/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 14 1.12.1.3 source/albert/graph2d/colprof.cpp, 2d, ioc.v400, 000302 
/*==========================================
||
||	File:	ColorProfile.c
||
||	What:	ColorProfile routines for Color matching
||
||	Change History:
||
*/

#include <colprof.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _COLPROF_CPP_
#include <ipagetun.h>
#endif
//----------------------------------------------------------------

const unsigned long  IColorProfile::fMatrixDimension = 3;


//-------------------------- Class IColorProfile ------------------------


IColorProfile::IColorProfile()
{
	defineTransformationMatrix();
}

IColorProfile::IColorProfile(const IColorProfile& aColorProfile)
{
	*this = aColorProfile;
}

IColorProfile::~IColorProfile ()
{
    // ### Put your Destructions Here
}

const icHeader* IColorProfile::header() const
{
    icHeader* result = 0;	// 0 /*NIL*/;
	// ### Put your stuff here
    return result;
}

const icTagList* IColorProfile::tagList() const
{
    icTagList* result = 0;	// 0 /*NIL*/;
	// ### Put your stuff here
    return result;
}

const iccTagData* IColorProfile::tagFromProfile(const iccTagSignature& signature) const
{
    iccTagData* result = 0;	// 0 /*NIL*/;
	// ### Put your stuff here
    return result;
}

IColorProfile& IColorProfile::operator=(const IColorProfile& source)
{
    if (&source != this)
    {
		long i,j;
		for (j =0; j<fMatrixDimension; j++)
			for (i =0; i<fMatrixDimension; i++) fRGB2XYZ[i][j] =source.fRGB2XYZ[i][j];
		for (j =0; j<fMatrixDimension; j++)
			for (i =0; i<fMatrixDimension; i++) fXYZ2RGB[i][j] =source.fXYZ2RGB[i][j];
    }
    return *this;
}


//----------------------------------------------------------------
void IColorProfile::convertRGBtoXYZ(
	GIntensity red,
	GIntensity green,
	GIntensity blue,
	GIntensity& X,
	GIntensity& Y,
	GIntensity& Z) const
{
	if (red == 0.0 && green==0.0 && blue==0.0)
	{
		red = green = blue = 0.000001000;
	}
	X = GIntensity (fRGB2XYZ[0][0]*red + fRGB2XYZ[0][1]*green + fRGB2XYZ[0][2]*blue);
	Y = GIntensity (fRGB2XYZ[1][0]*red + fRGB2XYZ[1][1]*green + fRGB2XYZ[1][2]*blue);
	Z = GIntensity (fRGB2XYZ[2][0]*red + fRGB2XYZ[2][1]*green + fRGB2XYZ[2][2]*blue);
}

//----------------------------------------------------------------
void IColorProfile::convertXYZtoRGB(
	GIntensity X,
	GIntensity Y,
	GIntensity Z,
	GIntensity& red,
	GIntensity& green,
	GIntensity& blue) const
{
    red   = GIntensity( fXYZ2RGB[0][0]*X + fXYZ2RGB[0][1]*Y + fXYZ2RGB[0][2]*Z);
    green = GIntensity( fXYZ2RGB[1][0]*X + fXYZ2RGB[1][1]*Y + fXYZ2RGB[1][2]*Z);
    blue  = GIntensity( fXYZ2RGB[2][0]*X + fXYZ2RGB[2][1]*Y + fXYZ2RGB[2][2]*Z);
}


//----------------------------------------------------------------
void IColorProfile::defineTransformationMatrix()
{
	// Taken From Color and the Computer chapter 4 Page 86
	// equations (4-3) and (4-4)  notation used is same as in book.

	// Apple 13" monitor for right now.

/*
    xyzY Red	(0.616900,	0.340800,	 21.230000);
    xyzY Green	(0.278000,	0.601700,	 67.209999);
    xyzY Blue	(0.151100,	0.070600,	 11.560000);
    xyzY White	(0.279000,	0.296100,	100.000000);
*/

	ChromaCoord xR,yR,zR;	// red
	xR = 0.616900;
	yR = 0.340800;
	zR = ChromaCoord(1.0 - xR - yR);

	ChromaCoord xG,yG,zG;	// green
	xG = 0.278000;
	yG = 0.601700;
	zG = ChromaCoord(1 - xG - yG);

	ChromaCoord xB,yB,zB;	// blue
	xB = 0.151100;
	yB = 0.070600;
	zB = ChromaCoord(1 - xB - yB);

	ChromaCoord xW,yW;		// white
	TristimulusCoord YW;
	xW = 0.279000;
	yW = 0.296100;
	YW = 100.000000;

	float D = xR*(yG-yB)+xG*(yB-yR)+xB*(yR-yG);

	float CR= (YW/yW)*((xW*(yG-yB) - yW*(xG-xB) + xG*yB - xB*yG )/D );

	float CG= (YW/yW)*((xW*(yB-yR) - yW*(xB-xR) - xR*yB + xB*yR )/D );
	float CB= (YW/yW)*((xW*(yR-yG) - yW*(xR-xG) + xR*yG - xG*yR )/D );

	float XR =xR*CR; float XG=xG*CG; float XB=xB*CB;
	float YR =yR*CR; float YG=yG*CG; float YB=yB*CB;
	float ZR= zR*CR; float ZG=zG*CG; float ZB=zB*CB;

	fRGB2XYZ[0][0] = XR; fRGB2XYZ[0][1] = XG; fRGB2XYZ[0][2] = XB;
	fRGB2XYZ[1][0] = YR; fRGB2XYZ[1][1] = YG; fRGB2XYZ[1][2] = YB;
	fRGB2XYZ[2][0] = ZR; fRGB2XYZ[2][1] = ZG; fRGB2XYZ[2][2] = ZB;

	makefXYZ2RGBMatrix();
}


//----------------------------------------------------------------
void IColorProfile::makefXYZ2RGBMatrix()
{
	// Brut Froce inversion of a 3x3 Matrix;
	// not very pretty but okay -> Adjoint/Determinant

	double Determinant=0;

	double a11=fRGB2XYZ[0][0], a12=fRGB2XYZ[0][1], a13=fRGB2XYZ[0][2];
	double a21=fRGB2XYZ[1][0], a22=fRGB2XYZ[1][1], a23=fRGB2XYZ[1][2];
	double a31=fRGB2XYZ[2][0], a32=fRGB2XYZ[2][1], a33=fRGB2XYZ[2][2];

	double c11, c12, c13;
	double c21, c22, c23;
	double c31, c32, c33;

	Determinant = a11*(a22*a33 - a23*a32) - a12*(a21*a33 - a23*a31) + a13*(a21*a32 - a22*a31);

	/* this can never happen case is well defined
	if(Determinant == 0.0 ) {
		qprintf("Not a valid fRGB2XYZ Matrix could not Invert\n");
    	exit(1);}
	*/

	c11 =      (a22*a33 - a23*a32);
	c12 = -1.0*(a21*a33 - a23*a31);
	c13 =      (a21*a32 - a22*a31);

	c21 = -1.0*(a12*a33 - a13*a32);
	c22 =      (a11*a33 - a13*a31);
	c23 = -1.0*(a11*a32 - a12*a31);

	c31 =      (a12*a23 - a13*a22);
	c32 = -1.0*(a11*a23 - a13*a21);
	c33 =      (a11*a22 - a12*a21);

	fXYZ2RGB[0][0] = c11/Determinant;
	fXYZ2RGB[0][1] = c21/Determinant;
	fXYZ2RGB[0][2] = c31/Determinant;

	fXYZ2RGB[1][0] = c12/Determinant;
	fXYZ2RGB[1][1] = c22/Determinant;
	fXYZ2RGB[1][2] = c32/Determinant;

	fXYZ2RGB[2][0] = c13/Determinant;
	fXYZ2RGB[2][1] = c23/Determinant;
	fXYZ2RGB[2][2] = c33/Determinant;
}

