// Revision: 27 1.4.1.4 source/albert/graph2d/conics.hpp, 2d, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
/*================
||
||  File:   Conics.h
||
||  What:   Declaration of Albert Graphic Conic Acceleration Support Classes
||
||  Change History:
||  18 Oct 96   tt      First version in OpenClass.
||
*/

#ifndef _CONICS_
#define _CONICS_

#include <igbase2d.hpp>
#include <imatrix.hpp>


#ifndef NO_CONIC_ACCELERATION_SUPPORT

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)
// -----------------------------------------------------------------------------
// internal class to implement conic acceleration support...
class IConicInfo {
  public:
    enum ECurveType { kArc, kEllipse, kPie, kChord };

    IConicInfo(ECurveType kind);
    IConicInfo(ECurveType kind, const IGRect2D& arcBounds, bool direction);
    IConicInfo(const IConicInfo& src);
    ~IConicInfo();

    IConicInfo& operator=(const IConicInfo& src);

    bool operator==(const IConicInfo& src);
    bool transformBy(const IGrafMatrix& matrix);
    GDegrees calculatePointAngle(const IGPoint2D& point);

    ECurveType fType;
    bool fDirection;
    IGRect2D fArcBounds;
	IGPoint2D fStart, fEnd;	// Begin and end of the arc of a pie/chord...
};
#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // NO_CONIC_ACCELERATION_SUPPORT

#endif // _CONICS_
