/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 21 1.9.1.3 source/albert/graph2d/cuspline.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:   CubicSplines.C
||
||  What:   Tools for generating "matrix" cubic splines
||
||  Change history:
||
*/

#include <igloop2d.hpp>
#include <grassert.hpp>
#include <prspline.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _CUSPLINE_CPP_
#include <ipagetun.h>
#endif

IGCardinalSpline::IGCardinalSpline( const IGPoint2DArray& points,
                                    GCoordinate tension )
                                : IGCurve2D( 4, points )
{
    if( tension < 0. ) tension = 0.;
    else
    if( tension > 1. ) tension = 1.;

    BasisMatrix m;
    GParametric *mp = &m[0];
    *mp++ = -tension;
    *mp++ = 2.-tension;
    *mp++ = tension-2.;
    *mp++ = tension;
    *mp++ = 2.*tension;
    *mp++ = tension-3.;
    *mp++ = 3.-2.*tension;
    *mp++ = -tension;
    *mp++ = -tension;
    *mp++ = 0.;
    *mp++ = tension;
    *mp++ = 0.;
    *mp++ = 0.;
    *mp++ = 1.;
    *mp++ = 0.;
    *mp++ = 0.;
    setBasisSpline( m, 1 );
}
IGCardinalSpline::IGCardinalSpline()
{
}
IGCardinalSpline::IGCardinalSpline(const IGCardinalSpline& source)
    : IGCurve2D (source)
{
}
IGCardinalSpline::~IGCardinalSpline()
{
}

IGHermiteSpline::IGHermiteSpline( const IGPoint2DArray& points )
            : IGCurve2D( 4, points )
{
    BasisMatrix m = {
         2., 1., -2., 1.,
         -3., -2., 3., -1.,
         0., 1., 0., 0.,
         1., 0., 0., 0.
     };

    setBasisSpline( m, 4 );
}
IGHermiteSpline::IGHermiteSpline()
{
}
IGHermiteSpline::IGHermiteSpline(const IGHermiteSpline& source)
    : IGCurve2D (source)
{
}
IGHermiteSpline::~IGHermiteSpline()
{
}

IGTensionSpline::IGTensionSpline( const IGPoint2DArray& points,
                                  GCoordinate tension, GCoordinate bias )
            : IGCurve2D( 4, points )
{
    GCoordinate b2,b3,d;

    b2 = bias*bias;
    b3 = b2*bias;

    d = 2./(tension+2.*b3+4.*(b2+bias)+2.); // note the +2. in the numerator
    BasisMatrix m;
    GParametric *mp = &m[0];
    *mp++ =  -d*b3;
    *mp++ =  d*(tension+b3+b2+bias);
    *mp++ =  -d*(tension+b2+bias+1.);
    *mp++ =  d;
    *mp++ =  3.*d*b3;
    *mp++ =  -3.*d/2.*(tension+2.*b3+2.*b2);
    *mp++ =  (3.*tension+6.*b2)*d/2.;
    *mp++ =  0.;
    *mp++ =  -3.*d*b3;
    *mp++ =  3.*d*(b3-bias);
    *mp++ =  3.*d*bias;
    *mp++ =  0.;
    *mp++ =  d*b3;
    *mp++ =  (tension+4.*b2+4*bias)*d/2.;
    *mp++ =  d;
    *mp++ =  0.;

    setBasisSpline( m, 1 );
}
IGTensionSpline::IGTensionSpline()
{
}
IGTensionSpline::IGTensionSpline(const IGTensionSpline& source)
    : IGCurve2D (source)
{
}
IGTensionSpline::~IGTensionSpline()
{
}
