/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 28 1.16.1.4 source/albert/graph2d/gdiimdev.hpp, 2d, ioc.v400, 000302 
/*

        name:   GdiImageDevice.h
        type:   C++

        This is the definition of the IGdiImageDevice class.

        A IGdiImageDevice is an IOC GrafDevice that is based
on a GDI memory device context. The device context is compatible with
the display and has a bitmap selected in it. The bitmap is part of
a IGHBITMAPImage object.

Programmers:
AGR     Arthur Ryman

Change History:

*/

#ifndef _GDIIMDEV_
#define _GDIIMDEV_

#include <ilanglvl.hpp>

#if defined(IC_WIN)
#include <gdidev.hpp>
#elif defined(IC_PM)
#include <gpidev.hpp>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGImage;

#pragma pack(push,4)

#if defined(IC_WIN)
class IGdiImageDevice: public IGdiDevice
#elif defined(IC_PM)
class IGdiImageDevice: public IGpiDevice
#endif
{
public:
    IGdiImageDevice(IGImage* aliasImage, IPresSpaceHandle hdcMem);
    virtual ~IGdiImageDevice ();

    virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState);

    virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState);

    virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState);

    virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState);

    virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState);

    virtual void renderArea(const IGArea& geometry, const IGrafState& grafState);

    virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState);

    virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState);

    virtual void renderImage(
    	const IGImage& image,
		const IGRect2D& source,
		const IGrafState& grafState);

   	virtual void renderMaskedImage(
		const IGImage& image,
		const IGImage& mask,
		const IGRect2D& source,
		const IGrafState& grafState);

	virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState);

#if !IC_RTTI_ENABLED
	GrafDeviceTypeDeclarationsMacro(IGdiImageDevice);
#endif // IC_RTTI_ENABLED

private:
    IGImage* fImage;

    IGdiImageDevice();
    IGdiImageDevice(const IGdiImageDevice&);
    IGdiImageDevice& operator=(const IGdiImageDevice& Src);
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _GDIIMDEV_
