/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 37 1.26.1.3 source/albert/graph2d/grafarea.cpp, 2d, ioc.v400, 000302 
/*
||
||  File: GraphicAreas.C
||
||
*/

#include <igraph2d.hpp>
#include <igrport.hpp>
#include <igrafatt.hpp>
#include <grtype.hpp>
#include <idatstrm.hpp>
#include <grstrmod.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _GRAFAREA_CPP_
#include <ipagetun.h>
#endif
/*================================
||  IEllipse::
||
*/

StreamableDefinitionsMacro(IEllipse, gGraphicsStreamModule);

// ----- Constructors and Destructors -----

IEllipse::IEllipse() : IMGraphic(), fBounds(0 /*NIL*/), fEllipse(0 /*NIL*/)
{}

IEllipse::IEllipse(const IEllipse& ellipse) :
	IMGraphic(ellipse),
	fBounds(0 /*NIL*/),
	fEllipse(0 /*NIL*/)
{
	if (ellipse.fEllipse != 0 /*NIL*/) {
		fEllipse = new IGEllipse2D(*ellipse.fEllipse);
		fMatrix = ellipse.fMatrix;
	}
}

IEllipse::IEllipse(IGEllipse2D* ellipse, IGrafBundle* bundle) :
	IMGraphic(bundle),
	fEllipse(ellipse),
	fBounds(0 /*NIL*/)
{}

IEllipse::~IEllipse()
{
	delete fBounds;
	delete fEllipse;
}

void IEllipse::adoptGeometry(IGEllipse2D* ellipse)
{
	delete fEllipse;
	fEllipse = ellipse;
}

IGEllipse2D* IEllipse::orphanGeometry()
{
	IGEllipse2D* ellipse = fEllipse;
	fEllipse = 0 /*NIL*/;
	return ellipse;
}

const IGEllipse2D* IEllipse::geometry()
{
	return fEllipse;
}

void IEllipse::draw(IGrafPort& port)const
{
	#if 0
	if (fEllipse)
	{
		const IAttributeState* state = attributeState();
		if (!state) {
			if (fMatrix.isIdentity())
				port.draw(*fEllipse);
			else
    			ILinkedGrafPort(&port, kModelMatrix, &fMatrix).draw(*fEllipse);
		}
		else {
    		ILinkedGrafPort bundlePort(&port, state);
			if (fMatrix.isIdentity())
    			bundlePort.draw(*fEllipse);
			else
    			ILinkedGrafPort(&bundlePort, kModelMatrix, &fMatrix).draw(*fEllipse);
		}
	}
#else
if (fEllipse)
{
   const IAttributeState* state = attributeState();
	 ILinkedGrafPort bundlePort(&port, state);
	 ILinkedGrafPort(&bundlePort, kModelMatrix, &fMatrix).draw(*fEllipse);
}
#endif
}

void IEllipse::transformBy(const IGrafMatrix& m)
{
	if (fEllipse == 0 /*NIL*/)return;

	fMatrix.concatWith(m);
	updateTimeStamp();
}

IGRect2D IEllipse::geometricBounds()const
{
	if (fEllipse == 0 /*NIL*/)return IGRect2D();
	if (fMatrix.isIdentity())
		return fEllipse->bounds();
	else
		return fMatrix.transformBounds(fEllipse->bounds());
}


bool IEllipse::intersects (const IGRect2D &  rect) const
{
  bool  retval = false;
  if (isHitEnabled () == true)
  {
    if (fEllipse != 0 /* NIL */)
    {
      retval = fEllipse->intersects (rect);
    }

    ((IEllipse *) this)->setHit (retval);  // for performance reasons,
                                           // hit state is cached.
  }

  return (retval);

} // IEllipse::intersects


IGRect2D IEllipse::looseFitBounds(const IGrafPort* port)const
{
	return IMGraphic::looseFitBounds(port);
}



void IEllipse::writeToStream(IDataStream& toWhere)const
{
	IStreamOutFrame streamFrame(toWhere);
    IMGraphic::writeToStream(toWhere);
    bool nilPointer = (fEllipse == 0 /*NIL*/)? true : false;
    long(nilPointer) >>= toWhere;
    if (! nilPointer)
    	*fEllipse >>=(toWhere);
}

void IEllipse::readFromStream(IDataStream& fromWhere)
{
	IStreamInFrame streamFrame(fromWhere);
    IMGraphic::readFromStream(fromWhere);

    bool nilPointer;
    long temp;
    temp <<= fromWhere;
    nilPointer = bool(temp);
    if (! nilPointer)
    {
	if (fEllipse == 0 /*NIL*/)
            fEllipse = new IGEllipse2D;
        *fEllipse <<=(fromWhere);
    }
    else
    {
       delete fEllipse;
       fEllipse = 0 /*NIL*/;
    }

    delete fBounds;
    fBounds = 0 /*NIL*/;
}

IEllipse& IEllipse::operator=(const IEllipse& other)
{
    if (&other != this)
    {
        IMGraphic::operator=(other);
	    if (other.fEllipse != 0 /*NIL*/)
	    {
	    	if (fEllipse == 0 /*NIL*/)	
	    		fEllipse = new IGEllipse2D(*other.fEllipse);
			else
	    		*fEllipse = (*other.fEllipse);
	    }else
	    {
	    	delete fEllipse;
	    	fEllipse = 0 /*NIL*/;
	    }

	    if (other.fBounds != 0 /*NIL*/)
	    {
	      if (fBounds == 0 /*NIL*/)	
		fBounds = new IGRect2D (*(other.fBounds));
	      else
		*fBounds = *(other.fBounds);
	    }else
	    {
	      delete fBounds;
	      fBounds = 0 /*NIL*/;
	    }

        fBoundsTimeStamp = other.fBoundsTimeStamp;
    }
    return (*this);
}

bool IEllipse::operator==(const IMDrawable& src)const
{
	if (!IMGraphic::operator==(src))
		return false;

	IEllipse* srcGraphic = (IEllipse*)&src;
	return (fEllipse == srcGraphic->fEllipse || *fEllipse == *srcGraphic->fEllipse);
}

/*================================
||  IPolygon::
||
*/

StreamableDefinitionsMacro(IPolygon, gGraphicsStreamModule);

// ----- Constructors and Destructors -----

IPolygon::IPolygon() : IMGraphic(), fBounds(0 /*NIL*/), fPolygon(0 /*NIL*/)
{}

IPolygon::IPolygon(const IPolygon& polygon) :
	IMGraphic(polygon),
	fBounds(0 /*NIL*/),
	fPolygon(0 /*NIL*/)
{
	if (polygon.fPolygon != 0 /*NIL*/) {
		fPolygon = new IGPolygon2D(*polygon.fPolygon);
	}
}

IPolygon::IPolygon(IGPolygon2D* polygon, IGrafBundle* bundle) :
	IMGraphic(bundle),
	fPolygon(polygon),
	fBounds(0 /*NIL*/)
{}

IPolygon::~IPolygon()
{
	delete fBounds;
	delete fPolygon;
}

void IPolygon::adoptGeometry(IGPolygon2D* polygon)
{
	delete fPolygon;
	fPolygon = polygon;
}

IGPolygon2D* IPolygon::orphanGeometry()
{
	IGPolygon2D* polygon = fPolygon;
	fPolygon = 0 /*NIL*/;
	return polygon;
}

const IGPolygon2D* IPolygon::geometry()
{
	return fPolygon;
}

void IPolygon::draw(IGrafPort& port)const
{
	if (fPolygon)
	{
		const IAttributeState* state = attributeState();
		if (!state) {
			port.draw(*fPolygon);
		}
		else {
			ILinkedGrafPort(&port, state).draw(*fPolygon);
		}
	}
}

void IPolygon::transformBy(const IGrafMatrix& m)
{
	if (fPolygon == 0 /*NIL*/)return;

	fPolygon->transformBy(m);
	updateTimeStamp();
}

IGRect2D IPolygon::geometricBounds()const
{
	if (fPolygon == 0 /*NIL*/)return IGRect2D();
	return fPolygon->bounds();
}


bool IPolygon::intersects (const IGRect2D &  rect) const
{
  bool  retval = false;
  if (isHitEnabled () == true)
  {
    if (fPolygon != 0 /* NIL */)
    {
      retval = fPolygon->intersects (rect);
    }

    ((IPolygon *) this)->setHit (retval);  // for performance reasons,
                                           // hit state is cached.
  }

  return (retval);

} // IPolygon::intersects


IGRect2D IPolygon::looseFitBounds(const IGrafPort* port)const
{
	return IMGraphic::looseFitBounds(port);
}



void IPolygon::writeToStream(IDataStream& toWhere)const
{
    IStreamOutFrame streamFrame(toWhere);
    IMGraphic::writeToStream(toWhere);

    bool  polygonNotNil (fPolygon != 0 /* NIL */);
    long (polygonNotNil) >>= toWhere;
    if (polygonNotNil)
    {
      *fPolygon >>= toWhere;
    }

    bool  boundsNotNil (fBounds != 0 /* NIL */);
    long (boundsNotNil) >>= toWhere;
    if (boundsNotNil)
    {
      *fBounds >>= toWhere;
    }

    fBoundsTimeStamp >>= toWhere;
}

void IPolygon::readFromStream(IDataStream& fromWhere)
{
	IStreamInFrame streamFrame(fromWhere);
    IMGraphic::readFromStream(fromWhere);

    long  temp;
    temp <<= fromWhere;
    bool  notNil = bool (temp);
    if (notNil)
    {
      if (fPolygon == 0 /* NIL */)
      {
	fPolygon = new IGPolygon2D;
      }

      *fPolygon <<= fromWhere;
    }
    else
    {
      delete fPolygon;
      fPolygon = 0 /* NIL */;
    }


    temp <<= fromWhere;
    notNil = bool (temp);
    if (notNil)
    {
      if (fBounds == 0 /* NIL */)
      {
    	fBounds = new IGRect2D;
      }

      *fBounds <<= fromWhere;
    }
    else
    {
      delete fBounds;
      fBounds = 0 /*NIL*/;
    }

    fBoundsTimeStamp <<= fromWhere;
}

IPolygon& IPolygon::operator=(const IPolygon& other)
{
  if (&other != this)
  {
    IMGraphic::operator= (other);

    if (other.fPolygon != 0 /* NIL */)
    {
      if (fPolygon == 0 /* NIL */)
	fPolygon = new IGPolygon2D (*(other.fPolygon));
      else
	*fPolygon = *(other.fPolygon);
    }
    else
    {
      delete fPolygon;
      fPolygon = 0 /* NIL */;
    }


    if (other.fBounds != 0 /* NIL */)
    {
      if (fBounds == 0 /* NIL */)
	fBounds = new IGRect2D (*(other.fBounds));
      else
	*fBounds = *(other.fBounds);
    }
    else
    {
      delete fBounds;
      fBounds = 0 /* NIL */;
    }

    fBoundsTimeStamp = other.fBoundsTimeStamp;

  } // if not self assignment

  return (*this);

} // IPolygon::operator=



bool IPolygon::operator==(const IMDrawable& src)const
{
	if (!IMGraphic::operator==(src))
		return false;

	IPolygon* srcGraphic = (IPolygon*)&src;
    return (fPolygon == srcGraphic->fPolygon || *fPolygon == *srcGraphic->fPolygon);
}


/*================================
||  IArea::
||
*/

StreamableDefinitionsMacro(IArea, gGraphicsStreamModule);
// ----- Constructors and Destructors -----

IArea::IArea() : IMGraphic(), fBounds(0 /*NIL*/), fArea(0 /*NIL*/)
{}

IArea::IArea(const IArea& area) :
	IMGraphic(area),
	fBounds(0 /*NIL*/),
	fArea(0 /*NIL*/)
{
	if (area.fArea != 0 /*NIL*/) {
		fArea = new IGArea(*area.fArea);
	}
}

IArea::IArea(IGArea* area, IGrafBundle* bundle) :
	IMGraphic(bundle),
	fArea(area),
	fBounds(0 /*NIL*/)
{}

IArea::~IArea()
{
	delete fBounds;
	delete fArea;
}

void IArea::adoptGeometry(IGArea* area)
{
	delete fArea;
	fArea = area;
}

IGArea* IArea::orphanGeometry()
{
	IGArea* area = fArea;
	fArea = 0 /*NIL*/;
	return area;
}

const IGArea* IArea::geometry()
{
	return fArea;
}

void IArea::draw(IGrafPort& port)const
{
	if (fArea)
	{
		const IAttributeState* state = attributeState();
		if (!state) {
			port.draw(*fArea);
		}
		else {
			ILinkedGrafPort(&port, state).draw(*fArea);
		}
	}
}

void IArea::transformBy(const IGrafMatrix& m)
{
	if (fArea == 0 /*NIL*/)return;

	fArea->transformBy(m);
	updateTimeStamp();
}

IGRect2D IArea::geometricBounds()const
{
	if (fArea == 0 /*NIL*/)
		return IGRect2D();
	return fArea->bounds();
}


bool IArea::intersects (const IGRect2D &  rect) const
{
  bool  retval = false;
  if (isHitEnabled () == true)
  {
    if (fArea != 0 /* NIL */)
    {
      retval = fArea->intersects (rect);
    }

    ((IArea *) this)->setHit (retval);  // for performance reasons,
                                        // hit state is cached.
  }

  return (retval);

} // IArea::intersects


IGRect2D IArea::looseFitBounds(const IGrafPort* port)const
{
	return IMGraphic::looseFitBounds(port);
}



void IArea::writeToStream(IDataStream& toWhere)const
{
	IStreamOutFrame streamFrame(toWhere);
    IMGraphic::writeToStream(toWhere);
    bool nilPointer = (fArea == 0 /*NIL*/)? true : false;
    long(nilPointer) >>= toWhere;
    if (! nilPointer)
        *fArea >>=(toWhere);
}

void IArea::readFromStream(IDataStream& fromWhere)
{
	IStreamInFrame streamFrame(fromWhere);
    IMGraphic::readFromStream(fromWhere);

    bool nilPointer;
    long temp;
    temp <<= fromWhere;
    nilPointer = bool(temp);
    if (! nilPointer)
    {
	if (fArea == 0 /*NIL*/)
            fArea = new IGArea;
        *fArea <<=(fromWhere);
    }
    else
    {
       delete fArea;
       fArea = 0 /*NIL*/;
    }

    delete fBounds;
    fBounds = 0 /*NIL*/;
}

IArea& IArea::operator=(const IArea& other)
{
    if (&other != this)
    {
        IMGraphic::operator=(other);

		if (other.fArea != 0 /*NIL*/)
		{
			if (fArea == 0 /*NIL*/)
				fArea = new IGArea(*other.fArea);
			else
				*fArea = (*other.fArea);
		}
		else
		{
			delete fArea;
			fArea = 0 /*NIL*/;
		}

		if (other.fBounds != 0 /*NIL*/)
		{
			if (fBounds == 0 /*NIL*/)
				fBounds = new IGRect2D (*(other.fBounds));
			else
				*fBounds = *(other.fBounds);
		}
		else
		{
			delete fBounds;
			fBounds = 0 /*NIL*/;
		}

		fBoundsTimeStamp = other.fBoundsTimeStamp;
    }
    return (*this);
}

bool IArea::operator==(const IMDrawable& src)const
{
	if (!IMGraphic::operator==(src))
		return false;

	IArea* srcGraphic = (IArea*)&src;
    return (fArea == srcGraphic->fArea || *fArea == *srcGraphic->fArea);
}


/*================================
||  ILoop::
||
*/

// ----- Constructors and Destructors -----

StreamableDefinitionsMacro(ILoop, gGraphicsStreamModule);

ILoop::ILoop() : IMGraphic(), fBounds(0 /*NIL*/), fLoop(0 /*NIL*/)
{}

ILoop::ILoop(const ILoop& loop) :
	IMGraphic(loop),
	fBounds(0 /*NIL*/),
	fLoop(0 /*NIL*/)
{
	if (loop.fLoop != 0 /*NIL*/) {
		fLoop = new IGLoop2D(*loop.fLoop);
	}
}

ILoop::ILoop(IGLoop2D* loop, IGrafBundle* bundle) :
	IMGraphic(bundle),
	fLoop(loop),
	fBounds(0 /*NIL*/)
{}

ILoop::~ILoop()
{
	delete fBounds;
	delete fLoop;
}

void ILoop::adoptGeometry(IGLoop2D* loop)
{
	delete fLoop;
	fLoop = loop;
}

IGLoop2D* ILoop::orphanGeometry()
{
	IGLoop2D* loop = fLoop;
	fLoop = 0 /*NIL*/;
	return loop;
}

const IGLoop2D* ILoop::geometry()
{
	return fLoop;
}

void ILoop::draw(IGrafPort& port)const
{
	if (fLoop)
	{
		const IAttributeState* state = attributeState();
		if (!state) {
			if (fMatrix.isIdentity())
				port.draw(*fLoop);
			else
    			ILinkedGrafPort(&port, kModelMatrix, &fMatrix).draw(*fLoop);
		}
		else {
    		ILinkedGrafPort bundlePort(&port, state);
			if (fMatrix.isIdentity())
    			bundlePort.draw(*fLoop);
			else
    			ILinkedGrafPort(&bundlePort, kModelMatrix, &fMatrix).draw(*fLoop);
		}
	}
}

void ILoop::transformBy(const IGrafMatrix& m)
{
	if (fLoop == 0 /*NIL*/)
		return;

	fMatrix.concatWith(m);
	updateTimeStamp();
}

IGRect2D ILoop::geometricBounds()const
{
	if (fLoop == 0 /*NIL*/)return IGRect2D();
	if (fMatrix.isIdentity())
  		return fLoop->bounds();
	else
		return fMatrix.transformBounds(fLoop->bounds());
}


bool ILoop::intersects (const IGRect2D &  rect) const
{
  bool  retval = false;
  if (isHitEnabled () == true)
  {
    if (fLoop != 0 /* NIL */)
    {
      retval = fLoop->intersects (rect);
    }

    ((ILoop *) this)->setHit (retval);  // for performance reasons,
                                        // hit state is cached.
  }

  return (retval);

} // ILoop::intersects


IGRect2D ILoop::looseFitBounds(const IGrafPort* port)const
{
	return IMGraphic::looseFitBounds(port);
}


void ILoop::writeToStream(IDataStream& toWhere)const
{
	IStreamOutFrame streamFrame(toWhere);
    IMGraphic::writeToStream(toWhere);
    bool notNil = false;
    if (fLoop != 0 /*NIL*/)
    	notNil = true;
    long(notNil) >>= toWhere;
    if (notNil)
    	*fLoop >>=(toWhere);

    notNil = false;
    if (fBounds != 0 /*NIL*/)
    	notNil = true;
    long(notNil) >>= toWhere;
    if (notNil)
    	*fBounds >>= toWhere;
    fBoundsTimeStamp >>= toWhere;
}

void ILoop::readFromStream(IDataStream& fromWhere)
{
    IStreamInFrame streamFrame(fromWhere);

    IMGraphic::readFromStream(fromWhere);
    delete fLoop;
    fLoop = 0 /*NIL*/;

    bool notNil;
    long temp;
    temp <<= fromWhere;
    notNil = bool(temp);

    if (notNil)
    {
    	fLoop = new IGLoop2D;
    	*fLoop <<=(fromWhere);
    }

    temp <<= fromWhere;
    notNil = bool (temp);
    if (notNil)
    {
      if (fBounds == 0 /* NIL */)
      {
    	fBounds = new IGRect2D;
      }

      *fBounds <<= fromWhere;
    }
    else
    {
      delete fBounds;
      fBounds = 0 /*NIL*/;
    }

    fBoundsTimeStamp <<= fromWhere;
}

ILoop& ILoop::operator=(const ILoop& other)
{
  if (&other != this)
  {
    IMGraphic::operator= (other);

    if (other.fLoop != 0 /* NIL */)
    {
      if (fLoop == 0 /* NIL */)
	fLoop = new IGLoop2D (*(other.fLoop));
      else
	*fLoop = *(other.fLoop);
    }
    else
    {
      delete fLoop;
      fLoop = 0 /* NIL */;
    }


    if (other.fBounds != 0 /* NIL */)
    {
      if (fBounds == 0 /* NIL */)
	fBounds = new IGRect2D (*(other.fBounds));
      else
	*fBounds = *(other.fBounds);
    }
    else
    {
      delete fBounds;
      fBounds = 0 /* NIL */;
    }

    fBoundsTimeStamp = other.fBoundsTimeStamp;

  } // if not self assignment

  return (*this);

} // ILoop::operator=


bool ILoop::operator==(const IMDrawable& src)const
{
	if (!IMGraphic::operator==(src))
		return false;

	ILoop* srcGraphic = (ILoop*)&src;
    return (fLoop == srcGraphic->fLoop || *fLoop == *srcGraphic->fLoop);
}
