/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 19 1.13.1.3 source/albert/graph2d/grafdevs.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:       GrafDevices.C
||
||  What:       Routines which support the generic graphics device and related classes
||
||  Change History:
||
*/

#include <igarea2d.hpp>
#include <grafdevs.hpp>
#include <imatrix.hpp>
#include <ipaints.hpp>
#include <igcurv2d.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _GRAFDEVS_CPP_
#include <ipagetun.h>
#endif

//=============//
// IGrafDevice //
//=============//


IGrafDevice::IGrafDevice() :
	fValid(true),
    fWorldBounds(IGRect2D::infiniteRect())
{}

IGrafDevice::IGrafDevice(const IGRect2D& worldbounds) :
	fValid(true),
    fWorldBounds(worldbounds)
{}

IGrafDevice::IGrafDevice(const IGrafDevice &source) :
	fValid(source.fValid),
    fWorldBounds(source.fWorldBounds)
{}

IGrafDevice::~IGrafDevice()
{}


IGRect2D IGrafDevice::worldBounds() const
{
    return fWorldBounds;
}

IGArea IGrafDevice::worldArea() const
{
    return IGArea(fWorldBounds);
}

const IGrafMatrix& IGrafDevice::deviceResolution(const IGRect2D&) const
{
    return IGrafMatrix::identity();
}

void IGrafDevice::flush()
{}

ICoordinateSystem::EOrientation IGrafDevice::coordinateSystem() const
{
	return ICoordinateSystem::kOriginUpperLeft;	// as default ...
}

bool IGrafDevice::isValid() const
{
	return fValid;
}

void IGrafDevice::setValidity(bool valid)
{
	fValid = valid;
}

void IGrafDevice::setWorldBounds(const IGRect2D& newBounds)
{
	fWorldBounds = newBounds;
}

// ----------------------------------------------------------------------------
// Stuff added to access friends of IGrafDevice by subclasses... (OCRESERVED)
//
#ifndef IC_IMAGE
#include <igimage.hpp>
#endif

#ifndef _CONICS_
#include <conics.hpp>
#endif

void IGrafDevice::renderImage(
	const IGImage& image,
	const IGrafState& grafState) {

	// turn arround and called the overloaded function which takes the sourceBounds
	renderImage(image, image.rasterBounds(), grafState);
}

void IGrafDevice::renderMaskedImage(
	const IGImage& image,
	const IGImage& mask,
	const IGrafState& grafState) {

	// turn arround and called the overloaded function which takes the sourceBounds
	renderMaskedImage(image, mask, image.rasterBounds(), grafState);
}

IGrafDevice::EMaskPattern IGrafDevice::maskPattern(const IMaskPattern* pattern)
{
	return (IGrafDevice::EMaskPattern) pattern->pattern();
}

const unsigned char* IGrafDevice::maskBits(const IMaskPattern* pattern)
{
	return pattern->bits();
}

void IGrafDevice::extract(
        const IGArea& area,
        IGrafExtractor& extractor,
        ICAGNode* parentNode,
        const IGrafMatrix& coordSys)
{
	area.extract(extractor, parentNode, coordSys);
}

IConicInfo* IGrafDevice::conicInfo(IGCurve2D& curve)
{
	return curve.fConicInfo;
}

void IGrafDevice::deleteConicInfo(IGCurve2D& curve)
{
	delete curve.fConicInfo;
	curve.fConicInfo = 0;	// NIL
}
