/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 45 1.9.1.3 source/albert/graph2d/grafextr.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:   GrafExtractor.c
||
||  What:   Implementation for Graphics Extractor
||
||  Classes defined in this file:
||
||  *IGrafExtractor
||
||  Change History:
||
*/

#include <igrtypes.hpp>
#include <cag.hpp>
#include <grafextr.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _GRAFEXTR_CPP_
#include <ipagetun.h>
#endif

/*================
||
||  IGrafExtractor constructors
||
*/

IGrafExtractor::IGrafExtractor ()
{
    fCAGNodes = 0 /*NIL*/;
    fNumCAGNodes = fNumAllocatedCAGNodes = 0;
}

IGrafExtractor::IGrafExtractor (const IGrafExtractor&)
{
    fCAGNodes = 0 /*NIL*/;
    fNumCAGNodes = fNumAllocatedCAGNodes = 0;
}


/*================
||
||  IGrafExtractor destructor
||
*/

IGrafExtractor::~IGrafExtractor ()
{
    deleteCAGNodes();
    delete [] fCAGNodes;
}


/*================
||
||  IGrafExtractor assignment operator
||
*/

IGrafExtractor& IGrafExtractor::operator= (const IGrafExtractor&)
{
    deleteCAGNodes();
    return *this;
}


/*================
||
||  IGrafExtractor cAG node function
||
*/

const long IGrafExtractor::kCAGNodeIncrement = 16;

void IGrafExtractor::adoptCAGNode (ICAGNode* node)
{
    if (fNumCAGNodes == fNumAllocatedCAGNodes)  // need more node space?
    {
        unsigned long oldNumAllocatedCAGNodes = fNumAllocatedCAGNodes;
        fNumAllocatedCAGNodes += kCAGNodeIncrement;
        ICAGNode** newCAGNodes = new ICAGNode*[fNumAllocatedCAGNodes];
        for (unsigned long i = 0; i < oldNumAllocatedCAGNodes; i++)
            newCAGNodes[i] = fCAGNodes[i];
        delete [] fCAGNodes;
        fCAGNodes = newCAGNodes;
    }

    fCAGNodes[fNumCAGNodes] = node; // remember for later delete
    fNumCAGNodes++;
}


/*================
||
||  IGrafExtractor deleteCAGNodes function
||
*/

void IGrafExtractor::deleteCAGNodes ()
{
    // Delete all cAG nodes
    ICAGNode** nodePtr = fCAGNodes;
    for (long i = 0; i < fNumCAGNodes; i++, nodePtr++)
        delete *nodePtr;

    fNumCAGNodes = 0;
}
