// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Test application that draws an IGraphicText centered in the window.

/*
        7/9/97 - Removed fCanvas from MainWindow - caused problems on OS/2
*/
// Revision: 80 1.11.1.2 source/albert/graph2d/graftext/app/main.cpp, 2d, ioc.v400, 000302 

#include <iframe.hpp>              
#include <icmdhdr.hpp>
#include <imenubar.hpp>
#include <ipainhdr.hpp>
#include <ipainevt.hpp>
#include <ihandle.hpp>
#include <icanvas.hpp>

#include <igraph2d.hpp>
#include <grafdevs.hpp>
#include <igrport.hpp>

#ifndef _IBCOLOR_
#include <ibcolor.hpp> // <Taligent>
#endif

#ifndef _IGRPORT_
#include <igrport.hpp> // <Taligent>
#endif

#include <itext.hpp>
#include <igraftxt.hpp>
#include <istyles.hpp>
#include <itrancod.hpp>

#include <igtest2d.hpp>

#include <iostream.h>

// Segment definitions
#ifdef IC_PAGETUNE
#define _MAIN_CPP_
#include <ipagetun.h>
#endif

#define WND_MAIN              0x1000
#define WND_DRAW              0x8008

//------------------------------------------------------------------------------------

class MainWindow : public IFrameWindow/*, public IPaintHandler*/ 
{
public:
  MainWindow(unsigned long windowId, const IText& text, unsigned long rot, IGraphicText::ELineBreak lineBreak);
  MainWindow(unsigned long windowId, const IText& text, unsigned long rot, GCoordinate wrapWidth);
  virtual ~MainWindow();
  
protected:
  virtual bool  paintWindow(IPaintEvent& event);
  
private:
//  ICanvas          fCanvas;
  IGraphicText     fGraphic;
  IPaintConnectionTo<MainWindow> fPaintConnection;
};

//------------------------------------------------------------------------------------

MainWindow::MainWindow(unsigned long windowId,
					   const IText& srctext,
					   unsigned long rot,
					   IGraphicText::ELineBreak lineBreak)
  : IFrameWindow(IFrameWindow::defaultStyle() | IFrameWindow::animated, windowId)
//  , fCanvas(WND_DRAW, this, this) 
  , fGraphic(IText(), lineBreak)
  , fPaintConnection(*this, MainWindow::paintWindow)
{
  // If I try to create the root graf port from the canvas in the initializer list
  // and cache it, when the graphic tries to get the device context I get a
  // segment violation.

  IText text = srctext;

  if (text.length() >= 10)
    text.addStyles(ITextPointSizeStyle(36), 0, 10);
  if (text.length() >= 30)
    text.addStyles(ITextPointSizeStyle(18), 20, 10);
  if (text.length() >= 15)
    text.addStyles(ITextBoldfaceStyle(true), 5, 10);
  if (text.length() >= 17)
    text.addStyles(ITextItalicStyle(true), 7, 10);
  if (text.length() >= 25)
    text.addStyles(ITextUnderlineStyle(true), 15, 10);
  if (text.length() >= 35)
    text.addStyles(ITextStrikethroughStyle(true), 20, 15);
  if (text.length() >= 25)
    text.addStyles(ITextColorStyle(0x80, 0x80, 0x00), 5, 20);

  fGraphic.setText(text);
  fGraphic.rotateBy(rot);

  //IPaintHandler::handleEventsFor(this);
  fPaintConnection.handleEventsFor(this);
  setFocus();
  show();
}

MainWindow::MainWindow(unsigned long windowId,
					   const IText& srctext,
					   unsigned long rot,
					   GCoordinate wrapWidth)
  : IFrameWindow(IFrameWindow::defaultStyle() | IFrameWindow::animated, windowId)
//  , fCanvas(WND_DRAW, this, this) 
  , fGraphic(IText(), wrapWidth)
  , fPaintConnection(*this, MainWindow::paintWindow)
{
  // If I try to create the root graf port from the canvas in the initializer list
  // and cache it, when the graphic tries to get the device context I get a
  // segment violation.

  IText text = srctext;

  if (text.length() >= 10)
    text.addStyles(ITextPointSizeStyle(36), 0, 10);
  if (text.length() >= 30)
    text.addStyles(ITextPointSizeStyle(18), 20, 10);
  if (text.length() >= 15)
    text.addStyles(ITextBoldfaceStyle(true), 5, 10);
  if (text.length() >= 17)
    text.addStyles(ITextItalicStyle(true), 7, 10);
  if (text.length() >= 25)
    text.addStyles(ITextUnderlineStyle(true), 15, 10);
  if (text.length() >= 35)
    text.addStyles(ITextStrikethroughStyle(true), 20, 15);
  if (text.length() >= 25)
    text.addStyles(ITextColorStyle(0x80, 0x80, 0x00), 5, 20);

  fGraphic.setText(text);
  fGraphic.rotateBy(rot);

//  IPaintHandler::handleEventsFor(this);
  fPaintConnection.handleEventsFor(this);
  setFocus();
  show();
}

MainWindow::~MainWindow()
{
//  IPaintHandler::stopHandlingEventsFor(this);
}

bool MainWindow::paintWindow(IPaintEvent& event)
{
//  fCanvas.refresh(IWindow::paintAllImmediate);

  IManagedPresSpaceHandle presSpace(this);
  IBaseRootGrafPort port(presSpace);

  IGRect2D portBounds = IGraph2DTestHelper::device(port)->worldBounds();
  IGRect2D textBounds = fGraphic.looseFitBounds(&port);
  IGRect2D localTextBounds = fGraphic.textBounds(&port);
  IGPoint2D location = fGraphic.location();

  location += portBounds.center() - textBounds.center();
  fGraphic.setLocation(location);

  textBounds = fGraphic.looseFitBounds(&port);

  port.draw(portBounds, IFillBundle(backgroundColor())); // erase background
  port.draw(textBounds);

  IPolygon	polygon(new IGPolygon2D(localTextBounds));
  polygon.transformBy(fGraphic.transform());
  polygon.draw(port);
  
  port.draw(IGLine2D(IGPoint2D(location.fX, portBounds.fTop), IGPoint2D(location.fX, portBounds.fBottom)));
  port.draw(IGLine2D(IGPoint2D(portBounds.fLeft, location.fY), IGPoint2D(portBounds.fRight, location.fY)));

  fGraphic.draw(port);

  return true;
}

int 
main(int argc, char** argv)
{
  const char* str = "This is some sample text to display.";
  unsigned long rot = 0;
  GCoordinate wrapWidth = 99999;	// some arbitrarily large number
  IGraphicText::ELineBreak lineBreak = IGraphicText::kAllOnOneLine;

  for (int i = 1; i < argc; ++i) {
    if (argv[i][0] == '-') {
      if (argv[i][1] == 'r' || argv[i][1] == 'R') {
		if (i + 1 < argc) {
		  i += 1;
		  if (argv[i][0] == '-') {
		    rot = 360 - atol(argv[i] + 1);
		  } else {
		    rot = atol(argv[i]);
		  }
		}
	   } else if (argv[i][1] == 'f' || argv[i][1] == 'F')
	     lineBreak = IGraphicText::kFirstLineOnly;
	   else if (argv[i][1] == 'a' || argv[i][1] == 'A')
	     lineBreak = IGraphicText::kAllOnOneLine;
	   else if (argv[i][1] == 'w' || argv[i][1] == 'W') {
	     if (i + 1 < argc) {
		   i += 1;
		   wrapWidth = atof(argv[i]);
		   lineBreak = IGraphicText::kMultipleLines;
		 }
	   } else if (argv[i][1] == 's' || argv[i][1] == 'S') {
		if (i + 1 < argc) {
		  i += 1;
		  str = argv[i];
		}
      }
    }
  }

  char* newstr = strdup(str);
  const char* p = str;
  const char* e = p + strlen(str);
  char* q = newstr;

  while (p != e) {
    char c = *p;
    if (c == '\\') {
      ++p;
      if (p != e) {
	switch (*p) {
	case 'n': c = '\n'; break;
	case 'r': c = 0x0b; break; // vertical tab, transcoder should convert to line separator
	case 't': c = '\t'; break;
	}
      }
    }
    *q = c;
    ++q;
    ++p;
  }
  *q = 0;

  cout << "rot: " << rot <<  ", str: '" << newstr << "'" << endl;

  if (lineBreak != IGraphicText::kMultipleLines) {
    MainWindow mainWindow(WND_MAIN, ILineBreakConverter::convert(IText(newstr)), rot, lineBreak);
    IApplication::current().run();
  }
  else {
    MainWindow mainWindow(WND_MAIN, ILineBreakConverter::convert(IText(newstr)), rot, wrapWidth);
    IApplication::current().run();
  }

  free(newstr);
} 
