/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 58 1.11.2.4 source/albert/graph2d/grrefcnt.hpp, 2d, ioc.v400, 000302 
///////////////////////////////////////////////////////////////////////
//
// File: GraphicReferenceCounting.h (ReferenceCounting.h)
//

#ifndef _GRREFCNT_
#define _GRREFCNT_

#ifndef _IGRAPHPL_H
#include <igraphpl.hpp>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

typedef unsigned long ReferenceCount;

class IMGraphicReferenceCounted
{
    public:

        virtual void                unreachable();
                void                addReference();
                void                removeReference();
                ReferenceCount      count() const;

    protected:
                                    IMGraphicReferenceCounted(ReferenceCount initial = 0);
                                    IMGraphicReferenceCounted(const IMGraphicReferenceCounted&);

                IMGraphicReferenceCounted&  operator=(const IMGraphicReferenceCounted&);
    private:
                ReferenceCount      fReferenceCount;
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

inline IMGraphicReferenceCounted::IMGraphicReferenceCounted(unsigned long initial)
{
    fReferenceCount = initial;
}

inline IMGraphicReferenceCounted::IMGraphicReferenceCounted(const IMGraphicReferenceCounted&)
{
    fReferenceCount = 0;
}

inline void IMGraphicReferenceCounted::unreachable()
{
	delete this;
}
	
inline IMGraphicReferenceCounted& IMGraphicReferenceCounted::operator=(const IMGraphicReferenceCounted& m)
{
	fReferenceCount = m.fReferenceCount;
	return *this;
}

inline void IMGraphicReferenceCounted::addReference()
{
    fReferenceCount++;
}

inline unsigned long IMGraphicReferenceCounted::count() const
{
    return fReferenceCount;
}

inline void IMGraphicReferenceCounted::removeReference()
{
    if (!--fReferenceCount)
        unreachable();
}

#endif // _GRREFCNT_
