/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       ibmpconv.hpp
||
||  What:       Routines which support loading of bmp Image format
||
*/

// Revision: 01 1.2.1.8 source/albert/graph2d/ibmpconv.hpp, 2d, ioc.v400, 000302 

#ifndef _IBMPCONV_
#define _IBMPCONV_

#include <imgconv.hpp>
#include <igimage.hpp>
#include <imghelp.h>

//***************************************************************************
//BMP
//***************************************************************************
#define BFT_BMAP        0x4d42
#define BFT_BITMAPARRAY 0x4142
#define BCA_UNCOMP      0x00000000L
#define BCA_RLE8        0x00000001L
#define BCA_RLE4        0x00000002L
#define BCA_HUFFFMAN1D  0x00000003L
#define BCA_RLE24       0x00000004L
#define MSWCC_EOL       0
#define MSWCC_EOB       1
#define MSWCC_DELTA     2

#ifdef IC_WIN
#define SV_CXSCREEN            SM_CXSCREEN
#define SV_CYSCREEN            SM_CYSCREEN
#define IQUERYSYSVALUE(idx)             (GetSystemMetrics( idx ))
#endif // IC_WIN
#ifdef IC_PM
#define IQUERYSYSVALUE(idx)             (WinQuerySysValue( HWND_DESKTOP, idx ))
#endif // IC_PM

#define AHEAD_BUF 0x4000

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

typedef struct _AHEAD_
{
	iBYTE buf[AHEAD_BUF];
	int inx, cnt;
	IDataStream *fromWhere;
} AHEAD;

class IBMPConverter : public IImageConverter
{
public:
	IBMPConverter();
	~IBMPConverter();

	IGImage* createFromStream( IDataStream& fromWhere );

	void writeTo( const IGImage& fromImage,
				IDataStream& toWhere) const;
#ifdef IC_WIN
	IPixelBuffer* createDIBFromResource(char *, unsigned long);
#endif //IC_WIN
	
protected:
	IGImage* createFromFile( const char* fromFile);
	int writeToFile( char *toFile, iBYTE *data);
private:
	int readBmp(IDataStream&, bool skipper=true);
	int bmp_rdata(IDataStream&, iBYTE *data);
	iCWORD writeInv(int fd, iBYTE* buffer, iCWORD count);
	int writeBmp(int fd, iBYTE* data);

	AHEAD* gbm_create_ahead(IDataStream&);
	void gbm_destroy_ahead(AHEAD *ahead);
	int gbm_read_ahead(AHEAD *ahead);

	iDWORD iBase, cbFix, ulCompression, offBits;
	bool bWindows;
	
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

//==============================================================================
#endif // _IBMPCONV_
