/*
******************************************************************************
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation,  1997           *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************
||
||  File:       igifconv.hpp
||
||  What:       Routines which support loading of gif Image format
||
*/

// Revision: 02 1.2.1.6 source/albert/graph2d/igifconv.hpp, 2d, ioc.v400, 000302 

#ifndef _IGIFCONV_
#define _IGIFCONV_

#include <imgconv.hpp>
#include <igimage.hpp>
#include <imghelp.h>

//***************************************************************************
//GIF
//***************************************************************************

#define GBM_ERR_OK		1
#define GBM_ERR_MEM		-1
#define GBM_ERR_READ		-2
#define GBM_ERR_BAD_MAGIC	-3
#define GBM_ERR_BAD_SIZE	-4
#define GBM_ERR_BPP		-5
#define GBM_ERR_WRITE		-6

#define GBM_ERR_GIF_TERM	-11
#define GBM_ERR_GIF_CODE_SIZE	-12
#define GBM_ERR_GIF_CORRUPT	-13

#define GBM_ERR_BMP_PLANES	-21
#define GBM_ERR_BMP_CBFIX	-22
#define GBM_ERR_BMP_COMP	-23

#define	GIF_INTERLACEMASK 	0x40
#define	GIF_HASCOLORMAPMASK 	0x80

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

typedef struct
{
	IDataStream *fromWhere;
        long inx, size;                  /* Index and size in bits */
        iBYTE buf[255+3];                /* Buffer holding bits */
        long code_size;                  /* Number of bits to return at once */
        iCWORD read_mask;                /* 2^code_size-1 */
} READ_CONTEXT;

/*output context*/
typedef struct
{
        int x, y, w, h, bpp, pass;
        bool ilace;
        int stride;
        iBYTE *data, *data_this_line;
} OUTPUT_CONTEXT;

/*write context*/
typedef struct
{
	int fd;                         /* Open file descriptor to write to */
	int inx;                        /* Bit index into buf */
	int code_size;                  /* Code size in bits */
	iBYTE buf[255+2];                /* Biggest block + overflow space */
} WRITE_CONTEXT;

typedef struct { iCWORD tail; iBYTE col; } DICT;

/*
hashvalue is calculated from a string of pixels cumulatively.
hashtable is searched starting at index hashvalue for to find the entry.
hashtable is big enough so that MAX_HASH > 4*MAX_DICT.
*/

#define MAX_HASH        17777                   /* Probably prime and > 4096 */
#define MAX_DICT        4096                    /* Dictionary size           */
#define INIT_HASH(p)    (((p)+3)*301)           /* Initial hash value        */

class IGIFConverter : public IImageConverter
{
public:
	IGIFConverter();
	~IGIFConverter();

	IGImage* createFromStream( IDataStream& fromWhere );


	void writeTo( const IGImage& fromImage,
				IDataStream& toWhere) const;
protected:
	IGImage* createFromFile( const char* fromFile);
	int writeToFile( char *toFile, iBYTE *data);
private:
	int readGif(IDataStream&, iBYTE **fImgData);
	int gif_rdata(IDataStream&, iBYTE **fImgData);
	int writeGif(int fd, iBYTE* data);
	iCWORD read_code(READ_CONTEXT *c);
	void output(iBYTE value, OUTPUT_CONTEXT *o);
	bool write_code(int code, WRITE_CONTEXT *w);
	bool flush_code(WRITE_CONTEXT *w);
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

//==============================================================================
#endif // _IGIFCONV_
