/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 51 1.9.1.3 source/albert/graph2d/igrexept.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:   IGraphicException.C
||
||  What:   IGraphicExeption class
||
||
||
||  Change history:
||
*/


#include <igrexept.hpp>
#include <grassert.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _IGREXEPT_CPP_
#include <ipagetun.h>
#endif
	
IGraphicException::IGraphicException(
	const char* errorText,
	unsigned long errorId,
	Severity severity) :
	IException(errorText, errorId, severity)
{}

IGraphicException::IGraphicException(const IGraphicException& other) :
	IException(other)
{}

IGraphicException::~IGraphicException()
{}

#include <stdio.h>

// InternalAssert are invoked when there is an internal consistancy problem
void IGraphicException::InternalAssert(bool condition, const char* str)
{
	if (!condition) {

		char sbuf[100];

		sprintf(sbuf, "Internal Assertion: %s\n", str);
		fprintf(stderr, "Internal Assertion: %s\n", str);
		IFUNCTRACE_ALL();
		ITRACE_ALL( sbuf );
		throw IGraphicException(str, IGraphicException::kInternalAssertException);
	}
}

// The calls to ASSERT_MESSAGE are just crude placeholders for now.
// In the Real Version, an enum indicating the type of failure should be
// passed in.  It may also be desirable to invoke the exception mechanism
// directly, rather than working through the ASSERT_MESSAGE macro.

// In many cases, the condition itself is enough to give a reasonable
// clue as to the cause of the problem ("index > maxPoints").  However,
// the second argument allows a message to describe the failure in case
// the condition itself ("numChickens < bigToeLength") doesn't make sense
// out of context.  Pass in "" if this isn't needed.

// I'm also assuming the run-time can generate a message indicating which
// routine (or routines/stack crawl) the failure occurred in.  Duplicating
// that information in the macro invocation is error prone and wastes string
// space.


// ParameterAsserts are invoked when an argument to an API method or function
// is out of range.  They signal developer's programming errors.

void IGraphicException::ParameterAssert(bool condition, const char* str)
{
	if (!condition) {

		char sbuf[100];

    sprintf(sbuf, "Parametre Assertion: %s\n", str);
    fprintf(stderr, "Parametre Assertion: %s\n", str);
		IFUNCTRACE_ALL();
    ITRACE_ALL( sbuf );

		throw IGraphicException(str, IGraphicException::kParameterAssertException);
	}
}


// Initalized assert is invoked on methods for objects that need to be
// initialized (e.g., a IGCurve2D created with an empty constructor must
// be initialized with <<= or = before any operations are attempted).

void IGraphicException::InitializedAssert(bool condition, const char* str)
{
	if (!condition) {

		char sbuf[100];

    sprintf(sbuf, "Initialised Assertion: %s\n", str);
    fprintf(stderr, "Initialised Assertion: %s\n", str);
		IFUNCTRACE_ALL();
    ITRACE_ALL( sbuf );

		throw IGraphicException(str, IGraphicException::kInitializedAssertException);
	}
}

// Use this assertion to mark methods that aren't implemented yet.
// Hopefully, these will be gone by 1.0...

void IGraphicException::ImplementedAssert(const char* str)
{
	char sbuf[100];

  sprintf(sbuf, "Implemented Assertion: %s\n", str);
  fprintf(stderr, "Implemented Assertion: %s\n", str);
	IFUNCTRACE_ALL();
  ITRACE_ALL( sbuf );

	throw IGraphicException(str, IGraphicException::kImplementedAssertException);
}
////////////////////////////////////////////////////////////
//
// The following are for throwing exceptions inside of
// Albert.  Exceptions should NOT be thrown using the THROW()
// macro from Exceptions.h because it may be pulled at any
// time, and it also prevents us from having control
// over whether or not exceptions are "on" in Albert.
//
////////////////////////////////////////////////////////////

// ParameterException throws the given TInvalidParameter exception
// with the given enum if the given condition is true

void IGraphicException::ParameterException(bool condition, int reason)
{
    if (condition) {
		char errorString [100];
		sprintf(errorString, "Parameter Exception: %d\n", long(reason));
    fprintf(stderr, "%s", errorString);
		IFUNCTRACE_ALL();
    ITRACE_ALL( errorString );

		throw IGraphicException(errorString, long(reason), IException::recoverable);
	}
}

// InternalException throws the given IGraphicsInternalException
// with the given enum if the given condition is true

void IGraphicException::InternalException(bool condition, int reason)
{
    if (condition) {
		char errorString [100];
		sprintf(errorString, "Internal Exception: %d\n", long(reason));
		fprintf(stderr,"%s", errorString);
		IFUNCTRACE_ALL();
		ITRACE_ALL( errorString );

		throw IGraphicException(errorString, long(reason), IException::recoverable);
	}
}

#if defined(IC_WIN)
#include <windows.h>
#elif defined(IC_PM)
#define INCL_WINERRORS
#include <os2.h>
#endif

void IGraphicException::GrafDeviceException(const char* str)
{
	char errorString [100];
	IFUNCTRACE_ALL();
#if defined(IC_WIN)
	long code = GetLastError();
	sprintf(errorString, "Exception: %s (code %d)\n", str, code);
	fprintf(stderr,"%s\n", errorString);
  ITRACE_ALL( errorString );
#elif defined(IC_PM)
	long code = WinGetLastError(0);
	sprintf(errorString, "Exception: %s (code 0x%X)\n", str, code);
	fprintf(stderr,"%s\n", errorString);
  ITRACE_ALL( errorString );
#else
	sprintf(errorString, "Exception: %s\n", str);
	fprintf(stderr,"%s\n", errorString);
  ITRACE_ALL( errorString );
#endif
	throw IGraphicException(errorString, IGraphicException::kGenericGraphicException);
}

extern "C" unsigned long ModError(int rc);

void IGraphicException::GraphicExceptionMod(int modErr)
    {
	char errorString [100];
	sprintf(errorString, "Exception: Mod Error (%d)\n", long(modErr));
	fprintf(stderr, "%s", errorString);
	IFUNCTRACE_ALL();
  ITRACE_ALL( errorString );

	throw IGraphicException(errorString, long(modErr), IException::recoverable);
    }

void IGraphicException::GraphicExceptionMod1(int modErr, const char* str)
    {
	char errorString [100];
	sprintf(errorString, "Exception: Mod Error (%d) %s\n", long(modErr), str);
	fprintf(stderr, "%s", errorString);
	IFUNCTRACE_ALL();
  ITRACE_ALL( errorString );
	throw IGraphicException(errorString, long(modErr), IException::recoverable);
    }

void IGraphicException::TransformException(int reason)
    {
	char errorString [100];
	sprintf(errorString, "Transforms Exception: %d\n", long(reason));
	fprintf(stderr, "%s", errorString);
	IFUNCTRACE_ALL();
  ITRACE_ALL( errorString );
	throw IGraphicException(errorString, long(reason), IException::recoverable);
    }
