/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 98 1.10.2.3 source/albert/graph2d/iknots.hpp, 2d, ioc.v400, 000302 
/*================
||
||	File:	Knots.h
||
||	What:	Knotvector related methods for implementing curves and surfaces
||			These are strictly private routines
||
||	Change history:
||	 9 Sep 94	jp		add moveKnot
||	 8 Feb 94	jp		Split off from prspline.hpp
||
*/

#ifndef _IKNOTS_
#define _IKNOTS_

#ifndef IC_GRAFTYPES
#include <igrtypes.hpp>
#endif

#ifndef IC_RAWARRAY
#include <irawarr.hpp>
#endif


// I've split this stuff out with the intention of creating an *Internal*
// knot vector class (not useful externally because knot vectors are
// meaningless without the number of points and order - keeping that
// information redundantly is a recipe for disaster...)

// ---------- Evaluation ----------

extern void ComputeSplineValues( // in the system (surface, 3D, etc) can
	GParametric u,				 // share the implementation.
	long brkPoint,				 // (Maybe move this out of the class def,
	unsigned long order,		 // and just make it a global extern???)
	const GParametric * const knots,
	GParametric * bvals,
	GParametric * dvals1 = 0 /*NIL*/,	 // Deriv's are optional, set to NULL to
	GParametric * dvals2 = 0 /*NIL*/ ); // avoid their computation

extern long FindInterval(		// Locate parametric Interval (Killer B's)
	const GParametric& u,
	const GParametric * const knots,
	unsigned long numPts,
	unsigned long order );

extern long FindNextInterval(	// Given a kv index, skip to the next interval)
	long curBrk,
	const GParametric * const knots,
	unsigned long numPts );

// ---------- Discontinuities ----------

extern unsigned long GetNextDiscon(								// (emacs)
	unsigned long startIndex,
	unsigned long discon,
	unsigned long fOrder,
	unsigned long fNumPoints,
	const GParametric * const fKnots );

extern long MakeBezierDiscons(									// (emacs)
	const GParametric * knots,
	unsigned long order,
	unsigned long numPoints,
	long *& discons );

extern void CreateKnotsFromDiscons(								// (emacs)
	GParametric * knots,
	unsigned long order,
	unsigned long numPoints,
	long * discons );

extern bool IsKnotvectorConsistent(							// (emacs)
	const GParametric *Knots,
	unsigned long numberKnots,
	unsigned long order );

// ---------- Refinement ----------

extern void ComputeAlphaMatrix(									// (emacs)
    const GParametric * oldKnots,
    const GParametric * newKnots,
    unsigned long numPts,
    unsigned long numNew,
    unsigned long order,
    GParametric **& alpha );
	
extern void RefineKVUniform(									// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	const GParametric* fKnots,
	IRawArray<GParametric>& newKnots );

extern void RefineKVToBezier(									// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	const GParametric* fKnots,
	IRawArray<GParametric>& newKnots );

extern void RefineKVToPinned(									// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	const IRawArray<GParametric>& fKnots,
	IRawArray<GParametric>& newKnots );

// ---------- Sets/Tests ----------

extern void SetBezierKnots(										// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	IRawArray<GParametric>& fKnots );
	
extern void SetPinnedKnots(										// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	IRawArray<GParametric>& fKnots );

extern bool IsBezierKnots(									// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	const IRawArray<GParametric>& fKnots );
	
extern bool IsPinnedKnots(									// (emacs)
	unsigned long fOrder,
	unsigned long fNumPoints,
	const IRawArray<GParametric>& fKnots );

extern void MoveAndSortKnot(									// (emacs)
    unsigned long index,
	GParametric u,
	unsigned long fOrder,
	unsigned long fNumPoints,
	IRawArray<GParametric>& fKnots );

extern void privateMergeKnots(	// used by IGCurve2D/3D, TGSurface3D MergeKnots()
	const GParametric *SrcKnots, // incoming knots to be merged
	unsigned long SrcNumberKnots, // number of new knots
	unsigned long SrcOrder,		  // order of curve of incoming knotvector
	const GParametric *DstKnots,  // original knot vector (fKnotvector)
	unsigned long DstNumberPoints, // original nr of points (fNumPoints)
	unsigned long DstOrder,		   // fOrder
	IRawArray<GParametric> &resultKnots		// merged knot vector
	);
	
#endif // _IKNOTS_
