/* NOSHIP */
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 13 1.11.2.4 source/albert/graph2d/inumeric.hpp, 2d, ioc.v400, 000302 

#ifndef _INUMERIC_
#define _INUMERIC_

#include <complex.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>

#include <ibase.hpp>


#ifdef TYPE_PORTABILITY_TEST

typedef int float_t;
typedef int double_t;

#else

#ifndef _FLOAT_T
#define _FLOAT_T
     typedef double float_t;
#endif

#ifndef _DOUBLE_T
#define _DOUBLE_T
     typedef double double_t;
#endif

#endif

#ifdef IC_MOTIF
#define _INFINITY DBL_INFINITY
#endif //MOTIF

#define kInfinity double(_INFINITY)

inline bool   IsInfinite (double x )     { return x == _INFINITY; }
inline bool   IsFinite ( double x )      { return x != _INFINITY; }
inline bool   IsNaN(double x)            { return (x != x); }
inline double Log( double x)             { return log(x); }
inline double Log10( double x)           { return log10(x); }
inline double Log2( double x)            { return (log10(x)/log10((double)2)); } // for ReduceCurveImp

#define kPi double(M_PI)
#define kDegreesToRadians double(0.01745329251994329576924L)
#define kRadiansToDegrees double(1 / kDegreesToRadians)

inline double ArcTan(double x)           { return atan(x); }
inline double ArcTan(double y, double x) { return atan2(y, x); }
inline double ArcCos(double x)           { return acos(x); }
inline double ArcSin(double x)           { return asin(x); }
inline double Cos(double x)              { return cos(x); }
inline double Sin(double x)              { return sin(x); }
inline double Abs(double x)              { return fabs(x); }


inline double Ceiling(double x)          { return ceil(x); }
inline double Floor(double x)            { return floor(x);} // used in graftype.hpp

inline double Max (double x,  double y)  { return ( (y <= x) ? x : y ); }
inline double Min (double x,  double y)  { return ( (y <= x) ? y : x ); }

inline long Max (long x,  long y)        { return ( (y <= x) ? x : y ); }
inline long Min (long x,  long y)        { return ( (y <= x) ? y : x ); }

inline double Square(double x)           { return (x * x); }
inline double Sqrt(double x)             { return sqrt(x); }

inline double SquareRoot( double x)      { return sqrt(x); }
inline double Power(double b, double e)  { return pow(b,e); }
inline double Remainder(double x, double y )   { return fmod ( x, y ); }
// AR inline double Round( double x )          {return (double) ( x + (double) 0.5 );}
inline double Round( double x )          {return Floor( x + (double) 0.5 );} // V100


#endif // _INUMERIC_
