// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
//
// Revision: 94 1.2 source/albert/graph2d/pmvtxeng.cpp, 2d, ioc.v400, 000302 

#ifndef INCL_WIN
#define INCL_WIN
#include <winemul.h>
#undef INCL_WIN
#endif
#include <pmvtxeng.hpp>

//---------------
// IPmPoint class definitions
//---------------

IPmPoint::IPmPoint(LONG x, LONG y) 
{
	fPoint.x = x;
	fPoint.y = y;
}

IPmPoint::IPmPoint() {}
IPmPoint::~IPmPoint() {}
	
IPmPoint::IPmPoint(const IPmPoint& src) 
{
	fPoint.x = src.fPoint.x;
	fPoint.y = src.fPoint.y;
}

IPmPoint::operator POINTL() const 
{
	return fPoint;
}

IDataStream& IPmPoint::operator>>= (IDataStream& toWhere) const 
{
	return toWhere;
}
IDataStream& IPmPoint::operator<<= (IDataStream& fromWhere) 
{
	return fromWhere;
}

IPmPoint& IPmPoint::operator=(const IPmPoint& src)
{
	fPoint.x = src.fPoint.x;
	fPoint.y = src.fPoint.y;
	return *this;
}
bool IPmPoint::operator==(const IPmPoint& src) const 
{
	return (fPoint.x == src.fPoint.x && fPoint.y == src.fPoint.y);
}
bool IPmPoint::operator!=(const IPmPoint& src) const 
{
	return (!operator==(src));
}

//---------------
// IPmPoly class definitions
//---------------

IPmPoly::IPmPoly(ULONG numPoints, long index) 
{
	fnPoints = numPoints;
	fIndex = index;
}
IPmPoly::IPmPoly() {}
IPmPoly::~IPmPoly() {}
	
IPmPoly::IPmPoly(const IPmPoly& src) {
	fnPoints = src.fnPoints;
	fIndex = src.fIndex;
}

IDataStream& IPmPoly::operator>>= (IDataStream& toWhere) const 
{
	return toWhere;
}
IDataStream& IPmPoly::operator<<= (IDataStream& fromWhere) 
{
	return fromWhere;
}

IPmPoly& IPmPoly::operator=(const IPmPoly& src) {
	fnPoints = src.fnPoints;
	fIndex = src.fIndex;
	return *this;
}
bool IPmPoly::operator==(const IPmPoly& src) const {
	return (fnPoints == src.fnPoints && fIndex == src.fIndex);
}
bool IPmPoly::operator!=(const IPmPoly& src) const {
	return (!operator==(src));
}

//---------------
// IPmPathVertexEngine class definitions
//---------------

IPmPathVertexEngine::IPmPathVertexEngine(IPresSpaceHandle hps) :
	fPoints(),
	fPolys(),
	fNumPoints(0),
	fHPS(hps)
{}

IPmPathVertexEngine::~IPmPathVertexEngine() {}

void IPmPathVertexEngine::SetCAGNode(ICAGUnary*) {}

void IPmPathVertexEngine::SetClosed(bool) {}

void IPmPathVertexEngine::Reset() {
	fPoints.resize(0);
	fPolys.resize(0);
	fNumPoints = 0;
}

void IPmPathVertexEngine::AddVertex(const IGPoint2D& vertex, bool joint)
{
	IPmPoint thePoint(
		IGpiDevice::GCoordinateToInt(vertex.fX),
		IGpiDevice::GCoordinateToInt(vertex.fY));
	fPoints.append(thePoint);	
}

void IPmPathVertexEngine::End() 
{
	// fPoints.append(fPoints[fNumPoints]);	// add the starting point to explicitly close the loop...
	unsigned long current = fPoints.numberOfValues();
	ULONG nPoints = current - fNumPoints;
	IPmPoly thePoly(nPoints, fNumPoints);
	if (nPoints > 2) {	// 3 if explicitly closed...
		fPolys.append(thePoly);
	}	
	fNumPoints = current;
}

void IPmPathVertexEngine::Render(const ICAGRoot&) 
{
	GpiBeginPath(fHPS, 1);
	long n = fPolys.numberOfValues();
	for (long i = 0; i < n; ++i) {
		PPOINTL pts = (PPOINTL)&fPoints[fPolys[i].fIndex];
		GpiMove(fHPS, pts);
		GpiPolyLine(fHPS, fPolys[i].fnPoints, pts);
		GpiCloseFigure(fHPS);
	}
	GpiEndPath(fHPS);
}


//---------------
// IPmClipPathVertextEngine class definitions
//---------------

IPmClipPathVertextEngine::IPmClipPathVertextEngine(IPresSpaceHandle hps) 
: IPmPathVertexEngine(hps) 
{}

IPmClipPathVertextEngine::~IPmClipPathVertextEngine() 
{}

void IPmClipPathVertextEngine::Render(const ICAGRoot& node) 
{
	GpiSetClipPath(fHPS, 0L, SCP_RESET );
	IPmPathVertexEngine::Render(node);
	// call the path to region function ...      
  GpiSetClipPath(fHPS, 1L, SCP_ALTERNATE|SCP_AND|SCP_INCL);
}	

//---------------
// IPmRenderPathVertextEngine class definitions
//---------------

IPmRenderPathVertextEngine::IPmRenderPathVertextEngine(IPresSpaceHandle hps) 
 : IPmPathVertexEngine(hps) 
{}

IPmRenderPathVertextEngine::~IPmRenderPathVertextEngine() 
{}

void IPmRenderPathVertextEngine::Render(const ICAGRoot& node) 
{
	IPmPathVertexEngine::Render(node);
}

int IPmRenderPathVertextEngine::FillArea(IPresSpaceHandle hps) 
{
	return FillPath(hps);
}

int IPmRenderPathVertextEngine::StrokeArea(IPresSpaceHandle hps) 
{
	return StrokePath(hps);
}


