/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 28 1.10.1.3 source/albert/graph2d/prratopr.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:       prRationalOperators.C
||
||  What:       Internally used operators for TGRPoints
||
||  Change History:
||
*/

// see prratopr.hpp for comments

#include <prratopr.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _PRRATOPR_CPP_
#include <ipagetun.h>
#endif

//================== 2D ===================//

IGRPoint2D& operator*=( IGRPoint2D& dest, const IGRPoint2D& src )
{
    dest.fX *= src.fX;
    dest.fY *= src.fY;
    dest.fW *= src.fW;
    return dest;
}

IGRPoint2D& operator*=( IGRPoint2D& dest, GCoordinate src )
{
    dest.fX *= src;
    dest.fY *= src;
    dest.fW *= src;
    return dest;
}

IGRPoint2D& operator/=( IGRPoint2D& dest, const IGRPoint2D& src )
{
    dest.fX /= src.fX;
    dest.fY /= src.fY;
    dest.fW /= src.fW;
    return dest;
}

IGRPoint2D& operator/=( IGRPoint2D& dest, GCoordinate src )
{
    dest.fX /= src;
    dest.fY /= src;
    dest.fW /= src;
    return dest;
}

IGRPoint2D& operator+=( IGRPoint2D& dest, const IGRPoint2D& src )
{
    dest.fX += src.fX;
    dest.fY += src.fY;
    dest.fW += src.fW;
    return dest;
}

IGRPoint2D& operator-=( IGRPoint2D& dest, const IGRPoint2D& src )
{
    dest.fX -= src.fX;
    dest.fY -= src.fY;
    dest.fW -= src.fW;
    return dest;
}

IGRPoint2D operator+( const IGRPoint2D& left, const IGRPoint2D& right )
{
    return IGRPoint2D( left.fX + right.fX,
                     left.fY + right.fY,
                     left.fW + right.fW );
}

IGRPoint2D operator-( const IGRPoint2D& left, const IGRPoint2D& right )
{
    return IGRPoint2D( left.fX - right.fX,
                     left.fY - right.fY,
                     left.fW - right.fW );
}

IGRPoint2D operator*( const IGRPoint2D& left, const IGRPoint2D& right )
{
    return IGRPoint2D( left.fX * right.fX,
                     left.fY * right.fY,
                     left.fW * right.fW );
}

IGRPoint2D operator/( const IGRPoint2D& left, const IGRPoint2D& right )
{
    return IGRPoint2D( left.fX / right.fX,
                     left.fY / right.fY,
                     left.fW / right.fW );
}

IGRPoint2D operator*( const IGRPoint2D& left, GCoordinate right )
{
    return IGRPoint2D( left.fX * right,
                     left.fY * right,
                     left.fW * right );
}

IGRPoint2D operator*( GCoordinate right, const IGRPoint2D& left )
{
    return IGRPoint2D( left.fX * right,
                     left.fY * right,
                     left.fW * right );
}

IGRPoint2D operator/( const IGRPoint2D& left, GCoordinate right )
{
    return IGRPoint2D( left.fX / right,
                     left.fY / right,
                     left.fW / right );
}

void SetToZero( IGRPoint2D& p )
{
    p.fX = 0.0; p.fY = 0.0; p.fW = 0.0;
}
