/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 30 1.12.1.3 source/albert/graph2d/prspline.cpp, 2d, ioc.v400, 000302 
/*================
||
||  File:   prSplineRoutines.c
||
||  What:   Useful routines for IGCurve2D and other objects
||          These are strictly private routines
||
||  Change history:
||
*/

#include <ipt2darr.hpp>
#include <prspline.hpp>
#include <igbase2d.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
#define _PRSPLINE_CPP_
#include <ipagetun.h>
#endif

// Combitorial function.  This belongs in a math library someplace
// also include the math-orphans at the end of igbase2d.hpp

long
comb( long d, long i )
{
    long j, a;

    a = 1;
    for (j = i + 1; j <= d; j++)
        a *= j;
    for (j = 1; j <= d - i; j++)
        a /= j;
    return( a );
}


IGRect2D
GetSegmentBounds( long brkPoint, long order,
                  const IGRPoint2DArray& pts, bool ratFlag )
{
    long j;
    IGRect2D segmentBounds;
    // Compute the bounds of the current segment

    segmentBounds = ratFlag ? IGRect2D( pts[brkPoint].divW(),
                                      pts[brkPoint - 1].divW() )
        : IGRect2D( pts[brkPoint].dropW(), pts[brkPoint -1].dropW());
    segmentBounds.orderPoints();
    for (j = brkPoint - 2; j >= (long)(brkPoint - (order - 1)); j--)
    {
        segmentBounds.extendTo( ratFlag ? pts[j].divW() : pts[j].dropW() );
    }

    return segmentBounds;
}

// Duplicate a tiny bit of code here rather than dragging in all the 3D
// transform stuff.

void CubicBasisMatrixMul( const BasisMatrix a,
                            const BasisMatrix b, BasisMatrix dest )
{
    long i, j, k;
    for (i = 0; i < 4; i++)
        for (j = 0; j < 4; j++)
        {
            dest[i * 4 + j] = 0.0;
            for (k = 0; k < 4; k++)
                dest[i * 4 + j] += a[i * 4 + k] * b[k * 4 + j];
        }
}
