// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 00 1.6.1.4 source/albert/graph2d/vrtxeng.hpp, 2d, ioc.v400, 000302 

//vrtxeng.hpp

#ifndef _VRTXENG_
#define _VRTXENG_

#include <igrexept.hpp>
#include <imstrmbl.hpp>
#include <ipt2darr.hpp>
#include <irawarr.hpp>
#include <igbase2d.hpp>
#include <igloop2d.hpp>

#ifdef __OS2__
#define NULL 0
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ICAGRoot;
class ICAGUnary;

#pragma enum(4)
#pragma pack(push,4)

class IVertexEngine {
public:
        virtual ~IVertexEngine();

        virtual void SetCAGNode(ICAGUnary*) = 0;        // set the CAG node for

        virtual void SetClosed(bool) = 0;       // true => closed list(s) of vertices

        virtual void AddVertex(
                const IGPoint2D& vertex,        // process this vertex
                bool joint = true) = 0;         // apply a joint at this vertex?

        virtual void End() = 0;         // end this list of vertices

        virtual void Render(const ICAGRoot&) = 0;       //process the vertices and reset

        virtual void Reset() = 0;       // reset the vertex engine

protected:
        IVertexEngine();
        IVertexEngine(const IVertexEngine&);

        IVertexEngine& operator=(const IVertexEngine&);
};

class IPolygonVertexEngine : public IVertexEngine {
        IGLoop2D fLoop;
        bool fInitialised;
        IGRPoint2DArray fPoints;
        IRawArray<GParametric> fKnots;
        double fK;
        int fStart;

        void addKnot(GCoordinate discountinuity);

public:
        IPolygonVertexEngine();
        ~IPolygonVertexEngine();
        virtual void SetCAGNode(ICAGUnary* node);
        virtual void SetClosed(bool closed);
        void Reset();
        void AddVertex(const IGPoint2D& vertex, bool joint = true);
        void End();
        void Render(const ICAGRoot& root);
        IGLoop2D& loop();
};

//OutLineMakerVertexEngine internal classes
class ICAGPointArray  // container for an input point array and CAG node
{
public:
        ICAGPointArray(ICAGUnary* node);
        ~ICAGPointArray();

        ICAGUnary* fCAGNode;
        IGPoint2DArray  fPoints;
};

class ICAGPointArrayPtr
{
        ICAGPointArray* fValue;
public:
        ICAGPointArrayPtr(ICAGPointArray* pArrayList);
        ICAGPointArrayPtr(const ICAGPointArrayPtr& src);
        ~ICAGPointArrayPtr();

        ICAGPointArrayPtr& operator=(const ICAGPointArrayPtr& Src );
        IDataStream& operator>>=(IDataStream& to) const;
        IDataStream& operator<<=(IDataStream& from);
        operator ICAGPointArray*();
};

inline IDataStream& ICAGPointArrayPtr::operator>>=(IDataStream& to) const
{
        return to;
}

inline IDataStream& ICAGPointArrayPtr::operator<<=(IDataStream& from)
{
        return from;
}

inline ICAGPointArrayPtr::operator ICAGPointArray*()
{
        return this->fValue;
}


class IOutputVertexList   // container for an output point array
{
        public:
        IOutputVertexList(bool direction);
        ~IOutputVertexList();

        IGPoint2DArray    fPoints;
        bool        fForward;   // direction of vertices
        bool        fProcessed;
        IOutputVertexList*  fNextEdgeList;
};

class IOutputVertexListPtr
{
        IOutputVertexList* fValue;
public:
        IOutputVertexListPtr(IOutputVertexList* pVertexList);
        IOutputVertexListPtr(const IOutputVertexListPtr& );
        ~IOutputVertexListPtr();

        IDataStream& operator>>=(IDataStream& to) const;
        IDataStream& operator<<=(IDataStream& from);
        operator IOutputVertexList*();
        IOutputVertexListPtr& operator=(const IOutputVertexListPtr& Src );
};

inline IDataStream& IOutputVertexListPtr::operator>>=(IDataStream& to) const
{
        return to;
}

inline IDataStream& IOutputVertexListPtr::operator<<=(IDataStream& from)
{
        return from;
}

inline IOutputVertexListPtr::operator IOutputVertexList*()
{
        return fValue;
}


class IPolyEdge   // polygonal edge: two points and a CAG node
{
public:
        IPolyEdge();
        IPolyEdge(IGPoint2D* p0, IGPoint2D* p1, ICAGUnary* node,
                                                                unsigned long index);
        ~IPolyEdge();

        void Set(IGPoint2D* p0, IGPoint2D* p1, ICAGUnary* node,
                                                                 unsigned long index);

        void Transition();

        IGPoint2D* fTop;
        IGPoint2D* fBottom;
        GCoordinate fSlope;             // the slope
        ICAGUnary* fCAGNode;    // parent CAG node
        unsigned long fIndex;   // unique index
        signed char fSign;              // +1 or -1
        bool fRelevant;                 // belongs to the current y-event?

        IOutputVertexList*  fOutputList;        // output vertex list this edge is
                                                                                // contributing to;     NIL if none
};

inline IPolyEdge::IPolyEdge()
{}

inline IPolyEdge::IPolyEdge(IGPoint2D* p0, IGPoint2D* p1, ICAGUnary* node, unsigned long index)
{
        Set(p0, p1, node, index);
}

inline IPolyEdge::~IPolyEdge()
{}


class IPolyYEvent
{
public:
        IPolyYEvent();
        IPolyYEvent(IPolyEdge* edge0, IPolyEdge* edge1, const IGPoint2D& point);
        ~IPolyYEvent();

        void Set(IPolyEdge* edge0, IPolyEdge* edge1, const IGPoint2D& point);

        // For sorting
        bool operator<(const IPolyYEvent&) const;

        // For use by IRawArray
        bool operator==(const IPolyYEvent&) const;
        IDataStream& operator>>=(IDataStream&) const;
        IDataStream& operator<<=(IDataStream&);

        IPolyEdge* fEdge[2];    // the two edges that intersect
        IGPoint2D fPoint;       // point of intersection
};

inline IPolyYEvent::IPolyYEvent()
{}
inline IPolyYEvent::IPolyYEvent(IPolyEdge* edge0, IPolyEdge* edge1, const IGPoint2D& point)
{
        Set(edge0, edge1, point);
}

inline IPolyYEvent::~IPolyYEvent()
{}

inline IDataStream& IPolyYEvent::operator>>=(IDataStream& to) const
{
        return to;
}

inline IDataStream& IPolyYEvent::operator<<=(IDataStream& from)
{
        return from;
}

class IOutlineMakerVertexEngine : public IVertexEngine {
public:
        IOutlineMakerVertexEngine(IVertexEngine* outputVertexEngine = NULL);

        virtual ~IOutlineMakerVertexEngine();

        // === IVertexEngine overrides ===
        virtual void SetCAGNode(ICAGUnary*);
        virtual void SetClosed(bool closed);
        virtual void AddVertex(const IGPoint2D&, bool joint = true);
        virtual void End();
        virtual void Render(const ICAGRoot&);
        virtual void Reset();

        void SetVertexEngine(IVertexEngine*);
        IVertexEngine* GetVertexEngine();

private:
        IRawArray<ICAGPointArrayPtr> fVertexArrays;     // list of vertex arrays
        ICAGPointArray* fCurrentArray;  // the array new points go into

        bool fClosed;   // closed loop
        ICAGUnary* fCAGNode;    // the current CAG node
        IVertexEngine* fVertexEngine;   // the output vertex engine

        void SendInputVerticesToOutput(const ICAGRoot&);

        static void YSortEdges(IPolyEdge** edges, unsigned long numEdges);
        static void SortYEvents(IPolyYEvent** edges, unsigned long numYEvents);
        static void MoveEdge(IPolyEdge** thisEdge, IPolyEdge**& edgeListStart);

        // Explicitly not implemented
        IOutlineMakerVertexEngine(const IOutlineMakerVertexEngine&);
        IOutlineMakerVertexEngine& operator=(const IOutlineMakerVertexEngine&);

        class IEdgeComparator
        {
        public:
                IEdgeComparator(unsigned long numberOfEdges);
                ~IEdgeComparator();

                void SetPoint(const IGPoint2D&);
                bool LessThan(const IPolyEdge&, const IPolyEdge&);

                int FindIntersection(const IPolyEdge&, const IPolyEdge&, IGPoint2D&, IGPoint2D&);

        private:
                IGPoint2D fPoint;

                enum EEdgeComparison { kIntersecting, kCoincident, kDisjoint };

                EEdgeComparison FindIntersection2(
                        const IPolyEdge& edge0,
                        const IPolyEdge& edge1,
                        IGPoint2D& p0,
                        IGPoint2D& p1,
                        bool& edge0Less);

                static void ComputeDistances(
                        const IPolyEdge&,
                        const IGPoint2D& p0,
                        const IGPoint2D& p1,
                        GCoordinate& d0,
                        GCoordinate& d1);

                        // computes y value of intersection point
                static GCoordinate ComputeY(const IPolyEdge&, const IPolyEdge&);

                        // computes x at y
                static GCoordinate ComputeX(GCoordinate y, const IPolyEdge&);
        };

        class IEdgeTransition
        {
        public:
                IEdgeTransition();
                IEdgeTransition(IPolyEdge* edge, bool transition);
                ~IEdgeTransition();

                // For use by IRawArray
                bool operator==(const IEdgeTransition&) const;
                IDataStream& operator>>=(IDataStream&) const;
                IDataStream& operator<<=(IDataStream&);

                IPolyEdge*  fEdge;
                bool fTransition;       // whether there is an in/out transition at this edge
        };

        class IOutputListDeleter  // responsible for deleting an output list
        {
        public:
                IOutputListDeleter(IRawArray<IOutputVertexListPtr>*);
                ~IOutputListDeleter();
        private:
                IRawArray<IOutputVertexListPtr>* fOutputLists;
        };

        class IOutlineMakerException : public IGraphicException
        {
        public:
                IOutlineMakerException();
                IOutlineMakerException(const IOutlineMakerException&);
                virtual ~IOutlineMakerException();
        };
};      //end of class IOutlineMakerVertexEngine


inline IOutlineMakerVertexEngine::IEdgeTransition::IEdgeTransition()
{}

inline IOutlineMakerVertexEngine::IEdgeTransition::IEdgeTransition(
        IPolyEdge* edge,
        bool transition)
{
        fEdge = edge;
        fTransition = transition;
}

inline IOutlineMakerVertexEngine::IEdgeTransition::~IEdgeTransition()
{}

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _VRTXENG_
