// NOSHIP ...
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 08 1.25.1.15 source/albert/graph2d/xdevice.hpp, 2d, ioc.v400, 000302 
/*================
||
||  File:       XDevice.h
||
||  What:       Routines which support the graphics device on XWindows platform
||
||
||
||  Change History:
||              20 Nov 96   aruna           -   Original
||
*/


#ifndef _XDEVICE_
#define _XDEVICE_

#include <ilanglvl.hpp>

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <Xm/Xm.h>

#include <igrtypes.hpp>
#include <grafdevs.hpp>
#include <grstate.hpp>
#include <ibundles.hpp>
#include <iexgrprt.hpp>                 // for EDeviceMappingMode

#include <ipoint.hpp>
#include <irect.hpp>

#include <i2dghand.hpp>
#include <iexcept.hpp>

#include <ixdc.hpp>

#include <deleter.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IGPoint2D;
class IGRect2D;
class IGLine2D;
class IGRect2D;
class IGPoint2D;
class IGEllipse2D;
class IGPolygon2D;
class IGPolyline2D;
class IGCurve2D;
class IGLoop2D;
class IGImage;
class IGTextRun;
class IColorTransferMode;
class IMaskPattern;
class IColorMap;

class IGrafState;
class IBaseColor;
class IPaint;
class ITextStyleSet;

class IRegionGrafExtractor;
class IXFontContext;

#pragma enum(4)
#pragma pack(push,4)

/*================================================================
||
||      IXDevice
||
*/

class IXDevice: public IGrafDevice {
public:
        IXDevice(const IPresSpaceHandle& deviceContext);
        IXDevice(const IPresSpaceHandle& deviceContext,
                                EDeviceMappingMode mappingMode,
                                bool baseRootFlag = true);

        IXDevice(const IPresSpaceHandle& deviceContext,
                const IGRect2D& clipRect,
                const IGrafMatrix& resolution = IGrafMatrix::identity());

        virtual ~IXDevice();

        virtual void renderLine(const IGLine2D& geometry, const IGrafState& grafState);
        virtual void renderPolyline(const IGPolyline2D& geometry, const IGrafState& grafState);
        virtual void renderCurve(const IGCurve2D& geometry, const IGrafState& grafState);
        virtual void renderRect(const IGRect2D& geometry, const IGrafState& grafState);
        virtual void renderEllipse(const IGEllipse2D& geometry, const IGrafState& grafState);
        virtual void renderPolygon(const IGPolygon2D& geometry, const IGrafState& grafState);
        virtual void renderLoop(const IGLoop2D& geometry, const IGrafState& grafState);
        virtual void renderArea(const IGArea& geometry, const IGrafState& grafState);

        virtual void renderImage(
                const IGImage& image,
                const IGRect2D& source,
                const IGrafState& grafState);

        virtual void renderMaskedImage(
                const IGImage& image,
                const IGImage& mask,
                const IGRect2D& source,
                const IGrafState& grafState);

        virtual IGPoint2D renderTextRun(const IGTextRun& geometry, const IGrafState& grafState);

        virtual const IGrafMatrix& deviceResolution(
                const IGRect2D& areaOfInterest = IGRect2D::infiniteRect()) const;

        virtual const IPresSpaceHandle& deviceContext() const;
        virtual void adoptDeviceContext(const IPresSpaceHandle& deviceContext);
        virtual IPresSpaceHandle orphanDeviceContext();

#if !IC_RTTI_ENABLED
        GrafDeviceTypeDeclarationsMacro(IXDevice);
#endif // IC_RTTI_ENABLED

        virtual void flush();
private:
friend class IGTextRun;
        static void determineTextMetrics(IGTextRun& textRun, IXDevice*, const IGrafState*);

        static int GCoordinateToInt(GCoordinate value);

friend class ConicMaker;
friend class LineDiscretizer;
friend class LoopDiscretizer;
        void UpdateTransferMode (const IGrafState& state);
        void UpdatePen(const IGrafState& state);
        void UpdateBrush(const IGrafState& state);
        void IGPoint2DToPOINTL(const IGPoint2D &p, XPoint &ptl);
        IPresSpaceHandle fHDC;
        IXDC *fIXDC;

protected:
        // this is disabled ... it can only be called by disabled functions to assist compile...
        IXDevice();

private:
        void initialise(int mmode = 0, const IGRect2D* clipRect = 0 /*NIL*/);

        unsigned long lookupColormap(const IBaseColor& color);

        bool FrameEqualFillPaintModes(const IGrafState& state) const;

        int SetClipArea(const IGrafState& state);
        void DrawArc(const IGCurve2D& curve, const IGrafState& grafState, bool isLoop);

        int imageTransferMode(const int);

        unsigned long imageSamplingControl(XImage* source, XImage* dest,
          IGPoint2D srcPoint, int destX, int destY, const int);

        void queryColormap();
        unsigned long ConstructClrPixel(
                                    unsigned char red,
                                    unsigned char green,
                                    unsigned char blue);
        void imageTransferControl(unsigned long& pixelVal,
                                    const int);


        void internalRenderImage(
                const IGRect2D& rect,
                long rasterOperation,
                IGImage* srcIGImage,
                IGImage* srcIGMatte);

        void SetWinWorldTransform(const IGrafMatrix* grafmatrix);

        // data:
        bool fOwnHDC;
        IPresSpaceHandle fUsedOwnHDC;

        IPresSpaceHandle fDeviceHandle ;

        IAttributeState::EDrawOperation fDrawOperation;
        IGrafMatrix* fResolution;
        IGrafMatrix  *resolutionMatrix();
        bool fBaseRootDevice; // true if the device is created by an IBaseRootGrafPort, otherwise, false.

        void setupTransforms(const IGrafState& state);
        IGrafMatrix fMatrix;                            // T(geometry) where T(model) = T(geometry) . T(view)
        const IGrafMatrix* fFrameMatrix;        // used for transform the frame geometry
        const IGrafMatrix* fModelMatrix;        // used for transform the fill geometry
        const IGrafMatrix* fViewMatrix;         // used for set to GDI WorldTransform
        const IGrafMatrix* fPenMatrix;          // used for calculate the pen width

        IGrafMatrix fMatrixCache;
        bool fMatrixCacheUsed;

        bool fClipAreaSet;
        IStateChange fStateChange; // used for storing the delta state for each render call

        IColorMap* fColors; //Pointer to the colormap for fast lookup.

        static bool fInitialized;
        static GCoordinate gMaxCoord;
        static GCoordinate gMinCoord;

        static enum EPlatform { kWin95, kWinNT, kOS2, kUnix, kUndefined } fPlatform;

        friend class IRegionGrafExtractor;

        friend class IXFontContext;

        IXFontContext* fFontCache;
        IXFontContext& makeFontContext(
        const IGTextRun& theText,
        const IGrafMatrix* modelMatrix,
        const IGrafState* state);

        // used for text colour caching:
        IBaseColor fTextColour;

        // disabled functions:
        IXDevice(const IXDevice&);
        IXDevice& operator=(const IXDevice& Src);

        XImage* convert24To8Image(Display *, Window , const IGImage &);
        unsigned long transparencyIndex(Display *, Window , const IGImage &);

};

// --------------------------------------------------------------------------------
// this is the arbitrator object for making the right device for the root grafport on
// the right platforms... may be this should be in a independent header file

class IGrafDeviceMaker {
public:
        static IGrafDevice* makeDevice(
                const IPresSpaceHandle& deviceContext,
                const IGrafPort::EGrafPortType = IGrafPort::kIBaseRootGrafPort,
                EDeviceMappingMode mappingMode = kPixel);
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _XDEVICE_

