// Revision: 74 1.4.1.2 source/core/base/iasynty.cpp, notification, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iasynty.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iasynty.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iasynty.hpp>

#ifdef IC_PAGETUNE
    #define _IASYNTY_CPP_
    #include <ipagetun.h>
#endif

//Class INotificationProcessor
//------------------------------------------------------------
INotificationProcessor::INotificationProcessor () :
  IRequestQueue(),
  fThreadId (INonGUIThread::currentId()),
  fUsageCount(0),
  fIsRunning (false),
  fRequestHandler (0)
{
	fRequestHandler = new IRequestHandler(this);
}

INotificationProcessor::INotificationProcessor (IThreadId tid) :
  IRequestQueue(tid),
  fThreadId (tid),
  fUsageCount(0),
  fIsRunning (false),
  fRequestHandler (0)
{
	fRequestHandler = new IRequestHandler(this);
}

INotificationProcessor::~INotificationProcessor ()
{
  Close();
  if (fRequestHandler)
	delete fRequestHandler;
}


const IThreadId
   &INotificationProcessor::threadId() const
{
  return fThreadId;
}

unsigned long
INotificationProcessor::addRef()
{
  fUsageCount++;
#ifdef IC_DEVELOP
  printf ("added reference to INotificationProcessor...%d\n", fUsageCount);
#endif
  return  fUsageCount;
}

unsigned long
INotificationProcessor::refCount ( )
{
  return  fUsageCount;
}

unsigned long
INotificationProcessor::removeRef ( )
{
  fUsageCount--;
#ifdef IC_DEVELOP
  printf ("removed reference to INotificationProcessor...%d\n",fUsageCount);
#endif
  return fUsageCount;
}

bool
INotificationProcessor::isRunning () const
{
  return fIsRunning;
}

INotificationProcessor& INotificationProcessor::setIsRunning ( bool running )
{
  fIsRunning = running;
  return *this;
}



//Class INotificationRequest
INotificationRequest::INotificationRequest(IObserver& targetObserver,
										   const INotificationEvent& anEvent) :
  fObserver(targetObserver),
  //  fEvent (new INotificationEvent(anEvent)) //Copy this for async?
  fEvent (anEvent.clone())  //polymorphic copy this for async
{
}


INotificationRequest::~INotificationRequest()
{
  delete fEvent; fEvent = 0;
}

void
INotificationRequest::Do()
{
    fObserver.dispatchNotificationEvent(*fEvent);
}


IRequestHandler*
INotificationProcessor::handlerAlias ()
{
  return fRequestHandler;
}
