/*----------------------------------------------------------------------------*/
/* FILE NAME: ibuffer1.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of the IBuffer testing             */
/*   functions.                                                               */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
#ifdef __MVS__
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS400__
/*   ????-???                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS2__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __WINDOWS__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _AIX
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"IBUFFER1")
    #pragma csect(static,"SBUFFER1")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

extern "C"
  {
  #include <ctype.h>
  }

#include <ibuffer.hpp>

#include <istrtest.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUFFER1_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBuffer::isAlnum                                                             |
| IBuffer::isAlpha                                                             |
| IBuffer::isAscii                                                             |
| IBuffer::isCntrl                                                             |
| IBuffer::isDigits                                                            |
| IBuffer::isGraphics                                                          |
| IBuffer::isHexDigits                                                         |
| IBuffer::isLowerCase                                                         |
| IBuffer::isPrintable                                                         |
| IBuffer::isPunctuation                                                       |
| IBuffer::isUpperCase                                                         |
| IBuffer::isWhitespace                                                        |
|                                                                              |
| Each of these functions is implemented by constructing (implicitly)          |
| an IStringTest with the corresponding standard library function.             |
| This IStringTest is then applied to the buffer contents using the            |
| indexOfAnyBut member function (which returns the index of the first          |
| character that fails the test).  This "index" is then negated to             |
| achieve the desired result:                                                  |
|   0  (no characters failed the test)  -> "true"                              |
|   >0 (some character failed the test) -> "false"                             |
|                                                                              |
| Notes:                                                                       |
|   1. This simplistic implementation fails NLS/DBCS criteria, the             |
|      determination of alpha/lowercase/etc. must be made on a per             |
|      code page basis.                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: isAlphanumeric ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isalnum ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isAlphabetic                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: isAlphabetic ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isalpha ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isASCII                                                             |
------------------------------------------------------------------------------*/
bool IBuffer :: isASCII ( ) const
  {
  unsigned    i = this->length();
  const unsigned char *p = (unsigned char *) this->contents();

  if (!i)  return false;      // return false for null string

  while ( i-- )
    if ( *p++ > 127 )
      return false;
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::isControl                                                           |
------------------------------------------------------------------------------*/
bool IBuffer :: isControl ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( iscntrl ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isDigits                                                            |
------------------------------------------------------------------------------*/
bool IBuffer :: isDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isdigit ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isGraphics                                                          |
------------------------------------------------------------------------------*/
bool IBuffer :: isGraphics ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isgraph ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isHexDigits                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isHexDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isxdigit ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isLowerCase                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isLowerCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( islower ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isPrintable                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isPrintable ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isprint ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isPunctuation                                                       |
------------------------------------------------------------------------------*/
bool IBuffer :: isPunctuation ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( ispunct ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isUpperCase                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isUpperCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isupper ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isWhiteSpace                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: isWhiteSpace ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isspace ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isDBCS                                                              |
------------------------------------------------------------------------------*/
bool IBuffer :: isDBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::isMBCS                                                              |
------------------------------------------------------------------------------*/
bool IBuffer :: isMBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::isSBCS                                                              |
------------------------------------------------------------------------------*/
bool IBuffer :: isSBCS ( ) const
  {
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesMBCS                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: includesMBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesDBCS                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: includesDBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesSBCS                                                        |
------------------------------------------------------------------------------*/
bool IBuffer :: includesSBCS ( ) const
  {
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::isValidDBCS                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isValidDBCS ( ) const
  {
  return true;
  }
/*------------------------------------------------------------------------------
| IBuffer::isValidMBCS                                                         |
------------------------------------------------------------------------------*/
bool IBuffer :: isValidMBCS ( ) const
  {
  return true;
  }
