// Revision: 58 1.14.1.3 source/core/base/iconditn.cpp, oss, ioc.v400, 000302 
/**
 * FILE NAME: iconditn.cpp
 *
 * DESCRIPTION:
 *   Implementation of the event sempahore classes:
 *     ICondition        - The base condition definition class.
 *     IPrivateCondition - Private non-shared condition class.
 *     ISharedCondition  - Condition capable of being shared across processes
 *
 * COPYRIGHT:
 *   IBM Open Class Library
 *   Licensed Materials - Property of IBM
 *   (C) Copyright IBM Corp. 1997
 */


#include <iconditn.hpp>
#include <istring.hpp>
#include <ireslock.hpp>


#if defined(IC_WIN)
#include <iwcondtn.hpp>
#endif

#if defined(IC_PM)
#include <iocondtn.hpp>
#endif

#if defined(IC_POSIX)
#include <ipcondtn.hpp>
#endif


#ifdef IC_PAGETUNE
    #define _ICONDITN_CPP_
    #include <ipagetun.h>
#endif

void ICondition::Relock(IResourceLock& toUnlock, long times)
{
    toUnlock.presCl->relockAll(times);
}   

long ICondition::Unlock(IResourceLock& toUnlock)
{
    return toUnlock.presCl->unlockAll();
}   


//
// IPrivateCondition
//
IPrivateCondition::IPrivateCondition() :

    fImpl(0)
{
    fImpl = new IPrivCondImpl;
}

IPrivateCondition::~IPrivateCondition()
{
    delete fImpl;
}

bool IPrivateCondition::wait(long timeOut)
{
    return fImpl->wait(timeOut);
}

bool IPrivateCondition::wait (IResourceLock & aLock, long timeOut)
{
    return fImpl->wait(aLock, timeOut);
}

void IPrivateCondition::signal()
{
    fImpl->signal();
}

void IPrivateCondition::broadcast()
{
    fImpl->broadcast();
}


//
// ISharedCondition
//
ISharedCondition::ISharedCondition() :

    fImpl(0)
{
    fImpl = new ISharedCondImpl("");
}

ISharedCondition::ISharedCondition(const char * keyName) :

    fImpl(0)
{
    fImpl = new ISharedCondImpl(keyName);
}

ISharedCondition::~ISharedCondition()
{
    delete fImpl;
}


bool ISharedCondition::wait(long timeOut)
{
    return fImpl->wait(timeOut);
}

bool ISharedCondition::wait(IResourceLock & aLock, long timeOut)
{
    return fImpl->wait(aLock, timeOut);
}

void ISharedCondition::signal()
{
    fImpl->signal();
}

void ISharedCondition::broadcast()
{
    fImpl->broadcast();
}

bool ISharedCondition::exists(const char* keyName)
{
    return ISharedCondImpl::exists(keyName);
}

IString ISharedCondition::keyName() const
{
    return fImpl->keyName();
}
