/*----------------------------------------------------------------------------*/
/* FILE NAME: idbcsb1.cpp                                                     */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of the IDBCSBuffer testing         */
/*   functions.                                                               */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"IDBCSB1")
    #pragma csect(static,"SDBCSB1")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

#include <istring.hpp>

#include <idbcsbuf.hpp>

#include <istrtest.hpp>

#include <iexcbase.hpp>

#include <icconsta.h>

#include <iexcept.hpp>

#include <errno.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDBCSB1_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_NLS
static int      /*  LOCALE sensitive compare,  mb support via wchar       */
  locale_wncmp      ( const void* pIBuf,
                      const void* pArg,
                      size_t      IBlen,
                      size_t      cmpLen,
                      bool &   pfWEqual);


/*------------------------------------------------------------------------------
| IDBCSBuffer::isAlnum                                                         |
| IDBCSBuffer::isAlpha                                                         |
| IDBCSBuffer::isAscii                                                         |
| IDBCSBuffer::isCntrl                                                         |
| IDBCSBuffer::isDigits                                                        |
| IDBCSBuffer::isGraphics                                                      |
| IDBCSBuffer::isHexDigits                                                     |
| IDBCSBuffer::isLowerCase                                                     |
| IDBCSBuffer::isPrintable                                                     |
| IDBCSBuffer::isPunctuation                                                   |
| IDBCSBuffer::isUpperCase                                                     |
| IDBCSBuffer::isWhitespace                                                    |
|                                                                              |
| Each of these functions is implemented by constructing (implicitly)          |
| an IStringTest with the corresponding wide char standard library function.   |
| This IStringTest is then applied to the buffer contents using the            |
| windexOfAnyBut member function (which returns the index of the first         |
| character that fails the test).  This "index" is then negated to             |
| achieve the desired result:                                                  |
|   0  (no characters failed the test)  -> "true"                              |
|   >0 (some character failed the test) -> "false"                             |
|                                                                              |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isAlphanumeric ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswalnum ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isAlphabetic                                                    |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isAlphabetic ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswalpha ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isASCII                                                         |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isASCII ( ) const
  {
  return this->IBuffer::isASCII();
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isControl                                                       |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isControl ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswcntrl ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isDigits                                                        |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswdigit ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isGraphics                                                      |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isGraphics ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswgraph ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isHexDigits                                                     |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isHexDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswxdigit ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isLowerCase                                                     |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isLowerCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswlower ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isPrintable                                                     |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isPrintable ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswprint ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isPunctuation                                                   |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isPunctuation ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswpunct ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isUpperCase                                                     |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isUpperCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswupper ) );
  }

/*------------------------------------------------------------------------------
| IDBCSBuffer::isWhiteSpace                                                    |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isWhiteSpace ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->windexOfAnyBut( IStringTest( iswspace ) );
  }
#endif

/*------------------------------------------------------------------------------
| IDBCSBuffer::prevCharLength                                                  |
------------------------------------------------------------------------------*/
size_t IDBCSBuffer :: prevCharLength ( unsigned pos ) const
{
  if ( pos == 1 )
    return 1;
  else
  {
    IStringEnum::CharType ctype =  charType ( pos-1 );
    if ( ctype == IStringEnum::sbcs )
      return 1;
    else
      return (size_t) ctype;
  }
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isDBCS                                                          |
|                                                                              |
| isDBCS returns true if the string consists of only DBCS characters           |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isDBCS( ) const
{
  return isMBCS( );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isMBCS                                                          |
|                                                                              |
| isMBCS returns true if the string consists of only MBCS characters           |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isMBCS( ) const
{
  unsigned    i;
#ifdef IC_HAS_RESTARTABLE
  mbstate_t  ss = 0;
#endif
  const char* p = this->contents();

  for ( i = 0; i < this->length(); )
  {
    // delete the code to use isSBC() as it uses charLength()
    // thus causing state change already

#ifdef IC_HAS_RESTARTABLE
    unsigned j = charLength( p, &ss );
#else
    unsigned j = charLength( p );
#endif

    if ( j <= 1) return false;    // SBC found

#ifndef IC_MVS400
    wchar_t temp;
    mbtowc(&temp, p, j);
    if (wcwidth(temp) <= 1) return false; // half width character found
#endif

    p += j;
    i += j;
  }
  return true;
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isSBCS                                                          |
|                                                                              |
| isSBCS returns true if the string consists of only SBCS characters           |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isSBCS( ) const
{
  unsigned    i;
#ifdef IC_HAS_RESTARTABLE
  mbstate_t  ss = 0;
#endif
  const char* p = this->contents();

  for ( i = 0; i < this->length();)
  {
#ifdef IC_HAS_RESTARTABLE
    unsigned j = charLength( p, &ss );
#else
    unsigned j = charLength( p );
#endif
    if (j > 1)
    {
#ifndef IC_MVS400
       wchar_t temp;
       mbtowc(&temp, p, j);
       if (wcwidth(temp) >= 2) // not half width character found
#endif
          return false;
    }
    p += j;
    i += j;
  }
  return true;
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isValidDBCS                                                     |
|                                                                              |
| isValidDBCS checks to see if a null character (\0) appears at the second     |
| byte position of double-byte characters.                                     |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isValidDBCS( ) const
{
  return isValidMBCS( );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isValidMBCS                                                     |
|                                                                              |
| isValidMBCS checks to see if a null character (\0) appears at the second     |
| byte position of multi-byte characters.                                      |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isValidMBCS( ) const
{
  unsigned    i;
#ifdef IC_HAS_RESTARTABLE
  mbstate_t  ss = 0;
#endif
  const char* p = this->contents();

  for ( i = 0; i < this->length(); )
  {
#ifdef IC_HAS_RESTARTABLE
    unsigned nextChar = i + charLength ( p, &ss );
#else
    unsigned nextChar = i + charLength ( p );
#endif
    p++;
    i++;
    if (i < nextChar)
    {
       for ( ; i < nextChar; i++, p++ )
         if ( *p == '\0' )
           return false;
    } else
        return false;
  }
  return true;
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::includesDBCS                                                    |
|                                                                              |
| includesDBCS returns true if the string contains any DBCS characters         |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: includesDBCS( ) const
{
  return includesMBCS( );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::includesMBCS                                                    |
|                                                                              |
| includesMBCS returns true if the string contains any MBCS characters         |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: includesMBCS( ) const
{
  return( !isSBCS() );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer :: includesSBCS                                                  |
|                                                                              |
| includesSBCS returns true if the string contains any SBCS characters         |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: includesSBCS( ) const
{
  return( !isDBCS() );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::next                                                            |
|                                                                              |
| next returns a pointer to the next logical character in a string.            |
|                                                                              |
| Notes:                                                                       |
|   This function makes the assumption that the input character string         |
|   (pChars) is either pointing to a SBCS character or the first byte of a     |
|   DBCS character i.e. pChars is NOT pointing to the 2nd byte of a DBCS       |
|   character.                                                                 |
------------------------------------------------------------------------------*/
char* IDBCSBuffer :: next( const char *pChars )
{
  return( (char*)pChars + charLength ( pChars ) );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::next                                                            |
------------------------------------------------------------------------------*/
const char* IDBCSBuffer :: next( const char *pChars ) const
{
  return( (char*)pChars + charLength ( pChars ) );
}

/*------------------------------------------------------------------------------
| IDBCSBuffer :: charType                                                      |
|                                                                              |
| This function evaluates the byte at position index in a string and returns   |
| SBCS if the byte is a single byte character, DBCS1 if the byte is the first  |
| byte of a double byte character, and DBCS2 if the byte is the second byte    |
| of a double byte character.                                                  |
|                                                                              |
| Notes:                                                                       |
|   All invocations of charType from within IDBCSBuffer functions ensure that  |
|   the index is valid (i.e. index > 0 and index <= this->length()) before     |
|   calling charType.                                                          |
------------------------------------------------------------------------------*/
IStringEnum::CharType IDBCSBuffer :: charType( unsigned index ) const
{
  unsigned i = 0;
#ifdef IC_HAS_RESTARTABLE
  mbstate_t  ss = 0;
#endif

  for (;;)
  {
#ifdef IC_HAS_RESTARTABLE
    unsigned next = i + charLength (&contents () [i], &ss);
#else
    unsigned next = i + charLength (&contents () [i] );
#endif
    if ( next >= index )
    {
      if ( next - i == 1 )
        return IStringEnum::sbcs;
      else
        return IStringEnum::CharType (index - i);
    }
    else
      i = next;
  } /* endfor */
}

/*------------------------------------------------------------------------------
| IDBCSBuffer::isCharValid                                                     |
|                                                                              |
| This function determines if a character, pChar (which can be either a SBCS   |
| or DBCS character) is in the string of valid characters, pValidChars (which  |
| can contain any combination of SBCS and DBCS characters).                    |
------------------------------------------------------------------------------*/
bool IDBCSBuffer :: isCharValid( unsigned    pos,
                                           const char* pValidChars,
                                           unsigned    numValidChars ) const
  {
  char const* c = &contents () [ pos - 1 ];
  size_t lc = charLength ( c );
#ifdef IC_HAS_RESTARTABLE
  mbstate_t  ss = 0;
#endif

  while ( numValidChars >= lc )
  {
#ifdef IC_HAS_RESTARTABLE
    size_t len = charLength ( pValidChars, &ss );
#else
    size_t len = charLength ( pValidChars );
#endif
    if ( len == lc )
    {
      bool match = true;
      for (unsigned i = 0; i < len; i++)
      {
        if ( pValidChars [i] != c [i] )
        {
          match = false;
          break;
        }
      }
      if ( match )
        return true;
    }
    pValidChars += len;
    numValidChars -= len;
  }
  return false;
}


#ifdef IC_NLS
/*------------------------------------------------------------------------------
| IDBCSBuffer::compare                                                         |
|                                                                              |
| Initialize result to "less than".                                            |
|                                                                              |
| If neither string is null, we compare as many bytes as in the                |
| shorter of the two, using memcmp.                                            |
|                                                                              |
| If these sections compare equal, then, if the lengths are the same,          |
| set the result to "equal."  Otherwise, set the result to either              |
| "less than" or "greater than," depending on which string is longer           |
| (the longer one is "greater").                                               |
|                                                                              |
| If these sections aren't equal, then simply set the result to                |
| "less than" or "greater than" according to the result of the call            |
| to memcmp.                                                                   |
|                                                                              |
| If the receiver isn't null, but the argument is, then return                 |
| "greater than."  If both the receiver and argument are null,                 |
| return "equal."                                                              |
|                                                                              |
| Notes:                                                                       |
|   1. A quick check is made to see if the receiver's buffer and the           |
|      argument coincide (in which case "equal" is returned).                  |
------------------------------------------------------------------------------*/
IBuffer::Comparison IDBCSBuffer :: compare ( const void*  pArg,
                                             unsigned     argLen ) const
  {
  Comparison result = lessThan;

  if (!pArg)  result = greaterThan;
  else
  if (IString::isInternationalized())
  {
     if ( this->contents() == pArg
          &&
          this->length() == argLen )
        result = equal;
     else
       if ( this->length() > 0 )
         if ( argLen > 0 )
          {
            int rc;
            bool fWEqual = false;

            /*  Use the longer length for imbedded SI/SO support  */
            rc = locale_wncmp( this->contents(), pArg,
                               this->length(), argLen, fWEqual);

            if (0 == rc)
             {
               if (fWEqual)  result = equal;
               else
                result = ( this->length() == argLen ) ?
                           equal
                         :
                           ( this->length() < argLen ) ?
                                              lessThan : greaterThan;
             }
            else   /*  else if rc is not 0  */
              result = ( rc < 0 ) ? lessThan : greaterThan;
          }
         else    /*  else if argLen = 0  */
           result = greaterThan;
       else  /*  else if this->length is 0  */
         if ( argLen == 0 )
            result = equal;
  } else
     result = IBuffer::compare (pArg, argLen);

  return result;
  }

/*------------------------------------------------------------------------------
| int locale_wncmp                                                             |
|                                                                              |
| Initialize rc  to  -1.                                                       |
|                                                                              |
| Duplicate buffer1 and buffer2 with null terminator.                          |
| Use wcscoll() to compare the 2 duplicated buffers, taking into               |
| consideration of the fact that there might be null characters inside         |
| the buffers itself.                                                          |
|                                                                              |
| Return code from wcscoll() is returned to the caller.                        |
|                                                                              |
| Notes:                                                                       |
------------------------------------------------------------------------------*/
static int  locale_wncmp ( const void*  pIBuf,
                           const void*  pArg,
                           size_t       IBlen,
                           size_t       cmpLen,
                           bool  &   pfWEqual)
{
  int     rc = -1;                              /*  Set rc to less than      */
  unsigned  iBuf1 = 0, iBuf2 = 0;               /*  Set buffers index        */

  pfWEqual = false;                             /*  Init to NOT equal        */

                                 /*  Set up initial MB work buffers pointer  */
  const char * pM1 = (char *) pIBuf;
  const char * pM2 = (char *) pArg;
  char * pMBuf1 = NULL;
  char * pMBuf2 = NULL;
                                 /*  Get wchar work buffers via new          */
  wchar_t * pWCBuf1 = new wchar_t[IBlen + 1];
  wchar_t * pWCBuf2 = new wchar_t[cmpLen + 1];


  /*  if buffer1 is NOT null ended            */
  if (pM1  &&  (*(pM1 + IBlen - 1) != '\0'))
   {
     pMBuf1 = new char[IBlen + 1];           /*  Get work buffer1 via new  */
     if (pMBuf1)                      /*  if work buffer1 exist     */
      {
        memcpy(pMBuf1, pIBuf, IBlen);        /*  Duplicate (work) buffer1  */
        *(pMBuf1 +  IBlen) = '\0';           /*  Null terminate it         */
        pM1 = pMBuf1;
      }
     else
       pM1 = NULL;                    /*  Memory storage problem  */
   }

                  /*  if work buffer1 exist and buffer2 is NOT null ended  */
  if (pM1  &&  pM2  &&  (*(pM2 +  cmpLen - 1) != '\0'))
   {
     pMBuf2 = new char[cmpLen + 1];   /*  Get work buffer2 via new  */
     if (pMBuf2)                      /*  if work buffer1 exist     */
      {
        memcpy(pMBuf2, pArg,  cmpLen);       /*  Duplicate (work) buffer2  */
        *(pMBuf2 + cmpLen) = '\0';           /*  Null terminate it         */
        pM2 = pMBuf2;
      }
     else
      {
        pM2 = NULL;                   /*  Memory storage problem  */
                          /*  if work buffer1 obtained via new, delete it  */
        delete [] pMBuf1;
      }
   }

                                           /*  if all work buffers exist  */
  if (pM1  &&  pM2  &&  pWCBuf1  &&  pWCBuf2)
   {
     unsigned sLen1 = 0, sLen2 = 0,
              iX1, iX2;
     const char *  pWS1, * pWS2;
#ifdef IC_HAS_RESTARTABLE
     mbstate_t     ss1, ss2;
#endif

     for (;;)
      {
        iX1 = iX2 = 0;
#ifdef IC_HAS_RESTARTABLE
        ss1 = ss2 = 0;
#endif

        while ('\0' == *pM1)     /*  Skip over the possible NULL characters  */
         {
           ++pM1;
           ++iX1;
           ++iBuf1;
           if (iBuf1 ==  IBlen)  break;     /*  Done, end of buffer1         */
         }

        while ('\0' == *pM2)     /*  Skip over the possible NULL characters  */
         {
           ++pM2;
           ++iX2;
           ++iBuf2;
           if (iBuf2 == cmpLen)  break;     /*  Done, end of buffer2         */
         }

        if (iX1 != iX2)       /*  Not equal number of NULL characters found  */
         {
           if (iX1 > iX2)  rc = 1;        /*  Indicate pIBuf > pArg          */
           break;                         /*  Done, no need to continue      */
         }

        if (iBuf1 ==  IBlen)     /*  if end of first buffer  */
         {
           if (iX1 ==  IBlen)       /*  if both buffers contain only NULL  */
              rc = 0;               /*  charcaters                         */
           break;                /*  Done with compare       */
         }

        pWS1 = pM1;                 /*  pWS1/pWS2 are updated to NULL if   */
        pWS2 = pM2;                 /*  null character is reached          */

        /*  Length of MBs to convert to wchar, including the null char     */
        sLen1 = strlen(pM1) + 1;
                                    /*  Convert MB string to wchar string  */
#ifdef IC_HAS_RESTARTABLE
        mbsrtowcs(pWCBuf1, &pWS1, sLen1, &ss1);
#else
        mbstowcs(pWCBuf1, pWS1, sLen1);
#endif
        sLen2 = strlen(pM2) + 1;
#ifdef IC_HAS_RESTARTABLE
        mbsrtowcs(pWCBuf2, &pWS2, sLen2, &ss2);
#else
        mbstowcs(pWCBuf2, pWS2, sLen2);
#endif

        errno = 0;                        /*  Clear errno                  */
        rc = wcscoll(pWCBuf1, pWCBuf2);   /*  Collate the 2 wchar strings  */

#ifdef IC_MOTIF
        if (EINVAL == errno)   // AIX
#else
        if (EILSEQ == errno)   // Others
#endif
         {
           errno = 0;                       // Clear illegal sequence RC
           rc = wcscmp(pWCBuf1, pWCBuf2);   // Try WCHAR compare
         }

        if (errno || (0 != rc))  break;   /*  if wcscoll() error or not    */
                                          /*  equal, done                  */
        iBuf1 += sLen1;
        iBuf2 += sLen2;
        /*  if either EOD1 or EOD2, no need to continue  */
        if ((iBuf1 >=  IBlen) || (iBuf2 >= cmpLen))
         {
           --iBuf1;      /*  Remove the last null character from the    */
           --iBuf2;      /*  buffer indexes.                            */
           break;
         }
        pM1 += sLen1;                  /*  Advance the pointer within   */
        pM2 += sLen2;                  /*  MB buffers                   */
      }                       /*  loop through possible MB strings in   */
                              /*  the buffers                           */

     delete [] pWCBuf1;       /*  Delete the work buffers created via new  */
     delete [] pWCBuf2;
     delete [] pMBuf1;
     delete [] pMBuf2;

     if (errno)       /*  Bad rc from wcscoll() call  */
      {
        char cMsg[20];
        int save_errno = errno;
        sprintf(cMsg, "wcscoll errno = %d\n", errno);
#ifdef IC_MOTIF
  IInvalidRequest exception(cMsg, save_errno, IException::recoverable);
  ITHROW( exception );
#else
        ITHROWCLIBERROR( cMsg, IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
#endif
      }
   }

 if ((0 == rc) && ((iBuf1 == IBlen) && (iBuf2 == cmpLen)))
    pfWEqual = true;

 return rc;
}

#ifdef IC_HAS_RESTARTABLE

// New functions which use restartable version of multi-byte family

size_t IDBCSBuffer :: charLength ( char const* p , mbstate_t  *ss)
  {
  if (IString::isInternationalized())
  {
    if ( *p == 0 )
       return 1;
    else {
       int temp;
       return ((temp = mbrlen ( p, MB_LEN_MAX , ss )) > 0 ? temp : 1);
    }
  } else {
#ifdef IC_PMWIN
    return (IBuffer::isDBCSLead(*p) ? 2 : 1);
#endif
    if ( *p == 0 )
       return 1;
    else {
       int temp;
       return ((temp = mbrlen ( p, MB_LEN_MAX , ss )) > 0 ? temp : 1);
    }
    }
  }

size_t IDBCSBuffer :: charLength ( unsigned pos , mbstate_t  *ss) const
  {
  return charLength ( (contents() + pos - 1 ) , ss );
  }


bool IDBCSBuffer :: isSBC ( char const* p , mbstate_t *ss)
  {
  return ( charLength ( p , ss) == 1 );
  }


size_t IDBCSBuffer :: prevCharLength ( unsigned pos , mbstate_t *ss) const
{
  return prevCharLength( pos );
}
#endif
#endif
