#ifndef _IDECIMAL_INL_
#define _IDECIMAL_INL_
/*----------------------------------------------------------------------------*/
/* FILE NAME: idecimal.inl                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the definition of the inline functions for the        */
/*   classes declared in idecimal.hpp.                                        */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*----------------------------------------------------------------------------*/

extern "C"
{
   #include <string.h>
   #include <float.h>
}

#ifdef IC_PAGETUNE
   #define _IDECIMAL_CPP_
   #include <ipagetun.h>
#endif

inline const bool IBinaryCodedDecimal::isPositive() const
{
   return (buf[numDig >> 1] & 0x0F) == 0x0F;
}

inline const bool IBinaryCodedDecimal::isNegative() const
{
   return (buf[numDig >> 1] & 0x0F) == 0x0D;
}

inline const char* IBinaryCodedDecimal::cData() const
{
   return buf;
}

inline IString IBinaryCodedDecimal::asString() const
{
   return IDecimalUtil(*this).asString();
}

inline ostream& operator<<(ostream& os, const IDecimalUtil& dut)
{
   return (os << dut.asString());
}

inline void IBinaryCodedDecimal::setPositive()
{
   buf[numDig >> 1] |= 0x0F;
}

inline void IBinaryCodedDecimal::setNegative()
{
   int LastByte = numDig >> 1;

   for (int counter = 0; counter < LastByte; counter++)
   {
      if (buf[counter] != 0)
      {
         buf[LastByte] = (buf[LastByte] & 0xF0) | 0x0D;
         break;
      }
   }
   if ((buf[LastByte] & 0xF0) != 0)
      buf[LastByte] = (buf[LastByte] & 0xF0) | 0x0D;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator-() const
{
   IBinaryCodedDecimal tmpBCD(*this);
   if (tmpBCD.isPositive())
      tmpBCD.setNegative();
   else
      tmpBCD.setPositive();
   return tmpBCD;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator+() const
{
   return (*this);
}

inline IBinaryCodedDecimal&
IBinaryCodedDecimal::operator=(const IBinaryCodedDecimal& bcd)
{
   *this = IDecimalUtil(bcd);
   return *this;
}


inline IBinaryCodedDecimal::IBinaryCodedDecimal() :

    numDig(DFT_DIG),
    numPrec(DFT_PREC)
{
    memset(buf, '\0', sizeof(buf));
    setPositive();
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec)
: numDig(nDig), numPrec(nPrec)
{
   if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
       (numPrec < 0)  || (numPrec > DEC_DIG) ||
       (numPrec > numDig)) {

        IDecimalUtil::throwOverflow();
   }
   memset(buf, '\0', sizeof(buf));
   setPositive();
}

inline IBinaryCodedDecimal::IBinaryCodedDecimal(int anInt) :

    numDig(DFT_INT_DIG),
    numPrec(0)
{
    memset(buf, '\0', sizeof(buf));
    if (anInt)
        IDecimalUtil::intToDec(anInt, numDig, numPrec, buf);
    else
        setPositive();
}

inline IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int auInt) :

    numDig(DFT_INT_DIG),
    numPrec(0)
{
    memset(buf, '\0', sizeof(buf));
    if (auInt)
        IDecimalUtil::uIntToDec(auInt, numDig, numPrec, buf);
    else
        setPositive();
}

inline IBinaryCodedDecimal::IBinaryCodedDecimal(long aLong) :

    numDig(DFT_LNG_DIG),
    numPrec(0)
{
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::longToDec(aLong, numDig, numPrec, buf);
}

inline IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned long auLong) :

    numDig(DFT_LNG_DIG),
    numPrec(0)
{
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::uLongToDec(auLong, numDig, numPrec, buf);
}

inline IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig,
                                                unsigned int nPrec,
                                                int anInt) :
    numDig(nDig),
    numPrec(nPrec)
{
   if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
       (numPrec < 0)  || (numPrec > DEC_DIG) ||
       (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::intToDec(anInt, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         unsigned int auInt) :
    numDig(nDig),
    numPrec(nPrec)
{
   if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
       (numPrec < 0)  || (numPrec > DEC_DIG) ||
       (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::uIntToDec(auInt, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         long aLong) :
    numDig(nDig),
    numPrec(nPrec)
{
   if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
       (numPrec < 0)  || (numPrec > DEC_DIG) ||
       (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::longToDec(aLong, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         unsigned long auLong) :
    numDig(nDig),
    numPrec(nPrec)
{
    if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
       (numPrec < 0)  || (numPrec > DEC_DIG) ||
       (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::uLongToDec(auLong, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         float aFloat) :
    numDig(nDig),
    numPrec(nPrec)
{
    if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
        (numPrec < 0)  || (numPrec > DEC_DIG) ||
        (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::doubleToDec(aFloat, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         double aDouble) :
    numDig(nDig),
    numPrec(nPrec)
{
    if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
        (numPrec < 0)  || (numPrec > DEC_DIG) ||
        (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::doubleToDec(aDouble, numDig, numPrec, buf);
}

inline
IBinaryCodedDecimal::IBinaryCodedDecimal(unsigned int nDig, unsigned int nPrec,
                                         long double alDouble) :
    numDig(nDig),
    numPrec(nPrec)
{
    if ((numDig  <= 0) || (numDig  > DEC_DIG) ||
        (numPrec < 0)  || (numPrec > DEC_DIG) ||
        (numPrec > numDig))
    {
        IDecimalUtil::throwOverflow();
    }
    memset(buf, '\0', sizeof(buf));
    IDecimalUtil::lDoubleToDec(alDouble, numDig, numPrec, buf);
}

inline int IBinaryCodedDecimal::precisionOf() const
{
    return numPrec;
}

inline int IBinaryCodedDecimal::digitsOf() const
{
    return numDig;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(int anInt)
{
    *this = IDecimalUtil(anInt);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(unsigned int auInt)
{
    *this = IDecimalUtil(auInt);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(long aLong)
{
    *this = IDecimalUtil(aLong);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(unsigned long auLong)
{
    *this = IDecimalUtil(auLong);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(float aFloat)
{
    *this = IDecimalUtil(aFloat);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(double aDouble)
{
    *this = IDecimalUtil(aDouble);
    return *this;
}

inline IBinaryCodedDecimal& IBinaryCodedDecimal::operator=(long double alDouble)
{
    *this = IDecimalUtil(alDouble);
    return *this;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator++()
{
    *this = IDecimalUtil(*this) + IDecimalUtil(1);
    return *this;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator++(int)
{
    IBinaryCodedDecimal tmpBCD(*this);
    *this = IDecimalUtil(*this) + IDecimalUtil(1);
    return tmpBCD;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator--()
{
    *this = IDecimalUtil(*this) + IDecimalUtil(-1);
    return *this;
}

inline IBinaryCodedDecimal IBinaryCodedDecimal::operator--(int)
{
    IBinaryCodedDecimal tmpBCD(*this);
    *this = IDecimalUtil(*this) + IDecimalUtil(-1);
    return tmpBCD;
}

inline long IBinaryCodedDecimal::asLong() const
{
    return (long)IDecimalUtil(*this);
}

inline double IBinaryCodedDecimal::asDouble() const
{
    return (double) IDecimalUtil(*this);
}

inline int IBinaryCodedDecimal::operator !() const
{
    return (IDecimalUtil(*this) == IDecimalUtil(0));
}

inline void IBinaryCodedDecimal::operator +=(const IDecimalUtil &dut)
{
    *this = IDecimalUtil(*this) + dut;
}

inline void IBinaryCodedDecimal::operator -=(const IDecimalUtil &dut)
{
    *this = IDecimalUtil(*this) - dut;
}

inline void IBinaryCodedDecimal::operator *=(const IDecimalUtil &dut)
{
    *this = IDecimalUtil(*this) * dut;
}

inline void IBinaryCodedDecimal::operator /=(const IDecimalUtil &dut)
{
    *this = IDecimalUtil(*this) / dut;
}

inline ostream& operator<<(ostream& os, const IBinaryCodedDecimal& bcd)
{
   return (os << IDecimalUtil(bcd));
}

inline istream& operator>>(istream& is, IBinaryCodedDecimal& bcd)
{
    char cstring[DEC_DIG >> 2];
    IDecimalUtil dut(cstring);
    bcd = dut;
    return is;
}

inline int IBinaryCodedDecimal::operator>(const IDecimalUtil& dut) const
{
    return (IDecimalUtil(*this) > dut);
}

inline int IBinaryCodedDecimal::operator<(const IDecimalUtil& dut) const
{
    return (IDecimalUtil(*this) < dut);
}

inline int IBinaryCodedDecimal::operator>=(const IDecimalUtil& dut) const
{
    return !(IDecimalUtil(*this) < dut);
}

inline int IBinaryCodedDecimal::operator<=(const IDecimalUtil& dut) const
{
    return !(IDecimalUtil(*this) > dut);
}

inline int IBinaryCodedDecimal::operator==(const IDecimalUtil& dut) const
{
    return (IDecimalUtil(*this) == dut);
}

inline int IBinaryCodedDecimal::operator!=(const IDecimalUtil& dut) const
{
    return !(IDecimalUtil(*this) == dut);
}

#ifdef IC_STREAMING
inline void IBinaryCodedDecimal::operator>>=(IDataStream& toWhere)
{
    writeToStream(toWhere);
}

inline void IBinaryCodedDecimal::operator<<=(IDataStream& fromWhere)
{
    readFromStream(fromWhere);
}
#endif

inline int IDecimalUtil::operator>=(const IDecimalUtil& dut) const
{
    return !(*this < dut);
}

inline int IDecimalUtil::operator<=(const IDecimalUtil& dut) const
{
    return !(*this > dut);
}

inline int IDecimalUtil::operator!=(const IDecimalUtil& dut) const
{
    return !(*this == dut);
}

inline unsigned long IDecimalUtil::genuLong(const char* bcdBuf)
{
    // using shifts will eliminates indianess
    unsigned long tmpLong = *bcdBuf++;
    tmpLong = (tmpLong << 8) | *bcdBuf++;
    tmpLong = (tmpLong << 8) | *bcdBuf++;
    tmpLong = (tmpLong << 8) | *bcdBuf;
    return tmpLong;
}

inline unsigned long IDecimalUtil::gen10Complement(unsigned long auLong)
{
    auLong = ~auLong;
    return IDecimalUtil::decAddWord(auLong - MASK_OF_6, 1UL,
                                    MASK_OF_6, MASK_OF_8);
}

inline void IDecimalUtil::genBCD(char* bcdBuf, unsigned long auLong)
{
    // using shifts will eliminates indianess
    char * cptr = bcdBuf + 3;
    *(cptr--) = (char)(auLong & 0x000000FFUL);
    auLong >>= 8;
    *(cptr--) = (char)(auLong & 0x000000FFUL);
    auLong >>= 8;
    *(cptr--) = (char)(auLong & 0x000000FFUL);
    auLong >>= 8;
    *(cptr) = (char)(auLong & 0x000000FFUL);
}

#endif
