/* NOSHIP */
#ifndef _IDLLIMP_
#define _IDLLIMP_
/*******************************************************************************
* FILE NAME: idllimp.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declarations of the classes:                                               *
*     IRefCountedDLLImp                                                        *
*     IWindowsDLLImp                                                           *
*     IOS2DLLImp                                                               *
*     IMotifDLLImp                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 86  1.5.1.6  source/core/base/idllimp.hpp, appsupport, ioc.v400, 000302  

#include <ibase.hpp>
#include <istring.hpp>
#include <ibhandle.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

/* Used to reference count dll loading */
class IRefCountedDLLImp
{
public:
  
virtual
 ~IRefCountedDLLImp();

virtual bool  
  operator ==(const IRefCountedDLLImp& hmod) const;


/*------------- DLL Name ---------------------------------------*/
virtual IString 
  initialCreationName   ( ) const,
  loadedPathName ( ) const = 0;
  

/*------------- DLL Loading ------------------------------------*/
virtual void
  openDLL   ( ) = 0;

virtual void
  closeDLL  ( ) = 0;

IModuleHandle
  handle() const    { return fModuleHandle; }

/*------------- Procedure Address ---- -------------------------*/
virtual void
 *procAddress ( const char*   procedureName ) const = 0,
 *procAddress ( unsigned long procedureOrdinal ) const = 0;

enum ProcType { k16Bit, k32Bit, k64Bit};
virtual ProcType
  procType    ( const char*   procedureName ) const = 0,
  procType    ( unsigned long procedureOrdinal ) const = 0;


/*------------- DLL Reference Counting -------------------------*/
unsigned long 
  addRef      ( ),
  removeRef   ( );
unsigned long
  useCount    ( ) const { return fUseCount; }   

/*------------- Debug Information ------------------------------*/
virtual IString 
  asDebugInfo ( ) const;

protected:
    IRefCountedDLLImp    ( const char* creationName,
                           bool        useNLSPath,
                           const char* defaultExtension); 
    IRefCountedDLLImp    ( const  IModuleHandle& handle);
    void setHandle       ( const IModuleHandle& handle ) 
        { fModuleHandle = handle; }
    void setCreationName ( const IString& creationName)
        { fCreationName = creationName; }
    IString getDefaultExtension() { return fDefaultExtension; }

virtual IString
  nlsPathName     ( ) const = 0;

bool
  useNLSPath ( ) const { return fUseNLSPath; }

private:
IString
  fCreationName,
  fDefaultExtension;
IModuleHandle
  fModuleHandle;
unsigned long
  fUseCount;
bool
  fUseNLSPath;
};

#ifdef IC_WIN
class IWindowsDLLImp : public IRefCountedDLLImp
{
typedef IRefCountedDLLImp
  Inherited;
  
public:
  IWindowsDLLImp    ( const char* creationName,
                      bool        searchNLSPath = false);
  IWindowsDLLImp    ( const  IModuleHandle& handle);
 virtual
  ~IWindowsDLLImp   ( );
/*------------- DLL Loading ------------------------------------*/
virtual void
  openDLL   ( );

virtual void
  closeDLL  ( );

/*------------- DLL Name ---------------------------------------*/
virtual IString 
  loadedPathName ( ) const;
  

/*------------- Procedure Address ---- -------------------------*/
virtual void
 *procAddress ( const char*   procName ) const,
 *procAddress ( unsigned long procedureOrdinal) const;

virtual ProcType
  procType    ( const char*   procName ) const,
  procType    ( unsigned long procedureOrdinal ) const;

protected:
virtual IString
  nlsPathName ( ) const;

private:
IString
  fNLSPathName,
  fLoadedPathName;
};
#endif // IC_WIN

#ifdef IC_PM
class IOS2DLLImp : public IRefCountedDLLImp
{
typedef IRefCountedDLLImp
  Inherited;
public:
  IOS2DLLImp    ( const char* creationName,
                  bool        searchNLSPath = false);
  IOS2DLLImp    ( const  IModuleHandle& handle);
 virtual
  ~IOS2DLLImp   ( );

/*------------- DLL Loading ------------------------------------*/
virtual void
  openDLL   ( );

virtual void
  closeDLL  ( );

/*------------- DLL Name ---------------------------------------*/
virtual IString 
  loadedPathName ( ) const;
  
/*------------- Procedure Address ---- -------------------------*/
virtual void
 *procAddress ( const char*   procedureName ) const,
 *procAddress ( unsigned long procedureOrdinal ) const;

virtual ProcType
  procType    ( const char*   procedureName ) const,
  procType    ( unsigned long procedureOrdinal ) const;

protected:
virtual IString
  nlsPathName    ( ) const;

private:
IString
  fNLSPathName,
  fLoadedPathName;
};
#endif // IC_PM

#ifdef IC_MOTIF

struct ldhdr;
struct ldsym; 
struct ld_info;

class IMotifDLLImp : public IRefCountedDLLImp
{
    typedef IRefCountedDLLImp Inherited;
public:
    IMotifDLLImp(const char* creationName,
                 bool        searchNLSPath = false);
#if 0  // Not currently supported on Motif
    IMotifDLLImp(const  IModuleHandle& handle);
#endif
    virtual ~IMotifDLLImp(); 

/*------------- DLL Loading ------------------------------------*/
    virtual void openDLL();
    virtual void closeDLL();

/*------------- DLL Name ---------------------------------------*/
    virtual IString loadedPathName() const;
  
/*------------- Procedure Address ---- -------------------------*/
    virtual void *procAddress(const char* procedureName) const;
    virtual void *procAddress(unsigned long procedureOrdinal) const;

    virtual ProcType procType(const char* procedureName) const;
    virtual ProcType procType(unsigned long procedureOrdinal) const;

protected:
    virtual IString nlsPathName() const;

private:
    void saveLoaderInformation();
    IString makeNLSPath();
    IString fLoadedPathName;
    int (*fEntryPoint)();  
    struct ldhdr *fLoaderHeader;                                    
    struct ldsym *fLoaderSymbols;
    struct ld_info *fLoaderInfo;
    void *fResources;
    unsigned long fLoaderBias;  
};
#endif // IC_MOTIF

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IDLLIMP_
