// Revision: 17 1.4.1.2 source/core/base/iintrst.cpp, notification, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iintrst.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iintrst.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iintrst.hpp>
#include <inotifev.hpp>
#include <iobslist.hpp>

#ifdef IC_PAGETUNE
    #define _IINTRST_CPP_
    #include <ipagetun.h>
#endif

IInterest::IInterest() :
  fName(INotifier::allChangesId),
  fHashValue(0),
  fpObserverCache(0)
{
  fHashValue = IInterest::computeHashValue(fName);
}

unsigned long IInterest::computeHashValue(const char* pString)
{
    //just using the ptr address as the hash value
    //beware if interests go across address spaces
    return (unsigned long) pString;
}

IInterest::IInterest(const IInterest& src)
{
  fName  = src.fName;
  fNotifierAddress = src.fNotifierAddress;
  fHashValue = src.fHashValue;
  fpObserverCache = 0;
}

IInterest&
IInterest::operator=(const IInterest& rhs )
{
  if (this != &rhs)
        {
          fName  = rhs.fName;
          fNotifierAddress = rhs.fNotifierAddress;
          fHashValue = rhs.fHashValue;
          fpObserverCache = 0;
        }

  return *this;
}

IInterest::IInterest(const INotifier& theNotifier, const INotificationId& theId ):
  fName(theId),
  fHashValue(0),
  fpObserverCache(0)
{
  fHashValue = IInterest::computeHashValue(fName);
  fNotifierAddress = theNotifier.notifierAddress();
}


IInterest::~IInterest()
{
}


INotificationId
IInterest::notificationId() const
{
  return fName;
}


unsigned long
IInterest::hashValue () const
{
  //check for hashvalue
        return fHashValue;
}


INotifierAddress
IInterest::notifierAddress() const
{
  return fNotifierAddress;
}

bool
IInterest::operator==(const IInterest& theInterest) const
{
  return ((fNotifierAddress == theInterest.fNotifierAddress) &&
                  fHashValue == theInterest.fHashValue);
}

bool
IInterest::operator!=(const IInterest& theInterest) const
{
  return (!this->operator==(theInterest) );
}


void
IInterest::notifyObservers(INotificationEvent& theEvent)
{
  INotifierAddress notifierAddress =  theEvent.notifierAddress();
  //Get the notifier from the fNotifierAddress

  if (! fpObserverCache)
        //Get the observer list from the notifier for this interest
        fpObserverCache = &(notifierAddress.notifier().observerList(this));

  fpObserverCache->notifyObservers(theEvent);
}

void
IInterest::notifyObserversAsync(INotificationEvent& theEvent)
{
  INotifierAddress notifierAddress =  theEvent.notifierAddress();
  //Get the notifier from the fNotifierAddress
  //Call the notifyObserver method on it passing it theEvent
  if (! fpObserverCache)
        //Get the observer list from the notifier for this interest
        fpObserverCache = &(notifierAddress.notifier().observerList(this));

  fpObserverCache->notifyObserversAsync(theEvent);
}

