/* IBM Confidential */
/*----------------------------------------------------------------------------*/
/* FILE NAME: imprintf.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of classes/functions declared      */
/*   in imprintf.hpp.                                                         */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
#ifdef __MVS__
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS400__
/*   ????-???                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS2__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __WINDOWS__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _AIX
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"IMPRINTF")
    #pragma csect(static,"SMPRINTF")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

/* -------------------- IBM Internal Use Only ---------------------- */
/* PRINTF: diverts PRINTF calls to an OS/2 Named Queue               */
/* Copyright (c) IBM Corporation, 1991                               */
/* ----------------------------------------------------------------- */
/*                                     Mike Cowlishaw, Jan-Mar 1991  */
/*                                                                   */
/*-------------------------------------------------------------------*/
/* Modifed: Roger S. Reider    5/30/92                               */
/*                                                                   */
/* Modified for use by the IMemMgr DLL.  Note: these changes         */
/* removed the use of malloc() from within this module so that       */
/* recursive calls to the IMemMgr DLL functions would not be needed. */
/* Consequently, I have employed a semaphore to address multithread  */
/* issues instead of allocating a separate buffer for each calling   */
/* thread.                                                           */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* This routine, when linked into an .EXE instead of the usual C     */
/* runtime, sends the edited result string to a named queue (if      */
/* it exists).  If the queue does not exist, then all printf data    */
/* are discarded (ignored).                                          */
/*                                                                   */
/* The result string is accumulated until a line feed (LF) character */
/* is received; the whole line is then sent to the queue.  Lines are */
/* automatically broken at a set (tailorable) length, if necessary.  */
/*                                                                   */
/* This routine may be tailored by altering the #defines at the      */
/* top:                                                              */
/*                                                                   */
/*   PRINTFID      - An ID string that is prefixed to each line of   */
/*                   data before being sent to the queue.  This      */
/*                   can be any string, or the null string.          */
/*   IC_PRINTFMAXLEN  - Maximum length of string that can be         */
/*                      formatted in a single call. Results are      */
/*                      unpredictable if this length is exceeded.    */
/*                      Default is 1024.  Defined in <imprintf.h>.   */
/*   PRINTFLINELEN - Maximum length of a line that will be sent.     */
/*                   This excludes the prefix an its blank.  If the  */
/*                   calls to printf cause a line to be generated    */
/*                   that is longer than this, the line will be      */
/*                   broken at this point.                           */
/*   PRINTFTHREADS - Maximum number of threads expected.  This may   */
/*                   need to be increased if the process limitation  */
/*                   is removed, or you can save a little storage    */
/*                   by decreasing it.  PRINTFs from threads larger  */
/*                   than this number are ignored.                   */
/*   PRINTFQNAME   - The name of the public queue that the result    */
/*                   is to be sent to.  Normally '\QUEUES\PMPRINTF'. */
/*                   Note that the \QUEUES\ part is required.        */
/*                                                                   */
/* Returns:                                                          */
/*   n: Count of data characters, if successfully received           */
/*   0: If no queue existed (i.e., no server)                        */
/*  <0: An error occurred (e.g., out of memory)                      */
/*                                                                   */
/* Restrictions:                                                     */
/*   1. Total length of data (length of PRINTFID, + IC_PRINTFMAXLEN) */
/*      must be less than 32K-1.                                     */
/*   2. This has only been tested under Large model compilation.     */
/*      It may need modification for other models.                   */
/*   3. This version uses a static array to point to the per-thread  */
/*      data.  The code could be made read-only by hanging this      */
/*      array (and the other static information) off a system-owned  */
/*      anchor of some kind.                                         */
/*   4. To use PRINTF within other than the main thread in a         */
/*      program, that thread must be started with _beginthread       */
/*      (not DosCreateThread).  This restriction is a consequence of */
/*      the use of C library routines (sprintf) in PRINTF <sigh>.    */
/*   5. If the last PRINTF done by a thread does not end in '\n'     */
/*      then the final part-line may be lost or appear later.        */
/*                                                                   */
/* Protocol:                                                         */
/*   PRINTF writes its data to the named queue using the following   */
/*   protocol:                                                       */
/*     Request -- Holds the Selector of the shared memory segment    */
/*                valid in the receiver's address space.  The        */
/*                shared memory segment hold the character data      */
/*                (a 0-terminated string) starting at offset 0.      */
/*     Length  -- The length of the data, including terminator.      */
/*                A negative length indicates a BELL in the data.    */
/*     Address -- Timestamp (when queue was written) in C long       */
/*                integer format (as returned by time()).            */
/*                This may be 0L if not required.                    */
/*                                                                   */
/* Notes:                                                            */
/*   1. PMPRINTF uses a queue and shared memory messages because:    */
/*        (a) It makes collection at the receiving end very easy.    */
/*        (b) I wanted to experiment with queues and shared memory.  */
/*      This make not be the most cost-effective method.             */
/*   2. Typical IBM C/2 compiler invocation:                         */
/*        cc -nologo -G2s -W3 -Zpe printf.c;                         */
/*      For multi-thread (and link with LLIBCMT):                    */
/*        cl -c -nologo -G2s -W3 -Zpel Alfw printf.c                 */
/*   3. PRINTF sends the timestamp across the queue as a GMT long    */
/*      integer, the result from a call to the C function time().    */
/*      This will only be correct if the environment variable TZ has */
/*      been set (e.g., TZ=EST5EDT), or you are in the same time     */
/*      zone as the default for your compiler (e.g., +5 for IBM C/2, */
/*      +8 for Microsoft C6).  Hence, if PRINTF is called by an MS   */
/*      C6 program and TZ has not been set, then timestamps logged   */
/*      will be wrong by 3 hours, because PMPRINTF currently uses    */
/*      the IBM C/2 runtime library.  (The MS compiler applies a     */
/*      default +8 or +7 hour shift to generate GMT, then when       */
/*      received, the IBM compiler will apply a default -5 or -4     */
/*      hour shift to generate the local time recorded in the log.)  */
/*      For more information, see the tzset() function description   */
/*      in the IBM C/2 manual.                                       */

/* ----- Includes and externals ----- */
#pragma priority( -2147482024 )

#ifdef _AIX
extern "C" int imprintf(const char *f,...)
{
   return(0);
}
#else
#ifdef __MVS__
#define TRUE  1
#define FALSE 0
#endif

extern "C"
{
  #include <stdlib.h>              /* standard C functions */
  #include <stddef.h>              /* (note NOT stdio.h, to avoid a */
  #include <string.h>              /*   warning of non-matching */
  #include <time.h>                /*   PRINTF prototype) */
  #include <stdarg.h>              /* .. */
  #include <stdio.h>               /* vsprintf */
  #define INCL_DOS                 /* Operating system definitions */
  #define INCL_PROCESS
#if defined(__OS2__)
  #include <os2.h>
#endif // __OS2__
#if defined(__WINDOWS__)
  #include <windows.h>
#endif // __WINDOWS__
  #include <imprintf.h>
}

#ifdef __OS400__
 // Replaced sys/sysopapi.h with correct header files; sysopapi was not
 // part of the supported AS/400 interface
 #include <qmhsndpm.h>
 #include <qmh.h>
 #include <qtes.h>
#endif

#ifdef IC_PAGETUNE
    #define _IMPRINTF_CPP_
    #include <ipagetun.h>
#endif

/* ----- Customization variables ----- */
#define PRINTFID      "ICLUI: "
#ifdef __OS400__
   // no threading for OS400 yet, so 1 thread
   #define PRINTFTHREADS  1
   #define TRUE 1
   #define FALSE 0
#else
   #define PRINTFTHREADS  54
#endif
#ifdef IC_PM
  #define PRINTFQNAME   "\\QUEUES\\PRINTF32"
#endif
#ifdef IC_WIN
  #define PRINTFMSNAME  "\\\\.\\mailslot\\PRINTF32"
#endif
#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif

/* ----- Local subroutine ----- */
extern "C"  int printf_(struct perthread *);


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMPRINTF_CPP_
  #include <ipagetun.h>
#endif

#if 0
  /* Next line defines sprintf (from mt\stdio.h) */
  extern "C" int  vsprintf(char *, const char *, va_list);
#endif

/* ----- Local defines ----- */
#define PRINTFIDSIZE sizeof(PRINTFID)
#define PRINTFMAXBUF PRINTFIDSIZE+IC_PRINTFMAXLEN

/* ----- Per-thread output buffer and current indices into line ---- */
/* ----- Per-thread output buffer and current indices into line ---- */
struct perthread {
  long   lineIndex;                /* where next char */
  long   tideMark;                 /* rightmost char */
  int    bell;                     /* TRUE if line has bell */
  char   line[PRINTFMAXBUF];       /* accumulator */
  };

/* ----- Local static variables ----- */
static unsigned long ourPid = 0;   /* our process ID */
static unsigned long servePid = 0; /* process IDs of the server */
                                   /* -> per-thread data */
static struct perthread *threads[PRINTFTHREADS+1];
static perthread ptBuffer;         /* single threaded buffer */

#ifdef IC_PM
  static HQUEUE hQueue = 0;        /* handle for the queue */
#endif
#ifdef IC_WIN
  static HANDLE hMailSlot = INVALID_HANDLE_VALUE; /* handle for the mailslot */
  static unsigned long winThreadID[PRINTFTHREADS+1] = {0};
#endif

/*------------------------------------------------------------------------------
| imprintf - Externalized C function that is used to format trace information  |
|            that is written to either a queue (PM) or a mailslot (Windows).   |
|                                                                              |
| Note that this function has a variable number of arguments.                  |
------------------------------------------------------------------------------*/
extern "C" int imprintf(const char *f,...)
{
  unsigned long ourTid;            /* thread ID */
  struct perthread *pThread;       /* pointer to per-thread data */
  int    rc;                       /* returncode */

#ifdef IC_PM
  TIB    *pTib;                    /* process/thread id structures */
  PIB    *pPib;                    /* .. */
  ULONG  urc;                      /* returncode */

  /*******************************************************************/
  /* Open the printf Queue -                                         */
  /* If it does not exist or cannot be opened, return                */
  /*******************************************************************/
  urc = DosOpenQueue( &servePid, &hQueue, (PSZ)PRINTFQNAME );
  if (urc == 343)
    return( 0 );                   /* queue does not exist, so quit */

  if (urc != 0)
    return( -1 );                  /* report any other error */

  /*******************************************************************/
  /* First determine our thread ID (and hence get access to the      */
  /* correct per-thread data.                                        */
  /*******************************************************************/
  DosGetInfoBlocks( &pTib, &pPib );
  ourTid = pTib->tib_ptib2->tib2_ultid;
#endif

#ifdef IC_WIN
  /*******************************************************************/
  /* Attempt to access the printf mailslot ...                       */
  /* If it does not exist, return                                    */
  /*******************************************************************/
  if (hMailSlot == INVALID_HANDLE_VALUE)
     hMailSlot = CreateFile( PRINTFMSNAME,
                             GENERIC_WRITE,
                             FILE_SHARE_READ,
                             (LPSECURITY_ATTRIBUTES)NULL,
                             OPEN_EXISTING,
                             FILE_ATTRIBUTE_NORMAL,
                             (HANDLE)NULL );

  if (hMailSlot == INVALID_HANDLE_VALUE)
    return( 0 );

  /*******************************************************************/
  /* First determine our thread ID (and hence get access to the      */
  /* correct per-thread data.                                        */
  /*******************************************************************/
  ourTid = GetCurrentThreadId();
  /* use a hash table since GetCurrentThreadID can return any long */
  int index = ourTid % (PRINTFTHREADS + 1);
  int count = PRINTFTHREADS + 1;
  while (count-- > 0 &&
         winThreadID[index] != ourTid && winThreadID[index] != 0)
  {
     if (++index > PRINTFTHREADS) index = 0;
  }
  if (count == 0)
     ourTid = PRINTFTHREADS + 1;
  else
  {
     winThreadID[index] = ourTid;
     ourTid = index;
  }

#endif

#ifdef __OS400__
  ourTid = 1;                   // no threading for OS400 yet
#endif

  if (ourTid > PRINTFTHREADS)      /* too many threads .. */
    return( 0 );                   /* .. so quit, quietly */

  pThread = threads[ourTid];       /* copy to local pointer */

  /*******************************************************************/
  /* If per-thread struct hasn't been allocated for this thread,     */
  /* do so here, and initialize it ...                               */
  /*                                                                 */
  /* Note:  It is never freed, once allocated, as PRINTF is never    */
  /* notified of the end-of-thread condition.                        */
  /*******************************************************************/
  if (pThread == NULL)
  {
    pThread = (struct perthread *)malloc(sizeof(struct perthread));
    if (pThread == NULL)
      return( -1 );                       /* out of memory -- return error */

    threads[ourTid] = pThread;            /* save for future calls */
    strcpy(pThread->line, PRINTFID);      /* initialize: line.. */
    pThread->lineIndex = PRINTFIDSIZE - 1;/* ..where next char */
    pThread->tideMark = PRINTFIDSIZE - 2; /* ..rightmost char */
    pThread->bell = FALSE;                /* ..if line has bell */

#ifndef __OS400__
    if (ourPid == 0)
    {
      #ifdef IC_PM
        ourPid = pPib->pib_ulpid;         /* save PID for all to use */
      #endif
      #ifdef IC_WIN
        ourPid = GetCurrentProcessId();   /* save PID for all to use */
      #endif
    }
#endif
  }

  /*******************************************************************/
  /* Block for declarations - only needed if queue/mailslot exists.  */
  /*******************************************************************/
  {
    long  count;                      /* count of characters formatted */
    char  buffer[IC_PRINTFMAXLEN + 1];/* formatting area */
    long  i, newIndex;                /* work */
    char  ch;                         /* .. */
    va_list argptr;                   /* -> variable argument list */

    va_start(argptr, f);              /* get pointer to argument list */
    count = vsprintf(buffer, f, argptr);

    if (count < 0)
      return( count - 1000 );         /* bad start */

    if (count > IC_PRINTFMAXLEN)
    {
      /***************************************************************/
      /* Disaster -- we are probably "dead", but just in case we are */
      /* not, carry on with truncated data.                          */
      /***************************************************************/
      count = IC_PRINTFMAXLEN;
    }

    buffer[count] = '\0';             /* ensure terminated */

    /*****************************************************************/
    /* OK, ready to go with the data now in BUFFER.  We copy from    */
    /* the formatted string to the output (line) buffer, taking note */
    /* of certain control characters and sending a line the queue/   */
    /* mailslot whenever we see a LF control, or when the line fills */
    /* (causing a forced break).                                     */
    /*****************************************************************/
    for (i=0; ;i++)
    {
      ch = buffer[i];
      if (!ch)
        break;

      switch(ch)
      {
        case '\r':                    /* carriage return */
                                      /* back to start of line */
          pThread->lineIndex = PRINTFIDSIZE - 1;
          break;
        case '\n':                    /* new line */
        case '\f':                    /* form feed */
          rc = printf_(pThread);      /* print a line */
          if (rc != 0)
            return rc;                /* error */
          break;
        case '\t':                    /* tab */
                                      /* offset into data */
          newIndex = pThread->lineIndex - PRINTFIDSIZE + 1;
                                      /* new index requested */
          newIndex = pThread->lineIndex + 5 - newIndex % 5;
          if (newIndex >= PRINTFMAXBUF)
            newIndex=PRINTFMAXBUF;    /* clamp */

          for (; pThread->lineIndex < newIndex; pThread->lineIndex++)
          {
            /*********************************************************/
            /* if beyond current end, reset tideMark to lineIndex    */
            /*********************************************************/
            if (pThread->lineIndex > pThread->tideMark)
            {
                                      /* add space */
              pThread->line[pThread->lineIndex] = ' ';
              pThread->tideMark = pThread->lineIndex;
            }
          }
          break;
        case '\v':                    /* vertical tab */
          /* ignore it */
          break;
        case '\b':                    /* backspace */
          pThread->lineIndex = max(pThread->lineIndex - 1, PRINTFIDSIZE);
          break;
        case '\a':                   /* alert (bell) */
          pThread->bell = TRUE;
          break;
        default:                     /* ordinary character */
          pThread->line[pThread->lineIndex] = ch;
          /***********************************************************/
          /* if lineIndex is rightmost character, reset tideMark     */
          /***********************************************************/
          if (pThread->lineIndex > pThread->tideMark)
            pThread->tideMark = pThread->lineIndex;

          pThread->lineIndex++;       /* step for next */
      } /* end switch */

      if (pThread->lineIndex >= PRINTFMAXBUF)
      {
        rc = printf_(pThread);        /* print a line */
        if (rc != 0)
          return( rc );               /* error */
      }

    } /* end for */

    return( count );                  /* all formatted data processed */
  } /* end block */

} /* imprintf */


/*------------------------------------------------------------------------------
| printf - Local C function that is used to do the actual writes to the        |
|          queue (PM) or mailslot (Windows).                                   |
|                                                                              |
------------------------------------------------------------------------------*/
int printf_(struct perthread *pThread)
{
  long    lSize;                   /* total size of output data */
  char    *pszSrc;                 /* source pointer */
#ifdef IC_PM
  void    *pShrMem;                /* address of shared memory buffer */
  time_t  timeNow;                 /* holds current time */
#endif

                                   /* add terminator */
  pThread->line[pThread->tideMark + 1] = '\0';
  lSize = pThread->tideMark + 2;   /* total length of data */

  pszSrc = &(pThread->line[0]);    /* pointer to source */

#ifdef IC_PM
  /*******************************************************************/
  /* Get some shared memory that can be given away                   */
  /*******************************************************************/
  if (DosAllocSharedMem( &pShrMem,
                         NULL,
                         (unsigned)lSize,
                         OBJ_GIVEABLE | PAG_WRITE | PAG_COMMIT ))
  {
    DosSleep( 200 );                        /* Failed! Wait 2/10 second       */
    if (DosAllocSharedMem( &pShrMem,        /* and try again                  */
                           NULL,
                           (unsigned)lSize,
                           OBJ_GIVEABLE | PAG_WRITE | PAG_COMMIT ))
      return( -2 );                  /* error */
  }

  strcpy((char *)pShrMem, pszSrc); /* copy the string to shared memory */

  /*******************************************************************/
  /* Give access iff we're in a separate process (vs the server)     */
  /*******************************************************************/
  if (ourPid != servePid)
  {
    if (DosGiveSharedMem( pShrMem,
                          servePid,
                          PAG_READ ))
      return( -3 );                /* error */
  }

  /*******************************************************************/
  /* Write the pointer, size, and timestamp to the queue.            */
  /*******************************************************************/
  if (pThread->bell)
    lSize =- lSize;                /* BELL passed by negation */

  time(&timeNow);                  /* optional - else use 0 */
  if (DosWriteQueue( hQueue,
                     (unsigned)timeNow,
                     (unsigned)lSize,
                     pShrMem,
                     0 ))
  {
      DosFreeMem( pShrMem );
      return( -4 );                /* error */
  }

  /*******************************************************************/
  /* Free this process' access if we're dealing with a separate      */
  /* server process ...                                              */
  /*******************************************************************/
  if (ourPid != servePid)
  {
    if (DosFreeMem( pShrMem ))
      return( -5 );                /* error */
  }
#endif

#ifdef IC_WIN
  DWORD  written;

  /*******************************************************************/
  /* Write the trace message to the mailslot                         */
  /*******************************************************************/
  if (!WriteFile( hMailSlot,
                  pszSrc,
                  (DWORD)lSize,
                  &written,
                  NULL ))
      return( -4 );                /* error */
#endif
#ifdef __OS400__
   // send the information out to the external message queue
   int msgqid;
   char msgkey[4];
   // Replaced unsupported error_code_t with equivalent _TE_ERROR_CODE_T
   _TE_ERROR_CODE_T errinfo;

   // Replaced unsupported byte_provided with BytesProvided
   errinfo.BytesProvided = sizeof(errinfo);
   // Added prefix QMH_ to Message macros
   QMHSNDPM("       ", "                    ", pszSrc, lSize,
   QMH_MSGT_INFO, QMH_INV_MSG_Q_EXT, msgqid, msgkey, &errinfo);
#endif

  /*******************************************************************/
  /* Reset the line buffer and indices                               */
  /*******************************************************************/
  pThread->lineIndex = PRINTFIDSIZE - 1; /* where next char */
  pThread->tideMark = PRINTFIDSIZE - 2;  /* rightmost char */
  pThread->bell = FALSE;                 /* true if line has bell */
  return( 0 );                           /* success! */

} /* printf_ */
#endif
