// Revision: 34 1.6.1.3 source/core/base/ingthrdv.hpp, oss, ioc.v400, 000302  
/*NOSHIP*/
#ifndef _INGTHRDV_
#define _INGTHRDV_
/*******************************************************************************
* FILE NAME: ingthrdv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*      IThreadVariable                                                         *
*      IThreadVariables                                                        *
*      ITLSHandle                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibhandle.hpp>
#include <irefcnt.hpp>
#include <ireslock.hpp>
#include <istring.hpp>
#include <ingthrd.hpp>
#include <ikshsh2.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IWindowList;

#pragma pack(push,4)
#pragma enum(4)

/*******************************************************************************
* The IThreadVariable class stores a thread variable.   The IThreadVariables   *
* collection stores the variables for each thread.  The IThread members        *
* variable and setVariable use this class and the collection of these objects. *
*******************************************************************************/
class IThreadVariable 
{
public:
  IThreadVariable( );
  IThreadVariable( const IThreadVariable& variable );
  IThreadVariable( const IString& key, const IString& value );
  IThreadVariable
    &operator=   ( const IThreadVariable& variable );
  ~IThreadVariable( );
  bool operator < ( const IThreadVariable& variable) const
       { return ( fkey < variable.fkey); }
  bool operator == ( const IThreadVariable& variable) const
       { return ( fkey == variable.fkey); }

IString
   fkey,
   fvalue;
};

/*------------------------------------------------------------------------------
| key                                                                          |
------------------------------------------------------------------------------*/
inline IString const& key( IThreadVariable const& element )
{ return element.fkey; };

/*------------------------------------------------------------------------------
| hash                                                                         |
------------------------------------------------------------------------------*/
inline unsigned long hash (IString const& key, unsigned long h)
{
   // use the default string hash function
   return hash( (const char*)key, h );
}


class ITLSHandle {
/*******************************************************************************
* The ITLSHandle class provides a portable interface to OS specific thread     *
* local storage handles, or slots.                                             *
*                                                                              *
*******************************************************************************/
public:
/*---------------------------------- Value -----------------------------------*/
#ifdef IC_POSIX
typedef pthread_key_t Value;
#endif
#ifdef IC_PM
typedef unsigned long* Value;
#endif
#ifdef IC_WIN
typedef unsigned long Value;
#endif

/*------------------------- Constructors/Destructors ---------------------------
   Use the allocate() function to create an ITLSHandle object.
------------------------------------------------------------------------------*/
  ITLSHandle   ( const ITLSHandle& tlsHandle );
  ITLSHandle
    &operator= ( const ITLSHandle& tlsHandle );


/*----------------------------- Slot Allocation --------------------------------
   In order to use thread local storage, the process must allocate
   an entry, or slot, for the storage.   The slot allocated by this class
   is large enough to hold a pointer.   In both OS/2 and Win32, the number
   of tls slots per process is limited, so applications should be
   conservative in their usage.
   A typical usage would be to call allocate during program initialization
   and use the returned handle to initialize a global variable accessable by
   all threads.  Then, each thread uses the setValue or value functions
   to set or retrieve its copy of the data.
   allocate    -  allocates a new slot for thread local storage for use
                  by this process.
   release     -  returns the slot to the OS.   Accesses to the slot by
                  the process are invalid after this call.
------------------------------------------------------------------------------*/
static  ITLSHandle
  allocate( );
static  void
  release ( ITLSHandle& handle );


/*---------------------------------- Value -------------------------------------
   The value functions provide access to the user defined value stored in
   the thread local storage slot.  You must call allocate() in the current
   process to obtain the handle used on the value functions.
   storage  - retrieves the current value of the tls slot.  If the value
              has not yet been set by this thread, the value is 0.
   setStorage - sets a new value into the tls slot.   The previous value
              is returned.
------------------------------------------------------------------------------*/
void
  *storage   ( ) const;
void
  *setStorage(void* value);

/*------------------------------- Diagnostics --------------------------------*/
unsigned long
  asUnsigned  ( ) const;

/*-------------------------------- Operators ---------------------------------*/
  operator Value ( ) const;


protected:
  ITLSHandle   ( Value value=0 );

private:
/*---------------------------------- Value -----------------------------------*/
Value
  fhandle;
};


/*------------------------------------------------------------------------------
| IThreadVariables                                                             |
|                                                                              |
| Collection of thread variables used for variable/setVariable.                |
------------------------------------------------------------------------------*/
class IThreadVariables : public IKeySetAsHshTable<IThreadVariable, IString> {
public:
  IThreadVariables();
  ~IThreadVariables();
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <ingthrdv.inl>

#endif
