// Revision: 40 1.2.2.2 source/core/base/inotifev.cpp, notification, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: inotifev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in inotifev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <inotifev.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTIFEV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| INotificationEvent::INotificationEvent                                       |
|                                                                              |
------------------------------------------------------------------------------*/
INotificationEvent::INotificationEvent  ( const INotificationId&   id,
                      INotifier&         notifier,
                      bool            hasChanged,
                      const IEventData&  eventData,
                      const IEventData&  observerData)
                 : fInterest(new IInterest(notifier,id)),
				   fAttrChanged(hasChanged),
                   fEvtData(eventData),
                   fObsData(observerData),
                   fExtendedNotificationData(0)
{
}

INotificationEvent::INotificationEvent  (const INotificationEvent& event)
{
  fInterest = new IInterest(event.interest());

  //DK obsolete this
  this->fEvtNotifier = event.fEvtNotifier;
  this->fAttrChanged = event.fAttrChanged;
  this->fEvtData = event.fEvtData;
  this->fObsData = event.fObsData;
  // Note if extendedNotificationData implemented, copy it here
  this->fExtendedNotificationData = 0;
}

//{1}
INotificationEvent::INotificationEvent():
  fEvtNotifier (0),
  fAttrChanged (false),
  fExtendedNotificationData (0),
  fInterest (0)
{}

INotificationEvent::~INotificationEvent ( )
{
  delete fInterest;
}

INotificationEvent& INotificationEvent::operator=( const INotificationEvent& event)
{
  if(this!=&event)
  {
	if (this->fInterest)
	  delete this->fInterest;

	this->fInterest = new IInterest(event.interest());

	//DK Obsolete These
    this->fEvtNotifier = event.fEvtNotifier;
    this->fAttrChanged = event.fAttrChanged;
    this->fEvtData = event.fEvtData;
    this->fObsData = event.fObsData;
    // Note if extendedNotificationData implemented, copy it here
    this->fExtendedNotificationData = 0;
  }
  return *this;
}

//{1}
const IInterest&    INotificationEvent::interest() const
{
  return *fInterest;
}


//{1}
INotificationEvent::INotificationEvent ( const IInterest& theInterest)
{
  fInterest = new IInterest(theInterest);
}

//{1}
INotificationEvent::INotificationEvent(INotifier& theNotifier,const INotificationId& theId)
{
  fInterest = new IInterest(theNotifier,theId);
}


//Polymorphic copy
INotificationEvent* 
INotificationEvent::clone() const {
  return new INotificationEvent (*this);
}

