/*----------------------------------------------------------------------------*/
/* FILE NAME: iperfset.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   Implementation of the class(es):                                         */
/*     IPerformanceSettings                                                   */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <iperfset.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPERFSET_CPP_
  #include <ipagetun.h>
#endif


// This is initialized with the settings that are enabled by default.
// ValidateWindowHandle enabled for debug builds only.
unsigned long IPerformanceSettings::fgSettings =
#ifdef IC_DEVELOP
       IPerformanceSettings::validateWindowHandle |
#endif
#ifdef IC_MOTIF
       IPerformanceSettings::singleThreadedGUI |
#endif
       0;

/*------------------------------------------------------------------------------
| IPerformanceSettings::enableSetting                                          |
------------------------------------------------------------------------------*/
void IPerformanceSettings::enableSetting ( Setting setting, bool enable )
{
   unsigned long newValue = setting;
#ifndef IC_MOTIF
   // Force singleThreadedGUI to be set/reset when singleThreaded is changed.
   // For Motif, leave it alone because GUI is not thread safe.
   if (setting == IPerformanceSettings::singleThreaded)
      newValue |= IPerformanceSettings::singleThreadedGUI;
#endif
   if (enable)
      fgSettings |= newValue;
   else
      fgSettings &= ~newValue;
}

/*------------------------------------------------------------------------------
| IPerformanceSettings::disableSetting                                         |
------------------------------------------------------------------------------*/
void IPerformanceSettings::disableSetting ( Setting setting )
{
   IPerformanceSettings::enableSetting( setting, false );
}
