// ----------------------------------------------------------------------------
// FILE NAME: ipoint.cpp
//
// DESCRIPTION:
//  This file contains the implementation of classes/functions declared
//  in ipoint.hpp.
//
// COPYRIGHT:
//  IBM Open Class Library
//  (C) Copyright International Business Machines Corporation 1992, 1997
//  Licensed Material - Program-Property of IBM - All Rights Reserved.
//  US Government Users Restricted Rights - Use, duplication, or disclosure
//  restricted by GSA ADP Schedule Contract with IBM Corp.
//
// Revision: 56 1.6.1.4 source/core/base/ipoint.cpp, appsupport, ioc.v400, 000302 
// ----------------------------------------------------------------------------

extern "C"
{
    #ifdef IC_PM
        #define INCL_WINWINDOWMGR
    #endif // IC_PM

    #ifdef IC_WIN
        #include <windows.h>
    #endif

    #ifdef IC_PM
        #include <os2.h>
    #endif
}

#ifdef _MSC_VER
// MSVC does not like C headers included with extern "C"
#include <math.h>
#else
extern "C"
{
    #include <math.h>
}
#endif

#include <ipoint.hpp>
#include <ibhandle.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
    #define _IPOINT_CPP_
    #include <ipagetun.h>
#endif

#define MIN(a, b) ( ( (a) <= (b) ) ? (a) : (b) )
#define MAX(a, b) ( ( (a) >= (b) ) ? (a) : (b) )

// ----------------------------------------------------------------------------
//  IPair::asString
// ----------------------------------------------------------------------------
IString IPair::asString() const
{
    IString str(IString("(") +
                IString(coord1()) +
                IString(',') +
                IString(coord2()) +
                IString(')'));
    return str;
}

// ----------------------------------------------------------------------------
//  IPair::asDebugInfo
// ----------------------------------------------------------------------------
IString IPair::asDebugInfo() const
{
    IString result( "IPair(coord1=" );
    result += IString( this->coord1() );
    result += ",coord2=";
    result += IString( this->coord2() );
    result += ")";
    return result;
}

// ----------------------------------------------------------------------------
//  IPair::distanceFrom
// ----------------------------------------------------------------------------
double IPair::distanceFrom(const IPair &aPair) const
{
    long int dx = this->coordCl1 - aPair.coordCl1;
    long int dy = this->coordCl2 - aPair.coordCl2;
    return sqrt((double)(dx * dx + dy * dy));
}


// ----------------------------------------------------------------------------
//  IPair::minimum
// ----------------------------------------------------------------------------
IPair IPair::minimum(const IPair& pair) const
{
    return IPair(MIN(coord1(), pair.coord1()), MIN(coord2(), pair.coord2()));
}


// ----------------------------------------------------------------------------
//  IPair::maximum
// ----------------------------------------------------------------------------
IPair IPair :: maximum(const IPair& pair) const
{
    return IPair(MAX(coord1(), pair.coord1()), MAX(coord2(), pair.coord2()));
}




#ifdef IC_PMWIN
// ----------------------------------------------------------------------------
//  IPoint::asPOINTL
// ----------------------------------------------------------------------------
#ifdef IC_WIN
IPoint::IPoint(const POINT& ptl) :

    IPair(ptl.x, ptl.y)
{
}
#endif	//IC_WIN

IPoint::IPoint(const POINTL& ptl) :

    IPair(ptl.x, ptl.y)
{
}

POINTL IPoint::asPOINTL() const
{
    POINTL ptl = { x(), y() };
    return ptl;
}

// ----------------------------------------------------------------------------
//  ISize::asSIZEL
// ----------------------------------------------------------------------------
SIZEL ISize::asSIZEL ( ) const
{
    SIZEL sizl = { width(), height() };
    return sizl;
}

ISize::ISize(const SIZEL &sizl) :
    IPair(sizl.cx, sizl.cy)
{
}

#ifdef IC_PM
ISize::ISize(const RECTL& rcl) :

    IPair(rcl.xRight - rcl.xLeft, rcl.yTop - rcl.yBottom)
{
}
#endif //IC_PM

#ifdef IC_WIN
ISize::ISize(const RECTL& rcl) :

    IPair(rcl.right - rcl.left, rcl.bottom - rcl.top)
{
}
#endif	//IC_WIN
#endif  //IC_PMWIN
