/*----------------------------------------------------------------------------*/
/* FILE NAME: istring4.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of the IString manipulation        */
/*   operators.                                                               */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
#ifdef __MVS__
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS400__
/*   ????-???                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS2__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __WINDOWS__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _AIX
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"ISTRING4")
    #pragma csect(static,"SSTRING4")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif

#include <istring.hpp>

#include <ibuffer.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTRING4_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IString::operator =                                                          |
|                                                                              |
| Assign the argument string to this IString by copying its contents.  Note    |
| that since IStrings can be constructed from any basic type, any basic type   |
| can be assigned to an IString object.                                        |
|                                                                              |
| Notes:                                                                       |
|   1. An IString is constructed from this object's IBuffer.  This will        |
|      ensure the previously referenced IBuffer's reference count gets         |
|      decremented at the appropriate time.  This technique is used            |
|      throughout the IString implementation; it will rarely be noted          |
|      elsewhere.                                                              |
|                                                                              |
|   2. We add the reference to the source IBuffer prior to removing            |
|      the reference to the receiver's IBuffer (which happens at exit          |
|      during deletion of "old"); this ensures the proper result when          |
|      assigning an IString to this->                                          |
------------------------------------------------------------------------------*/
// See istring.inl

/*------------------------------------------------------------------------------
| IString::operator ~                                                          |
|                                                                              |
| Create the one's complement of the receiver by negating each byte.           |
------------------------------------------------------------------------------*/
IString IString :: operator ~ ( ) const
  {
  unsigned len                 = this->length();
  const unsigned char *pSource = (unsigned char *) this->data();
  IString result( 0, len );
  unsigned char *pDest = (unsigned char *) result.data();
  while ( len-- )
     *pDest++ = ~*pSource++;
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator +                                                          |
|                                                                              |
| Concatenate the argument string to the receiver.  Each of these is           |
| implemented using the constructor accepting two buffers                      |
| (as pointer/length pairs).                                                   |
|                                                                              |
| Notes:                                                                       |
|   1. These functions (along with the other const operators) return           |
|      the result of the operation in a new IString.                           |
------------------------------------------------------------------------------*/
IString IString :: operator + ( const IString &aString ) const
  {
  IString result( this->data(), this->length(),
                  aString.data(), aString.length() );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator +                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator + ( const char *pString ) const
  {
  IString result( this->data(), this->length(),
                  pString, lengthOf( pString ) );
  return result;
  }

/*------------------------------------------------------------------------------
| operator +                                                                   |
------------------------------------------------------------------------------*/
IString operator + ( const char    *pString,
                            const IString &aString )
  {
  IString result( pString, IString::lengthOf( pString ),
                  (char*) aString, aString.length() );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator +=                                                         |
|                                                                              |
| Concatenate the argument string to the receiver's contents.  These are all   |
| implemented using the IBuffer->initBuffer function.  If the argument string  |
| is null, simply return the receiver unmodified.                              |
------------------------------------------------------------------------------*/
IString &
IString::operator+=(const IString &aString)
{
    unsigned int a_strlen = aString.length();

    if (a_strlen > 0) {
        
        unsigned int my_strlen = this->length();
        unsigned int allocate_size = (my_strlen | 31) + 1;
        unsigned int reserved_space = allocate_size - my_strlen;

        if (this->buffer()->count() > 1 ||
            ! IBuffer::is32bytesAligned || 
            my_strlen == 0 || a_strlen >= reserved_space) {
            IString old( this->buffer() );
            this->initBuffer( this->data(), my_strlen,
                              aString.data(), a_strlen,
                              0, 0,
                              0 );
        } else {
            // + 1 for the last null byte
            memcpy(this->data() + my_strlen, aString.data(), a_strlen + 1);
            this->buffer()->len = my_strlen + a_strlen;
        }
    }
    return *this;
}

/*------------------------------------------------------------------------------
| IString::operator +=                                                         |
------------------------------------------------------------------------------*/
IString &
IString::operator+=(const char *pString)
{
    unsigned int a_strlen = lengthOf(pString);
    
    if (a_strlen > 0) {

        unsigned int my_strlen = this->length();
        unsigned int allocate_size = (my_strlen | 31) + 1;
        unsigned int reserved_space = allocate_size - my_strlen;

        if (this->buffer()->count() > 1 ||
            ! IBuffer::is32bytesAligned ||
            my_strlen == 0 || a_strlen >= reserved_space) {
            IString old( this->buffer() );
            this->initBuffer( this->data(), my_strlen,
                              pString, a_strlen,
                              0, 0,
                              0 );
        } else {
            // + 1 for the last null byte
            memcpy(this->data() + my_strlen, pString, a_strlen + 1);
            this->buffer()->len = my_strlen + a_strlen;
        }
    }

    return *this;
}

/*------------------------------------------------------------------------------
| IString::operator &                                                          |
| IString::operator |                                                          |
| IString::operator ^                                                          |
|                                                                              |
| These functions return a IString corresponding to the receiver's contents    |
| operated on via one of the bit-wise operators & (and), | (or), or            |
| ^ (exclusive or) and the provided argument.  If the argument string is       |
| shorter than the receiver, then that string is replicated in order to        |
| perform the operation.                                                       |
|                                                                              |
| These are all implemented by constructing a result string from the receiver  |
| and invoking the protected member function applyBitOp().  The arguments to   |
| that function are the argument string and the bitOpFunction (either and, or, |
| or exclusiveOr).                                                             |
------------------------------------------------------------------------------*/
IString IString :: operator & ( const IString &aString ) const
  {
  IString result(*this);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     bitAnd );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator &                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator & ( const char *pString ) const
  {
  IString result(*this);
  result.applyBitOp( pString,
                     lengthOf( pString ),
                     bitAnd );
  return result;
  }

/*------------------------------------------------------------------------------
| operator &                                                                   |
------------------------------------------------------------------------------*/
IString operator & ( const char    *pString,
                     const IString &aString )
  {
  IString result(pString);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     IString::bitAnd );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator |                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator | ( const IString &aString ) const
  {
  IString result(*this);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     bitOr );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator |                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator | ( const char *pString ) const
  {
  IString result(*this);
  result.applyBitOp( pString,
                     lengthOf( pString ),
                     bitOr );
  return result;
  }

/*------------------------------------------------------------------------------
| operator |                                                                   |
------------------------------------------------------------------------------*/
IString operator | ( const char   *pString,
                     const IString &aString )
  {
  IString result(pString);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     IString::bitOr );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator ^                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator ^ ( const IString &aString ) const
  {
  IString result(*this);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     bitExclusiveOr );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator ^                                                          |
------------------------------------------------------------------------------*/
IString IString :: operator ^ ( const char *pString ) const
  {
  IString result(*this);
  result.applyBitOp( pString,
                     lengthOf( pString ),
                     bitExclusiveOr );
  return result;
  }

/*------------------------------------------------------------------------------
| operator ^                                                                   |
------------------------------------------------------------------------------*/
IString operator ^ ( const char   *pString,
                     const IString &aString )
  {
  IString result(pString);
  result.applyBitOp( aString.data(),
                     aString.length(),
                     IString::bitExclusiveOr );
  return result;
  }

/*------------------------------------------------------------------------------
| IString::operator &=                                                         |
| IString::operator |=                                                         |
| IString::operator ^=                                                         |
|                                                                              |
| The receiver's contents are operated on via a bit-wise operator and the      |
| argument string (which can be either a IString object or a char*).  These    |
| are all implemented by invoking the protected member function applyBitOp()   |
| with the appropriate BitOp (either "and", "or", or "exclusiveOr."            |
------------------------------------------------------------------------------*/
IString &IString :: operator &= ( const IString &aString )
  {
  this->applyBitOp( aString.data(),
                   aString.length(),
                   bitAnd );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::operator &=                                                         |
------------------------------------------------------------------------------*/
IString &IString :: operator &= ( const char *pString )
  {
  this->applyBitOp( pString,
                   lengthOf( pString ),
                   bitAnd );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::operator |=                                                         |
------------------------------------------------------------------------------*/
IString &IString :: operator |= ( const IString &aString )
  {
  this->applyBitOp( aString.data(),
                   aString.length(),
                   bitOr );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::operator |=                                                         |
------------------------------------------------------------------------------*/
IString &IString :: operator |= ( const char *pString )
  {
  this->applyBitOp( pString,
                   lengthOf( pString ),
                   bitOr );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::operator ^=                                                         |
------------------------------------------------------------------------------*/
IString &IString :: operator ^= ( const IString &aString )
  {
  this->applyBitOp( aString.data(),
                   aString.length(),
                   bitExclusiveOr );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::operator ^=                                                         |
------------------------------------------------------------------------------*/
IString &IString :: operator ^= ( const char *pString )
  {
  this->applyBitOp( pString,
                   lengthOf( pString ),
                   bitExclusiveOr );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::applyBitOp                                                          |
|                                                                              |
| If the argument is null, then the receiver is returned unmodified.           |
| Otherwise, the receiver is operated on according to the length of            |
| the argument:                                                                |
|                                                                              |
| if 1, then each character of the receiver is operated on with                |
| the single byte of the argument                                              |
|                                                                              |
| else, the receiver is broken up into chunks the size of the                  |
| argument and then each byte of the chunk is operated on with                 |
| the corresponding byte of the argument.                                      |
|                                                                              |
| To as great an extent as possible, the switch on operator type is            |
| performed outside of the loops that perform the operations.                  |
------------------------------------------------------------------------------*/
IString &IString :: applyBitOp ( const char  *pArg,
                                 unsigned     argLen,
                                 BitOperator  op )
  {
  if ( argLen != 0 )
    {
    this->prepareToChange();
    char     *pt = this->data();
    unsigned  n  = this->length();
    if ( argLen == 1 )
      switch ( op )
        {
        case bitAnd:
          while ( n-- )
            *pt++ &= *pArg;
          break;
        case bitOr:
          while ( n-- )
            *pt++ |= *pArg;
          break;
        case bitExclusiveOr:
          while ( n-- )
            *pt++ ^= *pArg;
        }
    else
      while ( n )
        {
        const char *pa = pArg;
        unsigned    m  = ( argLen < n ) ? argLen : n;
        n -= m;
        switch ( op )
          {
          case bitAnd:
            while ( m-- )
              *pt++ &= *pa++;
            break;
          case bitOr:
            while ( m-- )
              *pt++ |= *pa++;
            break;
          case bitExclusiveOr:
            while ( m-- )
              *pt++ ^= *pa++;
          }
        }
    }
  return *this;
  }
