/*----------------------------------------------------------------------------*/
/* FILE NAME: istring7.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of the editing functions of        */
/*   the class IString.                                                       */
/*                                                                            */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
#ifdef __MVS__
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS400__
/*   ????-???                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS2__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __WINDOWS__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _AIX
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"ISTRING7")
    #pragma csect(static,"SSTRING7")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif


#if 0
// -- Remove following comments in the shipped code
/*******************************************************************************
*       -- Center(), Change(), Insert(), leftJustify() functions save the      *
*          old string using IString(IBuffer) constructor which does not        *
*          increment the refCount on the string. These functions expect the    *
*          corresponding IBuffer functions to increment the refCount. This     *
*          works fine as long as an overflow exception is not raised from      *
*          corresponding IBuffer function. In the case of exception , the      *
*          refCount is not incremented and hence when the string destructor    *
*          is called it abends because the string was not constructed properly.*
*          This problem surfaced up during SOM enablement but it is actually   *
*          the problem in the C++ code. C++ does not complain if the object    *
*          is deleted twice provided the stack remains intact.                 *
*                                                                              *
*          This problem amy be there with other IString functions but has      *
*          surfaced up with our existing set of test cases                     *
*                                                                              *
* Tactical Solution -- Increment the refCount after constructing the string    *
*                      and just before return decrement the count.             *
*                                                                              *
*  CHANGES IN THE CODE ARE MARKED WITH                                         *
*            // dummy add ref                                                  *
*            // remove dummy add ref                                           *
*                                                                              *
* Long Term Solution -- Construct the string using version of constructor which*
*                       increment the refCount and modify the corresponding    *
*                       IBuffer functions so that the calling IString code     *
*                       is not dependent upon IBuffer functions to increment   *
*                       the refCount(). Requires more investigation into the   *
*                       interfaces of IBuffer code.                            *
*                                                                              *
*                                                                              *
*******************************************************************************/
#endif

extern "C"
  {
  #include <ctype.h>
  }

#include <istring.hpp>

#include <ibuffer.hpp>

#include <istrtest.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTRING7_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IString::center                                                              |
------------------------------------------------------------------------------*/
IString & IString :: center ( unsigned newLength,
                              char      padCharacter )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref

  this->setBuffer( this->buffer()->center( newLength, padCharacter ) );

  old.buffer()->removeRef();  // remove dummy add ref

  return *this;

  }

/*------------------------------------------------------------------------------
| IString::center                                                              |
------------------------------------------------------------------------------*/
IString IString :: center ( const IString &aString,
                unsigned        newLength,
                char            padCharacter )
  {
  return IString(aString).center( newLength, padCharacter );
  }


/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString & IString :: change ( const char *pPattern,
                              unsigned    patternLen,
                              const char *pReplacement,
                              unsigned    replacementLen,
                              unsigned    startPos,
                              unsigned    numChanges )
   {
   IString old( this->buffer() );

   old.buffer()->addRef();    // dummy add ref
   this->setBuffer( this->buffer() -> change( pPattern,
                                              patternLen,
                                              pReplacement,
                                              replacementLen,
                                              startPos,
                                              numChanges ) );
   old.buffer()->removeRef();  // remove dummy add ref

   return *this;
   }

/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString IString :: change ( const IString &aString,
                            const IString &aPattern,
                            const IString &aReplacement,
                            unsigned       startPos,
                            unsigned       numChanges )
  {
  return IString(aString).change( aPattern,
                                  aReplacement,
                                  startPos,
                                  numChanges );
  }

/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString IString :: change ( const IString &aString,
                            const char    *pPattern,
                            const char    *pReplacement,
                            unsigned       startPos,
                            unsigned       numChanges )
  {
  return IString(aString).change( pPattern,
                                  pReplacement,
                                  startPos,
                                  numChanges );
  }

/*------------------------------------------------------------------------------
| IString::copy                                                                |
------------------------------------------------------------------------------*/
IString& IString :: copy ( unsigned numCopies )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref

  this->setBuffer( this->buffer()->copy( numCopies ) );

  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::insert                                                              |
------------------------------------------------------------------------------*/
IString & IString :: insert ( const char *pString,
                              unsigned    len,
                              unsigned    index,
                              char        padCharacter )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref

  this->setBuffer( this->buffer()->insert( pString, len,
                                           index,
                                           padCharacter ) );

  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::leftJustify                                                         |
------------------------------------------------------------------------------*/
IString & IString :: leftJustify ( unsigned newLength,
                                   char     padCharacter )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref

  this->setBuffer( this->buffer()->leftJustify( newLength,
                                                padCharacter ) );

  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::lowerCase                                                           |
------------------------------------------------------------------------------*/
IString& IString :: lowerCase ( )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref
  this->setBuffer( this->buffer()->lowerCase() );
  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::overlayWith                                                         |
------------------------------------------------------------------------------*/
IString& IString :: overlayWith ( const char *pOverlay,
                                  unsigned    len,
                                  unsigned    index,
                                  char        padCharacter )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref
  this->setBuffer( this->buffer()->overlayWith( pOverlay,
                                                len,
                                                index,
                                                padCharacter ) );
  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::remove                                                              |
|                                                                              |
| Notes:                                                                       |
|   1. The member function IString::remove is implemented in istring0.cpp.     |
------------------------------------------------------------------------------*/
// See istring0.cpp

/*------------------------------------------------------------------------------
| IString::reverse                                                             |
------------------------------------------------------------------------------*/
IString& IString :: reverse ( )
  {
  IString old( this->buffer() );

  old.buffer()->addRef();    // dummy add ref
  this->setBuffer( this->buffer()->reverse() );
  old.buffer()->removeRef();  // remove dummy add ref

  return *this;
  }

/*------------------------------------------------------------------------------
| IString::rightJustify                                                        |
|                                                                              |
| Notes:                                                                       |
|   1. The member function IString::rightJustify is implemented in             |
|      istring0.cpp.                                                           |
------------------------------------------------------------------------------*/
// See istring0.cpp

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString & IString :: strip ( )
  {
  return this->strip( IStringTest( isspace ), IStringEnum::both );
  }

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString& IString :: strip ( const char             *pString,
                            unsigned                len,
                            IStringEnum::StripMode  mode )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->strip( pString, len, mode ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString& IString :: strip ( const IStringTest      &aTest,
                            IStringEnum::StripMode  mode )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->strip( aTest, mode ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::translate                                                           |
------------------------------------------------------------------------------*/
IString & IString :: translate ( const char *pInputChars,
                                 unsigned    inputLen,
                                 const char *pOutputChars,
                                 unsigned    outputLen,
                                 char        padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->translate( pInputChars, inputLen,
                                              pOutputChars, outputLen,
                                              padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::upperCase                                                           |
------------------------------------------------------------------------------*/
IString & IString :: upperCase ( )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->upperCase() );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::upperCase                                                           |
------------------------------------------------------------------------------*/
IString IString :: upperCase ( const IString &aString )
  {
  return IString(aString).upperCase();
  }
