/*----------------------------------------------------------------------------*/
/* FILE NAME: istring8.cpp                                                    */
/*                                                                            */
/* DESCRIPTION:                                                               */
/*   This file contains the implementation of the word manipulation           */
/*   functions of the class IString.                                          */
/*                                                                            */
/* COPYRIGHT:                                                                 */
/*   IBM Open Class Library                                                   */
/*   Licensed Materials - Property of IBM                                     */
/*                                                                            */
#ifdef __MVS__
/*   5645-001                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS400__
/*   ????-???                                                                 */
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __OS2__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __WINDOWS__
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _AIX
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef _sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
#ifdef __sparc
/*   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.           */
#endif
/*                                                                            */
/*   US Government Users Restricted Rights - Use, duplication, or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifdef __MVS__
    #pragma csect(code,"ISTRING8")
    #pragma csect(static,"SSTRING8")
    #pragma comment (copyright, \
    "Licensed Materials - Property of IBM  5645-001 (C) \
    Copyright IBM Corp. 1992, 1997  All Rights Reserved. \
    US Government Users Restricted Rights - Use, duplication, or \
    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.")
#endif
#pragma priority( -2147482024 )

extern "C"
  {
  #include <string.h>
  }

#include <istring.hpp>

#include <ibuffer.hpp>

#include <iostream.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTRING8_CPP_
  #include <ipagetun.h>
#endif


// Class to record occurrence of a word:
struct WordOccurrence {
WordOccurrence
 *pNext;
unsigned
  pos,
  len;
WordOccurrence ( unsigned p )
  : pos( p ), len( 0 ), pNext( 0 ) { }
WordOccurrence ( const WordOccurrence& that)
  : pos(that.pos), len(that.len), pNext(0) {}
~WordOccurrence ( )
  { if ( pNext != 0 ) delete pNext; };
};

// Class to hold length/position for all words:
struct Words {
unsigned
  count;
WordOccurrence
 *p;
Words ( const IString& aString );
~Words ( )
  { delete this->p; }
WordOccurrence
 &operator[] ( unsigned n ) const;
};

/*------------------------------------------------------------------------------
| Words::operator[]                                                            |
|                                                                              |
| The wordOccurance is returned by the operator []                             |
|                                                                              |
| Notes:                                                                       |
|   1) Index is 1 based.                                                       |
|   2) If the index is greater than the number of words then return the head   |
|      of the linked list which has a pos of 0.                                |
------------------------------------------------------------------------------*/
inline WordOccurrence & Words :: operator[] ( unsigned n ) const
  {
  if ( n > this->count ) n = 0;
  WordOccurrence *result = this->p;
  while ( n-- )
    result = result->pNext;
  return *result;
  }

/*------------------------------------------------------------------------------
| Words::Words                                                                 |
|                                                                              |
| The constructor for the Words class parses the argument IString for          |
| whitespace-delimited "words."  The resulting Words object contains a count   |
| of the number of words and a pointer to an array of WordOccurrence objects.  |
| Each Word object specifies the starting index of the word in the receiver    |
| and the length of the word.                                                  |
|                                                                              |
|   A word is parsed by:                                                       |
|     1. Looking for the first non-whitespace character (via the               |
|        function indexOfAnyBut()).                                            |
|     2. Recording this position as the start of the word.                     |
|     3. Scanning for the next whitespace character (via                       |
|        indexOfAnyOf()), or, the end of the argument IString.                 |
|     4. Calculating from this the length of the word.                         |
|     5. Performing steps 1-4 until all the words have been parsed.            |
|                                                                              |
|   During this process, the position and length of each word are              |
|   recorded in a chain of WordOccurrence objects.                             |
------------------------------------------------------------------------------*/
static const char whiteSpace[] = "\t\n\v\f\r ";

Words :: Words ( const IString &aString )
  : p( new WordOccurrence( 0 ) ), count( 0 )
  {
  WordOccurrence *pLast = this->p;
  unsigned pos = 1;
  while ( pos != 0 )
    {
    pos = aString.indexOfAnyBut( whiteSpace, pos );
    if ( pos != 0 )
      {
      this->count++;
      pLast->pNext = new WordOccurrence( pos );
      pLast = pLast->pNext;
      unsigned j = aString.indexOfAnyOf( whiteSpace, pos );
      if ( j == 0 )
        {
        j = aString.size() - pos + 1;
        pos = 0;
        }
      else
        {
        j   -= pos;
        pos += j;
        }
      pLast->len = j;
      }
    }
  };

/*------------------------------------------------------------------------------
| IString::removeWords                                                         |
|                                                                              |
| First, the position of the first word to be deleted is calculated via a call |
| to indexOfWord().  If the word is found, then the position of the next word  |
| that would not be deleted is calculated (again, via indexOfWord()), and the  |
| substring deleted via a call to remove().                                    |
------------------------------------------------------------------------------*/
IString & IString :: removeWords ( unsigned firstWord,
                                   unsigned numWords )
  {
  if ( numWords != 0 )
    {
    unsigned startIndex = this->indexOfWord( firstWord, 1, 0 );
    if ( startIndex != 0 )
      {
      // Get index of first remaining word:
      unsigned stopIndex = this->indexOfWord( firstWord + numWords,
                                              startIndex,
                                              firstWord - 1 );
      if ( !stopIndex )
        stopIndex = this->length() + 1;
      this->remove( startIndex, stopIndex - startIndex );
      }
    }
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::indexOfPhrase                                                       |
|                                                                              |
| Simply returns the character index of the phrase as determined by            |
| IString::findPhrase().                                                       |
------------------------------------------------------------------------------*/
// See istring.inl

/*------------------------------------------------------------------------------
| IString::findPhrase                                                          |
|                                                                              |
| This function first parses the words in both the phrase and the receiver.    |
| It then compares a word sequence from the receiver to the phrase, starting   |
| at the startWord-th word of the receiver. If all the words in the phrase are |
| matched, then the position of the current receiver word sequence is returned |
| as the result.                                                               |
|                                                                              |
| Otherwise, the next word in the receiver is advanced to and this sequence    |
| compared to the phrase.  If all possible word sequences in the receiver are  |
| tested and fail, then 0 is returned.  All word compares are via memcmp() and |
| are only done if the word lengths compare.                                   |
|                                                                              |
| The IndexType argument determines whether the returned result is the index   |
| within the string, or, the word index.                                       |
------------------------------------------------------------------------------*/
unsigned IString :: findPhrase ( const IString &aPhrase,
                                 unsigned       startWord,
                                 IndexType      charOrWord ) const
  {
  // Parse receiver words.
  Words stringWords( *this );
  // Parse phrase words.
  Words phraseWords( aPhrase );
  // Start at specified word.
  unsigned int stringIndex = startWord;
  // Search till we run off end of receiver (or till match found):
#if defined( IC_MVS400) && defined(IC_NLS)
  if (IString::isInternationalized()) {
#endif
    while ( stringIndex + phraseWords.count - 1 <= stringWords.count )
      {
      unsigned j = 0; // Index into phrase.
      while ( ++j <= phraseWords.count )
        {
#if defined(IC_MVS400) && defined(IC_NLS)
        IString stringWord(this->data() + stringWords[stringIndex+j-1].pos - 1,
                           stringWords[stringIndex+j-1].len);
        IString phraseWord(aPhrase.data() + phraseWords[j].pos - 1,
                           phraseWords[j].len);
        if (stringWord != phraseWord)
          break;
#else
		WordOccurrence	stringWord = stringWords[stringIndex + j - 1];
		WordOccurrence	phraseWord = phraseWords[j];
        
        if ( stringWord.len != phraseWord.len )
          break;

		char*	thisChars = this->data();
		char*	thoseChars = aPhrase.data();
		thisChars += stringWord.pos - 1;
		thoseChars += phraseWord.pos - 1;
        if ( memcmp( thisChars, thoseChars, phraseWord.len ) != 0 )
           break;
#endif
        }
      if ( j > phraseWords.count )
        // Loop exit completed, phrase matched at stringIndex.
        return ( ( charOrWord == charIndex )
                   ? stringWords[ stringIndex ].pos
                   : stringIndex );
      else
        // Loop exit was due to mismatch, advance index.
        stringIndex++;
      }
#if defined( IC_MVS400) && defined(IC_NLS)
  } else {
    while ( stringIndex + phraseWords.count - 1 <= stringWords.count )
      {
      unsigned j = 0; // Index into phrase.
      while ( ++j <= phraseWords.count )
        {
        if ( stringWords[ stringIndex + j - 1 ].len
             !=
             phraseWords[ j ].len )
          break;
        if ( memcmp( this->data() + stringWords[ stringIndex + j - 1 ].pos - 1,
                     aPhrase.data() + phraseWords[ j ].pos - 1,
                     phraseWords[ j ].len ) != 0 )
           break;
        }
      if ( j > phraseWords.count )
        // Loop exit completed, phrase matched at stringIndex.
        return ( ( charOrWord == charIndex )
                   ? stringWords[ stringIndex ].pos
                   : stringIndex );
      else
        // Loop exit was due to mismatch, advance index.
        stringIndex++;
      }
  }
#endif
  // No match was found.
  return 0;
  }

/*------------------------------------------------------------------------------
| IString::indexOfWord                                                         |
|                                                                              |
| Simply returns the result of evaluating the protected member function of the |
| same name that accepts additional arguments for the position at which to     |
| start searching and the number of words occurring prior to this position.    |
------------------------------------------------------------------------------*/
unsigned IString :: indexOfWord ( unsigned wordNumber,
                                  unsigned startPos,
                                  unsigned numWords ) const
  {
  unsigned result = 0;
  while ( startPos > 0
          &&
          result == 0 )
    {
    // Skip to start of next word:
    startPos = this->indexOfAnyBut( whiteSpace, startPos );
    // See if this is the word we are looking for:
    if ( startPos != 0 )
      if ( ++numWords == wordNumber )
        // This is the word we are looking for:
        result = startPos;
      else
        // Skip to end of this word:
        startPos = this->indexOfAnyOf( whiteSpace, startPos );
    }
  return result;
  }

/*------------------------------------------------------------------------------
| IString::lengthOfWord                                                        |
|                                                                              |
| First, the index of the specified word is determined via a call to           |
| indexOfWord().  Then, the receiver is scanned starting at this location for  |
| the next whiteSpace character (or the end of the receiver) via               |
| indexOfAnyOf().  The length of the word is calculated using the results of   |
| these two functions.                                                         |
------------------------------------------------------------------------------*/
unsigned IString :: lengthOfWord ( unsigned wordNumber ) const
  {
  unsigned len = 0;
  unsigned pos = this->indexOfWord( wordNumber, 1, 0 );
  if ( pos != 0 )
    {
    unsigned end = this->indexOfAnyOf( whiteSpace, pos + 1 );
    if ( end != 0 )
       len = end - pos;
    else
       len = this->length() - pos + 1;
    }
  return len;
  }

/*------------------------------------------------------------------------------
| IString::numWords                                                            |
|                                                                              |
| Simply parse the words in the receiver and return the resulting count.       |
------------------------------------------------------------------------------*/
unsigned IString :: numWords ( ) const
  {
  Words
    myWords(*this);
  unsigned
    result = myWords.count;
  return result;
  }

/*------------------------------------------------------------------------------
| IString::wordIndexOfPhrase                                                   |
|                                                                              |
| Invoke findPhrase and ask for the result as the word index.                  |
------------------------------------------------------------------------------*/
// See istring.inl

/*------------------------------------------------------------------------------
| IString::space                                                               |
|                                                                              |
| First, the words in the receiver are parsed.  Then, space is allocated for   |
| the resulting string (with the specified number of blanks between words).    |
| Finally, the words are processed, one at a time, inserting the requisite     |
| number of blanks between them.                                               |
------------------------------------------------------------------------------*/
IString & IString :: space ( unsigned numSpaces,
                             char     spaceChar )
  {
  if ( this->length() != 0 )
    {
    Words myWords( *this );
    if ( myWords.count != 0 )
      {
      IString old( this->buffer() );
      unsigned i = 1,  // Word index.
               j = 0;  // Total length (excluding first word).
      // Calculate length of spaces/words:
      while ( i++ < myWords.count )
        j = IBuffer::checkAddition( j,
                                    IBuffer::checkAddition( numSpaces,
                                                           myWords[ i ].len ) );
      // Construct new buffer, note first word is copied automatically:
      this->setBuffer( this->buffer()->
                         newBuffer( this->data() + myWords[ 1 ].pos - 1,
                                    myWords[ 1 ].len,
                                    0, j,
                                    0, 0,
                                    spaceChar ) );
      i = 1;
      j = 0;
      while ( i++ < myWords.count )
        // 1. Add length of previous word, plus numSpaces, to j
        // 2. Copy this word from pOld -> *this:
        memcpy( this->data() + ( j += myWords[ i - 1 ].len + numSpaces ),
                old.data() + myWords[ i ].pos - 1,
                myWords[ i ].len );
      }
    else
      *this = IString::null;
    }
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::word                                                                |
|                                                                              |
| The position of the specified word is obtained via a call to indexOfWord().  |
| Then, the length of this word is deduced via a call to indexOfAnyOf()        |
| (using whiteSpace as the set of valid characters).  Finally, a result string |
| is built from the portion of the receiver between these two positions.       |
------------------------------------------------------------------------------*/
IString IString :: word ( unsigned wordNumber ) const
  {
  // Treat word zero like word 1:
  if ( wordNumber == 0 ) wordNumber = 1;
  unsigned start = this->indexOfWord( wordNumber );
  unsigned end   = 0;
  if ( start != 0 )
    {
    end = this->indexOfAnyOf( whiteSpace, start + 1 );
    if ( end == 0 )
      end = this->length() + 1;
    }
  if ( end > start )
    return IString( this->data() + start - 1, end - start );
  else
     return IString::null;
  }

/*------------------------------------------------------------------------------
| IString::words                                                               |
|                                                                              |
| First, we locate the index of the first word.  If found, then we locate the  |
| end of the last word by getting its index and then searching for a           |
| whiteSpace character.  Once these indices have been calculated, we simply    |
| allocate space for the result and copy the specified portion of the receiver.|
------------------------------------------------------------------------------*/
IString IString :: words ( unsigned firstWord,
                           unsigned numWords ) const
  {
  // Treat word zero like word 1:
  if ( firstWord == 0 ) firstWord = 1;
  unsigned start = this->indexOfWord( firstWord );
  unsigned len   = 0; // Default is none.
  if ( start != 0 )
    {
    if ( numWords != 0 )
      {
      // Check for overflow...
      if ( numWords - 1 > UINT_MAX - firstWord )
        // Use maximum possible.
        numWords = UINT_MAX - firstWord + 1;
      // Get index of last word:
      unsigned end = this->indexOfWord( firstWord + numWords - 1 );
      // If found, get index of end of last word:
      if ( end == 0 )
      // Word not found, use last word.
      {
        unsigned pos = this->lastIndexOfAnyBut( whiteSpace );
        end = this->indexOfAnyOf( whiteSpace, pos + 1 );
        if ( end == 0 )
          end = this->length() + 1;
      }
      else
        // Word found, skip to next blank.
        {
        end = this->indexOfAnyOf( whiteSpace, end + 1 );
        if ( end == 0 )
          end = this->length() + 1;
        }
      len = end - start;
      }
    return this->subString( start, len );
    }
  return IString::null;
  }
