// ----------------------------------------------------------------------------
// FILE NAME: itime.cpp
//
// DESCRIPTION:
//   This file contains the implementation of classes/functions declared
//   in itime.hpp.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//
//   (C) Copyright IBM Corporation 1992, 1997  All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication, or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//                                                                            
// Revision: 99  1.5.1.4  source/core/base/itime.cpp, appsupport, ioc.v400, 000302  
// ----------------------------------------------------------------------------

#include <itime.hpp>

#ifdef IC_MVS
#include <ibhandle.hpp>  // Need _OPEN_THREADS for thread-safe version of USL
#endif // IC_MVS

extern "C"
{
    #include <time.h>
    #include <stdio.h>
    #include <string.h>

#ifdef IC_WIN
    #include <windows.h>
#endif

#ifdef IC_PM
    #define INCL_WINSTDCNR
    #include <os2.h>
#endif
}


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITIME_CPP_
  #include <ipagetun.h>
#endif


ITime ITime :: now ( )
  {
#ifdef IC_WIN
  SYSTEMTIME now;
  GetLocalTime(&now);
  return ITime( now.wHour, now.wMinute, now.wSecond );
#endif
#ifdef IC_PM
  time_t t    = time( 0 );
  tm     time = *localtime( &t );
  return ITime( time.tm_hour, time.tm_min, time.tm_sec );
#endif
#ifdef IC_POSIX
  time_t t    = time( 0 );
  tm     time;
  localtime_r( &t, &time );
  return ITime( time.tm_hour, time.tm_min, time.tm_sec );
#endif
  }

ITime :: ITime ( )
  {
  this->ticks = this->now().ticks;
  }

/*------------------------------------------------------------------------------
| ITime::ITime                                                                 |
|                                                                              |
| Construct from a container ICnrTime stucture using its hours/minutes/seconds.|
------------------------------------------------------------------------------*/
ITime :: ITime ( const ICnrTime &cnrTime )
  {
  long secs = cnrTime.hours;
  secs *= 60;
  secs += cnrTime.minutes;
  secs *= 60;
  secs += cnrTime.seconds;
  this->initialize( secs );
  }

/*------------------------------------------------------------------------------
| ITime::asString                                                              |
|                                                                              |
| Use strftime to get ascii representation as hh:mm:ss.                        |
------------------------------------------------------------------------------*/
IString ITime :: asString ( const char *fmt ) const
  {
  IString result( 0, ( ( fmt ) ? strlen( fmt ) : 0 ) + 6 );
  struct tm dummy = { 0 };
  dummy.tm_hour = this->hours();
  dummy.tm_min  = this->minutes();
  dummy.tm_sec  = this->seconds();
  mktime( &dummy );
  while ( !strftime( (char*)result, result.length(), fmt, &dummy ) )
    result += result;
  return IString( (char*)result );
  }

/*------------------------------------------------------------------------------
| ITime::asICnrTime                                                            |
|                                                                              |
| Construct a container CTIME result structure from the receiver's             |
| hours/minutes/seconds.                                                       |
------------------------------------------------------------------------------*/
ICnrTime ITime :: asICnrTime ( ) const
  {
  ICnrTime
    result = { (unsigned char)this->hours(),
               (unsigned char)this->minutes(),
               (unsigned char)this->seconds() };
  return result;
  }

#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITime::asCTIME                                                               |
|                                                                              |
| Construct a container CTIME result structure from the receiver's             |
| hours/minutes/seconds.                                                       |
------------------------------------------------------------------------------*/
_CTIME ITime :: asCTIME ( ) const
  {
  CTIME
    result = { (unsigned char)this->hours(),
               (unsigned char)this->minutes(),
               (unsigned char)this->seconds() };
  return result;
  }

/*------------------------------------------------------------------------------
| ITime::ITime                                                                 |
|                                                                              |
| Construct from a container CTIME stucture using its hours/minutes/seconds.   |
------------------------------------------------------------------------------*/
ITime :: ITime ( const _CTIME &cTime )
  {
  long secs = cTime.hours;
  secs *= 60;
  secs += cTime.minutes;
  secs *= 60;
  secs += cTime.seconds;
  this->initialize( secs );
  }
#endif
#endif

void ITime :: writeToStream( IDataStream &toWhere ) const {
  ticks >>= toWhere;
}

void ITime :: readFromStream( IDataStream &fromWhere ) {
  ticks <<= fromWhere;
}
