// Revision: 01 1.6.1.5 source/core/base/itime.inl, appsupport, ioc.v400, 000302 
#ifndef _ITIME_INL_
#define _ITIME_INL_
// ----------------------------------------------------------------------------
// FILE NAME: itime.inl
//
// DESCRIPTION:
//   This file contains the definition of the inline functions for the
//   classes declared in itime.hpp.
//
//
// COPYRIGHT:
//   IBM Open Class Library
//   Licensed Materials - Property of IBM
//   (C) Copyright IBM Corporation 1992, 1997
//
// ----------------------------------------------------------------------------

extern "C"
{
    #include <time.h>
}

/*------------------------------ Implementation ------------------------------*/
inline ITime& ITime :: initialize ( long seconds )
  {
  const long secondsPerDay = 24*60*60L;
  if ( seconds < 0 )
    this->ticks = secondsPerDay - (-seconds % secondsPerDay);
  else
    this->ticks = seconds % secondsPerDay;
  return *this;
  }

/*------------------------------- Constructors -------------------------------*/
inline ITime :: ITime ( unsigned hours,
                        unsigned minutes,
                        unsigned seconds )
  {
  long secs = hours;
  secs *= 60;
  secs += minutes;
  secs *= 60;
  secs += seconds;
  this->initialize( secs );
  }
inline ITime :: ITime ( long seconds )
  {
  this->initialize( seconds );
  }
inline ITime :: ITime ( const ITime &aTime )
  {
  this->ticks = aTime.ticks;
  }
/*-------------------------------- Accessors ---------------------------------*/
inline long ITime :: asSeconds ( ) const
  {
  return this->ticks;
  }
inline unsigned ITime :: hours ( ) const
  {
  // seconds/(seconds/hour) -> hours.
  return ((unsigned)(this->ticks / 3600));
  }
inline unsigned ITime :: minutes ( ) const
  {
  // seconds (past hour)/(seconds/minute) -> minutes.
  return ((unsigned)((this->ticks % 3600 ) / 60));
  }
inline unsigned ITime :: seconds ( ) const
  {
  // seconds (past minute)
  return ((unsigned)(this->ticks % 60));
  }
/*-------------------------------- Comparison --------------------------------*/
inline bool ITime :: operator == ( const ITime &aTime ) const
  {
  return this->ticks == aTime.ticks;
  }
inline bool ITime :: operator != ( const ITime &aTime ) const
  {
  return this->ticks != aTime.ticks;
  }
inline bool ITime :: operator <  ( const ITime &aTime ) const
  {
  return this->ticks <  aTime.ticks;
  }
inline bool ITime :: operator <= ( const ITime &aTime ) const
  {
  return this->ticks <= aTime.ticks;
  }
inline bool ITime :: operator >  ( const ITime &aTime ) const
  {
  return this->ticks >  aTime.ticks;
  }
inline bool ITime :: operator >= ( const ITime &aTime ) const
  {
  return this->ticks >= aTime.ticks;
  }
/*-------------------------- Manipulation Operators --------------------------*/
inline ITime& ITime :: operator += ( const ITime &aTime )
  {
  return this->initialize( this->ticks + aTime.ticks );
  }
inline ITime ITime :: operator + ( const ITime &aTime ) const
  {
  return ITime( this->ticks + aTime.ticks );
  }
inline ITime& ITime :: operator -= ( const ITime &aTime )
  {
  return this->initialize( this->ticks - aTime.ticks );
  }
inline ITime ITime :: operator - ( const ITime &aTime ) const
  {
  return ITime( this->ticks - aTime.ticks );
  }


inline void ITime::operator>>=(IDataStream& toWhere)
{
    writeToStream(toWhere);
}

inline void ITime::operator<<=(IDataStream& fromWhere)
{
    readFromStream(fromWhere);
}

inline ostream& operator<<(ostream& aStream, const ITime& aTime)
{
    return aStream << aTime.asString();
}

#endif // _ITIME_INL_
