// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 49 1.5.1.3 source/core/collect/iamap.h, collection, ioc.v400, 000302 
#ifndef _IAMAP_
#define _IAMAP_

#include <iaeqkey.h>
#include <iiamap.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key>
class IAMap :
  public IAEqualityKeyCollection <Element, Key> {

  typedef  IAEqualityKeyCollection <Element, Key> Inherited;

  typedef  IAMapImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IAMap
                            (INotifier&);

#endif
          ~IAMap
                            ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addDifference    (IAMap <Element, Key> const&,
                             IAMap <Element, Key> const&); 

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addIntersection  (IAMap <Element, Key> const&,
                             IAMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addUnion         (IAMap <Element, Key> const&,
                             IAMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     differenceWith   (IAMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     intersectionWith (IAMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     unionWith        (IAMap <Element, Key> const&);

  bool     operator==       (IAMap
                              <Element, Key> const&) const;

  bool     operator!=       (IAMap
                              <Element, Key> const&) const;

protected:

           IAMap
                            ();

           IAMap
                            (IAMap <Element, Key> const&);

  bool     locateNext       (Element const&, ICursor&) const;

  bool     locateNextElementWithKey
                            (Key const&, ICursor&) const;

  INumber  numberOfDifferentKeys
                            () const;

  INumber  numberOfElementsWithKey
                            (Key const&) const;

  INumber  numberOfOccurrences
                            (Element const&) const;

  INumber  removeAllOccurrences
                            (Element const&);

  INumber  removeAllElementsWithKey
                            (Key const&);

  bool     setToNextWithDifferentKey
                            (ICursor&) const;

private:

  static
  Implementation&
           ImplOf           (IAMap <Element, Key> const&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           addDifference    (IAMap <Element, Key> const,\
                             IAMap <Element, Key> const&),\
           addIntersection  (IAMap <Element, Key> const&,\
                             IAMap <Element, Key> const&),\
           addUnion         (IAMap <Element, Key> const&,
                             IAMap <Element, Key> const&),\
           differenceWith   (IAMap <Element, Key> const&),\
           intersectionWith (IAMap <Element, Key> const&),\
           unionWith        (IAMap <Element, Key> const&),\
           operator==       (IAMap\
                              <Element, Key> const&) const,\
           operator!=       (IAMap\
                              <Element, Key> const&) const,\
           ImplOf           (IAMap <Element, Key> const&)\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <iamap.inl>

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IAMAP_
