// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 53 1.4.1.3 source/core/collect/iaorder.h, collection, ioc.v400, 000302 
#ifndef _IAORDER_
#define _IAORDER_

#include <iacllct.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

template <class Element>
class IOrderedCursor : public IElementCursor <Element> 
{
  typedef  ICursorImpl Implementation;

public:

  bool setToLast        ();

  bool setToPrevious    ();

protected:

           IOrderedCursor   (Implementation*);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           setToLast        (),\
           setToPrevious    (),\
           IOrderedCursor   (Implementation*)\
                         )
#endif

};

template <class Element>
class IAOrderedCollection : public virtual IACollection <Element> {

  typedef  IACollectionImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IAOrderedCollection
                            (INotifier&);

#endif
          ~IAOrderedCollection
                            ();

  Element const&
           elementAtPosition
                            (IPosition) const;

  Element const&
           first            () const;

  bool isFirstAt        (ICursor const&) const;

  bool isLastAt         (ICursor const&) const;

  Element const&
           last             () const;

  IPosition
           positionAt       (ICursor const&) const;

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     removeAtPosition (IPosition);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     removeFirst      ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     removeLast       ();

  bool setToLast        (ICursor&) const;

  void     setToPosition    (IPosition, ICursor&) const;

  bool setToPrevious    (ICursor&) const;

  // for backward compatibility
  Element const&
           firstElement     () const;

  bool isFirst          (ICursor const&) const;

  bool isLast           (ICursor const&) const;

  Element const&
           lastElement      () const;

  IPosition
           position         (ICursor const&) const;

protected:

           IAOrderedCollection
                            ();

           IAOrderedCollection
                            (IAOrderedCollection <Element> const&);

private:

  static
  Implementation&
           ImplOf           (IAOrderedCollection <Element> const&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           elementAtPosition(IPosition) const,\
           firstElement     () const,\
           isFirst          (ICursor const&) const,\
           isLast           (ICursor const&) const,\
           lastElement      () const,\
           position         (ICursor&) const,\
           positionAt       (ICursor&) const,\
           removeAtPosition (IPosition),\
           removeFirst      (),\
           removeLast       (),\
           setToLast        (ICursor&) const,\
           setToPosition    (IPosition, ICursor&) const,\
           setToPrevious    (ICursor&) const,\
           ImplOf           (IAOrderedCollection <Element> const&))
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iaorder.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IAORDER_
