// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 73 1.5.1.3 source/core/collect/iasm.h, collection, ioc.v400, 000302 
#ifndef _IASM_
#define _IASM_

#include <iaeqksrt.h>
#include <iiasm.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key>
class IASortedMap :
  public IAEqualityKeySortedCollection <Element, Key> {

  typedef  IAEqualityKeySortedCollection <Element, Key> Inherited;

  typedef  IASortedMapImpl Implementation;

public:

#if ! defined (__INO_AVALON__)
           IASortedMap
                            (INotifier&);

#endif
          ~IASortedMap
                            ();

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addDifference    (IASortedMap <Element, Key> const&,
                             IASortedMap <Element, Key> const&); 

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addIntersection  (IASortedMap <Element, Key> const&,
                             IASortedMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     addUnion         (IASortedMap <Element, Key> const&,
                             IASortedMap <Element, Key> const&);

  long     compare          (IASortedMap <Element, Key> const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;
 
#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     differenceWith   (IASortedMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     intersectionWith (IASortedMap <Element, Key> const&);

#if ! defined (INO_VIRTUAL)
  virtual
#endif
  void     unionWith        (IASortedMap <Element, Key> const&);

  bool     operator==       (IASortedMap
                              <Element, Key> const&) const;

  bool     operator!=       (IASortedMap
                              <Element, Key> const&) const;

protected:

           IASortedMap
                            ();

           IASortedMap
                            (IASortedMap <Element, Key> const&);

  INumber  numberOfDifferentKeys
                            () const;

  INumber  numberOfElementsWithKey
                            (Key const&) const;

  INumber  numberOfOccurrences
                            (Element const&) const;

  INumber  removeAllOccurrences
                            (Element const&);

  INumber  removeAllElementsWithKey
                            (Key const&);

  bool     setToNextWithDifferentKey
                            (ICursor&) const;

private:

  static
  Implementation&
           ImplOf           (IASortedMap <Element, Key> const&);

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           addDifference    (IASortedMap <Element, Key> const,\
                             IASortedMap <Element, Key> const&),\
           addIntersection  (IASortedMap <Element, Key> const&,\
                             IASortedMap <Element, Key> const&),\
           addUnion         (IASortedMap <Element, Key> const&,
                             IASortedMap <Element, Key> const&),\
           compare          (IASortedMap <Element, Key> const&,\
                             long (*)\
                              (Element const&, Element const&) const,\
           differenceWith   (IASortedMap <Element, Key> const&),\
           intersectionWith (IASortedMap <Element, Key> const&),\
           unionWith        (IASortedMap <Element, Key> const&),\
           operator==       (IASortedMap\
                              <Element, Key> const&) const,\
           operator!=       (IASortedMap\
                              <Element, Key> const&) const,\
           ImplOf           (IASortedMap <Element, Key> const&)\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <iasm.inl>

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IASM_
