// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 94 1.4.1.4 source/core/collect/ibag.inl, collection, ioc.v400, 000302  
#ifndef _IBAG_INL_
#define _IBAG_INL_

// ---
// IGBagCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGBagCursor <Element, ElementOps>::
IGBagCursor
  (IGBag
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGBagCursor <Element, ElementOps>&
IGBagCursor <Element, ElementOps>::
operator=
  (IGBagCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagCursor <Element, ElementOps>::
IGBagCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGBagMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagMngCursor <Element, ElementOps>::
IGBagMngCursor
  (IGBag
    <Element, ElementOps> const& collection)
: IGBagCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGBagMngCursor <Element, ElementOps>::
IGBagMngCursor (ICursorImpl* impl)
: IGBagCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGBag
// ---

// public members

template <class Element, class ElementOps>
inline
IGBag <Element, ElementOps>::
IGBag (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGBag <Element, ElementOps>::
IGBag
  (IGBag
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGBag <Element, ElementOps>&
IGBag <Element, ElementOps>::
operator= (IGBag
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBag <Element, ElementOps>::
IGBag (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGBag
 <Element, ElementOps>::Implementation&
IGBag <Element, ElementOps>::
ImplOf (IGBag
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IBag
// ---

// public members

template <class Element>
inline
IBag <Element>::
IBag (INumber numberOfElements)
: IGBag
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IBag <Element>::
IBag (PureImplementation& impl)
: IGBag
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGBag
// ---

// public members

template <class Element, class ElementOps>
inline
IVGBag <Element, ElementOps>::
IVGBag (INumber numberOfElements)
: IGBag
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGBag <Element, ElementOps>::
IVGBag (PureImplementation& impl)
: IGBag
    <Element, ElementOps> (impl)
{
}

// ---
// IVBag
// ---

// public members

template <class Element>
inline
IVBag <Element>::
IVBag (INumber numberOfElements)
: IVGBag
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVBag <Element>::
IVBag (PureImplementation& impl)
: IVGBag
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _IBAG_INL_
