// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 97 1.5.1.5 source/core/collect/ibagbst.h, collection, ioc.v400, 000302 
#ifndef _IBAGBST_
#define _IBAGBST_

#include <ilanglvl.hpp>
#include <iabag.h>
#include <iicbag.h>
#if ! defined (__INO_AVALON__)
#include <iivbag.h>
#endif
#include <iibaglst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGBagAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGBagAsBstTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGBagAsBstTree
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGBagAsBstTreeCursor
                            (IGBagAsBstTree
                              <
                               Element, ElementOps> const&);

  IGBagAsBstTreeCursor <Element, ElementOps>&
           operator=        (IGBagAsBstTreeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsBstTreeCursor\
                            (IGBagAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGBagAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGBagAsBstTreeMngCursor :
  public IGBagAsBstTreeCursor <Element, ElementOps>  {

  typedef  IGBagAsBstTreeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGBagAsBstTree
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGBagAsBstTreeMngCursor
                            (IGBagAsBstTree
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsBstTreeMngCursor\
                            (IGBagAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           IGBagAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGBagAsBstTree :
  public IABag <Element> {
protected:

  typedef  IGBagAsBstTree <Element, ElementOps> Self;

  typedef  IBagAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICBagImpl
            <PureImplementation> Implementation;
#endif

#define IGBagAsBstTreeTypedef(Element, ElementOps)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGBagAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGBagAsBstTreeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGBagAsBstTreeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGBagAsBstTree
                            (INumber = 100);

           IGBagAsBstTree
                            (IGBagAsBstTree
                              <
                               Element, ElementOps> const&);

  IGBagAsBstTree <Element, ElementOps>&
           operator=        (IGBagAsBstTree
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGBagAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGBagAsBstTree
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGBagAsBstTreeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGBagAsBstTreeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsBstTree\
                            (INumber),\
           operator=,\
           IGBagAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGBagAsBstTree\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IBagAsBstTree :
  public IGBagAsBstTree
          <Element, ICOps <Element> > {

  typedef  IGBagAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IBagAsBstTree <Element> Self;

#define IBagAsBstTreeTypedef(Element)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IBagAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IBagAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IBagAsBstTree\
                              (INumber),\
           IBagAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGBagAsBstTree :
  public IGBagAsBstTree
          <Element, ElementOps > {

  typedef  IVGBagAsBstTree
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGBagAsBstTree
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICBagImpl
            <IVBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGBagAsBstTreeTypedef(Element, ElementOps)\
  typedef  IBagAsListOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGBagAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGBagAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGBagAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGBagAsBstTree\
                              (INumber),\
           IVGBagAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVBagAsBstTree :
  public IVGBagAsBstTree
          <Element, ICOps <Element> > {

  typedef  IVGBagAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVBagAsBstTree <Element> Self;

#define IVBagAsBstTreeTypedef(Element)\
  typedef  IVBagAsListOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVBagAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVBagAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVBagAsBstTree\
                              (INumber),\
           IVBagAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibagbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBAGBST_
