// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 98 1.4.1.4 source/core/collect/ibagbst.inl, collection, ioc.v400, 000302  
#ifndef _IBAGBST_INL_
#define _IBAGBST_INL_

// ---
// IGBagAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGBagAsBstTreeCursor <Element, ElementOps>::
IGBagAsBstTreeCursor
  (IGBagAsBstTree
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGBagAsBstTreeCursor <Element, ElementOps>&
IGBagAsBstTreeCursor <Element, ElementOps>::
operator=
  (IGBagAsBstTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsBstTreeCursor <Element, ElementOps>::
IGBagAsBstTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGBagAsBstTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsBstTreeMngCursor <Element, ElementOps>::
IGBagAsBstTreeMngCursor
  (IGBagAsBstTree
    <Element, ElementOps> const& collection)
: IGBagAsBstTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsBstTreeMngCursor <Element, ElementOps>::
IGBagAsBstTreeMngCursor (ICursorImpl* impl)
: IGBagAsBstTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGBagAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGBagAsBstTree <Element, ElementOps>::
IGBagAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGBagAsBstTree <Element, ElementOps>::
IGBagAsBstTree
  (IGBagAsBstTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGBagAsBstTree <Element, ElementOps>&
IGBagAsBstTree <Element, ElementOps>::
operator= (IGBagAsBstTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGBagAsBstTree <Element, ElementOps>::
IGBagAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGBagAsBstTree
 <Element, ElementOps>::Implementation&
IGBagAsBstTree <Element, ElementOps>::
ImplOf (IGBagAsBstTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IBagAsBstTree
// ---

// public members

template <class Element>
inline
IBagAsBstTree <Element>::
IBagAsBstTree (INumber numberOfElements)
: IGBagAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IBagAsBstTree <Element>::
IBagAsBstTree (PureImplementation& impl)
: IGBagAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGBagAsBstTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGBagAsBstTree <Element, ElementOps>::
IVGBagAsBstTree (INumber numberOfElements)
: IGBagAsBstTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGBagAsBstTree <Element, ElementOps>::
IVGBagAsBstTree (PureImplementation& impl)
: IGBagAsBstTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVBagAsBstTree
// ---

// public members

template <class Element>
inline
IVBagAsBstTree <Element>::
IVBagAsBstTree (INumber numberOfElements)
: IVGBagAsBstTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVBagAsBstTree <Element>::
IVBagAsBstTree (PureImplementation& impl)
: IVGBagAsBstTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _IBAGBST_INL_
