// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 02 1.5.1.5 source/core/collect/ibaghsh.h, collection, ioc.v400, 000302 
#ifndef _IBAGHSH_
#define _IBAGHSH_

#include <ilanglvl.hpp>
#include <iabag.h>
#include <iicbag.h>
#if ! defined (__INO_AVALON__)
#include <iivbag.h>
#endif
#include <iibaghsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGBagAsHshTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGBagAsHshTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGBagAsHshTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGBagAsHshTableCursor
                            (IGBagAsHshTable
                              <
                               Element, ElementOps> const&);

  IGBagAsHshTableCursor <Element, ElementOps>&
           operator=        (IGBagAsHshTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsHshTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsHshTableCursor\
                            (IGBagAsHshTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGBagAsHshTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGBagAsHshTableMngCursor :
  public IGBagAsHshTableCursor <Element, ElementOps>  {

  typedef  IGBagAsHshTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGBagAsHshTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGBagAsHshTableMngCursor
                            (IGBagAsHshTable
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsHshTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsHshTableMngCursor\
                            (IGBagAsHshTable\
                              <\
                               Element, ElementOps> const&),\
           IGBagAsHshTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGBagAsHshTable :
  public IABag <Element> {
protected:

  typedef  IGBagAsHshTable <Element, ElementOps> Self;

  typedef  IBagAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICBagImpl
            <PureImplementation> Implementation;
#endif

#define IGBagAsHshTableTypedef(Element, ElementOps)\
  typedef  IBagAsHshTableOps\
            <Element,\
             IWEHOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGBagAsHshTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGBagAsHshTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGBagAsHshTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGBagAsHshTable
                            (INumber = 100);

           IGBagAsHshTable
                            (IGBagAsHshTable
                              <
                               Element, ElementOps> const&);

  IGBagAsHshTable <Element, ElementOps>&
           operator=        (IGBagAsHshTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGBagAsHshTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGBagAsHshTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGBagAsHshTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGBagAsHshTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsHshTable\
                            (INumber),\
           operator=,\
           IGBagAsHshTable\
                            (PureImplementation&),\
           ImplOf           (IGBagAsHshTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IBagAsHshTable :
  public IGBagAsHshTable
          <Element, IEHOps <Element> > {

  typedef  IGBagAsHshTable
            <Element,
             IEHOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IBagAsHshTable <Element> Self;

#define IBagAsHshTableTypedef(Element)\
  typedef  IBagAsHshTableOps\
            <Element,\
             IWEHOps <Element, IEHOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IBagAsHshTable\
                              (INumber),\
           IBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGBagAsHshTable :
  public IGBagAsHshTable
          <Element, ElementOps > {

  typedef  IVGBagAsHshTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGBagAsHshTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICBagImpl
            <IVBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGBagAsHshTableTypedef(Element, ElementOps)\
  typedef  IBagAsHshTableOps\
            <Element,\
             IWEHOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGBagAsHshTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGBagAsHshTable\
                              (INumber),\
           IVGBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVBagAsHshTable :
  public IVGBagAsHshTable
          <Element, IEHOps <Element> > {

  typedef  IVGBagAsHshTable
            <Element,
             IEHOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVBagAsHshTable <Element> Self;

#define IVBagAsHshTableTypedef(Element)\
  typedef  IVBagAsHshTableOps\
            <Element,\
             IWEHOps <Element, IEHOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVBagAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVBagAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVBagAsHshTable\
                              (INumber),\
           IVBagAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibaghsh.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBAGHSH_
