// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 09 1.5.1.4 source/core/collect/ibagtab.h, collection, ioc.v400, 000302 
#ifndef _IBAGTAB_
#define _IBAGTAB_

#include <ilanglvl.hpp>
#include <iabag.h>
#include <iicbag.h>
#if ! defined (__INO_AVALON__)
#include <iivbag.h>
#endif
#include <iibagtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGBagAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGBagAsTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGBagAsTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGBagAsTableCursor
                            (IGBagAsTable
                              <
                               Element, ElementOps> const&);

  IGBagAsTableCursor <Element, ElementOps>&
           operator=        (IGBagAsTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsTableCursor\
                            (IGBagAsTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGBagAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGBagAsTableMngCursor :
  public IGBagAsTableCursor <Element, ElementOps>  {

  typedef  IGBagAsTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGBagAsTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGBagAsTableMngCursor
                            (IGBagAsTable
                              <
                               Element, ElementOps> const&);

protected:

           IGBagAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsTableMngCursor\
                            (IGBagAsTable\
                              <\
                               Element, ElementOps> const&),\
           IGBagAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGBagAsTable :
  public IABag <Element> {
protected:

  typedef  IGBagAsTable <Element, ElementOps> Self;

  typedef  IBagAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICBagImpl
            <PureImplementation> Implementation;
#endif

#define IGBagAsTableTypedef(Element, ElementOps)\
  typedef  IBagAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGBagAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGBagAsTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGBagAsTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGBagAsTable
                            (INumber = 100);

           IGBagAsTable
                            (IGBagAsTable
                              <
                               Element, ElementOps> const&);

  IGBagAsTable <Element, ElementOps>&
           operator=        (IGBagAsTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGBagAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGBagAsTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGBagAsTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGBagAsTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGBagAsTable\
                            (INumber),\
           operator=,\
           IGBagAsTable\
                            (PureImplementation&),\
           ImplOf           (IGBagAsTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IBagAsTable :
  public IGBagAsTable
          <Element, ICOps <Element> > {

  typedef  IGBagAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IBagAsTable <Element> Self;

#define IBagAsTableTypedef(Element)\
  typedef  IBagAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IBagAsTable\
                              (INumber),\
           IBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGBagAsTable :
  public IGBagAsTable
          <Element, ElementOps > {

  typedef  IVGBagAsTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGBagAsTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICBagImpl
            <IVBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGBagAsTableTypedef(Element, ElementOps)\
  typedef  IBagAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGBagAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGBagAsTable\
                              (INumber),\
           IVGBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVBagAsTable :
  public IVGBagAsTable
          <Element, ICOps <Element> > {

  typedef  IVGBagAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVBagAsTable <Element> Self;

#define IVBagAsTableTypedef(Element)\
  typedef  IVBagAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVBagAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVBagAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVBagAsTable\
                              (INumber),\
           IVBagAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibagtab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBAGTAB_
