// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 12 1.4.1.3 source/core/collect/ibcllct.h, collection, ioc.v400, 000302 
#ifndef _IBCLLCT_
#define _IBCLLCT_

#if defined (__RRBC_LIB__)                    
#define IACollectionBase IARRBCCollectionBase 
#define ICursor IRRBCCursor                   
#endif                                        

#include <imcllct.h>
#include <iglobals.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) 
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class IC_EXPORTC ICursor 
{
  typedef  ICursorImpl Implementation;

public:

  virtual  ~ICursor          ();

  void     copy             (ICursor const&);

  void     invalidate       ();

#if ! defined (__INO_MNGCURSOR__)
  bool isInBetween      () const;

  bool isMngCursor      () const;

#endif
  bool isValid          () const;

  bool setToFirst       ();

  bool setToNext        ();

  void     operator=        (ICursor const&);

  bool operator==       (ICursor const&) const;

  bool operator!=       (ICursor const&) const;

protected:

           ICursor          (Implementation*);

           ICursor          (ICursor const&);

  static
  inline
  Implementation&
           ImplOf           (ICursor const&);

private:

  Implementation*
           ivImpl;

  friend
  class    IACollectionBase;

#if ! defined (__INO_AVALON__)
  friend
  class    IVCollectionImpl;

#endif
#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           copy             (ICursor const&),\
           invalidate       (),\
           isValid          () const,\
           setToFirst       (),\
           setToNext        (),\
           operator=,\
           operator==       (ICursor const&) const,\
           operator!=       (ICursor const&) const,\
           ICursor          (Implementation*)\
                        )
#endif

};

#define forICursor(c) \
  for ((c).setToFirst (); (c).isValid (); (c).setToNext ())

// for backward compatibility
#define forCursor(c) \
  for ((c).setToFirst (); (c).isValid (); (c).setToNext ())

class IC_EXPORTC IACollectionBase : protected IMetaCollection {
public:

protected:

           IACollectionBase ();

           IACollectionBase (IACollectionBase const&);

          ~IACollectionBase ();

  static
  ICursorImpl&
           CrsrImplOf       (ICursor& cursor);

  static
  ICursorImpl const&
           CrsrImplOf       (ICursor const& cursor);

  static
  ICursor* InterfaceFor     (ICursorImpl* cursor);
 
#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           CrsrImplOf       (ICursor&),\
           CrsrImplOf       (ICursor const&),\
           InterfaceFor     (ICursorImpl*)\
                        )
#endif

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ibcllct.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IBCLLCT_
