// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 24 1.4.1.4 source/core/collect/idqu.inl, collection, ioc.v400, 000302  
#ifndef _IDQU_INL_
#define _IDQU_INL_

// ---
// IGDequeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGDequeCursor <Element, ElementOps>::
IGDequeCursor
  (IGDeque
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGDequeCursor <Element, ElementOps>&
IGDequeCursor <Element, ElementOps>::
operator=
  (IGDequeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeCursor <Element, ElementOps>::
IGDequeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGDequeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGDequeMngCursor <Element, ElementOps>::
IGDequeMngCursor
  (IGDeque
    <Element, ElementOps> const& collection)
: IGDequeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGDequeMngCursor <Element, ElementOps>::
IGDequeMngCursor (ICursorImpl* impl)
: IGDequeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGDeque
// ---

// public members

template <class Element, class ElementOps>
inline
IGDeque <Element, ElementOps>::
IGDeque (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGDeque <Element, ElementOps>::
IGDeque
  (IGDeque
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGDeque <Element, ElementOps>&
IGDeque <Element, ElementOps>::
operator= (IGDeque
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGDeque <Element, ElementOps>::
IGDeque (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGDeque
 <Element, ElementOps>::Implementation&
IGDeque <Element, ElementOps>::
ImplOf (IGDeque
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IDeque
// ---

// public members

template <class Element>
inline
IDeque <Element>::
IDeque (INumber numberOfElements)
: IGDeque
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IDeque <Element>::
IDeque (PureImplementation& impl)
: IGDeque
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGDeque
// ---

// public members

template <class Element, class ElementOps>
inline
IVGDeque <Element, ElementOps>::
IVGDeque (INumber numberOfElements)
: IGDeque
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGDeque <Element, ElementOps>::
IVGDeque (PureImplementation& impl)
: IGDeque
    <Element, ElementOps> (impl)
{
}

// ---
// IVDeque
// ---

// public members

template <class Element>
inline
IVDeque <Element>::
IVDeque (INumber numberOfElements)
: IVGDeque
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVDeque <Element>::
IVDeque (PureImplementation& impl)
: IVGDeque
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IDQU_INL_
