// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 55 1.5.1.4 source/core/collect/ihp.h, collection, ioc.v400, 000302 
#ifndef _IHP_
#define _IHP_

#include <ilanglvl.hpp>
#include <iahp.h>
#include <iichp.h>
#if ! defined (__INO_AVALON__)
#include <iivhp.h>
#endif
#include <iihplst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGHeap;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGHeapCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGHeap
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGHeapCursor
                            (IGHeap
                              <
                               Element, ElementOps> const&);

  IGHeapCursor <Element, ElementOps>&
           operator=        (IGHeapCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGHeapCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeapCursor\
                            (IGHeap\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGHeapCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGHeapMngCursor :
  public IGHeapCursor <Element, ElementOps>  {

  typedef  IGHeapCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGHeap
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGHeapMngCursor
                            (IGHeap
                              <
                               Element, ElementOps> const&);

protected:

           IGHeapMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeapMngCursor\
                            (IGHeap\
                              <\
                               Element, ElementOps> const&),\
           IGHeapMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGHeap :
  public IAHeap <Element> {
protected:

  typedef  IGHeap <Element, ElementOps> Self;

  typedef  IHeapAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICHeapImpl
            <PureImplementation> Implementation;
#endif

#define IGHeapTypedef(Element, ElementOps)\
  typedef  IHeapAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGHeapTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGHeapCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGHeapMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGHeap
                            (INumber = 100);

           IGHeap
                            (IGHeap
                              <
                               Element, ElementOps> const&);

  IGHeap <Element, ElementOps>&
           operator=        (IGHeap
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGHeap
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGHeap
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGHeapCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGHeapMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGHeap\
                            (INumber),\
           operator=,\
           IGHeap\
                            (PureImplementation&),\
           ImplOf           (IGHeap\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IHeap :
  public IGHeap
          <Element, IStdOps <Element> > {

  typedef  IGHeap
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IHeap <Element> Self;

#define IHeapTypedef(Element)\
  typedef  IHeapAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IHeap
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IHeap
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IHeap\
                              (INumber),\
           IHeap\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGHeap :
  public IGHeap
          <Element, ElementOps > {

  typedef  IVGHeap
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGHeap
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVHeapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICHeapImpl
            <IVHeapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGHeapTypedef(Element, ElementOps)\
  typedef  IHeapAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGHeapTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGHeap
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGHeap
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGHeap\
                              (INumber),\
           IVGHeap\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVHeap :
  public IVGHeap
          <Element, IStdOps <Element> > {

  typedef  IVGHeap
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVHeap <Element> Self;

#define IVHeapTypedef(Element)\
  typedef  IVHeapAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVHeap
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVHeap
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVHeap\
                              (INumber),\
           IVHeap\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ihp.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IHP_
