// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 60 1.4.1.4 source/core/collect/ihplst.inl, collection, ioc.v400, 000302  
#ifndef _IHPLST_INL_
#define _IHPLST_INL_

// ---
// IGHeapAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGHeapAsListCursor <Element, ElementOps>::
IGHeapAsListCursor
  (IGHeapAsList
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGHeapAsListCursor <Element, ElementOps>&
IGHeapAsListCursor <Element, ElementOps>::
operator=
  (IGHeapAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsListCursor <Element, ElementOps>::
IGHeapAsListCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGHeapAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeapAsListMngCursor <Element, ElementOps>::
IGHeapAsListMngCursor
  (IGHeapAsList
    <Element, ElementOps> const& collection)
: IGHeapAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsListMngCursor <Element, ElementOps>::
IGHeapAsListMngCursor (ICursorImpl* impl)
: IGHeapAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGHeapAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeapAsList <Element, ElementOps>::
IGHeapAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGHeapAsList <Element, ElementOps>::
IGHeapAsList
  (IGHeapAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGHeapAsList <Element, ElementOps>&
IGHeapAsList <Element, ElementOps>::
operator= (IGHeapAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsList <Element, ElementOps>::
IGHeapAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGHeapAsList
 <Element, ElementOps>::Implementation&
IGHeapAsList <Element, ElementOps>::
ImplOf (IGHeapAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IHeapAsList
// ---

// public members

template <class Element>
inline
IHeapAsList <Element>::
IHeapAsList (INumber numberOfElements)
: IGHeapAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IHeapAsList <Element>::
IHeapAsList (PureImplementation& impl)
: IGHeapAsList
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGHeapAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGHeapAsList <Element, ElementOps>::
IVGHeapAsList (INumber numberOfElements)
: IGHeapAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGHeapAsList <Element, ElementOps>::
IVGHeapAsList (PureImplementation& impl)
: IGHeapAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVHeapAsList
// ---

// public members

template <class Element>
inline
IVHeapAsList <Element>::
IVHeapAsList (INumber numberOfElements)
: IVGHeapAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVHeapAsList <Element>::
IVHeapAsList (PureImplementation& impl)
: IVGHeapAsList
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IHPLST_INL_
