// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 62 1.4.1.4 source/core/collect/ihptab.inl, collection, ioc.v400, 000302  
#ifndef _IHPTAB_INL_
#define _IHPTAB_INL_

// ---
// IGHeapAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGHeapAsTableCursor <Element, ElementOps>::
IGHeapAsTableCursor
  (IGHeapAsTable
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGHeapAsTableCursor <Element, ElementOps>&
IGHeapAsTableCursor <Element, ElementOps>::
operator=
  (IGHeapAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsTableCursor <Element, ElementOps>::
IGHeapAsTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGHeapAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeapAsTableMngCursor <Element, ElementOps>::
IGHeapAsTableMngCursor
  (IGHeapAsTable
    <Element, ElementOps> const& collection)
: IGHeapAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsTableMngCursor <Element, ElementOps>::
IGHeapAsTableMngCursor (ICursorImpl* impl)
: IGHeapAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGHeapAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGHeapAsTable <Element, ElementOps>::
IGHeapAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGHeapAsTable <Element, ElementOps>::
IGHeapAsTable
  (IGHeapAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGHeapAsTable <Element, ElementOps>&
IGHeapAsTable <Element, ElementOps>::
operator= (IGHeapAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGHeapAsTable <Element, ElementOps>::
IGHeapAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGHeapAsTable
 <Element, ElementOps>::Implementation&
IGHeapAsTable <Element, ElementOps>::
ImplOf (IGHeapAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IHeapAsTable
// ---

// public members

template <class Element>
inline
IHeapAsTable <Element>::
IHeapAsTable (INumber numberOfElements)
: IGHeapAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IHeapAsTable <Element>::
IHeapAsTable (PureImplementation& impl)
: IGHeapAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGHeapAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGHeapAsTable <Element, ElementOps>::
IVGHeapAsTable (INumber numberOfElements)
: IGHeapAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGHeapAsTable <Element, ElementOps>::
IVGHeapAsTable (PureImplementation& impl)
: IGHeapAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVHeapAsTable
// ---

// public members

template <class Element>
inline
IVHeapAsTable <Element>::
IVHeapAsTable (INumber numberOfElements)
: IVGHeapAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVHeapAsTable <Element>::
IVHeapAsTable (PureImplementation& impl)
: IVGHeapAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IHPTAB_INL_
