// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 65 1.5.1.2 source/core/collect/iiabag.cpp, collection, ioc.v400, 000302  
#ifdef IC_PAGETUNE
  #define _IIABAG_CPP_
  #include <ipagetun.h>
#endif

#include <iiabag.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (off)
#endif

#pragma pack (push,4)

class IPointer {
  IACollectionImpl* ivImpl;
public:
  IPointer (IACollectionImpl* impl)
  : ivImpl (impl) { ivImpl->Initialize (); }
 ~IPointer () { ivImpl->Delete (); }
  IACollectionImpl& operator* () { return *ivImpl; }
};

#pragma pack (pop)

#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif


// ---
// IABagImpl
// ---

// public members

IABagImpl::
~IABagImpl ()
{
}

void
IABagImpl::
AddAntivalence (IABagImpl const& collection1,
                IABagImpl const& collection2)
{
#if ! defined (__INO_INTERNAL__)
  if (&collection1 != &collection2) {
    bool hasNext;
    for (hasNext = CursorOf (collection1).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection1).SetToNext ()) {
      if (! collection2.Contains
           (collection1.ElementAt (CursorOf (collection1)))) {
        Add (collection1.ElementAt (CursorOf (collection1)),
             CursorOf (*this));
      }
    }
    for (hasNext = CursorOf (collection2).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection2).SetToNext ()) {
      if (! collection1.Contains
           (collection2.ElementAt (CursorOf (collection2)))) {
        Add (collection2.ElementAt (CursorOf (collection2)),
             CursorOf (*this));
      }
    }
  }
#endif
}

void
IABagImpl::
AddDifference (IABagImpl const& collection1,
               IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this != &collection2) {
      Union (*this);
      ICursorImpl& cursor2 = CursorOf (collection2);
      bool hasNext;
      for (hasNext = collection2.SetToFirst (cursor2);
           hasNext;
           hasNext = collection2.SetToNextDifferentElement (cursor2)) {
        void const* element = collection2.ElementAt (cursor2);
        INumber n1 = NumberOfOccurrences (element) / 2,
                n2 = collection2.NumberOfOccurrences (element),
                n  = (n1 < n2 ? n1 : n2);
        while (n--) {
          Remove (element);
        }
      }
    }
  }
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNextDifferentElement (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      INumber n1 = collection1.NumberOfOccurrences (element),
              n2 = collection2.NumberOfOccurrences (element),
              n  = (n1 < n2 ? 0 : n1 - n2);
      while (n--) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IABagImpl::
AddIntersection (IABagImpl const& collection1,
                 IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this == &collection2)
      Union (*this);
    else
      AddIntersection (collection2, collection1);
  }
  else {
    ICursorImpl& cursor2 = CursorOf (collection2);
    bool hasNext;
    for (hasNext = collection2.SetToFirst (cursor2);
         hasNext;
         hasNext = collection2.SetToNextDifferentElement (cursor2)) {
      void const* element = collection2.ElementAt (cursor2);
      INumber n1 = collection1.NumberOfOccurrences (element),
              n2 = collection2.NumberOfOccurrences (element),
              n  = (n1 < n2 ? n1 : n2);
      for ( ; n != 0; n--)
        Add (element, CursorOf (*this));
    }
  }
}

void
IABagImpl::
AddUnion (IABagImpl const& collection1,
          IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this == &collection2) {
      IPointer bag (Clone ());
      AddAllFrom ((IABagImpl&)*bag);
      AddAllFrom ((IABagImpl&)*bag);
    }
    else {
      Union (collection1);
      Union (collection2);
    }
  }
  else {
    Union (collection2);
    Union (collection1);
  }
}

void
IABagImpl::
Antivalence (IABagImpl const& collection)
{
#if ! defined (__INO_INTERNAL__)
  if (this == &collection) {
    RemoveAll ();
  }
  else {
    for (CursorOf (collection).SetToFirst ();
         CursorOf (collection).IsValid ();
         CursorOf (collection).SetToNext ()) {
      if (! Contains
           (collection.ElementAt (CursorOf (collection)))) {
        Add (collection.ElementAt (CursorOf (collection)),
             CursorOf (*this));
      }
    }
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsContained, (void*)eq);
  }
#endif
}

void
IABagImpl::
Difference (IABagImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IABagImpl::
Intersection (IABagImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IABagImpl::
Union (IABagImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
  else {
    IPointer bag (Clone ());
    AddAllFrom ((IABagImpl&)*bag);
  }
}

bool
IABagImpl::
operator!= (IABagImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IABagImpl::
IABagImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
