// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#include <iiacllct.h>
#include <iiexc.h>
#include <icconsta.h>
#include <iprimlck.hpp>

#ifdef IC_PAGETUNE
  #define _IIACLLCT_CPP_
  #include <ipagetun.h>
#endif


#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (ICursorImpl)
#endif

#if ! defined(__INO_L1_LOCKING__)
// ------------------------
// ICollectionResourceImpl
// ------------------------

ICollectionResourceImpl::
ICollectionResourceImpl ()
: ivResource (0)
{
    try
    {
        ivResource = new IPrivateResource;
    }
    
    catch (...)
    {
        bool alwaysFalse = false;
        ICHECK( alwaysFalse,ICollectionResourceExceptionImpl,"ICollectionResourceImpl",
                "ICollectionResourceImpl");
    }
}

void
ICollectionResourceImpl::
lock (long timeout)
{ try {
    ivResource->lock(timeout);
  } catch (IOutOfSystemResource &e) { e;
     bool alwaysFalse = false;
     ICHECK(alwaysFalse,ICollectionLockTimeOutExceptionImpl,
            "ICollectionResourceImpl","lock");
  } catch (...) {
     bool alwaysFalse = false;
     ICHECK(alwaysFalse,ICollectionLockExceptionImpl,
            "ICollectionResourceImpl","lock");
  }
}

void
ICollectionResourceImpl::
unlock ()
{ try {
    ivResource->unlock();
  } catch (...) {
    bool alwaysFalse = false;
    ICHECK(alwaysFalse,ICollectionUnlockExceptionImpl,
           "ICollectionResourceImpl","unlock");
  }
}
#endif

// -----------
// ICursorImpl
// -----------

// public members

bool
ICursorImpl::
IsMngCursor () const
{ return false;
}

// ----------------
// IACollectionImpl
// ----------------

// public members

IACollectionImpl::
~IACollectionImpl ()
{ delete ivDummyCursor;
}

void
IACollectionImpl::
AddAllFrom (IACollectionImpl const& collection)
{ ICHECK_Identical (ClassName (), "AddAllFrom")

  LockBeforeWriting ();
  ((IACollectionImpl&)collection).LockBeforeReading ();
  for (collection.SetToFirst (CursorOf (collection));
       CursorOf (collection).IsValid ();
       collection.SetToNext (CursorOf (collection))) {
    Add (collection.ElementAt (CursorOf (collection)),
         CursorOf (*this));
  }
  ((IACollectionImpl&)collection).UnlockAfterReading ();
  UnlockAfterWriting ();
}

void
IACollectionImpl::
AddAsFirst (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsLast (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsNext (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAtPosition (IPosition, void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsPrevious (void const*, ICursorImpl&)
{ // just a dummy
}

bool
IACollectionImpl::
AddOrReplaceElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

bool
IACollectionImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ for (SetToFirst (CursorOf (*this));
       CursorOf (*this).IsValid ();
       SetToNext (CursorOf (*this))) {
    if (! (*applFunc) (ElementAt (CursorOf (*this)), addArg))
      return false;
  }
  return true;
}

void*
IACollectionImpl::
CheckPointer (void* ptr)
{
    if (ptr == 0) 
    {
        throw IOutOfCollectionMemoryImpl
        (
            ICollectionExceptionText(IC_OUT_OF_COLLECTION_MEMORY)
        );
    }
    return ptr;
}

bool
IACollectionImpl::
ContainsAllFrom (IACollectionImpl const& collection) const
{ bool result = true;
  ICursorImpl& cursor = CursorOf (collection);

  for (collection.SetToFirst (cursor);
       cursor.IsValid ();
       collection.SetToNext (cursor)) {
    if (! Contains (collection.ElementAt (cursor))) {
      result = false;
      break;
    }
  }
  return result;
};

bool
IACollectionImpl::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ ICursorImpl& cursor = CursorOf (collection);
  bool result = true;
  for (collection.SetToFirst (cursor);
       cursor.IsValid ();
       collection.SetToNext (cursor)) {
    if (! ContainsElementWithKey
         (Key (collection.ElementAt (cursor)))) {
      result = false;
      break;
    }
  }
  return result;
}

bool
IACollectionImpl::
ContainsElementWithKey (void const*) const
{ return false; // just a dummy
}

void
IACollectionImpl::
Copy (IACollectionImpl const& collection)
{ if (this != &collection) {
    RemoveAll ();
    AddAllFrom (collection);
  }
#if ! defined (__INO_STREAMING__)
  StreamerOf(*this) = collection.ivStreamer;
#endif
}

void
IACollectionImpl::
Delete ()
{ LockBeforeWriting ();
  if (--ivReferenceCount == 0) {
#if ! defined(__INO_L1_LOCKING__)
    delete ivResource;
#endif
    delete ivOps; // quasimodo suicide
    return;       // ..thus, return as soon as possible
  }
  UnlockAfterWriting ();
}

void*
IACollectionImpl::
ElementAtPosition (IPosition) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
ElementWithKey (void const*) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
First () const
{ SetToFirst (CursorOf (*this));
  return ElementAt (CursorOf (*this));
}

void
IACollectionImpl::
Initialize ()
{ LockBeforeWriting ();
  if (ivReferenceCount++ == 0) {
    ivDummyCursor = CreateCursor ();
  }
  UnlockAfterWriting ();
}

bool
IACollectionImpl::
IsEmpty () const
{ return (NumberOfElements () == 0);
}

bool
IACollectionImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ SetToFirst (CursorOf (*this));
  return (cursor == CursorOf (*this));
}

bool
IACollectionImpl::
IsLastAt (ICursorImpl const& cursor) const
{ SetToLast (CursorOf (*this));
  return (cursor == CursorOf (*this));
}

void*
IACollectionImpl::
Key (void const*) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
Last () const
{ SetToLast (CursorOf (*this));
  return ElementAt (CursorOf (*this));
}

bool
IACollectionImpl::
Locate (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

bool
IACollectionImpl::
LocateNext (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

bool
IACollectionImpl::
LocateElementWithKey (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

bool
IACollectionImpl::
LocateNextElementWithKey (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

bool
IACollectionImpl::
LocateOrAdd (void const*, ICursorImpl&)
{ return false; // just a dummy
}

bool
IACollectionImpl::
LocateOrAddElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

void
IACollectionImpl::
LockBeforeReading (long)
{
}

void
IACollectionImpl::
LockBeforeWriting (long)
{
}

INumber
IACollectionImpl::
MaxNumberOfElements () const
{ ICHECK_NotBounded (ClassName (), "MaxNumberOfElements")
  return 0;
}

INumber
IACollectionImpl::
NumberOfDifferentKeys () const
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
NumberOfElementsWithKey (void const*) const
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
NumberOfOccurrences (void const*) const
{ return 0; // just a dummy
}

IPosition
IACollectionImpl::
PositionAt (ICursorImpl const&) const
{ return 0; // just a dummy
}

void
IACollectionImpl::
Release () const
{
#if ! defined(__INO_L1_LOCKING__)
  ivResource->unlock();
#endif
}

bool
IACollectionImpl::
Remove (void const*)
{ return false; // just a dummy
}

INumber
IACollectionImpl::
RemoveAllElementsWithKey (void const*)
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
RemoveAllOccurrences (void const*)
{ return 0; // just a dummy
}

void
IACollectionImpl::
RemoveAtPosition (IPosition)
{ // just a dummy
}

bool
IACollectionImpl::
RemoveElementWithKey (void const*)
{ return false; // just a dummy
}

void
IACollectionImpl::
RemoveFirst ()
{ SetToFirst (CursorOf (*this));
  RemoveAt (CursorOf (*this));
}

void
IACollectionImpl::
RemoveLast ()
{ SetToLast (CursorOf (*this));
  RemoveAt (CursorOf (*this));
}

void
IACollectionImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ SetToPosition (position, CursorOf (*this));
  ReplaceAt (CursorOf (*this), element);
}

bool
IACollectionImpl::
ReplaceElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

void IACollectionImpl::Reverse()
{ // just a dummy
}

void IACollectionImpl::Seize(long timeout)
{
#if ! defined(__INO_L1_LOCKING__)
    static IPrivateResource* thisCode = 0;

    // If the resource is not created yet, then create it first
    if (!thisCode)
    {
        // Lock the primal resource
        IPrimalLock lockInit;

        // Check again in case another thread got it, else create it
        if (!thisCode)
            thisCode = new IPrivateResource;
    }

    // Lock the resource now that its created
    IResourceLock protect(*thisCode);

    if (ivResource == 0)
        ivResource = new ICollectionResourceImpl;
    ivResource->lock (timeout);
#endif
}

bool IACollectionImpl::SetToLast (ICursorImpl&) const
{ return false; // just a dummy
}

bool
IACollectionImpl::
SetToNextWithDifferentKey (ICursorImpl&) const
{ return false; // just a dummy
}

void
IACollectionImpl::
SetToPosition (IPosition, ICursorImpl&) const
{ // just a dummy
}

bool
IACollectionImpl::
SetToPrevious (ICursorImpl&) const
{ return false; // just a dummy
}

void
IACollectionImpl::
Sort (ICompFunc, void*)
{ // just a dummy
}

void
IACollectionImpl::
UnlockAfterReading ()
{
}

void
IACollectionImpl::
UnlockAfterWriting ()
{
}

#if ! defined (__INO_STREAMING__)
void
IACollectionImpl::
WriteVersion (IDataStream& toWhere) const
{ unsigned long version = STREAMING_VERSION;
  size_t l = strlen(ClassName());
  version >>= toWhere;
  toWhere.writeBytes(ClassName(),l);
}

void
IACollectionImpl::
ReadVersion (IDataStream& fromWhere) // should be const also...
{ unsigned long version;
  size_t l = strlen(ClassName());
  version <<= fromWhere;

  ICHECK (version == STREAMING_VERSION, IStreamErrorExceptionImpl,
          "IACollectionImpl", "ReadVersion");

  char inClass[MAX_CLASSNAME_LENGTH+1];
  memset(inClass,'\0',MAX_CLASSNAME_LENGTH+1);
  fromWhere.readBytes(inClass,l);
  ICHECK (!strncmp(ClassName(),inClass,MAX_CLASSNAME_LENGTH),
          IStreamErrorExceptionImpl,
          "IACollectionImpl", "ReadVersion");
}
#endif

// protected members

IACollectionImpl::
IACollectionImpl (Ops& ops)
: ivOps (&ops), ivReferenceCount (0), ivDummyCursor (0)
#if ! defined(__INO_L1_LOCKING__)
  ,ivResource (0)
#endif
#if ! defined (__INO_STREAMING__)
  ,ivStreamer (0)
#endif
{
}

// ---------------------------
// IACollectionImpl::Predicate
// ---------------------------

// public members

bool
IACollectionImpl::Predicate::
IsContained (void const* element, void* addArg)
{ return ((IACollectionImpl*)addArg)->Contains (element);
}

bool
IACollectionImpl::Predicate::
IsNotContained (void const* element, void* addArg)
{ return ! ((IACollectionImpl*)addArg)->Contains (element);
}

// ---------------
// IACollectionOps
// ---------------

// public members

IACollectionOps::
~IACollectionOps ()
{
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
