// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 80 1.5.1.2 source/core/collect/iiakss.cpp, collection, ioc.v400, 000302  
#ifdef IC_PAGETUNE
  #define _IIAKSS_CPP_
  #include <ipagetun.h>
#endif

#include <iiakss.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif


// ---
// IAKeySortedSetImpl
// ---

// public members

IAKeySortedSetImpl::
~IAKeySortedSetImpl ()
{
}

long
IAKeySortedSetImpl::
Compare (IAKeySortedSetImpl const& collection,
         ICompFunc compFunc, void* addArg) const
{ long result = 0;

  if (this != &collection) {
    ICursorImpl& thisCursor = CursorOf (*this),
               & thatCursor = CursorOf (collection);
    bool thisHasNext,
             thatHasNext;

    for (thisHasNext = SetToFirst (thisCursor),
         thatHasNext = collection.SetToFirst (thatCursor);
         result == 0 && thisHasNext && thatHasNext;
         thisHasNext = SetToNext (thisCursor),
         thatHasNext = collection.SetToNext (thatCursor)) {
      result = (*compFunc) (ElementAt (thisCursor),
                            collection.ElementAt (thatCursor), addArg);
    }

    if (result == 0) {
      if (thisHasNext && ! thatHasNext)
        result = 1;
      else if (! thisHasNext && thatHasNext)
        result = -1;
    }
  }
  return result;
}

// protected members

IAKeySortedSetImpl::
IAKeySortedSetImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

INumber
IAKeySortedSetImpl::
NumberOfDifferentKeys () const
{ return NumberOfElements ();
}

INumber
IAKeySortedSetImpl::
NumberOfElementsWithKey (void const* key) const
{ if (ContainsElementWithKey (key))
    return 1;
  else
    return 0;
}

INumber
IAKeySortedSetImpl::
RemoveAllElementsWithKey (void const* key)
{ if (RemoveElementWithKey (key))
    return 1;
  else
    return 0;
}

bool
IAKeySortedSetImpl::
SetToNextWithDifferentKey (ICursorImpl& cursor) const
{ return SetToNext (cursor);
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
