// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 96 1.5.1.2 source/core/collect/iiass.cpp, collection, ioc.v400, 000302  
#ifdef IC_PAGETUNE
  #define _IIASS_CPP_
  #include <ipagetun.h>
#endif

#include <iiass.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif


// ---
// IASortedSetImpl
// ---

// public members

IASortedSetImpl::
~IASortedSetImpl ()
{
}

void
IASortedSetImpl::
AddAntivalence (IASortedSetImpl const& collection1,
                IASortedSetImpl const& collection2)
{
#if ! defined (__INO_INTERNAL__)
  if (&collection1 != &collection2) {
    bool hasNext;
    for (hasNext = CursorOf (collection1).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection1).SetToNext ()) {
      if (! collection2.Contains
           (collection1.ElementAt (CursorOf (collection1)))) {
        Add (collection1.ElementAt (CursorOf (collection1)),
             CursorOf (*this));
      }
    }
    for (hasNext = CursorOf (collection2).SetToFirst ();
         hasNext;
         hasNext = CursorOf (collection2).SetToNext ()) {
      if (! collection1.Contains
           (collection2.ElementAt (CursorOf (collection2)))) {
        Add (collection2.ElementAt (CursorOf (collection2)),
             CursorOf (*this));
      }
    }
  }
#endif
}

void
IASortedSetImpl::
AddDifference (IASortedSetImpl const& collection1,
               IASortedSetImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IASortedSetImpl::
AddIntersection (IASortedSetImpl const& collection1,
                 IASortedSetImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    bool hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IASortedSetImpl::
AddUnion (IASortedSetImpl const& collection1,
          IASortedSetImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IASortedSetImpl::
Antivalence (IASortedSetImpl const& collection)
{
#if ! defined (__INO_INTERNAL__)
  if (this == &collection) {
    RemoveAll ();
  }
  else {
    for (CursorOf (collection).SetToFirst ();
         CursorOf (collection).IsValid ();
         CursorOf (collection).SetToNext ()) {
      if (! Contains
           (collection.ElementAt (CursorOf (collection)))) {
        Add (collection.ElementAt (CursorOf (collection)),
             CursorOf (*this));
      }
    }
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsContained, (void*)eq);
  }
#endif
}

long
IASortedSetImpl::
Compare (IASortedSetImpl const& collection,
         ICompFunc compFunc, void* addArg) const
{ long result = 0;

  if (this != &collection) {
    ICursorImpl& thisCursor = CursorOf (*this),
               & thatCursor = CursorOf (collection);
    bool thisHasNext,
             thatHasNext;

    for (thisHasNext = SetToFirst (thisCursor),
         thatHasNext = collection.SetToFirst (thatCursor);
         result == 0 && thisHasNext && thatHasNext;
         thisHasNext = SetToNext (thisCursor),
         thatHasNext = collection.SetToNext (thatCursor)) {
      result = (*compFunc) (ElementAt (thisCursor),
                            collection.ElementAt (thatCursor), addArg);
    }

    if (result == 0) {
      if (thisHasNext && ! thatHasNext)
        result = 1;
      else if (! thisHasNext && thatHasNext)
        result = -1;
    }
  }
  return result;
}

void
IASortedSetImpl::
Difference (IASortedSetImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IASortedSetImpl::
Intersection (IASortedSetImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IASortedSetImpl::
Union (IASortedSetImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

bool
IASortedSetImpl::
operator!= (IASortedSetImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IASortedSetImpl::
IASortedSetImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

INumber
IASortedSetImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IASortedSetImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
