// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 40 1.5.1.2 source/core/collect/iicset.c, collection, ioc.v400, 000302 
#ifndef _IICSET_C_
#define _IICSET_C_

#include <iiexc.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// ICSetImpl
// ---

// public members

template <class CInherited>
ICSetImpl <CInherited>::
ICSetImpl (Ops& ops, INumber numberOfElements)
: CInherited (ops, numberOfElements)
{
}

template <class CInherited>
ICSetImpl <CInherited>::
ICSetImpl
  (Ops& ops, ICSetImpl <CInherited> const& collection)
: CInherited (ops, collection)
{
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
Add (void const* element, ICursorImpl& cursor)
{ ICHECK_IsFor      (this->ClassName (), "Add")

  bool hasBeenAdded = CInherited::Add (element, cursor);
  return hasBeenAdded;
}

template <class CInherited>
void
ICSetImpl <CInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ ICHECK_Identical (this->ClassName (), "AddAllFrom")

  CInherited::AddAllFrom (collection);
}

template <class CInherited>
void*
ICSetImpl <CInherited>::
Any () const
{ ICHECK_IsEmpty (this->ClassName (), "Any")

  return CInherited::Any ();
}

template <class CInherited>
void*
ICSetImpl <CInherited>::
ElementAt (ICursorImpl const& cursor) const
{ ICHECK_IsFor        (this->ClassName (), "ElementAt")
  ICHECK_IsValid      (this->ClassName (), "ElementAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "ElementAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "ElementAt")
#endif

  return CInherited::ElementAt (cursor);
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
Locate (void const* element, ICursorImpl& cursor) const
{ ICHECK_IsFor (this->ClassName (), "Locate")

  return CInherited::Locate (element, cursor);
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
LocateNext (void const* element, ICursorImpl& cursor) const
{ ICHECK_IsFor   (this->ClassName (), "LocateNext")
  ICHECK_IsValid (this->ClassName (), "LocateNext")
#if defined (IALL_CHECKS)
  ICHECK_Cursor  (this->ClassName (), "LocateNext")
#endif

  return CInherited::LocateNext (element, cursor);
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
LocateOrAdd (void const* element, ICursorImpl& cursor)
{ ICHECK_IsFor (this->ClassName (), "LocateOrAdd")

  return CInherited::LocateOrAdd (element, cursor);
}

template <class CInherited>
void
ICSetImpl <CInherited>::
RemoveAt (ICursorImpl& cursor)
{ ICHECK_IsFor        (this->ClassName (), "RemoveAt")
  ICHECK_IsValid      (this->ClassName (), "RemoveAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "RemoveAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "RemoveAt")
#endif

  CInherited::RemoveAt (cursor);
}

template <class CInherited>
void
ICSetImpl <CInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ ICHECK_IsFor        (this->ClassName (), "ReplaceAt")
  ICHECK_IsValid      (this->ClassName (), "ReplaceAt")
#if ! defined (__INO_MNGCURSOR__)
  ICHECK_NotInBetween (this->ClassName (), "ReplaceAt")
#endif
#if defined (IALL_CHECKS)
  ICHECK_Cursor       (this->ClassName (), "ReplaceAt")
#endif
  ICHECK_Replacement  (this->ClassName (), "ReplaceAt")

  CInherited::ReplaceAt (cursor, element);
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
SetToFirst (ICursorImpl& cursor) const
{ ICHECK_IsFor (this->ClassName (), "SetToFirst")

  return CInherited::SetToFirst (cursor);
}

template <class CInherited>
bool
ICSetImpl <CInherited>::
SetToNext (ICursorImpl& cursor) const
{ ICHECK_IsFor   (this->ClassName (), "SetToNext")
  ICHECK_IsValid (this->ClassName (), "SetToNext")
#if defined (IALL_CHECKS)
  ICHECK_Cursor  (this->ClassName (), "SetToNext")
#endif

  return CInherited::SetToNext (cursor);
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IICSET_C_
