// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#include <ilanglvl.hpp>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// --------------------------
// IMapOnKeySetMngCursorImpl
// --------------------------

// public members

template <class Base>
IMapOnKeySetMngCursorImpl <Base>::
IMapOnKeySetMngCursorImpl
  (IMapOnKeySetImpl <Base> const& collection)
: Inherited (IMapOnKeySetImpl <Base>::BaseOf (collection))
{ this->ivCollection = &collection;
}

template <class Base>
IMapOnKeySetMngCursorImpl <Base>::
~IMapOnKeySetMngCursorImpl ()
{
}

// ----------------
// IMapOnKeySetImpl
// ----------------

// inline protected members

template <class Base>
inline bool
IMapOnKeySetImpl <Base>::
EqualElements (void const* element1, void const* element2) const
{ return OpsOf (*this).EqualElements (element1, element2);
}

// inline private members

template <class Base>
inline Base&
IMapOnKeySetImpl <Base>::
BaseOf (IMapOnKeySetImpl <Base> const& collection)
{ return collection.ivBase;
}

template <class Base>
inline TYPENAME IMapOnKeySetImpl <Base>::Ops&
IMapOnKeySetImpl <Base>::
OpsOf (IMapOnKeySetImpl <Base> const& collection)
{ return (Ops&) Inherited::OpsOf (collection);
}

// public members

template <class Base>
IMapOnKeySetImpl <Base>::
IMapOnKeySetImpl (Ops& ops, INumber)
: IAMapImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IMapOnKeySetImpl <Base>::
IMapOnKeySetImpl (Ops& ops, IMapOnKeySetImpl <Base> const&)
: IAMapImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IMapOnKeySetImpl <Base>::
~IMapOnKeySetImpl ()
{
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
Add (void const* element, ICursorImpl& cursor)
{ return BaseOf (*this).Base::Add (element, cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ return BaseOf (*this).AddOrReplaceElementWithKey (element, cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ return BaseOf (*this).Base::AllElementsDo (applFunc, addArg);
}

template <class Base>
void*
IMapOnKeySetImpl <Base>::
Any () const
{ return BaseOf (*this).Base::Any ();
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
CheckCursor (ICursorImpl const& cursor) const
{ return BaseOf (*this).Base::CheckCursor (cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
CheckReplacement (ICursorImpl const& cursor, void const* element) const
{ return BaseOf (*this).Base::CheckReplacement (cursor, element);
}

template <class Base>
char const*
IMapOnKeySetImpl <Base>::
ClassName () const
{ return cvClassName;
}

template <class Base>
IACollectionImpl*
IMapOnKeySetImpl <Base>::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
Contains (void const* element) const
{ bool isContained = false;
  Cursor cursor (*this);

  if (Self::LocateElementWithKey (Key (element), cursor)) {
    isContained = EqualElements (Self::ElementAt (cursor), element);
  }

  return isContained;
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& map = (Self const&)collection;
    return BaseOf (*this).Base::ContainsAllKeysFrom (BaseOf (map));
  }
  else
    return Inherited::ContainsAllKeysFrom (collection);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
ContainsElementWithKey (void const* key) const
{ return BaseOf (*this).Base::ContainsElementWithKey (key);
}

template <class Base>
void
IMapOnKeySetImpl <Base>::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

template <class Base>
TYPENAME IMapOnKeySetImpl <Base>::Cursor*
IMapOnKeySetImpl <Base>::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

template <class Base>
TYPENAME IMapOnKeySetImpl <Base>::MngCursor*
IMapOnKeySetImpl <Base>::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

template <class Base>
void*
IMapOnKeySetImpl <Base>::
ElementAt (ICursorImpl const& cursor) const
{ return BaseOf (*this).Base::ElementAt (cursor);
}

template <class Base>
void*
IMapOnKeySetImpl <Base>::
ElementWithKey (void const* key) const
{ return BaseOf (*this).Base::ElementWithKey (key);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
IsBounded () const
{ return BaseOf (*this).Base::IsBounded ();
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
IsConsistent () const
{ return BaseOf (*this).Base::IsConsistent ();
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
IsEmpty () const
{ return BaseOf (*this).Base::IsEmpty ();
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
IsFull () const
{ return BaseOf (*this).Base::IsFull ();
}

template <class Base>
void*
IMapOnKeySetImpl <Base>::
Key (void const* element) const
{ return BaseOf (*this).Base::Key (element);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
Locate (void const* element, ICursorImpl& cursor) const
{ bool isContained =
    Self::LocateElementWithKey (Key (element), cursor);

  if (isContained) {
    isContained = EqualElements (Self::ElementAt (cursor), element);
  }
  if (! isContained)
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
LocateElementWithKey (void const* key, ICursorImpl& cursor) const
{ return BaseOf (*this).Base::LocateElementWithKey (key, cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
LocateOrAdd (void const* element, ICursorImpl& cursor)
{ bool isContained = Self::Locate (element, cursor);

  if (! isContained)
    Add (element, cursor);

  return isContained;
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
LocateOrAddElementWithKey (void const* element, ICursorImpl& cursor)
{ return BaseOf (*this).
    Base::LocateOrAddElementWithKey (element, cursor);
}

template <class Base>
INumber
IMapOnKeySetImpl <Base>::
NumberOfElements () const
{ return BaseOf (*this).Base::NumberOfElements ();
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
Remove (void const* element)
{ bool isContained =
    Self::LocateElementWithKey (Key (element), CursorOf (*this));

  if (isContained)
    isContained =
      EqualElements (Self::ElementAt (CursorOf (*this)), element);
  if (isContained)
    Self::RemoveAt (CursorOf (*this));

  return isContained;
}

template <class Base>
INumber
IMapOnKeySetImpl <Base>::
RemoveAll ()
{ return BaseOf (*this).Base::RemoveAll ();
}

template <class Base>
INumber
IMapOnKeySetImpl <Base>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ return BaseOf (*this).Base::RemoveAll (predFunc, addArg);
}

template <class Base>
void
IMapOnKeySetImpl <Base>::
RemoveAt (ICursorImpl& cursor)
{ BaseOf (*this).Base::RemoveAt (cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
RemoveElementWithKey (void const* key)
{ return BaseOf (*this).Base::RemoveElementWithKey (key);
}

template <class Base>
void
IMapOnKeySetImpl <Base>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ BaseOf (*this).Base::ReplaceAt (cursor, element);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ return BaseOf (*this).ReplaceElementWithKey (element, cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
SetToFirst (ICursorImpl& cursor) const
{ return BaseOf (*this).Base::SetToFirst (cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
SetToNext (ICursorImpl& cursor) const
{ return BaseOf (*this).Base::SetToNext (cursor);
}

template <class Base>
bool
IMapOnKeySetImpl <Base>::
operator== (IAMapImpl const& collection) const
{ return (Self::NumberOfElements () == collection.NumberOfElements () &&
          Self::ContainsAllFrom (collection));
}

#if ! defined (__INO_STREAMING__)
template <class Base>
void
IMapOnKeySetImpl <Base>::
StreamIn (IDataStream& stream)
{ ReadVersion (stream);
  BaseOf (*this).Base::StreamIn (stream);
}

template <class Base>
void
IMapOnKeySetImpl <Base>::
StreamOut (IDataStream& stream) const
{ WriteVersion (stream);
  BaseOf (*this).Base::StreamOut (stream);
}

template <class Base>
void
IMapOnKeySetImpl <Base>::
AdoptStreamer (IACollectionStreamer *streamer)
{ (BaseOf (*this).Base::AdoptStreamer(streamer));
}

template <class Base>
IStreamerPtr&
IMapOnKeySetImpl <Base>::
StreamerOf (IACollectionImpl const& collection)
{ return (BaseOf (*this).Base::StreamerOf(collection));
}
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
