// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 70 1.5.1.3 source/core/collect/iimwt.c, collection, ioc.v400, 000302 
#ifndef _IIMWT_C_
#define _IIMWT_C_

#include <new.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ----------------
// IMultiwayTreeOps
// ----------------

// public members

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
IMultiwayTreeOps ()
: Inherited (__isDTSClass (Element)),
  ivElementOps (),
  ivImpl (*this)
{
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
IMultiwayTreeOps (void* opsArg)
: Inherited (__isDTSClass (Element)),
  ivElementOps (opsArg),
  ivImpl (*this)
{
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
IMultiwayTreeOps
  (IMultiwayTreeOps
    <numOfChildren, Element, ElementOps, Implementation> const& tree)
: Inherited (__isDTSClass (Element)),
  ivElementOps (tree.ivElementOps),
  ivImpl (*this, tree.ivImpl)
{
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
~IMultiwayTreeOps ()
{
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
Assign (void* element1, void const* element2) const
{ ivElementOps.Assign (*(Element*)element1, *(Element const*)element2);
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IATreeImpl*
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
Clone () const
{ return (Implementation*) *new Self (*this);
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void*
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
CreateNode (void const* element) const
{ void* node = ivElementOps.Allocate (sizeof (Node)
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__) || defined (_AIX)
#if defined (__DEBUG_ALLOC__)
                                      , __FILE__, __LINE__
#endif
#endif
#if defined (__OSTORE__)
                                      , Node::get_os_typespec(), 1
#endif
                                     );
  return new (ivImpl.CheckPointer (node))
    Node (*(Element const*)element);
}

#if ! defined (__INO_STREAMING__)
template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void*
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
CreateNodeWithStreamer(IACollectionStreamer *streamer,
                       IDataStream& fromWhere) const
{ void* node = ivElementOps.Allocate (sizeof (Node)
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__) || defined (_AIX)
#if defined (__DEBUG_ALLOC__)
                                      , __FILE__, __LINE__
#endif
#endif
                                     );
  ivImpl.CheckPointer (node);
  ((Node*)node)->StreamIn (streamer, fromWhere);
  return node;
}
#endif

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
DeleteNode (void* node) const
{ ((Node*)node)->~Node ();
  ivElementOps.Deallocate (node
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__) ||  defined (_AIX)
#if defined (__DEBUG_ALLOC__)
                           , __FILE__, __LINE__
#endif
#endif
                          );
}

#if ! defined (__INO_STREAMING__)
template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
ReadTreeStructure (IDataStream& fromWhere)
{ INumber n;
  n <<= fromWhere;
  ICHECK_StreamInChildren ("IMultiwayTreeOps", "ReadTreeStucture");
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
void
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
WriteTreeStructure (IDataStream& toWhere) const
{ numOfChildren >>= toWhere;
}
#endif

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
operator INumber () const
{ return numOfChildren;
}

template <INumber numOfChildren, class Element,
          class ElementOps, class Implementation>
IMultiwayTreeOps <numOfChildren, Element, ElementOps, Implementation>::
operator Implementation* ()
{ ivImpl.CheckPointer (this);
  return &ivImpl;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIMWT_C_
