// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#include <iimwt.h>
#include <iiexc.h>

#ifdef IC_PAGETUNE
  #define _IIMWT_CPP_
  #include <ipagetun.h>
#endif


#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IAMultiwayTreeOps)
#endif

// -----------------------
// IMultiwayTreeCursorImpl
// -----------------------

IMultiwayTreeCursorImpl::
IMultiwayTreeCursorImpl (IATreeImpl const& tree)
: ITreeCursorImpl (tree), ivNode (0)
{
}

IMultiwayTreeCursorImpl::
IMultiwayTreeCursorImpl (IMultiwayTreeCursorImpl const& cursor)
: ITreeCursorImpl (cursor), ivNode (cursor.ivNode)
{
}

IMultiwayTreeCursorImpl::
~IMultiwayTreeCursorImpl ()
{
}

ITreeCursorImpl*
IMultiwayTreeCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
IMultiwayTreeCursorImpl::
Copy (ITreeCursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}

// -----------------
// IMultiwayTreeImpl
// -----------------

char const*
IMultiwayTreeImpl::cvClassName = "MultiwayTree";

// inline private members

inline IMultiwayTreeImpl::Ops&
IMultiwayTreeImpl::
OpsOf (IMultiwayTreeImpl const& tree)
{ return (Ops&) Inherited::OpsOf (tree);
}

inline void*
IMultiwayTreeImpl::
ElementOf (Node const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ?
    (void*) ((IMultiwayTreeNodeImpl <1, long>*)node)->ivElement :
    (void*)&((IMultiwayTreeNodeImpl <1, long>*)node)->ivElement);
}

inline IMultiwayTreeImpl::Node*&
IMultiwayTreeImpl::
NodeOf (ITreeCursorImpl const& cursor)
{ return ((Cursor&)cursor).ivNode;
}

// inline protected members

inline void
IMultiwayTreeImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void*
IMultiwayTreeImpl::
CreateNode (void const* element) const
{ return OpsOf (*this).CreateNode (element);
}

#if ! defined (__INO_STREAMING__)
inline void*
IMultiwayTreeImpl::
CreateNodeWithStreamer (IDataStream& stream) const
{ return OpsOf (*this).CreateNodeWithStreamer (ivStreamer, stream);
}

inline void
IMultiwayTreeImpl::
ReadTreeStructure (IDataStream& stream)
{ OpsOf (*this).ReadTreeStructure (stream);
}

inline void
IMultiwayTreeImpl::
WriteTreeStructure (IDataStream& stream) const
{ OpsOf (*this).WriteTreeStructure (stream);
}
#endif

inline void
IMultiwayTreeImpl::
DeleteNode (void* node) const
{ OpsOf (*this).DeleteNode (node);
}

// public members

IMultiwayTreeImpl::
IMultiwayTreeImpl (Ops& ops)
: IATreeImpl ((Inherited::Ops&)ops),
  ivRoot (0), ivNumberOfChildren (ops)
{
}

IMultiwayTreeImpl::
IMultiwayTreeImpl (Ops& ops, IMultiwayTreeImpl const& tree)
: IATreeImpl ((Inherited::Ops&)ops),
  ivRoot (0), ivNumberOfChildren (tree.ivNumberOfChildren)
{ operator= (tree);
}

IMultiwayTreeImpl::
~IMultiwayTreeImpl ()
{ Self::RemoveAll ();
}

void
IMultiwayTreeImpl::
AddAsRoot (void const* element)
{ ivRoot = (Node*) CreateNode (element);
  InitNode (ivRoot, 0);
}

void
IMultiwayTreeImpl::
AddAsChild (ITreeCursorImpl const& cursor,
            IPosition position, void const* element)
{ Node* parent = NodeOf (cursor);
  Node*& child = parent->ivChildren [position - 1];
  child = (Node*) CreateNode (element);
  InitNode (child, parent);
}

void
IMultiwayTreeImpl::
AttachAsChild (ITreeCursorImpl const& cursor, IPosition position,
               IMultiwayTreeImpl& tree)
{ Node* parent = NodeOf (cursor);
  Node*& child = parent->ivChildren [position - 1];
  child = tree.ivRoot;
  if (child != 0)
    child->ivParent = parent;
  tree.ivRoot = 0;
}

void
IMultiwayTreeImpl::
AttachAsRoot (IMultiwayTreeImpl& tree)
{ ivRoot = tree.ivRoot;
  tree.ivRoot = 0;
}

void
IMultiwayTreeImpl::
AttachSubtreeAsChild (ITreeCursorImpl const& cursorTo,
                      IPosition position,
                      IMultiwayTreeImpl& tree,
                      ITreeCursorImpl const& cursorFrom)
{ Node* parent = NodeOf (cursorTo);
  Node* nodeFrom = NodeOf (cursorFrom);
  if (nodeFrom->ivParent == 0)
    tree.ivRoot = 0;
  else {
    nodeFrom->ivParent->ivChildren
      [tree.ChildPositionAt (cursorFrom) - 1] = 0;
  }
  parent->ivChildren [position - 1] = nodeFrom;
  if (nodeFrom != 0)
    nodeFrom->ivParent = parent;
}

void
IMultiwayTreeImpl::
AttachSubtreeAsRoot (IMultiwayTreeImpl& tree,
                     ITreeCursorImpl const& cursor)
{ Node* node = NodeOf (cursor);
  if (node->ivParent == 0)
    tree.ivRoot = 0;
  else {
    node->ivParent->ivChildren
      [tree.ChildPositionAt (cursor) - 1] = 0;
  }
  ivRoot = node;
  if (ivRoot != 0)
    ivRoot->ivParent = 0;
}

bool
IMultiwayTreeImpl::
AllElementsDo (IApplFunc applFunc,
               ITreeIterationOrder order, void* addArg)
{ if (ivRoot == 0)
    return true;
  else
    return AllElementsDoSubtree (applFunc, order, addArg, ivRoot);
}

bool
IMultiwayTreeImpl::
AllSubtreeElementsDo (ITreeCursorImpl const& cursor,
                      IApplFunc applFunc,
                      ITreeIterationOrder order, void* addArg)
{  return AllElementsDoSubtree
     (applFunc, order, addArg, NodeOf (cursor));
}

bool
IMultiwayTreeImpl::
CheckChild (ITreeCursorImpl const& cursor, IPosition position) const
{ return (NodeOf (cursor)->ivChildren [position - 1] == 0);
}

bool
IMultiwayTreeImpl::
CheckCursor (ITreeCursorImpl const& cursor) const
{ if (ivRoot != 0)
    return CheckCursorSubtree (cursor, ivRoot);
  else
    return false;
}

bool
IMultiwayTreeImpl::
CheckCursor (ITreeCursorImpl const& cursor,
             ITreeCursorImpl const& thatCursor) const
{ return CheckCursorSubtree (cursor, NodeOf (thatCursor));
}

INumber
IMultiwayTreeImpl::
ChildPositionAt (ITreeCursorImpl const& cursor) const
{ Node* parent = NodeOf (cursor)->ivParent;
  IPosition position = 1;

  if (parent != 0) {
    for (INumber n = 0; n < ivNumberOfChildren; n++) {
      if (parent->ivChildren [n] == NodeOf (cursor))
        position = n + 1;
    }
  }
  return position;
}

char const*
IMultiwayTreeImpl::
ClassName () const
{ return cvClassName;
}

IATreeImpl*
IMultiwayTreeImpl::
Clone () const
{ return (IATreeImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
IMultiwayTreeImpl::
Copy (IMultiwayTreeImpl const& tree)
{ operator= (tree);
}

void
IMultiwayTreeImpl::
CopySubtree (IMultiwayTreeImpl const& tree,
             ITreeCursorImpl const& cursor)
{ if (this != &tree) {
    Self::RemoveAll ();
    ivRoot = Self::CopySubtree (NodeOf (cursor));
    IASSERT (ivRoot->ivParent == 0);
  }
  else {
    Node* root;
    Node* node = NodeOf (cursor);
    if (node->ivParent == 0)
      ivRoot = 0;
    else {
      node->ivParent->ivChildren
        [Self::ChildPositionAt (cursor) - 1] = 0;
    }
    root = node;
    if (root != 0)
      root->ivParent = 0;

    Self::RemoveAll ();

    ivRoot = root;
  }
}

IMultiwayTreeImpl::Cursor*
IMultiwayTreeImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

void*
IMultiwayTreeImpl::
ElementAt (ITreeCursorImpl const& cursor) const
{ return ElementOf (NodeOf (cursor));
}

bool
IMultiwayTreeImpl::
HasChild (IPosition position, ITreeCursorImpl const& cursor) const
{ return (NodeOf (cursor)->ivChildren [position - 1] != 0);
}

bool
IMultiwayTreeImpl::
IsConsistent () const
{ return (ivRoot == 0) ||
         (ivRoot->ivParent == 0 && IsConsistentSubtree (ivRoot));
}

bool
IMultiwayTreeImpl::
IsEmpty () const
{ return (ivRoot == 0);
}

bool
IMultiwayTreeImpl::
IsLeaf (ITreeCursorImpl const& cursor) const
{ for (INumber n = 0; n < ivNumberOfChildren; n++)
    if (NodeOf (cursor)->ivChildren [n] != 0)
      return false;
  return true;
}

bool
IMultiwayTreeImpl::
IsRoot (ITreeCursorImpl const& cursor) const
{ return (NodeOf (cursor) == ivRoot);
}

INumber
IMultiwayTreeImpl::
NumberOfChildren () const
{ return ivNumberOfChildren;
}

INumber
IMultiwayTreeImpl::
NumberOfElements () const
{ if (ivRoot == 0)
    return 0;
  else
    return Self::NumberOfSubtreeElements (ivRoot);
}

INumber
IMultiwayTreeImpl::
NumberOfLeaves () const
{ if (ivRoot == 0)
    return 0;
  else
    return Self::NumberOfSubtreeLeaves (ivRoot);
}

INumber
IMultiwayTreeImpl::
NumberOfSubtreeElements (ITreeCursorImpl const& cursor) const
{ return NumberOfSubtreeElements (NodeOf (cursor));
}

INumber
IMultiwayTreeImpl::
NumberOfSubtreeLeaves (ITreeCursorImpl const& cursor) const
{ return NumberOfSubtreeLeaves (NodeOf (cursor));
}

INumber
IMultiwayTreeImpl::
RemoveAll ()
{ INumber removed = 0;

  if (ivRoot != 0) {
    removed = DeleteSubtree (ivRoot);
    ivRoot = 0;
  }

  return removed;
}

INumber
IMultiwayTreeImpl::
RemoveSubtree (ITreeCursorImpl& cursor)
{ if (NodeOf (cursor)->ivParent == 0)
    ivRoot = 0;
  else {
    NodeOf (cursor)->ivParent->ivChildren
      [Self::ChildPositionAt (cursor) - 1] = 0;
  }
  INumber removed = DeleteSubtree (NodeOf (cursor));
  cursor.Invalidate ();
  return removed;
}

void
IMultiwayTreeImpl::
ReplaceAt (ITreeCursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

bool
IMultiwayTreeImpl::
SetToChild (IPosition position, ITreeCursorImpl& cursor) const
{ return ((NodeOf (cursor) =
    NodeOf (cursor)->ivChildren [position - 1]) != 0);
}

bool
IMultiwayTreeImpl::
SetToFirst (ITreeCursorImpl& cursor, ITreeIterationOrder order) const
{ NodeOf (cursor) = ivRoot;
  if (ivRoot == 0)
    return false;
  if (order == IPostorder) {
    Cursor firstChild = (Cursor&)cursor;
    while (Self::SetToFirstExistingChild (firstChild))
      ((Cursor&)cursor) = firstChild;
  }
  return true;
}

bool
IMultiwayTreeImpl::
SetToFirstExistingChild (ITreeCursorImpl& cursor) const
{ Node*& node = NodeOf (cursor);
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (node->ivChildren [n] != 0) {
      node = node->ivChildren [n];
      return true;
    }
  }
  node = 0;
  return false;
}

bool
IMultiwayTreeImpl::
SetToLast (ITreeCursorImpl& cursor, ITreeIterationOrder order) const
{ NodeOf (cursor) = ivRoot;
  if (ivRoot == 0)
    return false;
  if (order == IPreorder) {
    Cursor lastChild = (Cursor&)cursor;
    while (Self::SetToLastExistingChild (lastChild))
      ((Cursor&)cursor) = lastChild;
  }
  return true;
}

bool
IMultiwayTreeImpl::
SetToLastExistingChild (ITreeCursorImpl& cursor) const
{ Node*& node = NodeOf (cursor);
  for (INumber n = ivNumberOfChildren; n != 0; ) {
    n--;
    if (node->ivChildren [n] != 0) {
      node = node->ivChildren [n];
      return true;
    }
  }
  node = 0;
  return false;
}

bool
IMultiwayTreeImpl::
SetToNext (ITreeCursorImpl& cursor, ITreeIterationOrder order) const
{ Cursor child = (Cursor&)cursor;
  if (order == IPreorder) {
    if (Self::SetToFirstExistingChild (child)) {
      ((Cursor&)cursor) = child;
      return true;
    }
    else {
      child = (Cursor&)cursor;
      while (! Self::SetToNextExistingChild (child)) {
        if (! Self::SetToParent (cursor))
          return false;
        child = (Cursor&)cursor;
      }
      ((Cursor&)cursor) = child;
      return true;
    }
  }
  else {
    if (Self::SetToNextExistingChild (child)) {
      ((Cursor&)cursor) = child;
      while (Self::SetToFirstExistingChild (child))
        ((Cursor&)cursor) = child;
      return true;
    }
    else
      return Self::SetToParent (cursor);
  }
}

bool
IMultiwayTreeImpl::
SetToNextExistingChild (ITreeCursorImpl& cursor) const
{ Node*& node = NodeOf (cursor);
  Node* parent = node->ivParent;
  if (parent != 0) {
    bool skippedNode = false;
    for (INumber n = 0; n < ivNumberOfChildren; n++) {
      if (skippedNode) {
        if (parent->ivChildren [n] != 0) {
          node = parent->ivChildren [n];
          return true;
        }
      }
      else {
        if (parent->ivChildren [n] == node) {
          skippedNode = true;
        }
      }
    }
  }
  node = 0;
  return false;
}

bool
IMultiwayTreeImpl::
SetToParent (ITreeCursorImpl& cursor) const
{ return ((NodeOf (cursor) = NodeOf (cursor)->ivParent) != 0);
}

bool
IMultiwayTreeImpl::
SetToPrevious (ITreeCursorImpl& cursor, ITreeIterationOrder order) const
{ Cursor child = (Cursor&)cursor;
  if (order == IPostorder) {
    if (SetToLastExistingChild (child)) {
      ((Cursor&)cursor) = child;
      return true;
    }
    else {
      child = (Cursor&)cursor;
      while (! SetToPreviousExistingChild (child)) {
        if (! SetToParent (cursor))
          return false;
        child = (Cursor&)cursor;
      }
      ((Cursor&)cursor) = child;
      return true;
    }
  }
  else {
    if (Self::SetToPreviousExistingChild (child)) {
      ((Cursor&)cursor) = child;
      while (Self::SetToLastExistingChild (child))
        ((Cursor&)cursor) = child;
      return true;
    }
    else
      return Self::SetToParent (cursor);
  }
}

bool
IMultiwayTreeImpl::
SetToPreviousExistingChild (ITreeCursorImpl& cursor) const
{ Node*& node = NodeOf (cursor);
  Node* parent = node->ivParent;
  if (parent != 0) {
    bool skippedNode = false;
    for (INumber n = ivNumberOfChildren; n != 0; ) {
      n--;
      if (skippedNode) {
        if (parent->ivChildren [n] != 0) {
          node = parent->ivChildren [n];
          return true;
        }
      }
      else {
        if (parent->ivChildren [n] == node) {
          skippedNode = true;
        }
      }
    }
  }
  node = 0;
  return false;
}

bool
IMultiwayTreeImpl::
SetToRoot (ITreeCursorImpl& cursor) const
{ return ((NodeOf (cursor) = ivRoot) != 0);
}

void
IMultiwayTreeImpl::
operator= (IMultiwayTreeImpl const& tree)
{ if (this != &tree) {
    Self::RemoveAll ();
    if (tree.ivRoot != 0) {
      ivRoot = Self::CopySubtree (tree.ivRoot);
    }
#if ! defined (__INO_STREAMING__)
    ivStreamer = tree.ivStreamer;
#endif
  }
}

#if ! defined (__INO_STREAMING__)
void
IMultiwayTreeImpl::
StreamIn (IDataStream& fromWhere)
{
  ReadVersion (fromWhere);
  ReadTreeStructure (fromWhere);
  Self::RemoveAll ();
  short hasSubtree;
  hasSubtree <<= fromWhere;
  if (hasSubtree) {
    ivRoot = StreamInSubtree (fromWhere);
    ivRoot->ivParent = 0;
  }
}

void
IMultiwayTreeImpl::
StreamOut (IDataStream& toWhere) const
{
  WriteVersion (toWhere);
  WriteTreeStructure (toWhere);
  if (ivRoot != 0) {
    (short)true >>= toWhere;
    StreamOutSubtree (ivRoot, toWhere);
  }
  else
    (short)false >>= toWhere;
}

void
IMultiwayTreeImpl::
AdoptStreamer (IACollectionStreamer *streamer)
{ if (streamer != ivStreamer.PtrOf())
    ivStreamer = IStreamerPtr(streamer);
}
#endif

// private members

bool
IMultiwayTreeImpl::
AllElementsDoSubtree (IApplFunc applFunc,
                      ITreeIterationOrder order,
                      void* addArg, Node* current)
{ if (order == IPreorder) {
    if (! (*applFunc) (ElementOf (current), addArg))
      return false;
    for (INumber i = 0; i < ivNumberOfChildren; i++) {
      Node* child = current->ivChildren [i];
      if (child != 0)
        if (! AllElementsDoSubtree (applFunc, order, addArg, child))
          return false;
    }
  }
  else {
    for (INumber i = 0; i < ivNumberOfChildren; i++) {
      Node* child = current->ivChildren [i];
      if (child != 0)
        if (! AllElementsDoSubtree (applFunc, order, addArg, child))
          return false;
    }
    if (! (*applFunc) (ElementOf (current), addArg))
      return false;
  }
  return true;
}

bool
IMultiwayTreeImpl::
CheckCursorSubtree (ITreeCursorImpl const& cursor, Node* current) const
{ if (NodeOf (cursor) == current)
    return true;

  for (INumber i = 0; i < ivNumberOfChildren; i++)
    if (current->ivChildren [i] != 0 &&
        CheckCursorSubtree (cursor, current->ivChildren [i]))
      return true;

  return false;
}

IMultiwayTreeImpl::Node*
IMultiwayTreeImpl::
CopySubtree (Node const* current) const
{ IASSERT (current != 0);
  Node* node = (Node*) CreateNode (ElementOf (current));
  for (INumber i = 0; i < ivNumberOfChildren; i++) {
    if (current->ivChildren [i] == 0)
      node->ivChildren [i] = 0;
    else {
      node->ivChildren [i] = CopySubtree (current->ivChildren [i]);
      node->ivChildren [i]->ivParent = node;
    }
  }
  return node;
}

INumber
IMultiwayTreeImpl::
DeleteSubtree (Node* current) const
{ IASSERT (current != 0);
  INumber removed = 1;
  for (INumber i = 0; i < ivNumberOfChildren; i++) {
    if (current->ivChildren [i] != 0) {
      removed += DeleteSubtree (current->ivChildren [i]);
    }
  }
  DeleteNode (current);
  return removed;
}

void
IMultiwayTreeImpl::
InitNode (Node* node, Node* parent) const
{ node->ivParent = parent;
  for (INumber n = 0; n < ivNumberOfChildren; n++)
    node->ivChildren [n] = 0;
}

bool
IMultiwayTreeImpl::
IsConsistentSubtree (Node* current) const
{ for (INumber i = 0; i < ivNumberOfChildren; i++) {
    if (current->ivChildren [i] != 0 &&
        (current->ivChildren [i]->ivParent != current ||
         ! IsConsistentSubtree (current->ivChildren [i])))
      return false;
  }
  return true;
}

INumber
IMultiwayTreeImpl::
NumberOfSubtreeElements (Node const* current) const
{ IASSERT (current != 0);

  INumber result = 0;
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (current->ivChildren [n] != 0)
      result += Self::NumberOfSubtreeElements (current->ivChildren [n]);
  }
  return result + 1;
}

INumber
IMultiwayTreeImpl::
NumberOfSubtreeLeaves (Node const* current) const
{ IASSERT (current != 0);

  INumber result = 0;
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (current->ivChildren [n] != 0)
      result += Self::NumberOfSubtreeLeaves (current->ivChildren [n]);
  }
  if (result == 0)
    // node is a leaf
    result = 1;

  return result;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nopar, nouse)
#endif

#if ! defined (__INO_STREAMING__)
IMultiwayTreeImpl::Node*
IMultiwayTreeImpl::
StreamInSubtree (IDataStream& fromWhere) const
{ Node* current = 0;
  current = (Node*) CreateNodeWithStreamer (fromWhere);
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    short hasSubtree; // should be bool type
    hasSubtree <<= fromWhere;
    if (hasSubtree) {
      current->ivChildren [n] = StreamInSubtree (fromWhere);
      current->ivChildren [n]->ivParent = current;
    }
    else
      current->ivChildren [n] = (Node*) 0;
  }
  return current;
}

void
IMultiwayTreeImpl::
StreamOutSubtree (Node const* current, IDataStream& toWhere) const
{
  ivStreamer->streamOut (toWhere, ElementOf (current));
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (current->ivChildren [n] != 0) {
      (short)true >>= toWhere;
      StreamOutSubtree (current->ivChildren [n], toWhere);
    }
    else
      (short)false >>= toWhere;
  }
}
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
