// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#include <iiseqlst.h>

#ifdef IC_PAGETUNE
  #define _IISEQLST_CPP_
  #include <ipagetun.h>
#endif


#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IASequenceAsListOps)
#endif

// -------------------------
// ISequenceAsListCursorImpl
// -------------------------

// public members

ISequenceAsListCursorImpl::
ISequenceAsListCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivNode (0)
{
}

ISequenceAsListCursorImpl::
ISequenceAsListCursorImpl
  (ISequenceAsListCursorImpl const& cursor)
: ICursorImpl (cursor), ivNode (cursor.ivNode)
{
}

ISequenceAsListCursorImpl::
~ISequenceAsListCursorImpl ()
{
}

ICursorImpl*
ISequenceAsListCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsListCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}

#if ! defined (__INO_MNGCURSOR__)
// ----------------------------
// ISequenceAsListMngCursorImpl
// ----------------------------

ISequenceAsListNodeImpl <long> const*const
ISequenceAsListMngCursorImpl::kIsInBetween =
  (ISequenceAsListNodeImpl <long>*)&
   ISequenceAsListMngCursorImpl::kIsInBetween;

// public members

ISequenceAsListMngCursorImpl::
ISequenceAsListMngCursorImpl (ISequenceAsListImpl const& collection)
: ISequenceAsListCursorImpl (collection),
  ivMngCollection ((ISequenceAsListImpl*)&collection)
{ ivNext = ivMngCollection->ivMngCursors;
  ivMngCollection->ivMngCursors = this;
}

ISequenceAsListMngCursorImpl::
~ISequenceAsListMngCursorImpl ()
{ Self*& first = ivMngCollection->ivMngCursors;
  if (this == first)
    first = ivNext;
  else {
    // find previous cursor
    for (Self* cursor = first;
         cursor->ivNext != this;
         cursor = cursor->ivNext);

    cursor->ivNext = ivNext;
  }
}

ICursorImpl*
ISequenceAsListMngCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsListMngCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}

bool
ISequenceAsListMngCursorImpl::
IsInBetween () const
{ return ivMngCollection->IsInBetween (*this);
}

bool
ISequenceAsListMngCursorImpl::
IsMngCursor () const
{ return true;
}

void
ISequenceAsListMngCursorImpl::
operator= (ISequenceAsListMngCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivNextNode     = cursor.ivNextNode;
  ivPreviousNode = cursor.ivPreviousNode;
  if (ivMngCollection != cursor.ivMngCollection) {

    // detach 'this' from current list of mngcursors
    Self*& first = ivMngCollection->ivMngCursors;
    if (this == first)
      first = ivNext;
    else {
      // find previous cursor
      for (Self* cursor = first;
           cursor->ivNext != this;
           cursor = cursor->ivNext);

      cursor->ivNext = ivNext;
    }

    // insert 'this' in new list of mngcursors
    ivMngCollection = cursor.ivMngCollection;
    ivNext = (cursor.ivMngCollection)->ivMngCursors;
    ivMngCollection->ivMngCursors = this;
  }
}
#endif

// --------------------
// ISequenceAsListImpl
// --------------------

char const*
ISequenceAsListImpl::cvClassName = "SequenceAsList";

// inline private members

inline ISequenceAsListImpl::Ops&
ISequenceAsListImpl::
OpsOf (ISequenceAsListImpl const& collection)
{ return (Ops&) Inherited::OpsOf (collection);
}

inline void*
ISequenceAsListImpl::
ElementOf (Node const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ? (void*) node->ivElement :
                                               (void*)&node->ivElement);
}

inline bool
ISequenceAsListImpl::
IsInBetween (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return (((Cursor const&)cursor).ivNode == MngCursor::kIsInBetween);
#else
  return false;
#endif
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
NextNodeOf (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return ((MngCursor&)cursor).ivNextNode;
#else
  return ((Cursor&)cursor).ivNode; // just a dummy
#endif
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
NodeOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivNode;
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
PreviousNodeOf (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return ((MngCursor&)cursor).ivPreviousNode;
#else
  return ((Cursor&)cursor).ivNode; // just a dummy
#endif
}

// inline protected members

inline void
ISequenceAsListImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void*
ISequenceAsListImpl::
CreateNode (void const* element) const
{ return OpsOf (*this).CreateNode (element);
}

#if ! defined (__INO_STREAMING__)
inline void*
ISequenceAsListImpl::
CreateNodeWithStreamer (IDataStream& stream) const
{ return OpsOf (*this).CreateNodeWithStreamer (ivStreamer, stream);
}
#endif

inline void
ISequenceAsListImpl::
DeleteNode (void* node) const
{ OpsOf (*this).DeleteNode (node);
}

// inline public members

#define inline

inline void*
ISequenceAsListImpl::
Any () const
{ return ElementOf (ivFirst);
}

inline char const*
ISequenceAsListImpl::
ClassName () const
{ return cvClassName;
}

inline void*
ISequenceAsListImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (NodeOf (cursor));
}

inline void*
ISequenceAsListImpl::
First () const
{ return ElementOf (ivFirst);
}

inline bool
ISequenceAsListImpl::
IsBounded () const
{ return false;
}

inline bool
ISequenceAsListImpl::
IsEmpty () const
{ return (ivNumberOfElements == 0);
}

inline bool
ISequenceAsListImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ return (NodeOf (cursor) == ivFirst);
}

inline bool
ISequenceAsListImpl::
IsFull () const
{ return false;
}

inline bool
ISequenceAsListImpl::
IsLastAt (ICursorImpl const& cursor) const
{ return (NodeOf (cursor) == ivLast);
}

inline void*
ISequenceAsListImpl::
Last () const
{ return ElementOf (ivLast);
}

inline INumber
ISequenceAsListImpl::
NumberOfElements () const
{ return ivNumberOfElements;
}

inline void
ISequenceAsListImpl::
RemoveFirst ()
{ RemoveNode (ivFirst);
}

inline void
ISequenceAsListImpl::
RemoveLast ()
{ RemoveNode (ivLast);
}

inline void
ISequenceAsListImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

inline bool
ISequenceAsListImpl::
SetToFirst (ICursorImpl& cursor) const
{ NodeOf (cursor) = ivFirst;
  return (NodeOf (cursor) != 0);
}

inline bool
ISequenceAsListImpl::
SetToLast (ICursorImpl& cursor) const
{ NodeOf (cursor) = ivLast;
  return (NodeOf (cursor) != 0);
}

inline bool
ISequenceAsListImpl::
SetToNext (ICursorImpl& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor))
    NodeOf (cursor) = NextNodeOf (cursor);
  else
#endif
    NodeOf (cursor) = NodeOf (cursor)->ivNext;
  return (NodeOf (cursor) != 0);
}

inline void
ISequenceAsListImpl::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ SetNodeToPosition (position, NodeOf (cursor));
}

inline bool
ISequenceAsListImpl::
SetToPrevious (ICursorImpl& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor))
    NodeOf (cursor) = PreviousNodeOf (cursor);
  else
#endif
    NodeOf (cursor) = NodeOf (cursor)->ivPrevious;
  return (NodeOf (cursor) != 0);
}

#undef inline

// public members

ISequenceAsListImpl::
ISequenceAsListImpl (Ops& ops, INumber)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivFirst (0), ivLast (0), ivNumberOfElements (0)
{
}

ISequenceAsListImpl::
ISequenceAsListImpl (Ops& ops, ISequenceAsListImpl const& collection)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivFirst (0), ivLast (0), ivNumberOfElements (0)
{ Self::AddAllFrom (collection);
#if ! defined (__INO_STREAMING__)
  ivStreamer = collection.ivStreamer;
#endif
}

ISequenceAsListImpl::
~ISequenceAsListImpl ()
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    cursor->Invalidate();
  }
#endif
  Self::RemoveAll ();
}

bool
ISequenceAsListImpl::
Add (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (ivLast, NodeOf (cursor));
  return true;
}

void
ISequenceAsListImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& seq = (Self const&)collection;
    for (Node* node = seq.ivFirst; node != 0; node = node->ivNext)
      AddAsNextNode (ivLast, (Node*) CreateNode (ElementOf (node)));
  }
  else
    Inherited::AddAllFrom (collection);
}

void
ISequenceAsListImpl::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (0, NodeOf (cursor));
}

void
ISequenceAsListImpl::
AddAsLast (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (ivLast, NodeOf (cursor));
}

void
ISequenceAsListImpl::
AddAsNext (void const* element, ICursorImpl& cursor)
{ Node* node = (Node*) CreateNode (element);
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
    NodeOf(cursor) = (NextNodeOf (cursor) ?
                      NextNodeOf (cursor)->ivPrevious : ivLast);
  }
#endif
  AddAsNextNode (NodeOf (cursor), node);
  NodeOf (cursor) = node;
}

void
ISequenceAsListImpl::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{ Node* node = (Node*) CreateNode (element);
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
    AddAsNextNode (PreviousNodeOf (cursor), node);
  }
  else
#endif
  {
    AddAsNextNode (NodeOf (cursor)->ivPrevious, node);
  }
  NodeOf (cursor) = node;
}

void
ISequenceAsListImpl::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ Node* node;
  if (position == 1)
    node = 0;
  else {
    node = ivFirst;
    for (int i = 2; i < position; i++)
      node = node->ivNext;
  }
  NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (node, NodeOf (cursor));
}

bool
ISequenceAsListImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ for (Node* node = ivFirst; node != 0; node = node->ivNext) {
    if (! (*applFunc) (ElementOf (node), addArg))
      return false;
  }
  return true;
}

bool
ISequenceAsListImpl::
CheckCursor (ICursorImpl const& cursor) const
{ bool result = false;
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor))
    result = true;
  else
#endif
  {
    for (Node* node = ivFirst; node != 0; node = node->ivNext) {
      if (NodeOf (cursor) == node)
        result = true;
    }
  }
  return result;
}

IACollectionImpl*
ISequenceAsListImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
ISequenceAsListImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

ISequenceAsListImpl::Cursor*
ISequenceAsListImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

ISequenceAsListImpl::MngCursor*
ISequenceAsListImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

void*
ISequenceAsListImpl::
ElementAtPosition (IPosition position) const
{ Node* node = 0;
  SetNodeToPosition (position, node);
  return ElementOf (node);
}

bool
ISequenceAsListImpl::
IsConsistent () const
{ INumber numberOfElements = 0;
  if (ivFirst != 0 && ivFirst->ivPrevious != 0)
    return false;
  for (Node* node = ivFirst; node != 0; node = node->ivNext) {
    if (node->ivNext != 0&& node->ivNext->ivPrevious != node)
      return false;
    numberOfElements++;
  }
  return (numberOfElements == ivNumberOfElements);
}

IPosition
ISequenceAsListImpl::
PositionAt (ICursorImpl const& cursor) const
{ IPosition position = 1;
  for (Node* node = ivFirst;
       node != NodeOf (cursor);
       node = node->ivNext, position++);
  return position;
}

INumber
ISequenceAsListImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  for (Node *current = ivFirst, *next; current != 0; current = next) {
    next = current->ivNext;
    DeleteNode (current);
  }

#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (cursor->IsValid ()) {
      NodeOf (*cursor)         = (Node*) MngCursor::kIsInBetween;
      NextNodeOf (*cursor)     = 0;
      PreviousNodeOf (*cursor) = 0;
    }
  }
#endif

  ivFirst = ivLast = 0;
  ivNumberOfElements = 0;
  return n;
}

INumber
ISequenceAsListImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber removed = 0;

  for (Node* current = ivFirst, * next = ivFirst;
       current != 0;
       current = next) {
    next = next->ivNext;
    if ((*predFunc) (ElementOf (current), addArg)) {
      RemoveNode (current);
      removed++;
    }
  }

  return removed;
}

void
ISequenceAsListImpl::
RemoveAt (ICursorImpl& cursor)
{ RemoveNode (NodeOf (cursor));
  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

void
ISequenceAsListImpl::
RemoveAtPosition (IPosition position)
{ Node* node;
  SetNodeToPosition (position, node);
  RemoveNode (node);
}

void
ISequenceAsListImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ SetToPosition (position, CursorOf (*this));
  Self::ReplaceAt (CursorOf (*this), element);
}

void
ISequenceAsListImpl::
Reverse ()
{ Node* next;
  for (Node* current = ivFirst; current != 0; current = next) {
    next = current->ivNext;
    current->ivNext = current->ivPrevious;
    current->ivPrevious = next;
  }

#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor))
      cursor->Invalidate ();
  }
#endif

  next = ivLast;
  ivLast = ivFirst;
  ivFirst = next;
}

void
ISequenceAsListImpl::
Sort (ICompFunc compFunc, void* addArg)
{ for (Node* last = ivLast; last != ivFirst; last = last->ivPrevious) {
    for (Node* current = ivFirst; current != last; ) {
      Node* next = current->ivNext;
      if (0 < (*compFunc) (ElementOf (current),
                           ElementOf (current->ivNext), addArg)) {
        current->ivNext = next->ivNext;
        next->ivPrevious = current->ivPrevious;
        if (current->ivNext != 0)
          current->ivNext->ivPrevious = current;
        if (next->ivPrevious != 0)
          next->ivPrevious->ivNext = next;
        current->ivPrevious = next;
        next->ivNext = current;
        if (next == last) {
          last = current;
          if (next == ivLast)
            ivLast = current;
        }
        if (current == ivFirst) {
          ivFirst = next;
        }
      }
      else {
        current = next;
      }
    }
  }
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor))
      cursor->Invalidate ();
  }
#endif
}

void
ISequenceAsListImpl::
operator= (ISequenceAsListImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    Self::AddAllFrom (collection);
#if ! defined (__INO_STREAMING__)
    ivStreamer = collection.ivStreamer;
#endif
  }
}

#if ! defined (__INO_STREAMING__)
void
ISequenceAsListImpl::
StreamIn (IDataStream& fromWhere)
{ ReadVersion (fromWhere);
  Self::RemoveAll ();
  INumber n;
  n <<= fromWhere;
  while (n--) {
    NodeOf (CursorOf (*this)) = (Node*) CreateNodeWithStreamer (fromWhere);
    AddAsNextNode (ivLast, NodeOf (CursorOf (*this)));
  }
}

void
ISequenceAsListImpl::
StreamOut (IDataStream& toWhere) const
{ WriteVersion (toWhere);
  Self::NumberOfElements () >>= toWhere;
  for (Self::SetToFirst (CursorOf (*this));
       CursorOf (*this).IsValid ();
       Self::SetToNext (CursorOf (*this))) {
    ivStreamer->streamOut (toWhere, Self::ElementAt (CursorOf (*this)));
  }
}

void
ISequenceAsListImpl::
AdoptStreamer (IACollectionStreamer *streamer)
{ if (streamer != ivStreamer.PtrOf())
    ivStreamer = IStreamerPtr(streamer);
}

IStreamerPtr&
ISequenceAsListImpl::
StreamerOf (IACollectionImpl const& collection)
{ return ivStreamer;
}
#endif

// private members

void
ISequenceAsListImpl::
AddAsNextNode (Node* node, Node* newNode)
{ if (node == 0)
    newNode->ivNext = ivFirst;
  else
    newNode->ivNext = node->ivNext;

  newNode->ivPrevious = node;

  if (newNode->ivNext != 0)
    newNode->ivNext->ivPrevious = newNode;

  if (newNode->ivPrevious != 0)
    newNode->ivPrevious->ivNext = newNode;

  if (node == 0)
    ivFirst = newNode;

  if (node == ivLast)
    ivLast = newNode;

  ivNumberOfElements++;
}

void
ISequenceAsListImpl::
RemoveNode (Node* node)
{ if (node == ivFirst)
    ivFirst = node->ivNext;
  if (node == ivLast)
    ivLast = node->ivPrevious;

  if (node->ivNext != 0)
    node->ivNext->ivPrevious = node->ivPrevious;
  if (node->ivPrevious != 0)
    node->ivPrevious->ivNext = node->ivNext;

#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousNodeOf (*cursor) == node)
        PreviousNodeOf (*cursor) = PreviousNodeOf (*cursor)->ivPrevious;
      if (NextNodeOf (*cursor) == node)
        NextNodeOf (*cursor) = NextNodeOf (*cursor)->ivNext;
    }
    else  {
      if (NodeOf (*cursor) == node)
        SetInBetween (*cursor);
    }
  }
#endif

  DeleteNode (node);

  ivNumberOfElements--;
}

void
ISequenceAsListImpl::
SetInBetween (MngCursor& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (! IsInBetween (cursor)) {
    PreviousNodeOf (cursor) = NodeOf (cursor)->ivPrevious;
    NextNodeOf (cursor)     = NodeOf (cursor)->ivNext;
    NodeOf (cursor)         = (Node*) MngCursor::kIsInBetween;
  }
#endif
}

void
ISequenceAsListImpl::
SetNodeToPosition (IPosition position, Node*& node) const
{ node = ivFirst;
  for (int i = 1; i < position; i++)
    node = node->ivNext;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
