// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 09 1.5.1.3 source/core/collect/iiseqtab.c, collection, ioc.v400, 000302 
#ifndef _IISEQTAB_C_
#define _IISEQTAB_C_

#include <new.h>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// -------------------
// ISequenceAsTableOps
// -------------------

// public members

template <class Element, class ElementOps, class Implementation>
ISequenceAsTableOps <Element, ElementOps, Implementation>::
ISequenceAsTableOps (INumber numberOfElements)
: Inherited (__isDTSClass (Element)),
  ivElementOps (),
  ivImpl (*this, numberOfElements)
{
}

template <class Element, class ElementOps, class Implementation>
ISequenceAsTableOps <Element, ElementOps, Implementation>::
ISequenceAsTableOps (INumber numberOfElements, void* opsArg)
: Inherited (__isDTSClass (Element)),
  ivElementOps (opsArg),
  ivImpl (*this, numberOfElements)
{
}

template <class Element, class ElementOps, class Implementation>
ISequenceAsTableOps <Element, ElementOps, Implementation>::
ISequenceAsTableOps
  (ISequenceAsTableOps <Element, ElementOps,
                        Implementation> const& collection)
: Inherited (__isDTSClass (Element)),
  ivElementOps (collection.ivElementOps),
  ivImpl (*this, collection.ivImpl)
{
}

template <class Element, class ElementOps, class Implementation>
ISequenceAsTableOps <Element, ElementOps, Implementation>::
~ISequenceAsTableOps ()
{
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
Assign (void* element1, void const* element2) const
{ ivElementOps.Assign (*(Element*)element1,
                       *(Element const*)element2);
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
BlockLeft (INumber target, INumber count)
{ INumber i = target;
  new (TableOf (ivImpl).NodeAtIndex (i))
    Node (((Node const*) TableOf (ivImpl).
                           NodeAtIndex (i + 1))->ivElement);
#if ! defined (__INO_MNGCURSOR__)
  UpdateMngCursorsForRemoveOf (ivImpl, i);
  UpdateMngCursorsForShiftLeftOf (ivImpl, i + 1);
#endif
  for (i++, count--; count > 0; i++, count--) {
     ivElementOps.Assign
       (((Node*) TableOf (ivImpl).NodeAtIndex (i))->ivElement,
        ((Node const*) TableOf (ivImpl).
                         NodeAtIndex (i + 1))->ivElement);
#if ! defined (__INO_MNGCURSOR__)
     UpdateMngCursorsForShiftLeftOf (ivImpl, i + 1);
#endif
  }
  ((Node*) TableOf (ivImpl).NodeAtIndex (i))->~Node ();
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
BlockRight (INumber target, INumber count)
{ INumber i = target + count - 1;
  new (TableOf (ivImpl).NodeAtIndex (i))
    Node (((Node const*) TableOf (ivImpl).
                           NodeAtIndex (i - 1))->ivElement);
#if ! defined (__INO_MNGCURSOR__)
  UpdateMngCursorsForRemoveOf (ivImpl, i);
  UpdateMngCursorsForShiftRightOf (ivImpl, i - 1);
#endif
  for (i--, count--; count > 0; i--, count--) {
     ivElementOps.Assign
       (((Node*) TableOf (ivImpl).NodeAtIndex (i))->ivElement,
        ((Node const*) TableOf (ivImpl).
                         NodeAtIndex (i - 1))->ivElement);
#if ! defined (__INO_MNGCURSOR__)
     UpdateMngCursorsForShiftRightOf (ivImpl, i - 1);
#endif
  }
  ((Node*) TableOf (ivImpl).NodeAtIndex (i))->~Node ();
}

template <class Element, class ElementOps, class Implementation>
IACollectionImpl*
ISequenceAsTableOps <Element, ElementOps, Implementation>::
Clone () const
{ return (Implementation*) *new Self (*this);
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
CopyFrom (ITableImpl const& from)
{ INumber toIndex = TableOf (ivImpl).ivLast;
  INumber fromIndex = from.ivFirst;
  for (INumber i = 0; i < from.ivNumberOfElements; i++) {
    TableOf (ivImpl).IncMod (toIndex);
    new (TableOf (ivImpl).NodeAtIndex (toIndex))
      Node (((Node const*) from.NodeAtIndex (fromIndex))->ivElement);
    from.IncMod (fromIndex);
  }
  TableOf (ivImpl).ivNumberOfElements += from.ivNumberOfElements;
  TableOf (ivImpl).ivLast = toIndex;

}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
ConstructNode (void* node, void const* element) const
{ new (node) Node (*(Element const*)element);
}

#if ! defined (__INO_STREAMING__)
template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
ConstructNodeWithStreamer (void* node, IACollectionStreamer *streamer,
                           IDataStream& fromWhere) const
{ ((Node*)node)->StreamIn (streamer, fromWhere);
}
#endif

template <class Element, class ElementOps, class Implementation>
void*
ISequenceAsTableOps <Element, ElementOps, Implementation>::
CreateNodes (INumber n, size_t& size)
{ size = sizeof (Node);
  void* node = ivElementOps.Allocate ((size_t) (size * n)
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__) || defined (_AIX)
#if defined (__DEBUG_ALLOC__)
                                      , __FILE__, __LINE__
#endif
#endif
#if defined (__OSTORE__)
                                      , Node::get_os_typespec (), n
#endif
                                     );
  return IACollectionImpl::CheckPointer (node);
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
DeleteNodes (void* nodes)
{ ivElementOps.Deallocate (nodes
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__) || defined (_AIX)
#if defined (__DEBUG_ALLOC__)
                           , __FILE__, __LINE__
#endif
#endif
                          );
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
DestroyNodes (void* node, INumber count) const
{ while (count--)
    ((Node*)node) [count].~Node ();
}

template <class Element, class ElementOps, class Implementation>
void
ISequenceAsTableOps <Element, ElementOps, Implementation>::
Swap (INumber index1, INumber index2) const
{ Element e =
    ((Node*) TableOf (ivImpl).NodeAtIndex (index1))->ivElement;
  ivElementOps.Assign
    (((Node*) TableOf (ivImpl).NodeAtIndex (index1))->ivElement,
     ((Node const*) TableOf (ivImpl).NodeAtIndex (index2))->ivElement);
  ivElementOps.Assign
    (((Node*) TableOf (ivImpl).NodeAtIndex (index2))->ivElement, e);
#if ! defined (__INO_MNGCURSOR__)
  UpdateMngCursorsForSwap (ivImpl, index1, index2);
#endif
}

template <class Element, class ElementOps, class Implementation>
ISequenceAsTableOps <Element, ElementOps, Implementation>::
operator Implementation* ()
{ ivImpl.CheckPointer (this);
  return &ivImpl;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IISEQTAB_C_
