// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#include <iiseqtab.h>

#ifdef IC_PAGETUNE
  #define _IISEQTAB_CPP_
  #include <ipagetun.h>
#endif


#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IASequenceAsTableOps)
#endif

// --------------------------
// ISequenceAsTableCursorImpl
// --------------------------

// public members

ISequenceAsTableCursorImpl::
ISequenceAsTableCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivIndex (0)
{
}

ISequenceAsTableCursorImpl::
ISequenceAsTableCursorImpl
  (ISequenceAsTableCursorImpl const& cursor)
: ICursorImpl (cursor), ivIndex (cursor.ivIndex)
{
}

ISequenceAsTableCursorImpl::
~ISequenceAsTableCursorImpl ()
{
}

ICursorImpl*
ISequenceAsTableCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsTableCursorImpl::
Copy (ICursorImpl const&  cursor)
{ operator= ((Self const&)cursor);
}

#if ! defined (__INO_MNGCURSOR__)
// -----------------------------
// ISequenceAsTableMngCursorImpl
// -----------------------------

INumber const
ISequenceAsTableMngCursorImpl::kIsInBetween = (INumber) -1;

// public members

ISequenceAsTableMngCursorImpl::
ISequenceAsTableMngCursorImpl (ISequenceAsTableImpl const& collection)
: ISequenceAsTableCursorImpl (collection),
  ivMngCollection ((ISequenceAsTableImpl*)&collection)
{ ivNext = ivMngCollection->ivMngCursors;
  ivMngCollection->ivMngCursors = (Self*)this;
}

ISequenceAsTableMngCursorImpl::
~ISequenceAsTableMngCursorImpl ()
{ Self*& first = ivMngCollection->ivMngCursors;
  if (this == first) {
     first = ivNext;
  } else {
     // find previous cursor
     for (Self* cursor = first;
          cursor->ivNext != this;
          cursor = cursor->ivNext);

     cursor->ivNext = ivNext;
  }
}

ICursorImpl*
ISequenceAsTableMngCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsTableMngCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}

bool
ISequenceAsTableMngCursorImpl::
IsInBetween () const
{ return ivMngCollection->IsInBetween (*this);
}

bool
ISequenceAsTableMngCursorImpl::
IsMngCursor () const
{ return true;
}

void
ISequenceAsTableMngCursorImpl::
operator= (ISequenceAsTableMngCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivNextIndex     = cursor.ivNextIndex;
  ivPreviousIndex = cursor.ivPreviousIndex;
  if (ivMngCollection != cursor.ivMngCollection) {

    // detach 'this' from current list of mngcursors
    Self*& first = ivMngCollection->ivMngCursors;
    if (this == first)
      first = ivNext;
    else {
      // find previous cursor
      for (Self* cursor = first;
           cursor->ivNext != this;
           cursor = cursor->ivNext);

      cursor->ivNext = ivNext;
    }

    // insert 'this' in new list of mngcursors
    ivMngCollection = cursor.ivMngCollection;
    ivNext = (cursor.ivMngCollection)->ivMngCursors;
    ivMngCollection->ivMngCursors = this;
  }
}
#endif

// --------------------
// ISequenceAsTableImpl
// --------------------

char const*
ISequenceAsTableImpl::cvClassName = "SequenceAsTable";

// inline private members

inline ISequenceAsTableImpl::Ops&
ISequenceAsTableImpl::
OpsOf (ISequenceAsTableImpl const& collection)
{ return (Ops&) IACollectionImpl::OpsOf (collection);
}

inline void*
ISequenceAsTableImpl::
ElementOf (void const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ?
    (void*)*(void const**)node :
    (void*) (void const**)node);
};

inline INumber&
ISequenceAsTableImpl::
IndexOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivIndex;
}

inline bool
ISequenceAsTableImpl::
IsInBetween (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return (((Cursor const&)cursor).ivIndex == MngCursor::kIsInBetween);
#else
  return false;
#endif
}

inline INumber&
ISequenceAsTableImpl::
NextIndexOf (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return ((MngCursor&)cursor).ivNextIndex;
#else
  return ((Cursor&)cursor).ivIndex; // just a dummy
#endif
}

inline INumber&
ISequenceAsTableImpl::
PreviousIndexOf (ICursorImpl const& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  return ((MngCursor&)cursor).ivPreviousIndex;
#else
  return ((Cursor&)cursor).ivIndex; // just a dummy
#endif
}

// inline protected members

inline void
ISequenceAsTableImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void
ISequenceAsTableImpl::
BlockLeft (INumber target, INumber count)
{ OpsOf (*this).BlockLeft (target, count);
}

inline void
ISequenceAsTableImpl::
BlockRight (INumber target, INumber count)
{ OpsOf (*this).BlockRight (target, count);
}

inline void
ISequenceAsTableImpl::
CopyFrom (ITableImpl const& from)
{ OpsOf (*this).CopyFrom (from);
}

inline void
ISequenceAsTableImpl::
ConstructNode (void* node, void const* element) const
{ OpsOf (*this).ConstructNode (node, element);
}

#if ! defined (__INO_STREAMING__)
inline void
ISequenceAsTableImpl::
ConstructNodeWithStreamer (void* node, IDataStream& fromWhere) const
{ OpsOf (*this).ConstructNodeWithStreamer (node, ivStreamer, fromWhere);
}
#endif

inline void*
ISequenceAsTableImpl::
CreateNodes (INumber n, size_t& size)
{ return OpsOf (*this).CreateNodes (n, size);
}

inline void
ISequenceAsTableImpl::
DeleteNodes (void* nodes)
{ OpsOf (*this).DeleteNodes (nodes);
}

inline void
ISequenceAsTableImpl::
DestroyNodes (void* node, INumber count) const
{ OpsOf (*this).DestroyNodes (node, count);
}

inline void
ISequenceAsTableImpl::
Swap (INumber index1, INumber index2) const
{ OpsOf (*this).Swap (index1, index2);
}

// inline public members

#define inline

inline bool
ISequenceAsTableImpl::
Add (void const* element, ICursorImpl& cursor)
{ Self::AddAsLast (element, (Cursor&)cursor);
  return true;
}

inline void*
ISequenceAsTableImpl::
Any () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivFirst));
}

inline char const*
ISequenceAsTableImpl::
ClassName () const
{ return cvClassName;
}

inline void*
ISequenceAsTableImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (ivTable.NodeAtIndex (IndexOf (cursor)));
}

inline void*
ISequenceAsTableImpl::
ElementAtPosition (IPosition position) const
{ Self::SetToPosition (position, CursorOf (*this));
  return Self::ElementAt (CursorOf (*this));
}

inline void*
ISequenceAsTableImpl::
First () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivFirst));
}

inline bool
ISequenceAsTableImpl::
IsBounded () const
{ return false;
}

inline bool
ISequenceAsTableImpl::
IsEmpty () const
{ return (ivTable.ivNumberOfElements == 0);
}

inline bool
ISequenceAsTableImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivFirst);
}

inline bool
ISequenceAsTableImpl::
IsFull () const
{ return false;
}

inline bool
ISequenceAsTableImpl::
IsLastAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivLast);
}

inline void*
ISequenceAsTableImpl::
Last () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivLast));
}

inline INumber
ISequenceAsTableImpl::
NumberOfElements () const
{ return ivTable.ivNumberOfElements;
}

inline IPosition
ISequenceAsTableImpl::
PositionAt (ICursorImpl const& cursor) const
{ return ivTable.Pos (IndexOf (cursor));
}

inline void
ISequenceAsTableImpl::
RemoveAt (ICursorImpl& cursor)
{ RemoveAtIndex (IndexOf (cursor));
  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

inline void
ISequenceAsTableImpl::
RemoveFirst ()
{ RemoveAtIndex (ivTable.ivFirst);
}

inline void
ISequenceAsTableImpl::
RemoveLast ()
{ RemoveAtIndex (ivTable.ivLast);
}

inline void
ISequenceAsTableImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

inline void
ISequenceAsTableImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ Self::SetToPosition (position, CursorOf (*this));
  Self::ReplaceAt (CursorOf (*this), element);
}

inline bool
ISequenceAsTableImpl::
SetToFirst (ICursorImpl& cursor) const
{ return Self::IsEmpty () ? (IndexOf (cursor) = 0) != 0 :
                            (IndexOf (cursor) = ivTable.ivFirst) != 0;
}

inline bool
ISequenceAsTableImpl::
SetToLast (ICursorImpl& cursor) const
{ return Self::IsEmpty () ? (IndexOf (cursor) = 0) != 0 :
                            (IndexOf (cursor) = ivTable.ivLast) != 0;
}

inline bool
ISequenceAsTableImpl::
SetToNext (ICursorImpl& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
     IndexOf (cursor) = NextIndexOf (cursor);
     return (IndexOf (cursor) != 0);
  }
  else
#endif
    return Self::IsLastAt (cursor) ?
      (IndexOf (cursor) = 0) != 0 :
      ivTable.IncMod (IndexOf (cursor)) != 0;
}

inline void
ISequenceAsTableImpl::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ IndexOf (cursor) = ivTable.ivFirst;
  ivTable.AddMod (IndexOf (cursor), position - 1);
}

inline bool
ISequenceAsTableImpl::
SetToPrevious (ICursorImpl& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
     IndexOf (cursor) = PreviousIndexOf (cursor);
     return (IndexOf (cursor) != 0);
  }
  else
#endif
    return Self::IsFirstAt (cursor) ?
      (IndexOf (cursor) = 0) != 0 :
      ivTable.DecMod (IndexOf (cursor)) != 0;
}

inline void
ISequenceAsTableImpl::
Sort (ICompFunc compFunc, void* addArg)
{ if (Self::NumberOfElements () > 1)
    Quicksort (compFunc, addArg, ivTable.ivFirst, ivTable.ivLast);
}

#undef inline

// public members

ISequenceAsTableImpl::
ISequenceAsTableImpl (Ops& ops, INumber numberOfElements)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ if (numberOfElements == 0)
    numberOfElements++;
  Initialize (numberOfElements);
}

ISequenceAsTableImpl::
ISequenceAsTableImpl
  (Ops& ops, ISequenceAsTableImpl const& collection)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ Initialize (collection.ivTable.ivAllocatedElements);
  Self::AddAllFrom (collection);
#if ! defined (__INO_STREAMING__)
  ivStreamer = collection.ivStreamer;
#endif
}

ISequenceAsTableImpl::
~ISequenceAsTableImpl ()
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    cursor->Invalidate();
  }
#endif
  Self::RemoveAll ();
  DeleteNodes (ivTable.ivNodes);
}

void
ISequenceAsTableImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& seq = (Self const&)collection;
    if (! ivTable.CanAdd (seq.ivTable.ivNumberOfElements))
      GrowBy (seq.ivTable.ivNumberOfElements);

    CopyFrom (seq.ivTable);
  }
  else
    Inherited::AddAllFrom (collection);
}

void
ISequenceAsTableImpl::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  ivTable.DecMod (ivTable.ivFirst);

  ConstructNode (ivTable.NodeAtIndex (ivTable.ivFirst), element);
  ivTable.ivNumberOfElements++;
  IndexOf (cursor) = ivTable.ivFirst;
}

void
ISequenceAsTableImpl::
AddAsLast (void const* element, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  ivTable.IncMod (ivTable.ivLast);

  ConstructNode (ivTable.NodeAtIndex (ivTable.ivLast), element);
  ivTable.ivNumberOfElements++;
  IndexOf (cursor) = ivTable.ivLast;
}

#if ! defined (__INO_STREAMING__)
void
ISequenceAsTableImpl::
AddAsLastFromStreamer (IDataStream& fromWhere, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  ivTable.IncMod (ivTable.ivLast);

  ConstructNodeWithStreamer (ivTable.NodeAtIndex (ivTable.ivLast), fromWhere);
  ivTable.ivNumberOfElements++;
  IndexOf (cursor) = ivTable.ivLast;
}
#endif

void
ISequenceAsTableImpl::
AddAsNext (void const* element, ICursorImpl& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
    if (IndexOf (cursor) = NextIndexOf (cursor))
      Self::AddAsPrevious (element, cursor);
    else
      Self::AddAsLast (element, cursor);
  }
  else
#endif
  { if (IndexOf (cursor) == ivTable.ivLast)
      Self::AddAsLast (element, cursor);
    else {
      Self::SetToNext (cursor);
      Self::AddAsPrevious (element, cursor);
    }
  }
}

void
ISequenceAsTableImpl::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor)) {
    if ((IndexOf(cursor) = PreviousIndexOf (cursor)) == 0)
      Self::SetToFirst (cursor);
    else if (! Self::SetToNext (cursor)) {
      Self::AddAsLast (element, cursor);
      return;
    }
  }
#endif

  INumber index = IndexOf (cursor); // no reference 'cause of mng cursor

  if (! ivTable.CanAdd (1)) {
    index = ivTable.Pos (index);
    GrowBy (1);
  }

  if (ivTable.ivFirst == index) {
    ivTable.DecMod (ivTable.ivFirst);
    index = ivTable.ivFirst;
  }
  else if (ivTable.ivFirst <= ivTable.ivLast &&
           (ivTable.ivFirst == 1 ||
            (ivTable.ivLast < ivTable.ivAllocatedElements &&
             index - ivTable.ivFirst > ivTable.ivLast - index))) {
    BlockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else if (ivTable.ivFirst > ivTable.ivLast &&
           index <= ivTable.ivLast) {
    BlockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else {
    BlockLeft (ivTable.ivFirst - 1, index - ivTable.ivFirst);
    ivTable.ivFirst--;
    index--;
  }

  ConstructNode (ivTable.NodeAtIndex (index), element);
  IndexOf (cursor) = index;
  ivTable.ivNumberOfElements++;
}

void
ISequenceAsTableImpl::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ if (position == Self::NumberOfElements () + 1)
    Self::AddAsLast (element, cursor);
  else {
    Self::SetToPosition (position, cursor);
    Self::AddAsPrevious (element, cursor);
  }
}

bool
ISequenceAsTableImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < Self::NumberOfElements (); i++) {
    if (! (*applFunc) (ElementOf (ivTable.NodeAtIndex (index)), addArg))
      return false;
    ivTable.IncMod (index);
  }
  return true;
}

bool
ISequenceAsTableImpl::
CheckCursor (ICursorImpl const& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (IsInBetween (cursor))
    return true;
  else
#endif
  {
    INumber const& index = ((Cursor const&)cursor).ivIndex;
    if (ivTable.ivNumberOfElements == 0)
      return false;
    else if (ivTable.ivFirst <= ivTable.ivLast)
      return ivTable.ivFirst <= index && index <= ivTable.ivLast;
    else
      return (ivTable.ivFirst <= index &&
                index <= ivTable.ivAllocatedElements) ||
              index <= ivTable.ivLast;
  }
}

IACollectionImpl*
ISequenceAsTableImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
ISequenceAsTableImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

ISequenceAsTableImpl::Cursor*
ISequenceAsTableImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

ISequenceAsTableImpl::MngCursor*
ISequenceAsTableImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

bool
ISequenceAsTableImpl::
IsConsistent () const
{ INumber f = ivTable.ivFirst,
          l = ivTable.ivLast,
          n = ivTable.ivNumberOfElements,
          a = ivTable.ivAllocatedElements;
  return n % a == (l >= f ? l - f + 1 : l + a - f + 1) % a;
}

INumber
ISequenceAsTableImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  if (n != 0) {
    if (ivTable.ivFirst <= ivTable.ivLast) {
      DestroyNodes (ivTable.NodeAtIndex (ivTable.ivFirst),
                    ivTable.ivNumberOfElements);
    }
    else {
      DestroyNodes (ivTable.NodeAtIndex (ivTable.ivFirst),
                    ivTable.ivAllocatedElements - ivTable.ivFirst + 1);
      DestroyNodes (ivTable.ivNodes, ivTable.ivLast);
    }

#if ! defined (__INO_MNGCURSOR__)
    forMngCursors (cursor) {
      if (cursor->IsValid ()) {
        IndexOf (*cursor)         = MngCursor::kIsInBetween;
        NextIndexOf (*cursor)     = 0;
        PreviousIndexOf (*cursor) = 0;
      }
    }
#endif

    ivTable.ivFirst = 1;
    ivTable.ivLast = ivTable.ivAllocatedElements;
    ivTable.ivNumberOfElements = 0;
  }
  return n;
}


INumber
ISequenceAsTableImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber index = ivTable.ivFirst,
          current = ivTable.ivFirst;
  INumber removed = 0;
#if ! defined (__INO_MNGCURSOR__)
  INumber previous = 0;
#endif

  for (INumber i = Self::NumberOfElements (); i != 0; i--) {
    if ((*predFunc) (ElementOf (ivTable.NodeAtIndex (current)),
                     addArg)) {
#if ! defined (__INO_MNGCURSOR__)
      UpdateMngCursorsForPredRemove (current, previous);
#endif
      removed++;
    }
    else {
      if (index != current)  {
        Assign (ElementOf (ivTable.NodeAtIndex (index)),
                ElementOf (ivTable.NodeAtIndex (current)));
#if ! defined (__INO_MNGCURSOR__)
        UpdateMngCursorsForAssign (index, current);
#endif
      }
      ivTable.IncMod (index);
#if ! defined (__INO_MNGCURSOR__)
      previous =
        (previous != 0) ? ivTable.IncMod (previous) : ivTable.ivFirst;
#endif
    }
    ivTable.IncMod (current);
  }

  if (removed != 0) {
    if (index <= ivTable.ivLast) {
      DestroyNodes (ivTable.NodeAtIndex (index), removed);
    }
    else {
      DestroyNodes (ivTable.NodeAtIndex (index),
                    ivTable.ivAllocatedElements - index + 1);
      DestroyNodes (ivTable.NodeAtIndex (1),
                    removed -
                      (ivTable.ivAllocatedElements - index + 1));
    }
  }

#if ! defined (__INO_MNGCURSOR__)
  current = index;
#endif
  ivTable.ivLast = ivTable.DecMod (index);
#if ! defined (__INO_MNGCURSOR__)
  UpdateMngCursorsForPredRemove (current, ivTable.ivLast);
#endif
  ivTable.ivNumberOfElements -= removed;

  return removed;
}

void
ISequenceAsTableImpl::
RemoveAtPosition (IPosition position)
{ Self::SetToPosition (position, CursorOf (*this));
  RemoveAtIndex (IndexOf (CursorOf (*this)));
}

void
ISequenceAsTableImpl::
Reverse ()
{ if (Self::NumberOfElements () > 1) {
    INumber count = Self::NumberOfElements () / 2;
    INumber forward = ivTable.ivFirst,
            backward = ivTable.ivLast;
    while (count--) {
      Swap (forward, backward);
      forward++;
      backward--;
    }
  }

#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor))
      cursor->Invalidate ();
  }
#endif
}

void
ISequenceAsTableImpl::
operator= (ISequenceAsTableImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    if (! ivTable.CanAdd (collection.ivTable.ivNumberOfElements))
      GrowBy (collection.ivTable.ivNumberOfElements);
    CopyFrom (collection.ivTable);
#if ! defined (__INO_STREAMING__)
    ivStreamer = collection.ivStreamer;
#endif
  }
}

#if ! defined (__INO_STREAMING__)
void
ISequenceAsTableImpl::
StreamIn (IDataStream& fromWhere)
{ ReadVersion (fromWhere);
  Self::RemoveAll ();
  INumber n;
  n <<= fromWhere;
  while (n--) {
    Self::AddAsLastFromStreamer (fromWhere, CursorOf (*this));
  }
}

void
ISequenceAsTableImpl::
StreamOut (IDataStream& toWhere) const
{ WriteVersion (toWhere);
  Self::NumberOfElements () >>= toWhere;
  for (Self::SetToFirst (CursorOf (*this));
       CursorOf (*this).IsValid ();
       Self::SetToNext (CursorOf (*this))) {
    ivStreamer->streamOut (toWhere, Self::ElementAt (CursorOf (*this)));
  }
}

void
ISequenceAsTableImpl::
AdoptStreamer (IACollectionStreamer *streamer)
{ if (streamer != ivStreamer.PtrOf())
    ivStreamer = IStreamerPtr(streamer);
}

IStreamerPtr&
ISequenceAsTableImpl::
StreamerOf (IACollectionImpl const& collection)
{ return ivStreamer;
}
#endif

// private members

void
ISequenceAsTableImpl::
GrowBy (INumber n)
{
#if ! defined (__INO_MNGCURSOR__)
  UpdateMngCursorsForGrowBy ();
#endif

  n += ivTable.ivNumberOfElements;
  ITableImpl table = ivTable;

  INumber newAllocatedElements = ivTable.ivAllocatedElements;
  while (newAllocatedElements < n)
    newAllocatedElements *= 2;

  Initialize (newAllocatedElements);
  CopyFrom (table);

  if (table.ivFirst <= table.ivLast) {
    DestroyNodes (table.NodeAtIndex (table.ivFirst),
                  table.ivNumberOfElements);
  }
  else {
    DestroyNodes (table.NodeAtIndex (table.ivFirst),
                  table.ivAllocatedElements - table.ivFirst + 1);
    DestroyNodes (table.NodeAtIndex (1), table.ivLast);
  }
  DeleteNodes (table.ivNodes);
}

void
ISequenceAsTableImpl::
Initialize (INumber n)
{ ivTable.ivNodes = CreateNodes (n, ivTable.ivNodeSize);
  ivTable.ivFirst = 1;
  ivTable.ivLast = ivTable.ivAllocatedElements = n;
  ivTable.ivNumberOfElements = 0;
}

void
ISequenceAsTableImpl::
Quicksort (ICompFunc compFunc, void* addArg, INumber from, INumber to)
{ INumber i = from, j = to;
  ivTable.IncMod (i);
  for (;;) {
    while (i != j &&
           ((*compFunc) (ElementOf (ivTable.NodeAtIndex (i)),
                         ElementOf (ivTable.NodeAtIndex (from)),
                         addArg) <= 0))
      ivTable.IncMod (i);
    while (i != j &&
           ((*compFunc) (ElementOf (ivTable.NodeAtIndex (j)),
                         ElementOf (ivTable.NodeAtIndex (from)),
                         addArg) >= 0))
      ivTable.DecMod (j);
    if (i == j)
      break;
    Swap (i, j);
    ivTable.IncMod (i);
    if (i != j)
      ivTable.DecMod (j);
  }
  if ((*compFunc) (ElementOf (ivTable.NodeAtIndex (i)),
                   ElementOf (ivTable.NodeAtIndex (from)),
                   addArg) > 0)
    ivTable.DecMod (i);
  j = i;
  if (i != from) {
    Swap (from, i);
    ivTable.DecMod (i);
  }
  if (j != to)
    ivTable.IncMod (j);
  if (from != i)
    Quicksort (compFunc, addArg, from, i);
  if (j != to)
    Quicksort (compFunc, addArg, j, to);
}

void
ISequenceAsTableImpl::
RemoveAtIndex (INumber index)
{ DestroyNodes (ivTable.NodeAtIndex (index));
  if (ivTable.ivFirst == index) {
#if ! defined (__INO_MNGCURSOR__)
    UpdateMngCursorsForRemoveOf (index);
#endif
    ivTable.IncMod (ivTable.ivFirst);
  }
  else if (ivTable.ivLast == index) {
#if ! defined (__INO_MNGCURSOR__)
    UpdateMngCursorsForRemoveOf (index);
#endif
    ivTable.DecMod (ivTable.ivLast);
  }
  else if ((ivTable.ivFirst <= ivTable.ivLast &&
            index - ivTable.ivFirst > ivTable.ivLast - index) ||
           (ivTable.ivFirst > ivTable.ivLast &&
            index <= ivTable.ivLast)) {

    BlockLeft(index, ivTable.ivLast - index);
    ivTable.ivLast--;
  }
  else {
    BlockRight (ivTable.ivFirst + 1, index - ivTable.ivFirst);
    ivTable.ivFirst++;
    ivTable.IncMod (index);
  }
  ivTable.ivNumberOfElements--;
}


void
ISequenceAsTableImpl::
SetInBetween (MngCursor& cursor) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (! IsInBetween (cursor)) {
    if (IsLastAt (cursor))
      NextIndexOf (cursor) = 0;
    else {
      NextIndexOf (cursor) = IndexOf (cursor);
      ivTable.IncMod (NextIndexOf (cursor));
    }

    if (IsFirstAt (cursor))
      PreviousIndexOf (cursor) = 0;
    else {
      PreviousIndexOf (cursor) = IndexOf (cursor);
      ivTable.DecMod (PreviousIndexOf (cursor));
    }

    IndexOf (cursor) = MngCursor::kIsInBetween;
  }
#endif
}

#if defined (__INO_MNGCURSOR__)
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nopar, nouse)
#endif
#endif

void
ISequenceAsTableImpl::
UpdateMngCursorsForAssign
  (INumber targetIndex, INumber sourceIndex) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (sourceIndex != targetIndex) {
    IASSERT(targetIndex > 0)

    INumber nextAfterTarget = targetIndex;
    nextAfterTarget = (targetIndex == ivTable.ivLast) ?
                      0 : ivTable.IncMod (nextAfterTarget);

    INumber prevBeforeTarget = targetIndex;
    prevBeforeTarget = (targetIndex == ivTable.ivFirst) ?
                       0 : ivTable.DecMod(prevBeforeTarget);

    forMngCursors (cursor) {
      if (IsInBetween (*cursor)) {
        if (PreviousIndexOf (*cursor) == sourceIndex) {
          PreviousIndexOf (*cursor) = targetIndex;
          NextIndexOf (*cursor) = nextAfterTarget;
        }
        if (NextIndexOf (*cursor) == sourceIndex) {
          NextIndexOf (*cursor) = targetIndex;
          PreviousIndexOf (*cursor) = prevBeforeTarget;
        }
      }
      else {
        if (IndexOf (*cursor) == sourceIndex)
          IndexOf (*cursor) = targetIndex;
      }
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForCopyFrom (INumber fromIndex, INumber toIndex) const
{
#if ! defined (__INO_MNGCURSOR__)
  if (fromIndex != toIndex) {
    forMngCursors (cursor) {
      if (IsInBetween (*cursor)) {
        if (PreviousIndexOf (*cursor) == fromIndex)
          PreviousIndexOf (*cursor) = toIndex;
        else
          if (NextIndexOf (*cursor) == fromIndex)
            NextIndexOf (*cursor) = toIndex;
      }
      else {
        if (IndexOf (*cursor) == fromIndex)
          IndexOf (*cursor) = toIndex;
      }
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForGrowBy () const
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousIndexOf (*cursor) != 0) {
        PreviousIndexOf (*cursor)
          = ivTable.Pos (PreviousIndexOf (*cursor));
      }
      if (NextIndexOf (*cursor) != 0) {
        NextIndexOf (*cursor)
          = ivTable.Pos (NextIndexOf (*cursor));
      }
    }
    else if (IndexOf (*cursor) != 0) {
      IndexOf (*cursor) = ivTable.Pos (IndexOf (*cursor));
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForPredRemove
  (INumber removedIndex, INumber prevIndex) const
{
#if ! defined (__INO_MNGCURSOR__)
  INumber nextIndex;

  nextIndex = prevIndex;
  nextIndex =
     (prevIndex != 0) ?
     ((prevIndex == ivTable.ivLast) ? 0 : ivTable.IncMod (nextIndex)) :
     ivTable.ivFirst;

  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousIndexOf (*cursor) == removedIndex
          || NextIndexOf (*cursor) == removedIndex) {
        PreviousIndexOf (*cursor) = prevIndex;
        NextIndexOf (*cursor) = nextIndex;
      }
    }
    else {
      if (removedIndex == IndexOf (*cursor)) {
        PreviousIndexOf (*cursor) = prevIndex;
        NextIndexOf (*cursor) = nextIndex;
        IndexOf (*cursor) = MngCursor::kIsInBetween;
      }
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForRemoveOf (INumber index) const
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousIndexOf (*cursor) == index) {
        if (index == ivTable.ivFirst)
          PreviousIndexOf (*cursor) = 0;
        else
          ivTable.DecMod (PreviousIndexOf (*cursor));
      }
      if (NextIndexOf (*cursor) == index) {
        if (index == ivTable.ivLast)
          NextIndexOf (*cursor) = 0;
        else
          ivTable.IncMod (NextIndexOf (*cursor));
      }
    }
    else {
      if (index == IndexOf (*cursor))
        SetInBetween (*cursor);
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForShiftLeftOf (INumber index) const
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousIndexOf (*cursor) == index)
        ivTable.DecMod (PreviousIndexOf (*cursor));
      else
        if (NextIndexOf (*cursor) == index)
          ivTable.DecMod (NextIndexOf (*cursor));
    }
    else {
      if (index == IndexOf (*cursor))
        ivTable.DecMod (IndexOf (*cursor));
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForShiftRightOf (INumber index) const
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor)) {
      if (PreviousIndexOf (*cursor) == index)
        ivTable.IncMod (PreviousIndexOf (*cursor));
      else
        if (NextIndexOf (*cursor) == index)
          ivTable.IncMod (NextIndexOf (*cursor));
    }
    else {
      if (index == IndexOf (*cursor))
        ivTable.IncMod (IndexOf (*cursor));
    }
  }
#endif
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForSwap (INumber index1, INumber index2) const
{
#if ! defined (__INO_MNGCURSOR__)
  forMngCursors (cursor) {
    if (IsInBetween (*cursor))
      cursor->Invalidate ();
    else {
      if (IndexOf (*cursor) == index1)
        IndexOf (*cursor) = index2;
      else if (IndexOf (*cursor) == index2)
        IndexOf (*cursor) = index1;
    }
  }
#endif
}

#if defined (__INO_MNGCURSOR__)
#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif
