// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 97 1.5.1.3 source/core/collect/iivks.c, collection, ioc.v400, 000302 
#ifndef _IIVKS_C_
#define _IIVKS_C_

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// IVKeySetImpl
// ---

// public members

template <class VInherited>
IVKeySetImpl <VInherited>::
IVKeySetImpl (Ops& ops, INumber numberOfElements)
: VInherited (ops, numberOfElements)
{
}

template <class VInherited>
IVKeySetImpl <VInherited>::
IVKeySetImpl
  (Ops& ops, IVKeySetImpl <VInherited> const& collection)
: VInherited (ops, collection)
{
}

template <class VInherited>
bool
IVKeySetImpl <VInherited>::
Add (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenAdded = VInherited::Add (element, cursor);
  this->EnableNotification (isEnabled);
  if (hasBeenAdded) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVKeySetImpl <VInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAllFrom (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
bool
IVKeySetImpl <VInherited>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ bool hasBeenAdded = false;
  if (VInherited::LocateElementWithKey (this->Key (element), cursor)) {
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
    this->NotifyOfReplaceAt (cursor, element);
#endif
    VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  }
  else {
    hasBeenAdded = VInherited::Add (element, cursor);
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVKeySetImpl <VInherited>::
Copy (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Copy (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
bool
IVKeySetImpl <VInherited>::
LocateOrAddElementWithKey (void const* key, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenLocated =
    VInherited::LocateOrAddElementWithKey (key, cursor);
  this->EnableNotification (isEnabled);
  if (!hasBeenLocated) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenLocated;
}

template <class VInherited>
INumber
IVKeySetImpl <VInherited>::
RemoveAll ()
{ INumber result = VInherited::RemoveAll ();
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVKeySetImpl <VInherited>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber result = VInherited::RemoveAll (predFunc, addArg);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVKeySetImpl <VInherited>::
RemoveAllElementsWithKey (void const* key)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  INumber result = VInherited::RemoveAllElementsWithKey (key);
  this->EnableNotification (isEnabled);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
void
IVKeySetImpl <VInherited>::
RemoveAt (ICursorImpl& cursor)
{ this->NotifyOfRemoveAt (cursor);
  VInherited::RemoveAt (cursor);
}

template <class VInherited>
bool
IVKeySetImpl <VInherited>::
RemoveElementWithKey (void const* key)
{ if (this->LocateElementWithKey (key, this->CursorOf (*this))) {
    this->NotifyOfRemoveAt (this->CursorOf (*this));
  }
  return VInherited::RemoveElementWithKey (key);
}

template <class VInherited>
void
IVKeySetImpl <VInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
}

template <class VInherited>
bool
IVKeySetImpl <VInherited>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  bool hasBeenReplaced =
    VInherited::ReplaceElementWithKey (element, cursor);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  return hasBeenReplaced;
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIVKS_C_
