// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 07 1.5.1.3 source/core/collect/iivrel.c, collection, ioc.v400, 000302 
#ifndef _IIVREL_C_
#define _IIVREL_C_

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

// ---
// IVRelationImpl
// ---

// public members

template <class VInherited>
IVRelationImpl <VInherited>::
IVRelationImpl (Ops& ops, INumber numberOfElements)
: VInherited (ops, numberOfElements)
{
}

template <class VInherited>
IVRelationImpl <VInherited>::
IVRelationImpl
  (Ops& ops, IVRelationImpl <VInherited> const& collection)
: VInherited (ops, collection)
{
}

template <class VInherited>
bool
IVRelationImpl <VInherited>::
Add (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenAdded = VInherited::Add (element, cursor);
  this->EnableNotification (isEnabled);
  if (hasBeenAdded) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
AddAllFrom (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAllFrom (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
AddAntivalence (IARelationImpl const& collection1,
                IARelationImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddAntivalence (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
AddDifference (IARelationImpl const& collection1,
               IARelationImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddDifference (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
AddIntersection (IARelationImpl const& collection1,
               IARelationImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddIntersection (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
AddUnion (IARelationImpl const& collection1,
          IARelationImpl const& collection2)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::AddUnion (collection1, collection2);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
Antivalence (IARelationImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Antivalence (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}
template <class VInherited>
bool
IVRelationImpl <VInherited>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ bool hasBeenAdded = false;
  if (VInherited::LocateElementWithKey (this->Key (element), cursor)) {
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
    this->NotifyOfReplaceAt (cursor, element);
#endif
    VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
    this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  }
  else {
    hasBeenAdded = VInherited::Add (element, cursor);
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenAdded;
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
Copy (IACollectionImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Copy (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
Difference (IARelationImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Difference (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
Intersection (IARelationImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Intersection (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}
template <class VInherited>
bool
IVRelationImpl <VInherited>::
LocateOrAdd (void const* element, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenLocated = VInherited::LocateOrAdd (element, cursor);
  this->EnableNotification (isEnabled);
  if (! hasBeenLocated) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenLocated;
}

template <class VInherited>
bool
IVRelationImpl <VInherited>::
LocateOrAddElementWithKey (void const* key, ICursorImpl& cursor)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  bool hasBeenLocated =
    VInherited::LocateOrAddElementWithKey (key, cursor);
  this->EnableNotification (isEnabled);
  if (!hasBeenLocated) {
    this->NotifyOfAddAt (cursor);
  }
  return hasBeenLocated;
}

template <class VInherited>
bool
IVRelationImpl <VInherited>::
Remove (void const* element)
{ if (this->Locate (element, this->CursorOf (*this))) {
    this->NotifyOfRemoveAt (this->CursorOf (*this));
  }
  return VInherited::Remove (element);
}

template <class VInherited>
INumber
IVRelationImpl <VInherited>::
RemoveAll ()
{ INumber result = VInherited::RemoveAll ();
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVRelationImpl <VInherited>::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber result = VInherited::RemoveAll (predFunc, addArg);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVRelationImpl <VInherited>::
RemoveAllElementsWithKey (void const* key)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  INumber result = VInherited::RemoveAllElementsWithKey (key);
  this->EnableNotification (isEnabled);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
INumber
IVRelationImpl <VInherited>::
RemoveAllOccurrences (void const* element)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  INumber result = VInherited::RemoveAllOccurrences (element);
  this->EnableNotification (isEnabled);
  if (result > 0) {
    this->NotifyOfModification ();
  }
  return result;
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
RemoveAt (ICursorImpl& cursor)
{ this->NotifyOfRemoveAt (cursor);
  VInherited::RemoveAt (cursor);
}

template <class VInherited>
bool
IVRelationImpl <VInherited>::
RemoveElementWithKey (void const* key)
{ if (this->LocateElementWithKey (key, this->CursorOf (*this))) {
    this->NotifyOfRemoveAt (this->CursorOf (*this));
  }
  return VInherited::RemoveElementWithKey (key);
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  VInherited::ReplaceAt (cursor, element);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
}

template <class VInherited>
bool
IVRelationImpl <VInherited>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfRemoveAt (cursor); // to be removed in a future release
#else
  this->NotifyOfReplaceAt (cursor, element);
#endif
  bool hasBeenReplaced =
    VInherited::ReplaceElementWithKey (element, cursor);
#if defined (ICLCC_COMPAT_IVB)
  this->NotifyOfAddAt (cursor); // to be removed in a future release
#endif
  return hasBeenReplaced;
}

template <class VInherited>
void
IVRelationImpl <VInherited>::
Union (IARelationImpl const& collection)
{ bool isEnabled = this->IsEnabledForNotification ();
  this->DisableNotification ();
  VInherited::Union (collection);
  this->EnableNotification (isEnabled);
  this->NotifyOfModification ();
}

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IIVREL_C_
