// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 30 1.4.1.4 source/core/collect/ikbtab.inl, collection, ioc.v400, 000302  
#ifndef _IKBTAB_INL_
#define _IKBTAB_INL_

// ---
// IGKeyBagAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTableCursor <Element, Key, ElementOps>::
IGKeyBagAsTableCursor
  (IGKeyBagAsTable
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeyBagAsTableCursor <Element, Key, ElementOps>&
IGKeyBagAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGKeyBagAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTableCursor <Element, Key, ElementOps>::
IGKeyBagAsTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeyBagAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTableMngCursor <Element, Key, ElementOps>::
IGKeyBagAsTableMngCursor
  (IGKeyBagAsTable
    <Element, Key, ElementOps> const& collection)
: IGKeyBagAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTableMngCursor <Element, Key, ElementOps>::
IGKeyBagAsTableMngCursor (ICursorImpl* impl)
: IGKeyBagAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeyBagAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTable <Element, Key, ElementOps>::
IGKeyBagAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTable <Element, Key, ElementOps>::
IGKeyBagAsTable
  (IGKeyBagAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeyBagAsTable <Element, Key, ElementOps>&
IGKeyBagAsTable <Element, Key, ElementOps>::
operator= (IGKeyBagAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeyBagAsTable <Element, Key, ElementOps>::
IGKeyBagAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeyBagAsTable
 <Element, Key, ElementOps>::Implementation&
IGKeyBagAsTable <Element, Key, ElementOps>::
ImplOf (IGKeyBagAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeyBagAsTable
// ---

// public members

template <class Element, class Key>
inline
IKeyBagAsTable <Element, Key>::
IKeyBagAsTable (INumber numberOfElements)
: IGKeyBagAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeyBagAsTable <Element, Key>::
IKeyBagAsTable (PureImplementation& impl)
: IGKeyBagAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeyBagAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBagAsTable <Element, Key, ElementOps>::
IVGKeyBagAsTable (INumber numberOfElements)
: IGKeyBagAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeyBagAsTable <Element, Key, ElementOps>::
IVGKeyBagAsTable (PureImplementation& impl)
: IGKeyBagAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeyBagAsTable
// ---

// public members

template <class Element, class Key>
inline
IVKeyBagAsTable <Element, Key>::
IVKeyBagAsTable (INumber numberOfElements)
: IVGKeyBagAsTable
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeyBagAsTable <Element, Key>::
IVKeyBagAsTable (PureImplementation& impl)
: IVGKeyBagAsTable
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKBTAB_INL_
