// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 35 1.5.1.4 source/core/collect/iksavl.h, collection, ioc.v400, 000302 
#ifndef _IKSAVL_
#define _IKSAVL_

#include <ilanglvl.hpp>
#include <iaks.h>
#include <iicks.h>
#if ! defined (__INO_AVALON__)
#include <iivks.h>
#endif
#include <iiksavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeySetAsAvlTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeySetAsAvlTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeySetAsAvlTree
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeySetAsAvlTreeCursor
                            (IGKeySetAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>&
           operator=        (IGKeySetAsAvlTreeCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsAvlTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsAvlTreeCursor\
                            (IGKeySetAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeySetAsAvlTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeySetAsAvlTreeMngCursor :
  public IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>  {

  typedef  IGKeySetAsAvlTreeCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeySetAsAvlTree
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeySetAsAvlTreeMngCursor
                            (IGKeySetAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsAvlTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsAvlTreeMngCursor\
                            (IGKeySetAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeySetAsAvlTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeySetAsAvlTree :
  public IAKeySet <Element, Key> {
protected:

  typedef  IGKeySetAsAvlTree <Element, Key, ElementOps> Self;

  typedef  IKeySetAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySetImpl
            <PureImplementation> Implementation;
#endif

#define IGKeySetAsAvlTreeTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeySetAsAvlTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeySetAsAvlTreeCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeySetAsAvlTreeMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeySetAsAvlTree
                            (INumber = 100);

           IGKeySetAsAvlTree
                            (IGKeySetAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsAvlTree <Element, Key, ElementOps>&
           operator=        (IGKeySetAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeySetAsAvlTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeySetAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeySetAsAvlTreeCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeySetAsAvlTreeMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsAvlTree\
                            (INumber),\
           operator=,\
           IGKeySetAsAvlTree\
                            (PureImplementation&),\
           ImplOf           (IGKeySetAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeySetAsAvlTree :
  public IGKeySetAsAvlTree
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGKeySetAsAvlTree
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeySetAsAvlTree <Element, Key> Self;

#define IKeySetAsAvlTreeTypedef(Element, Key)\
  typedef  IKeySetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeySetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeySetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeySetAsAvlTree\
                              (INumber),\
           IKeySetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeySetAsAvlTree :
  public IGKeySetAsAvlTree
          <Element, Key, ElementOps > {

  typedef  IVGKeySetAsAvlTree
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeySetAsAvlTree
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySetImpl
            <IVKeySetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeySetAsAvlTreeTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeySetAsAvlTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeySetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeySetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeySetAsAvlTree\
                              (INumber),\
           IVGKeySetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeySetAsAvlTree :
  public IVGKeySetAsAvlTree
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGKeySetAsAvlTree
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeySetAsAvlTree <Element, Key> Self;

#define IVKeySetAsAvlTreeTypedef(Element, Key)\
  typedef  IVKeySetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeySetAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeySetAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeySetAsAvlTree\
                              (INumber),\
           IVKeySetAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iksavl.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKSAVL_
