// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 45 1.5.1.4 source/core/collect/iksbst.h, collection, ioc.v400, 000302 
#ifndef _IKSBST_
#define _IKSBST_

#include <ilanglvl.hpp>
#include <iaks.h>
#include <iicks.h>
#if ! defined (__INO_AVALON__)
#include <iivks.h>
#endif
#include <iiksbst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeySetAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeySetAsBstTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeySetAsBstTree
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeySetAsBstTreeCursor
                            (IGKeySetAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsBstTreeCursor <Element, Key, ElementOps>&
           operator=        (IGKeySetAsBstTreeCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsBstTreeCursor\
                            (IGKeySetAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeySetAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeySetAsBstTreeMngCursor :
  public IGKeySetAsBstTreeCursor <Element, Key, ElementOps>  {

  typedef  IGKeySetAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeySetAsBstTree
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeySetAsBstTreeMngCursor
                            (IGKeySetAsBstTree
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsBstTreeMngCursor\
                            (IGKeySetAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeySetAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeySetAsBstTree :
  public IAKeySet <Element, Key> {
protected:

  typedef  IGKeySetAsBstTree <Element, Key, ElementOps> Self;

  typedef  IKeySetAsBstTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySetImpl
            <PureImplementation> Implementation;
#endif

#define IGKeySetAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsBstTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeySetAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeySetAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeySetAsBstTreeMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeySetAsBstTree
                            (INumber = 100);

           IGKeySetAsBstTree
                            (IGKeySetAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsBstTree <Element, Key, ElementOps>&
           operator=        (IGKeySetAsBstTree
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeySetAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeySetAsBstTree
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeySetAsBstTreeCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeySetAsBstTreeMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsBstTree\
                            (INumber),\
           operator=,\
           IGKeySetAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGKeySetAsBstTree\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeySetAsBstTree :
  public IGKeySetAsBstTree
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGKeySetAsBstTree
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeySetAsBstTree <Element, Key> Self;

#define IKeySetAsBstTreeTypedef(Element, Key)\
  typedef  IKeySetAsBstTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeySetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeySetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeySetAsBstTree\
                              (INumber),\
           IKeySetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeySetAsBstTree :
  public IGKeySetAsBstTree
          <Element, Key, ElementOps > {

  typedef  IVGKeySetAsBstTree
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeySetAsBstTree
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySetImpl
            <IVKeySetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeySetAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsBstTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeySetAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeySetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeySetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeySetAsBstTree\
                              (INumber),\
           IVGKeySetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeySetAsBstTree :
  public IVGKeySetAsBstTree
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGKeySetAsBstTree
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeySetAsBstTree <Element, Key> Self;

#define IVKeySetAsBstTreeTypedef(Element, Key)\
  typedef  IVKeySetAsBstTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeySetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeySetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeySetAsBstTree\
                              (INumber),\
           IVKeySetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iksbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKSBST_
