// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 54 1.5.1.4 source/core/collect/ikslst.h, collection, ioc.v400, 000302 
#ifndef _IKSLST_
#define _IKSLST_

#include <ilanglvl.hpp>
#include <iaks.h>
#include <iicks.h>
#if ! defined (__INO_AVALON__)
#include <iivks.h>
#endif
#include <iikslst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeySetAsList;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeySetAsListCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGKeySetAsList
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeySetAsListCursor
                            (IGKeySetAsList
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsListCursor <Element, Key, ElementOps>&
           operator=        (IGKeySetAsListCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsListCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsListCursor\
                            (IGKeySetAsList\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeySetAsListCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeySetAsListMngCursor :
  public IGKeySetAsListCursor <Element, Key, ElementOps>  {

  typedef  IGKeySetAsListCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeySetAsList
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeySetAsListMngCursor
                            (IGKeySetAsList
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySetAsListMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsListMngCursor\
                            (IGKeySetAsList\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeySetAsListMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeySetAsList :
  public IAKeySet <Element, Key> {
protected:

  typedef  IGKeySetAsList <Element, Key, ElementOps> Self;

  typedef  IKeySetAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySetImpl
            <PureImplementation> Implementation;
#endif

#define IGKeySetAsListTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeySetAsListTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeySetAsListCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeySetAsListMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeySetAsList
                            (INumber = 100);

           IGKeySetAsList
                            (IGKeySetAsList
                              <
                               Element, Key, ElementOps> const&);

  IGKeySetAsList <Element, Key, ElementOps>&
           operator=        (IGKeySetAsList
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeySetAsList
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeySetAsList
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeySetAsListCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeySetAsListMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySetAsList\
                            (INumber),\
           operator=,\
           IGKeySetAsList\
                            (PureImplementation&),\
           ImplOf           (IGKeySetAsList\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeySetAsList :
  public IGKeySetAsList
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGKeySetAsList
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeySetAsList <Element, Key> Self;

#define IKeySetAsListTypedef(Element, Key)\
  typedef  IKeySetAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeySetAsList
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeySetAsList
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeySetAsList\
                              (INumber),\
           IKeySetAsList\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeySetAsList :
  public IGKeySetAsList
          <Element, Key, ElementOps > {

  typedef  IVGKeySetAsList
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeySetAsList
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySetImpl
            <IVKeySetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeySetAsListTypedef(Element, Key, ElementOps)\
  typedef  IKeySetAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeySetAsListTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeySetAsList
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeySetAsList
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeySetAsList\
                              (INumber),\
           IVGKeySetAsList\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeySetAsList :
  public IVGKeySetAsList
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGKeySetAsList
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeySetAsList <Element, Key> Self;

#define IVKeySetAsListTypedef(Element, Key)\
  typedef  IVKeySetAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeySetAsList
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeySetAsList
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeySetAsList\
                              (INumber),\
           IVKeySetAsList\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ikslst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKSLST_
