// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 55 1.4.1.4 source/core/collect/ikslst.inl, collection, ioc.v400, 000302  
#ifndef _IKSLST_INL_
#define _IKSLST_INL_

// ---
// IGKeySetAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsListCursor <Element, Key, ElementOps>::
IGKeySetAsListCursor
  (IGKeySetAsList
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsListCursor <Element, Key, ElementOps>&
IGKeySetAsListCursor <Element, Key, ElementOps>::
operator=
  (IGKeySetAsListCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsListCursor <Element, Key, ElementOps>::
IGKeySetAsListCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySetAsListMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsListMngCursor <Element, Key, ElementOps>::
IGKeySetAsListMngCursor
  (IGKeySetAsList
    <Element, Key, ElementOps> const& collection)
: IGKeySetAsListCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsListMngCursor <Element, Key, ElementOps>::
IGKeySetAsListMngCursor (ICursorImpl* impl)
: IGKeySetAsListCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySetAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsList <Element, Key, ElementOps>::
IGKeySetAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsList <Element, Key, ElementOps>::
IGKeySetAsList
  (IGKeySetAsList
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySetAsList <Element, Key, ElementOps>&
IGKeySetAsList <Element, Key, ElementOps>::
operator= (IGKeySetAsList
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySetAsList <Element, Key, ElementOps>::
IGKeySetAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySetAsList
 <Element, Key, ElementOps>::Implementation&
IGKeySetAsList <Element, Key, ElementOps>::
ImplOf (IGKeySetAsList
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySetAsList
// ---

// public members

template <class Element, class Key>
inline
IKeySetAsList <Element, Key>::
IKeySetAsList (INumber numberOfElements)
: IGKeySetAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySetAsList <Element, Key>::
IKeySetAsList (PureImplementation& impl)
: IGKeySetAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySetAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsList <Element, Key, ElementOps>::
IVGKeySetAsList (INumber numberOfElements)
: IGKeySetAsList
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySetAsList <Element, Key, ElementOps>::
IVGKeySetAsList (PureImplementation& impl)
: IGKeySetAsList
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySetAsList
// ---

// public members

template <class Element, class Key>
inline
IVKeySetAsList <Element, Key>::
IVKeySetAsList (INumber numberOfElements)
: IVGKeySetAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySetAsList <Element, Key>::
IVKeySetAsList (PureImplementation& impl)
: IVGKeySetAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSLST_INL_
