// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 56 1.5.1.4 source/core/collect/ikss.h, collection, ioc.v400, 000302 
#ifndef _IKSS_
#define _IKSS_

#include <ilanglvl.hpp>
#include <iakss.h>
#include <iickss.h>
#if ! defined (__INO_AVALON__)
#include <iivkss.h>
#endif
#include <iikssavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGKeySortedSet;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGKeySortedSetCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGKeySortedSet
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGKeySortedSetCursor
                            (IGKeySortedSet
                              <
                               Element, Key, ElementOps> const&);

  IGKeySortedSetCursor <Element, Key, ElementOps>&
           operator=        (IGKeySortedSetCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySortedSetCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySortedSetCursor\
                            (IGKeySortedSet\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGKeySortedSetCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGKeySortedSetMngCursor :
  public IGKeySortedSetCursor <Element, Key, ElementOps>  {

  typedef  IGKeySortedSetCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGKeySortedSet
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGKeySortedSetMngCursor
                            (IGKeySortedSet
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGKeySortedSetMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySortedSetMngCursor\
                            (IGKeySortedSet\
                              <\
                               Element, Key, ElementOps> const&),\
           IGKeySortedSetMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGKeySortedSet :
  public IAKeySortedSet <Element, Key> {
protected:

  typedef  IGKeySortedSet <Element, Key, ElementOps> Self;

  typedef  IKeySortedSetAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySortedSetImpl
            <PureImplementation> Implementation;
#endif

#define IGKeySortedSetTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedSetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGKeySortedSetTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGKeySortedSetCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGKeySortedSetMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGKeySortedSet
                            (INumber = 100);

           IGKeySortedSet
                            (IGKeySortedSet
                              <
                               Element, Key, ElementOps> const&);

  IGKeySortedSet <Element, Key, ElementOps>&
           operator=        (IGKeySortedSet
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGKeySortedSet
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGKeySortedSet
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGKeySortedSetCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGKeySortedSetMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGKeySortedSet\
                            (INumber),\
           operator=,\
           IGKeySortedSet\
                            (PureImplementation&),\
           ImplOf           (IGKeySortedSet\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IKeySortedSet :
  public IGKeySortedSet
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGKeySortedSet
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IKeySortedSet <Element, Key> Self;

#define IKeySortedSetTypedef(Element, Key)\
  typedef  IKeySortedSetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IKeySortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IKeySortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IKeySortedSet\
                              (INumber),\
           IKeySortedSet\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGKeySortedSet :
  public IGKeySortedSet
          <Element, Key, ElementOps > {

  typedef  IVGKeySortedSet
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGKeySortedSet
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySortedSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySortedSetImpl
            <IVKeySortedSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGKeySortedSetTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedSetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGKeySortedSetTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGKeySortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGKeySortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGKeySortedSet\
                              (INumber),\
           IVGKeySortedSet\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVKeySortedSet :
  public IVGKeySortedSet
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGKeySortedSet
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVKeySortedSet <Element, Key> Self;

#define IVKeySortedSetTypedef(Element, Key)\
  typedef  IVKeySortedSetAsAvlTreeOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVKeySortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVKeySortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVKeySortedSet\
                              (INumber),\
           IVKeySortedSet\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ikss.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IKSS_
