// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 59 1.4.1.4 source/core/collect/ikssavl.inl, collection, ioc.v400, 000302  
#ifndef _IKSSAVL_INL_
#define _IKSSAVL_INL_

// ---
// IGKeySortedSetAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTreeCursor
  (IGKeySortedSetAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps>&
IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedSetAsAvlTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedSetAsAvlTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTreeMngCursor
  (IGKeySortedSetAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTreeMngCursor (ICursorImpl* impl)
: IGKeySortedSetAsAvlTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedSetAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTree
  (IGKeySortedSetAsAvlTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsAvlTree <Element, Key, ElementOps>&
IGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
operator= (IGKeySortedSetAsAvlTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
IGKeySortedSetAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedSetAsAvlTree
 <Element, Key, ElementOps>::Implementation&
IGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
ImplOf (IGKeySortedSetAsAvlTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedSetAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IKeySortedSetAsAvlTree <Element, Key>::
IKeySortedSetAsAvlTree (INumber numberOfElements)
: IGKeySortedSetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedSetAsAvlTree <Element, Key>::
IKeySortedSetAsAvlTree (PureImplementation& impl)
: IGKeySortedSetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedSetAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
IVGKeySortedSetAsAvlTree (INumber numberOfElements)
: IGKeySortedSetAsAvlTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsAvlTree <Element, Key, ElementOps>::
IVGKeySortedSetAsAvlTree (PureImplementation& impl)
: IGKeySortedSetAsAvlTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedSetAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedSetAsAvlTree <Element, Key>::
IVKeySortedSetAsAvlTree (INumber numberOfElements)
: IVGKeySortedSetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedSetAsAvlTree <Element, Key>::
IVKeySortedSetAsAvlTree (PureImplementation& impl)
: IVGKeySortedSetAsAvlTree
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSSAVL_INL_
