// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 68 1.4.1.4 source/core/collect/iksslst.inl, collection, ioc.v400, 000302  
#ifndef _IKSSLST_INL_
#define _IKSSLST_INL_

// ---
// IGKeySortedSetAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsListCursor <Element, Key, ElementOps>::
IGKeySortedSetAsListCursor
  (IGKeySortedSetAsList
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsListCursor <Element, Key, ElementOps>&
IGKeySortedSetAsListCursor <Element, Key, ElementOps>::
operator=
  (IGKeySortedSetAsListCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsListCursor <Element, Key, ElementOps>::
IGKeySortedSetAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGKeySortedSetAsListMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsListMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsListMngCursor
  (IGKeySortedSetAsList
    <Element, Key, ElementOps> const& collection)
: IGKeySortedSetAsListCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsListMngCursor <Element, Key, ElementOps>::
IGKeySortedSetAsListMngCursor (ICursorImpl* impl)
: IGKeySortedSetAsListCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGKeySortedSetAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsList <Element, Key, ElementOps>::
IGKeySortedSetAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsList <Element, Key, ElementOps>::
IGKeySortedSetAsList
  (IGKeySortedSetAsList
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGKeySortedSetAsList <Element, Key, ElementOps>&
IGKeySortedSetAsList <Element, Key, ElementOps>::
operator= (IGKeySortedSetAsList
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGKeySortedSetAsList <Element, Key, ElementOps>::
IGKeySortedSetAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGKeySortedSetAsList
 <Element, Key, ElementOps>::Implementation&
IGKeySortedSetAsList <Element, Key, ElementOps>::
ImplOf (IGKeySortedSetAsList
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IKeySortedSetAsList
// ---

// public members

template <class Element, class Key>
inline
IKeySortedSetAsList <Element, Key>::
IKeySortedSetAsList (INumber numberOfElements)
: IGKeySortedSetAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IKeySortedSetAsList <Element, Key>::
IKeySortedSetAsList (PureImplementation& impl)
: IGKeySortedSetAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGKeySortedSetAsList
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsList <Element, Key, ElementOps>::
IVGKeySortedSetAsList (INumber numberOfElements)
: IGKeySortedSetAsList
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGKeySortedSetAsList <Element, Key, ElementOps>::
IVGKeySortedSetAsList (PureImplementation& impl)
: IGKeySortedSetAsList
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVKeySortedSetAsList
// ---

// public members

template <class Element, class Key>
inline
IVKeySortedSetAsList <Element, Key>::
IVKeySortedSetAsList (INumber numberOfElements)
: IVGKeySortedSetAsList
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVKeySortedSetAsList <Element, Key>::
IVKeySortedSetAsList (PureImplementation& impl)
: IVGKeySortedSetAsList
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IKSSLST_INL_
