// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 82 1.5.1.4 source/core/collect/imapbst.h, collection, ioc.v400, 000302 
#ifndef _IMAPBST_
#define _IMAPBST_

#include <ilanglvl.hpp>
#include <iamap.h>
#include <iicmap.h>
#if ! defined (__INO_AVALON__)
#include <iivmap.h>
#endif
#include <iimapbst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGMapAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGMapAsBstTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGMapAsBstTree
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGMapAsBstTreeCursor
                            (IGMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGMapAsBstTreeCursor <Element, Key, ElementOps>&
           operator=        (IGMapAsBstTreeCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGMapAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsBstTreeCursor\
                            (IGMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGMapAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGMapAsBstTreeMngCursor :
  public IGMapAsBstTreeCursor <Element, Key, ElementOps>  {

  typedef  IGMapAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGMapAsBstTree
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGMapAsBstTreeMngCursor
                            (IGMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGMapAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsBstTreeMngCursor\
                            (IGMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           IGMapAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGMapAsBstTree :
  public IAMap <Element, Key> {
protected:

  typedef  IGMapAsBstTree <Element, Key, ElementOps> Self;

  typedef  IMapAsBstTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICMapImpl
            <PureImplementation> Implementation;
#endif

#define IGMapAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  IMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGMapAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGMapAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGMapAsBstTreeMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGMapAsBstTree
                            (INumber = 100);

           IGMapAsBstTree
                            (IGMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGMapAsBstTree <Element, Key, ElementOps>&
           operator=        (IGMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGMapAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGMapAsBstTreeCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGMapAsBstTreeMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsBstTree\
                            (INumber),\
           operator=,\
           IGMapAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IMapAsBstTree :
  public IGMapAsBstTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IGMapAsBstTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IMapAsBstTree <Element, Key> Self;

#define IMapAsBstTreeTypedef(Element, Key)\
  typedef  IMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IMapAsBstTree\
                              (INumber),\
           IMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGMapAsBstTree :
  public IGMapAsBstTree
          <Element, Key, ElementOps > {

  typedef  IVGMapAsBstTree
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGMapAsBstTree
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVMapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICMapImpl
            <IVMapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGMapAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  IMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGMapAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGMapAsBstTree\
                              (INumber),\
           IVGMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVMapAsBstTree :
  public IVGMapAsBstTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IVGMapAsBstTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVMapAsBstTree <Element, Key> Self;

#define IVMapAsBstTreeTypedef(Element, Key)\
  typedef  IVMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVMapAsBstTree\
                              (INumber),\
           IVMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <imapbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IMAPBST_
