// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 83 1.4.1.4 source/core/collect/imapbst.inl, collection, ioc.v400, 000302  
#ifndef _IMAPBST_INL_
#define _IMAPBST_INL_

// ---
// IGMapAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTreeCursor <Element, Key, ElementOps>::
IGMapAsBstTreeCursor
  (IGMapAsBstTree
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGMapAsBstTreeCursor <Element, Key, ElementOps>&
IGMapAsBstTreeCursor <Element, Key, ElementOps>::
operator=
  (IGMapAsBstTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTreeCursor <Element, Key, ElementOps>::
IGMapAsBstTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGMapAsBstTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTreeMngCursor <Element, Key, ElementOps>::
IGMapAsBstTreeMngCursor
  (IGMapAsBstTree
    <Element, Key, ElementOps> const& collection)
: IGMapAsBstTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTreeMngCursor <Element, Key, ElementOps>::
IGMapAsBstTreeMngCursor (ICursorImpl* impl)
: IGMapAsBstTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGMapAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTree <Element, Key, ElementOps>::
IGMapAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTree <Element, Key, ElementOps>::
IGMapAsBstTree
  (IGMapAsBstTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGMapAsBstTree <Element, Key, ElementOps>&
IGMapAsBstTree <Element, Key, ElementOps>::
operator= (IGMapAsBstTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsBstTree <Element, Key, ElementOps>::
IGMapAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGMapAsBstTree
 <Element, Key, ElementOps>::Implementation&
IGMapAsBstTree <Element, Key, ElementOps>::
ImplOf (IGMapAsBstTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IMapAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IMapAsBstTree <Element, Key>::
IMapAsBstTree (INumber numberOfElements)
: IGMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IMapAsBstTree <Element, Key>::
IMapAsBstTree (PureImplementation& impl)
: IGMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGMapAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsBstTree <Element, Key, ElementOps>::
IVGMapAsBstTree (INumber numberOfElements)
: IGMapAsBstTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsBstTree <Element, Key, ElementOps>::
IVGMapAsBstTree (PureImplementation& impl)
: IGMapAsBstTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVMapAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IVMapAsBstTree <Element, Key>::
IVMapAsBstTree (INumber numberOfElements)
: IVGMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVMapAsBstTree <Element, Key>::
IVMapAsBstTree (PureImplementation& impl)
: IVGMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IMAPBST_INL_
