// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 87 1.5.1.4 source/core/collect/imaphsh.h, collection, ioc.v400, 000302 
#ifndef _IMAPHSH_
#define _IMAPHSH_

#include <ilanglvl.hpp>
#include <iamap.h>
#include <iicmap.h>
#if ! defined (__INO_AVALON__)
#include <iivmap.h>
#endif
#include <iimaphsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGMapAsHshTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGMapAsHshTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGMapAsHshTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGMapAsHshTableCursor
                            (IGMapAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGMapAsHshTableCursor <Element, Key, ElementOps>&
           operator=        (IGMapAsHshTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGMapAsHshTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsHshTableCursor\
                            (IGMapAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGMapAsHshTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGMapAsHshTableMngCursor :
  public IGMapAsHshTableCursor <Element, Key, ElementOps>  {

  typedef  IGMapAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGMapAsHshTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGMapAsHshTableMngCursor
                            (IGMapAsHshTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGMapAsHshTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsHshTableMngCursor\
                            (IGMapAsHshTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGMapAsHshTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGMapAsHshTable :
  public IAMap <Element, Key> {
protected:

  typedef  IGMapAsHshTable <Element, Key, ElementOps> Self;

  typedef  IMapAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICMapImpl
            <PureImplementation> Implementation;
#endif

#define IGMapAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IMapAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGMapAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGMapAsHshTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGMapAsHshTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGMapAsHshTable
                            (INumber = 100);

           IGMapAsHshTable
                            (IGMapAsHshTable
                              <
                               Element, Key, ElementOps> const&);

  IGMapAsHshTable <Element, Key, ElementOps>&
           operator=        (IGMapAsHshTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGMapAsHshTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGMapAsHshTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGMapAsHshTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGMapAsHshTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGMapAsHshTable\
                            (INumber),\
           operator=,\
           IGMapAsHshTable\
                            (PureImplementation&),\
           ImplOf           (IGMapAsHshTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IMapAsHshTable :
  public IGMapAsHshTable
          <Element, Key, IEKEHOps <Element, Key> > {

  typedef  IGMapAsHshTable
            <Element, Key,
             IEKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IMapAsHshTable <Element, Key> Self;

#define IMapAsHshTableTypedef(Element, Key)\
  typedef  IMapAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, IEKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IMapAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IMapAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IMapAsHshTable\
                              (INumber),\
           IMapAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGMapAsHshTable :
  public IGMapAsHshTable
          <Element, Key, ElementOps > {

  typedef  IVGMapAsHshTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGMapAsHshTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVMapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICMapImpl
            <IVMapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGMapAsHshTableTypedef(Element, Key, ElementOps)\
  typedef  IMapAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGMapAsHshTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGMapAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGMapAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGMapAsHshTable\
                              (INumber),\
           IVGMapAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVMapAsHshTable :
  public IVGMapAsHshTable
          <Element, Key, IEKEHOps <Element, Key> > {

  typedef  IVGMapAsHshTable
            <Element, Key,
             IEKEHOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVMapAsHshTable <Element, Key> Self;

#define IVMapAsHshTableTypedef(Element, Key)\
  typedef  IVMapAsHshTableOps\
            <Element, Key,\
             IWEKEHOps <Element, Key, IEKEHOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVMapAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVMapAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVMapAsHshTable\
                              (INumber),\
           IVMapAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <imaphsh.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IMAPHSH_
