// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 95 1.4.1.4 source/core/collect/imaptab.inl, collection, ioc.v400, 000302  
#ifndef _IMAPTAB_INL_
#define _IMAPTAB_INL_

// ---
// IGMapAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTableCursor <Element, Key, ElementOps>::
IGMapAsTableCursor
  (IGMapAsTable
    <Element, Key, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGMapAsTableCursor <Element, Key, ElementOps>&
IGMapAsTableCursor <Element, Key, ElementOps>::
operator=
  (IGMapAsTableCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTableCursor <Element, Key, ElementOps>::
IGMapAsTableCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGMapAsTableMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTableMngCursor <Element, Key, ElementOps>::
IGMapAsTableMngCursor
  (IGMapAsTable
    <Element, Key, ElementOps> const& collection)
: IGMapAsTableCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTableMngCursor <Element, Key, ElementOps>::
IGMapAsTableMngCursor (ICursorImpl* impl)
: IGMapAsTableCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGMapAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTable <Element, Key, ElementOps>::
IGMapAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGMapAsTable <Element, Key, ElementOps>::
IGMapAsTable
  (IGMapAsTable
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGMapAsTable <Element, Key, ElementOps>&
IGMapAsTable <Element, Key, ElementOps>::
operator= (IGMapAsTable
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGMapAsTable <Element, Key, ElementOps>::
IGMapAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGMapAsTable
 <Element, Key, ElementOps>::Implementation&
IGMapAsTable <Element, Key, ElementOps>::
ImplOf (IGMapAsTable
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IMapAsTable
// ---

// public members

template <class Element, class Key>
inline
IMapAsTable <Element, Key>::
IMapAsTable (INumber numberOfElements)
: IGMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IMapAsTable <Element, Key>::
IMapAsTable (PureImplementation& impl)
: IGMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGMapAsTable
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsTable <Element, Key, ElementOps>::
IVGMapAsTable (INumber numberOfElements)
: IGMapAsTable
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGMapAsTable <Element, Key, ElementOps>::
IVGMapAsTable (PureImplementation& impl)
: IGMapAsTable
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVMapAsTable
// ---

// public members

template <class Element, class Key>
inline
IVMapAsTable <Element, Key>::
IVMapAsTable (INumber numberOfElements)
: IVGMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVMapAsTable <Element, Key>::
IVMapAsTable (PureImplementation& impl)
: IVGMapAsTable
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IMAPTAB_INL_
