// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 03 1.4.1.4 source/core/collect/imwt.inl, collection, ioc.v400, 000302  
#ifndef _IMWT_INL_
#define _IMWT_INL_

// ---
// IGMultiwayTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>::
IGMultiwayTreeCursor
  (IGMultiwayTree
    <numOfChildren, Element, ElementOps> const& collection)
: IElementTreeCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <INumber numOfChildren, class Element, class ElementOps>
inline IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>&
IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>::
operator=
  (IGMultiwayTreeCursor
    <numOfChildren, Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>::
IGMultiwayTreeCursor (ITreeCursorImpl* impl)
: IElementTreeCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGMultiwayTreeMngCursor
// ---

// public members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTreeMngCursor <numOfChildren, Element, ElementOps>::
IGMultiwayTreeMngCursor
  (IGMultiwayTree
    <numOfChildren, Element, ElementOps> const& collection)
: IGMultiwayTreeCursor <numOfChildren, Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTreeMngCursor <numOfChildren, Element, ElementOps>::
IGMultiwayTreeMngCursor (ITreeCursorImpl* impl)
: IGMultiwayTreeCursor <numOfChildren, Element, ElementOps> (impl)
{
}

#endif
// ---
// IGMultiwayTree
// ---

// public members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTree <numOfChildren, Element, ElementOps>::
IGMultiwayTree ()
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation ());
}

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTree <numOfChildren, Element, ElementOps>::
IGMultiwayTree
  (IGMultiwayTree
    <numOfChildren, Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <INumber numOfChildren, class Element, class ElementOps>
inline IGMultiwayTree <numOfChildren, Element, ElementOps>&
IGMultiwayTree <numOfChildren, Element, ElementOps>::
operator= (IGMultiwayTree
            <numOfChildren, Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IGMultiwayTree <numOfChildren, Element, ElementOps>::
IGMultiwayTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <INumber numOfChildren, class Element, class ElementOps>
inline TYPENAME IGMultiwayTree
 <numOfChildren, Element, ElementOps>::Implementation&
IGMultiwayTree <numOfChildren, Element, ElementOps>::
ImplOf (IGMultiwayTree
         <numOfChildren, Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IMultiwayTree
// ---

// public members

template <INumber numOfChildren, class Element>
inline
IMultiwayTree <numOfChildren, Element>::
IMultiwayTree ()
: IGMultiwayTree
   <numOfChildren, Element, IStdOps <Element> > ()
{
}

// protected members

template <INumber numOfChildren, class Element>
inline
IMultiwayTree <numOfChildren, Element>::
IMultiwayTree (PureImplementation& impl)
: IGMultiwayTree
   <numOfChildren, Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGMultiwayTree
// ---

// public members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IVGMultiwayTree <numOfChildren, Element, ElementOps>::
IVGMultiwayTree ()
: IGMultiwayTree
    <numOfChildren, Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation ())
{
}

// protected members

template <INumber numOfChildren, class Element, class ElementOps>
inline
IVGMultiwayTree <numOfChildren, Element, ElementOps>::
IVGMultiwayTree (PureImplementation& impl)
: IGMultiwayTree
    <numOfChildren, Element, ElementOps> (impl)
{
}

// ---
// IVMultiwayTree
// ---

// public members

template <INumber numOfChildren, class Element>
inline
IVMultiwayTree <numOfChildren, Element>::
IVMultiwayTree ()
: IVGMultiwayTree
   <numOfChildren, Element, IStdOps <Element> > ()
{
}

// protected members

template <INumber numOfChildren, class Element>
inline
IVMultiwayTree <numOfChildren, Element>::
IVMultiwayTree (PureImplementation& impl)
: IVGMultiwayTree
   <numOfChildren, Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IMWT_INL_
