// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 07 1.5.1.4 source/core/collect/ipqu.h, collection, ioc.v400, 000302 
#ifndef _IPQU_
#define _IPQU_

#include <ilanglvl.hpp>
#include <iapqu.h>
#include <iicksb.h>
#if ! defined (__INO_AVALON__)
#include <iivksb.h>
#endif
#include <iiksblst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGPriorityQueue;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGPriorityQueueCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGPriorityQueue
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGPriorityQueueCursor
                            (IGPriorityQueue
                              <
                               Element, Key, ElementOps> const&);

  IGPriorityQueueCursor <Element, Key, ElementOps>&
           operator=        (IGPriorityQueueCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGPriorityQueueCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueueCursor\
                            (IGPriorityQueue\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGPriorityQueueCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGPriorityQueueMngCursor :
  public IGPriorityQueueCursor <Element, Key, ElementOps>  {

  typedef  IGPriorityQueueCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGPriorityQueue
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGPriorityQueueMngCursor
                            (IGPriorityQueue
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGPriorityQueueMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueueMngCursor\
                            (IGPriorityQueue\
                              <\
                               Element, Key, ElementOps> const&),\
           IGPriorityQueueMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGPriorityQueue :
  public IAPriorityQueue <Element, Key> {
protected:

  typedef  IGPriorityQueue <Element, Key, ElementOps> Self;

  typedef  IKeySortedBagAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySortedBagImpl
            <PureImplementation> Implementation;
#endif

#define IGPriorityQueueTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedBagAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGPriorityQueueTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

  typedef  IGPriorityQueueCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGPriorityQueueMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGPriorityQueue
                            (INumber = 100);

           IGPriorityQueue
                            (IGPriorityQueue
                              <
                               Element, Key, ElementOps> const&);

  IGPriorityQueue <Element, Key, ElementOps>&
           operator=        (IGPriorityQueue
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGPriorityQueue
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGPriorityQueue
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGPriorityQueueCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGPriorityQueueMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueue\
                            (INumber),\
           operator=,\
           IGPriorityQueue\
                            (PureImplementation&),\
           ImplOf           (IGPriorityQueue\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IPriorityQueue :
  public IGPriorityQueue
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGPriorityQueue
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IPriorityQueue <Element, Key> Self;

#define IPriorityQueueTypedef(Element, Key)\
  typedef  IKeySortedBagAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IPriorityQueue
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IPriorityQueue
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IPriorityQueue\
                              (INumber),\
           IPriorityQueue\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGPriorityQueue :
  public IGPriorityQueue
          <Element, Key, ElementOps > {

  typedef  IVGPriorityQueue
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGPriorityQueue
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySortedBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySortedBagImpl
            <IVKeySortedBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGPriorityQueueTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedBagAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGPriorityQueueTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVGPriorityQueue
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGPriorityQueue
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGPriorityQueue\
                              (INumber),\
           IVGPriorityQueue\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVPriorityQueue :
  public IVGPriorityQueue
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGPriorityQueue
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVPriorityQueue <Element, Key> Self;

#define IVPriorityQueueTypedef(Element, Key)\
  typedef  IVKeySortedBagAsListOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVPriorityQueue
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVPriorityQueue
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVPriorityQueue\
                              (INumber),\
           IVPriorityQueue\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ipqu.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IPQU_
