// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 08 1.4.1.4 source/core/collect/ipqu.inl, collection, ioc.v400, 000302  
#ifndef _IPQU_INL_
#define _IPQU_INL_

// ---
// IGPriorityQueueCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueCursor <Element, Key, ElementOps>::
IGPriorityQueueCursor
  (IGPriorityQueue
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueueCursor <Element, Key, ElementOps>&
IGPriorityQueueCursor <Element, Key, ElementOps>::
operator=
  (IGPriorityQueueCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueCursor <Element, Key, ElementOps>::
IGPriorityQueueCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGPriorityQueueMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueMngCursor <Element, Key, ElementOps>::
IGPriorityQueueMngCursor
  (IGPriorityQueue
    <Element, Key, ElementOps> const& collection)
: IGPriorityQueueCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueueMngCursor <Element, Key, ElementOps>::
IGPriorityQueueMngCursor (ICursorImpl* impl)
: IGPriorityQueueCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGPriorityQueue
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueue <Element, Key, ElementOps>::
IGPriorityQueue (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueue <Element, Key, ElementOps>::
IGPriorityQueue
  (IGPriorityQueue
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGPriorityQueue <Element, Key, ElementOps>&
IGPriorityQueue <Element, Key, ElementOps>::
operator= (IGPriorityQueue
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGPriorityQueue <Element, Key, ElementOps>::
IGPriorityQueue (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGPriorityQueue
 <Element, Key, ElementOps>::Implementation&
IGPriorityQueue <Element, Key, ElementOps>::
ImplOf (IGPriorityQueue
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IPriorityQueue
// ---

// public members

template <class Element, class Key>
inline
IPriorityQueue <Element, Key>::
IPriorityQueue (INumber numberOfElements)
: IGPriorityQueue
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IPriorityQueue <Element, Key>::
IPriorityQueue (PureImplementation& impl)
: IGPriorityQueue
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGPriorityQueue
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueue <Element, Key, ElementOps>::
IVGPriorityQueue (INumber numberOfElements)
: IGPriorityQueue
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGPriorityQueue <Element, Key, ElementOps>::
IVGPriorityQueue (PureImplementation& impl)
: IGPriorityQueue
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVPriorityQueue
// ---

// public members

template <class Element, class Key>
inline
IVPriorityQueue <Element, Key>::
IVPriorityQueue (INumber numberOfElements)
: IVGPriorityQueue
   <Element, Key, IKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVPriorityQueue <Element, Key>::
IVPriorityQueue (PureImplementation& impl)
: IVGPriorityQueue
   <Element, Key, IKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _IPQU_INL_
