// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 15 1.5.1.4 source/core/collect/ipqutab.h, collection, ioc.v400, 000302 
#ifndef _IPQUTAB_
#define _IPQUTAB_

#include <ilanglvl.hpp>
#include <iapqu.h>
#include <iicksb.h>
#if ! defined (__INO_AVALON__)
#include <iivksb.h>
#endif
#include <iiksbtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGPriorityQueueAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGPriorityQueueAsTableCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGPriorityQueueAsTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGPriorityQueueAsTableCursor
                            (IGPriorityQueueAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGPriorityQueueAsTableCursor <Element, Key, ElementOps>&
           operator=        (IGPriorityQueueAsTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGPriorityQueueAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueueAsTableCursor\
                            (IGPriorityQueueAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGPriorityQueueAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGPriorityQueueAsTableMngCursor :
  public IGPriorityQueueAsTableCursor <Element, Key, ElementOps>  {

  typedef  IGPriorityQueueAsTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGPriorityQueueAsTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGPriorityQueueAsTableMngCursor
                            (IGPriorityQueueAsTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGPriorityQueueAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueueAsTableMngCursor\
                            (IGPriorityQueueAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGPriorityQueueAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGPriorityQueueAsTable :
  public IAPriorityQueue <Element, Key> {
protected:

  typedef  IGPriorityQueueAsTable <Element, Key, ElementOps> Self;

  typedef  IKeySortedBagAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICKeySortedBagImpl
            <PureImplementation> Implementation;
#endif

#define IGPriorityQueueAsTableTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGPriorityQueueAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

  typedef  IGPriorityQueueAsTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGPriorityQueueAsTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGPriorityQueueAsTable
                            (INumber = 100);

           IGPriorityQueueAsTable
                            (IGPriorityQueueAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGPriorityQueueAsTable <Element, Key, ElementOps>&
           operator=        (IGPriorityQueueAsTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGPriorityQueueAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGPriorityQueueAsTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGPriorityQueueAsTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGPriorityQueueAsTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGPriorityQueueAsTable\
                            (INumber),\
           operator=,\
           IGPriorityQueueAsTable\
                            (PureImplementation&),\
           ImplOf           (IGPriorityQueueAsTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class IPriorityQueueAsTable :
  public IGPriorityQueueAsTable
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IGPriorityQueueAsTable
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IPriorityQueueAsTable <Element, Key> Self;

#define IPriorityQueueAsTableTypedef(Element, Key)\
  typedef  IKeySortedBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IPriorityQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IPriorityQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IPriorityQueueAsTable\
                              (INumber),\
           IPriorityQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGPriorityQueueAsTable :
  public IGPriorityQueueAsTable
          <Element, Key, ElementOps > {

  typedef  IVGPriorityQueueAsTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGPriorityQueueAsTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVKeySortedBagImpl
            <PureImplementation> Implementation;
#else
  typedef  ICKeySortedBagImpl
            <IVKeySortedBagImpl
              <PureImplementation> > Implementation;
#endif

#define IVGPriorityQueueAsTableTypedef(Element, Key, ElementOps)\
  typedef  IKeySortedBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGPriorityQueueAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVGPriorityQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGPriorityQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGPriorityQueueAsTable\
                              (INumber),\
           IVGPriorityQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVPriorityQueueAsTable :
  public IVGPriorityQueueAsTable
          <Element, Key, IKCOps <Element, Key> > {

  typedef  IVGPriorityQueueAsTable
            <Element, Key,
             IKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVPriorityQueueAsTable <Element, Key> Self;

#define IVPriorityQueueAsTableTypedef(Element, Key)\
  typedef  IVKeySortedBagAsTableOps\
            <Element, Key,\
             IWKCOps <Element, Key, IKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVPriorityQueueAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVPriorityQueueAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVPriorityQueueAsTable\
                              (INumber),\
           IVPriorityQueueAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ipqutab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _IPQUTAB_
