// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 18 1.6.1.3 source/core/collect/iptr.h, collection, ioc.v400, 000302 
#ifndef _IPTR_
#define _IPTR_

#include <iglobals.h>

#if ! defined (__INO_OBJSTR__)
#include <iobjstr.h>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif


#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

enum IExplicitInit
{
    IINIT
};

template <class Element>
class IElemPointer
{
public:

           IElemPointer     ();

           IElemPointer     (Element*, IExplicitInit = IINIT);

  Element& operator*        () const;

  Element* operator->       () const;

           operator Element*
                            () const;
  friend
  Element& elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
		(IElemPointer <Element>&);

  friend
  Element const&
           elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
    		(IElemPointer <Element> const&) ;
protected:

private:

  Element* ivPtr;

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IElemPointer     (Element*, IExplicitInit),\
           operator*        () const,\
           operator->       () const,\
           operator Element*() const\
                        )
#endif

};

#if ! defined (__INO_OBJSTR__)

template <class Element>
void save (IObjectOutStream&, IElemPointer <Element> const&);

template <class Element>
void restore (IObjectInStream &, IElemPointer <Element>&);

#endif

template <class Element>
class IMngPointer {
public:

           IMngPointer      ();

           IMngPointer      (Element*, IExplicitInit);

           IMngPointer      (IMngPointer <Element> const&);

          ~IMngPointer      ();

  IMngPointer <Element>&
           operator=        (IMngPointer <Element> const&);

  Element& operator*        () const;

  Element* operator->       () const;

           operator Element* () const;

#if ! defined (__INO_OBJSTR__)

  IAddress getPtrRc () const;
           { return IMngPointer <Element>::getPtrRc ();
           }

  void     setPtrRc (IAddress a)
           { IMngPointer <Element>::setPtrRc (a);
           }


  void     setPtrRc (IAddress);

#endif

private:

  struct PtrRc
  { Element *ivPtr;

    unsigned int ivRc;

             PtrRc (Element *ptr)
             : ivPtr (ptr), ivRc (1)
             {
             }

            ~PtrRc ()
             { delete ivPtr; }

  } *ivPtrRc;

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMAttribute (ivPtrRc)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IMngPointer      (Element*, IExplicitInit),\
           operator=        (IMngPointer <Element> const&),\
           operator*        () const,\
           operator->       () const,\
           operator Element*() const\
)
#endif

};

#if ! defined (__INO_STREAMING__)
template <class Element>
class IStreamablePointer {
public:

           IStreamablePointer      ();

           IStreamablePointer      (Element*, IExplicitInit);

           IStreamablePointer      (IStreamablePointer <Element> const&);

          ~IStreamablePointer      ();

  IStreamablePointer <Element>&
           operator=        (IStreamablePointer <Element> const&);

  Element& operator*        () const;

  Element* operator->       () const;

           operator Element* () const;

private:

  struct PtrRc
  { Element *ivPtr;

    unsigned int ivRc;

             PtrRc (Element *ptr)
             : ivPtr (ptr), ivRc (1)
             {
             }

            ~PtrRc ()
             { delete ivPtr; }

  } *ivPtrRc;

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMAttribute (ivPtrRc)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IStreamablePointer      (Element*, IExplicitInit),\
           operator=        (IStreamablePointer <Element> const&),\
           operator*        () const,\
           operator->       () const,\
           operator Element*() const\
)
#endif

};

class IDataStream;

template <class Element>
void operator<<=( IStreamablePointer<Element>&, IDataStream&);

template <class Element>
void operator>>=( const IStreamablePointer<Element>&, IDataStream&);
#endif

#if ! defined (__INO_OBJSTR__)

template <class Element>
void save (IObjectOutStream&, IMngPointer <Element> const&);

template <class Element>
void restore (IObjectInStream&, IMngPointer <Element>&);

#endif

template <class Element>
class IMngElemPointer : protected IMngPointer <Element> {
public:

           IMngElemPointer  () ;

           IMngElemPointer  (Element*, IExplicitInit);

#if defined (ICLCC_COMPAT_PTR)
           IMngElemPointer  (Element const&);

           IMngElemPointer  (Element*);
#endif

  Element& operator*        () const;

  Element* operator->       () const;

           operator Element*
                            () const;
  friend
  Element& elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
		(IMngElemPointer <Element>&);

  friend
  Element const&
           elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
		(IMngElemPointer <Element> const&);

#if ! defined (__INO_OBJSTR__)

  IAddress getPtrRc () const
           { return IMngPointer <Element>::getPtrRc ();
           }

  void     setPtrRc (IAddress a)
           { IMngPointer <Element>::setPtrRc (a);
           }

#endif

protected:

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IMngElemPointer  (Element*, IExplicitInit),\
           operator*        () const,\
           operator->       () const,\
           operator Element*() const)
#endif

};


#if ! defined (__INO_OBJSTR__)

class IObjectOutStream;
class IObjectInStream;

template <class Element>
void save (IObjectOutStream&, IMngElemPointer <Element> const&);

template <class Element>
void restore (IObjectInStream&, IMngElemPointer <Element>&);

#endif

template <class Element>
class IAutoPointer {
public:

           IAutoPointer     ();

           IAutoPointer     (Element*, IExplicitInit);

           IAutoPointer     (IAutoPointer <Element> const&);

          ~IAutoPointer     ();

  void     operator=        (IAutoPointer <Element> const&);

           operator Element*
                            () const;

  Element& operator*        () const;

  Element* operator->       () const;

protected:

private:

  Element* ivPtr;

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IAutoPointer     (Element*, IExplicitInit),\
           operator=        (IAutoPointer <Element> const&),\
           operator Element*() const,\
           operator*        () const,\
           operator->       () const\
)
#endif

};

#if ! defined (__INO_OBJSTR__)

template <class Element>
void save (IObjectOutStream&, IAutoPointer <Element> const&);

template <class Element>
void restore (IObjectInStream&, IAutoPointer <Element>&);

#endif

template <class Element>
class IAutoElemPointer : protected IAutoPointer <Element> {
public:

           IAutoElemPointer ();

           IAutoElemPointer (Element*, IExplicitInit);

  Element& operator*        () const;

  Element* operator->       () const;

           operator Element*
                            () const;
  friend
  Element& elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
		(IAutoElemPointer <Element>&);

  friend
  Element const&
           elementForOps
#if __IBMCPP__ >= 400
		<Element>
#endif
		(IAutoElemPointer <Element> const&);

protected:

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder ( \
           IAutoElemPointer (Element*, IExplicitInit),\
           operator*        () const,\
           operator->       () const,\
           operator Element*() const)
#endif

};

#if ! defined (__INO_OBJSTR__)

template <class Element>
void save (IObjectOutStream&, IAutoElemPointer <Element> const&);

template <class Element>
void restore (IObjectInStream&, IAutoElemPointer <Element>&);

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#include <iptr.inl>

#if ! defined (__INO_OBJSTR__)
#if defined (_AIX) || defined (__MVS__) ||\
    defined (__OS2__)     || defined (__OS400__) ||\
    defined (__WINDOWS__)
#if ! defined (__TEMPINC__)
#include <iptr.c>
#endif
#endif
#endif

#endif // _IPTR_
