// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 21 1.4.1.4 source/core/collect/iqu.inl, collection, ioc.v400, 000302  
#ifndef _IQU_INL_
#define _IQU_INL_

// ---
// IGQueueCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGQueueCursor <Element, ElementOps>::
IGQueueCursor
  (IGQueue
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGQueueCursor <Element, ElementOps>&
IGQueueCursor <Element, ElementOps>::
operator=
  (IGQueueCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueCursor <Element, ElementOps>::
IGQueueCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGQueueMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueueMngCursor <Element, ElementOps>::
IGQueueMngCursor
  (IGQueue
    <Element, ElementOps> const& collection)
: IGQueueCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueMngCursor <Element, ElementOps>::
IGQueueMngCursor (ICursorImpl* impl)
: IGQueueCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGQueue
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueue <Element, ElementOps>::
IGQueue (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGQueue <Element, ElementOps>::
IGQueue
  (IGQueue
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGQueue <Element, ElementOps>&
IGQueue <Element, ElementOps>::
operator= (IGQueue
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueue <Element, ElementOps>::
IGQueue (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGQueue
 <Element, ElementOps>::Implementation&
IGQueue <Element, ElementOps>::
ImplOf (IGQueue
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IQueue
// ---

// public members

template <class Element>
inline
IQueue <Element>::
IQueue (INumber numberOfElements)
: IGQueue
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IQueue <Element>::
IQueue (PureImplementation& impl)
: IGQueue
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGQueue
// ---

// public members

template <class Element, class ElementOps>
inline
IVGQueue <Element, ElementOps>::
IVGQueue (INumber numberOfElements)
: IGQueue
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGQueue <Element, ElementOps>::
IVGQueue (PureImplementation& impl)
: IGQueue
    <Element, ElementOps> (impl)
{
}

// ---
// IVQueue
// ---

// public members

template <class Element>
inline
IVQueue <Element>::
IVQueue (INumber numberOfElements)
: IVGQueue
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVQueue <Element>::
IVQueue (PureImplementation& impl)
: IVGQueue
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IQU_INL_
