// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 28 1.4.1.4 source/core/collect/iqulst.inl, collection, ioc.v400, 000302  
#ifndef _IQULST_INL_
#define _IQULST_INL_

// ---
// IGQueueAsListCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGQueueAsListCursor <Element, ElementOps>::
IGQueueAsListCursor
  (IGQueueAsList
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGQueueAsListCursor <Element, ElementOps>&
IGQueueAsListCursor <Element, ElementOps>::
operator=
  (IGQueueAsListCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsListCursor <Element, ElementOps>::
IGQueueAsListCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGQueueAsListMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueueAsListMngCursor <Element, ElementOps>::
IGQueueAsListMngCursor
  (IGQueueAsList
    <Element, ElementOps> const& collection)
: IGQueueAsListCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsListMngCursor <Element, ElementOps>::
IGQueueAsListMngCursor (ICursorImpl* impl)
: IGQueueAsListCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGQueueAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IGQueueAsList <Element, ElementOps>::
IGQueueAsList (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGQueueAsList <Element, ElementOps>::
IGQueueAsList
  (IGQueueAsList
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGQueueAsList <Element, ElementOps>&
IGQueueAsList <Element, ElementOps>::
operator= (IGQueueAsList
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGQueueAsList <Element, ElementOps>::
IGQueueAsList (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGQueueAsList
 <Element, ElementOps>::Implementation&
IGQueueAsList <Element, ElementOps>::
ImplOf (IGQueueAsList
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IQueueAsList
// ---

// public members

template <class Element>
inline
IQueueAsList <Element>::
IQueueAsList (INumber numberOfElements)
: IGQueueAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IQueueAsList <Element>::
IQueueAsList (PureImplementation& impl)
: IGQueueAsList
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGQueueAsList
// ---

// public members

template <class Element, class ElementOps>
inline
IVGQueueAsList <Element, ElementOps>::
IVGQueueAsList (INumber numberOfElements)
: IGQueueAsList
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGQueueAsList <Element, ElementOps>::
IVGQueueAsList (PureImplementation& impl)
: IGQueueAsList
    <Element, ElementOps> (impl)
{
}

// ---
// IVQueueAsList
// ---

// public members

template <class Element>
inline
IVQueueAsList <Element>::
IVQueueAsList (INumber numberOfElements)
: IVGQueueAsList
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVQueueAsList <Element>::
IVQueueAsList (PureImplementation& impl)
: IVGQueueAsList
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _IQULST_INL_
