// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 02 1.4.1.4 source/core/collect/iseqtab.inl, collection, ioc.v400, 000302  
#ifndef _ISEQTAB_INL_
#define _ISEQTAB_INL_

// ---
// IGSequenceAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSequenceAsTableCursor <Element, ElementOps>::
IGSequenceAsTableCursor
  (IGSequenceAsTable
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSequenceAsTableCursor <Element, ElementOps>&
IGSequenceAsTableCursor <Element, ElementOps>::
operator=
  (IGSequenceAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSequenceAsTableCursor <Element, ElementOps>::
IGSequenceAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSequenceAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSequenceAsTableMngCursor <Element, ElementOps>::
IGSequenceAsTableMngCursor
  (IGSequenceAsTable
    <Element, ElementOps> const& collection)
: IGSequenceAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSequenceAsTableMngCursor <Element, ElementOps>::
IGSequenceAsTableMngCursor (ICursorImpl* impl)
: IGSequenceAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSequenceAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGSequenceAsTable <Element, ElementOps>::
IGSequenceAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSequenceAsTable <Element, ElementOps>::
IGSequenceAsTable
  (IGSequenceAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSequenceAsTable <Element, ElementOps>&
IGSequenceAsTable <Element, ElementOps>::
operator= (IGSequenceAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSequenceAsTable <Element, ElementOps>::
IGSequenceAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSequenceAsTable
 <Element, ElementOps>::Implementation&
IGSequenceAsTable <Element, ElementOps>::
ImplOf (IGSequenceAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISequenceAsTable
// ---

// public members

template <class Element>
inline
ISequenceAsTable <Element>::
ISequenceAsTable (INumber numberOfElements)
: IGSequenceAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISequenceAsTable <Element>::
ISequenceAsTable (PureImplementation& impl)
: IGSequenceAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSequenceAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSequenceAsTable <Element, ElementOps>::
IVGSequenceAsTable (INumber numberOfElements)
: IGSequenceAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSequenceAsTable <Element, ElementOps>::
IVGSequenceAsTable (PureImplementation& impl)
: IGSequenceAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVSequenceAsTable
// ---

// public members

template <class Element>
inline
IVSequenceAsTable <Element>::
IVSequenceAsTable (INumber numberOfElements)
: IVGSequenceAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSequenceAsTable <Element>::
IVSequenceAsTable (PureImplementation& impl)
: IVGSequenceAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _ISEQTAB_INL_
