// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 03 1.6.1.4 source/core/collect/iset.h, collection, ioc.v400, 000302 
#ifndef _ISET_
#define _ISET_

#include <ilanglvl.hpp>
#include <iaset.h>
#include <iicset.h>
#if ! defined (__INO_AVALON__)
#include <iivset.h>
#endif
#include <iisetavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSet;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSetCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGSet
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSetCursor
                            (IGSet
                              <
                               Element, ElementOps> const&);

  IGSetCursor <Element, ElementOps>&
           operator=        (IGSetCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSetCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetCursor\
                            (IGSet\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSetCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSetMngCursor :
  public IGSetCursor <Element, ElementOps>  {

  typedef  IGSetCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSet
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSetMngCursor
                            (IGSet
                              <
                               Element, ElementOps> const&);

protected:

           IGSetMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetMngCursor\
                            (IGSet\
                              <\
                               Element, ElementOps> const&),\
           IGSetMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSet :
  public IASet <Element> {
protected:

  typedef  IGSet <Element, ElementOps> Self;

  typedef  ISetAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSetTypedef(Element, ElementOps)\
  typedef  ISetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSetTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSetCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSetMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSet
                            (INumber = 100);

           IGSet
                            (IGSet
                              <
                               Element, ElementOps> const&);

  IGSet <Element, ElementOps>&
           operator=        (IGSet
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSet
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSet
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSetCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSetMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSet\
                            (INumber),\
           operator=,\
           IGSet\
                            (PureImplementation&),\
           ImplOf           (IGSet\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISet :
  public IGSet
          <Element, ICOps <Element> > {

  typedef  IGSet
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISet <Element> Self;

#define ISetTypedef(Element)\
  typedef  ISetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISet\
                              (INumber),\
           ISet\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSet :
  public IGSet
          <Element, ElementOps > {

  typedef  IVGSet
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSet
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSetImpl
            <IVSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSetTypedef(Element, ElementOps)\
  typedef  ISetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSetTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSet\
                              (INumber),\
           IVGSet\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSet :
  public IVGSet
          <Element, ICOps <Element> > {

  typedef  IVGSet
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSet <Element> Self;

#define IVSetTypedef(Element)\
  typedef  IVSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSet\
                              (INumber),\
           IVSet\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iset.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISET_
