// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 06 1.4.1.4 source/core/collect/isetavl.inl, collection, ioc.v400, 000302  
#ifndef _ISETAVL_INL_
#define _ISETAVL_INL_

// ---
// IGSetAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSetAsAvlTreeCursor <Element, ElementOps>::
IGSetAsAvlTreeCursor
  (IGSetAsAvlTree
    <Element, ElementOps> const& collection)
: IElementCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSetAsAvlTreeCursor <Element, ElementOps>&
IGSetAsAvlTreeCursor <Element, ElementOps>::
operator=
  (IGSetAsAvlTreeCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsAvlTreeCursor <Element, ElementOps>::
IGSetAsAvlTreeCursor (ICursorImpl* impl)
: IElementCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSetAsAvlTreeMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSetAsAvlTreeMngCursor <Element, ElementOps>::
IGSetAsAvlTreeMngCursor
  (IGSetAsAvlTree
    <Element, ElementOps> const& collection)
: IGSetAsAvlTreeCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsAvlTreeMngCursor <Element, ElementOps>::
IGSetAsAvlTreeMngCursor (ICursorImpl* impl)
: IGSetAsAvlTreeCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSetAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IGSetAsAvlTree <Element, ElementOps>::
IGSetAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSetAsAvlTree <Element, ElementOps>::
IGSetAsAvlTree
  (IGSetAsAvlTree
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSetAsAvlTree <Element, ElementOps>&
IGSetAsAvlTree <Element, ElementOps>::
operator= (IGSetAsAvlTree
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSetAsAvlTree <Element, ElementOps>::
IGSetAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSetAsAvlTree
 <Element, ElementOps>::Implementation&
IGSetAsAvlTree <Element, ElementOps>::
ImplOf (IGSetAsAvlTree
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISetAsAvlTree
// ---

// public members

template <class Element>
inline
ISetAsAvlTree <Element>::
ISetAsAvlTree (INumber numberOfElements)
: IGSetAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISetAsAvlTree <Element>::
ISetAsAvlTree (PureImplementation& impl)
: IGSetAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSetAsAvlTree
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSetAsAvlTree <Element, ElementOps>::
IVGSetAsAvlTree (INumber numberOfElements)
: IGSetAsAvlTree
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSetAsAvlTree <Element, ElementOps>::
IVGSetAsAvlTree (PureImplementation& impl)
: IGSetAsAvlTree
    <Element, ElementOps> (impl)
{
}

// ---
// IVSetAsAvlTree
// ---

// public members

template <class Element>
inline
IVSetAsAvlTree <Element>::
IVSetAsAvlTree (INumber numberOfElements)
: IVGSetAsAvlTree
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSetAsAvlTree <Element>::
IVSetAsAvlTree (PureImplementation& impl)
: IVGSetAsAvlTree
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISETAVL_INL_
