// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 07 1.5.1.4 source/core/collect/isetbst.h, collection, ioc.v400, 000302 
#ifndef _ISETBST_
#define _ISETBST_

#include <ilanglvl.hpp>
#include <iaset.h>
#include <iicset.h>
#if ! defined (__INO_AVALON__)
#include <iivset.h>
#endif
#include <iisetbst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSetAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSetAsBstTreeCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGSetAsBstTree
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSetAsBstTreeCursor
                            (IGSetAsBstTree
                              <
                               Element, ElementOps> const&);

  IGSetAsBstTreeCursor <Element, ElementOps>&
           operator=        (IGSetAsBstTreeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsBstTreeCursor\
                            (IGSetAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSetAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSetAsBstTreeMngCursor :
  public IGSetAsBstTreeCursor <Element, ElementOps>  {

  typedef  IGSetAsBstTreeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSetAsBstTree
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSetAsBstTreeMngCursor
                            (IGSetAsBstTree
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsBstTreeMngCursor\
                            (IGSetAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           IGSetAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSetAsBstTree :
  public IASet <Element> {
protected:

  typedef  IGSetAsBstTree <Element, ElementOps> Self;

  typedef  ISetAsBstTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSetAsBstTreeTypedef(Element, ElementOps)\
  typedef  ISetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSetAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSetAsBstTreeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSetAsBstTreeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSetAsBstTree
                            (INumber = 100);

           IGSetAsBstTree
                            (IGSetAsBstTree
                              <
                               Element, ElementOps> const&);

  IGSetAsBstTree <Element, ElementOps>&
           operator=        (IGSetAsBstTree
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSetAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSetAsBstTree
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSetAsBstTreeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSetAsBstTreeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsBstTree\
                            (INumber),\
           operator=,\
           IGSetAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGSetAsBstTree\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISetAsBstTree :
  public IGSetAsBstTree
          <Element, ICOps <Element> > {

  typedef  IGSetAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISetAsBstTree <Element> Self;

#define ISetAsBstTreeTypedef(Element)\
  typedef  ISetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISetAsBstTree\
                              (INumber),\
           ISetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSetAsBstTree :
  public IGSetAsBstTree
          <Element, ElementOps > {

  typedef  IVGSetAsBstTree
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSetAsBstTree
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSetImpl
            <IVSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSetAsBstTreeTypedef(Element, ElementOps)\
  typedef  ISetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSetAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSetAsBstTree\
                              (INumber),\
           IVGSetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSetAsBstTree :
  public IVGSetAsBstTree
          <Element, ICOps <Element> > {

  typedef  IVGSetAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSetAsBstTree <Element> Self;

#define IVSetAsBstTreeTypedef(Element)\
  typedef  IVSetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSetAsBstTree\
                              (INumber),\
           IVSetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <isetbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISETBST_
