// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 12 1.5.1.4 source/core/collect/isethsh.h, collection, ioc.v400, 000302 
#ifndef _ISETHSH_
#define _ISETHSH_

#include <ilanglvl.hpp>
#include <iaset.h>
#include <iicset.h>
#if ! defined (__INO_AVALON__)
#include <iivset.h>
#endif
#include <iisethsh.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSetAsHshTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSetAsHshTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGSetAsHshTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSetAsHshTableCursor
                            (IGSetAsHshTable
                              <
                               Element, ElementOps> const&);

  IGSetAsHshTableCursor <Element, ElementOps>&
           operator=        (IGSetAsHshTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsHshTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsHshTableCursor\
                            (IGSetAsHshTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSetAsHshTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSetAsHshTableMngCursor :
  public IGSetAsHshTableCursor <Element, ElementOps>  {

  typedef  IGSetAsHshTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSetAsHshTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSetAsHshTableMngCursor
                            (IGSetAsHshTable
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsHshTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsHshTableMngCursor\
                            (IGSetAsHshTable\
                              <\
                               Element, ElementOps> const&),\
           IGSetAsHshTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSetAsHshTable :
  public IASet <Element> {
protected:

  typedef  IGSetAsHshTable <Element, ElementOps> Self;

  typedef  ISetAsHshTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSetAsHshTableTypedef(Element, ElementOps)\
  typedef  ISetAsHshTableOps\
            <Element,\
             IWEHOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSetAsHshTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSetAsHshTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSetAsHshTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSetAsHshTable
                            (INumber = 100);

           IGSetAsHshTable
                            (IGSetAsHshTable
                              <
                               Element, ElementOps> const&);

  IGSetAsHshTable <Element, ElementOps>&
           operator=        (IGSetAsHshTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSetAsHshTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSetAsHshTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSetAsHshTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSetAsHshTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsHshTable\
                            (INumber),\
           operator=,\
           IGSetAsHshTable\
                            (PureImplementation&),\
           ImplOf           (IGSetAsHshTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISetAsHshTable :
  public IGSetAsHshTable
          <Element, IEHOps <Element> > {

  typedef  IGSetAsHshTable
            <Element,
             IEHOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISetAsHshTable <Element> Self;

#define ISetAsHshTableTypedef(Element)\
  typedef  ISetAsHshTableOps\
            <Element,\
             IWEHOps <Element, IEHOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISetAsHshTable\
                              (INumber),\
           ISetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSetAsHshTable :
  public IGSetAsHshTable
          <Element, ElementOps > {

  typedef  IVGSetAsHshTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSetAsHshTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSetImpl
            <IVSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSetAsHshTableTypedef(Element, ElementOps)\
  typedef  ISetAsHshTableOps\
            <Element,\
             IWEHOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSetAsHshTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSetAsHshTable\
                              (INumber),\
           IVGSetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSetAsHshTable :
  public IVGSetAsHshTable
          <Element, IEHOps <Element> > {

  typedef  IVGSetAsHshTable
            <Element,
             IEHOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSetAsHshTable <Element> Self;

#define IVSetAsHshTableTypedef(Element)\
  typedef  IVSetAsHshTableOps\
            <Element,\
             IWEHOps <Element, IEHOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSetAsHshTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSetAsHshTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSetAsHshTable\
                              (INumber),\
           IVSetAsHshTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <isethsh.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISETHSH_
