// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 19 1.5.1.4 source/core/collect/isettab.h, collection, ioc.v400, 000302 
#ifndef _ISETTAB_
#define _ISETTAB_

#include <ilanglvl.hpp>
#include <iaset.h>
#include <iicset.h>
#if ! defined (__INO_AVALON__)
#include <iivset.h>
#endif
#include <iisettab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSetAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSetAsTableCursor :
  public IElementCursor <Element> {

  typedef  IElementCursor <Element> Inherited;

  typedef TYPENAME IGSetAsTable
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSetAsTableCursor
                            (IGSetAsTable
                              <
                               Element, ElementOps> const&);

  IGSetAsTableCursor <Element, ElementOps>&
           operator=        (IGSetAsTableCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsTableCursor\
                            (IGSetAsTable\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSetAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSetAsTableMngCursor :
  public IGSetAsTableCursor <Element, ElementOps>  {

  typedef  IGSetAsTableCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSetAsTable
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSetAsTableMngCursor
                            (IGSetAsTable
                              <
                               Element, ElementOps> const&);

protected:

           IGSetAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsTableMngCursor\
                            (IGSetAsTable\
                              <\
                               Element, ElementOps> const&),\
           IGSetAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSetAsTable :
  public IASet <Element> {
protected:

  typedef  IGSetAsTable <Element, ElementOps> Self;

  typedef  ISetAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSetAsTableTypedef(Element, ElementOps)\
  typedef  ISetAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSetAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSetAsTableCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSetAsTableMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSetAsTable
                            (INumber = 100);

           IGSetAsTable
                            (IGSetAsTable
                              <
                               Element, ElementOps> const&);

  IGSetAsTable <Element, ElementOps>&
           operator=        (IGSetAsTable
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSetAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSetAsTable
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSetAsTableCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSetAsTableMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSetAsTable\
                            (INumber),\
           operator=,\
           IGSetAsTable\
                            (PureImplementation&),\
           ImplOf           (IGSetAsTable\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISetAsTable :
  public IGSetAsTable
          <Element, ICOps <Element> > {

  typedef  IGSetAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISetAsTable <Element> Self;

#define ISetAsTableTypedef(Element)\
  typedef  ISetAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISetAsTable\
                              (INumber),\
           ISetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSetAsTable :
  public IGSetAsTable
          <Element, ElementOps > {

  typedef  IVGSetAsTable
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSetAsTable
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSetImpl
            <IVSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSetAsTableTypedef(Element, ElementOps)\
  typedef  ISetAsTableOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSetAsTableTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSetAsTable\
                              (INumber),\
           IVGSetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSetAsTable :
  public IVGSetAsTable
          <Element, ICOps <Element> > {

  typedef  IVGSetAsTable
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSetAsTable <Element> Self;

#define IVSetAsTableTypedef(Element)\
  typedef  IVSetAsTableOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSetAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSetAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSetAsTable\
                              (INumber),\
           IVSetAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <isettab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISETTAB_
