// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 23 1.5.1.4 source/core/collect/ismavl.h, collection, ioc.v400, 000302 
#ifndef _ISMAVL_
#define _ISMAVL_

#include <ilanglvl.hpp>
#include <iasm.h>
#include <iicsm.h>
#if ! defined (__INO_AVALON__)
#include <iivsm.h>
#endif
#include <iismavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGSortedMapAsAvlTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGSortedMapAsAvlTreeCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedMapAsAvlTree
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedMapAsAvlTreeCursor
                            (IGSortedMapAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsAvlTreeCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsAvlTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsAvlTreeCursor\
                            (IGSortedMapAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGSortedMapAsAvlTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGSortedMapAsAvlTreeMngCursor :
  public IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>  {

  typedef  IGSortedMapAsAvlTreeCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGSortedMapAsAvlTree
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedMapAsAvlTreeMngCursor
                            (IGSortedMapAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsAvlTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsAvlTreeMngCursor\
                            (IGSortedMapAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&),\
           IGSortedMapAsAvlTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGSortedMapAsAvlTree :
  public IASortedMap <Element, Key> {
protected:

  typedef  IGSortedMapAsAvlTree <Element, Key, ElementOps> Self;

  typedef  ISortedMapAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedMapImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedMapAsAvlTreeTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsAvlTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGSortedMapAsAvlTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedMapAsAvlTreeCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedMapAsAvlTreeMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGSortedMapAsAvlTree
                            (INumber = 100);

           IGSortedMapAsAvlTree
                            (IGSortedMapAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsAvlTree <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedMapAsAvlTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedMapAsAvlTree
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedMapAsAvlTreeMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsAvlTree\
                            (INumber),\
           operator=,\
           IGSortedMapAsAvlTree\
                            (PureImplementation&),\
           ImplOf           (IGSortedMapAsAvlTree\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class ISortedMapAsAvlTree :
  public IGSortedMapAsAvlTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IGSortedMapAsAvlTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedMapAsAvlTree <Element, Key> Self;

#define ISortedMapAsAvlTreeTypedef(Element, Key)\
  typedef  ISortedMapAsAvlTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedMapAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedMapAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedMapAsAvlTree\
                              (INumber),\
           ISortedMapAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGSortedMapAsAvlTree :
  public IGSortedMapAsAvlTree
          <Element, Key, ElementOps > {

  typedef  IVGSortedMapAsAvlTree
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedMapAsAvlTree
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedMapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedMapImpl
            <IVSortedMapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedMapAsAvlTreeTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsAvlTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGSortedMapAsAvlTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedMapAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedMapAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedMapAsAvlTree\
                              (INumber),\
           IVGSortedMapAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVSortedMapAsAvlTree :
  public IVGSortedMapAsAvlTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IVGSortedMapAsAvlTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedMapAsAvlTree <Element, Key> Self;

#define IVSortedMapAsAvlTreeTypedef(Element, Key)\
  typedef  IVSortedMapAsAvlTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedMapAsAvlTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedMapAsAvlTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedMapAsAvlTree\
                              (INumber),\
           IVSortedMapAsAvlTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ismavl.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISMAVL_
