// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 24 1.4.1.4 source/core/collect/ismavl.inl, collection, ioc.v400, 000302  
#ifndef _ISMAVL_INL_
#define _ISMAVL_INL_

// ---
// IGSortedMapAsAvlTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>::
IGSortedMapAsAvlTreeCursor
  (IGSortedMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>&
IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>::
operator=
  (IGSortedMapAsAvlTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>::
IGSortedMapAsAvlTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedMapAsAvlTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGSortedMapAsAvlTreeMngCursor
  (IGSortedMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
: IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTreeMngCursor <Element, Key, ElementOps>::
IGSortedMapAsAvlTreeMngCursor (ICursorImpl* impl)
: IGSortedMapAsAvlTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedMapAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTree <Element, Key, ElementOps>::
IGSortedMapAsAvlTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTree <Element, Key, ElementOps>::
IGSortedMapAsAvlTree
  (IGSortedMapAsAvlTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsAvlTree <Element, Key, ElementOps>&
IGSortedMapAsAvlTree <Element, Key, ElementOps>::
operator= (IGSortedMapAsAvlTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsAvlTree <Element, Key, ElementOps>::
IGSortedMapAsAvlTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedMapAsAvlTree
 <Element, Key, ElementOps>::Implementation&
IGSortedMapAsAvlTree <Element, Key, ElementOps>::
ImplOf (IGSortedMapAsAvlTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedMapAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
ISortedMapAsAvlTree <Element, Key>::
ISortedMapAsAvlTree (INumber numberOfElements)
: IGSortedMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedMapAsAvlTree <Element, Key>::
ISortedMapAsAvlTree (PureImplementation& impl)
: IGSortedMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedMapAsAvlTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsAvlTree <Element, Key, ElementOps>::
IVGSortedMapAsAvlTree (INumber numberOfElements)
: IGSortedMapAsAvlTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsAvlTree <Element, Key, ElementOps>::
IVGSortedMapAsAvlTree (PureImplementation& impl)
: IGSortedMapAsAvlTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedMapAsAvlTree
// ---

// public members

template <class Element, class Key>
inline
IVSortedMapAsAvlTree <Element, Key>::
IVSortedMapAsAvlTree (INumber numberOfElements)
: IVGSortedMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedMapAsAvlTree <Element, Key>::
IVSortedMapAsAvlTree (PureImplementation& impl)
: IVGSortedMapAsAvlTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISMAVL_INL_
