// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 25 1.5.1.4 source/core/collect/ismbst.h, collection, ioc.v400, 000302 
#ifndef _ISMBST_
#define _ISMBST_

#include <ilanglvl.hpp>
#include <iasm.h>
#include <iicsm.h>
#if ! defined (__INO_AVALON__)
#include <iivsm.h>
#endif
#include <iismbst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGSortedMapAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGSortedMapAsBstTreeCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedMapAsBstTree
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedMapAsBstTreeCursor
                            (IGSortedMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsBstTreeCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsBstTreeCursor\
                            (IGSortedMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGSortedMapAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGSortedMapAsBstTreeMngCursor :
  public IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>  {

  typedef  IGSortedMapAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGSortedMapAsBstTree
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedMapAsBstTreeMngCursor
                            (IGSortedMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsBstTreeMngCursor\
                            (IGSortedMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&),\
           IGSortedMapAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGSortedMapAsBstTree :
  public IASortedMap <Element, Key> {
protected:

  typedef  IGSortedMapAsBstTree <Element, Key, ElementOps> Self;

  typedef  ISortedMapAsBstTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedMapImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedMapAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGSortedMapAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedMapAsBstTreeCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedMapAsBstTreeMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGSortedMapAsBstTree
                            (INumber = 100);

           IGSortedMapAsBstTree
                            (IGSortedMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsBstTree <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedMapAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedMapAsBstTree
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedMapAsBstTreeMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsBstTree\
                            (INumber),\
           operator=,\
           IGSortedMapAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGSortedMapAsBstTree\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class ISortedMapAsBstTree :
  public IGSortedMapAsBstTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IGSortedMapAsBstTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedMapAsBstTree <Element, Key> Self;

#define ISortedMapAsBstTreeTypedef(Element, Key)\
  typedef  ISortedMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedMapAsBstTree\
                              (INumber),\
           ISortedMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGSortedMapAsBstTree :
  public IGSortedMapAsBstTree
          <Element, Key, ElementOps > {

  typedef  IVGSortedMapAsBstTree
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedMapAsBstTree
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedMapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedMapImpl
            <IVSortedMapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedMapAsBstTreeTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGSortedMapAsBstTreeTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedMapAsBstTree\
                              (INumber),\
           IVGSortedMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVSortedMapAsBstTree :
  public IVGSortedMapAsBstTree
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IVGSortedMapAsBstTree
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedMapAsBstTree <Element, Key> Self;

#define IVSortedMapAsBstTreeTypedef(Element, Key)\
  typedef  IVSortedMapAsBstTreeOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedMapAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedMapAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedMapAsBstTree\
                              (INumber),\
           IVSortedMapAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ismbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISMBST_
