// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 26 1.4.1.4 source/core/collect/ismbst.inl, collection, ioc.v400, 000302  
#ifndef _ISMBST_INL_
#define _ISMBST_INL_

// ---
// IGSortedMapAsBstTreeCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>::
IGSortedMapAsBstTreeCursor
  (IGSortedMapAsBstTree
    <Element, Key, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>&
IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>::
operator=
  (IGSortedMapAsBstTreeCursor
    <Element, Key, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>::
IGSortedMapAsBstTreeCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedMapAsBstTreeMngCursor
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTreeMngCursor <Element, Key, ElementOps>::
IGSortedMapAsBstTreeMngCursor
  (IGSortedMapAsBstTree
    <Element, Key, ElementOps> const& collection)
: IGSortedMapAsBstTreeCursor <Element, Key, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTreeMngCursor <Element, Key, ElementOps>::
IGSortedMapAsBstTreeMngCursor (ICursorImpl* impl)
: IGSortedMapAsBstTreeCursor <Element, Key, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedMapAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTree <Element, Key, ElementOps>::
IGSortedMapAsBstTree (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTree <Element, Key, ElementOps>::
IGSortedMapAsBstTree
  (IGSortedMapAsBstTree
    <Element, Key, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class Key, class ElementOps>
inline IGSortedMapAsBstTree <Element, Key, ElementOps>&
IGSortedMapAsBstTree <Element, Key, ElementOps>::
operator= (IGSortedMapAsBstTree
            <Element, Key, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IGSortedMapAsBstTree <Element, Key, ElementOps>::
IGSortedMapAsBstTree (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class Key, class ElementOps>
inline TYPENAME IGSortedMapAsBstTree
 <Element, Key, ElementOps>::Implementation&
IGSortedMapAsBstTree <Element, Key, ElementOps>::
ImplOf (IGSortedMapAsBstTree
         <Element, Key, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedMapAsBstTree
// ---

// public members

template <class Element, class Key>
inline
ISortedMapAsBstTree <Element, Key>::
ISortedMapAsBstTree (INumber numberOfElements)
: IGSortedMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
ISortedMapAsBstTree <Element, Key>::
ISortedMapAsBstTree (PureImplementation& impl)
: IGSortedMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedMapAsBstTree
// ---

// public members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsBstTree <Element, Key, ElementOps>::
IVGSortedMapAsBstTree (INumber numberOfElements)
: IGSortedMapAsBstTree
    <Element, Key, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class Key, class ElementOps>
inline
IVGSortedMapAsBstTree <Element, Key, ElementOps>::
IVGSortedMapAsBstTree (PureImplementation& impl)
: IGSortedMapAsBstTree
    <Element, Key, ElementOps> (impl)
{
}

// ---
// IVSortedMapAsBstTree
// ---

// public members

template <class Element, class Key>
inline
IVSortedMapAsBstTree <Element, Key>::
IVSortedMapAsBstTree (INumber numberOfElements)
: IVGSortedMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (numberOfElements)
{
}

// protected members

template <class Element, class Key>
inline
IVSortedMapAsBstTree <Element, Key>::
IVSortedMapAsBstTree (PureImplementation& impl)
: IVGSortedMapAsBstTree
   <Element, Key, IEKCOps <Element, Key> > (impl)
{
}
#endif

#endif // _ISMBST_INL_
