// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 34 1.5.1.4 source/core/collect/ismtab.h, collection, ioc.v400, 000302 
#ifndef _ISMTAB_
#define _ISMTAB_

#include <ilanglvl.hpp>
#include <iasm.h>
#include <iicsm.h>
#if ! defined (__INO_AVALON__)
#include <iivsm.h>
#endif
#include <iismtab.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class Key, class ElementOps>
class IGSortedMapAsTable;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class Key, class ElementOps>
class IGSortedMapAsTableCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedMapAsTable
            <Element, Key, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedMapAsTableCursor
                            (IGSortedMapAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsTableCursor <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsTableCursor
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsTableCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsTableCursor\
                            (IGSortedMapAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           operator=,\
           IGSortedMapAsTableCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class Key, class ElementOps>
class IGSortedMapAsTableMngCursor :
  public IGSortedMapAsTableCursor <Element, Key, ElementOps>  {

  typedef  IGSortedMapAsTableCursor
            <Element, Key, ElementOps> Cursor;

  typedef TYPENAME IGSortedMapAsTable
            <Element, Key, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedMapAsTableMngCursor
                            (IGSortedMapAsTable
                              <
                               Element, Key, ElementOps> const&);

protected:

           IGSortedMapAsTableMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsTableMngCursor\
                            (IGSortedMapAsTable\
                              <\
                               Element, Key, ElementOps> const&),\
           IGSortedMapAsTableMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class Key, class ElementOps>
class IGSortedMapAsTable :
  public IASortedMap <Element, Key> {
protected:

  typedef  IGSortedMapAsTable <Element, Key, ElementOps> Self;

  typedef  ISortedMapAsTableImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedMapImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedMapAsTableTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IGSortedMapAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedMapAsTableCursor
            <Element, Key, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedMapAsTableMngCursor
            <Element, Key, ElementOps> MngCursor;
#endif

           IGSortedMapAsTable
                            (INumber = 100);

           IGSortedMapAsTable
                            (IGSortedMapAsTable
                              <
                               Element, Key, ElementOps> const&);

  IGSortedMapAsTable <Element, Key, ElementOps>&
           operator=        (IGSortedMapAsTable
                              <
                               Element, Key, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedMapAsTable
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedMapAsTable
                              <
                               Element, Key, ElementOps> const&);

private:

  friend
  class    IGSortedMapAsTableCursor <Element, Key, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedMapAsTableMngCursor
            <Element, Key, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedMapAsTable\
                            (INumber),\
           operator=,\
           IGSortedMapAsTable\
                            (PureImplementation&),\
           ImplOf           (IGSortedMapAsTable\
                              <\
                               Element, Key, ElementOps> const&)\
                        )
#endif

};

template <class Element, class Key>
class ISortedMapAsTable :
  public IGSortedMapAsTable
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IGSortedMapAsTable
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedMapAsTable <Element, Key> Self;

#define ISortedMapAsTableTypedef(Element, Key)\
  typedef  ISortedMapAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedMapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedMapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedMapAsTable\
                              (INumber),\
           ISortedMapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class Key, class ElementOps>
class IVGSortedMapAsTable :
  public IGSortedMapAsTable
          <Element, Key, ElementOps > {

  typedef  IVGSortedMapAsTable
            <Element, Key, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedMapAsTable
            <Element, Key, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedMapImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedMapImpl
            <IVSortedMapImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedMapAsTableTypedef(Element, Key, ElementOps)\
  typedef  ISortedMapAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, ElementOps>,\
             Implementation> Instantiation

           IVGSortedMapAsTableTypedef (Element, Key, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedMapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedMapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedMapAsTable\
                              (INumber),\
           IVGSortedMapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element, class Key>
class IVSortedMapAsTable :
  public IVGSortedMapAsTable
          <Element, Key, IEKCOps <Element, Key> > {

  typedef  IVGSortedMapAsTable
            <Element, Key,
             IEKCOps <Element, Key> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedMapAsTable <Element, Key> Self;

#define IVSortedMapAsTableTypedef(Element, Key)\
  typedef  IVSortedMapAsTableOps\
            <Element, Key,\
             IWEKCOps <Element, Key, IEKCOps <Element, Key> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedMapAsTable
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedMapAsTable
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedMapAsTable\
                              (INumber),\
           IVSortedMapAsTable\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <ismtab.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISMTAB_
