// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 51 1.5.1.4 source/core/collect/iss.h, collection, ioc.v400, 000302 
#ifndef _ISS_
#define _ISS_

#include <ilanglvl.hpp>
#include <iass.h>
#include <iicss.h>
#if ! defined (__INO_AVALON__)
#include <iivss.h>
#endif
#include <iissavl.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSortedSet;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSortedSetCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedSet
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedSetCursor
                            (IGSortedSet
                              <
                               Element, ElementOps> const&);

  IGSortedSetCursor <Element, ElementOps>&
           operator=        (IGSortedSetCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetCursor\
                            (IGSortedSet\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSortedSetCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSortedSetMngCursor :
  public IGSortedSetCursor <Element, ElementOps>  {

  typedef  IGSortedSetCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSortedSet
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedSetMngCursor
                            (IGSortedSet
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetMngCursor\
                            (IGSortedSet\
                              <\
                               Element, ElementOps> const&),\
           IGSortedSetMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSortedSet :
  public IASortedSet <Element> {
protected:

  typedef  IGSortedSet <Element, ElementOps> Self;

  typedef  ISortedSetAsAvlTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedSetTypedef(Element, ElementOps)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSortedSetTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedSetCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedSetMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSortedSet
                            (INumber = 100);

           IGSortedSet
                            (IGSortedSet
                              <
                               Element, ElementOps> const&);

  IGSortedSet <Element, ElementOps>&
           operator=        (IGSortedSet
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedSet
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedSet
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSortedSetCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedSetMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSet\
                            (INumber),\
           operator=,\
           IGSortedSet\
                            (PureImplementation&),\
           ImplOf           (IGSortedSet\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISortedSet :
  public IGSortedSet
          <Element, ICOps <Element> > {

  typedef  IGSortedSet
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedSet <Element> Self;

#define ISortedSetTypedef(Element)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedSet\
                              (INumber),\
           ISortedSet\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSortedSet :
  public IGSortedSet
          <Element, ElementOps > {

  typedef  IVGSortedSet
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedSet
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedSetImpl
            <IVSortedSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedSetTypedef(Element, ElementOps)\
  typedef  ISortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSortedSetTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedSet\
                              (INumber),\
           IVGSortedSet\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSortedSet :
  public IVGSortedSet
          <Element, ICOps <Element> > {

  typedef  IVGSortedSet
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedSet <Element> Self;

#define IVSortedSetTypedef(Element)\
  typedef  IVSortedSetAsAvlTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedSet
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedSet
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedSet\
                              (INumber),\
           IVSortedSet\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <iss.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISS_
