// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 52 1.4.1.4 source/core/collect/iss.inl, collection, ioc.v400, 000302  
#ifndef _ISS_INL_
#define _ISS_INL_

// ---
// IGSortedSetCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGSortedSetCursor <Element, ElementOps>::
IGSortedSetCursor
  (IGSortedSet
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGSortedSetCursor <Element, ElementOps>&
IGSortedSetCursor <Element, ElementOps>::
operator=
  (IGSortedSetCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetCursor <Element, ElementOps>::
IGSortedSetCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGSortedSetMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSetMngCursor <Element, ElementOps>::
IGSortedSetMngCursor
  (IGSortedSet
    <Element, ElementOps> const& collection)
: IGSortedSetCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSetMngCursor <Element, ElementOps>::
IGSortedSetMngCursor (ICursorImpl* impl)
: IGSortedSetCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGSortedSet
// ---

// public members

template <class Element, class ElementOps>
inline
IGSortedSet <Element, ElementOps>::
IGSortedSet (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGSortedSet <Element, ElementOps>::
IGSortedSet
  (IGSortedSet
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGSortedSet <Element, ElementOps>&
IGSortedSet <Element, ElementOps>::
operator= (IGSortedSet
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGSortedSet <Element, ElementOps>::
IGSortedSet (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGSortedSet
 <Element, ElementOps>::Implementation&
IGSortedSet <Element, ElementOps>::
ImplOf (IGSortedSet
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// ISortedSet
// ---

// public members

template <class Element>
inline
ISortedSet <Element>::
ISortedSet (INumber numberOfElements)
: IGSortedSet
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
ISortedSet <Element>::
ISortedSet (PureImplementation& impl)
: IGSortedSet
   <Element, ICOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGSortedSet
// ---

// public members

template <class Element, class ElementOps>
inline
IVGSortedSet <Element, ElementOps>::
IVGSortedSet (INumber numberOfElements)
: IGSortedSet
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGSortedSet <Element, ElementOps>::
IVGSortedSet (PureImplementation& impl)
: IGSortedSet
    <Element, ElementOps> (impl)
{
}

// ---
// IVSortedSet
// ---

// public members

template <class Element>
inline
IVSortedSet <Element>::
IVSortedSet (INumber numberOfElements)
: IVGSortedSet
   <Element, ICOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVSortedSet <Element>::
IVSortedSet (PureImplementation& impl)
: IVGSortedSet
   <Element, ICOps <Element> > (impl)
{
}
#endif

#endif // _ISS_INL_
