// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 55 1.5.1.4 source/core/collect/issbst.h, collection, ioc.v400, 000302 
#ifndef _ISSBST_
#define _ISSBST_

#include <ilanglvl.hpp>
#include <iass.h>
#include <iicss.h>
#if ! defined (__INO_AVALON__)
#include <iivss.h>
#endif
#include <iissbst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGSortedSetAsBstTree;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGSortedSetAsBstTreeCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGSortedSetAsBstTree
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGSortedSetAsBstTreeCursor
                            (IGSortedSetAsBstTree
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsBstTreeCursor <Element, ElementOps>&
           operator=        (IGSortedSetAsBstTreeCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsBstTreeCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsBstTreeCursor\
                            (IGSortedSetAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGSortedSetAsBstTreeCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGSortedSetAsBstTreeMngCursor :
  public IGSortedSetAsBstTreeCursor <Element, ElementOps>  {

  typedef  IGSortedSetAsBstTreeCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGSortedSetAsBstTree
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGSortedSetAsBstTreeMngCursor
                            (IGSortedSetAsBstTree
                              <
                               Element, ElementOps> const&);

protected:

           IGSortedSetAsBstTreeMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsBstTreeMngCursor\
                            (IGSortedSetAsBstTree\
                              <\
                               Element, ElementOps> const&),\
           IGSortedSetAsBstTreeMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGSortedSetAsBstTree :
  public IASortedSet <Element> {
protected:

  typedef  IGSortedSetAsBstTree <Element, ElementOps> Self;

  typedef  ISortedSetAsBstTreeImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSortedSetImpl
            <PureImplementation> Implementation;
#endif

#define IGSortedSetAsBstTreeTypedef(Element, ElementOps)\
  typedef  ISortedSetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGSortedSetAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

  typedef  IGSortedSetAsBstTreeCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGSortedSetAsBstTreeMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGSortedSetAsBstTree
                            (INumber = 100);

           IGSortedSetAsBstTree
                            (IGSortedSetAsBstTree
                              <
                               Element, ElementOps> const&);

  IGSortedSetAsBstTree <Element, ElementOps>&
           operator=        (IGSortedSetAsBstTree
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGSortedSetAsBstTree
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGSortedSetAsBstTree
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGSortedSetAsBstTreeCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGSortedSetAsBstTreeMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGSortedSetAsBstTree\
                            (INumber),\
           operator=,\
           IGSortedSetAsBstTree\
                            (PureImplementation&),\
           ImplOf           (IGSortedSetAsBstTree\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class ISortedSetAsBstTree :
  public IGSortedSetAsBstTree
          <Element, ICOps <Element> > {

  typedef  IGSortedSetAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  ISortedSetAsBstTree <Element> Self;

#define ISortedSetAsBstTreeTypedef(Element)\
  typedef  ISortedSetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           ISortedSetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           ISortedSetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           ISortedSetAsBstTree\
                              (INumber),\
           ISortedSetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGSortedSetAsBstTree :
  public IGSortedSetAsBstTree
          <Element, ElementOps > {

  typedef  IVGSortedSetAsBstTree
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGSortedSetAsBstTree
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSortedSetImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSortedSetImpl
            <IVSortedSetImpl
              <PureImplementation> > Implementation;
#endif

#define IVGSortedSetAsBstTreeTypedef(Element, ElementOps)\
  typedef  ISortedSetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGSortedSetAsBstTreeTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVGSortedSetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGSortedSetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGSortedSetAsBstTree\
                              (INumber),\
           IVGSortedSetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVSortedSetAsBstTree :
  public IVGSortedSetAsBstTree
          <Element, ICOps <Element> > {

  typedef  IVGSortedSetAsBstTree
            <Element,
             ICOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVSortedSetAsBstTree <Element> Self;

#define IVSortedSetAsBstTreeTypedef(Element)\
  typedef  IVSortedSetAsBstTreeOps\
            <Element,\
             IWCOps <Element, ICOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef ICollectionGuard<Element> Guard;
#endif

           IVSortedSetAsBstTree
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVSortedSetAsBstTree
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVSortedSetAsBstTree\
                              (INumber),\
           IVSortedSetAsBstTree\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <issbst.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISSBST_
