// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 69 1.5.1.4 source/core/collect/istk.h, collection, ioc.v400, 000302 
#ifndef _ISTK_
#define _ISTK_

#include <ilanglvl.hpp>
#include <iastk.h>
#include <iicseq.h>
#if ! defined (__INO_AVALON__)
#include <iivseq.h>
#endif
#include <iiseqlst.h>

#if defined (__OSTORE__)
#include <ostore/ostore.hh>
#include <ostore/manschem.hh>
#endif

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#endif

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_IBM__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

template <class Element, class ElementOps>
class IGStack;

#pragma pack(push,4)
#pragma enum(4)

template <class Element, class ElementOps>
class IGStackCursor :
  public IOrderedCursor <Element> {

  typedef  IOrderedCursor <Element> Inherited;

  typedef TYPENAME IGStack
            <Element, ElementOps>::
              Implementation::Cursor Implementation;

public:

           IGStackCursor
                            (IGStack
                              <
                               Element, ElementOps> const&);

  IGStackCursor <Element, ElementOps>&
           operator=        (IGStackCursor
                              <
                               Element, ElementOps> const&);

protected:

           IGStackCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGStackCursor\
                            (IGStack\
                              <\
                               Element, ElementOps> const&),\
           operator=,\
           IGStackCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#if ! defined (__INO_MNGCURSOR__)
template <class Element, class ElementOps>
class IGStackMngCursor :
  public IGStackCursor <Element, ElementOps>  {

  typedef  IGStackCursor
            <Element, ElementOps> Cursor;

  typedef TYPENAME IGStack
            <Element, ElementOps>::
              Implementation::MngCursor Implementation;

public:

           IGStackMngCursor
                            (IGStack
                              <
                               Element, ElementOps> const&);

protected:

           IGStackMngCursor
                            (ICursorImpl*);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGStackMngCursor\
                            (IGStack\
                              <\
                               Element, ElementOps> const&),\
           IGStackMngCursor\
                            (ICursorImpl*)\
                        )
#endif

};

#endif
template <class Element, class ElementOps>
class IGStack :
  public IAStack <Element> {
protected:

  typedef  IGStack <Element, ElementOps> Self;

  typedef  ISequenceAsListImpl PureImplementation;

#if defined (INO_CHECKS)
  typedef  PureImplementation Implementation;
#else
  typedef  ICSequenceImpl
            <PureImplementation> Implementation;
#endif

#define IGStackTypedef(Element, ElementOps)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IGStackTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

  typedef  IGStackCursor
            <Element, ElementOps> Cursor;

#if ! defined (__INO_MNGCURSOR__)
  typedef  IGStackMngCursor
            <Element, ElementOps> MngCursor;
#endif

           IGStack
                            (INumber = 100);

           IGStack
                            (IGStack
                              <
                               Element, ElementOps> const&);

  IGStack <Element, ElementOps>&
           operator=        (IGStack
                              <
                               Element, ElementOps> const&);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Instantiation::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IGStack
                            (PureImplementation&);

  static
  Implementation&
           ImplOf           (IGStack
                              <
                               Element, ElementOps> const&);

private:

  friend
  class    IGStackCursor <Element, ElementOps>;

#if ! defined (__INO_MNGCURSOR__)
  friend
  class    IGStackMngCursor
            <Element, ElementOps>;
#endif

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine ()
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IGStack\
                            (INumber),\
           operator=,\
           IGStack\
                            (PureImplementation&),\
           ImplOf           (IGStack\
                              <\
                               Element, ElementOps> const&)\
                        )
#endif

};

template <class Element>
class IStack :
  public IGStack
          <Element, IStdOps <Element> > {

  typedef  IGStack
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IStack <Element> Self;

#define IStackTypedef(Element)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IStack
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IStack
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IStack\
                              (INumber),\
           IStack\
                              (PureImplementation&)\
                        )
#endif

};

#if ! defined (__INO_AVALON__)
template <class Element, class ElementOps>
class IVGStack :
  public IGStack
          <Element, ElementOps > {

  typedef  IVGStack
            <Element, ElementOps> Inherited;

protected:

  typedef TYPENAME
           IGStack
            <Element, ElementOps>::PureImplementation PureImplementation;

#if defined (INO_CHECKS)
  typedef  IVSequenceImpl
            <PureImplementation> Implementation;
#else
  typedef  ICSequenceImpl
            <IVSequenceImpl
              <PureImplementation> > Implementation;
#endif

#define IVGStackTypedef(Element, ElementOps)\
  typedef  ISequenceAsListOps\
            <Element,\
             IWStdOps <Element, ElementOps>,\
             Implementation> Instantiation

           IVGStackTypedef (Element, ElementOps);

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVGStack
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVGStack
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVGStack\
                              (INumber),\
           IVGStack\
                              (PureImplementation&)\
                        )
#endif

};

template <class Element>
class IVStack :
  public IVGStack
          <Element, IStdOps <Element> > {

  typedef  IVGStack
            <Element,
             IStdOps <Element> > Inherited;

protected:

  typedef TYPENAME Inherited::PureImplementation PureImplementation;

  typedef  IVStack <Element> Self;

#define IVStackTypedef(Element)\
  typedef  IVSequenceAsListOps\
            <Element,\
             IWStdOps <Element, IStdOps <Element> >,\
             Implementation> Instantiation

public:

#if ! defined(__INO_L1_LOCKING__)
  typedef IRestrictedAccessCollectionGuard<Element> Guard;
#endif

           IVStack
                            (INumber = 100);

#if defined (__OSTORE__)
  static
  os_typespec*
           get_os_typespec  ()
           { return 0; }

  static
  void     os_mark_schema_type ()
           {
             Inherited::os_mark_schema_type ();
#if ! defined (_AIX)
             OS_MARK_SCHEMA_TYPESPEC((Self));
#else
             { Self* _OSMT_ = 0; }
#endif
           }
#endif

protected:

           IVStack
                              (PureImplementation&);

private:

#if defined (__SOM_ENABLED__) && defined (__RRBC_IBM__)
#pragma SOMDefine (*)
#pragma SOMClassVersion (*, 5, 0)
#pragma SOMReleaseOrder (\
           IVStack\
                              (INumber),\
           IVStack\
                              (PureImplementation&)\
                        )
#endif

};

#endif

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#if defined (__SOM_ENABLED__) 
#pragma SOMAsDefault (pop)
#endif

#include <istk.inl>

#if defined (__OS2__) || defined (__OS400__) || defined (__WINDOWS__)
#pragma info (restore)
#endif

#endif // _ISTK_
