// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 77 1.4.1.4 source/core/collect/istktab.inl, collection, ioc.v400, 000302  
#ifndef _ISTKTAB_INL_
#define _ISTKTAB_INL_

// ---
// IGStackAsTableCursor
// ---

#include <ilanglvl.hpp>

// public members

template <class Element, class ElementOps>
inline
IGStackAsTableCursor <Element, ElementOps>::
IGStackAsTableCursor
  (IGStackAsTable
    <Element, ElementOps> const& collection)
: IOrderedCursor <Element>
    (collection.ivImpl->CreateCursor ())
{
}

template <class Element, class ElementOps>
inline IGStackAsTableCursor <Element, ElementOps>&
IGStackAsTableCursor <Element, ElementOps>::
operator=
  (IGStackAsTableCursor
    <Element, ElementOps> const& cursor)
{ Inherited::operator= (cursor);
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsTableCursor <Element, ElementOps>::
IGStackAsTableCursor (ICursorImpl* impl)
: IOrderedCursor <Element> (impl)
{
}

#if ! defined (__INO_MNGCURSOR__)
// ---
// IGStackAsTableMngCursor
// ---

// public members

template <class Element, class ElementOps>
inline
IGStackAsTableMngCursor <Element, ElementOps>::
IGStackAsTableMngCursor
  (IGStackAsTable
    <Element, ElementOps> const& collection)
: IGStackAsTableCursor <Element, ElementOps>
    (collection.ivImpl->CreateMngCursor ())
{
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsTableMngCursor <Element, ElementOps>::
IGStackAsTableMngCursor (ICursorImpl* impl)
: IGStackAsTableCursor <Element, ElementOps> (impl)
{
}

#endif
// ---
// IGStackAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IGStackAsTable <Element, ElementOps>::
IGStackAsTable (INumber numberOfElements)
{ this->SetImpl ((Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements));
}

template <class Element, class ElementOps>
inline
IGStackAsTable <Element, ElementOps>::
IGStackAsTable
  (IGStackAsTable
    <Element, ElementOps> const& collection)
{ this->CloneImplOf (collection);
}

template <class Element, class ElementOps>
inline IGStackAsTable <Element, ElementOps>&
IGStackAsTable <Element, ElementOps>::
operator= (IGStackAsTable
            <Element, ElementOps> const& collection)
{ this->ImplOf (*this).Copy (this->ImplOf (collection));
  return *this;
}

// protected members

template <class Element, class ElementOps>
inline
IGStackAsTable <Element, ElementOps>::
IGStackAsTable (PureImplementation& impl)
{ this->SetImpl (&impl);
}

template <class Element, class ElementOps>
inline TYPENAME IGStackAsTable
 <Element, ElementOps>::Implementation&
IGStackAsTable <Element, ElementOps>::
ImplOf (IGStackAsTable
         <Element, ElementOps> const& collection)
{ return *(Implementation*)collection.ivImpl;
}

// ---
// IStackAsTable
// ---

// public members

template <class Element>
inline
IStackAsTable <Element>::
IStackAsTable (INumber numberOfElements)
: IGStackAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IStackAsTable <Element>::
IStackAsTable (PureImplementation& impl)
: IGStackAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#if ! defined (__INO_AVALON__)

// ---
// IVGStackAsTable
// ---

// public members

template <class Element, class ElementOps>
inline
IVGStackAsTable <Element, ElementOps>::
IVGStackAsTable (INumber numberOfElements)
: IGStackAsTable
    <Element, ElementOps> (*(Implementation*) *new
#if defined (__OSTORE__)
             (os_segment::of (this),
              Instantiation::get_os_typespec ())
#endif
                Instantiation (numberOfElements))
{
}

// protected members

template <class Element, class ElementOps>
inline
IVGStackAsTable <Element, ElementOps>::
IVGStackAsTable (PureImplementation& impl)
: IGStackAsTable
    <Element, ElementOps> (impl)
{
}

// ---
// IVStackAsTable
// ---

// public members

template <class Element>
inline
IVStackAsTable <Element>::
IVStackAsTable (INumber numberOfElements)
: IVGStackAsTable
   <Element, IStdOps <Element> > (numberOfElements)
{
}

// protected members

template <class Element>
inline
IVStackAsTable <Element>::
IVStackAsTable (PureImplementation& impl)
: IVGStackAsTable
   <Element, IStdOps <Element> > (impl)
{
}
#endif

#endif // _ISTKTAB_INL_
