/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 41 1.9.1.3 source/core/filesys/fsalias.cpp, filesystem, ioc.v400, 000302 
// FSAlias.cpp

#include "fsalias.hpp"
#include "filesys.hpp"

#ifdef IC_PAGETUNE
    #define _FSALIAS_CPP_
    #include <ipagetun.h>
#endif

//---------------------------------------------------------------------------
// IFSAlias "special" methods...
//

extern IStreamModule gFileSystemModule;

StreamableDefinitionsMacro(IFSAlias, gFileSystemModule);


IFSAlias::IFSAlias( const IFileName &type, const IFileName& host, const IFileName &path )
      : fType(type)
      , fHost(host)
      , fPath(path)
{
}

IFSAlias::IFSAlias()
{
}

IFSAlias::IFSAlias( const IFSAlias &copy )
      : fType(copy.fType)
      , fHost(copy.fHost)
      , fPath(copy.fPath)
{
}

IFSAlias&
IFSAlias::operator=( const IFSAlias &copy )
{
      if (&copy != this) {
             fType = copy.fType;
             fHost = copy.fHost;
             fPath = copy.fPath;
      }
      return *this;
}

IFSAlias::~IFSAlias()
{
}


//---------------------------------------------------------------------------
// IFSAlias streaming operators...
//

void
IFSAlias::writeToStream(IDataStream& stream) const
{
        IStreamOutFrame frame(stream);
        fType >>= stream;
        fHost >>= stream;
        fPath >>= stream;
}

void
IFSAlias::readFromStream(IDataStream& stream)
{
        IStreamInFrame frame(stream);
        fType <<= stream;
        fHost <<= stream;
        fPath <<= stream;
}

