// Revision: 67 1.6.1.4 source/core/filesys/fsvolrec.hpp, filesystem, ioc.v400, 000302 
//NOSHIP
// fsvolrec.hpp
//
// Contains:    IFSVolumeRecord
//                              Internal filesystem class that encapsulates volume operations.
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#ifndef _FSVOLREC_
#define _FSVOLREC_

#include <ifilesys.hpp>
#include "fsdirrec.hpp"

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class IFSVolumeRecord : public IFSDirectoryRecord {
public:
//      Special Functions...
    virtual                                             ~IFSVolumeRecord();

// Type Safety Functions...
        // Subclasses may override kind to return their own kind.  If so, they
        // must override isA to check for own kind and call up if not equal.
        virtual IFileKind                       kind    () const;
        virtual bool                            isA (const IFileKind& kind) const;

// Access  properties...
        virtual IFileSize                       freeSpace () const = 0;
        virtual IFileSize                       totalSpace () const = 0;

        virtual IVolume::EState         remote () const = 0;
        virtual IVolume::EState         removable () const = 0;

        // The default implementation of this just tries a volume operation
        // and checks for an IVolumeOffline exception.
        virtual IVolume::EState         online () const;

        // This throws an exception; by default you can't change a volume's name
        virtual void                            setName(const IFileName& newName);

// Volume operations...
        // Volumes can't be deleted or moved; these methods throw exceptions
        virtual void                            deleteSelf (bool deleteChildren);
        virtual void                            doDeleteSelf ();
        virtual IFileSystemEntity       moveTo(IDirectory &to, const IFileName &newName);

protected:
// Subclass interface...
                                                                IFSVolumeRecord();

private:
// This class is ref-counted; copying and assignment aren't allowed...
                                                        IFSVolumeRecord(const IFSVolumeRecord& copy);
    IFSVolumeRecord&                    operator = (const IFSVolumeRecord& copy);
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

// This inline definition doesn't strictly belong here,  but it has
// to appear somewhere after the declaration of IFSVolumeRecord,
// and this is as good a place as any
inline IFSVolumeRecord*
IVolume::getRecord() const {
        return (IFSVolumeRecord*) IDirectory::getRecord();
}

#endif // _FSVOLREC_
