// Revision: 74 1.8.1.3 source/core/filesys/ifileitr.inl, filesystem, ioc.v400, 000302 
// ifileitr.inl
//
// Contains:
//      Inline functions for classes declared in ifileitr.hpp
//
//      Do not include this file directly!  Include <ifileitr.hpp> instead.
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//
#ifndef _IFILEITR_INL_
#define _IFILEITR_INL_

//------------------------------------------------------------
//      IDirectoryIterator inlines
//
inline IFileSystemEntity
IDirectoryIterator::operator*() const {
        return fEntity;
}

inline IFileSystemEntity*
IDirectoryIterator::operator->() {
        return (IFileSystemEntity*) &fEntity;
}

inline IDirectoryIterator&
IDirectoryIterator::operator++() {
        advance();
        return *this;
}

inline bool
IDirectoryIterator::operator== (const IDirectoryIterator& other) const {
        return fEntity == other.fEntity;
}

inline bool
IDirectoryIterator::operator!= (const IDirectoryIterator& other) const {
        return !operator==(other);
}

inline
IDirectoryIterator::operator bool() const {
        return fEntity.valid();
}


//------------------------------------------------------------
//      IVolumeIterator inlines
//
inline IVolume
IVolumeIterator::operator*() const {
        return fEntity;
}

inline IVolume*
IVolumeIterator::operator->() {
        return &fEntity;
}

inline IVolumeIterator&
IVolumeIterator::operator++() {
        advance();
        return *this;
}

inline bool
IVolumeIterator::operator== (const IVolumeIterator& other) const {
        return fEntity == other.fEntity;
}

inline bool
IVolumeIterator::operator!= (const IVolumeIterator& other) const {
        return !operator==(other);
}

inline
IVolumeIterator::operator bool() const {
        return fEntity.valid();
}


//------------------------------------------------------------
//      IRootDirectoryIterator inlines
//
inline IDirectory
IRootDirectoryIterator::operator*() const {
        return fEntity;
}

inline IDirectory*
IRootDirectoryIterator::operator->() {
        return &fEntity;
}

inline IRootDirectoryIterator&
IRootDirectoryIterator::operator++() {
        advance();
        return *this;
}

inline bool
IRootDirectoryIterator::operator== (const IRootDirectoryIterator& other) const {
        return fEntity == other.fEntity;
}

inline bool
IRootDirectoryIterator::operator!= (const IRootDirectoryIterator& other) const {
        return !operator==(other);
}

inline
IRootDirectoryIterator::operator bool() const {
        return fEntity.valid();
}

#endif // _IFILEITR_INL_
