// Revision: 57 1.15.1.3 source/core/filesys/ipathnam.cpp, filesystem, ioc.v400, 000302 
// ipathnam.cpp
//
// Contains:
//      IPathName                       - Portably stores and manipulates file system paths
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#include <ifilesys.hpp>
#include <ipathnam.hpp>
#include <iexcept.hpp>

#include "filesys.hpp"

#ifdef IC_PAGETUNE
    #define _IPATHNAM_CPP_
    #include <ipagetun.h>
#endif

extern IStreamModule gFileSystemModule;

StreamableDefinitionsMacro(IPathName, gFileSystemModule);

//--------------------------------------------------------------------------
// IPathName constructors, etc.
//
IPathName::IPathName ()
      : ICompoundName()
{
}

IPathName::IPathName (const IHostPathName&                      hostPath,
                                                const ICompoundNameParser&      aParser)
        : ICompoundName(hostPath, aParser)
{
}

IPathName::IPathName (const IDirectory&                         startHere,
                                                const IHostPathName&            partialPath,
                                                const ICompoundNameParser&      aParser)
        : ICompoundName(startHere.path())
{
        append( aParser.parseName(partialPath) );
}

IPathName::IPathName (const IDirectory&                         startHere,
                                                 const IPathName&                       partialPath)
        : ICompoundName(startHere.path())
{
        append(partialPath);
}

IPathName::IPathName (const ICompoundName& copy)
        : ICompoundName( copy )
{
}

IPathName::IPathName (const IPathName& copy)
        : ICompoundName( copy )
{
}

IPathName&
IPathName::operator= (const IPathName& copyMe)
{
        if (&copyMe != this) {
                ICompoundName::operator=(copyMe);
        }
        return *this;
}

IPathName::~IPathName ()
{
}


//--------------------------------------------------------------------------
// Streaming methods.
//       ICompoundName's streaming methods do all of the work.
//
void
IPathName::readFromStream (IDataStream& stream)
{
        IStreamInFrame frame(stream);
        ICompoundName::readFromStream(stream);
}

void
IPathName::writeToStream (IDataStream& stream) const
{
        IStreamOutFrame frame(stream);
        ICompoundName::writeToStream(stream);
}


//--------------------------------------------------------------------------
// defaultParser
//      Return the default path name parser for this host's file system.
//
const ICompoundNameParser&
IPathName::defaultParser ()
{
      return IFSFileSystem::instance()->getDefaultParser ();
}

