// Revision: 51 1.11.1.5 source/core/filesys/volume.cpp, filesystem, ioc.v400, 000302 

// volume.cpp
//
// Contains: implementation of IVolume
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#include <ifilesys.hpp>
#include "fsvolrec.hpp"
#include "filesys.hpp"

#ifdef IC_PAGETUNE
    #define _VOLUME_CPP_
    #include <ipagetun.h>
#endif
//------------------------------------------------------------------------------
// Constants/Globals
//
extern IStreamModule gFileSystemModule;

StreamableDefinitionsMacro(IVolume, gFileSystemModule);

#pragma info(nodcl)
const IFileKind _EXPORT IVolume::kKind = "Volume";
#pragma info(restore)

//------------------------------------------------------------------------------
// Constructors, etc...
//

IVolume::IVolume        ()
{
}

IVolume::IVolume (IFSVolumeRecord* record)
        :       IDirectory(record)
{
}

IVolume::IVolume (const IHostPathName &path, const ICompoundNameParser& parser)
{
        typeSafeAssign( IFSFileSystem::instance()->lookUp(path, parser, kMustExist, 0),
                                        kKind );
}

IVolume::IVolume (const IVolume& copy)
        : IDirectory(copy)
{
}

IVolume::~IVolume ()
{
}


//------------------------------------------------------------------------------
//      Downcast "Special" Functions
//  These functions will throw a kTypeMismatchNotAVolume exception if types are different.
//

IVolume::IVolume  (const IFileSystemEntity& copy)
{
        operator = (copy);
}

IVolume& IVolume::operator = (const IFileSystemEntity& right)
{
        typeSafeAssign(right, IVolume::kKind);
        return *this;
}


//------------------------------------------------------------------------------
//      Access volume properties
//

IFileSize IVolume::freeSpace () const
{
        IFSVolumeRecord* Record = (IFSVolumeRecord*) getRecord();
        return (Record->freeSpace());
}

IFileSize IVolume::totalSpace () const
{
        IFSVolumeRecord* Record = (IFSVolumeRecord*) getRecord();
        return (Record->totalSpace());
}

IVolume::EState IVolume::remote () const
{
        return getRecord()->remote();
}

IVolume::EState IVolume::removable () const
{
        return getRecord()->removable();
}

IVolume::EState IVolume::online () const
{
        return getRecord()->online();
}
