/*NOSHIP*/
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// Revision: 85 1.7.1.4 source/core/filesys/win32/windir.hpp, filesystem, ioc.v400, 000302 
// windir.h
// Created by laura on Thu, Jan 24, 1996 @ 12:42 PM.

#ifndef _WINDIR_
#define _WINDIR_

#include "fsdirrec.hpp"
#include "winpaths.hpp"
#include "fsiterat.hpp"

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)

class INTDirectoryRecord : public IFSDirectoryRecord {
public:
// Standard methods....
                                                                INTDirectoryRecord(const INTPathName &fullPath);
        virtual                                         ~INTDirectoryRecord();

// Comparison....
        virtual bool                            testConnection() const ;
        virtual bool                            isEqual ( const IFSRecord *other) const;

// Access properties...
        // IFileSystemEntity properties...
        virtual IFileName                       name() const ;
        virtual void                            setName(const IFileName& newName);

        virtual IHostPathName           hostPath() const ;

        virtual ITimeStamp                      modificationTime() const ;

// Manipulation...
        virtual IFileSystemEntity       moveTo(IDirectory &to, const IFileName &newName);
        virtual IFileSystemEntity       copySelf(IDirectory &to, const IFileName &name) const;

        virtual void                            doDeleteSelf () ;

// Navigation....
        virtual IDirectory                      parent() const ;
        virtual IDirectory                      topmostParent() const ;
        virtual IVolume                         volume () const ;

        virtual void                            setCurrent() const;

// Browsing...
        virtual IFileSystemEntity       lookUpByName(const IFileName& theName) const ;
        virtual IFSIterator *           createIterator() const ;

// Creation....
        virtual IDirectory                      createDirectory(const IFileName& name);
        virtual IFile                           createFile(const IFileName& name,
                                                                                IFileSize initialFileSize) ;

private:
        // Disallowed functions
                                                                INTDirectoryRecord(const INTDirectoryRecord &);
        INTDirectoryRecord &            operator = (const INTDirectoryRecord &);

        // Private data
        INTPathName                             fPath;
};

class INTDirectoryIterator : public IFSIterator {
public:
                                                                INTDirectoryIterator(const INTPathName path);
        virtual                                         ~INTDirectoryIterator();

        virtual IFileSystemEntity       first () ;
        virtual IFileSystemEntity       next () ;

private:
// Disallowed...
                                                                INTDirectoryIterator(const INTDirectoryIterator &copy);
        INTDirectoryIterator &          operator = (const INTDirectoryIterator &copy);

// Private...
        IFileSystemEntity                       makeEntity(const WIN32_FIND_DATAW &data);

        INTPathName                                     fPath;
        HANDLE                                          fHandle;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _WINDIR_
