/*NOSHIP*/
/******************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// Revision: 88 1.12.1.4 source/core/filesys/win32/winpaths.hpp, filesystem, ioc.v400, 000302 
// winpaths.hpp

#ifndef _WINPATHS_
#define _WINPATHS_

#include <istring.hpp>
#include <imrefcnt.hpp>
#include <icntptr.hpp>
#include <isynchr.hpp>

#include "ifilesys.hpp"

#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class INTPathNameImp;

#pragma enum(4)
#pragma pack(push,4)

class INTPathName {
public:

                                        INTPathName(const INTPathName &copy );
        INTPathName &   operator=(const INTPathName &copy);

                                        INTPathName( const IText &string );
                                        INTPathName( const char *string, DWORD length = 0 );
                                        INTPathName( const WCHAR *string, DWORD length = 0 );
                                        INTPathName( const IDirectory &parent, const IFileName &name);

                                        INTPathName( INTPathNameImp *adopt );

        virtual                 ~INTPathName();

        bool                    operator==( const INTPathName & );

                                        operator const char *() const;
                                        operator const WCHAR *() const;
                                        operator IText() const;

        DWORD                   length() const;

        bool                    isVolumeName() const;

        INTPathName             getVolumeName() const;
        INTPathName             getParentName() const;



        INTPathName             getLeafName( ) const;

        INTPathName&    operator += (const INTPathName &);
        INTPathName&    operator += (const IFileName &);

        friend INTPathName operator+ (const INTPathName &, const INTPathName &);
        friend INTPathName operator+ (const INTPathName &, const IFileName &);

private:
                                        INTPathName( const INTPathName &copy, DWORD reserveExtraSpace );
        void                    myOwnStorage();

        ICountedPointerTo<INTPathNameImp>       fImp;
};


class INTPathNameImp : public IMRefCounted {
public:
                                INTPathNameImp( const IText &string );
                                INTPathNameImp( const char *string,  DWORD length = 0 );
                                INTPathNameImp( const WCHAR *string, DWORD length = 0 );

                                INTPathNameImp( const INTPathNameImp &copy );
                                INTPathNameImp( const INTPathNameImp &other, DWORD reserveSpace );

        virtual         ~INTPathNameImp();

        bool            operator==(INTPathNameImp &other);

        CHAR *          getChars();
        WCHAR *         getUnicode();

        DWORD           length() const { return fLength; }

        void            truncate(DWORD length);

        void            reserveRoomFor(DWORD appendCount);

        void            append( const IText & );
        void            append( const INTPathNameImp & );
        void            append( const CHAR *string, DWORD length = 0 );
        void            append( const WCHAR* string,  DWORD length = 0 );
        bool            isVolumeName() const;
        INTPathName     getLeafName() const;
        INTPathName     getVolumeName() const;
        INTPathName     getParentName() const;

        void            canonicalize();
        void            nukeChars();

private:
// This class is ref-counted; assignment isn't allowed...
        void            operator=(const INTPathNameImp &copy);

// Utility functions
        WCHAR *         findLast(WCHAR c) const;


// Private data
        enum { kPathSeparator = '\\' };

        DWORD                                   fLength;                        // length of the string
        DWORD                                   fMaxLength;                     // # of chars allocated

        WCHAR *                                 fUniChars;

        ISynchronized<CHAR *>   fChars;                         // For non-Unicode systems
        DWORD                                   fCharsLength;           // To make comparisons faster

        static bool                             fUseUnicode;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _WINPATHS_
