/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 92 1.6.1.4 source/core/streams/idbgstio.hpp, oss, ioc.v400, 000302 
//
//
//
// idbgstio.hpp
//
#ifndef _IDBGSTIO_
#define _IDBGSTIO_

#include <istrmio.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//
// external class declarations.
//
class IStreamBuffer;
class IDebugStreamBuffer;
class IMStreamable;
class IStreamFinder;
class ITypeRepresentation;
class IVoidMasterPointer;

#pragma pack(push,4)

//
// IDebugStreamIO
//
class IDebugStreamIO : public IStreamIO
{
public:
        IDebugStreamIO(IStreamBuffer* streamBuffer);
        virtual ~IDebugStreamIO();

protected:
friend class IDebugStreamBuffer;
        //
        // called from the IDataStream constructor so that
        // the IStreamIO can stream subobjects.
        //
        virtual void    setStream(IDataStream* dataStream);

        //
        // reset the stream back to the beginning.
        //
        virtual void    reset();

        //
        // force all data out through the stream buffer.
        //
        virtual void    flush();

        //
        // write primitive data types.  encoding format is implemented by
        // subclasses.
        //
        virtual void    writeByte(char source);
        virtual void    writeShort(short source);
        virtual void    writeLong(long source);
        virtual void    writeFloat(float source);
        virtual void    writeDouble(double source);

        //
        // read primitive data types.
        //
        virtual char    readByte();
        virtual short   readShort();
        virtual long    readLong();
        virtual float   readFloat();
        virtual double  readDouble();

        //
        // support for transfers of arrays.
        //
        virtual void    writeBytes(const char* data, unsigned long howMany);
        virtual void    readBytes(char* data, unsigned long howMany);
        virtual void    writeShorts(const short* data, unsigned long howMany);
        virtual void    readShorts(short* data, unsigned long howMany);
        virtual void    writeLongs(const long* data, unsigned long howMany);
        virtual void    readLongs(long* data, unsigned long howMany);
        virtual void    writeFloats(const float* data, unsigned long howMany);
        virtual void    readFloats(float* data, unsigned long howMany);
        virtual void    writeDoubles(const double* data, unsigned long howMany);
        virtual void    readDoubles(double* data, unsigned long howMany);

        //
        // support for streamable objects.
        //
        virtual void    writeObject(const IMStreamable* object, const ITypeRepresentation* typeRep);
        virtual IMStreamable*   readObject(const IStreamFinder* finder, const ITypeRepresentation* typeRep);
        virtual bool    writeAliasedObject(const IMStreamable* object, const IMRefCounted* refCounted, const ITypeRepresentation* typeRep);
        virtual bool    readAliasedObject(IMStreamable*& object, IMRefCounted*& refCounted, const IStreamFinder* finder, const ITypeRepresentation* typeRep);

        //
        // support for stream frames.
        //
        virtual void    writeBeginFrame();
        virtual void    writeEndFrame();
        virtual void    readBeginFrame();
        virtual void    readEndFrame();
        virtual bool    atEndOfFrame();

        //
        // support for stream context.
        //
        virtual bool    establishContext();
        virtual void    clearContext();

        // support for our implementation class
        static  IMStreamable*   createUninitialized(const IStreamFinder* finder, const ITypeName& typeName, IMRefCounted*& refCounted);
        static  const ITypeRepresentation*      findTypeRepresentation(const IStreamFinder* finder, const ITypeName& typeName);

protected:
        IDebugStreamIO();
        IDebugStreamIO(const IDebugStreamIO& other);

private:
        IDebugStreamBuffer*     fImp;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IDBGSTIO_
