/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 74 1.13.1.3 source/core/streams/istrmctx.cpp, oss, ioc.v400, 000302 
//
//
//
// istrmctx.cpp
//
#include "istrmctx.hpp"

#include <istrmio.hpp>
#include <ityperep.hpp>
#include <imstrmbl.hpp>
#include <istrmfnd.hpp>
#include <istrmexc.hpp>
#include <imrefcnt.hpp>

#include <iexcbase.hpp>
#include "strmexc.h"

#include <iostream.h>
#include <string.h>

#ifdef IC_PAGETUNE
    #define _ISTRMCTX_CPP_
    #include <ipagetun.h>
#endif

//#define DEBUG_CONTEXT

IContextContainer::IContextContainer()
        :       fObjectCount(0)
        ,       fTypeCount(0)
        ,       fNext(0)
{
}

IContextContainer::IContextContainer(const IContextContainer& other)
        :       fObjectCount(0)
        ,       fTypeCount(0)
        ,       fNext(0)
{
}

IContextContainer::~IContextContainer()
{
}

long
IContextContainer::objectCount()
{
        return fObjectCount;
}

void
IContextContainer::setObjectCount(long objectCount)
{
        fObjectCount = objectCount;
}

long
IContextContainer::typeCount()
{
        return fTypeCount;
}

void
IContextContainer::setTypeCount(long typeCount)
{
        fTypeCount = typeCount;
}

IContextContainer*
IContextContainer::next()
{
        return fNext;
}

void
IContextContainer::setNext(IContextContainer* nextContainer)
{
        fNext = nextContainer;
}

//
// IStreamContext
//
IStreamContext::IStreamContext()
        :       fActive(false)
        ,       fUsingContainer(false)
        ,       fObjectTypeOnStream(fFastObjectTypeOnStream)
        ,       fObjects(fFastObjects)
        ,       fRefCounted(fFastRefCounted)
        ,       fTypes(fFastTypes)
        ,       fObjectCount(0)
        ,       fTypeCount(0)
        ,       fMaxObjectCount(kFastObjectsArraySize)
        ,       fMaxTypeCount(kFastTypesArraySize)
        ,       fContainers(0)
        ,       fLastContainer(0)
{
}

IStreamContext::IStreamContext(const IStreamContext& other)
        :       fActive(false)
        ,       fUsingContainer(false)
        ,       fObjectTypeOnStream(fFastObjectTypeOnStream)
        ,       fObjects(fFastObjects)
        ,       fRefCounted(fFastRefCounted)
        ,       fTypes(fFastTypes)
        ,       fObjectCount(0)
        ,       fTypeCount(0)
        ,       fContainers(0)
        ,       fLastContainer(0)
{
}

IStreamContext&
IStreamContext::operator=(const IStreamContext& other)
{
        return *this;
}

IStreamContext::~IStreamContext()
{
        reset();
        if (fObjectTypeOnStream != fFastObjectTypeOnStream)
                delete [] fObjectTypeOnStream;
        if (fObjects != fFastObjects)
                delete [] fObjects;
        if (fRefCounted != fFastRefCounted)
                delete [] fRefCounted;
        if (fTypes != fFastTypes)
                delete [] fTypes;
        IContextContainer* cc;
        IContextContainer* nextcc;
        for (cc = fContainers; cc != 0; cc = nextcc) {
                nextcc = cc->next();
                delete cc;
        }
}

bool
IStreamContext::establish()
{
        if (fActive) {
                return false;
        }
        fActive = true;
        return true;
}

void
IStreamContext::clear()
{
        fActive = false;
        reset();
}

void
IStreamContext::reset()
{
        for (int i = 0; i < fObjectCount; i++)
                if (fRefCounted[i] != 0)
                        fRefCounted[i]->removeRef();
        fObjectCount = 0;
        fTypeCount = 0;
}

long
IStreamContext::addObject(const IMStreamable* object, const IMRefCounted* refCounted, bool& typeOnStream, bool& newEntry)
{
#ifdef DEBUG_CONTEXT
cerr << "addObject called for object at " << ((long) object) << endl;
#endif
        long objectNumber = 0;
        for (long i = 0; i < fObjectCount; i++) {
                if (fObjects[i] == object) {
                        if (newEntry == false) {
                                objectNumber = i+1;
                                if (typeOnStream && fObjectTypeOnStream[i])
                                        typeOnStream = false;
#ifdef DEBUG_CONTEXT
cerr << "addObject returned number " << objectNumber << " and newEntry " << newEntry << endl;
#endif
                                return objectNumber;
                        }
                        IAddressAlreadyInContext exc;
                        ITHROW(exc);
                }
        }
        if (fObjectCount == fMaxObjectCount)
                growObjects();
        fObjectTypeOnStream[fObjectCount] = (unsigned char)typeOnStream;
        fObjects[fObjectCount] = (IMStreamable*)object;
        fRefCounted[fObjectCount] = (IMRefCounted*)refCounted;
        fRefCounted[fObjectCount]->addRef();
        newEntry = true;
        objectNumber = ++fObjectCount;
#ifdef DEBUG_CONTEXT
cerr << "addObject returned number " << objectNumber << " and newEntry " << newEntry << endl;
#endif
        return objectNumber;
}

long
IStreamContext::addObjectType(const ITypeRepresentation* objectType, bool& newEntry)
{
#ifdef DEBUG_CONTEXT
cerr << "addObjectType called for object type at " << ((long) objectType) << endl;
#endif
        long objectTypeNumber = 0;
        for (long i = 0; i < fTypeCount; i++) {
                if (fTypes[i] == objectType) {
                        if (newEntry == false) {
                                objectTypeNumber = i+1;
#ifdef DEBUG_CONTEXT
cerr << "addObjectType returned type number " << objectTypeNumber << " and newEntry " << newEntry << endl;
#endif
                                return objectTypeNumber;
                        }
                        IAddressAlreadyInContext exc;
                        ITHROW(exc);
                }
        }
        if (fTypeCount == fMaxTypeCount)
                growTypes();
        fTypes[fTypeCount] = objectType;
        newEntry = true;
        objectTypeNumber = ++fTypeCount;
#ifdef DEBUG_CONTEXT
cerr << "addObjectType returned type number " << objectTypeNumber << " and newEntry " << newEntry << endl;
#endif
        return objectTypeNumber;
}

IMStreamable*
IStreamContext::findObject(IMRefCounted*& refCounted, long objectNumber, const IStreamFinder* finder, const ITypeRepresentation* typeRep)
{
#ifdef DEBUG_CONTEXT
cerr << "findObject called for object number " << objectNumber << endl;
#endif
        if (objectNumber < 1 || objectNumber > fObjectCount) {
                IInvalidContextNumber exc;
                ITHROW(exc);
        }

        IMStreamable* object = fObjects[objectNumber-1];
        refCounted = fRefCounted[objectNumber-1];

        //
        // if we have any containers and this object is in a container,
        // then we need to read the object from the container.
        //
        if (fContainers != 0 && ((((long)object) & 1) == 1)) {
                IContextContainer* container = (IContextContainer*)(((long)object) & ~1);
#ifdef DEBUG_CONTEXT
cerr << "findObject fluffing up object in container at " << ((long)container) << endl;
#endif

                //
                // save the current context state and load the temporary state
                // from the container.
                //
                bool savedUsingContainer = fUsingContainer;
                long savedObjectCount = fObjectCount;
                long savedTypeCount = fTypeCount;
                fUsingContainer = true;
                fObjectCount = container->objectCount();
                fTypeCount = container->typeCount();

                //
                // read the object from the container.
                //
                object = container->readAliasedObject(refCounted, finder, typeRep);

                //
                // restore the context state.
                //
                fUsingContainer = savedUsingContainer;
                fObjectCount = savedObjectCount;
                fTypeCount = savedTypeCount;

                //
                // this container is now "used", so remove it from the container
                // list and delete it.
                //
                if (fContainers == container) {
                        fContainers = container->next();
                        if (fLastContainer == container)
                                fLastContainer = fContainers;
                } else {
                        for (IContextContainer* c = fContainers; c != 0; c = c->next()) {
                                if (c->next() == container) {
                                        c->setNext(container->next());
                                        if (fLastContainer == container)
                                                fLastContainer = c;
                                        break;
                                }
                        }
                }
                delete container;
#ifdef DEBUG_CONTEXT
cerr << "findObject remaining containers..." << endl;
if (fContainers) { for (IContextContainer* c = fContainers; c != 0; c = c->next()) c->printAll(); }
cerr << "findObject check object " << ((long)object) << " vs. " << ((long)fObjects[objectNumber-1]) << endl;
#endif
        }
#ifdef DEBUG_CONTEXT
cerr << "findObject returned object at " << ((long)object) << endl;
#endif

        return object;
}

const ITypeRepresentation*
IStreamContext::findObjectType(long objectTypeNumber)
{
#ifdef DEBUG_CONTEXT
cerr << "findObjectType called for object type number " << objectTypeNumber << endl;
#endif
        if (objectTypeNumber < 1 || objectTypeNumber > fTypeCount) {
                IInvalidContextNumber exc;
                ITHROW(exc);
        }

        const ITypeRepresentation* objectType = fTypes[objectTypeNumber-1];

        //
        // if we have any containers and this type is in a container,
        // then we need to read the type from the container.
        //
        if (fContainers != 0 && ((((long)objectType) & 1) == 1)) {
                IContextContainer* container = (IContextContainer*)(((long)objectType) & ~1);
#ifdef DEBUG_CONTEXT
cerr << "findObjectType fluffing up object type in container at " << ((long)container) << endl;
#endif

                //
                // save the current context state and load the temporary state
                // from the container.
                //
                bool savedUsingContainer = fUsingContainer;
                long savedObjectCount = fObjectCount;
                long savedTypeCount = fTypeCount;
                fUsingContainer = true;
                fObjectCount = container->objectCount();
                fTypeCount = container->typeCount();

                //
                // read the object type from the container.
                //
                objectType = container->readObjectType();

                //
                // restore the context state.
                //
                fUsingContainer = savedUsingContainer;
                fObjectCount = savedObjectCount;
                fTypeCount = savedTypeCount;

                //
                // a container containing a type also contains an object,
                // so we need to keep the container around in case the
                // object is needed.
                //
#ifdef DEBUG_CONTEXT
cerr << "findObjectType check object type " << ((long)objectType) << " vs. " << ((long)fTypes[objectTypeNumber-1]) << endl;
#endif
        }
#ifdef DEBUG_CONTEXT
cerr << "findObjectType returned object type at " << ((long)objectType) << endl;
#endif
        return objectType;
}

IMStreamable*
IStreamContext::nextObject(IMRefCounted*& refCounted)
{
        IMStreamable* object = 0;
        if (fUsingContainer) {
                object = fObjects[fObjectCount];
                if ((((long)object) & 1) == 1)
                        object = 0;
                else
                        refCounted = fRefCounted[fObjectCount];
        }
#ifdef DEBUG_CONTEXT
cerr << "IStreamContext::nextObject() returns " << ((long)object) << endl;
#endif
        return object;
}

void
IStreamContext::addObjectContainer(IContextContainer* container, bool addType)
{
        container->setObjectCount(fObjectCount);
        container->setTypeCount(fTypeCount);

        if (fContainers == 0)
                fContainers = container;
        else
                fLastContainer->setNext(container);
        fLastContainer = container;

        if (fObjectCount == fMaxObjectCount)
                growObjects();
        fObjectTypeOnStream[fObjectCount] = false;
        fObjects[fObjectCount] = (IMStreamable*)(((long)container) | 1);
        fRefCounted[fObjectCount++] = 0;
        if (addType) {
                if (fTypeCount == fMaxTypeCount)
                        growTypes();
                fTypes[fTypeCount++] = (const ITypeRepresentation*)(((long)container) | 1);
        }
#ifdef DEBUG_CONTEXT
cerr << "AddObjectContainer called..." << endl;
for (IContextContainer* c = fContainers; c != 0; c = c->next()) c->printAll();
#endif
}

void
IStreamContext::growObjects()
{
        const long kDynamicObjectsArraySize = 64;
        if (fObjects == fFastObjects)
                fMaxObjectCount = kDynamicObjectsArraySize;
        else
                fMaxObjectCount += kDynamicObjectsArraySize;
        unsigned char* oldObjectTypeOnStreamArray = fObjectTypeOnStream;
        IMStreamable** oldObjectArray = fObjects;
        IMRefCounted** oldRefCountedArray = fRefCounted;
        fObjectTypeOnStream = new unsigned char[fMaxObjectCount];
        fObjects = new IMStreamable*[fMaxObjectCount];
        fRefCounted = new IMRefCounted*[fMaxObjectCount];
        for (long j = 0; j < fObjectCount; j++) {
                fObjectTypeOnStream[j] = oldObjectTypeOnStreamArray[j];
                fObjects[j] = oldObjectArray[j];
                fRefCounted[j] = oldRefCountedArray[j];
        }
        if (oldObjectArray != fFastObjects) {
                delete [] oldObjectTypeOnStreamArray;
                delete [] oldObjectArray;
                delete [] oldRefCountedArray;
        }
}

void
IStreamContext::growTypes()
{
        const long kDynamicTypesArraySize = 64;
        if (fTypes == fFastTypes)
                fMaxTypeCount = kDynamicTypesArraySize;
        else
                fMaxTypeCount += kDynamicTypesArraySize;
        const ITypeRepresentation** oldTypeArray = fTypes;
        fTypes = new const ITypeRepresentation*[fMaxTypeCount];
        for (long j = 0; j < fTypeCount; j++)
                fTypes[j] = oldTypeArray[j];
        if (oldTypeArray != fFastTypes)
                delete [] oldTypeArray;
}
