/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 93 1.4.1.4 source/core/streams/istrmexc.cpp, oss, ioc.v400, 000302 
//
//
//
// istrmexc.cpp
//

#include <istrmexc.hpp>
#include "strmexc.h"

#ifdef IC_PAGETUNE
    #define _ISTRMEXC_CPP_
    #include <ipagetun.h>
#endif

//
// IInvalidDataOnStream
//
IInvalidDataOnStream::IInvalidDataOnStream(const char* text, long errorID, Severity severity)
        :       IException(text, errorID, severity)
{
        if (!errorID)
                setErrorId(kInvalidFormat);
        if (!text)
                setText(IMessageText(kInvalidFormat, kMsgFileName).text());
}

IInvalidDataOnStream::IInvalidDataOnStream(const IInvalidDataOnStream& other)
        : IException(other)
{
}

IInvalidDataOnStream::~IInvalidDataOnStream()
{
}

const char*
IInvalidDataOnStream::name() const
{
        return "IInvalidDataOnStream";
}

//
// IUnknownTypeOnStream
//
IUnknownTypeOnStream::IUnknownTypeOnStream(const char* text, long errorID, Severity severity)
        :       IException(text, errorID, severity)
{
        if (!errorID)
                setErrorId(kUnknownType);
        if (!text)
                setText(IMessageText(kUnknownType, kMsgFileName).text());
}

IUnknownTypeOnStream::IUnknownTypeOnStream(const IUnknownTypeOnStream& other)
        : IException(other)
{
}

IUnknownTypeOnStream::~IUnknownTypeOnStream()
{
}

const char*
IUnknownTypeOnStream::name() const
{
        return "IUnknownTypeOnStream";
}

//
// IEndOfStream
//
IEndOfStream::IEndOfStream(const char* text, long errorID, Severity severity)
        :       IInvalidRequest(text, errorID, severity)
{
        if (!errorID)
                setErrorId(kAtEndOfStream);
        if (!text)
                setText(IMessageText(kAtEndOfStream, kMsgFileName).text());
}

IEndOfStream::IEndOfStream(const IEndOfStream& other)
        : IInvalidRequest(other)
{
}

IEndOfStream::~IEndOfStream()
{
}

const char*
IEndOfStream::name() const
{
        return "IEndOfStream";
}

//
// IAddressAlreadyInContext
//
IAddressAlreadyInContext::IAddressAlreadyInContext(const char* text, long errorID, Severity severity)
        :       IException(text, errorID, severity)
{
        if (!errorID)
                setErrorId(kDuplicateAddress);
        if (!text)
                setText(IMessageText(kDuplicateAddress, kMsgFileName).text());
}

IAddressAlreadyInContext::IAddressAlreadyInContext(const IAddressAlreadyInContext& other)
        : IException(other)
{
}

IAddressAlreadyInContext::~IAddressAlreadyInContext()
{
}

const char*
IAddressAlreadyInContext::name() const
{
        return "IAddressAlreadyInContext";
}

//
// IInvalidContextNumber
//
IInvalidContextNumber::IInvalidContextNumber(const char* text, long errorID, Severity severity)
        :       IException(text, errorID, severity)
{
        if (!errorID)
                setErrorId(kInvalidContextNumber);
        if (!text)
                setText(IMessageText(kInvalidContextNumber, kMsgFileName).text());
}

IInvalidContextNumber::IInvalidContextNumber(const IInvalidContextNumber& other)
        : IException(other)
{
}

IInvalidContextNumber::~IInvalidContextNumber()
{
}

const char*
IInvalidContextNumber::name() const
{
        return "IInvalidContextNumber";
}
