/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 79 1.11.1.4 source/core/streams/istrmfnd.hpp, oss, ioc.v400, 000302 
//
//
//
// istrmfnd.hpp
//
#ifndef _ISTRMFND_
#define _ISTRMFND_

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//
// external class declarations.
//
class IMStreamable;
class IMRefCounted;
class ITypeName;
class ITypeRepresentation;
class IStreamableFactory;

#pragma pack(push,4)

//
// IStreamFinder
//
class IStreamFinder
{
public:
        //
        // returns a finder that implements the default behavior.
        //
        static  const IStreamFinder&    defaultFinder();

protected:
        //
        // default implementation is for IMStreamable objects linked with
        // the application.
        //
        virtual const IStreamableFactory& findFactory(const ITypeName& typeName) const;

        //
        // subclasses can use this method to access the ITypeRepresentation method.
        //
        const ITypeRepresentation*      findTypeWithName(const ITypeName& typeName) const;

        IStreamFinder();
        IStreamFinder(const IStreamFinder& other);
        virtual ~IStreamFinder();

private:
friend class IStreamIO;
        //
        // called from IStreamIO implementations to create objects to stream into.
        //
        IMStreamable*   createUninitialized(const ITypeName& typeName, IMRefCounted*& refCounted) const;

        //
        // called from IStreamIO implementations to find type information.
        //
        const ITypeRepresentation&      findTypeRepresentation(const ITypeName& typeName) const;
};

#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTRMFND_
