/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 84 1.9.1.3 source/core/streams/istrmmod.cpp, oss, ioc.v400, 000302 
//
//
//
// istrmmod.cpp
//
#include <istrmmod.hpp>
#include <ityperep.hpp>
#include "itypenam.hpp"

#include <string.h>

#ifdef IC_PAGETUNE
    #define _ISTRMMOD_CPP_
    #include <ipagetun.h>
#endif

//
// IStreamModule
//

//
// these must be left uninitialized !!
//
IStreamModule* IStreamModule::fgStreamModuleHead;
IStreamModule* IStreamModule::fgStreamModuleTail;

IStreamModule::IStreamModule(const char* moduleName)
        :       fModuleName(moduleName)
        ,       fNextStreamModule(0)
        //
        // must not initialize fTypeRepHead and fTypeRepTail !!
        //
{
        if (fgStreamModuleHead == 0)
                fgStreamModuleHead = this;
        else
                fgStreamModuleTail->fNextStreamModule = this;
        fgStreamModuleTail = this;
}

IStreamModule::IStreamModule()
        :       fModuleName(0)
        ,       fNextStreamModule(0)
{
        // should never be called...
}

IStreamModule::IStreamModule(const IStreamModule& other)
        :       fModuleName(0)
        ,       fNextStreamModule(0)
{
        // should never be called...
}

IStreamModule::~IStreamModule()
{
        // should only get called at static destructor time...
        // and we do not own any storage...
}

const IStreamModule*
IStreamModule::next() const
{
        return fNextStreamModule;
}

const ITypeRepresentation*
IStreamModule::firstTypeRepresentation() const
{
        return fTypeRepHead;
}

const char*
IStreamModule::name() const
{
        return fModuleName;
}

void
IStreamModule::registerType(ITypeRepresentation* typeRepresentation)
{
        if (fTypeRepHead == 0)
                fTypeRepHead = typeRepresentation;
        else
                fTypeRepTail->setNext(typeRepresentation);
        fTypeRepTail = typeRepresentation;
}

const ITypeRepresentation*
IStreamModule::findTypeWithName(const ITypeName& typeName)
{
        const ITypeRepresentation* typeRep = 0;
        const char* moduleName = typeName.componentAt(1);
        for (const IStreamModule* module = fgStreamModuleHead; module != 0; module = module->fNextStreamModule)
        {
                if (strcmp(moduleName, module->fModuleName) == 0) {
                        typeRep = module->fTypeRepHead;
                        break;
                }
        }
        if (typeRep != 0) {
                const char* className = typeName.componentAt(2);
                do {
                        if (strcmp(className, typeRep->name()) == 0)
                                break;
                } while ((typeRep = typeRep->nextTypeInModule()) != 0);
        }
        return typeRep;
}
