// NOSHIP
// Revision: 13 1.11.1.5 source/core/testfw/iheapchk.hpp, testfw, ioc.v400, 000302 
#ifndef _IHEAPCHK_
#define _IHEAPCHK_
/**
 * Class IHeadChecker: Used for detecing and analyzing memory leaks (Internal Only!)
 *
 *<PRE>
 * Contains:    IHeapChecker, a class that is useful for detecting
 *                              and analyzing memory leaks
 *
 * Usage:               Instantiate an IHeapChecker object on the stack just before
 *                              the block of code which you want to check for leaks.
 *                              After the code block, you can call currentBytes or currentBlocks
 *                              to see how many bytes or heap blocks were leaked, or you
 *                              can just call check, which will return true if there were no
 *                              leaks and false if there were.  It will also print the number
 *                              and sizes of the blocks that were leaked.
 *
 * Example:             void IMyTest::test() {
 *                                      ...
 *                                      IHeapChecker heapChecker;
 *                                      {
 *                                              // Code to be tested for leaks
 *                                      }
 *                                      if (!heapChecker.check(outputTextStream())) {
 *                                              setSuccess(false);
 *                                      }
 *                              }
 *
 * Bonus:               If you compile your code with the -Tm option, the check method
 *                              will also print out the file name and line number where each
 *                              leaked block was allocated.  This is *very* handy for tracking
 *                              down your memory leaks.
 *
 * Laura Werner, 5/14/96
 *
 *</PRE>
 * @package Test Framework
 * @category Testing
 *
 * @author Laura Werner
 * @task Creation
 *
 * @copyright
 *      IBM Open Class Library
 *      (C) Copyright International Business Machines Corporation 1997
 *      Licensed Material - Program-Property of IBM - All Rights Reserved.
 *      US Government Users Restricted Rights - Use, duplication, or disclosure
 *      restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 */

#include <stddef.h>
#include <iprimtyp.hpp>

#include <itfwpl.hpp>           // pragma library definitions

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ITieredTextBuffer;
class IHeapCheckerImp;

#pragma enum(4)
#pragma pack(push,4)

class IHeapChecker {
public:
                        IHeapChecker();
        virtual ~IHeapChecker();

        bool    check(ITieredTextBuffer &out);

        size_t  currentBytes();
        size_t  currentBlocks();

private:
        IHeapChecker(const IHeapChecker&);  // not allowed
        IHeapChecker& operator=(const IHeapChecker&); // not allowed
        IHeapCheckerImp*        fImp;

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IHEAPCHK_
