/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 06 1.6.1.6 source/core/testfw/istatset.hpp, testfw, ioc.v400, 000302 

#ifndef _ISTATSET_H
#define _ISTATSET_H

#include <math.h>
#include <assert.h>
#include <float.h>
#include "iprimtyp.hpp"

// Pragma library definitions
#include <itfwpl.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class IStatisticalSet
{
public:
                    IStatisticalSet();
                    IStatisticalSet(long tDist, long errorThresh, long corrThresh);
                    IStatisticalSet(const IStatisticalSet&);
    virtual         ~IStatisticalSet();
    IStatisticalSet& operator=(const IStatisticalSet&);

    void            setTdist(double tDist);
    double          tdist() const;
    void            setErrorThreshold(double errorThresh);
    double          errorThreshold() const;
    void            setCorrelationThreshold(double corrThresh);
    double          correlationThreshold() const;

    unsigned long   numberOfSamples() const;
    double          sum() const;
    double          sumOfSquares() const;
    double          mean() const;
    double          median() const;
    double          variance() const;
    double          standardDeviation() const;
    double          correlation(long j) const;
    double          relativeError() const;
    double          halfInterval() const;
    double          maxSample() const;
    double          minSample() const;

    bool            correlatedData() const;
    bool            errorTooLarge() const;

    void            clear();
    void            addNext(double delta);
    void            removeFirst();
    void            copySamples(long& count, double*& theArray) const;
    virtual void    coalesceSamples();

private:
    void            sort(long length, double*& theArray) const;

    double*         fSamples;
    unsigned long   fSamplesTaken;
    double          fSum;
    double          fSumOfSquares;
    double          fTdist;
    double          fErrorThreshold;
    double          fCorrelationThreshold;
    unsigned long   fArrayLimit;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

// Return number of samples
inline unsigned long IStatisticalSet::numberOfSamples () const
{
    return fSamplesTaken;
}

#endif // _ISTATSET_H
