// Revision: 15 1.13.1.3 source/core/testfw/itestmul.cpp, testfw, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

#include <idecwrap.hpp>
#include <ittxtbuf.hpp>
#include <itestmul.hpp>
#include <ism2.h>

#include <assert.h>
#include <stdio.h>

// Segment definitions
#ifdef IC_PAGETUNE
    #define _ITESTMUL_CPP_
    #include <ipagetun.h>
#endif

//---------------------------------------------------------------------------
//  ITestMultiplexer Constants
//---------------------------------------------------------------------------
static const IString kRealKAllDecisions("All");
#ifdef _AIX
const IString&  ITestMultiplexer::kAllDecisions = kRealKAllDecisions;
#else
#pragma info(nodcl)
const IString& _Export ITestMultiplexer::kAllDecisions = kRealKAllDecisions;
#pragma info(restore)
#endif

//---------------------------------------------------------------------------
//  ITestMultiplexer:  PUBLIC special member functions
//---------------------------------------------------------------------------

ITestMultiplexer::ITestMultiplexer() : fKey(ITestMultiplexer::kAllDecisions)
{
        fInputsAsKeys = 0;
        fIsLoaded = false;
        fDictionary = new ISortedMap<IDecisionFnWrapper, IString>;

}

ITestMultiplexer::ITestMultiplexer(const ITestMultiplexer& copyIn)
: ITest(copyIn), fKey(copyIn.fKey)
{
        fInputsAsKeys = copyIn.fInputsAsKeys;
        fIsLoaded = copyIn.fIsLoaded;
        fDictionary = new ISortedMap<IDecisionFnWrapper, IString>(*copyIn.fDictionary);
}

ITestMultiplexer::ITestMultiplexer(const IString& theKey, unsigned int howManyInputsAsKeys)
: fKey(theKey)
{
        fInputsAsKeys = howManyInputsAsKeys;
        fIsLoaded = false;
        fDictionary = new ISortedMap<IDecisionFnWrapper, IString>;

}

ITestMultiplexer& ITestMultiplexer::operator=(const ITestMultiplexer& copyIn)
{
        if (&copyIn != this)
    {
                ITest::operator=(copyIn);
                fKey = copyIn.fKey;
                fInputsAsKeys = copyIn.fInputsAsKeys;
                fIsLoaded = copyIn.fIsLoaded;
                *fDictionary = *copyIn.fDictionary;
    }
        return (*this);
}

ITestMultiplexer::~ITestMultiplexer()
{
        delete fDictionary;
}



//---------------------------------------------------------------------------
//   ITestMultiplexer: PUBLIC decision key functions
//---------------------------------------------------------------------------

bool ITestMultiplexer::setDecisionKey(const IString& theKey)
{
        loadDictionary();

        if (fDictionary->containsElementWithKey(theKey))
    {
                fKey = theKey;
                setInputsAsKeys(0);
                return true;
    }
        return false;
}


void ITestMultiplexer::decisionKey(IString& dest) const
{
        if (inputsAsKeys() == 0)
                dest = fKey;
        else
                dest = IString("");
}

void ITestMultiplexer::setInputsAsKeys(unsigned int howMany)
{
        fInputsAsKeys = howMany;
}

unsigned int ITestMultiplexer::inputsAsKeys() const
{
        return fInputsAsKeys;
}


//---------------------------------------------------------------------------
//   ITestMultiplexer: PROTECTED ITest overrides
//---------------------------------------------------------------------------

void ITestMultiplexer::test()
{
        ITestDecisionFn decisionFn;
        bool successes = success(), singleSuccess;
        ITieredTextBuffer& out = outputTextStream();

        loadDictionary();

        out << "ITestMultiplexer {\n" ;
        out.setRelativeIndent(1);

        if (fInputsAsKeys > 0)
    {
                out << "Input keys: {\n";
                ISequence<IString> inputs;
                copyInputs(inputs);
                ISequence<IString>::Cursor cursor(inputs);

                int input = 0;
                for (cursor.setToFirst(); input<fInputsAsKeys &&cursor.isValid();cursor.setToNext())
                {
                        IString arg = cursor.element();
                        decisionFn = findDecision(arg);
                        try {
                                if (arg == ITestMultiplexer::kAllDecisions)
                                        singleSuccess = (this->*decisionFn)();
                                else
                                        singleSuccess = handleDecision(decisionFn, arg);
                        }
                        catch(const IException& ie)
                        {
                                singleSuccess = false;
                                out.pushTier(ITieredTextBuffer::kHeadline);
                                out << "Exception in decision function for key \""
                                        << arg << "\"";
                                out.popTier();
                        }

                        out << "\"" << arg << "\"=" << (singleSuccess? "TRUE\n": "FALSE\n");
                        successes = successes && singleSuccess;
        input++;
                }
                out << "}\n";
    }
        else   // fInputsAsKeys <= 0
    {
                decisionFn = findDecision(fKey);
                try{
                        if (fKey == ITestMultiplexer::kAllDecisions)
                                singleSuccess = (this->*decisionFn)();
                        else
                                singleSuccess = handleDecision(decisionFn, fKey);
                }
                catch(const IException& ie)
                {
                        singleSuccess = false;
                        out.pushTier(ITieredTextBuffer::kHeadline);
                        out << "Exception in decision function for key \"" << fKey << "\"";
                        out.popTier();
                }

                out << "Programmatic key: \"" << fKey << "\"="
                        << (singleSuccess? "TRUE\n": "FALSE\n");
                successes = successes && singleSuccess;

    }
        out.setRelativeIndent(-1);
        out << "}\n";
        setSuccess(successes);
}


void ITestMultiplexer::print(ITieredTextBuffer& out)
{

        ITest::print(out);

        out << "ITestMultiplexer: ";

        if (fInputsAsKeys > 0)
                out << "first " << fInputsAsKeys << " input(s) used as key(s)\n";
        else if (fKey == ITestMultiplexer::kAllDecisions)
                out << "All decision functions run\n";
        else out << "Programmatic key \"" << fKey << "\"\n";

        out.pushTier(ITieredTextBuffer::kDebug);

        loadDictionary();
        out  << "Keys: ";

        ISortedMap<IDecisionFnWrapper,IString>::Cursor cursor(*fDictionary);

        for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
        {
                out << (char)'"' << cursor.element().getkey() << "\" ";
        }
        out << (char)'\n';
        out.popTier();

}


//-----------------------------------------------------------------------------
//  ITestMultiplexer: PROTECTED Decision key functions
//-----------------------------------------------------------------------------
void ITestMultiplexer::loadDecisions()
{
        addDecision("True", (ITestDecisionFn)privateTrueDecision);
}


void ITestMultiplexer::addDecision(const IString& key, ITestDecisionFn fun)
{
        fDictionary->add(IDecisionFnWrapper(fun, key));
}

bool ITestMultiplexer::andAllDecisions()
{
        bool success = true;
        bool singleSuccess;
        ITestDecisionFn decision;
        IString key;

        loadDictionary();
        outputTextStream() << "All Decisions: {\n";

        ISortedMap<IDecisionFnWrapper, IString>::Cursor cursor(*fDictionary);

        for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
        {
                decision = cursor.element().decisionFn();
                key = cursor.element().getkey();

                try{
                        singleSuccess = handleDecision(decision, key);
                }
                catch(const IException& ie)
                {
                        singleSuccess = false;
                        outputTextStream().pushTier(ITieredTextBuffer::kHeadline);
                        outputTextStream()      << "Exception in decision function for key \""
                                                << key << "\"";
                        outputTextStream().popTier();
                }

        outputTextStream() << "\"" << key << "\" = "
                        << (singleSuccess? "TRUE\n":"FALSE\n");
                success = success && singleSuccess;
    }
        outputTextStream() << "}\n";
        return success;
}


bool ITestMultiplexer::handleDecision(ITestDecisionFn fun, const IString& key)
{
        outputTextStream().setRelativeIndent(1);
        bool result = (this->*fun)();
        outputTextStream().setRelativeIndent(-1);
        return result;
}

//-----------------------------------------------------------------------------
//  ITestMultiplexer: PRIVATE methods
//-----------------------------------------------------------------------------
void ITestMultiplexer::loadDictionary()
{
        if (!fIsLoaded)
        {
                loadDecisions();
                fIsLoaded = true;
        }
}


ITestDecisionFn ITestMultiplexer::findDecision(const IString& theKey)
{

        if (theKey == ITestMultiplexer::kAllDecisions)
                return (ITestDecisionFn)&ITestMultiplexer::andAllDecisions;

        loadDictionary();
        IDecisionFnWrapper *coldec;

        if (fDictionary->containsElementWithKey(theKey))
                coldec = &fDictionary->elementWithKey(theKey);
        else
                return (ITestDecisionFn)&ITestMultiplexer::unknownDecision;

    return coldec->decisionFn();

}


bool ITestMultiplexer::privateTrueDecision()
{
        return true;
}

bool ITestMultiplexer::unknownDecision()
{
        outputTextStream() << "ITestMultiplexer - Unknown decision.\n";
        return false;
}
