/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 60 1.10.1.4 source/core/text/intl/compare/iclitimp.hpp, text, ioc.v400, 000302 
//===============================================================================
//
// File clitrimp.cpp : collation iterator implementation classes
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================
//
#ifndef _ICLITIMP_
#define _ICLITIMP_

#include <ilocale.hpp>
#include <icolitr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class IHostCollationIterator
{
        public:
                IHostCollationIterator();
                IHostCollationIterator(const IHostCollationIterator& that);
                ~IHostCollationIterator();
                // assignment
                IHostCollationIterator& operator=(const IHostCollationIterator& that);
                // comparison
                bool                            operator==(const IHostCollationIterator& that) const;
                bool                            operator!=(const IHostCollationIterator& that) const;
                // cursor actions
                void                                    reset();
                                                                operator bool() const;
                ICollation*                     create(ICollation::ECollationStrength strength) const;
                IText                                   localePOSIXID() const;
                IHostCollationIterator&         operator++();    // prefix
                IHostCollationIterator          operator++(int); // postfix
                // displayable key name
                IText                                   displayName(const ILocaleKey& desiredLanguage,
                                                                                        ILocaleKey::ELookupStrategy strategy) const;

        protected:
                ILocaleKeyIterator&             hostLocales();

        private:
                ILocaleKeyIterator              fLocales;

};

class IPortableCollationIterator
{
        public:
                IPortableCollationIterator();
                IPortableCollationIterator(const IPortableCollationIterator& that);
                ~IPortableCollationIterator();
                // assignment
                IPortableCollationIterator& operator=(const IPortableCollationIterator& that);
                // comparison
                bool                            operator==(const IPortableCollationIterator& that) const;
                bool                            operator!=(const IPortableCollationIterator& that) const;
                // cursor actions
                void                                    reset();
                                                                operator bool() const;
                IPortableCollation*                     create(ICollation::ECollationStrength strength) const;
                IText                                   localePOSIXID() const;
                IPortableCollationIterator&     operator++();    // prefix
                IPortableCollationIterator              operator++(int); // postfix
                // displayable key name
                IText                                   displayName(const ILocaleKey& desiredLanguage,
                                                                                        ILocaleKey::ELookupStrategy strategy) const;

        private:
//              IText                   *fCollations;

};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICLITIMP_
