/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 70 1.18.1.6 source/core/text/intl/compare/win32/wincoll.cpp, text, ioc.v400, 000302 
//===============================================================================
//
// File wincoll.cpp  : win32 collation object implementation
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================


#ifndef IC_ITXTCOLL
#include <itxtcoll.hpp>
#endif

#ifndef IC_ITXCLIMP
#include <itxclimp.hpp>
#endif

#ifndef IC_IEXCBASE
#include <iexcbase.hpp>
#endif

bool qDebugFlag = false;

#ifdef _WIN32

#if __IBMCPP__ >= 400
#ifndef UNICODE
#define UNICODE
#endif

#ifndef _UNICODE
#define _UNICODE
#endif
#endif

#include <stdio.h>
#include <wtypes.h>
#include <winnls.h> // LCMapStringW, CompareStringW

#ifdef IC_PAGETUNE
        #define _WINCOLL_CPP
        #include <ipagetun.h>
#endif

IWin32Collation::IWin32Collation()
        : ICollation()
{
}

IWin32Collation::IWin32Collation(const ILocaleKey& localeKey,
                                                                ICollation::ECollationStrength strength)
{
        this->setLocaleKey(localeKey);
        this->setStrength(strength);
}

IWin32Collation::IWin32Collation(const IWin32Collation& order)
        : ICollation(order)
{
}

IWin32Collation::~IWin32Collation(void)
{
        this->setLocaleKey(ILocaleKey::defaultLocale());   // ILocale::currentLocale();
        this->setStrength(ICollation::kTertiaryDifference);
}

IWin32Collation&
IWin32Collation::operator=(const IWin32Collation& that)
{
        return *this;
}

IText
IWin32Collation::displayName(const ILocaleKey& desiredLanguage,
                                                        ILocaleKey::ELookupStrategy strategy) const
{
#pragma info(none)
        return (localeKey().displayName(desiredLanguage, strategy));
#pragma info(restore)
}

char*
IWin32Collation::sortKeyToken(char *str, int len) const
{
        int cnt = 0;
        char* startPos = str;
        for (int i = 0; i < len; i++) {
                if (str[i] == 0x01) {
                        if (cnt == this->strength()) {
                                str[i] = 0;
                                break;
                        } else {
                                startPos = &(str[i+1]);
                        }
                        cnt++;
                }
        }
        if ((startPos == str) && (this->strength() != ICollation::kPrimaryDifference)) {
                return str;
        }
        return startPos;
}

#ifdef DEBUG
void
IWin32Collation::printSortKeyStr(char *str, int len)
{
    ITRACE_DEVELOP( IString("Sort Key String, length ") +
                    IString(len) + IString(" : "));

        for (int i = 0; i < len; i++) {
                if (str[i] == 0x01) {
                        ITRACE_DEVELOP(IString("<sep> "));
                } else {
                        ITRACE_DEVELOP(IString((int)str[i]) + IString(" "));
                }
        }
}
#endif
int
IWin32Collation::makeSortKey(unsigned long      flags,
                                                         const IText&   source,
                                                         UniChar*               sourceStr,
                                                         char*                  sourceBuffer,
                                                         int                    len) const
{
        IText::length_type sourceLen = source.length();
        int result = 0;

        result = LCMapStringW(this->localeKey().hostID(), flags, (WCHAR*)sourceStr, sourceLen, (wchar_t*)sourceBuffer, len);
        if (result == 0) {
                // this call does not work try with LCMapStringA
                // first, map to the native code page
                const char* nativeChars = (const char*)source;
                // call with different code page settings to try
                result = LCMapStringA(this->localeKey().hostID(), flags, nativeChars, strlen(nativeChars), sourceBuffer, len);
                if (result == 0) {
                        if (sourceBuffer != 0)
                                strcpy(sourceBuffer, nativeChars);
                        result = strlen(nativeChars);
                }
        }
        return result;          // result is number of bytes matched including NULL,
                                                        // therefore, -1 for NULL and -4 for 4 sort keys.
}

int
IWin32Collation::do_compare(const UniChar* sourceLow,
                                                        const UniChar* sourceHigh,
                                                        const UniChar* targetLow,
                                                        const UniChar* targetHigh) const
{
        if ((sourceHigh < sourceLow) || (targetHigh < targetLow)) {
                ITHROW(IInvalidParameter("Incorrect text offsets!"));
        }
        IText::length_type  sourceLen = sourceHigh - sourceLow + 1,
                                                targetLen = targetHigh - targetLow + 1;
        int result;
        DWORD flags;
    IText ySource(sourceLow, sourceLen);
    IText yTarget(targetLow, targetLen);
        if (this->strength() == ICollation::kPrimaryDifference) {
                IText xSource = do_transform(ySource);
                IText xTarget = do_transform(yTarget);
                if (xSource < xTarget) {
                        return -1;
                } else if (xSource > xTarget) {
                        return 1;
                } else {
                        return 0;
                }
        } else if (this->strength() == ICollation::kSecondaryDifference) {
                flags = NORM_IGNORECASE && NORM_IGNOREKANATYPE && NORM_IGNORENONSPACE;
        } else {
                flags = NORM_IGNOREWIDTH;
        }
        if (this->strength() != ICollation::kPrimaryDifference) {
                result = CompareStringW(this->localeKey().hostID(), flags, (LPCWSTR)sourceLow, sourceLen, (LPCWSTR)targetLow, targetLen);
        if (result == 0) {
            const char* nativeSource = (const char*)ySource;
            const char* nativeTarget = (const char*)yTarget;
            // call with different code page settings to try
            result = CompareStringA(this->localeKey().hostID(), flags, nativeSource, strlen(nativeSource), nativeTarget, strlen(nativeTarget));
            if (result == 0) {
                            // Instead of throwing system error, try to compare bitwise
                                result = strcmp(nativeSource, nativeTarget) + 2;
            }
        }
        }
        if ( (result - 2) < 0 ) {
                return kSourceLess;
        } else if ( (result - 2) > 0) {
                return kSourceGreater;
        } else {
                return kSourceEqual;
        }
}

IText
IWin32Collation::do_transform(const IText& source) const
{
        UniChar *sourceStr = 0;
        IText::length_type chunkLength = 0;
        char *sourceBuffer = 0;

        sourceStr = createUniCharChunk(source, chunkLength);
        int sourceBufferLen = makeSortKey(LCMAP_SORTKEY, source, (UniChar*)sourceStr, sourceBuffer, 0);
        try {
                sourceBuffer = new char[ sourceBufferLen ];
        } catch (...) {
                ITHROW( IOutOfMemory("String calling LCMapString, memory allocation error! In IWin32Collation::do_transform().") );
        }
        int sourceResult = makeSortKey(LCMAP_SORTKEY, source, (UniChar*)sourceStr, sourceBuffer, sourceBufferLen);
        char* tmpBuffer = sortKeyToken(sourceBuffer, sourceResult);
        IText tmp(tmpBuffer);
        delete [] sourceStr;
        delete [] sourceBuffer;
        return tmp;
}


#endif
