// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// *****************************************************************************
// This file was generated from the java source file SimpleTextBoundary.java
// *****************************************************************************
 
#include "isimtxbd.hpp"
#include "iwdbktbl.hpp"
#include "iunicdcm.hpp"

#include <ilanglvl.hpp>
#include <iexcept.hpp> 

#ifdef IC_PAGETUNE
	#define _ISIMTXBD_CPP
	#include <ipagetun.h>
#endif

// *****************************************************************************
// class ISimpleTextBoundary
// *****************************************************************************
 
// -------------------------------------
 
ISimpleTextBoundary::ISimpleTextBoundary(const ITextBoundaryData* data)
  : fData(data), fText(), fPos(0)
{
	fForward = fData->forward();
	fBackward = fData->backward();
	fMap = fData->map();
}

// -------------------------------------
 
ISimpleTextBoundary::~ISimpleTextBoundary()
{
}

// -------------------------------------

ISimpleTextBoundary::ISimpleTextBoundary(const ISimpleTextBoundary& rhs)
  : fData(rhs.fData), fText(rhs.fText), fPos(rhs.fPos)
{
	fForward = fData->forward();
	fBackward = fData->backward();
	fMap = fData->map();
}

// -------------------------------------
  
ISimpleTextBoundary*
ISimpleTextBoundary::clone() const
{
  return new ISimpleTextBoundary(*this);
}

// -------------------------------------
 
bool
ISimpleTextBoundary::operator==(const ITextBoundary& rhs) const
{
  if (this == &rhs)
    return true;

#if 0
// currently we only have one type of ITextBoundary and don't allow subclassing,
// so the RTTI code below is superfluous.  But we've commented it out
// mainly because we don't currently have consistent RTTI support among the
// compilers we need to support.  We've decided to defer this whole issue
// until the compiler situation is more stable, or we have other kinds of
// ITextBoundary, whichever comes first.

#if IC_RTTI_ENABLED
  const ISimpleTextBoundary* other = dynamic_cast<const ISimpleTextBoundary*>(&rhs);
  if (other == 0)
	return false;
#else

  // ugh.  ISimpleTextBoundary is the only concrete class in the system, so don't
  // worry about this yet.
#if 0
  try {
    throw &rhs;
  }
  catch (const ISimpleTextBoundary&) {
  }
  catch (...) {
    return false;
  }
#endif
  const ISimpleTextBoundary* other = (const ISimpleTextBoundary*)&rhs;
#endif

#endif
// TAKE THIS LINE OUT!!!
  const ISimpleTextBoundary* other = (const ISimpleTextBoundary*)&rhs;

  // !!! pointer equality on fData sufficices since these are singletons.
  return fData == other->fData && fPos == other->fPos && fText == other->fText;
}
 
// -------------------------------------
 
IText
ISimpleTextBoundary::text() const
{
    return fText;
}
 
// -------------------------------------
 
void
ISimpleTextBoundary::setText(const IText& text)
{
  fText = text;
  fPos = 0;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::first()
{
    fPos = 0;
    return fPos;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::last()
{
    fPos = fText.length();
    return fPos;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::nthFromCurrent(int increment)
{
    offset_type result = current();

    if (increment < 0) {
        for (int i = increment; (i < 0) && (result != kDone); ++i) {
            result = previous();
        }
    } else {
        for (int i = increment; (i > 0) && (result != kDone); --i) {
            result = next();
        }
    }

    return result;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::previous()
{
  if (fPos > 1) { // should we check for != kDone as well?
    offset_type startBoundary = fPos;
   	fPos = previousSafePosition(fPos - 1);
    offset_type prevPos = fPos;
    offset_type nextPos = next();
    while (nextPos < startBoundary && nextPos != kDone) {
      prevPos = nextPos;
      nextPos = next();
    }
    fPos = prevPos;
    return fPos;
  }
  else if (fPos == 1) {
	fPos = 0;
	return fPos;
  }
  else {
    return kDone;
  }
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::next()
{
    offset_type result = fPos;

    if (fPos < fText.length()) {
        fPos = nextPosition(fPos);
        result = fPos;
    }
    else {
        result = kDone;
    }

    return result;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::nextAfter(offset_type offset)
{
    if (offset >= fText.length()) 
      throw IException("nextAfter offset out of bounds", 0, IException::recoverable);

    fPos = previousSafePosition(offset);
    offset_type result;
    do {
        result = next();
    }
    while (result <= offset && result != kDone);

    return result;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::current() const
{
    return fPos;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::previousSafePosition(offset_type offset)
{
  offset_type result = 0;

  ITextBoundaryData::Node state = fBackward->initialState();
  offset_type p = offset;
  while (!fBackward->isEndState(state) && p < fText.length()) {
    UniChar c = fText[p];
    state = fBackward->get(state, fMap->mappedChar(c));
    if (fBackward->isMarkState(state)) {
	result = p;
    }
    --p;
  }

  return result;
}
 
// -------------------------------------
 
ITextBoundary::offset_type
ISimpleTextBoundary::nextPosition(offset_type offset)
{
  offset_type endIndex = fText.length();

  ITextBoundaryData::Node state = fForward->initialState();
  offset_type p = offset;
  while (!fForward->isEndState(state) && p < fText.length()) {
    UniChar c = fText[p];
    state = fForward->get(state, fMap->mappedChar(c));
    if (fForward->isMarkState(state)) {
	endIndex = p;
    }
    ++p;
  }
  if (fForward->isEndState(state)) {
    return endIndex;
  }
  else if (p < fText.length()) {
    return endIndex;
  }
  else {
    return fText.length();
  }
}
