/**
 * ISpecialMapping represents exceptions to the normal unicode category mapping.
 *
 * @package     Text and International
 * @category    Text Scanning
 *
 * @copyright
 *              IBM Open Class Library
 *              (C) Copyright International Business Machines Corporation 1997
 *              Licensed Material - Program-Property of IBM - All Rights Reserved.
 *              US Government Users Restricted Rights - Use, duplication, or disclosure
 *              restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _ISPCLMAP_
#define _ISPCLMAP_

#include <iprimtyp.hpp> // UniChar
#include <itxtbdat.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

/**
 * This class represents ranges of characters that are exceptions to the normal
 * unicode category mapping.  Characters from the start char to the end char,
 * inclusive, are mapped to the new value.
 */
class ISpecialMapping {
public:
    /** Create a special mapping from the single char ch to the value nv. */
    ISpecialMapping(UniChar ch, ITextBoundaryData::Type nv) : fStartChar(ch), fEndChar(ch), fNewValue(nv) {}

    /** Create a special mapping from the range of chars sch - ech, inclusive, to the value nv. */
    ISpecialMapping(UniChar sch, UniChar ech, ITextBoundaryData::Type nv) : fStartChar(sch), fEndChar(ech), fNewValue(nv) {}

    /** The first character of the range. */
    UniChar fStartChar;

    /** The last character of the range. */
    UniChar fEndChar;

    /** The character mapping to use. */
    ITextBoundaryData::Type     fNewValue;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISPCLMAP_
