// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// *****************************************************************************
// This file was generated from the java source file UnicodeClassMapping.java
// *****************************************************************************
 
#include "iunicdcm.hpp"
#include "iunicode.hpp"

#ifdef IC_PAGETUNE
	#define _IUNICDCM_CPP
	#include <ipagetun.h>
#endif

// *****************************************************************************
// class UnicodeClassMapping
// *****************************************************************************
 
// -------------------------------------
 
IUnicodeClassMapping::IUnicodeClassMapping()
: fMappedValue_length(0),
  fExceptionChars_length(0)
{
	// DO NOT DEFAULT CONSTRUCT!
}

IUnicodeClassMapping&
IUnicodeClassMapping::operator=(const IUnicodeClassMapping&) {
	return *this;
	// DO NOT ASSIGN!
}

IUnicodeClassMapping::IUnicodeClassMapping(	const Type mappedValue[],
											int mappedValue_length,
											const ISpecialMapping exceptionChars[],
											int exceptionChars_length,
											const bool* hasExceptionFlags)
  : fMappedValue(mappedValue)
  , fMappedValue_length(mappedValue_length)
  , fExceptionChars(exceptionChars)
  , fExceptionChars_length(exceptionChars_length)
{
	int i;

	/**
	 * Construct the fHasException[] array.  An element of this array is true if and only
	 * if the corresponding type (from Unicode::getType()) is represented in the
	 * SpecialMapping array fExceptionChars[].  We use this to optimize mappedChar().
	 */
	for (i=0; i<IUnicode::kCharacterPropertiesCount; ++i)
		fHasException[i] = hasExceptionFlags[i];
}
 
// -------------------------------------
 
IUnicodeClassMapping::Type
IUnicodeClassMapping::mappedChar(UniChar ch) const
{
	Type chType = (Type)(IUnicode::type(ch));

	/**
	 * Handle special cases.  These are, in order: No SpecialMapping objects exist;
	 * the character falls in a predefined range, which we know to be free of special
	 * mapping information; we have predetermined that this character type has no
	 * exception data (this data is stored in fHasException[]).
	 */
    if ((fExceptionChars_length == 0) ||
    	((UniChar)0x003F < ch && ch < (UniChar)0x00A0) ||
    	!fHasException[chType])
    {
        return fMappedValue[chType];
    }

	// The invariant during this loop is that the character ch is <= max and
	// >= min.  We iterate until min == max.
    int min = 0;
    int max = fExceptionChars_length - 1;
    while (max > min) {
        int pos = (max + min) >> 1;
        if (ch > fExceptionChars[pos].fEndChar)
            min = pos + 1;
        else
            max = pos;
    }
    const ISpecialMapping* sm = &fExceptionChars[min];
    if (sm->fStartChar <= ch && ch <= sm->fEndChar)
        return sm->fNewValue;
    else
        return fMappedValue[chType];
}
