/**
 * IUnicodeClassMapping maps characters to state change inputs for IWordBreakTable.
 *
 * @package     Text and International
 * @category    Text Scanning
 *
 * @copyright
 *              IBM Open Class Library
 *              (C) Copyright International Business Machines Corporation 1997
 *              Licensed Material - Program-Property of IBM - All Rights Reserved.
 *              US Government Users Restricted Rights - Use, duplication, or disclosure
 *              restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _IUNICDCM_
#define _IUNICDCM_

#include <ispclmap.hpp>
#include <iunicode.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

/**
 * This class maps characters to state change inputs for
 * IWordBreakTable.  If the character appears in the exception list,
 * the mapping there is retuned, otherwise the mapping returned by
 * IUnicode::type is returned.
 *
 * Note in this implementation characters from 0x0040 to 0x009f always use the
 * mapping returned by IUnicode::type and never the exception list.
 */
class IUnicodeClassMapping {
public:
        // For convenience
        typedef ITextBoundaryData::Type Type;

    /**
     * Create a mapping given a mapping from categories and a list
     * of exceptions.  Both the mapping list and exceptionChars list must
     * be sorted in ascending order.
     */
    IUnicodeClassMapping(const Type mappedValue[], int mappedValue_length,
                                                const ISpecialMapping exceptionChars[], int exceptionChars_length,
                                                const bool* hasExceptionFlags);

    /**
     * Map a character to a state change input for IWordBreakTable.
     * @param ch the character to map.
     * @return the mapped value.
     */
    Type mappedChar(UniChar ch) const;

private:
        IUnicodeClassMapping();
        IUnicodeClassMapping& operator=(const IUnicodeClassMapping&);

        bool                                    fHasException[IUnicode::kCharacterPropertiesCount];
        const Type*                             fMappedValue;
        const int                               fMappedValue_length;
        const ISpecialMapping*  fExceptionChars;
        const int                               fExceptionChars_length;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IUNICDCM_
