/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 52 1.30.1.4 source/core/text/intl/locale/ilocale.cpp, text, ioc.v400, 000302 
//===============================================================================
//
// File ilocale.cpp
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================

#ifndef IC_IEXCEPT
#include <iexcept.hpp>
#endif

#ifndef IC_IRESLOCK
#include <ireslock.hpp>
#endif

#ifndef IC_ILOCALE
#include <ilocale.hpp>
#endif

#ifndef IC_LOCMAP
#include "locmap.hpp"
#endif

#ifndef IC_ISTRMMOD
#include <istrmmod.hpp>
#endif

#ifndef IC_IGSTREAM
#include <igstream.hpp>
#endif

// platform specific headers
#ifdef _WIN32
#include <wtypes.h>
#include <winnls.h>
#include <tchar.h>
#endif

#ifdef USE_POSIX
#include <locale.h>

#ifndef IC_IFILEITR
#include <ifileitr.hpp>
#endif

#endif

#ifdef IC_PAGETUNE
        #define _ILOCALE_CPP
        #include <ipagetun.h>
#endif

IStreamModule gLocaleKeyStreamModule("ilocale");

StreamableDefinitionsMacro(ILocaleKey, gLocaleKeyStreamModule);

#ifdef USE_POSIX
void
parseHostPOSIXID(const char* hostStr,
                        IText& posixID)
{
        IText hostText(hostStr);
        IText::offset_type pos = hostText.find(IText("."), 0);
        if (pos == 0) {
                posixID = hostText;
        } else  {
                posixID = hostText.substr(0, pos);
        }
}
#endif

ILocaleKey::ILocaleKey(const IText& languageID,
                       const IText& regionID,
                       const IText& extension)
        : IMStreamable(),
          fLanguageID(languageID),
          fRegionID(regionID),
          fExtension(extension),
          fHostID(0)
{
                gGlobalLocale.initializeMapRegions();
        IText posixID = languageID + "_" + regionID + extension;
        // Check if it's valid before constructing
#ifdef _WIN32
    if (regionID.length() == 0) {
        if (languageID.length() == 0) {
                        fHostID = GetSystemDefaultLCID();
                        posixID = IGlobalLocales::convertToPosix(fHostID);
                        parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension);
        } else if (languageID == IText("C")) {
                        fLanguageID = IText("C");
                } else if (languageID == IText("POSIX")) {
                fLanguageID = IText("POSIX");
        } else {
                        ITHROW(IInvalidParameter("Invalid Locale Identifier"));
                }
        } else {
                unsigned long ntID = IGlobalLocales::convertToLCID(posixID);
                if (IsValidLocale(ntID, LCID_INSTALLED)) {
                        fHostID = ntID;
                } else {
                        unsigned int langID = IGlobalLocales::languageLCID(ntID);
                        ntID = langID + IGlobalLocales::fgStdLang;
                        if (IsValidLocale(ntID, LCID_INSTALLED)) {
                                posixID = IGlobalLocales::convertToPosix(ntID);
                                parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension);
                                fHostID = ntID;
                        } else if (IsValidLocale(ntID, LCID_SUPPORTED)){
                                ITHROW(IInvalidRequest("Locale Supported but not installed"));
                        } else {
                                ITHROW(IInvalidParameter("Invalid Locale Identifier"));
                        }
                }
        }
#endif

#ifdef USE_POSIX
//        IResourceLock reslock(gGlobalLocale.fRes, 100);
//        if (!gGlobalLocale.fgInited) {
//            gGlobalLocale.initializeMapRegions();
//            gGlobalLocale.fgInited = true;
//        }
        char *str = 0, *tmpStr = 0;
        tmpStr = setlocale(LC_ALL, NULL);
        if (tmpStr) {
            str = new char[strlen(tmpStr)+1];
            strcpy(str, tmpStr);
        }
#pragma info(none)
        if (regionID.length() == 0) {
            if (languageID.length() == 0) {
                if (NULL != (tmpStr = getenv("LC_ALL"))) {
                    posixID = tmpStr;
                } else if (NULL != (tmpStr = getenv("LANG"))) {
                    posixID = tmpStr;
                } else {
                    tmpStr = setlocale(LC_ALL, NULL);
                    posixID = tmpStr;
            }
#pragma info(restore)
            parseHostPOSIXID(tmpStr, posixID);
            if (parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension))
                                fHostID = IGlobalLocales::convertToLCID(posixID);
            } else if ( (languageID == IText("C")) || (languageID == IText("POSIX")) ) {
                fLanguageID = languageID;
            } else {
                ITHROW(IInvalidParameter("Invalid Locale Identifier"));
            }
        } else {
            // Check in POSIX table if it's valid
#pragma info(none)
            if (NULL == (tmpStr = setlocale(LC_ALL, posixID))) {
                 unsigned long ntID = IGlobalLocales::convertToLCID(posixID);
                 if (ntID) {
                       unsigned int langID = IGlobalLocales::languageLCID(ntID);
                       ntID = langID + IGlobalLocales::fgStdLang;
                       posixID = IGlobalLocales::convertToPosix(ntID);
                       if (NULL == (tmpStr = setlocale(LC_ALL, posixID))) {
                               ITHROW(IInvalidRequest("Locale supported but not installed"));
                       }
#pragma info(restore)
                 } else {
                        ITHROW(IInvalidParameter("Invalid POSIX identifier"));
                 }
            }
            if (parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension))
                                fHostID = IGlobalLocales::convertToLCID(posixID);
                        else
                ITHROW(IInvalidParameter("Invalid POSIX identifier"));
            if (str) {
                setlocale(LC_ALL, str);
                delete [] str;
            }
        }
#endif
}

ILocaleKey::~ILocaleKey()
{
}

ILocaleKey::ILocaleKey(const ILocaleKey& that)
        : IMStreamable(), fLanguageID(that.fLanguageID), fRegionID(that.fRegionID), fExtension(that.fExtension), fHostID(that.fHostID)
{
}

bool
ILocaleKey::isCLocale() const
{
        if ((fLanguageID == IText("C")) || (fLanguageID == IText("POSIX"))) {
                return true;
        }
        return false;
}

ILocaleKey&
ILocaleKey::operator=(const ILocaleKey& that)
{
    if (this != &that) {
        fLanguageID = that.fLanguageID;
        fRegionID = that.fRegionID;
        fExtension = that.fExtension;
        fHostID = that.fHostID;
    }
    return *this;
}

IText
ILocaleKey::posixID() const
{
        IText tmp;
        if (!isCLocale()) {
                tmp = fLanguageID + IText("_") + fRegionID + fExtension;
        } else {
                tmp += fLanguageID;
        }
        return tmp;
}

void
ILocaleKey::setLanguageID(const IText& langID)
{
        // check if langID is valid
        if (langID == IText("C") || langID == IText("POSIX")) {
                fRegionID = IText();
                fExtension = IText();
                return;
        }
        IText posixID = langID + "_" + fRegionID + fExtension;
        setPOSIXID(posixID);
}

void
ILocaleKey::setRegionID(const IText& regionID)
{
        // check if regionID is valid
        IText posixID = fLanguageID + "_" + regionID + fExtension;
        setPOSIXID(posixID);
}

void
ILocaleKey::setExtension(const IText& extension)
{
        // check if extension is valid
        IText posixID = fLanguageID + "_" + fRegionID + extension;
        setPOSIXID(posixID);
}

IText
ILocaleKey::languageID() const
{
        return fLanguageID;
}

IText
ILocaleKey::regionID() const
{
        return fRegionID;
}

IText
ILocaleKey::extension() const
{
        return fExtension;
}

bool
ILocaleKey::parsePOSIXID(const IText& posixID,
                        IText& langID,
                        IText& regionID,
                        IText& extension)
{
        if (posixID.length() > 6) {
                return false;
        }
        IText::offset_type pos = posixID.find(IText("_"), 0);
        if (pos == IText::npos) { // no "_", not a regular POSIX ID
                if ((posixID == IText("C")) || (posixID == IText("POSIX"))) {
                        langID = posixID;
                        regionID = IText();
                        extension = IText();
                } else { return false; }
        } else if (pos > 2) {
                return false;
        } else {
                langID = posixID.substr(0, 2);
                regionID = posixID.substr(pos+1, 2);
                if (posixID.length() > 5) {
                        extension = posixID.substr(pos+3, 1);
                } else {
                        extension = IText();
                }
        }
        return true;
}

void
ILocaleKey::setPOSIXID(const IText& newLocaleID)
{
        if (newLocaleID == IText("C") || newLocaleID == IText("POSIX")) {
                fRegionID = IText();
                fExtension = IText();
                return;
        }
#ifdef USE_POSIX
        IText posixID = newLocaleID;
        if (!this->parsePOSIXID(newLocaleID, fLanguageID, fRegionID, fExtension)) {
                ITHROW(IInvalidParameter("Invalid POSIX Identifier"));
        } else {
#pragma info(none)
                char *str = 0, *tmpStr = 0;
                tmpStr = setlocale(LC_ALL, NULL);
                if (tmpStr) {
                        str = new char[strlen(tmpStr)+1];
                        strcpy(str, tmpStr);
                }
                if (NULL == (tmpStr = setlocale(LC_ALL, posixID))) {
                        unsigned long ntID = IGlobalLocales::convertToLCID(posixID);
                        if (ntID) {
                                unsigned int langID = IGlobalLocales::languageLCID(ntID);
                                ntID = langID + IGlobalLocales::fgStdLang;
                                posixID = IGlobalLocales::convertToPosix(ntID);
                                if (NULL == (tmpStr = setlocale(LC_ALL, posixID))) {
                                        ITHROW(IInvalidRequest("Locale supported but not installed"));
                                }
                        } else {
                           ITHROW(IInvalidParameter("Invalid POSIX identifier"));
                        }

                }
#pragma info(restore)
                if (!parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension))
                        ITHROW(IInvalidParameter("Invalid POSIX identifier"));
                if (str) {
                      setlocale(LC_ALL, str);
                      delete [] str;
                }
        }

#endif
#ifdef _WIN32
        unsigned long localeID = IGlobalLocales::convertToLCID(newLocaleID);
        fHostID = localeID;
    if (IsValidLocale(localeID, LCID_INSTALLED)) {
                fHostID = localeID;
                parsePOSIXID(newLocaleID, fLanguageID, fRegionID, fExtension);
        // look up values for Language ID, region ID and extension.
        } else if (IsValidLocale(localeID, LCID_SUPPORTED)){
                ITHROW(IInvalidRequest("Locale Supported but not installed"));
        } else {
                if (localeID) {
                        unsigned int langID = IGlobalLocales::languageLCID(localeID);
                        fHostID = langID + IGlobalLocales::fgStdLang;
                        IText posixID = IGlobalLocales::convertToPosix(fHostID);
                        parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension);
                } else {
                        ITHROW(IInvalidParameter("Invalid POSIX Identifier"));
                }
        }
#endif

}

#pragma info(nouse)
IText
ILocaleKey::exactLocaleMatch(const ILocaleKey& desiredLanguage) const
{
        char *buffer = 0;
        int cchData = 0;
#ifdef _WIN32
        if (GetSystemDefaultLCID() != desiredLanguage.hostID()) {
            ITHROW( IInvalidRequest("No valid locale display name in the desired locale"));
        } else if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, NULL, 0)) == 0){
            cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, NULL, 0);
            if (cchData == 0)
                ITHROW( IOutOfSystemResource("No valid locale display name calling GetLocaleInfo()"));
        }
        int saveLen = cchData;
        buffer = new char[cchData];
        if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, (LPTSTR)buffer, cchData)) == 0) {
            cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, buffer, saveLen);
        }
#endif
        IText tmpText(buffer);
        if (cchData >= 1)
                delete [] buffer;
        return tmpText;
}

IText
ILocaleKey::exactLanguageMatch(const ILocaleKey& desiredLanguage) const
{
        char *buffer = 0;
        IText tmpText;
        int cchData = 0;
#ifdef _WIN32
        try {
                tmpText = exactLocaleMatch(desiredLanguage);
        } catch (const IException&) {
                unsigned int langID = IGlobalLocales::languageLCID(desiredLanguage.hostID());
                unsigned long ntID = langID + IGlobalLocales::fgStdLang;
                if (IsValidLocale(ntID, LCID_INSTALLED)) {
                        if (ntID == hostID()) {  // new language is native language
                                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SNATIVELANGNAME, NULL, 0)) == 0) {
                                    if ((cchData = GetLocaleInfoA(hostID(), LOCALE_SNATIVELANGNAME, NULL, 0)) == 0) {
                                        ITHROW( IOutOfSystemResource("No valid locale display name calling GetLocaleInfo()"));
                                    }
                                }
                                buffer = new char[cchData];
                                int saveLen = cchData;
                                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SNATIVELANGNAME, (LPTSTR)buffer, cchData)) == 0) {
                                    cchData = GetLocaleInfoA(hostID(), LOCALE_SNATIVELANGNAME, buffer, saveLen);
                                }
                        } else if (GetSystemDefaultLCID() == ntID) {
                                int cchData = 0;
                                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, NULL, 0)) == 0) {
                                    if ((cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, NULL, 0)) == 0) {
                                        ITHROW( IOutOfSystemResource("No valid locale display name calling GetLocaleInfo()"));
                                    }
                                }
                                buffer = new char[cchData];
                                int saveLen = cchData;
                                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, (LPTSTR)buffer, cchData)) == 0) {
                                    cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, buffer, saveLen);
                                }
                        } else {
                                                                ITHROW(IInvalidRequest("No valid locale display name in the same langauge") );
                                                }
                }
    }
#endif
        if (cchData >= 1) {
                tmpText = IText(buffer);
                delete [] buffer;
        }
        return tmpText;
}

IText
ILocaleKey::anyLanguageMatch(const ILocaleKey& desiredLanguage) const
{
        char *buffer = 0;
        IText tmpText;
        int cchData = 0;
        try {
        tmpText = exactLanguageMatch(desiredLanguage);
        } catch (const IException&) { // use the localized name
#ifdef _WIN32
                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, NULL, 0)) == 0) {
                    if ((cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, NULL, 0)) == 0) {
                        ITHROW( IOutOfSystemResource("No valid locale display name calling GetLocaleInfo()"));
                    }
                }
                int saveLen = cchData;
                buffer = new char[cchData];
                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SLANGUAGE, (LPTSTR)buffer, cchData)) == 0) {
                    cchData = GetLocaleInfoA(hostID(), LOCALE_SLANGUAGE, buffer, saveLen);
                }
                if (cchData >= 1) {
                    tmpText = IText(buffer);
                    delete [] buffer;
                }
#endif
        }
        return tmpText;
}

IText
ILocaleKey::displayName(const ILocaleKey& desiredLanguage,
                                        ELookupStrategy strategy) const
{
        IText tmpText;
        char *buffer = 0;
        int cchData = 0;
        if ( ( languageID() == IText("C") ) || (languageID() == IText("POSIX") ) )
                return languageID();
#ifdef USE_POSIX
        tmpText = posixID();
#elif defined(_WIN32) // before the mapping in place.
        // the desired langauge is the native langauge
        if (desiredLanguage == hostID()) {
                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SNATIVELANGNAME, NULL, 0)) == 0) {
                    if ((cchData == GetLocaleInfoA(hostID(), LOCALE_SNATIVELANGNAME, NULL, 0)) == 0) {
                        return IText();
                    }
                }
                int saveLen = cchData;
                buffer = new char[cchData];
                if ((cchData = GetLocaleInfo(hostID(), LOCALE_SNATIVELANGNAME, (LPTSTR)buffer, cchData)) == 0) {
                    cchData = GetLocaleInfoA(hostID(), LOCALE_SNATIVELANGNAME, buffer, saveLen);
                }
                tmpText = IText(buffer);
        } else {
                switch (strategy) {
                        case ILocaleKey::kExactLocale:
                                tmpText = exactLocaleMatch(desiredLanguage);
                                break;
                        case ILocaleKey::kExactLanguage:
                                tmpText = exactLanguageMatch(desiredLanguage);
                                break;
                        case ILocaleKey::kAnyLanguage:
                                tmpText = anyLanguageMatch(desiredLanguage);
                                break;
                        default:
                                ITHROW( IInvalidParameter("Invalid look-up strategy"));
                }
        }
#endif
        if (cchData >= 1)
                delete [] buffer;
        return tmpText;
}
#pragma info(restore)

ILocaleKey
ILocaleKey::defaultLocale()
{
                gGlobalLocale.initializeMapRegions();
#ifdef _WIN32
        ILocaleKey ret( GetThreadLocale() );
        return ret;
#elif  defined(USE_POSIX)
        char *tmpStr = 0;
        IText posixID, langID, regID, extID;
#pragma info(none)
        if (NULL != (tmpStr = getenv("LC_ALL"))) {
                        posixID = tmpStr;
                } else if (NULL != (tmpStr = getenv("LANG"))) {
                        posixID = tmpStr;
                } else {
                        tmpStr = setlocale(LC_ALL, NULL);
                        posixID = tmpStr;
                }
#pragma info(restore)
        parseHostPOSIXID(tmpStr, posixID);
        if (parsePOSIXID(posixID, langID, regID, extID))
                return (ILocaleKey(langID, regID, extID));
        else
                return (ILocaleKey("C"));
#endif
}

void
ILocaleKey::writeToStream(IDataStream& toWhere) const
{
    IStreamOutFrame localeKeyFrame(toWhere);
        this->fLanguageID >>= toWhere;
    this->fRegionID >>= toWhere;
    this->fExtension >>= toWhere;
}

void
ILocaleKey::readFromStream(IDataStream& fromWhere)
{
    IStreamInFrame localeKeyFrame(fromWhere);
        this->fLanguageID <<= fromWhere;
    this->fRegionID <<= fromWhere;
    this->fExtension <<= fromWhere;
}

bool
ILocaleKey::operator==(const ILocaleKey& other) const
{
    if (this != &other) {
        if (fLanguageID != other.fLanguageID) return false;
        if (fRegionID != other.fRegionID) return false;
        if (fExtension != other.fExtension) return false;
        if (fHostID != other.fHostID) return false;
    }
    return true;
}

bool
ILocaleKey::operator!=(const ILocaleKey& other) const
{
    return !(*this == other);
}

IText
ILocaleKey::displayName(const IText& posixID,
                        const ILocaleKey&  desiredLanguage,
                        ILocaleKey::ELookupStrategy lookup)
{
    IText langID, regID, extID;
    if (parsePOSIXID(posixID, langID, regID, extID)) {
        ILocaleKey tmp(langID, regID, extID);
        return tmp.displayName(desiredLanguage, lookup);
    } else {
        ITHROW( IInvalidParameter("Invalid POSIX ID"));
    }
}

ILocaleKey::ILocaleKey(unsigned long localeId)
{
                gGlobalLocale.initializeMapRegions();
        setHostID(localeId);
}

void
ILocaleKey::setHostID(unsigned long localeID)
{
        IText posixID;
        posixID = IGlobalLocales::convertToPosix(localeID);
        fHostID = localeID;
#ifdef _WIN32
    if (IsValidLocale(localeID, LCID_INSTALLED)) {
                parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension);
        // look up values for Language ID, region ID and extension.
        } else if (IsValidLocale(localeID, LCID_SUPPORTED)){
                ITHROW(IInvalidRequest("Locale Supported but not installed"));
        } else {
#endif
                if (posixID != IText("??_??")) {
                        if ((parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension)) &&
                                (fRegionID == IText("??"))) {
                                        unsigned int langID = IGlobalLocales::languageLCID(localeID);
                                        fHostID = langID + IGlobalLocales::fgStdLang;
                                        posixID = IGlobalLocales::convertToPosix(fHostID);
                                        parsePOSIXID(posixID, fLanguageID, fRegionID, fExtension);
                        }
                } else {
                        ITHROW(IInvalidParameter("Invalid Locale Identifier"));
                }
#ifdef _WIN32
        }
#endif
}

unsigned long
ILocaleKey::hostID() const
{
    if (isCLocale())
        return 0;
    return fHostID;
}

ILocaleKey::ILocaleKey()
{
        gGlobalLocale.initializeMapRegions();
    fHostID = (defaultLocale()).hostID();
    fLanguageID = (defaultLocale()).languageID();
    fRegionID  = (defaultLocale()).regionID();
    fExtension = (defaultLocale()).extension();
}

/* ------------------------------------------------------------------------
                 ILocaleKeyIterator Implementation
   ------------------------------------------------------------------------ */

#ifdef _WIN32
//.............................................................................
// Callback function for enumerating the system's locals
// global current LCID is referenced in ILocaleKeyIterator class.
//.............................................................................

BOOL CALLBACK
EnumLocalesProc( LPTSTR lpzLocale)
{
    BOOL fRC  = false;  // Assume failure
        LCID lcid = 0;      // For LCID value.

    // translate locale string to lcid value.
        int  cchData = _stscanf( lpzLocale, TEXT("%x"), &lcid);

        if ( cchData > 0 )
        {                          // Build the string from lang and ctry names.
            if (gGlobalLocale.fgLocales == 0) {
                        gGlobalLocale.fgLocaleCount++;
                } else {
                        gGlobalLocale.fgLocales[gGlobalLocale.fgLocaleCount].setHostID(lcid);
                        gGlobalLocale.fgLocaleCount++;
                }
        fRC = true;
    }
        return fRC;
}

#endif
#ifdef USE_POSIX
static bool getRealPath(const IText& path, int n, IText& result)
{
        IText::offset_type offset = 0;
#ifdef __OS2__
        IText   separator(";");
#else   // i.e., AIX
        IText   separator(":");
#endif
        if (n == 0) {
                offset = path.find(separator, 0);
                result = (path.substr(0, ((offset == IText::npos) ? path.length() : offset)));
                    return true;
        }
        for (int i = 0; i < n; i++) {
                offset = path.find(separator, offset+1);
                if ((offset == IText::npos) || (offset == path.length() - 1)) return false;
        }
        IText::offset_type nextOffset = path.find(separator, offset+1);
        result = (path.substr(offset+1,
            ((nextOffset == IText::npos) ? (path.length() - offset -1): (nextOffset - offset - 1))));
    return true;
}

static bool lookupPOSIX(IText& name, int n)
{
    bool result = false;
    if ((n == 0) || (gGlobalLocale.fgLocales == 0)) return result;
    for (int i = 0; i < gGlobalLocale.fgLocaleCount; i++) {
        if (gGlobalLocale.fgLocales[i].posixID() == name) {
            result = true;
            break;
        }
    }
    return result;
}

#pragma info(none)
int
ILocaleKeyIterator::enumPOSIXLocale()
{
    char *locpathstr = 0;
        int dirCount = 0;
        gGlobalLocale.initializeMapRegions();
    if (NULL != (locpathstr = getenv("LOCPATH"))) {
                IText realPath(locpathstr);
                IText str;
                while (getRealPath(realPath, dirCount, str)) {
                for (IDirectoryIterator locDirIter = IDirectory(IHostPathName(str)); locDirIter; locDirIter++) {
#ifdef __OS2__
                if ((*locDirIter).isA(IDirectory::kKind)) {
#endif
                        IText tmp((*locDirIter).name()), tmpLang, tmpRegion, tmpExt;
                    if ((tmp.find("_", 0)) && ILocaleKey::parsePOSIXID(tmp, tmpLang, tmpRegion, tmpExt)) {
                            if (gGlobalLocale.fgLocales == 0) {
                        char *isValid = setlocale(LC_ALL, tmp);
                        if ((isValid != 0) && (!lookupPOSIX(tmp, dirCount))) {
                                      gGlobalLocale.fgLocaleCount++;
                                   }
                            } else {
                                try {
                                        char *isValid = setlocale(LC_ALL, tmp);
                                        if ((isValid != 0) && (!lookupPOSIX(tmp, dirCount))) {
                                            gGlobalLocale.fgLocales[gGlobalLocale.fgLocaleCount].setPOSIXID(tmp);
                                            gGlobalLocale.fgLocaleCount++;
                                        }
                                    } /* end try */
                                catch (...) {  // locale supported but not installed
                                        continue;
                                } /* end catch */
                            }
                    }
#ifdef __OS2__
                    }
#endif
                        }
                        dirCount++;
        }
    } else ITHROW(IInvalidRequest("No locales path specified"));

    return gGlobalLocale.fgLocaleCount;
}
#pragma info(restore)
#endif

ILocaleKeyIterator::ILocaleKeyIterator()
{
                gGlobalLocale.initializeMapRegions();
        if (gGlobalLocale.fgLocales == 0) {
#ifdef _WIN32
                EnumSystemLocales( (LOCALE_ENUMPROC)EnumLocalesProc,
                                                        LCID_INSTALLED );
                if (gGlobalLocale.fgLocaleCount != 0) {
                        gGlobalLocale.fgLocales = new ILocaleKey[gGlobalLocale.fgLocaleCount];
                        gGlobalLocale.fgLocaleCount = 0;
                        EnumSystemLocales( (LOCALE_ENUMPROC)EnumLocalesProc,
                                                                LCID_INSTALLED );
                }  else {
                        // throw exception ; there's no locales
                        ITHROW(IInvalidRequest("No locales installed"));
                }
#elif defined(USE_POSIX)
//        if (!gGlobalLocale.fgInited) {
//            gGlobalLocale.initializeMapRegions();
//            gGlobalLocale.fgInited = true;
//        }
        if (enumPOSIXLocale()) {
                gGlobalLocale.fgLocales = new ILocaleKey[gGlobalLocale.fgLocaleCount];
                gGlobalLocale.fgLocaleCount = 0;
                enumPOSIXLocale();
        } else {
                // throw exception ; there's no locales
                ITHROW(IInvalidRequest("No locales installed"));
        }
#endif
        }
        fIndex = 0;
        fLocales = gGlobalLocale.fgLocales;
}

ILocaleKeyIterator::ILocaleKeyIterator(const ILocaleKeyIterator& that)
        : fLocales(that.fLocales), fIndex(that.fIndex)
{
}

ILocaleKeyIterator::~ILocaleKeyIterator()
{
    if (fLocales) {
        fLocales = 0;
        fIndex = 0;
    }
}

ILocaleKeyIterator&
ILocaleKeyIterator::operator=(const ILocaleKeyIterator& that)
{
    fLocales = that.fLocales;
    fIndex = that.fIndex;
    return *this;
}

bool
ILocaleKeyIterator::operator==(const ILocaleKeyIterator& that) const
{
    if (fIndex == that.fIndex) {
       return true;
    }
       return false;
}

bool
ILocaleKeyIterator::operator!=(const ILocaleKeyIterator& that) const
{
    return !( *this == that );
}

void
ILocaleKeyIterator::reset()
{
    fIndex = 0;
}

ILocaleKeyIterator
ILocaleKeyIterator::end()
{
    ILocaleKeyIterator tmp;
    tmp.fIndex = gGlobalLocale.fgLocaleCount;
    return tmp;
}

ILocaleKeyIterator::operator bool()      const
{
    if (fIndex >= gGlobalLocale.fgLocaleCount) {
        return false;
    }
        return true;
}


ILocaleKey
ILocaleKeyIterator::operator*()  const
{
    if (fIndex < gGlobalLocale.fgLocaleCount)
        return fLocales[fIndex];
    return fLocales[gGlobalLocale.fgLocaleCount - 1];
}

const ILocaleKey*
ILocaleKeyIterator::operator->()  const
{
    if (fIndex < gGlobalLocale.fgLocaleCount)
        return &(fLocales[fIndex]);
    return &(fLocales[gGlobalLocale.fgLocaleCount - 1]);
}

ILocaleKeyIterator&
ILocaleKeyIterator::operator++()
{
    // range checking, do not advance cursor
    if (fIndex + 1 > gGlobalLocale.fgLocaleCount) {
        ITHROW(IInvalidRequest("Invalid advancing request for locale key iterator"));
    } else if (fIndex + 1 == gGlobalLocale.fgLocaleCount) {
        fIndex = gGlobalLocale.fgLocaleCount;
        return *this;
    }
    fIndex++;
    return *this;
}

ILocaleKeyIterator
ILocaleKeyIterator::operator++(int)
{
    ILocaleKeyIterator tmp = *this;
    // range checking, do not advance cursor
    if (fIndex + 1 > gGlobalLocale.fgLocaleCount) {
        ITHROW(IInvalidRequest("Invalid advancing request for locale key iterator"));
    } else if (fIndex + 1 == gGlobalLocale.fgLocaleCount) {
        fIndex = gGlobalLocale.fgLocaleCount;
        return *this;
    }
    fIndex++;
    return tmp;
}
