#ifndef _LOCMAP_
#define _LOCMAP_

/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 54 1.15.1.4 source/core/text/intl/locale/locmap.hpp, text, ioc.v400, 000302 
//===============================================================================
//
// File locmap.hpp      : Locale Mapping Classes
//
//
//
// Created by: Helena Shih
//
// Modification History:
//
//      Date         Name          Description
//
//===============================================================================

#ifdef _WIN32
#include <wtypes.h>
#endif

#include <ilocale.hpp>

#include <ireslock.hpp>
/////////////////////////////////////////////////
//
// Internal Classes for LCID <--> POSIX Mapping
//
/////////////////////////////////////////////////

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)


class ILcidPosixElement
{
public:
  ILcidPosixElement(unsigned long, const char*);

  ILcidPosixElement();
  ILcidPosixElement(const ILcidPosixElement&);
  ILcidPosixElement& operator=(const ILcidPosixElement&);

  ~ILcidPosixElement();

private:

  unsigned long fHostID;
  char fPosixID[7];

  friend class ILcidPosixMap;
};

class ILcidPosixMap
{
public:

  ILcidPosixMap (unsigned long hostID,
                 const char* posixID,
                 unsigned int totalNumberOfRegions = 1);

  ~ILcidPosixMap();

  void addRegion (unsigned long hostID,
                  const char* posixID);

  unsigned int hostLangID() const
  { return fHostLangID; };

  const char* posixLangID() const
  { return fPosixLangID; };

  unsigned long hostID(const char* fromPosixID) const;
  const char* posixID(unsigned long fromHostID) const;

  static const char* fgWildCard;

  ILcidPosixMap(const ILcidPosixMap&);

private:

  ILcidPosixMap();
  ILcidPosixMap& operator=(const ILcidPosixMap&);

  unsigned int fHostLangID;
  char fPosixLangID[3];

  ILcidPosixElement* fRegionMaps;
  unsigned int fMapSize;
  unsigned int fNumRegions;
};

//

class IGlobalLocales {
    public:
                IGlobalLocales()
                        {
                        }
                ~IGlobalLocales()
                        {
                        }
                IGlobalLocales(const IGlobalLocales& that)
                        {
                        fgLocales = that.fgLocales;
                        fgPosixIDmap = that.fgPosixIDmap;
                        fgLocaleCount = that.fgLocaleCount;
                        }
                IGlobalLocales& operator=(const IGlobalLocales& that)
                        {
                        fgLocales = that.fgLocales;
                        fgPosixIDmap = that.fgPosixIDmap;
                        fgLocaleCount = that.fgLocaleCount;
                        return *this;
                        }
                       void     initializeMapRegions();
                static const char* convertToPosix(unsigned long hostid);
                static unsigned long convertToLCID(const char* posixID);
                static unsigned int languageLCID(unsigned long hostID);


                static ILocaleKey        *fgLocales;
                static ILcidPosixMap *fgPosixIDmap;
                static int                       fgLocaleCount;
                static unsigned long fgStdLang;
#ifdef USE_POSIX
//                static bool             fgInited;
#endif
                IPrivateResource        fRes;
};

#pragma pack(pop)

extern IGlobalLocales gGlobalLocale;

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
