// Revision: 34 1.9.1.4 source/core/text/intl/transcod/iso88591.hpp, text, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: iso88591.hpp
//  Function: Internal ISO8859-1 Transcoder Definition.
//  Author: Chen-Lieh Huang
//  Date created: 04/05/1996
//
#ifndef _ISO88591_
#define _ISO88591_

#include <itrancod.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma pack(push,4)
#pragma enum(4)

class IISO8859_1Transcoder : public ITranscoder {
public:
        //      Standard member functions
        IISO8859_1Transcoder();
        IISO8859_1Transcoder(const IISO8859_1Transcoder& source);
        IISO8859_1Transcoder& operator=(const IISO8859_1Transcoder& right);
        virtual                         ~IISO8859_1Transcoder();

        // Overrides
        length_type             maximumBytesPerUniChar() const;
        length_type             maximumUniCharsPerByte() const;

        length_type             byteBufferSize(const IText& uniText) const;
        length_type             byteBufferSize(const UniChar* from, const UniChar* from_end) const;
        length_type             uniCharBufferSize(const IString& text) const;
        length_type             uniCharBufferSize(const char* from, const char* from_end) const;

protected:
        //      called by toUnicode
        result  doToUnicode(const char* from, const char* from_end,
                                                const char*& from_next,
                                                UniChar* to, UniChar* to_limit, UniChar*& to_next);

        //      called by fromUnicode
        result  doFromUnicode(const UniChar* from, const UniChar* from_end,
                                                const UniChar*& from_next,
                                                char* to, char* to_limit, char*& to_next);

        // Sets the maximum number of bytes generated by converting a UniChar.
        void            setMaximumBytesPerUniChar(length_type bytes);

        // Sets the maximum number of UniChars generated by converting
        // a multibyte char.
        void            setMaximumUniCharsPerByte(length_type bytes);

private:
        length_type                     fMaximumBytesPerUniChar;
        length_type                     fMaximumUniCharsPerByte;
};

#pragma enum(pop)
#pragma pack(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISO88591_
