// Revision: 11 1.5.1.5 source/core/text/intl/transcod/utf8tran.cpp, text, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: utf8tran.cpp
//  Function: Portable UTF-8 Transcoder implementation.
//  Author: Chen-Lieh Huang
//  Date created: 06/03/1996
//
#ifndef IC_ITRANCOD
#include "itrancod.hpp"
#endif

#ifndef IC_UTF8TRAN
#include "utf8tran.hpp"
#endif

#ifndef IC_TRANSCOD
#include "transcod.h"
#endif

#ifdef IC_PAGETUNE
        #define _UTF8TRAN_CPP
        #include <ipagetun.h>
#endif


static const int                                                halfShift                                               = 10;
static const unsigned long      halfBase                                                = 0x0010000UL;
static const unsigned long  halfMask                                            = 0x3FFUL;
static const unsigned long  kSurrogateHighStart = 0xD800UL;
static const unsigned long  kSurrogateHighEnd           = 0xDBFFUL;
static const unsigned long  kSurrogateLowStart  = 0xDC00UL;
static const unsigned long  kSurrogateLowEnd            = 0xDFFFUL;

static const unsigned long      kReplacementCharacter = 0x0000FFFDUL;
static const unsigned long  kMaximumUTF16 =                     0x0010FFFFUL;
static const unsigned long  kMaximumUCS2 =                      0x0000FFFFUL;

typedef unsigned long UCS4;
typedef unsigned short UCS2;
typedef unsigned short UTF16;
typedef unsigned char UTF8;

//static char gBytesFromUTF8[256] = {
static short gBytesFromUTF8[256] = {
        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
//      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
//      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
        2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5};

static UCS4 gOffsetsFromUTF8[6] =       {
        0x00000000UL, 0x00003080UL, 0x000E2080UL,
        0x03C82080UL, 0xFA082080UL, 0x82082080UL};

static UTF8 gFirstByteMark[7] = {0x00, 0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC};

static const UniChar utf8tran[]={'U','T','F','-','8',0x0000};


UTF8Transcoder::UTF8Transcoder()
        : fMaximumBytesPerUniChar(4),
                fMaximumUniCharsPerByte(1)
{
        setCharacterEncoding(IText(utf8tran));
}


UTF8Transcoder::~UTF8Transcoder()
{
}


UTF8Transcoder::UTF8Transcoder(const UTF8Transcoder &source)
        : ITranscoder(source),
                fMaximumBytesPerUniChar(source.fMaximumBytesPerUniChar),
                fMaximumUniCharsPerByte(source.fMaximumUniCharsPerByte)
{
}


UTF8Transcoder &
UTF8Transcoder::operator=(const UTF8Transcoder &right)
{
        if (this != &right)
        {
                ITranscoder::operator=(right);
                fMaximumBytesPerUniChar = right.fMaximumBytesPerUniChar;
                fMaximumUniCharsPerByte = right.fMaximumUniCharsPerByte;
        }

        return *this;
}


ITranscoder::length_type
UTF8Transcoder::maximumBytesPerUniChar() const
{
        return fMaximumBytesPerUniChar;
}


ITranscoder::length_type
UTF8Transcoder::maximumUniCharsPerByte() const
{
        return fMaximumUniCharsPerByte;
}


void
UTF8Transcoder::setMaximumBytesPerUniChar(length_type bytes)
{
        fMaximumBytesPerUniChar = bytes;
}


void
UTF8Transcoder::setMaximumUniCharsPerByte(length_type bytes)
{
        fMaximumUniCharsPerByte = bytes;
}


ITranscoder::length_type
UTF8Transcoder::byteBufferSize(const IText& uniText) const
{
        return uniText.length() * fMaximumBytesPerUniChar;
}


ITranscoder::length_type
UTF8Transcoder::byteBufferSize(
        const UniChar* from, const UniChar* from_end) const
{
        if (from < from_end)
                return (from_end - from) * fMaximumBytesPerUniChar;
        else
                return (from - from_end) * fMaximumBytesPerUniChar;
}


ITranscoder::length_type
UTF8Transcoder::uniCharBufferSize(const IString& text) const
{
        return text.length() * fMaximumUniCharsPerByte;
}


ITranscoder::length_type
UTF8Transcoder::uniCharBufferSize(
        const char* from, const char* from_end) const
{
        if (from < from_end)
                return (from_end - from) * fMaximumUniCharsPerByte;
        else
                return (from - from_end) * fMaximumUniCharsPerByte;
}


ITranscoder::result
UTF8Transcoder::doToUnicode(
                const char* from, const char* from_end, const char*& from_next,
                UniChar* to, UniChar* to_limit, UniChar*& to_next)
{
        // Initial condition checking
        if (from == NULL || to == NULL ||
                        from > from_end || to > to_limit ||
                        from == (char*)to)
        {
                IMessageText text(kConvBufRangeError, kMessageFile);
                IInvalidParameter exc(text, kConvBufRangeError, IException::recoverable);
                ITHROW(exc);
        }

        const unsigned char* source = (const unsigned char*)from;
        UniChar*                target = to;
        const unsigned char* actual_from_end = (const unsigned char*)from_end;
        short   extraBytesToWrite;

        while (source < actual_from_end)
        {
                UCS4 ch = 0;

                // Validate single-byte UTF-8 input character
                if ((extraBytesToWrite=gBytesFromUTF8[*source]) == -1)
                {
                        IMessageText text(kInvalidInputCharacter, kMessageFile);
                        IInvalidRequest exc(text);
                        ITHROW(exc);
                }

                if (source + extraBytesToWrite > actual_from_end)
                        // Source exhausted!
                        break;

                // Validate multi-byte UTF-8 input character
                if (extraBytesToWrite >= 1)
                {
                        for (short i=1; i<=extraBytesToWrite; i++)
                                if (gBytesFromUTF8[*(source+i)] != -1)
                                {
                                IMessageText text(kInvalidInputCharacter, kMessageFile);
                                IInvalidRequest exc(text);
                                ITHROW(exc);
                                }
                }

                switch(extraBytesToWrite) {     // note: code falls through cases.
                        case 5: ch += *source++; ch <<= 6;
                        case 4: ch += *source++; ch <<= 6;
                        case 3: ch += *source++; ch <<= 6;
                        case 2: ch += *source++; ch <<= 6;
                        case 1: ch += *source++; ch <<= 6;
                        case 0: ch += *source++;
                }
                ch -= gOffsetsFromUTF8[extraBytesToWrite];

                if (ch <= kMaximumUCS2) {
                        *target++ = (UniChar) ch;
                } else if (ch > kMaximumUTF16) {
                        // There are unmapped chars, handle based on unmappedBehavior.
                        ITranscoder::EUnmappedBehavior ub = unmappedBehavior();
                        switch (ub)
                        {
                        case ITranscoder::kOmit:
                                break;
                        case ITranscoder::kUseSub:
                                *target++ = kReplacementCharacter;
                                break;
                        case ITranscoder::kStop:
                                from_next = (const char*)source;
                                to_next = target;
                                return codecvt_base::partial;
                        }
                } else {
                        if (target + 1 >= to_limit) {
                                // Target exhausted!  Back up the source pointer.
                                source -= extraBytesToWrite + 1;
                                break;
                        }
                        ch -= halfBase;
                        *target++ = (UniChar) ((ch >> halfShift) + kSurrogateHighStart);
                        *target++ = (UniChar) ((ch & halfMask) + kSurrogateLowStart);
                }

                if (target >= to_limit) {
                        // Target exhausted!
                        break;
                }
        }
        from_next = (const char*)source;
        to_next = target;

        if (from_next == from)
                return codecvt_base::noconv;
        else if (from_next < from_end)
                return codecvt_base::partial;
        else return codecvt_base::ok;
}


ITranscoder::result
UTF8Transcoder::doFromUnicode(
                const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
                char* to, char* to_limit, char*& to_next)
{
        // Initial condition checking
        if (from == NULL || to == NULL ||
                        from > from_end || to > to_limit ||
                        (const char*)from == to)
        {
                IMessageText text(kConvBufRangeError, kMessageFile);
                IInvalidParameter exc(text, kConvBufRangeError, IException::recoverable);
                ITHROW(exc);
        }

        const UniChar*  source = from;
        UTF8*                                           target = (UTF8*)to;
        const UCS4                      byteMask = 0xBF;
        const UCS4                      byteMark = 0x80;
        while (source < from_end) {
                UCS4 ch;
                unsigned short bytesToWrite = 0;

                ch = *source++;
                if (ch>=kSurrogateHighStart && ch<=kSurrogateHighEnd && source<from_end)
                {
                        // a double unichar "Surrogate Pair" character
                        UCS4 ch2 = *source;
                        if (ch2 >= kSurrogateLowStart && ch2 <= kSurrogateLowEnd)
                        {
                                ch = ((ch - kSurrogateHighStart) << halfShift) +
                                                        (ch2 - kSurrogateLowStart) + halfBase;
                                ++source;
                        }
                        else    // Invalid input "Surrogate Pair"
                        {
                                IMessageText text(kInvalidInputCharacter, kMessageFile);
                                IInvalidRequest exc(text);
                                ITHROW(exc);
                        }
                }

                if (ch < 0x80) {                                                bytesToWrite = 1;
                } else if (ch < 0x800) {                bytesToWrite = 2;
                } else if (ch < 0x10000) {      bytesToWrite = 3;
                } else if (ch < 0x200000) {     bytesToWrite = 4;
//      IOC only supports UCS 2
//              } else if (ch < 0x4000000) {    bytesToWrite = 5;
//              } else if (ch <= kMaximumUCS4){ bytesToWrite = 6;
                } else {
                        // There are unmapped chars, handle based on unmappedBehavior.
                        ITranscoder::EUnmappedBehavior ub = unmappedBehavior();
                        switch (ub)
                        {
                        case ITranscoder::kOmit:
                                bytesToWrite = 0;
                                break;
                        case ITranscoder::kUseSub:
                                bytesToWrite = 2;
                                ch = kReplacementCharacter;
                                break;
                        case ITranscoder::kStop:
                                if (bytesToWrite >= 4)
                                        source -= 2;
                                else source--;
                                from_next = source;
                                to_next = (char *)target;
                                return codecvt_base::partial;
                        }
                }

                target += bytesToWrite;
                if (target > (UTF8 *)to_limit) {
                        // Target exhausted!  Back up the source and target pointers.
                        if (bytesToWrite >= 4)
                                source -= 2;
                        else source--;
                        target -= bytesToWrite;
                        break;
                }
                switch (bytesToWrite) { // note: code falls through cases!
                        case 6: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
                        case 5: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
                        case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
                        case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
                        case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
                        case 1: *--target = (UTF8) (ch | gFirstByteMark[bytesToWrite]);
                }
                target += bytesToWrite;
        }

        from_next = source;
        to_next = (char *)target;

        if (from_next == from)
                return codecvt_base::noconv;
        else if (from_next < from_end)
                return codecvt_base::partial;
        else return codecvt_base::ok;
}
