// Revision: 53 1.18.2.1 source/core/text/intl/transcod/win32/wintrans.cpp, text, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//  File: wintrans.cpp
//  Function: Win32 implementation of ITranscoder::CreateTranscoder()
//  Author: Chen-Lieh Huang
//  Date created: 03/22/1996
//
#ifndef __string_h
#include <string.h>
#endif
#ifndef __tchar_h
#include <tchar.h>
#endif
#include <windows.h>

#ifndef IC_IGENEXC
#include <igenexc.hpp>
#endif
#ifndef IC_ITRANCOD
#include <itrancod.hpp>
#endif
#ifndef IC_ISO88591
#include "iso88591.hpp"
#endif
#ifndef IC_UTF8TRAN
#include "utf8tran.hpp"
#endif

#ifndef IC_TRANSCOD
#include "transcod.h"
#endif

#ifdef IC_PAGETUNE
        #define _WINTRANS_CPP
        #include <ipagetun.h>
#endif


// None of the methods of this class can be overriden.
class IWin32Transcoder : public ITranscoder {
public:
        //      Standard member functions
        IWin32Transcoder();
        IWin32Transcoder(const IText& charSet, EMappingProximity proximity);
        IWin32Transcoder(const IWin32Transcoder& source);
        IWin32Transcoder& operator=(const IWin32Transcoder& right);
        virtual                         ~IWin32Transcoder();

        // Overrides
        length_type             maximumBytesPerUniChar() const;
        length_type             maximumUniCharsPerByte() const;

        length_type             byteBufferSize(const IText& uniText) const;
        length_type             byteBufferSize(const UniChar* from, const UniChar* from_end) const;
        length_type             uniCharBufferSize(const IString& text) const;
        length_type             uniCharBufferSize(const char* from, const char* from_end) const;

        static const UniChar*   portableCSet(const UINT codePage);
                // used by ITranscoder::characterSet()

protected:
        //      called by toUnicode
        result doToUnicode(const char* from, const char* from_end,
                                                const char*& from_next,
                                                UniChar* to, UniChar* to_limit, UniChar*& to_next);

        //      called by fromUnicode
        result doFromUnicode(const UniChar* from, const UniChar* from_end,
                                                const UniChar*& from_next,
                                                char* to, char* to_limit, char*& to_next);

        // Sets the maximum number of bytes generated by converting a UniChar.
        void                    setMaximumBytesPerUniChar(length_type bytes);

        // Sets the maximum number of UniChars generated by converting
        // a multibyte char.
        void                    setMaximumUniCharsPerByte(length_type bytes);

private:
        UINT                                    codePage(const IText& characterSet, EMappingProximity proximity) const;

        UINT                                    fCurrentCP;
        length_type     fMaximumBytesPerUniChar;
        length_type     fMaximumUniCharsPerByte;
};


typedef struct CharacterSetToCodePageMappingType {
        char* cSet;
        UINT    codePage;
        UINT    proximity;
} CSetToCodePageMappingType;

// The pre-sorted mapping table that maps the portable name of
// a character set to a corresponding Microsoft supported code page.
static const CSetToCodePageMappingType cSetMap[] =      // index
{       {"GB-2312",                     936, ITranscoder::kExactMapping},               // 0
        {"ISO-8859-1", 1252, ITranscoder::kExactMapping},               // 1
        {"ISO-8859-2", 1250, ITranscoder::kCloseMapping},               // 2
        {"ISO-8859-7", 1253, ITranscoder::kCloseMapping},               // 3
        {"ISO-8859-8", 1255, ITranscoder::kCloseMapping},               // 4
        {"ISO-8859-9", 1254, ITranscoder::kSupersetMapping},// 5
        {"KSC-5601",            949, ITranscoder::kExactMapping},               // 6
        {"MSCP-10000",10000, ITranscoder::kExactMapping},               // 7
        {"MSCP-1250",  1250, ITranscoder::kExactMapping},               // 8
        {"MSCP-1251",  1251, ITranscoder::kExactMapping},               // 9
        {"MSCP-1252",  1252, ITranscoder::kExactMapping},               // 10
        {"MSCP-1253",  1253, ITranscoder::kExactMapping},               // 11
        {"MSCP-1254",  1254, ITranscoder::kExactMapping},               // 12
        {"MSCP-1255",  1255, ITranscoder::kExactMapping},               // 13
        {"MSCP-1256",    1256, ITranscoder::kExactMapping},             // 14
        {"MSCP-437",            437, ITranscoder::kExactMapping},               // 15
        {"MSCP-850",            850, ITranscoder::kExactMapping},               // 16
        {"MSCP-936",            936, ITranscoder::kExactMapping},               // 17
        {"Shift-JIS",           932, ITranscoder::kExactMapping},               // 18
        {"US-ASCII",     1252, ITranscoder::kExactMapping},             // 19
        {"UTF-8",                                       0, ITranscoder::kExactMapping}          // 20
};

static const UINT numCSets = sizeof(cSetMap)/sizeof(CSetToCodePageMappingType);


typedef struct CodePageToCharacterSetMapping {
        UINT            codePage;
        UniChar cSet[20];
} CodePageToCSetMappingType;

// The pre-sorted mapping table that maps a Microsoft supported
// code page to the corresponding portable name of a character set.
static const CodePageToCSetMappingType cpMap[] =                                        // index
{       {437,           {'M','S','C','P','-','4','3','7',0x0000}},                              // 0
        {850,           {'M','S','C','P','-','8','5','0',0x0000}},                              // 1
        {932,           {'S','h','i','f','t','-','J','I','S',0x0000}},          // 2
        {936,           {'G','B','-','2','3','1','2',0x0000}},                                          // 3
        {949,           {'K','S','C','-','5','6','0','1',0x0000}},                              // 4
        {1250,  {'M','S','C','P','-','1','2','5','0',0x0000}},          // 5
        {1251,  {'M','S','C','P','-','1','2','5','1',0x0000}},          // 6
        {1252,  {'M','S','C','P','-','1','2','5','2',0x0000}},          // 7
        {1253,  {'M','S','C','P','-','1','2','5','3',0x0000}},          // 8
        {1254,  {'M','S','C','P','-','1','2','5','4',0x0000}},          // 9
        {1255,  {'M','S','C','P','-','1','2','5','5',0x0000}},          // 10
        {1256,  {'M','S','C','P','-','1','2','5','6',0x0000}},          // 11
        {10000, {'M','S','C','P','-','1','0','0','0','0',0x0000}}       // 12
};

static const UINT numCodePages = sizeof(cpMap)/sizeof(CodePageToCSetMappingType);

static const UniChar iso88591[]={'I','S','O','-','8','8','5','9','-','1',0x0000};
static const UniChar utf8[]={'U','T','F','-','8',0x0000};
//static const UniChar usascii[]={'U','S','-','A','S','C','I','I',0x0000};


//
//      Platform-specific APIs implementation.
//
ILineBreakConverter::ELineBreakConvention
ILineBreakConverter::hostConvention()
{
        return ILineBreakConverter::kCRLF;
}


const IText
ICharacterSetIterator::operator*()
{
        if (fCurCharSetIdx >= numCSets)
                return IText(); // pass-the-end
        else return (IText(cSetMap[fCurCharSetIdx].cSet));
}


ICharacterSetIterator::operator bool() const
{
        if (fCurCharSetIdx >= numCSets)
                return false;   // pass-the-end
        else return true;
}


const IText
ITranscoder::characterSet(const ILocaleKey& key)
{
        LCID    lcid = key.hostID();
        char    lcData[256];
        UINT    codePage;

        GetLocaleInfo(lcid, LOCALE_IDEFAULTANSICODEPAGE, (LPTSTR)lcData, 256);

        if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
                int             lcDataLen = GetLocaleInfo(lcid, LOCALE_IDEFAULTANSICODEPAGE, NULL, 0);
                char*   lcData2 = new char[lcDataLen+1];

                GetLocaleInfo(lcid, LOCALE_IDEFAULTANSICODEPAGE, (LPTSTR)lcData2, lcDataLen);
                codePage = atoi(lcData2);
                delete [] lcData2;
        }
        else
        {
                codePage = atoi(lcData);
        if (!IsValidCodePage(codePage))
        {
            IMessageText text(kTranscoderNotInstalled, kMessageFile);
            IObjectNotFound exc(text);
            ITHROW(exc);
            }
        }

        return IText(IWin32Transcoder::portableCSet(codePage));
}


//  Win32 implementation of ITranscoder::createTranscoder().
ITranscoder *
ITranscoder::createTranscoder(const IText& charSet, EMappingProximity proximity)
{
        // If charSet is ISO-8859-1, the conversion is handled
        // with zero-extended to Unicode.
        if (charSet == IText(iso88591))
                return new IISO8859_1Transcoder();
        else if (charSet == IText(utf8))
                return new UTF8Transcoder();
        else if (charSet.empty())
                return new IWin32Transcoder();
        else
                return new IWin32Transcoder(charSet, proximity);
}


//
//      This returns the default host transcoder.  If the default host character set
//  can be exchangeable with ISO8859-1, the IISO8859_1Transcoder will be
//      returned for fast performance.
//
ITranscoder *
ITranscoder::createTranscoder()
{
        return new IWin32Transcoder();
}


//
//      Win32 Transcoder implementation
//
UINT
IWin32Transcoder::codePage(const IText& charSet, EMappingProximity proximity) const
{
        // Convert charSet from IText to char* algoritmically, which
        // turns IText into UniChar, and then converts from UniChar
        // to char* using the ISO-8859-1 transcoding algorithm.
        length_type     length = charSet.length();
        size_t  dummy1, dummy2;

        const UniChar* uniCharSet = charSet.storage_chunk(0, dummy1, dummy2);

        char csName[256];       // charSet name size never greater than 256.
        for (int i=0; i<length; i++)
                csName[i] = (unsigned char)uniCharSet[i];
        csName[length] = '\0';

        // Binary search for the codePage using csName.
        // Should enhance BS with appropriate collection when it's available.
        int     low = 0, mid = 0;
        int     high = numCSets - 1;

        while (low <= high) {
                mid = (low + high) / 2;
                if (strcmp(csName, cSetMap[mid].cSet) < 0)
                        high = mid - 1;
                else if (strcmp(csName, cSetMap[mid].cSet) > 0)
                        low = mid + 1;
                else if (proximity >= cSetMap[mid].proximity) // match found!
                        return cSetMap[mid].codePage;
                else return 0;  // charset matched, but proximity was not met!
                }
        return 0;
}


const UniChar*
IWin32Transcoder::portableCSet(const UINT codePage)
{
        if (!IsValidCodePage(codePage))
        {
                IMessageText text(kTranscoderNotInstalled, kMessageFile);
                IObjectNotFound exc(text);
                ITHROW(exc);
        }

        // Binary search for the mapping entry for codePage specified.
        int     low = 0, mid = 0;
        int     high = numCodePages - 1;

        while (low <= high) {
                mid = (low + high) / 2;
                if (codePage < cpMap[mid].codePage)
                        high = mid - 1;
                else if (codePage > cpMap[mid].codePage)
                        low = mid + 1;
                else return cpMap[mid].cSet;    // found match!
                }
        return NULL;
}


IWin32Transcoder::IWin32Transcoder()
{
        fCurrentCP = GetACP();
        CPINFO  cpInfoStruct;
        GetCPInfo (fCurrentCP, &cpInfoStruct);

        fMaximumBytesPerUniChar = cpInfoStruct.MaxCharSize;
        fMaximumUniCharsPerByte = 1;

        // Search the name of the character set based on
        // the default code page fCurrentCP, and set the
        // Character Encoding using this name.
        const UniChar *cSet;
        if ((cSet=portableCSet(fCurrentCP)) != NULL)
                setCharacterEncoding(IText(cSet));
        else setCharacterEncoding(IText());
}


IWin32Transcoder::IWin32Transcoder(const IText& charSet, EMappingProximity proximity)
{
        // Map charSet to Code Page. If the specified charSet
        // can't be mapped to a non-zero code page, that means
        // the charSet specified is not supported, and should throw
        // kNoAdequateTranscoder exception.
        if ((fCurrentCP=codePage(charSet, proximity)) == 0)
        {
                IMessageText text(kNoAdequateTranscoder, kMessageFile);
                IObjectNotFound exc(text);
                ITHROW(exc);
        }

        // Check if the code page is installed in this Win32 system.
        // If it isn't, throw kTranscoderNotInstalled exception.
        if (!IsValidCodePage(fCurrentCP))
        {
                IMessageText text(kTranscoderNotInstalled, kMessageFile);
                IObjectNotFound exc(text);
                ITHROW(exc);
        }

        CPINFO  cpInfoStruct;
        GetCPInfo (fCurrentCP, &cpInfoStruct);

        fMaximumBytesPerUniChar = cpInfoStruct.MaxCharSize;
        fMaximumUniCharsPerByte = 1;
        setCharacterEncoding(charSet);
}


IWin32Transcoder::~IWin32Transcoder()
{
}


IWin32Transcoder::IWin32Transcoder(const IWin32Transcoder& source)
        : ITranscoder(source),
                fCurrentCP(source.fCurrentCP),
                fMaximumBytesPerUniChar(source.fMaximumBytesPerUniChar),
                fMaximumUniCharsPerByte(source.fMaximumUniCharsPerByte)
{
}


IWin32Transcoder &
IWin32Transcoder::operator=(const IWin32Transcoder& right)
{
        if (this != &right)
        {
                ITranscoder::operator=(right);
                fCurrentCP = right.fCurrentCP;
                fMaximumBytesPerUniChar = right.fMaximumBytesPerUniChar;
                fMaximumUniCharsPerByte = right.fMaximumUniCharsPerByte;
        }

        return *this;
}


ITranscoder::length_type
IWin32Transcoder::maximumBytesPerUniChar() const
{
        return fMaximumBytesPerUniChar;
}


ITranscoder::length_type
IWin32Transcoder::maximumUniCharsPerByte() const
{
        return fMaximumUniCharsPerByte;
}


void
IWin32Transcoder::setMaximumBytesPerUniChar(length_type bytes)
{
        fMaximumBytesPerUniChar = bytes;
}


void
IWin32Transcoder::setMaximumUniCharsPerByte(length_type bytes)
{
        fMaximumUniCharsPerByte = bytes;
}


ITranscoder::length_type
IWin32Transcoder::byteBufferSize(const IText& uniText) const
{
        return uniText.length() * fMaximumBytesPerUniChar;
}


ITranscoder::length_type
IWin32Transcoder::byteBufferSize(
        const UniChar* from, const UniChar* from_end) const
{
        if (from < from_end)
                return (from_end - from) * fMaximumBytesPerUniChar;
        else
                return (from - from_end) * fMaximumBytesPerUniChar;
}


ITranscoder::length_type
IWin32Transcoder::uniCharBufferSize(const IString& text) const
{
        return text.length() * fMaximumUniCharsPerByte;
}


ITranscoder::length_type
IWin32Transcoder::uniCharBufferSize(
        const char* from, const char* from_end) const
{
        if (from < from_end)
                return (from_end - from) * fMaximumUniCharsPerByte;
        else
                return (from - from_end) * fMaximumUniCharsPerByte;
}


ITranscoder::result
IWin32Transcoder::doToUnicode(
        const char* from, const char* from_end, const char*& from_next,
        UniChar* to, UniChar* to_limit, UniChar*& to_next)
{
        // Initial conditions checking
        if (from == NULL || to == NULL ||
                        from > from_end || to > to_limit ||
                        from == (char*)to)
        {
                IInvalidParameter       exc(
                        IMessageText(kConvBufRangeError, kMessageFile),
                        kConvBufRangeError,
                        IException::recoverable);
                ITHROW(exc);
        }

        UniChar conversionBuffer[1024]; // fix-size conversion buffer

        int     dwSizeIn = from_end - from;
        int     dwSizeOut = sizeof(conversionBuffer)/sizeof(UniChar);

        LPCSTR  inBuf = from;
        LPWSTR  outBuf = (WCHAR*) conversionBuffer;
        long    lengthConverted, lengthToCopy, inputConverted;

        from_next = from;
        to_next = to;
        while (from_next < from_end)
        {
                // this length is number of wide chars
                lengthConverted = MultiByteToWideChar(fCurrentCP,
                                                      MB_ERR_INVALID_CHARS,   // returns 0 if hits an unmapped/invalid character.
                                                      inBuf,
                                                      dwSizeIn,
                                                      outBuf,
                                                      dwSizeOut);

                // now find out how many bytes from input have been processed
                if (lengthConverted) {
                   inputConverted = WideCharToMultiByte(fCurrentCP, // ANSI code page is the default
                                                        WC_COMPOSITECHECK,  // Convert composite chars to precomposed chars.
                                                        outBuf,
                                                        lengthConverted,
                                                        NULL,
                                                        0, // just tell us how many bytes are needed
                                                        NULL, // faster if last 2 are NULL
                                                        NULL
                                                        );
                }
                else {
                   inputConverted = lengthConverted;
                }

                DWORD   dwErrCode = NO_ERROR;

                if (lengthConverted == 0)
                {
                        // OutBuf might be insufficient, or ecountered unmapped character.
                        dwErrCode = GetLastError();

                        if (dwErrCode == ERROR_INSUFFICIENT_BUFFER)
                                // OutBuf size is insufficient, continue to convert.
                                lengthConverted = dwSizeOut;
                        else if (dwErrCode == ERROR_NO_UNICODE_TRANSLATION)
                        {
                                // Use WinNT default Unicode sub char - could be 0xFFFD.
                                lengthConverted = MultiByteToWideChar (
                                        fCurrentCP,
                                        MB_USEGLYPHCHARS,       // MB_ERR_INVALID_CHARS not set to translate to
                                        inBuf,                                          // WinNT default sub char.
                                        dwSizeIn,
                                        outBuf,
                                        dwSizeOut);
                                if (lengthConverted == 0 && GetLastError() == ERROR_INSUFFICIENT_BUFFER)
                                        lengthConverted = dwSizeOut;
                        }
                }

                // Copy out conversionBuffer so that it can be reused in next round of conversion.
                if ((to_next + lengthConverted) - to_limit > 0)
                        lengthToCopy = to_limit - to_next;
                else lengthToCopy = lengthConverted;

                from_next += inputConverted;

                // Copy out the fix buffer conversionBuffer.
                UniChar* up = NULL;
                if (dwErrCode == ERROR_NO_UNICODE_TRANSLATION)
                {
                        // There are unmapped chars, handle based on unmappedBehavior.
                        ITranscoder::EUnmappedBehavior ub = unmappedBehavior();
                        switch (ub)
                        {
                        case ITranscoder::kOmit:
                                for (up = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        if (*up == 0xFFFD)
                                                up++;   // omit the WinNT unisub which is converted from an unmapped char.
                                        else
                                                *to_next++ = *up++;
                                break;
                        case ITranscoder::kUseSub:      // WinNT unisub ==> uniCharSubstitute()
                                for (up = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        if (*up == 0xFFFD)      // needed if WinNT unisub is not 0xFFFD
                                        {                                                                               // Otherwise, remove this code.
                                                *to_next++ = uniCharSubstitute();
                                                up++;
                                        }
                                        else
                                                *to_next++ = *up++;
                                break;
                        case ITranscoder::kStop:
                                for (up = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        if (*up == 0xFFFD)
                                        {
                                                from_next -= lengthToCopy;      // reset from_next to first WinNT unisub
                                                return codecvt_base::partial;
                                        }
                                        else
                                                *to_next++ = *up++;
                                break;
                        }
                }
                else    // There is no unmapped char, use normal copy.
                        for (up = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                *to_next++ = *up++;

                if (to_next == to_limit)        // we are done with required conversion
                        break;

                inBuf = from_next;
                dwSizeIn = from_end - from_next;
        }

        if (lengthConverted == 0)
                return codecvt_base::error;
        else if (from_next < from_end)
                return codecvt_base::partial;
        else if (from_next == from)
                return codecvt_base::noconv;
        else return codecvt_base::ok;
}


ITranscoder::result
IWin32Transcoder::doFromUnicode(
        const UniChar* from, const UniChar* from_end, const UniChar*& from_next,
        char* to, char* to_limit, char*& to_next)
{
        // Initial conditions checking
        if (from == NULL || to == NULL ||
                        from > from_end || to > to_limit ||
                        (const char*)from == to)
        {
                IInvalidParameter       exc(
                        IMessageText(kConvBufRangeError, kMessageFile),
                        kConvBufRangeError,
                        IException::recoverable);
                ITHROW(exc);
        }

        char conversionBuffer[1024];    // fix-size conversion buffer

        int     dwSizeIn = from_end - from;
        int     dwSizeOut = sizeof(conversionBuffer);

        LPCWSTR inBuf = (LPCWSTR) from;
        LPSTR           outBuf = conversionBuffer;
        long            lengthConverted, lengthToCopy;

        BOOL bUsedDef = FALSE;

        // Always use substitute char if unmapped char exists.
        char subChar[2];
        subChar[0] = charSubstitute();
        subChar[1] = '\0';

        from_next = from;
        to_next = to;
        while (from_next < from_end)
        {
                lengthConverted = WideCharToMultiByte (
                        fCurrentCP,     // ANSI code page is the default
                        WC_COMPOSITECHECK,     // Convert composite chars to precomposed chars.
                        inBuf,
                        dwSizeIn,
                        outBuf,
                        dwSizeOut,
                        subChar,
                        &bUsedDef       // Indicates if subChar used during conversion?
                        );

                if (lengthConverted == 0)               // OutBuf may be insufficient
                {
                        if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
                                // OutBuf is insufficient, continue to convert.
                                lengthConverted = dwSizeOut;
                        else
                                // Other 2 possible errors this API can generate are
                                // ERROR_INVALID_FLAGS and ERROR_INVALID_PARAMETER,
                                // which should never happen since they have been
                                // assured before reaching this point.
                                break;
                }

                if ((to_next + lengthConverted) - to_limit > 0)
                        lengthToCopy = to_limit - to_next;
                else lengthToCopy = lengthConverted;

                from_next += lengthToCopy;

                // Copy out the fix buffer conversionBuffer.
                char* p = NULL;
                if (bUsedDef)
                {
                        // There are unmapped chars, handle based on unmappedBehavior.
                        ITranscoder::EUnmappedBehavior ub = unmappedBehavior();
                        switch (ub)
                        {
                        case ITranscoder::kOmit:
                                for (p = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        if (*p == subChar[0])
                                                p++;    // omit the subChar which is converted from an unmapped char.
                                        else
                                                *to_next++ = *p++;
                                break;
                        case ITranscoder::kUseSub:      // Sub done in WideCharToMultiByte, simply copy out!
                                for (p = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        *to_next++ = *p++;
                                break;
                        case ITranscoder::kStop:
                                for (p = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                        if (*p == subChar[0])
                                        {
                                                from_next -= lengthToCopy;      // reset from_next to first subChar[0].
                                                return codecvt_base::partial;
                                        }
                                        else
                                                *to_next++ = *p++;
                                break;
                        }
                }
                else    // There is no unmapped char, simply copy out.
                        for (p = conversionBuffer; lengthToCopy > 0; lengthToCopy--)
                                *to_next++ = *p++;

                if (to_next == to_limit)        // we are done with required conversion
                        break;

                inBuf = (LPCWSTR) from_next;
                dwSizeIn = from_end - from_next;
        }

        if (lengthConverted == 0)
                return codecvt_base::error;
        else if (from_next < from_end)
                return codecvt_base::partial;
        else if (from_next == from)
                return codecvt_base::noconv;
        else return codecvt_base::ok;
}
