/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 57 1.15.1.7 source/core/text/itext/istyles.cpp, text, ioc.v400, 000302 

#ifndef IC_ISTYLES
#include "istyles.hpp"
#endif

#ifndef IC_ISTYLSET
#include "istylset.hpp"
#endif

#ifdef IC_PAGETUNE
        #define _ISTYLES_CPP
        #include <ipagetun.h>
#endif

//*******************************************************
//      class ITextTypefaceStyle
//*******************************************************

ITextTypefaceStyle::ITextTypefaceStyle() :
                                        ICharacterStyle(),
                                        fValue()
{
}


ITextTypefaceStyle::ITextTypefaceStyle( const IText& name ) :
                                        ICharacterStyle(),
                                        fValue( name )
{
}



ITextTypefaceStyle::ITextTypefaceStyle( const ITextTypefaceStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextTypefaceStyle::~ITextTypefaceStyle()
{
}


ITextTypefaceStyle&
ITextTypefaceStyle::operator=( const ITextTypefaceStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


IText
ITextTypefaceStyle::typeface() const
{
        return fValue;
}


void
ITextTypefaceStyle::setTypeface( const IText& value )
{
        fValue = value;
}


void
ITextTypefaceStyle::addToSet(ITextStyleSet&     set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasTypefaceStyle);
        set.setTypefaceValue(fValue);
}

void
ITextTypefaceStyle::removeFromSet(ITextStyleSet&        set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasTypefaceStyle);
}

void
ITextTypefaceStyle::getValueFromSet(const ITextStyleSet&        set)
{
        fValue = set.typefaceValue();
}

bool
ITextTypefaceStyle::isContainedInSet(const ITextStyleSet&       set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasTypefaceStyle));
}

bool
ITextTypefaceStyle::isValueContainedInSet(const ITextStyleSet&  set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasTypefaceStyle))
                return set.typefaceValue() == fValue;
        else
                return false;
}

bool
ITextTypefaceStyle::operator==( const ITextTypefaceStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextTypefaceStyle::operator!=( const ITextTypefaceStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextPointSizeStyle
//*******************************************************

ITextPointSizeStyle::ITextPointSizeStyle() :
                                        ICharacterStyle(),
                                        fValue( 0 )
{
}


ITextPointSizeStyle::ITextPointSizeStyle( float value ) :
                                        ICharacterStyle(),
                                        fValue( value )
{
}


ITextPointSizeStyle::ITextPointSizeStyle( const ITextPointSizeStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextPointSizeStyle::~ITextPointSizeStyle()
{
}


ITextPointSizeStyle&
ITextPointSizeStyle::operator=( const ITextPointSizeStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


float
ITextPointSizeStyle::pointSize() const
{
        return fValue;
}

void
ITextPointSizeStyle::setPointSize( float value )
{
        fValue = value;
}


void
ITextPointSizeStyle::addToSet(ITextStyleSet&    set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasPointSizeStyle);
        set.setFloatValue(ITextStyleSet::kPointSize, fValue);
}

void
ITextPointSizeStyle::removeFromSet(ITextStyleSet&       set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasPointSizeStyle);
}

void
ITextPointSizeStyle::getValueFromSet(const ITextStyleSet&       set)
{
        fValue = set.floatValue(ITextStyleSet::kPointSize);
}

bool
ITextPointSizeStyle::isContainedInSet(const ITextStyleSet&      set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasPointSizeStyle));
}

bool
ITextPointSizeStyle::isValueContainedInSet(const ITextStyleSet& set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasPointSizeStyle))
                return fValue == set.floatValue(ITextStyleSet::kPointSize);
        else
                return false;
}

bool
ITextPointSizeStyle::operator==( const ITextPointSizeStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextPointSizeStyle::operator!=( const ITextPointSizeStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextBoldfaceStyle
//*******************************************************

ITextBoldfaceStyle::ITextBoldfaceStyle() :
                                        ICharacterStyle()
{
}


ITextBoldfaceStyle::ITextBoldfaceStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextBoldfaceStyle::ITextBoldfaceStyle( const ITextBoldfaceStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextBoldfaceStyle::~ITextBoldfaceStyle()
{
}


ITextBoldfaceStyle&
ITextBoldfaceStyle::operator=( const ITextBoldfaceStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextBoldfaceStyle::setBoldface( bool value )
{
        fValue = value;
}


bool
ITextBoldfaceStyle::isBoldface() const
{
        return fValue;
}


void
ITextBoldfaceStyle::addToSet(ITextStyleSet&     set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasBoldfaceStyle);
        set.setBooleanValue(ITextStyleSet::kBoldface, fValue);
}

void
ITextBoldfaceStyle::removeFromSet(ITextStyleSet&        set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasBoldfaceStyle);
}

void
ITextBoldfaceStyle::getValueFromSet(const ITextStyleSet&        set)
{
        fValue = set.booleanValue(ITextStyleSet::kBoldface);
}

bool
ITextBoldfaceStyle::isContainedInSet(const ITextStyleSet&       set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasBoldfaceStyle));
}

bool
ITextBoldfaceStyle::isValueContainedInSet(const ITextStyleSet&  set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasBoldfaceStyle))
                return fValue == set.booleanValue(ITextStyleSet::kBoldface);
        else
                return false;
}

bool
ITextBoldfaceStyle::operator==( const ITextBoldfaceStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextBoldfaceStyle::operator!=( const ITextBoldfaceStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextItalicStyle
//*******************************************************

ITextItalicStyle::ITextItalicStyle() :
                                        ICharacterStyle()
{
}


ITextItalicStyle::ITextItalicStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextItalicStyle::ITextItalicStyle( const ITextItalicStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextItalicStyle::~ITextItalicStyle()
{
}


ITextItalicStyle&
ITextItalicStyle::operator=( const ITextItalicStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextItalicStyle::setItalic( bool value )
{
        fValue = value;
}


bool
ITextItalicStyle::isItalic() const
{
        return fValue;
}


void
ITextItalicStyle::addToSet(ITextStyleSet&       set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasItalicStyle);
        set.setBooleanValue(ITextStyleSet::kItalic, fValue);
}

void
ITextItalicStyle::removeFromSet(ITextStyleSet&  set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasItalicStyle);
}

void
ITextItalicStyle::getValueFromSet(const ITextStyleSet&  set)
{
        fValue = set.booleanValue(ITextStyleSet::kItalic);
}

bool
ITextItalicStyle::isContainedInSet(const ITextStyleSet& set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasItalicStyle));
}

bool
ITextItalicStyle::isValueContainedInSet(const ITextStyleSet&    set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasItalicStyle))
                return fValue == set.booleanValue(ITextStyleSet::kItalic);
        else
                return false;
}

bool
ITextItalicStyle::operator==( const ITextItalicStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextItalicStyle::operator!=( const ITextItalicStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextUnderlineStyle
//*******************************************************

ITextUnderlineStyle::ITextUnderlineStyle() :
                                        ICharacterStyle()
{
}


ITextUnderlineStyle::ITextUnderlineStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextUnderlineStyle::ITextUnderlineStyle( const ITextUnderlineStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextUnderlineStyle::~ITextUnderlineStyle()
{
}


ITextUnderlineStyle&
ITextUnderlineStyle::operator=( const ITextUnderlineStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextUnderlineStyle::setUnderlined( bool value )
{
        fValue = value;
}


bool
ITextUnderlineStyle::isUnderlined() const
{
        return fValue;
}


void
ITextUnderlineStyle::addToSet(ITextStyleSet&    set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasUnderlineStyle);
        set.setBooleanValue(ITextStyleSet::kUnderline, fValue);
}

void
ITextUnderlineStyle::removeFromSet(ITextStyleSet&       set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasUnderlineStyle);
}

void
ITextUnderlineStyle::getValueFromSet(const ITextStyleSet&       set)
{
        fValue = set.booleanValue(ITextStyleSet::kUnderline);
}

bool
ITextUnderlineStyle::isContainedInSet(const ITextStyleSet&      set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasUnderlineStyle));
}

bool
ITextUnderlineStyle::isValueContainedInSet(const ITextStyleSet& set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasUnderlineStyle))
                return fValue == set.booleanValue(ITextStyleSet::kUnderline);
        else
                return false;
}

bool
ITextUnderlineStyle::operator==( const ITextUnderlineStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextUnderlineStyle::operator!=( const ITextUnderlineStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextStrikethroughStyle
//*******************************************************

ITextStrikethroughStyle::ITextStrikethroughStyle() :
                                        ICharacterStyle()
{
}


ITextStrikethroughStyle::ITextStrikethroughStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextStrikethroughStyle::ITextStrikethroughStyle( const ITextStrikethroughStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextStrikethroughStyle::~ITextStrikethroughStyle()
{
}


ITextStrikethroughStyle&
ITextStrikethroughStyle::operator=( const ITextStrikethroughStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextStrikethroughStyle::setStrikethrough( bool value )
{
        fValue = value;
}


bool
ITextStrikethroughStyle::isStrikethrough() const
{
        return fValue;
}


void
ITextStrikethroughStyle::addToSet(ITextStyleSet&        set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasStrikethroughStyle);
        set.setBooleanValue(ITextStyleSet::kStrikethrough, fValue);
}

void
ITextStrikethroughStyle::removeFromSet(ITextStyleSet&   set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasStrikethroughStyle);
}

void
ITextStrikethroughStyle::getValueFromSet(const ITextStyleSet&   set)
{
        fValue = set.booleanValue(ITextStyleSet::kStrikethrough);
}

bool
ITextStrikethroughStyle::isContainedInSet(const ITextStyleSet&  set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasStrikethroughStyle));
}

bool
ITextStrikethroughStyle::isValueContainedInSet(const ITextStyleSet&     set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasStrikethroughStyle))
                return fValue == set.booleanValue(ITextStyleSet::kStrikethrough);
        else
                return false;
}

bool
ITextStrikethroughStyle::operator==( const ITextStrikethroughStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextStrikethroughStyle::operator!=( const ITextStrikethroughStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextOutlineStyle
//*******************************************************

ITextOutlineStyle::ITextOutlineStyle() :
                                        ICharacterStyle()
{
}


ITextOutlineStyle::ITextOutlineStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextOutlineStyle::ITextOutlineStyle( const ITextOutlineStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextOutlineStyle::~ITextOutlineStyle()
{
}


ITextOutlineStyle&
ITextOutlineStyle::operator=( const ITextOutlineStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextOutlineStyle::setOutlined( bool value )
{
        fValue = value;
}


bool
ITextOutlineStyle::isOutlined() const
{
        return fValue;
}


void
ITextOutlineStyle::addToSet(ITextStyleSet&      set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasOutlineStyle);
        set.setBooleanValue(ITextStyleSet::kOutline, fValue);
}

void
ITextOutlineStyle::removeFromSet(ITextStyleSet& set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasOutlineStyle);
}

void
ITextOutlineStyle::getValueFromSet(const ITextStyleSet& set)
{
        fValue = set.booleanValue(ITextStyleSet::kOutline);
}

bool
ITextOutlineStyle::isContainedInSet(const ITextStyleSet&        set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasOutlineStyle));
}

bool
ITextOutlineStyle::isValueContainedInSet(const ITextStyleSet&   set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasOutlineStyle))
                return fValue == set.booleanValue(ITextStyleSet::kOutline);
        else
                return false;
}

bool
ITextOutlineStyle::operator==( const ITextOutlineStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextOutlineStyle::operator!=( const ITextOutlineStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextUneditableStyle
//*******************************************************

ITextUneditableStyle::ITextUneditableStyle() :
                                        ICharacterStyle()
{
}


ITextUneditableStyle::ITextUneditableStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextUneditableStyle::ITextUneditableStyle( const ITextUneditableStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextUneditableStyle::~ITextUneditableStyle()
{
}


ITextUneditableStyle&
ITextUneditableStyle::operator=( const ITextUneditableStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextUneditableStyle::setUneditable( bool value )
{
        fValue = value;
}


bool
ITextUneditableStyle::isUneditable() const
{
        return fValue;
}


void
ITextUneditableStyle::addToSet(ITextStyleSet&   set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasUneditableStyle);
        set.setBooleanValue(ITextStyleSet::kUneditable, fValue);
}

void
ITextUneditableStyle::removeFromSet(ITextStyleSet&      set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasUneditableStyle);
}

void
ITextUneditableStyle::getValueFromSet(const ITextStyleSet&      set)
{
        fValue = set.booleanValue(ITextStyleSet::kUneditable);
}

bool
ITextUneditableStyle::isContainedInSet(const ITextStyleSet&     set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasUneditableStyle));
}

bool
ITextUneditableStyle::isValueContainedInSet(const ITextStyleSet&        set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasUneditableStyle))
                return fValue == set.booleanValue(ITextStyleSet::kUneditable);
        else
                return false;
}

bool
ITextUneditableStyle::operator==( const ITextUneditableStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextUneditableStyle::operator!=( const ITextUneditableStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextBitmapStyle
//*******************************************************

ITextBitmapStyle::ITextBitmapStyle() :
                                        ICharacterStyle()
{
}


ITextBitmapStyle::ITextBitmapStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextBitmapStyle::ITextBitmapStyle( const ITextBitmapStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextBitmapStyle::~ITextBitmapStyle()
{
}


ITextBitmapStyle&
ITextBitmapStyle::operator=( const ITextBitmapStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextBitmapStyle::setBitmap( bool value )
{
        fValue = value;
}


bool
ITextBitmapStyle::isBitmap() const
{
        return fValue;
}


void
ITextBitmapStyle::addToSet(ITextStyleSet&       set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasBitmapStyle);
        set.setBooleanValue(ITextStyleSet::kBitmap, fValue);
}

void
ITextBitmapStyle::removeFromSet(ITextStyleSet&  set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasBitmapStyle);
}

void
ITextBitmapStyle::getValueFromSet(const ITextStyleSet&  set)
{
        fValue = set.booleanValue(ITextStyleSet::kBitmap);
}

bool
ITextBitmapStyle::isContainedInSet(const ITextStyleSet& set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasBitmapStyle));
}

bool
ITextBitmapStyle::isValueContainedInSet(const ITextStyleSet&    set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasBitmapStyle))
                return fValue == set.booleanValue(ITextStyleSet::kBitmap);
        else
                return false;
}

bool
ITextBitmapStyle::operator==( const ITextBitmapStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextBitmapStyle::operator!=( const ITextBitmapStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextColorStyle
//*******************************************************

ITextColorStyle::ITextColorStyle() :
                                        ICharacterStyle(),
                                        fRed(0),
                                        fGreen(0),
                                        fBlue(0)
{
}


ITextColorStyle::ITextColorStyle(       unsigned char   red,
                                                                        unsigned char   green,
                                                                        unsigned char   blue) :
                                        ICharacterStyle(),
                                        fRed(red),
                                        fGreen(green),
                                        fBlue(blue)
{
}


ITextColorStyle::ITextColorStyle( const ITextColorStyle& other ) :
                                        ICharacterStyle( other ),
                                        fRed(other.fRed),
                                        fGreen(other.fGreen),
                                        fBlue(other.fBlue)
{
}



ITextColorStyle::~ITextColorStyle()
{
}


ITextColorStyle&
ITextColorStyle::operator=( const ITextColorStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fRed = other.fRed;
                fGreen = other.fGreen;
                fBlue = other.fBlue;
        }
        return *this;
}


void
ITextColorStyle::colors(        unsigned char&  red,
                                                        unsigned char&  green,
                                                        unsigned char&  blue) const
{
        red = fRed;
        green = fGreen;
        blue = fBlue;
}

void
ITextColorStyle::setColors(     unsigned char   newRed,
                                                        unsigned char   newGreen,
                                                        unsigned char   newBlue)
{
        fRed = newRed;
        fGreen = newGreen;
        fBlue = newBlue;
}

unsigned char
ITextColorStyle::red() const
{
        return fRed;
}

unsigned char
ITextColorStyle::green() const
{
        return fGreen;
}

unsigned char
ITextColorStyle::blue() const
{
        return fBlue;
}

void
ITextColorStyle::setRed(unsigned char newValue)
{
        fRed = newValue;
}

void
ITextColorStyle::setGreen(unsigned char newValue)
{
        fGreen = newValue;
}

void
ITextColorStyle::setBlue(unsigned char  newValue)
{
        fBlue = newValue;
}


void
ITextColorStyle::addToSet(ITextStyleSet&        set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasColorStyle);
        set.setColorComponent(ITextStyleSet::kRed, fRed);
        set.setColorComponent(ITextStyleSet::kGreen, fGreen);
        set.setColorComponent(ITextStyleSet::kBlue, fBlue);
}

void
ITextColorStyle::removeFromSet(ITextStyleSet&   set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasColorStyle);
}

void
ITextColorStyle::getValueFromSet(const ITextStyleSet&   set)
{
        fRed = set.colorComponent(ITextStyleSet::kRed);
        fGreen = set.colorComponent(ITextStyleSet::kGreen);
        fBlue = set.colorComponent(ITextStyleSet::kBlue);
}

bool
ITextColorStyle::isContainedInSet(const ITextStyleSet&  set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasColorStyle));
}

bool
ITextColorStyle::isValueContainedInSet(const ITextStyleSet&     set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasColorStyle))
                return (fRed == set.colorComponent(ITextStyleSet::kRed)) &&
                           (fGreen == set.colorComponent(ITextStyleSet::kGreen)) &&
                           (fBlue == set.colorComponent(ITextStyleSet::kBlue));
        else
                return false;
}

bool
ITextColorStyle::operator==( const ITextColorStyle& other ) const
{
        return fRed == other.fRed && fGreen == other.fGreen && fBlue == other.fBlue;
}

bool
ITextColorStyle::operator!=( const ITextColorStyle& other ) const
{
        return fRed != other.fRed || fGreen != other.fGreen || fBlue != other.fBlue;
}

//*******************************************************
//      class ITextSuperSubscriptStyle
//*******************************************************

ITextSuperSubscriptStyle::ITextSuperSubscriptStyle() :
                                        ICharacterStyle(),
                                        fValue( 0 )
{
}


ITextSuperSubscriptStyle::ITextSuperSubscriptStyle( float value ) :
                                        ICharacterStyle(),
                                        fValue( value )
{
}


ITextSuperSubscriptStyle::ITextSuperSubscriptStyle( const ITextSuperSubscriptStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextSuperSubscriptStyle::~ITextSuperSubscriptStyle()
{
}


float
ITextSuperSubscriptStyle::offset() const
{
        return fValue;
}


void
ITextSuperSubscriptStyle::setOffset( float value )
{
        fValue = value;
}


ITextSuperSubscriptStyle&
ITextSuperSubscriptStyle::operator=( const ITextSuperSubscriptStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextSuperSubscriptStyle::addToSet(ITextStyleSet&       set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasSuperSubscriptStyle);
        set.setFloatValue(ITextStyleSet::kSuperSubscript, fValue);
}

void
ITextSuperSubscriptStyle::removeFromSet(ITextStyleSet&  set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasSuperSubscriptStyle);
}

void
ITextSuperSubscriptStyle::getValueFromSet(const ITextStyleSet&  set)
{
        fValue = set.floatValue(ITextStyleSet::kSuperSubscript);
}

bool
ITextSuperSubscriptStyle::isContainedInSet(const ITextStyleSet& set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasSuperSubscriptStyle));
}

bool
ITextSuperSubscriptStyle::isValueContainedInSet(const ITextStyleSet&    set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasSuperSubscriptStyle))
                return fValue == set.floatValue(ITextStyleSet::kSuperSubscript);
        else
                return false;
}

bool
ITextSuperSubscriptStyle::operator==( const ITextSuperSubscriptStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextSuperSubscriptStyle::operator!=( const ITextSuperSubscriptStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class ITextFixedPitchStyle
//*******************************************************

ITextFixedPitchStyle::ITextFixedPitchStyle() :
                                        ICharacterStyle()
{
}


ITextFixedPitchStyle::ITextFixedPitchStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextFixedPitchStyle::ITextFixedPitchStyle( const ITextFixedPitchStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextFixedPitchStyle::~ITextFixedPitchStyle()
{
}


ITextFixedPitchStyle&
ITextFixedPitchStyle::operator=( const ITextFixedPitchStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextFixedPitchStyle::setFixedPitch( bool value )
{
        fValue = value;
}


bool
ITextFixedPitchStyle::isFixedPitch() const
{
        return fValue;
}


void
ITextFixedPitchStyle::addToSet(ITextStyleSet&     set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasFixedPitchStyle);
        set.setBooleanValue(ITextStyleSet::kFixedPitch, fValue);
}

void
ITextFixedPitchStyle::removeFromSet(ITextStyleSet&        set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasFixedPitchStyle);
}

void
ITextFixedPitchStyle::getValueFromSet(const ITextStyleSet&        set)
{
        fValue = set.booleanValue(ITextStyleSet::kFixedPitch);
}

bool
ITextFixedPitchStyle::isContainedInSet(const ITextStyleSet&       set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasFixedPitchStyle));
}

bool
ITextFixedPitchStyle::isValueContainedInSet(const ITextStyleSet&  set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasFixedPitchStyle))
                return fValue == set.booleanValue(ITextStyleSet::kFixedPitch);
        else
                return false;
}

bool
ITextFixedPitchStyle::operator==( const ITextFixedPitchStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
ITextFixedPitchStyle::operator!=( const ITextFixedPitchStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class IParagraphJustificationStyle
//*******************************************************

IParagraphJustificationStyle::IParagraphJustificationStyle() :
                                        IParagraphStyle(),
                                        fValue( IParagraphJustificationStyle::kStart )
{
}


IParagraphJustificationStyle::IParagraphJustificationStyle( IParagraphJustificationStyle::EJustification value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphJustificationStyle::IParagraphJustificationStyle( const IParagraphJustificationStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphJustificationStyle::~IParagraphJustificationStyle()
{
}


IParagraphJustificationStyle&
IParagraphJustificationStyle::operator=( const IParagraphJustificationStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphJustificationStyle::setJustification( IParagraphJustificationStyle::EJustification value )
{
        fValue = value;
}


IParagraphJustificationStyle::EJustification
IParagraphJustificationStyle::justification() const
{
        return fValue;
}


void
IParagraphJustificationStyle::addToSet(ITextStyleSet&   set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasJustificationStyle);
        set.setJustificationValue(fValue);
}

void
IParagraphJustificationStyle::removeFromSet(ITextStyleSet&      set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasJustificationStyle);
}

void
IParagraphJustificationStyle::getValueFromSet(const ITextStyleSet&      set)
{
        fValue = set.justificationValue();
}

bool
IParagraphJustificationStyle::isContainedInSet(const ITextStyleSet&     set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasJustificationStyle));
}

bool
IParagraphJustificationStyle::isValueContainedInSet(const ITextStyleSet&        set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasJustificationStyle))
                return fValue == set.justificationValue();
        else
                return false;
}

bool
IParagraphJustificationStyle::operator==( const IParagraphJustificationStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
IParagraphJustificationStyle::operator!=( const IParagraphJustificationStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class IParagraphSpreadStyle
//*******************************************************

IParagraphSpreadStyle::IParagraphSpreadStyle() :
                                        IParagraphStyle(),
                                        fValue( IParagraphSpreadStyle::kNone )
{
}


IParagraphSpreadStyle::IParagraphSpreadStyle( IParagraphSpreadStyle::ESpread value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphSpreadStyle::IParagraphSpreadStyle( const IParagraphSpreadStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphSpreadStyle::~IParagraphSpreadStyle()
{
}


IParagraphSpreadStyle&
IParagraphSpreadStyle::operator=( const IParagraphSpreadStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphSpreadStyle::setSpread( IParagraphSpreadStyle::ESpread value )
{
        fValue = value;
}


IParagraphSpreadStyle::ESpread
IParagraphSpreadStyle::spread() const
{
        return fValue;
}


void
IParagraphSpreadStyle::addToSet(ITextStyleSet&  set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasSpreadStyle);
        set.setSpreadValue(fValue);
}

void
IParagraphSpreadStyle::removeFromSet(ITextStyleSet&     set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasSpreadStyle);
}

void
IParagraphSpreadStyle::getValueFromSet(const ITextStyleSet&     set)
{
        fValue = set.spreadValue();
}

bool
IParagraphSpreadStyle::isContainedInSet(const ITextStyleSet&    set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasSpreadStyle));
}

bool
IParagraphSpreadStyle::isValueContainedInSet(const ITextStyleSet&       set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasSpreadStyle))
                return fValue == set.spreadValue();
        else
                return false;
}

bool
IParagraphSpreadStyle::operator==( const IParagraphSpreadStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
IParagraphSpreadStyle::operator!=( const IParagraphSpreadStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class IParagraphStartIndentStyle
//*******************************************************

IParagraphStartIndentStyle::IParagraphStartIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphStartIndentStyle::IParagraphStartIndentStyle( float value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphStartIndentStyle::IParagraphStartIndentStyle( const IParagraphStartIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphStartIndentStyle::~IParagraphStartIndentStyle()
{
}


IParagraphStartIndentStyle&
IParagraphStartIndentStyle::operator=( const IParagraphStartIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphStartIndentStyle::setStartIndent( float value )
{
        fValue = value;
}


float
IParagraphStartIndentStyle::startIndent() const
{
        return fValue;
}



void
IParagraphStartIndentStyle::addToSet(ITextStyleSet&     set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasStartIndentStyle);
        set.setFloatValue(ITextStyleSet::kStartIndent, fValue);
}

void
IParagraphStartIndentStyle::removeFromSet(ITextStyleSet&        set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasStartIndentStyle);
}

void
IParagraphStartIndentStyle::getValueFromSet(const ITextStyleSet&        set)
{
        fValue = set.floatValue(ITextStyleSet::kStartIndent);
}

bool
IParagraphStartIndentStyle::isContainedInSet(const ITextStyleSet&       set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasStartIndentStyle));
}

bool
IParagraphStartIndentStyle::isValueContainedInSet(const ITextStyleSet&  set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasStartIndentStyle))
                return fValue == set.floatValue(ITextStyleSet::kStartIndent);
        else
                return false;
}

bool
IParagraphStartIndentStyle::operator==( const IParagraphStartIndentStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
IParagraphStartIndentStyle::operator!=( const IParagraphStartIndentStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class IParagraphEndIndentStyle
//*******************************************************

IParagraphEndIndentStyle::IParagraphEndIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphEndIndentStyle::IParagraphEndIndentStyle( float value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphEndIndentStyle::IParagraphEndIndentStyle( const IParagraphEndIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphEndIndentStyle::~IParagraphEndIndentStyle()
{
}


IParagraphEndIndentStyle&
IParagraphEndIndentStyle::operator=( const IParagraphEndIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphEndIndentStyle::setEndIndent( float value )
{
        fValue = value;
}


float
IParagraphEndIndentStyle::endIndent() const
{
        return fValue;
}



void
IParagraphEndIndentStyle::addToSet(ITextStyleSet&       set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasEndIndentStyle);
        set.setFloatValue(ITextStyleSet::kEndIndent, fValue);
}

void
IParagraphEndIndentStyle::removeFromSet(ITextStyleSet&  set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasEndIndentStyle);
}

void
IParagraphEndIndentStyle::getValueFromSet(const ITextStyleSet&  set)
{
        fValue = set.floatValue(ITextStyleSet::kEndIndent);
}

bool
IParagraphEndIndentStyle::isContainedInSet(const ITextStyleSet& set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasEndIndentStyle));
}

bool
IParagraphEndIndentStyle::isValueContainedInSet(const ITextStyleSet&    set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasEndIndentStyle))
                return fValue == set.floatValue(ITextStyleSet::kEndIndent);
        else
                return false;
}

bool
IParagraphEndIndentStyle::operator==( const IParagraphEndIndentStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
IParagraphEndIndentStyle::operator!=( const IParagraphEndIndentStyle& other ) const
{
        return (fValue != other.fValue);
}

//*******************************************************
//      class IParagraphFirstIndentStyle
//*******************************************************

IParagraphFirstIndentStyle::IParagraphFirstIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphFirstIndentStyle::IParagraphFirstIndentStyle( float value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphFirstIndentStyle::IParagraphFirstIndentStyle( const IParagraphFirstIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphFirstIndentStyle::~IParagraphFirstIndentStyle()
{
}


IParagraphFirstIndentStyle&
IParagraphFirstIndentStyle::operator=( const IParagraphFirstIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphFirstIndentStyle::setFirstIndent( float value )
{
        fValue = value;
}


float
IParagraphFirstIndentStyle::firstIndent() const
{
        return fValue;
}


void
IParagraphFirstIndentStyle::addToSet(ITextStyleSet&     set) const
{
        set.setContainmentFlag(ITextStyleSet::kHasFirstIndentStyle);
        set.setFloatValue(ITextStyleSet::kFirstIndent, fValue);
}

void
IParagraphFirstIndentStyle::removeFromSet(ITextStyleSet&        set) const
{
        set.clearContainmentFlag(ITextStyleSet::kHasFirstIndentStyle);
}

void
IParagraphFirstIndentStyle::getValueFromSet(const ITextStyleSet&        set)
{
        fValue = set.floatValue(ITextStyleSet::kFirstIndent);
}

bool
IParagraphFirstIndentStyle::isContainedInSet(const ITextStyleSet&       set) const
{
        return !!(set.testContainmentFlag(ITextStyleSet::kHasFirstIndentStyle));
}

bool
IParagraphFirstIndentStyle::isValueContainedInSet(const ITextStyleSet&  set) const
{
        if (set.testContainmentFlag(ITextStyleSet::kHasFirstIndentStyle))
                return fValue == set.floatValue(ITextStyleSet::kFirstIndent);
        else
                return false;
}

bool
IParagraphFirstIndentStyle::operator==( const IParagraphFirstIndentStyle& other ) const
{
        return (fValue == other.fValue);
}

bool
IParagraphFirstIndentStyle::operator!=( const IParagraphFirstIndentStyle& other ) const
{
        return (fValue != other.fValue);
}

//*****************************************
//*
//*****************************************

ITabRulerStyle::ITabRulerStyle()
        : IParagraphStyle(), fRuler(NULL)
{
        fRuler = new IStandardTabRuler();
}

ITabRulerStyle::ITabRulerStyle(const ITabRulerStyle& source)
        : IParagraphStyle(source), fRuler(source.fRuler->clone())
{
}

ITabRulerStyle::ITabRulerStyle(GCoord autoSpacing)
        : IParagraphStyle(), fRuler(NULL)
{
        fRuler = new IStandardTabRuler(autoSpacing);
}

ITabRulerStyle::ITabRulerStyle(size_t tabCount, const ITabStop* tabs, GCoord autoSpacing)
        : IParagraphStyle(),fRuler(NULL)
{
        fRuler = new IStandardTabRuler(tabCount, tabs, autoSpacing);
}

ITabRulerStyle::ITabRulerStyle(const IMTabRuler& ruler)
        : IParagraphStyle(), fRuler(NULL)
{
        fRuler = ruler.clone();
}

ITabRulerStyle::~ITabRulerStyle()
{
        delete fRuler;
        fRuler = NULL;
}

void ITabRulerStyle::setRuler(const IMTabRuler& ruler)
{
        delete fRuler;
        fRuler = ruler.clone();
}

IMTabRuler* ITabRulerStyle::ruler()
{
        return fRuler;
}

void
ITabRulerStyle::addToSet(ITextStyleSet& set) const
{
        delete set.fRuler;
        set.fRuler = fRuler->clone();
        set.fStyleContainmentFlags |= ITextStyleSet::kHasTabRuler;
}

void
ITabRulerStyle::removeFromSet(ITextStyleSet& set) const
{
        delete set.fRuler;
        set.fRuler = NULL;
        set.fStyleContainmentFlags &= (unsigned long)~ITextStyleSet::kHasTabRuler;
}

void
ITabRulerStyle::getValueFromSet(const ITextStyleSet& set)
{
        delete fRuler;
        if (set.fRuler) {
                fRuler = set.fRuler->clone();
        }
        else {
                fRuler = NULL;
        }
}

bool
ITabRulerStyle::isContainedInSet(const ITextStyleSet& set) const
{
        return !!(set.fStyleContainmentFlags & ITextStyleSet::kHasTabRuler);
}

bool
ITabRulerStyle::isValueContainedInSet(const ITextStyleSet& set) const
{
        return isContainedInSet(set) && (*fRuler == *set.fRuler);
}
