// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
// Revision: 17 1.7.1.3 source/core/text/itext/itxtplmt.cpp, text, ioc.v400, 000302  
#include "itxtplmt.hpp"

#ifdef IC_PAGETUNE
	#define _ITXTPLMNT_CPP
	#include <ipagetun.h>
#endif

ITextPlacement::ITextPlacement(const ITextIterator& pos)
	: fPos(pos), fPlacement(AFTER_OFFSET)
{
}

ITextPlacement::ITextPlacement(const ITextIterator& pos, ITextPlacement::ECharPlacement placement)
	: fPos(pos), fPlacement(placement)
{
}

ITextPlacement::ITextPlacement(const ITextPlacement& source)
	: fPos(source.fPos), fPlacement(source.fPlacement)
{
}

ITextPlacement::~ITextPlacement()
{
}

ITextPlacement&
ITextPlacement::operator=(const ITextPlacement& source)
{
	if (this != &source) {
		fPos = source.fPos;
		fPlacement = source.fPlacement;
	}
	return *this;
}

bool
ITextPlacement::operator==(const ITextPlacement& other) const
{
	return fPos == other.fPos && fPlacement == other.fPlacement;
}

bool
ITextPlacement::operator!=(const ITextPlacement& other) const
{
	return fPos != other.fPos || fPlacement != other.fPlacement;
}

bool
ITextPlacement::operator<(const ITextPlacement& other) const
{
	return fPos < other.fPos || (fPos == other.fPos && fPlacement < other.fPlacement);
}

bool
ITextPlacement::operator<=(const ITextPlacement& other) const
{
	return fPos < other.fPos || (fPos == other.fPos && fPlacement <= other.fPlacement);
}

bool
ITextPlacement::operator>(const ITextPlacement& other) const
{
	return fPos > other.fPos || (fPos == other.fPos && fPlacement > other.fPlacement);
}

bool
ITextPlacement::operator>=(const ITextPlacement& other) const
{
	return fPos > other.fPos || (fPos == other.fPos && fPlacement >= other.fPlacement);
}

ITextPlacement&
ITextPlacement::operator++()
{
	if (fPlacement == BEFORE_OFFSET) {
		fPlacement = AFTER_OFFSET;
	}
	else {
		fPos++;
		fPlacement = BEFORE_OFFSET;
	}
	return *this;
}

ITextPlacement&
ITextPlacement::operator++(int)
{
	return ITextPlacement::operator++();
}

ITextPlacement&
ITextPlacement::operator--()
{
	if (fPlacement == AFTER_OFFSET) {
		fPlacement = BEFORE_OFFSET;
	}
	else {
		fPos--;
		fPlacement = AFTER_OFFSET;
	}
	return *this;
}

ITextPlacement&
ITextPlacement::operator--(int)
{
	return ITextPlacement::operator--();
}

