// Revision: 68 1.4.1.4 source/core/text/styles/istyle.cpp, text, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
#ifndef _ISTYLE_HPP
#include "istyle.hpp"
#endif

#ifndef _ISTRING_
#include <istring.hpp>
#endif

#ifdef IC_PAGETUNE
        #define _ISYLE_CPP
        #include <ipagetun.h>
#endif

//*******************************************************
//      class ITextStyle
//*******************************************************


ITextStyle::ITextStyle()
{
}


ITextStyle::ITextStyle( const ITextStyle& )
{
}


ITextStyle::~ITextStyle()
{
}


ITextStyle&
ITextStyle::operator=( const ITextStyle& other )
{
        if ( &other != this ) {
                }
        return *this;
}


//*******************************************************
//      class ICharacterStyle
//*******************************************************


ICharacterStyle::ICharacterStyle() :
                                        ITextStyle()
{
}


ICharacterStyle::ICharacterStyle( const ICharacterStyle& other ) :
                                        ITextStyle( other )
{
}


ICharacterStyle::~ICharacterStyle()
{
}


ICharacterStyle&
ICharacterStyle::operator=( const ICharacterStyle& other )
{
        if ( &other != this ) {
                ITextStyle::operator=( other );
                }
        return *this;
}


ITextStyle::EStylePropagation
ICharacterStyle::propagates() const
{
        return ITextStyle::kPropagateByCharacter;
}

//*******************************************************
//      class IParagraphStyle
//*******************************************************

IParagraphStyle::IParagraphStyle() :
                                        ITextStyle()
{
}


IParagraphStyle::IParagraphStyle( const IParagraphStyle& other ) :
                                        ITextStyle( other )
{
}



IParagraphStyle::~IParagraphStyle()
{
}


IParagraphStyle&
IParagraphStyle::operator=( const IParagraphStyle& other )
{
        if ( &other != this ) {
                ITextStyle::operator=( other );
                }
        return *this;
}


ITextStyle::EStylePropagation
IParagraphStyle::propagates() const
{
        return ITextStyle::kPropagateByParagraph;
}



//*******************************************************
//      class ITextTypefaceStyle
//*******************************************************

ITextTypefaceStyle::ITextTypefaceStyle() :
                                        ICharacterStyle(),
                                        fValue()
{
}


ITextTypefaceStyle::ITextTypefaceStyle( const IText& name ) :
                                        ICharacterStyle(),
                                        fValue( name )
{
}



ITextTypefaceStyle::ITextTypefaceStyle( const ITextTypefaceStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextTypefaceStyle::ITextTypefaceStyle( const IFont& font ) :
                                        ICharacterStyle()
{
        fValue = IText( font.name() );
}


ITextTypefaceStyle::~ITextTypefaceStyle()
{
}


ITextTypefaceStyle&
ITextTypefaceStyle::operator=( const ITextTypefaceStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


IText
ITextTypefaceStyle::typeface() const
{
        return fValue;
}


void
ITextTypefaceStyle::setTypeface( const IText& value )
{
        fValue = value;
}


ITextStyle::EStyleClass
ITextTypefaceStyle::styleClass() const
{
        return ITextStyle::kTextTypefaceStyle;
}


//*******************************************************
//      class ITextPointSizeStyle
//*******************************************************

ITextPointSizeStyle::ITextPointSizeStyle() :
                                        ICharacterStyle(),
                                        fValue( 0 )
{
}


ITextPointSizeStyle::ITextPointSizeStyle( double value ) :
                                        ICharacterStyle(),
                                        fValue( value )
{
}


ITextPointSizeStyle::ITextPointSizeStyle( const ITextPointSizeStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextPointSizeStyle::ITextPointSizeStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.pointSize() )
{
}


ITextPointSizeStyle::~ITextPointSizeStyle()
{
}


ITextPointSizeStyle&
ITextPointSizeStyle::operator=( const ITextPointSizeStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


double
ITextPointSizeStyle::pointSize() const
{
        return fValue;
}

void
ITextPointSizeStyle::setPointSize( const double& value )
{
        fValue = value;
}


ITextStyle::EStyleClass
ITextPointSizeStyle::styleClass() const
{
        return ITextStyle::kTextPointSizeStyle;
}


//*******************************************************
//      class ITextBoldfaceStyle
//*******************************************************

ITextBoldfaceStyle::ITextBoldfaceStyle() :
                                        ICharacterStyle()
{
}


ITextBoldfaceStyle::ITextBoldfaceStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextBoldfaceStyle::ITextBoldfaceStyle( const ITextBoldfaceStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextBoldfaceStyle::ITextBoldfaceStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.isBold() )
{
}


ITextBoldfaceStyle::~ITextBoldfaceStyle()
{
}


ITextBoldfaceStyle&
ITextBoldfaceStyle::operator=( const ITextBoldfaceStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextBoldfaceStyle::setBoldface( bool value )
{
        fValue = value;
}


bool
ITextBoldfaceStyle::isBoldface() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextBoldfaceStyle::styleClass() const
{
        return ITextStyle::kTextBoldfaceStyle;
}


//*******************************************************
//      class ITextItalicStyle
//*******************************************************

ITextItalicStyle::ITextItalicStyle() :
                                        ICharacterStyle()
{
}


ITextItalicStyle::ITextItalicStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextItalicStyle::ITextItalicStyle( const ITextItalicStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextItalicStyle::ITextItalicStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.isItalic() )
{
}


ITextItalicStyle::~ITextItalicStyle()
{
}


ITextItalicStyle&
ITextItalicStyle::operator=( const ITextItalicStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextItalicStyle::setItalic( bool value )
{
        fValue = value;
}


bool
ITextItalicStyle::isItalic() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextItalicStyle::styleClass() const
{
        return ITextStyle::kTextItalicStyle;
}


//*******************************************************
//      class ITextUnderlineStyle
//*******************************************************

ITextUnderlineStyle::ITextUnderlineStyle() :
                                        ICharacterStyle()
{
}


ITextUnderlineStyle::ITextUnderlineStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextUnderlineStyle::ITextUnderlineStyle( const ITextUnderlineStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextUnderlineStyle::ITextUnderlineStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.isUnderscore() )
{
}


ITextUnderlineStyle::~ITextUnderlineStyle()
{
}


ITextUnderlineStyle&
ITextUnderlineStyle::operator=( const ITextUnderlineStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextUnderlineStyle::setUnderlined( bool value )
{
        fValue = value;
}


bool
ITextUnderlineStyle::isUnderlined() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextUnderlineStyle::styleClass() const
{
        return ITextStyle::kTextUnderlineStyle;
}


//*******************************************************
//      class ITextStrikethroughStyle
//*******************************************************

ITextStrikethroughStyle::ITextStrikethroughStyle() :
                                        ICharacterStyle()
{
}


ITextStrikethroughStyle::ITextStrikethroughStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextStrikethroughStyle::ITextStrikethroughStyle( const ITextStrikethroughStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextStrikethroughStyle::ITextStrikethroughStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.isStrikeout() )
{
}


ITextStrikethroughStyle::~ITextStrikethroughStyle()
{
}


ITextStrikethroughStyle&
ITextStrikethroughStyle::operator=( const ITextStrikethroughStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextStrikethroughStyle::setStrikethrough( bool value )
{
        fValue = value;
}


bool
ITextStrikethroughStyle::isStrikethrough() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextStrikethroughStyle::styleClass() const
{
        return ITextStyle::kTextStrikethroughStyle;
}


//*******************************************************
//      class ITextOutlineStyle
//*******************************************************

ITextOutlineStyle::ITextOutlineStyle() :
                                        ICharacterStyle()
{
}


ITextOutlineStyle::ITextOutlineStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextOutlineStyle::ITextOutlineStyle( const ITextOutlineStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextOutlineStyle::ITextOutlineStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( font.isStrikeout() )
{
}


ITextOutlineStyle::~ITextOutlineStyle()
{
}


ITextOutlineStyle&
ITextOutlineStyle::operator=( const ITextOutlineStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextOutlineStyle::setOutlined( bool value )
{
        fValue = value;
}


bool
ITextOutlineStyle::isOutlined() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextOutlineStyle::styleClass() const
{
        return ITextStyle::kTextOutlineStyle;
}


//*******************************************************
//      class ITextUneditableStyle
//*******************************************************

ITextUneditableStyle::ITextUneditableStyle() :
                                        ICharacterStyle()
{
}


ITextUneditableStyle::ITextUneditableStyle( bool isSet ) :
                                        ICharacterStyle(),
                                        fValue( isSet )
{
}


ITextUneditableStyle::ITextUneditableStyle( const ITextUneditableStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextUneditableStyle::~ITextUneditableStyle()
{
}


ITextUneditableStyle&
ITextUneditableStyle::operator=( const ITextUneditableStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextUneditableStyle::setUneditable( bool value )
{
        fValue = value;
}


bool
ITextUneditableStyle::isUneditable() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextUneditableStyle::styleClass() const
{
        return ITextStyle::kTextUneditableStyle;
}


//*******************************************************
//      class ITextColorStyle
//*******************************************************

ITextColorStyle::ITextColorStyle() :
                                        ICharacterStyle(),
                                        fValue( IColor::black )
{
}


ITextColorStyle::ITextColorStyle( const IColor& color ) :
                                        ICharacterStyle(),
                                        fValue( color )
{
}


ITextColorStyle::ITextColorStyle( const ITextColorStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextColorStyle::~ITextColorStyle()
{
}


ITextColorStyle&
ITextColorStyle::operator=( const ITextColorStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
ITextColorStyle::setColor( const IColor& value )
{
        fValue = value;
}


IColor
ITextColorStyle::color() const
{
        return fValue;
}


ITextStyle::EStyleClass
ITextColorStyle::styleClass() const
{
        return ITextStyle::kTextColorStyle;
}


//*******************************************************
//      class ITextSuperSubscriptStyle
//*******************************************************

ITextSuperSubscriptStyle::ITextSuperSubscriptStyle() :
                                        ICharacterStyle(),
                                        fValue( 0 )
{
}


ITextSuperSubscriptStyle::ITextSuperSubscriptStyle( double value ) :
                                        ICharacterStyle(),
                                        fValue( value )
{
}


ITextSuperSubscriptStyle::ITextSuperSubscriptStyle( const ITextSuperSubscriptStyle& other ) :
                                        ICharacterStyle( other ),
                                        fValue( other.fValue )
{
}



ITextSuperSubscriptStyle::ITextSuperSubscriptStyle( const IFont& font ) :
                                        ICharacterStyle(),
                                        fValue( 0 )
{
        //      I'm rather unclear on how to extract this information
        //              from a font, since the headers I have are inconsistent
}


ITextSuperSubscriptStyle::~ITextSuperSubscriptStyle()
{
}


double
ITextSuperSubscriptStyle::offset() const
{
        return fValue;
}


void
ITextSuperSubscriptStyle::setOffset( const double& value )
{
        fValue = value;
}


ITextSuperSubscriptStyle&
ITextSuperSubscriptStyle::operator=( const ITextSuperSubscriptStyle& other )
{
        if ( &other != this ) {
                ICharacterStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


ITextStyle::EStyleClass
ITextSuperSubscriptStyle::styleClass() const
{
        return ITextStyle::kTextSuperSubscriptStyle;
}


//*******************************************************
//      class IParagraphJustificationStyle
//*******************************************************

IParagraphJustificationStyle::IParagraphJustificationStyle() :
                                        IParagraphStyle(),
                                        fValue( IParagraphJustificationStyle::kStart )
{
}


IParagraphJustificationStyle::IParagraphJustificationStyle( const IParagraphJustificationStyle::EParagraphJustification& value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphJustificationStyle::IParagraphJustificationStyle( const IParagraphJustificationStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphJustificationStyle::~IParagraphJustificationStyle()
{
}


IParagraphJustificationStyle&
IParagraphJustificationStyle::operator=( const IParagraphJustificationStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphJustificationStyle::setJustification( IParagraphJustificationStyle::EParagraphJustification value )
{
        fValue = value;
}


IParagraphJustificationStyle::EParagraphJustification
IParagraphJustificationStyle::justification() const
{
        return fValue;
}


ITextStyle::EStyleClass
IParagraphJustificationStyle::styleClass() const
{
        return ITextStyle::kParagraphJustificationStyle;
}


//*******************************************************
//      class IParagraphSpreadStyle
//*******************************************************

IParagraphSpreadStyle::IParagraphSpreadStyle() :
                                        IParagraphStyle(),
                                        fValue( IParagraphSpreadStyle::kNone )
{
}


IParagraphSpreadStyle::IParagraphSpreadStyle( const IParagraphSpreadStyle::EParagraphSpread& value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphSpreadStyle::IParagraphSpreadStyle( const IParagraphSpreadStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphSpreadStyle::~IParagraphSpreadStyle()
{
}


IParagraphSpreadStyle&
IParagraphSpreadStyle::operator=( const IParagraphSpreadStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphSpreadStyle::setSpread( IParagraphSpreadStyle::EParagraphSpread value )
{
        fValue = value;
}


IParagraphSpreadStyle::EParagraphSpread
IParagraphSpreadStyle::spread() const
{
        return fValue;
}


ITextStyle::EStyleClass
IParagraphSpreadStyle::styleClass() const
{
        return ITextStyle::kParagraphSpreadStyle;
}


//*******************************************************
//      class IParagraphStartIndentStyle
//*******************************************************

IParagraphStartIndentStyle::IParagraphStartIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphStartIndentStyle::IParagraphStartIndentStyle( const double& value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphStartIndentStyle::IParagraphStartIndentStyle( const IParagraphStartIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphStartIndentStyle::~IParagraphStartIndentStyle()
{
}


IParagraphStartIndentStyle&
IParagraphStartIndentStyle::operator=( const IParagraphStartIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphStartIndentStyle::setStartIndent( const double& value )
{
        fValue = value;
}


double
IParagraphStartIndentStyle::startIndent() const
{
        return fValue;
}



ITextStyle::EStyleClass
IParagraphStartIndentStyle::styleClass() const
{
        return ITextStyle::kParagraphStartIndentStyle;
}


//*******************************************************
//      class IParagraphEndIndentStyle
//*******************************************************

IParagraphEndIndentStyle::IParagraphEndIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphEndIndentStyle::IParagraphEndIndentStyle( const double& value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphEndIndentStyle::IParagraphEndIndentStyle( const IParagraphEndIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphEndIndentStyle::~IParagraphEndIndentStyle()
{
}


IParagraphEndIndentStyle&
IParagraphEndIndentStyle::operator=( const IParagraphEndIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphEndIndentStyle::setEndIndent( const double& value )
{
        fValue = value;
}


double
IParagraphEndIndentStyle::endIndent() const
{
        return fValue;
}



ITextStyle::EStyleClass
IParagraphEndIndentStyle::styleClass() const
{
        return ITextStyle::kParagraphEndIndentStyle;
}


//*******************************************************
//      class IParagraphFirstIndentStyle
//*******************************************************

IParagraphFirstIndentStyle::IParagraphFirstIndentStyle() :
                                        IParagraphStyle(),
                                        fValue( 0 )
{
}


IParagraphFirstIndentStyle::IParagraphFirstIndentStyle( const double& value ) :
                                        IParagraphStyle(),
                                        fValue( value )
{
}


IParagraphFirstIndentStyle::IParagraphFirstIndentStyle( const IParagraphFirstIndentStyle& other ) :
                                        IParagraphStyle( other ),
                                        fValue( other.fValue )
{
}



IParagraphFirstIndentStyle::~IParagraphFirstIndentStyle()
{
}


IParagraphFirstIndentStyle&
IParagraphFirstIndentStyle::operator=( const IParagraphFirstIndentStyle& other )
{
        if ( &other != this ) {
                IParagraphStyle::operator=( other );
                fValue = other.fValue;
                }
        return *this;
}


void
IParagraphFirstIndentStyle::setFirstIndent( const double& value )
{
        fValue = value;
}


double
IParagraphFirstIndentStyle::firstIndent() const
{
        return fValue;
}



ITextStyle::EStyleClass
IParagraphFirstIndentStyle::styleClass() const
{
        return ITextStyle::kParagraphFirstIndentStyle;
}


