// Revision: 69 1.6.1.4 source/core/text/styles/istyle.hpp, text, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
#ifndef _ISTYLE_
#define _ISTYLE_

#include <itext.hpp>
#include <ifont.hpp>
#include <icolor.hpp>
#include <ipoint.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ITextStyleSet;

#pragma enum(4)
#pragma pack(push,4)

class ITextStyle {

        public:

                virtual ~ITextStyle();

                enum EStylePropagation {
                                kPropagateByCharacter,
                                kPropagateByParagraph,

                                kMaximumPropagation = 127
                                };

                virtual EStylePropagation       propagates() const = 0;

        protected:

                ITextStyle();
                ITextStyle( const ITextStyle& );
                ITextStyle& operator=( const ITextStyle& );

                enum EStyleClass {
                                                kTextTypefaceStyle,
                                                kTextPointSizeStyle,
                                                kTextBoldfaceStyle,
                                                kTextItalicStyle,
                                                kTextUnderlineStyle,
                                                kTextStrikethroughStyle,
                                                kTextOutlineStyle,
                                                kTextUneditableStyle,
                                                kTextColorStyle,
                                                kTextSuperSubscriptStyle,
                                                kParagraphJustificationStyle,
                                                kParagraphSpreadStyle,
                                                kParagraphStartIndentStyle,
                                                kParagraphEndIndentStyle,
                                                kParagraphFirstIndentStyle,

                                                kMaximumStyleClass = 127

                                        };

                virtual EStyleClass styleClass() const = 0;
                friend ITextStyleSet;


        private:

};


class ICharacterStyle : public ITextStyle {

        public:

                virtual ~ICharacterStyle();

                virtual ITextStyle::EStylePropagation   propagates() const;

        protected:

                        ICharacterStyle();
                        ICharacterStyle( const ICharacterStyle& );
                        ICharacterStyle&        operator=( const ICharacterStyle& );

        private:

};



class IParagraphStyle : public ITextStyle {

        public:

                virtual ~IParagraphStyle();

                virtual ITextStyle::EStylePropagation   propagates() const;

        protected:

                        IParagraphStyle();
                        IParagraphStyle( const IParagraphStyle& );
                        IParagraphStyle&        operator=( const IParagraphStyle& );

        private:

};




class ITextTypefaceStyle : public ICharacterStyle
{

        public:
                        ITextTypefaceStyle();
                        ITextTypefaceStyle( const IText& );
                        ITextTypefaceStyle( const ITextTypefaceStyle& );
                        ITextTypefaceStyle( const IFont& );

        virtual ~ITextTypefaceStyle();

        bool    operator==( const ITextTypefaceStyle& ) const;
        bool    operator!=( const ITextTypefaceStyle& ) const;

                        IText   typeface() const;
                        void    setTypeface( const IText& );

                        ITextTypefaceStyle& operator=( const ITextTypefaceStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                IText   fValue;

};


class ITextPointSizeStyle : public ICharacterStyle
{

        public:
                        ITextPointSizeStyle();
                        ITextPointSizeStyle( double );
                        ITextPointSizeStyle( const ITextPointSizeStyle& );
                        ITextPointSizeStyle( const IFont& );

        virtual ~ITextPointSizeStyle();

        bool    operator==( const ITextPointSizeStyle& ) const;
        bool    operator!=( const ITextPointSizeStyle& ) const;

        double  pointSize() const;
        void            setPointSize( const double& );

                                ITextPointSizeStyle& operator=( const ITextPointSizeStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                unsigned long   fValue;

};



class ITextBoldfaceStyle : public ICharacterStyle
{

        public:
                        ITextBoldfaceStyle();
                        ITextBoldfaceStyle( bool );
                        ITextBoldfaceStyle( const ITextBoldfaceStyle& );
                        ITextBoldfaceStyle( const IFont& );

        virtual ~ITextBoldfaceStyle();

        bool    operator==( const ITextBoldfaceStyle& ) const;
        bool    operator!=( const ITextBoldfaceStyle& ) const;

                                bool    isBoldface() const;
                                void    setBoldface( bool );

                                ITextBoldfaceStyle& operator=( const ITextBoldfaceStyle& );
        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool    fValue;

};


class ITextItalicStyle : public ICharacterStyle
{

        public:
                        ITextItalicStyle();
                        ITextItalicStyle( bool );
                        ITextItalicStyle( const ITextItalicStyle& );
                        ITextItalicStyle( const IFont& );

        virtual ~ITextItalicStyle();

        bool    operator==( const ITextItalicStyle& ) const;
        bool    operator!=( const ITextItalicStyle& ) const;

                                bool    isItalic() const;
                                void    setItalic( bool );

                                ITextItalicStyle& operator=( const ITextItalicStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool    fValue;
};


class ITextUnderlineStyle : public ICharacterStyle
{

        public:
                        ITextUnderlineStyle();
                        ITextUnderlineStyle( bool );
                        ITextUnderlineStyle( const ITextUnderlineStyle& );
                        ITextUnderlineStyle( const IFont& );

        virtual ~ITextUnderlineStyle();

        bool    operator==( const ITextUnderlineStyle& ) const;
        bool    operator!=( const ITextUnderlineStyle& ) const;

                                bool    isUnderlined() const;
                                void    setUnderlined( bool );

                                ITextUnderlineStyle& operator=( const ITextUnderlineStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool    fValue;

};


class ITextStrikethroughStyle : public ICharacterStyle
{

        public:
                        ITextStrikethroughStyle();
                        ITextStrikethroughStyle( bool );
                        ITextStrikethroughStyle( const ITextStrikethroughStyle& );
                        ITextStrikethroughStyle( const IFont& );

        virtual ~ITextStrikethroughStyle();

        bool    operator==( const ITextStrikethroughStyle& ) const;
        bool    operator!=( const ITextStrikethroughStyle& ) const;

                                bool    isStrikethrough() const;
                                void    setStrikethrough( bool );

                        ITextStrikethroughStyle& operator=( const ITextStrikethroughStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool fValue;

};




class ITextOutlineStyle : public ICharacterStyle
{

        public:
                        ITextOutlineStyle();
                        ITextOutlineStyle( bool );
                        ITextOutlineStyle( const ITextOutlineStyle& );
                        ITextOutlineStyle( const IFont& );

        virtual ~ITextOutlineStyle();

        bool    operator==( const ITextOutlineStyle& ) const;
        bool    operator!=( const ITextOutlineStyle& ) const;

                                bool    isOutlined() const;
                                void    setOutlined( bool );

                                ITextOutlineStyle& operator=( const ITextOutlineStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool    fValue;

};





class ITextUneditableStyle : public ICharacterStyle
{

        public:
                        ITextUneditableStyle();
                        ITextUneditableStyle( bool );
                        ITextUneditableStyle( const ITextUneditableStyle& );

        virtual ~ITextUneditableStyle();

        bool    operator==( const ITextUneditableStyle& ) const;
        bool    operator!=( const ITextUneditableStyle& ) const;

                                bool    isUneditable() const;
                                void    setUneditable( bool );

                                ITextUneditableStyle& operator=( const ITextUneditableStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                bool    fValue;

};



class ITextColorStyle : public ICharacterStyle
{

        public:
                        ITextColorStyle();
                        ITextColorStyle( const IColor& );
                        ITextColorStyle( const ITextColorStyle& );

        virtual ~ITextColorStyle();

        bool    operator==( const ITextColorStyle& ) const;
        bool    operator!=( const ITextColorStyle& ) const;

                                IColor  color() const;
                                void            setColor( const IColor& );

                                ITextColorStyle& operator=( const ITextColorStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                IColor  fValue;

};



class ITextSuperSubscriptStyle : public ICharacterStyle
{

        public:
                        ITextSuperSubscriptStyle();
                        ITextSuperSubscriptStyle( double offset );
                        ITextSuperSubscriptStyle( const ITextSuperSubscriptStyle& );
                        ITextSuperSubscriptStyle( const IFont& );

        virtual ~ITextSuperSubscriptStyle();

                                double  offset() const;
                                void            setOffset( const double& );

        bool    operator==( const ITextSuperSubscriptStyle& ) const;
        bool    operator!=( const ITextSuperSubscriptStyle& ) const;

        ITextSuperSubscriptStyle& operator=( const ITextSuperSubscriptStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

        long    fValue;

};



class IParagraphJustificationStyle : public IParagraphStyle
{

        public:

        enum    EParagraphJustification {

                                kStart,
                                kCenter,
                                kEnd,

                                kMaximumParagraphJustification = 127
                                };


                                IParagraphJustificationStyle();
                                IParagraphJustificationStyle( const EParagraphJustification& );
                                IParagraphJustificationStyle( const IParagraphJustificationStyle& );

                virtual ~IParagraphJustificationStyle();

        bool    operator==( const IParagraphJustificationStyle& ) const;
        bool    operator!=( const IParagraphJustificationStyle& ) const;

        EParagraphJustification justification() const;
        void setJustification( EParagraphJustification );

        IParagraphJustificationStyle&
                                operator=( const IParagraphJustificationStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                EParagraphJustification fValue;

};



class IParagraphSpreadStyle : public IParagraphStyle
{

        public:

        enum    EParagraphSpread {

                                kNone,
                                kAll,
                                kNotLastLine,

                                kMaximumParagraphSpread = 127
                                };


                                IParagraphSpreadStyle();
                                IParagraphSpreadStyle( const EParagraphSpread& );
                                IParagraphSpreadStyle( const IParagraphSpreadStyle& );

                virtual ~IParagraphSpreadStyle();

        bool    operator==( const IParagraphSpreadStyle& ) const;
        bool    operator!=( const IParagraphSpreadStyle& ) const;

        EParagraphSpread        spread() const;
        void setSpread( EParagraphSpread );

        IParagraphSpreadStyle&
                                operator=( const IParagraphSpreadStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                EParagraphSpread        fValue;

};





class IParagraphStartIndentStyle : public IParagraphStyle
{

        public:
                                IParagraphStartIndentStyle();
                                IParagraphStartIndentStyle( const double& );
                                IParagraphStartIndentStyle( const IParagraphStartIndentStyle&
);

                virtual ~IParagraphStartIndentStyle();

        bool    operator==( const IParagraphStartIndentStyle& ) const;
        bool    operator!=( const IParagraphStartIndentStyle& ) const;

        double  startIndent() const;
        void            setStartIndent( const double& );

                                IParagraphStartIndentStyle& operator=( const IParagraphStartIndentStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                double fValue;

};



class IParagraphEndIndentStyle : public IParagraphStyle
{

        public:
                                IParagraphEndIndentStyle();
                                IParagraphEndIndentStyle( const double& );
                                IParagraphEndIndentStyle( const IParagraphEndIndentStyle& );

                virtual ~IParagraphEndIndentStyle();

        bool    operator==( const IParagraphEndIndentStyle& ) const;
        bool    operator!=( const IParagraphEndIndentStyle& ) const;

        double  endIndent() const;
        void            setEndIndent( const double& );

        IParagraphEndIndentStyle& operator=( const IParagraphEndIndentStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                double fValue;

};



class IParagraphFirstIndentStyle : public IParagraphStyle
{

        public:
                                IParagraphFirstIndentStyle();
                                IParagraphFirstIndentStyle( const double& );
                                IParagraphFirstIndentStyle( const IParagraphFirstIndentStyle&
);

                virtual ~IParagraphFirstIndentStyle();

        bool    operator==( const IParagraphFirstIndentStyle& ) const;
        bool    operator!=( const IParagraphFirstIndentStyle& ) const;

        double  firstIndent() const;
        void            setFirstIndent( const double& );

                                IParagraphFirstIndentStyle& operator=( const IParagraphFirstIndentStyle& );

        protected:

                virtual ITextStyle::EStyleClass styleClass() const;

        private:

                double fValue;

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ISTYLE_

