// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
#ifndef _ICPARRAY_INL_
#define _ICPARRAY_INL_

#include <stdlib.h>
#include <stdio.h>

template <class AType>
const bool ICompactArrayOf<AType>::debugShowOverlap =false;

template <class AType>
const unsigned long ICompactArrayOf<AType>::debugShowOverlapLimit = 100;

template <class AType>
const bool ICompactArrayOf<AType>::debugTrace = false;

template <class AType>
const bool ICompactArrayOf<AType>::debugSmall = false;

template <class AType>
const unsigned long ICompactArrayOf<AType>::debugSmallLimit = 30000;

// internal constants
template <class AType>
const long ICompactArrayOf<AType>::kUnicodeCount = 65536;

template <class AType>
const long ICompactArrayOf<AType>::kBlockShift = 7;

template <class AType>
const long ICompactArrayOf<AType>::kBlockCount = 128; // (1<<kBlockShift)

template <class AType>
const long ICompactArrayOf<AType>::kIndexShift = 9; // (16-kBlockShift)

template <class AType>
const long ICompactArrayOf<AType>::kIndexCount = 512; // (1<<kIndexShift)

template <class AType>
const unsigned long ICompactArrayOf<AType>::kBlockMask = 127; // (kBlockCount-1)


// debug flags
//=======================================================

template <class AType>
ICompactArrayOf<AType>::ICompactArrayOf(AType defaultValue)
{
	unsigned long i;
	this->fArray = new AType[kUnicodeCount];
	this->fIndex = new unsigned short[kIndexCount];
	for (i = 0; i < kUnicodeCount; ++i) {
		this->fArray[i] = defaultValue;
	}
	for (i = 0; i < kIndexCount; ++i) {
		this->fIndex[i] = (unsigned short)(i<<kBlockShift);
	}
	this->fCount = kUnicodeCount;
	this->fCompact = false;
}

template <class AType>
ICompactArrayOf<AType>::ICompactArrayOf(unsigned short *indexArray, AType *newValues, long count)
{
 	this->fArray = newValues;
	this->fIndex = indexArray;
	this->fCount = count;
	this->fCompact = (count < kUnicodeCount) ? true : false;
}

//=======================================================
 
template <class AType>
ICompactArrayOf<AType>::~ICompactArrayOf() {
	delete [] fArray;
	fArray = 0;
	delete [] fIndex;
	fIndex = 0;
	fCount = 0;
	fCompact = false;
}


template <class AType>
ICompactArrayOf<AType>::ICompactArrayOf(const ICompactArrayOf<AType>& that) : 
	fCount(that.fCount),
	fCompact(that.fCompact)
{
	long i = 0;
	this->fArray = new AType[fCount];
	this->fIndex = new unsigned short[kIndexCount];
	for (i = 0; i < fCount; ++i) {
		this->fArray[i] = that.fArray[i];
	}
	for (i = 0; i < kIndexCount; ++i) {
		this->fIndex[i] = that.fIndex[i];
	}
}


template <class AType>
ICompactArrayOf<AType>&
ICompactArrayOf<AType>::operator=(const ICompactArrayOf<AType>& that)
{
	if (this != &that) {
		long i = 0;
		fCount = that.fCount;
		fCompact = that.fCompact;
		delete [] fArray;
		fArray = 0;
		fArray = new AType[fCount];
		for (i = 0; i < fCount; ++i) {
			fArray[i] = that.fArray[i];	
		}
		delete [] fIndex;
		fIndex = 0;
		fIndex = new unsigned short[kIndexCount];
		for (i = 0; i < kIndexCount; ++i) {
			fIndex[i] = that.fIndex[i];	
		}
	}
	return *this;
}

//=======================================================
 
template <class AType>
void
ICompactArrayOf<AType>::expand() {
// can optimize later.
// if we have to expand, then walk through the blocks instead of using Get
	unsigned long i;
	if (this->fCompact) {
		AType*	tempArray;
		tempArray = new AType[kUnicodeCount];
		for (i = 0; i < kUnicodeCount; ++i) {
			tempArray[i] = this->get((UniChar)i);  // HSYS : How expand?
		}
		for (i = 0; i < kIndexCount; ++i) {
			this->fIndex[i] = (unsigned short)(i<<kBlockShift);
		}
		delete [] (this->fArray);
		this->fArray = tempArray;
		this->fCompact = false;
	}
}

 
template <class AType>
void 
ICompactArrayOf<AType>::printIndex(long start, long count)
{
	long i;
	for (i = start; i < count; ++i)
		{
		fprintf(stderr, "%i -> %i;   ", i, this->fIndex[i]);
		}
	fprintf(stderr, "\n");
}

 
template <class AType>
void 
ICompactArrayOf<AType>::printPlainArray(
		long start,
		long count,
		const UniChar *tempIndex)
{
	long iIndex;
	if (tempIndex)
		{
		for (iIndex	= start; iIndex < start + count; ++iIndex)
			{
			fprintf(stderr, "%i", this->fArray[tempIndex[iIndex]]);
			}
		}
	else 
		{
		for (iIndex = start; iIndex < start + count; ++iIndex)
			{
			fprintf(stderr, "%i", this->fArray[iIndex]);
			}
		}
	fprintf(stderr, "	Range: start %i, count%i\n", start, count);
}
		

//=======================================================
// this->fArray:	an array to be overlapped
// start and count:	specify the block to be overlapped
// tempIndex:	the overlapped array (actually indices back into inputContents)
// inputHash:	an index of hashes for tempIndex, where
//		inputHash[i] = XOR of values from i-count+1 to i
 
template <class AType>
long
ICompactArrayOf<AType>::findOverlappingPosition(
					unsigned long start,
					const UniChar* tempIndex,
				    long tempIndexCount,
				    unsigned long cycle) {
	long i;
	long j;
	long currentCount;
		
	if (debugShowOverlap && start < debugShowOverlapLimit) {
		this->printPlainArray(start, kBlockCount, 0);
		this->printPlainArray(0, tempIndexCount, tempIndex);
	}
	for (i = 0; i < tempIndexCount; i += cycle) {
		currentCount = kBlockCount;
		if (i + kBlockCount > tempIndexCount) {
			currentCount = tempIndexCount - i;
		} 
		for (j = 0; j < currentCount; ++j) {
			if (this->fArray[start + j] != this->fArray[tempIndex[i + j]]) break;
		}
		if (j == currentCount) break;
	}
	if (debugShowOverlap && start < debugShowOverlapLimit) {
		for (j = 1; j < i; ++j) {
			fprintf(stderr, " ");
		}
		this->printPlainArray(start, kBlockCount, 0);
		fprintf(stderr, "	Found At: %i\n", i);
	}
	return i;
}

template <class AType>
const AType*
ICompactArrayOf<AType>::getArray()
{
	return fArray;
}

template <class AType>
const unsigned short*
ICompactArrayOf<AType>::getIndex()
{
	return fIndex;
}

template <class AType>
long 
ICompactArrayOf<AType>::getCount()
{
	return fCount;
}


template <class AType>
void 
ICompactArrayOf<AType>::setCount(long cnt)
{
	fCount = cnt;
}


template <class AType>
void 
ICompactArrayOf<AType>::set(UniChar c, AType value)
{
    if (fCompact == true)
        expand();
    fArray[(long)c] = value;
}


template <class AType>
void 
ICompactArrayOf<AType>::set(UniChar start, UniChar end, AType value)
{
    long i;
    if (fCompact == true) {
        expand();
    }
    for (i = start; i <= end; ++i) {
        fArray[i] = value;
    }
}


//=======================================================
 
template <class AType>
void 
ICompactArrayOf<AType>::compact (unsigned long cycle) {
	if (!this->fCompact) {
		UniChar*	tempIndex;
		long		tempIndexCount;
		AType*		tempArray;
		long		iBlock, iIndex;
		
		// fix cycle, must be 0 < cycle <= blockcount
		if (cycle < 0) cycle = 1;
		else if (cycle > kBlockCount) cycle = kBlockCount;

		// make temp storage, larger than we need
		tempIndex = new UniChar[kUnicodeCount];
		
		// set up first block.
		tempIndexCount = kBlockCount;
		for (iIndex = 0; iIndex < kBlockCount; ++iIndex) {
			tempIndex[iIndex] = (unsigned short)iIndex;
		}; // endfor (iIndex = 0; .....)
		this->fIndex[0] = 0;
	
	// for each successive block, find out its first position in the compacted array
		for (iBlock = 1; iBlock < kIndexCount; ++iBlock) {
			long	newCount, firstPosition, block;
			block = iBlock<<kBlockShift;
			if (debugSmall) if (block > debugSmallLimit) break;
			firstPosition = findOverlappingPosition(
				block, tempIndex, tempIndexCount, cycle);
			
		// if not contained in the current list, copy the remainder
		// invariant; cumulativeHash[iBlock] = XOR of values from iBlock-kBlockCount+1 to iBlock
		// we do this by XORing out cumulativeHash[iBlock-kBlockCount]
			newCount = firstPosition + kBlockCount;
			if (newCount > tempIndexCount) {
				for (iIndex = tempIndexCount; iIndex < newCount; ++iIndex) {
					tempIndex[iIndex] = (unsigned short)(iIndex - firstPosition + block);
				} // endfor (iIndex = tempIndexCount....)
				tempIndexCount = newCount;
			} // endif (newCount > tempIndexCount)
			this->fIndex[iBlock] = (unsigned short)firstPosition;
		} // endfor (iBlock = 1.....)
		
		if (debugTrace) {
			this->printPlainArray(0, tempIndexCount, tempIndex);
		} // endif debugTrace
		
	// now allocate and copy the items into the array
		tempArray = new AType[tempIndexCount];
		for (iIndex = 0; iIndex < tempIndexCount; ++iIndex) {
			tempArray[iIndex] = this->fArray[tempIndex[iIndex]];
		}
		delete []  (this->fArray);
		this->fArray = tempArray;
		this->fCount = tempIndexCount;
	
		if (debugTrace) {
			this->printPlainArray(0, tempIndexCount, tempIndex);
		} // endfor (iIndex....)
	
	// free up temp storage
		delete []  (tempIndex);
		this->fCompact = true;
	} // endif (!this->fCompact)
}

#endif // _ICPARRAY_INL_
