/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 86 1.9.1.4 source/core/text/unicode/ilyprpty.hpp, text, ioc.v400, 000302 
//
//      File:           LayoutProperties.h
//      Function:       Management and access to database for inquiries about Unicode character properties
//      CREATED
//              Monday, February 1, 1996
//
//      CREATED BY
//              Text & International Team
//
//      29 Feb 1996             hsys            conform OpenClass coding guideline and update unicode info
//
//      DESCRIPTION
//              This file defines the unicode classes
//
//      COPYRIGHT
//
//
//
//********************************************************************************************


#ifndef _ILYPRPTY_
#define _ILYPRPTY_

#include <iunicode.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

// LayoutProperties

// This code is temporary and only used by Line Layout. See ldc or jhj if you think you need to use it

class ILayoutProperties {
        public:
                enum EDirectionClass            {
                                                                cLL = 0, cLR, cLNl, cLNr, cRL, cRR, cRNl,
                                                                cRNr, cNlL, cNlR, cNlNl, cNrL, cNrR, cNrNr,
                                                                eot, dummy
                                                                };

                typedef struct aCharPropsInitializationRecord {
                                //THIS BIT ORDERING IS INTERESTING BECAUSE IT IS FILED IN THE trueTYPE FONTS!!!
                                IUnicode::EDirectionProperty    charDir : 2;
                                IUnicode::EDirectionProperty    runDir  : 2;
                                IUnicode::ECharacterLayoutType  charClass : 2;
                                bool                            hangLeft  : 1;
                                bool                            hangRight : 1;
                                } CharPropsInitializationRecord;

                void                            set(IUnicode::ICharacterLayoutProperties lc);
                void                            set(CharPropsInitializationRecord cpir);

                IUnicode::EDirectionProperty    runDirection();
                void                                    setRunDirection(IUnicode::EDirectionProperty);
                IUnicode::EDirectionProperty    characterDirection();
                void                                    setCharacterDirection(IUnicode::EDirectionProperty);
                IUnicode::ECharacterLayoutType  characterClass();
                void                                    setCharacterClass(IUnicode::ECharacterLayoutType);
                bool                                    hangLeft();
                void                                    setHangLeft(bool);
                bool                                    hangRight();
                void                                    setHangRight(bool);

                EDirectionClass                 directionClass();
                void                            print();


                enum {
                        gNeutralProperty = 0xAC
                        };

                //      Public because it's used by ILayoutGraphic.C

        private:
                typedef union aCPIR {
                         CharPropsInitializationRecord cp;
                         IUnicode::ICharacterLayoutProperties uc;
                        } CPIR;

//              static const IUnicode::ICharacterLayoutProperties gNeutralProperty;
                friend class IUnicode;
                friend IUnicode::ICharacterLayoutProperties characterLayoutProperties(UniChar uc);
                CPIR fLayoutProperties;

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include "ilyprpty.inl"

#endif // _ILYPRPTY_
