/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 87 1.14.1.5 source/core/text/unicode/unicode.cpp, text, ioc.v400, 000302 
//********************************************************************************************
//      NAME
//              unicode.cpp
//
//      CREATED
//              Monday, February 1, 1996
//
//      CREATED BY
//              Text & International Team
//
//      DESCRIPTION
//              This file defines the unicode classes
//
//      COPYRIGHT
//
//
//
//********************************************************************************************

#include <string.h>
#include <ctype.h>

#include <iunicode.hpp>
#include <iugnrl.hpp>
#include <iucmpbty.hpp>
#include <iueasia.hpp>


// A private header containing the character and layout property information
#include <isynchr.hpp>
#include <icparray.hpp>

#ifdef IC_PAGETUNE
        #define _UNICODE_CPP
        #include <ipagetun.h>
#endif

#pragma pack(push,4)

struct UniCharShortPair{
        UniChar fUnicode;
        short   fValue;
};

//======================================================================
// The following are the data tables for the Unicode 2.0 character-
// property functions.  I ported the Unicode 2.0 character-property
// stuff over from the Unicode Analytics Package, but have done
// nothing else to bring this unit up to compatibility with Unicode
// 2.0.  This is why we still have the Unicode 1.1 character-property
// tables in the project.  All this data probably belongs in another
// file, but I didn't know where.  Moving this stuff to an appropriate
// place needs to be part of the process of bringing the rest of this
// code up to Unicode 2.0 conformance.
//                                                              --rtg 4/17/97
//======================================================================

// array of indices into compacted array of Unicode character-property data
const unsigned short IUnicode::fgCharPropIndices[] = {
    0, 127, 255, 382, 504, 591, 719, 846, 973, 1099,
    1221, 1341, 1458, 1571, 1693, 1693, 1693, 1693, 1820,
    1947, 2073, 2200, 2327, 2453, 2580, 2706, 2832, 1693,
    2959, 3086, 3214, 3342, 1693, 3438, 3566, 3621, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 3749, 3855, 3983, 4111,
    4239, 4367, 4495, 4620, 4748, 4762, 4890, 1693, 4976,
    5076, 5204, 5310, 5438, 1693, 5565, 5683, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 5811, 5918, 6045, 6158,
    6286, 6414, 6542, 6665, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6883, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693, 1693,
    1693, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793, 6793,
    6793, 6793, 6793, 6793, 6793, 6793, 6793, 6885, 7013,
    7013, 7013, 7013, 7013, 7013, 7013, 7013, 7013, 7013,
    7013, 7013, 7013, 7013, 7013, 7013, 7141, 7141, 7141,
    7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141,
    7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141,
    7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141,
    7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141,
    7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141, 7141,
    7141, 7141, 6793, 6793, 6875, 1693, 7269, 7347, 6793,
    6793, 7430, 7542, 7666, 7784, 7912, 8025 };

// compacted array of Unicode character-property data
const char IUnicode::fgCharPropValues[] = {
        15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
        15, 12, 23, 23, 23, 25, 23, 23, 23, 20, 21, 23, 24, 23, 19,
        23, 23, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 23, 23, 24,
        24, 24, 23, 23, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        20, 23, 21, 26, 22, 26, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 20, 24, 21, 24, 15, 15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15, 12, 23, 25, 25, 25, 25, 27,
        27, 26, 27, 2, 20, 24, 19, 27, 26, 27, 24, 11, 11, 26, 2,
        27, 23, 26, 11, 2, 21, 11, 11, 11, 23, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 24, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 24, 2, 2, 2, 2, 2, 2, 2, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        1, 2, 1, 2, 1, 2, 2, 1, 1, 2, 1, 2, 1, 1, 2,
        1, 1, 1, 2, 2, 1, 1, 1, 1, 2, 1, 1, 2, 1, 1,
        1, 2, 2, 2, 1, 1, 2, 1, 1, 2, 1, 2, 1, 2, 1,
        1, 2, 1, 5, 2, 1, 2, 1, 1, 2, 1, 1, 1, 2, 1,
        2, 1, 1, 2, 2, 5, 1, 2, 5, 5, 5, 5, 5, 5, 1,
        3, 2, 1, 3, 2, 1, 3, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 1,
        3, 2, 1, 2, 0, 0, 0, 0, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 0, 0, 0, 0, 0, 0, 0, 4, 4, 4, 4, 4, 4, 4,
        4, 4, 26, 26, 4, 4, 4, 4, 4, 4, 4, 26, 26, 26, 26,
        26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 4, 4, 26, 26, 26,
        26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 0, 4, 4, 4, 4,
        4, 26, 26, 26, 26, 26, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 23, 23, 0, 0, 0, 0,
        4, 0, 0, 0, 23, 0, 0, 0, 0, 26, 26, 1, 23, 1, 1,
        1, 0, 1, 0, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 2, 2, 1, 1, 1,
        2, 2, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 2, 2, 2,
        5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        0, 2, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 27, 6, 6, 6, 6, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 5, 1, 2, 1, 2, 0, 0, 1,
        2, 0, 0, 1, 2, 0, 0, 0, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 0, 0, 1, 2, 1, 2, 1, 2, 1,
        2, 0, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 0, 0, 4, 23, 23, 23, 23, 23, 23, 0, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 23,
        0, 0, 0, 0, 0, 0, 0, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 0, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 0, 6, 6, 6, 23, 6, 23, 6, 6, 23, 6, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 5, 5, 5,
        23, 23, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 23,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 23,
        0, 0, 0, 23, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 0, 0, 0, 0, 0, 4, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9,
        9, 9, 9, 23, 23, 23, 23, 0, 0, 6, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 0, 0, 5, 5, 5, 5, 5, 0, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0,
        5, 5, 5, 5, 23, 5, 6, 6, 6, 6, 6, 6, 6, 7, 7,
        6, 6, 6, 6, 6, 6, 4, 4, 6, 6, 27, 6, 6, 6, 6,
        0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 6, 6, 8, 0, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 0, 0, 6, 5, 8, 8, 8, 6, 6, 6, 6, 6, 6,
        6, 6, 8, 8, 8, 8, 6, 0, 0, 27, 6, 6, 6, 6, 0,
        0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 23,
        23, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 23, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 8, 8,
        0, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 5, 5, 0, 0,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 0, 5, 5, 5, 5, 5, 5, 5,
        0, 5, 0, 0, 0, 5, 5, 5, 5, 0, 0, 6, 0, 8, 8,
        8, 6, 6, 6, 6, 0, 0, 8, 8, 0, 0, 8, 8, 6, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 5, 5,
        0, 5, 5, 5, 6, 6, 0, 0, 9, 9, 9, 9, 9, 9, 9,
        9, 9, 9, 5, 5, 25, 25, 11, 11, 11, 11, 11, 11, 27, 0,
        0, 0, 0, 0, 6, 0, 0, 5, 5, 5, 5, 5, 5, 0, 0,
        0, 0, 5, 5, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5,
        5, 5, 5, 5, 5, 5, 0, 5, 5, 0, 5, 5, 0, 5, 5,
        0, 0, 6, 0, 8, 8, 8, 6, 6, 0, 0, 0, 0, 6, 6,
        0, 0, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 5, 5, 5, 5, 0, 5, 0, 0, 0, 0, 0, 0, 0, 9,
        9, 9, 9, 9, 9, 9, 9, 9, 9, 6, 6, 5, 5, 5, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 8, 0, 5,
        5, 5, 5, 5, 5, 5, 0, 5, 0, 5, 5, 5, 0, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 0, 5, 5, 5, 5, 5, 5, 5, 0, 5,
        5, 0, 5, 5, 5, 5, 5, 0, 0, 6, 5, 8, 8, 8, 6,
        6, 6, 6, 6, 0, 6, 6, 8, 0, 8, 8, 6, 0, 0, 27,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        5, 0, 0, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9,
        9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 6, 8, 8, 0, 5, 5, 5, 5, 5, 5, 5, 5, 0,
        0, 5, 5, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5,
        5, 5, 5, 5, 5, 0, 5, 5, 0, 0, 5, 5, 5, 5, 0,
        0, 6, 5, 8, 6, 8, 6, 6, 6, 0, 0, 0, 8, 8, 0,
        0, 8, 8, 6, 0, 0, 0, 0, 0, 0, 0, 0, 6, 8, 0,
        0, 0, 0, 5, 5, 0, 5, 5, 5, 0, 0, 0, 0, 9, 9,
        9, 9, 9, 9, 9, 9, 9, 9, 27, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 8, 0, 5, 5, 5,
        5, 5, 5, 0, 0, 0, 5, 5, 5, 0, 5, 5, 5, 5, 0,
        0, 0, 5, 5, 0, 5, 0, 5, 5, 0, 0, 0, 5, 5, 0,
        0, 0, 5, 5, 5, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5,
        5, 0, 5, 5, 5, 0, 0, 0, 0, 8, 8, 6, 8, 8, 0,
        0, 0, 8, 8, 8, 0, 8, 8, 8, 6, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 11,
        11, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        8, 8, 8, 0, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5,
        5, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 0, 5, 5, 5, 5, 5, 0, 0, 0,
        0, 6, 6, 6, 8, 8, 8, 8, 0, 6, 6, 6, 0, 6, 6,
        6, 6, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 9, 9, 9, 9,
        9, 9, 9, 9, 9, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 8, 8, 0, 5, 5, 5, 5, 5,
        5, 5, 5, 0, 5, 5, 5, 0, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5,
        5, 5, 5, 0, 0, 0, 0, 8, 6, 8, 8, 8, 8, 8, 0,
        6, 8, 8, 0, 8, 8, 6, 6, 0, 0, 0, 0, 0, 0, 0,
        8, 8, 0, 0, 0, 0, 0, 0, 0, 5, 0, 5, 5, 0, 0,
        0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8,
        0, 5, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5, 5, 0, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 0, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 8, 8,
        8, 6, 6, 6, 0, 0, 8, 8, 8, 0, 8, 8, 8, 6, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0,
        0, 0, 5, 5, 0, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9,
        9, 9, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 23, 5, 6, 5, 5, 6, 6, 6, 6, 6,
        6, 6, 0, 0, 0, 0, 25, 5, 5, 5, 5, 5, 5, 4, 6,
        6, 6, 6, 6, 6, 6, 6, 27, 9, 9, 9, 9, 9, 9, 9,
        9, 9, 9, 23, 23, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 5,
        0, 0, 5, 5, 0, 5, 0, 0, 5, 0, 0, 0, 0, 0, 0,
        5, 5, 5, 5, 0, 5, 5, 5, 5, 5, 5, 5, 0, 5, 5,
        5, 0, 5, 0, 5, 0, 0, 5, 5, 0, 5, 5, 23, 5, 6,
        5, 5, 6, 6, 6, 6, 6, 6, 0, 6, 6, 5, 0, 0, 5,
        5, 5, 5, 5, 0, 4, 0, 6, 6, 6, 6, 6, 6, 0, 0,
        9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 0, 0, 5, 5, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 27, 27, 27, 27, 23, 23, 23, 23, 23, 23, 23, 23,
        23, 23, 23, 23, 23, 23, 23, 27, 27, 27, 27, 27, 6, 6, 27,
        27, 27, 27, 27, 27, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
        11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 27, 6, 27, 6, 27,
        6, 20, 21, 20, 21, 8, 8, 5, 5, 5, 5, 5, 5, 5, 5,
        0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 8, 6, 6, 6, 6,
        6, 23, 6, 6, 6, 6, 6, 6, 0, 0, 0, 0, 6, 6, 6,
        6, 6, 6, 0, 6, 0, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 0, 0, 0,
        6, 6, 6, 6, 6, 6, 6, 0, 6, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 0, 0, 0, 0, 23, 0, 0, 0, 0, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0,
        0, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2,
        1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1,
        2, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2,
        1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 0,
        0, 1, 1, 1, 1, 1, 1, 0, 0, 2, 2, 2, 2, 2, 2,
        2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2,
        2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2,
        2, 2, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 2, 2, 2,
        2, 2, 2, 2, 2, 0, 1, 0, 1, 0, 1, 0, 1, 2, 2,
        2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0,
        2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1,
        1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1,
        1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
        1, 1, 1, 2, 2, 2, 2, 2, 0, 2, 2, 1, 1, 1, 1,
        1, 26, 1, 26, 26, 26, 2, 2, 2, 0, 2, 2, 1, 1, 1,
        1, 1, 26, 26, 26, 2, 2, 2, 2, 0, 0, 2, 2, 1, 1,
        1, 1, 0, 26, 26, 26, 2, 2, 2, 2, 2, 2, 2, 2, 1,
        1, 1, 1, 1, 26, 26, 26, 0, 0, 2, 2, 2, 0, 2, 2,
        1, 1, 1, 1, 1, 26, 26, 0, 12, 12, 12, 12, 12, 12, 12,
        12, 12, 12, 12, 12, 16, 16, 16, 16, 19, 19, 19, 19, 19, 19,
        23, 23, 20, 21, 20, 20, 20, 21, 20, 20, 23, 23, 23, 23, 23,
        23, 23, 23, 13, 14, 16, 16, 16, 16, 16, 0, 23, 23, 23, 23,
        23, 23, 23, 23, 23, 20, 21, 23, 23, 23, 23, 22, 22, 23, 23,
        23, 24, 20, 21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 16, 16, 16, 16, 16, 16,
        11, 0, 0, 0, 11, 11, 11, 11, 11, 11, 24, 24, 24, 20, 21,
        2, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 24, 24, 24, 20,
        21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25,
        25, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 6, 6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 7, 7, 7, 7, 6, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 27, 27, 1, 27, 27, 27,
        27, 1, 27, 27, 2, 1, 1, 1, 2, 2, 1, 1, 1, 2, 27,
        1, 27, 27, 1, 1, 1, 1, 1, 1, 27, 27, 27, 27, 27, 27,
        1, 27, 1, 27, 1, 27, 1, 1, 1, 1, 2, 2, 1, 1, 27,
        1, 2, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
        10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
        10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
        10, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        24, 24, 24, 24, 24, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 24, 27, 24, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 27,
        0, 27, 27, 27, 27, 27, 27, 24, 24, 24, 24, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 24, 24, 27, 27, 27, 27, 27, 27, 27, 20, 21, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
        11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
        11, 11, 11, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 11, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0,
        0, 0, 0, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        27, 27, 27, 27, 0, 27, 27, 27, 27, 0, 0, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        0, 27, 0, 27, 27, 27, 27, 0, 0, 0, 27, 0, 27, 27, 27,
        27, 27, 27, 27, 0, 0, 27, 27, 27, 27, 27, 27, 27, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 11, 11, 11,
        11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
        11, 11, 27, 0, 0, 0, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        0, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 12, 23, 23, 23, 27, 4, 23, 10, 20, 21,
        20, 21, 20, 21, 20, 21, 20, 21, 27, 27, 20, 21, 20, 21, 20,
        21, 20, 21, 19, 20, 21, 20, 27, 10, 10, 10, 10, 10, 10, 10,
        10, 10, 6, 6, 6, 6, 6, 6, 19, 4, 4, 4, 4, 4, 27,
        27, 0, 0, 0, 0, 0, 0, 0, 27, 0, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0,
        0, 0, 6, 6, 4, 4, 4, 4, 0, 0, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 23, 4, 4, 4, 0,
        0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0,
        0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 0, 27, 27, 11, 11, 11, 11, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0,
        0, 0, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 0, 0, 0, 27, 11, 11, 11, 11, 11, 11, 11,
        11, 11, 11, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 0, 0, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        0, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        0, 0, 0, 0, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 0, 0, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27,
        27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 0, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17,
        17, 17, 17, 17, 17, 17, 17, 17, 2, 2, 2, 2, 2, 2, 2,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2,
        2, 2, 0, 0, 0, 0, 0, 0, 6, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 24, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 0, 5, 5, 5, 5, 5, 0, 5, 0, 5, 5, 0,
        5, 5, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 20, 21, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 23, 19, 19, 22, 22, 20, 21, 20, 21, 20, 21, 20,
        21, 20, 21, 20, 21, 20, 21, 20, 21, 0, 0, 0, 0, 23, 23,
        23, 23, 22, 22, 22, 23, 23, 23, 0, 23, 23, 23, 23, 19, 20,
        21, 20, 21, 20, 21, 23, 23, 23, 24, 19, 24, 24, 24, 0, 23,
        25, 23, 23, 0, 0, 0, 0, 5, 5, 5, 0, 5, 0, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 0, 0, 16, 0, 23, 23, 23, 25, 23, 23, 23, 20,
        21, 23, 24, 23, 19, 23, 23, 9, 9, 9, 9, 9, 9, 9, 9,
        9, 9, 23, 23, 24, 24, 24, 23, 23, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 20, 23, 21, 26, 22, 26, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
        2, 2, 2, 2, 2, 2, 2, 20, 24, 21, 24, 0, 0, 23, 20,
        21, 23, 23, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 4, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 4,
        4, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
        5, 5, 0, 0, 0, 5, 5, 5, 5, 5, 5, 0, 0, 5, 5,
        5, 5, 5, 5, 0, 0, 5, 5, 5, 5, 5, 5, 0, 0, 5,
        5, 5, 0, 0, 0, 25, 25, 24, 26, 27, 25, 25, 0, 24, 24,
        24, 24, 24, 27, 27, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 27, 0, 0 };

// offset count for compacted character-property array
const long IUnicode::fgCharPropOffsetCount = 8153;


const unsigned short IUnicode::fgCharDirIndices[] = {
    0, 123, 243, 253, 375, 462, 590, 717, 844, 965, 1087,
    1207, 1324, 1436, 1558, 1558, 1558, 1558, 1685, 1812, 1938,
    2065, 2192, 2318, 2445, 2571, 2697, 1558, 2824, 2951, 3079,
    3195, 1558, 3291, 3419, 3474, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 243,
    3602, 3730, 3858, 3986, 4114, 4212, 4337, 4447, 1558, 4521,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 4649, 4756, 4883, 4996, 5124,
    5251, 252, 5379, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 5507, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558, 1558,
    1558, 1558, 1558, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    5509, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 243,
    243, 243, 243, 243, 243, 243, 243, 243, 243, 5637,
    1558, 5765, 5843, 5926, 5926, 5992, 6104, 6228, 5929, 6356,
    6457
};

const char IUnicode::fgCharDirValues[] = {
    10, 10, 10, 10, 10, 10, 10, 10, 10, 8, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 9, 10, 10, 4, 4, 4, 0, 10, 10,
    10, 10, 4, 6, 4, 3, 3, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 6, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 9, 10, 4, 4, 4, 4,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    4, 4, 2, 2, 10, 10, 10, 10, 10, 2,
    10, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 10, 10, 10,
    10, 0, 10, 10, 10, 0, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 10, 0, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 10, 10, 10, 0, 10, 0, 10,
    0, 10, 0, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 10, 0, 0, 10, 10, 0,
    0, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 10, 0, 0, 0, 0, 0,
    0, 0, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 0, 10, 10, 10, 10,
    10, 10, 10, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    10, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 10, 10, 10, 10, 10, 1, 1,
    1, 1, 1, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 1, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    1, 10, 10, 10, 1, 10, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 10, 10, 10, 10, 10, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 5,
    5, 5, 5, 5, 5, 5, 5, 5, 5, 4,
    5, 5, 1, 10, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 10, 10, 1, 1, 1, 1, 1, 10, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 10, 10, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 0, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 10,
    10, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 0, 0, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 10, 10, 10, 0, 0, 0, 0, 10,
    10, 0, 10, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 0, 0, 10, 10, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 0, 10,
    10, 10, 10, 0, 0, 10, 0, 0, 0, 0,
    0, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 0,
    10, 10, 0, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 0, 0, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 10, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 10,
    0, 0, 10, 0, 0, 10, 10, 0, 10, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 0, 0,
    10, 10, 0, 0, 0, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    10, 0, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 0, 10, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 10, 0,
    0, 0, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 10, 0, 0, 10, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 0, 0, 0, 10, 0,
    0, 0, 10, 10, 0, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    0, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 0, 0, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 0, 10, 10, 0, 0, 0, 0, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 0, 0, 10, 10, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 0, 0, 10,
    10, 10, 10, 0, 0, 10, 0, 0, 0, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 10, 0, 0, 0, 0, 0, 0, 10, 10,
    10, 0, 0, 0, 10, 0, 0, 0, 0, 10,
    10, 10, 0, 0, 10, 0, 10, 0, 0, 10,
    10, 10, 0, 0, 10, 10, 10, 0, 0, 0,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 0, 0, 0, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 10, 10, 10, 0, 0, 0,
    10, 0, 0, 0, 0, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 0, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 0, 0,
    0, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 10, 0, 0, 0, 10, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    0, 0, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 0, 0, 10, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 0, 0, 0, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 0, 0, 0, 0, 0, 0, 0, 10,
    0, 0, 0, 10, 0, 0, 0, 0, 10, 10,
    10, 10, 10, 10, 10, 0, 0, 10, 10, 10,
    10, 10, 10, 10, 0, 10, 0, 0, 10, 10,
    10, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 0, 0,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    0, 0, 0, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 0, 0,
    0, 0, 0, 0, 10, 10, 0, 0, 0, 10,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 0, 0, 10, 10, 10, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 0, 0, 10, 0, 10, 10, 0, 0, 10,
    0, 10, 10, 0, 10, 10, 10, 10, 10, 10,
    0, 0, 0, 0, 10, 0, 0, 0, 0, 0,
    0, 0, 10, 0, 0, 0, 10, 0, 10, 0,
    10, 10, 0, 0, 10, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 10, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 10,
    0, 10, 0, 0, 0, 0, 0, 0, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 10, 0, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 10, 0, 10, 10, 10, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 10, 10, 10, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 10, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 0, 0, 0, 0, 0, 0, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 0, 0, 0, 0, 0, 0, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 0, 10,
    0, 10, 0, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 0, 0, 0, 0, 0, 0, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 0, 0, 0, 10, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 9, 9, 9, 9, 9,
    9, 9, 3, 9, 9, 9, 9, 10, 10, 0,
    1, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 7, 7, 10, 10, 10,
    10, 10, 10, 4, 4, 4, 4, 4, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 2, 10, 10,
    10, 2, 2, 2, 2, 2, 2, 4, 4, 10,
    10, 10, 10, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 4, 4, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 4, 4, 4, 4, 4,
    4, 4, 4, 4, 4, 4, 4, 4, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 4, 4, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 10, 10, 10, 10, 10, 9, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 10, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 10, 10,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    10, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 0, 0, 0, 0, 0, 0,
    0, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 0, 0, 0, 0, 0, 10, 10,
    10, 10, 10, 10, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 10,
    1, 1, 1, 1, 1, 10, 1, 10, 1, 1,
    10, 1, 1, 10, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 10,
    10, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 1,
    1, 1, 10, 1, 10, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 10, 10, 10, 10, 10, 10, 10, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 10, 10, 10, 10, 10, 10,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 10,
    10, 10, 0, 0, 0, 0, 0, 0, 10, 10,
    0, 0, 0, 0, 0, 0, 10, 10, 0, 0,
    0, 0, 0, 0, 10, 10, 0, 0, 0, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
    10, 10, 10, 10
        };

const long IUnicode::fgCharDirOffsetCount = 6585;

// compact array of Unicode character-property data
// (this is declared as a global rather than as amember of IUnicode
// so that I don't have to export the declaration and implementation
// of ICompactArray to the world)
ISynchronized<ICompactArrayOf<char>*> gCharPropTables;

ISynchronized<ICompactArrayOf<char>*> gCharDirTables;
//=================================================================================
// Declarations of streaming operators for enums
//=================================================================================

// HSYS : replaced with host streaming
void operator>>=( IUnicode::EUnicodeBounds&, IDataStream& ) ;
void operator<<=( IUnicode::EUnicodeBounds&, IDataStream& );
void operator>>=( IUnicode::EUnicodeScript&, IDataStream& ) ;
void operator<<=( IUnicode::EUnicodeScript&, IDataStream& );
void operator>>=( IUnicode::ECharacterProperty&, IDataStream& ) ;
void operator<<=( IUnicode::ECharacterProperty&, IDataStream& );
void operator>>=( IUnicode::EDirectionProperty&, IDataStream& ) ;
void operator<<=( IUnicode::EDirectionProperty&, IDataStream& );

const IUnicode::IBlockScriptMap IUnicode::fgScriptIndex[] = {
    {    0x0000, 0x007F },
    {    0x0080, 0x00FF },
    {    0x0100, 0x017F },
    {    0x0180, 0x024F },
    {    0x0250, 0x02AF },
    {    0x02B0, 0x02FF },
    {    0x0300, 0x036F },
    {    0x0370, 0x03FF },
    {    0x0400, 0x04FF },
    {    0x0530, 0x058F },
    {    0x0590, 0x05FF },
    {    0x0600, 0x06FF },
    {    0x0900, 0x097F },
    {    0x0980, 0x09FF },
    {    0x0A00, 0x0A7F },
    {    0x0A80, 0x0AFF },
    {    0x0B00, 0x0B7F },
    {    0x0B80, 0x0BFF },
    {    0x0C00, 0x0C7F },
    {    0x0C80, 0x0CFF },
    {    0x0D00, 0x0D7F },
    {    0x0E00, 0x0E7F },
    {    0x0E80, 0x0EFF },
    {    0x0F00, 0x0FBF },
    {    0x10A0, 0x10FF },
    {    0x1100, 0x11FF },
    {    0x1E00, 0x1EFF },
    {    0x1F00, 0x1FFF },
    {    0x2000, 0x206F },
    {    0x2070, 0x209F },
    {    0x20A0, 0x20CF },
    {    0x20D0, 0x20FF },
    {    0x2100, 0x214F },
    {    0x2150, 0x218F },
    {    0x2190, 0x21FF },
    {    0x2200, 0x22FF },
    {    0x2300, 0x23FF },
    {    0x2400, 0x243F },
    {    0x2440, 0x245F },
    {    0x2460, 0x24FF },
    {    0x2500, 0x257F },
    {    0x2580, 0x259F },
    {    0x25A0, 0x25FF },
    {    0x2600, 0x26FF },
    {    0x2700, 0x27BF },
    {    0x3000, 0x303F },
    {    0x3040, 0x309F },
    {    0x30A0, 0x30FF },
    {    0x3100, 0x312F },
    {    0x3130, 0x318F },
    {    0x3190, 0x319F },
    {    0x3200, 0x32FF },
    {    0x3300, 0x33FF },
    {    0x4E00, 0x9FFF },
    {    0xAC00, 0xD7A3 },
    {    0xD800, 0xDB7F },
    {    0xDB80, 0xDBFF },
    {    0xDC00, 0xDFFF },
    {    0xE000, 0xF8FF },
    {    0xF900, 0xFAFF },
    {    0xFB00, 0xFB4F },
    {    0xFB50, 0xFDFF },
    {    0xFE20, 0xFE2F },
    {    0xFE30, 0xFE4F },
    {    0xFE50, 0xFE6F },
    {    0xFE70, 0xFEFE },
    {    0xFEFF, 0xFEFF },
    {    0xFF00, 0xFFEF },
    {    0xFFF0, 0xFFFF }
};

#pragma info(notrd)
UniCharShortPair gCodeDigitTable[] = {
        {       0x0030, 0       },
        {       0x0031, 1       },
        {       0x0032, 2       },
        {       0x0033, 3       },
        {       0x0034, 4       },
        {       0x0035, 5       },
        {       0x0036, 6       },
        {       0x0037, 7       },
        {       0x0038, 8       },
        {       0x0039, 9       },
        {       0x00B2, 2       },
        {       0x00B3, 3       },
        {       0x00B9, 1       },
        {       0x0660, 0       },
        {       0x0661, 1       },
        {       0x0662, 2       },
        {       0x0663, 3       },
        {       0x0664, 4       },
        {       0x0665, 5       },
        {       0x0666, 6       },
        {       0x0667, 7       },
        {       0x0668, 8       },
        {       0x0669, 9       },
        {       0x06F0, 0       },
        {       0x06F1, 1       },
        {       0x06F2, 2       },
        {       0x06F3, 3       },
        {       0x06F4, 4       },
        {       0x06F5, 5       },
        {       0x06F6, 6       },
        {       0x06F7, 7       },
        {       0x06F8, 8       },
        {       0x06F9, 9       },
        {       0x0966, 0       },
        {       0x0967, 1       },
        {       0x0968, 2       },
        {       0x0969, 3       },
        {       0x096A, 4       },
        {       0x096B, 5       },
        {       0x096C, 6       },
        {       0x096D, 7       },
        {       0x096E, 8       },
        {       0x096F, 9       },
        {       0x09E6, 0       },
        {       0x09E7, 1       },
        {       0x09E8, 2       },
        {       0x09E9, 3       },
        {       0x09EA, 4       },
        {       0x09EB, 5       },
        {       0x09EC, 6       },
        {       0x09ED, 7       },
        {       0x09EE, 8       },
        {       0x09EF, 9       },
        {       0x0A66, 0       },
        {       0x0A67, 1       },
        {       0x0A68, 2       },
        {       0x0A69, 3       },
        {       0x0A6A, 4       },
        {       0x0A6B, 5       },
        {       0x0A6C, 6       },
        {       0x0A6D, 7       },
        {       0x0A6E, 8       },
        {       0x0A6F, 9       },
        {       0x0AE6, 0       },
        {       0x0AE7, 1       },
        {       0x0AE8, 2       },
        {       0x0AE9, 3       },
        {       0x0AEA, 4       },
        {       0x0AEB, 5       },
        {       0x0AEC, 6       },
        {       0x0AED, 7       },
        {       0x0AEE, 8       },
        {       0x0AEF, 9       },
        {       0x0B66, 0       },
        {       0x0B67, 1       },
        {       0x0B68, 2       },
        {       0x0B69, 3       },
        {       0x0B6A, 4       },
        {       0x0B6B, 5       },
        {       0x0B6C, 6       },
        {       0x0B6D, 7       },
        {       0x0B6E, 8       },
        {       0x0B6F, 9       },
        {       0x0BE7, 1       },
        {       0x0BE8, 2       },
        {       0x0BE9, 3       },
        {       0x0BEA, 4       },
        {       0x0BEB, 5       },
        {       0x0BEC, 6       },
        {       0x0BED, 7       },
        {       0x0BEE, 8       },
        {       0x0BEF, 9       },
        {       0x0C66, 0       },
        {       0x0C67, 1       },
        {       0x0C68, 2       },
        {       0x0C69, 3       },
        {       0x0C6A, 4       },
        {       0x0C6B, 5       },
        {       0x0C6C, 6       },
        {       0x0C6D, 7       },
        {       0x0C6E, 8       },
        {       0x0C6F, 9       },
        {       0x0CE6, 0       },
        {       0x0CE7, 1       },
        {       0x0CE8, 2       },
        {       0x0CE9, 3       },
        {       0x0CEA, 4       },
        {       0x0CEB, 5       },
        {       0x0CEC, 6       },
        {       0x0CED, 7       },
        {       0x0CEE, 8       },
        {       0x0CEF, 9       },
        {       0x0D66, 0       },
        {       0x0D67, 1       },
        {       0x0D68, 2       },
        {       0x0D69, 3       },
        {       0x0D6A, 4       },
        {       0x0D6B, 5       },
        {       0x0D6C, 6       },
        {       0x0D6D, 7       },
        {       0x0D6E, 8       },
        {       0x0D6F, 9       },
        {       0x0E50, 0       },
        {       0x0E51, 1       },
        {       0x0E52, 2       },
        {       0x0E53, 3       },
        {       0x0E54, 4       },
        {       0x0E55, 5       },
        {       0x0E56, 6       },
        {       0x0E57, 7       },
        {       0x0E58, 8       },
        {       0x0E59, 9       },
        {       0x0ED0, 0       },
        {       0x0ED1, 1       },
        {       0x0ED2, 2       },
        {       0x0ED3, 3       },
        {       0x0ED4, 4       },
        {       0x0ED5, 5       },
        {       0x0ED6, 6       },
        {       0x0ED7, 7       },
        {       0x0ED8, 8       },
        {       0x0ED9, 9       },
        {       0x0F20, 0       },
        {       0x0F21, 1       },
        {       0x0F22, 2       },
        {       0x0F23, 3       },
        {       0x0F24, 4       },
        {       0x0F25, 5       },
        {       0x0F26, 6       },
        {       0x0F27, 7       },
        {       0x0F28, 8       },
        {       0x0F29, 9       },
        {       0x2070, 0       },
        {       0x2074, 4       },
        {       0x2075, 5       },
        {       0x2076, 6       },
        {       0x2077, 7       },
        {       0x2078, 8       },
        {       0x2079, 9       },
        {       0x2080, 0       },
        {       0x2081, 1       },
        {       0x2082, 2       },
        {       0x2083, 3       },
        {       0x2084, 4       },
        {       0x2085, 5       },
        {       0x2086, 6       },
        {       0x2087, 7       },
        {       0x2088, 8       },
        {       0x2089, 9       },
        {       0x2460, 1       },
        {       0x2461, 2       },
        {       0x2462, 3       },
        {       0x2463, 4       },
        {       0x2464, 5       },
        {       0x2465, 6       },
        {       0x2466, 7       },
        {       0x2467, 8       },
        {       0x2468, 9       },
        {       0x2474, 1       },
        {       0x2475, 2       },
        {       0x2476, 3       },
        {       0x2477, 4       },
        {       0x2478, 5       },
        {       0x2479, 6       },
        {       0x247A, 7       },
        {       0x247B, 8       },
        {       0x247C, 9       },
        {       0x2488, 1       },
        {       0x2489, 2       },
        {       0x248A, 3       },
        {       0x248B, 4       },
        {       0x248C, 5       },
        {       0x248D, 6       },
        {       0x248E, 7       },
        {       0x248F, 8       },
        {       0x2490, 9       },
        {       0x24EA, 0       },
        {       0x2776, 1       },
        {       0x2777, 2       },
        {       0x2778, 3       },
        {       0x2779, 4       },
        {       0x277A, 5       },
        {       0x277B, 6       },
        {       0x277C, 7       },
        {       0x277D, 8       },
        {       0x277E, 9       },
        {       0x2780, 1       },
        {       0x2781, 2       },
        {       0x2782, 3       },
        {       0x2783, 4       },
        {       0x2784, 5       },
        {       0x2785, 6       },
        {       0x2786, 7       },
        {       0x2787, 8       },
        {       0x2788, 9       },
        {       0x278A, 1       },
        {       0x278B, 2       },
        {       0x278C, 3       },
        {       0x278D, 4       },
        {       0x278E, 5       },
        {       0x278F, 6       },
        {       0x2790, 7       },
        {       0x2791, 8       },
        {       0x2792, 9       },
        {       0xFF10, 0       },
        {       0xFF11, 1       },
        {       0xFF12, 2       },
        {       0xFF13, 3       },
        {       0xFF14, 4       },
        {       0xFF15, 5       },
        {       0xFF16, 6       },
        {       0xFF17, 7       },
        {       0xFF18, 8       },
        {       0xFF19, 9       },
        {       0xFFFF, 0xFFFF}
};

#pragma pack(pop)
#pragma info(restore)

// StaticInits
double IUnicode::fVersion = 2.0;        // this is currently a lie: only the character peoperties
                                                                        // have been upgraded to 2.0


IUnicode::IUnicode()
{
}


IUnicode::IUnicode( const IUnicode& )
{
        //      We do nothing right now; we assume the property tables have been loaded successfully
}


IUnicode::~IUnicode()
{

        //      Destructor included only for the sake of completeness

}


IUnicode& IUnicode::operator=( const IUnicode& other )
{

        if ( &other != this ) {
                }
        return *this;

}



IUnicode::EUnicodeScript
IUnicode::script( register UniChar uc )
{
        register IUnicode::EUnicodeScript returnValue = IUnicode::kNoScript;

        int index = -1;
        int j;
        for ( j = 0; index == -1 && fgScriptIndex[j].fFirstCode != 0xFFFF; j++ )
                if ( fgScriptIndex[j].fFirstCode <= uc && uc <= fgScriptIndex[j].fLastCode ) {
                        index = j;
                }
        if (index >= IUnicode::kScriptCount) {
                returnValue = IUnicode::kNoScript;
        }
        else if ( index != -1 ) {
                returnValue = IUnicode::EUnicodeScript(index);
        }
        return returnValue;
}


IUnicode::ECharacterProperty
IUnicode::type( register UniChar uc )
{
        register IUnicode::ECharacterProperty returnValue = IUnicode::kUncased;

        ICompactArrayOf<char>*  dataTable = gCharPropTables;

        if (gCharPropTables == 0) {
                if (dataTable == 0) {
                        dataTable = new ICompactArrayOf<char>(
                                                                        (unsigned short*)fgCharPropIndices,
                                                                        (char*)fgCharPropValues, fgCharPropOffsetCount);
                                                                        // is there a better way to do this than
                                                                        // casting away the const?
                        dataTable = synchronizedInit(gCharPropTables, dataTable);
                }
        }
        returnValue =  (IUnicode::ECharacterProperty)(dataTable->get(uc));
        return returnValue;
}

IUnicode::EDirectionProperty
IUnicode::characterDirection( register UniChar uc )
{
        register IUnicode::EDirectionProperty returnValue = IUnicode::kLeftToRight;

        ICompactArrayOf<char>*  dataTable = gCharDirTables;

        if (gCharDirTables == 0) {
                if (dataTable == 0) {
                        dataTable = new ICompactArrayOf<char>(
                                                                        (unsigned short*)fgCharDirIndices,
                                                                        (char*)fgCharDirValues, fgCharDirOffsetCount);
                                                                        // is there a better way to do this than
                                                                        // casting away the const?
                        dataTable = synchronizedInit(gCharDirTables, dataTable);
                }
        }
        returnValue =  (IUnicode::EDirectionProperty)(dataTable->get(uc));
        return returnValue;
}

// The following routines mimic the ANSI CType.h routines for the complete Unicode
// set. This may in fact affect the interpretation of the C definition.
// This may require additional tuning or even a parallel set of routines that duplicate the
// exact behaviour.


// NOT IN ANSI C!       kLineSeparator && !kParagraphSeparator

bool
IUnicode::isLineSeparator(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props != IUnicode::kParagraphSeparator && props == IUnicode::kLineSeparator);
}

// NOT IN ANSI C!       !kLineSeparator && kParagraphSeparator

bool
IUnicode::isParagraphSeparator(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props == IUnicode::kParagraphSeparator && props != IUnicode::kLineSeparator);
}

// NOT IN ANSI C!       kLineSeparator || kParagraphSeparator

bool
IUnicode::isLineOrParagraphSeparator(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props == IUnicode::kParagraphSeparator || props == IUnicode::kLineSeparator);
}



// kWhiteSpace | isLineTerminator
bool
IUnicode::isInvisible(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return ((props >= IUnicode::kFirstSeparator) && (props <= IUnicode::kFormatCharacter));
}

bool
IUnicode::isTrailingInvisible(UniChar uc)
{
        return ( isInvisible( uc ) && !( uc == UWhiteSpace::kNoBreakSpace || uc == UWhiteSpace::kZeroWidthNoBreakSpace ) );
}

// NOT IN ANSI C!       kWhiteSpace && !isParagraphTerminator
bool
IUnicode::isASpace(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (        (props >= IUnicode::kFirstSeparator) &&
                                (props <= IUnicode::kFormatCharacter) &&
                                props != IUnicode::kLineSeparator &&
                                props != IUnicode::kParagraphSeparator);
}



bool
IUnicode::isUpper(UniChar uc)
{
        return (type(uc) == IUnicode::kUppercaseLetter);
}

bool
IUnicode::isLower(UniChar uc)
{
        return (type(uc) == IUnicode::kLowercaseLetter);
}

// need to add an isTitlecase() in here

bool
IUnicode::isUncased(UniChar uc)
{
        return (type(uc) == IUnicode::kOtherLetter);
}

//      Can take a diacritic
bool
IUnicode::isBaseForm(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
#pragma info(none)
        return (        ( props >= IUnicode::kFirstLetter && props <= IUnicode::kLastLetter ) ||
                                ( props >= IUnicode::kFirstNumber && props <= IUnicode::kLastNumber ) ||
                                props == IUnicode::kSpaceSeparator );
#pragma info(restore)
}

bool
IUnicode::isDiacritic(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return ( props >= IUnicode::kFirstMark && props <= IUnicode::kLastMark );
}

bool
IUnicode::isAlpha(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props >= IUnicode::kFirstLetter && props <= IUnicode::kLastLetter);
}

bool
IUnicode::isAlphaNumeric(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return ( props >= IUnicode::kFirstLetter && props <= IUnicode::kLastLetter ) ||
                   ( props >= IUnicode::kFirstNumber && props <= IUnicode::kLastNumber );
}

bool
IUnicode::isGraphic(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return ((props >= kFirstLetter && props <= kLastNumber) ||
                (props >= kFirstPunctuation && props <= kLastSymbol));
}

bool
IUnicode::isDigit(UniChar uc)
{
        // I've left this alone, but is it right?  Shouldn't it be
        // type(uc) >= IUnicode::kFirstNumber && type(uc) <= IUnicode::kLastNumber?
        return (type(uc) == IUnicode::kDecimalNumber);
}

bool
IUnicode::isHexDigit(UniChar uc)
{
        return ((type(uc) == IUnicode::kDecimalNumber)
                || (uc >= ULatin::kSmallLetterA && uc <= ULatin::kSmallLetterF)
                || (uc >= ULatin::kCapitalLetterA && uc <= ULatin::kCapitalLetterF)
                );
}

bool
IUnicode::isXDigit(UniChar uc)
{
        // slightly less efficient than repeating the code of isHexDigit, but guarantees
        //      that the two stay the same

        return isHexDigit (uc);
}


bool
IUnicode::isASCII(UniChar uc)
{
        return (uc < 0x0080);
}

bool
IUnicode::isControl(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props == IUnicode::kControlCharacter ||
                        props == IUnicode::kFormatCharacter ||
                        props == IUnicode::kLineSeparator ||
                        props == IUnicode::kParagraphSeparator);
}

bool
IUnicode::isPrint(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return ((props >= kFirstLetter && props <= kLastSeparator) ||
                (props >= kFirstPunctuation && props <= kLastSymbol));
}

// kOpenPunctuation | kClosePunctuation | kWordFormPunctuation | kOtherPunctuation
bool
IUnicode::isPunctuation (UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props >= IUnicode::kFirstPunctuation && props <= IUnicode::kLastPunctuation);
}

bool
IUnicode::isOpenPunctuation(UniChar uc)
{
        return (type(uc) == IUnicode::kOpenPunctuation);
}

bool
IUnicode::isClosePunctuation(UniChar uc)
{
        return (type(uc) == IUnicode::kClosePunctuation);
}

// kMathSymbol | kGeneralTechnicalSymbol | kCurrencySymbol | kOtherSymbol
bool
IUnicode::isSymbol(UniChar uc)
{
        register IUnicode::ECharacterProperty props = type(uc);
        return (props >= IUnicode::kFirstSymbol && props <= IUnicode::kLastSymbol);
}

bool
IUnicode::isInSet(UniChar uc)
{
        return (type(uc) != IUnicode::kNonCharacter);
}


bool
IUnicode::isCJK( UniChar uc )
{
        if ( !isInSet( uc ) )
                return false;
        if ( uc < UHangulChoseong::kKiyeok )
                return false;
        if ( uc < ULatin::kCapitalLetterAWithRingBelow )
                return true;
        if ( uc < UWhiteSpace::kIdeographicSpace )
                return false;
        if ( uc <= IUnicode::kHighBoundHan )
                return true;
        if ( uc < UCJKCompatibility::kF900 )
                return false;
        if ( uc < ULatinSmallLigature::kFf )
                return true;
        if ( uc < UHalfwidth::kIdeographicFullStop )
                return false;
        if ( uc < UFullwidth::kCentSign )
                return true;
        return false;
}

#pragma pack(push,4)

typedef struct punctmatch {
                                        UniChar fLeftMatch;
                                        UniChar fRightMatch;
                                        } TPunctMatch;

TPunctMatch     gPunctuationMatches[] = {

                                {       UGeneralPunctuation::kLeftParenthesis, UGeneralPunctuation::kRightParenthesis   },
                                {       UGeneralPunctuation::kLeftSquareBracket, UGeneralPunctuation::kRightSquareBracket       },
                                {       UGeneralPunctuation::kLeftCurlyBracket, UGeneralPunctuation::kRightCurlyBracket },
                                {       UGeneralPunctuation::kLeftSquareBracketWithQuill, UGeneralPunctuation::kRightSquareBracketWithQuill     },
                                {       UGeneralPunctuation::kLeftSingleQuotationMark, UGeneralPunctuation::kRightSingleQuotationMark   },
                                {       UGeneralPunctuation::kLeftDoubleQuotationMark, UGeneralPunctuation::kRightDoubleQuotationMark   },
                                {       UGeneralPunctuation::kLeftAngleBracket, UGeneralPunctuation::kRightAngleBracket },
                                {       UGeneralPunctuation::kLeftDoubleAngleBracket, UGeneralPunctuation::kRightDoubleAngleBracket     },
                                {       UGeneralPunctuation::kLeftAngleBracket, UGeneralPunctuation::kRightAngleBracket },
                                {       UGeneralPunctuation::kLeftCornerBracket, UGeneralPunctuation::kRightCornerBracket       },
                                {       UGeneralPunctuation::kLeftWhiteCornerBracket, UGeneralPunctuation::kRightWhiteCornerBracket     },
                                {       UGeneralPunctuation::kLeftBlackLenticularBracket, UGeneralPunctuation::kRightBlackLenticularBracket     },
                                {       UGeneralPunctuation::kLeftTortoiseShellBracket, UGeneralPunctuation::kRightTortoiseShellBracket },
                                {       UGeneralPunctuation::kLeftWhiteLenticularBracket, UGeneralPunctuation::kRightWhiteLenticularBracket     },
                                {       UGeneralPunctuation::kLeftWhiteTortoiseShellBracket, UGeneralPunctuation::kRightWhiteTortoiseShellBracket       },
                                {       UGeneralPunctuation::kLeftWhiteSquareBracket, UGeneralPunctuation::kRightWhiteSquareBracket     },

                                {       UUnicodeSpecial::kNotAUnicode, UUnicodeSpecial::kNotAUnicode    }
                                };

#pragma pack(pop)

UniChar
IUnicode::matchPunctuation( register UniChar searchChar )
{

        register TPunctMatch* matches = gPunctuationMatches;
        register UniChar matchChar = UUnicodeSpecial::kNotAUnicode;

        do {
                if ( searchChar == matches->fLeftMatch )
                        matchChar = matches->fRightMatch;
                else if ( searchChar == matches->fRightMatch )
                        matchChar = matches->fLeftMatch;
                matches++;
        } while (matchChar == UUnicodeSpecial::kNotAUnicode && matches->fLeftMatch != UUnicodeSpecial::kNotAUnicode);
        if ( matchChar == UUnicodeSpecial::kNotAUnicode )
                matchChar = searchChar;
        return matchChar;
}


int
IUnicode::digitValue(UniChar ch)
{

        register int returnValue = -1;

        if ( ( ch >= IUnicode::kLowBoundHan && ch <= IUnicode::kHighBoundHan ) || ( ch == UUnicodeDigit::kHanZero ) ) {
                switch ( ch ) {
                        case UUnicodeDigit::kHanZero    :       returnValue = 0;        break;  // HSYS: 0x3007
                        case UUnicodeDigit::kHanOne             :       returnValue = 1;        break;  // HSYS: 0x4E00
                        case UUnicodeDigit::kHanTwo             :       returnValue = 2;        break;
                        case UUnicodeDigit::kHanThree   :       returnValue = 3;        break;
                        case UUnicodeDigit::kHanFour    :       returnValue = 4;        break;
                        case UUnicodeDigit::kHanFive    :       returnValue = 5;        break;
                        case UUnicodeDigit::kHanSix             :       returnValue = 6;        break;
                        case UUnicodeDigit::kHanSeven   :       returnValue = 7;        break;
                        case UUnicodeDigit::kHanEight   :       returnValue = 8;        break;
                        case UUnicodeDigit::kHanNine    :       returnValue = 9;        break;
                        }
                }
        else {
                for ( register UniCharShortPair* uc = gCodeDigitTable;
                                                returnValue == -1 && uc->fUnicode != UUnicodeSpecial::kNotAUnicode;
                                                uc++ ) {
                        if ( ch == uc->fUnicode )
                                returnValue = uc->fValue;
                        }
                }

        return returnValue;

}

double
IUnicode::currentVersion()
{
        return fVersion;
}

#ifdef UNICODE_DEBUG
// string constants for the print functions

#pragma pack(push,4)

const unsigned short kCharTypeCount = IUnicode::kLastType+1;
char* gCharTypeStrings[kCharTypeCount] = {
                "kNonCharacter",                                //        0
                "kUppercaseLetter",                             //        1
                "kLowercaseLetter",                             //        2
                "kTitlecaseLetter",                             //    3
                "kModifierLetter",                              //        4
                "kOtherLetter",                                 //        5
                "kNonSpacingMark",                              //        6
                "kEnclosingMark",                               //    7
                "kCombiningSpacingMark",                //        8
                "kDecimalNumber",                               //        9
                "kLetterNumber",                                //   10
                "kOtherNumber",                                 //       11
                "kSpaceSeparator",                              //       12
                "kLineSeparator",                               //       13
                "kParagraphSeparator",                  //       14
                "kControlCharacter",                    //       15
                "kFormatCharacter",                             //   16
                "kPrivateUseCharacter",                 //       17
                "kSurrogate",                                   //   18
                "kDashPunctuation",                             //       19
                "kOpenPunctuation",                             //       20
                "kClosePunctuation",                    //       21
                "kConnectorPunctuation",                //   22
                "kOtherPunctuation",                    //       23
                "kMathSymbol",                                  //       24
                "kCurrencySymbol",                              //       25
                "kModifierSymbol",                              //   26
                "kOtherSymbol",                                 //       27
                "kCharacterPropertiesCount",    //       28
        };


char* gScriptStrings[IUnicode::kScriptCount] = {
                    "kBasicLatin",
            "kLatin1Supplement",
                    "kLatinExtendedA",
                    "kLatinExtendedB",
                    "kIPAExtensions",
                    "kSpacingModifier",
                    "kCombiningDiacritical",
                    "kGreek",
                    "kCyrillic",
                    "kArmenian",
                    "kHebrew",
                    "kArabic",
                    "kDevanagari",
                    "kBengali",
                    "kGurmukhi",
                    "kGujarati",
                    "kOriya",
                    "kTamil",
                    "kTelugu",
                    "kKannada",
                    "kMalayalam",
                    "kThai",
                    "kLao",
                    "kTibetan",
                    "kGeorgian",
                    "kHangulJamo",
                    "kLatinExtendedAdditional",
                    "kGreekExtended",
                    "kGeneralPunctuation",
                    "kSuperSubScript",
                    "kCurrencySymbolScript",
                    "kSymbolCombiningMark",
                    "kLetterlikeSymbol",
                    "kNumberForms",
                    "kArrows",
                    "kMathOperator",
                    "kMiscTechnical",
                    "kControlPicture",
                    "kOpticalCharacter",
                    "kEnclosedAlphanumeric",
                    "kBoxDrawing",
                    "kBlockElement",
                    "kGeometricShape",
                    "kMiscSymbol",
                    "kDingbats",
                    "kCJKSymbolPunctuation",
                    "kHiragana",
                    "kKatakana",
                    "kBopomofo",
                    "kHangulCompatibilityJamo",
                    "kKanbun",
                    "kEnclosedCJKLetterMonth",
                    "kCJKCompatibility",
                    "kCJKUnifiedIdeograph",
                    "kHangulSyllable",
                    "kHighSurrogate",
                    "kHighPrivateUseSurrogate",
                    "kLowSurrogate",
                    "kPrivateUse",
                    "kCJKCompatiblityIdeograph",
                    "kAlphabeticPresentation",
                    "kArabicPresentationA",
                    "kCombiningHalfMark",
                    "kCJKCompatilityForm",
                    "kSmallFormVariant",
                    "kArabicPresentationB",
                    "kNoScript",
                    "kHalfwidthFullWidthForm"
};

const unsigned short kEDirectionPropertyCount = IUnicode::kOtherNeutral+1;

char* gEDirectionPropertyStrings[kEDirectionPropertyCount] = {
                        "kLeftToRight",
                        "kRightToLeft",
            "kEuropeanNumber",
            "kEuropeanNumberSeparator",
            "kEuropeanNumberTerminator",
            "kArabicNumber",
                        "kCommonNumberSeparator",
                        "kBlockSeparator",
                        "kSegmentSeparator",
                        "kWhiteSpaceNeutral",
                        "kOtherNeutral"
};

#pragma pack(pop)

#endif

#pragma info(nouse)
void
IUnicode::printCharacterProperties(UniChar u)
{
#ifdef UNICODE_DEBUG
// HSYS : rid of qprintf
        sprintf("%s %s char dir %s",
                gScriptStrings[script(u)],
                gCharTypeStrings[type(u)],
                gEDirectionPropertyStrings[characterDirection(u)]);
#endif
}
#pragma info(restore)


// UnicodeNameConversion
#pragma enum(4)
enum State {startWord, inWord};
#pragma enum(pop)

// HSYS Replaced with host streaming

void
operator >>=( IUnicode::EUnicodeBounds& e, IDataStream& toWhere )
{
        (unsigned short)(e) >>= toWhere;
}


void
operator <<=( IUnicode::EUnicodeBounds& e, IDataStream& fromWhere )
{
        unsigned short c;
        c <<= fromWhere;
        e = IUnicode::EUnicodeBounds(c);
}


void
operator >>=( IUnicode::EUnicodeScript& e, IDataStream& toWhere )
{
        (unsigned short)(e) >>= toWhere;
}


void
operator <<=( IUnicode::EUnicodeScript& e, IDataStream& fromWhere )
{
        unsigned short c;
        c <<= fromWhere;
        e = IUnicode::EUnicodeScript(c);
}


void
operator >>=( IUnicode::ECharacterProperty& e, IDataStream& toWhere )
{
        (unsigned short)(e) >>= toWhere;
}


void
operator <<=( IUnicode::ECharacterProperty& e, IDataStream& fromWhere )
{
        unsigned short c;
        c <<= fromWhere;
        e = IUnicode::ECharacterProperty(c);
}


void
operator >>=( IUnicode::EDirectionProperty& e, IDataStream& toWhere )
{
        (unsigned short)(e) >>= toWhere;
}


void
operator <<=( IUnicode::EDirectionProperty& e, IDataStream& fromWhere )
{
        unsigned short c;
        c <<= fromWhere;
        e = IUnicode::EDirectionProperty(c);
}

//=================================================================================
//      class ICharacterPropertyIterator
//=================================================================================



ICharacterPropertyIterator::ICharacterPropertyIterator( IUnicode::ECharacterProperty lowPropertyBound,
                                                                                                                IUnicode::ECharacterProperty highPropertyBound ) :
                                                                                        fLowPropertyBound( lowPropertyBound ),
                                                                                        fHighPropertyBound( highPropertyBound ),
                                                                                        fCurrentPosition( UUnicodeSpecial::kNull )
{

        fUnicodeImplementation = new IUnicode;

}




ICharacterPropertyIterator::ICharacterPropertyIterator( const ICharacterPropertyIterator& other ) :
                                                                                        fLowPropertyBound( other.fLowPropertyBound ),
                                                                                        fHighPropertyBound( other.fHighPropertyBound ),
                                                                                        fCurrentPosition( other.fCurrentPosition )
{
        fUnicodeImplementation = new IUnicode( *other.fUnicodeImplementation );
}




ICharacterPropertyIterator::~ICharacterPropertyIterator()
{
        delete fUnicodeImplementation;
}




ICharacterPropertyIterator&
ICharacterPropertyIterator::operator=( const ICharacterPropertyIterator& other )
{
        if ( this != &other ) {
                fLowPropertyBound = other.fLowPropertyBound;
                fHighPropertyBound = other.fHighPropertyBound;
                fCurrentPosition = other.fCurrentPosition;
                delete fUnicodeImplementation;
                fUnicodeImplementation = new IUnicode( *other.fUnicodeImplementation );
                }
        return *this;
}




void
ICharacterPropertyIterator::setRange( IUnicode::ECharacterProperty lowPropertyBound,
                                                                          IUnicode::ECharacterProperty highPropertyBound )
{

        fLowPropertyBound = lowPropertyBound;
        fHighPropertyBound = highPropertyBound;
}




void
ICharacterPropertyIterator::range( IUnicode::ECharacterProperty& lowPropertyBound,
                                                                          IUnicode::ECharacterProperty& highPropertyBound ) const
{
        lowPropertyBound = fLowPropertyBound;
        highPropertyBound = fHighPropertyBound;
}


ICharacterPropertyIterator::operator bool()      const
{
    if (fCurrentPosition == UUnicodeSpecial::kNotAUnicode) {
        return false;
    }
    return true;
}


ICharacterPropertyIterator&
ICharacterPropertyIterator::operator++()
{
        UniChar returnValue = UUnicodeSpecial::kNotAUnicode;

        if ( fCurrentPosition != UUnicodeSpecial::kNotAUnicode ) {
                do {
                        fCurrentPosition++;
                        IUnicode::ECharacterProperty theProperty = fUnicodeImplementation->type( fCurrentPosition );
                        if ( theProperty >= fLowPropertyBound && theProperty <= fHighPropertyBound ) {
                                returnValue = fCurrentPosition;
                                }
                        }
                while ( returnValue == UUnicodeSpecial::kNotAUnicode &&  fCurrentPosition != UUnicodeSpecial::kNotAUnicode );
        }
        return *this;
}

ICharacterPropertyIterator
ICharacterPropertyIterator::operator++(int)
{
        UniChar returnValue = UUnicodeSpecial::kNotAUnicode;
        ICharacterPropertyIterator tmp(*this);

        if ( fCurrentPosition != UUnicodeSpecial::kNotAUnicode ) {
                do {
                        fCurrentPosition++;
                        IUnicode::ECharacterProperty theProperty = fUnicodeImplementation->type( fCurrentPosition );
                        if ( theProperty >= fLowPropertyBound && theProperty <= fHighPropertyBound ) {
                                returnValue = fCurrentPosition;
                                }
                        }
                while ( returnValue == UUnicodeSpecial::kNotAUnicode &&  fCurrentPosition != UUnicodeSpecial::kNotAUnicode );
        }
        return tmp;
}

UniChar
ICharacterPropertyIterator::operator*() const
{

        if ( fCurrentPosition != UUnicodeSpecial::kNotAUnicode ) {
                return fCurrentPosition;
                }
        return UUnicodeSpecial::kNotAUnicode;
}


void
ICharacterPropertyIterator::reset()
{
        for (   fCurrentPosition = UUnicodeSpecial::kNull;
                        fCurrentPosition != UUnicodeSpecial::kNotAUnicode;
                        fCurrentPosition++ ) {
                IUnicode::ECharacterProperty theProperty = fUnicodeImplementation->type( fCurrentPosition );
                if ( theProperty >= fLowPropertyBound && theProperty <= fHighPropertyBound ) {
                        break;
                        }
                }
}
