/* Revision: 75 1.5.1.3 source/core/utils/fpbitdef.h, utilities, ioc.v400, 000302  */

/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

#ifndef _FPBITDEF_
#define _FPBITDEF_

/*************************************************************************************
*
*       File: FPBitDef.h
*
*       Not intended for general use.
*       For library and compiler internal use.
*
*       Abstracts platform data dependencies.
*       Representations for floating-point values as integer fields.
*
*       Note: File currently understands:
*               __X86__ => Intel byte ordering-- changed to __HHW_INTEL__ which is defined by VACPP
*               _AIX_   => PowerPC byte ordering
*               __68K__ => Motorola byte ordering
*
*       Warning: This file expects "long" to be 32-bit and "short" 16-bit.
*
*       Author: Clayton Lewis
*       All rights reserved
*
*       Modification History:-
*               SJP changed "long" to "unsigned long" so that type is treated as a bit-pattern
*               08/24/93        CRL     reworked else clauses for 68K, X86 and PPC
*       02/09/94    SJP changed __PPC__ to _AIX_ for SDK1
*
**************************************************************************************/

/*****************************
 *      For guts of float precision
 *****************************/

typedef union {
        unsigned long _I;
        float _F;
} _FloatOrLong;

/*****************************
 *      For guts of double precision
 *****************************/

typedef struct {
        #ifdef __HHW_INTEL__
                unsigned long _Two;
                unsigned long _One;
        #elif   __68K__
                unsigned long _One;
                unsigned long _Two;
        #elif   _AIX_
                unsigned long _One;
                unsigned long _Two;
        #endif
} _TwoPieces;

typedef union {
        _TwoPieces _I;
        double _F;
} _DoubleOrTwoPieces;

/*****************************
 *      For guts of IEEE extended precision
 *****************************/
#ifndef _AIX_

typedef struct {
        #ifdef __HHW_INTEL__
                unsigned long  _Three;
                unsigned long  _Two;
                unsigned short _One;
        #elif   __68K__
                unsigned short _One;
                unsigned short _Junk;
                unsigned long  _Two;
                unsigned long  _Three;
        #endif
} _ThreePieces;

typedef union {
        _ThreePieces _I;
        long double _F;
} _ExtendedOrThreePieces;

#endif                                          // #ifndef _AIX_

/*****************************
 *      For guts of double-double precision
 *****************************/

#ifdef _AIX_
        typedef struct {
                        unsigned long  _One;
                        unsigned long  _Two;
                        unsigned long   _Three;
                        unsigned long   _Four;
        } _FourPieces;

        typedef struct {
                        double _One;
                        double _Two;
        } _TwoDoubles;

        typedef union {
                _FourPieces _I4;
                long double _LD;
                _TwoDoubles _DD;
        } _DoubleDouble;
#endif

#ifdef __OS2__
#include <os2/os2utils.hpp>
#endif

#endif // _FPBITDEF_
