// Revision: 54 1.9.1.4 source/core/utils/iautoptr.hpp, filesystem, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

#ifndef _IAUTOPTR_
#define _IAUTOPTR_

#ifndef _IBASEPL_
#include <ibasepl.hpp>          // Need the base library
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

// Temporary version of auto_ptr until we get the ANSI libraries
template<class T> class auto_ptr
{
public:
        // _lib.auto.ptr.cons_ construct/copy/destroy:
                                        auto_ptr(T* p =0);
                                        auto_ptr(auto_ptr<T>&);
        auto_ptr<T>&    operator=(auto_ptr<T>&);
                                        ~auto_ptr();

        // _lib.auto.ptr.members_ members:
        T&                              operator*() const;
        T*                              operator->() const;
        T*                              get() const;
        T*                              release();
        void                    reset(T* p =0);

private:
        T*                              fObject;
};

template <class T> class IAutoArray
{
public:
                                        IAutoArray(T* p =0);
                                        IAutoArray(IAutoArray<T>&);
        IAutoArray<T>&  operator=(IAutoArray<T>&);
                                        ~IAutoArray();

        T&                              operator[](int i) const;
        T*                              get() const;
        T*                              release();
        void                    reset(T* p =0);

private:
        T*                              fObject;
};


//
//  This one is just for raw help buffer cleanup. The above auto_ptr class
//  won't work for allocation of non-object and non-structure buffers because
//  the -> operator is not legal for them. And, this one does not need to be
//  a template either since it can work on any raw heap buffer.
//
class IAutoHeap
{
public :
                    IAutoHeap(void* p = 0);
                    ~IAutoHeap();

    void*           get() const;
    void*           release();
    void            reset(void* p = 0);

private :
                    IAutoHeap(const IAutoHeap&) {}
    void            operator=(const IAutoHeap&) {}

    void*           fBuffer;
};

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <iautoptr.inl>

#endif // _IAUTOPTR_
