// Revision: 86 1.5.1.3 source/core/utils/icntptr.cpp, utilities, ioc.v400, 000302 
// icntptr.cpp
//
// Contains:
//      ICountedPointerTo - A smart pointer class that does reference counting
//                                              on the object to which it points.
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#include "icntptr.hpp"

#include <iexcept.hpp>
#include <icconsta.h>
#include <istrmmod.hpp>

#ifdef IC_PAGETUNE
    #define _ICNTPTR_CPP_
    #include <ipagetun.h>
#endif

//---------------------------------------------------------------------------------
// IVoidMasterPointer
//
static IStreamModule gModule("IVoidMasterPointer");
StreamableDefinitionsMacro(IVoidMasterPointer, gModule);

IVoidMasterPointer::IVoidMasterPointer(void* adoptObject, Deleter function)
        : fObject(adoptObject), fDeleter(function)
{
}

IVoidMasterPointer::~IVoidMasterPointer()
{
        (*fDeleter)(fObject);
}

void
IVoidMasterPointer::throwInvalid()
{
     ITHROWLIBRARYERROR(IC_NULL_IREFERENCE, IBaseErrorInfo::invalidRequest,
                                                IException::recoverable);
}

IVoidMasterPointer::IVoidMasterPointer()
        : fObject(0), fDeleter(0)
{
}

IVoidMasterPointer::IVoidMasterPointer(const IVoidMasterPointer& other)
        : fObject(0), fDeleter(other.fDeleter)
{
}

void
IVoidMasterPointer::operator=(const IVoidMasterPointer& other)
{
}

void
IVoidMasterPointer::writeToStream(IDataStream&) const
{
}

void
IVoidMasterPointer::readFromStream(IDataStream&)
{
}
