// Revision: 12 1.6.1.3 source/core/utils/igenexc.cpp, filesystem, ioc.v400, 000302 
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/

// igenexc.cpp
//
// Contains:    Implementations of the generic exception classes
//                              declared in igenexc.hpp
//
// Author:
//      Mike Seilnacht, Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#ifndef _IMSGTEXT_
#include <imsgtext.hpp>
#endif

#ifndef IC_IGENEXC
#include "igenexc.hpp"
#endif

#ifndef _IEXCBASE_
#include <iexcbase.hpp>
#endif

#ifndef _ISTRING_
#include <istring.hpp>
#endif

#include "igenmsg.h"

#ifdef IC_PAGETUNE
    #define _IGENEXC_CPP_
    #include <ipagetun.h>
#endif


//--------------------------------------------------------------------
// exceptionText
//      A handy utility method that will construct an exception's message
//      string.  It lets us avoid putting nasty ?: expressions inside
//      of ctor initializer lists.
//
static IString
exceptionText(
        const char *textArg,
        unsigned long errorId,
        const char*   insert1 = 0,
        const char*   insert2 = 0,
        const char*   insert3 = 0,
        const char*   insert4 = 0,
        const char*   insert5 = 0,
        const char*   insert6 = 0,
        const char*   insert7 = 0,
        const char*   insert8 = 0,
        const char*   insert9 = 0 )
{
        IString result;
        if (textArg) {
                result = textArg;
        } else {
                IMessageText message( errorId, kMsgFileName,
                                                                insert1, insert2, insert3,
                                                                insert4, insert5, insert6,
                                                                insert7, insert8, insert9 );
                result = message.text();
        }
        return result;
}

//--------------------------------------------------------------------
// IObjectNotFound
//
IObjectNotFound::IObjectNotFound(const char *textArg)
        : IAccessError(exceptionText(textArg, kObjectNotFound),
                         kObjectNotFound,
                         recoverable)
{
}

IObjectNotFound::IObjectNotFound(const IObjectNotFound &copy)
        : IAccessError(copy)
        {
}

IObjectNotFound::~IObjectNotFound()
{
}

const char * IObjectNotFound::name() const
{
        return "IObjectNotFound";
}


//--------------------------------------------------------------------
// IInvalidName
//
IInvalidName::IInvalidName(const char *text)
        : IInvalidRequest(exceptionText(text, kInvalidName),
                                                 kInvalidName,
                                                 recoverable)
{
}

IInvalidName::IInvalidName(const IInvalidName &copy)
        : IInvalidRequest(copy)
{
}

IInvalidName::~IInvalidName()
{
}

const char * IInvalidName::name() const
{
        return "IInvalidName";
}


//--------------------------------------------------------------------
// ICannotProceed
//
ICannotProceed::ICannotProceed(const char *text)
        : IException(exceptionText(text, kCannotProceed),
                                                 kCannotProceed,
                                                 recoverable)
{
}

ICannotProceed::ICannotProceed(const ICannotProceed &copy)
        : IException(copy)
{
}

ICannotProceed::~ICannotProceed()
{
}

const char * ICannotProceed::name() const
{
        return "ICannotProceed";
}


//--------------------------------------------------------------------
// IAlreadyExists
//
IAlreadyExists::IAlreadyExists(const char *text)
        : IAccessError(exceptionText(text, kAlreadyExists),
                                                 kAlreadyExists,
                                                 recoverable)
{
}

IAlreadyExists::IAlreadyExists(const IAlreadyExists &copy)
        : IAccessError(copy)
{
}

IAlreadyExists::~IAlreadyExists()
{
}

const char * IAlreadyExists::name() const
{
        return "IAlreadyExists";
}


//--------------------------------------------------------------------
// IMustBeEmpty
//
IMustBeEmpty::IMustBeEmpty(const char *text)
        : IAccessError(exceptionText(text, kMustBeEmpty),
                                         kMustBeEmpty,
                                         recoverable)
{
}

IMustBeEmpty::IMustBeEmpty(const IMustBeEmpty &copy)
        : IAccessError(copy)
{
}

IMustBeEmpty::~IMustBeEmpty()
{
}

const char * IMustBeEmpty::name() const
{
        return "IMustBeEmpty";
}
