// Revision: 49 1.11.1.3 source/core/utils/imrefcnt.cpp, utilities, ioc.v400, 000302 
// imrefcnt.cpp
//
// Contains:    IMRefCounted: a thread-safe, reference-counted base class
//
// Author:
//      Laura Werner
//
// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//
#include <imrefcnt.hpp>

#ifdef IC_PAGETUNE
    #define _IMREFCNT_CPP_
    #include <ipagetun.h>
#endif

//---------------------------------------------------------------------------------
// IMRefCounted
//
IMRefCounted::IMRefCounted()
        : fCount(0)
{
}

//
//  Note that we do not copy the other's count! This is not an error, it is
//  desired behaviour! We are constructing a new object which happens to
//  mix in IMRefCounted, so we want to start with a 0 count.
//
IMRefCounted::IMRefCounted(const IMRefCounted&)
        : fCount(0)
{
}

IMRefCounted::~IMRefCounted()
{
}

//
//  Note that we do not copy the other's count! This is not an error, it is
//  desired behaviour! The content of an object that has mixed in IMRefCnt
//  is being assigned, but that should not change the reference count of the
//  object.
//
IMRefCounted&
IMRefCounted::operator=(const IMRefCounted& other)
{
        return *this;
}
