// COPYRIGHT:
//      IBM Open Class Library
//      (C) Copyright International Business Machines Corporation 1992, 1997
//      (C) Copyright Talgent, Inc., 1996
//      Licensed Material - Program-Property of IBM - All Rights Reserved.
//      US Government Users Restricted Rights - Use, duplication, or disclosure
//      restricted by GSA ADP Schedule Contract with IBM Corp.
//

// Revision: 50 1.3.2.2 source/core/utils/utiltest.cpp, utilities, ioc.v400, 000302  
// utiltest.cpp

#include <stdio.h>
#include "isynchr.hpp"
#include "imrefcnt.hpp"

ISynchronized<long> testMe;

class ITestRefCounted : public IMRefCounted {
public:
	ITestRefCounted();
	~ITestRefCounted();
};
ITestRefCounted::ITestRefCounted() {}
ITestRefCounted::~ITestRefCounted() {}

int main()
{
	int result;
	
	printf("&testMe = %08lx\n", &testMe);

	printf("Calling SetConditional...\n");
	result = testMe.SetConditional(0, 1);
	printf("SetConditional returned %d, testMe.fAtom=%ld\n", result, testMe.fAtom);
	
	printf("Calling Get...\n");
	long value = testMe.Get();
	printf("Get returned %ld, testMe.fAtom=%ld\n", value, testMe.fAtom);

	printf("Calling Set...\n");
	testMe.Set( 2 );
	printf("after Set, testMe.fAtom=%ld\n", testMe.fAtom);
	
	printf("\nSimple ICountedPointerTo test...\n");
	ICountedPointerTo<ITestRefCounted> ptr1;
	ICountedPointerTo<ITestRefCounted> ptr2 = new ITestRefCounted();
	printf("ptr1: getAlias=%lx, count()=%ld\n",   ptr1.getAlias(), ptr1.count());
	printf("ptr2: getAlias=%lx, count() = %ld\n", ptr2.getAlias(), ptr2.count());

	printf("ptr1 = ptr2...\n");
	ptr1 = ptr2;
	printf("ptr1: getAlias=%lx, count() = %ld\n", ptr1.getAlias(), ptr1.count());
	printf("ptr2: getAlias=%lx, count() = %ld\n", ptr2.getAlias(), ptr2.count());

	printf("ptr1 = 0...\n");
	ptr1 = 0;
	printf("ptr1: getAlias=%lx, count() = %ld\n", ptr1.getAlias(), ptr1.count());
	printf("ptr2: getAlias=%lx, count() = %ld\n", ptr2.getAlias(), ptr2.count());
	
	return 0;
}
