// Revision: 91 1.1.1.2 source/dde/iddecset.cpp, dde, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrcset.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of:                                  *
*     IDDEActiveServerSet                                                      *
*     IDDEClientHotLinkSet                                                     *
*                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <itrace.hpp>
#include <iddecset.hpp>
#include <iset2.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDDECSET_CPP_
  #include <ipagetun.h>
#endif


extern bool IDdeleteElemCHLS ( IDDEClientHotLinkEvent* const& hotLinkEvent,
                               void*                          item );

extern bool IDdeleteElemASS ( IDDEActiveServer* const& activeServer,
                              void*  );


class IDDEActiveServerSetImp : public IVPtrSet<IDDEActiveServer*>
{
  
};

                                       

/****************************************************************/
/*   IDDEActiveServerSet::Cursor Member Functions               */
/****************************************************************/

IDDEActiveServerSet::Cursor::Cursor (const IDDEActiveServerSet& controlList)
         : fServerSetImp(controlList.fServerSetImp),
           fServerCursorImp(new IDDEActiveServerSetImp::Cursor(*fServerSetImp))  
{}
       

IDDEActiveServerSet::Cursor::~Cursor ( )
{
   delete (IDDEActiveServerSetImp::Cursor*)fServerCursorImp;
}


bool IDDEActiveServerSet::Cursor::setToFirst ()
{ 
   return ((IDDEActiveServerSetImp::Cursor*)fServerCursorImp)->setToFirst();
}


bool IDDEActiveServerSet::Cursor::setToNext ()
{ 
   return ((IDDEActiveServerSetImp::Cursor*)fServerCursorImp)->setToNext();
}


bool IDDEActiveServerSet::Cursor::isValid() const
{ 
   return ((IDDEActiveServerSetImp::Cursor*)fServerCursorImp)->isValid();
}

void IDDEActiveServerSet::Cursor::invalidate ( )
{ 
   ((IDDEActiveServerSetImp::Cursor*)fServerCursorImp)->invalidate();
}


IDDEActiveServer* const IDDEActiveServerSet::Cursor::element ( ) const
{ 
   return ((const IDDEActiveServerSetImp::Cursor*)fServerCursorImp)->element();
}     



/****************************************************************/
/*   IDDEActiveServerSet Member Functions                       */
/****************************************************************/

IDDEActiveServerSet :: IDDEActiveServerSet() 
    : fServerSetImp(new IDDEActiveServerSetImp)
{}


IDDEActiveServerSet :: ~IDDEActiveServerSet() 
{
  removeAll(&IDdeleteElemASS);
  delete fServerSetImp;   
}


bool IDDEActiveServerSet::add ( IDDEActiveServer* const& object)
{
   return fServerSetImp->add(object);
}


IDDEActiveServer* IDDEActiveServerSet::elementAt   ( const Cursor& cursor) const
{
  return fServerSetImp->elementAt(*(IDDEActiveServerSetImp::Cursor*)(cursor.fServerCursorImp));
}

bool IDDEActiveServerSet::isEmpty ( ) const 
{
  return fServerSetImp->isEmpty();
}

unsigned long IDDEActiveServerSet::numberOfElements ( ) const 
{
  return fServerSetImp->numberOfElements();
}

unsigned long IDDEActiveServerSet::removeAll ( )
{
  return fServerSetImp->removeAll();
}

unsigned long IDDEActiveServerSet::removeAll ( bool (*predicateFunction)(IDDEActiveServer* const& , void*), void* data)
{
  return fServerSetImp->removeAll(predicateFunction, data);
}

void IDDEActiveServerSet::removeAt ( Cursor& cursor)
{
  fServerSetImp->removeAt(*(IDDEActiveServerSetImp::Cursor*)cursor.fServerCursorImp);
}


void IDDEActiveServerSet::replaceAt ( const Cursor& cursor, 
                                IDDEActiveServer* const& activeServer)
{
  fServerSetImp->replaceAt(*(const IDDEActiveServerSetImp::Cursor*)cursor.fServerCursorImp, activeServer);
}


bool IDDEActiveServerSet::allElementsDo( bool (*function) (IDDEActiveServer* &, void*),
                    void* additionalArgument)
{
   return fServerSetImp->allElementsDo(function, additionalArgument);
}



class IDDEClientHotLinkSetImp : public IVPtrSet<IDDEClientHotLinkEvent*>
{
  
};

                                       


IDDEClientHotLinkSet::Cursor::Cursor (const IDDEClientHotLinkSet& hotLinkSet)
         : fClientSetImp(hotLinkSet.fClientSetImp),
           fClientCursorImp(new IDDEClientHotLinkSetImp::Cursor(*fClientSetImp))  
{}
       

IDDEClientHotLinkSet::Cursor::~Cursor ( )
{
   delete (IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp;
}


bool IDDEClientHotLinkSet::Cursor::setToFirst ()
{ 
   return ((IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp)->setToFirst();
}


bool IDDEClientHotLinkSet::Cursor::setToNext ()
{ 
   return ((IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp)->setToNext();
}


bool IDDEClientHotLinkSet::Cursor::isValid() const
{ 
   return ((IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp)->isValid();
}

void IDDEClientHotLinkSet::Cursor::invalidate ( )
{ 
   ((IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp)->invalidate();
}


IDDEClientHotLinkEvent* const IDDEClientHotLinkSet::Cursor::element ( ) const
{ 
   return ((const IDDEClientHotLinkSetImp::Cursor*)fClientCursorImp)->element();
}     




IDDEClientHotLinkSet :: IDDEClientHotLinkSet() 
    : fClientSetImp(new IDDEClientHotLinkSetImp)
{}


 IDDEClientHotLinkSet :: ~IDDEClientHotLinkSet() 
{
  // remove and delete all old members of the client hot link set
  removeAll(&IDdeleteElemCHLS);

  delete fClientSetImp;   
}


bool IDDEClientHotLinkSet::add ( IDDEClientHotLinkEvent* const& hotLinkEvent)
{
   return fClientSetImp->add(hotLinkEvent);
}


IDDEClientHotLinkEvent* IDDEClientHotLinkSet::elementAt   ( const Cursor& cursor) const
{
  return fClientSetImp->elementAt(*(IDDEClientHotLinkSetImp::Cursor*)(cursor.fClientCursorImp));
}

bool IDDEClientHotLinkSet::isEmpty ( ) const 
{
  return fClientSetImp->isEmpty();
}

unsigned long IDDEClientHotLinkSet::numberOfElements ( ) const 
{
  return fClientSetImp->numberOfElements();
}

unsigned long IDDEClientHotLinkSet::removeAll ( )
{
  return fClientSetImp->removeAll();
}

unsigned long IDDEClientHotLinkSet::removeAll ( bool (*predicateFunction)(IDDEClientHotLinkEvent* const& , void*), void* data)
{
  return fClientSetImp->removeAll(predicateFunction, data);
}

void IDDEClientHotLinkSet::removeAt ( Cursor& cursor)
{
  fClientSetImp->removeAt(*(IDDEClientHotLinkSetImp::Cursor*)cursor.fClientCursorImp);
}


void IDDEClientHotLinkSet::replaceAt ( const Cursor& cursor, 
                                IDDEClientHotLinkEvent* const& hotLinkEvent)
{
  fClientSetImp->replaceAt(*(const IDDEClientHotLinkSetImp::Cursor*)cursor.fClientCursorImp, hotLinkEvent);
}



bool IDDEClientHotLinkSet::allElementsDo( bool (*function) (IDDEClientHotLinkEvent* &, void*),
                    void* additionalArgument )
{
   return fClientSetImp->allElementsDo(function, additionalArgument);
}


/*------------------------------------------------------------------------------
| IDDEClientHotLinkSet::semaphor                                               |
------------------------------------------------------------------------------*/
IPrivateResource& IDDEClientHotLinkSet::semaphor ( )
{
   return priResCl;
}
