// Revision: 70 1.4.1.5 source/ui/baseapp/iaccelbd.hpp, userinput, ioc.v400, 000302  
/* NOSHIP */
#ifndef _IACCELBD_
#define _IACCELBD_
/*******************************************************************************
* FILE NAME: iaccelbd.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Contains the declaration of the IAccelBld class. This class is used        *
*   by IResourceLibrary::loadAccelTable() to parse Application Resources that  *
*   represent accelerator definition as defined in a PM .rc file.              *
*   This class is not intended to be used by customers.                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandle.hpp>
#include <istring.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

struct rtacceltable;

#pragma enum(4)              
#pragma pack(push,4)
                
class IAccelBld {
public:

  IAccelBld ( char*         res,
              IWindowHandle menu );
 ~IAccelBld ( );

void
  getTokens     ( );
void
  parseIt       ( IString& curTok );
void
  subParseIt    ( IString& curTok );
void
  accelitemDone ( IString& curTok );
void
  clearItemData ( );

static IAccelTblHandle
  rtacceltableToHandle ( const rtacceltable* resAccelTable );

private:
  // Holds the string representing the entire accelerator table that
  // was obtained from the resource database.  Passed in on the ctor.
IString
  resString;

  // Holds the position of the current word in resString.
unsigned long
  curPos;

  // Handle of the menu object (menu bBar or pop-up menu) to be updated.
  // Passed into loadAccelTable() and passed in on the ctor.
IWindowHandle
  menu;

  // States for 2 parsers.
  // States used in parseIt()
enum State
  { looking, accelitem };
  // States used in subParseIt()
enum SubState
  { startString, cmdId, style };
State
  state;
SubState
  subState;

  // Data for accelerator item currently being built up.
IString
  accelString;
unsigned long
  menuId;
bool
  isCtrl,
  isShift,
  isAlt,
  isChar;

  // Alt key mask for the current server.
unsigned long
  altMask;
}; // IAccelBld

#pragma pack(pop)    
#pragma enum(pop)
      
#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
