// Revision: 71 1.4.1.2 source/ui/baseapp/iaccelhd.cpp, userinput, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: iaccelhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iaccelhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

#define INCL_WINACCELERATORS
#define INCL_WINFRAMEMGR
extern "C"
{
  #include <iwindefs.h>
}

#include <iaccelhd.hpp>
#include <ihandle.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IACCELHD_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_PM
/*------------------------------------------------------------------------------
| IAcceleratorKeyHandlerStatics                                                |
|                                                                              |
| Class to manage destruction of the static IAcceleratorKeyHandler object.     |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IAcceleratorKeyHandlerStatics {
public:
 ~IAcceleratorKeyHandlerStatics ( )
  {
    if ( staticHandler )
    {
       delete staticHandler;
    }
  }
IAcceleratorKeyHandler
 *staticHandler;
}; // IAcceleratorKeyHandlerStatics

#pragma pack(pop)
#pragma enum(pop)

static IAcceleratorKeyHandlerStatics
  IAcceleratorKeyHandlerStaticsObject;


/*------------------------------------------------------------------------------
| IAcceleratorKeyHandler::IAcceleratorKeyHandler                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IAcceleratorKeyHandler::IAcceleratorKeyHandler ( )
  : IAcceleratorKeyHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| IAcceleratorKeyHandler::~IAcceleratorKeyHandler                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IAcceleratorKeyHandler::~IAcceleratorKeyHandler ( )
{ }

/*------------------------------------------------------------------------------
| IAcceleratorKeyHandler::dispatchHandlerEvent                                 |
|                                                                              |
| Dispatch potential accelerator key events for this handler.                  |
------------------------------------------------------------------------------*/
bool IAcceleratorKeyHandler::dispatchHandlerEvent ( IEvent & event )
{
  bool
    stopProcessing = false;

  if ( event.eventId() == WM_TRANSLATEACCEL )
  {
     stopProcessing = this->key( event );
  }

  return stopProcessing;
}

/*------------------------------------------------------------------------------
| IAcceleratorKeyHandler::key                                                  |
|                                                                              |
| Translate the event using the window's accelerator table.                    |
------------------------------------------------------------------------------*/
bool IAcceleratorKeyHandler::key ( IEvent& event )
{
  bool
    stopProcessing = false;
  IWindow
   *accelWindow = event.dispatchingWindow();
  IAccelTblHandle
    accelHandle( accelWindow->acceleratorHandle() );
  if ( accelHandle  &&
       WinTranslateAccel( IThread::current().anchorBlock(),
                          accelWindow->handle(),
                          accelHandle,
                          (PQMSG)(char*)event.parameter1() ) )
  {  // The message was translated.
     event.setResult( true );
     stopProcessing = true;
  }
  return stopProcessing;
}

/*------------------------------------------------------------------------------
| IAcceleratorKeyHandler::defaultHandler                                       |
|                                                                              |
| Return the static handler object used by all windows.                        |
------------------------------------------------------------------------------*/
IAcceleratorKeyHandler* IAcceleratorKeyHandler::defaultHandler ( )
{
  if ( ! IAcceleratorKeyHandlerStaticsObject.staticHandler )
  {
     IAcceleratorKeyHandlerStaticsObject.staticHandler =
                                            new IAcceleratorKeyHandler();
  }
  return IAcceleratorKeyHandlerStaticsObject.staticHandler;
}

#endif // IC_PM
