// Revision: 73 1.9.1.3 source/ui/baseapp/iaccelky.cpp, userinput, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iaccelky.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iaccelky.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #define INCL_WINACCELERATORS          // For struct _ACCEL.
  #define INCL_WINFRAMEMGR              // For SC_xxx.
  #define INCL_WININPUT                 // For VK_xxx.
  #include <iwindefs.h>
  }

#ifdef IC_MOTIF
extern "C" {
  #ifdef IC_SUN
    #include <X11/Sunkeysym.h>
  #else
    #include <X11/aix_keysym.h>
  #endif
  #include <X11/keysym.h>
  }
#endif

#include <iaccelky.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <iframe.hpp>
#include <ihandle.hpp>
#include <istrenum.hpp>
#include <istring.hpp>

// Flags used to map to/from an ACCEL structure.
#ifdef IC_WIN
  #define AF_VIRTUALKEY  FVIRTKEY       // TRUE
  #define AF_SHIFT       FSHIFT         // 0x04
  #define AF_CONTROL     FCONTROL       // 0x08
  #define AF_ALT         FALT           // 0x10
#endif
#ifdef IC_MOTIF
  #define AF_SHIFT       ShiftMask      // 0x01 (1<<0)
  #define AF_CONTROL     ControlMask    // 0x04 (1<<2)
  #define AF_ALT         Mod1Mask       // 0x08 (1<<3)
  #define AF_VIRTUALKEY  0x0040
  #define AF_SYSCOMMAND  0x0080
  #define AF_HELP        0x0100
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IACCELKY_CPP_
  #include <ipagetun.h>
#endif

// Private implementation classes.
#pragma enum(4)
#pragma pack(push,4)

/*------------------------------------------------------------------------------
| IAcceleratorKeyData                                                          |
------------------------------------------------------------------------------*/
class IAcceleratorKeyData  {
public:
  IAcceleratorKeyData ( );
  IAcceleratorKeyData ( const ACCEL&               accel );
  IAcceleratorKeyData ( const IAcceleratorKeyData& keyData );
IAcceleratorKeyData
 &operator=           ( const IAcceleratorKeyData& keyData );
bool
  operator==          ( const IAcceleratorKeyData& keyData ) const;

enum EKeyType
  { eCharacter, eVirtualKey, eScanCode };
EKeyType
  fKeyType;
#ifdef IC_MOTIF
KeySym
  fKey;
#else
unsigned short
  fKey;
#endif
IKey::KeyModifier
  fKeyModifier;
ICommand::CommandId
  fCmd;
ICommand::ActionType
  fActionType;
bool
  fFromACCEL;
unsigned long
  fUniqueKey;
};  // IAcceleratorKeyData

#pragma pack(pop)
#pragma enum(pop)

IAcceleratorKeyData::IAcceleratorKeyData ( )
  : fKeyType( IAcceleratorKeyData::eCharacter ),
    fKey( 0 ),
    fKeyModifier( IKey::noModifier ),
    fCmd( 0 ),
    fActionType( ICommand::applicationCommand ),
    fFromACCEL( false ),
    fUniqueKey( 0 )
{ }

IAcceleratorKeyData::IAcceleratorKeyData ( const ACCEL& accel )
  : fKeyType( IAcceleratorKeyData::eCharacter ),
    fKey( accel.key ),
    fKeyModifier( IKey::noModifier ),
#ifdef IC_PMWIN
    fCmd( accel.cmd ),
#endif
#ifdef IC_MOTIF
    fCmd( accel.commandId ),
#endif
    fActionType( ICommand::applicationCommand ),
    fFromACCEL( true ),
    fUniqueKey( 0 )          // Set by IAcceleratorKey.
{
#ifdef IC_PM
  unsigned short flags = accel.fs;
#endif
#ifdef IC_WIN
  unsigned short flags = accel.fVirt;
#endif
#ifdef IC_MOTIF
  unsigned short flags = accel.modifiers;
#endif

  if ( flags & AF_CONTROL )
  {
     fKeyModifier |= IKey::ctrl;
  }
  if ( flags & AF_ALT )
  {
     fKeyModifier |= IKey::alt;
  }
  if ( flags & AF_SHIFT )
  {
     fKeyModifier |= IKey::shift;
  }
  // Remove the default value if some modifier is being used.
  if ( fKeyModifier != IKey::noModifier )
  {
     fKeyModifier &= ~IKey::noModifier;
  }

#ifdef IC_MOTIF
  flags = accel.flags;
#endif
  if ( flags & AF_VIRTUALKEY )
  {
     fKeyType = IAcceleratorKeyData::eVirtualKey;
  }
#ifdef IC_PM
  else if ( flags & AF_SCANCODE )
  {
     fKeyType = IAcceleratorKeyData::eScanCode;
  }
#endif
#ifdef IC_MOTIFPM
  if ( flags & AF_SYSCOMMAND )
  {
     fActionType = ICommand::systemCommand;
  }
  else if ( flags & AF_HELP )
  {
     fActionType = ICommand::help;
  }
#endif
#ifdef IC_WIN
  if ( fCmd >= SC_SIZE )
  {           // Range of system command values in winuser.h.
     fActionType = ICommand::systemCommand;
  }

  // Note: No way to define a help key via an ACCEL structure.
#endif
}

IAcceleratorKeyData::IAcceleratorKeyData ( const IAcceleratorKeyData& keyData )
  : fKeyType( keyData.fKeyType ),
    fKey( keyData.fKey ),
    fKeyModifier( keyData.fKeyModifier ),
    fCmd( keyData.fCmd ),
    fActionType( keyData.fActionType ),
    fFromACCEL( keyData.fFromACCEL ),
    fUniqueKey( keyData.fUniqueKey )
{ }

IAcceleratorKeyData&
  IAcceleratorKeyData::operator= ( const IAcceleratorKeyData& keyData )
{
  if ( this != &keyData )
  {           // Not copying self.
     this->fKeyType = keyData.fKeyType;
     this->fKey = keyData.fKey;
     this->fKeyModifier = keyData.fKeyModifier;
     this->fCmd = keyData.fCmd;
     this->fActionType = keyData.fActionType;
     this->fFromACCEL = keyData.fFromACCEL;
     this->fUniqueKey = keyData.fUniqueKey;
  }
  return *this;
}

bool IAcceleratorKeyData::operator== ( const IAcceleratorKeyData& keyData ) const
{
  return ( this->fKeyType == keyData.fKeyType  &&
           this->fKey == keyData.fKey  &&
           this->fKeyModifier == keyData.fKeyModifier  &&
           this->fActionType == keyData.fActionType  &&
           this->fCmd == keyData.fCmd );
}


/*------------------------------------------------------------------------------
| ::key                                                                        |
|                                                                              |
| Global function for key set of IAcceleratorKey objects.  Allow only one      |
| accelerator table entry for a given key and modifier.                        |
------------------------------------------------------------------------------*/
unsigned long const& key ( IAcceleratorKey const& item )
{
  return item.fData->fUniqueKey;
}


/*------------------------------------------------------------------------------
| IAcceleratorKey::IAcceleratorKey                                             |
|                                                                              |
| Constructor for virtual key accelerator.                                     |
------------------------------------------------------------------------------*/
IAcceleratorKey::IAcceleratorKey ( IKey::VirtualKey         virtualKey,
                                   const IKey::KeyModifier& modifier,
                                   ICommand::ActionType     actionType,
                                   ICommand::CommandId      commandId )
  : fData( new IAcceleratorKeyData() )
{
  (*this)
   .setKey( virtualKey, modifier );

  switch ( actionType )
  {
     case (ICommand::applicationCommand):
        this->setCommand( commandId );
        break;
     case (ICommand::systemCommand):
        this->setSystemCommand( commandId );
        break;
     case (ICommand::help):
        this->setHelpKey();
        break;
     default:
        ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                            IBaseErrorInfo::invalidParameter,
                            IException::recoverable );
        break;
  }
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::IAcceleratorKey                                             |
|                                                                              |
| Constructor for character key accelerator.                                   |
------------------------------------------------------------------------------*/
IAcceleratorKey::IAcceleratorKey ( const IString&           commandKey,
                                   const IKey::KeyModifier& modifier,
                                   ICommand::ActionType     actionType,
                                   ICommand::CommandId      commandId )
  : fData( new IAcceleratorKeyData() )
{
  (*this)
   .setKey( commandKey, modifier );

  switch ( actionType )
  {
     case (ICommand::applicationCommand):
        this->setCommand( commandId );
        break;
     case (ICommand::systemCommand):
        this->setSystemCommand( commandId );
        break;
     case (ICommand::help):
        this->setHelpKey();
        break;
     default:
        ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                            IBaseErrorInfo::invalidParameter,
                            IException::recoverable );
        break;
  }
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::IAcceleratorKey                                             |
|                                                                              |
| Constructor from an ACCEL structure.                                         |
#ifdef IC_WIN                                                            
| Note: For Windows, ACCEL has no way to hold command type information.        |
#endif                                                                  
------------------------------------------------------------------------------*/
IAcceleratorKey::IAcceleratorKey ( const ACCEL& accelerator )
  : fData( new IAcceleratorKeyData( accelerator ) )
{
  this->fData->fUniqueKey = IAcceleratorKey::uniqueKeyFor( *this );
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::IAcceleratorKey                                             |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
IAcceleratorKey::IAcceleratorKey ( const IAcceleratorKey& key )
  : fData( new IAcceleratorKeyData( *( key.fData ) ) )
{ }

/*------------------------------------------------------------------------------
| IAcceleratorKey::IAcceleratorKey                                             |
------------------------------------------------------------------------------*/
IAcceleratorKey::IAcceleratorKey ( )
   : fData( new IAcceleratorKeyData() )
{ }

/*------------------------------------------------------------------------------
| IAcceleratorKey::operator=                                                   |
|                                                                              |
| Assignment operator.                                                         |
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::operator= ( const IAcceleratorKey& key )
{
  if ( this != &key )
  {           // Not copying self.
     *(this->fData) = *(key.fData);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::~IAcceleratorKey                                            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IAcceleratorKey::~IAcceleratorKey ( )
{
  delete this->fData;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::operator==                                                  |
|                                                                              |
| Equality check.                                                              |
------------------------------------------------------------------------------*/
bool IAcceleratorKey::operator== ( const IAcceleratorKey& key ) const
{
  return ( *(this->fData) == *(key.fData) );
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::isLike                                                      |
|                                                                              |
| Check for same key and modifier.                                             |
------------------------------------------------------------------------------*/
bool IAcceleratorKey::isLike ( const IAcceleratorKey& key ) const
{
  return ( ::key( *this ) == ::key( key ) );
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::operator <                                                  |
------------------------------------------------------------------------------*/
bool IAcceleratorKey::operator < ( const IAcceleratorKey& key ) const
{
  return ( ::key( *this ) < ::key( key ) );
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setKey                                                      |
|                                                                              |
| Assign character accelerator key.                                            |
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::setKey ( const IString&           accelKey,
                                           const IKey::KeyModifier& modifier )
{
  char accelChar[ 2 ];
  accelChar[ 0 ] = accelKey[ 1 ];
  if ( accelKey.charType( 1 ) == IStringEnum::sbcs )
  {
     accelChar[ 1 ] = 0;
  }
  else
  {
     accelChar[ 1 ] = accelKey[ 2 ];
  }

#ifdef IC_MOTIF
  if ( accelChar[ 1 ] == 0  &&
       ( accelChar[ 0 ] >= XK_space  &&  accelChar[ 0 ] <= XK_asciitilde ) )
  {  // This range (0x20-0x7e) of Latin I characters maps straight to
     // KeySym values.
     this->fData->fKey = (KeySym) accelChar[ 0 ];
  }
  else
  {  // Silently ignore this case, because we have no way to intelligently
     // map this to a keysym (the application should pass the corresponding
     // keysym as a virtual key code to the other version of setKey).
     this->fData->fKey = NoSymbol;
     ITRACE_DEVELOP( "Keycode " +
                     IString( *(unsigned short *)accelChar ).d2x() +
                     "ignored (not in range 0x20-0x7e)." );
  }
#else
  this->fData->fKey = *(unsigned short *)accelChar;
#endif
  this->fData->fKeyType = IAcceleratorKeyData::eCharacter;

  this->setKeyModifier( modifier );
  this->fData->fFromACCEL = false;
  this->fData->fUniqueKey = IAcceleratorKey::uniqueKeyFor( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setKey                                                      |
|                                                                              |
| Assign virtual key accelerator key.                                          |
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::setKey ( IKey::VirtualKey         virtualKey,
                                           const IKey::KeyModifier& modifier )
{
  IASSERTPARM( virtualKey != IKey::kNoKey );

#ifdef IC_MOTIF
  this->fData->fKey = (KeySym) virtualKey;
  // (An application can pass any keysym value here.)
#else
  this->fData->fKey = (unsigned short)virtualKey;
#endif
  this->fData->fKeyType = IAcceleratorKeyData::eVirtualKey;

  this->setKeyModifier( modifier );
  this->fData->fFromACCEL = false;
  this->fData->fUniqueKey = IAcceleratorKey::uniqueKeyFor( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::character                                                   |
|                                                                              |
| Return the character key accelerator key.                                    |
------------------------------------------------------------------------------*/
IString IAcceleratorKey::character ( ) const
{
  if ( this->fData->fKeyType != IAcceleratorKeyData::eCharacter )
  {
     return IString();
  }
  else
  {
#ifdef IC_MOTIF
     KeySym keysym = this->fData->fKey;
     char accelChar = (char) keysym;
#endif
#ifdef IC_PMWIN
     char accelChar[ 3 ];
  *(unsigned short *)accelChar = this->fData->fKey;
  accelChar[ 2 ] = 0;
#endif
     return IString( accelChar );
  }
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::virtualKey                                                  |
|                                                                              |
| Return the virtual key accelerator key.                                      |
------------------------------------------------------------------------------*/
IKey::VirtualKey IAcceleratorKey::virtualKey ( ) const
{
  if ( this->fData->fKeyType != IAcceleratorKeyData::eVirtualKey )
  {
     return IKey::kNoKey;
  }
  else
  {
     return (IKey::VirtualKey) this->fData->fKey;
  }
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::keyModifier                                                 |
|                                                                              |
| Return the keyboard modifier for the accelerator key.                        |
------------------------------------------------------------------------------*/
IKey::KeyModifier IAcceleratorKey::keyModifier ( ) const
{
  return this->fData->fKeyModifier;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setCommand                                                  |
|                                                                              |
| Assign the command that the accelerator key will run.                        |
#ifdef IC_WIN
| Note: The command ID should be below SC_SIZE.                                |
#endif
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::setCommand ( unsigned long commandId )
{
  // Set the flags to show this key will run an application command.
  this->fData->fActionType = ICommand::applicationCommand;

  // Now store the ID of the application command.
  this->fData->fCmd = commandId;

  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setSystemCommand                                            |
|                                                                              |
| Assign the system command that the accelerator key will run.                 |
#ifdef IC_WIN
| Note: The command ID should be SC_SIZE or greater in value.                  |
#endif
------------------------------------------------------------------------------*/
IAcceleratorKey&
  IAcceleratorKey::setSystemCommand ( unsigned long systemCommand )
{
  // Set the flags to show this key will run a system command.
  this->fData->fActionType = ICommand::systemCommand;

  // Now store the ID of the system command.
  this->fData->fCmd = systemCommand;

  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setHelpKey                                                  |
|                                                                              |
| Specify that this key will invoke a help request.                            |
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::setHelpKey ( )
{
  // Set the flags to show this key will invoke help.
  this->fData->fActionType = ICommand::help;

  this->fData->fCmd = IC_ID_HELP;
  // This command generates a help request.  Support for the command
  // is provided in IFrameHandler.
  // Note that this command will not automatically be propagated up
  // the parent window chain to a frame window, unless this
  // accelerator key is defined for a canvas or container (or the
  // application provides a command handler on the control
  // associated with this accelerator key to propagate the command).

  return *this;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::actionType                                                  |
|                                                                              |
| Return the type of event that the accelerator key will run.                  |
------------------------------------------------------------------------------*/
ICommand::ActionType IAcceleratorKey::actionType ( ) const
{
  return this->fData->fActionType;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::commandId                                                   |
|                                                                              |
| Return the identifier of the command that the accelerator key will run.      |
------------------------------------------------------------------------------*/
unsigned long IAcceleratorKey::commandId ( ) const
{
  unsigned long cmdId = this->fData->fCmd;
  if ( this->actionType() == ICommand::help )
  {
     cmdId = 0;
  }
  return cmdId;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::asACCEL                                                     |
|                                                                              |
| Convert to ACCEL structure.                                                  |
------------------------------------------------------------------------------*/
ACCEL IAcceleratorKey::asACCEL ( ) const
{
  ACCEL accel;
  unsigned short flags = 0;
  if ( this->fData->fKeyModifier & IKey::alt )
  {
     flags |= AF_ALT;
  }
  if ( this->fData->fKeyModifier & IKey::ctrl )
  {
     flags |= AF_CONTROL;
  }
  if ( this->fData->fKeyModifier & IKey::shift )
  {
     flags |= AF_SHIFT;
  }
#ifdef IC_MOTIF
  accel.modifiers = flags;
  flags = 0;
#endif

  if ( this->fData->fKeyType == IAcceleratorKeyData::eVirtualKey )
  {
     flags |= AF_VIRTUALKEY;
  }
#ifdef IC_PM
  else if ( this->fData->fKeyType == IAcceleratorKeyData::eScanCode )
  {
     flags |= AF_SCANCODE;
  }
  else
  {  // Assume eCharacter.
     flags |= AF_CHAR;
  }
#endif

#ifdef IC_MOTIFPM
  if ( this->fData->fActionType == ICommand::systemCommand )
  {
     flags |= AF_SYSCOMMAND;
  }
  else if ( this->fData->fActionType == ICommand::help )
  {
     flags |= AF_HELP;
  }
#endif

#ifdef IC_PM
  accel.fs = flags;
  accel.cmd = (unsigned short)this->fData->fCmd;
#endif
#ifdef IC_WIN
  accel.fVirt = (unsigned char) flags;
  accel.cmd = (unsigned short)this->fData->fCmd;
#endif
#ifdef IC_MOTIF
  accel.flags = flags;
  accel.commandId = (unsigned short)this->fData->fCmd;
#endif
  accel.key = this->fData->fKey;

#ifdef IC_WIN
  // Map the key, if appropriate, to a format better suited for
  // the presentation system.
  if ( ! ( this->fData->fFromACCEL ) )
  {  // If possible, map the key to a virtual key.  For some reason,
     // non-virtual keys in Windows do not support Alt/Ctrl/Shift.
     if ( ! ( accel.fVirt & AF_VIRTUALKEY )  &&
          ! ( accel.key & 0xFF00 ) )
     {  // AF_CHAR (character key), and single-byte for VkKeyScan.
        // We use VkKeyScan to see if the character really maps to
        // a virtual key.
        short virtKey = VkKeyScan( (TCHAR)accel.key );
        if ( virtKey != -1 )
        {  // Character key can be represented as a virtual key.
           // Note that this translation will not generate virtual
           // keys for the numeric keypad (e.g. VK_NUMPAD0 - VK_DIVIDE).
           accel.fVirt |= AF_VIRTUALKEY;
           accel.key = (WORD)( virtKey & 0xFF );  // Low word.

           // Check if shift/ctrl/alt need to be added (high word).

           // Some compilers get confused by the following expression
           // when there is a template function "key" defined.
           //     ( accel.key < 'A'  ||  accel.key > 'Z' ) )
           // The key member is qualified to avoid this problem.
           if ( ( virtKey & 0x0100 )  &&
                ( accel.ACCEL::key < 'A'  ||  accel.ACCEL::key > 'Z' ) )
           {          // Shift key pressed (and not alphabetic).
              accel.fVirt |= AF_SHIFT;
           }
           if ( virtKey & 0x0200 )
           {       // Ctrl key pressed.
              accel.fVirt |= AF_CONTROL;
           }
           if ( virtKey & 0x0400 )
           {       // Alt key pressed.
              accel.fVirt |= AF_ALT;
           }
        }
     }
  }

  // Ignore flag for application vs. system command.  Let Windows
  // ultimately distinguish the type of command by the ID value
  // instead.
#endif

  return accel;
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::uniqueKeyFor                                                |
|                                                                              |
| Convert accelerator key object to a unique key value for a keyed collection. |
------------------------------------------------------------------------------*/
unsigned long
  IAcceleratorKey::uniqueKeyFor ( const IAcceleratorKey& key )
{
  ACCEL accelData = key.asACCEL();
  unsigned long
    keysym = accelData.key,
#ifdef IC_PM
    flags = ( accelData.fs &
               ( AF_CHAR | AF_VIRTUALKEY | AF_SCANCODE |
                 AF_SHIFT | AF_CONTROL | AF_ALT | AF_LONEKEY ) );
#endif
#ifdef IC_WIN
    flags = ( accelData.fVirt &
               ( AF_VIRTUALKEY | AF_SHIFT | AF_CONTROL | AF_ALT ) );
#endif
#ifdef IC_MOTIF
    flags = accelData.modifiers;
#endif
  return ( keysym | ( flags << 16 ) );
}

/*------------------------------------------------------------------------------
| IAcceleratorKey::setKeyModifier                                              |
|                                                                              |
| Assign a key modifier for the accelerator.                                   |
------------------------------------------------------------------------------*/
IAcceleratorKey& IAcceleratorKey::setKeyModifier ( const IKey::KeyModifier& modifier )
{
  this->fData->fKeyModifier = modifier;

  return *this;
}
