// Revision: 98 1.8.1.3 source/ui/baseapp/icolor.cpp, datatype, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icolor.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icolor.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_GPILOGCOLORTABLE
#define INCL_WINSYS
#define INCL_WINDESKTOP
#include <iwindefs.h>
#ifdef IC_MOTIF
  #include <X11/Intrinsic.h>
#endif


#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICOLOR_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_PMWIN

/*------------------------------------------------------------------------------
| IDeviceColor::IDeviceColor                                                   |
|                                                                              |
| Construct an IDeviceColor from a DeviceColor enumerator value.               |
------------------------------------------------------------------------------*/
IDeviceColor::IDeviceColor(DeviceColor dclr)
   : IColor()
{
   switch (dclr) {
   case background:
      dcClDeviceColor = background;
#ifdef IC_WIN
      this->setGreen(255);
      this->setRed(255);
      this->setBlue(255);
#endif
      break;
   case neutral:
      dcClDeviceColor = neutral;
#ifdef IC_WIN
      this->setGreen(0);
      this->setRed(0);
      this->setBlue(0);
#endif
      break;
   case defaultColor:
      /************************************************************************/
      /* CLR_DEFAULT is not valid for GpiQueryRGBColor.  It is device         */
      /* dependent and is the default window text color (SYSCLR_WINDOWTEXT)   */
      /* for displays.                                                        */
      /************************************************************************/
      {
      IGUIColor guiColor(IGUIColor::kWindowText);
      this->setGreen(guiColor.greenMix());
      this->setRed(guiColor.redMix());
      this->setBlue(guiColor.blueMix());
      dcClDeviceColor = defaultColor;
      }
      break;
   default:
     ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      break;
   } /* endswitch */

#ifdef IC_PM
   if (dclr != defaultColor)
   {
      IPresSpaceHandle hps( IWindow::desktopWindow()->presSpace() );
      long lRGBColor = GpiQueryRGBColor(hps, 0,
                        (dclr==background) ? CLR_BACKGROUND : CLR_NEUTRAL);

      IWindow::desktopWindow()->releasePresSpace(hps);

      if (lRGBColor == GPI_ALTERROR)
         ITHROWGUIERROR2("GpiQueryRGBColor",
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

      RGB2* prgb2 = (RGB2*)&lRGBColor;
      this->setGreen(prgb2->bGreen);
      this->setRed(prgb2->bRed);
      this->setBlue(prgb2->bBlue);
   }
#endif
}

/*------------------------------------------------------------------------------
| IDeviceColor::~IDeviceColor                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDeviceColor::~IDeviceColor( )
{ }

/*------------------------------------------------------------------------------
| IGUIColor::IGUIColor                                                         |
|                                                                              |
| Construct an IGUIColor from a SysColor enumerator value.                     |
------------------------------------------------------------------------------*/
IGUIColor::IGUIColor(ESystemColor scValue)
  : IColor(scValue)
{ }

/*------------------------------------------------------------------------------
| IGUIColor::~IGUIColor                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGUIColor::~IGUIColor( )
{ }

/*------------------------------------------------------------------------------
| IGUIColor::setColor                                                          |
|                                                                              |
| Change the GUI component system color.                                       |
------------------------------------------------------------------------------*/
IGUIColor& IGUIColor::setColor(const IColor& newColor)
{
   ITRACE_MOTIF_NOP();
   //Entry structure to API.
#ifdef IC_WIN
   typedef struct {
      int      lIndex;
      COLORREF lColor;
   } Entry;
#else
   typedef struct {
      long lIndex;
      long lColor;
   } Entry;
#endif

   Entry alTable[1];
   alTable[0].lColor = newColor.asRGBLong();
   switch (this->systemColor())
   {
   case kShadowIconHiliteBgnd:
      alTable[0].lIndex = SYSCLR_SHADOWHILITEBGND;
      break;
   case kShadowIconHiliteFgnd:
      alTable[0].lIndex = SYSCLR_SHADOWHILITEFGND;
      break;
   case kShadowIconText:
      alTable[0].lIndex = SYSCLR_SHADOWTEXT;
      break;
   case kEntryFieldBgnd:
   case kListBoxBgnd:
      alTable[0].lIndex = SYSCLR_ENTRYFIELD;
      break;
   case kDisableMenuText:
      alTable[0].lIndex = SYSCLR_MENUDISABLEDTEXT;
      break;
   case kMenuHiliteText:
      alTable[0].lIndex = SYSCLR_MENUHILITE;
      break;
   case kMenuHiliteBgnd:
      alTable[0].lIndex = SYSCLR_MENUHILITEBGND;
      break;
   case kNotebookPageBgnd:
      alTable[0].lIndex = SYSCLR_PAGEBACKGROUND;
      break;
   case kInactiveScrollBar:
   case kDefaultControl:
      alTable[0].lIndex = SYSCLR_FIELDBACKGROUND;
      break;
   case kButtonLight:
      alTable[0].lIndex = SYSCLR_BUTTONLIGHT;
      break;
   case kButtonMiddle:
      alTable[0].lIndex = SYSCLR_BUTTONMIDDLE;
      break;
   case kButtonDark:
      alTable[0].lIndex = SYSCLR_BUTTONDARK;
      break;
   case kTitleLine:
      alTable[0].lIndex = SYSCLR_TITLEBOTTOM;
      break;
   case kMenuShadow:
   case kDialogShadow:
      alTable[0].lIndex = SYSCLR_SHADOW;
      break;
   case kIconText:
      alTable[0].lIndex = SYSCLR_ICONTEXT;
      break;
   case kDialogBgnd:
      alTable[0].lIndex = SYSCLR_DIALOGBACKGROUND;
      break;
   case kHiliteFgnd:
      alTable[0].lIndex = SYSCLR_HILITEFOREGROUND;
      break;
   case kHiliteBgnd:
      alTable[0].lIndex = SYSCLR_HILITEBACKGROUND;
      break;
   case kInactiveTitleTextBgnd:
      alTable[0].lIndex = SYSCLR_INACTIVETITLETEXTBGND;
      break;
   case kActiveTitleTextBgnd:
      alTable[0].lIndex = SYSCLR_ACTIVETITLETEXTBGND;
      break;
   case kInactiveTitleText:
      alTable[0].lIndex = SYSCLR_INACTIVETITLETEXT;
      break;
   case kActiveTitleText:
      alTable[0].lIndex = SYSCLR_ACTIVETITLETEXT;
      break;
   case kOutputText:
      alTable[0].lIndex = SYSCLR_OUTPUTTEXT;
      break;
   case kWindowStaticText:
      alTable[0].lIndex = SYSCLR_WINDOWSTATICTEXT;
      break;
   case kScrollBar:
      alTable[0].lIndex = SYSCLR_SCROLLBAR;
      break;
   case kDesktopBgnd:
      alTable[0].lIndex = SYSCLR_BACKGROUND;
      break;
   case kActiveTitleBgnd:
      alTable[0].lIndex = SYSCLR_ACTIVETITLE;
      break;
   case kInactiveTitleBgnd:
      alTable[0].lIndex = SYSCLR_INACTIVETITLE;
      break;
   case kMenuBgnd:
      alTable[0].lIndex = SYSCLR_MENU;
      break;
   case kWindowBgnd:
      alTable[0].lIndex = SYSCLR_WINDOW;
      break;
   case kFrameBorder:
      alTable[0].lIndex = SYSCLR_WINDOWFRAME;
      break;
   case kMenuText:
      alTable[0].lIndex = SYSCLR_MENUTEXT;
      break;
   case kWindowText:
      alTable[0].lIndex = SYSCLR_WINDOWTEXT;
      break;
   case kTitleText:
   case kSizeBar:
   case kScrollArrow:
      alTable[0].lIndex = SYSCLR_TITLETEXT;
      break;
   case kActiveFrameBorder:
      alTable[0].lIndex = SYSCLR_ACTIVEBORDER;
      break;
   case kInactiveFrameBorder:
      alTable[0].lIndex = SYSCLR_INACTIVEBORDER;
      break;
   case kMainWindowBgnd:
      alTable[0].lIndex = SYSCLR_APPWORKSPACE;
      break;
   case kHelpWindowBgnd:
      alTable[0].lIndex = SYSCLR_HELPBACKGROUND;
      break;
   case kHelpText:
      alTable[0].lIndex = SYSCLR_HELPTEXT;
      break;
   case kHelpHiliteText:
      alTable[0].lIndex = SYSCLR_HELPHILITE;
      break;
   case kDefaultButton:
#ifdef IC_WIN
      return *this;             // this area not supported
#else
      alTable[0].lIndex = SYSCLR_BUTTONDEFAULT;
      break;
#endif
   default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
      break;
   } /* endswitch */

   if (ISETSYSCOLORS(1, &alTable[0].lIndex, &alTable[0].lColor) == FALSE) {
      ITHROWGUIERROR("WinSetSysColors");
   } /* endif */

   return *this;
}
#endif


#ifdef IC_MOTIF
#define MAX_INTENSITY 255

/*------------------------------------------------------------------------------
| IDeviceColor::IDeviceColor                                                   |
|                                                                              |
| Construct an IDeviceColor from a DeviceColor enumerator value.               |
------------------------------------------------------------------------------*/
IDeviceColor::IDeviceColor(DeviceColor dclr)
   : IColor()

  {
  dcClDeviceColor = dclr;

  switch (dclr )
    {
    case background  :
      // Set background color to be white
      setRed( MAX_INTENSITY);
      setGreen( MAX_INTENSITY);
      setBlue( MAX_INTENSITY);
      break;
    case neutral :
      // the default of black is used for neutral
      break;
    case defaultColor :
      // the default of black is used for default
      break;
    }
  }

/*------------------------------------------------------------------------------
| IDeviceColor::~IDeviceColor                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDeviceColor::~IDeviceColor( )
{ }

/*------------------------------------------------------------------------------
| IGUIColor::IGUIColor                                                         |
|                                                                              |
| Construct an IGUIColor from a SystemColor enumerator value.                  |
------------------------------------------------------------------------------*/
IGUIColor::IGUIColor(ESystemColor scValue)
  : IColor(scValue)

  {
  scClSysColor = scValue;
  switch (scValue)
    {
    case shadowIconHiliteBgnd:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case shadowIconHiliteFgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case shadowIconText:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case entryFieldBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 204 );
      break;
      }
    case listBoxBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 204 );
      break;
      }
    case disableMenuText:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case menuHiliteText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case menuHiliteBgnd:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case notebookPageBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case inactiveScrollBar:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case defaultControl:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case buttonLight:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case buttonMiddle:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case buttonDark:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case defaultButton:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case titleLine:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case menuShadow:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case dialogShadow:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case iconText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case dialogBgnd:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case hiliteFgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case hiliteBgnd:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case inactiveTitleTextBgnd:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case activeTitleTextBgnd:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case inactiveTitleText:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case activeTitleText:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case outputText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case windowStaticText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 128 );
      break;
      }
    case scrollBar:
      {
      setRed( 192 );
      setGreen( 192 );
      setBlue( 192 );
      break;
      }
    case desktopBgnd:
      {
      setRed( 0 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case activeTitleBgnd:
      {
      setRed( 64 );
      setGreen( 128 );
      setBlue( 204 );
      break;
      }
    case inactiveTitleBgnd:
      {
      setRed( 204);
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case menuBgnd:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case windowBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case frameBorder:
      {
      setRed( 128 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    case menuText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case windowText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 0 );
      break;
      }
    case titleText:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case sizeBar:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case scrollArrow:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case activeFrameBorder:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 128 );
      break;
      }
    case inactiveFrameBorder:
      {
      setRed( 204 );
      setGreen( 204 );
      setBlue( 204 );
      break;
      }
    case mainWindowBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 224 );
      break;
      }
    case helpWindowBgnd:
      {
      setRed( 255 );
      setGreen( 255 );
      setBlue( 255 );
      break;
      }
    case helpText:
      {
      setRed( 0 );
      setGreen( 0 );
      setBlue( 128 );
      break;
      }
    case helpHiliteText:
      {
      setRed( 0 );
      setGreen( 128 );
      setBlue( 128 );
      break;
      }
    }
  }

/*------------------------------------------------------------------------------
| IGUIColor::~IGUIColor                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGUIColor::~IGUIColor( )
{ }

/*------------------------------------------------------------------------------
| IGUIColor::setColor                                                          |
|                                                                              |
| Change the GUI component system color.                                       |
| UNX_NOP_INVESTIGATE                                                          |
------------------------------------------------------------------------------*/
IGUIColor& IGUIColor::setColor(const IColor& newColor)
{
  return *this;
}
#endif
