// Revision: 04 1.6.1.2 source/ui/baseapp/icritsec.cpp, thread, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icritsec.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     ICritSec                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481624 )

extern "C" {
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include <signal.h>
    #include <errno.h>
  #endif
}

#include <icritsec.hpp>
#include <iexcept.hpp>
#include <iprimlck.hpp>

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICRITSEC_CPP_
  #include <ipagetun.h>
#endif

// Define static members.
unsigned long
  ICritSec :: fgCount = 0;

#ifdef IC_WIN

//  Deleting this causes race conditions at process shutdown.  We
//  let the critical section be cleaned up by the system.
static CRITICAL_SECTION
 *gCriticalSection = 0;

#endif

#ifdef IC_MOTIF
#pragma enum(4)
#pragma pack(push,4)

class ICritSecData {
/*******************************************************************************
*  Use ICritSecData to maintain any private data areas needed by        *
*  the ICritSec class.   An instance of this class is created when an          *
*  ICritSec object is created and deleted when the ICritSec is deleted.        *
*******************************************************************************/
public:
   sigset_t   oldSigMask;       // saved signal mask
};

#pragma pack(pop)
#pragma enum(pop)

#endif


/*------------------------------------------------------------------------------
| ICritSec::ICritSec                                                           |
|                                                                              |
| Issue DosEnterCritSec and increment count.                                   |
------------------------------------------------------------------------------*/
ICritSec::ICritSec ( )
   : fCritSecData( 0 )
{
#ifdef IC_WIN
  if ( !gCriticalSection )
  {
     IPrimalLock lock;
     if ( !gCriticalSection )
     {
         gCriticalSection= new CRITICAL_SECTION;
         InitializeCriticalSection(gCriticalSection);
     }
  }
  EnterCriticalSection(gCriticalSection);
#endif
#ifdef IC_PM
  DosEnterCritSec();
#endif
#ifdef IC_MOTIF
  fCritSecData = new ICritSecData;
  sigset_t   newMask;

  // Create a mask with all signals blocked.   Despite this the kernel
  // will not block SIGKILL and SIGSTOP.
  sigfillset( &newMask );
  // allow debugging signals
  sigdelset( &newMask, SIGTRAP );

  // Set the signal mask to the mask with signals blocked and save
  // previous setting in fCritSecData->oldSigMask;
  if ( 0 != sigprocmask( SIG_BLOCK, &newMask, &(fCritSecData->oldSigMask) ) )
     {
     ITHROWCLIBERROR("sigprocmask",
                     IBaseErrorInfo::accessError,
                     IException::recoverable);
     }
#endif
  ICritSec::fgCount++;
}

/*------------------------------------------------------------------------------
| ICritSec::~ICritSec                                                          |
|                                                                              |
| Issue DosExitCritSec and decrement count.                                    |
------------------------------------------------------------------------------*/
ICritSec::~ICritSec ( )
{
#ifdef IC_WIN
  LeaveCriticalSection(gCriticalSection);
#endif
#ifdef IC_PM
  DosExitCritSec();
#endif
#ifdef IC_MOTIF
  if (fCritSecData != NULL)
     {
     // Restore the signal mask to the value saved when this object
     // was constructed.
     if ( 0 != sigprocmask( SIG_SETMASK, &(fCritSecData->oldSigMask) , NULL ) )
        {
        ITHROWCLIBERROR("sigprocmask",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
        }
     delete fCritSecData;
     fCritSecData = NULL;
     }
#endif
  ICritSec::fgCount--;
}

/*------------------------------------------------------------------------------
| ICritSec::count                                                              |
|                                                                              |
| Retrieve the critical section inuse count                                    |
------------------------------------------------------------------------------*/
unsigned long ICritSec::count( )
{
  return fgCount;
}
