// Revision: 11 1.4.1.2 source/ui/baseapp/iedithdr.cpp, editctls, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: iedithdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iedithdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define IC_MULTIMEDIA
#define INCL_CIRCULARSLIDER
#define INCL_SW
#define INCL_WINDIALOGS
#define INCL_WINENTRYFIELDS
#define INCL_WINMLE
#define INCL_WINSTDSLIDER
#define INCL_WINSTDSPIN
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
  #ifdef IC_MOTIF
    #include <Xm/Text.h>
    #include <Xm/TextF.h>
  #endif
}

#ifdef IC_WIN
  #ifndef _ICLSLIDERW_
    #include <iclsldw.h>
  #endif
  #ifndef _ICLSPINBTNW_
    #include <iclspbw.h>
  #endif
  #ifndef _ICLCSLIDERW_
    #include <iclcsldw.h>
  #endif
#endif
#include <iedithdr.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>
#include <iwcname.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IEDITHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IEditHandler::IEditHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IEditHandler::IEditHandler ( )
  : IEditHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IEditHandler::~IEditHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IEditHandler::~IEditHandler ( )
{ }

/*------------------------------------------------------------------------------
| IEditHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool IEditHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_COMBOBOX )
        {                              // Combo box case.
           if ( subMsg == CBN_EFCHANGE )
           {
              bRc = this->edit( ctlevt );
           }
#ifdef IC_WIN
           else if (subMsg == IC_UM_CBN_EFCHANGE)
           {
              bRc = this->edit( ctlevt );
           }
#endif
        }
        else if ( className == WC_ENTRYFIELD )
        {                              // Entry field case.
           if ( subMsg == EN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
#ifndef IC_WIN
        // Don't process MLE separately, since on Windows it is the
        // same as the entry field, which was just processed above.
        else if ( className == WC_MLE )
        {                              // Multi-line edit case.
           if ( subMsg == MLN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
#endif
#ifdef IC_WIN
        else if ((className == WC_SPINBUTTON) ||
                 (className == WC_NATIVESPINBUTTON))
#else
        else if (className == WC_SPINBUTTON)
#endif
        {                              // Spin button case.
           if ( subMsg == SPBN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
#ifdef IC_WIN
        else if ( className == WC_SLIDER ||
                  className == WC_NATIVESLIDER )
#else
        else if ( className == WC_SLIDER )
#endif
        {                              // Slider case.
           if ( subMsg == SLN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_CHANGED )
           {
              bRc = this->edit( ctlevt );
           }
        }
#endif

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  }
#endif

#ifdef IC_MOTIF
   /***********************************************/
   /*The eventId is the callback reason in Motif. */
   /***********************************************/
   unsigned long reason  = event.eventId();
   IControlEvent ctlevt(event);

   /*********************************************************************/
   /*Value changed callback.  EntryField, MultiLineEdit, ComboBox (which*/
   /*is implemented as Entryfield + listbox),  Slider                   */
   /*********************************************************************/
   if ( reason == motifEvent(XmCR_VALUE_CHANGED) ||
        ( reason == WM_CONTROL &&
          event.parameter1().number2() == SLN_CHANGE ) )
      {
      bRc = edit( ctlevt );
      }

   if (bRc)
      {                           // processing done
      event.setResult(ctlevt.result());  // copy result
      }
#endif

  return bRc;
}
