// Revision: 23 1.8.1.6 source/ui/baseapp/ifilehdr.cpp, views, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ifilehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifilehdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
{
  #define INCL_WINMESSAGEMGR
  #define INCL_WINSTDFILE
  #define INCL_WINWINDOWMGR
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include <string.h>
    #include <Xm/FileSB.h>
    #include <Xm/List.h>
  #endif
}

#include <iexcept.hpp>
#include <ifildprv.hpp>
#include <ifiledlg.hpp>
#include <ifilehdr.hpp>
#include <istring.hpp>
#ifdef IC_PM
  #include <ipmexsta.hpp>
#endif
#ifdef IC_MOTIF
  #include <imstring.hpp>
#endif
#include <itrace.hpp>


// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IFILEHDR_CPP_
  #include <ipagetun.h>
#endif

//
// Implementation for IFileDialogHandler.
//

/*------------------------------------------------------------------------------
| IFileDialogHandler::IFileDialogHandler                                       |
------------------------------------------------------------------------------*/
IFileDialogHandler::IFileDialogHandler ( )
  : IFileDialogHandler::Inherited()
{
#if 0
#ifdef IC_WIN
  fRegisteredOkMsg = RegisterWindowMessage( FILEOKSTRING );
#endif
#endif // if 0
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::~IFileDialogHandler                                      |
------------------------------------------------------------------------------*/
IFileDialogHandler::~IFileDialogHandler ( )
{ }

/*------------------------------------------------------------------------------
| IFileDialogHandler::handleEventsFor                                          |
------------------------------------------------------------------------------*/
IFileDialogHandler& IFileDialogHandler::handleEventsFor ( IFileDialog* fileDialog )
{
  IASSERTPARM( fileDialog != 0 );
  Inherited::handleEventsFor( fileDialog );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::stopHandlingEventsFor                                    |
------------------------------------------------------------------------------*/
IFileDialogHandler& IFileDialogHandler::stopHandlingEventsFor
                                                   ( IFileDialog* fileDialog )
{
  IASSERTPARM( fileDialog != 0 );
  Inherited::stopHandlingEventsFor( fileDialog );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::handleEventsFor                                          |
------------------------------------------------------------------------------*/
IHandler& IFileDialogHandler::handleEventsFor( IWindow *window )
{
  IFileDialog *filedlg = dynamic_cast<IFileDialog *>( window );
  if ( filedlg )
  {
     return this->handleEventsFor( filedlg );
  }
  else
  {
     ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::stopHandlingEventsFor                                    |
------------------------------------------------------------------------------*/
IHandler& IFileDialogHandler::stopHandlingEventsFor( IWindow *window )
{
  IFileDialog *filedlg = dynamic_cast<IFileDialog *>( window );
  if ( filedlg )
  {
     return this->stopHandlingEventsFor( filedlg );
  }
  else
  {
     ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::dispatchHandlerEvent                                     |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool retcond = false;

  IFileDialogEvent fileEvt( event );
  fileEvt.setResult( true );

  unsigned long idOfEvent = event.eventId();
  switch ( idOfEvent )
  {
#ifdef IC_PM
    case FDM_FILTER:
      retcond = this->filter( fileEvt );
      break;
    case FDM_VALIDATE:
      retcond = this->validate( fileEvt );
      break;
    case WM_DESTROY:
    case IC_UM_DLG_APPLY_PRESSED:
    {
      /*****************************************************************/
      /* Get the "this" pointer for window object from the window word */
      /* which is set during dialog initialization                     */
      /*****************************************************************/
      FILEDLG* pfildData = (FILEDLG*)WinQueryWindowPtr(event.handle(),QWL_USER);
      IFileDialog* pfildlg = (IFileDialog*)pfildData->ulUser;
      /*****************************************************************/
      /* Verify we have a non-null object to reference before using    */
      /*****************************************************************/
      if (pfildlg != (IFileDialog*)0)
      {
        /***************************************************************/
        /* If modeless, call handler function, dependent on event type */
        /***************************************************************/
        if (pfildlg->isModeless())
        {
          /*************************************************************/
          /* Even if dialog cancelled, still call results on destroy   */
          /*************************************************************/
          if (idOfEvent == WM_DESTROY)
            modelessResults( pfildlg );

          /*************************************************************/
          /* otherwise this is apply button, so notify of file to use  */
          /*************************************************************/
          else if (idOfEvent == IC_UM_DLG_APPLY_PRESSED)
            modelessApply( pfildlg, (const char*)event.parameter1() );
        }
      }
      break;
    }
#endif
#ifdef IC_WIN
    case IC_UM_DLG_APPLY_PRESSED:
    {
      IFileDialog* pfildlg =
         dynamic_cast<IFileDialog*>(event.controlWindow());
      // Verify we have a non-null object to reference before using
      if (pfildlg != (IFileDialog*)0)
      {
        // If modeless, call handler function.
        if (pfildlg->isModeless())
        {
          // Call modelessResults when the dialog is being destroyed.
          // The parameter2 value of the event is non 0 in this case.
          // Otherwise, this is apply button, so notify of file to use
          if (event.parameter2().asUnsignedLong() != 0)
             modelessResults( pfildlg );
          else
             modelessApply( pfildlg, (const char*)event.parameter1() );
        }
      }
      break;
    }
    case WM_NOTIFY:
    {
        OFNOTIFY* ofn = (OFNOTIFY*)event.parameter2().asUnsignedLong();
        if ( ofn->hdr.code == CDN_FILEOK)
        {
           IFileDialog* pfildlg =
              dynamic_cast<IFileDialog*>(event.controlWindow());
           if (pfildlg)
           {
              // Get the filename and add it to the private data of the event
              // so that queries to the events fileName() function can be
              // resolved
              if (pfildlg->fFileDialogData->fStyle & IFileDialog::multiSelection)
              {
                 //Call validate on each file selected.
                 int  selectedCount( pfildlg->selectedFileCount() );
                 bool fileValid    ( true );
                 for (int i=0; fileValid && (i < selectedCount); i++)
                 {
                    fileEvt.fFileNameSelected = pfildlg->fileName(i);
                    retcond = this->validate( fileEvt );
                    fileValid = fileEvt.result().asUnsignedLong() ? true : false;
                 }
              }
              else
              {
                 fileEvt.fFileNameSelected = pfildlg->fileName();
                 retcond = this->validate( fileEvt );
              }

              // Set pending result to cancel if file was rejected by validate().
              if ( !fileEvt.result().asUnsignedLong() )
                 pfildlg->fFileDialogData->fReturnVal = DID_CANCEL;
           }
        }
        break;
    }
#endif

#ifdef IC_MOTIF
    case FDM_FILTER:
      // This event comes from the search procedure in ifiledlg.
      retcond = filter(fileEvt);
      event.setResult( fileEvt.result() );
      break;
    case motifEvent(XmCR_OK):
    // This is when ok or double click
    {
       IFileDialog* pfildlg =
          dynamic_cast<IFileDialog*>(fileEvt.controlWindow()) ;

       //Call validate on each file selected.
       int  selectedCount( pfildlg->selectedFileCount() );
       bool fileValid    ( true );
       for (int i=0; fileValid && (i < selectedCount); i++)
       {
          IString fileNameSelected = pfildlg->fileName(i);
          IEvent tmpEvent(pfildlg,  FDM_VALIDATE, (char*)fileNameSelected, 0);
          IFileDialogEvent validateEvent( tmpEvent );
          retcond = this->validate( validateEvent );
          fileValid = validateEvent.result().asUnsignedLong() ? true : false;
          fileEvt.setResult( fileValid );
       }

       event.setResult( fileEvt.result() );

       // If validate() did not veto, process the OK.
       if ( (bool)fileEvt.result() )
       {
         pfildlg->setResult(DID_OK);
         if ( !( pfildlg->fFileDialogData->fStyle & IFileDialog::applyButton) )
         {
           // generate a command event
           pfildlg->postEvent(IWindow::command,
                          IEventData( DID_OK_PB ),
                          IEventData(ICommandEvent::pushButton, 0) );
           if ( pfildlg->isModeless())
           {
             pfildlg->close();
           }
           else
           {
             pfildlg->dismiss(DID_OK);
           }
         }
         else
         {
           // Call modelessApply function.
           modelessApply( pfildlg,  pfildlg->fileName(0) );

           // generate a command event
           pfildlg->postEvent(IWindow::command,
                          IEventData( DID_APPLY_PB ),
                          IEventData(ICommandEvent::pushButton, 0) );
         }
       }
       // Event is processed...insure no more handlers are called.
       retcond = true;
       break;
    }
    case WM_COMMAND:
       if (event.parameter1().asUnsignedLong() != IC_ID_CANCEL)
       {
          break;
       }
       // If parm1 is IC_ID_CANCEL (generated from the escape key), then
       // intentionally fall through to the cancel case below.
    case motifEvent(XmCR_CANCEL):
    // This is when the cancel button is pressed.
    {
      IFileDialog *pfildlg = (IFileDialog*)fileEvt.controlWindow() ;
      pfildlg->setResult(DID_CANCEL);
      // generate a command event
      pfildlg->postEvent(IWindow::command,
                     IEventData( DID_CANCEL_PB ),
                     IEventData(ICommandEvent::pushButton, 0) );
      if ( pfildlg->isModeless())
      {
        pfildlg->close();
      }
      else
      {
        pfildlg->dismiss(DID_CANCEL);
      }
      // Event is processed...insure no more handlers are called.
      retcond = true;
      break;
    }
    case WM_DESTROY:
    // This is when the system menu is double-clicked, or Alt+F4 is pressed,
    // or Close is selected from the system menu.
    // Also handles case where they clicked OK or Cancel and we did a close
    // or a dismiss.
    {
      IFileDialog *pfildlg = (IFileDialog*)fileEvt.controlWindow() ;
      if (pfildlg != (IFileDialog*)0) {
         unsigned long pfdResult = pfildlg->result();
         if ((pfdResult != DID_OK) && (pfdResult != DID_CANCEL))
            pfildlg->setResult(DID_CANCEL);
         if (pfildlg->isModeless())
            modelessResults( pfildlg );
      }
      break;
    }
#endif //IC_MOTIF
    default:
      break;
  } /* endswitch */

#ifdef IC_PM
  // filter() and validate() will call setResult on fileEvt to indicate if
  // the file is OK to process.  We have to pass this on to the original event.
  event.setResult( fileEvt.result() );
#endif

#ifdef IC_WIN
#if 0
  if ( idOfEvent == fRegisteredOkMsg )
  {
    IFileDialog* pfildlg =
       dynamic_cast<IFileDialog*>(event.controlWindow());
    // Get the filename and add it to the private data of the event
    //  so that queries to the events fileName() function can be resolved
    if (pfildlg->fFileDialogData->fStyle & IFileDialog::multiSelection)
    {
       //Call validate on each file selected.
       int  selectedCount( pfildlg->selectedFileCount() );
       bool fileValid    ( true );
       for (int i=0; fileValid && (i < selectedCount); i++)
       {
          fileEvt.fFileNameSelected = pfildlg->fileName(i);
          retcond = this->validate( fileEvt );
          fileValid = fileEvt.result().asUnsignedLong() ? true : false;
       }
    }
    else
    {
       fileEvt.fFileNameSelected = pfildlg->fileName();
       retcond = this->validate( fileEvt );
    }
  }
#endif //if 0
  // Result is defaulted to true, therefore only on a validate message
  // will it change.  Therefore default result to false and return false
  // on Windows.  If validate returns false, then continue dialog so set
  // result to true and return true.
  event.setResult( !fileEvt.result().asUnsignedLong() );

  if ( event.result().asUnsignedLong() )
#endif

#ifdef IC_MOTIFPM
  if ( !fileEvt.result().asUnsignedLong() )
#endif
  {
     // If validate() or filter() setResult to false, the file should not be
     // processed, and therefore no further handlers should be called.
     return true;
  }
  else
  {
     return retcond;
  }
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::filter                                                   |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::filter ( IFileDialogEvent& event )
{
  ITRACE_WIN_NOP();

  event.setResult( this->filterName( event.fileName(),
                                     (IFileDialog*)event.window() ));
  return false;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::filterName                                               |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::filterName ( const IString& fileName,
                                      IFileDialog* )
{
  ITRACE_WIN_NOP();

  return true;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::validate                                                 |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::validate ( IFileDialogEvent& event )
{
  event.setResult( this->validateName( event.fileName(),
                                       (IFileDialog*)event.window() ));
  return false;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::validateName                                             |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::validateName ( const IString& fileName,
                                        IFileDialog* )
{
  return true;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::modelessResults                                          |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::modelessResults ( IFileDialog* )
{
  return true;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::modelessApply                                            |
------------------------------------------------------------------------------*/
bool IFileDialogHandler::modelessApply ( IFileDialog*,
                                         const IString& )
{
  return true;
}


//
// Implementation for IFileDialogEvent.
//

/*------------------------------------------------------------------------------
| IFileDialogEvent::~IFileDialogEvent                                          |
------------------------------------------------------------------------------*/
IFileDialogEvent::~IFileDialogEvent ( )
{ }

/*------------------------------------------------------------------------------
| IFileDialogEvent::IFileDialogEvent                                           |
------------------------------------------------------------------------------*/
IFileDialogEvent::IFileDialogEvent ( IEvent& event )
  : IEvent( event )
{ }

/*------------------------------------------------------------------------------
| IFileDialogEvent::fileName                                                   |
------------------------------------------------------------------------------*/
IString IFileDialogEvent::fileName ( ) const
{
#ifdef IC_MOTIFPM
  char* temp = this->parameter1();
  return temp;
#endif
#ifdef IC_WIN
  return this->fFileNameSelected;
#endif
}

/*------------------------------------------------------------------------------
| IFileDialogEvent::fileLength                                                 |
------------------------------------------------------------------------------*/
unsigned long IFileDialogEvent::fileLength ( ) const
{
  return fileName().length();
}
