// Revision: 57 1.3.1.2 source/ui/baseapp/ihandler.cpp, basewin, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: ihandler.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihandler.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iwindefs.h>
#include <ihandler.hpp>

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHANDLER_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IHandler::IHandler                                                           |
|                                                                              |
| Constructor for the IHandler astabled state.                                 |
------------------------------------------------------------------------------*/
IHandler :: IHandler ( )
  : active( true ),
    numWindows( 0 )
  {
  }

/*------------------------------------------------------------------------------
| IHandler::~IHandler                                                          |
------------------------------------------------------------------------------*/
IHandler :: ~IHandler ( )

{
  active = 0xFFFFFFF;
  if ( this->numWindows )
    IWindow::cleanUpHandler( *this );
}


/*------------------------------------------------------------------------------
| IHandler::enable                                                             |
------------------------------------------------------------------------------*/
IHandler &IHandler :: enable ( bool setting )
  {
  active = setting;
  return *this;
  }

/*------------------------------------------------------------------------------
| IHandler::disable                                                            |
------------------------------------------------------------------------------*/
IHandler &IHandler :: disable ( )
  {
  return this->enable( false );
  }

/*------------------------------------------------------------------------------
| IHandler::handleEventsFor                                                    |
|                                                                              |
| Attach the handler to the specified window.                                  |
------------------------------------------------------------------------------*/
IHandler &IHandler :: handleEventsFor( IWindow *window )
  {
  IASSERTPARM(window);
  window -> addHandler( this );
  return *this;
  }

/*------------------------------------------------------------------------------
| IHandler::stopHandlingEventsFor                                              |
|                                                                              |
| Detach the handler from the specified window.                                |
------------------------------------------------------------------------------*/
IHandler &IHandler :: stopHandlingEventsFor( IWindow *window )
  {
  IASSERTPARM(window);
  window -> removeHandler( this );
  return *this;
  }

/*------------------------------------------------------------------------------
| IHandler::defaultProcedure                                                   |
|                                                                              |
| Send the specified event object to the original window procedure for the     |
| underlying PM window.                                                        |
------------------------------------------------------------------------------*/
IEventResult IHandler :: defaultProcedure ( IEvent &event )
  {
  event.window() -> defaultProcedure( event );
  return event.result();
  }

/*------------------------------------------------------------------------------
| IHandler::asString                                                           |
|                                                                              |
| Return IHandler(enabled) or IHandler(disabled).                              |
------------------------------------------------------------------------------*/
IString IHandler::asString() const
  {
  IString
    strStatus = ( this->active ) ? "enabled)": "disabled)";
  return "IHandler(" + strStatus;
  }

/*------------------------------------------------------------------------------
| IHandler::asDebugInfo                                                        |
|                                                                              |
| Return diagnostic information about the handler.                             |
------------------------------------------------------------------------------*/
IString IHandler :: asDebugInfo ( ) const
  {
  IString
    result( "IHandler(" );
  result += ",active=";
  result += ( this->active ) ? "true" : "false";
  result += ",windows=";
  result += IString( this->numWindows );
  result += ")";
  return result;
  }
