// Revision: 67 1.7.1.2 source/ui/baseapp/ihelphdr.cpp, help, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: ihelphdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IHelpHandler - This class deals with help events                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINHELP
extern "C" {
  #include <iwindefs.h>
  #ifdef IC_MOTIFWIN
     #include <ipfx.h>
  #endif
}

#include <ihelphdr.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <iframe.hpp>
#include <imsgbox.hpp>
#include <ireslib.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IHELPHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IHelpHandler::IHelpHandler                                                   |
------------------------------------------------------------------------------*/
IHelpHandler::IHelpHandler ( )
{ }

/*------------------------------------------------------------------------------
| IHelpHandler::~IHelpHandler                                                  |
------------------------------------------------------------------------------*/
IHelpHandler::~IHelpHandler ( )
{ }


/*------------------------------------------------------------------------------
| IHelpHandler::dispatchHandlerEvent                                           |
------------------------------------------------------------------------------*/
bool IHelpHandler::dispatchHandlerEvent ( IEvent& event )
/***************************************************************/
/* After the event is dispatched to handlers in the list, this */
/* function is dispatched. Window subclass may override this   */
/* function to handle the event.                               */
/***************************************************************/
{
  bool stopProcessingEvent = false;
  switch ( event.eventId() )
  {
    case HM_ACTIONBAR_COMMAND:
    {
      IHelpMenuBarEvent menuBarEvent( event );
      // Don't need to copy the event result (reserved 0 value).
      stopProcessingEvent = this->menuBarCommand( menuBarEvent );
      break;
    }

    case HM_ERROR:
    {
      IHelpErrorEvent errorEvent( event );
      // Don't need to copy the event result (reserved 0 value).
      stopProcessingEvent = this->handleError( errorEvent );
      break;
    }

    case HM_HELPSUBITEM_NOT_FOUND:
    {
      IHelpSubitemNotFoundEvent hsnfEvent( event );
      stopProcessingEvent = subitemNotFound( hsnfEvent );
      event.setResult( hsnfEvent.result() );
      break;
    }

    case HM_GENERAL_HELP_UNDEFINED:
    {
      stopProcessingEvent = this->helpUndefined( event );
      break;
    }

    case HM_INFORM:
    {
      IHelpHyperlinkEvent hyperlinkEvent( event );
      // Don't need to copy the event result (reserved 0 value).
      stopProcessingEvent = this->hyperlinkSelect( hyperlinkEvent );
      break;
    }

    case HM_NOTIFY:
    {
      IHelpNotifyEvent notifyEvent( event );

#ifdef IC_MOTIFPM
       switch ( event.parameter1().number1() )
#endif
#ifdef IC_WIN
       switch ( event.parameter2().number1() )
#endif
        {
          case CONTROL_SELECTED:
             stopProcessingEvent = this->controlSelect( notifyEvent );
             break;

#ifdef IC_PM   // the following message not supported in IPF/Win
          case OPEN_COVERPAGE:
             stopProcessingEvent = this->showCoverPage( notifyEvent );
             break;

          case OPEN_PAGE:
             stopProcessingEvent = this->showPage( notifyEvent );
             break;

          case SWAP_PAGE:
             stopProcessingEvent = this->swapPage( notifyEvent );
             break;

          case OPEN_INDEX:
             stopProcessingEvent = this->showIndex( notifyEvent );
             break;

          case OPEN_TOC:
             stopProcessingEvent = this->showContents( notifyEvent );
             break;

          case OPEN_HISTORY:
             stopProcessingEvent = this->showHistory( notifyEvent );
             break;

          case OPEN_LIBRARY:
             stopProcessingEvent = this->openLibrary( notifyEvent );
             break;

          case OPEN_SEARCH_HIT_LIST:
             stopProcessingEvent = this->showSearchList( notifyEvent );
             break;
#endif  // IC_PM

          default:
             break;
          } /* endswitch */
        event.setResult( notifyEvent.result() );
        break;
      } /* endcase HM_NOTIFY */

    case HM_QUERY_KEYS_HELP:
      stopProcessingEvent = this->keysHelpId( event );
      break;

#ifdef IC_PM
    case HM_TUTORIAL:
    {
      IHelpTutorialEvent tutorialEvent( event );
      // Don't need to copy the event result (reserved 0 value).
      stopProcessingEvent = this->showTutorial( tutorialEvent );
      break;
    }
#endif

    default:
      break;
  } /* endswitch */

  return stopProcessingEvent;
}


/*------------------------------------------------------------------------------
| IHelpHandler::menuBarCommand                                                 |
------------------------------------------------------------------------------*/
bool IHelpHandler::menuBarCommand ( IHelpMenuBarEvent& menuBarEvent )
{
  return false;
}


/*------------------------------------------------------------------------------
| IHelpHandler::handleError                                                    |
------------------------------------------------------------------------------*/
bool IHelpHandler::handleError ( IHelpErrorEvent& errorEvent )
{
  unsigned long ulMessage = 0;
  switch ( errorEvent.error() )
  {
    case IHelpErrorEvent::loadingDLL:
       ulMessage = IC_HMERR_LOAD_DLL;
       break;
    case IHelpErrorEvent::noWindow:
       ulMessage = IC_HMERR_NO_FRAME_WND_IN_CHAIN;
       break;
    case IHelpErrorEvent::invalidAppHandle:
       ulMessage = IC_HMERR_INVALID_ASSOC_APP_WND;
       break;
    case IHelpErrorEvent::invalidInstanceHandle:
       ulMessage = IC_HMERR_INVALID_ASSOC_HELP_INST;
       break;
    case IHelpErrorEvent::invalidQueryHandle:
       ulMessage = IC_HMERR_INVALID_QUERY_APP_WND;
       break;
    case IHelpErrorEvent::noHelpTable:
       ulMessage = IC_HMERR_HELPTABLE_UNDEFINE;
       break;
    case IHelpErrorEvent::noHelpInstance:
       ulMessage = IC_HMERR_HELP_INSTANCE_UNDEFINE;
       break;
    case IHelpErrorEvent::invalidId:
       ulMessage = IC_HMERR_HELPITEM_NOT_FOUND;
       break;
    case IHelpErrorEvent::noIndex:
       ulMessage = IC_HMERR_INDEX_NOT_FOUND;
       break;
    case IHelpErrorEvent::noContent:
       ulMessage = IC_HMERR_CONTENT_NOT_FOUND;
       break;
    case IHelpErrorEvent::openHelpFile:
       ulMessage = IC_HMERR_OPEN_LIB_FILE;
       break;
    case IHelpErrorEvent::readHelpFile:
       ulMessage = IC_HMERR_READ_LIB_FILE;
       break;
    case IHelpErrorEvent::closeHelpFile:
       ulMessage = IC_HMERR_CLOSE_LIB_FILE;
       break;
    case IHelpErrorEvent::invalidHelpFile:
       ulMessage = IC_HMERR_INVALID_LIB_FILE;
       break;
    case IHelpErrorEvent::notEnoughMemory:
       ulMessage = IC_HMERR_NO_MEMORY;
       break;
    case IHelpErrorEvent::unableFreeMemory:
       ulMessage = IC_HMERR_FREE_MEMORY;
       break;
    case IHelpErrorEvent::invalidHelpWindow:
       ulMessage = IC_HMERR_PANEL_NOT_FOUND;
       break;
    case IHelpErrorEvent::unopenDatabase:
       ulMessage = IC_HMERR_DATABASE_NOT_OPEN;
       break;
    case IHelpErrorEvent::fontNotFound:
       ulMessage = IC_HMERR_FONT_ERROR;
       break;
    case IHelpErrorEvent::unknown:
    default:
       ulMessage = IC_HMERR_UNKNOWN;
       break;
  } /* endswitch */

  IMessageBox msgbox( errorEvent.window() );
  msgbox.setTitle( IMessageText( IC_HELPERRORMSGBOX_TITLE ) );
  msgbox.show( IMessageText( ulMessage ),
               IMessageBox::information );

  return true;
}


/*------------------------------------------------------------------------------
| IHelpHandler::helpUndefined                                                  |
------------------------------------------------------------------------------*/
bool IHelpHandler::helpUndefined ( IEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::hyperlinkSelect                                                |
------------------------------------------------------------------------------*/
bool IHelpHandler::hyperlinkSelect ( IHelpHyperlinkEvent& hyperlinkEvent )
{
#if (IC_OBSOLETE <= IC_OBSOLETE_1)
  return this->hypertextSelect( hyperlinkEvent );
#else
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IHelpHandler::subitemNotFound                                                |
------------------------------------------------------------------------------*/
bool IHelpHandler::subitemNotFound ( IHelpSubitemNotFoundEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showTutorial                                                   |
------------------------------------------------------------------------------*/
bool IHelpHandler::showTutorial ( IHelpTutorialEvent& tutorialEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::keysHelpId                                                     |
------------------------------------------------------------------------------*/
bool IHelpHandler::keysHelpId ( IEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showContents                                                   |
------------------------------------------------------------------------------*/
bool IHelpHandler::showContents ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showCoverPage                                                  |
------------------------------------------------------------------------------*/
bool IHelpHandler::showCoverPage ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showPage                                                       |
------------------------------------------------------------------------------*/
bool IHelpHandler::showPage ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::swapPage                                                       |
------------------------------------------------------------------------------*/
bool IHelpHandler::swapPage ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showIndex                                                      |
------------------------------------------------------------------------------*/
bool IHelpHandler::showIndex ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showHistory                                                    |
------------------------------------------------------------------------------*/
bool IHelpHandler::showHistory ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showSearchList                                                 |
------------------------------------------------------------------------------*/
bool IHelpHandler::showSearchList ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::openLibrary                                                    |
------------------------------------------------------------------------------*/
bool IHelpHandler::openLibrary ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::controlSelect                                                  |
------------------------------------------------------------------------------*/
bool IHelpHandler::controlSelect ( IHelpNotifyEvent& notifyEvent )
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::handleEventsFor                                                |
------------------------------------------------------------------------------*/
IHandler& IHelpHandler::handleEventsFor ( IWindow *window )
{
  IFrameWindow *frame = dynamic_cast<IFrameWindow *>( window );
  if ( frame )
     return this->handleEventsFor( frame );
  else
     ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::handleEventsFor                                                |
------------------------------------------------------------------------------*/
IHelpHandler& IHelpHandler::handleEventsFor ( IFrameWindow *applicationWindow )
{
  IASSERTPARM( applicationWindow != 0 );
  IHandler::handleEventsFor( applicationWindow );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::stopHandlingEventsFor                                          |
------------------------------------------------------------------------------*/
IHandler& IHelpHandler::stopHandlingEventsFor ( IWindow *window )
{
  IFrameWindow *frame = dynamic_cast<IFrameWindow *>( window );
  if ( frame )
     return this->stopHandlingEventsFor( frame );
  else
     ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::stopHandlingEventsFor                                          |
------------------------------------------------------------------------------*/
IHelpHandler& IHelpHandler::stopHandlingEventsFor ( IFrameWindow
                                                      *applicationWindow )
{
  IASSERTPARM( applicationWindow != 0 );
  IHandler::stopHandlingEventsFor( applicationWindow );
  return *this;
}
