// Revision: 69 1.5.1.3 source/ui/baseapp/ihelpprv.hpp, help, ioc.v400, 000302 
/* NOSHIP */
#ifndef _IHELPPRV_
#define _IHELPPRV_
/*******************************************************************************
* FILE NAME: ihelpprv.hpp.                                                     *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IHelpWindow                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandle.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <ihandler.hpp>
extern "C" {
  #ifdef IC_MOTIFWIN
    #include <ipfx.h>
  #endif
}

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IFrameWindow;
class IHelpWindow;
class IHelpWindowData;
class IHelpTable;

#pragma enum(4)
#pragma pack(push,4)

#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| class : IWinHelpHandler                                                      |
|  This class handles help requests initiated by the user, such as pressing    |
|  the F1 key.  An instance of this handler is created for each IHelpWindow    |
|  object, and is attached to each IFrameWindow which is associated to the     |
|  IHelpWindow object.                                                         |
|  The dispatchHandlerEvents function looks for help requests for the          |
|  frame window and any of its children.  When it detects one, it attempts     |
|  to locate the associated help panel id via the IWindow::helpId or           |
|  IMenu::itemHelpId functions.  If the help panel id cannot be found in       |
|  this way, the help table, if any, associated with the IHelpWindow is        |
|  examined.  If the help panel id is still not found a message                |
|  is sent to the frame window to allow its help handler to process the        |
|  request.                                                                    |
|  If the help handler did not process the request, the help engine is called  |
|  to display either the panel id located or general help.                     |
------------------------------------------------------------------------------*/
class IWinHelpHandler : public IHandler
{
  typedef IHandler
    Inherited;
public:
    IWinHelpHandler(IHelpWindow& helpWindow, IHelpWindowData& helpWindowData);
  virtual
   ~IWinHelpHandler();

protected:
  virtual bool
    dispatchHandlerEvent( IEvent&     event ),
    winHelpRequested    ( IEvent&     event);

private:
  IHelpWindow
    &fHelpWindow;
  IHelpWindowData
    &fHlpWndData;

  IWinHelpHandler( const IWinHelpHandler& );
  IWinHelpHandler& operator=( const IWinHelpHandler& );
};
#endif

/*------------------------------------------------------------------------------
| IHelpWindowData class                                                        |
|                                                                              |
| Private data class for IHelpWindow.                                          |
------------------------------------------------------------------------------*/
class IHelpWindowData {
public:
  IHelpWindowData( IHelpWindow& helpwindow );

  ~IHelpWindowData( );

IWindowHandle
  helpHandle;                   // handle of help instance
IResourceLibrary
 *pHTResLib,                    // help table resource library
 *pMAResLib;
IString
 helpLibraries;                 // list of help files

bool
 isUsingIPF,
 newHTResLib;

#ifdef IC_MOTIFWIN
IFrameWindow*
   activeFrameWindow();         // determine active frame window

IWindowHandle
   lastActiveHandle;
IWinHelpHandler
   winHelpHandler;
IHelpTable
  *pHlpTable;
IWindowHandle
  ownerForIPF;
unsigned long
  usingHelpId;
#endif

private:
  IHelpWindowData( const IHelpWindowData& );
  IHelpWindowData& operator=( const IHelpWindowData& );

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
