// Revision: 34 1.1.1.3 source/ui/baseapp/iinhratt.cpp, basewin, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iinhratt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     This file contains the implementation of classes/functions for           *
*     inherited color attribute support :                                      *
*     IInheritColorAttribute                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iinhratt.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <iattribt.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IINHRATT_CPP_
    #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IInheritColorAttribute::IInheritColorAttribute                               |
|                                                                              |
| Constructor to create an alignment attribute from enum value                 |
------------------------------------------------------------------------------*/
IInheritColorAttribute::IInheritColorAttribute ( EInheritColor inheritColor )
   : IInheritColorAttribute::Inherited( )
{
  this->fInheritColor = inheritColor;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::IInheritColorAttribute                               |
|                                                                              |
| Constructor to create an alignment attribute from another alignment attribute|
------------------------------------------------------------------------------*/
IInheritColorAttribute::IInheritColorAttribute ( 
                                           const IInheritColorAttribute& aAttr )
   : IInheritColorAttribute::Inherited( )
{
  this->fInheritColor = aAttr.fInheritColor;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::~IInheritColorAttribute                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IInheritColorAttribute::~IInheritColorAttribute( )
{ }

/*------------------------------------------------------------------------------
| IInheritColorAttribute::operator=                                            |
|                                                                              |
| Class assignment                                                             |
| NOTES: Required by base IAttribute class                                     |
------------------------------------------------------------------------------*/
IInheritColorAttribute& IInheritColorAttribute::operator= (
                                           const IInheritColorAttribute& aAttr )
{
  this->fInheritColor = aAttr.fInheritColor;
  return *this;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::operator==                                           |
|                                                                              |
| Class comparison                                                             |
| NOTES: Required by base IAttribute class                                     |
------------------------------------------------------------------------------*/
bool IInheritColorAttribute::operator== (const IAttribute& aAttr) const
{
  const IInheritColorAttribute& a =
    dynamic_cast<const IInheritColorAttribute&>(aAttr);
  if ( this->fInheritColor == a.fInheritColor )
      return true;
  return false;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::inheritColor                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IInheritColorAttribute::EInheritColor 
                                   IInheritColorAttribute::inheritColor( ) const
{
  return this->fInheritColor;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::setInheritColor                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IInheritColorAttribute& IInheritColorAttribute::setInheritColor( 
                                                    EInheritColor inheritColor )
{
  this->fInheritColor = inheritColor;
  return *this;
}

/*------------------------------------------------------------------------------
| IInheritColorAttribute::clone                                                |
|                                                                              |
| In lieu of Copy constructor                                                  |
| NOTES: Required by IAttribute                                                |
------------------------------------------------------------------------------*/
IAttribute* IInheritColorAttribute::clone( ) const
{
  IAttribute* attr = new IInheritColorAttribute( this->fInheritColor );
  return attr;
}
