// Revision: 82 1.4.1.2 source/ui/baseapp/ikeyhdr.cpp, userinput, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ikeyhdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ikeyhdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
#ifdef IC_MOTIFPM
*   Licensed Materials - Property of IBM                                       *
#ifdef IC_MOTIF
*   (C) Copyright IBM Corporation 1992, 1997                                   *
#endif
#ifdef IC_PM
*   (C) Copyright IBM Corporation 1992, 1997                                   *
#endif
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
#endif
#ifdef IC_WIN
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
#endif
*                                                                              *
*******************************************************************************/
#ifdef IC_MOTIF
#include <iwindefs.h>
extern "C"
{
  #include <X11/Xlib.h>
}
#else // ! IC_MOTIF
extern "C"
{
  #define INCL_WININPUT
#ifdef IC_WIN
  #include <iwindefs.h>
  #include <limits.h>
#endif
#ifdef IC_PM
  #include <os2.h>
#endif
}
#endif // ! IC_MOTIF

#include <ikeyhdr.hpp>
#ifdef IC_WIN
#include <imenu.hpp>
#include <imnitem.hpp>
#include <istring.hpp>
#include <iwindow.hpp>
#endif //IC_WIN

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IKEYHDR_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
#define IC_DIACRITIC_FOUND     0x0001
#define IC_COMPOSITE_IS_NEXT   0x0002
#endif

/*------------------------------------------------------------------------------
| IKeyboardHandler::IKeyboardHandler                                           |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IKeyboardHandler::IKeyboardHandler ( )
  : IKeyboardHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| IKeyboardHandler::~IKeyboardHandler                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IKeyboardHandler::~IKeyboardHandler ( )
{ }

/*------------------------------------------------------------------------------
| IKeyboardHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Distribute key events to the appropriate virtual functions.                  |
------------------------------------------------------------------------------*/
#ifdef IC_MOTIFPM
bool IKeyboardHandler::dispatchHandlerEvent ( IEvent & event )
{
  bool bRc = false;

#ifdef IC_MOTIF
  if ( event.eventId() == xEvent( KeyPress )  ||
       event.eventId() == xEvent( KeyRelease ) )
#else
  if ( event.eventId() == WM_CHAR )
#endif
  {                               // Key event found
     IKeyboardEvent keyEvent( event );
     if ( keyEvent.isUpTransition() == false )
     {                            // Key press
        IKeyboardEvent::VirtualKey vkKey;
        if ( keyEvent.isScanCode()  &&
             this->scanCodeKeyPress( keyEvent ) )
        {                         // Have a scan code
           bRc = true;            // No additional processing needed.
        }
        else if ( keyEvent.isVirtual()  &&
                  ( keyEvent.isCharacter() == false  ||
                    ( ( vkKey = keyEvent.virtualKey() ) != IKeyboardEvent::insert  &&
                      vkKey != IKeyboardEvent::deleteKey  &&
                      vkKey != IKeyboardEvent::home  &&
                      vkKey != IKeyboardEvent::end  &&
                      vkKey != IKeyboardEvent::pageUp  &&
                      vkKey != IKeyboardEvent::pageDown  &&
                      vkKey != IKeyboardEvent::left  &&
                      vkKey != IKeyboardEvent::right  &&
                      vkKey != IKeyboardEvent::up  &&
                      vkKey != IKeyboardEvent::down ) )  &&
                  this->virtualKeyPress( keyEvent ) )
        {       // (Exclude NumLock + numeric keypad from virtual keys.)
           bRc = true;            // No additional processing needed.
        }
        else if ( keyEvent.isCharacter()  &&
                  ( keyEvent.isVirtual() == false  ||
                    ( ( vkKey = keyEvent.virtualKey() ) != IKeyboardEvent::tab  &&
                      vkKey != IKeyboardEvent::enter  &&
                      vkKey != IKeyboardEvent::newLine  &&
                      vkKey != IKeyboardEvent::backSpace  &&
                      vkKey != IKeyboardEvent::esc ) )  &&
                  this->characterKeyPress( keyEvent ) )
        {     // (Exclude tab, new line, backspace, esc from char keys.)
           bRc = true;            // No additional processing needed.
        }
     }
     if ( bRc == false  &&  this->key( keyEvent ) )
     {                            // Catch-all virtual function
        bRc = true;               // No additional processing needed.
     }

     if ( bRc )
     {                            // This key should not be passed on.
        event.setResult( keyEvent.result() );
     }
  } // endif WM_CHAR

  return bRc;
}
#endif // IC_MOTIFPM

#ifdef IC_WIN
bool IKeyboardHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;
  unsigned long keyEventId = event.eventId();

  switch ( keyEventId )
  {
    case WM_SYSCHAR:
    {
      // To preserve common behavior with OS/2, do not pass a keystroke
      // that is a menu bar mnemonic to any virtual functions of the
      // keyboard handler.  Just pass these on to the window's window
      // procedure to allow the appropriate menu choice to be selected.
      bool menuBarMnemonic = false;
      IKeyboardEvent keyEvent( event );
      if ( keyEvent.isCharacter()  &&  ! keyEvent.isUpTransition() )
      {  // Alt + character key press.
         // Look for a window with a menu bar.
         IWindow* window = keyEvent.dispatchingWindow();
         bool done = false;
         while ( ! done )
         {
            if ( window == 0  ||
                 window == IWindow::desktopWindow()  ||
                 window == IWindow::objectWindow() )
            {
               done = true;
            }
            else if ( ! ( ISTYLEOF( window->handle() ) & WS_CHILD ) )
            {     // GetMenu only reliable for non-WS_CHILD windows.
               HMENU menuHandle = GetMenu( window->handle() );
               if ( menuHandle )
               {  // Found a window with a menu bar.
                  // Note: No need to test for an existing IMenu object,
                  //       since neither IMenu nor IMenuBar constructors
                  //       call IWindow::startHandlingEvents.
                  IMenu menu( menuHandle );

                  // Iterate all menu bar items to see if the key press
                  // is a mnemonic (the underlined character in the menu
                  // item text, identified by preceding '&' character).
                  IMenu::Cursor cursor( menu );
                  for ( cursor.setToFirst();
                        cursor.isValid();
                        cursor.setToNext() )
                  {
                     IMenuItem menuItem = menu.elementAt( cursor );
                     IString menuItemText = menuItem.text();
                     unsigned long mnemonic = menuItemText.indexOf( '&' );
                     if ( mnemonic  &&
                          ( keyEvent.mixedCharacter().upperCase() ==
                              menuItemText.upperCase()[ mnemonic + 1 ] ) )
                     {  // Match found with mnemonic character.
                        menuBarMnemonic = true;
                     }
                  }

                  // Even if match not found, stop at first menu bar.
                  done = true;
               }
               else
               {  // See if parent window has a menu bar.
                  window = window->parent();
               }
            }
            else
            {     // GetMenu not reliable for WS_CHILD windows, so
                  // try next window in parent chain.
               window = window->parent();
            }
         }  // End search for a window with a menu bar.
      }  // End Alt + character key press.

      if ( menuBarMnemonic )
      {       // Don't pass menu bar mnemonic to virtual functions.
         break;
      }       // If not menu bar mnemonic, fall to next case.
    }  // End WM_SYSCHAR.

    case WM_DEADCHAR:
    case WM_SYSDEADCHAR:
      if ( keyEventId == WM_DEADCHAR  ||  keyEventId == WM_SYSDEADCHAR )
      {         // (Be sure to screen out WM_SYSCHAR.)
         fKeyStateFlag |= IC_DIACRITIC_FOUND;
         fDiacriticKey = event.parameter1();
      }
      // Fall through (and let WM_SYSCHAR fall through too).

    case WM_KEYDOWN:
    case WM_SYSKEYDOWN:
    case WM_KEYUP:
    case WM_SYSKEYUP:
    case WM_CHAR:
    {
      IKeyboardEvent keyEvent( event );

      // Check if this is a press of a key.
      if ( keyEvent.isUpTransition() == false )
      {
         // If this is a character message, then check for the special
         // case processing needed for diacritic codes.
         if ((keyEventId == WM_CHAR) || (keyEventId == WM_SYSCHAR))
         {
            // If a diacritic key was the previous character...
            if ( fKeyStateFlag & IC_DIACRITIC_FOUND )
            {
              // Reset flag and check if this character is the same.
              fKeyStateFlag &= (unsigned long)( ~IC_DIACRITIC_FOUND );
              if ( keyEvent.parameter1() == fDiacriticKey )
              {
                 // If so, the the composite was invalid, so expect another.
                 keyEvent.fInvalidComposite = true;
                 fKeyStateFlag |= IC_COMPOSITE_IS_NEXT;
              }
              else
                 keyEvent.fValidComposite = true;

              // Reset diacritic key value.
              fDiacriticKey = 0;
           }
           // If a diacritic key was the previous character...
           if ( fKeyStateFlag & IC_COMPOSITE_IS_NEXT )
           {
              // Reset flag and check if this character is the same.
              fKeyStateFlag &= (unsigned long)( ~IC_COMPOSITE_IS_NEXT );
              keyEvent.fInvalidComposite = true;
           }
        }

        // Process the event by calling the potentially overridden functions.
        if ( keyEvent.isScanCode()  &&
             this->scanCodeKeyPress( keyEvent ) )
        {                         // Have a scan code.
           bRc = true;            // No additional processing needed.
        }
        else if ( keyEvent.isVirtual()  &&
                  this->virtualKeyPress( keyEvent ) )
        {
           bRc = true;            // No additional processing needed.
        }
        else if ( keyEvent.isCharacter()  &&
                  this->characterKeyPress( keyEvent ) )
        {
           bRc = true;            // No additional processing needed.
        }
      }

      if ( bRc == false  &&  this->key( keyEvent ) )
      {                           // Catch-all virtual function.
         bRc = true;              // No additional processing needed.
      }

      if ( bRc )
      {                           // Key should not be passed on.
         event.setResult( keyEvent.result() );
      }
    }
    break;

    default:
      break;
  }

  return bRc;
}
#endif // IC_WIN

/*------------------------------------------------------------------------------
| IKeyboardHandler::scanCodeKeyPress                                           |
|                                                                              |
| Process the press of a key with a scan code value.                           |
------------------------------------------------------------------------------*/
bool IKeyboardHandler::scanCodeKeyPress ( IKeyboardEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IKeyboardHandler::virtualKeyPress                                            |
|                                                                              |
| Process the press of a virtual key.                                          |
------------------------------------------------------------------------------*/
bool IKeyboardHandler::virtualKeyPress ( IKeyboardEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IKeyboardHandler::characterKeyPress                                          |
|                                                                              |
| Process the press of a character key.                                        |
------------------------------------------------------------------------------*/
bool IKeyboardHandler::characterKeyPress ( IKeyboardEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IKeyboardHandler::key                                                        |
|                                                                              |
| Process any and all key events.                                              |
------------------------------------------------------------------------------*/
bool IKeyboardHandler::key ( IKeyboardEvent& event )
{
  return false;
}
