// Revision: 93 1.5.1.2 source/ui/baseapp/imenu1.cpp, menu, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imenu1.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imenunhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#if 0

#pragma priority( -2147481324 )

#define INCL_WINSYS
#define INCL_WINMENUS
extern "C" {
   #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <imenu.hpp>
#include <imenunhd.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMENU1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
#pragma enum(4)
#pragma pack(push,4)

class IMenuNotifyStatics
{
public:

  ~IMenuNotifyStatics ()
  {
    delete menuNotifyHandler;
    menuNotifyHandler = 0;
  }

static IMenuNotifyHandler
  &notificationHandler ( );

static IMenuNotifyHandler
  *menuNotifyHandler;
};

#pragma pack(pop)
#pragma enum(pop)

// Initialize static objects and notification strings.
static IMenuNotifyStatics menuStatics;

IMenuNotifyHandler* IMenuNotifyStatics::menuNotifyHandler = 0;

/*------------------------------------------------------------------------------
| IMenuNotifyStatics::notificationHandler                                      |
|                                                                              |
| Static function to create and return a notification handler for IMenu.       |
------------------------------------------------------------------------------*/
IMenuNotifyHandler& IMenuNotifyStatics::notificationHandler ()
{
  if (!(IMenuNotifyStatics::menuNotifyHandler))
  {
    IMenuNotifyStatics::menuNotifyHandler =
                                   new IMenuNotifyHandler();
  }
  return(*IMenuNotifyStatics::menuNotifyHandler);
}

/*------------------------------------------------------------------------------
| IMenu::enableNotification                                                    |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IMenu, does not send notifications to observer objects until it is enabled.  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IMenu& IMenu::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IMenuNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IMenuNotifyHandler::IMenuNotifyHandler                                       |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IMenuNotifyHandler::IMenuNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IMenuNotifyHandler::~IMenuNotifyHandler                                      |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IMenuNotifyHandler::~IMenuNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IMenuNotifyHandler::dispatchHandlerEvent                                     |
|                                                                              |
| This notifies IMenu Windows of the following events:                         |
|                                                                              |
| foregroundColor(Id)                                                          |
| backgroundColor(Id)                                                          |
| disabledForegroundColor(Id)                                                  |
| disabledBackgroundColor(Id)                                                  |
| hiliteForegroundColor(Id)                                                    |
| hiliteBackgroundColor(Id)                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
bool IMenuNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PM
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_PRESPARAMCHANGED:

        switch (anEvent.parameter1())
        {
          case PP_MENUFOREGROUNDCOLOR:
          case PP_MENUBACKGROUNDCOLOR:
          case PP_MENUDISABLEDFGNDCOLOR:
          case PP_MENUDISABLEDBGNDCOLOR:
          case PP_MENUHILITEFGNDCOLOR:
          case PP_MENUHILITEBGNDCOLOR:

            /**********************************************************/
            /* Post private message to delay notification until after */
            /* the selection state has been modified                  */
            /* Put color identifier into parameter 1 for later use    */
            /**********************************************************/
            thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                              IEventParameter1( anEvent.parameter1() ),
                              IEventParameter2( IC_NOTDEL_MENU_COLOR ));
            break;

          default:
            break;
        }
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* is used to determine which notification id to send           */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_MENU_COLOR)
        {
          switch (anEvent.parameter1())
          {
            case PP_MENUFOREGROUNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::foregroundColorId,
                                          *thisWindow));
              break;

            case PP_MENUBACKGROUNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::backgroundColorId,
                                          *thisWindow));
              break;

            case PP_MENUDISABLEDFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::disabledForegroundColorId,
                                          *thisWindow));
              break;

            case PP_MENUDISABLEDBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::disabledBackgroundColorId,
                                          *thisWindow));
              break;
            case PP_MENUHILITEFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::hiliteForegroundColorId,
                                          *thisWindow));
              break;

            case PP_MENUHILITEBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                          IWindow::hiliteBackgroundColorId,
                                          *thisWindow));
              break;
          }
        }
        break;

      default:
        break;
    }  /* end switch */
  }
#endif // IC_PM

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

#ifdef IC_MOTIF

bool IMenuNotifyHandler::dispatchHandlerEvent(IEvent& anEvent)
{
   return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_MOTIF

#endif  //#if 0
