// Revision: 97 1.6.1.8 source/ui/baseapp/imenubld.cpp, menu, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imenubld.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the IMenuBld class.                                      *
*   By constructing a IMenuBld object the resChar string that is passed in     *
*   is parsed and the appropriate IMenuItems are added to to the pMenu that    *
*   is passed in.                                                              *
*   IResourceLibrary::loadMenu() creates an IMenuBld object on the stack       *
*   to effect menu loading.                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1997, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ilanglvl.hpp>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <imenubld.hpp>
#include <imenuprv.hpp>
#include <ixconst.h>
#include <iexcept.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IMENUBLD_CPP_
    #include <ipagetun.h>
#endif

//
// Constructor merely calls getTokens() which makes everything happen.
//
IMenuBld :: IMenuBld(char* resChar,
                     IWindowHandle menuHandle,
                     IMenu* pMenu)
            : resString(resChar),
              menuHandle(menuHandle),
              pMenu(pMenu),
              state(looking),
              subState(startString),
              subIdStack(),
              isSeparator(false),
              isChecked(false),
              isDisabled(false),
              miStyle(0)
{
   pMenu->fMenuData->creatingMenuUnmanaged = true;
   getTokens();
   manageMenu(menuHandle);
   pMenu->fMenuData->creatingMenuUnmanaged = false;
}

IMenuBld :: ~IMenuBld() { }

IAixMenuImp :: IAixMenuImp(char* resChar,
                           IWindowHandle menuHandle,
                           IMenu* pMenu)
            : idStack(),
              menuHandle(menuHandle),
              pMenu(pMenu)
{
   pMenu->fMenuData->creatingMenuUnmanaged = true;
   processMenuTemplateData((rtmenu*) resChar);
   manageMenu(menuHandle);
   pMenu->fMenuData->creatingMenuUnmanaged = false;
}

IAixMenuImp :: ~IAixMenuImp() { }

/*------------------------------------------------------------------------------
| IAixMenuImp::processMenuTemplateData - ProcessMenuTemplate                   |
------------------------------------------------------------------------------*/
char *IAixMenuImp::processMenuTemplateData(rtmenu* pResource)
{
  unsigned short miNext = 0;
  unsigned short i;
  menuitem* pMenuItem;
  unsigned short intoSubmenu=0;

  if (!idStack.isEmpty())
     intoSubmenu = idStack.top();
  else                                   //AJ26395
    intoSubmenu = ((unsigned short)pMenu->fMenuData->itemWithHandle(menuHandle)); //AJ26395

  if( pResource->version == 1)
  {
    pMenuItem = (menuitem*) ((char*)pResource + sizeof(rtmenuv1));
  }
  else
  {
    pMenuItem = (menuitem*) ((char*)pResource + sizeof(rtmenu));
  }

  for( i=1; i<=pResource->itemCount; i++)
  {
    menuitem* pMenuItemPrev = pMenuItem;

    pMenuItem = (menuitem*) processMenuItemData(pMenuItem, intoSubmenu);
    if( pMenuItemPrev->style & MIS_SUBMENU)
    {
      idStack.push(pMenuItemPrev->id);
      pMenuItem = (menuitem*) processMenuTemplateData((rtmenu*) pMenuItem);
    }
  }
  return (char*) pMenuItem;
}

/*------------------------------------------------------------------------------
| IAixMenuImp::processMenuItemData - Process menuitem                          |
------------------------------------------------------------------------------*/
char* IAixMenuImp::processMenuItemData(menuitem *pMenuItem,
//                                                IMenu* pMenu,
                                                unsigned long intoSubmenu)
{
  char *result=0;
  unsigned long miAttr = 0;
  unsigned long miStyle = 0;

  IMenuItem mi(pMenuItem->id);

  miStyle |= ((pMenuItem->style & MIS_SYSCOMMAND) |
              (pMenuItem->style & MIS_SUBMENU) |
              (pMenuItem->style & MIS_HELP));
  mi.setStyle(miStyle);

  miAttr  |= ((pMenuItem->attributes & MIA_CHECKED) |
              (pMenuItem->attributes & MIA_DISABLED));
  mi.setAttribute(miAttr);

  if(pMenuItem->style & MIS_SEPARATOR)
  {
    mi.setSeparator();
    result = (((char*) pMenuItem + sizeof(menuitem)) - sizeof(char));
  }
  else if(pMenuItem->style & MIS_OWNERDRAW)
  {
      mi.setDrawItem();
      result = (((char*) pMenuItem + sizeof(menuitem)) - sizeof(char));
  }
  else
  {
    mi.setText(IString(&(pMenuItem->text)));
    result = (((char*) pMenuItem + IString(&(pMenuItem->text)).length() +
               sizeof(menuitem)));
  }
  pMenu->IMenu::addItem(mi, intoSubmenu);        //27557 - added IMenu::
  if( pMenuItem->style & MIS_SUBMENU)
    pMenu->IMenu::addSubmenu(pMenuItem->id);     //27557 - added IMenu::

  return result;
}

//
// manageMenu - recursively traverse menu managing created widgets
//
void IAixMenuImp :: manageMenu( const IWindowHandle &whnd  )
{
   Cardinal numChildren;
   WidgetList children;
   Widget whndMenu;

   if ( !XtIsComposite( (Widget) whnd) )
     return;

   XtVaGetValues( ( Widget) whnd,
     XmNnumChildren, &numChildren,
     XmNchildren, &children,
     NULL );
   for ( int i = 0; i < numChildren; i++ ) {
     if (XmIsCascadeButton(children[i])) {
       XtVaGetValues(children[i],
                     XmNsubMenuId, &whndMenu,
                     NULL);
       if (whndMenu)
         manageMenu((IWindowHandle)whndMenu);
     }
   }    // end for
   XtManageChildren(children, numChildren);
   return;
}

//
// getTokens - get all blank-delimited tokens from resString and call
//             parseIt() for each token
//
void IMenuBld :: getTokens(void)
{
   IString curToken;
   unsigned long resStringSize = resString.size();
   unsigned long leftPos;
   curPos = 1;
   while (curToken != "MENUEND") {
     leftPos = curPos;
     // skip leading white space
     while (resString[leftPos] == ' ' || resString[leftPos] == '\t')
       leftPos++;
     curPos = leftPos;
     // Move forward till next white space delimiting token or end of string
     while (resString[curPos] != ' ' &&
            resString[curPos] != '\t' &&
            curPos <= resStringSize)
       curPos++;

     curToken = resString.subString(leftPos, curPos - leftPos);
     parseIt(curToken);
   }
}

//
// parseIt - top level parser controlled by the state enum
//
void IMenuBld :: parseIt(IString& curTok)
{
   switch (state) {

     // looking for the next SUBMENU or MENUITEM
     case looking:
        if (curTok == "SUBMENU") {
          state = submenu;
          subState = startString;
        }
        else if (curTok == "MENUITEM") {
          state = menuitem;
          subState = startString;
        }
        else if (curTok == "END") {
          if (!subIdStack.isEmpty())
            subIdStack.pop();
        }
        break;

     // building up a SUBMENU
     case submenu:
        if (curTok == "BEGIN")
          submenuDone(curTok);
        else
          subParseIt(curTok);
        break;

     // building up a MENUITEM
     case menuitem:
        if (curTok == "MENUITEM" || curTok == "SUBMENU" ||
            curTok == "END"      || curTok == "MENUEND") {
          menuitemDone(curTok);
        }
        else
          subParseIt(curTok);
        break;
   }
}

//
// subParseIt - used to parse the info that follows the SUBMENU or MENUITEM
//              keyword.
//
void IMenuBld :: subParseIt(IString& curTok)
{
   switch (subState) {

     // expecting the start of a quoted string
     case startString:
      {
        if (curTok[1]  != '"')
          if (curTok.upperCase() == "SEPARATOR") {
            isSeparator = true;
            menuitemDone(curTok);
            break;
          }
          else
            ITHROWGUIERROR("String literal not found in Menu resource");

        // Not a Separator, so let's get the entire quoted string
        unsigned long curTokSize = curTok.size();
        unsigned long quoteStart = curPos - curTokSize + 1;

        // move curPos up to ending double quote
        // Deal with curTok that contains whole menu string (ie. "File")
        if (curTokSize > 1 && curTok[curTokSize] == '"')
           curPos--;  // back up to ending quote
        else
           while (resString[curPos] != '"') curPos++;  //advance to ending quote

        // now we've got the menuString
        //    and curPos points at the ending quote
        menuString = resString.subString(quoteStart, curPos - quoteStart);

        // make curPos point at next character after the ending double quote
        curPos++;
        subState = cmdId;
        break;
       }

     // expecting the menu item id (numeric value)
     case cmdId:
        if (!curTok.isDigits())
          ITHROWGUIERROR("Expected numeric menu id and found: " + curTok);
        menuId = curTok.asUnsigned();
        subState = style;
        break;

     // expecting one or more style or attribute flags
     // currently only support MIA_CHECKED and MIA_DISABLED
     // any other Attribute or Style is ignored
     case style:
        curTok.upperCase();
        if (curTok.includes("MIA_CHECKED"))
          isChecked = true;
        if (curTok.includes("MIA_DISABLED"))
          isDisabled = true;
        if (curTok.includes("MIS_SYSCOMMAND"))
          miStyle |= MIS_SYSCOMMAND;
        if (curTok.includes("MIS_HELP"))
          miStyle |= MIS_HELP;
        break;
   }
}

//
// submenuDone - called once we've parsed a complete SUBMENU
//
void IMenuBld :: submenuDone(IString& curTok)
{
   unsigned long intoSubmenu = 0;
   unsigned long afAttribute = 0;
   if (!subIdStack.isEmpty())
      intoSubmenu = subIdStack.top();

   IMenuItem mi(menuId);
   mi.setText(menuString);
   miStyle |= MIS_SUBMENU;
   mi.setStyle(miStyle);
   if (isChecked)
      afAttribute |= MIA_CHECKED;
   if (isDisabled)
      afAttribute |= MIA_DISABLED;
   mi.setAttribute(afAttribute);
   pMenu->IMenu::addItem(mi, intoSubmenu);    //27557 - added IMenu::
   pMenu->IMenu::addSubmenu(menuId);          //27557 - added IMenu::

   subIdStack.push(menuId);
   state = looking;

   clearItemData();
}

//
// menuitemDone - called once we've parsed a complete MENUITEM
//
void IMenuBld :: menuitemDone(IString& curTok)
{
   unsigned long intoSubmenu = 0;
   unsigned long afAttribute = 0;
   unsigned char rcType;
   if (subIdStack.isEmpty()) {
     XtVaGetValues(menuHandle,
                   XmNrowColumnType, &rcType,
                   NULL);
     if (rcType != XmMENU_BAR) {
        IString menuName = XtName((Widget)menuHandle);
        if (menuName.size())
           intoSubmenu = menuName.asInt();
        subIdStack.push(intoSubmenu);
        }
     }
   else
     intoSubmenu = subIdStack.top();

   IMenuItem mi(menuId);
   mi.setStyle(miStyle);
   if (isSeparator)
     mi.setSeparator();
   else
     mi.setText(menuString);
   if (isChecked)
      afAttribute |= MIA_CHECKED;
   if (isDisabled)
      afAttribute |= MIA_DISABLED;
   mi.setAttribute(afAttribute);
   pMenu->IMenu::addItem(mi, intoSubmenu);      //27557 - added IMenu::

   if (curTok == "MENUITEM") {
     state = menuitem;
     subState = startString;
   }
   else if (curTok == "SUBMENU") {
     state = submenu;
     subState = startString;
   }
   else if ( (curTok == "END") || (curTok == "MENUEND") ) {
     if (!subIdStack.isEmpty())
        subIdStack.pop();
     state = looking;
   }
   else
     state = looking;

   clearItemData();
}

//
// clearItemData - reset data about item being built
//
void IMenuBld :: clearItemData(void)
{
   menuString = "";
   menuId = 0;
   isSeparator = false;
   isChecked = false;
   isDisabled = false;
   miStyle = 0;
}

//
// manageMenu - recursively traverse menu managing created widgets
//
void IMenuBld :: manageMenu( const IWindowHandle &whnd  )
{
   Cardinal numChildren;
   WidgetList children;
   Widget whndMenu;

   if ( !XtIsComposite( (Widget) whnd) )
     return;

   XtVaGetValues( ( Widget) whnd,
     XmNnumChildren, &numChildren,
     XmNchildren, &children,
     NULL );
   for ( int i = 0; i < numChildren; i++ ) {
     if (XmIsCascadeButton(children[i])) {
       XtVaGetValues(children[i],
                     XmNsubMenuId, &whndMenu,
                     NULL);
       if (whndMenu)
         manageMenu((IWindowHandle)whndMenu);
     }
   }    // end for
   XtManageChildren(children, numChildren);
   return;
}

