// Revision: 98 1.7.1.3 source/ui/baseapp/imenubld.hpp, menu, ioc.v400, 000302 
/* NOSHIP */
#ifndef _IMENUBLD_
#define _IMENUBLD_
/*******************************************************************************
* FILE NAME: imenubld.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Contains the declaration of the IMenuBld class. This class is used         *
*   by IResourceLibrary::loadMenu() to parse Application Resources that        *
*   represent a Menu definition as defined in a PM .rc file.                   *
*   This class is not intended to be used by customers.                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
********************************************************************************/

#include <ibase.hpp>
#include <istring.hpp>
#include <imenu.hpp>
#include <iresprv.hpp>
#include <istack2.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

typedef IStack<unsigned long> SubIdStack;
typedef IStack<unsigned short> IdStack;

#pragma pack(push,4)
#pragma enum(4)

class IAixMenuImp  {
public:
  IAixMenuImp(char*  res,
              IWindowHandle menuHandle,
              IMenu* pMenu);
  ~IAixMenuImp();

  void manageMenu(const IWindowHandle &whnd);
  char * processMenuTemplateData(rtmenu* pResource);
  char * processMenuItemData(menuitem *pMenuItem,
                             unsigned long intoSubmenu);

private:
  IdStack idStack;
  IMenu*  pMenu;
  IWindowHandle  menuHandle;

};

class IMenuBld {
public:
  // States used in parseIt()
  enum State {looking, submenu, menuitem};
  // States used in subParseIt()
  enum SubState {startString, cmdId, style};

  IMenuBld(char*  res,
           IWindowHandle menuHandle,
           IMenu* pMenu);
  ~IMenuBld();

   void getTokens(void);
   void parseIt(IString& curTok);
   void subParseIt(IString& curTok);
   void submenuDone(IString& curTok);
   void menuitemDone(IString& curTok);
   void clearItemData(void);
   void manageMenu(const IWindowHandle &whnd);

private:
  // Holds string representing entire menu that was obtained from the
  // Resource DataBase. Passed in on cstor.
  IString    resString;

  // Holds position of current word in resString
  unsigned long curPos;

  // Submenu hwnd that will be build up when no SUBMENU is specified.
  // Passed into loadMenu() and passed in on the cstro.
  IWindowHandle  menuHandle;

  // Menu object that will be built up. Passed into loadMenu() and passed in
  // on the cstor.
  IMenu*     pMenu;

  // States for 2 parsers.
  State      state;
  SubState   subState;

  // Data for Menu item currently being built up
  IString    menuString;
  unsigned long menuId;
  Boolean   isSeparator, isChecked, isDisabled;
  unsigned long miStyle;

  // Use this BB stack to keep track of the id of the SubMenu that
  // we are currently adding items to.
  SubIdStack subIdStack;
};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif
