// Revision: 43  1.8.1.3  source/ui/baseapp/imngpres.cpp, datatype, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: imngpres.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iwindow.hpp for the class IManagedPresSpaceHandle.                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <iexcept.hpp>
#include <icconst.h>

#ifdef IC_MOTIF
#include <ixdc.hpp>
#endif // IC_MOTIF

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IMNGPRES_CPP_
    #include <ipagetun.h>
#endif

IManagedPresSpaceHandle::IManagedPresSpaceHandle(const IWindowHandle& windowHandle)
  : fWindow(0)
  , fWindowHandle(&windowHandle)
  , fPresSpaceHandle()
{
#ifdef IC_PMWIN
#ifdef IC_WIN
  HDC hps =
#else
  HPS hps =
#endif
            IGETPS(*fWindowHandle);

  if (!hps)
     ITHROWGUIERROR("WinGetPS");

  fPresSpaceHandle = IPresSpaceHandle(hps, *fWindowHandle);
#endif // IC_PMWIN
#ifdef IC_MOTIF
  unsigned long valuemask = 0;     //Ignore XGCvalues & use defaults
  XGCValues values;
  fPresSpaceHandle = new IXDC( XtDisplay( (Widget) *fWindowHandle ),
                                XtWindow ( (Widget) *fWindowHandle ) );
  if (!fPresSpaceHandle)
     ITHROWGUIERROR("IXDC");
#endif // IC_MOTIF
}

void
IManagedPresSpaceHandle::releasePresSpace()
{
#ifdef IC_PMWIN
   // Need to verify that handle() call is NOT made in OS/2 (since not needed).
   if (fWindowHandle->isValid()) {
      if (!IRELEASEPS(*fWindowHandle,fPresSpaceHandle) ) {
         ITHROWLIBRARYERROR(IC_RELEASEPSFAIL,
                            IBaseErrorInfo::invalidRequest,
                            IException::recoverable);
      } /* endif */
   }
#endif // IC_PMWIN
#if 0
//This was causing crash in 2d test cases(on AIX), so is commented.  
//the GC can't be freed here as its not created in this class. Its only using IXDC.
// For more details check up defect 23306.
#ifdef IC_MOTIF
   if (fWindowHandle->isValid()) {
       XFreeGC( XtDisplay( (Widget) *fWindowHandle ), ((IXDC*)fPresSpaceHandle)->xGC );
   }
#endif // IC_MOTIF
#endif
}

IManagedPresSpaceHandle::IManagedPresSpaceHandle ( IWindow* window )
  : fWindow( window ),
    fWindowHandle( 0 ),
    fPresSpaceHandle( window->presSpace() )
{
}

IManagedPresSpaceHandle::~IManagedPresSpaceHandle ( )
{
  if (fWindow != 0)
    fWindow->releasePresSpace(fPresSpaceHandle);
  else
    releasePresSpace();
}
