// Revision: 18 1.16.1.8 source/ui/baseapp/imsgbox.cpp, views, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imsgbox.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imsgbox.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481624 )

#define INCL_WINFRAMEMGR
#define INCL_WINHOOKS
#define INCL_WINWINDOWMGR
extern "C" {
   #include <iwindefs.h>
   #include <string.h>
   #ifdef IC_MOTIF
     #include <langinfo.h>
     #include <Xm/MessageB.h>
     #include <Xm/PushB.h>
     #include <Xm/MwmUtil.h>
     #include <X11/Shell.h>
     #include <Xm/DialogS.h>
   #endif //IC_MOTIF
}

#include <imsgbox.hpp>
#include <ibidiset.hpp>
#include <icconst.h>
#include <ievtdata.hpp>
#include <iexcept.hpp>
#include <iframe.hpp>
#include <ihelp.hpp>
#include <imsghlpi.hpp>
#include <iprimlck.hpp>
#include <ireslib.hpp>
#include <ireslock.hpp>
#include <iseq2.h>
#include <ithread.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <imsgtext.hpp>
#include <iplatfrm.hpp>
#ifdef IC_MOTIF
  #include <imstring.hpp>
  #include <ixmlabel.hpp>
#endif //IC_MOTIF

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IMSGBOX_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
// Define ICLUI only styles to get correct enumeration return value.
// These are not valid styles to pass to Windows.
#define  MB_BASESTYLEMASK  0x0000000fL      // Mask to get base style bits.
#define  MB_CANCEL         0x0000000dL
#define  MB_ENTER          0x0000000eL
#define  MB_ENTERCANCEL    0x0000000fL
#define  MB_MOVEABLE       0                // Ignored.
#define  MBID_ENTER        0x0000000fL
#endif

const IMessageBox::Style IMessageBox::okButton(MB_OK);
const IMessageBox::Style IMessageBox::okCancelButton(MB_OKCANCEL);
#ifndef IC_WIN_FLAGNOP
const IMessageBox::Style IMessageBox::cancelButton(MB_CANCEL);
const IMessageBox::Style IMessageBox::enterButton(MB_ENTER);
const IMessageBox::Style IMessageBox::enterCancelButton(MB_ENTERCANCEL);
#endif
const IMessageBox::Style IMessageBox::retryCancelButton(MB_RETRYCANCEL);
const IMessageBox::Style
  IMessageBox::abortRetryIgnoreButton(MB_ABORTRETRYIGNORE);
const IMessageBox::Style IMessageBox::yesNoButton(MB_YESNO);
const IMessageBox::Style IMessageBox::yesNoCancelButton(MB_YESNOCANCEL);
const IMessageBox::Style IMessageBox::defButton1(MB_DEFBUTTON1);
const IMessageBox::Style IMessageBox::defButton2(MB_DEFBUTTON2);
const IMessageBox::Style IMessageBox::defButton3(MB_DEFBUTTON3);
#ifndef IC_WIN_FLAGNOP
const IMessageBox::Style IMessageBox::moveable(MB_MOVEABLE);
#endif
const IMessageBox::Style IMessageBox::noIcon(MB_NOICON);
const IMessageBox::Style IMessageBox::informationIcon(MB_INFORMATION);
const IMessageBox::Style IMessageBox::queryIcon(MB_ICONQUESTION);
const IMessageBox::Style IMessageBox::warningIcon(MB_WARNING);
const IMessageBox::Style IMessageBox::errorIcon(MB_ERROR);
const IMessageBox::Style IMessageBox::applicationModal(MB_APPLMODAL);
const IMessageBox::Style IMessageBox::systemModal(MB_SYSTEMMODAL);

#ifdef IC_MOTIF

extern "C" {

   void imessageboxHelpCallback( Widget     w,
                                 XtPointer  clientData,
                                 XtPointer  callData);
   void messageBoxButtonCallback( Widget w,
                                  XtPointer clientData,
                                  XtPointer callData );
}

static IString loadButtonLabel( unsigned long messageId,
                                const char   *defaultText ) ;

// Max width of message text in pixels.  This value is intended to cause the
// entire message box to still appear even at VGA resolution (640 screen width)
#define MESSAGE_TEXT_MAXWIDTH 600
#endif //IC_MOTIF

#ifdef IC_PM
class IMessageHelpList;
bool IMessageHelpListDeleter ( IMessageHelpItem* const& element,
                                         void* )
{        // Called to free all item storage in the message help list.
  delete element;
  return true;
}

/*------------------------------------------------------------------------------
| IMessageHelpList                                                             |
|                                                                              |
| Sequence of IMessageHelpItem objects.  This sequence represents all message  |
| boxes for which the IMsgBoxHelpHookFn is responsible for displaying help.    |
| This is an implementation class.                                             |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IMessageHelpList : public ISequence < IMessageHelpItem* >
{
public:
  IMessageHelpList ( );
 ~IMessageHelpList ( );
};

IMessageHelpList::IMessageHelpList ( )
  : ISequence < IMessageHelpItem* > ( )
{ }

IMessageHelpList :: ~IMessageHelpList ( )
{
  this->removeAll( &IMessageHelpListDeleter );
}

/*------------------------------------------------------------------------------
| IMessageHelpStatics                                                          |
|                                                                              |
| Class to handle destruction of the static objects for message help.          |
------------------------------------------------------------------------------*/
class IMessageHelpStatics {
public:
 ~IMessageHelpStatics ( );
static IMessageHelpList
 *messageHelpList     ( );
static IPrivateResource
 *helpListKey         ( );
private:
static IMessageHelpList
 *fMessageHelpList;
static IPrivateResource
 *fHelpListKey;
}; // IMessageHelpStatics

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IMessageHelpStatics::~IMessageHelpStatics                                    |
------------------------------------------------------------------------------*/
IMessageHelpStatics::~IMessageHelpStatics ( )
{
  IMODTRACE_DEVELOP("IMessageHelpStatics::~IMessageHelpStatics");
#if IC_STATIC_PRIORITY_SUPPORTED
  // Deletes are only safe if static object init ordering can be controlled.
  if ( IMessageHelpStatics::fMessageHelpList )
  {
     delete IMessageHelpStatics::fMessageHelpList;
  }
  if ( IMessageHelpStatics::fHelpListKey )
  {
     delete IMessageHelpStatics::fHelpListKey;
  }
#endif
}

/*------------------------------------------------------------------------------
| IMessageHelpStatics::messageHelpList                                         |
------------------------------------------------------------------------------*/
IMessageHelpList* IMessageHelpStatics::messageHelpList ( )
{
  if ( ! IMessageHelpStatics::fMessageHelpList )
  {
     IMessageHelpStatics::fMessageHelpList = new IMessageHelpList;
  }
  return IMessageHelpStatics::fMessageHelpList;
}

/*------------------------------------------------------------------------------
| IMessageHelpStatics::helpListKey                                             |
------------------------------------------------------------------------------*/
IPrivateResource* IMessageHelpStatics::helpListKey ( )
{
  if ( !fHelpListKey )
  {
      IPrimalLock lock;
      if ( !fHelpListKey )
         fHelpListKey = new IPrivateResource;
  }
  return fHelpListKey;
}

static IMessageHelpStatics MessageHelpStatics;
IMessageHelpList* IMessageHelpStatics::fMessageHelpList = 0;
IPrivateResource* IMessageHelpStatics::fHelpListKey = 0;


extern "C" {
  BOOL EXPENTRY IMsgBoxHelpHookFn ( HAB hab,
                                    ULONG mode,
                                    ULONG topic,
                                    ULONG subTopic,
                                    PRECTL prclPosition );
}
#endif


/*------------------------------------------------------------------------------
| IMessageBox::IMessageBox                                                     |
------------------------------------------------------------------------------*/
IMessageBox::IMessageBox ( IWindow* owner )
  : strClTitle(),
    pwndClOwner( owner ),
    fMessageBoxData( 0 )
{
  IMODTRACE_DEVELOP( "IMessageBox::ctor" );

  IThread::current().initializeGUI();
}

/*------------------------------------------------------------------------------
| IMessageBox::~IMessageBox                                                    |
------------------------------------------------------------------------------*/
IMessageBox::~IMessageBox ( )
{ }

/*------------------------------------------------------------------------------
| IMessageBox::setTitle                                                        |
|  Set the message box title. If not set, show will query                      |
|  owner's title.                                                              |
------------------------------------------------------------------------------*/
IMessageBox& IMessageBox::setTitle ( const char* title )
{
  strClTitle = title;
  return *this;
}

/*------------------------------------------------------------------------------
| IMessageBox::setTitle                                                        |
------------------------------------------------------------------------------*/
IMessageBox& IMessageBox::setTitle ( const IResourceId& title )
{
  return this->setTitle( title.resourceLibrary().loadString( title ));
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const char* message,
                                          Severity severity,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show" );

  Style style = IMessageBox::convertSeverity( severity );
  return this->show( message, style, helpPanelId );
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const IResourceId& message,
                                          Severity severity,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show" );

  Style style = IMessageBox::convertSeverity( severity );
  return this->show( message, style, helpPanelId );
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const char* message,
                                          const Style& style,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show - own Style" );

  Style newStyle = style;
  char* pszTitle = 0;
  bool freeMe = false;

  if ( strClTitle.length() == 0 )
  {
#ifdef IC_PMWIN
     unsigned uLen = 0;
     IWindowHandle titleHandle;
     if ( pwndClOwner  &&
          pwndClOwner != IWindow::objectWindow()  &&
          pwndClOwner != IWindow::desktopWindow() )
     {
        // Query window title text.
#ifdef IC_WIN
        // In Windows NT, title is not separate window, so query
        // frame directly for text.
        titleHandle = pwndClOwner->handle();
#else
        titleHandle =
          IWindow::handleWithParent( FID_TITLEBAR, pwndClOwner->handle() );
#endif
        uLen = IQUERYWINDOWTEXTLENGTH( titleHandle );
     }
     pszTitle = new char[ uLen + 1 ];
     freeMe = true;
     *pszTitle = '\0';

     if ( uLen != 0 )
     {
        IQUERYWINDOWTEXT( titleHandle, uLen+1, (PSZ)pszTitle );
     }
#endif //IC_PMWIN
#ifdef IC_MOTIF
     //Query window title text.
     _XtString  windowTitle;
     Widget     parent;

     if (pwndClOwner != 0)
     {
        for ( parent = (Widget)pwndClOwner->handle();
              parent != NULL;
              parent = XtParent( parent ))
        {
          if ( XtIsApplicationShell( parent ) ||
               XtIsTransientShell( parent )   ||
               XtIsTopLevelShell( parent )    ||
               XmIsVendorShell( parent )      ||
               XmIsDialogShell( parent )      ||
               XtIsWMShell( parent ))
          {
            XtVaGetValues( parent, XmNtitle, &windowTitle, NULL );
            break;
          } /* endif */
        }

        if ((Widget)pwndClOwner->handle() == IThread::current().applicationShell())
          pszTitle = XtName( IThread::current().appShell() );
        else
        {
          if ( ( windowTitle != NULL ) && (strlen( windowTitle ) != 0 ) )
            pszTitle = (char*)windowTitle;
        }
     }
     else
     {
        pszTitle = new char[2];
        freeMe = true;
        pszTitle[0] = ' ';
        pszTitle[1] = '\0';
     }
#endif //IC_MOTIF
  }

  Response result =
    IMessageBox::convertResponse( this->showMessageBox( message,
                                                        strClTitle.length() ?
                                                          (char*)strClTitle :
                                                          pszTitle,
                                                        helpPanelId,
                                                        newStyle ));
  if (freeMe)
     delete pszTitle;
  return result;
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const IResourceId& message,
                                          const Style& style,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show" );

  // First check for message in a a string table, then a message resource.
  IString str = message.resourceLibrary().tryToLoadString( message );
  if ( str.length() == 0 )
  {
     str = message.resourceLibrary().loadMessage( message );
  }
  return this->show( str, style, helpPanelId );
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const IException& exception,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show - IException" );
  // Set the severity flag. Unrecovrable exception is catastrophic
  // and recoverable is critical.
  Severity sev = catastrophic;
  if ( exception.isRecoverable() )
  {
     sev = critical;
  }

  return this->show( exception.text(), sev, helpPanelId );
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::show                                      |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::show ( const IBaseErrorInfo& error,
                                          unsigned long helpPanelId )
{
  IMODTRACE_DEVELOP( "IMessageBox::show - IBaseErrorInfo" );
  return this->show( error.text(), catastrophic, helpPanelId );
}

/*------------------------------------------------------------------------------
| IMessageBox::convertSeverity                                                 |
|  Convert severity to appropriate style.                                      |
------------------------------------------------------------------------------*/
IMessageBox::Style IMessageBox::convertSeverity ( Severity severity )
{
  Style style = okButton;

  switch ( severity )
  {
     case information:
     {
        style = okButton | informationIcon;
        break;
     }
     case warning:
     {
        style = enterCancelButton | warningIcon;
        break;
     }
     case action:
     {
        style = retryCancelButton | queryIcon;
        break;
     }
     case critical:
     {
        style = retryCancelButton | errorIcon;
        break;
     }
     case catastrophic:
     {
        style = okButton | errorIcon;
        break;
     }
     default:
     {
        ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                            IBaseErrorInfo::invalidParameter,
                            IException::recoverable );
        break;
     }
  } /* endswitch */

  return style;
}

/*------------------------------------------------------------------------------
| IMessageBox::Response IMessageBox::convertResponse                           |
|  Convert response return value to enum value.                                |
------------------------------------------------------------------------------*/
IMessageBox::Response IMessageBox::convertResponse ( unsigned long buttonId )
{
  switch ( buttonId )
  {
     case MBID_ENTER:
        return enter;
     case MBID_OK:
        return ok;
     case MBID_CANCEL:
        return cancel;
     case MBID_ABORT:
        return abort;
     case MBID_RETRY:
        return retry;
     case MBID_IGNORE:
        return ignore;
     case MBID_YES:
        return yes;
     case MBID_NO:
        return no;
     case MBID_ERROR:
        ITHROWGUIERROR( "IMESSAGEBOX" );
     default:
        return unknown;
  } /* endswitch */
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IMsgBoxHelpHookFn                                                            |
|  OS/2 help hook for displaying help panels for message boxes.                |
------------------------------------------------------------------------------*/
BOOL EXPENTRY IMsgBoxHelpHookFn ( HAB, ULONG mode, ULONG, ULONG, PRECTL )
{
  IMODTRACE_DEVELOP( "IMsgBoxHelpHookFn" );
  bool helpProcessed = false;

  if ( (unsigned short)mode == (unsigned short)HLPM_WINDOW )
  // The checking of only the lower word of the mode value is a hack
  // for an apparent OS/2 Warp GA bug.  This check ignores the upper
  // word of the value, which contains an unpredictable value.
  {           // Only process "dialog" windows.
     /*************************************************************/
     /* Try to verify that help was requested for a message box   */
     /* created with IMessageBox.  Unfortunately, there is no way */
     /* to completely verify this (for example, there is no       */
     /* HLPM_MESSAGE value passed to a help hook to even detect   */
     /* if we are dealing with help for a message box, in         */
     /* contrast to what the OS/2 Toolkit documentation           */
     /* describes).  We are at least guaranteed that the help     */
     /* request if for a window that was created on the same      */
     /* thread as where the message box was created, because that */
     /* is also the thread that calls WinRegisterHook.  For the   */
     /* rest, we only try verifying:                              */
     /*                                                           */
     /*   - The active window is not wrappered with an IWindow    */
     /*     (IMessageBox has no access to the message box itself, */
     /*     and we assume the application is not wrappering the   */
     /*     message box through some PM hook).                    */
     /*   - The active window has the desktop window as parent    */
     /*     (this is how IMessageBox creates the message box).    */
     /*   - The owner window is associated with an IHelpWindow.   */
     /*   - The window ID is between 1-64000 (IPF restricts       */
     /*     help panel IDs that way).                             */
     /*   - The combination of window ID, owner window, and       */
     /*     thread match a message box created by IMessageBox.    */
     /*                                                           */
     /* Note that any checks based on PM's implementation of      */
     /* message boxes (for example, the message box is a WC_FRAME */
     /* without a client window or sizing border...) could break  */
     /* in a future release of OS/2, if PM ever changes their     */
     /* implementation of message boxes.                          */
     /*************************************************************/
     IWindowHandle active =       // Window help was requested for.
        IQUERYACTIVEWINDOW( IWindow::desktopWindow()->handle() );
     if ( ! IWindow::windowWithHandle( active, false ) )
     {             // Active window has no IWindow.
        try
        {          // Don't need an exception killing the application.
           IFrameWindow msgBox( active );
           IWindow* msgOwner = msgBox.owner();
           if ( msgOwner  &&  msgBox.parent() == IWindow::desktopWindow() )
           {      // Message box has some owner, and desktop for parent.
              unsigned long windowId = msgBox.id();
              IHelpWindow* help = IHelpWindow::helpWindow( msgOwner );
              if ( help  &&  windowId >= 1  &&  windowId <= 64000 )
              {
                 IResourceLock
                    helpListLock( *IMessageHelpStatics::helpListKey() );
                 IMessageHelpList* helpList =
                                 IMessageHelpStatics::messageHelpList();
                 IMessageHelpList::Cursor cursor( *helpList );
                 for ( cursor.setToFirst();
                       cursor.isValid();
                       cursor.setToNext() )
                 {           // Look for match.
                    IMessageHelpItem* helpItem = helpList->elementAt( cursor );
                    if ( helpItem->messageBoxId() == windowId  &&
                         helpItem->messageBoxThreadId() ==
                                        IThread::current().id() )
                    {        // See if owner window matches.
                       IWindowHandle ownerHwnd =
                              helpItem->messageBoxOwnerHandle();

                       // Note: The owner window may have been silently
                       // modified by PM.  If so, it will be in the parent
                       // window chain of the specified owner, and a child
                       // window of the message box's parent (the desktop
                       // window).
                       if ( helpItem->messageBoxOwner() == msgOwner  ||
                            ( ownerHwnd.isValid()  &&
                              ownerHwnd
                               .sendEvent( WM_QUERYFOCUSCHAIN,
                                           IEventParameter1( QFC_PARTOFCHAIN ),
                                           IEventParameter2( msgOwner->handle() ))
                                   != 0  &&
                              msgOwner->parent() ==
                                      IWindow::desktopWindow() ))
                       {     // Appears to be an IMessageBox.
                          help->show( (IResourceId)windowId );
                          helpProcessed = true;
                          break;
                       }
                    }
                 }
                 if ( ! helpProcessed )
                 {
                    ITRACE_DEVELOP( "No IMessageBox match found." );
                 }
              }
              else
              {
                 ITRACE_DEVELOP( "Owner has no associated IHelpWindow," +
                                  IString( "or window ID out of range." ));
              }
           }
           else
           {
              ITRACE_DEVELOP( "No IWindow owner, or non-desktop parent." );
           }
        }
        catch ( IException& )
        {          // Let normal help manager processing occur.
           ITRACE_RUNTIME( "Exception during message box help processing." );
           helpProcessed = true;
        }
     }
     else
     {
        ITRACE_DEVELOP( "Help requested for an IWindow." );
     }
  }

  return helpProcessed;
}
#endif

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| imessageboxHelpCallback                                                      |
| The clientData with this callback contains a pointer to an                   |
| IMessageHelpItem object, which contains the help id and                      |
| the IWindow* to dispatch the help message to.                                |
------------------------------------------------------------------------------*/
void imessageboxHelpCallback( Widget     w,
                              XtPointer  clientData,
                              XtPointer  callData)
{
   IMODTRACE_DEVELOP("IMessageBox::imessageboxHelpCallback");

   IMessageHelpItem* messageHelpItem = (IMessageHelpItem*) clientData;
   if (messageHelpItem)
   {
      // If there is an owner window, construct the help message
      // and send it.
      if (messageHelpItem->messageBoxOwner())
      {
         HELPINFO helpinfo;
         helpinfo.cbSize       = sizeof(HELPINFO);
         helpinfo.iContextType = HELPINFO_WINDOW;
         helpinfo.hItemHandle  = w;
         helpinfo.iCtrlId      = messageHelpItem->messageBoxId();
         helpinfo.dwContextId  = messageHelpItem->messageBoxId();
         messageHelpItem->messageBoxOwner()->sendEvent( WM_HELP, 0, &helpinfo );
      }
   }
   return;
}

/*------------------------------------------------------------------------------
| Callback function for the message box pushbuttons created in showMessageBox  |
| The clientData field contains the button ID (MBID_*) of the button causing   |
| the callback.                                                                |
------------------------------------------------------------------------------*/
void messageBoxButtonCallback( Widget w,
                               XtPointer clientData,
                               XtPointer callData )
{
  XmAnyCallbackStruct *pacb = (XmAnyCallbackStruct *)callData;
  unsigned long ulButtonId = (unsigned long)clientData;
  Widget MessageBox = NULL;

  switch ( pacb->reason )
  {
    case XmCR_OK:
    case XmCR_CANCEL:
      MessageBox = w;
      break;
    case XmCR_ACTIVATE:
      MessageBox = XtParent( w );
      break;
  }
  if (MessageBox)
     XtVaSetValues( MessageBox, XmNuserData, ulButtonId, NULL );
}

/*------------------------------------------------------------------------------
| loadButtonLabel - loads the string for the button label from the             |
| application message file.                                                    |
------------------------------------------------------------------------------*/
IString loadButtonLabel( unsigned long messageId,
                         const char   *defaultText)
{
   IMessageText msgText = IMessageText( messageId );
   msgText.setDefaultText( defaultText );
   return IString( msgText.text() );
}

#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IMessageBox::showMessageBox                                                  |
------------------------------------------------------------------------------*/
unsigned long IMessageBox::showMessageBox ( const char* message,
                                            const char* title,
                                            unsigned long helpPanelId,
                                            const Style& style )
{
  IMODTRACE_DEVELOP( "IMessageBox::showMessageBox" );
#ifdef IC_PMWIN
  unsigned long ulStyle = style.asUnsignedLong();
  unsigned long ulResponse = 0;
  if ( helpPanelId )         // Add help style and find help window.
  {
     ulStyle |= MB_HELP; // Always allow a help button so HTML help will work.
#ifdef IC_PM
     // Ensure help is associated,
     if ( this->pwndClOwner  &&
          IHelpWindow::helpWindow( this->pwndClOwner ))
     {

        // Add an entry for the message box into the message help list,
        // which the help hook will use to display help for the message.
        IMessageHelpItem* helpItem =
                 new IMessageHelpItem( helpPanelId, this->pwndClOwner );
        {          // (Reduce scope of the resource lock.)
           IResourceLock
              helpListLock( *IMessageHelpStatics::helpListKey() );
           IMessageHelpStatics::messageHelpList()->addAsFirst( helpItem );
        }

        // We want the IMsgBoxHelpHookFn called when the user requests
        // help while the message box is displayed, to have a chance
        // at displaying help for the message box.
        WinSetHook( IThread::current().anchorBlock(),
                    HMQ_CURRENT,
                    HK_HELP,
                    PFN( IMsgBoxHelpHookFn ),
                    0 );
     }
#endif // IC_PM
  }

#ifdef IC_WIN
  // Save the specified style.  Then convert ICLUI only styles to
  // ones supported by Windows.
  unsigned long icStyle = ulStyle & MB_BASESTYLEMASK;
  switch ( icStyle )
  {
     case MB_CANCEL:
     case MB_ENTER:
        ulStyle = (ulStyle & ~MB_BASESTYLEMASK) | MB_OK;
        break;
     case MB_ENTERCANCEL:
        ulStyle = (ulStyle & ~MB_BASESTYLEMASK) | MB_OKCANCEL;
        break;
     default:
        break;
  }
  ulStyle |= MB_SETFOREGROUND;  // make the message box rise to foreground
#endif

#ifdef IC_WIN
  // Windows appears to eat all messages on the queue when displaying
  // a message box.  This causes breakage if the message box is displayed
  // just after a modal dialog has exited and there is an IC_UM_QUIT on
  // the queue.  If there is such a message we save it and post it later
  // if the message box ate it.
  MSG msg;
  bool savedMsg =
    PeekMessage( &msg, 0, IC_UM_QUIT, IC_UM_QUIT, PM_NOREMOVE );

  // Provide bidi enablement here.  Here on Windows, we emulate the way
  // a window inherits bidi attributes on OS/2 (a window parented to
  // the desktop uses the application-wide bidi settings).  For a
  // right-to-left message box, add in the appropriate styles.
  if ( IBidiSettings::isBidiSupported() )
  {
     IBidiSettings
       bidiValues = IBidiSettings::applicationDefaults();
     if ( bidiValues.windowLayout() == IBidiSettings::layoutRightToLeft )
     {
        ulStyle |= MB_RIGHT;
     }
     if ( bidiValues.textOrientation() == IBidiSettings::textRightToLeft )
     {
        ulStyle |= MB_RTLREADING;
     }
  }

  // To support Help Buttons on messageboxes in Windows NT and 95,
  // you must use the MessageBoxIndirect function.

  MSGBOXPARAMS msgboxParams;
  msgboxParams.cbSize           = sizeof(MSGBOXPARAMS);
  msgboxParams.hwndOwner        = this->pwndClOwner ?
                                    this->pwndClOwner->handle() :
                                    IWindowHandle( 0 );
  msgboxParams.hInstance        = 0;
  msgboxParams.lpszText         = message;
  msgboxParams.lpszCaption      = title;
  msgboxParams.dwStyle          = ulStyle;
  msgboxParams.lpszIcon         = 0;
  msgboxParams.dwContextHelpId  = helpPanelId;
  msgboxParams.lpfnMsgBoxCallback = 0;
  msgboxParams.dwLanguageId     = 0;
  ulResponse = MessageBoxIndirect( (LPMSGBOXPARAMS) &msgboxParams );
  if (ulResponse == 0)
     {
     ITHROWGUIERROR2( "MessageBoxIndirect",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable );
     }
#else
    ulResponse = IMESSAGEBOX( IWindow::desktopWindow()->handle(),
                              this->pwndClOwner ?
                                this->pwndClOwner->handle() :
                                IWindowHandle( 0 ),
                              message, title,
                              helpPanelId, ulStyle );
#endif

#ifdef IC_WIN
  // If there was a message eaten, post it now.
  if (savedMsg)
     {
     MSG msg2;
     if ( !PeekMessage( &msg2, 0, IC_UM_QUIT, IC_UM_QUIT, PM_NOREMOVE))
       {
       ITRACE_DEVELOP(
          IString("Re-posting quit on queue: hwnd=") +
          IString((unsigned long)msg.hwnd).d2x()+
          IString(", Msg=") + IString(msg.message).d2x()+
          IString(", wParam=") + IString(msg.wParam).d2x()+
          IString(", lParam=") + IString(msg.lParam).d2x() );

       PostMessage( 0, msg.message, msg.wParam, msg.lParam );
       }
     }


  // Convert the response back to the one expected in the interface
  // for the ICLUI only styles.
  switch ( icStyle )
  {
     case MB_CANCEL:
        ulResponse = MBID_CANCEL;
        break;
     case MB_ENTER:
        ulResponse = MBID_ENTER;
        break;
     case MB_ENTERCANCEL:
        if ( ulResponse == MBID_OK )
        {
           ulResponse = MBID_ENTER;
        }
        break;
     default :
        break;
  }
#endif

#ifdef IC_PM
  if ( helpPanelId )
  {
     WinReleaseHook( IThread::current().anchorBlock(),
                     HMQ_CURRENT,
                     HK_HELP,
                     PFN( IMsgBoxHelpHookFn ),
                     0 );

     // Remove the entry for the message box from the message help list.
     IResourceLock
        helpListLock( *IMessageHelpStatics::helpListKey() );
     IMessageHelpList* helpList = IMessageHelpStatics::messageHelpList();
     IMessageHelpList::Cursor cursor( *helpList );
     for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
     {             // Look for match.
        IMessageHelpItem* helpItem = helpList->elementAt( cursor );
        if ( helpItem->messageBoxId() == helpPanelId  &&
             helpItem->messageBoxOwner() == this->pwndClOwner  &&
             helpItem->messageBoxThreadId() == IThread::current().id() )
        {          // Found item to remove.
           helpList->removeAt( cursor );
           delete helpItem;
           break;
        }
     }
  }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
   unsigned long ulStyle = style.asUnsignedLong();
   unsigned long ulResponse = 0;
   Widget   w;
   Widget   userButton = NULL;
   IString       okLabel, cancelLabel, userLabel;
   unsigned long okButtonId, cancelButtonId, userButtonId;
   // default buttons
   const unsigned long
      xmDialogOK = 1,
      xmDialogCancel = 2,
      xmDialogUser = 4;
   unsigned long buttonsUsed = 0;

   if (helpPanelId)                   //Add help style and find help window
      {
      ulStyle |= MB_HELP;
      }

   /*--------------------------------------------------------------------------
      Map the OS/2 styles to the corresponding button text
   --------------------------------------------------------------------------*/
   switch ( ulStyle & 0x0000000F)
      {
      case  MB_OKCANCEL:
        buttonsUsed = xmDialogOK | xmDialogCancel;
        okButtonId = MBID_OK;
        cancelButtonId = MBID_CANCEL;
        break;
      case  MB_CANCEL:
        buttonsUsed = xmDialogCancel;
        cancelButtonId = MBID_CANCEL;
        break;
      case  MB_ENTERCANCEL:
        buttonsUsed = xmDialogCancel;
        cancelButtonId = MBID_CANCEL;
        // Intentional fall thru to next case
      case  MB_ENTER:
        buttonsUsed |= xmDialogOK;
        okLabel = loadButtonLabel(IC_LABEL_ENTER, "Enter");
        okButtonId = MBID_ENTER;
        break;
      case  MB_RETRYCANCEL:
        buttonsUsed = xmDialogOK | xmDialogCancel;
        okLabel = loadButtonLabel(IC_LABEL_RETRY, "Retry");
        okButtonId = MBID_RETRY;
        cancelButtonId = MBID_CANCEL;
        break;
      case  MB_ABORTRETRYIGNORE:
        buttonsUsed = xmDialogOK | xmDialogCancel | xmDialogUser;
        okLabel = loadButtonLabel(IC_LABEL_ABORT, "Abort");
        userLabel = loadButtonLabel(IC_LABEL_RETRY, "Retry");
        cancelLabel = loadButtonLabel(IC_LABEL_IGNORE, "Ignore");
        okButtonId = MBID_ABORT;
        userButtonId = MBID_RETRY;
        cancelButtonId = MBID_IGNORE;
        break;
      case  MB_YESNOCANCEL:
        buttonsUsed = xmDialogOK | xmDialogCancel | xmDialogUser;
        okLabel = loadButtonLabel(IC_LABEL_YES, "Yes");
        userLabel = loadButtonLabel(IC_LABEL_NO, "No");
        okButtonId = MBID_YES;
        userButtonId = MBID_NO;
        cancelButtonId = MBID_CANCEL;
        break;
      case  MB_YESNO:
        buttonsUsed |= xmDialogOK | xmDialogCancel;
        okLabel = loadButtonLabel(IC_LABEL_YES, "Yes");
        cancelLabel = loadButtonLabel(IC_LABEL_NO, "No");
        okButtonId = MBID_YES;
        cancelButtonId = MBID_NO;
        break;
      default:   // includes case MB_OK
        buttonsUsed = xmDialogOK;
        okButtonId = MBID_OK;
        break;
      }

   unsigned char defaultButton;
   switch (ulStyle & 0x00000F00)
      {
      case MB_DEFBUTTON1:
          defaultButton = XmDIALOG_OK_BUTTON;
          break;
      case MB_DEFBUTTON2:
          if ( buttonsUsed & xmDialogUser )
             defaultButton = XmDIALOG_NONE;
           else
             defaultButton = XmDIALOG_CANCEL_BUTTON;
          break;
      case MB_DEFBUTTON3:
          if ( buttonsUsed & xmDialogUser )
             defaultButton = XmDIALOG_CANCEL_BUTTON;
          else
             defaultButton = XmDIALOG_HELP_BUTTON;
          break;
      default:
          defaultButton = XmDIALOG_OK_BUTTON;
      }

   /*--------------------------------------------------------------------------
      Map the OS/2 styles to the corresponding icon.
   --------------------------------------------------------------------------*/
   unsigned char dialogType;
   switch ( ulStyle & 0x000000F0 )
      {
      case MB_ICONHAND :
         dialogType = XmDIALOG_ERROR;
         break;
      case MB_ICONASTERISK :
         dialogType = XmDIALOG_INFORMATION;
         break;
      case MB_ICONEXCLAMATION :
         dialogType = XmDIALOG_WARNING;
         break;
      case MB_ICONQUESTION :
         dialogType = XmDIALOG_QUESTION;
         break;
      default:
         dialogType = XmDIALOG_MESSAGE;
         break;
      }

   /*--------------------------------------------------------------------------
      Set up the message box defaults and create it.
   --------------------------------------------------------------------------*/
   IArgList
     argList;
   // Get bidi resources for the message box, based on the
   // application bidi attributes.
   if ( IBidiSettings::isBidiSupported() )
   {
      // A message box is basically a top-level window, so have it
      // use application-wide bidi settings.
      IBidiSettings
        bidiValues = IBidiSettings::applicationDefaults();
      argList = bidiValues.asArgList();
   }

   IMString xms = IMString( title );
   argList
    .setResource( XmNdialogTitle, (long) (XmString) xms );

   // Determine the parent of the message box.  If the provided
   // IWindow is null or the desktopWindow, use the appShell as
   // the parent.  Otherwise, use the provided window as parent.
   IWindowHandle  hParent;
   IFrameWindow * ownerFrame = 0;
   bool        ownerDestroyOnClose = false;
   if ((pwndClOwner != 0) && (pwndClOwner != IWindow::desktopWindow()))
     {
     hParent = pwndClOwner->handle();
     if ( pwndClOwner->isFrameWindow() )
       {
       ownerFrame = (IFrameWindow *)pwndClOwner;
       ownerDestroyOnClose =(bool) ownerFrame->willDestroyOnClose();
       ownerFrame->setDestroyOnClose( false );
      }
    }
   else
     hParent = IThread::current().appShell();
   w = XmCreateMessageDialog(hParent,
                             "IMessageBox",
                             (ArgList) argList.argList(),
                             argList.numberOfArgs() );

   XtVaSetValues( w,
                  XmNdialogType, dialogType,
                  XmNuserData, MBID_CANCEL,
                  NULL);
   /*--------------------------------------------------------------------------
      Change the window decoration to allow/disallow window movement, and
      to destroy widgets when dismissed.
   --------------------------------------------------------------------------*/
   if ( ulStyle & MB_MOVEABLE )
     XtVaSetValues( XtParent(w),
                    XmNmwmFunctions, MWM_FUNC_MOVE,
                    XmNmwmDecorations, MWM_DECOR_BORDER | MWM_DECOR_TITLE,
                    XmNdeleteResponse, XmDESTROY,
                    NULL );
   else
     XtVaSetValues( XtParent(w),
                    XmNmwmFunctions, 0,
                    XmNmwmDecorations, MWM_DECOR_BORDER | MWM_DECOR_TITLE,
                    XmNdeleteResponse, XmDESTROY,
                    NULL );


   /*--------------------------------------------------------------------------
     Setup button 1
   --------------------------------------------------------------------------*/
   if (buttonsUsed & xmDialogOK )
      {
      // Assign the label text to button1
      // d5895 - Explicitly set button label text, to get ICLUI NLS version.
      if ( okLabel.length() == 0 )
         okLabel = loadButtonLabel(IC_LABEL_OK, "OK");
      IMString labelStr = IMString( okLabel );
      XtVaSetValues( w, XmNokLabelString, (XmString)labelStr, NULL );

      // Add the callback for button1
      XtAddCallback( w,
                     XmNokCallback, messageBoxButtonCallback,
                     (XtPointer)okButtonId);
      }
   else
      {
      XtUnmanageChild( XmMessageBoxGetChild( w, XmDIALOG_OK_BUTTON ));
      }

   /*--------------------------------------------------------------------------
     Setup button 2
   --------------------------------------------------------------------------*/
   if (buttonsUsed & xmDialogCancel )
      {
      // Assign the label text to button2
      // d5895 - Explicitly set button label text, to get ICLUI NLS version.
      if ( cancelLabel.length() == 0 )
         cancelLabel = loadButtonLabel(IC_LABEL_CANCEL, "Cancel");
      IMString labelStr = IMString( cancelLabel );
      XtVaSetValues( w, XmNcancelLabelString, (XmString)labelStr, NULL );
      //   Add the callback for button2
      XtAddCallback( w,
                     XmNcancelCallback, messageBoxButtonCallback,
                     (XtPointer)cancelButtonId);
      }
   else
      {
      XtUnmanageChild( XmMessageBoxGetChild( w, XmDIALOG_CANCEL_BUTTON ));
      }
   /*--------------------------------------------------------------------------
     Setup button 3.  We create this one ourselves if needed
   --------------------------------------------------------------------------*/
   if (buttonsUsed & xmDialogUser )
      {
      IMString labelStr = IMString( userLabel );
      Arg      args[5];
      int      n = 0;
      XtSetArg( args[n], XmNlabelString, (XmString)labelStr ); n++;
      userButton = XmCreatePushButton( w, "userButton", args, n );
      XtManageChild( userButton );
      // Add the callback for userButton
      XtAddCallback( userButton,
                     XmNactivateCallback, messageBoxButtonCallback,
                    (XtPointer)userButtonId);
      }

   /*--------------------------------------------------------------------------
      if a help button was not specified, unmanage it.  Otherwise, add
      the help callback
   --------------------------------------------------------------------------*/
   IMessageHelpItem* messageHelpItem = 0;
   if (!(ulStyle & MB_HELP))
     {
     XtUnmanageChild( XmMessageBoxGetChild( w, XmDIALOG_HELP_BUTTON ));
     }
   else
     {
     // Allocate an IMessageHelpItem object for use by the help callback
     // to dispatch the help message.
     messageHelpItem = new IMessageHelpItem( helpPanelId, pwndClOwner);

     // Explicitly set button label text, to get ICLUI NLS version.
     IString helpLabel = loadButtonLabel(IC_LABEL_HELP, "Help");
     IMString labelStr = IMString( helpLabel );
     XtVaSetValues( w, XmNhelpLabelString, (XmString)labelStr, NULL );
     XtAddCallback( w,
                    XmNhelpCallback, imessageboxHelpCallback,
                    (XtPointer)messageHelpItem);
     }

   /*--------------------------------------------------------------------------
      Set the default button in the message box.
   --------------------------------------------------------------------------*/
   XtVaSetValues( w,
                  XmNdefaultButtonType, defaultButton,
                  NULL );
   if ( (defaultButton == XmDIALOG_NONE ) && (userButton) )
      XtVaSetValues( w,
                     XmNdefaultButton, userButton,
                     NULL );

   /*--------------------------------------------------------------------------
      Determine modality.
   --------------------------------------------------------------------------*/
   if ( ulStyle & MB_SYSTEMMODAL )
     XtVaSetValues( w, XmNdialogStyle, XmDIALOG_SYSTEM_MODAL, NULL );
   else
     XtVaSetValues( w, XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, NULL );

   /*--------------------------------------------------------------------------
      assign the message text
   --------------------------------------------------------------------------*/
   IString messageText( message );
   IXmLabel::applyWordBreak( MESSAGE_TEXT_MAXWIDTH, w, messageText );
   IMString  messageString = IMString( messageText );
   XtVaSetValues( w, XmNmessageString, (XmString)messageString, NULL );

   // Note: X will calculate the right size for the message box

   XtManageChild( w );

   /*--------------------------------------------------------------------------
    local event loop to allow for OS/2 style modality
   --------------------------------------------------------------------------*/
   {
     XEvent   event;
     XtAppContext app = IThread::current().anchorBlock();
     bool continueLoop = true;
     while (continueLoop)
     {
        XtAppNextEvent( app, &event );
        XtDispatchEvent( &event );
        if ( event.type == UnmapNotify )
        {
           if (XtWindowToWidget( event.xunmap.display, event.xunmap.window ) ==
               XtParent( w ))
           {
              continueLoop = false;
              // If the window got an unmap event because of a minimized frame
              // parent, ignore it and continue this loop.
              IWindow* window = IWindow::windowWithHandle(XtParent(XtParent(w)));
              if (window)
              {
                 IFrameWindow* frame = dynamic_cast<IFrameWindow*>(window);
                 if (frame && frame->isMinimized())
                 {
                    continueLoop = true;
                 }
              }
           }
        }
     }
   }

   XtVaGetValues( w, XmNuserData, &ulResponse, NULL );
   XtDestroyWidget( XtParent(w) );

   // Delete the message help info if it was allocated.
   if (messageHelpItem)
      delete messageHelpItem;

  if (ownerFrame)
     {
     ownerFrame->setDestroyOnClose( ownerDestroyOnClose);
     }
#endif //IC_MOTIF
  return ulResponse;
}
