// Revision: 04 1.6.1.4 source/ui/baseapp/imsgwin.cpp, views, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imsgwin.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imsgwin.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1997             *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481624 )

#include <imsgwin.hpp>
#include <ibmpctl.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandle.hpp>
#include <iiconctl.hpp>
#include <imcelcv.hpp>
#include <ipushbut.hpp>
#include <isetcv.hpp>
#include <istring.hpp>
#include <itext.hpp>
#include <itextctl.hpp>
#include <ititle.hpp>
extern "C"
  {
  #define INCL_NLS
  #include <iwindefs.h>
  }

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IMSGWIN_CPP_
    #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMessageWindow private data class: IMessageWindowData                        |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IMessageWindowData {
/*------------------------------------------------------------------------------
| The IMessageWindowData class encapsulates private data and functions         |
| used by the IMessageWindow class.  An object of this class is created in the |
| IMessageWindow constructors.                                                 |
------------------------------------------------------------------------------*/
public:
  IMessageWindowData ( IMessageWindow* theFrame );
 ~IMessageWindowData ( );

// Visual components of the message window.
IBitmapHandle
  bitmapHandle;

IBitmapControl
 *messageImage;

ISystemPointerHandle::Identifier
  identifier;

IWindow
 *clientWindow;

IPushButton
 *buttons[9];

IMultiCellCanvas
 *buttonMCCanvas,
 *clientCanvas;

ISetCanvas
 *buttonSetCanvas,
 *imageSetCanvas;

ITextControl
 *message,
 *messageId;

IText
  messageText,
  messageIdText;

// Information about the frame.
unsigned long
  id;

IWindow
 *owner;

IMessageWindow::EMessageType
  type;

IMessageWindow::Style
  style;

IMessageWindow*
  myFrame;

// Flags and various.
bool
  isInitialized,
  useBitmapHandle,
  useImage,
  ourClient;

/*---------------------------- Private Functions -----------------------------*/
void
  initialize ();

bool
  initialized ();

void doBeep ();

void createButtons ();

void createAButton ( unsigned long, bool isDefault = false );

void addChildrenToClient ();

void moveAndSizeWindow ();

}; // IMessageWindowData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IMessageWindowData::IMessageWindowData                                       |
| Private data for the IMessageWindow class.  Contains all controls that make  |
| up the message window.                                                       |
| Initialize all pointers to zero.  They are allocated when the message window |
| is constructed or shown.                                                     |
------------------------------------------------------------------------------*/
IMessageWindowData::IMessageWindowData ( IMessageWindow* theFrame )
  : bitmapHandle(0)
    ,messageImage(0)
    ,identifier(ISystemPointerHandle::information)
    ,clientWindow(0)
    ,buttonMCCanvas(0)
    ,clientCanvas(0)
    ,buttonSetCanvas(0)
    ,imageSetCanvas(0)
    ,message(0)
    ,messageId(0)
    ,messageText()
    ,messageIdText()
    ,id(0)
    ,owner(0)
    ,type(IMessageWindow::kInformationMessage)
    ,style(IWindow::noStyle)
    ,myFrame(theFrame)
    ,isInitialized(false)
    ,useBitmapHandle(false)
    ,useImage(false)
    ,ourClient(false)
{
   for (int i=0 ; i < 9; i++)
   {
      buttons[i] = (IPushButton*) 0;
   }
}

/*------------------------------------------------------------------------------
| IMessageWindowData::~IMessageWindowData                                      |
------------------------------------------------------------------------------*/
IMessageWindowData::~IMessageWindowData ( )
{
}


/*------------------------------------------------------------------------------
| IMessageWindow::Style definitions                                            |
------------------------------------------------------------------------------*/
const IMessageWindow::Style
  IMessageWindow::deferClientCreation ( 0, IMGW_DEFERCLIENTCREATION );

// Make default style have the look and feel of a regular message box.  Users
// can override it if they wish.
#ifdef IC_PM
IMessageWindow::Style
  IMessageWindow::currentDefaultStyle ( (FS_DLGBORDER  |
                                         FS_TASKLIST   |
                                         FS_DBE_APPSTAT|
                                         FS_AUTOICON   ),
                                        (FCF_TITLEBAR  |
                                         FCF_SYSMENU   ));
#endif
#ifdef IC_MOTIFWIN
IMessageWindow::Style
  IMessageWindow::currentDefaultStyle ( (WS_CAPTION      |
                                         WS_SYSMENU      ),
                                        (WS_EX_APPWINDOW |
                                         IFS_APPDBCS     |
                                         IFS_DLGBACKGND  |
                                         WS_EX_DLGMODALFRAME));
#endif
/*------------------------------------------------------------------------------
| IMessageWindow constants                                                     |
------------------------------------------------------------------------------*/
const unsigned MGW_MESSAGEID_LIMIT (12);
const unsigned MGW_MAIN_MCCANVAS   (IC_ID_BASE + 0x1000);
const unsigned MGW_BUTTON_SETCANVAS(IC_ID_BASE + 0x1001);
const unsigned MGW_BUTTON_MCCANVAS (IC_ID_BASE + 0x1002);
const unsigned MGW_ICON_SETCANVAS  (IC_ID_BASE + 0x1003);
const unsigned MGW_ICON            (IC_ID_BASE + 0x1004);
const unsigned MGW_MESSAGE_TEXT    (IC_ID_BASE + 0x1005);
const unsigned MGW_MESSAGE_ID      (IC_ID_BASE + 0x1006);

/*------------------------------------------------------------------------------
| IMessageWindow::IMessageWindow                                               |
------------------------------------------------------------------------------*/
IMessageWindow::IMessageWindow (unsigned long id,
                                IWindow* owner,
                                EMessageType type,
                                const Style& style)
  : IFrameWindow (IFrameWindow::deferCreation, id),
    fMessageWindowData (new IMessageWindowData(this))
{
   fMessageWindowData->id = id;
   fMessageWindowData->owner = owner;
   fMessageWindowData->type = type;
   fMessageWindowData->style = style;
   if (!(style & deferClientCreation))
   {
      // Initialize does all the work.
      fMessageWindowData->initialize();
   }
}

/*------------------------------------------------------------------------------
| IMessageWindow::~IMessageWindow                                              |
------------------------------------------------------------------------------*/
IMessageWindow::~IMessageWindow ()
{
  delete fMessageWindowData;
}


/*------------------------------------------------------------------------------
| IMessageWindow::messageType                                                  |
------------------------------------------------------------------------------*/
IMessageWindow::EMessageType IMessageWindow::messageType () const
{
  return fMessageWindowData->type;
}

/*------------------------------------------------------------------------------
| IMessageWindow::defaultStyle                                                 |
------------------------------------------------------------------------------*/
IMessageWindow::Style IMessageWindow::defaultStyle ()
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setDefaultStyle                                              |
------------------------------------------------------------------------------*/
void IMessageWindow::setDefaultStyle (const IMessageWindow::Style &style)
{
  currentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageText                                               |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageText (const IText& message)
{
  fMessageWindowData->messageText = message;
  if (fMessageWindowData->message != (ITextControl *) 0)
  {
     fMessageWindowData->message->setText(fMessageWindowData->messageText);
     IStaticText* msg = dynamic_cast<IStaticText*> (fMessageWindowData->message);
     if (msg)
        msg->setLimit(fMessageWindowData->messageText.length());
     if (isVisible())
        fMessageWindowData->moveAndSizeWindow();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageText                                               |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageText (const IResourceId& message)
{
  return setMessageText (message.resourceLibrary().loadString(message));
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageText                                               |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageText (const IException& exception)
{
  return setMessageText ((IText)exception.text());
}

/*------------------------------------------------------------------------------
| IMessageWindow::messageText                                                  |
------------------------------------------------------------------------------*/
IText IMessageWindow::messageText () const
{
  return fMessageWindowData->messageText;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageId                                                 |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageId (const IText& messageId)
{
  IText tempMsg (messageId);
  if (tempMsg.length() > MGW_MESSAGEID_LIMIT)  // If the limit is exceeded, the
     tempMsg.resize (MGW_MESSAGEID_LIMIT);     // id string is truncated.
  fMessageWindowData->messageIdText = tempMsg;
  if (fMessageWindowData->messageId != (ITextControl *) 0)
  {
     fMessageWindowData->messageId->setText(fMessageWindowData->messageIdText);
     IStaticText* msgId = dynamic_cast<IStaticText*> (fMessageWindowData->messageId);
     if (msgId)
        msgId->setLimit(fMessageWindowData->messageIdText.length());
     if (isVisible())
        fMessageWindowData->moveAndSizeWindow();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageId                                                 |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageId (const IResourceId& messageId)
{
  return setMessageId (messageId.resourceLibrary().loadString(messageId));
}

/*------------------------------------------------------------------------------
| IMessageWindow::messageId                                                    |
------------------------------------------------------------------------------*/
IText IMessageWindow::messageId () const
{
  return fMessageWindowData->messageIdText;
}

/*------------------------------------------------------------------------------
| IMessageWindow::addButton                                                    |
| Add a predefined button.                                                     |
------------------------------------------------------------------------------*/
IPushButton* IMessageWindow::addButton (EButton buttonType,
                                        bool isDefaultButton)
{
  fMessageWindowData->initialize();
  if (fMessageWindowData->ourClient)
  {
     fMessageWindowData->createAButton ((unsigned long) buttonType,
                                                   isDefaultButton);
     if (isVisible())
        fMessageWindowData->moveAndSizeWindow();
     if (buttonType == kHelp)
        return fMessageWindowData->buttons[(unsigned long) 8];
     else
        return fMessageWindowData->buttons[(unsigned long) buttonType];
  }
  else
     return (IPushButton *) 0;
}

/*------------------------------------------------------------------------------
| IMessageWindow::button                                                       |
------------------------------------------------------------------------------*/
IPushButton* IMessageWindow::button (EButton buttonType)
{
  fMessageWindowData->initialize();
  if (fMessageWindowData->ourClient)
  {
     if (buttonType == IMessageWindow::kHelp)
        return fMessageWindowData->buttons[8];
     else
        return fMessageWindowData->buttons[buttonType - 1];
  }
  else
     return (IPushButton *) 0;
}

/*------------------------------------------------------------------------------
| IMessageWindow::buttonParent                                                 |
------------------------------------------------------------------------------*/
IWindow* IMessageWindow::buttonParent ()
{
  fMessageWindowData->initialize();
  return fMessageWindowData->buttonSetCanvas;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageImage                                              |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageImage (const IBitmapHandle&
                                                        bitmapHandle)
{
  fMessageWindowData->useBitmapHandle = true;
  fMessageWindowData->useImage = true;
  fMessageWindowData->bitmapHandle = bitmapHandle;
  if (fMessageWindowData->initialized() && fMessageWindowData->imageSetCanvas)
  {
     delete fMessageWindowData->messageImage;
     fMessageWindowData->messageImage = new IBitmapControl (MGW_ICON,
                                 fMessageWindowData->imageSetCanvas,
                                 fMessageWindowData->imageSetCanvas,
                                 fMessageWindowData->bitmapHandle,
                                 IRectangle(),
                                 IBitmapControl::classDefaultStyle |
                                 IBitmapControl::sizeToBitmap);

     fMessageWindowData->messageImage->setAutoDeleteObject();
     if (isVisible())
        fMessageWindowData->moveAndSizeWindow();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMessageWindow::setMessageImage                                              |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::setMessageImage (EImageType image)
{
   fMessageWindowData->useBitmapHandle = false;
   fMessageWindowData->useImage = true;
   switch (image)
   {
      case kInformationImage:
         fMessageWindowData->identifier = ISystemPointerHandle::information;
         break;
      case kQuestionImage:
         fMessageWindowData->identifier = ISystemPointerHandle::question;
         break;
      case kWarningImage:
         fMessageWindowData->identifier = ISystemPointerHandle::warning;
         break;
      case kErrorImage:
         fMessageWindowData->identifier = ISystemPointerHandle::error;
         break;
   }
   if (fMessageWindowData->initialized() && fMessageWindowData->imageSetCanvas)
   {
      delete fMessageWindowData->messageImage;
      fMessageWindowData->messageImage = new IIconControl (MGW_ICON,
                                fMessageWindowData->imageSetCanvas,
                                fMessageWindowData->imageSetCanvas,
                                fMessageWindowData->identifier,
                                IRectangle(),
                                IIconControl::classDefaultStyle |
                                IIconControl::sizeToIcon);

      fMessageWindowData->messageImage->setAutoDeleteObject();
      if (isVisible())
         fMessageWindowData->moveAndSizeWindow();
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IMessageWindowData::doBeep                                                   |
------------------------------------------------------------------------------*/
void IMessageWindowData::doBeep ()
{
   switch (type)
   {
      case IMessageWindow::kWarningMessage:
#ifdef IC_PM
         WinAlarm (IWindow::desktopWindow()->handle(), WA_WARNING);
#endif // IC_PM
#ifdef IC_WIN
         MessageBeep (MB_WARNING);
#endif // IC_WIN
#ifdef IC_MOTIF
         XBell (XtDisplay(IWindow::desktopWindow()->handle()), 30);
#endif // IC_MOTIF
         break;
      case IMessageWindow::kConfirmationMessage:
      case IMessageWindow::kRetryMessage:
#ifdef IC_PM
         WinAlarm (IWindow::desktopWindow()->handle(), WA_NOTE);
#endif // IC_PM
#ifdef IC_WIN
         MessageBeep (MB_ICONQUESTION);
#endif // IC_WIN
#ifdef IC_MOTIF
         XBell (XtDisplay(IWindow::desktopWindow()->handle()), 30);
#endif // IC_MOTIF
         break;
      case IMessageWindow::kCatastrophicMessage:
#ifdef IC_PM
         WinAlarm (IWindow::desktopWindow()->handle(), WA_ERROR);
#endif // IC_PM
#ifdef IC_WIN
         MessageBeep (MB_ERROR);
#endif // IC_WIN
#ifdef IC_MOTIF
         XBell (XtDisplay(IWindow::desktopWindow()->handle()), 30);
#endif // IC_MOTIF
         break;
      // Don't beep for these.
      case IMessageWindow::kInformationMessage:
      default:
         break;
   }
}

/*------------------------------------------------------------------------------
| IMessageWindow::show                                                         |
------------------------------------------------------------------------------*/
IMessageWindow& IMessageWindow::show (bool showWindow)
{
   fMessageWindowData->initialize();
   if (showWindow)
   {
      fMessageWindowData->moveAndSizeWindow();
      fMessageWindowData->doBeep();
      this->setFocus();
   }
   Inherited::show (showWindow);
   return *this;
}

/*------------------------------------------------------------------------------
| IMessageWindow::showModally                                                  |
------------------------------------------------------------------------------*/
unsigned long IMessageWindow::showModally ()
{
   fMessageWindowData->initialize();
   fMessageWindowData->moveAndSizeWindow();
   fMessageWindowData->doBeep();
   this->setFocus();
   return Inherited::showModally ();
}

/*------------------------------------------------------------------------------
| IMessageWindowData::initialize                                               |
------------------------------------------------------------------------------*/
void IMessageWindowData::initialize ()
{
  if (!initialized())
  {
     // Setting this flag must be first, or else if they create the window
     // visible, we will get into an endless loop of IMessageWindow::initialize,
     // IFrameWindow::initialize, IWindow::show...
     isInitialized = true;
     myFrame->IFrameWindow::initialize(id,
                                      (IFrameWindow::Style&)style,
                                       IWindow::desktopWindow(),
                                       owner);
     myFrame->setDestroyOnClose();
     clientWindow = myFrame->createClient();
     if (clientWindow)
        myFrame->setClient (clientWindow);
     else
        ITHROWLIBRARYERROR(IC_CONTROL_NOT_FOUND,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
  }
}

/*------------------------------------------------------------------------------
| IMessageWindowData::isInitialized                                            |
------------------------------------------------------------------------------*/
bool IMessageWindowData::initialized ()
{
  return isInitialized;
}

/*------------------------------------------------------------------------------
| IMessageWindow::createClient                                                 |
------------------------------------------------------------------------------*/
IWindow* IMessageWindow::createClient ()
{
   fMessageWindowData->ourClient = true;
   // Create the multi-cell canvas to hold the message window components and the
   // set canvas to hold the buttons.

   fMessageWindowData->clientCanvas = new IMultiCellCanvas (MGW_MAIN_MCCANVAS,
                                        this, this);
   fMessageWindowData->clientCanvas->setAutoDeleteObject();


   // We have to have a multicell to hold the set canvas for the buttons
   // so that the buttons can be centered.

   fMessageWindowData->buttonMCCanvas = new IMultiCellCanvas (
                                            MGW_BUTTON_MCCANVAS,
                                            fMessageWindowData->clientCanvas,
                                            fMessageWindowData->clientCanvas);
   fMessageWindowData->buttonMCCanvas->setAutoDeleteObject();

   // The button set canvas is owned by an embedded multicell.
   fMessageWindowData->buttonSetCanvas = new ISetCanvas (
                                            MGW_BUTTON_SETCANVAS,
                                            fMessageWindowData->buttonMCCanvas,
                                            fMessageWindowData->buttonMCCanvas);
   fMessageWindowData->buttonSetCanvas->setAutoDeleteObject();

   fMessageWindowData->message = createMessageText (fMessageWindowData->clientCanvas);
   if (!fMessageWindowData->message)
      ITHROWLIBRARYERROR(IC_CONTROL_NOT_FOUND,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

   fMessageWindowData->messageId = createMessageIdText (fMessageWindowData->clientCanvas);
   if (!fMessageWindowData->messageId)
      ITHROWLIBRARYERROR(IC_CONTROL_NOT_FOUND,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);


   // Create the icon.  It goes in the icon set canvas.  A set canvas to
   // hold the icon makes it keep its relative position to the message
   // text when the text covers multiple lines.
   fMessageWindowData->imageSetCanvas = new ISetCanvas (
                                           MGW_ICON_SETCANVAS,
                                           fMessageWindowData->clientCanvas,
                                           fMessageWindowData->clientCanvas);
   fMessageWindowData->imageSetCanvas->setAutoDeleteObject();

   fMessageWindowData->messageImage = createMessageImage (fMessageWindowData->imageSetCanvas);
   if (!fMessageWindowData->messageImage &&
       (fMessageWindowData->type != kUserDefinedMessage))
      ITHROWLIBRARYERROR(IC_CONTROL_NOT_FOUND,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

   // Do other setup in private functions.
   fMessageWindowData->createButtons();
   fMessageWindowData->addChildrenToClient();
   return (IWindow *) fMessageWindowData->clientCanvas;
}

/*------------------------------------------------------------------------------
| IMessageWindow::createMessageImage                                           |
------------------------------------------------------------------------------*/
IBitmapControl* IMessageWindow::createMessageImage (IWindow* parentAndOwner)
{
   // If they've called setMessageImage, useImage will be true and we need to
   // create one.  Otherwise check the default message type for which image
   // to create.  May still be none if they specified kUserDefinedMessage.
   if (!fMessageWindowData->useImage)
      if (fMessageWindowData->type != kUserDefinedMessage)
      {
         switch (fMessageWindowData->type)
         {
            case kInformationMessage:
               fMessageWindowData->identifier =
                  ISystemPointerHandle::information;
               break;
            case kWarningMessage:
               fMessageWindowData->identifier =
                  ISystemPointerHandle::warning;
               break;
            case kConfirmationMessage:
               fMessageWindowData->identifier =
                  ISystemPointerHandle::question;
               break;
            case kRetryMessage:
            case kCatastrophicMessage:
               fMessageWindowData->identifier =
                  ISystemPointerHandle::error;
               break;
         }
         fMessageWindowData->useImage = true;
      }
   if (fMessageWindowData->useImage)
   {
      IBitmapControl* image;
      if (fMessageWindowData->useBitmapHandle)
         image = new IBitmapControl (MGW_ICON,
                                     parentAndOwner,
                                     parentAndOwner,
                                     fMessageWindowData->bitmapHandle,
                                     IRectangle(),
                                     IBitmapControl::classDefaultStyle |
                                     IBitmapControl::sizeToBitmap);
      else
         image = new IIconControl (MGW_ICON,
                                   parentAndOwner,
                                   parentAndOwner,
                                   fMessageWindowData->identifier,
                                   IRectangle(),
                                   IIconControl::classDefaultStyle |
                                   IIconControl::sizeToIcon);
      image->setAutoDeleteObject();
      return (IBitmapControl *) image;
   }
   else
      return (IBitmapControl *) 0;
}

/*------------------------------------------------------------------------------
| IMessageWindow::createMessageText                                            |
------------------------------------------------------------------------------*/
ITextControl* IMessageWindow::createMessageText (IWindow* parentAndOwner)
{
   // Create the message text as a static text control.  Use wordBreak style
   // so the text will reflow.  Set the text to the message text.  Use
   // setLimit to tell the control where to break.

   IStaticText* msg;
   msg = new IStaticText (MGW_MESSAGE_TEXT,
                          parentAndOwner,
                          parentAndOwner,
                          IRectangle(),
                          IStaticText::classDefaultStyle |
                          IStaticText::wordBreak);
   msg->setAutoDeleteObject();
   if (fMessageWindowData->messageText.length() != 0)
   {
      msg->setText (fMessageWindowData->messageText);
      msg->setLimit (fMessageWindowData->messageText.length());
   }

   return (ITextControl *) msg;
}

/*------------------------------------------------------------------------------
| IMessageWindow::createMessageIdText                                          |
------------------------------------------------------------------------------*/
ITextControl* IMessageWindow::createMessageIdText (IWindow* parentAndOwner)
{
   // Create the message id as a static text control.

   IStaticText* id;
   id = new IStaticText (MGW_MESSAGE_ID,
                         parentAndOwner,
                         parentAndOwner,
                         IRectangle(),
                         IStaticText::classDefaultStyle);
   id->setAutoDeleteObject();

   // The message id is intended to be short.  Twelve characters is the limit.
   id->setLimit(MGW_MESSAGEID_LIMIT);

   // Use bottomRight alignment so that the text is in the bottom right corner
   // of the message window.  Set the messageId text.

   id->setAlignment (IStaticText::bottomRight);
   if (fMessageWindowData->messageIdText.length() != 0)
      id->setText (fMessageWindowData->messageIdText);

   // Change the font to a "small" font so the message id is unobtrusive.
   // If this font is not found, it will just default to the system font, which
   // is OK.

   id->setFont (IFont ("Helvetica", 8));

   return (ITextControl *) id;
}

/*------------------------------------------------------------------------------
| IMessageWindowData::createButtons                                            |
| Create the message window buttons.                                           |
------------------------------------------------------------------------------*/
void IMessageWindowData::createButtons ()
{
  // Determine the buttons.
  // When each button is created, check its minimum size.  We want each button
  // to be at least a certain number of pels wide and of uniform size.

  switch (type)
  {
     case IMessageWindow::kInformationMessage:
     case IMessageWindow::kCatastrophicMessage:
        createAButton(IC_ID_OK, true);
        break;
     case IMessageWindow::kWarningMessage:
        createAButton(IC_ID_OK, true);
        createAButton(IC_ID_CANCEL);
        break;
     case IMessageWindow::kConfirmationMessage:
     case IMessageWindow::kRetryMessage:
        createAButton(IC_ID_RETRY, true);
        createAButton(IC_ID_CANCEL);
        break;
     // These don't have buttons.
     case IMessageWindow::kUserDefinedMessage:
     default:
        break;
  }

  return;
}
/*------------------------------------------------------------------------------
| IMessageWindowData::createAButton                                            |
| Create a button with a specific id.                                          |
------------------------------------------------------------------------------*/
void IMessageWindowData::createAButton (unsigned long buttonId,
                                        bool isDefault)
{
   // Use the button id as an index into the array of predefined buttons
   // unless help is used.  The id for that is not consistent (see icconst.h).
   unsigned long index = (buttonId != IC_ID_HELP) ? (buttonId-1) : 8;

   // There may have been an old button there already.  Delete it.
   // If they add one with the same ID, it will replace the old one.
   delete buttons[index];

   buttons[index] =
      new IPushButton (buttonId,
                       buttonSetCanvas,
                       buttonSetCanvas);

   IString buttonText;
   // Query the button text from the resource file and set it on the button.
   // If load fails for some reason, set the text to a default.
   try
   {
      buttonText = IApplication::current().resourceLibrary().
                                        loadString(buttonId);
   }
   catch (...)
   {
      switch (buttonId)
      {
         case IMessageWindow::kEnter:
            buttonText = "Enter";
         break;

         case IMessageWindow::kOk:
            buttonText = "OK";
         break;

         case IMessageWindow::kCancel:
            buttonText = "Cancel";
         break;

         case IMessageWindow::kAbort:
            buttonText = "Abort";
         break;

         case IMessageWindow::kRetry:
            buttonText = "Retry";
         break;

         case IMessageWindow::kIgnore:
            buttonText = "Ignore";
         break;

         case IMessageWindow::kYes:
            buttonText = "Yes";
         break;

         case IMessageWindow::kNo:
            buttonText = "No";
         break;

         case IMessageWindow::kHelp:
            buttonText = "Help";
         break;

         default:
            buttonText = "OK";
         break;

      }
   }

   buttons[index]->setText(buttonText);

   // If this is a help button, set the appropriate styles.
   if (buttonId == IC_ID_HELP)
   {
      buttons[index]->enableHelp();
      buttons[index]->disableMouseClickFocus();
   }

   // If this is the default button, set the appropriate styles.
   if (isDefault)
   {
      buttons[index]->enableDefault();
      buttons[index]->setFocus();
   }

   buttons[index]->setAutoDeleteObject();
   return;
}

/*------------------------------------------------------------------------------
| IMessageWindowData::addChildrenToClient                                      |
| Add all the child controls to the multi-cell canvas and do layout.           |
------------------------------------------------------------------------------*/
void IMessageWindowData::addChildrenToClient()
{
  // Add everything to the multi-cell canvas.

  // The message text is centered on the icon, so they begin in
  // different rows.
  clientCanvas->addToCell(message, 4, 3, 2, 1);
  clientCanvas->addToCell(imageSetCanvas, 2, 2, 1, 3);
  clientCanvas->setRowHeight (2, 0, true);

  // Set the horizontal space between the icon and text.
  clientCanvas->setColumnWidth (3,
                                3*myFrame->font().avgCharWidth(),
                                false);

  // Make the column the text starts in and the one it traverses expandable.
  clientCanvas->setColumnWidth (4, 0, true);
  clientCanvas->setColumnWidth (5, 0, true);

  // Add the button canvas to the embedded multicell, and
  // then add it to the client multicell so that the buttons will be centered.
  buttonMCCanvas->addToCell(buttonSetCanvas, 2, 1);
  clientCanvas->addToCell(buttonMCCanvas, 2, 5, 5, 1);
  buttonMCCanvas->setColumnWidth(1, 0, true);
  buttonMCCanvas->setColumnWidth(3, 0, true);

  // The message id is on the row after the buttons.
  clientCanvas->addToCell(messageId, 5, 6);

  // Make column one the same size as the space between the icon and text.
  clientCanvas->setColumnWidth (1,
                                (5*myFrame->font().avgCharWidth())/3,
                                false);

  // Make the empty row between the text and the buttons expandable.
  clientCanvas->setRowHeight (4, 0, true);

  // Put some extra padding on the right and bottom edges.
  clientCanvas->setColumnWidth (7,
                                (5*myFrame->font().avgCharWidth())/3,
                                false);
  clientCanvas->setRowHeight (6,
                              myFrame->font().maxCharHeight(),
                              false);

  // Set the configuration of the button set canvas.

  buttonSetCanvas->setDeckOrientation (ISetCanvas::horizontal);
  buttonSetCanvas->setAlignment (ISetCanvas::bottomLeft);
  buttonSetCanvas->setPackType (ISetCanvas::expanded);

  // The padding between the buttons is small.
  buttonSetCanvas->setPad (ISize (6,buttonSetCanvas->pad().height()));
  buttonSetCanvas->setMargin (ISize (0,myFrame->font().maxCharHeight()/2));

  // Set the configuration of the icon set canvas.

  imageSetCanvas->setDeckOrientation (ISetCanvas::vertical);
  imageSetCanvas->setAlignment (ISetCanvas::topCenter);
  imageSetCanvas->setPackType (ISetCanvas::tight);
  imageSetCanvas->setPad (ISize (0,0));
  imageSetCanvas->setMargin (ISize (0,myFrame->font().maxCharHeight()/2));

  return;
}

/*------------------------------------------------------------------------------
| IMessageWindowData::moveAndSizeWindow                                        |
| Size and position the message window.                                        |
------------------------------------------------------------------------------*/
void IMessageWindowData::moveAndSizeWindow ()
{
  // Only do this moving and sizing if we actually have a client canvas to work
  // with.  If they overrode createClient, then skip the whole thing.
  if (ourClient)
  {
     // Set minimum size for the buttons.  We want a minimum
     // width for a uniform look.  Have to do this by enumerating
     // the children of the button set canvas because they may
     // have created custom buttons that we don't know about.

     IWindow::ChildCursor buttonCursor(*buttonSetCanvas);

     // Initial minimum button width should match the system - determined
     // through empirical means: eyeballing.
     long minButtonWidth = 10*myFrame->font().avgCharWidth();

     // Cursor through once to find the largest minimum width (i.e.,
     // the width of the largest button).  Cursor through again
     // to set all the minimum sizes.
     for (buttonCursor.setToFirst(); buttonCursor.isValid(); buttonCursor.setToNext())
     {
        // Reset the minimum button width if necessary.
        minButtonWidth = MAX (IWindow::windowWithHandle(buttonSetCanvas->childAt(buttonCursor))->
                               minimumSize().width(), minButtonWidth);
     }

     for (buttonCursor.setToFirst(); buttonCursor.isValid(); buttonCursor.setToNext())
     {
        IWindow::windowWithHandle(buttonSetCanvas->childAt(buttonCursor))->setMinimumSize(ISize (minButtonWidth,
           (IWindow::windowWithHandle(buttonSetCanvas->childAt(buttonCursor))->minimumSize().height())));
     }

     // Center the message window on the desktop.
     long desktopHeight = IWindow::desktopWindow()->size().height(),
          desktopWidth = IWindow::desktopWindow()->size().width(),
          msgWidth = clientCanvas->minimumSize().width(),
          msgHeight = clientCanvas->minimumSize().height();

     // Get the title bar.
     if (style & IFrameWindow::titleBar)
     {
        ITitle titleBar (myFrame);

        // If the title is wider than the message window minimum size, expand the
        // message window width to include the title but no wider than the screen.
        if (titleBar.displaySize().width() > msgWidth)
           msgWidth = MIN(titleBar.displaySize().width(),
                         (desktopWidth-2*myFrame->borderWidth()));

     }
     // Limit width to 5/8 of the screen also.
     msgWidth = MIN (msgWidth, (5*desktopWidth)/8);

     // We have to size and position once and then check for a few things and maybe
     // do it again.
     myFrame->moveSizeToClient (IRectangle ((desktopWidth/2 - msgWidth/2),
                                            (desktopHeight/2 - msgHeight/2),
                                            (desktopWidth/2 + msgWidth/2),
                                            (desktopHeight/2 + msgHeight/2)));

     // If the width of the message static text control turned out larger than
     // average due to a long title or a large number of buttons, recalculate the
     // number of lines needed for the message text and resize.

     long textColumnWidths,
        remainingColumnWidths;
     textColumnWidths = clientCanvas->columnWidth(4)+
                        clientCanvas->columnWidth(5);
     remainingColumnWidths = clientCanvas->columnWidth(1)+
                             clientCanvas->columnWidth(2)+
                             clientCanvas->columnWidth(3)+
                             clientCanvas->columnWidth(6)+
                             clientCanvas->columnWidth(7);

     if (textColumnWidths > (msgWidth - remainingColumnWidths))
     {

        textColumnWidths = msgWidth - remainingColumnWidths;
        // Calculate the new number of lines based on existing column width.
        long newNumberOfLines = myFrame->font().textLines(messageText,
                                                 textColumnWidths);

        // We only need to recalculate and reposition if the new number of lines
        // needed is different from the old.  If the message is short  and the
        // title long, the number of lines will be the same.

        if ((newNumberOfLines*message->font().maxCharHeight()) >
             message->minimumSize().height())
        {

           // Force a recalculation of the size.  Setting the limit causes a call
           // to setLayoutDistorted.
           long newLimit = textColumnWidths/myFrame->font().avgCharWidth();
           dynamic_cast<IStaticText*>(message)->setLimit (newLimit);

           // Recalculate and resize the message window.
           // Make sure message window is no more than 2/3 the height of the screen.
           msgHeight = MIN(clientCanvas->minimumSize().height(),
                              desktopHeight);

           myFrame->moveSizeToClient (IRectangle ((desktopWidth/2 - msgWidth/2),
                                                (desktopHeight/2 - msgHeight/2),
                     (desktopWidth/2 + msgWidth/2 + (2*myFrame->borderWidth())),
                                              (desktopHeight/2 + msgHeight/2)));
        }
     }
  }
  return;
}
