// Revision: 22 1.6.1.2 source/ui/baseapp/imstring.cpp, basewin, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imstring.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imstring.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ilanglvl.hpp>

extern "C"
  {
  #include <Xm/BulletinB.h>
  }

#include <imstring.hpp>

#include <itrace.hpp>
#include <icconst.h>
#include <iexcept.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IMSTRING_CPP_
    #include <ipagetun.h>
#endif

/*-------------------------------------------------------------------------
                       S T A T I C   O B J E C T S
--------------------------------------------------------------------------*/
XmStringCharSet
    IMString :: dfltStringCharSet = XmFONTLIST_DEFAULT_TAG;

XmStringDirection
    IMString :: dfltStringDirection = XmSTRING_DIRECTION_L_TO_R;

bool
    IMString :: dfltAddSeparator = False;

/*------------------------ Compound String Support -------------------------

--------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
| IMString::defaultCharSet                                                     |
------------------------------------------------------------------------------*/
XmStringCharSet IMString :: defaultCharSet( )
    {
    return dfltStringCharSet;
    }

/*------------------------------------------------------------------------------
| IMString::setDefaultCharSet                                                  |
------------------------------------------------------------------------------*/
IMString &IMString :: setDefaultCharSet ( XmStringCharSet charSet )
    {
    dfltStringCharSet = charSet;
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString::defaultStringDirection                                             |
------------------------------------------------------------------------------*/
XmStringDirection IMString :: defaultStringDirection( )
    {
    return dfltStringDirection;
    }

/*------------------------------------------------------------------------------
| IMString::setDefaultStringDirection                                          |
------------------------------------------------------------------------------*/
IMString &IMString :: setDefaultStringDirection(
                          XmStringDirection stringDirection )
    {
    dfltStringDirection = stringDirection;
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString :: defaultAddSeparator                                              |
------------------------------------------------------------------------------*/
bool IMString :: defaultAddSeparator( )
    {
    return dfltAddSeparator;
    }

/*------------------------------------------------------------------------------
| IMString::setDefaultAddSeparator                                             |
------------------------------------------------------------------------------*/
IMString &IMString :: setDefaultAddSeparator( bool addSeparator)
    {
    dfltAddSeparator = addSeparator;
    return *this;
    }


/*------------------------------------------------------------------------------
| IMString::IMString                                                           |
------------------------------------------------------------------------------*/
IMString::IMString( const IString& aString,
                    XmStringCharSet charSet,
                    XmStringDirection stringDirection,
                    bool addSeparator)
{
 //
 // It is safe to scan for \n in a multibyte string, since it
 // is in the range x'00' to x'3f'.
 //
 char* restOfString = (char*)aString;
 unsigned long segmentLength = strcspn( restOfString, "\n" )+1;
 if ( restOfString[segmentLength-1] ) // Found a new line.
  {
   // build up string with a separator for each new line
   char* segment = new char[segmentLength];
   strncpy(segment, restOfString, segmentLength);
   segment[segmentLength-1] = '\0';
   XmString localString = XmStringSegmentCreate( segment, charSet,
     stringDirection, true );
   delete [] segment;
   restOfString += segmentLength;
   while ( *restOfString )
    {
     XmString restString;
     unsigned long segmentLength = strcspn( restOfString, "\n" )+1;
     if ( restOfString[segmentLength-1] )  // Found a new line.
      {
       char* segment = new char[segmentLength];
       strncpy(segment, restOfString, segmentLength);
       segment[segmentLength-1] = '\0';
       restString = XmStringSegmentCreate( segment, charSet,
         stringDirection, true );
       delete [] segment;
       restOfString += segmentLength;
      }
     else // No more new lines.
      {
       restString = XmStringSegmentCreate( restOfString, charSet,
         stringDirection, addSeparator );
       restOfString += segmentLength-1; // Point to the null terminator.
      }
     XmString tempString = XmStringConcat( localString, restString );
     XmStringFree( localString );
     XmStringFree( restString );
     localString = XmStringCopy( tempString );
     XmStringFree( tempString );
    }
   motifString = XmStringCopy( localString );
   XmStringFree( localString );
  }
 else
   motifString = XmStringSegmentCreate( restOfString, charSet, stringDirection,
     addSeparator);
}

//@@ The above re-implements this.  IString seems to go into an infinite loop.
//@@ We can do it properly mbcs enabled without using IString...  Rob.
//@@
//@@  {
//@@  if ( aString.indexOf('\n') != 0 )
//@@    {
//@@    // build up string with a separator for each new line
//@@    IString segment = aString.subString(1, aString.indexOf('\n') - 1 );
//@@    XmString localString = XmStringSegmentCreate( segment, charSet,
//@@      stringDirection, true );
//@@    IString restOfString = aString.subString( aString.indexOf('\n') + 1,
//@@      aString.length() - aString.indexOf('\n') );
//@@    bool needASeparator;
//@@    while ( restOfString.length( ) )
//@@      {
//@@      needASeparator = addSeparator;
//@@      if ( restOfString.indexOf('\n') )
//@@        {
//@@        segment = restOfString.subString(1, restOfString.indexOf('\n') - 1 );
//@@        restOfString = restOfString.subString(
//@@          restOfString.indexOf('\n') + 1,
//@@          restOfString.length() - restOfString.indexOf('\n') );
//@@        needASeparator = true;
//@@        }
//@@      else
//@@        {
//@@        segment = restOfString;
//@@        restOfString = "";
//@@        }
//@@      XmString restString = XmStringSegmentCreate( segment, charSet,
//@@        stringDirection, needASeparator);
//@@      XmString tempString = XmStringConcat( localString, restString );
//@@      XmStringFree( localString );
//@@      XmStringFree( restString );
//@@      localString = XmStringCopy( tempString );
//@@      XmStringFree( tempString );
//@@      }
//@@    motifString = XmStringCopy( localString );
//@@    XmStringFree( localString );
//@@    }
//@@  else
//@@    motifString = XmStringSegmentCreate( aString, charSet, stringDirection,
//@@      addSeparator);
//@@
//@@//  if (motifString == 0)
//@@//      IOutOfMemory("XmStringCreate new failed", IC_HEAP_EXHAUSTED,
//@@//          IException::unrecoverable);
//@@//      THROWLIBRARYERROR( IC_HEAP_EXHAUSTED, IBaseErrorInfo::outOfMemory,
//@@//                         IException::unrecoverable );
//@@  }

/*------------------------------------------------------------------------------
| IMString::IMString                                                           |
------------------------------------------------------------------------------*/
IMString :: IMString ( )
    {
    motifString = XmStringSegmentCreate( "", defaultCharSet(),
        defaultStringDirection(), defaultAddSeparator() );
//    if (motifString == 0)
//        ITHROWLIBRARYERROR( IC_HEAP_EXHAUSTED, IBaseErrorInfo::outOfMemory,
//                            IException::unrecoverable );
    }

/*------------------------------------------------------------------------------
| IMString::IMString                                                           |
------------------------------------------------------------------------------*/
IMString::IMString( const IMString& mString)
    {
    motifString = XmStringCopy(mString.motifString);
    }

/*------------------------------------------------------------------------------
| IMString::IMString                                                           |
------------------------------------------------------------------------------*/
IMString::IMString( XmString string )
    {
    motifString = string;
    }

/*------------------------------------------------------------------------------
| IMString::IMString                                                           |
------------------------------------------------------------------------------*/
IMString::IMString( const IWindowHandle& widget, const char * resourceName)
{
    XtVaGetValues(
      (Widget)widget,
      resourceName,  &motifString,
      NULL);
   // if its null create a null XmString
   if (motifString == 0)
      {
      motifString = XmStringSegmentCreate( "", defaultCharSet(),
                       defaultStringDirection(), defaultAddSeparator() );
      }
}

/*------------------------------------------------------------------------------
| IMString::~IMString                                                          |
------------------------------------------------------------------------------*/
IMString::~IMString()
    {
    XmStringFree(motifString);
    }

/*------------------------------------------------------------------------------
| IMString::lineCount                                                          |
------------------------------------------------------------------------------*/
int IMString::lineCount()
  {
  return XmStringLineCount( motifString );
  }

/*------------------------------------------------------------------------------
| IMString::size                                                               |
------------------------------------------------------------------------------*/
ISize IMString::size(const IWindowHandle &whnd)
  {
  Dimension width = 0, height = 0;
  if ( !XmStringEmpty( motifString ) )
     {
     XmFontList fontList;

     if ( XtIsSubclass( whnd, xmBulletinBoardWidgetClass ) )
       XtVaGetValues( whnd,
                      XmNtextFontList, &fontList,
                      NULL );
     else
       XtVaGetValues( whnd,
                      XmNfontList, &fontList,
                      NULL);
     XmStringExtent( fontList, motifString, &width, &height );
     }
  return ISize(width, height);
  }

/*------------------------------------------------------------------------------
| operator ==                                                                  |
------------------------------------------------------------------------------*/
bool operator == ( const IMString &mString1, const IMString &mString2 )
    {
    return XmStringCompare(mString1, mString2);
    }

/*------------------------------------------------------------------------------
| operator ==                                                                  |
------------------------------------------------------------------------------*/
bool operator == ( const IMString &mString, const IString &aString)
    {
    return (aString == (IString) mString);
    }

/*------------------------------------------------------------------------------
|  operator ==                                                                 |
------------------------------------------------------------------------------*/
bool operator == ( const IMString &mString, const char * pString)
    {
    return ( (IString) mString == pString);
    }

/*------------------------------------------------------------------------------
|  operator ==                                                                 |
------------------------------------------------------------------------------*/
bool operator == ( const IString &aString, const IMString &mString)
    {
    return (aString == (IString) mString);
    }

/*------------------------------------------------------------------------------
|  operator ==                                                                 |
------------------------------------------------------------------------------*/
bool operator == ( const char * pString, const IMString &mString)
    {
    return (pString == (IString) mString);
    }


/*------------------------------------------------------------------------------
|  operator !=                                                                 |
------------------------------------------------------------------------------*/
bool operator != ( const IMString &mString1, const IMString &mString2 )
    {
    return (!(mString1 == mString2));
    }

/*------------------------------------------------------------------------------
|  operator !=                                                                 |
------------------------------------------------------------------------------*/
bool operator != ( const IMString &mString, const IString &aString)
    {
    return (aString != (IString) mString);
    }

/*------------------------------------------------------------------------------
|  operator !=                                                                 |
------------------------------------------------------------------------------*/
bool operator != ( const IMString &mString, const char * pString)
    {
    return ( (IString) mString != pString);
    }

/*------------------------------------------------------------------------------
| operator !=                                                                  |
------------------------------------------------------------------------------*/
bool operator != ( const IString &aString, const IMString &mString)
    {
    return (aString != ( IString ) mString);
    }

/*------------------------------------------------------------------------------
|  operator !=                                                                 |
------------------------------------------------------------------------------*/
bool operator != ( const char * pString, const IMString &mString)
    {
    return (pString != ( IString ) mString);
    }


/*------------------------------------------------------------------------------
| IMString::operator const XmString                                            |
------------------------------------------------------------------------------*/
IMString:: operator const XmString () const
    {
    return motifString;
    }

/*------------------------------------------------------------------------------
| IMString::operator const XtArgVal                                            |
------------------------------------------------------------------------------*/
IMString :: operator const XtArgVal ( ) const
    {
    return (XtArgVal) motifString;
    }

/*------------------------------------------------------------------------------
| IMString::operator IString                                                   |
------------------------------------------------------------------------------*/
IMString :: operator IString ( ) const
    {
    return asString();
    }


/*------------------------------------------------------------------------------
| IMString::asString                                                           |
|   static function to convert XmString to IString                             |
------------------------------------------------------------------------------*/
IString IMString::asString( const XmString& xmstring )
{
    // If the XmString is null return empty IString.  Otherwise
    // do the conversion.  Need to check this since this is a static fn
    if (xmstring == 0)
       {
       return IString();
       }
    else
       {
       XmStringContext context;
       IString convertedText;
       char *text;
       XmStringCharSet charset;
       XmStringDirection direction;
       ::Boolean separator;

       if ( XmStringInitContext(&context, xmstring) )
         {
         while ( XmStringGetNextSegment(context, &text, &charset, &direction,
                  &separator) )
           {
           convertedText += IString(text);
           if (separator == True)
             convertedText += IString('\n');
           XtFree(text);
           // d7879
           // It seems we need to free this too, although it is not documented.
           XtFree(charset);
           }
         XmStringFreeContext(context);
         }
       else
         {
         ITHROWLIBRARYERROR( IC_XMSTRING_INIT_FAILURE,
                             IBaseErrorInfo::accessError,
                             IException::recoverable );
         }
       return convertedText;
      }
}

/*------------------------------------------------------------------------------
| IMString::asString                                                           |
------------------------------------------------------------------------------*/
IString IMString :: asString () const
    {
    return IMString::asString( motifString );
    }

/*------------------------------------------------------------------------------
| IMString::asDebugInfo                                                        |
------------------------------------------------------------------------------*/
IString IMString :: asDebugInfo () const
    {
    return asString();  //Need to add direction, separator, and charset to this
    }

/*------------------------------------------------------------------------------
| IMString::operator =                                                         |
------------------------------------------------------------------------------*/
IMString& IMString :: operator = ( const IMString &mString)
    {
    if ( *this == mString)
        return *this;
    XmStringFree( motifString );
    motifString = XmStringCopy(mString.motifString);
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString::operator +=                                                        |
------------------------------------------------------------------------------*/
IMString& IMString :: operator += ( const IMString &mString )
    {
    XmString tempMotifString = XmStringConcat(motifString, mString.motifString);
    XmStringFree( motifString );
    motifString = tempMotifString;
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString::operator +                                                         |
------------------------------------------------------------------------------*/
IMString IMString :: operator + ( const IMString &mString ) const
    {
    IMString temp(*this);
    return (temp += mString);
    }

/*------------------------------------------------------------------------------
| IMString::operator +=                                                        |
------------------------------------------------------------------------------*/
IMString& IMString :: operator += ( const IString &aString )
    {
    IMString *nonConstThis = (IMString *) this;
    IMString temp(aString);
    *nonConstThis += temp;
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString::operator +                                                         |
------------------------------------------------------------------------------*/
IMString IMString :: operator + ( const IString &aString ) const
    {
    IMString temp(*this);
    return ( temp += aString);
    }

/*------------------------------------------------------------------------------
| IMString::operator +=                                                        |
------------------------------------------------------------------------------*/
IMString& IMString :: operator += ( const char *pString )
    {
    IMString *nonConstThis = (IMString *) this;
    IMString temp(pString);
    *nonConstThis += temp;
    return *this;
    }

/*------------------------------------------------------------------------------
| IMString::operator +                                                         |
------------------------------------------------------------------------------*/
IMString IMString :: operator + ( const char *pString ) const
    {
    IMString temp(*this);
    return ( temp += pString );
    }

/*------------------------------------------------------------------------------
| +                                                                            |
------------------------------------------------------------------------------*/
IMString operator + (const IString &aString,
                     const IMString &mString)
    {
    IMString temp(aString);
    return ( temp += mString );
    }

/*------------------------------------------------------------------------------
| +                                                                            |
------------------------------------------------------------------------------*/
IMString operator + ( const char *pString,
                      const IMString &mString)
    {
    IMString temp(pString);
    return ( temp += mString);
    }
