// Revision: 28 1.15.1.2 source/ui/baseapp/iobjwin.cpp, frame, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iobjwin.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Definition of the class(es):                                               *
*     IObjectWindow -  Invisible Window Class                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define INCL_WINWINDOWMGR
  #define INCL_WINERRORS
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include <Xm/Label.h>
  #endif
}

#include <icconst.h>
#include <iobjwin.hpp>
#ifdef IC_MOTIF
    #include <iarglist.hpp>
#endif
#include <iexcept.hpp>
#include <iguilock.hpp>
#include <ihandler.hpp>
#include <irect.hpp>
#ifdef IC_WIN
  #include <ireslib.hpp>
#endif
#include <ithread.hpp>
#include <itrace.hpp>
#include <iwinlsts.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IOBJWIN_CPP_
  #include <ipagetun.h>
#endif

// Move this to iwindefs.h
#define IOS_BOUNDTOMESSAGEQUEUE 0x00010000ul


/*------------------------------------------------------------------------------
| Public entry field styles.                                                   |
------------------------------------------------------------------------------*/
const IObjectWindow::Style
  IObjectWindow::boundToMessageQueue( 0, IOS_BOUNDTOMESSAGEQUEUE),
  IObjectWindow::noStyle           = 0,
  IObjectWindow::classDefaultStyle ( 0, IOS_BOUNDTOMESSAGEQUEUE );
/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
IObjectWindow::Style
 IObjectWindow::fCurrentDefaultStyle ( 0, IOS_BOUNDTOMESSAGEQUEUE );


/*------------------------------------------------------------------------------
| IObjectWindowCloseHandler                                                    |
|                                                                              |
| Close the object window upon receiving an IC_UM_CLOSE message.               |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IObjectWindowCloseHandler : public IHandler {
typedef IHandler
  Inherited;
public:
  IObjectWindowCloseHandler ( );
virtual
 ~IObjectWindowCloseHandler ( );
virtual IObjectWindowCloseHandler
 &handleEventsFor       ( IObjectWindow* objectWindow ),
 &stopHandlingEventsFor ( IObjectWindow* objectWindow );
protected:
virtual bool
  dispatchHandlerEvent ( IEvent& event );
private:
virtual IHandler
 &handleEventsFor       ( IWindow* window ),
 &stopHandlingEventsFor ( IWindow* window );
}; // IObjectWindowCloseHandler

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IObjectWindowData                                                            |
|                                                                              |
| Private data for IObjectWindow.                                              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IObjectWindowData {
public:
 IObjectWindowData ( ) : fCloseHandler() { };
 ~IObjectWindowData ( ) { };

IObjectWindowCloseHandler
  fCloseHandler;
}; // IObjectWindowData

#pragma pack(pop)
#pragma enum(pop)


/*************************************************************************/
/* IObjectWindow :: IObjectWindow                                        */
/*  Constructor for a minimal object window                              */
/*************************************************************************/
IObjectWindow :: IObjectWindow( const IObjectWindow::Style& style )
  : fObjectWindowData( new IObjectWindowData )
{
#ifdef IC_PMWIN

  static char* pszObjClass=0;

  IThread::current().initializeGUI();    // insure the GUI is started

  if (pszObjClass==0)
      {
      // Serialize access and verify that window class is not registered.
      // NOTE:  Do not hold this lock across operations that would cause
      // windows to be added or deleted, because it is the same key that
      // IWindow uses.
      IGUIResourceLock resLock( IWindowThreadData::libraryKey() );
      if (pszObjClass==0)
          {
          pszObjClass="ICLWC_OBJECT";
#ifdef IC_WIN
          HINSTANCE  hinst = GetModuleHandle(0);
          WNDCLASS   wndclass;
          // Check for it already registered.  Need this in some static
          // linking scenarios.
          bool    bSuc = GetClassInfo( hinst, pszObjClass, &wndclass);
          if (!bSuc)
             {
             // have to register it
             wndclass.style       = CS_NOCLOSE;
             wndclass.lpfnWndProc = (WNDPROC)IDEFWINDOWPROC;
             wndclass.cbClsExtra  = 0;
             wndclass.cbWndExtra  = 0;
             wndclass.hInstance   = hinst;
             wndclass.hIcon       = LoadIcon( (HINSTANCE)NULL, IDI_APPLICATION );
             wndclass.hCursor     = LoadCursor( (HINSTANCE)NULL, IDC_ARROW );
             wndclass.hbrBackground = (HBRUSH)(COLOR_BACKGROUND+1);
             wndclass.lpszMenuName  = NULL;
             wndclass.lpszClassName = pszObjClass;
             if (RegisterClass(&wndclass))
                bSuc = true;
             }
#else
          bool bSuc=WinRegisterClass((HAB)0,
                                        pszObjClass,
                                        WinDefWindowProc,
                                        0,    // no class style
                                        4);   // need QWL_USER only.
#endif
          if (!bSuc)
              ITHROWGUIERROR("WinRegisterClass");
          }
      }

  IWindowHandle whObjWin =
      this -> create( 0,
                      0,
#ifdef IC_WIN
                      WS_CHILD,
#else
                      0,
#endif
                      pszObjClass,
                      IWindow::objectWindow()->handle(),
                      0,
                      IRectangle(),
                      0,
                      0,
                      IWindow::onTopOfSiblings );


  startHandlingEventsFor(whObjWin);

#endif //IC_PMWIN

#ifdef IC_MOTIF
  IArgList args;
  args.setResource( XmNx, 0 );
  args.setResource( XmNy, 0 );
  args.setResource( XmNwidth, 1 );
  args.setResource( XmNheight, 1 );
  args.setResource( XmNmappedWhenManaged, False );
  IWindowHandle whObjWin = XtCreateManagedWidget(
                               "IObjectWindow",
                               xmLabelWidgetClass,
                               IWindow::objectWindow()->handle(),
                               (ArgList)args.argList(),
                               args.numberOfArgs());

  startHandlingEventsFor(whObjWin);

  // We can't use the object window unless its been managed and realized.
  // The widget is created managed, but is not realized until the appShell
  // is realized. Since object windows can be used without a frame window and
  // object windows are not shown, we need to realize the appShell now.
  if ( !XtIsRealized( IWindow::desktopWindow()->handle() ) )
    XtRealizeWidget( IWindow::desktopWindow()->handle() );

#endif //IC_MOTIF

  fObjectWindowData->fCloseHandler.handleEventsFor( this );
  // bind the message queue if style is specified.
  if ( style & IObjectWindow::boundToMessageQueue )
  {
    bindMessageQueue();
  }
}

/*************************************************************************/
/* IObjectWindow :: ~IObjectWindow                                       */
/*  Destructor for a minimal object window                               */
/*************************************************************************/
IObjectWindow :: ~IObjectWindow ( )
{
   fObjectWindowData->fCloseHandler.stopHandlingEventsFor( this );
   delete fObjectWindowData;
}

/*************************************************************************/
/* IObjectWindow :: close                                                */
/*  Easy way to close an object window.                                  */
/*************************************************************************/
IObjectWindow& IObjectWindow::close ( )
{
  this->postEvent( IC_UM_CLOSE );
  return *this;
}


/*------------------------------------------------------------------------------
| IObjectWindow::defaultStyle                                                    |
------------------------------------------------------------------------------*/
IObjectWindow::Style  IObjectWindow :: defaultStyle()
{
  return IObjectWindow::fCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IFrameWindow::setDefaultStyle                                                |
|                                                                              |
| Set the current default static member to the argument style.                 |
------------------------------------------------------------------------------*/
void IObjectWindow::setDefaultStyle ( const Style &style )
{
  IObjectWindow::fCurrentDefaultStyle = style;
}

/*************************************************************************/
/* IObjectWindowCloseHandler :: IObjectWindowCloseHandler                */
/*  Default constructor for page tuning.                                 */
/*************************************************************************/
IObjectWindowCloseHandler::IObjectWindowCloseHandler ( )
  : IObjectWindowCloseHandler::Inherited ( )
{ }

/*************************************************************************/
/* IObjectWindowCloseHandler :: ~IObjectWindowCloseHandler               */
/*  Empty destructor for page tuning.                                    */
/*************************************************************************/
IObjectWindowCloseHandler::~IObjectWindowCloseHandler ( )
{ }

/*************************************************************************/
/* IObjectWindowCloseHandler :: dispatchHandlerEvent                     */
/*  Handle request to destroy an object window.                          */
/*************************************************************************/
bool IObjectWindowCloseHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool stopProcessingEvent = false;
  if ( event.eventId() == IC_UM_CLOSE )
  {
#ifdef IC_PMWIN
     IDESTROYWINDOW( event.window()->handle() );
#endif
#ifdef IC_MOTIF
     XtDestroyWidget( event.window()->handle() );
#endif
     stopProcessingEvent = true;
  }
  return stopProcessingEvent;
}

/*************************************************************************/
/* IObjectWindowCloseHandler :: handleEventsFor                          */
/*  Attach handler to an object window.                                  */
/*************************************************************************/
IObjectWindowCloseHandler
 &IObjectWindowCloseHandler::handleEventsFor ( IObjectWindow* objectWindow )
{
  IASSERTPARM( objectWindow != 0 );
  Inherited::handleEventsFor( objectWindow );
  return *this;
}

/*************************************************************************/
/* IObjectWindowCloseHandler :: stopHandlingEventsFor                    */
/*  Detach handler from an object window.                                */
/*************************************************************************/
IObjectWindowCloseHandler
 &IObjectWindowCloseHandler::stopHandlingEventsFor ( IObjectWindow* objectWindow )
{
  IASSERTPARM( objectWindow != 0 );
  Inherited::stopHandlingEventsFor( objectWindow );
  return *this;
}

/*************************************************************************/
/* IObjectWindowCloseHandler :: handleEventsFor                          */
/*  Hide general version in IHandler.                                    */
/*************************************************************************/
IHandler& IObjectWindowCloseHandler::handleEventsFor ( IWindow* )
{
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}

/*************************************************************************/
/* IObjectWindowCloseHandler :: stopHandlingEventsFor                    */
/*  Hide general version in IHandler.                                    */
/*************************************************************************/
IHandler& IObjectWindowCloseHandler::stopHandlingEventsFor ( IWindow* )
{
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}
