// Revision: 38 1.4.1.3 source/ui/baseapp/ipainhdr.cpp, genhdr, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: ipainhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipainhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#include <ipainhdr.hpp>
#include <ipainevt.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPAINHDR_CPP_
  #include <ipagetun.h>
#endif


/**************************************************************/
/* Default constructor for page tuning.                       */
/**************************************************************/
IPaintHandler :: IPaintHandler ( )
{ ; }


/**************************************************************/
/* Empty destructor for page tuning.                          */
/**************************************************************/
IPaintHandler :: ~IPaintHandler ( )
{ ; }


/**************************************************************/
/* dispatchHandler Event - Dispatch command functions for this handler. */
/**************************************************************/
bool IPaintHandler :: dispatchHandlerEvent(IEvent & evt)
{
  bool bRc = false;

#ifdef IC_PMWIN
  if (evt.eventId() == WM_PAINT)
  {
    IPaintEvent ptevt(evt);
    // if the rect is 0 sized then don't call paintWindow.
    if ( ptevt.rect() != IRectangle() )
    {
      // call derived class (if any) to paint
      bRc = paintWindow(ptevt);
      if (bRc)
      {
#ifdef IC_WIN
#ifdef IC_TRACE_ALL
        ITRACE_ALL(IString("IPaintHandler::dispatch (true) hwnd=")+
                   IString(evt.handle().asUnsigned()).d2x() );
        ITRACE_ALL(IString("rect=(") +
                   IString(rct.left) + IString(",") +
                   IString(rct.top) + IString(",") +
                   IString(rct.right) + IString(",") +
                   IString(rct.bottom) + IString(")") );
#endif
        PAINTSTRUCT ptst;
        BeginPaint( evt.handle(), &ptst );
        EndPaint( evt.handle(), &ptst );
#endif  // IC_WIN
#ifdef IC_PM
        ptevt.window()->validateRect( ptevt.rect());
#endif // IC_PM
        evt.setResult(ptevt.result());  // copy result
      }
    }
  } /* endif */
#endif //IC_PMWIN

#ifdef IC_MOTIF
  if (evt.eventId() == xEvent(Expose) )
  {
    XEvent *xevt = (XEvent *) evt.parameter2().asUnsignedLong();
    if (xevt->xexpose.count == 0 )
    {
      IPaintEvent ptevt(evt);
      // if the rect is 0 sized then don't call paintWindow.
      if ( ptevt.rect() != IRectangle() )
      {
        bRc = paintWindow(ptevt);    // call derived class (if any) to paint
        if (bRc)
        {
          ptevt.window()->validateRect( ptevt.rect());
          evt.setResult(ptevt.result());  // copy result 
        }
      }
    }
  }
  else if (evt.eventId() == IC_UM_PAINT )
  {
    IPaintEvent ptevt(evt);
    // if the rect is 0 sized then don't call paintWindow.
    if ( ptevt.rect() != IRectangle() )
    {
      bRc = paintWindow(ptevt);    // call derived class (if any) to paint
      if (bRc)
      {
        ptevt.window()->validateRect( ptevt.rect());
        evt.setResult(ptevt.result());  // copy result 
      }
    }
  }
#endif //IC_MOTIF

  return bRc;
}
