/*******************************************************************************
* FILE NAME: ireslib1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of IDynamicLinkLibrary and it's      *
*   related classes.                                                           *
*                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 56  1.8.1.2  source/ui/baseapp/ireslib1.cpp, resource, ioc.v400, 000302  



#include <ireslib.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <itrace.hpp>
#include <iset2.h>
#include <iexcept.hpp>
#include <ireslock.hpp>
#include <idllmod.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRESLIB1_CPP_
  #include <ipagetun.h>
#endif




/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary::IDynamicLinkLibrary(const char* resourceFileName,
                                          ESearchLocation searchLocation )
   : fDLLModule(0),
     fPrivateData(0)
{
  IFUNCTRACE_DEVELOP();
  if(searchLocation == kNLSPathSearch)
    fDLLModule = new IDLLModule(resourceFileName, IDLLModule::kNLSPathSearch);
  else
    fDLLModule = new IDLLModule(resourceFileName, IDLLModule::kOSDefaultSearch);

}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
|   Copy Constructor                                                           |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary::IDynamicLinkLibrary(const IDynamicLinkLibrary& reslib)
              : fDLLModule(new IDLLModule(*reslib.fDLLModule)),
                fPrivateData(0)
{
  IFUNCTRACE_DEVELOP();
}

#ifndef IC_MOTIF
/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary::IDynamicLinkLibrary   (const IModuleHandle& moduleHandle)
       : fDLLModule(new IDLLModule(moduleHandle)),
         fPrivateData(0)
{
  IFUNCTRACE_DEVELOP();
}
#endif

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::operator=                                               |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary::operator=(const IDynamicLinkLibrary& reslib)
{
  IFUNCTRACE_DEVELOP();

  if (this != &reslib)
  {
    fPrivateData = 0;   // Allocate if used
    IDLLModule* fDLLModuleNew = new IDLLModule(*reslib.fDLLModule);
    delete fDLLModule;
    fDLLModule = fDLLModuleNew;
  }

  return( *this);
}



/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::~IDynamicLinkLibrary                                    |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary::~IDynamicLinkLibrary()
{
  IFUNCTRACE_DEVELOP();

  delete fDLLModule;
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::open                                                    |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary::open()
{
  IFUNCTRACE_DEVELOP();

  fDLLModule->open();

  return( *this );
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::close                                                   |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary::close()
{
  IFUNCTRACE_DEVELOP();

  fDLLModule->close();

  return( *this );
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::fileName                                                |
------------------------------------------------------------------------------*/
IString IDynamicLinkLibrary::fileName() const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->fileName();
}



/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isOpen                                                  |
------------------------------------------------------------------------------*/
bool IDynamicLinkLibrary::isOpen() const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->isOpen();
}




/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::procAddress                                             |
|   Get proceudure address by name                                             |
------------------------------------------------------------------------------*/
void* IDynamicLinkLibrary::procAddress(const char* procedureName) const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->procAddress(procedureName);
}

#ifndef IC_MOTIF
/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::procAddress                                             |
|  Get procedure address from Ordinal                                          |
------------------------------------------------------------------------------*/
void* IDynamicLinkLibrary::procAddress(unsigned long procedureOrdinal) const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->procAddress(procedureOrdinal);

}
#endif

#ifndef IC_MOTIF
/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isEntryPoint32Bit                                       |
------------------------------------------------------------------------------*/
bool IDynamicLinkLibrary::isEntryPoint32Bit
                                 ( const char *procedureName ) const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->isEntryPoint32Bit(procedureName);

}
#endif

#ifndef IC_MOTIF
/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isEntryPoint32Bit                                       |
------------------------------------------------------------------------------*/
bool IDynamicLinkLibrary::isEntryPoint32Bit 
                                ( unsigned long procedureOrdinal ) const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->isEntryPoint32Bit(procedureOrdinal);
}
#endif

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::handle                                                  |
|  Return the Handle used to load Resources                                    |
------------------------------------------------------------------------------*/
IModuleHandle IDynamicLinkLibrary::handle() const
{
  IFUNCTRACE_DEVELOP();

  return fDLLModule->handle();
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::resourceAddress - Returns the address of resources in   |
|                      a shared library.                                       |
------------------------------------------------------------------------------*/
void* IDynamicLinkLibrary::resourceAddress ( ) const
{
   return fDLLModule->procAddress("__rsrc");
}
#endif
