/*******************************************************************************
* FILE NAME: ireslib3.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of Pointer functions and classes.    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 58 1.9.1.6 source/ui/baseapp/ireslib3.cpp, resource, ioc.v400, 000302 

extern "C" {
  #define INCL_WINPOINTERS
  #include <iwindefs.h>
}

#include <ireslib.hpp>
#include <iresstat.hpp>
#include <ibmpstat.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <itrace.hpp>
#include <iset2.h>
#include <ithread.hpp>
#include <iguilock.hpp>
#include <ireslock.hpp>
#include <iexcept.hpp>
#include <iplatfrm.hpp>

#ifdef IC_MOTIF
  #include <iresprv.hpp>
#endif //IC_MOTIF


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRESLIB3_CPP_
  #include <ipagetun.h>
#endif



/*------------------------------------------------------------------------------
| IResourceLibrary::loadIcon                                                   |
------------------------------------------------------------------------------*/
IPointerHandle IResourceLibrary::loadIcon(unsigned long iconId,
                                        bool cached) const
{

  IFUNCTRACE_DEVELOP();

  IPointerHandle hptr = tryToLoadIcon(iconId, cached);
  if(hptr == 0)
      ITHROWGUIERROR(IString("ILOADICON(")+
                     "IResourceId:"+IString(iconId)+")");

  return hptr;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadPointer                                                |
------------------------------------------------------------------------------*/
IPointerHandle IResourceLibrary::loadPointer(unsigned long iconId,
                                               bool cached) const
{

  IFUNCTRACE_DEVELOP();

  IPointerHandle hptr = tryToLoadPointer(iconId, cached);
  if(hptr == 0)
      ITHROWGUIERROR(IString("ILOADPOINTER(")+
                     "IResourceId:"+IString(iconId)+")");

  return hptr;
}




#ifndef IC_NOOLDRESOURCES
#ifdef IC_MOTIF
// The following function is temporary until we stop supporting resources
// loaded the old way.
IPointerHandle IBitmapStaticPtr__loadIcon(unsigned long iconId,
                                         const IModuleHandle& moduleHandle,
                                         const IString& fileName);
#endif
#endif


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadIcon                                              |
------------------------------------------------------------------------------*/
IPointerHandle IResourceLibrary::tryToLoadIcon(unsigned long iconId,
                                                 bool cached) const
{

  IFUNCTRACE_ALL();

#ifdef IC_PMWIN
  return IBitmapStaticPtr::loadIcon( iconId, handle(), fileName(), cached);
#endif //IC_PMWIN

#ifdef IC_MOTIF
  /*------------------------------------------------------------------*/
  /* Try to load the icon resource first from memory. If this fails   */
  /* load the icon resource the old way.                              */
  /*                                                                  */
  /* We also need to insure that the GUI has been initialize so that  */
  /* the graphics library can create images (the creation requires    */
  /* the display)                                                     */
  /*------------------------------------------------------------------*/
  if(!IThread::current().isGUIInitialized() )
    IThread::current().initializeGUI();
  IAixResourceImp xResource(*this);
  char * iconResource = xResource.resourceByteAddress(RT_POINTER, iconId);

  if (iconResource)
  {
    char * xpmData;
    xResource.resourceData((rtbitmap*) iconResource, &xpmData);
    return IBitmapStaticPtr::loadIcon( iconId, 
                                       fileName(),
                                       xpmData, 
                                       cached, 
                                       false);
  }
#ifndef IC_NOOLDRESOURCES
  else
    return IBitmapStaticPtr__loadIcon(iconId, handle(), fileName());
#endif // IC_NOOLDRESOURCES

#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadPointer                                           |
------------------------------------------------------------------------------*/
IPointerHandle IResourceLibrary::tryToLoadPointer(unsigned long iconId,
                                                 bool cached) const
{

  IFUNCTRACE_ALL();

#ifdef IC_PMWIN
  return IBitmapStaticPtr::loadIcon( iconId, handle(), fileName(), cached);
#endif //IC_PMWIN

#ifdef IC_MOTIF
  /*------------------------------------------------------------------*/
  /* Insure that the GUI has been initialize so that                  */
  /* the graphics library can create images (the creation requires    */
  /* the display)                                                     */
  /*------------------------------------------------------------------*/
  if(!IThread::current().isGUIInitialized() )
    IThread::current().initializeGUI();
  IAixResourceImp xResource(*this);
  char * iconResource = xResource.resourceByteAddress(RT_POINTER, iconId);

  if (iconResource)
  {
    char * xpmData;
    xResource.resourceData((rtbitmap*) iconResource, &xpmData);
    return IBitmapStaticPtr::loadIcon( iconId, 
                                       fileName(),
                                       xpmData, 
                                       cached, 
                                       true);
  }
  else
	  return IPointerHandle(0);
#endif //IC_MOTIF
}



#ifndef IC_NOOLDRESOURCES
#ifdef IC_MOTIF
// The following function is temporary until we remove resource loading the
// old way
#include <ibmpdata.hpp>
/*------------------------------------------------------------------------------
| IBitmapStaticPtr::loadBitmap                                                 |
------------------------------------------------------------------------------*/
IPointerHandle IBitmapStaticPtr__loadIcon(unsigned long iconId,
                                         const IModuleHandle& moduleHandle,
                                         const IString& fileName)
{
  IFUNCTRACE_DEVELOP();

  IPointerData* pointerData = 0;
  bool fSuccess = 0;

  /* Build a cursor to see if we have already             */
  /* loaded this Icon.  If so just update the use count   */
  bool fFound = false;
  /* Lock Resource Table */
  IResourceLock reslibLock(IBitmapStaticPtr::bitmapKey());
  IPointerSet::Cursor cursor(IBitmapStaticPtr::pointerSet());
  for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
     pointerData = IBitmapStaticPtr::pointerSet().elementAt(cursor);
     if( pointerData->id() == iconId)
     {
       ITRACE_ALL("BMP Match found");
       fFound = true;
       break;
     }
  }

  if(!fFound)
  {
    pointerData = 0;
    IString pixFileName(IResLibStaticPtr::getAppRes(iconId, "ICLPIXMAP_", false));  
    if (pixFileName != "")
    {
      // Note that we are creating the IBitmap object whether or
      // not managed is true. We have to have the IBitmap in the
      // collection if we are going to be able to dynamically resize.

      IResourceLock reslibLock(IBitmapStaticPtr::bitmapKey());

      pointerData = new IPointerData(iconId,
                             pixFileName, ISize());
      IBitmapStaticPtr::pointerSet().add(pointerData);

    }
  }  /* endif !found */

  if (pointerData)
    return IPointerHandle(pointerData->handle());
  else
    return IPointerHandle(0);
}
#endif //IC_MOTIF
#endif // IC_NOOLDRESOURCES




