// Revision: 61 1.9 source/ui/baseapp/iscrlevt.cpp, slidectls, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: iscrlevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iscrlevt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR
#define INCL_WINSCROLLBARS
extern "C" 
  {
  #include <iwindefs.h>
  
#ifdef IC_MOTIF
  #include <Xm/ScrollBar.h>
#endif //IC_MOTIF
  }

#include <iscrlevt.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <iscroll.hpp>
#include <itrace.hpp>
#include <istring.hpp>
#include <imsgtext.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISCRLEVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IScrollEvent::IScrollEvent                                                   |
------------------------------------------------------------------------------*/
IScrollEvent :: IScrollEvent ( const IEvent& event )
  : IScrollEvent::Inherited ( event ),
    ulClPrevScrollBoxPos ( 0 )
{
#ifdef IC_PMWIN
  IScrollBar * scrollWindow;

  /*****************************************************************/
  /* Try to keep scrollAmount() independent of when/if the user    */
  /* repositions the scroll box.  The case where this is still a   */
  /* problem is if the scroll box is repositioned by one handler,  */
  /* and the second handler needs to call scrollAmount(), since    */
  /* the old scroll box position has now been lost so the delta    */
  /* can't be computed.                                            */
  /*****************************************************************/
  scrollWindow = scrollBarWindow();
  if (scrollWindow)
    ulClPrevScrollBoxPos = scrollWindow->scrollBoxPosition();
#endif //IC_PMWIN
}


/*------------------------------------------------------------------------------
| IScrollEvent::~IScrollEvent                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IScrollEvent :: ~IScrollEvent ( )
{ ; }


/*------------------------------------------------------------------------------
| IScrollEvent::scrollBarWindow                                                |
|                                                                              |
| Return the scroll bar object.                                                |
------------------------------------------------------------------------------*/
IScrollBar  *IScrollEvent :: scrollBarWindow ( ) const
{
#ifdef IC_PMWIN

#ifdef IC_WIN
  HWND hwndCtl = parameter2();
#else
  HWND hwndCtl = IWINDOWFROMID(handle(), parameter1().number1());
#endif

  IWindow* pwin = 0;

  if (hwndCtl)
  {                                    //Valid handle
     pwin = IWindow::windowWithHandle(hwndCtl);
  }
  else
  {                                   //Parent was not the owner
     IWindow* owner = IWindow::windowWithHandle(handle());
     if (owner)
     {
#ifdef IC_WIN
        pwin = IWindow::windowWithOwner(IIDOF(parameter2()), owner);
#else
        pwin = IWindow::windowWithOwner(parameter1().number1(), owner);
#endif
     }
  }
  return dynamic_cast<IScrollBar*>(pwin);
#endif //IC_PMWIN

#ifdef IC_MOTIF
  if ( !XtIsSubclass( controlHandle(), xmScrollBarWidgetClass ) )
    ITHROWLIBRARYERROR1( IC_INVALID_CLASS,
      IBaseErrorInfo::invalidRequest,
      IException::recoverable,
      "Scroll Bar");
  return dynamic_cast<IScrollBar*>(IWindow::windowWithHandle( controlHandle() ));
#endif //IC_MOTIF
}


/*------------------------------------------------------------------------------
| IScrollEvent::scrollAmount                                                   |
|                                                                              |
| Return the number of units the scroll box has moved.                         |
------------------------------------------------------------------------------*/
long  IScrollEvent :: scrollAmount ( ) const
{
   long lScrolledUnits = 0;
   IScrollBar* pscrbar = scrollBarWindow();

   if (pscrbar != 0 )
   {
#ifdef IC_PMWIN

#ifdef IC_WIN
      switch (parameter1().number1())
#else
      switch (parameter2().number2())
#endif
      {
         case SB_LINEUP:               //(same as SB_LINELEFT)
            lScrolledUnits = -pscrbar->minScrollIncrement();
            break;
         case SB_LINEDOWN:             //(same as SB_LINERIGHT)
            lScrolledUnits = pscrbar->minScrollIncrement();
            break;
         case SB_PAGEUP:               //(same as SB_PAGELEFT)
            lScrolledUnits = -pscrbar->pageScrollIncrement();
            break;
         case SB_PAGEDOWN:             //(same as SB_PAGERIGHT)
            lScrolledUnits = pscrbar->pageScrollIncrement();
            break;
         case SB_SLIDERTRACK:
         case SB_SLIDERPOSITION:
            lScrolledUnits = newScrollBoxPosition() -
                                 ulClPrevScrollBoxPos;
            break;
         case SB_ENDSCROLL:
            lScrolledUnits = 0;        //All scrolling is done
            break;
         default:
         {                             //Unknown scroll type
            ITHROWLIBRARYERROR(IC_INVALIDEVENT,
                               IBaseErrorInfo::invalidRequest,
                               IException::recoverable);

            break;
         }
      }

      /*************************************************************/
      /* Check for an out-of-scroll-unit bound condition and       */
      /* adjust the position of the scroll box accordingly.        */
      /*************************************************************/
      IRange rangeScrollBox = pscrbar->scrollBoxRange();
      long lMaxScroll =
              rangeScrollBox.lowerBound() - ulClPrevScrollBoxPos;
      if (lScrolledUnits < lMaxScroll)
      {                                //Hitting minimum limit
         lScrolledUnits = lMaxScroll;
      }
      else
      {                               //Scrolling past top/right?
         lMaxScroll =
            rangeScrollBox.upperBound() - ulClPrevScrollBoxPos;
         if (lScrolledUnits > lMaxScroll)
         {                            //Hitting maximum limit
            lScrolledUnits = lMaxScroll;
         }
      }
#endif //IC_PMWIN

#ifdef IC_MOTIF
    switch(eventId()) 
      {
      case motifEvent( XmCR_DECREMENT ):
      case IC_UM_CR_DECREMENT:
         lScrolledUnits = -pscrbar->minScrollIncrement();
         break;
      case motifEvent( XmCR_INCREMENT ):
      case IC_UM_CR_INCREMENT:
         lScrolledUnits = pscrbar->minScrollIncrement();
         break;
      case motifEvent( XmCR_PAGE_DECREMENT ):   
      case IC_UM_CRPAGE_DECREMENT:
         lScrolledUnits = -pscrbar->pageScrollIncrement();
         break;
      case motifEvent( XmCR_PAGE_INCREMENT ):    
      case IC_UM_CRPAGE_INCREMENT:
         lScrolledUnits = pscrbar->pageScrollIncrement();
         break;
      case motifEvent( XmCR_DRAG ):
      case motifEvent( XmCR_VALUE_CHANGED ):
      case motifEvent( XmCR_TO_BOTTOM ):
      case motifEvent( XmCR_TO_TOP ):
         lScrolledUnits = newScrollBoxPosition() -
                             pscrbar->prevScrollBoxPosition();
         break;
      case SB_ENDSCROLL:
        lScrolledUnits = 0;
        break;
      } /* endswitch */

      /*************************************************************/
      /* Check for an out-of-scroll-unit bound condition and       */
      /* adjust the position of the scroll box accordingly.        */
      /*************************************************************/
    IRange rangeScrollBox = pscrbar->scrollBoxRange();

    long lMaxScroll =
            rangeScrollBox.lowerBound() - pscrbar->prevScrollBoxPosition();
    if (lScrolledUnits < lMaxScroll)
      {                           // hitting minimum limit
      lScrolledUnits = lMaxScroll;
      }
    else
      {                           // scrolling past top/right?
      lMaxScroll =
          rangeScrollBox.upperBound() - pscrbar->prevScrollBoxPosition();
      if (lScrolledUnits > lMaxScroll)
        {                         // hitting maximum limit
        lScrolledUnits = lMaxScroll;
        }
      } 
#endif //IC_MOTIF
   }

   return lScrolledUnits;
}


/*------------------------------------------------------------------------------
| IScrollEvent::newScrollBoxPosition                                           |
|                                                                              |
| Return the new scroll box position.  Only valid for SB_SLIDERTRACK and       |
| SB_SLIDERPOSITION.                                                           |
------------------------------------------------------------------------------*/
unsigned long  IScrollEvent :: newScrollBoxPosition ( ) const
{
#ifdef IC_PMWIN
#ifdef IC_WIN
  switch (parameter1().number1())
#else
  switch (parameter2().number2())
#endif
  {                                    //Switch on scroll bar event
     case SB_SLIDERPOSITION:
     case SB_SLIDERTRACK:              //Above are valid cases
        break;
     case SB_LINEUP:                   //(Same as SB_LINELEFT)
     case SB_LINEDOWN:                 //(Same as SB_LINERIGHT)
     case SB_PAGEUP:                   //(Same as SB_PAGELEFT)
     case SB_PAGEDOWN:                 //(Same as SB_PAGERIGHT)
     case SB_ENDSCROLL:
     default:                          //Above are invalid cases
     {
        ITHROWLIBRARYERROR(IC_INVALIDEVENT,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        break;
     }
  }

#ifdef IC_WIN
  return parameter1().number2();
#else
  return parameter2().number1();
#endif
#endif //IC_PMWIN

#ifdef IC_MOTIF
  switch(eventId())
    {
    case motifEvent( XmCR_DRAG ):
    case motifEvent( XmCR_VALUE_CHANGED ):
    case motifEvent( XmCR_TO_BOTTOM ):
    case motifEvent( XmCR_TO_TOP ):
       break;             
    // These don't need to throw exception's but the OS/2 version does so these
    // will as well to be consistent.
    case motifEvent( XmCR_INCREMENT ): 
    case motifEvent( XmCR_DECREMENT ):
    case motifEvent( XmCR_PAGE_INCREMENT ):
    case motifEvent( XmCR_PAGE_DECREMENT ):
    default:
      {
      IInvalidRequest ex(IMessageText(IC_INVALIDEVENT),
                         IC_INVALIDEVENT, IException::recoverable);
      ITHROW(ex);
      break;
      }
    } /* endswitch */
  return ((XmScrollBarCallbackStruct *) parameter1().asUnsignedLong())->value;
#endif //IC_MOTIF
}
