// Revision: 65 1.4.1.3 source/ui/baseapp/iselhdr.cpp, genhdr, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: iselhdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iselhdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINBUTTONS
#define INCL_WINENTRYFIELDS
#define INCL_WINLISTBOXES
#define INCL_WINSTDCNR
#define INCL_WINSTDVALSET
#define INCL_WINWINDOWMGR

extern "C"
  {
  #include <iwindefs.h>
  #include <stdlib.h>
  #ifdef IC_MOTIF
    #include <Xm/List.h>
    #include <Xm/ToggleB.h>
    #include <Xm/Text.h>
    #include <Xm/TextF.h>
  #endif //IC_MOTIF
  }

#ifdef IC_WIN
  #ifndef _ICLCONTAINERW_
  #include <iclcnrw.h>
  #endif
#endif
#include <iselhdr.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>
#include <iwcname.hpp>
#include <itrace.hpp>
#include <ilistbas.hpp>
#include <icconst.h>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISELHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISelectHandler::ISelectHandler                                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISelectHandler::ISelectHandler ( )
  : ISelectHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| ISelectHandler::~ISelectHandler                                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISelectHandler::~ISelectHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISelectHandler::dispatchHandlerEvent                                         |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool ISelectHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

#ifdef IC_PMWIN
  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_COMBOBOX )
        {                              // Combo-box case.
           if ( subMsg == CBN_LBSELECT )
           {
              bRc = this->selected( ctlevt );
           }
           else if ( subMsg == CBN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
#ifdef IC_WIN
           else if (subMsg == IC_UM_CBN_ENTER)
           {
              bRc = this->enter( ctlevt );
           }
#endif
        }
        else if ( className == WC_BUTTON )
        {                              // Button case.
           ITRACE_DEVELOP("we have a button");
           unsigned long flStyle = ISTYLEOF( hwndControl );
#ifdef IC_WIN
           if ( ((flStyle & BS_PRIMARYSTYLES) != BS_PUSHBUTTON) &&
                ((flStyle & BS_PRIMARYSTYLES) != BS_DEFPUSHBUTTON) )
#else
           if ( (flStyle & BS_PRIMARYSTYLES) != BS_PUSHBUTTON )
#endif
           {                           // Not a push button.
              if ( subMsg == BN_CLICKED )
              {
                 bRc = this->selected( ctlevt );
              }
              else if ( subMsg == BN_DBLCLICKED )
              {
                 bRc = this->enter( ctlevt );
              }
           }
        }
        else if ( className == WC_LISTBOX )
        {                              // List box case.
           if ( subMsg == LN_SELECT )
           {
#ifdef IC_WIN
              IBaseListBox *listBox =
                 (IBaseListBox*)(IWindow::windowWithHandle( hwndControl ));
              if (!listBox->isMultipleSelect() ||
                  (listBox->isMultipleSelect() && listBox->isExtendedSelect()))
#endif
                 bRc = this->selected( ctlevt );
           }
#ifdef IC_WIN
           else if ( subMsg == IC_UM_LN_SELECT )
           {
              bRc = this->selected( ctlevt );
           }
#endif
           else if ( subMsg == LN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
#ifdef IC_WIN
        else if ( (className == WC_CONTAINER) ||
                  (className == WC_NATIVECONTAINER))
#else
        else if ( className == WC_CONTAINER )
#endif
        {                              // Container case.
           if ( subMsg == CN_EMPHASIS )
           {                           // Check for selected emphasis.
              NOTIFYRECORDEMPHASIS* pnre =
                  (NOTIFYRECORDEMPHASIS*)(void*)ctlevt.parameter2();
              if ( pnre->fEmphasisMask & CRA_SELECTED )
              {
                 bRc = this->selected( ctlevt );
              }
           }
           else if ( subMsg == CN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
#ifndef IC_WIN
        else if ( className == WC_VALUESET )
        {                              // Value set case.
           if ( subMsg == VN_SELECT )
           {
              bRc = this->selected( ctlevt );
           }
           else if ( subMsg == VN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
#endif
        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  } /* endif WM_CONTROL */

#ifdef IC_WIN
  else if ( event.eventId() == IC_UM_LBUTTONDBLCLK )
    {
    ITRACE_DEVELOP("We have a double click-WM_LBUTTONDBLCLK");
    IWindowHandle hwndControl = event.controlHandle();

    if ( hwndControl )
      {
      IWindowClassName className( hwndControl );
      ITRACE_DEVELOP( className );
      if ( className == WC_BUTTON )
        {                              // Button case.
        ITRACE_DEVELOP("Its a button");
        unsigned long flStyle = ISTYLEOF( hwndControl ) & BS_PRIMARYSTYLES;
        if ( (flStyle != BS_PUSHBUTTON) &&
             (flStyle != BS_DEFPUSHBUTTON) &&
             (flStyle != BS_OWNERDRAW) &&       // owner draw and radio button
             (flStyle != BS_RADIOBUTTON) )      // generate BN_DOUBLECLICKED
          {
          // The control event is assumed to go to the parent of the
          // control initially.  Therefore, set event handle to the
          // parent of the control.
          IControlEvent ctlevt( IEvent( IPARENTOF(hwndControl),
                                        WM_CONTROL,
                                        IEventData( (short)IIDOF( hwndControl ),
                                                    BN_DOUBLECLICKED ) ,
                                        IEventData(0) ) );

          bRc = this->enter( ctlevt );
          if ( bRc )
            {                              // Processing done.
            event.setResult( ctlevt.result() );  // Copy event result.
            }
          }
        }
      }
      else
        ITRACE_DEVELOP( "hwnd is null");
    }
  else if ( event.eventId() == WM_CHAR &&
            (unsigned long)event.parameter1() == VK_RETURN)
    {
      IWindowHandle hwndControl = event.controlHandle();

      if ( hwndControl )
      {
         IWindowClassName className( hwndControl );
         if ( className == WC_LISTBOX )
         {
            // The control event is assumed to go to the parent of the
            // control initially.  Therefore, set event handle to the
            // parent of the control.
            IControlEvent ctlevt( IEvent( IPARENTOF(hwndControl),
                                          WM_CONTROL,
                                          IEventData(
                                             (short)IIDOF( hwndControl ),
                                             LBN_DBLCLK ) ,
                                          IEventData(0) ) );

            // Handle the case for the enter key on a listbox or combobox.
            // Windows doesn't send a notification for the enter key with
            // these controls.
            bRc = this->enter( ctlevt );
            if ( bRc )
               event.setResult( ctlevt.result() );  // Copy event result.
         }
      }
    }
  else if (event.eventId() == WM_LBUTTONUP ||
           (event.eventId() == WM_CHAR && event.parameter1() == VK_SPACE))
    {
       // Since Windows sends LBN_SELCHANGE notifications on multiple
       // select listboxes whenever the user presses the arrow keys,
       // we need to handle the key/mouse events responsible for
       // selection.  This will ensure correct behavior of ISelectHandler
       // and selectId notifications.
       IWindowHandle hwndControl = event.controlHandle();
       if (hwndControl)
       {
          IWindowClassName className( hwndControl );
          if (className == WC_LISTBOX)
          {
             IBaseListBox *listBox = (IBaseListBox*)(event.controlWindow());
             if (listBox->isMultipleSelect() && !listBox->isExtendedSelect())
             {
                IControlEvent ctlevt( IEvent( IPARENTOF( hwndControl ),
                                              WM_CONTROL,
                                           IEventData(
                                              (short)IIDOF( hwndControl ),
                                              LN_SELECT ),
                                           IEventData(0) ));
                bRc = this->selected( ctlevt );
                if (bRc)
                   event.setResult( ctlevt.result() );
             }
          }
       }
    }
#endif

#endif //IC_PMWIN

#ifdef IC_MOTIF
   IControlEvent ctlevt(event);
   Widget       widget;

   /***********************************************************/
   /* Invoke the appropriate virtual function ("callback").   */
   /***********************************************************/
     switch ( event.eventId() )
     {
        case motifEvent(XmCR_BROWSE_SELECT):
           bRc = selected( ctlevt );
           break;

        case motifEvent(XmCR_EXTENDED_SELECT):
           bRc = selected( ctlevt );
           break;

        case motifEvent(XmCR_MULTIPLE_SELECT):
           bRc = selected( ctlevt );
           break;

        case motifEvent(XmCR_SINGLE_SELECT):
           bRc = selected( ctlevt );
           break;

        case motifEvent(XmCR_DEFAULT_ACTION):
           bRc = enter( ctlevt );
           break;

        case motifEvent(XmCR_VALUE_CHANGED):
           // only handle value changed for toggle buttons. For
           // text the IEditHandler gets this one.
           widget = event.controlHandle();
           if (XtIsSubclass (widget, xmToggleButtonWidgetClass))
              {
              bRc = selected( ctlevt );
              }
           break;

        case motifEvent(XmCR_ACTIVATE):     // Text widgets (MLE and entryfield)
           bRc = enter( ctlevt );
           break;

        case motifEvent(XmCR_GAIN_PRIMARY):  // Text widgets (MLE and entryfield)
           bRc = selected( ctlevt );
           break;

     }

   // process enter for combobox
   unsigned long subMsg = ctlevt.parameter1().number2();
   if (subMsg == IC_UM_CBN_ENTER) 
   {
     bRc = enter(ctlevt);
   }

   if (bRc)
   {                          // processing done
      event.setResult(ctlevt.result());  // copy result
   }
#endif //IC_MOTIF

  return bRc;
}

/*------------------------------------------------------------------------------
| ISelectHandler::enter                                                        |
|                                                                              |
| Dummy virtual handler function.  If a button generated the event,            |
| call selected.                                                               |
------------------------------------------------------------------------------*/
bool ISelectHandler::enter ( IControlEvent& event )
{
  bool bRc = false;
#ifdef IC_PMWIN
  ITRACE_DEVELOP("dummy virtual - call selected");
  if ( IWindowClassName( event.controlHandle() ) == WC_BUTTON )
  {
     bRc = this->selected( event );
  }
#endif //IC_PMWIN
  return bRc;
}

/*------------------------------------------------------------------------------
| ISelectHandler::selected                                                     |
|                                                                              |
| Dummy virtual handler function.                                              |
------------------------------------------------------------------------------*/
bool ISelectHandler::selected ( IControlEvent& event )
{
  return false;
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ISelectHandler::control                                                                             |
|                                                                                                                 |
| Return the window handle of the control which        generated the event.           |
------------------------------------------------------------------------------*/
IWindowHandle ISelectHandler::control( const IControlEvent& evt        ) const
{
   // We can simply return the handle from the event.  In Motif, even
   // if this handler is attached to the window's parent, the child
   // must be wrappered for this IControlEvent to have been generated.
   // This assumption is valid since the callback registration is done
   // in the window's constructor (and assuming that a parent manager
   // widget doesn't define a callback of the same name).
   return evt.controlHandle();
}
#endif //IC_MOTIF
