// Revision: 71 1.6 source/ui/baseapp/isldahdr.cpp, slidectls, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: isldahdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isldahdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define IC_MULTIMEDIA
  #define INCL_CIRCULARSLIDER
  #define INCL_SW
  #define INCL_WINDIALOGS
  #define INCL_WINENTRYFIELDS
  #define INCL_WINMLE
  #define INCL_WINSTDSLIDER
  #define INCL_WINSTDSPIN
  #define INCL_WINWINDOWMGR
  #define WINVER 0x0400
  #include <iwindefs.h>
  #ifdef IC_WIN
    #include <commctrl.h>
  #endif
  #include <stdlib.h>
}

#ifdef IC_WIN
  #ifndef _ICLSLIDERW_
    #include <iclsldw.h>
  #endif
  #ifndef _ICLCSLIDERW_
    #include <iclcsldw.h>
  #endif
#endif
#include <isldahdr.hpp>
#include <ictlevt.hpp>
#include <ievent.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <iwcname.hpp>
#include <icconst.h>

#ifdef IC_MOTIF
  #ifndef _IXCONST_
    #include <ixconst.h>
  #endif
  #include <islider.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISLDAHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISliderArmHandler::ISliderArmHandler                                         |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISliderArmHandler::ISliderArmHandler ( )
  : ISliderArmHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| ISliderArmHandler::~ISliderArmHandler                                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISliderArmHandler::~ISliderArmHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISliderArmHandler::dispatchHandlerEvent                                      |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
bool ISliderArmHandler::dispatchHandlerEvent ( IEvent& event )
{
  bool bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

#ifdef IC_PMWIN
#ifdef IC_WIN
        if ( className == WC_SLIDER ||
             className == WC_NATIVESLIDER )
#else
        if ( className == WC_SLIDER )
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
        // Use a dynamic cast to verify that we have a progress indicator
        // or slider.  Since objects of these classes are our own composites,
        // we don't have a single widget class we can check.  Also, we can't
        // have a case of the handler attached to the parent/owner of an
        // unwrappered instance of these controls.
        IProgressIndicator
           *progressInd = dynamic_cast<IProgressIndicator*>(ctlevt.controlWindow());
        if (progressInd)
#endif // IC_MOTIF
        {                              // Slider case.
           if ( subMsg == SLN_CHANGE )
           {
              bRc = this->positionChanged( ctlevt );
           }
           else if ( subMsg == SLN_SLIDERTRACK )
           {
              bRc = this->moving( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_TRACKING )
           {
              bRc = this->moving( ctlevt );
           }
           if ( subMsg == CSN_CHANGED )
           {
              bRc = this->positionChanged( ctlevt );
           }
        }
#endif
     }

     if ( bRc )
     {                              // Processing done.
        event.setResult( ctlevt.result() );  // Copy event result.
     }
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| ISliderArmHandler::moving                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
bool ISliderArmHandler::moving ( IControlEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISliderArmHandler::positionChanged                                           |
|                                                                              |
------------------------------------------------------------------------------*/
bool ISliderArmHandler::positionChanged ( IControlEvent& event )
{
  return false;
}
