// Revision: 86 1.6.1.6 source/ui/baseapp/isysmenu.cpp, menu, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: isysmenu.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isysmenu.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINFRAMEMGR
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

#include <isysmenu.hpp>
#include <iframe.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <imenuprv.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISYSMENU_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_MOTIFPM
const unsigned long ISystemMenu :: idHide(SC_HIDE);
#endif
#ifdef IC_WIN
// Not really supported in Windows 32...simulate with minimize.
const unsigned long ISystemMenu :: idHide(SC_MINIMIZE);
#endif
const unsigned long ISystemMenu :: idPulldown(SC_SYSMENU);
const unsigned long ISystemMenu :: idRestore(SC_RESTORE);
const unsigned long ISystemMenu :: idMove(SC_MOVE);
const unsigned long ISystemMenu :: idSize(SC_SIZE);
const unsigned long ISystemMenu :: idMinimize(SC_MINIMIZE);
const unsigned long ISystemMenu :: idMaximize(SC_MAXIMIZE);
const unsigned long ISystemMenu :: idClose(SC_CLOSE);
const unsigned long ISystemMenu :: idWindowList(SC_TASKMANAGER);

/*------------------------------------------------------------------------------
| ISystemMenu::ISystemMenu                                                     |
------------------------------------------------------------------------------*/
ISystemMenu::ISystemMenu( IFrameWindow* owner )
                        : fSystemMenuData( 0 ), fMenuWindowNewdInCtor(false)
{
#ifdef IC_PMWIN
   IASSERTPARM(owner != 0);

#ifdef IC_WIN
   fowner = owner->handle();
   fmenuFrame = owner;
   fhMenu = GetSystemMenu( fowner, false );
   if (!fhMenu)
#else
//only called for PM 
   IWindowHandle hwndSysMenu = IWindow::handleWithParent(FID_SYSMENU,owner->handle());

   if (hwndSysMenu)
   {
      // Find existing wrapper of the underlying PM window, or create one
     IWindow* menuWindow = IWindow::windowWithHandle(hwndSysMenu);
	   if (!menuWindow)
	   {
	        menuWindow = new MenuWindow(hwndSysMenu, 0);
	        menuWindow->setAutoDestroyWindow( false );
	        menuWindow->reserveUserWindowWord( false );
     }
      const IAttributeName menuName(IMenuAttribute::menuAttrName);
      //26467: Save any existing menu attr (like when fixedSubmenuHandle is our menubar!)
      const IMenuAttribute* priorMenuAttr =
           (const IMenuAttribute*)(menuWindow->attributeWithName(menuName));
      if (priorMenuAttr)
        fMenuFromPriorMenuAttribute = priorMenuAttr->fMenu;
      //Replace existing with our menu attribute value (existing IMenuAttribute is deleted)
      IMenuAttribute menuAttr(this);
      menuWindow->addOrReplaceAttribute(menuName, menuAttr);
      
      setWindow(menuWindow);
   }
   else
#endif
   {
     ITHROWLIBRARYERROR(IC_NO_SYSTEMMENU,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   } /* endif */

#endif //IC_PMWIN
}


/*------------------------------------------------------------------------------
| ISystemMenu::~ISystemMenu                                                    |
------------------------------------------------------------------------------*/
ISystemMenu::~ISystemMenu()
{
#ifdef IC_PM
  // Remove the reference to this menu which the "menu" IWindow now holds
  //  If we don't do this we may try to delete this menu again from AutoDeleteHandler
  const IAttributeName menuName(IMenuAttribute::menuAttrName);
  window()->removeAttribute(menuName);
  if (fMenuFromPriorMenuAttribute) {
    IMenuAttribute menuAttr(fMenuFromPriorMenuAttribute);
    window()->addOrReplaceAttribute(menuName, menuAttr);
    fMenuFromPriorMenuAttribute = 0;
  }

// the deletion of the fWindow pointer will be handled in the parent destructor

#endif
}
