/*******************************************************************************
* FILE NAME: itexthdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class ITextChangeHandler                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1994, 1995                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/


extern "C" {
  #define INCL_WINMLE
  #define INCL_WINENTRYFIELDS
  #define INCL_WININPUT
  #include <iwindefs.h>
#ifdef IC_MOTIF
  #include <Xm/Xm.h>
#endif
}

#include <itexthdr.hpp>
#include <iwindow.hpp>
#include <istring.hpp>
#include <ientryfd.hpp>
#include <icmdevt.hpp>
#include <ictlevt.hpp>
#include <iwcname.hpp>
#include <iclipbrd.hpp>
#include <imle.hpp>
#include <ispinbas.hpp>
#include <ispinszh.hpp>
#include <ikeyevt.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IEDVHDR_CPP_
  #include <ipagetun.h>
#endif

ITextChangeHandler :: ITextChangeHandler ( )
  : ITextChangeHandler::Inherited ( ),
    fResendEvent(false)
/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
{ ; }


ITextChangeHandler :: ~ITextChangeHandler ( )
/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
{ ; }


bool  ITextChangeHandler :: dispatchHandlerEvent ( IEvent& evt )
/**************************************************************/
/* Dispatch command functions for this handler.               */
/**************************************************************/
{
   bool bRc = false;

   /*************************************************************/
   /* PM-specific.  The following scenario will NOT work in     */
   /* Motif since the routing of messages to the owner is an    */
   /* ICLUI function.                                           */
   /* This handler is either attached to the control generating */
   /* the WM_CONTROL message or to its owner.  If               */
   /* IControlEvent::controlWindow returns 0, this must mean we */
   /* have a non-wrappered control with a wrappered owner       */
   /* window and that the handler is attached to the owner      */
   /* window.  This case is supported only if the owner of the  */
   /* control is also the control's parent (this is in order    */
   /* for WinWindowFromID to work).                             */
   /*                                                           */
   /* ???aj: what if X routes event from non-wrappered control  */
   /* (with no callback) to parent (with callback)? since       */
   /* parent=owner is required in OS2 as well, won't this let   */
   /* Motif work as OS2 does? Check it out. ??????????          */
   /*************************************************************/

#ifdef IC_WIN
  unsigned long keyEventId = evt.eventId();

  switch ( keyEventId )
  {
    case WM_CHAR:
    case IC_UM_WM_CHAR:
    {

       IKeyboardEvent keyEvent(evt);
       if (!keyEvent.isVirtual() && !keyEvent.isUpTransition())
       {
         char keyInput;
         keyInput = (char)evt.parameter1();
         IString key(keyInput);
         IEvent newEvt(evt.window(),
                       WM_CHAR, //evt.eventId(),
                       (char*)key,
                       0);

         // if CTRL key not pressed
         if (!(IGETKEYSTATE(evt.window()->handle(),VK_CTRL) & 0x8000))
         {
           ITextChangeEvent txtchevt(newEvt);
           textChange( txtchevt );
           if ( txtchevt.isChangeAllowed() )       // if handler said to allow the change 
           {
             IString propText = txtchevt.proposedText();

             IMultiLineEdit* mle = dynamic_cast<IMultiLineEdit *>(evt.window());
             IEntryField* ef;   
             IBaseSpinButton* sb;   
             if (mle)
             {
//             mle->setText(txtchevt.resultingText());
               mle->disableUpdate();
               ISETWINDOWTEXT(mle->handle(), (PSZ)txtchevt.resultingText());
               mle->setCursorPosition(txtchevt.startPosition()+propText.length());        
               mle->enableUpdate();
               mle->handle().sendEvent(EM_SCROLLCARET);
             }
             else if ( ef = dynamic_cast<IEntryField *>(evt.window()) )
             {                
//             ef->setText(txtchevt.resultingText());
               ISETWINDOWTEXT(ef->handle(), (PSZ)txtchevt.resultingText());
               ef->setCursorPosition(txtchevt.startPosition()+propText.length());        
               ef->handle().sendEvent(EM_SCROLLCARET);
             }  
             else if ( sb = dynamic_cast<IBaseSpinButton *>(evt.window()) )
             {
               IWindowHandle efHandle = ISpinButtonStatics::buddyHandle( sb->handle() );
               ISETWINDOWTEXT(efHandle, (PSZ)txtchevt.resultingText());

               IEntryField spinEf = IEntryField(efHandle);
               spinEf.setCursorPosition(txtchevt.startPosition()+propText.length());        
             }
           }
           evt.setResult(true);
           return true;
         }
       }
    }
    break;

    case EM_PASTE:
    case IC_UM_EM_PASTE:
    {
      IClipboard clipBoard(evt.window()->handle());
      IString clipText = clipBoard.text();

      IEvent newEvt(evt.window(),
                    evt.eventId(),
                    (char*)clipText,
                    0);

      ITextChangeEvent txtchevt(newEvt);
//      txtchevt.setProposedText(clipText);
      textChange( txtchevt );
      if ( txtchevt.isChangeAllowed() )       // if handler said to allow the change 
      {
        IString propText = txtchevt.proposedText();
 
        IMultiLineEdit* mle = dynamic_cast<IMultiLineEdit *>(evt.window());
        IEntryField* ef;      
        IBaseSpinButton* sb;   
        if (mle)
        {
//          mle->setText(txtchevt.resultingText());
          ISETWINDOWTEXT(mle->handle(), (PSZ)txtchevt.resultingText());
          mle->setCursorPosition(txtchevt.startPosition()+propText.length());        

        }
        else if ( ef = dynamic_cast<IEntryField *>(evt.window()) )
        {                
//          ef->setText(txtchevt.resultingText());
          ISETWINDOWTEXT(ef->handle(), (PSZ)txtchevt.resultingText());
          ef->setCursorPosition(txtchevt.startPosition()+propText.length());        
        }

        else if ( sb = dynamic_cast<IBaseSpinButton *>(evt.window()) )
        {
          IWindowHandle efHandle = ISpinButtonStatics::buddyHandle( sb->handle() );
          ISETWINDOWTEXT(efHandle, (PSZ)txtchevt.resultingText());
          IEntryField spinEf = IEntryField(efHandle);
          spinEf.setCursorPosition(txtchevt.startPosition()+propText.length());        
        }


      }
      evt.setResult(true);
      return true;
    }

    case IC_UM_EDIT_SETTEXT:
    {
        IWindowHandle hwndControl = (IWindowHandle)evt.parameter2();
       
        IString newText = (char *)evt.parameter1();
        ITextChangeEvent txtchevt(evt);
        txtchevt.setProposedText(newText);
        textChange( txtchevt );

        if ( txtchevt.isChangeAllowed() && hwndControl )
        {
          IWindowClassName className( hwndControl );
          if ( className == WC_ENTRYFIELD || className == WC_COMBOBOX )
          {
            ISETWINDOWTEXT(hwndControl, (PSZ)txtchevt.proposedText());
          }
          if ( className == WC_SPINBUTTON || className == WC_NATIVESPINBUTTON )
          {
            IWindowHandle ef = ISpinButtonStatics::buddyHandle( hwndControl );
            ISETWINDOWTEXT(ef, (PSZ)txtchevt.proposedText());
          }

        }
        evt.setResult(true);
        return true;
    }
    break;

    case EM_REPLACESEL:
    {
      if (fResendEvent)
        {
        fResendEvent = false;
        return false;
        }
      IString newText = (char *)evt.parameter2();
      ITextChangeEvent txtchevt(evt);
      txtchevt.setProposedText(newText);
      textChange( txtchevt );

      if ( txtchevt.isChangeAllowed() )
        {
        IMultiLineEdit* mle = dynamic_cast<IMultiLineEdit *>(evt.window());
        fResendEvent = true;
        mle->handle().sendEvent( EM_REPLACESEL,
                                 IEventParameter1(0),
                                 IEventParameter2((PSZ)txtchevt.proposedText()) );

        return true;
        }
      else
        return false;
    }

    default:
      break;
  }

#endif //IC_WIN

#ifdef IC_MOTIF
   /***********************************************/
   /*The eventId is the callback reason in Motif. */
   /***********************************************/
   unsigned long reason  = evt.eventId();

   /*********************************************************************/
   /*Modify verify callback.  EntryField, MultiLineEdit, ComboBox (which*/
   /* is implemented as Entryfield + listbox),  SpinButton (which is    */
   /* implemented as Entryfield + two buttons) get these events.        */
   /*********************************************************************/
   if ( reason == motifEvent(XmCR_MODIFYING_TEXT_VALUE) )
      {
      ITextChangeEvent txtchevt(evt);
      bRc = textChange( txtchevt );

      // Move the (possibly modified) data from txtchevt's verifyText back into
      //   the X calldata (both evt and txtchevt point to the same calldata).
      //   Note that, if the original request involved only deleting, we don't
      //   allow the handler to add any text; this would break Motif look & feel. 
      XmTextVerifyCallbackStruct *tcbs =
              (XmTextVerifyCallbackStruct *)evt.parameter1().asUnsignedLong();
      if ( txtchevt.isChangeAllowed() )         // if handler said to allow the change 
        {
        IString newtext = txtchevt.proposedText();
        int newlength = newtext.length();

        if (tcbs->text->length < newlength) {                               
            if (tcbs->text->length != 0) {                                  
                tcbs->text->ptr = XtRealloc (tcbs->text->ptr, newlength);   
            } else {                                                        
                tcbs->text->ptr = XtMalloc (newlength);                     
            }                                                               
        }

        for (int len = 0; len < newlength; len++)       // store the text
          tcbs->text->ptr[len] = newtext.asString()[len+1];      //IString is 1-based
        tcbs->text->length = newlength;                 // store the length
        }             // endif allowChange

      if (bRc)
         {                           // processing done
         evt.setResult(txtchevt.result());  // copy result
         }
      }             // endif MODIFYING_TEXT_VALUE
#endif //IC_MOTIF

   return bRc;
}
