// Revision: 93 1.9.1.4 source/ui/baseapp/itimer.cpp, thread, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: itimer.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     ITimer                                                                   *
*     ITimerFn                                                                 *
*     ITimerMemberFn                                                           *
*     ITimerMemberFn0                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define INCL_WINTIMER
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}

#include <itimer.hpp>

#include <icconst.h>
#include <iexcept.hpp>
#include <ihandler.hpp>
#include <iobjwin.hpp>
#include <iprimlck.hpp>
#include <ireslock.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <itrace.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ITIMER_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PM

/*------------------------------------------------------------------------------
| ITimerHandler                                                                |
|                                                                              |
| This class is used to handle WM_TIMER messages and dispatch them to the      |
| correct ITimerFn object.                                                     |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ITimerHandler : public IHandler {
typedef IHandler
  Inherited;
public:
  ITimerHandler ( );
 ~ITimerHandler ( );
protected:
virtual bool
  dispatchHandlerEvent ( IEvent& event );
}; // ITimerHandler

#pragma pack(pop)
#pragma enum(pop)

#endif

/*------------------------------------------------------------------------------
| ITimerRecord                                                                 |
|                                                                              |
| This class is used to store information for active (or referenced) timers.   |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ITimerRecord {
public:
  ITimerRecord ( );
unsigned long
  interval;
ITimerFn
 *fn;

#ifdef IC_PM
IObjectWindow
 *win;
ITimerHandler
 *fHandler;          //NOTE: We want the default copy ctor here, because
                     //      this object is managed by ITimer.
#endif

#ifdef IC_WIN
unsigned long
  timerId;
#endif

#ifdef IC_MOTIF
XtIntervalId
  timerId;
XtAppContext
  appContext;
#endif

unsigned long
  refCnt;
}; // ITimerRecord

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| ITimerStatics                                                                |
| Class to manage ITimer static data.                                          |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ITimerStatics {
public:
  ITimerStatics ( )
  { }
 ~ITimerStatics ( );

IPrivateResource
 &timerLock ( );

static ITimerRecord
 *fgTimerList;          // List of referenced timers.
static unsigned long
  fgTimerListSize;      // Current size of timer list.

private:
static IPrivateResource
 *fgTimerSem;           // Semaphore to access timer list--use timerLock()
                        // to access this.
}; // ITimerStatics

#pragma pack(pop)
#pragma enum(pop)

static ITimerStatics statics;    // Instance of static object for convenience
                                 // of notation and destructor.
ITimerRecord*     ITimerStatics::fgTimerList = 0;
unsigned long     ITimerStatics::fgTimerListSize = 0;
IPrivateResource* ITimerStatics::fgTimerSem = 0;

/*------------------------------------------------------------------------------
| ITimerRecord::ITimerRecord                                                   |
------------------------------------------------------------------------------*/
ITimerRecord::ITimerRecord ( )
  : interval( 1000 ),
    fn( 0 ),
#ifdef IC_PM
    win( 0 ),
    fHandler( 0 ),
#endif
#ifdef IC_WIN
    timerId( 0 ),
#endif
#ifdef IC_MOTIF
    timerId( 0 ),
    appContext( 0 ),
#endif
    refCnt( 0 )
{ }


#ifdef IC_WIN
// The windows timer class uses a timer callback procedure.  The original
// implementation used an object window like OS/2.  The problem with that
// approach is that when the object window is re-parented, it would steal
// the focus from the application.
VOID CALLBACK TimerProc ( HWND hwnd,
                          UINT uMsg,
                          UINT timerId,
                          DWORD time )
{
  unsigned long appTimerId;

  ITimerFn* fn = 0;
  {
    int fIndex;

    IResourceLock lock( statics.timerLock() );

    // Iterate through the timer list searching for the timer ID
    // that was returned from the SetTimer api.
    for ( fIndex = 1; fIndex < statics.fgTimerListSize; fIndex++ )
    {
      if ( statics.fgTimerList[ fIndex ].timerId == timerId  &&
           statics.fgTimerList[ fIndex ].fn != 0 )
      {
         fn = statics.fgTimerList[ fIndex ].fn;
         appTimerId = (unsigned long)( fIndex + IC_RESERVED_TIMERS );
         break;
      }
    }
  }

  // If the timer was found in the list.
  if ( fn )
  {
     // Call the correct timer expired function.  Add a reference to
     // the function while we are calling it to be sure that it does
     // not get deleted while in use.

     fn->addRef();
     fn->timerExpired( appTimerId );
     fn->removeRef();
  }
}
#endif // IC_WIN

#ifdef IC_MOTIF

// The X timer class uses XtAppAddTimeout to get a callback when the
// time has expired.  This is the callback function.  The implementation
// is very similar to the Windows one except that XtAppAddTimeOut has to
// be restarted after it fires, unlike the Windows SetTimer function.
void _System TimerProc ( XtPointer client_data,
                         XtIntervalId* timerId )
{
  unsigned long appTimerId;

  ITimerFn* fn = 0;
  {
    // The client data contains the index of the timer in our timer list.
    int fIndex = (int)client_data;

    IResourceLock lock( statics.timerLock() );

    // Verify that things are as we expect...insure that the stored
    // ID is the one returned by XtAppAddTimeOut and is valid.
    if ( ( fIndex < statics.fgTimerListSize ) &&
         ( statics.fgTimerList[ fIndex ].timerId == *timerId ) &&
         ( statics.fgTimerList[ fIndex ].fn != 0 ) )
      {
      fn = statics.fgTimerList[ fIndex ].fn;
      appTimerId = (unsigned long)( fIndex + IC_RESERVED_TIMERS );

      // Set up for the next timer cycle.  The timer that caused us
      // to be running now is cleared automatically by Xt.
      statics.fgTimerList[ fIndex ].timerId =
         XtAppAddTimeOut( statics.fgTimerList[ fIndex ].appContext,
                          statics.fgTimerList[ fIndex ].interval,
                          TimerProc,
                          (XtPointer)fIndex );
      }
  }  // end resource lock

  // If the timer was found in the list.
  if ( fn )
  {
     // Call the correct timer expired function.  Add a reference to
     // the function while we are calling it to be sure that it does
     // not get deleted while in use.
     fn->addRef();
     fn->timerExpired( appTimerId );
     fn->removeRef();
  }
}
#endif // IC_MOTIF


#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITimerHandler::ITimerHandler                                                 |
------------------------------------------------------------------------------*/
ITimerHandler::ITimerHandler ( )
{
}

/*------------------------------------------------------------------------------
| ITimerHandler::~ITimerHandler                                                |
------------------------------------------------------------------------------*/
ITimerHandler::~ITimerHandler ( )
{
}

/*------------------------------------------------------------------------------
| ITimerHandler::dispatchHandlerEvent                                          |
------------------------------------------------------------------------------*/
bool ITimerHandler::dispatchHandlerEvent ( IEvent& event )
{
   if ( event.eventId() == WM_TIMER )
   {
      unsigned long timerId = (unsigned long) event.parameter1();
      ITimerFn* fn = 0;
      {
        IResourceLock  lock(statics.timerLock());
        if ( ( timerId > IC_RESERVED_TIMERS ) &&
             ( ( timerId - IC_RESERVED_TIMERS ) < statics.fgTimerListSize ) )
          fn = statics.fgTimerList[ timerId - IC_RESERVED_TIMERS ].fn;
      }
      if ( fn )
      {
         // Call the correct timer expired function.  Add a reference to
         // the function while we are calling it to be sure that it does
         // not get deleted while in use.
         fn->addRef();
         fn->timerExpired( timerId );
         fn->removeRef();

         return true;
      }
   }
   return false;
}
#endif  // IC_PM

/*------------------------------------------------------------------------------
| ITimerStatics::~ITimerStatics                                                |
------------------------------------------------------------------------------*/
ITimerStatics::~ITimerStatics ( )
{
   IMODTRACE_DEVELOP( "ITimerStatics::~ITimerStatics" );
#if IC_STATIC_PRIORITY_SUPPORTED
   // It is only safe to do this if we know that destruction is occurring
   // in the correct order and is single threaded.
   if ( fgTimerList )
      delete [] fgTimerList;
   if ( fgTimerSem )
      delete fgTimerSem;
#endif
}

/*------------------------------------------------------------------------------
| ITimerStatics::timerLock                                                     |
------------------------------------------------------------------------------*/
IPrivateResource& ITimerStatics::timerLock ( )
{
   if ( ! fgTimerSem )
   {
      IMODTRACE_DEVELOP( "Allocating ITimerStatics::fgTimerSem" );
      IPrimalLock lock;
      if ( ! fgTimerSem )
         fgTimerSem = new IPrivateResource;
   }
   return *fgTimerSem;
}

/*------------------------------------------------------------------------------
| ITimer::ITimer                                                               |
|                                                                              |
| Get a free entry from the timer list and add reference.                      |
------------------------------------------------------------------------------*/
ITimer::ITimer ( )
{
  fIndex = 0;
  IResourceLock lock( statics.timerLock() );
  int i;

  for ( i = 1; ( ( i < statics.fgTimerListSize )  &&  ( fIndex == 0 ) ); i++ )
  {
     if ( ( statics.fgTimerList[ i ].refCnt == 0 ) &&
          ( statics.fgTimerList[ i ].fn == 0 ) )
     {
        fIndex = i;
     }
  }

  if ( fIndex == 0 )
  {
     if ( ( statics.fgTimerListSize + 10 + IC_RESERVED_TIMERS ) >= TID_USERMAX )
     {
       ITHROWLIBRARYERROR( IC_TIMER_ID_UNAVAILABLE,
                           IBaseErrorInfo::outOfSystemResource,
                           IException::recoverable );
     }
     else
     {
        // Increase the timer list by 10 records.

        fIndex = statics.fgTimerListSize;
        if ( fIndex == 0 )
        {
           fIndex = 1;
        }

        ITimerRecord* oldList = statics.fgTimerList;
        statics.fgTimerList = new ITimerRecord[ statics.fgTimerListSize + 10 ];
        for ( i = 1; i < statics.fgTimerListSize; i++ )
        {
           statics.fgTimerList[ i ] = oldList[ i ];
        }
        delete [] oldList;
        statics.fgTimerListSize += 10;
     }
  }

  statics.fgTimerList[ fIndex ].refCnt = 1;
}

/*------------------------------------------------------------------------------
| ITimer::ITimer                                                               |
|                                                                              |
| Find a free entry in the timer list and add a reference.                     |
------------------------------------------------------------------------------*/
ITimer::ITimer ( unsigned long timerIdentifer )
{
  fIndex = timerIdentifer - IC_RESERVED_TIMERS;
  IResourceLock lock( statics.timerLock() );

  if ( ( fIndex == 0 )  ||  ( fIndex >= statics.fgTimerListSize ) )
  {
     ITHROWLIBRARYERROR( IC_INVALID_TIMER_ID,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable );
  }

  if ( statics.fgTimerList[ fIndex ].fn == 0 )
  {
     ITHROWLIBRARYERROR( IC_INVALID_TIMER_ID,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable );
  }

  statics.fgTimerList[ fIndex ].refCnt++;
}

/*------------------------------------------------------------------------------
| ITimer::ITimer                                                               |
|                                                                              |
| Add a reference to a timer list entry.                                       |
------------------------------------------------------------------------------*/
ITimer::ITimer ( const ITimer &timer )
{
  fIndex = timer.fIndex;
  IResourceLock lock( statics.timerLock() );
  statics.fgTimerList[ fIndex ].refCnt++;
}

/*------------------------------------------------------------------------------
| ITimer::ITimer                                                               |
|                                                                              |
| Get a free entry from the timer list, add a reference, and start the timer.  |
------------------------------------------------------------------------------*/
ITimer::ITimer ( const ICountedPointerTo<ITimerFn>& timerFunction,
                 unsigned long timerInterval )
{
  ITimer newTimer;                      // Use default ctor to get free fIndex.
  fIndex = newTimer.fIndex;

  // Block for modifying the timer list.
  {
    IResourceLock lock( statics.timerLock() );
    statics.fgTimerList[ fIndex ].refCnt++;
  }

  this->start( timerFunction, timerInterval );
}

/*------------------------------------------------------------------------------
| ITimer::operator=                                                            |
|                                                                              |
| Assignment operator for ITimer.                                              |
------------------------------------------------------------------------------*/
ITimer& ITimer::operator= ( const ITimer& timer )
{
  IResourceLock lock( statics.timerLock() );

  // Remove the reference to the current timer index.
  statics.fgTimerList[ fIndex ].refCnt--;

  // Add a reference to the new timer index.
  fIndex = timer.fIndex;
  statics.fgTimerList[ fIndex ].refCnt++;

  return *this;
}

/*------------------------------------------------------------------------------
| ITimer::~ITimer                                                              |
|                                                                              |
| Remove a reference from the timer list.                                      |
------------------------------------------------------------------------------*/

ITimer::~ITimer ( )
{
  IResourceLock lock( statics.timerLock() );
  statics.fgTimerList[ fIndex ].refCnt--;
}

/*------------------------------------------------------------------------------
| ITimer::start                                                                |
------------------------------------------------------------------------------*/
ITimer& ITimer::start ( const ICountedPointerTo<ITimerFn>& timerFunction )
{
  if ( this->isStarted() )
  {
     ITHROWLIBRARYERROR( IC_TIMER_STARTED,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }

  unsigned long timerInterval;

#ifdef IC_MOTIF
  // Need to insure that the GUI is initialized because we need the
  // app context (IAnchorBlockHandle) to set the timer.
  if ( !IThread::current().isGUIInitialized() )
     IThread::current().initializeGUI();
#endif

#ifdef IC_PM
  IObjectWindow* win = 0;
#endif

  // Block for working with the timer list.
  {
    IResourceLock lock( statics.timerLock() );
    timerInterval = statics.fgTimerList[ fIndex ].interval;
    statics.fgTimerList[ fIndex ].fn = timerFunction.getAlias();
#ifdef IC_WIN
    // In windows when you create a timer using a call back proc, the system
    // determines the timer ID that it returns.  This value is saved as part
    // of the timer record and is used to uniquely identify the timer.
    statics.fgTimerList[ fIndex ].timerId =
       SetTimer( 0,
                 (unsigned)( fIndex + IC_RESERVED_TIMERS ),
                 (unsigned)timerInterval,
                 (TIMERPROC)TimerProc );
#endif
#ifdef IC_MOTIF
    // In Motif when you create a timer using a call back proc, the system
    // determines the timer ID that it returns.  This value is saved as part
    // of the timer record and is used to uniquely identify the timer.
    statics.fgTimerList[ fIndex ].appContext = IThread::current().anchorBlock();
    statics.fgTimerList[ fIndex ].timerId =
       XtAppAddTimeOut( statics.fgTimerList[ fIndex ].appContext,
                        statics.fgTimerList[ fIndex ].interval,
                        TimerProc,
                        (XtPointer)fIndex );
#endif
#ifdef IC_PM
    // Create an object window that will not prevent shutdown of the
    // message queue.  This object window will receive WM_TIMER messages
    // from the timer.
    statics.fgTimerList[ fIndex ].win = new IObjectWindow( IObjectWindow::noStyle );
    win = statics.fgTimerList[ fIndex ].win;
    // Allocate a handler and attach it to the window.
    statics.fgTimerList[ fIndex ].fHandler = new ITimerHandler;
    statics.fgTimerList[ fIndex ].fHandler->handleEventsFor( win );
#endif
  }

#ifdef IC_PM
  if ( ISTARTTIMER ( IThread::current().anchorBlock(),
                     win->handle(),
                     fIndex + IC_RESERVED_TIMERS,
                     timerInterval ) == 0 )
  {
     ITHROWGUIERROR("WinStartTimer");
  }
#endif

  ITRACE_DEVELOP( IString( "ITimer::start(" ) +
                  IString( fIndex + IC_RESERVED_TIMERS ) +
                  IString( ')' ) );

  timerFunction->addRef();

  return *this;
}

/*------------------------------------------------------------------------------
| ITimer::start                                                                |
------------------------------------------------------------------------------*/

ITimer& ITimer::start ( const ICountedPointerTo<ITimerFn>& timerFunction,
                        unsigned long timerInterval )
{
  this->setInterval( timerInterval );
  return this->start( timerFunction );
}

/*------------------------------------------------------------------------------
| ITimer::stop                                                                 |
------------------------------------------------------------------------------*/
ITimer& ITimer::stop ( )
{
  if ( this->isStarted() )
  {
#ifdef IC_PM
    ITimerHandler* hdr = 0;
    IObjectWindow* win = 0;

    {  // Block for reading from the list.
       IResourceLock lock( statics.timerLock() );
       win = statics.fgTimerList[ fIndex ].win;
       hdr = statics.fgTimerList[ fIndex ].fHandler;
    }

    // Check it the window still exists before calling PM.  The window may
    // have been destroyed by a terminating thread without stopping the
    // timer.

    if ( win->isValid() )
    {
      if ( ISTOPTIMER ( IThread::current().anchorBlock(),
                        win->handle(),
                        fIndex + IC_RESERVED_TIMERS ) == FALSE )
      {
         ITHROWGUIERROR( "WinStopTimer" );
      }
    }
#endif

    { // Block for removing data from the list.
      IResourceLock lock( statics.timerLock() );
#ifdef IC_WIN
      KillTimer( 0, (unsigned)statics.fgTimerList[ fIndex ].timerId );
      statics.fgTimerList[ fIndex ].timerId = 0;
#endif
#ifdef IC_MOTIF
      XtRemoveTimeOut( statics.fgTimerList[ fIndex ].timerId );
      statics.fgTimerList[ fIndex ].timerId = 0;
#endif
      if ( statics.fgTimerList[ fIndex ].fn )
      {
         statics.fgTimerList[ fIndex ].fn->removeRef();
         statics.fgTimerList[ fIndex ].fn = 0;
      }
#ifdef IC_PM
      statics.fgTimerList[ fIndex ].win = 0;
      statics.fgTimerList[ fIndex ].fHandler = 0;
#endif
    }

#ifdef IC_PM
    if ( win )
    {
      // Remove and delete the handler.
      hdr->stopHandlingEventsFor( win );
      delete hdr;

      if ( win->isValid() )
      {
         // Destroy the window and automatically delete the instance
         // when it is safe (ie. when we are not nested)
         win->setAutoDeleteObject();
         IDESTROYWINDOW( win->handle() );
      }
      else
      {
         // The PM window does not exist, so we are sure this is not
         // a nested call for the window and it is safe to delete the
         // object instance.
         delete win;
      }
    }
#endif

    ITRACE_DEVELOP( IString( "ITimer::stop(" ) +
                    IString( fIndex + IC_RESERVED_TIMERS ) +
                    IString( ')' ) );
  }

  return *this;
}

/*------------------------------------------------------------------------------
| ITimer::id                                                                   |
------------------------------------------------------------------------------*/
unsigned long ITimer::id ( ) const
{
  return ( fIndex + IC_RESERVED_TIMERS );
}

/*------------------------------------------------------------------------------
| ITimer::asString                                                             |
------------------------------------------------------------------------------*/
IString ITimer::asString ( ) const
{
  return ( IString( "ITimer(" ) +
           IString( fIndex+IC_RESERVED_TIMERS ) + ")" );
}

/*------------------------------------------------------------------------------
| ITimer::asDebugInfo                                                          |
------------------------------------------------------------------------------*/
IString ITimer::asDebugInfo ( ) const
{
  IString str;
  str = IString( "ITimer: id = " ) + IString( fIndex+IC_RESERVED_TIMERS );
  str += IString( ", interval = " ) + IString( interval() );
  if ( this->isStarted() )
     str += ", timer started";
  return str;
}

/*------------------------------------------------------------------------------
| ITimer::interval                                                             |
------------------------------------------------------------------------------*/
unsigned long ITimer::interval ( ) const
{
  IResourceLock lock( statics.timerLock() );
  return statics.fgTimerList[ fIndex ].interval;
}

/*------------------------------------------------------------------------------
| ITimer::setInterval                                                          |
------------------------------------------------------------------------------*/
ITimer& ITimer::setInterval ( unsigned long timerInterval )
{
  if ( this->isStarted() )
  {
    ITimerFn* fn;
    bool change;
    {
      IResourceLock lock( statics.timerLock() );
      if ( statics.fgTimerList[ fIndex ].interval == timerInterval )
      {
         change = false;
      }
      else
      {
         change = true;
         fn = statics.fgTimerList[ fIndex ].fn;
         statics.fgTimerList[ fIndex ].interval = timerInterval;
      }
    }
    if ( change )
    {
       fn->addRef();
       this->stop();
       this->start( fn );
    }
  }
  else
  {
    IResourceLock lock( statics.timerLock() );
    statics.fgTimerList[ fIndex ].interval = timerInterval;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ITimer::isStarted                                                            |
------------------------------------------------------------------------------*/
bool ITimer::isStarted ( ) const
{
  IResourceLock lock( statics.timerLock() );
  if ( statics.fgTimerList[ fIndex ].fn == 0 )
     return false;
  return true;
}

/*------------------------------------------------------------------------------
| ITimerFn::ITimerFn                                                           |
|                                                                              |
| Empty ctor to prevent generated static version.                              |
------------------------------------------------------------------------------*/
ITimerFn::ITimerFn ( )
  : ITimerFn::Inherited()
{
}

/*------------------------------------------------------------------------------
| ITimerFn::~ITimerFn                                                          |
|                                                                              |
| Empty dtor to prevent generated static version.                              |
------------------------------------------------------------------------------*/
ITimerFn::~ITimerFn ( )
{
}

/*------------------------------------------------------------------------------
| ITimer::Cursor::Cursor                                                       |
------------------------------------------------------------------------------*/
ITimer::Cursor :: Cursor ( )
  : fIndex ( 0 )
{
}

/*------------------------------------------------------------------------------
| ITimer::Cursor::~Cursor                                                      |
------------------------------------------------------------------------------*/
ITimer::Cursor :: ~Cursor ( )
{
}

/*------------------------------------------------------------------------------
| ITimer::Cursor::setToFirst                                                   |
|                                                                              |
| Point to the first element of the array of started timers.                   |
------------------------------------------------------------------------------*/
bool ITimer::Cursor :: setToFirst ( )
{
  IResourceLock lock( statics.timerLock() );
  for ( fIndex = 1; fIndex < statics.fgTimerListSize; fIndex++ )
    if ( statics.fgTimerList[ fIndex ].fn != 0 )
       return true;
  fIndex = 0;
  return false;
}

/*------------------------------------------------------------------------------
| Iimer::Cursor::setToNext                                                     |
|                                                                              |
| Advance the cursor and return whether it is still valid.                     |
------------------------------------------------------------------------------*/
bool ITimer::Cursor :: setToNext ( )
{
  fIndex++;
  IResourceLock lock( statics.timerLock() );
  for ( ; fIndex < statics.fgTimerListSize; fIndex++ )
    if ( statics.fgTimerList[ fIndex ].fn != 0 )
       return true;
  fIndex = 0;
  return false;
}

/*------------------------------------------------------------------------------
| ITimer::Cursor::isValid                                                      |
------------------------------------------------------------------------------*/
bool ITimer::Cursor :: isValid ( ) const
{
  if ( fIndex == 0 )
     return false;

  IResourceLock lock( statics.timerLock() );
  if ( statics.fgTimerList[ fIndex ].fn == 0 )
    return false;

  return true;
}

/*------------------------------------------------------------------------------
| Iimer::Cursor::invalidate                                                    |
------------------------------------------------------------------------------*/
void ITimer::Cursor :: invalidate ( )
{
   fIndex = 0;
}

/*------------------------------------------------------------------------------
| ITimer::timerAt                                                              |
|                                                                              |
| Check the validity of the cursor and return the timer at the cursor.         |
------------------------------------------------------------------------------*/
ITimer ITimer::timerAt ( const ITimer::Cursor& cursor )
{
  IASSERTPARM( cursor.isValid() );
  return ITimer( cursor.fIndex + IC_RESERVED_TIMERS );
}

/*------------------------------------------------------------------------------
| ITimer::operator ==                                                          |
|                                                                              |
| Check if two timers are identical.                                           |
------------------------------------------------------------------------------*/
bool ITimer::operator == ( const ITimer timer )
{
   if ( timer.id() == this->id() )
   {
      return true;
   }
   return false;
}
