// Revision: 95 1.4.1.3 source/ui/baseapp/iwcname.cpp, datatype, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iwcname.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iwcname.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #ifdef IC_MOTIF
     #include <X11/IntrinsicP.h>
     #include <X11/CoreP.h>
  #endif
  }

#include <iwcname.hpp>
#include <ihandle.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWCNAME_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IWindowClassName::IWindowClassName                                           |
|                                                                              |
| Construct the class name from argument.  char* values are converted to       |
| corresponding "#n" "class names."  The class name for a window handle is     |
| obtained via WinQueryWindowClass.                                            |
------------------------------------------------------------------------------*/
IWindowClassName :: IWindowClassName ( const char *className )
  : name( )
  {
  this -> initialize( className );
  }

IWindowClassName :: IWindowClassName ( const IWindowHandle &handle )
  : name( )
  {
  IASSERTPARM( handle.isValid() );

#ifdef IC_PMWIN
  #define BUFSIZE 32
  char     buffer[BUFSIZE];
  unsigned len = IQUERYCLASSNAME( handle, BUFSIZE, buffer );
  if (len < (BUFSIZE - 1) )
     this->name = IString(buffer);
  else
  {
     char*    pbuffer = 0;
     unsigned bufferSize = BUFSIZE;
     do
     {
        delete [] pbuffer;
        bufferSize += BUFSIZE;
        pbuffer = new char[bufferSize];
        len = IQUERYCLASSNAME( handle, bufferSize, pbuffer );
     }  while (len >= bufferSize - 1);
     this->name = IString(pbuffer);
     delete [] pbuffer;
  }
#endif // IC_PMWIN
#ifdef IC_MOTIF
  this->name = IString(XtClass( (Widget)handle )->core_class.class_name );
#endif

  }

IWindowClassName :: IWindowClassName ( const IWindowClassName &className )
  : name( className.name )
  {
  }

/*------------------------------------------------------------------------------
| IWindowClassName::~IWindowClassName                                          |
------------------------------------------------------------------------------*/
IWindowClassName :: ~IWindowClassName ( )
  {
  }

/*------------------------------------------------------------------------------
| IWindowClassName::asString                                                   |
|                                                                              |
| Simply returns the name data member.                                         |
------------------------------------------------------------------------------*/
IString IWindowClassName :: asString ( ) const
  {
  return this->name;
  }

/*------------------------------------------------------------------------------
| IWindowClassName::operator const char *                                      |
|                                                                              |
| If the class name is one of the #nnnnn values, convert to corresponding      |
| WC_* value.                                                                  |
------------------------------------------------------------------------------*/
IWindowClassName :: operator const char * ( ) const
  {
  const char
   *result = this->name;
#ifdef IC_PMWIN
  if ( result[0] == '#' )
    {
#ifdef IC_WIN
    // Make an atom out of the digits following the # in the stored value
    result = MAKEINTATOM( this->name.subString(2).asInt() );
#else
    unsigned short
      nnnnn = 0;
    const char
     *p = result;
    while ( *(++p) )
      nnnnn = nnnnn * 10 + ( *p % 16 );
    result = (const char *)( 0xFFFF0000ul + nnnnn );
#endif
    }
#endif
  return result;
  }

/*------------------------------------------------------------------------------
| IWindowClassName::operator ==                                                |
| IWindowClassName::operator !=                                                |
|                                                                              |
| If either argument is 0xFFFFnnnn, convert both to const char * and           |
| compare, otherwise, compare names.  In Windows, the magic value for an       |
| atom is 0x0000nnnn.                                                          |
|                                                                              |
| For the "not equal" comparisons", call the "equal" member functions and      |
| negate the result.                                                           |
------------------------------------------------------------------------------*/
bool IWindowClassName :: operator == ( const char *className ) const
  {
#ifdef IC_PMWIN
#ifdef IC_WIN
  if ( (unsigned long)className >> 16 == 0      )
#else
  if ( (unsigned long)className >> 16 == 0xFFFF )
#endif
    {
    const char
     *self = *this;
    return self == className;
    }
  else
    {
    return this->name == className;
    }
#endif
#ifdef IC_MOTIF
  return (this->name == className);
#endif
  }

bool IWindowClassName :: operator == (       char *className ) const
{
   return (*this == (const char*)className);
}

bool IWindowClassName :: operator != ( const char *className ) const
  {
  return !( *this == className );
  }

bool IWindowClassName :: operator == ( const IWindowClassName &className ) const
  {
  return this->name == className.name;
  }

bool IWindowClassName :: operator != ( const IWindowClassName &className ) const
  {
  return this->name != className.name;
  }

bool operator == (       char *className1, const IWindowClassName &className2 )
  {
  // Invoke member function...
  return className2 == (const char*)className1;
  }

bool operator == ( const char *className1, const IWindowClassName &className2 )
  {
  // Invoke member function...
  return className2 == className1;
  }

bool operator != ( const char *className1, const IWindowClassName &className2 )
  {
  return !( className2 == className1 );
  }


/*------------------------------------------------------------------------------
| IWindowClassName::initialize                                                 |
|                                                                              |
| If the argument is of the form 0xFFFFnnnn, convert to "#nnnn".  (OS/2)       |
| If the argument is of the form 0x0000nnnn, convert to "#nnnn".  (Windows)    |
------------------------------------------------------------------------------*/
IWindowClassName &IWindowClassName :: initialize ( const char *className )
  {
#ifdef IC_PMWIN
  unsigned
    ptr = (unsigned long)className >> 16;
  /* If upper word of pointer is max (reserved value) */
#ifdef IC_WIN
  if ( ptr == 0      )
#else
  if ( ptr == 0xFFFF )
#endif
    {
    /* Then keep just the lower word by stripping off upper word      */
    ptr = (unsigned long)className & 0xFFFF;
    this->name = "#" + IString( ptr );
    }
  else
    this->name = className;
#endif
#ifdef IC_MOTIF
  this->name = className;
#endif
  return *this;
  }
