// Revision: 08 1.9.1.4 source/ui/baseapp/iwindow6.cpp, basewin, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iwindow6.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions used for        *
*   notification in IWindow.                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #include <iwindefs.h>
}


#include <iwinnhdr.hpp>
#include <iwindow.hpp>
#include <iexcept.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <iwinpriv.hpp>
#include <iwinlsts.hpp>
#include <iobslist.hpp>
#include <iintrst.hpp>
#include <iintpair.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWINDOW6_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects
INotificationId const IWindow::sizeId                    = "IWindow::size";
INotificationId const IWindow::positionId                = "IWindow::position";
INotificationId const IWindow::visibleId                 = "IWindow::visible";
INotificationId const IWindow::enableId                  = "IWindow::enable";
INotificationId const IWindow::focusId                   = "IWindow::focus";
INotificationId const IWindow::foregroundColorId         = "IWindow::foregroundColor";
INotificationId const IWindow::backgroundColorId         = "IWindow::backgroundColor";
INotificationId const IWindow::borderColorId             = "IWindow::borderColor";
INotificationId const IWindow::shadowColorId             = "IWindow::shadowColor";
INotificationId const IWindow::disabledForegroundColorId = "IWindow::disabledForegroundColor";
INotificationId const IWindow::disabledBackgroundColorId = "IWindow::disabledBackgroundColor";
INotificationId const IWindow::activeColorId             = "IWindow::activeColor";
INotificationId const IWindow::inactiveColorId           = "IWindow::inactiveColor";
INotificationId const IWindow::hiliteForegroundColorId   = "IWindow::hiliteForegroundColor";
INotificationId const IWindow::hiliteBackgroundColorId   = "IWindow::hiliteBackgroundColor";
INotificationId const IWindow::fontId                    = "IWindow::font";
INotificationId const IWindow::commandId                 = "IWindow::command";
INotificationId const IWindow::systemCommandId           = "IWindow::systemCommand";
INotificationId const IWindow::attributeAddReplaceId     = "IWindow::attributeAddReplace";
INotificationId const IWindow::attributeRemoveId         = "IWindow::attributeRemove";


/*------------------------------------------------------------------------------
| IWindow::enableNotification                                                  |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IWindow, does not send notifications to observer objects until it is enabled.|
|                                                                              |
| In the window hierarchy we override this function to allocate the  correct   |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IWindow& IWindow::enableNotification( bool enabled )
{
  this->pWindowData->observerData().enabled = enabled;
  if ( notificationHandler() == 0 )
    setNotificationHandler( new IWindowNotifyHandler() );

  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::disableNotification                                                 |
|                                                                              |
| Flags this part as being not ready to accept notifications.  A Notifier,     |
| like IWindow, does not send notifications to observer objects unless it is   |
| enabled                                                                      |
------------------------------------------------------------------------------*/
IWindow& IWindow::disableNotification( )
{
  this->pWindowData->observerData().enabled = false;
  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::isEnabledForNotification                                            |
|                                                                              |
| Returns true if this part is enabled for notification.                       |
------------------------------------------------------------------------------*/
bool IWindow::isEnabledForNotification( ) const
{
  if ( this->pWindowData->_observerData &&
       (this->pWindowData->observerData().enabled == true) )
    return true;
  else
    return false;
}

#if (IC_OBSOLETE <= IC_OBSOLETE_3)
/*------------------------------------------------------------------------------
| IWindow::addObserver                                                         |
|                                                                              |
| This is the implementation of the INotifier::addObserver protocol.  We just  |
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::addObserver( IObserver&       anObserver,
                               const IEventData& userData)
{
  observerList().add(anObserver, userData);
  return *this;
}
#endif // IC_OBSOLETE

/*------------------------------------------------------------------------------
| IWindow::addObserver                                                         |
|                                                                              |
| This is the implementation of the INotifier::addObserver protocol.  We just  |
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::addObserver( IObserver&       theObserver,
                               const IInterest& theInterest)
{
  //Add the connection to the connection list
  //{2} Obsolete - need to remove IEventData parameter from observer list seq
  observerList( &theInterest ).add( theObserver, 0 );
  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::removeObserver                                                      |
|                                                                              |
| This is the implementation of the INotifier::removeObserver protocol. We just|
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::removeObserver( IObserver& anObserver )
{
  IResourceLock
    threadsLock( this->pWindowData->observerData().dictionaryKey() );

  INotifierImplementation*
    pNotifierImp = this->pWindowData->observerData().fNotifierImp;

  IInterestDictionary::Cursor
    cursor( pNotifierImp->interestDict() );

  //Remove observer for every interest (including allInterest)
  forCursor( cursor )
  {
    IInterestConnectionListPair&
      thePair = pNotifierImp->interestDict().elementAt( cursor );

    //Get the connection list and remove the observer
    thePair.getElement()->remove( anObserver );
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::removeObserver                                                      |
|                                                                              |
| This is the implementation of the INotifier::removeObserver protocol. We just|
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::removeObserver( IObserver& theObserver,
                                  const IInterest& theInterest )
{
  INotifierImplementation*
    pNotifierImp = this->pWindowData->observerData().fNotifierImp;

  //Look in the dict for this interest
  bool found =
    pNotifierImp->interestDict().containsElementWithKey( theInterest );

  //Does this interest exist?
  if (found)
  {
    //Get the connections
    IInterestConnectionListPair&
      thePair = pNotifierImp->interestDict().elementWithKey( theInterest );

   //Remove this observers  connections
   IConnectionList*
     theConnections = thePair.getElement();

   if (theConnections)
     theConnections->remove(theObserver);
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::removeAllObservers                                                  |
|                                                                              |
| This is the implementation of the INotifier::removeAllObservers protocol. We |
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::removeAllObservers ( )
{
  IResourceLock
    threadsLock( this->pWindowData->observerData().dictionaryKey() );

  INotifierImplementation*
    pNotifierImp = this->pWindowData->observerData().fNotifierImp;

  IInterestDictionary::Cursor
    cursor( pNotifierImp->interestDict() );

  //For every interest
  forCursor( cursor )
  {
    IInterestConnectionListPair&
      thePair = pNotifierImp->interestDict().elementAt( cursor );

    //Get the connection list and remove the connections
    thePair.getElement()->removeAll();
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::notifyObservers                                                     |
|                                                                              |
| This is the implementation of the INotifier::notifyObservers protocol. We    |
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::notifyObservers( const INotificationEvent& anEvent)
{
  if ( isEnabledForNotification() )
   {
     static const unsigned int maxInterestArraySz = 2;
     //perhaps it is better to create allChangesInterest once by adding fAllChangesInterest field
     IInterest allChangesInterest( *this, INotifier::allChangesId );
     //use this array to check for *both* allChangesInterest and the event interest
     const IInterest* interestArray[maxInterestArraySz] = {&allChangesInterest, &anEvent.interest()};
     int numInterests = maxInterestArraySz;

     //Only check for 1 if these are the same
     if (allChangesInterest == anEvent.interest())
     numInterests = 1;

     for (int i = 0; i < numInterests; i++) {
     const IInterest& theInterest = *interestArray[i];

     INotifierImplementation*
       pNotifierImp = this->pWindowData->observerData().fNotifierImp;

     //Does this interest exist?
     bool
       found = pNotifierImp->interestDict().containsElementWithKey( theInterest);

     //Get the connections
     if (found)
       {
        //Get the connections for the interest
        IInterestConnectionListPair&
          thePair = pNotifierImp->interestDict().elementWithKey( theInterest );

        //pass event to connection list
        IConnectionList*
          theConnections = thePair.getElement();

        theConnections->notifyObservers( anEvent );
       }
     }
   }
  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::notifyObserversAsync                                                |
|                                                                              |
| This is the implementation of the INotifier::notifyObserversAsync protocol.  |
| We pass requests on to the observer list.                                    |
------------------------------------------------------------------------------*/
IWindow& IWindow::notifyObserversAsync( const INotificationEvent& anEvent)
{
  if ( isEnabledForNotification() )
   {
     static const unsigned int maxInterestArraySz = 2;
     //perhaps it is better to create allChangesInterest once by adding fAllChangesInterest field
     IInterest allChangesInterest( *this, INotifier::allChangesId );
     //use this array to check for *both* allChangesInterest and the event interest
     const IInterest* interestArray[maxInterestArraySz] = {&allChangesInterest, &anEvent.interest()};
     int numInterests = maxInterestArraySz;

     //Only check for 1 if these are the same
     if (allChangesInterest == anEvent.interest())
     numInterests = 1;

     for (int i = 0; i < numInterests; i++) {
     const IInterest& theInterest = *interestArray[i];

     INotifierImplementation*
       pNotifierImp = this->pWindowData->observerData().fNotifierImp;

     //Does this interest exist?
     bool
       found = pNotifierImp->interestDict().containsElementWithKey( theInterest);

     //Get the connections
     if (found)
       {
        //Get the connections for the interest
        IInterestConnectionListPair&
          thePair = pNotifierImp->interestDict().elementWithKey( theInterest );

        //pass event to connection list
        IConnectionList*
          theConnections = thePair.getElement();

        // The connectionList just stores a sequence of connections, which could
        // live in the same/diff thread as this thread and may or may not have
        // a request processor.  So let it create a bunch of INotificationRequests
        // for each connection and let it try to stick them on the correct request
        // processor.
        theConnections->notifyObserversAsync( anEvent );
       }
     }
   }
  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::notifierAddress                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
INotifierAddress IWindow::notifierAddress() const
{
  return INotifierAddress( this );
}

/*------------------------------------------------------------------------------
| IWindow::notifyObservers                                                     |
|                                                                              |
| This is the implementation of the INotifier::notifyObservers protocol. We    |
| pass requests on to the observer list.                                       |
------------------------------------------------------------------------------*/
IWindow& IWindow::notifyObservers(const INotificationId& nId)
{
  if (isEnabledForNotification())
  {
    IInterest theInterest( *this, nId );
    notifyObservers( INotificationEvent( theInterest ) );
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IWindow::observerList                                                        |
|                                                                              |
| Returns a reference to the observer list. The creation of the observer       |
| list is delayed until this function is called for the first time -- all      |
| access to the observer list MUST be obtained through this function.          |
------------------------------------------------------------------------------*/
IObserverList& IWindow::observerList( const IInterest* anInterest ) const
{
  IInterest allChangesInterest( *this, INotifier::allChangesId );

  INotifierImplementation*
    pNotifierImp = this->pWindowData->observerData().fNotifierImp;

  bool
    found = false;

  if ( !anInterest )
    anInterest = &allChangesInterest;

  found = pNotifierImp->interestDict().containsElementWithKey( *anInterest );
  if ( !found )
  {
    //If not, then enter an empty connectionlist
    IConnectionList* pConnections = new IConnectionList();

    //pConnections will get adopted by the Pair
    pNotifierImp->interestDict().add(
                                 IInterestConnectionListPair( *anInterest,
                                                              pConnections ) );
  }

  //Get the connections for the interest
  IInterestConnectionListPair&
    thePair = pNotifierImp->interestDict().elementWithKey( *anInterest );

  //return reference to connection list
  return( *(thePair.getElement()) );
}

/*------------------------------------------------------------------------------
| IWindow::notificationHandler                                                 |
|                                                                              |
| Returns a pointer to the notification handler or zero if it has not been     |
| constructed.                                                                 |
------------------------------------------------------------------------------*/
IWindowNotifyHandler* IWindow::notificationHandler( ) const
{
  return this->pWindowData->observerData().notifyHandler;
}

/*------------------------------------------------------------------------------
| IWindow::setNotifcationHandler                                               |
|                                                                              |
| Sets the notification handler to the passed value.                           |
------------------------------------------------------------------------------*/
IWindow& IWindow::setNotificationHandler( IWindowNotifyHandler* handler )
{
  IASSERT(handler!=0);
  if (this->notificationHandler())
     delete this->notificationHandler();
  this->pWindowData->observerData().notifyHandler = handler;
  return *this;
}

/*------------------------------------------------------------------------------
| IObserverData::IObserverData                                                 |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IObserverData::IObserverData()
    : fNotifierImp( new INotifierImplementation )
    , notifyHandler(0)
    , enabled(false)
{}

/*------------------------------------------------------------------------------
| IObserverData::~IObserverData                                                |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IObserverData::~IObserverData ()
{
  delete fNotifierImp;
  if (notifyHandler)
     delete notifyHandler;
}

/*------------------------------------------------------------------------------
| IWindowPrivateData::observerData                                             |
|                                                                              |
| Returns a reference to the observerData object.                              |
------------------------------------------------------------------------------*/
IObserverData& IWindowPrivateData::observerData( )
{ if (_observerData == 0)
     _observerData = new IObserverData;
  return *_observerData;
}


/*------------------------------------------------------------------------------
| IObserverData::dictionaryKey                                                 |
------------------------------------------------------------------------------*/
IPrivateResource& IObserverData::dictionaryKey( )
{
  if (!IWindowStaticData::fgDictionaryKey)
  {
      IResourceLock lock( IWindowStaticData::libraryKey() );
      if (!IWindowStaticData::fgDictionaryKey)
         IWindowStaticData::fgDictionaryKey = new IPrivateResource;
  }
  return *IWindowStaticData::fgDictionaryKey;
}
