/* Revision: 69 1.11.1.10 source/ui/baseapp/ixconst.h, basewin, ioc.v400, 000302  */
/* NOSHIP */
#ifndef _IXCONST_
  #define _IXCONST_
/*******************************************************************************
* FILE NAME: ixconst.h                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*  This file contains constants for the Motif environment.  Most of            *
*  these defined values are taken from the PM toolkit for convenience.         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1994, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or               *
*   disclosure                                                                 *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

/***************************************************/
/*Make sure we don't use this with real PM toolkit */
/***************************************************/
#ifndef OS2_INCLUDED

#define     ICLUI_MESSAGE "ICLUI_MESSAGE"
#define     ICLUI_WMDELETE "WM_DELETE_WINDOW"
#define     ICLUI_WMSAVE   "WM_SAVE_YOURSELF"
#define     ICLUI_WMFOCUS  "WM_TAKE_FOCUS"

// The event ids are stored in the file ixevtids.h
extern "C"
{
  #include <X11/Xlib.h>
  #include <X11/Intrinsic.h>
  #include <Xm/Xm.h>
  #include <stdlib.h>
  #ifndef _IXEVTIDS_
    #include <ixevtids.h>
  #endif
}

/***********************************************************************
   Windows style masks definitions
   There are 3 basic types of masks defined.
   _MASK       - mask for the base styles
   _EXTMASK    - mask for ICLUI defined extended styles
   _EXTGUIMASK - mask for GUI defined extended styles.  In Windows,
                 these are the extended styles for the first argument
                 to CreateWindowEx.
   The convertToGUIStyle function in the classes will return valid
   style bits to be passed to the GUI.  The ICLUI defined styles
   are translated to appropriate GUI flags by this function if
   needed.  For Windows, specifying extendedOnly=true will cause
   convertToGUIStyle to return any user style bits (24-31) ORed with
   the Windows extended styles (WS_EX_*) supported by the class.

   Classes which dont support any GUI extended styles typically use
   IS_EXTMASK to mask the extended styles.  If support is added for
   one or more GUI extended styles,  a _EXTGUIMASK definition should
   be added here and used instead.
***********************************************************************/
#define IWS_MASK        0xFFFF0000ul       // Window (WS_) reserved styles
#define IS_EXTMASK      0xFF000000ul       // Generic extended styles mask
#define IFS_MASK        0x00000000ul       // Frame window (FS_)
#define IFS_EXTMASK     0x00FB0000ul       // Frame window extended
#define IFS_EXTGUIMASK  0x000404C9ul       // Frame window. GUI defined extended
                                           // Excludes all WS_EX_* styles EXCEPT:
                                           //  WS_EX_DLGMODALFRAME
                                           //  WS_EX_TOPMOST
                                           //  WS_MDICHILD
                                           //  WS_EX_TOOLWINDOW
                                           //  WS_EX_CONTEXTHELP
                                           //  WS_EX_APPWINDOW,
#define IDT_MASK        0x0000FFE0ul       // DrawText (DT_) styles  Ref:SS_TYPEMASK
#define IDT_EXTMASK     0x007004D0ul       // DrawText extended styles
#define ITC_EXTGUIMASK  0x00000200ul       // Text Control(entry field and mle)
                                           //  WS_EX_CLIENTEDGE
#define IOB_EXTGUIMASK  0x00020000ul       // Outline Box extended GUI styles
                                           //  WS_EX_STATICEDGE
#define IGB_EXTGUIMASK  0x00020000ul       // Group Box extended GUI styles
                                           //  WS_EX_STATICEDGE
#define ISS_EXTGUIMASK  0x000FFF20ul       // Static  extended styles  EXCLUDES:
                                           //    WS_EX_DLGMODALFRAME
                                           //    WS_EX_NOPARENTNOTIFY
                                           //    WS_EX_TOPMOST
                                           //    WS_EX_ACCEPTFILES
                                           //    WS_EX_MDICHILD
                                           //    WS_EX_TOOLWINDOW
                                           //    WS_EX_CONTEXTHELP
                                           //    WS_EX_CLIENTEDGE
#define IFDS_MASK       0x002FFFFFul       // File Dialog (OFN_)
#define IFDS_EXTMASK    0x00008000ul       // File Dialog extended mask
#define IFNTS_MASK      0x01FFFFFFul       // Font Dialog (CF_)
#define IFNTS_EXTMASK   0x00008000ul       // Font Dialog extended mask
#define IMLS_MASK       0x00001DFFul       // MLE (ES_ styles)
#define IMS_MASK        0x00000045ul       // Menu control
#define IMS_STYLEMASK   0x0000e974ul       // Menu styles (MF_)
#define IMS_ATTRMASK    0x0000028bul       // Menu attributes (MF_)
#define IMBS_MASK       0x00000000ul       // Menubar
#define IMBS_EXTMASK    0x00000003ul       // Menubar extended
#define IMIS_MASK       0x0000FFFFul       // Menu item
#define IMIS_EXTMASK    0x00000001ul       // Menu item extended
#define ILS_MASK        0x00007FFDul       // List box (LBS_) (strip LBS_SORT)
#define ILS_EXTGUIMASK  0x00000200ul       // List box extended
#define IBKS_MASK       0x0000F7FFul       // Notebook (BKS_)
#define ICCS_MASK       0x0000007Ful       // Container (CCS_)
#define ISLS_SHAFTMASK  0x00000FCFul       // Slider (SLS_) Shaft styles
#define ISLS_BUTTONMASK 0x00000030ul       // Slider (SLS_) Button styles
#define ISLS_EXTMASK    0x00000010ul       // Slider extended styles
#define ISLS_EXTGUIMASK 0x00000200ul       // Slider GUI extended styles
#define IES_MASK        0x00001DFFul       // Entry field styles
#define IES_EXTMASK     0x00000001ul       // Entry field extended styles
#define ICBS_MASK       0x00100FF3ul       // Combo box styles
#define ICBS_EXTMASK    0x00100008ul       // Combo box extended styles
#define ICBS_EXTGUIMASK 0x00000200ul       // Combo box GUI extended styles
#define IBS_MASK        0x00000020ul       // Button controls
#define IBS_EXTCHECKBX  0x00000006ul       // Button controls extended checkbox
#define IBS_EXTRADIOBTN 0x00000018ul       // Button controls extended radiobtn
#define IBS_EXT3STATE   0x00000060ul       // Button controls extended 3state
#define IBS_EXTPUSH     0x00000700ul       // Button controls extended push
#define IBS_EXTGPUSH    0x00000f00ul       // Graphic Button controls extended push
#define ISBS_MASK       0x0000000Ful       // Scroll bar styles
// need one of these for BS_NOPOINTERFOCUS (IButton)
#define ISPBS_MASK      0x000001BFul       // Spin button styles
#define ISPBS_EXTMASK   0x00003FF8ul       // Spin button extended styles
#define ISS_BMPMASK     0x00000000ul       // Static bitmap style (none in Win)
#define ISS_EXTBMPMASK  0x00000001ul       // Bitmap Extended
#define IGB_MASK        0x00000000ul       // Graphic button style (none in Win)
#define IOBS_EXTMASK    0x00000007ul       // Outline box extended styles

/*------------------------------------------------------------------------
   The constant IC_XALLEVENTSMASK covers all the Input Event Masks
   defined in X11/X.h file and is used in the ICLUI code implementation.
   This constant needs to be updated to match with the masks defined in
   X11/X.h all the time. If this constant is provided by X11 later, the
   constant IC_XALLEVENTSMASK in ICLUI code should be replaced.
-------------------------------------------------------------------------*/
#define     IC_XALLEVENTSMASK           0x01FFFFFF

/*------------------------------------------------------------------------------
   Control message notification codes
------------------------------------------------------------------------------*/
#define     CBN_SHOWLIST              6
#define     SPBN_UPARROW              0x20A
#define     SPBN_DOWNARROW            0x20B
#define     SPBN_ENDSPIN              0x20C
#define     SLN_CHANGE                1
#define     SLN_SLIDERTRACK           2

/*------------------------------------------------------------------------------
   dialog
------------------------------------------------------------------------------*/
#define DID_OK      1
#define DID_CANCEL  2
#define DID_ERROR   0xffff

/*------------------------------------------------------------------------------
   general window and frame
------------------------------------------------------------------------------*/
// IWindow Styles
#define WS_PARENTCLIP   0x00000000
#define WS_SAVEBITS     0x00000000
#define WS_SYNCPAINT    0x00000000
#define WS_TABSTOP      0x00010000
#define WS_GROUP        0x00020000
#define WS_CLIPCHILDREN 0x02000000
#define WS_CLIPSIBLINGS 0x04000000
#define WS_DISABLED     0x08000000
#define WS_VISIBLE      0x10000000

// IFrameWindow styles
#define  WS_MAXIMIZEBOX  0x00010000
#define  WS_MINIMIZEBOX  0x00020000
#define  WS_THICKFRAME   0x00040000
#define  WS_SYSMENU      0x00080000
#define  WS_HSCROLL      0x00100000
#define  WS_VSCROLL      0x00200000
#define  WS_BORDER       0x00800000
#define  WS_CAPTION      0x00C00000
#define  WS_MAXIMIZE     0x01000000
#define  WS_MINIMIZE     0x20000000
// extended frame styles.
#define  IFS_DEFERCREATE 0x00000100
#define  IFS_ACCELERATOR 0x00000200
#define  IFS_MENU        0x00000400
#define  IFS_MINICON     0x00000800
#define  IFS_ALIGNNOADJ  0x00001000
#define  IFS_SHELLPOS    0x00002000
#define  IFS_DLGBACKGND  0x00800000
#define  IFS_ANIMATED    0x00000000
#define  IFS_APPDBCS     0x00000000
#define  IFS_NOMOVEOWNER 0x00000000
#define  IFS_SYSMODAL    0x00000000
#define  WS_EX_APPWINDOW 0x00040000
#define  WS_EX_DLGMODALFRAME 0x00000001

#define WS_MULTISELECT             0x00040000L
#define WS_ANIMATE                 0x00400000L


/****************/
/* Entry Field. */
/****************/

// the following are the windows values from winuser.h
#define ES_LEFT             0x00000000L
#define ES_CENTER           0x00000001L
#define ES_RIGHT            0x00000002L
#define ES_MULTILINE        0x00000004L
#define ES_UPPERCASE        0x00000008L
#define ES_LOWERCASE        0x00000010L
#define ES_PASSWD           0x00000020L
#define ES_AUTOVSCROLL      0x00000040L
#define ES_AUTOHSCROLL      0x00000080L
#define ES_NOHIDESEL        0x00000100L
#define ES_OEMCONVERT       0x00000400L
#define ES_READONLY         0x00000800L
#define ES_WANTRETURN       0x00001000L
#define ES_NUMBER           0x00002000L
//#define WS_BORDER           0x00800000L

#define ES_AUTOSCROLL       ES_AUTOHSCROLL
#define ES_MARGIN           WS_BORDER
#define ES_AUTOTAB          0
#define ES_COMMAND          0
#define ES_UNREADABLE       ES_PASSWD
#define ES_SBCS             0
#define ES_DBCS             0

/*--------------------------------------------------------------------*/
/* Entry field control extended styles                                */
/*--------------------------------------------------------------------*/
#define IES_LEFTALIGN       0x00000001ul   //IEntryField::leftAlign
#define IES_ANYDATA         0x00000000ul   //no equivalent in Motif
#define IES_MIXEDDATA       0x00000000ul

#define IWS_BORDER3D        0   // nop on motif

/************/
/* List Box */
/************/
#define LS_MULTIPLESEL             0x00000001
#define LS_OWNERDRAW               0x00000002
#define LS_NOADJUSTPOS             0x00000004
#define LS_HORZSCROLL              0x00000008
#define LS_EXTENDEDSEL             0x00000010
#define LS_VERTSCROLL              0x00000100
#define LS_SCROLLSTATIC            0x00000200
#define LS_AUTOSELECT              0x00000400
#define IC_LS_AUTOSCROLL           0x00000020

/********************************************/
/* List Box Constants                  @RKK */
/* (Values are based on os2/pmwin.h)   @RKK */
/********************************************/
#define LIT_ERROR                  (-3)
#define LIT_MEMERROR               (-2)
#define LIT_NONE                   (-1)
#define LIT_END                    (-1)

/********************************************/
/* List Box Messages                   @RKK */
/* (Values are based on os2/pmwin.h)   @RKK */
/********************************************/
#define LM_INSERTITEM              0x0161
#define LM_DELETEITEM              0x0163
#define LM_DELETEALL               0x016e

/*************/
/* Combo Box */
/*************/
#define CBS_SIMPLE                 0x0001L
#define CBS_DROPDOWN               0x0002L
#define CBS_DROPDOWNLIST           0x0004L
#define IC_CBS_ANYDATA             0x0200L
#define IC_CBS_MIXEDDATA           0x4000L

/***************/
/* Spin Button */
/***************/
#define SPBS_MASTER                0x0010L
#define IC_SPBS_SERVANT            0x4000L
#define IC_SPBS_ALLCHARACTERS      0x8000L
#define SPBS_NUMERICONLY           0x0001L
#define SPBS_READONLY              0x0002L
#define SPBS_JUSTLEFT              0x0008L
#define IC_SPBS_JUSTCENTER         0x2000L
#define SPBS_JUSTRIGHT             0x0004L
#define SPBS_NOBORDER              0x0020L
#define SPBS_FASTSPIN              0x0100L
#define SPBS_PADWITHZEROS          0x0080L

/**********/
/* Slider */
/**********/
#define IC_SLS_HORIZONTAL          0x0800L
#define SLS_VERTICAL               0x0001L
#define IC_SLS_CENTER              0x1000L
#define SLS_BOTTOM                 0x0002L
#define SLS_TOP                    0x0004L
#define SLS_LEFT                   0x0002L
#define SLS_RIGHT                  0x0004L
#define SLS_SNAPTOINCREMENT        0x0008L
#define SLS_BUTTONSBOTTOM          0x0010L
#define SLS_BUTTONSTOP             0x0020L
#define SLS_BUTTONSLEFT            0x0010L
#define SLS_BUTTONSRIGHT           0x0020L
#define SLS_OWNERDRAW              0x0040L
#define SLS_READONLY               0x0080L
#define SLS_RIBBONSTRIP            0x0100L
#define IC_SLS_HOMEBOTTOM          0x2000L
#define SLS_HOMETOP                0x0200L
#define IC_SLS_HOMELEFT            0x2000L
#define SLS_HOMERIGHT              0x0200L
#define IC_SLS_PRIMARYSCALE1       0x4000L
#define SLS_PRIMARYSCALE2          0x0400L
#define IC_SLS_THUMBSIZE           30L

/*******/
/* MLE */
/*******/
#define MLS_WORDWRAP               0x00000001L
#define MLS_BORDER                 0x00000002L
#define MLS_VSCROLL                0x00000004L
#define MLS_HSCROLL                0x00000008L
#define MLS_READONLY               0x00000010L
#define MLS_IGNORETAB              0x00000020L
#define MLS_DISABLEUNDO            0x00000040L

/***************/
/* Static Text */
/***************/
#define DT_LEFT                    0x0000L
#define DT_QUERYEXTENT             0x0002L
#define DT_UNDERSCORE              0x0010L
#define DT_STRIKEOUT               0x0020L
#define DT_TEXTATTRS               0x0040L
#define DT_EXTERNALLEADING         0x0080L
#define DT_CENTER                  0x0100L
#define DT_RIGHT                   0x0200L
#define DT_TOP                     0x0000L
#define DT_VCENTER                 0x0400L
#define DT_BOTTOM                  0x0800L
#define DT_HALFTONE                0x1000L
#define DT_MNEMONIC                0x2000L
#define DT_WORDBREAK               0x4000L
#define DT_ERASERECT               0x8000L


/**********/
/* Button */
/**********/
#define BS_PUSHBUTTON              0L
#define BS_CHECKBOX                1L
#define BS_AUTOCHECKBOX            2L
#define BS_RADIOBUTTON             3L
#define BS_AUTORADIOBUTTON         4L
#define BS_BITMAP                  0x0040L
#define BS_ICON                    0x0080L
#define BS_PRIMARYSTYLES           0x000fL
#define BS_HELP                    0x0100L
#define BS_SYSCOMMAND              0x0200L
#define BS_DEFAULT                 0x0400L
#define BS_NOPOINTERFOCUS          0x0800L
#define BS_NOBORDER                0x1000L
#define BS_NOCURSORSELECT          0x2000L

#define IBS_DEFAULT         0x00000100ul   // default button overlaps in win
#define IBS_HELP            0x00000200ul   // not a native style in win
#define IBS_SYSCOMMAND      0x00000400ul   // not a native style in win
#define IBS_NOPOINTERFOCUS  0x00000800ul   // not a native style in win
#define IBS_ICON            0x00001000ul   // BS_ICON not a native style in NT
#define IBS_NOBORDER        0x00002000ul

/*************************/
/* Static Control Styles */
/*************************/
#define SS_GROUPBOX                0x0002L
/*#define SS_BITMAP                  0x0004L*/

/****************************************/
/* Indices into the default color table */
/****************************************/
#if 0   // ipfx defines its own values and we don't seem to be using these
#define CLR_WHITE                    -2
#define CLR_BLACK                    -1
#define CLR_BLUE                      1
#define CLR_RED                       2
#define CLR_PINK                      3
#define CLR_GREEN                     4
#define CLR_CYAN                      5
#define CLR_YELLOW                    6
#define CLR_DARKGRAY                  7
#define CLR_DARKBLUE                  8
#define CLR_DARKRED                   9
#define CLR_DARKPINK                  10
#define CLR_DARKGREEN                 11
#define CLR_DARKCYAN                  12
#define CLR_BROWN                     13
#define CLR_PALEGRAY                  14
#endif

/**************************************/
/* Message Box Styles & return values */
/**************************************/
#define MB_OK                      0x0000
#define MB_OKCANCEL                0x0001
#define MB_RETRYCANCEL             0x0002
#define MB_ABORTRETRYIGNORE        0x0003
#define MB_YESNO                   0x0004
#define MB_YESNOCANCEL             0x0005
#define MB_CANCEL                  0x0006
#define MB_ENTER                   0x0007
#define MB_ENTERCANCEL             0x0008
#define MB_NOICON                  0x0000
#define MB_CUANOTIFICATION         0x0000
#define MB_ICONQUESTION            0x0010
#define MB_ICONEXCLAMATION         0x0020
#define MB_CUAWARNING              0x0020
#define MB_ICONASTERISK            0x0030
#define MB_ICONHAND                0x0040
#define MB_CUACRITICAL             0x0040
#define MB_QUERY                   MB_ICONQUESTION
#define MB_WARNING                 MB_CUAWARNING
#define MB_INFORMATION             MB_ICONASTERISK
#define MB_CRITICAL                MB_CUACRITICAL
#define MB_ERROR                   MB_CRITICAL
#define MB_DEFBUTTON1              0x0000
#define MB_DEFBUTTON2              0x0100
#define MB_DEFBUTTON3              0x0200
#define MB_APPLMODAL               0x0000
#define MB_SYSTEMMODAL             0x1000
#define MB_HELP                    0x2000
#define MB_MOVEABLE                0x4000
#define MBID_OK                    1
#define MBID_CANCEL                2
#define MBID_ABORT                 3
#define MBID_RETRY                 4
#define MBID_IGNORE                5
#define MBID_YES                   6
#define MBID_NO                    7
#define MBID_HELP                  8
#define MBID_ENTER                 9
#define MBID_ERROR                 0xffff

/**************************************/
/* File Dialog Styles & return values */
/**************************************/
#define FDS_CENTER           0x00000001L /* Center within owner wnd   */
#define FDS_CUSTOM           0x00000002L /* Use custom user template  */
#define FDS_FILTERUNION      0x00000004L /* Use union of filters      */
#define FDS_HELPBUTTON       0x00000008L /* Display Help button       */
#define FDS_APPLYBUTTON      0x00000010L /* Display Apply button      */
#define FDS_PRELOAD_VOLINFO  0x00000020L /* Preload volume info       */
#define FDS_MODELESS         0x00000040L /* Make dialog modeless      */
#define FDS_INCLUDE_EAS      0x00000080L /* Always load EA info       */
#define FDS_OPEN_DIALOG      0x00000100L /* Select Open dialog        */
#define FDS_SAVEAS_DIALOG    0x00000200L /* Select SaveAs dialog      */
#define FDS_MULTIPLESEL      0x00000400L /* Enable multiple selection */
#define FDS_ENABLEFILELB     0x00000800L /* Enable SaveAs Listbox     */

/*--------------------------------------------------------------------*/
/* File Dialog - dialog and control ids                               */
/*--------------------------------------------------------------------*/
#define   DID_FILE_DIALOG             256
#define   DID_FILENAME_TXT            257
#define   DID_FILENAME_ED             258
#define   DID_DRIVE_TXT               259
#define   DID_DRIVE_CB                260
#define   DID_FILTER_TXT              261
#define   DID_FILTER_CB               262
#define   DID_DIRECTORY_TXT           263
#define   DID_DIRECTORY_LB            264
#define   DID_FILES_TXT               265
#define   DID_FILES_LB                266
#define   DID_HELP_PB                 267
#define   DID_APPLY_PB                268
#define   DID_OK_PB                   DID_OK
#define   DID_CANCEL_PB               DID_CANCEL

/**********************************************************************/
/* Font Dialog Style Flags                                            */
/**********************************************************************/
#define FNTS_CENTER              0x00000001L /* Center in owner dialog*/
// #define FNTS_CUSTOM              0x00000002L /* Use custom template   */
// #define FNTS_OWNERDRAWPREVIEW    0x00000004L /* Allow app to draw     */
#define FNTS_HELPBUTTON          0x00000008L /* Display Help button   */
#define FNTS_APPLYBUTTON         0x00000010L /* Display Apply button  */
#define FNTS_RESETBUTTON         0x00000020L /* Display Reset button  */
#define FNTS_MODELESS            0x00000040L /* Make dialog modeless  */
#define FNTS_INITFROMFATTRS      0x00000080L /* Initialize from FATTRs*/
#define FNTS_BITMAPONLY          0x00000100L /* Only allow bitmap font*/
#define FNTS_VECTORONLY          0x00000200L /* Only allow vector font*/
#define FNTS_FIXEDWIDTHONLY      0x00000400L /* Only allow monospaced */
#define FNTS_PROPORTIONALONLY    0x00000800L /* Only proportional font*/
#define FNTS_NOSYNTHESIZEDFONTS  0x00001000L /* Don't synthesize fonts*/

/**********************************************************************/
/* font dialog and control id's                                       */
/**********************************************************************/
#define DID_FONT_DIALOG          300
#define DID_NAME                 301
#define DID_STYLE                302
#define DID_DISPLAY_FILTER       303
#define DID_PRINTER_FILTER       304
#define DID_SIZE                 305
#define DID_SAMPLE               306
#define DID_OUTLINE              307
#define DID_UNDERSCORE           308
#define DID_STRIKEOUT            309
#define DID_HELP_BUTTON          310
#define DID_APPLY_BUTTON         311
#define DID_RESET_BUTTON         312
#define DID_OK_BUTTON            DID_OK
#define DID_CANCEL_BUTTON        DID_CANCEL
#define DID_NAME_PREFIX          313
#define DID_STYLE_PREFIX         314
#define DID_SIZE_PREFIX          315
#define DID_SAMPLE_GROUPBOX      316
#define DID_EMPHASIS_GROUPBOX    317
#define DID_FONT_ISO_SUPPORT     318
#define DID_FONT_ISO_UNTESTED    319
#define DID_CANVAS               320   // added for Motif
#define DID_CANVAS2              321   // added for Motif
#define DID_STYLEBOX             322
#define DID_STYLEBOXLABEL        323

/***************************/
/* Presentation Parameters */
/***************************/
#define PP_FOREGROUNDCOLOR                      1L
#define PP_BACKGROUNDCOLOR                      3L
#define PP_HILITEFOREGROUNDCOLOR                5L
#define PP_HILITEBACKGROUNDCOLOR                7L
#define PP_DISABLEDFOREGROUNDCOLOR              9L
#define PP_DISABLEDBACKGROUNDCOLOR              11L
#define PP_BORDERCOLOR                          13L
#define PP_FONTNAMESIZE                         15L
#define PP_FONTHANDLE                           16L
#define PP_RESERVED                             17L
#define PP_ACTIVECOLOR                          18L
#define PP_INACTIVECOLOR                        20L
#define PP_ACTIVETEXTFGNDCOLOR                  22L
#define PP_ACTIVETEXTBGNDCOLOR                  24L
#define PP_INACTIVETEXTFGNDCOLOR                26L
#define PP_INACTIVETEXTBGNDCOLOR                28L
#define PP_SHADOW                               30L
#define PP_MENUFOREGROUNDCOLOR                  31L
#define PP_MENUBACKGROUNDCOLOR                  33L
#define PP_MENUHILITEFGNDCOLOR                  35L
#define PP_MENUHILITEBGNDCOLOR                  37L
#define PP_MENUDISABLEDFGNDCOLOR                39L
#define PP_MENUDISABLEDBGNDCOLOR                41L
#define PP_TROUGHCOLOR                          42L
/****************************/
/* GroupBox Controls Styles */
/****************************/
#define GB_RADIO                   0x4000L

/*********************************/
/* Menu Item Styles & Attributes */
/*********************************/

#define MIS_TEXT                   0x0001L
#define MIS_BITMAP                 0x0002L
#define MIS_SEPARATOR              0x0004L
#define MIS_OWNERDRAW              0x0008L
#define MIS_SUBMENU                0x0010L
#define MIS_MULTMENU               0x0020L /* multiple choice submenu     */
#define MIS_SYSCOMMAND             0x0040L
#define MIS_HELP                   0x0080L
#define MIS_STATIC                 0x0100L
#define MIS_BUTTONSEPARATOR        0x0200L
#define MIS_BREAK                  0x0400L
#define MIS_BREAKSEPARATOR         0x0800L
#define MIS_GROUP                  0x1000L /* multiple choice group start */
/* In multiple choice submenus a style of 'single' denotes the item is a
** radiobutton.  Absence of this style defaults the item to a checkbox.   */
#define MIS_SINGLE                 0x2000L
#define MIA_NODISMISS              0x0020L
#define MIA_FRAMED                 0x1000L
#define MIA_CHECKED                0x2000L
#define MIA_DISABLED               0x4000L
#define MIA_HILITED                0x8000L

#define MIT_END                    (-1)
#define MIT_NONE                   (-1)
#define MS_VERTICALFLIP           0x00000004L

#define KEYSTRING     "<Key>"
#define ALTSTRING     "Alt"
#define CTRLSTRING    "Ctrl"
#define SHIFTSTRING   "Shift"


/******************************************************************/
/* SC_ messages.   These values allow support of certain system   */
/* command messages as menu item ids.   This is primarily to      */
/* support use of SC_HELPEXTENDED, SC_HELPKEYS, and SC_HELPINDEX  */
/* in user defined menus.   Currently the only WM_SYSCOMMAND      */
/* events generated are the help related ones.                    */
/******************************************************************/
// the base is a large integer to avoid conflict with user defined
// message IDs, which PM limits to 16 bits.
#define HELP_FROMMENU_BIT   0x20000000
#define SC_SIZE             0x8000
#define SC_MOVE             0x8001
#define SC_MINIMIZE         0x8002
#define SC_MAXIMIZE         0x8003
#define SC_CLOSE            0x8004
#define SC_SYSMENU          0x8007
#define SC_RESTORE          0x8008
#define SC_TASKMANAGER      0x8011
#define SC_HELPKEYS         0x8012
#define SC_HELPINDEX        0x8013
#define SC_HELPEXTENDED     0x8014
#define SC_HIDE             0x802A

/*******************************************************************/
/*Help messages and defines.  Most of the help defines are defined */
/*by ipfx.h, but there are several we define in ICLUI for PM       */
/*compatibility.                                                   */
/*******************************************************************/
// the ipfx.h file uses 1000 thru 1034 currently and some in 2000 range
#define HMERR_NO_FRAME_WND_IN_CHAIN 1100
#define HMERR_NO_HELP_INST_IN_CHAIN 1101
#define HLPM_FRAME  0x0001
#define HLPM_WINDOW 0x0002
#define HLPM_MENU   0x0004

// Define the HELPINFO structure.  Members are named exactly like Win32 SDK
// structure for source code compatibility.
#define HELPINFO_WINDOW    0x0001
#define HELPINFO_MENUITEM  0x0002

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

typedef struct tagHELPINFO      /* Structure pointed to by lParam of WM_HELP */
{
    unsigned int  cbSize;       /* Size in bytes of this struct  */
    int           iContextType; /* Either HELPINFO_WINDOW or HELPINFO_MENUITEM */
    int           iCtrlId;      /* Control Id or a Menu item Id. */
    _WidgetRec*   hItemHandle;  /* hWnd of control or hMenu.     */
    unsigned long dwContextId;  /* Context Id associated with this item */
//    POINT         MousePos;     /* Mouse Position in screen co-ordinates */
}  HELPINFO, *LPHELPINFO;

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

/*************/
/* ScrollBar */
/*************/
#define SBS_VERT                   0x00000001L
#define SBS_HORZ                   0x00000002L
#define SBS_AUTOSIZE               0x00000004L

/************************/
/* Application defaults */
/************************/
#define DEFAULTAPPCLASS "icluiclass"

/*** WM_COMMAND msg source codes ****************************************/
#define CMDSRC_MENU            0       //value used by Windows
#define CMDSRC_ACCELERATOR     1       //value used by Windows
#define CMDSRC_PUSHBUTTON      2       //value defined for ICLUI
#define CMDSRC_OTHER           3       //value defined for ICLUI


/**********************/
/* Frame control IDs  */
/**********************/
#define FID_SYSMENU                0x8002
#define FID_TITLEBAR               0x8003
#define FID_MINMAX                 0x8004
#define FID_MENU                   0x8005
#define FID_VERTSCROLL             0x8006
#define FID_HORZSCROLL             0x8007
#define FID_CLIENT                 0x8008

/*************************/
/* WM_CHAR fs field bits */
/*************************/
#define KC_CHAR                   0x0001
#define KC_VIRTUALKEY             0x0002
#define KC_SCANCODE               0x0004
#define KC_SHIFT                  0x0008
#define KC_CTRL                   0x0010
#define KC_ALT                    0x0020
#define KC_KEYUP                  0x0040
#define KC_PREVDOWN               0x0080
#define KC_LONEKEY                0x0100
#define KC_DEADKEY                0x0200
#define KC_COMPOSITE              0x0400
#define KC_INVALIDCOMP            0x0800

/**********************/
/* Notebook styles    */
/**********************/
/**********************************************************************/
/* Page insertion/query styles (usPageStyle,usQueryEnd)               */
/**********************************************************************/
#define BKA_STATUSTEXTON         0x0001  /* status area text          */
#define BKA_MAJOR                0x0040  /* Major Tab                 */
#define BKA_MINOR                0x0080  /* Minor Tab                 */
#define BKA_AUTOPAGESIZE         0x0100  /* Page window position/size */
#define BKA_END                  0x0200  /* Query to end of book      */

/**********************************************************************/
/* Tab window contents (usTabDisplay)                                 */
/**********************************************************************/
#define BKA_TEXT                 0x0400  /* text data                 */
#define BKA_BITMAP               0x0800  /* bitmap                    */

/**********************************************************************/
/* Notebook window styles (ulNotebookStyles)                          */
/**********************************************************************/

/**********************************************************************/
/* Backpage Orientation                                               */
/**********************************************************************/
#define BKS_BACKPAGESBR          0x00000001  /* Bottom right          */
#define BKS_BACKPAGESBL          0x00000002  /* Bottom left           */
#define BKS_BACKPAGESTR          0x00000004  /* Top right             */
#define BKS_BACKPAGESTL          0x00000008  /* Top left              */

/**********************************************************************/
/* Major Tab Side                                                     */
/**********************************************************************/
#define BKS_MAJORTABRIGHT        0x00000010  /* Major tabs right      */
#define BKS_MAJORTABLEFT         0x00000020  /* Major tabs left       */
#define BKS_MAJORTABTOP          0x00000040  /* Major tabs top        */
#define BKS_MAJORTABBOTTOM       0x00000080  /* Major tabs bottom     */

/**********************************************************************/
/* Tab Type                                                           */
/**********************************************************************/
#define BKS_SQUARETABS           0x00000000  /* Square edged tabs     */
#define BKS_ROUNDEDTABS          0x00000100  /* Round edged tabs      */
#define BKS_POLYGONTABS          0x00000200  /* Polygon edged tabs    */

/**********************************************************************/
/* Binding type                                                       */
/**********************************************************************/
#define BKS_SOLIDBIND            0x00000000  /* Solid binding         */
#define BKS_SPIRALBIND           0x00000400  /* Spiral binding        */

/**********************************************************************/
/* Status line text justification                                     */
/**********************************************************************/
#define BKS_STATUSTEXTLEFT       0x00000000  /* Left justify text     */
#define BKS_STATUSTEXTRIGHT      0x00001000  /* Right justify text    */
#define BKS_STATUSTEXTCENTER     0x00002000  /* Center text           */

/**********************************************************************/
/* Tab text justification                                             */
/**********************************************************************/
#define BKS_TABTEXTLEFT          0x00000000  /* Left justify tab text */
#define BKS_TABTEXTRIGHT         0x00004000  /* Right justify tab text*/
#define BKS_TABTEXTCENTER        0x00008000  /* Center tab text       */

/***************/
/* Outline Box */
/***************/
#define OB_BACKGROUNDFRAME         0x0001L
#define OB_FOREGROUNDFRAME         0x0002L
#define OB_HALFTONEFRAME           0x0004L

/*
 * Static Control Constants
 */
#define SS_LEFT             0x00000000L
#define SS_CENTER           0x00000001L
#define SS_RIGHT            0x00000002L
#define SS_ICON             0x00000003L
#define SS_BLACKRECT        0x00000004L
#define SS_GRAYRECT         0x00000005L
#define SS_WHITERECT        0x00000006L
#define SS_BLACKFRAME       0x00000007L
#define SS_GRAYFRAME        0x00000008L
#define SS_WHITEFRAME       0x00000009L
#define SS_USERITEM         0x0000000AL
#define SS_SIMPLE           0x0000000BL
#define SS_LEFTNOWORDWRAP   0x0000000CL
#define SS_BITMAP           0x0000000EL
#define SS_OWNERDRAW        0x0000000DL
#define SS_ENHMETAFILE      0x0000000FL
#define SS_ETCHEDHORZ       0x00000010L
#define SS_ETCHEDVERT       0x00000011L
#define SS_ETCHEDFRAME      0x00000012L
#define SS_TYPEMASK         0x0000001FL
#define SS_NOPREFIX         0x00000080L /* Don't do "&" character translation */
#define SS_NOTIFY           0x00000100L
#define SS_CENTERIMAGE      0x00000200L
#define SS_RIGHTJUST        0x00000400L
#define SS_REALSIZEIMAGE    0x00000800L

#define SS_BKGNDRECT             SS_WHITERECT
#define SS_FGNDRECT              SS_BLACKRECT
#define SS_HALFTONERECT          SS_GRAYRECT
#define SS_BKGNDFRAME            SS_WHITEFRAME
#define SS_FGNDFRAME             SS_BLACKFRAME
#define SS_HALFTONEFRAME         SS_GRAYFRAME
//#define DT_QUERYEXTENT           DT_CALCRECT


/**********************************************************************/
/* Window position flags (usage is like PM)                           */
/**********************************************************************/
#define SWP_MOVE     0x0001
#define SWP_SIZE     0x0002
#define SWP_NOADJUST 0x0004
#define SWP_HIDE     0x0008
#define SWP_ZORDER   0x0010
#define SWP_SHOW     0x0020

#define HWND_TOP     (_WidgetRec *)0      
#define HWND_BOTTOM  (_WidgetRec *)1      

/* ----------------------- ICL misc values ---------------------------*/
#define TID_USERMAX           0x7FFF     // From PMWIN.H (Max number of timers)


//**********************************************************************
//* The following functions have definitions which map fairly          *
//* well between environments except as specified                      *
//**********************************************************************


  //
  // If we convert the window identifier to a character string
  // and use that as the name of the widget on XmCreate...
  // then we can extract the name from the widget instead
  // of keeping a local copy of the (unused) resource ID.
  // (ie. IString resid( XtName( (Widget)handle() );
  //      return ( resid.asInt() );                     )
  //*** EXCEPT it doesn't work if this was created from an existing
  //*** window handle. In this case, the widget name is really a string.
  //*** In this case, the return value here is undefined. For now, watch
  //*** for problems; we assume it's safer to do it this way & insure the id
  //*** stays with the widget than to keep a local copy & guard against misuse.
  //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< >>>>>>>>>>>>>>>>>>>>>>>>>>>>
inline _WidgetRec*   IPARENTOF(_WidgetRec* hwnd)
                           {return XtParent(hwnd); };
inline unsigned long IIDOF(_WidgetRec* hwnd)
                           {return strtol(XtName(hwnd), 0 ,10); };

#define IISWINDOW(hab,hwnd)        (XtIsSubclass( (Widget)hwnd, widgetClass ))
#define IWINDOWFROMID(hwnd,id)     (XtNameToWidget(hwnd,               \
                                                  (char *)IString(id)))
#define ISHOWWINDOW(hwnd,sw) sw ? XtManageChild(hwnd) : XtUnmanageChild(hwnd)
#define IENABLEWINDOW(hwnd,sw)  XtSetSensitive( hwnd, sw )
#define ISETIDOF(hwnd,oldhwnd) (0)

/* This value is used to determine that we unmanaged a widget because it was sized
   to 0. */
#define SIZED_TO_0 65232

/**********************************************************************/
/* Definitions for accelerator key support.                           */
/**********************************************************************/
#pragma enum(4)
#pragma pack(push,4)

struct ACCEL {
  unsigned short modifiers;
  unsigned short flags;
  KeySym         key;
  unsigned short commandId;
  };
struct ACCELTABLE {
  unsigned short version;
  unsigned short accelNumber;
  unsigned short codepage;
  ACCEL          accelerators[ 1 ];
  };

#pragma pack(pop)
#pragma enum(pop)

#endif /* OS2_INCLUDED */
#endif
