// Revision: 24 1.2 source/ui/baseapp/win/ibrshmgr.hpp, basewin, ioc.v400, 000302 
#ifndef _IBRSHMGR_
#define _IBRSHMGR_
/*******************************************************************************
* FILE NAME: ibrshmgr.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IBrushElement IBrushManager                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #include <iwindefs.h>
}
#include <ikeyset.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)              
#pragma pack(push,4)

class IColor;

/******************************************************************************
* class IBrushElement                                                             *
*******************************************************************************/
class IBrushElement {

public:
/*------------------------------- Constructors -------------------------------*/
  IBrushElement(long lClr);
  IBrushElement();
  ~IBrushElement();

/*------------------------------ Public Members ------------------------------*/
HBRUSH  
  handle() const;
long    
  color() const;

long    
  useBrush(),
  freeBrush(),
  useCount();

bool 
  operator==(IBrushElement& );

// Required to support IBrushElement's in IKeySet's
friend const long&       
  key(const IBrushElement* k);

/*--------------------------------- Private ----------------------------------*/
private:
    long     lColor;
    HBRUSH   hBrush;
    long     lUseCount;
};

/******************************************************************************
* class IBrushManager                                                          *
*******************************************************************************/
class IBrushManager {

public:

typedef IKeySet<IBrushElement*, long > BrushList;

/*------------------------------ Public Members ------------------------------*/
static HBRUSH  
  findBrush( const IColor& lClr );
static long
  useBrush( HBRUSH hBr);
static void    
  freeBrush( HBRUSH hBr );
static void    
  freeBrush( const IColor& lClr );
static long    
  brushCount( );

protected:
/*------------------------------ Hidden Members ------------------------------*/
static IBrushElement* 
  IBrushManager::brushElement( const IColor & lClr );
static IBrushElement* 
  IBrushManager::brushElement( HBRUSH hBr );

/*--------------------------------- Private ----------------------------------*/
private:

static BrushList 
  brushList;
static long 
  lBrushCount;
};

#pragma pack(pop)    
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif

