// Revision: 26 1.5 source/ui/baseapp/win/ibrushdr.cpp, basewin, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ibrushdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibrushdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
{
  #define INCL_WINSTDSLIDER
  #include <iwindefs.h>
}

#include <ibrushdr.hpp>
#include <ibrshmgr.hpp>
#include <icombobx.hpp>
#include <iclsldw.h>
#include <iwindow.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>
#include <icolor.hpp>
#include <iwcname.hpp>
                         
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBRUSHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBrushHandler::IBrushHandler                                                 |
|                                                                              |
| Constructor for IBrushHandler.                                               |
------------------------------------------------------------------------------*/
IBrushHandler::IBrushHandler ( )
  : IHandler()
{
  IFUNCTRACE_DEVELOP();   
}

  
/*------------------------------------------------------------------------------
| IBrushHandler::~IBrushHandler                                                |
|                                                                              |
| Destructor for IBrushHandler.                                                |
------------------------------------------------------------------------------*/
IBrushHandler::~IBrushHandler ( )
{
   IFUNCTRACE_DEVELOP();   
}

/*------------------------------------------------------------------------------
| IBrushHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Win: Set the brush in the event for windows to use.                          |
------------------------------------------------------------------------------*/
bool IBrushHandler::dispatchHandlerEvent(IEvent& event)
{
  bool fStopDispatch = false;
  switch (event.eventId())
  {
    case WM_CTLCOLOREDIT:
    case WM_CTLCOLORSTATIC:
    case WM_CTLCOLORSLIDER:
    case WM_CTLCOLORLISTBOX:
    {
      // the parent case is needed for combobox.
      if ( (event.dispatchingWindow()->handle() == event.parameter2()) || 
           (event.dispatchingWindow()->handle() == IPARENTOF(event.parameter2())) ||
           (IWindow::desktopWindow()->handle() == IPARENTOF(event.parameter2())) )
      {
        IWindow* ctlwin = IWindow::windowWithHandle(IWindowHandle(event.parameter2()));
        
        // if the control window is a combo box then the event.parameter2 may be
        // the entry field or the combo box( parent ). In that case use the control
        // window instead.
        IComboBox *comboWin =
          dynamic_cast<IComboBox *>(event.controlWindow());
        if (comboWin)
        {
          ctlwin = event.controlWindow();
        }

        if (ctlwin)
        {
          IColor clrFgnd = ctlwin->foregroundColor();
          IColor clrBgnd = ctlwin->backgroundColor();

          // Return HBRUSH for painting controls background
          HBRUSH fNewBrush = IBrushManager::findBrush( clrBgnd );
          // note that the above line only finds the brush, it does not increment its
          // use count.
          if (fNewBrush)
          {
            // Set foreground and background text colors in device context
            SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
            SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

            // the result should be the brush that should be used for the painting of the
            // background.
            event.setResult( fNewBrush );
         
            ITRACE_ALL( " BrushHandler:: Brush " + IString((unsigned long)fNewBrush).d2x() +
                            " for window " + IString( (unsigned long) ctlwin ).d2x() +
                            " with handle " + IString( (unsigned long) (void *)ctlwin->handle()).d2x() +
                            " for color " + IString( clrBgnd.asRGBLong() ).d2x() );
          }
        }
        fStopDispatch = true;
      }
      break;
    }

    case WM_CTLCOLORSCROLLBAR:
    {
      // For scrollbars, use the foreground color for the background brush to
      // maintain portability.  We also don't need to call SetBkColor or
      // SetTextColor for scrollbars since these have no effect.
      if (event.dispatchingWindow()->handle() == event.parameter2())
      {
        IWindow* ctlwin = IWindow::windowWithHandle(IWindowHandle(event.parameter2()));
        
        if (ctlwin)
        {
          IColor clrFgnd = ctlwin->foregroundColor();

          // Return HBRUSH for painting controls background
          HBRUSH fNewBrush = IBrushManager::findBrush( clrFgnd );
          
          if (fNewBrush)
          {
            // the result should be the brush that should be used for the painting of the
            // background.
            event.setResult( fNewBrush );
         
            ITRACE_ALL( " BrushHandler:: Brush " + IString((unsigned long)fNewBrush).d2x() +
                            " for window " + IString( (unsigned long) ctlwin ).d2x() +
                            " with handle " + IString( (unsigned long) (void *)ctlwin->handle()).d2x() +
                            " for color " + IString( clrFgnd.asRGBLong() ).d2x() );
          }
        }
        fStopDispatch = true;
      }
    }
    break;
  }
  return fStopDispatch;
}
