// Revision: 24 1.11.1.8 source/ui/basectl/i3statbx.cpp, buttonctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: i3statbx.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in i3statbx.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINBUTTONS
  #define INCL_WINERRORS
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <i3statbx.hpp>
#include <ibidiset.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <icolor.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <inotifev.hpp>
#ifdef IC_WIN
 #include <ibrushdr.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _I3STATBX_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public three-state check box styles.                                         |
------------------------------------------------------------------------------*/
const I3StateCheckBox::Style
  I3StateCheckBox::autoSelect        ( 0, IBS_AUTO3STATE ),
  I3StateCheckBox::classDefaultStyle ( WS_VISIBLE, IBS_AUTO3STATE );


/*------------------------------------------------------------------------------
| Default style for new three-state check box objects (initial value).         |
------------------------------------------------------------------------------*/
  I3StateCheckBox::Style
         I3StateCheckBox::currentDefaultStyle ( WS_VISIBLE, IBS_AUTO3STATE );

#ifdef IC_PM
/*------------------------------------------------------------------------------
| The I3StateCheckBoxData class only contains static data for OS/2 platform,   |
| so an object of this class is not required.  Once non-static data is added   |
| to this class, then the I3StateCheckBoxData constructors must be changed to  |
| create an object of this class.                                              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class I3StateCheckBoxData  {
public:
static ISize
  gBitmapSize;
};  // I3StateCheckBoxData

#pragma pack(pop)
#pragma enum(pop)

#endif // IC_PM

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| The I3StateCheckBoxData class encapsulates private data and functions        |
| used by the I3StateCheckBox class.  An object of this class is created in    |
| the I3StateCheckBox constructors.  This is the windows version.              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class I3StateCheckBoxData {
public:
  I3StateCheckBoxData ( );

virtual
 ~I3StateCheckBoxData ( );

static ISize
  gBitmapSize;

IBrushHandler
  fBrushHandler;
}; // I3StateCheckBoxData

#pragma pack(pop)
#pragma enum(pop)

#endif // IC_WIN

ISize I3StateCheckBoxData:: gBitmapSize ( 0, 0 );

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| I3StateCheckBoxData::I3StateCheckBoxData                                     |
|                                                                              |
------------------------------------------------------------------------------*/
I3StateCheckBoxData::I3StateCheckBoxData ( )
  : fBrushHandler()
{ }

/*------------------------------------------------------------------------------
| I3StateCheckBoxData::~I3StateCheckBoxData                                    |
|                                                                              |
------------------------------------------------------------------------------*/
I3StateCheckBoxData::~I3StateCheckBoxData ( )
{
}
#endif //IC_WIN

/*------------------------------------------------------------------------------
| I3StateCheckBox::I3StateCheckBox                                             |
|                                                                              |
| Constructor to create a three-state check box on a standard window.          |
------------------------------------------------------------------------------*/
I3StateCheckBox :: I3StateCheckBox(unsigned long ulId,
                                   IWindow* pwndParent,
                                   IWindow* pwndOwner,
                                   const IRectangle& rectInit,
                                   const Style& s3bsStyle)
#ifdef IC_WIN
                                 : f3StateCheckBoxData( new I3StateCheckBoxData )
#else
                                 : f3StateCheckBoxData( 0 )
#endif // IC_WIN
{
  IASSERTPARM(pwndParent!=0);          //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | s3bsStyle.asExtendedUnsignedLong() );

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (pwndParent)
    parentHandle = pwndParent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();

  unsigned long
    createStyle = this->convertToGUIStyle( s3bsStyle );

#ifdef IC_WIN
  // Check if we need to supply bidi support to turn the check box
  // into a right-to-left one.
  if ( IBidiSettings::isBidiSupported() )
  {
     // IWindow bidi styles take precedence over inherited bidi
     // attributes.
     if ( s3bsStyle & IWindow::rightToLeft )
     {  // Change the button into a right-to-left one.
        createStyle |= BS_RIGHTBUTTON;
     }
     else if ( ! ( s3bsStyle & IWindow::leftToRight ) )
     {
        // The check box inherits bidi attributes from its parent
        // window.
        IBidiSettings
          bidiValues( *pwndParent );
        if ( bidiValues.windowLayout() ==
                           IBidiSettings::layoutRightToLeft )
        {
           // Change the control into a right-to-left one.
           createStyle |= BS_RIGHTBUTTON;
        }
     }
  }
#endif // IC_WIN

  //Create three-state box control
  IWindowHandle wh3StateBox =
      this -> create( ulId,
                      0,
                      createStyle,
                      WC_BUTTON,
                      parentHandle,
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( s3bsStyle, true ) );

#ifdef IC_WIN
  f3StateCheckBoxData->fBrushHandler.handleEventsFor(this);
#endif
  startHandlingEventsFor(wh3StateBox);
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::I3StateCheckBox                                             |
|                                                                              |
| Constructor to instantiate an object for a a dialog template.                |
------------------------------------------------------------------------------*/
I3StateCheckBox :: I3StateCheckBox(unsigned long ulId,
                                   IWindow* pdlgwndParent)
#ifdef IC_WIN
                                 : f3StateCheckBoxData( new I3StateCheckBoxData )
#else
                                 : f3StateCheckBoxData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  f3StateCheckBoxData->fBrushHandler.handleEventsFor(this);
#endif
  reserveUserWindowWord( false );
  startHandlingEventsFor(ulId, pdlgwndParent);
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::I3StateCheckBox                                             |
|                                                                              |
| Constructor to instantiate from an existing three-state check box control.   |
------------------------------------------------------------------------------*/
I3StateCheckBox :: I3StateCheckBox(const IWindowHandle& wh)
#ifdef IC_WIN
                                 : f3StateCheckBoxData( new I3StateCheckBoxData )
#else
                                 : f3StateCheckBoxData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  f3StateCheckBoxData->fBrushHandler.handleEventsFor(this);
#endif
  reserveUserWindowWord( false );
  startHandlingEventsFor(wh);
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::~I3StateCheckBox                                            |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
I3StateCheckBox :: ~I3StateCheckBox()
{
#ifdef IC_WIN
  f3StateCheckBoxData->fBrushHandler.stopHandlingEventsFor(this);
#endif //IC_WIN
  if ( f3StateCheckBoxData != 0 )
    delete f3StateCheckBoxData;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::defaultStyle                                                |
|                                                                              |
| Return the default style for new three-state check box objects.              |
------------------------------------------------------------------------------*/
I3StateCheckBox::Style  I3StateCheckBox :: defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::setDefaultStyle                                             |
|                                                                              |
| Replace the default style for new three-state check box objects.             |
------------------------------------------------------------------------------*/
void  I3StateCheckBox :: setDefaultStyle(const Style& s3bsStyle)
{
  currentDefaultStyle = s3bsStyle;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long I3StateCheckBox :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   bool bExtOnly) const
{
  // Obtain the style from the class (ISettingButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXT3STATE;

    // Handle overlapping styles
    if (ulTmp & IBS_AUTO3STATE)
      ulStyle |= BS_AUTO3STATE;
    else
      ulStyle |= BS_3STATE;

#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::enableAutoSelect                                            |
|                                                                              |
| Sets or removes the autoSelect style for the three-state check box control.  |
------------------------------------------------------------------------------*/
I3StateCheckBox&  I3StateCheckBox :: enableAutoSelect(bool bTurnOn)
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking style from pmwin.h
    ulStyle |= BS_AUTO3STATE;
  }
  else
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking style from pmwin.h
    ulStyle |= BS_3STATE;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                          //Force repaint
  }
  return *this;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::disableAutoSelect                                           |
------------------------------------------------------------------------------*/
I3StateCheckBox&  I3StateCheckBox :: disableAutoSelect()
{
  enableAutoSelect(false);
  return *this;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::isAutoSelect                                                |
------------------------------------------------------------------------------*/
bool  I3StateCheckBox :: isAutoSelect() const
{
  return( (style() & BS_AUTO3STATE & (unsigned long)~(BS_3STATE)) ? true : false );
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::selectHalftone                                              |
|                                                                              |
| Selects a halftone three-state check box.  Halftone is equivalent to         |
| undetermined state.                                                          |
------------------------------------------------------------------------------*/
I3StateCheckBox&  I3StateCheckBox :: selectHalftone()
{
  if (!(this->isHalftone()))
  {
    handle().sendEvent(BM_SETCHECK, IEventParameter1(2),
                                    IEventParameter2(0));
  }
  return *this;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::isHalftone                                                  |
------------------------------------------------------------------------------*/
bool  I3StateCheckBox :: isHalftone() const
{
  IEventResult evt = handle().sendEvent(BM_QUERYCHECK,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  if (evt.asUnsignedLong() == 2)
     return true;
  else
     return false;
}


/*------------------------------------------------------------------------------
| I3StateCheckBox::calcMinimumSize                                             |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The minimum        |
| size is based on the maximum height and width of the font of the             |
| three-state check box text (plus the size of the check box itself).          |
------------------------------------------------------------------------------*/
ISize  I3StateCheckBox :: calcMinimumSize() const
{
   long bmpWidth = I3StateCheckBoxData::gBitmapSize.width();
   if ( bmpWidth == 0 )
   {                                    // Find size of 3state-checkbox bitmap.
#ifdef IC_PM
      BITMAPINFOHEADER pbmpData;
      pbmpData.cbFix=sizeof(BITMAPINFOHEADER);
#endif
#ifdef IC_WIN
      BITMAP           pbmpData;
#endif
      HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
      IQUERYBITMAPPARAMETERS(hbit, &pbmpData);
      IDELETEBITMAP(hbit);

#ifdef IC_PM
      bmpWidth = pbmpData.cx;
      I3StateCheckBoxData::gBitmapSize.setWidth( bmpWidth );
      I3StateCheckBoxData::gBitmapSize.setHeight( pbmpData.cy );
#endif
#ifdef IC_WIN
      bmpWidth = pbmpData.bmWidth;
      I3StateCheckBoxData::gBitmapSize.setWidth( bmpWidth );
      I3StateCheckBoxData::gBitmapSize.setHeight( pbmpData.bmHeight );
#endif
   }

   ISize size = displaySize(text());
   unsigned long ulWidth = size.width();
   unsigned long i=textLength();

   if (i==0)
     ulWidth += ((bmpWidth/4) + 2 + (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   else
     ulWidth += (ulWidth/i + (bmpWidth/4) + 2 +
                                    (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   unsigned long ulHeight = size.height() +
                                     (2 * IQUERYSYSVALUE(SV_CYBORDER));
   return ISize(ulWidth, ulHeight);
}
