/*******************************************************************************
* FILE NAME: ialgnatt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     This file contains the implementation of classes/functions for           *
*     horizontal and vertical alignment attribute support :                    *
*     IAlignmentAttribute                                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ialgnatt.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <iattribt.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
    #define _IALGNATT_CPP_
    #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IAlignmentAttribute::IAlignmentAttribute                                     |
|                                                                              |
| Constructor to create an alignment attribute from two enum values            |
------------------------------------------------------------------------------*/
IAlignmentAttribute::IAlignmentAttribute ( EHorizontalAlignment hAlign,
                                           EVerticalAlignment vAlign )
   : IAlignmentAttribute::Inherited( )
{
  this->fHorizontalAlignment = hAlign;
  this->fVerticalAlignment = vAlign;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::IAlignmentAttribute                                     |
|                                                                              |
| Constructor to create an alignment attribute from another alignment attribute|
------------------------------------------------------------------------------*/
IAlignmentAttribute::IAlignmentAttribute ( const IAlignmentAttribute& aAttr )
   : IAlignmentAttribute::Inherited( )
{
  this->fHorizontalAlignment = aAttr.fHorizontalAlignment;
  this->fVerticalAlignment = aAttr.fVerticalAlignment;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::~IAlignmentAttribute                                    |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IAlignmentAttribute::~IAlignmentAttribute( )
{ }

/*------------------------------------------------------------------------------
| IAlignmentAttribute::operator=                                               |
|                                                                              |
| Class assignment                                                             |
| NOTES: Required by base IAttribute class                                     |
------------------------------------------------------------------------------*/
IAlignmentAttribute& IAlignmentAttribute::operator= (const IAlignmentAttribute& aAttr )
{
  this->fHorizontalAlignment = aAttr.fHorizontalAlignment;
  this->fVerticalAlignment = aAttr.fVerticalAlignment;
  return *this;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::operator==                                              |
|                                                                              |
| Class comparison                                                             |
| NOTES: Required by base IAttribute class                                     |
------------------------------------------------------------------------------*/
bool IAlignmentAttribute::operator== (const IAttribute& aAttr) const
{
  const IAlignmentAttribute& a =
    dynamic_cast<const IAlignmentAttribute&>(aAttr);
  if ( ( this->fHorizontalAlignment == a.fHorizontalAlignment ) &&
       ( this->fVerticalAlignment == a.fVerticalAlignment ) )
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::horizontalAlignment                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IAlignmentAttribute::EHorizontalAlignment IAlignmentAttribute::horizontalAlignment( ) const
{
  return this->fHorizontalAlignment;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::setHorizontalAlignment                                  |
|                                                                              |
------------------------------------------------------------------------------*/
IAlignmentAttribute& IAlignmentAttribute::setHorizontalAlignment( EHorizontalAlignment hAlign )
{
  this->fHorizontalAlignment = hAlign;
  return *this;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::verticalAlignment                                       |
|                                                                              |
------------------------------------------------------------------------------*/
IAlignmentAttribute::EVerticalAlignment IAlignmentAttribute::verticalAlignment( ) const
{
  return this->fVerticalAlignment;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::setVerticalAlignment                                    |
|                                                                              |
------------------------------------------------------------------------------*/
IAlignmentAttribute& IAlignmentAttribute::setVerticalAlignment( EVerticalAlignment vAlign )
{
  this->fVerticalAlignment = vAlign;
  return *this;
}

/*------------------------------------------------------------------------------
| IAlignmentAttribute::clone                                                   |
|                                                                              |
| In lieu of Copy constructor                                                  |
| NOTES: Required by IAttribute                                                |
------------------------------------------------------------------------------*/
IAttribute* IAlignmentAttribute::clone( ) const
{
  IAttribute* attr = new IAlignmentAttribute( this->fHorizontalAlignment,
                                              this->fVerticalAlignment );
  return attr;
}
