// Revision: 56 1.7.1.12 source/ui/basectl/icombovw.c, listctls, ioc.v400, 000302 
#ifndef _ICOMBOVW_C_
#define _ICOMBOVW_C_
/*******************************************************************************
* FILE NAME: icombovw.c                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icombovw.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp. 1992, 1997  All Rights Reserved.                   *
*                                                                              *
*******************************************************************************/

#include  <icollvwi.hpp>
#include  <icombovw.hpp>
#include  <icolvwd.hpp>
#include  <iseq2.h>
#include  <icconst.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

template <class Element, class Collection>
  const unsigned long
    ICollectionViewComboBox<Element, Collection>::noSelection = 0;


template <class Element, class Collection>
  class ICollectionViewComboBoxData {

public:

  ICollectionViewComboBoxData( const IStringGenerator<Element>&
                                  stringGenerator );

 ~ICollectionViewComboBoxData ( );

ICollectionViewData< Element,
                     ICollectionViewComboBox< Element, Collection >,
                     Collection >
  *fCollectionViewData;

bool
  saveSelectionState;

private:
/*------------------------- Hidden Members -----------------------------------*/
  ICollectionViewComboBoxData
             ( const ICollectionViewComboBoxData< Element, Collection >& source );
ICollectionViewComboBoxData<Element,Collection>
 &operator=  ( const ICollectionViewComboBoxData< Element, Collection >& source );

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

template <class Element, class Collection>
  ICollectionViewComboBoxData<Element, Collection>::
    ICollectionViewComboBoxData( const IStringGenerator<Element>& stringGenerator )
      : saveSelectionState( false )
{
  fCollectionViewData = new
     ICollectionViewData< Element,
                          ICollectionViewComboBox< Element, Collection >,
                          Collection > ( stringGenerator);
}

template <class Element, class Collection>
  ICollectionViewComboBoxData<Element,Collection>::
   ~ICollectionViewComboBoxData ( )
{
  delete fCollectionViewData;
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>::
    ICollectionViewComboBox ( unsigned long                    identifier,
                              IWindow*                         parent,
                              IWindow*                         owner ,
                              const IRectangle&                initial,
                              const IBaseComboBox::Style&      style,
                              const IStringGenerator<Element>& stringGenerator )
    : IBaseComboBox( identifier, parent, owner, initial, style )
{
  fCollectionViewComboBoxData = new ICollectionViewComboBoxData<Element, Collection>( stringGenerator );
  fCollectionViewComboBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}


#ifdef IC_PMWIN
template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>::
    ICollectionViewComboBox( unsigned long                    identifier,
                             IWindow*                         parent,
                             const IStringGenerator<Element>& stringGenerator )
    : IBaseComboBox( identifier, parent )
{
  fCollectionViewComboBoxData = new ICollectionViewComboBoxData<Element, Collection>( stringGenerator );
  fCollectionViewComboBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>::
    ICollectionViewComboBox( const IWindowHandle&             handle,
                             const IStringGenerator<Element>& stringGenerator )
    : IBaseComboBox( handle )
{
  fCollectionViewComboBoxData =
     new ICollectionViewComboBoxData<Element, Collection>( stringGenerator );
  fCollectionViewComboBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}
#endif //IC_PMWIN

// implement empty copy constructor to work-around compiler RTTI bug
template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>::
    ICollectionViewComboBox( const ICollectionViewComboBox< Element, Collection > & source )
   : IBaseComboBox (9500, 0, 0)
{}


template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>::
   ~ICollectionViewComboBox ( )
{
  disableNotification();

  delete fCollectionViewComboBoxData;
}


#include <inotifev.hpp>
#include <iexcept.hpp>


template <class Element, class Collection>
  Collection*  ICollectionViewComboBox<Element,Collection>:: items( ) const
{
  return  (Collection *)fCollectionViewComboBoxData->fCollectionViewData->collectionObserver().collection();
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&   ICollectionViewComboBox<Element,Collection>::
    setItems( Collection* collection )
{
  fCollectionViewComboBoxData->fCollectionViewData->collectionObserver().setCollection( collection );

  return *this;
}

template <class Element, class Collection>
  IStringGenerator<Element>&  ICollectionViewComboBox<Element,Collection>::
    stringGenerator ( )
{
  return  fCollectionViewComboBoxData->fCollectionViewData->generator();
}

template <class Element, class Collection>
  IStringGenerator<Element>&
    ICollectionViewComboBox<Element,Collection>::
      setStringGenerator ( const IStringGenerator<Element>&  newGenerator )
{
  fCollectionViewComboBoxData->fCollectionViewData->setGenerator( (IStringGenerator<Element>&)newGenerator );

  fCollectionViewComboBoxData->saveSelectionState = true;
  elementsChanged ( );
  fCollectionViewComboBoxData->saveSelectionState = false;

  return  fCollectionViewComboBoxData->fCollectionViewData->generator();
}


template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>& ICollectionViewComboBox<Element,Collection> :: elementsChanged ( )
{
  if ( isWindowValid( this ) )
  {
  ICursor*                  pCursor;
  Collection*               collection;
  ISequence<unsigned long>  selectionIndexes( IBaseComboBox::numberOfSelections());

  collection = items( );
  IWindow::enableUpdate( false );

  /* Don't forget the case where collection is NULL  */
  if ( collection )
  {
    if ( ( collection->numberOfElements() > 0 ) &&
         fCollectionViewComboBoxData->saveSelectionState )
    {
      IBaseComboBox::Cursor    cursor( *this );
      forCursor( cursor )
        selectionIndexes.add( cursor.asIndex());
    }
    else   // this is a removeall so deselect list box items
    {
      IBaseComboBox::deselectAll( );
    }
  }

  /*  Clear list box and repopulate w/ current string representations */
  deleteAll( );
  if ( collection )
  {
    pCursor = collection->newCursor();
    for (pCursor->setToFirst(); pCursor->isValid(); pCursor->setToNext() )
    {
      insert( ICollectionViewConstants::listEnd,
        fCollectionViewComboBoxData->fCollectionViewData->generator().stringFor( collection->elementAt( *pCursor ) ) );
    }
    delete pCursor;
  }

  if ( !IBaseComboBox::isEmpty( ) &&
       fCollectionViewComboBoxData->saveSelectionState )
    for ( unsigned long i = selectionIndexes.numberOfElements(); i; --i )
       IBaseComboBox::select( selectionIndexes.elementAtPosition(i), true );

  IWindow::enableUpdate( );
  IWindow::show( );
  }
  return *this;
}


template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&  ICollectionViewComboBox<Element,Collection>::
    elementAdded ( unsigned long position, const Element&  element )
{
  insert( position - 1,
    fCollectionViewComboBoxData->fCollectionViewData->generator().stringFor( element ) );
  return *this;
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&  ICollectionViewComboBox<Element,Collection>::
    elementDeleted ( unsigned long position )
{
  deleteItem( position - 1 );

  return *this;
}


template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&  ICollectionViewComboBox<Element,Collection>::
    elementChanged ( unsigned long  position, const Element&  element )
{
  IBaseComboBox::setItemText(
     position - 1,
     fCollectionViewComboBoxData->fCollectionViewData->generator().stringFor(element) );
  INotificationEvent  anEvent(
     IBaseComboBox::itemChangedId,
    *this,
     true ,
     IEventData((void *)&element) );
  notifyObservers( anEvent );
  return *this;
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&  ICollectionViewComboBox<Element,Collection> :: collectionReplaced( )
{
  if ( isWindowValid( this ) )
  {
    INotificationEvent  anEvent( ICollectionViewComboBox<Element,Collection>::itemsId,
                                *this,
                                 true,
                                 IEventData( (void *)items( ) ) );
    notifyObservers( anEvent );
  }

  return *this;
}


template <class Element, class Collection>
  unsigned long ICollectionViewComboBox<Element,Collection>::
    selectedCollectionPosition ( )
{
  return IBaseComboBox::selection( ) + 1;
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&
    ICollectionViewComboBox<Element,Collection>::
      select ( unsigned long collectionPosition ,
               bool       select )
{
  if ( (collectionPosition > 0 ) &&
       (collectionPosition <= IBaseComboBox::count()) )
    IBaseComboBox::select( collectionPosition - 1, select );

  return *this;
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>&
    ICollectionViewComboBox<Element,Collection>::
      deselect ( unsigned long collectionPosition )
{
  return select( collectionPosition, false );
}

template <class Element, class Collection>
  Element       ICollectionViewComboBox<Element,Collection>::
    selectedElement ( ) const
{
  unsigned long                    selectedPosition;
  Element*						   pElement = 0;
  IPartOrderedCollection<Element>* aCollection = items( );

  if( aCollection )
  {
    IAutoPointer<ICursor>            pCursor( aCollection->newCursor( ), IINIT );

	selectedPosition = IBaseComboBox::selection( );
	if ( selectedPosition == ICollectionViewConstants::listNone )
	{
	  ITHROWLIBRARYERROR( IC_NO_SELECTION,
	                      IBaseErrorInfo::invalidRequest,
	                      IException::recoverable );
	}

    aCollection->setToPosition( selectedPosition + 1, *pCursor );
    pElement = &(aCollection->elementAt( *pCursor ));
  }
  else // no collection associated with this combo box
  {
    ITHROWLIBRARYERROR( IC_NO_SELECTION,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable );
  }

  return *pElement;
}

template <class Element, class Collection>
  unsigned long ICollectionViewComboBox<Element,Collection>::
    insert( unsigned long index,
            const char*   text )
{
   incrementChangeCount( );   // add may result in an invalid cursor.
   unsigned long anIndex = count();
   if ((long)anIndex > (long)index)
     anIndex = index;

   IEventResult evt = handle().sendEvent(
#ifdef IC_MOTIFPM
                               ICollectionViewConstants::listInsert,
#else
                               ICollectionViewConstants::comboInsert,
#endif
                               IEventParameter1(anIndex),
                               IEventParameter2((unsigned long) text));

   if ( ( evt.asLong() == ICollectionViewConstants::listMemoryError ) ||
        ( evt.asLong() == ICollectionViewConstants::listError ) )
     ITHROWGUIERROR("ICollectionViewConstants::listInsert");

   return evt.asUnsignedLong();  // Return index of inserted item.
}


template <class Element, class Collection>
  unsigned long ICollectionViewComboBox<Element,Collection>::
    deleteItem ( unsigned long index )
{
   bool selected = isSelected( index );

   incrementChangeCount( );   // add may result in an invalid cursor.
   IEventResult evt = handle().sendEvent(
#ifdef IC_MOTIFPM
                               ICollectionViewConstants::listDelete,
#else
                               ICollectionViewConstants::comboDelete,
#endif
                               IEventParameter1(index),
                               IEventParameter2(0));

   if ( selected )
   {
     INotificationEvent  anEvent( ICollectionViewComboBox<Element,Collection>::selectId,
                                 *this );
     notifyObservers( anEvent );
   }


   return evt.asUnsignedLong();
}

template <class Element, class Collection>
  ICollectionViewComboBox<Element,Collection>& ICollectionViewComboBox<Element,Collection>::
      deleteAll ( )
{
   IEventResult evt = handle().sendEvent(
#ifdef IC_MOTIFPM
                               ICollectionViewConstants::listDeleteAll,
#else
                               ICollectionViewConstants::comboDeleteAll,
#endif
                               IEventParameter1(0),
                               IEventParameter2(0));
#ifdef IC_PM
   if (!(evt.asUnsignedLong()))
      ITHROWGUIERROR("ICollectionViewConstants::listDeleteAll");
#endif
   incrementChangeCount( );   // add may result in an invalid cursor.
   return *this;
}

#endif // _ICOMBOVW_C_
