// Revision: 60 1.6.1.2 source/ui/basectl/icslide1.cpp, slidectls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icslide1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icsliden.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
#define INCL_WINDIALOGS
#define INCL_WINBUTTONS
#define INCL_OS2MM
#define INCL_SW
#define INCL_CIRCULARSLIDER
#define IC_MULTIMEDIA
extern "C" {
  #include <iwindefs.h>
}

#include <icslider.hpp>
#include <icsliden.hpp>
#include <inotifev.hpp>
#include <ictlevt.hpp>
#include <icconst.h>
#ifdef IC_WIN
#include <iclcsldw.h>
#endif
#include <itrace.hpp>

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICSLIDE1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const ICircularSlider::valueId = "ICircularSlider::value";
INotificationId const ICircularSlider::trackId = "ICircularSlider::track";

/*------------------------------------------------------------------------------
| ICircularSlider::enableNotification                                          |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ICircularSlider, does not send notifications to observer objects until it    |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::enableNotification (bool enabled)
{
  IMODTRACE_DEVELOP("ICircularSlider::enableNotification") ;
  if (!notificationHandler())
  {
    setNotificationHandler( new ICircularSliderNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSliderNotifyHandler::ICircularSliderNotifyHandler                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ICircularSliderNotifyHandler::ICircularSliderNotifyHandler()
{
   IMODTRACE_DEVELOP("ICircularSliderNotifyHandler::ICircularSliderNotifyHandler") ;
}

/*------------------------------------------------------------------------------
| ICircularSliderNotifyHandler::~ICircularSliderNotifyHandler                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ICircularSliderNotifyHandler::~ICircularSliderNotifyHandler()
{
   IMODTRACE_DEVELOP("ICircularSliderNotifyHandler::~ICircularSliderNotifyHandler") ;
}

/*------------------------------------------------------------------------------
| ICircularSliderNotifyHandler::dispatchHandlerEvent                           |
|                                                                              |
| This notifies ICircularSlider Windows of the following events:               |
|                                                                              |
| value(Id)                                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
bool ICircularSliderNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IMODTRACE_DEVELOP("ICircularSliderNotifyHandler::dispatchHandlerEvent") ;
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
        if (className == WC_CIRCULARSLIDER)
        {
          if (ulMsg == CSN_CHANGED)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        ICircularSlider::valueId,
                                        *thisWindow, true,
#ifdef IC_PM
                                        anEvent.parameter2()));
#endif
#ifdef IC_WIN
             (int)((PCSDIALPOS)(void*)(anEvent.parameter2()))->position ));
#endif
          }
          else if (ulMsg == CSN_TRACKING)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        ICircularSlider::trackId,
                                        *thisWindow, true,
#ifdef IC_PM
                                        anEvent.parameter2()));
#endif
#ifdef IC_WIN
             (int)((PCSDIALPOS)(void*)(anEvent.parameter2()))->position ));
#endif
          }
        }
      }
    break;
    } /* end WM_CONTROL */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

