// Revision: 65 1.13 source/ui/basectl/icvhdr.hpp, canvas, ioc.v400, 000302 
/* NOSHIP */
#ifndef _ICVHDR_
  #define _ICVHDR_
/*******************************************************************************
* FILE NAME: icvhdr.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ICanvasHandler                                                           *
*     ICanvasStatics                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandler.hpp>
#include <ikeyhdr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IWindow;
class ICanvas;
class IFrameWindow;
class IEvent;
class IKeyboardEvent;
class ICanvasHandlerData;
class ICVKeyboardHandlerData;


#pragma enum(4)
#pragma pack(push,4)

class ICVKeyboardHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* This class provides keyboard event processing for canvas objects.
* Its primary purpose is to provide tabbing and cursor movement between
* controls in a manner similar to dialogs.  It also handle SETFOCUS events
* for the canvas.
*   In OS/2, a default object of this class is attached to ICanvas objects.
*   In Windows and Motif, a default object of this class is attached to ICanvas
*      objects and to IFrameWindow objects.
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any parameters.                     |
------------------------------------------------------------------------------*/
  ICVKeyboardHandler        ( );
virtual
 ~ICVKeyboardHandler        ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a canvas window:                                                             |
|   handleEventsFor       - Attaches the handler to the specified canvas.      |
|   stopHandlingEventsFor - Detaches the handler from the specified canvas.    |
| In Windows, the handler object may also be attached to an IFrameWindow       |
| object.                                                                      |
------------------------------------------------------------------------------*/
virtual ICVKeyboardHandler
 &handleEventsFor       ( ICanvas* canvas ),
 &stopHandlingEventsFor ( ICanvas* canvas );

#ifdef IC_MOTIFWIN
virtual ICVKeyboardHandler
 &handleEventsFor       ( IFrameWindow* canvas ),
 &stopHandlingEventsFor ( IFrameWindow* canvas );
#endif


protected:
/*-------------------------------- Overrides -----------------------------------
| This class overrides the following inherited functions:                      |
|   dispatchHandlerEvent - Router for (and provides processing for other)      |
|                          class-specific events.                              |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent  ( IEvent& event );

/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   key          - Processes tabbing and cursor movement keys                  |
------------------------------------------------------------------------------*/
virtual bool
  key                  ( IKeyboardEvent& event );

/*------------------------------ Implementation --------------------------------
| These functions are supplied in order to provide a canvas window with        |
| support typically expected from a dialog window.                             |
|   gotFocus     - Gives input focus to the appropriate child window           |
|   handleCursorMovement - Handles tabbing between groups and canvases and     |
|                          cursor movement within a group.                     |
|   handleEnter          - Handles selection of a default pushbutton when the  |
|                          Enter key is pressed.                               |
------------------------------------------------------------------------------*/
bool
  gotFocus              ( IEvent& event ),
  handleCursorMovement  ( IKeyboardEvent& event ),
  handleEnter           ( IKeyboardEvent& event );

#ifdef IC_PMWIN
/*------------------------------ Button State ----------------------------------
|   buttonSetup          - Handles cursor selection of radio buttons and       |
|                          default push button hiliting with cursor movement.  |
------------------------------------------------------------------------------*/
enum State {
  enteringGroup, withinGroup, leavingGroup
};
static void
  buttonSetup           ( const IWindow* canvas,
                          const IWindowHandle& child,
                          State state,
                          bool  clickRadio = true );
#endif
private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );

ICVKeyboardHandlerData
 *fCVKeyboardHandlerData;
};    // class ICVKeyboardHandler

#ifdef IC_MOTIF
class ICanvasHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* This class provides event processing for canvas objects.  This includes:     *
*   - expected client area behavior                                            *
*     (e.g. ability to be resized, passing of command events to the owner      *
*   - calling the layout routine when the window is repainted                  *
*                                                                              *
* The management of tabbing and cursor movement between controls,              *
* auto-selecting of radio buttons, processing of mnemonics, etc. is handled    *
* by ICVKeyboardHandler.  For canvas controls default ICanvasHandler and       *
* ICVKeyboardHandler objects are attached to the canvas.  In Windows, the      *
* ICVKeyboardHandler object is also used on IFrameWindows but the              *
* ICanvasHandler object is not.                                                *
*                                                                              *
* The only way to construct instances of this class is by using the            *
* default constructor, which does not take any parameters.                     *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any parameters.                     |
------------------------------------------------------------------------------*/
  ICanvasHandler        ( );
virtual
 ~ICanvasHandler        ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a canvas window:                                                             |
|   handleEventsFor       - Attaches the handler to the specified canvas.      |
|   stopHandlingEventsFor - Detaches the handler from the specified canvas.    |
------------------------------------------------------------------------------*/
virtual ICanvasHandler
 &handleEventsFor       ( ICanvas* canvas ),
 &stopHandlingEventsFor ( ICanvas* canvas );

protected:
/*-------------------------------- Overrides -----------------------------------
| This class overrides the following inherited functions:                      |
|   dispatchHandlerEvent - Router for (and provides processing for other)      |
|                          class-specific events.                              |
------------------------------------------------------------------------------*/
virtual bool
  dispatchHandlerEvent  ( IEvent& event );

/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   layoutCanvas - Calls the virtual layout function.                          |
------------------------------------------------------------------------------*/
virtual bool
  layoutCanvas          ( IEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );

ICanvasHandlerData
 *fCanvasHandlerData;
}; // class ICanvasHandler
#endif // IC_MOTIF


class ICanvasStatics  {
/*******************************************************************************
* This class is a wrapper for static canvas functions and process level        *
* state data.                                                                  *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
 ~ICanvasStatics      ( );

/*-------------------------------- Accessors -----------------------------------
|  Functions to determine and set the state of the window class.               |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
enum EWindowClassStyle {
   kClassStyleMaxPaints, kClassStyleMinPaints
};
static void
  setWindowClassRegistered ( EWindowClassStyle cs );
static bool
  isWindowClassRegistered  ( EWindowClassStyle cs );
#endif

/*---------------------------- Dialog navigation -------------------------------
| Dialog navigation functions support dialog like keyboard navigation in       |
| canvases (and frame windows in some environments)                            |
|   matchForMnemonic        - Returns the first child window that uses the     |
|                             specified character as a mnemonic.               |
|   nextDialogItem          - Portable interface for enumerating child         |
|                             controls.  Usage like PM WinEnumDlgItem.         |
|                                                                              |
------------------------------------------------------------------------------*/
static IWindowHandle
  matchForMnemonic       ( const IWindow* window,
                           unsigned short character );

static IWindowHandle
  nextDialogItem         ( const IWindowHandle& dialog,
                           const IWindowHandle& child,
                           unsigned long        flags );

private:
/*--------------------------------- Private ----------------------------------*/
#ifdef IC_PMWIN
static bool
  fHasBeenRegistered,
  fAllPaintsHasBeenRegistered;
#endif
}; // class ICanvasStatics

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif


#ifdef IC_PM
/**********************************************************************/
/* support for dialog enumeration and button setting                  */
/**********************************************************************/
#define IENUMDLGITEM(dlg,ctl,flag ) \
         (ICanvasStatics::nextDialogItem(dlg, ctl, flag))

#endif

#ifdef IC_WIN
/**********************************************************************/
/* support for dialog enumeration and button setting                  */
/**********************************************************************/
#define IENUMDLGITEM(dlg,ctl,flag)      \
         (ICanvasStatics::nextDialogItem(dlg, ctl, flag))

// Define these flags for Win.  They are used in ICanvasStatics::nextDialogItem
#define EDI_FIRSTTABITEM           0
#define EDI_LASTTABITEM            1
#define EDI_NEXTTABITEM            2
#define EDI_PREVTABITEM            3
#define EDI_FIRSTGROUPITEM         4
#define EDI_LASTGROUPITEM          5
#define EDI_NEXTGROUPITEM          6
#define EDI_PREVGROUPITEM          7
#endif

#endif /* _ICVHDR_ */
