// Revision: 79 1.14.1.9 source/ui/basectl/igroupbx.cpp, staticctls, ioc.dev 
/*******************************************************************************
* FILE NAME: igroupbx.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igroupbx.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define INCL_WINERRORS
  #define INCL_WINSTATICS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/DrawingA.h>
  #include <ixconst.h>
  #include <Xm/Label.h>
#endif

#include <igroupbx.hpp>
#ifdef IC_WIN
  #include <igbpriv.hpp>
#endif
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandle.hpp>
#include <iplatfrm.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <itrace.hpp>
#ifdef IC_WIN
  #include <ibrushdr.hpp>
#endif

#ifdef IC_MOTIF
  #include <imstring.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGROUPBX_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public group box styles                                                      |
------------------------------------------------------------------------------*/
const IGroupBox::Style
  IGroupBox::classDefaultStyle (WS_VISIBLE);


/*------------------------------------------------------------------------------
| Default style for new group box objects (initial value).                     |
------------------------------------------------------------------------------*/
IGroupBox::Style IGroupBox::currentDefaultStyle = WS_VISIBLE;


#ifdef IC_MOTIFWIN
#pragma enum(4)
#pragma pack(push,4)

class IGroupBoxData 
{
/*------------------------------------------------------------------------------
| The IGroupBoxData class encapsulates private data and functions used by      |
| the IGroupBox class.  An object of this class is created in the IGroupBox    |
| constructors.                                                                |
------------------------------------------------------------------------------*/
public:
  IGroupBoxData ( );

 ~IGroupBoxData ( );

#ifdef IC_MOTIF
Widget
  whGroupBox,
  whGroupBoxLabel;
#endif
#ifdef IC_WIN
IBrushHandler
  fBrushHandler;
#endif
/*------------------------------- Private -----------------------------------*/
private:
  IGroupBoxData::IGroupBoxData( const IGroupBoxData& gbd ) {}
  IGroupBoxData& operator=    ( const IGroupBoxData& gbd );
}; // IGroupBoxData

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| IGroupBoxData::IGroupBoxData                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IGroupBoxData::IGroupBoxData ( )
#ifdef IC_WIN
  : fBrushHandler()
#endif
{ }

/*------------------------------------------------------------------------------
| IGroupBoxData::~IGroupBoxData                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IGroupBoxData::~IGroupBoxData ( )
{
}
#endif // IC_MOTIFWIN

#ifdef IC_MOTIF
void _System iobResizeCallback (_WidgetRec *w,
                                void       *client_data,
                                void       *call_data);
void _System iobExposeCallback (_WidgetRec *w,
                                void       *client_data,
                                void       *call_data);
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IGroupBox::IGroupBox                                                         |
|                                                                              |
| Constructor to create a group box control on a standard window.              |
------------------------------------------------------------------------------*/
IGroupBox :: IGroupBox(unsigned long ulId,
                       IWindow* pwndParent,
                       IWindow* pwndOwner,
                       const IRectangle& rectInit,
                       const Style& gbsStyle)
#ifdef IC_MOTIFWIN
  : fGroupBoxData( new IGroupBoxData )
#endif
#ifdef IC_PM
  : fGroupBoxData( 0 )
#endif
{
  IASSERTPARM(pwndParent!=0);          //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | gbsStyle.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  //Create group box control
  IWindowHandle ownerHandle = IWindowHandle(0);
  if (pwndOwner)
     ownerHandle = pwndOwner->handle();
  IWindowHandle whGroupBox =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( gbsStyle ),
#ifdef IC_PM
                      WC_STATIC,
#endif
#ifdef IC_WIN
                      WC_BUTTON,
#endif
                      pwndParent->handle(),
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( gbsStyle, true ) );

  startHandlingEventsFor(whGroupBox);
#ifdef IC_WIN
  fGroupBoxData->fBrushHandler.handleEventsFor( this );
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // Create the Group Box.
  Arg    args[5];
  int    n = 0;
  Pixel parentBackground, parentForeground;
  if (pwndParent->handle())
  {
     XtVaGetValues ( pwndParent->handle(),
                  XmNforeground, &parentForeground,
                  XmNbackground, &parentBackground,  
                  NULL);
      XtSetArg( args[n], XmNforeground, parentForeground); n++;
      XtSetArg( args[n], XmNbackground, parentBackground); n++;
  }
  XtSetArg( args[n], XmNmarginHeight, 0); n++;
  XtSetArg( args[n], XmNborderWidth, 0); n++;
// Added to fix defect 26864
  XtSetArg( args[n], XmNresizePolicy, XmRESIZE_NONE); n++;

  fGroupBoxData->whGroupBox = XmCreateDrawingArea(
         pwndParent->handleForChildCreation(),
         (char *)( IString(ulId) ),
         args,
         n);
  if (fGroupBoxData->whGroupBox == 0)
  {
     ITHROWGUIERROR("XmCreateDrawingArea");
  }
  else
  {
   n = 0;
   XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING);  n++;
   XtSetArg(args[n], XmNlabelString, (XmString)IMString() ) ;  n++;
   XtSetArg( args[n], XmNforeground, parentForeground); n++;
   XtSetArg( args[n], XmNbackground, parentBackground); n++;
   fGroupBoxData->whGroupBoxLabel = XmCreateLabel(
         fGroupBoxData->whGroupBox,
         (char *)( IString(ulId) + IString("L") ),  // widget name
         args,
         n);
   XtManageChild(fGroupBoxData->whGroupBoxLabel);
   startHandlingEventsFor (fGroupBoxData->whGroupBox);
   XtManageChild(fGroupBoxData->whGroupBox);
   setOwner (pwndOwner);
  }
  XtAddCallback (fGroupBoxData->whGroupBox,
                 XmNresizeCallback,
                 iobResizeCallback,
                 NULL);
  XtAddCallback (fGroupBoxData->whGroupBox,
                 XmNexposeCallback,
                 iobExposeCallback,
                 NULL);
  XtAddCallback (fGroupBoxData->whGroupBox,
                 XmNexposeCallback,
                 iobResizeCallback,
                 NULL);
  moveSizeTo(rectInit);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IGroupBox::IGroupBox                                                         |
|                                                                              |
| Constructor to create instance for a template window.                        |
------------------------------------------------------------------------------*/
IGroupBox :: IGroupBox(unsigned long ulId,
                       IWindow* pdlgwndParent)
#ifdef IC_MOTIFWIN
  : fGroupBoxData( new IGroupBoxData )
#endif
#ifdef IC_PM
  : fGroupBoxData( 0 )
#endif
{
  setAutoDestroyWindow(false);
#ifdef IC_PMWIN
  reserveUserWindowWord( false );
  startHandlingEventsFor(ulId, pdlgwndParent);
#ifdef IC_WIN
  fGroupBoxData->fBrushHandler.handleEventsFor( this );
#endif
#endif
#ifdef IC_MOTIF
  IWindowHandle whDummy =
         XtNameToWidget(pdlgwndParent->handle(), (char *)IString(ulId) );
  if (whDummy == 0)
     ITHROWGUIERROR("XtNameToWidget");

  prepareForUse(whDummy);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IGroupBox::IGroupBox                                                         |
|                                                                              |
| Constructor to instantiate from an existing group box control.               |
------------------------------------------------------------------------------*/
IGroupBox :: IGroupBox(const IWindowHandle& wh)
#ifdef IC_MOTIFWIN
  : fGroupBoxData( new IGroupBoxData )
#endif
#ifdef IC_PM
  : fGroupBoxData( 0 )
#endif
{
  setAutoDestroyWindow(false);
  reserveUserWindowWord( false );
  startHandlingEventsFor(wh);
#ifdef IC_WIN
  fGroupBoxData->fBrushHandler.handleEventsFor( this );
#endif
}


/*------------------------------------------------------------------------------
| IGroupBox::~IGroupBox                                                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IGroupBox :: ~IGroupBox()
{
#ifdef IC_WIN
  fGroupBoxData->fBrushHandler.stopHandlingEventsFor( this );
#endif
#ifdef IC_MOTIFWIN
  delete fGroupBoxData;
#endif
}

/*------------------------------------------------------------------------------
| IGroupBox::defaultStyle                                                      |
|                                                                              |
| Return the default style for new group box objects.                          |
------------------------------------------------------------------------------*/
IGroupBox::Style  IGroupBox :: defaultStyle()
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IGroupBox::setDefaultStyle                                                   |
|                                                                              |
| Replace the default style for new group box objects.                         |
------------------------------------------------------------------------------*/
void  IGroupBox :: setDefaultStyle(const IGroupBox::Style& gbsStyle)
{
  currentDefaultStyle = gbsStyle;
}

/*------------------------------------------------------------------------------
| IGroupBox::convertToGUIStyle                                                 |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IGroupBox::convertToGUIStyle(const IBitFlag& guiStyle,
                                           bool bExtOnly) const
{
  // Obtain the style from the class (IControl) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK ;
  }
  else
  {
    // Let the clean styles flow thru and add the groupbox style
    ulStyle |= guiStyle.asUnsignedLong();
#ifdef IC_MOTIFPM
    ulStyle |= SS_GROUPBOX;
#endif
#ifdef IC_WIN
    ulStyle |= (BS_GROUPBOX | WS_CHILD);
#endif
  }

  return( ulStyle );
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IGroupBox::calcMinimumSize                                                   |
------------------------------------------------------------------------------*/
ISize  IGroupBox :: calcMinimumSize() const
{
  ISize siz = displaySize(text());

  /*----------------------------------------------------------------------------
  | We need to pad the width of 2 characters to allow for the border           |
  ----------------------------------------------------------------------------*/
  int pad = IFont(this).avgCharWidth() << 1;
  siz.setWidth(siz.width() + pad);

  return siz;
}
#endif // ICPMWIN

/*------------------------------------------------------------------------------
| WIN32 Color Support                                                          |
|                                                                              |
| Windows has no API equivalent for Presentation Parameters.  The following    |
| group box color function is NOPed for Windows.  The query function returns   |
| an appropriate default IColor object.                                        |
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
| IGroupBox::foregroundColor                                                   |
|                                                                              |
| Returns the foreground color of the IGroupBox.                               |
------------------------------------------------------------------------------*/
IColor IGroupBox::foregroundColor () const
{
#ifdef IC_PM
  return (IWindow::color(PP_FOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::windowStaticText)));
#endif //IC_PM
#ifdef IC_WIN
  return IGUIColor( IGUIColor::windowStaticText );
#endif //IC_WIN
#ifdef IC_MOTIF
  Pixel groupColor;
  XtVaGetValues ( fGroupBoxData->whGroupBox,
                  XmNforeground, &groupColor, 
                  NULL ) ;
  return ( IColor(groupColor) );
#endif // IC_MOTIF
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IGroupBox::setForegroundColor                                                |
|                                                                              |
| Sets the foreground color of the outline box                                 |
------------------------------------------------------------------------------*/
IGroupBox& IGroupBox :: setForegroundColor    (const IColor &color)
{
  XtVaSetValues ( fGroupBoxData->whGroupBox,
                  XmNforeground, 
                  color.index(),
                  NULL ) ;
  XtVaSetValues ( fGroupBoxData->whGroupBoxLabel,
                  XmNforeground, 
                  color.index(),
                  NULL ) ;
  refresh ();
  return *this;
}
/*------------------------------------------------------------------------------
| IGroupBox::setBackgroundColor                                                |
|                                                                              |
| Sets the background color of the outline box                                 |
------------------------------------------------------------------------------*/
IGroupBox& IGroupBox :: setBackgroundColor    (const IColor &color)
{
  XtVaSetValues ( fGroupBoxData->whGroupBox,
                  XmNbackground, 
                  color.index(),
                  NULL ) ;
  XtVaSetValues ( fGroupBoxData->whGroupBoxLabel,
                  XmNbackground, 
                  color.index(),
                  NULL ) ;
  refresh ();
  return *this;
}
/*------------------------------------------------------------------------------
| IGroupBox::setColor                                                         |
|                                                                             |
| Sets the foreground color of the outline box, other PP values are passed    |
| onto the inherited implementation.                                          |
------------------------------------------------------------------------------*/
IGroupBox& IGroupBox :: setColor( unsigned long colorArea,
                                  const IColor &color)
{
  if (colorArea == PP_FOREGROUNDCOLOR )
  {
    XtVaSetValues ( fGroupBoxData->whGroupBox,
                    XmNforeground, 
                    color.index(),
                    NULL ) ;
    XtVaSetValues ( fGroupBoxData->whGroupBoxLabel,
                    XmNforeground, 
                    color.index(),
                    NULL ) ;
    refresh ();
  }
  else if (colorArea == PP_BACKGROUNDCOLOR )
  {
    XtVaSetValues ( fGroupBoxData->whGroupBox,
                    XmNbackground, 
                    color.index(),
                    NULL ) ;
    XtVaSetValues ( fGroupBoxData->whGroupBoxLabel,
                    XmNbackground, 
                    color.index(),
                    NULL ) ;
    refresh ();
  }
  else
    Inherited::setColor( colorArea, color );
  return *this;
}
#endif //IC_MOTIF

/*------------------------------------------------------------------------------
| IGroupBox::setText                                                           |
|                                                                              |
| Changes the text used by the group box.                                      |
------------------------------------------------------------------------------*/
IGroupBox &  IGroupBox :: setText ( const char* pszText )
{
#ifdef IC_PMWIN
  Inherited::setText(pszText);
  setLayoutDistorted(IWindow::minimumSizeChanged, 0);
#endif // IC_PMWIN
#ifdef IC_MOTIF
  IMString newText;
  newText = IMString (pszText);
  XtVaSetValues ( fGroupBoxData->whGroupBoxLabel,
                  XmNlabelString, 
                  (XmString)newText,
                  NULL ) ;
  refresh();
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IGroupBox::setText                                                           |
|                                                                              |
| Changes the group box text from a string table.                              |
------------------------------------------------------------------------------*/
IGroupBox&  IGroupBox::setText ( const IResourceId& residText )
{
   Inherited::setText( residText );
   return *this;
}

/*------------------------------------------------------------------------------
| IGroupBox::visibleRectangle                                                  |
|                                                                              |
| Returns the area painted by the group box.                                   |
------------------------------------------------------------------------------*/
IRectangle IGroupBox :: visibleRectangle () const
{
  return IRectangle(position(),ISize(0,0));
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IGroupBox::text                                                              |
|                                                                              |
| Returns the text of the group box.                                           |
------------------------------------------------------------------------------*/
// Need to override this function in Motif, since the inherited one 
// assume that handle() returns the text widget handle.
IString  IGroupBox::text( ) const
{
  IString currentText;
  currentText = IMString( fGroupBoxData->whGroupBoxLabel, 
            XmNlabelString ).asString( );
   return currentText;
}
/*------------------------------------------------------------------------------
| IGroupBox::displaySize                                                       |
|                                                                              |
| Returns the display size of the text of the group box.                       |
------------------------------------------------------------------------------*/
// Need to override this function in Motif, since the inherited one 
// assume that handle() returns the text widget handle.
ISize IGroupBox::displaySize (const char* pszText) const
{
  if (pszText == 0) pszText = (char*)text();
  IMString theText = IMString( IString( pszText ) ); 
  return theText.size( fGroupBoxData->whGroupBoxLabel );
} 

#pragma options info=nouse
extern void _System iobExposeCallback (Widget    w,
                                       XtPointer client_data,
                                       XtPointer call_data)
{ 
  XLowerWindow ( XtDisplay(w), XtWindow (w) );
  return;
}

/*------------------------------------------------------------------------------
|  iobResizeCallback is the procedure invoked when the outline box is          |
|  resized.  It catches the resize request for the handle widget and then      |
|  draws the necessary rectangle.                                              |
------------------------------------------------------------------------------*/
extern void _System iobResizeCallback (Widget    w,
                                       XtPointer client_data,
                                       XtPointer call_data)
{
  Position  x, y;
  Dimension width, height, labelWidth, labelHeight;
  XGCValues gcv;
  GC        gc;
  Colormap  colorMap;
  Pixel     background, foreground;
  Widget    *labelChild;
  Cardinal  numChildren;

  XtVaGetValues( w,
                 XmNx,         &x,
                 XmNy,         &y,
                 XmNwidth ,    &width,
                 XmNheight,    &height,
                 XmNbackground,&background,
                 XmNforeground,&foreground,
                 XmNcolormap,  &colorMap,
                 XmNchildren,  &labelChild,
                 XmNnumChildren, &numChildren,
                 NULL );

  XtVaGetValues( 
                 (Widget) labelChild[0],
                 XmNwidth ,    &labelWidth,
                 XmNheight,    &labelHeight,
                 NULL );

  XmGetColors (XtScreen (w),
               colorMap,
               background,
               &gcv.foreground,
               NULL,
               NULL,
               NULL);

  gc = XCreateGC (XtDisplay (w),
                  XtWindow (w),
                  GCForeground,
                  &gcv);

  XSetForeground (XtDisplay (w),
                  gc,
		  background);

  XFillRectangle (XtDisplay (w),
                  XtWindow(w),
                  gc,
                  0,
                  0,
                  width,
                  height);

  XSetForeground (XtDisplay (w),
                  gc,
		  foreground);

  XDrawRectangle (XtDisplay (w),
                  XtWindow(w),
                  gc,
                  0,
                  labelHeight/2,
                  width-2,
                  height-(labelHeight/2) - 1);

  XtVaSetValues(
       (Widget) labelChild[0],
        XmNx, labelHeight/2,
        XmNy, 0,
        XmNmarginHeight, 0,
        XmNborderWidth, 0,
        NULL);

  return;
}
#endif // IC_MOTIF
