// IBM Open Class Library
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// (C) Copyright IBM Corporation, 1992, 1997 All Rights Reserved
//
/* NOSHIP */
#ifndef _IHITDTEC_
#define _IHITDTEC_

// Revision: 84 1.8 source/ui/basectl/ihitdtec.hpp, slidectls, ioc.v400, 000302 

#include <grtype.hpp>
#include <ihandle.hpp>
#include <igraph2d.hpp>
#include <igbase2d.hpp>
#include <iimage.hpp>
#include <igraftxt.hpp>
#include <irawarr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

//
//  The following classes have been defined in this file.
//

class IHitGraphicGroupIterator;
class IHitGraphicGroup;
class IKeyGraphicPair;

#pragma enum(4)
#pragma pack(push,4)

//
//  Name:          IHitGraphicGroupIterator
//
//  Description:
//    This class provides hit-detection for IGraphicGroupIterator objects.
//

class IHitGraphicGroupIterator : public IGraphicGroupIterator
{
  public:
    IHitGraphicGroupIterator (const IHitGraphicGroup *  hitGroup);
    IHitGraphicGroupIterator (const IHitGraphicGroup *  hitGroup,
			      const IGPoint2D &         point);
    IHitGraphicGroupIterator (const IHitGraphicGroup *  hitGroup,
			      const IGRect2D &          rect);

    virtual                    ~IHitGraphicGroupIterator();

    virtual IMGraphic *      first ();
    virtual IMGraphic *      last ();
    virtual IMGraphic *      next ();
    virtual IMGraphic *      previous ();

  private:
    enum EIteratorMode
    {
      kAll,
      kHitRectangle
    };

    EIteratorMode          fMode;
    IGRect2D               fRect;


}; // IHitGraphicGroupIterator




//
//  Name:          IHitGraphicGroup
//
//  Description:
//    This class provides hit-detection for IGraphicGroup objects.
//

class IHitGraphicGroup : public IGraphicGroup
{
  public:
    IHitGraphicGroup ();
    IHitGraphicGroup (const IHitGraphicGroup&);
    IHitGraphicGroup (IGrafBundle*   adoptBundle);

    virtual ~IHitGraphicGroup ();

    virtual void  adoptBefore (const IMGraphic &  befObj,
			             IMGraphic *  obj);
    virtual void  adoptAfter (const IMGraphic &  aftObj,
			            IMGraphic *  obj);

    IHitGraphicGroup &  operator= (const IHitGraphicGroup& source);
    virtual bool operator== (const IMDrawable&) const;
    virtual bool operator!= (const IMDrawable&) const;

    virtual const IMGraphic *
      bottomGraphicUnderPoint (const IGPoint2D &  point);
    virtual const IMGraphic *
      topGraphicUnderPoint (const IGPoint2D &  point);

  private:

}; // IHitGraphicGroup



class IKeyGraphicPair
{
  public:

    IMGraphic *      fObj;
    unsigned long    fKey;

    IKeyGraphicPair ();
    ~IKeyGraphicPair ();

    IKeyGraphicPair &  operator= (const IKeyGraphicPair &  src);

    //
    // Implementation for the following functions are dummy.
    //

    bool operator== (const IKeyGraphicPair & other) const;
    IDataStream&    operator>>= (IDataStream &) const;
    IDataStream&    operator<<= (IDataStream &);

}; // IKeyGraphicPair

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _IHITDTEC_
