// Revision: 88 1.12.1.5 source/ui/basectl/iinfoa.cpp, staticctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iinfoa.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iinfoa.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// #define IC_TRACE_ALL
extern "C"
{
  #define INCL_WINWINDOWMGR
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
#include <Xm/RowColumn.h>
#endif

#include <iinfoa.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>
#include <iframe.hpp>
#include <imenu.hpp>
#include <imnitem.hpp>
#include <isubmenu.hpp>
#include <ifont.hpp>
#include <iapp.hpp>
#include <iwcname.hpp>
#include <inotifev.hpp>
#include <imenuhdr.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IINFOA_CPP_
  #include <ipagetun.h>
#endif

#define DEFAULT_TEXT_LIMIT  80

/*------------------------------------------------------------------------------
| IInfoAreaMenuHandler                                                         |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class IInfoAreaMenuHandler : public IMenuHandler  {

public:

 IInfoAreaMenuHandler (IInfoArea* infoArea );
 ~IInfoAreaMenuHandler ( );

protected:

 virtual bool
  dispatchHandlerEvent( IEvent& event ),
  menuSelected( IMenuEvent& menuEvent );

private:

#ifdef IC_MOTIF
 unsigned long sItem;
#endif
 IInfoArea *fInfoArea;
};

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
| &IInfoArea::setStringTableOffset                                             |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setStringTableOffset(long newOffset)
{
  if (stringTableOffset() != newOffset)
  {
    this->strTblOffset = newOffset;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IInfoArea::stringTableOffset                                                 |
------------------------------------------------------------------------------*/
long IInfoArea :: stringTableOffset ( ) const
  {
  return this->strTblOffset;
  }


/*------------------------------------------------------------------------------
| IInfoArea::IInfoArea                                                         |
------------------------------------------------------------------------------*/
IInfoArea :: IInfoArea ( IFrameWindow        *frame,
                         unsigned long        id )
  : IStaticText( id,                   // Id as given.
                 frame,                // Parent is the frame.
                 frame,                // Owner is the frame.
                 IRectangle(),         // Doesn't matter.
                 IWindow::visible |    // Style.
                   IStaticText::left |
                   IStaticText::top |
                   IStaticText::fillBackground),
    pResLib( 0 ),
    numberOfLines( 1 ),
    strTblOffset( 0 ),
    defaultString(" "),
    disabledString(" "),
    missingString(" "),
    status( kDefaultText ),
    fInfoAreaMenuHandler( new IInfoAreaMenuHandler( this ) ),
    preventFlyUpdate(false)
/*------------------------------------------------------------------------------
  Leave pResLib at zero to indicate to utilize user resource library.
------------------------------------------------------------------------------*/
  {
  IMODTRACE_DEVELOP( "IInfoArea::ctor 1" );
  this -> setLimit( DEFAULT_TEXT_LIMIT );
  this -> addTo( frame );
  }

/*------------------------------------------------------------------------------
| IInfoArea::IInfoArea                                                         |
------------------------------------------------------------------------------*/
IInfoArea :: IInfoArea ( IFrameWindow        *frame,
                         unsigned long        id,
                         const char          *resDLLName )
  : IStaticText( id,                   // Id as given.
                 frame,                // Parent is the frame.
                 frame,                // Owner is the frame.
                 IRectangle(),         // Doesn't matter.
                 IWindow::visible |    // Style.
                   IStaticText::left |
                   IStaticText::top |
                   IStaticText::fillBackground),
    pResLib( 0 ),
    numberOfLines( 1 ),
    strTblOffset( 0 ),
    defaultString(" "),
    disabledString(" "),
    missingString(" "),
    status( kDefaultText ),
    fInfoAreaMenuHandler( new IInfoAreaMenuHandler( this ) ),
    preventFlyUpdate(false)
  {
  IMODTRACE_DEVELOP( "IInfoArea::ctor 2" );
  this -> setResourceLibrary( resDLLName );
  this -> setLimit( DEFAULT_TEXT_LIMIT );
  this -> addTo( frame );
  }

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IInfoArea::IInfoArea                                                         |
------------------------------------------------------------------------------*/
IInfoArea :: IInfoArea ( IFrameWindow        *frame,
                         const IModuleHandle &resMod,
                         unsigned long        id )
  : IStaticText( id,                   // Id as given.
                 frame,                // Parent is the frame.
                 frame,                // Owner is the frame.
                 IRectangle(),         // Doesn't matter.
                 IWindow::visible |    // Style.
                   IStaticText::left |
                   IStaticText::top |
                   IStaticText::fillBackground),
    pResLib( 0 ),
    numberOfLines( 1 ),
    strTblOffset( 0 ),
    defaultString(" "),
    disabledString(" "),
    missingString(" "),
    status( kDefaultText ),
    fInfoAreaMenuHandler( new IInfoAreaMenuHandler( this ) ),
    preventFlyUpdate(false)
  {
  IMODTRACE_DEVELOP( "IInfoArea::ctor 3" );
  this -> setResourceLibrary( resMod );
  this -> setLimit( DEFAULT_TEXT_LIMIT );
  this -> addTo( frame );
  }
#endif
/*------------------------------------------------------------------------------
| IInfoArea::IInfoArea                                                         |
------------------------------------------------------------------------------*/
IInfoArea :: IInfoArea ( IFrameWindow        *frame,
                         const char          *resDLLName,
                         unsigned long        id )
  : IStaticText( id,                   // Id as given.
                 frame,                // Parent is the frame.
                 frame,                // Owner is the frame.
                 IRectangle(),         // Doesn't matter.
                 IWindow::visible |    // Style.
                   IStaticText::left |
                   IStaticText::top |
                   IStaticText::fillBackground),
    pResLib( 0 ),
    numberOfLines( 1 ),
    strTblOffset( 0 ),
    defaultString(" "),
    disabledString(" "),
    missingString(" "),
    status( kDefaultText ),
    fInfoAreaMenuHandler( new IInfoAreaMenuHandler( this ) ),
    preventFlyUpdate(false)
  {
  IMODTRACE_DEVELOP( "IInfoArea::ctor 4" );
  this -> setResourceLibrary( resDLLName );
  this -> setLimit( DEFAULT_TEXT_LIMIT );
  this -> addTo( frame );
  }

/*------------------------------------------------------------------------------
| IInfoArea::~IInfoArea                                                        |
------------------------------------------------------------------------------*/
IInfoArea :: ~IInfoArea()
  {
  IMODTRACE_DEVELOP( "IInfoArea::dtor" );

  // Remove extension...
  if ( this->isValid() )
  {
    IFrameWindow* pfrmParent = (IFrameWindow*)(this->parent());
    if ( pfrmParent && pfrmParent->isFrameWindow() )
      {
      pfrmParent -> removeExtension( this );
      }
  }

  // Delete resource library if we created one...
  delete this->pResLib;
  delete this->fInfoAreaMenuHandler;
  }


/*------------------------------------------------------------------------------
| &IInfoArea::resourceLibrary                                                  |
------------------------------------------------------------------------------*/
IResourceLibrary &IInfoArea :: resourceLibrary ( ) const
/*------------------------------------------------------------------------------
  If we've got our own, return it, else use current user default.
------------------------------------------------------------------------------*/
  {
  if ( this->pResLib )
    return *this->pResLib;
  else
    return IApplication::current().userResourceLibrary();
  }

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| &IInfoArea::setResourceLibrary                                               |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea :: setResourceLibrary ( const IModuleHandle &resMod )
/*------------------------------------------------------------------------------
  Construct an DLL from the input handle.
------------------------------------------------------------------------------*/
{
  delete this->pResLib;
  if (resMod)
  {
    pResLib = new IDynamicLinkLibrary( resMod );
  }
  else
  {
    pResLib = new IResourceLibrary();
  }
  notifyObservers(INotificationEvent(IInfoArea::resourceLibraryId,
                                     *this, true, (void*)pResLib));
  return *this;
}
#endif
/*------------------------------------------------------------------------------
| &IInfoArea::setResourceLibrary                                               |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea :: setResourceLibrary
                          ( const char *resDLLName,
                          IDynamicLinkLibrary::ESearchLocation searchLocation )
/*------------------------------------------------------------------------------
  Construct an DLL from the input dll name.
------------------------------------------------------------------------------*/
{
  delete this->pResLib;
  if (resDLLName)
  {
    pResLib = new IDynamicLinkLibrary( resDLLName, searchLocation );
  }
  else
  {
    pResLib = new IResourceLibrary();
  }
  notifyObservers(INotificationEvent(IInfoArea::resourceLibraryId,
                                     *this, true, (void*)pResLib));
  return *this;
}

/*------------------------------------------------------------------------------
| IInfoArea::defaultText                                                      |
------------------------------------------------------------------------------*/
IString IInfoArea :: defaultText ( ) const
  {
  return this->defaultString;
  }

/*------------------------------------------------------------------------------
| IInfoArea::disabledText                                                      |
------------------------------------------------------------------------------*/
IString IInfoArea :: disabledText ( ) const
  {
  return this->disabledString;
  }

/*------------------------------------------------------------------------------
| IInfoArea::missingText                                                       |
------------------------------------------------------------------------------*/
IString IInfoArea :: missingText  ( ) const
  {
  return this->missingString;
  }

/*------------------------------------------------------------------------------
|  IInfoArea::setDefaultText                                                   |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setDefaultText(unsigned long id)
{
  IString newString(this->resourceLibrary().loadString(id));
  setDefaultText(newString);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setDefaultText                                                   |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setDefaultText(const IString &aString)
{
  if (this->defaultString != aString)
  {
    this->defaultString = aString;
    if (status == kDefaultText)
    {
      this->showText(itemEnabled, 0, kDefaultText, defaultText());
    }
    notifyObservers(INotificationEvent(IInfoArea::defaultTextId,
                                       *this, true, (char*)defaultString));
  }
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setDisabledText                                                  |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setDisabledText(unsigned long id)
{
  IString newString(this->resourceLibrary().loadString(id));
  setDisabledText(newString);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setDisabledText                                                  |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setDisabledText(const IString &aString)
{
  if (this->disabledString != aString)
  {
    this->disabledString = aString;
    if (status == kDisabledText)
    {
      this->showText(itemEnabled, currentItem, kDisabledText, disabledText());
    }
    notifyObservers(INotificationEvent(IInfoArea::disabledTextId,
                                       *this, true, (char*)disabledString));
  }
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setMissingText                                                   |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setMissingText(unsigned long id)
{
  IString newString(this->resourceLibrary().loadString(id));
  setMissingText(newString);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setMissingText                                                   |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setMissingText(const IString &aString)
{
  if (this->missingString != aString)
  {
    this->missingString = aString;
    if (status == kMissingText)
    {
      this->showText(itemEnabled, currentItem, kMissingText, missingText());
    }
    notifyObservers(INotificationEvent(IInfoArea::missingTextId,
                                       *this, true, (char*)missingString));
  }
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::lineCount                                                        |
------------------------------------------------------------------------------*/
unsigned long IInfoArea::lineCount ( ) const
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return this->numberOfLines;
#endif

#ifdef IC_MOTIF
  return 1;
#endif
}

/*------------------------------------------------------------------------------
|  IInfoArea::setLineCount                                                     |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setLineCount ( unsigned long lines )
{
ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  IASSERTPARM(lines != 0);
  if (this->lineCount() != lines)
  {
     this->numberOfLines = lines;

     if (lines > 1)
     {
        this->setAlignment( IStaticText::topLeftWrapped );
     }
     else if (this->alignment() == IStaticText::topLeftWrapped)
     {
        this->setAlignment( IStaticText::topLeft );
     }

     this->setLayoutDistorted( IWindow::minimumSizeChanged, 0 );
  }
#endif // IC_PMWIN
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::calcMinimumSize                                                  |
------------------------------------------------------------------------------*/
ISize IInfoArea::calcMinimumSize ( ) const
{
  ISize sizMin = Inherited::calcMinimumSize();
  unsigned long lines = this->lineCount();
  return ISize( sizMin.width() * 2 / (lines + 1),
                sizMin.height() * lines );
}


/*------------------------------------------------------------------------------
|  IInfoArea::startShowingMenuInfo                                             |
------------------------------------------------------------------------------*/
IInfoArea  &IInfoArea :: startShowingMenuInfoFor ( IFrameWindow *frame )
{
  IASSERTPARM(frame != 0);
   fInfoAreaMenuHandler->handleEventsFor(frame);
   return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::stopShowingMenuInfo                                              |
------------------------------------------------------------------------------*/
IInfoArea  &IInfoArea :: stopShowingMenuInfoFor ( IFrameWindow *frame )
{
  IASSERTPARM(frame != 0);
  fInfoAreaMenuHandler->stopHandlingEventsFor(frame);
  return *this;
}

/*------------------------------------------------------------------------------
| IInfoArea::showText                                                          |
------------------------------------------------------------------------------*/
void IInfoArea :: showText ( bool itemState,  unsigned long itemId,
                               ETextType newStatus, const IString& text )
  {
    status = newStatus;
    if (newStatus == kDefaultText)
    // Allow fly over help to set help text into the information area.
      preventFlyUpdate = false;
    else
    // Prevent fly over help from setting help text into the information area.
      preventFlyUpdate = true;
    showMenuInfo ( itemEnabled, itemId, newStatus, text);
    return;
  }

/*------------------------------------------------------------------------------
| IInfoArea::showMenuInfo                                                      |
------------------------------------------------------------------------------*/
IInfoArea& IInfoArea :: showMenuInfo ( bool itemState,  unsigned long itemId,
                               ETextType newStatus, const IString& text )
  {
    this->setText( text );
    return *this;
  }

/*------------------------------------------------------------------------------
| IInfoArea::informationFor                                                    |
------------------------------------------------------------------------------*/
IString IInfoArea :: informationFor( unsigned long itemId ) const
/*------------------------------------------------------------------------------
  Load the string with menuId + offset and return it.
  If it can't be loaded, throw an exception.
------------------------------------------------------------------------------*/
  {
  IString result =
    this->resourceLibrary().tryToLoadString( itemId +
                                             this->stringTableOffset() );
  return result;
  }

/*------------------------------------------------------------------------------
| IInfoArea::addTo                                                             |
------------------------------------------------------------------------------*/
void IInfoArea :: addTo( IFrameWindow *frame )
/*------------------------------------------------------------------------------
  Add this object as a frame extension at bottom of client area.
------------------------------------------------------------------------------*/
  {
  frame -> addExtension ( this,
                          IFrameWindow::belowClient );
  // The handler is attached to the info area as well as the frame
  // so that it can respond to a user message from fly over help.
  fInfoAreaMenuHandler -> handleEventsFor( this );
  fInfoAreaMenuHandler -> handleEventsFor( frame );
  }


/*------------------------------------------------------------------------------
| IInfoAreaMenuHandler::IInfoAreaMenuHandler                                   |
------------------------------------------------------------------------------*/
IInfoAreaMenuHandler :: IInfoAreaMenuHandler ( IInfoArea    *infoArea)
         : IMenuHandler (),
           fInfoArea( infoArea )
  {
  }

/*------------------------------------------------------------------------------
| IInfoAreaMenuHandler::~IInfoAreaMenuHandler                                  |
------------------------------------------------------------------------------*/
IInfoAreaMenuHandler :: ~IInfoAreaMenuHandler()
{
  if (fInfoArea->isValid() )
  {
    IFrameWindow* pfrmParent = (IFrameWindow*)(fInfoArea->parent());
    if ( pfrmParent && pfrmParent->isFrameWindow() )
    {
       this->stopHandlingEventsFor( pfrmParent );
       this->stopHandlingEventsFor( fInfoArea );
    }
  }
}

/*------------------------------------------------------------------------------
| IInfoAreaMenuHandler::dispatchHandlerEvent                                   |
------------------------------------------------------------------------------*/
bool IInfoAreaMenuHandler::dispatchHandlerEvent(IEvent& evt)
{
#ifdef IC_PMWIN
   // Check for menu item is being de-selected (losing focus)
   if ((evt.eventId() == WM_MENUSELECT) &&
#ifdef IC_WIN
       (evt.parameter1().highNumber() == 0xFFFF))
#endif
#ifdef IC_PM
       (evt.parameter1().lowNumber() == 0xFFFF))
#endif
     {

#ifdef IC_WIN
     // It seems that this MENUSELECT only occurs when the
     // menu is closed...not when a submenu is closed.
     fInfoArea -> itemEnabled = false;
     fInfoArea -> showText( fInfoArea->itemEnabled, 0, IInfoArea::kDefaultText,
                                   fInfoArea->defaultText() );
#endif

#ifndef IC_WIN
     // Get owner window.  If the owner is another menu, then reset the
     // info area text to the menu text, otherwise set inactive text
     IWindowHandle whOwner = IOWNEROF(evt.parameter2());
     if (whOwner)
     {
       IWindowClassName  className(whOwner);
       if ( className == WC_MENU )
       {
         int sItem = whOwner.sendEvent(MM_QUERYSELITEMID).number1();
         if (( sItem == MIT_ERROR) || ( sItem == MIT_NONE ))
         {
            fInfoArea -> itemEnabled = false;
            fInfoArea -> showText( fInfoArea->itemEnabled, 0,
                          IInfoArea::kDefaultText, fInfoArea->defaultText() );
         }
         else
         {
           evt.window()->sendEvent( WM_MENUSELECT, sItem,
                                    (unsigned long) whOwner);
         }
       }
       else
       {
           fInfoArea -> itemEnabled = false;
           fInfoArea -> showText( fInfoArea->itemEnabled, 0,
                         IInfoArea::kDefaultText, fInfoArea->defaultText() );
       }
     }
     else
     {
        fInfoArea -> itemEnabled = false;
        fInfoArea -> showText( fInfoArea->itemEnabled, 0,
                       IInfoArea::kDefaultText, fInfoArea->defaultText() );
     }
#endif

   }
   else if (evt.eventId() == IC_UM_QRY_PREVENTUPDATE)
   {
        evt.setResult(fInfoArea->preventFlyUpdate);
        return true;
   }
#endif // IC_PMWIN

#ifdef IC_MOTIF
   IFUNCTRACE_ALL();
   // Check for menu item is being de-selected (losing focus)
     if (evt.eventId() == WM_MENUUNMAP)
     {
       ITRACE_ALL( IString("IInfoArea WM_MENUUNMAP parm1=") +
                   IString(evt.parameter1().asUnsignedLong()) +
                   IString(" parm2=") +
                   IString(evt.parameter2().asUnsignedLong()).d2x() );
       // IMenu dispatches WM_MENUUNMAP synchronously from UNMAP and DISARM
       // callbacks.
       unsigned long oItem = 0;
       Widget whOwner = (Widget)evt.parameter2().asUnsignedLong();
       if (whOwner)
       {
         IString handleName = XtName(whOwner);
         if (handleName.size()) oItem = handleName.asInt();
         ITRACE_ALL( IString("IInfoArea oItem=") + IString(oItem) );
         fInfoArea -> itemEnabled = false;
         fInfoArea -> showText( fInfoArea->itemEnabled, 0,
                        IInfoArea::kDefaultText, fInfoArea->defaultText() );
        }
     }
   // Check for a new menu being displayed, save the ID, and show the info.
    if (evt.eventId() == WM_INITMENU) {
     ITRACE_ALL( IString("IInfoArea WM_INITMENU parm1=") +
                 IString(evt.parameter1().asUnsignedLong()) +
                 IString(" parm2=") +
                 IString(evt.parameter2().asUnsignedLong()).d2x() );
     Widget whOwner = (Widget)evt.parameter2().asUnsignedLong();
     if (whOwner)
     {
       IString handleName = XtName(whOwner);
       if (handleName.size())
              sItem = handleName.asInt();
       ITRACE_ALL( IString("IInfoArea sItem=") + IString(sItem) );
       if ( sItem == NULL )
       {
         fInfoArea -> itemEnabled = false;
         fInfoArea -> showText( fInfoArea->itemEnabled, 0,
                       IInfoArea::kDefaultText, fInfoArea->defaultText() );
       }
       else
       {
         IString strInfoText=IString();
         strInfoText = fInfoArea->informationFor( sItem );
         if (strInfoText.length() == 0) {
             fInfoArea -> itemEnabled = true;
             fInfoArea -> showText( fInfoArea->itemEnabled, sItem,
                              IInfoArea::kMissingText, fInfoArea->missingText() );
         }/* end if */
         else
         {
           fInfoArea -> itemEnabled = true;
           fInfoArea -> showText( fInfoArea->itemEnabled, sItem,
                                  IInfoArea::kSelectedText, strInfoText );
         }
       }
     }
   } /* end if */
#endif // IC_MOTIF

   return IMenuHandler::dispatchHandlerEvent(evt);
}

/*------------------------------------------------------------------------------
| IInfoAreaMenuHandler::menuSelected                                                         |
------------------------------------------------------------------------------*/
bool IInfoAreaMenuHandler :: menuSelected     ( IMenuEvent& menuEvent )

{
  unsigned long ulStrId=menuEvent.menuItemId();

  ITRACE_ALL(IString("IInfoArea menuSelected  id=") + IString(ulStrId) );
  // Special routine to handle disabled menu items.
  // If user has called setDisabledText() then that text will be
  // displayed for any disabled menu item.
  // Otherwise just display the string from the table.

  bool isDisabled;
  IString strInfoText=IString();

  try {
    isDisabled = menuEvent.menuItem().isDisabled();    //change in IMenuHandler
  }
  catch(...) {
    /* ulStrId not found in menu */
    return false;
  }/* end catch */

  if (isDisabled)
  {
    fInfoArea -> itemEnabled = false;
    if (fInfoArea->disabledText().size())
    {
      fInfoArea -> showText( fInfoArea->itemEnabled, ulStrId,
                    IInfoArea::kDisabledText, fInfoArea->disabledText() );
    }
    else
    {
      strInfoText=fInfoArea -> informationFor( ulStrId );
      if (strInfoText.length() == 0)
      {                // string not found in resource
         fInfoArea -> itemEnabled = false;
         fInfoArea -> showText( fInfoArea->itemEnabled, ulStrId,
                        IInfoArea::kMissingText, fInfoArea->missingText() );
      } /* end if */

    }
    return false;
  }

  strInfoText=fInfoArea -> informationFor( ulStrId );
  if (strInfoText.length() == 0)
  {                // string not found in resource
     fInfoArea -> itemEnabled = true;
     fInfoArea -> showText( fInfoArea->itemEnabled, ulStrId,
                    IInfoArea::kMissingText, fInfoArea->missingText() );
  } /* end if */
  else
  {
    fInfoArea -> itemEnabled = true;
    fInfoArea -> showText( fInfoArea->itemEnabled, ulStrId,
                  IInfoArea::kSelectedText, strInfoText );
  }
  return false;
}

/*------------------------------------------------------------------------------
|  Obsoleted functions                                                         |
------------------------------------------------------------------------------*/
#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )

/*------------------------------------------------------------------------------
|  IInfoArea::handleEventsFor                                                  |
------------------------------------------------------------------------------*/
IInfoArea  &IInfoArea :: handleEventsFor ( IFrameWindow *frame )
{
  IASSERTPARM(frame != 0);
  fInfoAreaMenuHandler->handleEventsFor(frame);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::stopHandlingEventsFor                                            |
------------------------------------------------------------------------------*/
IInfoArea  &IInfoArea :: stopHandlingEventsFor ( IFrameWindow *frame )
{
  IASSERTPARM(frame != 0);
  fInfoAreaMenuHandler->stopHandlingEventsFor(frame);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setInactiveText                                                  |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setInactiveText(unsigned long id)
{
  setDefaultText(id);
  return *this;
}

/*------------------------------------------------------------------------------
|  IInfoArea::setInactiveText                                                  |
------------------------------------------------------------------------------*/
IInfoArea &IInfoArea::setInactiveText(const IString &aString)
{
  setDefaultText(aString);
  return *this;
}

/*------------------------------------------------------------------------------
| IInfoArea::inactiveText                                                      |
------------------------------------------------------------------------------*/
IString IInfoArea :: inactiveText ( ) const
  {
  return defaultText();
  }

#endif // IC_OBSOLETE

/* end IInfoArea  */


