// Revision: 00 1.5.1.10 source/ui/basectl/ilistcvw.c, listctls, ioc.v400, 000302 
#ifndef _ILISTCVW_C_
#define _ILISTCVW_C_
/*******************************************************************************
* FILE NAME: ilistcvw.c                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistcvw.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp. 1992, 1997  All Rights Reserved.                   *
*                                                                              *
*******************************************************************************/

#include  <icollvwi.hpp>
#include  <ilistcvw.hpp>
#include  <icolvwd.hpp>
#include  <iseq2.h>
#include  <icconst.h>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

template <class Element, class Collection>
  const unsigned long
    ICollectionViewListBox<Element, Collection>::noSelection = 0;


template <class Element, class Collection>
  class ICollectionViewListBoxData {

public:

  ICollectionViewListBoxData( const IStringGenerator<Element>& stringGenerator );

 ~ICollectionViewListBoxData ( );

ICollectionViewData< Element,
                     ICollectionViewListBox< Element, Collection >,
                     Collection >
 *fCollectionViewData;

bool saveSelectionState;

private:
/*------------------------- Hidden Members -----------------------------------*/
  ICollectionViewListBoxData
             ( const ICollectionViewListBoxData< Element, Collection >& source );
ICollectionViewListBoxData<Element,Collection>
 &operator=  ( const ICollectionViewListBoxData< Element, Collection >& source );

};

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

template <class Element, class Collection>
  ICollectionViewListBoxData<Element, Collection>::
    ICollectionViewListBoxData( const IStringGenerator<Element>& stringGenerator )
      : saveSelectionState( false )
{
  fCollectionViewData = new
        ICollectionViewData< Element,
                             ICollectionViewListBox< Element, Collection >,
                             Collection > ( stringGenerator) ;
}

template <class Element, class Collection>
  ICollectionViewListBoxData<Element,Collection>::
   ~ICollectionViewListBoxData ( )
{
  delete fCollectionViewData;
}


template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>::
    ICollectionViewListBox ( unsigned long                    identifier,
                             IWindow*                         parent,
                             IWindow*                         owner ,
                             const IRectangle&                initial,
                             const IBaseListBox::Style&       style,
                             const IStringGenerator<Element>& stringGenerator )
    : IBaseListBox( identifier, parent, owner, initial, style )
{
  fCollectionViewListBoxData = new ICollectionViewListBoxData<Element, Collection>( stringGenerator );
  fCollectionViewListBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}


template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>::
    ICollectionViewListBox ( unsigned long                    identifier,
                             IWindow*                         parent,
                             const IStringGenerator<Element>& stringGenerator )
    : IBaseListBox( identifier, parent )
{
  fCollectionViewListBoxData = new ICollectionViewListBoxData<Element, Collection>( stringGenerator );
  fCollectionViewListBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>::
    ICollectionViewListBox( const IWindowHandle&             handle,
                            const IStringGenerator<Element>& stringGenerator )
    : IBaseListBox( handle )
{
  fCollectionViewListBoxData = new ICollectionViewListBoxData<Element, Collection>( stringGenerator );
  fCollectionViewListBoxData->fCollectionViewData->collectionObserver().setViewer( this );
  enableNotification();
}

// implement empty copy constructor to work-around compiler RTTI bug
template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>::
    ICollectionViewListBox( const ICollectionViewListBox< Element, Collection >& source )
   : IBaseListBox (9200, 0, 0)
{}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>::
   ~ICollectionViewListBox ( )
{
  disableNotification();

  delete fCollectionViewListBoxData;
}


template <class Element, class Collection>
  ICollectionViewListBox<Element, Collection>& ICollectionViewListBox<Element, Collection>::
    selectedElements ( Collection& elements )
{

  elements.removeAll( );

  IPartOrderedCollection<Element>* pSequence = items();

  if( pSequence )
  {
    ICursor*                         pCursor   = pSequence->newCursor();
    IBaseListBox::Cursor             cursor( *this );//defaults to selected items


    forCursor( cursor )
    {
      pSequence->setToPosition( cursor.asIndex()+1, *pCursor );
      elements.add( pSequence->elementAt( *pCursor ));
    }
    delete pCursor;
  }

  return *this;
}


template <class Element, class Collection>
  ICollectionViewListBox<Element, Collection>&
    ICollectionViewListBox<Element, Collection>::
      enableExtendedSelect( bool  extended )
{

  if ( extended != IBaseListBox::isExtendedSelect( ) )
  {
    IBaseListBox::enableExtendedSelect ( extended );

    INotificationEvent  anEvent(
         IBaseListBox::extendedSelectChangedId,
         *this );
    notifyObservers( anEvent );
  }
  return *this;
}


#include <inotifev.hpp>
#include <iexcept.hpp>


template <class Element, class Collection>
  Collection*  ICollectionViewListBox<Element,Collection>:: items( ) const
{
  return  (Collection *)fCollectionViewListBoxData->fCollectionViewData->collectionObserver().collection();
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&   ICollectionViewListBox<Element,Collection>::
    setItems( Collection* collection )
{
  fCollectionViewListBoxData->fCollectionViewData->collectionObserver().setCollection( collection );

  return *this;
}

template <class Element, class Collection>
  IStringGenerator<Element>&  ICollectionViewListBox<Element,Collection>::
    stringGenerator ( )
{
  return  fCollectionViewListBoxData->fCollectionViewData->generator();
}

template <class Element, class Collection>
  IStringGenerator<Element>&
    ICollectionViewListBox<Element,Collection>::
      setStringGenerator ( const IStringGenerator<Element>&  newGenerator )
{
  fCollectionViewListBoxData->fCollectionViewData->setGenerator( (IStringGenerator<Element>&)newGenerator );

  fCollectionViewListBoxData->saveSelectionState = true;
  elementsChanged ( );
  fCollectionViewListBoxData->saveSelectionState = false;

  return  fCollectionViewListBoxData->fCollectionViewData->generator();
}


template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>& ICollectionViewListBox<Element,Collection> :: elementsChanged ( )
{
  if ( isWindowValid( this ) )
  {
  ICursor*                  pCursor;
  Collection*               collection;
  long                      selection;

  collection = items( );
  IWindow::enableUpdate( false );

  /* Don't forget the case where collection is NULL  */
  if ( collection )
  {
    if ( ( collection->numberOfElements() > 0 ) &&
         fCollectionViewListBoxData->saveSelectionState )
    {
      selection  = IBaseListBox::selection();
    }
    else   // this is a removeall so deselect list box items
    {
      IBaseListBox::deselectAll( );
    }
  }

  /*  Clear list box and repopulate w/ current string representations */
  deleteAll( );
  if ( collection )
  {
    pCursor = collection->newCursor();
    for (pCursor->setToFirst(); pCursor->isValid(); pCursor->setToNext() )
    {
      insert( ICollectionViewConstants::listEnd,
        fCollectionViewListBoxData->fCollectionViewData->generator().stringFor( collection->elementAt( *pCursor ) ) );
    }
    delete pCursor;
  }

  if ( !IBaseListBox::isEmpty( ) &&
       fCollectionViewListBoxData->saveSelectionState )
    if ( selection != ICollectionViewConstants::listNone )
      IBaseListBox::select( selection, true );

  IWindow::enableUpdate( );
  IWindow::show( );
  }
  return *this;
}


template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&  ICollectionViewListBox<Element,Collection>::
    elementAdded ( unsigned long position, const Element&  element )
{
  insert( position - 1,
    fCollectionViewListBoxData->fCollectionViewData->generator().stringFor( element ) );
  return *this;
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&  ICollectionViewListBox<Element,Collection>::
    elementDeleted ( unsigned long position )
{
  deleteItem( position - 1 );

  return *this;
}


template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&  ICollectionViewListBox<Element,Collection>::
    elementChanged ( unsigned long  position, const Element&  element )
{
  IBaseListBox::setItemText(
     position - 1,
     fCollectionViewListBoxData->fCollectionViewData->generator().stringFor(element) );
  INotificationEvent  anEvent(
     IBaseListBox::itemChangedId,
    *this,
     true ,
     IEventData((void *)&element) );
  notifyObservers( anEvent );
  return *this;
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&  ICollectionViewListBox<Element,Collection> :: collectionReplaced( )
{
  if ( isWindowValid( this ) )
  {
    INotificationEvent  anEvent( ICollectionViewListBox<Element,Collection>::itemsId,
                                *this,
                                 true,
                                 IEventData( (void *)items( ) ) );
    notifyObservers( anEvent );
  }

  return *this;
}


template <class Element, class Collection>
  unsigned long ICollectionViewListBox<Element,Collection>::
    selectedCollectionPosition ( )
{
  return IBaseListBox::selection( ) + 1;
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&
    ICollectionViewListBox<Element,Collection>::
      select ( unsigned long collectionPosition ,
               bool       select )
{
  if ( (collectionPosition > 0 ) &&
       (collectionPosition <= IBaseListBox::count()) )
    IBaseListBox::select( collectionPosition - 1, select );

  return *this;
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>&
    ICollectionViewListBox<Element,Collection>::
      deselect ( unsigned long collectionPosition )
{
  return select( collectionPosition, false );
}

template <class Element, class Collection>
  Element       ICollectionViewListBox<Element,Collection>::
    selectedElement ( ) const
{
  unsigned long                    selectedPosition;
  Element*						   pElement;
  IPartOrderedCollection<Element>* aCollection = items( );

  if( aCollection )
  {
    IAutoPointer<ICursor>            pCursor( aCollection->newCursor( ), IINIT );

    selectedPosition = IBaseListBox::selection( );
    if ( selectedPosition == ICollectionViewConstants::listNone )
    {
      ITHROWLIBRARYERROR( IC_NO_SELECTION,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable );
    }

    aCollection->setToPosition( selectedPosition + 1, *pCursor );
    pElement = &(aCollection->elementAt( *pCursor ));
  }
  else // no collection associated with this list box
  {
    ITHROWLIBRARYERROR( IC_NO_SELECTION,
                    IBaseErrorInfo::invalidRequest,
                    IException::recoverable );
  }

  return *pElement;

}

template <class Element, class Collection>
  unsigned long ICollectionViewListBox<Element,Collection>::
    insert( unsigned long index,
            const char*   text )
{
   incrementChangeCount( );   // add may result in an invalid cursor.
   unsigned long anIndex = count();
   if ((long)anIndex > (long)index)
     anIndex = index;

   IEventResult evt = handle().sendEvent(
                               ICollectionViewConstants::listInsert,
                               IEventParameter1(anIndex),
                               IEventParameter2((unsigned long) text));

   if ( ( evt.asLong() == ICollectionViewConstants::listMemoryError ) ||
        ( evt.asLong() == ICollectionViewConstants::listError ) )
     ITHROWGUIERROR("ICollectionViewConstants::listInsert");

   return evt.asUnsignedLong();  // Return index of inserted item.
}


template <class Element, class Collection>
  unsigned long ICollectionViewListBox<Element,Collection>::
    deleteItem ( unsigned long index )
{
   bool selected = isSelected( index );

   incrementChangeCount( );   // add may result in an invalid cursor.
   IEventResult evt = handle().sendEvent(
                               ICollectionViewConstants::listDelete,
                               IEventParameter1(index),
                               IEventParameter2(0));

   if ( selected )
   {
     INotificationEvent  anEvent( ICollectionViewListBox<Element,Collection>::selectId,
                                 *this );
     notifyObservers( anEvent );
   }


   return evt.asUnsignedLong();
}

template <class Element, class Collection>
  ICollectionViewListBox<Element,Collection>& ICollectionViewListBox<Element,Collection>::
      deleteAll ( )
{
   IEventResult evt = handle().sendEvent(
                               ICollectionViewConstants::listDeleteAll,
                               IEventParameter1(0),
                               IEventParameter2(0));
#ifdef IC_PM
   if (!(evt.asUnsignedLong()))
      ITHROWGUIERROR("ICollectionViewConstants::listDeleteAll");
#endif
   incrementChangeCount( );   // add may result in an invalid cursor.
   return *this;
}

#endif // _ILISTCVW_C_
