// Revision: 14 1.6.1.2 source/ui/basectl/imle1.cpp, editctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imle1.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imlenhdr.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )


extern "C"
{
  #define INCL_WINMLE
  #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <imle.hpp>
#include <imlenhdr.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMLE1_CPP_
  #include <ipagetun.h>
#endif


// Initialize static objects and notification strings.
INotificationId const IMultiLineEdit::dataUpdateId = "IMultiLineEdit::dataUpdate";
INotificationId const IMultiLineEdit::limitId      = "IMultiLineEdit::limit";

/*------------------------------------------------------------------------------
| IMultiLineEdit::enableNotification                                           |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IMultiLineEdit, does not send notifications to observer objects until it is  |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IMultiLineEdit& IMultiLineEdit::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new IMultiLineEditNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::IMultiLineEditNotifyHandler                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IMultiLineEditNotifyHandler::IMultiLineEditNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::~IMultiLineEditNotifyHandler                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IMultiLineEditNotifyHandler::~IMultiLineEditNotifyHandler()
{}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::dispatchHandlerEvent                            |
|                                                                              |
| This notifies IMultiLineEdit Windows of the following events:                |
|                                                                              |
| dataUpdate(Id)                                                               |
| limit(Id)                                                                    |
------------------------------------------------------------------------------*/
bool IMultiLineEditNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case MLM_SETREADONLY:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the readonly state has been modified                       */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_MLE_DATAUPD ));
        break;

      case MLM_SETTEXTLIMIT:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the limit has actually been set                            */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_MLE_LIMIT ));
        break;

    /******************************************************************/
    /* This is the delayed notification message.  Check to see which  */
    /* notification should be sent to the observers based on info     */
    /* stored in parameter 2 of the private message.  Parameter 1     */
    /* contains the information passed in the original event that     */
    /* needs to be sent with the notification                         */
    /******************************************************************/
    case IC_UM_DELAYNOTIFY:

      if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_MLE_DATAUPD )
      {
        thisWindow->notifyObservers(INotificationEvent(
                                    IMultiLineEdit::dataUpdateId,
                                    *thisWindow, true,
                    (void*)(!(anEvent.parameter1().asUnsignedLong()))));
      }
      else if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_MLE_LIMIT )
      {
        thisWindow->notifyObservers(INotificationEvent(
                                    IMultiLineEdit::limitId,
                                    *thisWindow, true,
                                    (void*)anEvent.parameter1()));
      }
      break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN
#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::dispatchHandlerEvent                            |
|                                                                              |
| This notifies IMultiLineEdit Windows of the following events:                |
|                                                                              |
| dataUpdate(Id)                                                               |
| limit(Id)                                                                    |
------------------------------------------------------------------------------*/
bool IMultiLineEditNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif

