// Revision: 16 1.6.1.2 source/ui/basectl/imphdr.cpp, userinput, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: imphdr.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imphdr.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <imphdr.hpp>
#include <iwindow.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMPHDR_CPP_
  #include <ipagetun.h>
#endif

/**************************************************************************/
/* Class to handle destruction of the static IMousePointerHandler object. */
/**************************************************************************/
#pragma enum(4)
#pragma pack(push,4)

class IMousePointerHandlerStatics {
public:
 ~IMousePointerHandlerStatics ( )
  {
    IMODTRACE_DEVELOP("IMousePointerHandlerStatics::~IMousePointerHandlerStatics");
#if IC_STATIC_PRIORITY_SUPPORTED
    // Delete is only safe if static init/term ordering can be controlled.
    if ( pDefHandler )
    {
       delete pDefHandler;
    }
#endif
  }

IMousePointerHandler
 *pDefHandler;
};  // IMousePointerHandlerStatics

#pragma pack(pop)
#pragma enum(pop)

static IMousePointerHandlerStatics
  IDefaultMPStatics;


/*******************************************************************/
/* Default constructor.                                            */
/*******************************************************************/
IMousePointerHandler::IMousePointerHandler ( )
  : IMousePointerHandler::Inherited( IMouseHandler::someMouseMoves )
{ }

/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IMousePointerHandler::~IMousePointerHandler ( )
{ }

/***********************************************************************/
/* IC_OBSOLETE note:                                                   */
/* When IMouseHandler::mousePointerChange is obsoleted, a              */
/* dispatchHandlerEvent function must be added to IMousePointerHandler */
/* to call its changeMousePointer function, so that derived classes    */
/* overriding this function will continue to be called.                */
/***********************************************************************/

#ifdef IC_PMWIN
/*********************************************************************/
/* IMousePointerHandler::changeMousePointer                          */
/*                                                                   */
/* Pass the WM_CONTROLPOINTER/WM_SETCURSOR message to the owner if   */
/* this handler is attached to is an aggregate control (for example, */
/* a spin button or combo box).  This message passing allows a       */
/* fly-over help handler and IWindow::setMousePointer to support     */
/* composite controls which would not ordinarily pass this message   */
/* to its parent or owner window.                                    */
/*********************************************************************/
bool IMousePointerHandler::mousePointerChange( IMousePointerEvent& event )
{
  IWindow *powner = event.window()->owner();
  if ( powner  &&
       event.window() != powner  &&
       event.window() == event.dispatchingWindow() )
  {
     event.setResult( powner->sendEvent( event ) );
     return true;
  }
  return false;
}
#endif // IC_PMWIN

/*******************************************************************/
/* IMousePointerHandler::defaultHandler                            */
/*******************************************************************/
IMousePointerHandler* IMousePointerHandler::defaultHandler ( )
{
   if ( ! IDefaultMPStatics.pDefHandler )
   {
      IDefaultMPStatics.pDefHandler = new IMousePointerHandler();
   }
   return IDefaultMPStatics.pDefHandler;
}
