// Revision: 29 1.16.1.11 source/ui/basectl/iradiobt.cpp, buttonctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: iradiobt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iradiobt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

// #define IC_TRACE_ALL

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINERRORS
  #define INCL_GPIBITMAPS
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#ifdef IC_MOTIF
  #include <Xm/Label.h>
  #include <Xm/ToggleB.h>
#endif  // IC_MOTIF

#include <ibidiset.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <inotifev.hpp>
#include <ipoint.hpp>
#include <iradiobt.hpp>
#include <iradiobp.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#ifdef IC_WIN
  #include <ibrushdr.hpp>
#endif
#ifdef IC_MOTIF
  #include <imstring.hpp>
  #include <ixmlabel.hpp>
#endif // IC_MOTIF


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRADIOBT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public radio button styles.                                                  |
------------------------------------------------------------------------------*/
const IRadioButton::Style
#ifdef IC_MOTIFPM
  IRadioButton::noCursorSelect    = BS_NOCURSORSELECT,
#endif // IC_MOTIFPM
#ifdef IC_WIN
  IRadioButton::noCursorSelect    = 0,    // Style ignored
#endif
  IRadioButton::autoSelect        ( 0, IBS_AUTORADIOBUTTON ),
  IRadioButton::classDefaultStyle ( WS_VISIBLE, IBS_AUTORADIOBUTTON );

const unsigned long
  IRadioButton::noSelections      = (unsigned long)ULONG_MAX;

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
  IRadioButton::Style
        IRadioButton::currentDefaultStyle ( WS_VISIBLE, IBS_AUTORADIOBUTTON );


#ifdef IC_PMWIN
  ISize IRadioButtonData::gBitmapSize ( 0, 0 );
#endif


#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| IRadioButtonData::IRadioButtonData                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IRadioButtonData::IRadioButtonData ( )
#ifdef IC_WIN
  : fdefaultHandler( )
  , fBrushHandler()
#endif //IC_WIN
{ }

/*------------------------------------------------------------------------------
| IRadioButtonData::~IRadioButtonData                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IRadioButtonData::~IRadioButtonData ( )
{
}
#endif //IC_MOTIFWIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| IRadioButtonData::groupSelection                                             |
| Returns the index from the start of the group containing "widget" of the     |
| first radio button with a "set" state.  If the "resetOthers" flag is true,   |
| the set state of all radio buttons except "widget" in the group is set to    |
| false.                                                                       |
------------------------------------------------------------------------------*/
int IRadioButtonData::groupSelection( Widget widget, bool resetOthers)
{
   int result( (int)IRadioButton::noSelections );
   Widget parent = XtParent( widget );
   WidgetList children(0);
   Cardinal   numChildren(0);
   if (XtIsComposite( parent ) )
   {
      XtVaGetValues( parent,
                     XmNchildren    , &children,
                     XmNnumChildren , &numChildren,
                     0);
   }

   if (numChildren > 1)
   {   // The parent has children, so there may be something to do.
      int current(-1);
      // Find the current widget in the list as a starting point.
      int i;
      for (i = 0; (i < numChildren) && (current < 0); i++)
      {
         if (children[i] == widget)
            current = i;
      }
      // Find the first window in the group.  By default, it is the
      // first child.
      int first(numChildren);
      for (i = current; first == numChildren; i--)
      {
         IWindow* window = IWindow::windowWithHandle( children[i] );
         if ( (window && window->isGroup()) || (i == 0) )
            first = i;
      }
      // Now, start with the first window in group and clear the
      // set state of any toggle buttons in the group.
      i = first;
      while (i < numChildren)
      {
         Widget child = children[i];

         // See if the child is a radio button.
         if ( XtIsSubclass(child, xmToggleButtonWidgetClass) )
         {
            unsigned char indicatorType;
            XtVaGetValues( child,
                           XmNindicatorType, &indicatorType,
                           0);
            if (indicatorType == XmONE_OF_MANY)
            {
               // If we have been asked to reset all the other widgets
               // in the group and this is not the "current" widget,
               // reset the indicator.
               if ( resetOthers && (i != current))
               {
                  // Do not generate a value changed callback.  (possiblity
                  // of recursion)
                  XmToggleButtonSetState( child, false, false);

                  // Because we do not generate a value changed callback,
                  // we need to generate a notification event.  Fire the
                  // notification event that the button/ state is changed.
                  IWindow* notifyWin = IWindow::windowWithHandle( child );
                  if (notifyWin)
                  {
                     notifyWin->notifyObservers(
                        INotificationEvent(ISettingButton::selectId,
                                           *notifyWin, true, false));
                  }
               }

               // Check to see if this button is set for our return value.
               if ( (result == (int) IRadioButton::noSelections) &&
                    XmToggleButtonGetState( child ) )
                  result = i - first;
            }  // if radio button
         }  // if toggle button

         // Move to the next child.
         i++;

         // If the next window is start of the next group, terminate loop.
         if (i < numChildren)
         {
            IWindow* window = IWindow::windowWithHandle( children[i] );
            if ( window && window->isGroup() )
               i = numChildren;
         }
      }  // while
   }  // if children
   else
   {
      // Single child assumed to be radio button.  Return its index (0) if
      // it is set.
      if ( XmToggleButtonGetState( widget ) )
         result = 0;
   }

   return result;
}

/*------------------------------------------------------------------------------
| iradioButtonCallback                                                         |
------------------------------------------------------------------------------*/
void _System iradioButtonCallback( Widget widget,
                                   XtPointer client_data,
                                   XtPointer call_data)

{
  XmToggleButtonCallbackStruct *callbackStruct =
    (XmToggleButtonCallbackStruct * ) call_data;
  if ( callbackStruct->reason == XmCR_VALUE_CHANGED )
  {
     // If button is being set, then clear indicators in all others in group.
     if ( XmToggleButtonGetState( widget ) )
     {
        // Button is being set. Clear indicators in all others in group.
        IRadioButtonData::groupSelection( widget, true );
     }
     else if ( IRadioButtonData::groupSelection( widget, false ) < 0 )
     {
        // Button is being reset and there are no others set.  Set this one
        // because it is due to clicking on already set button.
        XmToggleButtonSetState( widget, true, false );
     }
  }  // if value changed
}
#endif // IC_MOTIF

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IRadioHandler::IRadioHandler                                                 |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IRadioHandler::IRadioHandler()
{ }

/*------------------------------------------------------------------------------
| IRadioHandler::~IRadioHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IRadioHandler::~IRadioHandler()
{ }

/*------------------------------------------------------------------------------
| IRadioHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Intercepts control color event                                               |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
bool IRadioHandler::dispatchHandlerEvent( IEvent& event )
{
  bool
    result = false;
  switch( event.eventId() )
  {
    case BM_SETCHECK:
      if (event.parameter1().asUnsignedLong())
         {
         // This radio button is being checked.  We have to find any disabled
         // radio buttons in the group ourselves and uncheck them, since
         // Windows does not do this.
         IWindowHandle
            hwnd    ( event.handle() ),
            current ( event.handle() );

         // Find the beginning of the group.  We don't use GetNextDlgGroupItem
         // since it ignores disabled controls...
         while ( (current) && ((ISTYLEOF(current) & WS_GROUP) == 0) )
            {
            current = GetWindow( current, GW_HWNDPREV );
            }

         // handle case where WS_GROUP is not set...Windows does this when
         // you click
         if (!current)
            current = GetWindow( hwnd, GW_HWNDFIRST );

         // Loop through the group, unchecking any disabled radio buttons
         do
            {
            unsigned long style = ISTYLEOF(current);
            if ((current != hwnd) &&
                (style & WS_DISABLED) &&
                (current.sendEvent(WM_QUERYDLGCODE, 0, 0) & DLGC_RADIOBUTTON))
               {
               // Uncheck the button.
               current.sendEvent(BM_SETCHECK,
                                 IEventParameter1(0),
                                 IEventParameter2(0));
               // If tabstop is set turn it off.  Otherwise we have
               // key navigation problems.
               if (style & WS_TABSTOP)
                  {
                  style &= (unsigned long)~WS_TABSTOP;
                  ISETWINDOWSTYLE(current, style);
                  }
               }
            current = GetWindow( current, GW_HWNDNEXT);
            }
         while ( (current) && ((ISTYLEOF(current) & WS_GROUP) == 0) );

         }
      break;

    default:
      break;
  }
  return result;
}
#endif //IC_WIN

/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to create a radio button on a standard window.                   |
------------------------------------------------------------------------------*/
IRadioButton::IRadioButton ( unsigned long ulId,
                             IWindow* parent,
                             IWindow* pwndOwner,
                             const IRectangle& rectInit,
                             const Style& rbsStyle )
#ifdef IC_MOTIFWIN
  : fRadioButtonData( new IRadioButtonData )
#endif // IC_MOTIFWIN
#ifdef IC_PM
  : fRadioButtonData( 0 )
#endif // IC_PM
{
#ifdef IC_PMWIN
  IASSERTPARM(parent!=0);             //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | rbsStyle.asExtendedUnsignedLong() );

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (parent)
    parentHandle = parent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();

  unsigned long
    createStyle = this->convertToGUIStyle( rbsStyle );

#ifdef IC_WIN
  // Check if we need to supply bidi support to turn the radio button
  // into a right-to-left one.
  if ( IBidiSettings::isBidiSupported() )
  {
     // IWindow bidi styles take precedence over inherited bidi
     // attributes.
     if ( rbsStyle & IWindow::rightToLeft )
     {  // Change the button into a right-to-left one.
        createStyle |= BS_RIGHTBUTTON;
     }
     else if ( ! ( rbsStyle & IWindow::leftToRight ) )
     {
        // The radio button inherits bidi attributes from its parent
        // window.
        IBidiSettings
          bidiValues( *parent );
        if ( bidiValues.windowLayout() ==
                           IBidiSettings::layoutRightToLeft )
        {
           // Change the control into a right-to-left one.
           createStyle |= BS_RIGHTBUTTON;
        }
     }
  }
#endif // IC_WIN

  //Create radio button control
  IWindowHandle whRadioButton =
      this -> create( ulId,
                      0,
                      createStyle,
                      WC_BUTTON,
                      parentHandle,
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( rbsStyle, true ) );

  startHandlingEventsFor(whRadioButton);
#ifdef IC_WIN
  fRadioButtonData->fdefaultHandler.handleEventsFor( this );
  fRadioButtonData->fBrushHandler.handleEventsFor(this);
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Widget widgetID;
  Arg argList[3];
  int i;
  // assertions on input parms
  IASSERTPARM(parent!=0);


  i=0;

  XtSetArg( argList[i], XmNindicatorType, XmONE_OF_MANY   ); i++;

  // Determine the effect of recompute size on the button
  XtSetArg( argList[i], XmNrecomputeSize, false);i++;

  // Left align button text
  XtSetArg( argList[i], XmNalignment, XmALIGNMENT_BEGINNING); i++;

  IWindowHandle parentHandle(0);
  IWindowHandle ownerHandle(0);
  if (parent)
    parentHandle = parent->handleForChildCreation();
  else
    parentHandle = IWindow::desktopWindow()->handle();
  if (pwndOwner)
    ownerHandle = pwndOwner->handle();
  widgetID =
    Inherited::create(
      ulId,
      NULL,
      rbsStyle.asUnsignedLong(),
      (IXmCreateFunction)XmCreateToggleButton,
      parentHandle,
      ownerHandle,
      rectInit,
      argList,
      i);

  if (widgetID == 0)
     ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "radio button toggle");

  startHandlingEventsFor(widgetID);
  // If the label string is equal to the widget name, set it to the null
  // string.  MUST do this conditionally after create to allow the use of
  // resource files.
  IXmLabel::initializeXmLabelString( widgetID, IString( ulId ), true );

  // Register the valueChanged callback for private callback.
  XtAddCallback (widgetID,
                 XmNvalueChangedCallback,
                 iradioButtonCallback,
                 (XtPointer) this);

  // Register the valueChanged callback for client handlers.
  XtAddCallback (widgetID,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
IRadioButton::IRadioButton ( unsigned long id,
                             IWindow* parentDialog)
#ifdef IC_MOTIFWIN
                               : fRadioButtonData( new IRadioButtonData )
#endif // IC_MOTIFWIN
#ifdef IC_PM
                               : fRadioButtonData( 0 )
#endif // IC_PM
{
#ifdef IC_PMWIN
  setAutoDestroyWindow( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor(id, parentDialog);
#ifdef IC_WIN
  fRadioButtonData->fdefaultHandler.handleEventsFor( this );
  fRadioButtonData->fBrushHandler.handleEventsFor(this);
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // assertions on input parms
  IASSERTPARM(parentDialog!=0);

  IString widgetName = IString (id);
  IWindowHandle whRadioButton =
                  XtNameToWidget(parentDialog->handle(), widgetName);
  if (whRadioButton == 0)
      ITHROWLIBRARYERROR1( IC_CONTROL_NOT_FOUND,
                           IBaseErrorInfo::accessError,
                           IException::recoverable,
                           "XtNameToWidget (radio button toggle)");
   if ( !(XtIsSubclass(whRadioButton, xmToggleButtonWidgetClass)) )
     ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "XmToggleButton");
  setAutoDestroyWindow(false);
  prepareForUse(whRadioButton);
  startHandlingEventsFor(whRadioButton);

  // Register the valueChanged callback for private callback.
  XtAddCallback (whRadioButton,
                 XmNvalueChangedCallback,
                 iradioButtonCallback,
                 (XtPointer) this);
  // Register the valueChanged callback for client handlers.
  XtAddCallback (whRadioButton,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to instantiate from an existing radio button control.            |
------------------------------------------------------------------------------*/
IRadioButton::IRadioButton ( const IWindowHandle& wh )
#ifdef IC_MOTIFWIN
                               : fRadioButtonData( new IRadioButtonData )
#endif // IC_MOTIFWIN
#ifdef IC_PM
                               : fRadioButtonData( 0 )
#endif // IC_PM
{
#ifdef IC_PMWIN
  setAutoDestroyWindow( false );
  reserveUserWindowWord( false );
  startHandlingEventsFor( wh );
#ifdef IC_WIN
  fRadioButtonData->fdefaultHandler.handleEventsFor( this );
  fRadioButtonData->fBrushHandler.handleEventsFor(this);
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IASSERTPARM(wh!=0);

  if ( !(XtIsSubclass(wh, xmToggleButtonWidgetClass)) )
    ITHROWLIBRARYERROR1( IC_WRONG_CONTROL_TYPE,
                         IBaseErrorInfo::accessError,
                         IException::recoverable,
                         "XmToggleButton");
  setAutoDestroyWindow(false);
  prepareForUse(wh);
  startHandlingEventsFor(wh);

  // Register the valueChanged callback for private callback.
  XtAddCallback (wh,
                 XmNvalueChangedCallback,
                 iradioButtonCallback,
                 (XtPointer) this);
  // Register the valueChanged callback for client handlers.
  XtAddCallback (wh,
                 XmNvalueChangedCallback,
                 iwindowMotifCallback,
                 (XtPointer) this);
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IRadioButton:~IRadioButton                                                   |
------------------------------------------------------------------------------*/
IRadioButton::~IRadioButton ( )
{
#ifdef IC_WIN
  fRadioButtonData->fdefaultHandler.stopHandlingEventsFor( this );
  fRadioButtonData->fBrushHandler.stopHandlingEventsFor(this);
#endif //IC_WIN
#ifdef IC_MOTIF
  if (this->isValid())
  {
     // Remove the valueChanged callback.
     XtRemoveCallback (handle(),
                       XmNvalueChangedCallback,
                       iwindowMotifCallback,
                       (XtPointer) this);
     // Remove the valueChanged callback.
     XtRemoveCallback (handle(),
                       XmNvalueChangedCallback,
                       iradioButtonCallback,
                       (XtPointer) this);
  }
#endif //IC_MOTIF
  if ( fRadioButtonData != 0 )
    delete fRadioButtonData;
}


/*------------------------------------------------------------------------------
| IRadioButton:defaultStyle                                                    |
|                                                                              |
| Return the default style for new radio button objects.                       |
------------------------------------------------------------------------------*/
IRadioButton::Style  IRadioButton::defaultStyle ( )
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IRadioButton:setDefaultStyle                                                 |
|                                                                              |
| Replace the default style for new radio button objects.                      |
------------------------------------------------------------------------------*/
void IRadioButton::setDefaultStyle ( const Style& rbsStyle )
{
  currentDefaultStyle = rbsStyle;
}

/*------------------------------------------------------------------------------
| IRadioButton::convertToGUIStyle                                              |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IRadioButton::convertToGUIStyle(const IBitFlag& guiStyle,
                                              bool bExtOnly) const
{
  // Obtain the style from the class (ISettingButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTRADIOBTN;

    // Handle overlapping styles
    if (ulTmp & IBS_AUTORADIOBUTTON)
      ulStyle |= BS_AUTORADIOBUTTON;
    else
      ulStyle |= BS_RADIOBUTTON;

#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IRadioButton:enableAutoSelect                                                |
|                                                                              |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
| -- Motif always uses autoSelect.                                             |
| Sets or removes the autoSelect style for the radio button control.           |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton::enableAutoSelect ( bool bTurnOn )
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking flag from pmwin.h
    ulStyle |= BS_AUTORADIOBUTTON;
  }
  else
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking flag from pmwin.h
    ulStyle |= BS_RADIOBUTTON;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                //Force refresh
  }
#endif // IC_PMWIN
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:disableAutoSelect                                               |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
| -- Motif always uses autoSelect.                                             |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton::disableAutoSelect()
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  enableAutoSelect(false);
#endif // IC_PMWIN
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:isAutoSelect                                                    |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
| -- Motif always uses autoSelect.                                             |
------------------------------------------------------------------------------*/
bool  IRadioButton::isAutoSelect() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return( (style() & BS_AUTORADIOBUTTON & (unsigned long)~(BS_RADIOBUTTON)) ?
          true : false );
#endif // IC_PMWIN
#ifdef IC_MOTIF
  return true;
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IRadioButton:enableCursorSelect                                              |
|                                                                              |
| Removes or sets the noCursorSelect style.  The noCursorSelect style causes   |
| the radio button not to select itself when given the focus as the result     |
| of an arrow key or tab key.  This style only applies to autoSelect radio     |
| buttons.                                                                     |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton::enableCursorSelect(bool bTurnOn)
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~noCursorSelect.asUnsignedLong();
  }
  else
  {
    ulStyle |= noCursorSelect.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:disableCursorSelect                                             |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton::disableCursorSelect()
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PMWIN
  enableCursorSelect(false);
#endif // IC_PMWIN
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:isCursorSelect                                                  |
| -- UNX_NOP_NOT_APPLICABLE                                                    |
------------------------------------------------------------------------------*/
bool  IRadioButton::isCursorSelect() const
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PMWIN
#ifdef IC_PM
  unsigned long ulStyle = style();

  if ((ulStyle & BS_AUTORADIOBUTTON & (unsigned long)~(BS_RADIOBUTTON)) &&
      (ulStyle & noCursorSelect.asUnsignedLong()))
    return false;
  else
#endif
    return true;
#endif // IC_PMWIN
#ifdef IC_MOTIF
  return false;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IRadioButton:selectedIndex                                                   |
|                                                                              |
| Returns the zero-based index of a selected radio button in a group.          |
| NOTE:  noSelections (-1) is returned if nothing is selected.                 |
------------------------------------------------------------------------------*/
unsigned long  IRadioButton::selectedIndex() const
{
#ifdef IC_PMWIN
  unsigned long index = noSelections;
#ifdef IC_PM
  IEventResult evt = handle().sendEvent(BM_QUERYCHECKINDEX,
                             IEventParameter1(0),
                             IEventParameter2(0));
  if (evt.asLong() != -1)
  {                // A button is selected, so return its index.
     index = evt.asUnsignedLong();
  }
#endif
#ifdef IC_WIN
   // Find the beginning of the group
   IWindowHandle hwnd = this->handle();
   while ( (hwnd) && ((ISTYLEOF(hwnd) & WS_GROUP) == 0) )
      {
      hwnd = GetWindow( hwnd, GW_HWNDPREV );
      }
   // find the checked button
   unsigned long count = 0;
   while (hwnd && (index == noSelections))
   {
      if ( ( hwnd.sendEvent(WM_QUERYDLGCODE, 0, 0) & DLGC_RADIOBUTTON) &&
           ( hwnd.sendEvent( BM_GETCHECK, 0 ,0 ).asUnsignedLong() == BST_CHECKED) )
      {
         index = count;
      }
      else
      {
         hwnd = GetWindow( hwnd, GW_HWNDNEXT );
         count++;
      }
   }
#endif

  return index;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return (unsigned long) IRadioButtonData::groupSelection( this->handle(),
                                                           false );
#endif // IC_MOTIF
}


/*------------------------------------------------------------------------------
| IRadioButton:calcMinimumSize                                                 |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The minimum size   |
| is based on the maximum height and width of the font of the radio button     |
| text (plus the size of the radio button itself).                             |
------------------------------------------------------------------------------*/
ISize  IRadioButton::calcMinimumSize() const
{
   unsigned long i        ( textLength() );
   ISize         size     ( i ? displaySize(text()) : ISize());
   unsigned long ulWidth  ( size.width() );
   unsigned long ulHeight ( size.height() );

#ifdef IC_PMWIN
   ISize bmpSize = IRadioButtonData::gBitmapSize;
   if ( bmpSize == ISize() )
   {                    // Find size of radio button bitmap.
   #ifdef IC_PM
      BITMAPINFOHEADER pbmpData;
      pbmpData.cbFix=sizeof(BITMAPINFOHEADER);
      HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
      IQUERYBITMAPPARAMETERS(hbit, &pbmpData);
      IDELETEBITMAP(hbit);
      IRadioButtonData::gBitmapSize.setWidth( pbmpData.cx );
      IRadioButtonData::gBitmapSize.setHeight( pbmpData.cy );
   #endif
   #ifdef IC_WIN
      BITMAP           pbmpData;
      HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
      IQUERYBITMAPPARAMETERS(hbit, &pbmpData);
      IDELETEBITMAP(hbit);
      IRadioButtonData::gBitmapSize.setWidth( pbmpData.bmWidth);
      IRadioButtonData::gBitmapSize.setHeight( pbmpData.bmHeight );
   #endif
      bmpSize = IRadioButtonData::gBitmapSize;
   }

   if (ulHeight < (bmpSize.height()/4) )
     ulHeight = bmpSize.height()/4;
   ulHeight += (2 * IQUERYSYSVALUE(SV_CYBORDER));

   if (i!=0)
     ulWidth += ulWidth/i;             //Has text
   ulWidth += ((bmpSize.width()/4) + 2 + (2 * IQUERYSYSVALUE(SV_CXBORDER)));
#endif //IC_PMWIN

#ifdef IC_MOTIF
   Dimension shadowThickness, borderWidth, highlightThickness,
             marginHeight, marginWidth,
             marginLeft, marginRight, marginTop, marginBottom,
             indicatorSize;

   XtVaGetValues(this->handle(),
                 XmNshadowThickness,    &shadowThickness,
                 XmNborderWidth,        &borderWidth,
                 XmNhighlightThickness, &highlightThickness,
                 XmNmarginHeight,       &marginHeight,
                 XmNmarginWidth,        &marginWidth,
                 XmNmarginLeft,         &marginLeft,
                 XmNmarginRight,        &marginRight,
                 XmNmarginTop,          &marginTop,
                 XmNmarginBottom,       &marginBottom,
                 XmNindicatorSize,      &indicatorSize,
                 0);
  ITRACE_ALL(IString("RadioButton:") +
             IString(" ulWidth=") +               IString(ulWidth) +
             IString(" ulHeight=") +              IString(ulHeight) +
             IString(" XmNshadowThickness=") +    IString(shadowThickness) +
             IString(" XmNborderWidth=") +        IString(borderWidth) +
             IString(" XmNhighlightThickness=") + IString(highlightThickness) +
             IString(" XmNmarginHeight=") +       IString(marginHeight) +
             IString(" XmNmarginWidth=") +        IString(marginWidth) +
             IString(" XmNmarginLeft=") +         IString(marginLeft) +
             IString(" XmNmarginRight=") +        IString(marginRight) +
             IString(" XmNmarginTop=") +          IString(marginTop) +
             IString(" XmNmarginBottom=") +       IString(marginBottom) +
             IString(" XmNindicatorSize=") +      IString(indicatorSize) );

   unsigned long edges = 2*(shadowThickness+borderWidth+highlightThickness);
   ulHeight += 2*marginHeight + marginTop + marginBottom;
   if (ulHeight < indicatorSize)
     ulHeight = indicatorSize;
   ulHeight += edges;
   ulWidth += 2*marginWidth;
   if (i==0)
      ulWidth += indicatorSize;             //no text
   else
      ulWidth += marginLeft + marginRight;  //has text
   ulWidth  += edges;
#endif // IC_MOTIF
   return ISize(ulWidth, ulHeight);
}
