// Revision: 45 1.6 source/ui/basectl/isetcvob.inl, canvas, ioc.v400, 000302  
/* NOSHIP */
#ifndef _ISETCVOB_INL_
#define _ISETCVOB_INL_
/*******************************************************************************
* FILE NAME: isetcvob.inl                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of the inline functions for the          *
*   classes declared in isetcvob.hpp.                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ipoint.hpp>
#include <iwindow.hpp>

/*----------------------------- ISetCanvasObject -----------------------------*/
inline ISetCanvasObject::ISetCanvasObject ( IWindow* pwin )
  : rectCl( 0,0,0,0 ), pwinCl( pwin ),
    bClFirstInDeck( false ), ulClDeckWidth ( 0 )
  {
  }
inline const IWindow  *ISetCanvasObject::window ( ) const
  {
  return pwinCl;
  }
inline IPoint  ISetCanvasObject::topLeft ( ) const
  {
  return rectCl.bottomLeft();
  }
inline ISize  ISetCanvasObject::size ( ) const
  {
  return rectCl.size();
  }
inline IRectangle  ISetCanvasObject::rect ( ) const
  {
  return rectCl;
  }
inline bool  ISetCanvasObject::isFirstInDeck ( ) const
  {
  return bClFirstInDeck;
  }
inline unsigned long  ISetCanvasObject::deckWidth ( ) const
  {
  return ulClDeckWidth;
  }
inline ISetCanvasObject  &ISetCanvasObject::moveTo ( const IPoint& pt )
  {
  rectCl.moveTo( pt );
  return *this;
  }
inline ISetCanvasObject  &ISetCanvasObject::moveTopTo ( IPair::Coord y )
  {
  rectCl.moveBy( IPoint( 0, y - rectCl.bottom() ) );
  return *this;
  }
inline ISetCanvasObject  &ISetCanvasObject::moveLeftTo ( IPair::Coord x )
  {
  rectCl.moveBy( IPoint( x - rectCl.left(), 0 ) );
  return *this;
  }
inline ISetCanvasObject  &ISetCanvasObject::sizeTo ( const ISize& siz )
  {
  rectCl.sizeTo( siz );
  return *this;
  }
inline ISetCanvasObject  &ISetCanvasObject::sizeWidthTo ( IPair::Coord width )
  {
  return sizeTo( ISize( width, 0 ) );
  }
inline ISetCanvasObject  &ISetCanvasObject::sizeHeightTo ( IPair::Coord height )
  {
  return sizeTo( ISize( 0, height ) );
  }
inline ISetCanvasObject  &ISetCanvasObject::setFirstInDeck ( bool first )
  {
  bClFirstInDeck = first;
  return *this;
  }
inline ISetCanvasObject  &ISetCanvasObject::setDeckWidth ( unsigned long width )
  {
  ulClDeckWidth = width;
  return *this;
  }
#endif // _ISETCVOB_INL_
