// Revision: 53 1.11.1.7 source/ui/basectl/ispbdata.hpp, listctls, ioc.v400, 000302 
/* NOSHIP */
#ifndef _ISPBDATA_
  #define _ISPBDATA_
/*******************************************************************************
* FILE NAME: ispbdata.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IBaseSpinButtonData - This class encapsulates private data and functions *
*                           used by the IBaseSpinButton class.  An object of   *
*                           this class is created in the IBaseSpinButton       *
*                           constructor.                                       *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifdef IC_MOTIF
//#include <Xm/Xm.h>
#endif

#include <ispinbas.hpp>
#include <ilanglvl.hpp>

#ifdef IC_PMWIN
  #include <imphdr.hpp>
  #include <ispinszh.hpp>
#ifdef IC_WIN
  #include <ibrushdr.hpp>
#endif
#endif
#ifdef IC_MOTIF
#pragma info(none)
  #include <ikeyset2.h>
#pragma info(restore)

#include <ispinbdh.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IEntryFieldData;

#pragma enum(4)
#pragma pack(push,4)

/*------------------------------------------------------------------------------
| IServantElement class                                                        |
------------------------------------------------------------------------------*/
class IServantElement {
public:
  IServantElement ( IWindowHandle servantHandle,
                    bool          isNumeric )
    : servantKey( servantHandle ),
      isNumeric( isNumeric )
    { }
 ~IServantElement ( )
    { }
bool
  isNumericOnly   ( )
    { return isNumeric; }

private:
IWindowHandle
  servantKey;
bool
  isNumeric;

friend class IServantSet;
friend IWindowHandle const& key (IServantElement* const&);
}; // IServantElement

// Key access
#ifdef IC_SUN
// For the SUN platform, this function is defined as inline.
inline IWindowHandle const& key ( IServantElement* const& pServantElement )
#endif
#ifdef IC_MOTIF
IWindowHandle const& key ( IServantElement* const& pServantElement )
#endif
{
  return pServantElement->servantKey;
}

/*------------------------------------------------------------------------------
| IServantSet class                                                            |
------------------------------------------------------------------------------*/
class IServantSet : public IKeySet < IServantElement*, IWindowHandle > {
public:
  IServantSet ( )
    : IKeySet <IServantElement*, IWindowHandle> ( 5 )
    { }
 ~IServantSet ( )
    { }
}; // IServantSet

/*------------------------------------------------------------------------------
| The spin button private data class.  (Motif version)                         |
------------------------------------------------------------------------------*/
class IBaseSpinButtonData  {
public:
  IBaseSpinButtonData ( );
 ~IBaseSpinButtonData ( );

enum SpinType
  { notSpinning=0,
    mouse=1,
    mouseUpSuspend,
    mouseDownSuspend,
    keyUp,
    keyDown };

Widget
  upArrow,
  downArrow;
XtIntervalId
  timer,             // Timer to control spinning.
  accelerationTimer; // Timer to control spin interval.
bool
  isFastSpin,        // Should spin speed double every 2 seconds?
  isMaster,          // Is this a master spin?
  globalAutoRepeat,  // True if keyboard had auto repeat set on prior to
                     // arrow key spinning.
  isNumericOnly;     // True if numeric only allowed.  Need to set this in
                     //    initialize to be used in callback and timeout
                     //    procs.  These procedures are not member functions
                     //    so they cannot take advantage of polymorphism.
                     //    Checking this field will provide polymorhpic
                     //    type capabilities.
unsigned long
  limit;             // Number of characters in spin field.
Dimension
  minMarginHeight,   // Used by calcMinimumSize for minimum margin values;
  minMarginWidth;    //    set by a widget query in initialize.
long
  position,          // Current position or numeric value.
  initialDelay,      // Initial interval for spin.
  minDelay,          // Minimum interval for spin.
  currentDelay;      // Current spin interval.
IServantSet
  servantSet;        // Set of servant spinbuttons if this is a master.
IEntryFieldData
  *pefpd;            // Pointer to entry field private data.
IWindowHandle
  efHandle;          // Window handle for the entryfield widget
IBaseSpinDefaultHandler
  fdefaultHandler;

void
  registerCallbacks   ( IBaseSpinButton* spinButton );
void
  unregisterCallbacks ( IBaseSpinButton* spinButton );

friend void _System
  ientryfMotifCallback    ( Widget               w,
                            void*                client_data,
                            void*                call_data );
friend void _System
  ibsbArmCallback         ( Widget               w,
                            void*                client_data,
                            void*                call_data );
friend void _System
  ibsbDisarmCallback      ( Widget               w,
                            XtPointer            client_data,
                            XtPointer            call_data);
friend void _System
  ibsbDestroyCallback     ( Widget               w,
                            void*                client_data,
                            void*                call_data );
friend void _System
  ibsbSpinUp              ( IBaseSpinButtonData* pData,
                            XtIntervalId*        id );
friend void _System
  ibsbSpinDown            ( IBaseSpinButtonData* pData,
                            XtIntervalId*        id );
friend void _System
  ibsbFastSpin            ( IBaseSpinButtonData* pData,
                            XtIntervalId*        id );
friend void
  ibsbStopSpin            ( IBaseSpinButton*     spinButton,
                            IBaseSpinButtonData* pData );
friend class IBaseSpinDefaultHandler;

private:
#if IC_EXTENDED_RTTI_ENABLED
  IBaseSpinButtonData ( const IBaseSpinButtonData& )
  { }
#else
  IBaseSpinButtonData ( const IBaseSpinButtonData& );
#endif
IBaseSpinButtonData
 &operator=           ( const IBaseSpinButtonData& );
}; // IBaseSpinButtonData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // IC_MOTIF

#ifdef IC_PMWIN

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

/*------------------------------------------------------------------------------
| The spin button private data class.  (Intel version)                         |
------------------------------------------------------------------------------*/
class IBaseSpinButtonData {
public:
  IBaseSpinButtonData ( );
 ~IBaseSpinButtonData ( );
void
  setLimit ( unsigned long ulLimit )
    { textLimit = ulLimit; }
unsigned long
  limit    ( ) const
    { return textLimit; }
unsigned long
  textLimit;

IMousePointerHandler
  fMousePointerHandler;
IBaseSpinDefaultHandler
  fdefaultHandler;
#ifdef IC_WIN
IBrushHandler
  fBrushHandler;
#endif
}; // IBaseSpinButtonData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // IC_PMWIN

#endif // _ISPBDATA_
