// Revision: 56 1.7.1.2 source/ui/basectl/ispbttn1.cpp, listctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ispbttn1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispbttnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINSTDSPIN

extern "C" {
  #include <iwindefs.h>
}

#include <ispintxt.hpp>
#include <ispbttnh.hpp>
#include <inotifev.hpp>
#include <ictlevt.hpp>
#include <icconst.h>

#ifndef IC_MOTIF
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
  #include <iclspbw.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPBTTN1_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects and notification strings.
INotificationId const IBaseSpinButton::textId = "IBaseSpinButton::text";

/*------------------------------------------------------------------------------
| ITextSpinButton::enableNotification                                          |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ITextSpinButton, does not send notifications to observer objects until it    |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new ITextSpinButtonNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::ITextSpinButtonNotifyHandler                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITextSpinButtonNotifyHandler::ITextSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::~ITextSpinButtonNotifyHandler                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITextSpinButtonNotifyHandler::~ITextSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::dispatchHandlerEvent                           |
|                                                                              |
| This notifies ITextSpinButton Windows of the following events:               |
|                                                                              |
| text(Id)                                                                     |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
bool ITextSpinButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
#ifdef IC_WIN
        if ((className == WC_SPINBUTTON) || (className == WC_NATIVESPINBUTTON))
#else
        if (className == WC_SPINBUTTON)
#endif
        {
          if (ulMsg == SPBN_CHANGE)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseSpinButton::textId,
                                        *thisWindow));
          }
#ifdef IC_WIN
          else if (ulMsg == SPBN_SETFOCUS || ulMsg == SPBN_KILLFOCUS)
          {
            thisWindow->notifyObservers( INotificationEvent(
              IBaseSpinButton::focusId, *thisWindow, false,
              (ulMsg == SPBN_SETFOCUS) ? (void*)true : (void*)false ));
          }
#endif
        }
      }
    break;
    } /* end WM_CONTROL */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

#ifdef IC_MOTIF
bool ITextSpinButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
   // This is a stub for Motif needed for vft.
   return false;
}
#endif // IC_MOTIF
