// Revision: 58 1.18.1.22 source/ui/basectl/ispinbas.cpp, listctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ispinbas.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinbas.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define INCL_WINSTDSPIN
  #define INCL_WINSYS
  #include <iwindefs.h>
#ifdef IC_MOTIF
  #include <X11/Xlib.h>
  #include <X11/IntrinsicP.h>     // for CoreP.h
  #include <X11/CoreP.h>          // for core.being_destroyed
  #include <Xm/ArrowB.h>
  #include <Xm/Form.h>
  #include <Xm/Text.h>
#endif // IC_MOTIF
}

#ifdef IC_WIN
  #include <iclspbw.h>
  #include <commctrl.h>
  #include <icctlsta.hpp>         // Statics to load dll.
#endif
#include <ispinbas.hpp>
#include <ibidiset.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <imphdr.hpp>
#include <inotifev.hpp>
#include <irect.hpp>
#include <ispbdata.hpp>
#include <itrace.hpp>
#ifdef IC_PMWIN
  #include <ispinszh.hpp>
#endif
#ifdef IC_WIN
  #include <iplatfrm.hpp>
  #include <iwcname.hpp>
  #include <ibrushdr.hpp>
#endif
#ifdef IC_MOTIF
  #include <ispinbdh.hpp>
  #include <iefpriv.hpp>
  #include <ispinnum.hpp>
  #include <ispintxt.hpp>
#endif // IC_MOTIF

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISPINBAS_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public spin button styles.                                                   |
------------------------------------------------------------------------------*/
const IBaseSpinButton::Style
  IBaseSpinButton::master            ( 0, ISPBS_MASTER ),
  IBaseSpinButton::servant           ( 0, ISPBS_SERVANT ),
  IBaseSpinButton::readOnly          ( 0, ISPBS_READONLY),
  IBaseSpinButton::leftAlign         ( 0, ISPBS_JUSTLEFT),
  IBaseSpinButton::centerAlign       ( 0, ISPBS_CENTERALIGN ),
  IBaseSpinButton::rightAlign        ( 0, ISPBS_JUSTRIGHT),
  IBaseSpinButton::noBorder          ( 0, ISPBS_NOBORDER),
  IBaseSpinButton::pmCompatible      ( 0, ISPBS_PMCOMPATIBILITY),
  IBaseSpinButton::fastSpin          ( 0, ISPBS_FASTSPIN),
#ifdef IC_PMWIN
  IBaseSpinButton::border3D          ( 0, IWS_BORDER3D );
#endif
#ifdef IC_MOTIF
  IBaseSpinButton::border3D          ( 0 );
#endif

#ifdef IC_MOTIF
// Initialize static objects and notification strings.
INotificationId const IBaseSpinButton::textId      = "IBaseSpinButton::text";
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| The static object that IBaseSpinButton uses to properly change its size.     |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
static ISpinButtonStatics SpinButtonStatics;
#endif

/*------------------------------------------------------------------------------
| IBaseSpinButtonData::IBaseSpinButtonData                                     |
------------------------------------------------------------------------------*/
IBaseSpinButtonData::IBaseSpinButtonData()
    : fdefaultHandler()
#ifdef IC_MOTIF
    , upArrow (0)
    , downArrow (0)
    , isFastSpin (false)
    , isMaster (true)
    , isNumericOnly (false)
    , limit (255)
    , position (0)
    , timer (0)
    , accelerationTimer (0)
    , initialDelay (500)
    , minDelay (50)
    , currentDelay (0)
    , globalAutoRepeat (false)
    , servantSet ()
    , minMarginWidth (0)
    , minMarginHeight (0)
    , efHandle(0)
#endif
#ifdef IC_PMWIN
    , fMousePointerHandler( )
#endif
#ifdef IC_WIN
    , fBrushHandler()
#endif
  { }

/*------------------------------------------------------------------------------
| IBaseSpinButtonData::~IBaseSpinButtonData                                    |
------------------------------------------------------------------------------*/
IBaseSpinButtonData::~IBaseSpinButtonData()
{ }

#ifdef IC_MOTIF

void  iBaseSpinButtonMotifCallback( Widget handle,
                                   XtPointer clientdata,
                                   XtPointer calldata)
{
    ientryfMotifCallback( handle, clientdata, calldata );
    
    // if value has changed, notify observers
    IEntryFieldData* pefd = (IEntryFieldData*)clientdata;
    IBaseSpinButton* button = dynamic_cast<IBaseSpinButton*>( pefd->dispatchingWindow );
    if( button )
    {
        button->notifyObservers(INotificationEvent(
                    IBaseSpinButton::valueId, *button));
    }
}

/*------------------------------------------------------------------------------
| IBaseSpinButtonData::registerCallbacks                                       |
------------------------------------------------------------------------------*/
void IBaseSpinButtonData :: registerCallbacks (IBaseSpinButton* spinButton)
{
  // Add the callback to enable IEditVerifyHandler.
  XtAddCallback (efHandle,
                 XmNmodifyVerifyCallback,
                 ientryfMotifCallback,
                 (XtPointer) pefpd);

  // Add the callback to enable IEditHandler.
  XtAddCallback (efHandle,
                 XmNvalueChangedCallback,
                 iBaseSpinButtonMotifCallback,
                 (XtPointer) pefpd);

  if (isMaster)
  {
     // Arrow button callbacks.
     XtAddCallback (upArrow,
                    XmNarmCallback,
                    ibsbArmCallback,
                    (XtPointer) this);
     XtAddCallback (downArrow,
                    XmNarmCallback,
                    ibsbArmCallback,
                    (XtPointer) this);
     XtAddCallback (upArrow,
                    XmNdisarmCallback,
                    ibsbDisarmCallback,
                    (XtPointer) this);
     XtAddCallback (downArrow,
                    XmNdisarmCallback,
                    ibsbDisarmCallback,
                    (XtPointer) this);
  }


  // Explicitly add X event handler for spinbutton focus events
  XtAddEventHandler(
     (Widget)efHandle,               // widget
     FocusChangeMask,                // ask for FocusIn and FocusOut events
     True,                           // indicate we also want nonmaskable events
     iwindowXEventCallback,          // event handler routine
     (XtPointer) spinButton);        // save reference to this IWindow in client data

}

/*------------------------------------------------------------------------------
| IBaseSpinButtonData::unregisterCallbacks                                       |
------------------------------------------------------------------------------*/
void IBaseSpinButtonData :: unregisterCallbacks (IBaseSpinButton* spinButton)
{
  if (spinButton->isValid())
  {
    Widget wHandle = (Widget)spinButton->handle();
    // Only unregister callbacks if widget not to be destroyed
    if (!wHandle->core.being_destroyed)
    {
      XtRemoveCallback (efHandle,
                      XmNmodifyVerifyCallback,
                      ientryfMotifCallback,
                      (XtPointer) pefpd);

      XtRemoveCallback (efHandle,
                        XmNvalueChangedCallback,
                        ientryfMotifCallback,
                        (XtPointer) pefpd);

      if (isMaster)
      {
         // Arrow button callbacks.
         XtRemoveCallback (upArrow,
                           XmNarmCallback,
                           ibsbArmCallback,
                           (XtPointer) this);
         XtRemoveCallback (downArrow,
                           XmNarmCallback,
                           ibsbArmCallback,
                           (XtPointer) this);
         XtRemoveCallback (upArrow,
                           XmNdisarmCallback,
                           ibsbDisarmCallback,
                           (XtPointer) this);
         XtRemoveCallback (downArrow,
                           XmNdisarmCallback,
                           ibsbDisarmCallback,
                          (XtPointer) this);
      }

      XtRemoveEventHandler(
         (Widget)efHandle,               // widget
         FocusChangeMask,                // ask for FocusIn and FocusOut events
         True,                           // indicate we also want nonmaskable events
         iwindowXEventCallback,          // event handler routine
         (XtPointer) spinButton);        // save reference to this IWindow in client data
    }
  }
}

#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| IBaseSpinButton::IBaseSpinButton                                             |
|                                                                              |
| Constructor to create a spin button control.                                 |
------------------------------------------------------------------------------*/
IBaseSpinButton :: IBaseSpinButton ( )
{
   fBaseSpinButtonData = new IBaseSpinButtonData();
#ifdef IC_PMWIN
   fBaseSpinButtonData->setLimit(255);
#endif // IC_PMWIN
#ifdef IC_MOTIF
   fBaseSpinButtonData->pefpd = new IEntryFieldData( this );
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::~IBaseSpinButton                                            |
------------------------------------------------------------------------------*/
IBaseSpinButton::~IBaseSpinButton ( )
{
#ifdef IC_PMWIN
   fBaseSpinButtonData->fdefaultHandler.stopHandlingEventsFor(this);
   fBaseSpinButtonData->fMousePointerHandler.stopHandlingEventsFor(this);
   delete fBaseSpinButtonData;
#ifdef IC_WIN
   fBaseSpinButtonData->fBrushHandler.stopHandlingEventsFor(this);
#endif
#endif // IC_PMWIN
#ifdef IC_MOTIF
  if (fBaseSpinButtonData->isMaster)
  {
     // If this spin button is still spinning, remove the timer.  Only remove
     // timers for a master.  Master's private data holds all timers.  We
     // can't remove the timers from a servant's master since the master may
     // have been destructed first.
     if (fBaseSpinButtonData->timer)
        XtRemoveTimeOut (fBaseSpinButtonData->timer);
     if (fBaseSpinButtonData->accelerationTimer)
        XtRemoveTimeOut (fBaseSpinButtonData->accelerationTimer);

     // If this is a master, delete servant elements.
     IServantSet::Cursor cursor(fBaseSpinButtonData->servantSet);
     IServantElement* servant;
     for (cursor.setToFirst(); cursor.isValid(); )
     {
        servant = fBaseSpinButtonData->servantSet.elementAt (cursor);
        fBaseSpinButtonData->servantSet.removeAt (cursor);
        XtRemoveCallback (key (servant),
                          XmNdestroyCallback,
                          ibsbDestroyCallback,
                          fBaseSpinButtonData);
        delete servant;
        cursor.setToFirst();
     }
  }

  fBaseSpinButtonData->unregisterCallbacks(this);

  fBaseSpinButtonData->fdefaultHandler.IKeyboardHandler::stopHandlingEventsFor (this);
  fBaseSpinButtonData->fdefaultHandler.IResizeHandler::stopHandlingEventsFor (this);

  delete fBaseSpinButtonData->pefpd;
  delete fBaseSpinButtonData;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::initialize                                                  |
------------------------------------------------------------------------------*/
void IBaseSpinButton :: initialize ( unsigned long        windowId,
                                     const IWindowHandle& parent,
                                     const IWindowHandle& owner,
                                     unsigned long        ulStyle,
                                     const IRectangle&    initial )
{
  IASSERTPARM(parent!=0);
#ifdef IC_PMWIN
  IWindowHandle spb(0);
#ifdef IC_WIN
  if ( this->isPMCompatible() )
  {
    // If needed, load the DLL containing the control code.
    IControlStatics::loadControlDLL();
#endif
    spb =
        this -> create( windowId,
                        0,
                        ulStyle,
                        WC_SPINBUTTON,
                        parent,
                        owner,
                        initial,
                        0,
                        0,
                        defaultOrdering(),
                        convertToGUIStyle(        // convert extended styles
                          IBaseSpinButton::Style(0, extendedStyle() ),
                          true)     );
#ifdef IC_WIN
  } // if pmCompatible
  else
  {
    // Native Up/Down control

    // Create the dummy outer control of class WC_NATIVESPINBUTTON

    HINSTANCE  hinst = GetModuleHandle(0);
    WNDCLASS   wndClass;
    // Check for it already registered.  Need this in some static
    // linking scenarios.
    bool    registered = GetClassInfo( hinst, WC_NATIVESPINBUTTON, &wndClass);

    if (!registered)
    {
      InitCommonControls();     // make sure common controls loaded

      wndClass.style         = CS_HREDRAW | CS_VREDRAW;
      wndClass.cbClsExtra    = 0;
      wndClass.cbWndExtra    = NATIVESPINBUTTONEXTRA;
      wndClass.hCursor       = LoadCursor(0,IDC_ARROW);
      wndClass.hInstance     = GetModuleHandle(0);
      wndClass.lpfnWndProc   = (WNDPROC)IDEFWINDOWPROC;
      wndClass.hIcon         = 0;
      wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
      wndClass.lpszMenuName  = 0;
      wndClass.lpszClassName = WC_NATIVESPINBUTTON;

      if (!RegisterClass(&wndClass))
        ITHROWGUIERROR( "RegisterClass" );
    }
    spb =
        this -> create( windowId,
                        0,
                        ulStyle,
                        WC_NATIVESPINBUTTON,
                        parent,
                        owner,
                        IRectangle(),
                        0,
                        0,
                        defaultOrdering(),
                        convertToGUIStyle(        // convert extended styles
                          IBaseSpinButton::Style(0, extendedStyle() ),
                          true)     );

    // If this is a right-to-left spin button, make its child entry field
    // right-to-left as well.  Do this by adding in some extended styles
    // for alignment.
    unsigned long exStyleBuddy(0);
    if (IBidiSettings::isBidiSupported())
    {
       IBidiSettings
         bidiSettings( spb );
       if ( bidiSettings.windowLayout() == IBidiSettings::layoutRightToLeft )
       {
          exStyleBuddy |= WS_EX_RIGHT;
       }
       if ( bidiSettings.textOrientation() == IBidiSettings::textRightToLeft )
       {
          exStyleBuddy |= WS_EX_RTLREADING;
       }
    }

    // Create the real buddy control (Edit control ).  It uses the same Id
    // as the dummy wrapper.
    IWindowHandle buddy =
       ICREATEWINDOW(
          spb,                                  // parent
          WC_ENTRYFIELD,                        // class
          (ulStyle & INSPBS_NUMERICONLY) ? "0" : "",  //Window text
          ISpinButtonStatics::convertToBuddyStyle( extendedStyle() ),
          0,                                    // x
          0,                                    // y
          0,
          0,
          spb,                                  // owner
          0,                                    // behind (unused)
          windowId,                             // Id
          0,                                    // control parms
          //BIDI 25652 0,                                    // extended styles
          exStyleBuddy,                         // extended styles
          hinst );                              // instance
    if ( buddy == 0)  // Error: throw exception.
      ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId)+
                     IString(" Class=")+IString(WC_ENTRYFIELD));

    WNDPROC defproc = (WNDPROC)ISUBCLASSWINDOW( buddy, fnwpSpinButtonEditProc );
    ISETBUDDYPROC( spb, defproc );
    ISETACTIVEBUDDY( spb, buddy.asUnsigned() );

    // If this is a master spinbutton, we need to create the updown control,
    // a fake buddy so that it works correctly, and setup fast spin.
    // For a servant, we will be using the updown control of the master
    // so we skip creation here.
    if (ulStyle & INSPBS_MASTER)
    {
       IWindowHandle fakebuddy =
          ICREATEWINDOW(
             spb,                               // parent
             WC_ENTRYFIELD,                     // class
             (ulStyle & INSPBS_NUMERICONLY) ? "0" : "",  //Window text
             WS_CHILD,                          // style
             0,                                 // x
             0,                                 // y
             0,                                 // width
             0,                                 // height
             spb,                               // owner
             0,                                 // behind (unused)
             ((windowId-2) & 0x0000ffff),       // id
             0,                                 // control parms
             0,                                 // extended styles
             hinst );                           // instance

       if ( fakebuddy == 0)  // Error: throw exception.
         ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId-1)+
                        IString(" Class=")+IString(WC_ENTRYFIELD));

       // Create the up/down control.  Its id is one less than the
       // dummy wrapper.
       unsigned long udStyle =
          WS_CHILD | WS_BORDER | WS_VISIBLE |
          UDS_AUTOBUDDY | UDS_SETBUDDYINT | UDS_NOTHOUSANDS |
          UDS_WRAP | UDS_ARROWKEYS;
       IWindowHandle updown =
          ICREATEWINDOW(
             spb,                               // parent
             UPDOWN_CLASS,                      // class
             0,                                 // Window text
             udStyle,                           // style
             0,
             0,                                 // y
             0,
             0,
             spb,                               // owner
             0,                                 // behind (unused)
             ((windowId-1) & 0x0000ffff),       // id
             0,                                 // control parms
             0,                                 // extended styles
             hinst );                           // instance

       if ( updown == 0)  // Error: throw exception.
         ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId-1)+
                        IString(" Class=")+IString(UPDOWN_CLASS));

       // Set the handle of the updown control into the extra window word
       // of the wrapper control.  The active buddy is initially this one.
       ISETUPDOWNHANDLE( spb, updown.asUnsigned() );

       // Set the initial range to be 0,0
       updown.sendEvent( UDM_SETRANGE,
                         IEventParameter1(0),
                         IEventParameter2(0, 0) );

       // Set up table needed for fast spin
       if (extendedStyle() & ISPBS_FASTSPIN)
       {
          updown.sendEvent( UDM_SETACCEL,
                   IEventParameter1(ISpinButtonStatics::fastSpinTableSize()),
                   IEventParameter2(ISpinButtonStatics::fastSpinTable()) );
       }
       else
       {
          updown.sendEvent( UDM_SETACCEL,
                   IEventParameter1(1),
                   IEventParameter2(ISpinButtonStatics::normalSpinTable()));
       }
    }   // master
    else
    {
       // servant.  Insure updown handle value is 0
       ISETUPDOWNHANDLE( spb, 0 );
    }
    // Initialize the rest of the window words
   ISETRANGE(spb, 0);
   ISETLASTPOS(spb, 0);

  }
#endif
  initialize();
  startHandlingEventsFor(spb);

#ifdef IC_WIN
  if ( (!this->isPMCompatible() ) &&
       ( initial.size() != ISize(0,0) ) )
  {
    // If the initial size specified is not 0, resize now.  This is done
    // to let our handler position the controls
    this->moveSizeTo( initial );
  }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  int        n = 0;
  Arg        args[10];
  Widget     entryField, arrowForm;

  fBaseSpinButtonData->isFastSpin = (extendedStyle() & ISPBS_FASTSPIN) ? true : false;
  fBaseSpinButtonData->isNumericOnly =
    (ulStyle & SPBS_NUMERICONLY) ? true : false;

  // Create top level XmForm widget, and set it up to be an IWindow.
  IWindowHandle sbForm =
     Inherited::create(
        windowId,
        NULL,
        ulStyle,
        (IXmCreateFunction)XmCreateForm,
        parent,
        owner,
        initial,
        0,
        0);

  // Check if the spin button is left-to-right (the entry field is left
  // of the spin arrows) or right-to-left (the entry field is right of
  // the spin arrows).
  bool
    leftToRight = true;
  if ( IBidiSettings::isBidiSupported() )
  {
     IBidiSettings
       bidiSettings( sbForm );
     if ( bidiSettings.windowLayout() == IBidiSettings::layoutRightToLeft )
     {
        leftToRight = false;
     }
  }

  n = 0;
  unsigned long extStyle = extendedStyle();

  // Set editable resource if it is read-only.
  if (extStyle & IBaseSpinButton::readOnly.asExtendedUnsignedLong())
  {
     XtSetArg (args[n], XmNeditable, false); n++;
  }

  XtSetArg (args[n], XmNeditMode,         XmSINGLE_LINE_EDIT); n++;
  XtSetArg (args[n], leftToRight ?
                     XmNleftAttachment : XmNrightAttachment,
                                          XmATTACH_FORM); n++;
  XtSetArg (args[n], XmNtopAttachment,    XmATTACH_FORM); n++;
  XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
  XtSetArg (args[n], XmNuserData,
                        (XtPointer)IBaseSpinButtonData::notSpinning); n++;

  if (extStyle & IBaseSpinButton::noBorder.asExtendedUnsignedLong())
  {
     XtSetArg (args[n], XmNborderWidth,   0); n++;
  }
  else
  {
     XtSetArg (args[n], XmNborderWidth,   2); n++;
  }
  // Create the entry field as an XmText widget.  Attach it to the form
  // at the top, bottom, and left.
  entryField = XtCreateManagedWidget (IString (windowId),
                                      xmTextWidgetClass,
                                      (Widget)sbForm,
                                      args, n);
  if ((unsigned long) entryField == 0)
     ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                          IBaseErrorInfo::accessError,
                          IException::recoverable,
                          "spin button entry field");

  XtVaSetValues ((Widget)sbForm,
                 XmNinitialFocus, entryField,
                 NULL);

  fBaseSpinButtonData->isMaster = (ulStyle & IC_SPBS_SERVANT) ? false : true;

  if (fBaseSpinButtonData->isMaster)
  {
     arrowForm = XtVaCreateWidget (IString (windowId),
                                   xmFormWidgetClass,
                                   (Widget)sbForm,
                                   XmNleftAttachment,   leftToRight ?
                                                        XmATTACH_NONE : XmATTACH_FORM,
                                   XmNrightAttachment,  leftToRight ?
                                                        XmATTACH_FORM : XmATTACH_NONE,
                                   XmNtopAttachment,    XmATTACH_FORM,
                                   XmNbottomAttachment, XmATTACH_FORM,
                                   XmNfractionBase,     2,
                                   NULL);
     if ((unsigned long) arrowForm == 0)
        ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                             IBaseErrorInfo::accessError,
                             IException::recoverable,
                             "spin button arrow form");

     if ( leftToRight )
     {  // The spin arrows are to the right of the entry field.
        XtVaSetValues( entryField,
                       XmNrightAttachment, XmATTACH_WIDGET,
                       XmNrightWidget,     arrowForm,
                       NULL );
     }
     else
     {  // The spin arrows are to the left of the entry field.
        XtVaSetValues( entryField,
                       XmNleftAttachment, XmATTACH_WIDGET,
                       XmNleftWidget,     arrowForm,
                       NULL );
     }

     fBaseSpinButtonData->upArrow = XtVaCreateManagedWidget (
                                   IString (windowId),
                                   xmArrowButtonWidgetClass,
                                   arrowForm,
                                   XmNarrowDirection,     XmARROW_UP,
                                   XmNtraversalOn,        False,
                                   XmNhighlightThickness, 0,
                                   XmNleftAttachment,     XmATTACH_FORM,
                                   XmNrightAttachment,    XmATTACH_FORM,
                                   XmNtopAttachment,      XmATTACH_POSITION,
                                   XmNtopPosition,        0,
                                   XmNbottomAttachment,   XmATTACH_POSITION,
                                   XmNbottomPosition,     1,
                                   NULL);
     if ((unsigned long) fBaseSpinButtonData->upArrow == 0)
        ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                             IBaseErrorInfo::accessError,
                             IException::recoverable,
                             "spin button up arrow button");

     fBaseSpinButtonData->downArrow = XtVaCreateManagedWidget (
                                   IString (windowId),
                                   xmArrowButtonWidgetClass,
                                   arrowForm,
                                   XmNarrowDirection,     XmARROW_DOWN,
                                   XmNtraversalOn,        False,
                                   XmNhighlightThickness, 0,
                                   XmNleftAttachment,     XmATTACH_FORM,
                                   XmNrightAttachment,    XmATTACH_FORM,
                                   XmNtopAttachment,      XmATTACH_POSITION,
                                   XmNtopPosition,        1,
                                   XmNbottomAttachment,   XmATTACH_POSITION,
                                   XmNbottomPosition,     2,
                                   NULL);
     if ((unsigned long) fBaseSpinButtonData->downArrow == 0)
        ITHROWLIBRARYERROR1 (IC_XMCREATE_FAILURE,
                             IBaseErrorInfo::accessError,
                             IException::recoverable,
                             "spin button down arrow button");

     XtManageChild (arrowForm);
  }
  else
  {
     XtVaSetValues (entryField,
                    XmNrightAttachment,  XmATTACH_FORM,
                    NULL);
  }


  // Query the XmText margin height and width.  These values will
  // be used by calcMinimumSize.
  XtVaGetValues (entryField,
                 XmNmarginHeight, &fBaseSpinButtonData->minMarginHeight,
                 XmNmarginWidth,  &fBaseSpinButtonData->minMarginWidth,
                 NULL);

  // Stash away edit widget handle
  fBaseSpinButtonData->efHandle = entryField;

  startHandlingEventsFor (sbForm);

  fBaseSpinButtonData->registerCallbacks(this);

  setOwner (windowWithHandle (owner));
  setStyle (ulStyle);

  // Attach default handler.  It is derived from IKeyboardHandler (for
  // up and down arrows) and IResizeHandler (for window resize)
  fBaseSpinButtonData->fdefaultHandler.IKeyboardHandler::handleEventsFor (this);
  fBaseSpinButtonData->fdefaultHandler.IResizeHandler::handleEventsFor (this);

  // Ensure that the windows distorted flag is set.
  setLayoutDistorted (windowCreated | colorChanged | fontChanged, 0);
#endif // IC_MOTIF
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::initialize                                                  |
------------------------------------------------------------------------------*/
void IBaseSpinButton :: initialize ( )
{
   fBaseSpinButtonData->fdefaultHandler.handleEventsFor(this);
   fBaseSpinButtonData->fMousePointerHandler.handleEventsFor(this);
#ifdef IC_WIN
   fBaseSpinButtonData->fBrushHandler.handleEventsFor(this);
#endif
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| IBaseSpinButton::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IBaseSpinButton :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   bool bExtOnly) const
{
  // Obtain the style from the class that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & ISPBS_EXTMASK;
#ifdef IC_WIN
    if (guiStyle.asExtendedUnsignedLong() &
        ( IBaseSpinButton::noBorder.asExtendedUnsignedLong() |
          IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() ))
    {
      // if noBorder and\or pmCompatible is specified then remove border3D
      ulStyle &= ~WS_EX_CLIENTEDGE;
    }
#endif
  }
  else
  {
    unsigned long guiExtUL = guiStyle.asExtendedUnsignedLong();
#ifdef IC_WIN
    if ( guiExtUL & IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() )
#endif
       {
       // PM Compatible spinbutton
       // Don't need to handle servant, SPBS_ALLCHARACTERS since these are 0
       if ( guiExtUL & IBaseSpinButton::master.asExtendedUnsignedLong()      )
          ulStyle |= SPBS_MASTER;
       if ( guiExtUL & IBaseSpinButton::readOnly.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_READONLY;
       if ( guiExtUL & IBaseSpinButton::leftAlign.asExtendedUnsignedLong()   )
          ulStyle |= SPBS_JUSTLEFT;
       if ( guiExtUL & IBaseSpinButton::centerAlign.asExtendedUnsignedLong() )
#ifdef IC_PMWIN
          ulStyle |= SPBS_JUSTCENTER;
#endif
#ifdef IC_MOTIF // sync this up!!!!
          ulStyle |= IC_SPBS_JUSTCENTER;
       if ( guiExtUL & IBaseSpinButton::servant.asExtendedUnsignedLong() )
          ulStyle |= IC_SPBS_SERVANT;
#endif
       if ( guiExtUL & IBaseSpinButton::rightAlign.asExtendedUnsignedLong()  )
          ulStyle |= SPBS_JUSTRIGHT;
       if ( guiExtUL & IBaseSpinButton::noBorder.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_NOBORDER;
       if ( guiExtUL & IBaseSpinButton::fastSpin.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_FASTSPIN;
       }
#ifdef IC_WIN
    else
       {
       ulStyle |= WS_BORDER;
       // For the native Windows up/down control, this is the style
       // of the wrapper.
       if ( guiExtUL & IBaseSpinButton::master.asExtendedUnsignedLong()   )
          ulStyle |= INSPBS_MASTER;
       if ( guiExtUL & IBaseSpinButton::noBorder.asExtendedUnsignedLong() )
          ulStyle &= (unsigned long)~WS_BORDER;
       }

    // For Windows it is always a child window
    ulStyle |= WS_CHILD;
#endif
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IBaseSpinButton::enableDataUpdate                                            |
|                                                                              |
| Set or remove the readOnly style on the spinbutton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::enableDataUpdate( bool writeable )
{
#ifdef IC_PMWIN
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;

  if (writeable)
  {
    ulStyle &= ~readOnly.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle |= readOnly.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
#ifdef IC_WIN
    else
    {
      // Native Up/Down control
      IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
      ef.sendEvent(EM_SETREADONLY,
                   IEventParameter1(writeable ? false : true),
                   IEventParameter2(0));
    }
#endif
    refresh();                      //force refresh
  }

#endif // IC_PMWIN

#ifdef IC_MOTIF
  if (writeable)
  {
     XtVaSetValues (fBaseSpinButtonData->efHandle,
                    XmNeditable, True,
                    NULL);
  }
  else
  {
     XtVaSetValues (fBaseSpinButtonData->efHandle,
                    XmNeditable, False,
                    NULL);
  }
#endif // IC_MOTIF
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::disableDataUpdate                                           |
|                                                                              |
| Remove the readOnly style on the spinbutton.                                 |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::disableDataUpdate ( )
{
   return (enableDataUpdate(false));
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::enableFastSpin                                              |
|                                                                              |
| Set or remove the fastSpin style on the the spinbutton.                      |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::enableFastSpin (bool fast)
{
#ifdef IC_PMWIN
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;
  if (fast)
  {
    ulStyle |= fastSpin.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle &= ~fastSpin.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
#ifdef IC_WIN
    else
    {
      // Native Up/Down control
      IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
      if (updown.asUnsigned())  // in servant case could be empty
      {
        if (fast)
          {
          if ( !updown.sendEvent(
                   UDM_SETACCEL,
                   IEventParameter1(ISpinButtonStatics::fastSpinTableSize()),
                   IEventParameter2(ISpinButtonStatics::fastSpinTable()) )
                .asUnsignedLong()  )
             ITHROWGUIERROR2("UDM_SETACCEL",
                             IBaseErrorInfo::invalidParameter,
                             IException::recoverable);
          }
        else
          {
          if ( !updown.sendEvent(
                   UDM_SETACCEL,
                   IEventParameter1(1),
                   IEventParameter2(ISpinButtonStatics::normalSpinTable()) )
                .asUnsignedLong() )
             ITHROWGUIERROR2("UDM_SETACCEL",
                             IBaseErrorInfo::invalidParameter,
                             IException::recoverable);
          }
      }  // updown set

    }
#endif
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  fBaseSpinButtonData->isFastSpin = fast ? true : false;
#endif // IC_MOTIF
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::disableFastSpin                                             |
|                                                                              |
| Remove the fastSpin style on the the spinbutton.                             |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::disableFastSpin ()
{
   return (enableFastSpin(false));
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setAlignment                                                |
|                                                                              |
| Change the spin field text alignment.                                        |
| --- UNX_NOP                                                                  |
| --- Alignment not supported with XmText widget.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::setAlignment(Alignment alignment)
{
  ITRACE_MOTIF_NOP();
  ITRACE_WIN_NOP();
#ifdef IC_PM  // changing on the fly not supported in Windows
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;
  ulStyle &= ~(leftAlign.asExtendedUnsignedLong() |
               centerAlign.asExtendedUnsignedLong() |
               rightAlign.asExtendedUnsignedLong() );
  switch (alignment)
  {
    case left:
      ulStyle |= leftAlign.asExtendedUnsignedLong();
    break;

    case right:
      ulStyle |= rightAlign.asExtendedUnsignedLong();
    break;

    case center:
      ulStyle |= centerAlign.asExtendedUnsignedLong();
    break;
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
   refresh();                                          //Force refresh
  }
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::addBorder                                                   |
|                                                                              |
| Add or remove the border on the spinbutton.                                  |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::addBorder (bool add)
{
#ifdef IC_PMWIN
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;

  if (add)
  {
    ulStyle &= ~noBorder.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle |= noBorder.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    IBaseSpinButton::Style newStyle( style(), ulStyle);
    setStyle( convertToGUIStyle(newStyle, false) );     //Change style
#ifdef IC_WIN
    if ( !this->isPMCompatible() )
       {
       long extGUI = GetWindowLong( this->handle(), GWL_EXSTYLE);
       if (add)
          extGUI |= WS_EX_CLIENTEDGE;
       else
          extGUI &= ~WS_EX_CLIENTEDGE;
       SetWindowLong( this->handle(), GWL_EXSTYLE, extGUI );

       // Need to force a resizing event to get border change to show

       RECT wrect;
       GetWindowRect( this->handle(), &wrect );
       MapWindowPoints( HWND_DESKTOP,
                        IPARENTOF( this->handle() ),
                        (POINT*)(&wrect),
                        2 );
       SetWindowPos( this->handle(), 0,
                     (int)wrect.left,
                     (int)wrect.top,
                     (int)(wrect.right - wrect.left),
                     (int)(wrect.bottom - wrect.top),
                     SWP_FRAMECHANGED | SWP_NOZORDER | SWP_NOACTIVATE );
       }
#endif
    refresh();                                         // force refresh
  }
#endif // IC_PMWIN

#ifdef IC_MOTIF
  // Handle the border style.  Change the border size if and only if
  // this request reflects a change.
  bool currentBorder = hasBorder();
  if (add ^ currentBorder)
  {
     Dimension height, width;
     XtVaGetValues (fBaseSpinButtonData->efHandle,
                    XmNheight, &height,
                    XmNwidth,  &width,
                    NULL);
     if (add)
     {
        // Force default value needed to enable border.
        XtVaSetValues (fBaseSpinButtonData->efHandle,
                       XmNborderWidth, 2,
                       NULL);
     }
     else
     {
        XtVaSetValues (fBaseSpinButtonData->efHandle,
                       XmNborderWidth, 0,
                       NULL);
     }

     // Need to resize text widget since border is outside its size.
     sizeTo (size());
     setLayoutDistorted (minimumSizeChanged, 0);
  }
#endif // IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::removeBorder                                                |
|                                                                              |
| Remove the spin field border.                                                |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: removeBorder ( )
{
   return addBorder(false);
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isMaster                                                    |
|                                                                              |
| Is the spin button a master?                                                 |
------------------------------------------------------------------------------*/
bool IBaseSpinButton :: isMaster ( ) const
{
#ifdef IC_PMWIN
  return (extendedStyle() & master.asExtendedUnsignedLong()) ? true : false;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return fBaseSpinButtonData->isMaster;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isServant                                                   |
|                                                                              |
| Is the spin button a servant?                                                |
------------------------------------------------------------------------------*/
bool IBaseSpinButton :: isServant  ( ) const
{
   return (!isMaster());
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isWriteable                                                 |
|                                                                              |
| Is the spin field read only?                                                 |
------------------------------------------------------------------------------*/
bool IBaseSpinButton :: isWriteable ( ) const
{
#ifdef IC_PMWIN
   return (extendedStyle() & readOnly.asExtendedUnsignedLong()) ? false : true;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  ::Boolean isEditable;
  XtVaGetValues (fBaseSpinButtonData->efHandle,
                 XmNeditable,        &isEditable,
                 NULL);
  return (isEditable == True);
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isFastSpinEnabled                                           |
|                                                                              |
| Is fast spin activated?                                                      |
------------------------------------------------------------------------------*/
bool IBaseSpinButton :: isFastSpinEnabled ( ) const
{
#ifdef IC_PMWIN
   return (extendedStyle() & fastSpin.asExtendedUnsignedLong()) ? true : false;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return fBaseSpinButtonData->isFastSpin;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::hasBorder                                                   |
|                                                                              |
| Does the spin field have a border?                                           |
------------------------------------------------------------------------------*/
bool IBaseSpinButton :: hasBorder   ( ) const
{
#ifdef IC_PMWIN
   return (extendedStyle() & noBorder.asExtendedUnsignedLong()) ? false : true;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Dimension     borderWidth;
  XtVaGetValues (fBaseSpinButtonData->efHandle,
                 XmNborderWidth, &borderWidth,
                 NULL);
  return (borderWidth != 0);
#endif // IC_MOTIF
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::hasFocus                                                    |
------------------------------------------------------------------------------*/
bool IBaseSpinButton::hasFocus() const
{
  HWND hwndFocus = IQUERYFOCUS(HWND_DESKTOP);
  if (handle() == IPARENTOF( hwndFocus ))
  {
     IWindowClassName wcName( hwndFocus );
     if (wcName == WC_ENTRYFIELD)
        return true;
  }
  return false;
}
#endif

/*------------------------------------------------------------------------------
| IBaseSpinButton::alignment                                                   |
|                                                                              |
| Returns the spin field alignment.                                            |
------------------------------------------------------------------------------*/
IBaseSpinButton::Alignment IBaseSpinButton :: alignment  ( ) const
{
  ITRACE_MOTIF_NOP();

#ifdef IC_PMWIN
  unsigned long ulStyle = extendedStyle();
  if ( ulStyle & leftAlign.asExtendedUnsignedLong())
     return left;
  else if (ulStyle & rightAlign.asExtendedUnsignedLong())
     return right;
  else
     return center;
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return IBaseSpinButton::left;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::limit                                                       |
|                                                                              |
| Returns the number of characters allowed.                                    |
------------------------------------------------------------------------------*/
unsigned long IBaseSpinButton::limit ( ) const
{
#ifdef IC_PMWIN
   return this->fBaseSpinButtonData->limit();
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return fBaseSpinButtonData->limit;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setLimit                                                    |
|                                                                              |
| Sets the number of characters permitted in a spin field.                     |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: setLimit ( unsigned long aNumber )
{
#ifdef IC_PMWIN
   this->fBaseSpinButtonData->setLimit(aNumber);
   if (this->isPMCompatible())
   {

     unsigned long ulOk = handle().sendEvent(SPBM_SETTEXTLIMIT,
//                                           IEventParameter1((USHORT)(aNumber)),
                                             IEventParameter1(aNumber),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SETTEXTLIMIT",
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
     ef.sendEvent( EM_SETTEXTLIMIT,
                   IEventParameter1( aNumber ),
                   IEventParameter2( 0 ));

/* Now handled in canvas handler
     // force an update of the text
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     if (updown.asUnsigned())
     {
        IEventResult pos = IQUERYLASTPOS(ef);
        if ( !pos.highNumber() )
        {
           // Generate a message to be sent.  This simulates the message that
           // is sent when the user clicks on the arrow buttons.
           this->sendEvent( IC_UM_VSCROLL,
                            IEventParameter1( SB_THUMBPOSITION, pos.lowNumber() ),
                            IEventParameter2( updown.asUnsigned() ) );
        }
     }
*/

   }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  fBaseSpinButtonData->limit = aNumber;
  XtVaSetValues (fBaseSpinButtonData->efHandle,
                 XmNmaxLength, aNumber,
                 NULL);
#endif // IC_MOTIF

   setLayoutDistorted(IWindow::minimumSizeChanged, 0);
   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setMaster                                                   |
|                                                                              |
| Assign the master for this servant spin button.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: setMaster ( IBaseSpinButton& master )
{
   IASSERTSTATE(isServant());
   IASSERTPARM(master.isMaster());

#ifdef IC_PMWIN
   if (this->isPMCompatible())
   {
     unsigned long ulOk =
             handle().sendEvent(SPBM_SETMASTER,
                                IEventParameter1(master.handle()),
                                IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SETMASTER",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle updown = IQUERYUPDOWNHANDLE( master.handle() );
     if ( updown.asUnsigned() )
        ISETUPDOWNHANDLE( this->handle(), updown.asUnsigned() );
     else
        ITHROWLIBRARYERROR(IC_INVALIDHANDLE,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
   }
#endif
#endif // IC_PMWIN

#ifdef IC_MOTIF
  IServantElement* servantElement =
    new IServantElement (fBaseSpinButtonData->efHandle, fBaseSpinButtonData->isNumericOnly);
  master.fBaseSpinButtonData->
    servantSet.addOrReplaceElementWithKey (servantElement);
  fBaseSpinButtonData->upArrow = master.fBaseSpinButtonData->upArrow;
  fBaseSpinButtonData->downArrow = master.fBaseSpinButtonData->downArrow;

  // Register a destroy callback to be invoked when this servant is destroyed.
  // The callback will remove this servant from the master's servant set.
  XtAddCallback (fBaseSpinButtonData->efHandle,
                 XmNdestroyCallback,
                 ibsbDestroyCallback,
                 master.fBaseSpinButtonData);
#endif // IC_MOTIF

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::spinDown                                                    |
|                                                                              |
| Spin the button down (backward) the specified amount of times.               |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: spinDown ( unsigned long spinBy )
{
   ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_SPINDOWN,
                                             IEventParameter1(spinBy),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SPINDOWN",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle spinBt = this->handle();
     IWindowHandle updown = IQUERYUPDOWNHANDLE( spinBt );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) !=
           ISpinButtonStatics::buddyHandle( spinBt ) ) )
        updown = 0;

     // Try to gather info from the updown control.  If that fails,
     // use stored information to handle the servant case.
     IEventResult pos, range;
     if (updown.asUnsigned())
     {
        pos = updown.sendEvent( UDM_GETPOS );
        if ( pos.highNumber() )
           ITHROWGUIERROR2("UDM_GETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        range = updown.sendEvent( UDM_GETRANGE );
     }
     else
     {
        pos = IQUERYLASTPOS( spinBt );
        range = IQUERYRANGE( spinBt );
     }
     int newpos = (short)pos.lowNumber() - (int)spinBy;
     // lowNumber is maximum, highNumber is minimum in the event result
     if ((short)range.highNumber() > newpos)
     {
        newpos += (short)range.lowNumber() - (short)range.highNumber() + 1;
        if ((short)range.highNumber() > newpos)
        {
           // Still out of range ... error
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        }
     }

     if (updown.asUnsigned())
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2(newpos));

     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)newpos ),
                      IEventParameter2( updown.asUnsigned() ) );

     // Generate the spin up/down event expected by handlers
     IWindowHandle parent = IPARENTOF( spinBt );
     parent.sendEvent( WM_COMMAND,
                       IEventParameter1( (short)IIDOF( spinBt ),
                                         (short)SPBN_DOWNARROW ),
                       IEventParameter2( spinBt.asUnsigned()) );
   }
#endif
#endif // IC_PMWIN

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::spinUp                                                      |
|                                                                              |
| Spin the button up (forward) the specified amount of times.                  |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: spinUp   ( unsigned long spinBy )
{
   ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_SPINUP,
                                             IEventParameter1(spinBy),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SPINUP",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle spinBt = this->handle();
     IWindowHandle updown = IQUERYUPDOWNHANDLE( spinBt );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) !=
           ISpinButtonStatics::buddyHandle( spinBt ) ) )
        updown = 0;

     // Try to gather info from the updown control.  If that fails,
     // use stored information to handle the servant case.
     IEventResult pos, range;
     if (updown.asUnsigned())
     {
        pos = updown.sendEvent( UDM_GETPOS );
        if ( pos.highNumber() )
           ITHROWGUIERROR2("UDM_GETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        range = updown.sendEvent( UDM_GETRANGE );
     }
     else
     {
        pos = IQUERYLASTPOS( spinBt );
        range = IQUERYRANGE( spinBt );
     }
     int  newpos = (short)pos.lowNumber() + (int)spinBy;
     // lowNumber is maximum, highNumber is minimum in the event result
     if ((short)range.lowNumber() < newpos)
     {
        newpos -= (short)range.lowNumber() - (short)range.highNumber() + 1;
        if ((short)range.lowNumber() < newpos)
        {
           // Still out of range ... error
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        }
     }
     if (updown.asUnsigned())
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2(newpos));

     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)newpos ),
                      IEventParameter2( updown.asUnsigned() ) );

     // Generate the spin up/down event expected by handlers
     IWindowHandle parent = IPARENTOF( spinBt );
     parent.sendEvent( WM_COMMAND,
                       IEventParameter1( (short)IIDOF( spinBt ),
                                         (short)SPBN_UPARROW ),
                       IEventParameter2( spinBt.asUnsigned()) );
   }
#endif
#endif // IC_PMWIN

   return *this;
}

#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::enable                                                      |
|                                                                              |
| Enables the window to accept keyboard and mouse input.                       |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: enable ( bool enableControl )
{
   IWindow::enable( enableControl);
#ifdef IC_WIN
   // If this is a native windows spin button, we need to notify the
   // entry field about this call (so that text is normal or grayed properly).
   if (!this->isPMCompatible())
   {
     // Get the entry field handle and enable/disable it.
     IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
     ef.sendEvent( WM_ENABLE,
                   IEventParameter1(enableControl),
                   IEventParameter2(0));
   }
#endif
   refresh();
   return *this;
}
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| IBaseSpinButton::isPMCompatible                                              |
------------------------------------------------------------------------------*/
bool IBaseSpinButton::isPMCompatible ( ) const
{
#ifdef IC_MOTIFWIN
   return ((extendedStyle() & pmCompatible.asExtendedUnsignedLong()) ?
             true : false );
#else
   return true;
#endif
}



#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::foregroundColor                                             |
|                                                                              |
| Returns the foreground color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IColor IBaseSpinButton::foregroundColor () const
{
  IColor guiColor( IColor::kWindowText );

  if (!isWriteable())
  {
    guiColor = IColor( IColor::kOutputText );
  }
#ifdef IC_PM
  return (IWindow::color(PP_FOREGROUNDCOLOR, guiColor));
#else
  {
    if (this->isPMCompatible())
    {
      unsigned char ucRed,
                    ucGreen,
                    ucBlue;
      unsigned long ulResult =
         handle().sendEvent(SPBM_QUERYSPINCOLORS,
                            IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                            IEventParameter2((unsigned long) NULL));
      ucBlue = (unsigned char)((ulResult & 0x00ff0000) >> 16);
      ucGreen = (unsigned char)((ulResult & 0x0000ff00) >> 8);
      ucRed = (unsigned char)(ulResult & 0x000000ff);
      return (IColor(ucRed, ucGreen, ucBlue));
    }
  else
    {
      return (IWindow::color(PP_FOREGROUNDCOLOR, guiColor));
    }
  }
#endif
}
#endif

/*------------------------------------------------------------------------------
| IBaseSpinButton::calcMinimumSize                                             |
|                                                                              |
| Return the minimum size of the spin button based on the font and the text    |
| limit.                                                                       |
------------------------------------------------------------------------------*/
ISize IBaseSpinButton :: calcMinimumSize() const
{
#ifdef IC_PMWIN
  unsigned long ulClTextLimit = limit();

  IFont font( this );
  unsigned long ulWidth = font.avgCharWidth();   // Average char width
  unsigned long ulHeight = font.maxCharHeight();

  /*****************************************************************/
  /* Use a step function to reflect the fact that you are more     */
  /* likely to exceed an average character width with an           */
  /* individual character, but a long string will likely be ok     */
  /* with this average value.                                      */
  /*****************************************************************/
  if (ulClTextLimit <= 2)
     ulWidth = ulWidth * ulClTextLimit * 3;
  else
     ulWidth = ulWidth * (ulClTextLimit + 5);

  /*****************************************************************/
  /* Add room for the spin arrows, but still be sure that the size */
  /* does not exceed 35 avg chars                                  */
  /*                                                               */
  /* Note: Spin arrows are 17 pels wide.                           */
  /*****************************************************************/
  if (this->isPMCompatible())
  {
    ulWidth += 17;
  }
#ifdef IC_WIN
  else
  {
    ulWidth += (unsigned long) GetSystemMetrics( SM_CXVSCROLL );
  }
#endif
  unsigned long maxWidth = font.avgCharWidth() * 35;
  if ( ulWidth > maxWidth )
  {
     ulWidth = maxWidth;
  }

  if (hasBorder())
  {                     // Entry field has border.
     ulWidth += (3 * IQUERYSYSVALUE(SV_CXBORDER));
#ifdef IC_PM
     ulHeight+= (3 * IQUERYSYSVALUE(SV_CYBORDER));
#endif
#ifdef IC_WIN
     // Use 1/2 a character since no conversion for lowercase api
     ulHeight += font.maxCharHeight()/2;
     // Oldshell needs a bit more.
     if (IPlatform::isWinNT() && !IPlatform::isNTNewShell())
        ulHeight += 2;
     if ( ( extendedStyle() & border3D.asExtendedUnsignedLong() ) &&
         !( extendedStyle() & pmCompatible.asExtendedUnsignedLong()) &&
	  ( IPlatform::isWin9x() || IPlatform::isNTNewShell() ))
     {
       ulWidth += (3 * IQUERYSYSVALUE(SV_CXBORDER));
       ulHeight+= (3 * IQUERYSYSVALUE(SV_CYBORDER));
     }
#endif
  }

  return ISize(ulWidth, ulHeight);
#endif // IC_PMWIN

#ifdef IC_MOTIF
  Dimension borderWidth, shadowThickness, highlightThickness, marginHeight;
  ISize     spinSize;
  int       maxLength;

  XtVaGetValues (fBaseSpinButtonData->efHandle,
                 XmNmaxLength,          &maxLength,
                 XmNborderWidth,        &borderWidth,
                 XmNshadowThickness,    &shadowThickness,
                 XmNhighlightThickness, &highlightThickness,
                 XmNmarginHeight,       &marginHeight,
                 NULL);

  spinSize = ((IBaseSpinButton*) this)->characterSize();

  if (maxLength > 35)
     maxLength = 35;
  else
     //add fudge factor consistent w/PMWIN implementation
     maxLength += 5;

  spinSize.setWidth (spinSize.width() * maxLength +
                     2 * (borderWidth + shadowThickness +
                          highlightThickness +
                          fBaseSpinButtonData->minMarginWidth));
  if (isMaster())
  {
     // Pad width to account for arrow button width.
     Dimension arrowWidth, arrowBorder;
     XtVaGetValues (fBaseSpinButtonData->upArrow,
                    XmNwidth,       &arrowWidth,
                    XmNborderWidth, &arrowBorder,
                    NULL);
     if (arrowWidth)
        spinSize.setWidth (spinSize.width() + arrowWidth + 2 * arrowBorder);
     else
        // Use a fudge factor if the spin button isn't visible.
        spinSize.setWidth (spinSize.width() +
                        3 * ((IBaseSpinButton*) this)->characterSize().width() +
                        2 * arrowBorder);
  }

  spinSize.setHeight (spinSize.height() +
                      2 * (borderWidth + shadowThickness +
                           highlightThickness +
                           marginHeight));
//                         fBaseSpinButtonData->minMarginHeight));

  return spinSize;
#endif // IC_MOTIF
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::setForegroundColor                                          |
|                                                                              |
| Updates the foreground color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton::setForegroundColor ( const IColor &color)
{
  if (this->isPMCompatible())
  {
    unsigned long ulResult =
       handle().sendEvent(SPBM_SETSPINCOLORS,
                          IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                          IEventParameter2((unsigned long) color.asRGBLong()));
    if (ulResult)
    {
      setLayoutDistorted(IWindow::colorChanged, 0);
      this->refresh( );
    }
    else
    {
      ITHROWGUIERROR2("SPBM_SETSPINCOLORS",
                      IBaseErrorInfo::invalidParameter,
                      IException::recoverable);
    } /* endif */
  }
  else
  {
    IWindow::setColor(PP_FOREGROUNDCOLOR, color);
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::backgroundColor                                             |
|                                                                              |
| Returns the background color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IColor IBaseSpinButton::backgroundColor () const
{
  unsigned char ucRed,
                ucGreen,
                ucBlue;
  if (this->isPMCompatible())
  {
    unsigned long ulResult =
       handle().sendEvent(SPBM_QUERYSPINCOLORS,
                          IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
                          IEventParameter2((unsigned long) NULL));
    ucBlue = (unsigned char)((ulResult & 0x00ff0000) >> 16);
    ucGreen = (unsigned char)((ulResult & 0x0000ff00) >> 8);
    ucRed = (unsigned char)(ulResult & 0x000000ff);
    return (IColor(ucRed, ucGreen, ucBlue));
  }
  else
  {
    return IWindow::color( PP_BACKGROUNDCOLOR,
                           IColor( IColor::kEntryFieldBgnd ) );
  }
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setBackgroundColor                                          |
|                                                                              |
| Updates the background color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton::setBackgroundColor ( const IColor &color)
{
  // inherited setBackgroundColor calls setColor, so put pmCompatible vs.
  // native specific code in setColor.
  Inherited::setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton :: setColor                                                 |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: setColor( unsigned long colorArea,
                                              const IColor& color)
{
  if (colorArea == PP_BACKGROUNDCOLOR)
  {
    if (this->isPMCompatible())
    {
      unsigned long ulResult =
         handle().sendEvent(SPBM_SETSPINCOLORS,
         IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
         IEventParameter2(NEARESTPALETTECOLOR((unsigned long) color.asRGBLong())));
      if (ulResult)
      {
        setLayoutDistorted(IWindow::colorChanged, 0);
        this->refresh( );
      }
      else
      {
        ITHROWGUIERROR2("SPBM_SETSPINCOLORS",
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
      } /* endif */
    }
    else
    {
      IWindow::setColor(PP_BACKGROUNDCOLOR, color);
    }
  }
  else
    Inherited::setColor( colorArea, color );

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton :: resetForegroundColor                                      |
|                                                                              |
| Resets the foreground color of the IBaseSpinButton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: resetForegroundColor ( )
{
  if (this->isPMCompatible())
  {
    handle().sendEvent( SPBM_RESETSPINCOLORS,
                        IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                        IEventParameter2((unsigned long) NULL));
    this->refresh( );
  }
  else
  {
    IWindow::resetColor(PP_FOREGROUNDCOLOR);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton :: resetBackgroundColor                                      |
|                                                                              |
| Resets the background color of the IBaseSpinButton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton :: resetBackgroundColor ( )
{
  if (this->isPMCompatible())
  {
    handle().sendEvent( SPBM_RESETSPINCOLORS,
                        IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
                        IEventParameter2((unsigned long) NULL));
    this->refresh( );
  }
  else
  {
    IWindow::resetColor(PP_BACKGROUNDCOLOR);
  }
  return *this;
}
#endif  // IC_WIN

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IBaseSpinButton::visibleRectangle                                            |
|                                                                              |
| Calculate the actual displayed (painted) ares of the spin button.            |
------------------------------------------------------------------------------*/
IRectangle IBaseSpinButton::visibleRectangle() const
{
  if ( !hasBorder() )
  {
    IFont         font( this );
    unsigned long ulHeight = font.maxCharHeight();
    IRectangle    trueRect = nativeRect();
    trueRect = IRectangle ( trueRect.bottomLeft(),
                            IPoint(trueRect.right(),
                                   trueRect.bottom()+ulHeight));
    return ICoordinateSystem::convertToApplication( trueRect, parentSize() );
  }
  return Inherited::visibleRectangle();
}
#endif // IC_PM

#ifdef IC_MOTIF
#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbSpinUp is the procedure invoked when the spin timer elapses.  It will   |
|  spin the entry field and start a new timer.                                 |
------------------------------------------------------------------------------*/
extern void _System ibsbSpinUp (IBaseSpinButtonData *fBaseSpinButtonData,
                                XtIntervalId *id)
{
  IBaseSpinButton* spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent((Widget)fBaseSpinButtonData->efHandle));

  Widget w = (Widget)fBaseSpinButtonData->efHandle;

  // Spin up one position.
  spinButton->spinUp();

  bool isNumericSpin = fBaseSpinButtonData->isNumericOnly;

  // If the spin button wrapped, reset the spin speed if fastSpin is enabled.
  if (spinButton->isFastSpinEnabled())
     if ((isNumericSpin &&
          ((INumericSpinButton*)spinButton)->value() ==
               ((INumericSpinButton*)spinButton)->range().lowerBound()) ||
         (!isNumericSpin &&
          ((ITextSpinButton*)spinButton)->text() ==
               ((ITextSpinButton*)spinButton)->elementAt (0)))
     {
        XtRemoveTimeOut (fBaseSpinButtonData->accelerationTimer);
        fBaseSpinButtonData->currentDelay = fBaseSpinButtonData->initialDelay;
        fBaseSpinButtonData->accelerationTimer =
              XtAppAddTimeOut (XtWidgetToApplicationContext (w),
                               2000,    // every 2 seconds
                               (XtTimerCallbackProc)ibsbFastSpin,
                               fBaseSpinButtonData);
     }

  // Generate SPBN_UPARROW event.
  spinButton->postEvent (IWindow::control,
               IEventData ((unsigned short)spinButton->id(), SPBN_UPARROW),
               IEventData ((Widget)fBaseSpinButtonData->efHandle));

  fBaseSpinButtonData->timer = XtAppAddTimeOut (
                   XtWidgetToApplicationContext (w),
                   fBaseSpinButtonData->currentDelay,
                   (XtTimerCallbackProc)ibsbSpinUp,
                   fBaseSpinButtonData);
  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbSpinDown is the procedure invoked when the spin timer elapses.  It will |
|  spin the entry field and start a new timer.                                 |
------------------------------------------------------------------------------*/
extern void _System ibsbSpinDown (IBaseSpinButtonData *fBaseSpinButtonData,
                                  XtIntervalId *id)
{
  IBaseSpinButton* spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent((Widget)fBaseSpinButtonData->efHandle));

  Widget w = (Widget)fBaseSpinButtonData->efHandle;

  
  // Spin down one position.
  spinButton->spinDown();

  bool isNumericSpin = fBaseSpinButtonData->isNumericOnly;

  // If the spin button wrapped, reset the spin speed if fastSpin is enabled.
  if (spinButton->isFastSpinEnabled())
  {
     bool resetFastSpin = false;
     if (isNumericSpin &&
          ((INumericSpinButton*)spinButton)->value() ==
             ((INumericSpinButton*)spinButton)->range().upperBound())
        resetFastSpin = true;
     else if (!isNumericSpin)
     {
        ITextSpinButton::Cursor tc ((ITextSpinButton&)*spinButton);
        tc.setToLast();
        if (((ITextSpinButton*)spinButton)->text() ==
               ((ITextSpinButton*)spinButton)->elementAt (tc))
           resetFastSpin = true;
     }
     if (resetFastSpin)
        {
           XtRemoveTimeOut (fBaseSpinButtonData->accelerationTimer);
           fBaseSpinButtonData->currentDelay =
             fBaseSpinButtonData->initialDelay;
           fBaseSpinButtonData->accelerationTimer =
                 XtAppAddTimeOut (XtWidgetToApplicationContext (w),
                                  2000,    // every 2 seconds
                                  (XtTimerCallbackProc)ibsbFastSpin,
                                  fBaseSpinButtonData);
        }
  }

  // Generate SPBN_DOWNARROW event.
  spinButton->postEvent (IWindow::control,
               IEventData ((unsigned short)spinButton->id(), SPBN_DOWNARROW),
               IEventData ((Widget)fBaseSpinButtonData->efHandle));

  fBaseSpinButtonData->timer = XtAppAddTimeOut (
                   XtWidgetToApplicationContext (w),
                   fBaseSpinButtonData->currentDelay,
                   (XtTimerCallbackProc) ibsbSpinDown,
                   fBaseSpinButtonData);
  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbFastSpin is the procedure invoked when the acceleration timer elapses.  |
|  It will halve the currentDelay time for this spinner and start a new timer. |
|  accelerationTimer.                                                          |
------------------------------------------------------------------------------*/
extern void _System ibsbFastSpin (IBaseSpinButtonData *fBaseSpinButtonData,
                                  XtIntervalId *id)
{
  IBaseSpinButton* spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent((Widget)fBaseSpinButtonData->efHandle));

  if (fBaseSpinButtonData->currentDelay >= fBaseSpinButtonData->minDelay)
     fBaseSpinButtonData->currentDelay /= 2;

  fBaseSpinButtonData->accelerationTimer = XtAppAddTimeOut
               (XtWidgetToApplicationContext ((Widget) fBaseSpinButtonData->efHandle),
                                              2000,
                                              (XtTimerCallbackProc)ibsbFastSpin,
                                              fBaseSpinButtonData);
  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbArmCallback is the callback invoked when the user presses the           |
|  up or down arrow button to spin values in the entry field.                  |
------------------------------------------------------------------------------*/
extern void _System ibsbArmCallback (Widget w,
                                     XtPointer client_data,
                                     XtPointer call_data)
{
  // Get private data pointer from client data.
  IBaseSpinButtonData* fBaseSpinButtonData = (IBaseSpinButtonData*)client_data;
  
  // Get the spin button with the current focus if there is one.
  Widget focusWidget = XmGetFocusWidget (w);


  // First get master spin button.
  IBaseSpinButton* spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent((Widget)(fBaseSpinButtonData->efHandle)));

  // If the master is not the one with focus, see if it is one of the
  // servants.
  if (focusWidget != fBaseSpinButtonData->efHandle)
  {
     if (fBaseSpinButtonData->servantSet.containsElementWithKey ((IWindowHandle) focusWidget))
     {
        // It must be one of the servant spin buttons.
        spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent(focusWidget));
        fBaseSpinButtonData = spinButton->fBaseSpinButtonData;
     }
     else
     {
        // Set focus on the master spin button.
        XmProcessTraversal (fBaseSpinButtonData->efHandle,
                            XmTRAVERSE_CURRENT);
     }
  }

  // If this spin button is already actively spinning, just return.
  XtPointer userData;
  XtVaGetValues (fBaseSpinButtonData->efHandle,
                 XmNuserData, &userData,
                 NULL);
  IBaseSpinButtonData::SpinType spinType =
     (IBaseSpinButtonData::SpinType)userData;

  if (spinType == IBaseSpinButtonData::mouse ||
      spinType == IBaseSpinButtonData::keyUp ||
      spinType == IBaseSpinButtonData::keyDown)
     return;

  if (spinType == IBaseSpinButtonData::notSpinning)
  {
     // Grab input to insure that we get the disarm event.  (Prevents callback
     // from doing a grab or otherwise changing input focus to another widget.)
     XButtonEvent* buttonEvent =
         (XButtonEvent*)((XmPushButtonCallbackStruct*)call_data)->event;
     if (XtGrabPointer (w,                  // Grabbing widget.
                        False,              // Pointer events to the grab window.
                        ButtonReleaseMask,  // Interested in ButtonReleases.
                        GrabModeAsync,      // Process pointer events.
                        GrabModeSync,       // Process keyboard events.
                        None,               // Don't confine cursor.
                        None,               // Use normal cursor.
                        buttonEvent->time)  // Use XButtonEvent time.
         != GrabSuccess)
        return;
  }

  XtVaSetValues (fBaseSpinButtonData->efHandle,
                 XmNuserData, (XtPointer)IBaseSpinButtonData::mouse,
                 NULL);

  fBaseSpinButtonData->currentDelay = fBaseSpinButtonData->initialDelay;
  // If it is a fast spin button, start timer to increase spin speed after
  // 2 seconds.
  if (spinButton->isFastSpinEnabled())
     fBaseSpinButtonData->accelerationTimer =
           XtAppAddTimeOut (XtWidgetToApplicationContext (w),
                            2000,    // every 2 seconds
                            (XtTimerCallbackProc)ibsbFastSpin,
                            fBaseSpinButtonData);

  // Spin up/down one position.
  if (w == fBaseSpinButtonData->upArrow)
  {
     spinButton->spinUp();

     // Generate SPBN_UPARROW event.
     spinButton->postEvent (IWindow::control,
                  IEventData ((unsigned short)spinButton->id(), SPBN_UPARROW),
                  IEventData ((Widget)fBaseSpinButtonData->efHandle));

     fBaseSpinButtonData->timer = XtAppAddTimeOut (
									 XtWidgetToApplicationContext (w),
                                     fBaseSpinButtonData->currentDelay,
                                     (XtTimerCallbackProc)ibsbSpinUp,
                                     fBaseSpinButtonData);
  }
  else
  {
     spinButton->spinDown();

     // Generate SPBN_DOWNARROW event.
     spinButton->postEvent (IWindow::control,
                  IEventData ((unsigned short)spinButton->id(), SPBN_DOWNARROW ),
                  IEventData ((Widget)fBaseSpinButtonData->efHandle));

     fBaseSpinButtonData->timer = XtAppAddTimeOut (
									 XtWidgetToApplicationContext (w),
                                     fBaseSpinButtonData->currentDelay,
                                     (XtTimerCallbackProc)ibsbSpinDown,
                                     fBaseSpinButtonData);
  }

  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbDisarmCallback is the callback invoked when the user releases the       |
|  up or down arrow button to end the spin in the entry field.                 |
------------------------------------------------------------------------------*/
extern void _System ibsbDisarmCallback (Widget w,
                                        XtPointer client_data,
                                        XtPointer call_data)
{
  // Get private data pointer from client data.
  IBaseSpinButtonData* fBaseSpinButtonData = (IBaseSpinButtonData*)client_data;
  
  // Get the spin button with the current focus if there is one.
  Widget focusWidget = XmGetFocusWidget (w);


  // First get master spin button.
  IBaseSpinButton* spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent((Widget)(fBaseSpinButtonData->efHandle)));

  // If the master is not the one with focus, see if it is one of the
  // servants.
  if (focusWidget != fBaseSpinButtonData->efHandle)
  {
     if (fBaseSpinButtonData->servantSet.containsElementWithKey ((IWindowHandle) focusWidget))
     {
        // It must be one of the servant spin buttons.
        spinButton = (IBaseSpinButton*) IWindow::windowWithHandle
                                (XtParent(focusWidget));
        fBaseSpinButtonData = spinButton->fBaseSpinButtonData;
     }
     else
     {
        // Set focus on the master spin button.
        XmProcessTraversal (fBaseSpinButtonData->efHandle,
                            XmTRAVERSE_CURRENT);
     }
  }
  
  // If this spin button is not spinning via arrow button, return.  This is
  // possible if both the mouse button and cursor keys were used together to
  // start spinning.
  XtPointer userData;
  XtVaGetValues (focusWidget,
                 XmNuserData, &userData,
                 NULL);
  IBaseSpinButtonData::SpinType spinType =
      (IBaseSpinButtonData::SpinType)userData;

  if (spinType != IBaseSpinButtonData::mouse &&
      spinType != IBaseSpinButtonData::mouseUpSuspend &&
      spinType != IBaseSpinButtonData::mouseDownSuspend)
     return;

  XtVaSetValues (focusWidget,
                 XmNuserData, IBaseSpinButtonData::notSpinning,
                 NULL);

  ibsbStopSpin (spinButton, fBaseSpinButtonData);

  // Remove the input grab.
  XtUngrabPointer (w, CurrentTime);

  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbStopSpin is called by ibsbDisarmCallback to restore spin button to the  |
|  non-spinning state.                                                         |
------------------------------------------------------------------------------*/
extern void ibsbStopSpin (IBaseSpinButton*     spinButton,
                          IBaseSpinButtonData* fBaseSpinButtonData)
{
  // Cancel the spin timer.
  if (fBaseSpinButtonData->timer)
  {
     XtRemoveTimeOut (fBaseSpinButtonData->timer);
     fBaseSpinButtonData->timer = 0;
  }

  // If it is a fast spin button, remove acceleration timer as well.
  if (fBaseSpinButtonData->accelerationTimer)
  {
     XtRemoveTimeOut (fBaseSpinButtonData->accelerationTimer);
     fBaseSpinButtonData->accelerationTimer = 0;
  }

  // Generate SPBN_ENDSPIN event.
  spinButton->postEvent (IWindow::control,
               IEventData ((unsigned short)spinButton->id(), SPBN_ENDSPIN),
               IEventData ((Widget)fBaseSpinButtonData->efHandle));
  return;
}
#pragma options info=reset

#pragma options info=nouse
/*------------------------------------------------------------------------------
|  ibsbDestroyCallback is the callback invoked when a servant spin button      |
|  is destroyed.  The servant will be removed from the master's servant set.   |
------------------------------------------------------------------------------*/
extern void _System ibsbDestroyCallback (Widget w,
                                         XtPointer client_data,
                                         XtPointer call_data)
{
  // Get private data pointer from client data.
  IBaseSpinButtonData *masterData = (IBaseSpinButtonData*) client_data;

  // Remove this servant from the servant set of my master.
  IServantElement*
     servant = masterData->servantSet.elementWithKey ((IWindowHandle)w);
  masterData->servantSet.removeElementWithKey ((IWindowHandle)w);
  delete servant;

  return;
}
#pragma options info=reset
#endif // IC_MOTIF
