// Revision: 66 1.5.1.3 source/ui/basectl/ispinszh.hpp, listctls, ioc.v400, 000302 
/* NOSHIP */
#ifndef _ISPINSZH_
  #define _ISPINSZH_
/*******************************************************************************
* FILE NAME: ispinszh.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IBaseSpinDefaultHandler - Windows - handles messages from updown control.*
*                               PM -  allows a spin button to properly         *
*                               size its child entry field.  Replaces          *
*                               ISpinButtonSizeHandler                         *
*     ISpinButtonStatics     - This class manages the creation and deletion of *
*                              the spin button size handler object.            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <isizehdr.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class ISpinButton;
class IBaseSpinButton;
struct _UDACCEL;

#pragma enum(4)
#pragma pack(push,4)

#ifdef IC_WIN
class IBaseSpinDefaultHandler : public IHandler {
typedef IHandler
  Inherited;
#else
class IBaseSpinDefaultHandler : public IResizeHandler {
typedef IResizeHandler
  Inherited;
#endif
/*******************************************************************************
* This class is the default handler for an IBaseSpinButton.   In Windows,      *
* it allows an IBaseSpinButton window to handle the notification               *
* events from the up/down control.                                             *
* In PM, it allows an IBaseSpinButton window to properly change its size       *
* (actually the size of its child entry field and reposition its spin arrows)  *
* to compensate for the use of SWP_NOADJUST in IWindow::sizeTo.                *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
  IBaseSpinDefaultHandler ( );
virtual
 ~IBaseSpinDefaultHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a spin button:                                                               |
|   handleEventsFor       - Attaches the handler to the specified control.     |
|   stopHandlingEventsFor - Detaches the handler from the specified control.   |
------------------------------------------------------------------------------*/
virtual IBaseSpinDefaultHandler
 &handleEventsFor        ( IBaseSpinButton* spinButton ),
 &stopHandlingEventsFor  ( IBaseSpinButton* spinButton );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   dispatchHandlerEvent - processes events sent to the spinbutton control.    |
|                  Handles native up/down notifications as well as the         |
|                  positioning of the entry field and spin arrows.             |
|                  (Windows only)                                              |
|   windowResize - Allows the spin button to correctly resize and position     |
|                  its child entry field and spin arrows. (PM only)            |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
virtual bool
  dispatchHandlerEvent   ( IEvent& event );
#else
virtual bool
  windowResize           ( IResizeEvent& event );
#endif

private:
/*--------------------------------- Private ----------------------------------*/
IHandler
 &handleEventsFor        ( IWindow *window ),
 &stopHandlingEventsFor  ( IWindow *window );

#ifdef IC_WIN
long
 iDelta;
IWindowHandle
 hwndDelta;
HBRUSH
  bkgrndbrush;
#endif
}; // class IBaseSpinDefaultHandler

class ISpinButtonStatics  {
/*******************************************************************************
* This class manages the lifetime of the static spin button size handler.  An  *
* object of this class is constructed during static object initialization.     *
* The handler object is constructed only if needed, and is deleted at          *
* termination of the application.                                              *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
 ~ISpinButtonStatics ( );

/*-------------------------------- Accessors -----------------------------------
| This function is used to access the static spin button size handler.  This   |
| allows one object to be constructed when needed, and shared by all spin      |
| buttons.                                                                     |
|   buddyHandle   - Returns the handle of the spin button edit field.          |
------------------------------------------------------------------------------*/
static IWindowHandle
  buddyHandle( const IWindowHandle& spinbutton );

/*-------------------------------- Styles --------------------------------------
| This function is used to convert extended spin button styles to the          |
| appropriate style for the buddy control.                                     |
|   convertToBuddyStyle - returns the style flags for the buddy control.       |
|   convertToExtendedStyle - returns the extended style flags based on the     |
|                            window sytles used in a PM compatible spinbutton  |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
static unsigned long
  convertToBuddyStyle( unsigned long extendedStyle );
#endif
static unsigned long
  convertToExtendedStyle( unsigned long guiStyle );

/*-------------------------------- Fast Spin -----------------------------------
| This function is used to access the fast spin acceleration table.            |
|   fastSpinTableSize   - returns the number of entries in the fast spin       |
|                         acceleration table.                                  |
|   fastSpinTable       - returns pointer to the fast spin acceleration table. |
|   normalSpinTable     - returns pointer to the normal spin acceleration table|
------------------------------------------------------------------------------*/
#ifdef IC_WIN
static unsigned long
  fastSpinTableSize( );
static struct _UDACCEL
  *fastSpinTable( ),
  *normalSpinTable( );
#endif


private:
/*--------------------------------- Private ----------------------------------*/
static IBaseSpinDefaultHandler
 *fsBaseSpinHandler;
#ifdef IC_WIN
static struct _UDACCEL
 *fsfastSpinTable,
 *fsnormalSpinTable;
#endif
}; // class ISpinButtonStatics


#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#ifdef IC_WIN
// Definition of the extra window words in the WC_NATIVESPINBUTTON class,
// and macros to access them.  For all functions, the spbthwnd argument is
// the handle() of the IBaseSpinButton object, which is a WC_NATIVESPINBUTTON.
// Range and position values are the same format as UDM_GETRANGE and UDM_GETPOS
// messages.
// IQUERYUPDOWNHANDLE.  Returns the handle of the updown control.
// ISETUPDOWNHANDLE.    Sets the handle of the updown control.
// IQUERYRANGE.         Returns the range of the updown control for this
//                      spin field.
// ISETRANGE.           Sets the range of the updown control for this
//                      spin field.
// IQUERYLASTPOS.       Returns the last position.
// ISETLASTPOS.         Sets the last position, returning previous value.
// IQUERYACTIVEBUDDY.   Returns the active buddy control (edit field)
// ISETACTIVEBUDDY.     Sets the active buddy control (edit field)
// IQUERYBUDDYPROC      Returns the default procedure for edit field
// ISETBUDDYPROC        Sets the default procedure for edit field
#define QWL_UPDOWNHANDLE 0
#define QWL_SPBRANGE     4
#define QWL_LASTPOS      8
#define QWL_ACTIVEBUDDY  12
#define QWL_BUDDYPROC    16          // default procedure for buddy control
#define QWL_TEXTEMPTY    20          // Flag for empty text for handler
#define NATIVESPINBUTTONEXTRA  24    //number of extra bytes
#define IQUERYUPDOWNHANDLE(spbthwnd) \
            ((HWND)(GetWindowLong(spbthwnd,QWL_UPDOWNHANDLE)))
#define ISETUPDOWNHANDLE(spbthwnd,hwndud) \
            (SetWindowLong(spbthwnd,QWL_UPDOWNHANDLE,(LONG)hwndud))
#define IQUERYRANGE(spbthwnd) \
            ((unsigned long)(GetWindowLong(spbthwnd,QWL_SPBRANGE)))
#define ISETRANGE(spbthwnd,rng) \
            (SetWindowLong(spbthwnd,QWL_SPBRANGE,(LONG)rng))
#define IQUERYLASTPOS(spbthwnd) \
            ((unsigned long)(GetWindowLong(spbthwnd,QWL_LASTPOS)))
#define ISETLASTPOS(spbthwnd,pstn) \
            (SetWindowLong(spbthwnd,QWL_LASTPOS,(LONG)pstn))
#define IQUERYACTIVEBUDDY(spbthwnd) \
            ((HWND)(GetWindowLong(spbthwnd,QWL_ACTIVEBUDDY)))
#define ISETACTIVEBUDDY(spbthwnd,hwndbdy) \
            (SetWindowLong(spbthwnd,QWL_ACTIVEBUDDY,(LONG)hwndbdy))
#define IQUERYBUDDYPROC(spbthwnd) \
            ((WNDPROC)(GetWindowLong(spbthwnd,QWL_BUDDYPROC)))
#define ISETBUDDYPROC(spbthwnd,prc) \
            (SetWindowLong(spbthwnd,QWL_BUDDYPROC,(LONG)prc))
#define IQUERYEMPTYFLAG(spbthwnd) \
            ((bool)(GetWindowLong(spbthwnd,QWL_TEXTEMPTY)))
#define ISETEMPTYFLAG(spbthwnd,fl) \
            (SetWindowLong(spbthwnd,QWL_TEXTEMPTY,(LONG)fl))

LRESULT CALLBACK fnwpSpinButtonEditProc(
                    HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam );
#endif

#endif /* _ISPINSZH_ */
