// Revision: 76 1.7 source/ui/basectl/isplitpc.hpp, canvas, ioc.v400, 000302  
/* NOSHIP */
#ifndef _ISPLITPC_
  #define _ISPLITPC_
/*******************************************************************************
* FILE NAME: isplitpc.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IPaneObj         - This class keeps up with the windows occupying the    *
*                        panes of a canvas.                                    *
*     ISplitPercentage - This class keeps up with the percentage data for      *
*                        all windows in the canvas.                            *
*     INumberSplitBar                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icanvas.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IPaneSeq;
class ISplitBar;
class ISplitCanvas;

#pragma enum(4)
#pragma pack(push,4)

class IPaneObj {
public:
  IPaneObj ( IWindow* window )
    : pwinCl( window ), ulControlSize( 0 ), ulClPercentage( 10 ), pNext( 0 )
  { }
private:
friend class ISplitCanvas;
friend class IPaneSeq;
IWindow
 *pwinCl;
unsigned long
  ulControlSize,
  ulClPercentage;
IPaneObj
 *pNext;
}; // IPaneObj

class ISplitPercentage {
public:
  ISplitPercentage ( IWindow* window,
                     unsigned long percentage )
    : pwinCl( window ), ulClPercentage( percentage ),
      ulClResolvedPercentage( 0 )
  { }
private:
friend class ISplitCanvas;
friend IWindow* const
 &key ( ISplitPercentage* const& element );
IWindow
 *pwinCl;
unsigned long
  ulClPercentage,
  ulClResolvedPercentage;
}; // ISplitPercentage


inline IWindow* const& key ( ISplitPercentage* const& element )
{
  return element->pwinCl;
}


#ifdef IC_MOTIF
class INumberSplitBar {
public:
  INumberSplitBar ( ISplitBar* splitBar,
                    unsigned long barNumber )
    : pwinCl( splitBar ),
      ulClBarNumber( barNumber )
  { }
ISplitBar
 *splitBar ( ) const
  {
    return pwinCl;
  }
private:
friend unsigned long const
 &key ( INumberSplitBar* const& element );
ISplitBar
 *pwinCl;
unsigned long
  ulClBarNumber;
}; // INumberSplitBar


inline unsigned long const& key ( INumberSplitBar* const& element )
{
  return element->ulClBarNumber;
}
#endif

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _ISPLITPC_ */
