// Revision: 69 1.8 source/ui/basectl/itabpriv.hpp, notebook, ioc.v400, 000302  
/* NOSHIP */
#ifndef _ITABPRIV_
#define _ITABPRIV_
/*******************************************************************************
* FILE NAME: itabpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ITabControlData                                                          *
*     ITabControlDefaultHandler                                                *
*     ITabControlButton                                                        *
*     ITabControlButtonSequence                                                *
*     ITabPageData                                                             *
*                                                                              *
*   This file also contains the declaration of the following classes which     *
*   are specific to the Windows environment:                                   *
*     ITabControlButtonCanvas                                                  *
*     ITabControlClipPaintHandler                                              *
*     ITabControlClipWindow                                                    *
*     ITabControlPageSequence                                                  *
*     ITabPageKeyboardHandler                                                  *
*                                                                              *
*   This file also contains the declaration of the following classes which     *
*   are specific to the OS/2 environment:                                      *
*     ITabPageResizeHandler                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   Licensed Material - Property of IBM                                        *
*   (C) Copyright IBM Corp.  1997  All Rights Reserved.                        *
*                                                                              *
*******************************************************************************/
#include <ihandler.hpp>
#include <itabctl.hpp>

#ifdef IC_WIN
#include <ikeyevt.hpp>
#include <ipainhdr.hpp>
#include <isetcv.hpp>
#endif // IC_WIN

#ifdef IC_PM
#include <isizehdr.hpp>
#endif // IC_PM

#pragma info(none)
  #include <ies2.h>
  #include <ikeyset2.h>
#pragma info(restore)

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IEvent;
class IPaintEvent;
class ITabControlButton;
class ITabControlButtonCanvas;
class ITabControlButtonSequence;
class ITabControlClipWindow;
class ITabControlDefaultHandler;
class ITabControlPageSequence;
class ITabPage;
class ITabPageKeyboardHandler;
class ITabPageResizeHandler;

#define NUM_STANDARD_BUTTONS 5

#pragma enum(4)
#pragma pack(push,4)

/*------------------------------------------------------------------------------
| ITabControlData                                                              |
|                                                                              |
| Private data class for the tab control.                                      |
------------------------------------------------------------------------------*/
class ITabControlData
{
public:
  ITabControlData ( ITabControl* tabControl );
 ~ITabControlData ( );

/*---- Private functions necessary for the implementation of ITabControl -----*/
IPushButton
  *createButton      ( ITabControl::EButton buttonType );

unsigned long
  tabPageIndex       ( const ITabPage*      tabPage ) const,
  buttonIndex        ( const ITabControl::EButton buttonType ) const;

ITabControl::EButton
  buttonEnum         ( const IPushButton*   button ) const;

void
  addPage            ( unsigned long        pageIndex,
                       const ITabPage*      tabPage ),
  deletePage         ( ITabPage*            tabPage,
                       bool                 bRemoveAll ),
  addCommonButton    ( ITabControlButton*   button,
                       bool                 isDefaultButton ),
  removeCommonButton ( ITabControlButton*   button );

#ifdef IC_WIN
void
  showTopPage        ( unsigned long        pageIndex,
                       bool                 bNotResizing = true ),
  adjustRect         ( bool                 bTabCtrlSize,
                       void*                pRect ) const,
  tabControlResize   ( const ISize&         newSize ),
  layoutChildren     ( const ISize&         size ),
  alignButtons       ( ),
  showButtons        ( const ITabPage*      prevPage,
                       const ITabPage*      newPage ),
  processTabSelect   ( unsigned long        currentPageIndex,
                       unsigned long        newPageIndex );

unsigned long
  buttonCanvasHeight ( ) const;
#endif // IC_WIN

#ifdef IC_PM
ITabPage
  *queryTabPage      ( unsigned short       pageOrder,
                       const ITabPage*      referencePage ) const;
#endif // IC_PM

/*------- Private data necessary for the implementation of ITabControl -------*/
ITabControl
 *fTabControl;

ITabControlDefaultHandler
 *fDefaultHandler;

ITabControlPageSequence
 *fPageSequence;

ITabControlButtonSequence
 *fCommonButtons;

ITabControlButton
 *fDefaultButton;

IPushButton
 *fStandardButton[5];

bool
  fAutoDeletePages,
  fPageBkgrndColorSet;

IColor
  fPageBkgrndColor;

unsigned long
  fValidate;

#ifdef IC_WIN
ITabPageKeyboardHandler
 *fPageKeyboardHandler;

ITabControlButtonCanvas
 *fButtonCanvas;

unsigned long
  fButtonCanvasHeight;

ISize
  fDefButtonMargin;

bool
  fInCreate;

void*
  fImageList;

ITabControlClipWindow
 *fClipWindow;
#endif // IC_WIN

#ifdef IC_PM
ITabPageResizeHandler
 *fTabPageResizeHandler;
#endif // IC_PM
}; // ITabControlData


/*------------------------------------------------------------------------------
| ITabControlDefaultHandler                                                    |
------------------------------------------------------------------------------*/
class ITabControlDefaultHandler : public IHandler {
typedef IHandler
  Inherited;

public:
  ITabControlDefaultHandler ( );
virtual
 ~ITabControlDefaultHandler ( ) { };

protected:
virtual bool
  dispatchHandlerEvent ( IEvent &anEvent );

private:
#ifdef IC_WIN
bool
  gotFocus   ( IEvent& event ),
  handleTab  ( IKeyboardEvent& event );

bool
  fFocusSet;

ITabPage
 *fPreviousPage;
#endif // IC_WIN

#ifdef IC_PM
bool
  fFocusPage;
#endif // IC_PM
}; // ITabControlDefaultHandler


/*------------------------------------------------------------------------------
| ITabControlPageSequence                                                      |
------------------------------------------------------------------------------*/
class ITabControlPageSequence : public IEqualitySequence< ITabPage* >
{
public:
  ITabControlPageSequence ( );
 ~ITabControlPageSequence ( );
}; // ITabControlPageSequence


/*------------------------------------------------------------------------------
| ITabControlButton                                                            |
------------------------------------------------------------------------------*/
class ITabControlButton
{
public:
  ITabControlButton ( );
 ~ITabControlButton ( );

IPushButton*
  button ( const ITabControl* tabControl ) const;

IPushButton
 *fApplicationButton;

ITabControl::EButton
  fStandardButton;
};


/*------------------------------------------------------------------------------
| ITabControlButtonSequence                                                    |
------------------------------------------------------------------------------*/
class ITabControlButtonSequence : public IEqualitySequence< ITabControlButton* >
{
public:
  ITabControlButtonSequence ( );
 ~ITabControlButtonSequence ( );
}; // ITabControlButtonSequence


#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITabControlButtonCanvas                                                      |
|                                                                              |
| We override ISetCanvas in order to handle font changes and button deletions. |
------------------------------------------------------------------------------*/
class ITabControlButtonCanvas : public ISetCanvas
{
typedef ISetCanvas
  Inherited;

public:
  ITabControlButtonCanvas ( ITabControl* tabControl );
 ~ITabControlButtonCanvas ( );

virtual ITabControlButtonCanvas
 &setLayoutDistorted ( unsigned long layoutAttributesOn,
                       unsigned long layoutAttributesOff );

/*----------------------------- Hidden Members -------------------------------*/
ITabControlButtonCanvas ( const ITabControlButtonCanvas& buttonCanvas );
ITabControlButtonCanvas
 &operator= ( const ITabControlButtonCanvas& buttonCanvas );

}; // ITabControlButtonCanvas
#endif // IC_WIN


/*------------------------------------------------------------------------------
| ITabPageData                                                                 |
|                                                                              |
| Private data class for the tab page class.                                   |
------------------------------------------------------------------------------*/
class ITabPageData
{
public:
  ITabPageData ( ITabPage *tabPage );
 ~ITabPageData ( );

/*---- Private functions necessary for the implementation of ITabPage --------*/
void
  addPageButton    ( ITabControlButton*         button,
                     bool                       isDefaultButton ),
  removePageButton ( ITabControlButton*         button );

#ifdef IC_PM
void
  setButtons       ( ITabControlButtonSequence* buttons );
#endif // IC_PM

static unsigned long
  autoTabColor ( ITabPage* tabPage );

/*--------- Private data necessary for the implementation of ITabPage --------*/
ITabControl
 *fTabControl;

ITabPage
 *fTabPage;
 
IWindow
 *fPageWindow;

IText
  fTabText;
 
IBitmapHandle
  fTabBitmap;

unsigned long
  fHelpId;
 
ITabControlButtonSequence
 *fPageButtons;

unsigned long
  fStyle;

ITabControlButton
 *fDefaultButton;

bool
  fTabBkgrndColorSet;
 
IColor
  fTabBkgrndColor;
 
#ifdef IC_WIN
bool
  fAutoPageSize;

IWindow
 *fPageWindowParent;
#endif // IC_WIN

#ifdef IC_PM
unsigned long
  fPageId;

static const unsigned long autoColors[10];
#endif // IC_PM
}; // ITabPageData


#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITabControlClipPaintHandler                                                  |
|                                                                              |
| Handle paint requests for the page clipping window.                          |
------------------------------------------------------------------------------*/
class ITabControlClipPaintHandler : public IPaintHandler
{
typedef IPaintHandler
  Inherited;
public:
  ITabControlClipPaintHandler  ( );
virtual
  ~ITabControlClipPaintHandler ( );

protected:
virtual bool
  paintWindow          ( IPaintEvent& event ),
  dispatchHandlerEvent ( IEvent&      event );
}; // ITabControlClipPaintHandler


/*------------------------------------------------------------------------------
| ITabControlClipWindow                                                        |
|                                                                              |
| Page clipping window class for the Windows tab control implementation.       |
| Clips the application page window to the allowable display area within the   |
| tab control to prevent it from painting over areas of the control.           |
------------------------------------------------------------------------------*/
class ITabControlClipWindow : public IWindow
{
public:
  ITabControlClipWindow  ( ITabControl* tabControl );

  ITabControlClipWindow  ( const ITabControlClipWindow& tabPageClipWindow );

ITabControlClipWindow
 &operator =          ( const ITabControlClipWindow& tabPageClipWindow );

virtual
  ~ITabControlClipWindow ( );

private:
ITabControlClipPaintHandler
 *fPaintHandler;
}; // ITabControlClipWindow


/*------------------------------------------------------------------------------
| ITabPageKeyboardHandler                                                      |
|                                                                              |
| Process Alt+PageUp, Alt+PageDown, and Alt+UpArrow keys for the application   |
| page window that is on top.  This handler is only attached to the            |
| application page window that is on top.  When a new page becomes the top     |
| page this handler is removed from the current page and attached to the new   |
| top page.                                                                    |
------------------------------------------------------------------------------*/
class ITabPageKeyboardHandler : public IHandler
{
typedef IHandler
  Inherited;
public:
  ITabPageKeyboardHandler  ( );
virtual
  ~ITabPageKeyboardHandler ( );

protected:
  dispatchHandlerEvent ( IEvent& event );
}; // ITabPageKeyboardHandler
#endif // IC_WIN


#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITabPageResizeHandler                                                        |
|                                                                              |
| This handler is used to workaround a PM problem. The problem manifests       |
| itself when a page window is resized subsequent to its being set as a page   |
| window in the notebook.  When this occurs, the notebook doesn't correctly    |
| manage the geometry for the resized page unless it is the page window for    |
| the current top page.  The fix for this is to add a resize handler to the    |
| page windows as they are set in the notebook.  The resize handler will reset |
| the page window for the corresponding page if and when the page window is    |
| resized.  This resetting of the page window causes the PM notebook to        |
| correctly cache the new page size and manage the page window geometry before |
| it becomes the top page.                                                     |
------------------------------------------------------------------------------*/
class ITabPageResizeHandler : public IResizeHandler
{
public:
  ITabPageResizeHandler ( ) { };
virtual
  ~ITabPageResizeHandler ( ) { };

protected:
bool
  windowResize ( IResizeEvent& event );
}; // ITabPageResizeHandler
#endif // IC_PM

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _ITABPRIV_ */
