// Revision: 89 1.8.1.7 source/ui/basectl/ititle.cpp, staticctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ititle.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ititle.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
   #define INCL_WINSWITCHLIST
   #define INCL_WINFRAMEMGR
   #define INCL_WINSYS
   #include <iwindefs.h>
   #include <string.h>
   #include <stdlib.h>
  }

#ifdef IC_MOTIF
  #include <Xm/MainW.h>
#endif //IC_MOTIF

#include <ititle.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <inotifev.hpp>
#include <iplatfrm.hpp>
#include <irect.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <iwindow.hpp>
#include <ithread.hpp>


// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ITITLE_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| titleBarHwnd                                                                 |
|  Returns FID_TITLEBAR HWND for owner.                                        |
|  Throws an exception if owner doesn't have FID_TITLEBAR.                     |
------------------------------------------------------------------------------*/
static IWindowHandle titleBarHwnd ( IWindow* owner )
{
#ifdef IC_PM
  IWindowHandle hwnd =
     IWindow::handleWithParent( FID_TITLEBAR, owner->handle() );
  if ( hwnd == 0 )
  {
     ITHROWLIBRARYERROR( IC_NO_TITLEBAR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }
  return hwnd;
#endif
#ifdef IC_WIN
  // No separate window handle in Windows.  We use this function to
  // validate that there is a title bar and throw if not.
  if ( ( ISTYLEOF( owner->handle() ) & WS_CAPTION ) != WS_CAPTION )
     ITHROWLIBRARYERROR( IC_NO_TITLEBAR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  return owner->handle();
#endif
}
#endif //IC_PMWIN


/*------------------------------------------------------------------------------
| ITitle::ITitle                                                               |
------------------------------------------------------------------------------*/
ITitle :: ITitle ( IWindow*      owner,
                   const char*   objectName,
                   const char*   viewName,
                   unsigned long viewNumber )
  : pwinClOwner( owner )
    ,fTitleData( 0 )
{
#ifdef IC_PM
  // Title bar not a window in Windows.
  // We override some of the IWindow and ITextControl functions in this class
  // because our handle is the same as the owning IFrameWindow handle.
  this->setAutoDestroyWindow( false );
  this->reserveUserWindowWord( false );
  // test that the title window is not already wrappered. We only support one
  // title wrapper
  IWindow *win = IWindow::windowWithHandle( titleBarHwnd( this->pwinClOwner ));
  IASSERTPARM( win == 0 );
  this->startHandlingEventsFor( titleBarHwnd( this->pwinClOwner ));
#endif

  if ( objectName )
  {
     this->setTitleText( objectName, viewName, viewNumber );
  }
}

/*------------------------------------------------------------------------------
| ITitle::ITitle                                                               |
------------------------------------------------------------------------------*/
ITitle :: ITitle ( IWindow*           owner,
                   const IResourceId& objectName,
                   const IResourceId& viewName,
                   unsigned long      viewNumber )
  : pwinClOwner( owner )
    ,fTitleData( 0 )
{
#ifdef   IC_PM
  // Title bar not a window in Windows.
  this->setAutoDestroyWindow( false );
  this->reserveUserWindowWord( false );
  // test that the title window is not already wrappered. We only support one
  // title wrapper
  IWindow *win = IWindow::windowWithHandle( titleBarHwnd( this->pwinClOwner ));
  IASSERTPARM( win == 0 );
  this->startHandlingEventsFor( titleBarHwnd( this->pwinClOwner ));
#endif   //IC_PM

  IString strObjName = objectName.resourceLibrary().loadString( objectName );

  IString strViewName;
  if ( viewName != 0 )
     strViewName = viewName.resourceLibrary().loadString( viewName );

  this->setTitleText( strObjName, strViewName, viewNumber );
}

/*------------------------------------------------------------------------------
| ITitle::~ITitle                                                              |
------------------------------------------------------------------------------*/
ITitle :: ~ITitle ( )
{ }

/*------------------------------------------------------------------------------
| ITitle::objectText                                                           |
------------------------------------------------------------------------------*/
IString ITitle :: objectText ( ) const
{
#ifdef IC_PMWIN
  IWindowHandle hwndTitleBar = titleBarHwnd( this->pwinClOwner );
  unsigned long uLen = IQUERYWINDOWTEXTLENGTH( hwndTitleBar );
  ++uLen;                                    // Add 1 for null terminator.

  IString strObj;
  if ( uLen > 1 )
  {
     CHAR* pszClBuf;                         // To store entire title string.
     pszClBuf = new CHAR[ uLen ];
     *pszClBuf = '\0';

     IQUERYWINDOWTEXT( this->pwinClOwner->handle(), uLen, (PSZ)pszClBuf );

     strObj = IString( pszClBuf );
     unsigned pos = strObj.indexOf(" - ");   // Locate " - ".
     if ( pos )
        strObj.remove( pos );

     delete [] pszClBuf;
  } /* endif */

  return strObj;
#endif //IC_PMWIN

#ifdef IC_MOTIF
   IString title = text();

   unsigned pos = title.indexOf("-");
   if (pos)
     title.remove(pos);

   return title;
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::viewText                                                             |
------------------------------------------------------------------------------*/
IString ITitle :: viewText ( ) const
{
#ifdef IC_PMWIN
  IWindowHandle hwndTitleBar = titleBarHwnd( this->pwinClOwner );

  unsigned long uLen = IQUERYWINDOWTEXTLENGTH( hwndTitleBar );
  ++uLen;                                    // Add 1 for null terminator.

  IString strView;

  if ( uLen > 1 )
  {
     char* pszClBuf = new char[ uLen ];

     IQUERYWINDOWTEXT( this->pwinClOwner->handle(), uLen, (PSZ)pszClBuf );

     IString strObj = IString( pszClBuf );
     unsigned pos = strObj.indexOf(" - ");   // Locate " - ".
     if ( pos )
     {
        strView = strObj.subString( pos + 3 ); // View name.
        pos = strView.lastIndexOf( ':' );      // Get ptr at last ":".
        if ( pos && ( strView.subString(pos+1).isDigits() ) )
        {
           // Remove : and numeric part.
           strView.remove( pos );
        }
     } /* endif */

     delete [] pszClBuf;
  } /* endif */

  return strView;
#endif //IC_PMWIN

#ifdef IC_MOTIF
   IString view = text();

   unsigned posDash = view.indexOf("-");
   unsigned posColon = view.lastIndexOf(":");

   if (posDash == 0)
     view = "";
   else
     if (posColon)
       view = view.subString(posDash+1, posColon - posDash - 1);
     else
       view = view.subString(posDash+1);

   return view;
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::viewNumber                                                           |
------------------------------------------------------------------------------*/
unsigned long ITitle :: viewNumber ( ) const
{
#ifdef IC_PMWIN
  unsigned long ulViewNum = 0;

  IWindowHandle hwndTitleBar = titleBarHwnd( this->pwinClOwner );
  unsigned long uLen = IQUERYWINDOWTEXTLENGTH( hwndTitleBar );
  ++uLen;                                    // Add 1 for null terminator.

  if ( uLen > 1 )
  {
     char* pszClBuf = new char[ uLen ];

     IQUERYWINDOWTEXT( this->pwinClOwner->handle(), uLen, (PSZ)pszClBuf );

     IString strObj = IString( pszClBuf );
     unsigned delimPos = strObj.indexOf(" - ");
     unsigned viewPos  = strObj.lastIndexOf( ':' );
     if ( (delimPos) &&                     // Found view delimiter.
          (viewPos > delimPos) &&           // Found view number.
          (viewPos < strObj.length()) )     // : not last char.
     {
        ulViewNum = strObj.subString( viewPos + 1 ).asUnsigned();
     }

     delete [] pszClBuf;
  } /* endif */

  return ulViewNum;
#endif //IC_PMWIN

#ifdef IC_MOTIF
  unsigned long num = 0;

  IString title= text();
  unsigned pos = title.lastIndexOf(":");
  if (pos) {
    title = title.subString(pos+1);
    num = title.asUnsigned();
  }
  return num;
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::setObjectText                                                        |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setObjectText ( const char* objectName )
{
  IString strObj = this->objectText();
  if ( objectName )
  {
     if (strcmp( (char*)strObj, objectName ))
     {
        this->setTitleText( objectName, this->viewText(), this->viewNumber() );
        this->notifyObservers( INotificationEvent( ITitle::objectTextId,
                                                   *this,
                                                   true,
                                                   (char*)objectName ));
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setObjectText                                                        |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setObjectText ( const IResourceId& objectNameResId )
{
  IString strObjName =
     objectNameResId.resourceLibrary().loadString( objectNameResId );
  if ( strObjName.length() )
     this->setObjectText( strObjName );
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setViewText                                                          |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setViewText ( const char* viewName )
{
  IString strView = this->viewText();

  if ( strcmp( (char*)strView, viewName ))
  {
     this->setTitleText( this->objectText(), viewName, this->viewNumber() );
     this->notifyObservers( INotificationEvent( ITitle::viewTextId,
                                                *this,
                                                true,
                                                (char*)viewName ));
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setViewText                                                          |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setViewText ( const IResourceId& viewNameResId )
{
  IString strViewName =
     (viewNameResId.resourceLibrary()).loadString( viewNameResId );
  if ( strViewName.length() )
     this->setViewText( strViewName );
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setViewNumber                                                        |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setViewNumber ( unsigned long viewNum )
{
  /****************************************/
  /* Only change and notify if different. */
  /****************************************/
  if ( viewNum != this->viewNumber() )
  {
     this->setTitleText( this->objectText(), this->viewText(), viewNum );
     this->notifyObservers( INotificationEvent( ITitle::viewNumberId,
                                                *this,
                                                true,
                                                (void*)viewNum ));
  }
  return *this;
}


/*------------------------------------------------------------------------------
| ITitle::setTitleText                                                         |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setTitleText ( const char*   objectName,
                                 const char*   viewName,
                                 unsigned long viewNum )
{
#ifdef IC_PMWIN
  IString strTitleText;

  // Check to insure a non-NULL string pointer was passed
  if ( objectName )
  {
     strTitleText = objectName;

     // If object name exists, then append view name and number
     if ( *objectName != '\0' )
     {
        if ( viewName && *viewName != '\0' )
           strTitleText += IString( " - " ) + IString( viewName );

        if ( viewNum )
           strTitleText += IString( ":" ) + IString( viewNum );
     }
  }
  this->setText( strTitleText );

#ifdef IC_PM
  // Update Task List or Window List.

  // Don't need to do anything in Windows since the task list or task bar is
  // automatically updated.

  HSWITCH hsw = WinQuerySwitchHandle( this->pwinClOwner->handle(), (PID)0 );
  if ( hsw )
  {
     SWCNTRL swctl;
     WinQuerySwitchEntry( hsw, &swctl );

     // Length of title cannot be greater than MAXNAMEL (60) + 1,
     // because SWCNTRL.szSwtitle is only that big.
     if ( strTitleText.length() + 1 > MAXNAMEL )
     {
       strTitleText = strTitleText.subString( 1, MAXNAMEL + 1 );
     }
     else if ( !strTitleText.length() )
       strTitleText = " ";

     strcpy( swctl.szSwtitle, strTitleText );
     WinChangeSwitchEntry( hsw, &swctl );
  } /* endif */
#endif  //IC_PM
#endif //IC_PMWIN

#ifdef IC_MOTIF
  IString strTitleText(" ");
  // Enforce CUA no title if no object text
  if (objectName && *objectName != '\0' ) {
     strTitleText = objectName;

     if (viewName && *viewName != '\0')
        strTitleText += IString(" - ") + IString(viewName);

     if (viewNum)
        strTitleText += IString(":") + IString(viewNum);
  }
      // Set the application shell title resource
      XtVaSetValues(XtParent(XtParent(pwinClOwner->handleForChildCreation())),
                    XmNtitle, (char*)strTitleText,
                    XmNiconName, (char*)strTitleText,
                    NULL);
      if (((char*)strTitleText != NULL) &&  (strlen((char*)strTitleText) != 0))
        XtVaSetValues(XtParent(XtParent(pwinClOwner->handleForChildCreation())),
                    XmNiconName, (char*)strTitleText,
                    NULL);
      else
        XtVaSetValues(XtParent(XtParent(pwinClOwner->handleForChildCreation())),
                    XmNiconName, XtName(IWindow::desktopWindow()->handle()),
                    NULL);
#endif //IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setTitleText                                                         |
------------------------------------------------------------------------------*/
ITitle& ITitle :: setTitleText ( const IResourceId& objectNameResId,
                                 const IResourceId& viewNameResId,
                                 unsigned long      viewNum )
{
  IString strObjName =
     objectNameResId.resourceLibrary().loadString( objectNameResId );

  if (strObjName.length())
  {
   IString strViewName;
   if ( viewNameResId )
     strViewName = viewNameResId.resourceLibrary().loadString( viewNameResId );
   else
     strViewName = "";

   // Enforcement of requiring an object name string will be done below
   this->setTitleText( strObjName, strViewName, viewNum );
  }
  return *this;
}

/*------------------------------------------------------------------------------
| WIN32 Color Support                                                          |
|                                                                              |
| Windows has no API equivalent for Presentation Parameters.  The following    |
| menu color functions are NOPed for Windows.  The query functions return      |
| an appropriate default IColor object.  The set and reset functions just      |
| return *this.                                                                |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| ITitle::borderColor                                                         |
|                                                                             |
| Returns the border color for the ITitle.                                    |
------------------------------------------------------------------------------*/
IColor ITitle::borderColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_BORDERCOLOR,
                         IGUIColor( IGUIColor::titleLine ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::titleLine );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::activeColor                                                         |
|                                                                             |
| Returns the active color for the ITitle.                                    |
------------------------------------------------------------------------------*/
IColor ITitle::activeColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_ACTIVECOLOR,
                         IGUIColor( IGUIColor::activeTitleBgnd ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::activeTitleBgnd );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::inactiveColor                                                       |
|                                                                             |
| Returns the inactive color for the ITitle.                                  |
------------------------------------------------------------------------------*/
IColor ITitle::inactiveColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_INACTIVECOLOR,
                         IGUIColor( IGUIColor::inactiveTitleBgnd ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::inactiveTitleBgnd );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::activeTextForegroundColor                                           |
|                                                                             |
| Returns the active text foreground color for the ITitle.                    |
------------------------------------------------------------------------------*/
IColor ITitle::activeTextForegroundColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_ACTIVETEXTFGNDCOLOR,
                         IGUIColor( IGUIColor::activeTitleText ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::activeTitleText );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::activeTextBackgroundColor                                           |
|                                                                             |
| Returns the active text background color for the ITitle.                    |
------------------------------------------------------------------------------*/
IColor ITitle::activeTextBackgroundColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_ACTIVETEXTBGNDCOLOR,
                         IGUIColor( IGUIColor::activeTitleTextBgnd ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::activeTitleTextBgnd );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::inactiveTextForegroundColor                                         |
|                                                                             |
| Returns the inactive text foreground color for the ITitle.                  |
------------------------------------------------------------------------------*/
IColor ITitle::inactiveTextForegroundColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_INACTIVETEXTFGNDCOLOR,
                         IGUIColor( IGUIColor::inactiveTitleText ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::inactiveTitleText );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::inactiveTextBackgroundColor                                         |
|                                                                             |
| Returns the inactive text background color for the ITitle.                  |
------------------------------------------------------------------------------*/
IColor ITitle::inactiveTextBackgroundColor ( ) const
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef IC_PM
  return IWindow::color( PP_INACTIVETEXTBGNDCOLOR,
                         IGUIColor( IGUIColor::inactiveTitleTextBgnd ));
#endif //IC_PM

#ifdef IC_WIN
  return IGUIColor( IGUIColor::inactiveTitleTextBgnd );
#endif //IC_WIN

#ifdef IC_MOTIF
  return ( IColor(0,0,0) );
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::setActiveTextForegroundColor                                        |
|                                                                             |
| Sets the active text foreground color for the ITitle.                       |
------------------------------------------------------------------------------*/
ITitle& ITitle::setActiveTextForegroundColor ( const IColor &color )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  IWindow::setColor( PP_ACTIVETEXTFGNDCOLOR, color );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setActiveTextBackgroundColor                                        |
|                                                                             |
| Sets the active text background color for the ITitle.                       |
------------------------------------------------------------------------------*/
ITitle& ITitle::setActiveTextBackgroundColor ( const IColor &color )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  IWindow::setColor( PP_ACTIVETEXTBGNDCOLOR, color );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setInactiveTextForegroundColor                                      |
|                                                                             |
| Sets the inactive text foreground color for the ITitle.                     |
------------------------------------------------------------------------------*/
ITitle& ITitle::setInactiveTextForegroundColor ( const IColor &color )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  IWindow::setColor( PP_INACTIVETEXTFGNDCOLOR, color );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setInactiveTextBackgroundColor                                      |
|                                                                             |
| Sets the inactive text background color for the ITitle.                     |
------------------------------------------------------------------------------*/
ITitle& ITitle::setInactiveTextBackgroundColor ( const IColor &color )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  IWindow::setColor( PP_INACTIVETEXTBGNDCOLOR, color );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::resetActiveTextForegroundColor                                      |
|                                                                             |
| Resets the active text foreground color by undoing a previous set.          |
------------------------------------------------------------------------------*/
ITitle& ITitle::resetActiveTextForegroundColor ( )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  this->resetColor( PP_ACTIVETEXTFGNDCOLOR );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::resetActiveTextBackgroundColor                                      |
|                                                                             |
| Resets the active text background color by undoing a previous set.          |
------------------------------------------------------------------------------*/
ITitle& ITitle::resetActiveTextBackgroundColor ( )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  this->resetColor( PP_ACTIVETEXTBGNDCOLOR );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::resetInactiveTextForegroundColor                                    |
|                                                                             |
| Resets the active text foreground color by undoing a previous set.          |
------------------------------------------------------------------------------*/
ITitle& ITitle::resetInactiveTextForegroundColor ( )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  this->resetColor( PP_INACTIVETEXTFGNDCOLOR );
#endif   //IC_PM
  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::resetInactiveTextBackgroundColor                                    |
|                                                                             |
| Resets the active text background color by undoing a previous set.          |
------------------------------------------------------------------------------*/
ITitle& ITitle::resetInactiveTextBackgroundColor ( )
{
ITRACE_MOTIF_NOP();
ITRACE_WIN_NOP();
#ifdef   IC_PM
  this->resetColor( PP_INACTIVETEXTBGNDCOLOR );
#endif   //IC_PM
  return *this;
}


#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| ITitle::setText                                                              |
|                                                                              |
| Set the title text.  Overridden since title is not a window in Windows.      |
| and to skip some of the tests done in ITextControl                           |
------------------------------------------------------------------------------*/
ITitle& ITitle::setText    ( const char* text )
{
#ifdef IC_WIN
  ISETWINDOWTEXT( titleBarHwnd( this->pwinClOwner ), text );
#endif //IC_WIN

#ifdef IC_MOTIF
  IString theText = (char *) text;
  setTitleText(theText, NULL, NULL);
#endif //IC_MOTIF

  return *this;
}

/*------------------------------------------------------------------------------
| ITitle::setText                                                              |
|                                                                              |
| Set the title text.  Overridden since title is not a window in Windows.      |
------------------------------------------------------------------------------*/
ITitle& ITitle::setText    ( const IResourceId& textResId )
{
   Inherited::setText( textResId );
   return *this;

}

/*------------------------------------------------------------------------------
| ITitle::text                                                                 |
|                                                                              |
| Get the title text.  Overridden since title is not a window in Windows.      |
------------------------------------------------------------------------------*/
IString ITitle::text( ) const
{
#ifdef IC_WIN
  unsigned long uLen = this->textLength();
  ++uLen;                                    // Add 1 for null terminator.

  IString result;
  if ( uLen > 1 )
  {
     char* pszClBuf = new char[ uLen ];

     IQUERYWINDOWTEXT( titleBarHwnd( this->pwinClOwner ),
                       uLen,
                       (PSZ)pszClBuf );
     result = pszClBuf;
     delete [] pszClBuf;
  }
  return result;
#endif //IC_WIN

#ifdef IC_MOTIF
   String data;
   XtVaGetValues(XtParent(XtParent(pwinClOwner->handleForChildCreation())),
                 XmNtitle, &data,
                 NULL);

   IString title((char *)data);
   return title;
#endif //IC_MOTIF
}

/*------------------------------------------------------------------------------
| ITitle::textLength                                                           |
|                                                                              |
| Get the title text length.  Overridden since title is not a window in Windows|
------------------------------------------------------------------------------*/
unsigned long ITitle::textLength ( ) const
{
#ifdef IC_WIN
  return IQUERYWINDOWTEXTLENGTH( titleBarHwnd( this->pwinClOwner ) );
#endif //IC_WIN

#ifdef IC_MOTIF
   return strlen( text() );
#endif //IC_MOTIF
}

#endif //IC_MOTIFWIN

#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| ITitle::owner                                                                |
|                                                                              |
| Return title owner window.  Overridden since title is not a window in Windows|
------------------------------------------------------------------------------*/
IWindow* ITitle::owner( ) const
{
   return pwinClOwner;
}

#endif //IC_MOTIFWIN

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITitle::isValid                                                              |
|                                                                              |
| True if owner has title.    Overridden since title is not a window in Windows|
------------------------------------------------------------------------------*/
bool ITitle::isValid( ) const
{
  if ( ( pwinClOwner ) &&
       ( ISTYLEOF( pwinClOwner->handle() ) & WS_CAPTION ) )
    return true;
  else
    return false;
}
#endif //IC_WIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ITitle::isValid                                                              |
|                                                                              |
| True if owner has title.    Overridden since title is not a window in Windows|
------------------------------------------------------------------------------*/
bool ITitle::isValid( ) const
{
  if  ( pwinClOwner ) 
    return true;
  else
    return false;
}
#endif //IC_MOTIF

#ifdef IC_MOTIFWIN
/*------------------------------------------------------------------------------
| ITitle::handle                                                               |
|                                                                              |
| Returns the handle of the owner window.   This is to allow font and          |
| similar operations to work correctly.                                        |
------------------------------------------------------------------------------*/
IWindowHandle ITitle::handle( ) const
{
   //Throw exception if window handle is not valid window.
   if ( !isValid() ) {
      ITHROWLIBRARYERROR( IC_INVALIDHANDLE,
                          IBaseErrorInfo::invalidRequest,
                          IException::recoverable );
   }
   return pwinClOwner->handle();
}
#endif //IC_MOTIFWIN
#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITitle::nativeRect                                                           |
|                                                                              |
| Returns the rectangle for the title bar.                                     |
------------------------------------------------------------------------------*/
IRectangle  ITitle::nativeRect                ( ) const
{
   // Reference : "How to Draw a Custom Window Caption" in Win32 SDK
   // Knowledge base.
   RECT rc;
   IWindowHandle hTitle = this->handle();
   GetWindowRect( hTitle, &rc );

   unsigned long style = ISTYLEOF( hTitle );
   // size of bitmaps on title bar
   ISize bmpSize( GetSystemMetrics(SM_CXSIZE),
                  GetSystemMetrics(SM_CYSIZE) );
   // size of window border
   ISize borderSize;

   if (style & WS_THICKFRAME )
      {
      // account for thick border
      borderSize = ISize(GetSystemMetrics(SM_CXSIZEFRAME),
                         GetSystemMetrics(SM_CYSIZEFRAME) );
      }
   else
      {
      if ( IPlatform::isWin9x() || IPlatform::isNTNewShell() )
        borderSize = ISize( GetSystemMetrics(SM_CXFIXEDFRAME),
                            GetSystemMetrics(SM_CYFIXEDFRAME) );
      else
        {
        borderSize = ISize( GetSystemMetrics(SM_CXBORDER),
                            GetSystemMetrics(SM_CYBORDER) );
        }
      }

   rc.left += borderSize.width() + 1;  // extra pel pad for title text
   if (style & WS_SYSMENU)
     rc.left += bmpSize.width();
   rc.top += borderSize.height();
   rc.right -= borderSize.width();
   if (style & WS_MAXIMIZEBOX)
     rc.right -= (bmpSize.width() + GetSystemMetrics(SM_CXBORDER));
   if (style & WS_MINIMIZEBOX)
     rc.right -= (bmpSize.width() + GetSystemMetrics(SM_CXBORDER));

   if ( (IPlatform::isWin9x()) || (IPlatform::isNTNewShell()) )
   {
      // Account for the close button on the title bar
      if (style & WS_SYSMENU)
         rc.right -= (bmpSize.width() + GetSystemMetrics(SM_CXBORDER));
      // Account for the "whats this?" button on the title bar
      if (GetWindowLong( hTitle, GWL_EXSTYLE ) & WS_EX_CONTEXTHELP)
         rc.right -= (bmpSize.width() + GetSystemMetrics(SM_CXBORDER));
   }
   rc.bottom = rc.top + bmpSize.height() - 1;

   // Map the results (now in screen coordinates) to coordinates
   // of the owner of the ITitle, whose handle is this->handle() since
   // it is not a separate window.
   MapWindowPoints( 0, hTitle, (POINT*)&rc, 2 );

   // Adjust mapping since map points uses client rect for mapping
   unsigned long addYDelta = 0;
   if ( !(style & WS_CHILD) && GetMenu( hTitle ) )
     addYDelta = GetSystemMetrics( SM_CYMENU );
   borderSize.setHeight( borderSize.height() + GetSystemMetrics(SM_CYCAPTION)
                         + addYDelta );
   rc.right  += borderSize.width();
   rc.left   += borderSize.width();
   rc.top    += borderSize.height();
   rc.bottom += borderSize.height();
   return IRectangle(rc);
}

/*------------------------------------------------------------------------------
| ITitle::parentSize                                                           |
|                                                                              |
| Returns the size of the window owning the title bar.                         |
------------------------------------------------------------------------------*/
ISize ITitle::parentSize ( ) const
{
   RECT rect;
   GetWindowRect(this->handle(), &rect);
   ITRACE_ALL( IString("parentSize=") +
               ISize(rect.right - rect.left, rect.bottom - rect.top ).asString() );
   return  ISize(rect.right - rect.left, rect.bottom - rect.top );
}

#endif //IC_WIN
