// Revision: 91 1.7.1.2 source/ui/basectl/ititle1.cpp, staticctls, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: ititle1.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ititlenh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481324 )

#define INCL_WINSYS
extern "C" {
   #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <ititle.hpp>
#include <ititlenh.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITITLE1_CPP_
  #include <ipagetun.h>
#endif


// Initialize static objects and notification strings.
INotificationId const ITitle::activeTextForegroundColorId =
                                         "ITitle::activeTextForegroundColor";
INotificationId const ITitle::activeTextBackgroundColorId =
                                         "ITitle::activeTextBackgroundColor";
INotificationId const ITitle::inactiveTextForegroundColorId =
                                         "ITitle::inactiveTextForegroundColor";
INotificationId const ITitle::inactiveTextBackgroundColorId =
                                         "ITitle::inactiveTextBackgroundColor";
INotificationId const ITitle::objectTextId = "ITitle::objectText";
INotificationId const ITitle::viewTextId   = "ITitle::viewText";
INotificationId const ITitle::viewNumberId = "ITitle::viewNumber";

/*------------------------------------------------------------------------------
| ITitle::enableNotification                                                   |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ITitle, does not send notifications to observer objects until it is          |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ITitle& ITitle::enableNotification (bool enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler( new ITitleNotifyHandler() );
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::ITitleNotifyHandler                                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITitleNotifyHandler::ITitleNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::~ITitleNotifyHandler                                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITitleNotifyHandler::~ITitleNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::dispatchHandlerEvent                                    |
|                                                                              |
| This notifies ITitle Windows of the following events:                        |
|                                                                              |
| activeTextForegroundColor(Id)                                                |
| activeTextBackgroundColor(Id)                                                |
| inactiveTextForegroundColor(Id)                                              |
| inactiveTextBackgroundColor(Id)                                              |
------------------------------------------------------------------------------*/
bool ITitleNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PM
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_PRESPARAMCHANGED:

        switch (anEvent.parameter1())
        {
          case PP_ACTIVETEXTFGNDCOLOR:
          case PP_ACTIVETEXTBGNDCOLOR:
          case PP_INACTIVETEXTFGNDCOLOR:
          case PP_INACTIVETEXTBGNDCOLOR:
            /**********************************************************/
            /* Post private message to delay notification until after */
            /* the selection state has been modified                  */
            /* Put color identifier into parameter 1 for later use    */
            /**********************************************************/
            thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                             IEventParameter1( anEvent.parameter1() ),
                             IEventParameter2( IC_NOTDEL_TITLE_COLOR ));
            break;

          default:
            break;
        }
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* is used to determine which notification id to send           */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_TITLE_COLOR)
        {
          switch (anEvent.parameter1())
          {
            case PP_ACTIVETEXTFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::activeTextForegroundColorId,
                                      *thisWindow));
              break;

            case PP_ACTIVETEXTBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::activeTextBackgroundColorId,
                                      *thisWindow));
              break;

            case PP_INACTIVETEXTFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::inactiveTextForegroundColorId,
                                      *thisWindow));
              break;

            case PP_INACTIVETEXTBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::inactiveTextBackgroundColorId,
                                      *thisWindow));
              break;
          }
        }
        break;

      default:
        break;
    }  /* end switch */
  }
#endif  // IC_PM
  return (Inherited::dispatchHandlerEvent(anEvent));
}
