// Revision: 95 1.8 source/ui/basectl/ivporhdr.hpp, canvas, ioc.v400, 000302 
/* NOSHIP */
#ifndef _IVPORHDR_
  #define _IVPORHDR_
/*******************************************************************************
* FILE NAME: ivporhdr.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IViewPortResizeHandler   - This class provides processing of window      *
*                                sizing events, sizing its scroll bars and     *
*                                view rectangle window appropriately, and      *
*                                positioning the view window appropriately.    *
*     IViewPortScrollHandler   - This class provides processing of scroll bar  *
*                                events, repositioning the view window         *
*                                approriately.                                 *
*     IViewPortKeyboardHandler - This class provides support for keyboard      *
*                                scrolling of the view window.                 *
*     IViewPortPaintHandler    - This class handles painting of the view port  *
*                                canvas window.                                *
*     IViewRectanglePaintHandler - This class handles painting of the view     *
*                                rectangle window.                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ievent.hpp>
#include <ikeyevt.hpp>
#include <ikeyhdr.hpp>
#include <ipainevt.hpp>
#include <ipainhdr.hpp>
#include <iscrlevt.hpp>
#include <iscrlhdr.hpp>
#include <isizeevt.hpp>
#include <isizehdr.hpp>


#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IWindow;
class IViewPort;
class IViewRectangle;


#pragma enum(4)
#pragma pack(push,4)

class IViewPortResizeHandler : public IResizeHandler {
typedef IResizeHandler
  Inherited;
/*******************************************************************************
* This class provides processing of resize events for a view port object, so   *
* that a view port can be redraw its scroll bars and client window when it is  *
* resized.                                                                     *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any arguments.                      |
------------------------------------------------------------------------------*/
  IViewPortResizeHandler ( );
virtual
 ~IViewPortResizeHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a view port object:                                                          |
|   handleEventsFor       - Attaches the handler to the specified view port    |
|                           object.                                            |
|   stopHandlingEventsFor - Detaches the handler from the specified view port  |
|                           object.                                            |
------------------------------------------------------------------------------*/
virtual IViewPortResizeHandler
 &handleEventsFor       ( IViewPort* viewport ),
 &stopHandlingEventsFor ( IViewPort* viewport );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   windowResize    - Resizes the view port window and adjusts the scroll bars |
|                     as necessary.                                            |
------------------------------------------------------------------------------*/
virtual bool
  windowResize          ( IResizeEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );
};  // class IViewPortResizeHandler


class IViewPortScrollHandler : public IScrollHandler {
typedef IScrollHandler
  Inherited;
/*******************************************************************************
* This class provides processing of scroll events for a view port object, so   *
* that a view port can reposition its client window (or allow the client       *
* window to redraw itself) when the user interacts with the scroll bars.       *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any arguments.                      |
------------------------------------------------------------------------------*/
  IViewPortScrollHandler ( );
virtual
 ~IViewPortScrollHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a view port object:                                                          |
|   handleEventsFor       - Attaches the handler to the specified view port    |
|                           object.                                            |
|   stopHandlingEventsFor - Detaches the handler from the specified view port  |
|                           object.                                            |
------------------------------------------------------------------------------*/
virtual IViewPortScrollHandler
 &handleEventsFor       ( IViewPort* viewport ),
 &stopHandlingEventsFor ( IViewPort* viewport );

protected:
/*------------------------ Scroll Handling Overrides ---------------------------
| This class overrides the following scroll handler virtual functions:         |
|   lineUp         - Scrolls the view window upward when the up scroll button  |
|                    of the vertical scroll bar is clicked.                    |
|   lineDown       - Scrolls the view window downward when the down scroll     |
|                    button of the vertical scroll bar is clicked.             |
|   lineLeft       - Scrolls the view window left when the left scroll button  |
|                    of the horizontal scroll bar is clicked.                  |
|   lineRight      - Scrolls the view window right when the right scroll       |
|                    button of the horizontal scroll bar is clicked.           |
|   pageUp         - Scrolls the view window up a page when the scroll shaft   |
|                    above the scroll box of the vertical scroll bar is        |
|                    clicked.                                                  |
|   pageDown       - Scrolls the view window down a page when the scroll shaft |
|                    below the scroll box of the vertical scroll bar is        |
|                    clicked.                                                  |
|   pageLeft       - Scrolls the view window left a page when the scroll shaft |
|                    to the left of the scroll box of the horizontal scroll    |
|                    bar is clicked.                                           |
|   pageRight      - Scrolls the view window right a page when the scroll      |
|                    shaft to the right of the scroll box of the horizontal    |
|                    scroll bar is clicked.                                    |
|   scrollBoxTrack - Scrolls the view window each time the position of the     |
|                    scroll box is changed.                                    |
------------------------------------------------------------------------------*/
virtual bool
  lineUp                ( IScrollEvent& event ),
  lineDown              ( IScrollEvent& event ),
  lineLeft              ( IScrollEvent& event ),
  lineRight             ( IScrollEvent& event ),
  pageUp                ( IScrollEvent& event ),
  pageDown              ( IScrollEvent& event ),
  pageLeft              ( IScrollEvent& event ),
  pageRight             ( IScrollEvent& event ),
  scrollBoxTrack        ( IScrollEvent& event );

/*------------------------------ Implementation --------------------------------
| These functions provide a common place to process various scroll events:     |
|   verticalScroll   - Processes vertical scroll events.                       |
|   horizontalScroll - Processes horizontal scroll events.                     |
------------------------------------------------------------------------------*/
virtual bool
  verticalScroll         ( IScrollEvent& event ),
  horizontalScroll       ( IScrollEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );

  IViewPortScrollHandler( const IViewPortScrollHandler &);
  IViewPortScrollHandler
    &operator=          ( const IViewPortScrollHandler &);
};  // class IViewPortScrollHandler


class IViewPortKeyboardHandler : public IKeyboardHandler {
typedef IKeyboardHandler
  Inherited;
/*******************************************************************************
* This class provides processing of keyboard events for a view port object, so *
* that a view port can be scrolled using the keyboard.                         *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any arguments.                      |
------------------------------------------------------------------------------*/
  IViewPortKeyboardHandler ( );
virtual
 ~IViewPortKeyboardHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a view port object:                                                          |
|   handleEventsFor       - Attaches the handler to the specified view port    |
|                           object.                                            |
|   stopHandlingEventsFor - Detaches the handler from the specified view port  |
|                           object.                                            |
------------------------------------------------------------------------------*/
virtual IViewPortKeyboardHandler
 &handleEventsFor       ( IViewPort* viewport ),
 &stopHandlingEventsFor ( IViewPort* viewport );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   virtualKeyPress - Processes keys scrolling the view port client window.    |
------------------------------------------------------------------------------*/
virtual bool
  virtualKeyPress       ( IKeyboardEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );
};  // class IViewPortKeyboardHandler


#ifdef IC_PM
class IViewPortPaintHandler : public IPaintHandler {
typedef IPaintHandler
  Inherited;
/*******************************************************************************
* This class provides processing of paint events for a view port object.       *
* Since a view port potentially does not need any painting (the view rectangle *
* and scroll bars together consume all of the view port window), it does       *
* nothing (in order to optimize painting and minimize screen flicker).         *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any arguments.                      |
------------------------------------------------------------------------------*/
  IViewPortPaintHandler ( );
virtual
 ~IViewPortPaintHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a view port object:                                                          |
|   handleEventsFor       - Attaches the handler to the specified view port    |
|                           object.                                            |
|   stopHandlingEventsFor - Detaches the handler from the specified view port  |
|                           object.                                            |
------------------------------------------------------------------------------*/
virtual IViewPortPaintHandler
 &handleEventsFor       ( IViewPort* viewport ),
 &stopHandlingEventsFor ( IViewPort* viewport );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   paint - Processes painting of the view port canvas itself.                 |
------------------------------------------------------------------------------*/
virtual bool
  paintWindow           ( IPaintEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );
};  // class IViewPortPaintHandler


class IViewRectanglePaintHandler : public IPaintHandler {
typedef IPaintHandler
  Inherited;
/*******************************************************************************
* This class provides processing of paint events for the view rectangle        *
* object.  The view rectangle will only paint the area not occupied by the     *
* child view window, if appropriate (otherwise it will paint its entire        *
* rectangle, for compatibility reasons).                                       *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any arguments.                      |
------------------------------------------------------------------------------*/
  IViewRectanglePaintHandler ( );
virtual
 ~IViewRectanglePaintHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a view port object:                                                          |
|   handleEventsFor       - Attaches the handler to the specified view         |
|                           rectangle object.                                  |
|   stopHandlingEventsFor - Detaches the handler from the specified view       |
|                           rectangle object.                                  |
------------------------------------------------------------------------------*/
virtual IViewRectanglePaintHandler
 &handleEventsFor       ( IViewRectangle* viewRect ),
 &stopHandlingEventsFor ( IViewRectangle* viewRect );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   paint - Processes painting of the view rectangle canvas.                   |
------------------------------------------------------------------------------*/
virtual bool
  paintWindow           ( IPaintEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );
};  // class IViewRectanglePaintHandler
#endif // IC_PM


#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IVPORHDR_ */
