/* Revision: 03 1.2.2.2 source/ui/cclnt/cnr/cnrdlg.c, cclcnr, ioc.v400, 000302  */
/*---------------------------------------------------------------------------*
 *
 *                              IBM CONFIDENTIAL
 *
 *            (IBM Confidential-Restricted when combined with the
 *              aggregated OCO source modules for this program)
 *
 *                            OCO SOURCE MATERIALS
 *
 *             20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)
 *
 *---------------------------------------------------------------------------*
 * cnrdlg.c
 *
 * Container interface routines for the Windows SDK Dialog Editor.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

#include <windows.h>
#include <custcntl.h>
#include <string.h>
#include "dlgedit.h"
#include "cnr2win.h"
#include "iclcnrw.h"
#include "cnrdlg.h"

#ifdef   SEGMENTOR
#define  CNRDLG_C
#include "cnrpragw.h"
#endif

HANDLE FAR PASCAL CnrInfo  (void);
BOOL   FAR PASCAL CnrStyle (HWND, HANDLE, LPFNSTRTOID, LPFNIDTOSTR);
WORD   FAR PASCAL CnrFlags (WORD, LPSTR, WORD);
BOOL   FAR PASCAL CnrDlgFn (HWND, WORD, WORD, DWORD);

void   EnableMajors (HWND, WORD, WORD, WORD, WORD, WORD);
WORD   QueryStyles  (HWND);
void   SetStyles    (HWND, DWORD);

extern HANDLE vhCNRInstance;


/*---------------------------------------------------------------------------*
 * CnrInfo
 *
 * Allocates a CTLINFO structure from the global heap to identify the
 * controls that are supported by this DLL.  The caller of this function
 * is responsible for freeing the structure.
 *
 * This function must be ordinal 2.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

HANDLE FAR PASCAL CnrInfo (void)
{
  HANDLE    hCtlInfo;
  LPCTLINFO lpCtlInfo;

  /*-----------------------------------------------------*/
  /* allocate a CTLINFO that the Dialog Editor will free */
  /*-----------------------------------------------------*/
  if (hCtlInfo = GlobalAlloc (GHND, sizeof (CTLINFO)))
  {
    /*-----------------------------------*/
    /* lock it down so we can address it */
    /*-----------------------------------*/
    if (lpCtlInfo = (LPCTLINFO) GlobalLock (hCtlInfo))
    {
      lpCtlInfo->wVersion        = CCL_VERSION;
      lpCtlInfo->wCtlTypes       = 1;

      lpCtlInfo->Type[0].wType   = 0;
      lpCtlInfo->Type[0].wWidth  = 75;
      lpCtlInfo->Type[0].wHeight = 50;
      lpCtlInfo->Type[0].dwStyle = WS_CHILD | WS_CLIPCHILDREN |
                                   WS_CLIPSIBLINGS | CCS_SINGLESEL;

      lstrcpy (lpCtlInfo->szClass, ICL_CONTAINER);
      lstrcpy (lpCtlInfo->szTitle, CCL_COPYRIGHT);
      lstrcpy (lpCtlInfo->Type[0].szDescr, ICL_CONTAINER);

      GlobalUnlock (hCtlInfo);
    }

    /*---------------------------------------------------------------*/
    /* otherwise, we couldn't lock it down to initialize it, so punt */
    /*---------------------------------------------------------------*/
    else
    {
      GlobalFree (hCtlInfo);
      hCtlInfo = NULL;
    }
  }

  return (hCtlInfo);
}


/*---------------------------------------------------------------------------*
 * CnrStyle
 *
 * This function displays a modal dialog that is to collect the style of
 * the container from the user.
 *
 * This function must be ordinal 3.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

BOOL FAR PASCAL CnrStyle (HWND hwnd, HANDLE hCtlStyle,
                          LPFNSTRTOID lpfnStrToId, LPFNIDTOSTR lpfnIdToStr)
{
  PSTYLESTRUCT pss;
  BOOL         rc;

  if (!(pss = (PSTYLESTRUCT) LocalAlloc (LPTR, sizeof (STYLESTRUCT))))
    return (FALSE);

  pss->hCtlStyle   = hCtlStyle;
  pss->lpCtlStyle  = NULL;
  pss->lpfnStrToId = lpfnStrToId;
  pss->lpfnIdToStr = lpfnIdToStr;

  rc = DialogBoxParam (vhCNRInstance, MAKEINTRESOURCE (ID_CNRSTYLES), hwnd,
                       (FARPROC) CnrDlgFn, (WORD) pss);

  LocalFree ((HANDLE) pss);

  return (rc);
}


/*---------------------------------------------------------------------------*
 * CnrDlgFn
 *
 * This function must be ordinal 6.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

BOOL FAR PASCAL CnrDlgFn (HWND hwnd, WORD msg, WORD wParam, DWORD lParam)
{
  PSTYLESTRUCT pss;
  DWORD        dwRet;
  char         szID[33];

  switch (msg)
  {
    case WM_INITDIALOG:
      pss = (PSTYLESTRUCT) lParam;

      /*----------------------------------------------------------------*/
      /* set the property that contains the pointer to the STYLESTRUCT, */
      /* lock down the CTLSTYLE structure, and make sure this is really */
      /* a container; if any of these failed, bail out now              */
      /*----------------------------------------------------------------*/
      if (!SetProp (hwnd, MAKEINTRESOURCE (1), (HANDLE) pss) ||
          !(pss->lpCtlStyle = (LPCTLSTYLE) GlobalLock (pss->hCtlStyle)))
      {
        EndDialog (hwnd, FALSE);
        break;
      }

      /*-------------------------*/
      /* initialize the ID field */
      /*-------------------------*/
      if ((*pss->lpfnIdToStr) (pss->lpCtlStyle->wId, szID, sizeof (szID)))
      {
        SetDlgItemText (hwnd, ID_ID, szID);
      }
      else
      {
        SetDlgItemInt (hwnd, ID_ID, pss->lpCtlStyle->wId, FALSE);
      }

      /*-------------------------------------*/
      /* set the text limit for the ID field */
      /*-------------------------------------*/
      SendDlgItemMessage (hwnd, ID_ID, EM_LIMITTEXT, sizeof (szID)-1, 0);

      /*---------------------------------*/
      /* check the default style buttons */
      /*---------------------------------*/
      SetStyles (hwnd, pss->lpCtlStyle->dwStyle);
      break;

    case WM_DESTROY:
      if (pss = (PSTYLESTRUCT) RemoveProp (hwnd, MAKEINTRESOURCE (1)))
      {
        if (pss->lpCtlStyle)  GlobalUnlock (pss->hCtlStyle);
      }
      break;

    case WM_COMMAND:
      switch (wParam)
      {
        case IDOK:
          pss = (PSTYLESTRUCT) GetProp (hwnd, MAKEINTRESOURCE (1));

          /*----------------------------------------------*/
          /* if the ID field was modified, get the new ID */
          /*----------------------------------------------*/
          if (SendDlgItemMessage (hwnd, ID_ID, EM_GETMODIFY, 0, 0))
          {
            GetDlgItemText (hwnd, ID_ID, szID, sizeof (szID));

            /*---------------------------------------------------*/
            /* if the new ID is not a defined constant, bail out */
            /*---------------------------------------------------*/
            if (!LOWORD (dwRet = (*pss->lpfnStrToId) (szID)))  break;

            pss->lpCtlStyle->wId = HIWORD (dwRet);
          }

          LOWORD (pss->lpCtlStyle->dwStyle) = QueryStyles (hwnd);
          /* flow-through */

        case IDCANCEL:
          EndDialog (hwnd, (wParam == IDOK));
          break;
      }
      break;
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * QueryStyles
 *
 * Queries the styles selected from the dialog box.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

WORD QueryStyles (HWND hwnd)
{
  DWORD flStyle;

  flStyle  = IsDlgButtonChecked (hwnd, ID_SINGLESEL)   ? CCS_SINGLESEL   :
             IsDlgButtonChecked (hwnd, ID_MULTIPLESEL) ? CCS_MULTIPLESEL :
                                                         CCS_EXTENDSEL;

  if (IsDlgButtonChecked (hwnd, ID_AUTOPOSITION))    flStyle |= CCS_AUTOPOSITION;
  if (IsDlgButtonChecked (hwnd, ID_MINIRECORDCORE))  flStyle |= CCS_MINIRECORDCORE;
  if (IsDlgButtonChecked (hwnd, ID_READONLY))        flStyle |= CCS_READONLY;
  if (IsDlgButtonChecked (hwnd, ID_VERIFYPOINTERS))  flStyle |= CCS_VERIFYPOINTERS;

  return (LOWORD (flStyle));
}


/*---------------------------------------------------------------------------*
 * SetStyles
 *
 * Sets the styles for the dialog box from what was passed in by the
 * Dialog Editor
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

void SetStyles (HWND hwnd, DWORD flStyle)
{
  CheckDlgButton (hwnd, (flStyle & CCS_SINGLESEL)   ? ID_SINGLESEL   :
                        (flStyle & CCS_MULTIPLESEL) ? ID_MULTIPLESEL :
                                                      ID_EXTENDSEL,    TRUE);

  if (flStyle & CCS_AUTOPOSITION)   CheckDlgButton (hwnd, ID_AUTOPOSITION,   TRUE);
  if (flStyle & CCS_MINIRECORDCORE) CheckDlgButton (hwnd, ID_MINIRECORDCORE, TRUE);
  if (flStyle & CCS_READONLY)       CheckDlgButton (hwnd, ID_READONLY,       TRUE);
  if (flStyle & CCS_VERIFYPOINTERS) CheckDlgButton (hwnd, ID_VERIFYPOINTERS, TRUE);
}


/*---------------------------------------------------------------------------*
 * CnrFlags
 *
 * This function must be ordinal 4.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

WORD FAR PASCAL CnrFlags (WORD fs, LPSTR lpStyle, WORD cbMax)
{
  char  szStyles[196];

  /*-------------------------*/
  /* format the style string */
  /*-------------------------*/
  strcpy (szStyles, (fs & CCS_SINGLESEL)   ? "CCS_SINGLESEL"   :
                    (fs & CCS_MULTIPLESEL) ? "CCS_MULTIPLESEL" :
                                             "CCS_EXTENDSEL");

  if (fs & CCS_AUTOPOSITION)   strcat (szStyles, " | CCS_AUTOPOSITION");
  if (fs & CCS_MINIRECORDCORE) strcat (szStyles, " | CCS_MINIRECORDCORE");
  if (fs & CCS_READONLY)       strcat (szStyles, " | CCS_READONLY");
  if (fs & CCS_VERIFYPOINTERS) strcat (szStyles, " | CCS_VERIFYPOINTERS");

//strcat (szStyles, "| WS_CLIPCHILDREN | WS_CLIPSIBLINGS");

  /*-------------------------------------------------------------*/
  /* copy the style string we build into DlgEdit's buffer,       */
  /* make sure it's null-terminated, and return the bytes copied */
  /*-------------------------------------------------------------*/
  strncpy (lpStyle, szStyles, cbMax);
  lpStyle[cbMax-1] = '\0';
  return (lstrlen (lpStyle));
}
