/* Revision: 07 1.3.1.2 source/ui/cclnt/cnr/cnrs1p1.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS1P1.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS1P1.C - Details view, column & NLS routines for the  */
/*                       container control                            */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* D6386  1/16/96  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* DMS1   2/12/96  DMS  Defect 6685 - Paint outside drawable area     */
/*                      causes entire refresh.                        */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/* DAB2   8/01/96  DAB  Defect 8172 - Fixed CM_PAINTBACKGROUND parms  */
/* D7144  8/06/96  LJO  Fix null popup menu object over title.        */
/**********************************************************************/
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */     //D6386
#include <stdlib.h>
#include <string.h>
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef IC_WU
#include <stdio.h>      /* To get sprintf() */                  /*RKK1*/
#endif

#ifdef SEGMENTOR
#define CNRS1P1_C
#include "cnrpragw.h"
#endif

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrDetailsVCtlProc                                */
/*                                                                    */
/*  Descriptive Name:  Details view control procedure for the         */
/*                     container control                              */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LRESULT CALLBACK CnrDetailsVCtlProc( HWND hwndCnr, UINT msg,            /*DAB1*/
                                     WPARAM wParam, LPARAM lParam )     /*DAB1*/
{
 LPCNRINFON pccinfon;
 LPCNRINFO  pccinfo;
 HDC        hps;
 RECT       Rect;
 PAINTSTRUCT paintStruct;

 /********************************************************************/
 /* Get ptr to internal CNRINFO control data.  The pointer to the    */
 /* external control data is beyond the pointer to the internal      */
 /* control data.                                                    */
 /********************************************************************/
 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

   switch (msg)
   {
      case WM_CREATE:
        return FALSE;

      case WM_PAINT:
         hps = BeginPaint (hwndCnr, (LPPAINTSTRUCT)&paintStruct);

         PMFlipDC(hwndCnr, hps);

         PMGetClientRect (hwndCnr, &Rect);
         pccinfon->ulFlagSet |= FL_CNRWINOPEN;

         if (!(IsIconic(hwndCnr)))
         {
           pccinfon->ulFlagSet |= FL_CNRWINOPEN;
           pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;

           /*************************************************************/
           /* If this flag is not set, it means we have to resize the   */
           /* Container title window.                                   */
           /*************************************************************/
           if (!(pccinfon->ulFlagSet & FL_SETCNRTITLE))
           {
              /**********************************************************/
              /* The FL_SETCNRTITLE flag is set off in the              */
              /* CnrSizeCnrTitle code.                                  */
              /**********************************************************/
              CnrSizeCnrTitle (hwndCnr, hps, Rect);

              pccinfon->ulFlagSet |= FL_SETCNRTITLE;
              pccinfon->ulFlagSet &= ~(FL_SIZECOLTITLES | FL_SIZECHILDREN);
           }

           if (!(pccinfon->ulFlagSet & FL_POSITIONCOLTITLES))
           {
              if (pccinfo->flWindowAttr & CA_DETAILSVIEWTITLES)
              {
                CnrPositionColumnTitles (hwndCnr, hps);
                pccinfon->ulFlagSet |= FL_POSITIONCOLTITLES;
                pccinfon->ulFlagSet &= ~(FL_SIZECOLTITLES | FL_CNRPOSITION);
              }
           }

           /*************************************************************/
           /* If this flag is not set, we have to resize the column     */
           /* titles.                                                   */
           /*************************************************************/
           if (!(pccinfon->ulFlagSet & FL_SIZECOLTITLES))
           {
              CnrSizeColumnTitleWindows (hwndCnr, Rect.right, Rect.bottom);

              /*******************************************************/
              /* Set the flag so it is off.  I know it's backwards,  */
              /* I hate it too, but I had no choice...I had to       */
              /* follow the convention with the rest of the code.    */
              /*******************************************************/
              pccinfon->ulFlagSet |= FL_SIZECOLTITLES;
              pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
           }

           if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
           {
              CnrSize (hwndCnr, Rect.right, Rect.bottom);
              pccinfon->ulFlagSet |= FL_SIZECHILDREN;
              pccinfon->ulFlagSet &= ~FL_SETSCROLL;
           }

           if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
           {
              if (!(CnrPositionDetailsView( GetDlgItem (hwndCnr,
                                            CID_LEFTDVWND), hps,
                                            NULL)))
              {
                 EndPaint (hwndCnr, (LPPAINTSTRUCT)&paintStruct);
                 return ((long)NULL);
              }

              pccinfon->ulFlagSet |= FL_CNRPOSITION;
              pccinfon->ulFlagSet &= ~FL_SETSCROLL;
           }

           /*************************************************************/
           /* Set up the Scroll bars in the correct positions.          */
           /*************************************************************/
           if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
           {
              CnrSetDetailsViewScroll (hwndCnr);
              pccinfon->ulFlagSet |= FL_SETSCROLL;
           }

           /*************************************************************/
           /* If Details View is split into 2 windows, we have to paint */
           /* the splitbar.  No problem.                                */
           /*************************************************************/
           if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
           {
              CnrPaintSplitbar (hwndCnr, hps);
           }

           if (pccinfon->ulFlagSet2 & FL2_WINDOWSIZED)
           {
              EnumChildWindows (hwndCnr, CnrEnumWindowsProc, 0);        /*DAB1*/
              pccinfon->ulFlagSet2 &= ~FL2_WINDOWSIZED;                 /*DMS1*/
           }
         }

         pccinfon->ulFlagSet2 |= FL2_INITPAINT;
         EndPaint (hwndCnr, (LPPAINTSTRUCT)&paintStruct);
         return 0;

      case WM_VSCROLL:

         CnrVScrollDetailsView (hwndCnr, (HWND)lParam,                  /*DAB1*/
                               (int)HIWORD(wParam),                     /*DAB1*/
                               (UINT)LOWORD(wParam));                   /*DAB1*/
         return 0;
      break;

      case WM_HSCROLL:
         CnrHScrollDetailsView (hwndCnr, (HWND)lParam,                  /*DAB1*/
                               (int)HIWORD(wParam),                     /*DAB1*/
                               (UINT)LOWORD(wParam));                   /*DAB1*/
         return 0;
      break;

      case WM_SIZE:
         pccinfon->ulFlagSet2 |= FL2_WINDOWSIZED;
         /*************************************************************/
         /* If the size of the container changed, then we know we     */
         /* have to resize the left and right details windows, and    */
         /* their scroll bars.                                        */
         /*************************************************************/
         pccinfon->ulFlagSet &= ~(FL_SIZECHILDREN | FL_SETSCROLL);

         if (IsIconic(hwndCnr))
         {
           pccinfon->ulFlagSet2 |= FL2_MINIMIZED;
           pccinfon->ulFlagSet &= ~FL_CNRWINOPEN;
         }
         else
         {
           if (pccinfon->ulFlagSet2 & FL2_MINIMIZED)
           {
             pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;
             pccinfon->ulFlagSet &= ~(FL_CNRWINOPEN | FL_CNRPOSITION |
                                 FL_SETSCROLL | FL_NEWVIEWSCROLLSET |
                                 FL_SETCNRTITLE);
           }
         }

         if (pccinfon->ulFlagSet & FL_CNRWINOPEN)
         {
            /**********************************************************/
            /* If the horizontal size of the container window changed,*/
            /* then we also have to size the Cnr Title window and the */
            /* column title windows (if they exist).                  */
            /**********************************************************/
            pccinfon->ulFlagSet &= ~(FL_SETCNRTITLE | FL_SIZECOLTITLES );
         }
         else
         {
           pccinfon->ulFlagSet |= FL_CNRWINOPEN;
         }

         /**********************************************************/
         /* If the window is unsplit, or the splitbar is not       */
         /* visible, we need to explicitly send the WM_PAINT       */
         /* message to the Details View control proc.              */
         /**********************************************************/
         if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
             ((WORD)pccinfo->xVertSplitbar > LOWORD(lParam)))
         {
            CnrDetailsVCtlProc (hwndCnr, WM_PAINT, 0, 0);               /*DAB1*/
         }

         InvalidateChildren (hwndCnr, FALSE, TRUE);
      return 0;

    default:
      return (DefWindowProc (hwndCnr, msg, wParam, lParam));
   }
   return 0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LRESULT CALLBACK CnrLeftColumnTitlesProc( HWND hwnd, UINT msg,          /*DAB1*/
                                          WPARAM wParam, LPARAM lParam )/*DAB1*/
{
 LPCNRINFON         pccinfon;
 LPCNRINFO          pccinfo;
 HDC                hps;
 RECT               rctlInvalid;
 HANDLE             hwndCnr;
 PAINTSTRUCT        paintStruct;
 NOTIFYRECORDENTER  nRecordEnter;
 OWNERBACKGROUND    OwnerBack;
 BOOL               OwnerPaintBG = FALSE;

 hwndCnr = PARENTOF (hwnd);
 pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 switch (msg)
 {
    case WM_CREATE:
      return FALSE;

    case WM_PAINT:
       /***************************************************************/
       /* Get pointers to control data.                               */
       /***************************************************************/
       hps = BeginPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);

       PMFlipDC(hwnd, hps);
       rctlInvalid = paintStruct.rcPaint;
       PMFlipRect((LPRECT)&rctlInvalid);

       if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
       {
          OwnerBack.hwnd = hwnd;
          OwnerBack.hps  = hps;
          OwnerBack.rclBackground = paintStruct.rcPaint;
          OwnerBack.idWindow = getWindowId( GetDlgItem( hwndCnr,      /*DAB1*/
                                              CID_LEFTCOLTITLEWND));  /*DAB1*/

          OwnerPaintBG = LOWORD( SENDMESSAGE( hwndCnr,
                                              CM_PAINTBACKGROUND,       /*DAB1*/
                           (DWORD)((LPOWNERBACKGROUND)&OwnerBack), 0 ));/*DAB2*/
       }

       /***************************************************************/
       /* If the application didn't paint the background, then do it  */
       /***************************************************************/
       if (!OwnerPaintBG)
       {
         HBRUSH hBrush;

         /*************************************************************/
         /* Paint the background with the current setting             */
         /*************************************************************/
         if (hBrush = CreateSolidBrush (
                        pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
         {
           PMFillRect( hps, &paintStruct.rcPaint, hBrush);
           DeleteObject( hBrush );
         }
       }

       /***************************************************************/
       /* Draw the column titles in the left column title window.     */
       /***************************************************************/
       if (!(CnrDrawColumnTitlesLeft (hwnd, hps, paintStruct.rcPaint)))
       {
          EndPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);
          return ((long)NULL);
       }

       EndPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);
    break;

    case WM_LBUTTONDOWN:
       SetFocus (hwndCnr);

       /***************************************************************/
       /* Get pointers to control data.                               */
       /***************************************************************/
       pccinfon->ulFlagSet2 |= FL2_FOCUSLEFT;

       if (pccinfon->ulFlagSet & FL_EDITMODE)
       {
          if (GetKeyState (VK_MENU) < 0)
          {
             CnrCloseMLE (hwndCnr);
             return ((long)TRUE);
          }
          else
          {
             if (!(CnrSaveTextCloseMLE (hwndCnr)))
             {
                return ((long)NULL);
             }
          }
       }
       else
       {
         if (GetKeyState (VK_MENU) < 0)
         {
           /***********************************************************/
           /* Open Multi Line Entry Field to edit the title text by   */
           /* calling CnrOpenColTitleTextMLE                          */
           /***********************************************************/
           CnrOpenColTitleTextMLE (hwndCnr, LOWORD(lParam),
                                   HIWORD(lParam), TRUE);
         }
       }

       return 0;
    break;

    case WM_MOUSEMOVE:
      return ((long)CnrGateProc (hwndCnr, msg, wParam, lParam));

    case WM_MBUTTONDBLCLK:
    case WM_RBUTTONDBLCLK:
       pccinfon->ulFlagSet2 |= FL2_FOCUSLEFT;
       SetFocus (hwndCnr);
    break;

    case WM_LBUTTONDBLCLK:
       nRecordEnter.pRecord = NULL;
       nRecordEnter.hwndCnr = hwndCnr;
       nRecordEnter.fKey = FALSE;
       NotifyOwner (hwndCnr, CN_ENTER, (LPVOID)&nRecordEnter);
       break;

    case WM_RBUTTONUP:                                              /* D7144 */
       break;

    default:
       return (DefWindowProc (hwnd, msg, wParam, lParam));
 }
 return 0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LRESULT CALLBACK CnrRightColumnTitlesProc( HWND hwnd, UINT msg,         /*DAB1*/
                                          WPARAM wParam, LPARAM lParam )/*DAB1*/
{
 LPCNRINFON         pccinfon;
 LPCNRINFO          pccinfo;
 HDC                hps;
 HANDLE             hwndCnr;
 PAINTSTRUCT        paintStruct;
 NOTIFYRECORDENTER  nRecordEnter;
 OWNERBACKGROUND    OwnerBack;
 BOOL               OwnerPaintBG = FALSE;

 hwndCnr = PARENTOF (hwnd);
 pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 switch (msg)
 {
    case WM_CREATE:
      return FALSE;

    case WM_PAINT:
       /***************************************************************/
       /* Get pointers to control data.                               */
       /***************************************************************/
       hps = BeginPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);

       PMFlipDC(hwnd, hps);
       PMFlipRect((LPRECT)&(paintStruct.rcPaint));

       if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
       {
          OwnerBack.hwnd = hwnd;
          OwnerBack.hps  = hps;
          OwnerBack.rclBackground = paintStruct.rcPaint;
          OwnerBack.idWindow = getWindowId( GetDlgItem( hwndCnr,      /*DAB1*/
                                              CID_RIGHTCOLTITLEWND)); /*DAB1*/

          OwnerPaintBG = LOWORD( SENDMESSAGE( hwndCnr,
                                              CM_PAINTBACKGROUND,       /*DAB1*/
                           (DWORD)((LPOWNERBACKGROUND)&OwnerBack), 0 ));/*DAB2*/
       }

       /***************************************************************/
       /* If the application didn't paint the background, then do it  */
       /***************************************************************/
       if (!OwnerPaintBG)
       {
         HBRUSH hBrush;

         /***************************************************************/
         /* Paint the background with the current setting               */
         /***************************************************************/
         if (hBrush = CreateSolidBrush (
                     pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
         {
           PMFillRect (hps, &paintStruct.rcPaint, hBrush);
           DeleteObject(hBrush);
         }
       }

       /***************************************************************/
       /* Draw the column titles in the right column title window.    */
       /***************************************************************/
       if (!(CnrDrawColumnTitlesRight (hwnd, hps, paintStruct.rcPaint)))
       {
          EndPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);
          return ((long)NULL);
       }

       EndPaint (hwnd, (LPPAINTSTRUCT)&paintStruct);
    break;

    case WM_LBUTTONDOWN:
       SetFocus (hwndCnr);

       /***************************************************************/
       /* Get pointers to control data.                               */
       /***************************************************************/
       pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;

       if (pccinfon->ulFlagSet & FL_EDITMODE)
       {
          if (GetKeyState (VK_MENU) < 0)
          {
             CnrCloseMLE (hwndCnr);
             return ((long)TRUE);
          }
          else
          {
             if (!(CnrSaveTextCloseMLE (hwndCnr)))
             {
                return ((long)NULL);
             }
          }
       }
       else
       {
          if (GetKeyState (VK_MENU) < 0)
          {
             if (CnrVerifyFieldInfoPtrs (hwndCnr,
                                         pccinfon->pFieldInfoLast,
                                         1))
             {
               /******************************************************/
               /* Open Multi Line Entry Field to edit the title text */
               /* by calling CnrOpenColTitleTextMLE                  */
               /******************************************************/
               CnrOpenColTitleTextMLE( hwndCnr, LOWORD(lParam),
                                       HIWORD(lParam), FALSE );
             }
          }
       }

       return 0;
    break;

    case WM_MOUSEMOVE:
      return ((long)CnrGateProc (hwndCnr, msg, wParam, lParam));

    case WM_MBUTTONDBLCLK:
    case WM_RBUTTONDBLCLK:
       pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;
       SetFocus (hwndCnr);
    break;

    case WM_LBUTTONDBLCLK:
       nRecordEnter.pRecord = NULL;
       nRecordEnter.hwndCnr = hwndCnr;
       nRecordEnter.fKey = FALSE;
       NotifyOwner (hwndCnr, CN_ENTER, (LPVOID)&nRecordEnter);
       break;

    case WM_RBUTTONUP:                                              /* D7144 */
       break;

    default:
       return (DefWindowProc (hwnd, msg, wParam, lParam));
 }
 return 0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrAllColumnsFixedWidth                           */
/*                                                                    */
/*  Descriptive Name:  Are all columns in Details View Fixed Width?   */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will loop through all the inserted columns for      */
/*  Details View and check to see if all of them are a fixed width,   */
/*  ie. specifically set by the application.  If any of them are not  */
/*  a fixed width, this function returns FALSE.                       */
/*                                                                    */
/*  Notes:  N/A                                                       */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*  Output:  TRUE  - All the columns are a fixed width.               */
/*           FALSE - All the columns are not a fixed width.  Some are */
/*                   autosize.                                        */
/*                                                                    */
/*  Exit-Normal:  TRUE or FALSE                                       */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrAllColumnsFixedWidth (hwndCnr)

HANDLE  hwndCnr;  /* Handle of the container window.                  */

{
 LPCNRINFON   pccinfon;    /* Pointer to Internal Container Info.     */
 LPFIELDINFO  pFieldInfo;  /* Pointer to a FieldInfo structure.       */
 LPFIELDINFO  pLastInView;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* First check the left window's columns.                            */
 /*********************************************************************/
 if (!(pFieldInfo = pccinfon->pFIFirstColLeftInDV))
 {
    pFieldInfo = pccinfon->pFirstVisibleFI;
 }

 pLastInView = CnrQueryLastColumnInView (hwndCnr, pFieldInfo, TRUE);

 while ((pFieldInfo) && (pFieldInfo != NEXTVISIBLEFI(pLastInView)))
 {
    if (!(pFieldInfo->cxWidth))
    {
       return (FALSE);
    }

    pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
 }

 /*********************************************************************/
 /* Now check the right window's columns.                             */
 /*********************************************************************/
 if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
     (CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1)))
 {
    if (!(pFieldInfo = pccinfon->pFIFirstColRightInDV))
    {
       pFieldInfo = NEXTVISIBLEFI(pccinfon->pFieldInfoLast);
    }

    pLastInView = CnrQueryLastColumnInView (hwndCnr, pFieldInfo, FALSE);

    if (pLastInView)
    {
       while ((pFieldInfo) &&
              (pFieldInfo != NEXTVISIBLEFI(pLastInView)))
       {
          if (!(pFieldInfo->cxWidth))
          {
             return (FALSE);
          }

          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
       }
    }
 }

 /*********************************************************************/
 /* If we get to here, all the columns are a fixed width.  Return     */
 /* TRUE.                                                             */
 /*********************************************************************/
 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCreateNLSDateString( HANDLE hwndCnr, LPCDATE pDate,
                                 LPSTR pszDate )
{
 LPCNRINFON  pccinfon;     /* Pointer to Internal Container Info.     */
 int         aDate[3];
 int         Field1;
 int         Field2;
 int         Field3;
 int         DAY   = 0;
 int         MONTH = 1;
 int         YEAR  = 2;
 BOOL        LeadingZero;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 LeadingZero = pccinfon->LeadingZero;

 /*********************************************************************/
 /* Fill up my array with the values in the CDATE structure.          */
 /*********************************************************************/
 aDate[DAY]   = pDate->day;
 aDate[MONTH] = pDate->month;
 aDate[YEAR]  = pDate->year;

 /*********************************************************************/
 /* Determine the Date Format that is to be displayed.  There are 3   */
 /* possibilities. Month, Day, Year is the default.  The values of 2, */
 /* 1, and 0 for pccinfon->DateFormat are the values returned by the  */
 /* system during the query.  See CnrInitNLSInfo.                     */
 /*********************************************************************/
 switch ((int)pccinfon->DateFormat)
 {
    case 2:
       Field1 = YEAR;
       Field2 = MONTH;
       Field3 = DAY;
    break;

    case 1:
       Field1 = DAY;
       Field2 = MONTH;
       Field3 = YEAR;
    break;

    case 0:
    default:
       Field1 = MONTH;
       Field2 = DAY;
       Field3 = YEAR;
    break;
 }

 if (Field1 == YEAR)
 {
    LeadingZero = TRUE;
 }

 pszDate = CnrBuildDateTimeString (aDate[Field1],
                                   pszDate,
                                   pccinfon->DateSep,
                                   LeadingZero);

 LeadingZero = pccinfon->LeadingZero;

 if (Field2 == YEAR)
 {
    LeadingZero = TRUE;
 }

 pszDate = CnrBuildDateTimeString (aDate[Field2],
                                   pszDate,
                                   pccinfon->DateSep,
                                   LeadingZero);

 LeadingZero = pccinfon->LeadingZero;

 if (Field3 == YEAR)
 {
    LeadingZero = TRUE;
 }

 /*********************************************************************/
 /* Don't pass in the Date Separator for the last field.              */
 /*********************************************************************/
 pszDate = CnrBuildDateTimeString (aDate[Field3],
                                   pszDate,
                                   NULL,
                                   LeadingZero);

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCreateNLSTimeString( HANDLE hwndCnr, LPCTIME pTime,
                                 LPSTR pszTime )
{
 LPCNRINFON  pccinfon;   /* Pointer to Internal Container Info.       */
 int         Hours;
 BOOL        PM = FALSE;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 Hours = (int)pTime->hours;

 /*********************************************************************/
 /* If we are in the 12 hour format, we need to convert the hours to  */
 /* this format.                                                      */
 /*********************************************************************/
 if (!pccinfon->MilitaryFormat)
 {
    if (Hours >= 12)
    {
       Hours -= 12;
       PM = TRUE;
    }

    if (Hours == 0)
    {
       Hours = 12;
    }
 }

 pszTime = CnrBuildDateTimeString (Hours,
                                   pszTime,
                                   pccinfon->TimeSep,
                                   pccinfon->LeadingZero);

 pszTime = CnrBuildDateTimeString ((int)pTime->minutes,
                                   pszTime,
                                   pccinfon->TimeSep,
                                   TRUE);

 /*********************************************************************/
 /* Don't pass in the Time Separator for the last field.              */
 /*********************************************************************/
 pszTime = CnrBuildDateTimeString ((int)pTime->seconds,
                                   pszTime,
                                   NULL,
                                   TRUE);

 /*********************************************************************/
 /* If we are in the 12 hour format, add the am/pm indicator.         */
 /*********************************************************************/
 if (!pccinfon->MilitaryFormat)
 {
    lstrcat (pszTime, " ");
    lstrcat (pszTime, (PM) ? pccinfon->PM : pccinfon->AM);
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LPSTR FAR CnrBuildDateTimeString( int Field, LPSTR pszDateTime,
                                  LPSTR DateTimeSep, BOOL LeadingZero )
{
 if ((Field < 10) && (LeadingZero))
 {
    *pszDateTime++ = '0';
 }

 itoa (Field, pszDateTime, 10);

 /*********************************************************************/
 /* If there is a separator, add it to the end of the string.         */
 /*********************************************************************/
 if (DateTimeSep)
 {
    lstrcat (pszDateTime, DateTimeSep);
 }

 /*********************************************************************/
 /* Return a pointer to the end of the string, ie. to the terminating */
 /* NULL.                                                             */
 /*********************************************************************/
 while (*pszDateTime)
 {
    pszDateTime++;
 }

 return (pszDateTime);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrCreateNLSNumberString                          */
/*                                                                    */
/*  Description:    Builds a string from a DWORD number.              */
/*                                                                    */
/*  Input:          hwndCnr   - container window handle               */
/*                  dwNumber  - DWORD number to be converted to a     */
/*                              string.                               */
/*                  lpNumber - pointer to a buffer to hold string.    */
/*                                                                    */
/*  Output:         TRUE  - Success                                   */
/*                  FALSE - Error                                     */
/*                                                                    */
/*  NOTES:  This function only works on unsigned long numbers.        */
/*          Negative numbers are not supported.                       */
/*                                                                    */
/*  HISTORY:                                                          */
/*  02 Jan 92  Peter Haggar - Created it                              */
/**********************************************************************/
BOOL CnrCreateNLSNumberString( HWND hwndCnr, DWORD dwNumber,
                               LPSTR lpNumber )
{
 LPCNRINFON pccinfon;          /* Pointer to Internal Container Info. */
 LPSTR      pszLead;           /* Temp Pointer used to create string. */
 LPSTR      pszBack;           /* Temp Pointer used to create string. */
 LPSTR      pszTemp;           /* Temp Pointer used to  free the      */
                               /* storage.                            */
// LPVOID     pSatPtr;           /* Pointer to the storage space.       */
 WORD       i;                 /* Loop counter.                       */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
// pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);

 /* If the number is such that we don't have to worry about adding
  * the thousands separater, convert it and leave.
  */
 if (dwNumber < 1000)
 {
#ifdef IC_WU
    sprintf( lpNumber, "%lu", dwNumber );                       /*RKK1*/
#else
    ultoa (dwNumber, lpNumber, 10);
#endif
    return (TRUE);
 }

 /*********************************************************************/
 /* Allocate a temporary pointer to be used to create the number      */
 /* string.                                                           */
 /*********************************************************************/
// if (!(pszLead = (LPSTR)StmAllocateStorage (pSatPtr, NUMSIZE)))
 if (!(pszLead = (LPSTR)GlobalAlloc( GPTR, NUMSIZE)))                   /*DAB1*/
 {
    SETERROR(PMERR_INSUFFICIENT_MEMORY);
    return (FALSE);
 }

 /*********************************************************************/
 /* Save this pointer so we can free the memory at the bottom of this */
 /* function.                                                         */
 /*********************************************************************/
 pszTemp = pszLead;

 /*********************************************************************/
 /* Convert the number to a string.                                   */
 /*********************************************************************/
#ifdef IC_WU
 sprintf( pszLead, "%lu", dwNumber );                           /*RKK1*/
#else
 ultoa (dwNumber, pszLead, 10);
#endif

 /*********************************************************************/
 /* Reverse the string.                                               */
 /*********************************************************************/
 reverse (pszLead);

 /* Loop through the string and add in the thousands separator.
  */
 while (*pszLead)
 {
    i = 0;
    pszBack = pszLead;

    while ((*pszLead) && (i < 3))
    {
       pszLead = AnsiNext(pszLead);
       i++;
    }

    if (*pszLead)
    {
       strncat (lpNumber, pszBack, 3);
       lstrcat (lpNumber, pccinfon->Thousands);
    }
    else
    {
       lstrcat (lpNumber, pszBack);
    }
 }

 /*********************************************************************/
 /* Reverse the string again before we return.                        */
 /*********************************************************************/
 reverse (lpNumber);

 /*********************************************************************/
 /* Free the temporary storage that was allocated previously.         */
 /*********************************************************************/
// StmFreeStorage (pSatPtr, (LPVOID)pszTemp);
   GlobalFree( pszTemp );                                               /*DAB1*/

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  reverse                                           */
/*                                                                    */
/*  Descriptive Name:  Reverse the string                             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*                                                                    */
/*  Notes:  N/A                                                       */
/*                                                                    */
/*  Input:  psz     - String to reverse                               */
/*                                                                    */
/*  Output: None                                                      */
/*                                                                    */
/*  Exit-Normal:  N/A                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:                                                           */
/**********************************************************************/
VOID FAR reverse(LPSTR psz)
{
    int     i, j;
    char    c;

    for (i = 0, j = lstrlen(psz) - 1; i<j; i++, j-- ) {
        c = psz[i];
        psz[i] = psz[j];
        psz[j] = c;
    }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrColTitleHeightChanged                          */
/*                                                                    */
/*  Descriptive Name:  Column Title height changed                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will check to see if the height of the column title */
/*  windows has changed due to a direct edit operation or an          */
/*  invalidation resulting in different data.  At least one of the    */
/*  column titles could have changed from 1 line of text to 2 lines   */
/*  of text, which would result in this function returning TRUE.      */
/*                                                                    */
/*  Notes:  N/A                                                       */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*  Output: TRUE  - The column Title height has changed.              */
/*          FALSE - The column Title height has not changed.          */
/*                                                                    */
/*  Exit-Normal:  TRUE or FALSE                                       */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrColTitleHeightChanged (hwndCnr)

HANDLE  hwndCnr;  /* Handle of the container window.                  */

{
 LPCNRINFON  pccinfon;       /* Pointer to Internal Container Info.   */
 LPFIELDINFO pFieldInfo;     /* Pointer to a FieldInfo structure.     */
 POINT       IconSize;       /* Width and Height of an Icon.          */
 SIZEL       TextSize;       /* Width and Height of a Text string.    */
 HDC         hps;            /* Handle to the presentation space.     */
 long        RowHeight = 0;  /* Initial height of the column Title.   */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 pFieldInfo = pccinfon->pFirstVisibleFI;

 if (!pFieldInfo)
 {
    return (FALSE);
 }

 hps = PMGetFlippedDC (hwndCnr);
 CnrCalcIconSize (hwndCnr, &IconSize);

 /*********************************************************************/
 /* Loop through all of the columns for the specified row, and        */
 /* calculate the new height of the row.                              */
 /*********************************************************************/
 while (pFieldInfo)
 {
    if (pFieldInfo->flTitle & CFA_BITMAPORICON)
    {
       if ((long)IconSize.y > RowHeight)
       {
          RowHeight = IconSize.y;
       }
    }
    else
    {
       IcnCalcTextSize (hps, IconSize, pccinfon,
                        pFieldInfo->pTitleData, IS_SIDETEXT, &TextSize);

       if (TextSize.cy > RowHeight)                                     /*DAB1*/
       {
          RowHeight = TextSize.cy;                                      /*DAB1*/
       }
    }

    pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
 }

 ReleaseDC (hwndCnr, hps);

 /*********************************************************************/
 /* If the new height of the row is different than the old height, we */
 /* return TRUE.  Otherwise, return FALSE.                            */
 /*********************************************************************/
 if (RowHeight != (long)(pccinfon->MaxColTitleHeight))
 {
    return (TRUE);
 }
 else
 {
    return (FALSE);
 }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrCalcCellRectForDV                              */
/*                                                                    */
/*  Descriptive Name:  Calculate a Cell Rectangle for Data in Details */
/*                     View.                                          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will calculate the writeable area for column titles */
/*  and column data in Details View.  For Column Data, this is the    */
/*  intersection of the row and column that are passed as parameters. */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Not the entire area that results from the intersection of a row   */
/*  and a column is to be written in.  There is "dead space" that     */
/*  exists between the rows and between the columns, that data is not */
/*  written into.  This space is filled when selection emphasis is    */
/*  displayed.                                                        */
/*                                                                    */
/*  Input:  hwndCnr        - Handle of the Container Window.          */
/*          pRecord        - Pointer to the record in which combined  */
/*                           with a FieldInfo will result in a Cell.  */
/*          pFieldInfo     - Pointer to the FieldInfo in which to     */
/*                           calculate a Cell.                        */
/*          LeftWindow     - Determines if we are calculating a Cell  */
/*                           rectangle for the left or right window.  */
/*          ColumnTitleWnd - Determines if we are calculating a Cell  */
/*                           rectangle for a Column Title.            */
/*          CellRect       - Pointer to a RECT structure which is to */
/*                           contain the calculated Cell rectangle.   */
/*                                                                    */
/*  Output:  Pointer to a RECT structure which contains the Cell     */
/*           Rectangle that was calculated.                           */
/*                                                                    */
/*  Exit-Normal:  Pointer to a RECT Structure is returned.           */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcCellRectForDV (hwndCnr, pRecord, pFieldInfo, LeftWindow,
                           ColumnTitleWnd, CellRect)

HANDLE        hwndCnr;        /* Handle to the Container window.      */
LPRECORDCOREN pRecord;        /* Pointer to a Record to be combined   */
                              /* with a FieldInfo will result in a    */
                              /* Cell.                                */
LPFIELDINFO   pFieldInfo;     /* Pointer to a FieldInfo structure in  */
                              /* which to calculate a Cell.           */
BOOL          LeftWindow;     /* Calculate a Cell for the Left or     */
                              /* Right window.                        */
BOOL          ColumnTitleWnd; /* Calculate a Cell for a Column Title. */
LPRECT        CellRect;       /* Pointer to a RECT structure in      */
                              /* which to place the calculated Cell   */
                              /* Rectangle.                           */

{
 LPCNRINFON   pccinfon;      /* Pointer to Internal Container Info.   */
 PLPFIELDINFO pStartFI;      /* Pointer to a Pointer to a FieldInfo   */
                             /* Structure for left or right window.   */
 LPLONG       WindowOrgx;    /* Pointer to the address of the origin  */
                             /* for the left or right window.         */
 HANDLE       hwndTitleWnd;  /* The handle of the left or right       */
                             /* column title window.                  */
 RECT         TitleWndRect;  /* Rectangle of the left or right column */
                             /* title window.                         */
 int          ColumnWidth;   /* Width of column(FieldInfo).           */

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Determine the width of the column we are working with.            */
 /*********************************************************************/
 if (pFieldInfo->cxWidth)
 {
    ColumnWidth = pFieldInfo->cxWidth;
 }
 else
 {
    ColumnWidth = PFINFROMX(pFieldInfo)->usLongestLine;
 }

 /*********************************************************************/
 /* Determine if we are calculating a Cell Rectangle for a column     */
 /* title window.                                                     */
 /*********************************************************************/
 if (ColumnTitleWnd)
 {
    /******************************************************************/
    /* Determine if we are calculating a Cell Rectangle for the left  */
    /* or right window, and set up the starting FieldInfo structure,  */
    /* the correct window origin, and the handle to the left or right */
    /* window.                                                        */
    /******************************************************************/
    if (LeftWindow)
    {
       pStartFI = &pccinfon->pFirstVisibleFI;
       WindowOrgx = (LPLONG)&(pccinfon->ptlNonIconOrg.x);
       hwndTitleWnd = GetDlgItem (hwndCnr, CID_LEFTCOLTITLEWND);
    }
    else
    {
       pStartFI = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
       WindowOrgx = &pccinfon->RightWndxOrg;
       hwndTitleWnd = GetDlgItem (hwndCnr, CID_RIGHTCOLTITLEWND);
    }

    /******************************************************************/
    /* Get the rectangle of the title window.                         */
    /******************************************************************/
    PMGetClientRect (hwndTitleWnd, &TitleWndRect);

    /******************************************************************/
    /* Calcultate the rectangle for the title that the appplication,  */
    /* and I, can write into.                                         */
    /******************************************************************/
    CellRect->left = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                            pFieldInfo, TRUE, FALSE );

    CellRect->right = CellRect->left + ColumnWidth;

    CellRect->bottom = TitleWndRect.bottom -
                        pccinfon->lAveCharWidth;

    CellRect->top = CellRect->bottom -
                    pccinfon->MaxColTitleHeight;

    /******************************************************************/
    /* Now convert the OwnerItem rectangle into window coordinates.   */
    /* I only adjust the left and not the top and bottom because the  */
    /* Title window only scrolls horizontally, not vertically.        */
    /******************************************************************/
    CellRect->left  -= *WindowOrgx;
    CellRect->right -= *WindowOrgx;
 }
 else
 {
    /******************************************************************/
    /* We are calculating a Cell Rectangle for column data.  Set up   */
    /* the parameters for the left or right window.                   */
    /******************************************************************/
    if (LeftWindow)
    {
       pStartFI   = &pccinfon->pFirstVisibleFI;
       WindowOrgx = (LPLONG)&(pccinfon->ptlNonIconOrg.x);
    }
    else
    {
       pStartFI = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
       WindowOrgx = &pccinfon->RightWndxOrg;
    }

    /******************************************************************/
    /* Now calculate the rectangle for this FieldInfo that the        */
    /* application, and I, may write into.                            */
    /******************************************************************/
    CellRect->left = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                            pFieldInfo, TRUE, FALSE );

    CellRect->right = CellRect->left + ColumnWidth;

    CellRect->bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);
    CellRect->top    = CnrQueryRowPosInDV (pRecord, PT_TOP);

    /******************************************************************/
    /* Now convert the OwnerItem rectangle into window coordinates.   */
    /******************************************************************/
    CellRect->left   -= *WindowOrgx;
    CellRect->right  -= *WindowOrgx;
    CellRect->bottom -= pccinfon->ptlNonIconOrg.y;
    CellRect->top    -= pccinfon->ptlNonIconOrg.y;
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrOpenColDataTextMLE                             */
/*                                                                    */
/*  Descriptive Name:  Open an MLE window for Direct Edit for column  */
/*  Data.                                                             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will create and open an MLE window over the proper  */
/*  data item in Details View.  This data item is a result of the     */
/*  intersection of a row and a column.                               */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  The MLE will not be opened if the contaienr is Read-Only, the     */
/*  data is Read-Only, or the data does not have the CFA_STRING       */
/*  attribute associated with it.                                     */
/*                                                                    */
/*  Input:  hwndCnr    - Handle of the Container Window.              */
/*          xMousePos  - x Position of the mouse at the time the      */
/*                       Direct Edit operation is to start.           */
/*          yMousePos  - y Position of the mouse at the time the      */
/*                       Direct Edit operation is to start.           */
/*          LeftWindow - TRUE if the Direct Edit operation is to the  */
/*                       Left window in Details View, FALSE if for    */
/*                       the right window.                            */
/*                                                                    */
/*  Output:  TRUE  - There were no errors in creating the MLE.  The   */
/*                   MLE may not have been created, but due to other  */
/*                   reasons than an error.                           */
/*           FALSE - An Out of Memory Condition exists.               */
/*                                                                    */
/*  Exit-Normal:  TRUE                                                */
/*                                                                    */
/*  Exit-Error:   FALSE                                               */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrOpenColDataTextMLE (hwndCnr, xMousePos,
                                yMousePos, LeftWindow)

HANDLE      hwndCnr;     /* Handle of the container window.           */
long        xMousePos;   /* x Position of mouse when Direct Editing   */
                         /* is started.                               */
long        yMousePos;   /* y Position of mouse when Direct Editing   */
                         /* is started.                               */
BOOL        LeftWindow;  /* Indicates if we are processing the Direct */
                         /* Edit for the left or right window.        */

{
 LPFIELDINFO   pFieldInfo;  /* Pointer to the FieldInfo being edited. */
 LPRECORDCOREN pRecord;     /* Pointer to the Record being edited.    */

 /*********************************************************************/
 /* Check to see if the container is read-only.  If it is, we won't   */
 /* do any direct editing.                                            */
 /*********************************************************************/
 if (!(GetWindowLong (hwndCnr, GWL_STYLE) & CCS_READONLY))
 {
    pFieldInfo = CnrOverColumn( hwndCnr, xMousePos, yMousePos,
                                LeftWindow );

    pRecord = CnrOverRecord (hwndCnr, (int)xMousePos, (int)yMousePos);

    return (OpenColDataTextMLE (hwndCnr, pRecord,
                                pFieldInfo, LeftWindow));
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL OpenColDataTextMLE (hwndCnr, pRecord, pFieldInfo, LeftWindow)

HANDLE        hwndCnr;
LPRECORDCOREN pRecord;
LPFIELDINFO   pFieldInfo;
BOOL          LeftWindow;

{
 LPCNRINFON   pccinfon;    /* Pointer to Internal Container Info.     */
 WORD         Window;      /* ID of the window being edited.          */
 PLPFIELDINFO pStartFI;    /* First FieldInfo structure in the left   */
                           /* or right window.                        */
 POINT        IconSize;    /* Width and Height of an Icon.            */
 RECT         ViewPortLeft;/* Window coordinates of the left window.  */
 RECT         ViewPortRight;/* Window coordinates of the left window. */
 LPSTR        String;      /* The String being edited.                */
 HDC          hps;         /* Handle to a presentation space.         */
 SIZEL        TextSize;    /* Width and Height of the Text being      */
                           /* edited.                                 */
 RECT         TextRect;    /* Rectangle that surrounds the text being */
                           /* edited.                                 */
 int          xColumnPos;  /* x position of the column we are dealing */
                           /* with.                                   */
 RECT         CellRect;    /* Cell that the Text resides in.  This is */
                           /* the intersection of the Row and column. */
// LPVOID       pSatPtr;     /* Pointer to the storage space.           */
// ERRORID      ErrorID;     /* ID of any Errors that may occur.        */
 int          cyChar;
 BOOL         FirstColLeft = FALSE;  /* Determines if we are dealing  */
                           /* with the first column in the left wnd.  */
 MLEINFO      mleInfo;
 RECT         rctlColumnTitle;
 RECT         rctlCnrTitle;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Check to see if we are over a valid record(row)/FieldInfo         */
 /* (column) combination.                                             */
 /*********************************************************************/
 if ((pFieldInfo) && (pRecord))
 {
    /******************************************************************/
    /* Determine which window we are working with and assign the      */
    /* fields as appropriate.                                         */
    /******************************************************************/
    if (LeftWindow)
    {
       Window   = CID_LEFTDVWND;
       pStartFI = &pccinfon->pFirstVisibleFI;
    }
    else
    {
       Window   = CID_RIGHTDVWND;
       pStartFI = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
    }

    /******************************************************************/
    /* Get the rectangle for the left window.                         */
    /******************************************************************/
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPortLeft);

    /******************************************************************/
    /* Calculate the Icon size.                                       */
    /******************************************************************/
    CnrCalcIconSize (hwndCnr, &IconSize);

    /******************************************************************/
    /* Now check to be sure that the record(row) and FieldInfo        */
    /* (column) are not read-only.  Also check to be sure that        */
    /* what we are trying edit is string data.  If these              */
    /* conditions are not satisfied, we will not do any direct        */
    /* editing.                                                       */
    /******************************************************************/
    if ((!(pRecord->flRecordAttr & CRA_RECORDREADONLY)) &&
        (!(pFieldInfo->flData & CFA_FIREADONLY)) &&
        (pFieldInfo->flData & CFA_STRING))
    {
       pccinfon->preccEdit = pRecord;
       pccinfon->pFIEdit   = pFieldInfo;
       pccinfon->hwndEdit  = GetDlgItem (hwndCnr, Window);
       pccinfon->ulFlagSet |= FL_EDITMODE;

       if (!(String = CnrGetColumnData (hwndCnr,
                                        pRecord->preccX,
                                        pFieldInfo)))
       {
//        ErrorID = WinGetLastError (WinQueryAnchorBlock
//                                   (HANDLE_DESKTOP));

//        if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//        {
//           SETERROR(ErrorID);
//           return (FALSE);
//        }

//        SETERROR(ErrorID);
       }

       pccinfon->ceData.ppszText = (LPPSTR)((LPBYTE)pRecord->preccX +
                                   pFieldInfo->offStruct);

       pccinfon->ceData.cbText = 0;
       COPYNDATATOX(pRecord);
       pccinfon->ceData.pRecord = pRecord->preccX;
       pccinfon->ceData.hwndCnr = hwndCnr;
       pccinfon->ceData.pFieldInfo = pFieldInfo;

       /***************************************************************/
       /* Get the size of the text that we are editing.               */
       /***************************************************************/
       hps = PMGetFlippedDC (hwndCnr);

       if ((!String) || (*String == '\0'))
       {
         TextSize.cx = PFINFROMX(pFieldInfo)->usLongestLine;            /*DAB1*/
         TextSize.cy = pRecord->RowHeight;                              /*DAB1*/
       }
       else
       {
         IcnCalcTextSize( hps, IconSize, pccinfon, (LPSTR)String,
                          IS_SIDETEXT, &TextSize);
       }

       ReleaseDC (hwndCnr, hps);

       xColumnPos = CnrQueryColumnPosInDV (hwndCnr,
                                           *pStartFI,
                                           pFieldInfo,
                                           TRUE,
                                           FALSE);

       if ((LeftWindow) && (xColumnPos == pccinfon->lAveCharWidth))
       {
          FirstColLeft = TRUE;
       }

       CnrCalcCellRectForDV (hwndCnr, pRecord, pFieldInfo,
                             LeftWindow, FALSE, &CellRect);

       /***************************************************************/
       /* Calculate the rectangle that represents the text we are     */
       /* editing.                                                    */
       /***************************************************************/
       CnrCalcRectForDV (hwndCnr, pRecord, pFieldInfo, TextSize,
                         FirstColLeft, CellRect, &TextRect);

       /***************************************************************/
       /* We want the text rectangle to be the width of the column,   */
       /* not the width of the text.  If it was the width of the text */
       /* and the text was only 1 character, it would look stupid.    */
       /* This should make it look at least a little intelligent.     */
       /***************************************************************/
       TextRect.right = CellRect.right;
       TextRect.left  = CellRect.left;

       cyChar = GETCYCHAR(pccinfon);
       TextRect.bottom = max( TextRect.bottom, (TextRect.top + cyChar));

       /***************************************************************/
       /* Because the MLE is the way it is, get the fontmetrics       */
       /* for the DeskTop.                                            */
       /***************************************************************/
       CnrQueryFontMetrics( hwndCnr, NULL, TRUE, &mleInfo, NULL );

       /***************************************************************/
       /* Get the heights of the Container title and the Column Title */
       /* Windows ...                                                 */
       /***************************************************************/
       CnrQueryTitleRect (hwndCnr, &rctlCnrTitle, NULL);
       CnrQueryColumnTitleWndRect (hwndCnr, TRUE, &rctlColumnTitle);

       TextRect.left  -= mleInfo.lAveCharWidth / 2;

       TextRect.right += (mleInfo.lAveCharWidth / 2) +
                          TXT_SIDEMARGIN;

       TextRect.bottom += rctlCnrTitle.bottom + rctlColumnTitle.bottom +
                          mleInfo.lMaxBaselineExt / 6;

       TextRect.top    += rctlCnrTitle.bottom + rctlColumnTitle.bottom -
                          mleInfo.lMaxBaselineExt / 6;

       /***************************************************************/
       /* If we are direct editing text in the right window, we       */
       /* need to make the TextRect relative to hwndCnr.  Therefore   */
       /* we need to add the width of the left window plus the        */
       /* width of the splitbar to the left and right sides of the    */
       /* text rectangle.                                             */
       /***************************************************************/
       if (!LeftWindow)
       {
          TextRect.left  += ViewPortLeft.right +
                             GetSystemMetrics (SM_CXFRAME);
          TextRect.right += ViewPortLeft.right +
                             GetSystemMetrics (SM_CXFRAME);
       }

       /***************************************************************/
       /* If the right side of the text is going to fall outside      */
       /* of the window, position the right side of the rectangle     */
       /* to be even with the right side of the window.  The MLE      */
       /* will now scroll.                                            */
       /***************************************************************/
       if (LeftWindow)
       {
          TextRect.right = min (ViewPortLeft.right,
                                 TextRect.right);
          TextRect.left = max (ViewPortLeft.left, TextRect.left);
       }
       else
       {
          PMGetClientRect (GetDlgItem (hwndCnr, CID_RIGHTDVWND),
                           &ViewPortRight);

          ViewPortRight.right += ViewPortLeft.right +
                                  GetSystemMetrics (SM_CXFRAME);
          ViewPortRight.left += ViewPortLeft.right +
                                  GetSystemMetrics (SM_CXFRAME);

          TextRect.right = min (ViewPortRight.right,
                                 TextRect.right);
          TextRect.left = max (ViewPortRight.left, TextRect.left);
       }

       /***************************************************************/
       /* Create the MLE.                                             */
       /***************************************************************/
       CnrCreateMLE( pccinfon->hwndEdit, String, TextRect, 2 );

       /***************************************************************/
       /* Free the storage that was allocated for String, only if     */
       /* we are not processing CFA_STRING data.  No extra memory     */
       /* is allocated in this case that needs to be freed.           */
       /***************************************************************/
       if (!(pFieldInfo->flData & CFA_STRING))
       {
//        pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);
//        StmFreeStorage (pSatPtr, (LPVOID)String);
          GlobalFree( String );                                         /*DAB1*/
       }
    }
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrOpenColTitleTextMLE                            */
/*                                                                    */
/*  Descriptive Name:  Open an MLE window for Direct Edit for a       */
/*  Column Title.                                                     */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will create and open an MLE window over the proper  */
/*  column title in Details View.                                     */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  The MLE will not be opened if the contaienr is Read-Only, the     */
/*  data is Read-Only, or the data does not have the CFA_STRING       */
/*  attribute associated with it.                                     */
/*                                                                    */
/*  Input:  hwndCnr    - Handle of the Container Window.              */
/*          xMousePos  - x Position of the mouse at the time the      */
/*                       Direct Edit operation is to start.           */
/*          yMousePos  - y Position of the mouse at the time the      */
/*                       Direct Edit operation is to start.           */
/*          LeftWindow - TRUE if the Direct Edit operation is to the  */
/*                       Left window in Details View, FALSE if for    */
/*                       the right window.                            */
/*                                                                    */
/*  Output:  void                                                     */
/*                                                                    */
/*  Exit-Normal:  Function returns.                                   */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrOpenColTitleTextMLE (hwndCnr, xMousePos,
                                  yMousePos, LeftWindow)

HANDLE      hwndCnr;     /* Handle of the container window.           */
int         xMousePos;   /* x Position of mouse when Direct Editing   */
                         /* is started.                               */
int         yMousePos;   /* y Position of mouse when Direct Editing   */
                         /* is started.                               */
BOOL        LeftWindow;  /* Indicates if we are processing the Direct */
                         /* Edit for the left or right window.        */

{
 LPFIELDINFO  pFieldInfo;   /* Pointer to the FieldInfo being edited. */

 /*********************************************************************/
 /* Check to see if the container is read-only.  If it is, we won't   */
 /* do any direct editing.                                            */
 /*********************************************************************/
 if (!(GetWindowLong (hwndCnr, GWL_STYLE) & CCS_READONLY))
 {
    pFieldInfo = CnrOverColumn( hwndCnr, xMousePos, yMousePos,
                                LeftWindow );

    OpenColTitleTextMLE (hwndCnr, pFieldInfo, LeftWindow);
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL OpenColTitleTextMLE (hwndCnr, pFieldInfo, LeftWindow)

HANDLE      hwndCnr;
LPFIELDINFO pFieldInfo;
BOOL        LeftWindow;

{
 LPCNRINFON   pccinfon;     /* Pointer to Internal Container Info.    */
 WORD         ColTitleWnd;  /* ID of the Title Window being edited.   */
 PLPFIELDINFO pStartFI;     /* First FieldInfo structure in the left  */
                            /* or right window.                       */
 RECT         TextRect;     /* Rectangle which surrounds the Text     */
                            /* that is to be edited.                  */
 RECT         ViewPortLeft; /* Window coordinates of the left window. */
 RECT         ViewPortRight;/* Window coordinates of the left window. */
 RECT         rctlTitle;    /* Window coordinates of the title.       */
 HDC          hps;          /* Handle to a Presentation space.        */
 POINT        IconSize;     /* Width and Height of an Icon.           */
 SIZEL        TextSize;     /* Width and Height of the text being     */
                            /* edited.                                */
 int          xColumnPos;   /* x Position of the column we are        */
                            /* dealing with.                          */
 RECT         CellRect;     /* Rectangle that houses the Text and     */
                            /* area around it.                        */
 LPSTR        String;       /* String to be edited.                   */
 int          cyChar;
 BOOL         FirstColLeft = FALSE;  /* Determines if we are dealing  */
                           /* with the first column in the left wnd.  */
 MLEINFO      mleInfo;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 if (pFieldInfo)
 {
    /******************************************************************/
    /* Get the rectangle for the left window.                         */
    /******************************************************************/
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPortLeft);

    /******************************************************************/
    /* Calculate the Icon size.                                       */
    /******************************************************************/
    CnrCalcIconSize (hwndCnr, &IconSize);

    /******************************************************************/
    /* Determine which window we are working with and assign the      */
    /* fields as appropriate.                                         */
    /******************************************************************/
    if (LeftWindow)
    {
       ColTitleWnd = CID_LEFTCOLTITLEWND;
       pStartFI    = &pccinfon->pFirstVisibleFI;
    }
    else
    {
       ColTitleWnd = CID_RIGHTCOLTITLEWND;
       pStartFI    = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
    }

    /******************************************************************/
    /* Now check to be sure that the column that we are over is       */
    /* not read-only.  Also check to make sure it is text data.       */
    /* If these conditions are not satisfied, we will not do any      */
    /* direct editing.                                                */
    /******************************************************************/
    if ((!(pFieldInfo->flTitle & CFA_FITITLEREADONLY)) &&
        (!(pFieldInfo->flTitle & CFA_BITMAPORICON)))
    {
       pccinfon->preccEdit = NULL;
       pccinfon->pFIEdit   = pFieldInfo;
       pccinfon->hwndEdit  = (HWND)GetDlgItem (hwndCnr, ColTitleWnd);   /*DAB1*/
       pccinfon->ulFlagSet |= FL_EDITMODE;

       String = pFieldInfo->pTitleData;

       /***************************************************************/
       /* Fill in the CnrEdit structure to be passed back to the      */
       /* app.                                                        */
       /***************************************************************/
       pccinfon->ceData.ppszText = (PSZ *)&(pFieldInfo->pTitleData);    /*DAB1*/
       pccinfon->ceData.cbText = 0;
       pccinfon->ceData.pRecord = NULL;
       pccinfon->ceData.hwndCnr = hwndCnr;
       pccinfon->ceData.pFieldInfo = pFieldInfo;

       /***************************************************************/
       /* Get the size of the text that we are editing.               */
       /***************************************************************/
       hps = PMGetFlippedDC (hwndCnr);

       if ((!String) || (*((LPSTR)(String)) == '\0'))
       {
          TextSize.cx = PFINFROMX(pFieldInfo)->usLongestLine;           /*DAB1*/
          TextSize.cy = pccinfon->MaxColTitleHeight;                    /*DAB1*/
       }
       else
       {
         IcnCalcTextSize( hps, IconSize, pccinfon, (LPSTR)String,
                          IS_SIDETEXT, &TextSize);
       }

       ReleaseDC (hwndCnr, hps);

       xColumnPos = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                           pFieldInfo, TRUE, FALSE );

       if ((LeftWindow) &&
           (xColumnPos == pccinfon->lAveCharWidth))
       {
          FirstColLeft = TRUE;
       }

       CnrCalcCellRectForDV (hwndCnr, NULL, pFieldInfo,
                             LeftWindow, TRUE, &CellRect);

       /***************************************************************/
       /* Calculate the rectangle that represents the text we are     */
       /* editing.                                                    */
       /***************************************************************/
       CnrCalcRectForDVTitles (hwndCnr, pFieldInfo, TextSize,
                               FirstColLeft, CellRect, &TextRect);

       /***************************************************************/
       /* We want the text rectangle to be the width of the column,   */
       /* not the width of the text.  If it was the width of the text */
       /* and the text was only 1 character, it would look stupid.    */
       /* This should make it look at least a little intelligent.     */
       /***************************************************************/
       TextRect.right = CellRect.right;
       TextRect.left  = CellRect.left;

       cyChar = GETCYCHAR(pccinfon);
       TextRect.bottom = max(TextRect.bottom, (TextRect.top + cyChar));

       /*************************************************************/
       /* Calculate the size of the title bar.                      */
       /*************************************************************/
       CnrQuerySizeRect(hwndCnr, NULL, &rctlTitle, NULL, NULL, NULL);

       /***************************************************************/
       /* Because the MLE is the way it is, get the fontmetrics       */
       /* for the DeskTop.                                            */
       /***************************************************************/
       CnrQueryFontMetrics( hwndCnr, NULL, TRUE, &mleInfo, NULL );

       TextRect.left   -= mleInfo.lAveCharWidth / 2;

       TextRect.right  += (mleInfo.lAveCharWidth / 2) +
                           TXT_SIDEMARGIN;

       TextRect.bottom += rctlTitle.bottom +
                          mleInfo.lMaxBaselineExt / 4;

       TextRect.top    += rctlTitle.bottom -
                          mleInfo.lMaxBaselineExt / 2;

       /***************************************************************/
       /* If we are direct editing text in the right window, we       */
       /* need to make the TextRect relative to hwndCnr.  Therefore   */
       /* we need to add the width of the left window plus the        */
       /* width of the splitbar to the left and right sides of the    */
       /* text rectangle.                                             */
       /***************************************************************/
       if (!LeftWindow)
       {
          TextRect.left  += ViewPortLeft.right +
                             GetSystemMetrics (SM_CXFRAME);
          TextRect.right += ViewPortLeft.right +
                             GetSystemMetrics (SM_CXFRAME);
       }

       /***************************************************************/
       /* If the right side of the text is going to fall outside      */
       /* of the window, position the right side of the rectangle     */
       /* to be even with the right side of the window.  The MLE      */
       /* will now scroll.                                            */
       /***************************************************************/
       if (LeftWindow)
       {
          TextRect.right = min (ViewPortLeft.right,
                                 TextRect.right);
          TextRect.left = max (ViewPortLeft.left, TextRect.left);
       }
       else
       {
          PMGetClientRect (GetDlgItem (hwndCnr,
                                               CID_RIGHTDVWND),
                              &ViewPortRight);

          ViewPortRight.right += ViewPortLeft.right +
                                  GetSystemMetrics (SM_CXFRAME) +
                                  GetSystemMetrics (SM_CXVSCROLL);
          ViewPortRight.left += ViewPortLeft.right +
                                  GetSystemMetrics (SM_CXFRAME);

          TextRect.right = min (ViewPortRight.right,
                                 TextRect.right);
          TextRect.left = max (ViewPortRight.left, TextRect.left);
       }

       /***************************************************************/
       /* Create the MLE.                                             */
       /***************************************************************/
       CnrCreateMLE( pccinfon->hwndEdit, String, TextRect, 2 );
    }
 }

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcColumnRect (hwndCnr, pFieldInfo, LeftWindow, ColumnRect)

HANDLE      hwndCnr;
LPFIELDINFO pFieldInfo;
BOOL        LeftWindow;
LPRECT      ColumnRect;

{
 LPCNRINFON   pccinfon;
 RECT         ViewPort;
 RECT         ColTitleRect;
 PLPFIELDINFO pStartFI;
 LPLONG       WindowOrgx;
 long         CxBetweenCols;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Get the distance between the columns.                             */
 /*********************************************************************/
 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

 if (LeftWindow)
 {
    pStartFI   = &pccinfon->pFirstVisibleFI;
    WindowOrgx = (LPLONG)&pccinfon->ptlNonIconOrg.x;
 }
 else
 {
    pStartFI   = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
    WindowOrgx = &pccinfon->RightWndxOrg;
 }

 /*********************************************************************/
 /* Query the rectangles of the left viewport and the left column     */
 /* title window.  We use the left because if will always exist in    */
 /* Details View.  Also, all we want from these queries is the height */
 /* of the windows.  Both the left and the right windows have the     */
 /* same height, so if we are dealing with the right window...it's    */
 /* ok.                                                               */
 /*********************************************************************/
 PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                     &ViewPort);

 CnrQueryColumnTitleWndRect (hwndCnr, TRUE, &ColTitleRect);

 ColumnRect->bottom = ViewPort.bottom + ColTitleRect.bottom;
 ColumnRect->top    = ColTitleRect.top;

 ColumnRect->left = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                           pFieldInfo, TRUE, FALSE );

 ColumnRect->right = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                            pFieldInfo, FALSE, FALSE );

 /*********************************************************************/
 /* Take into account the distance between the columns.               */
 /*********************************************************************/
 ColumnRect->left  -= (CxBetweenCols / 2);
 ColumnRect->right += (CxBetweenCols / 2);

 /*********************************************************************/
 /* Map the left and the right to the window.                         */
 /*********************************************************************/
 ColumnRect->left  -= *WindowOrgx;
 ColumnRect->right -= *WindowOrgx;

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LPFIELDINFO FAR CnrOverColumn (hwndCnr, xMousePos,
                               yMousePos, LeftWindow)

HANDLE  hwndCnr;
long    xMousePos;
long    yMousePos;
BOOL    LeftWindow;

{
 LPCNRINFON  pccinfon;
 POINT       MousePt;
 LPFIELDINFO pFieldInfo;
 LPFIELDINFO pLastInView;
 RECT        ColumnRect;
 BOOL        Found = FALSE;

 /*********************************************************************/
 /* Get the pointer to the internal data for the container.           */
 /*********************************************************************/
 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 MousePt.x = (int)xMousePos;
 MousePt.y = (int)yMousePos;

 /*********************************************************************/
 /* If we are finding a column in the left window, get the first and  */
 /* last in view for the left window.                                 */
 /*********************************************************************/
 if (LeftWindow)
 {
    pFieldInfo = pccinfon->pFIFirstColLeftInDV;

    if (!pFieldInfo)
    {
       pFieldInfo = pccinfon->pFirstVisibleFI;
    }

    /******************************************************************/
    /* Get the last column in view for the left window.               */
    /******************************************************************/
    pLastInView = CnrQueryLastColumnInView (hwndCnr, pFieldInfo, TRUE);
 }
 else
 /*********************************************************************/
 /* Get the first and last columns in view for the right window.      */
 /*********************************************************************/
 {
    pFieldInfo = pccinfon->pFIFirstColRightInDV;

    if ((!pFieldInfo) && (pccinfon->pFieldInfoLast))
    {
       pFieldInfo = NEXTVISIBLEFI(pccinfon->pFieldInfoLast);
    }

    /******************************************************************/
    /* Get the last column in view for the right window.              */
    /******************************************************************/
    pLastInView = CnrQueryLastColumnInView (hwndCnr, pFieldInfo, FALSE);
 }

 while ((pFieldInfo) && (!Found) &&
        (pFieldInfo != NEXTVISIBLEFI(pLastInView)))
 {
    CnrCalcColumnRect (hwndCnr, pFieldInfo, LeftWindow, &ColumnRect);

    Found = PMPtInRect (&ColumnRect, MousePt);

    if (!Found)
    {
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
    }
 }

 if (Found)
 {
    return (pFieldInfo);
 }
 else
 {
    return (NULL);
 }

}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcRectForDVTitles (hwndCnr, pFieldInfo, ItemSize,
                                  FirstColLeft, CellRect, ItemRect)

HANDLE      hwndCnr;
LPFIELDINFO pFieldInfo;
SIZEL       ItemSize;
BOOL        FirstColLeft;
RECT        CellRect;
LPRECT      ItemRect;

{
 LPCNRINFON pccinfon;
 int        ColumnWidth;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Determine the width of the column we are working with.            */
 /*********************************************************************/
 if (pFieldInfo->cxWidth)
 {
    ColumnWidth = pFieldInfo->cxWidth;
 }
 else
 {
    ColumnWidth = PFINFROMX(pFieldInfo)->usLongestLine;
 }

 /*********************************************************************/
 /* First, do any vertical justification on the rectange.  Handle the */
 /* case where the app. specified either Bottom or Top justification. */
 /*********************************************************************/
 if (pFieldInfo->flTitle & CFA_BOTTOM)
 {
    ItemRect->bottom = CellRect.bottom;
    ItemRect->top    = (ItemRect->bottom - ItemSize.cy);                /*DAB1*/
 }
 else if (pFieldInfo->flTitle & CFA_TOP)
 {
    ItemRect->top    = CellRect.top;
    ItemRect->bottom = (CellRect.top + ItemSize.cy);                    /*DAB1*/
 }
 else
 {
    /******************************************************************/
    /* Handle the default justification which is vertically centered. */
    /* If we are computing the leftmost column title rectangle in the */
    /* left window, then do so here.  We single this out because the  */
    /* computation is different for this item.  The rest of the       */
    /* column titles are computed using the same computation.         */
    /******************************************************************/
    if (FirstColLeft)
    {
       ItemRect->top    = pccinfon->FirstTitlePos.y;
       ItemRect->bottom = (ItemRect->top +                              /*DAB1*/
                          pccinfon->slFirstTitleSize.cy);               /*DAB1*/
    }
    else
    {
       ItemRect->bottom = (pccinfon->FirstTitlePos.y +
                            (pccinfon->slFirstTitleSize.cy +            /*DAB1*/
                            (ItemSize.cy -                              /*DAB1*/
                            pccinfon->slFirstTitleSize.cy) / 2));       /*DAB1*/
       ItemRect->top    = (ItemRect->bottom - ItemSize.cy);             /*DAB1*/

    }
 }

 /*********************************************************************/
 /* Now handle any horizontal justification.                          */
 /*********************************************************************/
 if (pFieldInfo->flTitle & CFA_CENTER)
 {
    ItemRect->left  = max ((CellRect.left +
                            (ColumnWidth - ItemSize.cx) / 2),           /*DAB1*/
                             CellRect.left);
    ItemRect->right = min ((ItemRect->left + ItemSize.cx),              /*DAB1*/
                             CellRect.right);
 }
 else if (pFieldInfo->flTitle & CFA_RIGHT)
 {
    ItemRect->right = CellRect.right;
    ItemRect->left  = max ((CellRect.left +                             /*DAB1*/
                            ColumnWidth - ItemSize.cx),                 /*DAB1*/
                             CellRect.left);
 }
 else
 {
    /******************************************************************/
    /* Otherwise the default horizontal justification is used.  The   */
    /* default is left justified.                                     */
    /******************************************************************/
    if (FirstColLeft)
    {
       ItemRect->left   = CellRect.left;
       ItemRect->right  = min ((ItemRect->left +                        /*DAB1*/
                                pccinfon->slFirstTitleSize.cx),         /*DAB1*/
                                CellRect.right);
    }
    else
    {
       ItemRect->left   = CellRect.left;
       ItemRect->right  = min ((ItemRect->left + ItemSize.cx),          /*DAB1*/
                                 CellRect.right);
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcRectForDV (hwndCnr, pRecord, pFieldInfo, ItemSize,
                            FirstColLeft, CellRect, ItemRect)

HANDLE        hwndCnr;
LPRECORDCOREN pRecord;
LPFIELDINFO   pFieldInfo;
SIZEL         ItemSize;
BOOL          FirstColLeft;
RECT          CellRect;
LPRECT        ItemRect;

{
 LPCNRINFON  pccinfon;      /* Pointer to Internal Control Info.      */
 int         ColumnWidth;   /* Width of the column in pels.           */

 /*********************************************************************/
 /* Get the internal container information pointer and the view info  */
 /*  for the record passed                                            */
 /*********************************************************************/
 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Determine the width of the column we are working with.            */
 /*********************************************************************/
 if (pFieldInfo->cxWidth)
 {
    ColumnWidth = pFieldInfo->cxWidth;
 }
 else
 {
    ColumnWidth = PFINFROMX(pFieldInfo)->usLongestLine;
 }

 /******************************************************************/
 /* First, do any vertical justification on the rectange.  Handle  */
 /* the case where the app. specified either Bottom or Top         */
 /* justification.                                                 */
 /******************************************************************/
 if (pFieldInfo->flData & CFA_BOTTOM)
 {
    ItemRect->bottom = CellRect.bottom;
    ItemRect->top    = (ItemRect->bottom - ItemSize.cy);                /*DAB1*/
 }
 else if (pFieldInfo->flData & CFA_TOP)
 {
    ItemRect->top = CellRect.top;
    ItemRect->bottom = (ItemRect->top + ItemSize.cy);                   /*DAB1*/
 }
 else
 {
    /******************************************************************/
    /* Otherwise, the default centering of the item is used.  If we   */
    /* are processing the first column in the left window, we need    */
    /* to handle this case separately because its x,y position is     */
    /* stored in the RECORDCORE.                                      */
    /******************************************************************/
    if (FirstColLeft)
    {
       ItemRect->top    = pRecord->ptlNonIconPos.y -
                          pccinfon->ptlNonIconOrg.y;
       ItemRect->bottom = (ItemRect->top +                              /*DAB1*/
                          pRecord->slTextSize.cy);                      /*DAB1*/
    }
    else
    {
       ItemRect->top    = ((pRecord->ptlNonIconPos.y -
                            (ItemSize.cy -                              /*DAB1*/
                            pRecord->slTextSize.cy) / 2) -              /*DAB1*/
                            pccinfon->ptlNonIconOrg.y);
       ItemRect->bottom = ItemRect->top + ItemSize.cy;                  /*DAB1*/
    }
 }

 /******************************************************************/
 /* Now handle any horizontal justification.                       */
 /******************************************************************/
 if (pFieldInfo->flData & CFA_CENTER)
 {
    ItemRect->left  = max ((CellRect.left +                             /*DAB1*/
                            (ColumnWidth - ItemSize.cx) / 2),           /*DAB1*/
                            CellRect.left);
    ItemRect->right = min ((ItemRect->left + ItemSize.cx),              /*DAB1*/
                             CellRect.right);
 }
 else if (pFieldInfo->flData & CFA_RIGHT)
 {
    ItemRect->right = CellRect.right;
    ItemRect->left  = max ((CellRect.left +                             /*DAB1*/
                             (ColumnWidth - ItemSize.cx)),              /*DAB1*/
                             CellRect.left);
 }
 else
 {
    /******************************************************************/
    /* Otherwise the default left justification is used.  Check to    */
    /* see if this is the first column in the left window.  If so, do */
    /* its special case.                                              */
    /******************************************************************/
    if (FirstColLeft)
    {
       ItemRect->left   = CellRect.left;
       ItemRect->right  = min ((ItemRect->left +                        /*DAB1*/
                                 pRecord->slTextSize.cx),               /*DAB1*/
                                 CellRect.right);
    }
    else
    {
       ItemRect->left   = CellRect.left;
       ItemRect->right  = min ((ItemRect->left + ItemSize.cx),          /*DAB1*/
                                 CellRect.right);
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcRowRect (hwndCnr, pRecord, LeftWindow, RowRect)

HANDLE        hwndCnr;
LPRECORDCOREN pRecord;
BOOL          LeftWindow;
LPRECT        RowRect;

{
 LPCNRINFON pccinfon;
 LPCNRINFO  pccinfo;
 RECT       ViewPort;
 long       CyBetweenRows;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* Get the distance between the rows.                                */
 /*********************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 /*********************************************************************/
 /* Check to see if CyBetweeRows is an odd number.  If it is we add 1 */
 /* to it so that it will be even.  This is done so that there will   */
 /* not be a gap between the rows during selection.  A gap would      */
 /* occur if CyBetweenRows was odd because dividing it by 2 would not */
 /* give a fractional result, and round down.                         */
 /*********************************************************************/
 if (CyBetweenRows & NUM_ODD)
 {
    CyBetweenRows++;
 }

 if (LeftWindow)
 {
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPort);

    RowRect->left   = ViewPort.left;
    RowRect->right  = ViewPort.right;
    RowRect->bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);
    RowRect->top    = CnrQueryRowPosInDV (pRecord, PT_TOP);

    /******************************************************************/
    /* Adjust the top and bottom of the row.                          */
    /******************************************************************/
    RowRect->bottom += (CyBetweenRows / 2);
    RowRect->top    -= (CyBetweenRows / 2);

    /******************************************************************/
    /* Convert the row rectangle into window coordinates.  However,   */
    /* don't convert right.  We want to leave this equal to the      */
    /* right side of the viewport.                                    */
    /******************************************************************/
    RowRect->left   -= pccinfon->ptlNonIconOrg.x;
    RowRect->bottom -= pccinfon->ptlNonIconOrg.y;
    RowRect->top    -= pccinfon->ptlNonIconOrg.y;
 }
 else
 /*********************************************************************/
 /* Calculate we need to calculate the row rectangle for the right    */
 /* window.                                                           */
 /*********************************************************************/
 {
    PMGetClientRect (GetDlgItem (hwndCnr, CID_RIGHTDVWND),
                        &ViewPort);

    RowRect->left   = ViewPort.left;
    RowRect->right  = ViewPort.right;
    RowRect->bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);
    RowRect->top    = CnrQueryRowPosInDV (pRecord, PT_TOP);

    /******************************************************************/
    /* Adjust the top and bottom of the row.                          */
    /******************************************************************/
    RowRect->bottom += (CyBetweenRows / 2);
    RowRect->top    -= (CyBetweenRows / 2);

    /******************************************************************/
    /* Convert the row rectangle into window coordinates.  Again, we  */
    /* don't change right.  We want it to be the right side of the   */
    /* viewport.                                                      */
    /******************************************************************/
    RowRect->left   -= pccinfon->RightWndxOrg;
    RowRect->bottom -= pccinfon->ptlNonIconOrg.y;
    RowRect->top    -= pccinfon->ptlNonIconOrg.y;
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrCalcSelectionRect (hwndCnr, pRecord, pFieldInfo,
                                LeftWindow, SelectionRect)

HANDLE        hwndCnr;
LPRECORDCOREN pRecord;
LPFIELDINFO   pFieldInfo;
BOOL          LeftWindow;
LPRECT        SelectionRect;

{
 LPCNRINFON   pccinfon;
 LPCNRINFO    pccinfo;
 long         CyBetweenRows;
 long         CxBetweenCols;
 PLPFIELDINFO pStartFI;
 LPLONG       WindowOrgx;
 LPFIELDINFO  pLast;
 RECT         ViewPort;

 /******************************************************************/
 /* Get the pointers to the internal and external control          */
 /* information.                                                   */
 /******************************************************************/
 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /******************************************************************/
 /* Set the distance between the rows.  If the application has     */
 /* specified it, use it...otherwise use our constant.             */
 /******************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 /*********************************************************************/
 /* Check to see if CyBetweeRows is an odd number.  If it is we add 1 */
 /* to it so that it will be even.  This is done so that there will   */
 /* not be a gap between the rows during selection.  A gap would      */
 /* occur if CyBetweenRows was odd because dividing it by 2 would not */
 /* give a fractional result, and round down.                         */
 /*********************************************************************/
 if (CyBetweenRows & NUM_ODD)
 {
    CyBetweenRows++;
 }

 /******************************************************************/
 /* This is the distance between columns.                          */
 /******************************************************************/
 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);

 if (CxBetweenCols & NUM_ODD)
 {
    CxBetweenCols++;
 }

 /******************************************************************/
 /* If we are calculating a rectangle for the left window, we need */
 /* the first column in the left window.  Otherwise we need the    */
 /* first column in the right window.  We need this for the call   */
 /* to CnrQueryColumnPosInDV.  Also get the window origin for the  */
 /* left or right window.                                          */
 /******************************************************************/
 if (LeftWindow)
 {
    pStartFI   = &pccinfon->pFirstVisibleFI;
    WindowOrgx = (LPLONG)&pccinfon->ptlNonIconOrg.x;
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPort);
 }
 else
 {
    pStartFI = &(NEXTVISIBLEFI(pccinfon->pFieldInfoLast));
    WindowOrgx = &pccinfon->RightWndxOrg;
    PMGetClientRect (GetDlgItem (hwndCnr, CID_RIGHTDVWND),
                        &ViewPort);
 }

 /*********************************************************************/
 /* Query the left and right sides of the column.                     */
 /*********************************************************************/
 SelectionRect->left = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                              pFieldInfo, TRUE, FALSE );

 SelectionRect->right = CnrQueryColumnPosInDV( hwndCnr, *pStartFI,
                                             pFieldInfo, FALSE, FALSE );

 /*********************************************************************/
 /* Query the Top and Bottom of the row.                              */
 /*********************************************************************/
 SelectionRect->top    = CnrQueryRowPosInDV (pRecord, PT_TOP);
 SelectionRect->bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);

 /*********************************************************************/
 /* Adjust all sides of the rectangle to take into account the        */
 /* distance between the columns and the distance between the rows    */
 /*********************************************************************/
 SelectionRect->left   -= (CxBetweenCols / 2);
 SelectionRect->right  += (CxBetweenCols / 2);
 SelectionRect->top    -= (CyBetweenRows / 2);
 SelectionRect->bottom += (CyBetweenRows / 2);

 /******************************************************************/
 /* Now adjust the selection rectangle to map to the window.       */
 /******************************************************************/
 SelectionRect->left   -= *WindowOrgx;
 SelectionRect->right  -= *WindowOrgx;
 SelectionRect->bottom -= pccinfon->ptlNonIconOrg.y;
 SelectionRect->top    -= pccinfon->ptlNonIconOrg.y;

 /***************************************************************/
 /* Get the last FieldInfo (column) in the list.                */
 /***************************************************************/
 pLast = pccinfon->pFirstVisibleFI;

 while (NEXTVISIBLEFI(pLast))
 {
    pLast = NEXTVISIBLEFI(pLast);
 }

 /******************************************************************/
 /* First handle the case where there is a left and right window.  */
 /******************************************************************/
 if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
     (CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1)))
 {
    /***************************************************************/
    /* Now check to see if we are processing the last column       */
    /* in one of the windows.  If we are, we have to handle the    */
    /* case where the last column in either window is to the left  */
    /* of the rightmost part of the window.  Therefore, some       */
    /* amount of dead space exists.  We have to add that on to     */
    /* the rectangle.                                              */
    /***************************************************************/
    if (pFieldInfo == pccinfon->pFieldInfoLast)
    {
       if (SelectionRect->right < ViewPort.right)
       {
          SelectionRect->right = ViewPort.right;
       }
    }
    else if (pFieldInfo == pLast)
    {
       if (SelectionRect->right < ViewPort.right)
       {
          SelectionRect->right = ViewPort.right;
       }
    }
 }
 else
 /******************************************************************/
 /* There is no right window, only a left.                         */
 /******************************************************************/
 {
    /***************************************************************/
    /* If we processing the last column, then check for dead space */
    /***************************************************************/
    if (pFieldInfo == pLast)
    {
       if (SelectionRect->right < ViewPort.right)
       {
          SelectionRect->right = ViewPort.right;
       }
    }
 }

 return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrDrawColumnTitlesLeft (hwnd, hps, InvalidRect)

HANDLE hwnd;
HDC    hps;
RECT   InvalidRect;

{
 HANDLE      hwndCnr;
 LPCNRINFON  pccinfon;       /*                                       */
 LPFIELDINFO pFieldInfo;     /*                                       */
 long        xColumnPos;
 long        CxBetweenCols;  /*                                       */
 BOOL        FirstColLeft = FALSE;

 hwndCnr   = PARENTOF (hwnd);
 pccinfon  = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);
 pccinfon->ulFlagSet &= ~FL_RIGHTOFINVRECT;

 pFieldInfo = pccinfon->pFIFirstColLeftInDV;

 if (!pFieldInfo)
 {
    pFieldInfo = pccinfon->pFirstVisibleFI;
    FirstColLeft = TRUE;
 }
 else if (pFieldInfo == pccinfon->pFirstVisibleFI)
 {
    FirstColLeft = TRUE;
 }

 xColumnPos = CnrQueryColumnPosInDV (hwndCnr,
                                     pccinfon->pFirstVisibleFI,
                                     pFieldInfo,
                                     TRUE,
                                     FALSE);

 if (pFieldInfo)
 {
    /******************************************************************/
    /* If the app. specified Details View to be split into 2 windows  */
    /* and pFieldInfoLast is valid, we know to paint Details View in  */
    /* both the left and right window.  pFieldInfoLast indicates the  */
    /* last column in the left window.                                */
    /******************************************************************/
    if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
        (CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1)))
    {
       if (!(CnrProcessTitleData (hwndCnr, hps, pFieldInfo,
                          xColumnPos, InvalidRect, TRUE, FirstColLeft)))
       {
          return (FALSE);
       }

       if (pFieldInfo->cxWidth)
       {
          xColumnPos += pFieldInfo->cxWidth + CxBetweenCols;
       }
       else
       {
          xColumnPos += PFINFROMX(pFieldInfo)->usLongestLine +
                        CxBetweenCols;
       }

       /***************************************************************/
       /* Go to the next column.                                      */
       /***************************************************************/
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

       /***************************************************************/
       /* Now draw the remaining column titles in the left window.    */
       /***************************************************************/
       while ((pFieldInfo) && (pFieldInfo !=
                  NEXTVISIBLEFI(pccinfon->pFieldInfoLast)) &&
                  (!(pccinfon->ulFlagSet & FL_RIGHTOFINVRECT)))
       {
          if (!(CnrProcessTitleData (hwndCnr, hps, pFieldInfo,
                               xColumnPos, InvalidRect, TRUE, FALSE)))
          {
             return (FALSE);
          }

          if (pFieldInfo->cxWidth)
          {
             xColumnPos += pFieldInfo->cxWidth + CxBetweenCols;
          }
          else
          {
             xColumnPos += PFINFROMX(pFieldInfo)->usLongestLine +
                           CxBetweenCols;
          }

          /************************************************************/
          /* Go to the next column.                                   */
          /************************************************************/
          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
       }
    }
    else
    /******************************************************************/
    /* This indicates that either the window is not to be split or    */
    /* pFieldInfoLast field was not valid.  In this case we paint     */
    /* everything in the left window.                                 */
    /******************************************************************/
    {
       if (!(CnrProcessTitleData (hwndCnr, hps, pFieldInfo,
                          xColumnPos, InvalidRect, TRUE, FirstColLeft)))
       {
          return (FALSE);
       }

       if (pFieldInfo->cxWidth)
       {
          xColumnPos += pFieldInfo->cxWidth + CxBetweenCols;
       }
       else
       {
          xColumnPos += PFINFROMX(pFieldInfo)->usLongestLine +
                        CxBetweenCols;
       }

       /***************************************************************/
       /* Go to the next column.                                      */
       /***************************************************************/
       pFieldInfo = NEXTVISIBLEFI(pFieldInfo);

       /***************************************************************/
       /* Now draw the remaining column titles in the left window.    */
       /***************************************************************/
       while ((pFieldInfo) &&
              (!(pccinfon->ulFlagSet & FL_RIGHTOFINVRECT)))
       {
          if (!(CnrProcessTitleData (hwndCnr, hps, pFieldInfo,
                               xColumnPos, InvalidRect, TRUE, FALSE)))
          {
             return (FALSE);
          }

          if (pFieldInfo->cxWidth)
          {
             xColumnPos += pFieldInfo->cxWidth + CxBetweenCols;
          }
          else
          {
             xColumnPos += PFINFROMX(pFieldInfo)->usLongestLine +
                           CxBetweenCols;
          }

          /************************************************************/
          /* Go to the next column.                                   */
          /************************************************************/
          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
       }
    }
 }

 /*********************************************************************/
 /* Turn the flag off when we leave the proc.                         */
 /*********************************************************************/
 pccinfon->ulFlagSet &= ~FL_RIGHTOFINVRECT;

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrDrawColumnTitlesRight (hwnd, hps, InvalidRect)

HANDLE   hwnd;
HDC      hps;
RECT     InvalidRect;

{
 HANDLE      hwndCnr;
 LPCNRINFON  pccinfon;     /*                                         */
 LPFIELDINFO pFieldInfo;
 long        xColumnPos;
 long        CxBetweenCols;

 hwndCnr   = PARENTOF (hwnd);
 pccinfon  = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 CxBetweenCols = 3 * (pccinfon->lAveCharWidth);
 pccinfon->ulFlagSet &= ~FL_RIGHTOFINVRECT;

 if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
     (CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1)))
 {
    /******************************************************************/
    /* Assign pFieldInfo to point to the first column in the right    */
    /* window.                                                        */
    /******************************************************************/
    pFieldInfo = pccinfon->pFIFirstColRightInDV;

    if (!pFieldInfo)
    {
       pFieldInfo = NEXTVISIBLEFI(pccinfon->pFieldInfoLast);
    }

    if (pFieldInfo)
    {
       xColumnPos = CnrQueryColumnPosInDV (hwndCnr,
                             NEXTVISIBLEFI(pccinfon->pFieldInfoLast),
                                           pFieldInfo,
                                           TRUE,
                                           FALSE);

       while ((pFieldInfo) &&
              (!(pccinfon->ulFlagSet & FL_RIGHTOFINVRECT)))
       {
          if (!(CnrProcessTitleData (hwndCnr, hps, pFieldInfo,
                               xColumnPos, InvalidRect, FALSE, FALSE)))
          {
             return (FALSE);
          }

          /************************************************************/
          /* Set the new column position and go to the next column    */
          /************************************************************/
          if (pFieldInfo->cxWidth)
          {
             xColumnPos += pFieldInfo->cxWidth + CxBetweenCols;
          }
          else
          {
             xColumnPos += PFINFROMX(pFieldInfo)->usLongestLine +
                           CxBetweenCols;
          }

          pFieldInfo = NEXTVISIBLEFI(pFieldInfo);
       }
    }
 }

 /*********************************************************************/
 /* Turn the flag off when we leave the proc.                         */
 /*********************************************************************/
 pccinfon->ulFlagSet &= ~FL_RIGHTOFINVRECT;

 return (TRUE);
}
