/* Revision: 11 1.3.1.2 source/ui/cclnt/cnr/cnrs1p5.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS1P5.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS1P5.C - Name/Text view and icon drawing routines for */
/*                       the container control                        */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/17/93  DAB  Initial modifications for NT execution        */
/* DAB2  10/11/95  DAB  Fixed problem with ownerdraw cnr id in mp1    */
/* DAB3  11/26/95  DAB  Adding logic for CRA_SOURCE emphasis          */
/* D6386  1/16/95  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/**********************************************************************/
#include <string.h>
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS1P5_C
#include "cnrpragw.h"
#endif

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrDrawEraseNametext                              */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Draw/Erase Name/Text         */
/*                                                                   */
/* USAGES : External                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints/erases the specific */
/*   item by calling the function DrawNameTextViewItem or            */
/*   CnrPaintBackground if it is the viewport                        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         PRECORDCORE pRecord - pointer to the record structure     */
/*         BOOL blIsDraw - specify draw or erase                     */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR CnrDrawEraseNameText( hwndCnr, hps, precord, blIsDraw)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precord;
BOOL          blIsDraw;
{
  BOOL         blAction = FALSE;   /* flag for the draw/erase process */
  long         yItemTop;           /* top of the record */
  long         yItemBottom;        /* bottom of the record */
  long         left;               /* x pos of the left boundary */
                                   /* of the record */
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  RECT         rctlClient;         /* rectangle of the client area */
  RECT         rctlHScroll;        /* rectangle of the H scroll    */
  RECT         rctlIcon;           /* rectangle of the icon        */
  RECT         rctlText;           /* rectangle of the text        */
  RECT         rctlTitle;          /* rectangle of the title       */
  POINT        iconSize;           /* size of the icon             */

  if (precord && ISRECORDVISIBLE(precord))
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    /*****************************************************************/
    /* Set icon dimension                                            */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_NAME)
    {
      CnrCalcIconSize (hwndCnr, &iconSize);
    }
    else
    {
      iconSize.x = 0;
      iconSize.y = 0;
    }

   /******************************************************************/
   /* Get viewport rect and  H scroll bar rect.                      */
   /* left-bottom point is (0, 0)                                    */
   /******************************************************************/
   CnrQuerySizeRect(hwndCnr, NULL, &rctlTitle, &rctlClient, NULL,
                    &rctlHScroll);

    /*****************************************************************/
    /* Calculate the icon and text rectangle.                        */
    /*****************************************************************/
    CnrQueryIconTextRect(hwndCnr, precord, &iconSize,
                         TRUE, &rctlIcon, &rctlText);

    /*****************************************************************/
    /* Draw/Erase the record if it is in the viewport.               */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_FLOW)
    {
      /***************************************************************/
      /* Set the leftmost point. If it is name view leftmost point   */
      /* is left of the icon rect and if it is text view left of   */
      /* the text rect is the leftmost.                              */
      /***************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        left = rctlIcon.left;
      }
      else
      {
        left = rctlText.left;
      }

      /***************************************************************/
      /* Draw/Erase the record if it is in the viewport              */
      /***************************************************************/
      if ((rctlText.right > 0) && (left < rctlClient.right) )
      {
        blAction = TRUE;
      }
    }
    else
    /*****************************************************************/
    /* Because it is not flowed, check the item y pos. If an item    */
    /* is in the viewport draw/erase it(blAction = TRUE)             */
    /*****************************************************************/
    {
      /***************************************************************/
      /* Get the highest point and lowest point of the item.         */
      /***************************************************************/
      yItemTop = CnrQueryPtOfItemInNTV(hwndCnr, precord,
                                       &iconSize, PT_TOP);

      yItemBottom = CnrQueryPtOfItemInNTV(hwndCnr, precord,
                                       &iconSize, PT_BOTTOM);

      /***************************************************************/
      /* If it is in the viewport draw/erase  it.                    */
      /***************************************************************/
      if ( ((yItemTop > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)) &&
            (yItemTop < (pccinfon->ptlNonIconOrg.y +
             rctlClient.bottom + rctlTitle.bottom)))  ||
           ((yItemBottom > (pccinfon->ptlNonIconOrg.y + rctlTitle.bottom)) &&
            (yItemBottom < (pccinfon->ptlNonIconOrg.y +
             rctlClient.bottom + rctlTitle.bottom))) )
      {
        blAction = TRUE;
      }
    }

    /*****************************************************************/
    /* If blAction is on draw or erase the item                      */
    /*****************************************************************/
    if (blAction)
    {
      /***************************************************************/
      /* If blIsDraw is FALSE or blIsDraw is TRUE and the record is  */
      /* not targeted and not selected, we erase the background.     */
      /***************************************************************/
      if (!blIsDraw)
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrEraseRect(hwndCnr, hps, rctlIcon);
          rctlText.right++;
          CnrEraseRect(hwndCnr, hps, rctlText);
        }
        else
        {
          rctlText.right+= 2 * TXT_SIDEMARGIN;
          rctlText.left-= 2 * TXT_SIDEMARGIN;
          rctlText.bottom++;
          rctlText.top--;
          CnrEraseRect(hwndCnr, hps, rctlText);
        }
      }

      if (blIsDraw)
      {
        if (!(CnrDrawNameTextViewItem (hwndCnr, hps, precord,
                                       &iconSize)))
        {
           return (FALSE);
        }
      }
    }
  }

  return (TRUE);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrDrawNameTextViewItem                           */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Paints Item                  */
/*                                                                   */
/* USAGES : Internal                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints an item.            */
/*   It calculates the text rectangle by using text size             */
/*   (IcnCalcTextSize), max text length(passed as a parameter) and   */
/*   icon rectangle. If LS_OWNERDRAWN bit is set, the WM_DRAWITEM    */
/*   message is sent to the owner of the container.  If the return   */
/*   value is FALSE (owner did not draw record) or LS_OWNERDRAW bit  */
/*   is not set, then control draws the item.                        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC hps - handle of the current presentation space        */
/*         PRECORD precord - pointer to the record to be painted     */
/*         POINT iconSize                                            */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CnrDrawNameTextViewItem( hwndCnr, hps, precord, piconSize)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precord;
LPPOINT       piconSize;
{
  BOOL            blOwnerdrawIcon; /* Flag for the owner draw style   */
  BOOL            blOwnerdrawText; /* Flag for the owner draw style   */
  HANDLE          hwndOwner;       /* handle of the owner window      */
  HBITMAP         hbmDraw;         /* handle of bitmap to be drawn.   */
  DRAWITEMSTRUCT  DrawItem;        /* structure passed to owner window*/
                                   /* for the owner draw              */
  CNRDRAWITEMINFO CnrDrawItemInfo;
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  LPICONRESIZED   pirsDraw;
  RECT            rctlIcon;        /* rectangle of the icon */
  RECT            rctlText;        /* rectangle of the text */
  RECT            rctlIconCur;     /* rectangle of the icon */
  RECT            rctlTextCur;     /* rectangle of the text */
  UINT            wWinID;          /* window ID */                      /*DAB1*/
  int             nSavedDC;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Calculate the icon and text rectangle.                          */
  /*******************************************************************/
  CnrQueryIconTextRect(hwndCnr, precord, piconSize,
                       FALSE, &rctlIcon, &rctlText);

  if (pccinfo->flWindowAttr & CV_TEXT)
  {
    rctlTextCur.left = rctlText.left - TXT_TOPMARGIN;
    rctlTextCur.right = rctlText.right + TXT_TOPMARGIN;
    rctlTextCur.bottom = rctlText.bottom + TXT_TOPMARGIN;
    rctlTextCur.top = rctlText.top - TXT_TOPMARGIN;
  }
  else
  {
    rctlIconCur.left = rctlIcon.left - ICN_EMPHSPACE;
    rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
    rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;
    rctlIconCur.top = rctlIcon.top - ICN_EMPHSPACE +
                         ICN_SIDEMARGIN;
  }


  /*******************************************************************/
  /* If the container has the OWNERDRAW style, then send the         */
  /* WM_DRAWITEM message to the owner.  If, either the owner did     */
  /* not draw the record (returned FALSE), or the container was not  */
  /* OWNERDRAW, then we will paint the record.                       */
  /*******************************************************************/
  blOwnerdrawIcon = FALSE;
  blOwnerdrawText = FALSE;
  hwndOwner = PARENTOF (hwndCnr);
  wWinID = getWindowId( hwndCnr );                                     /*DAB1*/

  if (pccinfo->flWindowAttr & CA_OWNERDRAW)
  {
    nSavedDC = SaveDC (hps);

    /******************************************************************/
    /* Set DrawItem data by calling CnrSetDrawItem (in cnrdfunc.c)    */
    /******************************************************************/
    CnrSetDrawItem (hwndCnr, hps, &DrawItem, 0);

    /******************************************************************/
    /* Set container control ID.                                      */
    /******************************************************************/
    DrawItem.CtlID = wWinID;

    /******************************************************************/
    /* Set up the CnrDrawItemInfo structure, and pass it back in the  */
    /* hItem field.                                                   */
    /******************************************************************/
    COPYNDATATOX(precord);
    CnrDrawItemInfo.pRecord = precord->preccX;
    CnrDrawItemInfo.pFieldInfo = NULL;
    DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

    /*****************************************************************/
    /* If in the name view, draw icon and text. If in the text view  */
    /* draw text only.                                               */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_NAME) {

      /****************************************************************/
      /* Set rcItem as the rectangle of the icon.                     */
      /****************************************************************/
      DrawItem.rcItem = rctlIconCur;

      /****************************************************************/
      /* Set the itemID field.                                        */
      /****************************************************************/
      DrawItem.itemID = CMA_ICON;

      /****************************************************************/
      /* Send WM_DRAWITEM for the owner drawing if the icon.          */
      /****************************************************************/
      blOwnerdrawIcon = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/

      /****************************************************************/
      /* Set rcItem as the rectangle of the text.                     */
      /****************************************************************/
      DrawItem.rcItem = rctlText;

      /****************************************************************/
      /* Set the itemID field.                                        */
      /****************************************************************/
      DrawItem.itemID = CMA_TEXT;

      /****************************************************************/
      /* Send WM_DRAWITEM for the owner drawing if the text.          */
      /****************************************************************/
      blOwnerdrawText = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
    }
    else
    {
      /***************************************************************/
      /* Set rcItem as the rectangle of the text.                    */
      /* In text view, we need to provide the space for the cursored */
      /* emphasis and target emphasis.                               */
      /***************************************************************/
      DrawItem.rcItem = rctlTextCur;

      /****************************************************************/
      /* Set the itemID field.                                        */
      /****************************************************************/
      DrawItem.itemID = CMA_TEXT;

      /****************************************************************/
      /* Send WM_DRAWITEM for the owner drawing if the text.          */
      /****************************************************************/
      blOwnerdrawText = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/

    } /* endif */

    if (nSavedDC)
      RestoreDC (hps, nSavedDC);
  }


  if (pccinfo->flWindowAttr & CV_NAME) {
    if (!blOwnerdrawIcon)
    {
      /****************************************************************/
      /* Get the information to use for drawing                       */
      /****************************************************************/
      CnrSetupDrawInfo( hwndCnr, precord, &hbmDraw, &pirsDraw );

      /****************************************************************/
      /* If no icon or bitmap to draw exists and WM_DRAWITEM is not   */
      /* sent before, send WM_DRAWITEM to the owner of the container  */
      /* for the app to provide a bitmap or icon or to draw it.       */
      /* (James Taylor requirement)                                   */
      /****************************************************************/
      if (!(hbmDraw || pirsDraw) &&
          !(pccinfo->flWindowAttr & CA_OWNERDRAW))
      {
        CnrSetDrawItem (hwndCnr, hps, &DrawItem, CMA_ICON);

        /**************************************************************/
        /* Set container control ID.                                  */
        /**************************************************************/
        DrawItem.CtlID = wWinID;

        /**************************************************************/
        /* Set rcItem as the rectangle of the icon.                   */
        /**************************************************************/
        DrawItem.rcItem = rctlIconCur;

        /*************************************************************/
        /* Set up the CnrDrawItemInfo structure, and pass it back in */
        /* hItem field.                                              */
        /*************************************************************/
        COPYNDATATOX(precord);
        CnrDrawItemInfo.pRecord = precord->preccX;
        CnrDrawItemInfo.pFieldInfo = NULL;
        DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

        blOwnerdrawIcon = SENDMESSAGE( hwndOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
        if (!blOwnerdrawIcon)
        {
          CnrSetupDrawInfo( hwndCnr, precord, &hbmDraw, &pirsDraw );
        }
      }

      if ((hbmDraw || pirsDraw ) && (!blOwnerdrawIcon))
      {
        IcnDrawIcon (hps, pirsDraw, hbmDraw,
                     precord->flRecordAttr, rctlIcon, pccinfon);
      }

      if ((precord->flRecordAttr & CRA_CURSORED) && (!blOwnerdrawIcon))
      {
        /**************************************************************/
        /* Draw a box around the icon if necessary.                   */
        /**************************************************************/
        CnrDrawCursorBox(hwndCnr, hps, precord, rctlIconCur, TRUE);
      }
    } /* endif ! Owner draw */

    if (!blOwnerdrawText)
    {
      IcnDrawText (hps, GETLPSTRFORNV(precord->preccX), rctlText,
                   precord->flRecordAttr, IS_SIDETEXT, pccinfon);

    } /* endif ! Owner draw */

  } else {

    if (!blOwnerdrawText)
    {
      IcnDrawText (hps, GETLPSTRFORTV(precord->preccX), rctlText,
                   precord->flRecordAttr, IS_SIDETEXT, pccinfon);

      /***************************************************************/
      /* In text view, we need to provide the cursored emphasis,     */
      /* and target emphasis on the text by drawing proper box       */
      /* around.                                                     */
      /***************************************************************/
      if (precord->flRecordAttr & CRA_CURSORED)
      {
        /**************************************************************/
        /* Draw a box around the icon if necessary.                   */
        /**************************************************************/
        CnrDrawCursorBox(hwndCnr, hps, precord, rctlTextCur, TRUE);
      }
    } /* endif ! Owner draw */
  } /* endif */

  return (TRUE);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrPaintNameTextView                              */
/*                                                                   */
/* USAGES : External                                                 */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Paint Name/Text View         */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints the name/text view. */
/*   BeginPaint API is used to get the painting region.  The         */
/*   first-item-in-view value in the container data structure is     */
/*   used as the starting record to begin painting.                  */
/*   Painting is then done until the position of the next record     */
/*   indicates that it is out of view.                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE WndHdl - handle to the container window            */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL: TRUE - the records were painted successfully.        */
/*                                                                   */
/* EXIT-ERROR: FALSE - an error occurred.                            */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR  CnrPaintNameTextView (hwndCnr, hps, prctlWindow)
HANDLE        hwndCnr;
HDC           hps;
LPRECT        prctlWindow;
{
  long          yItemTop;          /* Top of the record */
  long          yItemBottom;       /* Bottom of the record */
  long          left;              /* Left x pos of the record */
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  RECT          rctlClient;        /* rectangle of the client area */
  RECT          rctlHScroll;       /* rectangle of the H scroll bar */
  RECT          rctlIcon;          /* rectangle of the icon */
  RECT          rctlText;          /* rectangle of the text */
  LPRECORDCOREN drawingRecord;     /* the record to be drawn */
  POINT         iconSize;          /* size of the icon */

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect and  H scroll bar rect.                       */
  /* left-bottom point is (0, 0)                                     */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, NULL, &rctlClient, NULL,
                   &rctlHScroll);

  /*******************************************************************/
  /* Find the first item to be drawn and set icon dimensions.        */
  /*******************************************************************/
  drawingRecord = CnrGetFirstItemInRect(hwndCnr, *prctlWindow);

#if 0
  if (pccinfo->flWindowAttr & CV_NAME)
  {
    if (pccinfo->flWindowAttr & CV_FLOW)
    {
      if (pccinfon->preccFirstinFNV)
      {
        drawingRecord = pccinfon->preccFirstinFNV;
      }
      else
      {
        drawingRecord = pccinfon->preccItem;
      }
    }
    else
    {
      if (pccinfon->preccFirstinNV)
      {
        drawingRecord = pccinfon->preccFirstinNV;
      }
      else
      {
        drawingRecord = pccinfon->preccItem;
      }
    }

    CnrCalcIconSize (hwndCnr, &iconSize);
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_FLOW)
    {
      if (pccinfon->preccFirstinFTV)
      {
        drawingRecord = pccinfon->preccFirstinFTV;
      }
      else
      {
        drawingRecord = pccinfon->preccItem;
      }
    }
    else
    {
      if (pccinfon->preccFirstinTV)
      {
        drawingRecord = pccinfon->preccFirstinTV;
      }
      else
      {
        drawingRecord = pccinfon->preccItem;
      }
    }
    iconSize.x = 0;
    iconSize.y = 0;
  }
#endif

  if (drawingRecord)
  {
    /*****************************************************************/
    /* Paint records until positioning takes you out of the viewport.*/
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_FLOW)
    {
      /***************************************************************/
      /* Paint the records which are in the viewport                 */
      /***************************************************************/
      while (drawingRecord)
      {
        if (ISRECORDVISIBLE(drawingRecord))
        {
          CnrQueryIconTextRect (hwndCnr, drawingRecord, &iconSize, TRUE,
                                &rctlIcon, &rctlText);
          /***********************************************************/
          /* Set the leftmost point. If it is name view leftmost     */
          /* point is left of the icon rect and if it is text        */
          /* view left of the text rect is the leftmost.             */
          /***********************************************************/
          if (pccinfo->flWindowAttr & CV_NAME)
          {
            left = rctlIcon.left;
          }
          else
          {
            left = rctlText.left;
          }

          if ((rctlText.right > prctlWindow->left) &&
              (left < prctlWindow->right) )
          {
            if (!(CnrDrawNameTextViewItem (hwndCnr, hps, drawingRecord,
                                           &iconSize)))
            {
               return (FALSE);
            }
          }

          /***********************************************************/
          /* If the item is to the right of (out of)the invalidated  */
          /* rectangle, don't need to check the rest of the items    */
          /* because all of them are to the right of the window.     */
          /***********************************************************/
          if (left >= prctlWindow->right)
          {
            break;
          }
        }
        drawingRecord = drawingRecord->preccNextItem;
      }  /* end while */
    }
    else
    /*****************************************************************/
    /* Because it is not flowed, check from the first item. If an    */
    /* item is in the viewport draw it.                              */
    /*****************************************************************/
    {
      while (drawingRecord)
      {
        if (ISRECORDVISIBLE(drawingRecord))
        {
          /***********************************************************/
          /* Get the highest point and lowest point of the item.     */
          /***********************************************************/
          yItemTop = CnrQueryPtOfItemInNTV(hwndCnr, drawingRecord,
                                           &iconSize, PT_TOP);

          yItemBottom = CnrQueryPtOfItemInNTV(hwndCnr, drawingRecord,
                                           &iconSize, PT_BOTTOM);

          if ((yItemTop <
               (prctlWindow->bottom + pccinfon->ptlNonIconOrg.y))  &&
               (yItemBottom > (pccinfon->ptlNonIconOrg.y +
               prctlWindow->top)) )
          {
            if (!(CnrDrawNameTextViewItem (hwndCnr, hps, drawingRecord,
                                           &iconSize)))
            {
               return (FALSE);
            }
          }

          /***********************************************************/
          /* If the item is below of (out of)the invalidated rect, we*/
          /* don't need to check the rest of the items because all   */
          /* of them are   below         the window.                 */
          /***********************************************************/
          if (yItemTop >=
              (prctlWindow->bottom + pccinfon->ptlNonIconOrg.y))
          {
            break;
          }
        }
        drawingRecord = drawingRecord->preccNextItem;
      }  /* end while */
    }
  }

  return (TRUE);
}

/*********************************************************************/
/* FUNCTION NAME:  CnrDrawIcon                                       */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Draw Icon                    */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints the specific icon   */
/*   by calling the function DrawIcon if it is in the viewport.      */
/*   It changes the Z-order if the item overlaps other items         */
/*   and flChangeZorder is TRUE                                      */
/*   The Z-order of the item is on the top of the items overlapped.  */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         PRECORDCORE pRecord - pointer to the record structure     */
/*         BOOL flChangeZorder - flag to tell to change z-order      */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR  CnrDrawIcon( hwndCnr, hps, precord, flChangeZorder)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precord;
BOOL          flChangeZorder;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN prcZ;            /* record to be checked if it is */
                                 /* overlapped by the new item to be drawn */
  RECT        windowRect;        /* rectangle of the window */
  RECT        iconRect;          /* rectangle of the icon */
  RECT        textRect;          /* rectangle of the text */
  RECT        arcl[2];           /* array of the rectangles of the */
                                 /* record to be drawn */
  RECT        rclDest;           /* intersection of the rectangles */
  POINT       iconSize;          /* size of the icon */

  if (!precord)
  {
    return (FALSE);
  }

  if (precord && ISRECORDVISIBLE(precord))
  {
    PMGetClientRect(hwndCnr, &windowRect);
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    /*****************************************************************/
    /* Get icon size.                                                */
    /*****************************************************************/
    CnrQuerySizeRect(hwndCnr, &iconSize, NULL, NULL, NULL, NULL);

    /*****************************************************************/
    /* If the flag flChangeZorder is TRUE, we change the z-order so  */
    /* that the z order of the item to be drawn is the next of the   */
    /* topmost item on the rectangle on which the new item will be   */
    /* drawn. (then new item becomes the topmost item.)              */
    /*****************************************************************/
    if (flChangeZorder &&
        !(GetWindowLong(hwndCnr, GWL_STYLE) & CCS_AUTOPOSITION))
    {
      LPMARQUEEREC  pmrMarquee;

      /***************************************************************/
      /* Query the icon and text rectangle of the item to be drawn.  */
      /***************************************************************/
      CnrQueryIconTextRect(hwndCnr, precord, &iconSize,
                           TRUE, &iconRect, &textRect);

      /***************************************************************/
      /* Array of the rectangles of the item to be drawn. It will be */
      /* used to search the topmost item on those rectangles.        */
      /***************************************************************/
      arcl[0] = textRect;
      arcl[1] = iconRect;

      /***************************************************************/
      /* Set prcZ as the topmost item in the list.                   */
      /***************************************************************/
      if (!pccinfon->fFirstMouseMove)
      {
        pmrMarquee = pccinfon->pmrFirstMarquee;
        prcZ = pmrMarquee->pRecord;
      }
      else
      {
        prcZ = pccinfon->preccZTail;
      }

      /***************************************************************/
      /* Find the first item from the tail of the z-order which      */
      /* overlaps the rectangles of the new item to be drawn.        */
      /***************************************************************/
      while (prcZ)
      {

        /*************************************************************/
        /* Query the icon and text rectangle of the item to be       */
        /* compared.                                                 */
        /*************************************************************/
        CnrQueryIconTextRect(hwndCnr, prcZ, &iconSize,
                             FALSE, &iconRect, &textRect);

        /*************************************************************/
        /* Check the rectangles if they are overlapped. If they are, */
        /* then change the Z-order.                                  */
        /* The new item is placed on the top of the topmost item of  */
        /* the items which are overlapped by the new item.           */
        /*************************************************************/
        if (PMIntersectRect(&rclDest, &arcl[0], &textRect) ||
            PMIntersectRect(&rclDest, &arcl[0], &iconRect) ||
            PMIntersectRect(&rclDest, &arcl[1], &textRect) ||
            PMIntersectRect(&rclDest, &arcl[1], &iconRect) )
        {
          /***********************************************************/
          /* if overlapped item is itself, don't change the Z-order  */
          /***********************************************************/
          if (precord != prcZ) {

            /*********************************************************/
            /* Delete the record to be drawn from the current z-drder*/
            /*********************************************************/
            if (precord->preccPrevZ)
            {
              precord->preccPrevZ->preccNextZ = precord->preccNextZ;
            }
            else
            {
              pccinfon->preccZ = precord->preccNextZ;
            } /* endif */

            if (precord->preccNextZ)
            {
              precord->preccNextZ->preccPrevZ = precord->preccPrevZ;
            }
            else
            {
              pccinfon->preccZTail = precord->preccPrevZ;
            }

            /**********************************************************/
            /* Insert the record to be drawn to the top of the topmost*/
            /* record of records to be overlapped by the new record.  */
            /**********************************************************/
            precord->preccPrevZ = prcZ;
            precord->preccNextZ = prcZ->preccNextZ;

            if (prcZ->preccNextZ)
            {
              prcZ->preccNextZ->preccPrevZ = precord;
            }
            else
            {
              pccinfon->preccZTail = precord;
            }

            prcZ->preccNextZ = precord;

            if (!pccinfon->fFirstMouseMove)
            {
              LPMARQUEEREC  pmrCurrent;
              BOOL         fExit=FALSE;

              pmrCurrent = pmrMarquee;
              while(pmrCurrent && !fExit)
              {
                if (precord == pmrCurrent->pRecord)
                {
                  fExit = TRUE;
                }
                else
                {
                  pmrCurrent = pmrCurrent->pmrNextItem;
                }
              }
              if (pmrCurrent != pmrMarquee)
              {
                /* First remove pmrCurrent from list */
                if (pmrCurrent->pmrPrevItem && pmrCurrent->pmrNextItem)
                {
                  (pmrCurrent->pmrPrevItem)->pmrNextItem =
                                                pmrCurrent->pmrNextItem;
                  (pmrCurrent->pmrNextItem)->pmrPrevItem =
                                                pmrCurrent->pmrPrevItem;
                }
                else
                {
                  if (pmrCurrent->pmrPrevItem)
                  {
                    (pmrCurrent->pmrPrevItem)->pmrNextItem = NULL;
                    pccinfon->pmrLastMarquee = pmrCurrent->pmrPrevItem;
                  }
                  else
                  {
                    (pmrCurrent->pmrNextItem)->pmrPrevItem = NULL;
                    pccinfon->pmrFirstMarquee = pmrCurrent->pmrNextItem;
                  }
                }
                /* Then insert pmrCurrent into new position */
                pmrCurrent->pmrNextItem = pmrMarquee;
                if (pmrMarquee->pmrPrevItem)
                {
                  (pmrMarquee->pmrPrevItem)->pmrNextItem = pmrCurrent;
                  pmrCurrent->pmrPrevItem = pmrMarquee->pmrPrevItem;
                }
                else
                {
                  pccinfon->pmrFirstMarquee  = pmrCurrent;
                  pmrCurrent->pmrPrevItem = NULL;
                }
                pmrMarquee->pmrPrevItem = pmrCurrent;
              }
            }

          } /* endif */
          break;

        } /* endif */
        if (!pccinfon->fFirstMouseMove)
        {
          pmrMarquee = pmrMarquee->pmrNextItem;
          prcZ = pmrMarquee->pRecord;
        }
        else
        {
          prcZ = prcZ->preccPrevZ;
        }
      } /* endwhile */
    } /* endif */

    /*****************************************************************/
    /* Paint the record if it is in the viewport.                    */
    /*****************************************************************/
    if (IsInIconViewport(precord, &(pccinfo->ptlOrigin),
                         &windowRect, &iconSize))

    {
      if (!(xDrawIcon( hwndCnr, hps, precord, &iconSize)))
      {
         return (FALSE);
      }
    }
  } /* endif */

  return (TRUE);
}

/*********************************************************************/
/*                                                                   */
/*   FUNCTION NAME:  CnrEraseIcon                                    */
/*                                                                   */
/*   Usage: External                                                 */
/*                                                                   */
/*   DESCRIPTIVE NAME:  Container Control Erase Icon                 */
/*                                                                   */
/*   FUNCTIONAL DESCRIPTION: This function erase an item in the      */
/*      icon view. If flag flRedraw is TRUE, redraw all the items    */
/*      which are overlapped with the erased item.                   */
/*                                                                   */
/*   NOTES:                                                          */
/*      FORWARD DECLARATION:                                         */
/*                                                                   */
/*   INPUT:  HANDLE hwndCnr - handle to the container window         */
/*           PRECORDCORE pRecord - pointer to the record structure   */
/*           BOOL flRedraw - flag to tell to redraw the item         */
/*                           which were overlapped with erased item  */
/*                                                                   */
/*   OUTPUT:                                                         */
/*                                                                   */
/*                                                                   */
/*   EXIT-NORMAL:                                                    */
/*                                                                   */
/*   EXIT-ERROR:                                                     */
/*                                                                   */
/*   SIDE EFFECTS:                                                   */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrEraseIcon (hwndCnr, hps, precord, flRedraw)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precord;
BOOL          flRedraw;
{
  HRGN           regionHdl = NULL; /* handle of the new clipping */
                                   /* region */
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  prcZ;             /* record to be compared */
  RECT           iconRect;         /* rectangle of the icon */
  RECT           textRect;         /* rectangle of the text */
  RECT           arcl[3];          /* array of the rectangles of the */
                                   /* erased item */
  RECT           rclDest;          /* intersection of the rectangles */
  RECT           windowRect;       /* rectangle of the window */
  POINT          iconSize;         /* size of the icon */
  int            nSavedDC;


  PMGetClientRect(hwndCnr, &windowRect);
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size.                                                  */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, NULL, NULL, NULL, NULL);

  /*******************************************************************/
  /* Erase the record if it is in the viewport.                      */
  /*******************************************************************/
  if (IsInIconViewport(precord, &(pccinfo->ptlOrigin),
                       &windowRect, &iconSize))
  {
    /*****************************************************************/
    /* Calculate text size                                           */
    /*****************************************************************/
    CnrQueryIconTextRect(hwndCnr, precord, &iconSize,
                         TRUE, &iconRect, &textRect);

    textRect.right++;

    /*****************************************************************/
    /* Save the rectangles to be erased in the array.                */
    /*****************************************************************/
    arcl[1].right = textRect.right + 2 * TXT_SIDEMARGIN;
    arcl[1].left = textRect.left - TXT_SIDEMARGIN;
    arcl[1].top = textRect.top - 2 * TXT_TOPMARGIN;
    arcl[1].bottom = textRect.bottom + TXT_TOPMARGIN;
    arcl[2].right = iconRect.right + 1;
    arcl[2].left = iconRect.left - 1;
    arcl[2].top = iconRect.top - 1;
    arcl[2].bottom = iconRect.bottom + 1;

    /*****************************************************************/
    /* Save the current DC attributes.                               */
    /*****************************************************************/
    nSavedDC = SaveDC (hps);

    /*****************************************************************/
    /* Create the clipping region. Save old region as oldRegionHdl   */
    /*****************************************************************/
    UnionRect (&arcl[0], &arcl[1], &arcl[2]);
    if (regionHdl = CreateRectRgn (arcl[0].left, arcl[0].top,
                               arcl[0].right, arcl[0].bottom))
    {
      SelectClipRgn (hps, regionHdl);
    }

    CnrEraseRect(hwndCnr, hps, iconRect);
    CnrEraseRect(hwndCnr, hps, textRect);

    /*****************************************************************/
    /* If flRedraw is TRUE, redraw the overlapped items.             */
    /*****************************************************************/
    if ((flRedraw) &&
        !(GetWindowLong(hwndCnr, GWL_STYLE) & CCS_AUTOPOSITION))
    {
      /***************************************************************/
      /* Set prcZ as the bottommost item.                            */
      /***************************************************************/
      prcZ = pccinfon->preccZ;

      /***************************************************************/
      /* Redraw the overlapped items from top to bottom.             */
      /***************************************************************/
      while (prcZ)
      {
        /*************************************************************/
        /* If the item to be checked is the item erased, we don't    */
        /* draw again.                                               */
        /*************************************************************/
        if (prcZ != precord)
        {
          CnrQueryIconTextRect(hwndCnr, prcZ, &iconSize,
                               TRUE, &iconRect, &textRect);

          /***********************************************************/
          /* If the item checked was overlapped, redraw it and       */
          /* it's text and icon rectangle from clipping region so    */
          /* that the item under it is not drawn over it.            */
          /***********************************************************/
          if (PMIntersectRect(&rclDest, &arcl[1], &textRect) ||
              PMIntersectRect(&rclDest, &arcl[1], &iconRect) ||
              PMIntersectRect(&rclDest, &arcl[2], &textRect) ||
              PMIntersectRect(&rclDest, &arcl[2], &iconRect) )
          {
            EraseTextOnlyInIV(hwndCnr, hps, prcZ);
            xDrawIcon (hwndCnr, hps, prcZ, &iconSize);
          }
        }
        prcZ = prcZ->preccNextZ;
      }
    }

    /*****************************************************************/
    /* Restore the device context.                                   */
    /*****************************************************************/
    if (nSavedDC)
      RestoreDC (hps, nSavedDC);

    /*****************************************************************/
    /* Destroy the current clipping region                           */
    /*****************************************************************/
    if (regionHdl)
      DeleteObject (regionHdl);
  } /* endif */

  return;
}

/*********************************************************************/
/*                                                                   */
/*  FUNCTION NAME:  xDrawIcon                                        */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/*  DESCRIPTIVE NAME:  Container Control Draw Icon                   */
/*                                                                   */
/*  FUNCTIONAL DESCRIPTION:  This function paints the specific icon  */
/*    with given information as parameters by calling the functions  */
/*    IcnDrawIcon and IcnDrawText.                                   */
/*    If the container specifies LS_OWNERDRAW, then the WM_DRAWITEM  */
/*    message is sent to the owner of the container.  If the return  */
/*    value is FALSE (owner did not draw record), then the control   */
/*    draw it.                                                       */
/*                                                                   */
/*  NOTES:                                                           */
/*     FORWARD DECLARATION:                                          */
/*                                                                   */
/*  INPUT:  HANDLE hwndCnr - handle to the container window          */
/*          HDC hps - handle of the presentation space               */
/*          PRECORDCORE pRecord - pointer to the record structure    */
/*          LPPOINT pslIcon - size of an icon rectangle              */
/*                                                                   */
/*  OUTPUT:                                                          */
/*                                                                   */
/*                                                                   */
/*  EXIT-NORMAL:                                                     */
/*                                                                   */
/*  EXIT-ERROR:                                                      */
/*                                                                   */
/*  SIDE EFFECTS:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR  xDrawIcon( hwndCnr, hps, pRecord, pslIcon)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN pRecord;
LPPOINT       pslIcon;

{
  BOOL            OwnerDrawnIcon;
  BOOL            OwnerDrawnText;
  HANDLE          WindowOwner;
  HBITMAP         hbmDraw;         /* handle of the bitmap to be drawn */
  DRAWITEMSTRUCT  DrawItem;        /* info to be passed to the owner   */
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  LPICONRESIZED   pirsDraw;        /* ptr to the resized icon to be drawn */
  RECT            iconRect;        /* rectangle of the icon */
  RECT            textRect;        /* rectangle of the text */
  RECT            rctlBack;
  UINT            wWinID;          /* ID of the window      */          /*DAB1*/
  CNRDRAWITEMINFO CnrDrawItemInfo;
  int             nSavedDC;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /******************************************************************/
  /* Query the icon and the text rectangles.                        */
  /******************************************************************/
  CnrQueryIconTextRect(hwndCnr, pRecord, pslIcon,
                       FALSE, &iconRect, &textRect);

  /******************************************************************/
  /* Draw a box around the icon if necessary.                       */
  /******************************************************************/
  rctlBack.left = iconRect.left - ICN_EMPHSPACE;
  rctlBack.bottom = iconRect.bottom + ICN_EMPHSPACE;

  /******************************************************************/
  /* Note: Draw Bitmap doesn't paint top row and rightmost clmn     */
  /*    so, we need to take out those space by ICN_SIDEMARGIN       */
  /*    and ICN_TOPMARGIN                                           */
  /******************************************************************/
  rctlBack.right = iconRect.right + ICN_EMPHSPACE;
  rctlBack.top = iconRect.top - ICN_EMPHSPACE +
                 ICN_SIDEMARGIN;

  /******************************************************************/
  /* If the container has the OWNERDRAW style, then send the        */
  /* WM_DRAWITEM message to the owner.  If, either the owner did    */
  /* not draw the record (returned FALSE), or the container was     */
  /* not OWNERDRAW, then we will paint the record.                  */
  /******************************************************************/
  OwnerDrawnIcon = FALSE;
  OwnerDrawnText = FALSE;

  /****************************************************************/
  /* Get the owner of the window.                                 */
  /****************************************************************/
  WindowOwner = PARENTOF (hwndCnr);

  /****************************************************************/
  /* Get the ID of the container window.                          */
  /****************************************************************/
  wWinID = getWindowId( hwndCnr );                                    /*DAB1*/

  if (pccinfo->flWindowAttr & CA_OWNERDRAW)
  {
    nSavedDC = SaveDC (hps);
    /****************************************************************/
    /* Set the DrawItem data.                                       */
    /****************************************************************/
    CnrSetDrawItem (hwndCnr, hps, &DrawItem, CMA_ICON);

    /******************************************************************/
    /* Set container control ID.                                      */
    /******************************************************************/
    DrawItem.CtlID = wWinID;

    /*******************************************************************/
    /* Set up the CnrDrawItemInfo structure, and pass it back in the   */
    /* hItem field.                                                    */
    /*******************************************************************/
    COPYNDATATOX(pRecord);
    CnrDrawItemInfo.pRecord = pRecord->preccX;
    CnrDrawItemInfo.pFieldInfo = NULL;
    DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

    /****************************************************************/
    /* Set rcItem as the rectangle of the icon.                     */
    /****************************************************************/
    DrawItem.rcItem = rctlBack;

    /****************************************************************/
    /* Send WM_DRAWITEM for the owner drawing if the icon.          */
    /****************************************************************/
    OwnerDrawnIcon = SENDMESSAGE( WindowOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/

    /****************************************************************/
    /* Set rcItem as the rectangle of the text.                     */
    /****************************************************************/
    DrawItem.rcItem = textRect;

    /****************************************************************/
    /* Reset the itemID field.                                      */
    /****************************************************************/
    DrawItem.itemID = CMA_TEXT;

    /****************************************************************/
    /* Send WM_DRAWITEM for the owner drawing if the text.          */
    /****************************************************************/
    OwnerDrawnText = SENDMESSAGE( WindowOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/

    if (nSavedDC)
      RestoreDC (hps, nSavedDC);
  }

  if (!OwnerDrawnIcon)
  {
    /*****************************************************************/
    /* Get the information to use for drawing                        */
    /*****************************************************************/
    CnrSetupDrawInfo (hwndCnr, pRecord, &hbmDraw, &pirsDraw);

    /****************************************************************/
    /* If no icon or bitmap to draw exists and WM_DRAWITEM is not   */
    /* sent before, send WM_DRAWITEM to the owner of the container  */
    /* for the app to provide a bitmap or icon or to draw it.       */
    /* (James Taylor requirement)                                   */
    /****************************************************************/
    if (!(hbmDraw || pirsDraw) &&
        !(pccinfo->flWindowAttr & CA_OWNERDRAW))
    {
      CnrSetDrawItem (hwndCnr, hps, &DrawItem, CMA_ICON);

      /****************************************************************/
      /* Set container control ID.                                    */
      /****************************************************************/
      DrawItem.CtlID = wWinID;

      /***************************************************************/
      /* Set up the CnrDrawItemInfo structure, and pass it back in   */
      /* hItem field.                                                */
      /***************************************************************/
      COPYNDATATOX(pRecord);
      CnrDrawItemInfo.pRecord = pRecord->preccX;
      CnrDrawItemInfo.pFieldInfo = NULL;
      DrawItem.itemData = (DWORD)(LPVOID)&CnrDrawItemInfo;

      /**************************************************************/
      /* Set rcItem as the rectangle of the icon.                   */
      /**************************************************************/
      DrawItem.rcItem = rctlBack;

      OwnerDrawnIcon = SENDMESSAGE( WindowOwner, WM_DRAWITEM,
                  (WPARAM)wWinID, (DWORD)((LPDRAWITEMSTRUCT)&DrawItem));/*DAB2*/
      if (!OwnerDrawnIcon)
      {
        CnrSetupDrawInfo( hwndCnr, pRecord, &hbmDraw, &pirsDraw );
      }
    }

    if ((hbmDraw || pirsDraw ) && (!OwnerDrawnIcon))
    {
      IcnDrawIcon (hps, pirsDraw, hbmDraw,
                   pRecord->flRecordAttr, iconRect, pccinfon);
    }

    if ((pRecord->flRecordAttr & CRA_CURSORED) && (!OwnerDrawnIcon))
    {
      CnrDrawCursorBox(hwndCnr, hps, pRecord, rctlBack, TRUE);
    }

  } /* endif ! Owner draw */

  if (!OwnerDrawnText)
  {
    IcnDrawText (hps, pRecord->preccX->pszIcon, textRect,
                 pRecord->flRecordAttr, IS_BENEATHTEXT, pccinfon );
  } /* endif ! Owner draw */

  return (TRUE);
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnCalcTextSize                                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Calculate Text Size                    */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function calculate the text size.   */
/*   For the icon with it's text beneath to it, the function         */
/*   calculates the exact text rectangle that is shorter than or     */
/*   equal to the size alloweded by the system control panel or      */
/*   by the app API to set maximum size of the text rectangle.       */
/*   For the icon with the text to the right of it, the function     */
/*   calculates the exact text rectangle without limitation.         */
/*   If no string is specified or string is NULL string,             */
/*       cx = cx of the icon rectangle if text is beneath to the     */
/*            icon                                                   */
/*          = 0 if text is to the right of the icon                  */
/*       cy = 0 if text is beneath to the icon                       */
/*          = lMaxBaselineExt.                                       */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         POINT iconSize - icon size                                */
/*         LPSTR textString - text string that is painted            */
/*         DWORD iconStyle - iocn style                              */
/*         LPPOINT ptextSize - point to the text size in which the   */
/*                           calculated text size saved.             */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR  IcnCalcTextSize(hps,  iconSize, pccinfon,
                          textString, iconStyle, ptextSize)
HDC          hps;
POINT        iconSize;
LPCNRINFON   pccinfon;
LPSTR        textString;
DWORD        iconStyle;
PSIZEL       ptextSize;                                                 /*DAB1*/
{

  long       maxTextWidth;           /* length of longest text line   */
  long       textWidth;              /* length of each line of text   */
  long       maxTextHeight;          /* maximum height of text rect.  */
  long       maxTextlines;           /* maximum lines of text         */
  long       maxLinechars;           /* maximum number of characters  */
  WORD       lineCount;              /* counter of lines              */
  WORD       textIndex;              /* index of the text             */
  WORD       textLength;             /* length of text in # of chars  */
  WORD       usSkipChar;             /* number of the chars to skip   */
  SIZE       textData;               /* contains text width & height  *//*DAB1*/
  HFONT      hOldFont = NULL;        /* save old font                 */

  if ((!textString) || (*textString == '\0') )
  {

    /*****************************************************************/
    /* If no string, give default size                               */
    /*****************************************************************/
    if (iconStyle & IS_BENEATHTEXT)
    {
      /***************************************************************/
      /* Because the upmost row and right-most column of iconRect    */
      /* one pixel should be taken out                               */
      /***************************************************************/
      ptextSize->cx = iconSize.x + ICN_EMPHSPACE - TXT_SIDEMARGIN;      /*DAB1*/
      ptextSize->cy = 0;                                                /*DAB1*/
    }
    if (iconStyle & IS_SIDETEXT)
    {
      ptextSize->cx = 0;                                                /*DAB1*/
      ptextSize->cy = pccinfon->lMaxBaselineExt;                        /*DAB1*/
    }

  }
  else
  {
    if (iconStyle & IS_BENEATHTEXT)
    {
       maxTextlines = 32767;  /* Will catch from control panel later */
       maxLinechars = 32767; /* From the control panel               */

    }
    else
    {
       maxTextlines = 32767;  /* Will catch from control panel later */
       maxLinechars = 32767;  /* From the control panel               */

    } /* endif */

    lineCount = 0;
    textIndex = 0;
    maxTextWidth = 0;

    /*****************************************************************/
    /* Initialize text size as max size possible.                    */
    /*****************************************************************/
    ptextSize->cx = (maxLinechars * pccinfon->lAveCharWidth);           /*DAB1*/
    ptextSize->cy = (maxTextlines * GETCYCHAR(pccinfon));               /*DAB1*/

    /******************************************************************/
    /* Set the user specified font - If NULL use default (System)     */
    /******************************************************************/
    if (pccinfon->hCnrFont)
      hOldFont = SelectObject (hps, pccinfon->hCnrFont);

    /*****************************************************************/
    /* Get the longest text line                                     */
    /*****************************************************************/
    while ((textIndex < (WORD)lstrlen(textString)) &&
           (lineCount < (WORD)maxTextlines))
    {
      textLength = CnrStrcspn(&textString[textIndex],"\r\n\0");
      if (*(textString + (textIndex + textLength )) == '\r')
        usSkipChar = 2;
      else
        usSkipChar = 1;

      /***************************************************************/
      /* Query the rectangle of each line.                           */
      /*     textData - LOWORD = text width.                         */
      /*              - HIWORD = text height.                        */
      /***************************************************************/
      GetTextExtentPoint (hps, (LPSTR)&textString[textIndex],           /*DAB1*/
                          textLength, &textData);                       /*DAB1*/

      /***************************************************************/
      /* The text width is the difference between the right and      */
      /* left boundaries of the text(handling italic fonts).         */
      /***************************************************************/
      textWidth = textData.cx;                                          /*DAB1*/
      maxTextWidth = max( maxTextWidth, textWidth );

      if (maxTextWidth >= ptextSize->cx) {                              /*DAB1*/
         /************************************************************/
         /* the text rectangle width is the max text width           */
         /* allowed if the length of a line od the text is           */
         /* longer than that.                                        */
         /************************************************************/
         maxTextWidth = ptextSize->cx;                                  /*DAB1*/
         lineCount ++;
         break;
      } /* endif */
      textIndex += (textLength + usSkipChar);
      lineCount++;
    } /* end while */

    /******************************************************************/
    /* Reset the old font if it was changed                           */
    /******************************************************************/
    if (hOldFont)
      SelectObject (hps, hOldFont);

    /*****************************************************************/
    /* Get max text rect height.                                     */
    /*****************************************************************/
    if (lineCount > (WORD)maxTextlines)
       lineCount = (WORD)maxTextlines;

    maxTextHeight = lineCount * GETCYCHAR(pccinfon);

    ptextSize->cx = maxTextWidth;                                       /*DAB1*/
    ptextSize->cy = maxTextHeight;                                      /*DAB1*/

    /*****************************************************************/
    /* If the width of rectangle is less than                        */
    /* the width of the icon rect then set it same as icon one.      */
    /* Because the upmost row and right-most column of iconRect      */
    /* one pixel should be taken out                                 */
    /*****************************************************************/
    if ((iconStyle & IS_BENEATHTEXT) &&
        (ptextSize->cx  < iconSize.x + ICN_EMPHSPACE))                  /*DAB1*/
    {
      ptextSize->cx = iconSize.x + ICN_EMPHSPACE - TXT_SIDEMARGIN;      /*DAB1*/
    } /* endif */
  } /* endif */
  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawBox                                        */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Draw a Box                             */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION: Draw a box  around the rectangle given    */
/*    according to the attributes given                              */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         DWORD iconAttr - attribute of the icon                    */
/*         LPRECT prctlBox - rectangle  around which a box is drawn  */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/

VOID FAR  IcnDrawBox( hps, iconAttr, prctlBox)
HDC        hps;
DWORD      iconAttr;
LPRECT     prctlBox;
{
  long       lLineColor;           /* color of the line of the box */
  int        sLineType;            /* type of the line */
  HPEN       hpenCurrent;
  HPEN       hpenOld;
  HBRUSH     hbrushOld;
  int        nSavedDC;
  int        oldBkMode;            /* save old background mode */

  if (iconAttr & (IC_TARGET | IC_CURSORED))
  {
    /*****************************************************************/
    /* Select Color and type of line.                                */
    /*****************************************************************/
    if (iconAttr & IC_TARGET)
    {
      lLineColor = BLACK_PEN;
      sLineType = PS_SOLID;
    }
    else
    {
      if (iconAttr & IC_CURSORED)
      {
        lLineColor = BLACK_PEN;
        sLineType = PS_DOT;
        oldBkMode = SetBkMode (hps, TRANSPARENT);
      } /* endif */
    } /* endif */

    /*****************************************************************/
    /* Save current PS attributes                                    */
    /*****************************************************************/
    if (!(nSavedDC = SaveDC (hps)))
      return;

    /*****************************************************************/
    /* Select the pen color & line type for the lines of the box     */
    /*****************************************************************/
    if (hpenCurrent = CreatePen (sLineType, 1, lLineColor))
    {
      hpenOld = SelectObject (hps, hpenCurrent);

      /*****************************************************************/
      /* Draw the box from the current pos to the top right pos        */
      /*****************************************************************/
      hbrushOld = SelectObject (hps, GetStockObject (NULL_BRUSH));
      Rectangle (hps, prctlBox->left, prctlBox->top, prctlBox->right,
                 prctlBox->bottom);

      /*****************************************************************/
      /* Select the old brush                                          */
      /*****************************************************************/
      SelectObject (hps, hbrushOld);

      /*****************************************************************/
      /* Select the old pen color and free resource.                   */
      /*****************************************************************/
      DeleteObject (SelectObject (hps, hpenOld));

      /*****************************************************************/
      /* Select the old background mode                                */
      /*****************************************************************/
      SetBkMode (hps, oldBkMode);
    }

    /*****************************************************************/
    /* Restore the previous PS attributes                            */
    /*****************************************************************/
    RestoreDC (hps, nSavedDC);
  } /* endif */

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawIcon                                       */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Draw an Icon                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints an icon.            */
/*   It calculates the rectangle that includes icon rectangle and    */
/*   the white space for the selection emphasis, if the item is      */
/*   selected/in_use, draw selection/in_use emphasis and draw the    */
/*   graphic object using icon (if pirsIcon is given) or bitmap      */
/*   (if pirs is NULL and hbmIcon is given.).                        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         PICONRESIZED pirsIcon - resized icon                      */
/*         HBITMAP hbmIcon - bitmap to be drawn if pirsIcon is NULL  */
/*         DWORD iconAttr - attr of the item                         */
/*         RECT iconRect - rectangle of the icon                     */
/*         PCNRINFON pccinfon - pointer to internal container Info   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR IcnDrawIcon (hps, pirsIcon, hbmIcon, iconAttr, iconRect,
                      pccinfon)
HDC           hps;
LPICONRESIZED pirsIcon;
HBITMAP       hbmIcon;
DWORD         iconAttr;
RECT          iconRect;
LPCNRINFON    pccinfon;
{
  RECT       rctlBack;
  BOOL       flIsForIcon;
  int        oldFgndMode;         /* save old foreground mode */
  int        oldBkMode;           /* save old background mode */
  HPEN       hpenOld = NULL;      /* save old pen */
  HPEN       hpenCurrent;
  HBRUSH     hbrushOld;           /* save old brush */
  HBRUSH     hbrushCurrent;
  BITMAP     BitmapHdr;
  HBITMAP    hbmOld;              /* Old bitmap handle */
  HDC        hdcMemory;           /* handle to memory device context */
  int        nSavedDC;

  /*******************************************************************/
  /* If it not filtered out ..........                               */
  /*******************************************************************/
  if (!(iconAttr & IC_FILTERED))
  {
     /****************************************************************/
     /* Get rectangle for the selection/in_use emphasis.             */
     /****************************************************************/
     rctlBack.left = iconRect.left - ICN_SELSPACE;
     rctlBack.right = iconRect.right + ICN_SELSPACE;
     rctlBack.bottom = iconRect.bottom + ICN_SELSPACE;
     rctlBack.top = iconRect.top - ICN_SELSPACE +
                    ICN_SIDEMARGIN;


     /****************************************************************/
     /* Save current PS attributes.                                  */
     /****************************************************************/
     if (!(nSavedDC = SaveDC (hps)))
       return;

     /****************************************************************/
     /* Set current foreground mix mode which the foreground color   */
     /* replaces the existing color.                                 */
     /****************************************************************/
     oldFgndMode = SetROP2 (hps, R2_COPYPEN);

     /****************************************************************/
     /* Set current background mix mode which the background color   */
     /* is ignored and the existing color remains unchanged.         */
     /****************************************************************/
     oldBkMode = SetBkMode (hps, TRANSPARENT);

     /****************************************************************/
     /* If the item is selected........                              */
     /****************************************************************/
     if (iconAttr & IC_SELECTED)
     {
       /**************************************************************/
       /* Set current foreground color as HILITEFOREGROUND color     */
       /**************************************************************/
       if (!(hpenCurrent = CreatePen (PS_SOLID, 1,
                pccinfon->ulColorPPSupport[CMA_SELECTFOREGROUNDCOLOR])))
       {
         goto Err;
       }

       hpenOld = SelectObject (hps, hpenCurrent);

       /**************************************************************/
       /* If pirsIcon is given, paint whole rctlBack with the        */
       /* hilited background color because the icon will be          */
       /* transparent. Otherwise, draw only two boxes with hilited   */
       /* background color around iconRect for sel emph because      */
       /* whatever in iconRect will be overpainted by the icon.      */
       /**************************************************************/
       if (pirsIcon)
       {
         flIsForIcon = TRUE;
       }
       else
       {
         flIsForIcon = FALSE;
       } /* endif */

       IcnDrawSelectEmph(hps, rctlBack, flIsForIcon, pccinfon);
     } /* endif */

     /****************************************************************/
     /* If the item is in use..........                              */
     /****************************************************************/
     if (iconAttr & IC_INUSE)
     {
       /**************************************************************/
       /* Draw in_use emphasis on rctlBack.                          */
       /**************************************************************/
       IcnDrawInUseEmph(hps, rctlBack, pccinfon);
     } /* endif */

     /****************************************************************/
     /* Select the old pen color and free resource.                  */
     /****************************************************************/
     if (hpenOld)
        DeleteObject (SelectObject (hps, hpenOld));

     /****************************************************************/
     /* If pirs is given...............                              */
     /****************************************************************/
     if (pirsIcon)
     {
       /**************************************************************/
       /* Draw the icon by calling IcnDrawScaledIcon.                */
       /**************************************************************/
       IcnDrawScaledIcon( hps, pirsIcon, iconRect.left,
                          iconRect.top, pccinfon,
                          (iconAttr & CRA_SOURCE ? TRUE : FALSE));      /*DAB3*/
     }
     /****************************************************************/
     /* Otherwise......................                              */
     /****************************************************************/
     else
     {
       /**************************************************************/
       /* Draw the icon by calling WinDrawBitmap.                    */
       /**************************************************************/

       /**************************************************************/
       /* Set current foreground color as FOREGROUND color           */
       /**************************************************************/
       if (!(hpenCurrent = CreatePen (PS_SOLID, 1,
                    pccinfon->ulColorPPSupport[CMA_FOREGROUNDCOLOR])))
       {
         goto Err;
       }

       /**************************************************************/
       /* Set current background color as BACKGROUND color           */
       /**************************************************************/
       if (!(hbrushCurrent = CreateSolidBrush (
                      pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR])))
       {
         goto ErrBrush;
       }

       if (!(hdcMemory = CreateCompatibleDC (hps)))
       {
         goto ErrCompDC;
       }

       if (!(GetObject (hbmIcon, sizeof(BITMAP), (LPSTR)&BitmapHdr)))
       {
         goto ErrObject;
       }

       hpenOld = SelectObject (hps, hpenCurrent);
       hbrushOld = SelectObject (hps, hbrushCurrent);
       hbmOld = SelectObject (hdcMemory, hbmIcon);

       StretchBlt (hps, iconRect.left, (iconRect.top + 1),
                   (iconRect.right - iconRect.left),
                   (iconRect.bottom - iconRect.top - 1),
                   hdcMemory, 0, 0, BitmapHdr.bmWidth,
                   BitmapHdr.bmHeight, SRCCOPY);

       SelectObject (hdcMemory, hbmOld);

       /**************************************************************/
       /* Select the old pen color and free resource.                */
       /**************************************************************/
       SelectObject (hps, hpenOld);

       /**************************************************************/
       /* Select the old brush color and free resource.              */
       /**************************************************************/
       SelectObject (hps, hbrushOld);

ErrObject:
       DeleteDC (hdcMemory);

ErrCompDC:
       DeleteObject (hbrushCurrent);

ErrBrush:
       DeleteObject (hpenCurrent);
     } /* endif */

Err:
     /****************************************************************/
     /* Restore the modes                                            */
     /****************************************************************/
     SetBkMode (hps, oldBkMode);
     SetROP2 (hps, oldFgndMode);

     RestoreDC (hps, nSavedDC);

  }   /* end if not filtered */

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawText                                       */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Draw the text of an Icon               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function draw the text of the icon. */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         LPSTR textstring - text string that is painted            */
/*         RECT textRect - rectangle in which the text is drawn      */
/*         RECT iconRect - icon rectangle                            */
/*         DWORD iconAttr - icon attribute                           */
/*         DWORD iconStyle - iocn style                              */
/*         PCNRINFON pccinfon - pointer to internal container Info   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR IcnDrawText (hps, textString, textRect, iconAttr, iconStyle,
                      pccinfon)
HDC          hps;
LPSTR        textString;
RECT         textRect;
DWORD        iconAttr;
DWORD        iconStyle;
LPCNRINFON   pccinfon;
{
  int        right;                  /* right boundary of the text */
  int        textBottommost;         /* bottom of the item */
  int        textWidth;              /* width of the text */
//POINT      textBox[TXTBOX_COUNT];  /* array of the point of the text */
                                     /* rectangle */
  POINT      textInitPtl;            /* initial point to start drawing */
                                     /* the text */
  WORD       wOptions;
  WORD       lineCount;              /* counter */
  WORD       textIndex;              /* index of the text */
  WORD       textLength;             /* length of the text */
  WORD       usSkipChar;             /* number of the chars to skip */
  int        oldBkMode = 0;          /* save old background mode      *//*DAB1*/
  HBRUSH     hBrush;                 /* text background color brush   */
  HFONT      hOldFont = NULL;        /* save old font                 */
  SIZE       textData;                                                  /*DAB1*/
  int        nSavedDC;

  if (textString && (*textString!='\0'))
  {
    if (!(iconAttr & IC_FILTERED))
    {
      /***************************************************************/
      /* Save the current DC                                         */
      /***************************************************************/
      if (!(nSavedDC = SaveDC (hps)))
        return;

      /***************************************************************/
      /* Set the color according to the attributes.                  */
      /***************************************************************/
      if (iconAttr & IC_SELECTED)
      {
        SetTextColor(hps,
                     pccinfon->ulColorPPSupport[CMA_SELECTFOREGROUNDCOLOR]);
        SetBkColor(hps,
                   pccinfon->ulColorPPSupport[CMA_SELECTBACKGROUNDCOLOR]);
        oldBkMode = SetBkMode (hps, OPAQUE);
        wOptions = ETO_CLIPPED | ETO_OPAQUE;

        /*************************************************************/
        /* Get the brush for selected emphasis backfround            */
        /*************************************************************/
        if (hBrush = CreateSolidBrush (
                      pccinfon->ulColorPPSupport[CMA_SELECTBACKGROUNDCOLOR]))
        {
          /*************************************************************/
          /* Fill the entire text rectangle with the proper color      */
          /*************************************************************/
          FillRect (hps, &textRect, hBrush);
          DeleteObject(hBrush);
        }
      }
      else
      {  /* not selected */
        SetTextColor(hps,
                     pccinfon->ulColorPPSupport[CMA_FOREGROUNDCOLOR]);
        oldBkMode = SetBkMode (hps, TRANSPARENT);
        wOptions = ETO_CLIPPED;

        /*************************************************************/
        /* Get the brush for selected emphasis backfround            */
        /*************************************************************/

      } /* endif selected  */

      /****************************************************************/
      /* Set the user specified font - If NULL use default (System)   */
      /****************************************************************/
      if (pccinfon->hCnrFont)
        hOldFont = SelectObject (hps, pccinfon->hCnrFont);

      textIndex = 0;
      lineCount = 0;
      textBottommost = textRect.bottom;

      /***************************************************************/
      /* Make text rectangle for the first line of the text          */
      /***************************************************************/
      textRect.bottom =  textRect.top + GETCYCHAR(pccinfon);

      /***************************************************************/
      /* Initialize the init point of the text                       */
      /***************************************************************/
      textInitPtl.x = textRect.left;

      /***************************************************************/
      /* Since icon view text may be broken up into several          */
      /* lines separated by line-feed characters, each "line" of     */
      /* of text is drawn individually.                              */
      /***************************************************************/
      while (((int)textIndex < lstrlen(textString)) &&
             (textRect.bottom <= textBottommost))
      {
        textLength = CnrStrcspn(&textString[textIndex],"\r\n\0");
        /*************************************************************/
        /* If both '\r' and '\n' are at the end of each line,        */
        /* skip both  chars for next line. If only one of them is,   */
        /* skip one char.                                            */
        /*************************************************************/
        if (*(textString + (textIndex + textLength )) == '\r')
        {
          usSkipChar = 2;
        } else {
          usSkipChar = 1;
        } /* endif */

        if (iconStyle & (IS_BENEATHTEXT | IS_SIDERIGHTTEXT)) {

         /************************************************************/
         /* Query the rectangle of each line.                        */
         /*     textData - LOWORD = text width.                      */
         /*              - HIWORD = text height.                     */
         /************************************************************/
         GetTextExtentPoint (hps, (LPSTR)&textString[textIndex],        /*DAB1*/
                             textLength, &textData);                    /*DAB1*/

          /***********************************************************/
          /* The text width is the difference between the right      */
          /* and left boundaries of the text.                        */
          /***********************************************************/
          textWidth = textData.cx;                                      /*DAB1*/
          right = textRect.left + textWidth;

          /***********************************************************/
          /* Left justified if the text is longer than the           */
          /* rectangle                                               */
          /***********************************************************/
          if (right >= textRect.right )
          {
             textInitPtl.x = textRect.left;
          }
          else
          {
            if (iconStyle & IS_BENEATHTEXT ) {
              /*******************************************************/
              /* Centered if the text is shorter than or equal to    */
              /* the rectangle.                                      */
              /*******************************************************/
              textInitPtl.x = textRect.left + (textRect.right - right) / 2;
            }
            else
            {

              /*******************************************************/
              /* Right justified if the icon style is                */
              /* IS_SIDERIGHTTEXT.                                   */
              /*******************************************************/
              textInitPtl.x = textRect.right - textWidth;
            } /* endif */

          } /* endif */

        }           /* endif  */
        /*************************************************************/
        /* Set initial y position for the drawing a line of the text.*/
        /*************************************************************/
        textInitPtl.y = textRect.top;

        ExtTextOut (hps, textInitPtl.x, textInitPtl.y,
                    wOptions, &textRect,
                    (LPSTR)&textString[textIndex],
                    textLength, NULL);

        /*************************************************************/
        /* The text location is shifted down one line in             */
        /* preparation for the next line of text to be drawn.        */
        /*************************************************************/
        textIndex += (textLength + usSkipChar);
        textRect.top = textRect.bottom;

        textRect.bottom = textRect.top + GETCYCHAR(pccinfon);


        lineCount++;
      } /* end while */

      /***************************************************************/
      /* Reset the background mode.                                  */
      /***************************************************************/
      if (oldBkMode)
         SetBkMode (hps, oldBkMode);

      /****************************************************************/
      /* Reset the old font if it was changed                         */
      /****************************************************************/
      if (hOldFont)
        SelectObject (hps, hOldFont);

      RestoreDC (hps, nSavedDC);
    } /* endif  filtered */
  } /* endif */

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawIconInDV                                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Draw an Icon                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints an icon.            */
/*   It calculates the rectangle that includes icon rectangle and    */
/*   the white space for the selection emphasis, if the item is      */
/*   selected/in_use, draw selection/in_use emphasis and draw the    */
/*   graphic object using icon (if pirsIcon is given) or bitmap      */
/*   (if pirs is NULL and hbmIcon is given.).                        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         PICONRESIZED pirsIcon - resized icon                      */
/*         HBITMAP hbmIcon - bitmap to be drawn if pirsIcon is NULL  */
/*         DWORD iconAttr - attr of the item                         */
/*         RECT iconRect - rectangle of the icon                    */
/*         PCNRINFON pccinfon - pointer to internal container Info   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/

VOID FAR IcnDrawIconInDV (hps, pirsIcon, hbmIcon, iconAttr,
                           iconRect, rctlEmph, DrawEmph, pccinfon)
HDC           hps;
LPICONRESIZED pirsIcon;
HBITMAP       hbmIcon;
DWORD         iconAttr;
RECT          iconRect;
RECT          rctlEmph;
BOOL          DrawEmph;
LPCNRINFON    pccinfon;
{
  int        oldFgndMode;         /* save old foreground mode */
  int        oldBkMode;           /* save old background mode */
  HPEN       hpenOld = NULL;      /* save old pen */
  HPEN       hpenCurrent;
  HBRUSH     hbrushOld;           /* save old brush */
  HPEN       hbrushCurrent;
  BITMAP     BitmapHdr;
  HBITMAP    hbmOld;              /* Old bitmap handle */
  HDC        hdcMemory;           /* handle to memory device context */
  int        nSavedDC;

  /*******************************************************************/
  /* If it not filtered out ..........                               */
  /*******************************************************************/
  if (!(iconAttr & IC_FILTERED))
  {
     /****************************************************************/
     /* Save current PS attributes.                                  */
     /****************************************************************/
     if (!(nSavedDC = SaveDC (hps)))
       return;

     /****************************************************************/
     /* Set current foreground mix mode which the foreground color   */
     /* replaces the existing color.                                 */
     /****************************************************************/
     oldFgndMode = SetROP2 (hps, R2_COPYPEN);

     /****************************************************************/
     /* Set current background mix mode which the background color   */
     /* is ignored and the existing color remains unchanged.         */
     /****************************************************************/
     oldBkMode = SetBkMode (hps, TRANSPARENT);

     /****************************************************************/
     /* If the item is selected........                              */
     /****************************************************************/
     if (iconAttr & IC_SELECTED)
     {
       /**************************************************************/
       /* Set current foreground color as HILITEFOREGROUND color     */
       /**************************************************************/
       if (!(hpenCurrent = CreatePen (PS_SOLID, 1,
                   pccinfon->ulColorPPSupport[CMA_SELECTFOREGROUNDCOLOR])))
       {
         goto Err;
       }

       hpenOld = SelectObject (hps, hpenCurrent);

       /**************************************************************/
       /* If pirsIcon is given, paint whole rctlBack with the        */
       /* hilited background color because the icon will be          */
       /* transparent.                                               */
       /**************************************************************/
       IcnDrawSelectEmph(hps, rctlEmph, TRUE, pccinfon);

       /**************************************************************/
       /* Select the old pen color and free resource.                */
       /**************************************************************/

     } /* endif */

     /****************************************************************/
     /* If the item is in use..........                              */
     /****************************************************************/
     if ((iconAttr & IC_INUSE) && (DrawEmph))
     {
       /**************************************************************/
       /* Draw in_use emphasis on rctlBack.                          */
       /**************************************************************/
       rctlEmph.top++;
       rctlEmph.bottom--;
       IcnDrawInUseEmph(hps, rctlEmph, pccinfon);
       rctlEmph.top--;
       rctlEmph.bottom++;
     } /* endif */

     /****************************************************************/
     /* Select the old pen color and free resource.                  */
     /****************************************************************/
     if (hpenOld)
        DeleteObject (SelectObject (hps, hpenOld));

     /****************************************************************/
     /* If pirs is given...............                              */
     /****************************************************************/
     if (pirsIcon)
     {
       /**************************************************************/
       /* Draw the icon by calling IcnDrawScaledIcon.                */
       /**************************************************************/
       IcnDrawScaledIcon( hps, pirsIcon, iconRect.left,
                          iconRect.top, pccinfon,
                          (iconAttr & CRA_SOURCE ? TRUE : FALSE));      /*DAB3*/
     }
     /****************************************************************/
     /* Otherwise......................                              */
     /****************************************************************/
     else
     {
       /**************************************************************/
       /* Draw the icon by calling WinDrawBitmap.                    */
       /**************************************************************/

       /**************************************************************/
       /* Set current foreground color as BACKGROUND color           */
       /**************************************************************/
       if (!(hpenCurrent = CreatePen (PS_SOLID, 1,
                           pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR])))
       {
         goto Err;
       }

       /**************************************************************/
       /* Set current background color as BACKGROUND color           */
       /**************************************************************/
       if (!(hbrushCurrent = CreateSolidBrush (
                      pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR])))
       {
         goto ErrBrush;
       }

       if (!(hdcMemory = CreateCompatibleDC (hps)))
       {
         goto ErrCompDC;
       }

       if (!(GetObject (hbmIcon, sizeof(BITMAP), (LPSTR)&BitmapHdr)))
       {
         goto ErrObject;
       }

       hpenOld = SelectObject (hps, hpenCurrent);
       hbrushOld = SelectObject (hps, hbrushCurrent);
       hbmOld = SelectObject (hdcMemory, hbmIcon);

       StretchBlt (hps, iconRect.left, iconRect.top,
                   (iconRect.right - iconRect.left),
                   (iconRect.bottom - iconRect.top),
                   hdcMemory, 0, 0, BitmapHdr.bmWidth,
                   BitmapHdr.bmHeight, SRCCOPY);

       SelectObject (hdcMemory, hbmOld);

       /**************************************************************/
       /* Select the old pen color and free resource.                */
       /**************************************************************/
       SelectObject (hps, hpenOld);

       /**************************************************************/
       /* Select the old brush color and free resource.              */
       /**************************************************************/
       SelectObject (hps, hbrushOld);

ErrObject:
       DeleteDC (hdcMemory);

ErrCompDC:
       DeleteObject (hbrushCurrent);

ErrBrush:
       DeleteObject (hpenCurrent);
     } /* endif */

Err:
     /****************************************************************/
     /* Restore the modes                                            */
     /****************************************************************/
     SetBkMode (hps, oldBkMode);
     SetROP2 (hps, oldFgndMode);

     /****************************************************************/
     /* Restore the state of the presentation space from the PS stack*/
     /****************************************************************/
     RestoreDC (hps, nSavedDC);

  }   /* end if not filtered */

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawTextInDV                                   */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Control Draw the text of an Icon               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function draw the text of the icon. */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HDC hps - handle of the presentation space                */
/*         LPSTR textstring - text string that is painted            */
/*         RECT textRect - rectangle in which the text is drawn      */
/*         RECT iconRect - icon rectangle                            */
/*         DWORD iconAttr - icon attribute                           */
/*         DWORD iconStyle - iocn style                              */
/*         PCNRINFON pccinfon - pointer to internal container Info   */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR IcnDrawTextInDV ( hps, textString, textRect, rctlEmph,
                   iconAttr, iconStyle, pccinfon )
HDC          hps;
LPSTR        textString;
RECT         textRect;
RECT         rctlEmph;
DWORD        iconAttr;
DWORD        iconStyle;
LPCNRINFON   pccinfon;
{
  int        right;                  /* right boundary of the text */
  int        textBottommost;         /* bottom of the item */
  int        textWidth;              /* width of the text */
  POINT      textInitPtl;            /* initial point to start drawing */
                                     /* the text */
  WORD       lineCount;              /* counter */
  WORD       textIndex;              /* index of the text */
  WORD       textLength;             /* length of the text */
  WORD       usSkipChar;             /* number of the chars to skip */
  int        oldFgndMode = 0;        /* save old foreground mode */     /*DAB1*/
  int        oldBkMode = 0;          /* save old background mode */     /*DAB1*/
  HFONT      hOldFont = NULL;        /* save old font */
  SIZE       textData;               /* contains text width & height  *//*DAB1*/
  int        nSavedDC;

    if (!(iconAttr & IC_FILTERED))
    {
      /***************************************************************/
      /* Save the current DC                                         */
      /***************************************************************/
      if (!(nSavedDC = SaveDC (hps)))
        return;

      /***************************************************************/
      /* Set current foreground mix mode which the foreground color  */
      /* replaces the existing color.                                */
      /***************************************************************/
      oldFgndMode = SetROP2 (hps, R2_COPYPEN);

      /***************************************************************/
      /* Set current background mix mode which the background color  */
      /* is ignored and the existing color remains unchanged.        */
      /***************************************************************/
      oldBkMode = SetBkMode (hps, TRANSPARENT);

      /***************************************************************/
      /* Set the color according to the attributes.                  */
      /***************************************************************/
      if (iconAttr & IC_SELECTED)
      {
        /**************************************************************/
        /* Set current foreground color as HILITEFOREGROUND color     */
        /**************************************************************/
        SetTextColor(hps,
                     pccinfon->ulColorPPSupport[CMA_SELECTFOREGROUNDCOLOR]);

        /**************************************************************/
        /* Set current background color as HILITEBACKGROUND color     */
        /**************************************************************/
        SetBkColor(hps,
                   pccinfon->ulColorPPSupport[CMA_SELECTBACKGROUNDCOLOR]);


        /**************************************************************/
        /* If pirsIcon is given, paint whole rctlBack with the        */
        /* hilited background color because the icon will be          */
        /* transparent.                                               */
        /**************************************************************/
        IcnDrawSelectEmph(hps, rctlEmph, TRUE, pccinfon);
      }
      else /* not selected */
      {
        /**************************************************************/
        /* Set current foreground color as FOREGROUND color           */
        /**************************************************************/
        SetTextColor(hps,
                     pccinfon->ulColorPPSupport[CMA_FOREGROUNDCOLOR]);

        /**************************************************************/
        /* Set current background color as BACKGROUND color           */
        /**************************************************************/
      }

      /****************************************************************/
      /* Set the user specified font - If NULL use default (System)   */
      /****************************************************************/
      if (pccinfon->hCnrFont)
        hOldFont = SelectObject (hps, pccinfon->hCnrFont);

      textIndex = 0;
      lineCount = 0;
      textBottommost = textRect.bottom;

      /***************************************************************/
      /* Make text rectangle for the first line of the text          */
      /***************************************************************/
      textRect.bottom =  textRect.top + GETCYCHAR(pccinfon);

      /***************************************************************/
      /* Initialize the init point of the text                       */
      /***************************************************************/
      textInitPtl.x = textRect.left;

      /***************************************************************/
      /* Since icon view text may be broken up into several          */
      /* lines separated by line-feed characters, each "line" of     */
      /* of text is drawn individually.                              */
      /***************************************************************/
      if ((textString) && (*textString != '\0'))
      {
        while (((int)textIndex < lstrlen(textString)) &&
               (textRect.bottom <= textBottommost))
        {
          textLength = CnrStrcspn(&textString[textIndex],"\r\n\0");
          /*************************************************************/
          /* If both '\r' and '\n' are at the end of each line,        */
          /* skip both  chars for next line. If only one of them is,   */
          /* skip one char.                                            */
          /*************************************************************/
          if (*(textString + (textIndex + textLength )) == '\r')
          {
            usSkipChar = 2;
          } else {
            usSkipChar = 1;
          } /* endif */

          if (iconStyle & (IS_BENEATHTEXT | IS_SIDERIGHTTEXT)) {

            /**********************************************************/
            /* Query the rectangle of each line.                      */
            /*     textData - LOWORD = text width.                    */
            /*              - HIWORD = text height.                   */
            /**********************************************************/
            GetTextExtentPoint (hps, (LPSTR)&textString[textIndex],     /*DAB1*/
                                textLength, &textData);                 /*DAB1*/

            /**********************************************************/
            /* The text width is the difference between the right     */
            /* and left boundaries of the text.                       */
            /**********************************************************/
            textWidth = textData.cx;                                    /*DAB1*/
            right = textRect.left + textWidth;

            /***********************************************************/
            /* Left justified if the text is longer than the           */
            /* rectangle                                               */
            /***********************************************************/
            if (right >= textRect.right )
            {
               textInitPtl.x = textRect.left;
            }
            else
            {
              if (iconStyle & IS_BENEATHTEXT)
              {
                /*******************************************************/
                /* Centered if the text is shorter than or equal to    */
                /* the rectangle.                                      */
                /*******************************************************/
                textInitPtl.x = textRect.left +
                                (textRect.right - right) / 2;
              }
              else
              {
                /*******************************************************/
                /* Right justified if the icon style is                */
                /* IS_SIDERIGHTTEXT.                                   */
                /*******************************************************/
                textInitPtl.x = textRect.right - textWidth;
              } /* endif */

            } /* endif */

          }           /* endif  */
          /*************************************************************/
          /* Set initial y position for the drawing a line of the text.*/
          /*************************************************************/
          textInitPtl.y = textRect.top;

          ExtTextOut (hps, textInitPtl.x, textInitPtl.y,
                      ETO_CLIPPED, &textRect,
                      (LPSTR)&textString[textIndex],
                      textLength, NULL);

          /*************************************************************/
          /* The text location is shifted down one line in             */
          /* preparation for the next line of text to be drawn.        */
          /*************************************************************/
          textIndex += (textLength + usSkipChar);
          textRect.top = textRect.bottom;

          textRect.bottom = textRect.top + GETCYCHAR(pccinfon);


          lineCount++;
        } /* end while */
      }

      /****************************************************************/
      /* Reset the old font if it was changed                         */
      /****************************************************************/
      if (hOldFont)
        SelectObject (hps, hOldFont);

      RestoreDC (hps, nSavedDC);
    } /* endif  filtered */

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawInUseEmph                                  */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Draw InUse Emphasis                            */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*    This function draws In-Use Emphasis on the given               */
/*    rectangle(rctlBack).                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:                                                            */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR IcnDrawInUseEmph(hps, rctlBack, pccinfon)
HDC        hps;
RECT       rctlBack;
LPCNRINFON pccinfon;

{
  int      xInd;               /* x pos indicator                    */
  int      yInd;               /* y pos indicator                    */
  POINT    ptlInit;            /* initial point of the line          */
  POINT    ptlEnd;             /* end point of the line to be drawn  */

  /*******************************************************************/
  /* Initialize ....                                                 */
  /*******************************************************************/
  rctlBack.left--;
  rctlBack.right--;
  xInd = ptlInit.x = ptlEnd.x = rctlBack.left;
  yInd = ptlInit.y = ptlEnd.y = rctlBack.top;

  while (xInd < (rctlBack.right + (rctlBack.bottom - rctlBack.top)))
  {
    if (xInd < rctlBack.right)
    {
      /***************************************************************/
      /* If xInd is less than right of the rectangle, the x of the  */
      /* init point of the line is xInd and the y of the init point  */
      /* doesn't change.                                             */
      /***************************************************************/
      ptlInit.x = xInd;
    }
    else
    {
      /***************************************************************/
      /* Otherwise, the x of the init point is right of the         */
      /* rectangle and the y of the init point is lowered by         */
      /* the difference between xInd and the right of the rectangle.*/
      /***************************************************************/
      ptlInit.x = rctlBack.right;
      ptlInit.y = rctlBack.top + (xInd - rctlBack.right);
    } /* endif */

    if (yInd < rctlBack.bottom)
    {
      /***************************************************************/
      /* If yInd is more than right of the rectangle, the y of the  */
      /* end  point of the line is yInd and the x of the end  point  */
      /* doesn't change.                                             */
      /***************************************************************/
      ptlEnd.y = yInd;
    }
    else
    {
      /***************************************************************/
      /* Otherwise, the y of the end point is bottom of the          */
      /* rectangle and the x of the init point is added   by         */
      /* the difference between yInd and the bottom of               */
      /* the rectangle.                                              */
      /***************************************************************/
      ptlEnd.y = rctlBack.bottom;
      ptlEnd.x = rctlBack.left + (yInd - rctlBack.bottom);
    } /* endif */

    /*****************************************************************/
    /* Draw a line from the init point to the end point.             */
    /*****************************************************************/
    MoveToEx (hps, ptlInit.x, ptlInit.y, NULL);                         /*DAB1*/

    /*****************************************************************/
    /* Draw a line from the first point to the second point.         */
    /*****************************************************************/
    LineTo (hps, ptlEnd.x, ptlEnd.y);

    /*****************************************************************/
    /* update xInd and yInd                                          */
    /*****************************************************************/
    xInd += CX_BETWEENLINES;
    yInd += CY_BETWEENLINES;
  } /* endwhile */

  rctlBack.left++;
  rctlBack.right++;

  return;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IcnDrawSelectEmph                                 */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Draw Select  Emphasis                          */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*    This function draws Select Emphasis on the given               */
/*    rectangle(rctlBack).                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:                                                            */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/

VOID FAR IcnDrawSelectEmph(hps, rctlBack, flIsForIcon, pccinfon)
HDC        hps;
RECT       rctlBack;
BOOL       flIsForIcon;
LPCNRINFON pccinfon;

{
  POINT     ptl;
  POINT     ptl2;
  HPEN      hpenOld;             /* save old pen */
  HPEN      hpenCurrent;
  HBRUSH    hbrushOld;           /* save old brush */
  HBRUSH    hBrush;

  if (flIsForIcon)
  {

    if (hBrush = CreateSolidBrush (
                    pccinfon->ulColorPPSupport[CMA_SELECTBACKGROUNDCOLOR]))
    {
      PMFillRect (hps, &rctlBack, hBrush);
      DeleteObject(hBrush);
    }
  }
  else
  {
    if (!(hpenCurrent = CreatePen (PS_SOLID, 1,
                   pccinfon->ulColorPPSupport[CMA_SELECTBACKGROUNDCOLOR])))
    {
      return;
    }

    hpenOld = SelectObject (hps, hpenCurrent);

    ptl.x = rctlBack.right;
    ptl.y = rctlBack.top;

    hbrushOld = SelectObject (hps, GetStockObject (NULL_BRUSH));
    Rectangle (hps, rctlBack.left, rctlBack.bottom, ptl.x, ptl.y);

    ptl.x = rctlBack.left + ICN_SIDEMARGIN;
    ptl.y = rctlBack.bottom - ICN_TOPMARGIN;

    ptl2.x = rctlBack.right - ICN_SIDEMARGIN;
    ptl2.y = rctlBack.top + ICN_TOPMARGIN;

    Rectangle (hps, ptl.x, ptl.y, ptl2.x, ptl2.y);
    SelectObject (hps, hbrushOld);

    /*****************************************************************/
    /* Select the old pen color and free resource.                   */
    /* Assume that we are reselecting the Hilite Foreground color... */
    /*****************************************************************/
    DeleteObject (SelectObject (hps, hpenOld));
  } /* endif */
  return;
}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrPaintBackground                             */
/*                                                                    */
/*  Descriptive Name:   Paint Background                              */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function paint the given rectangle with SYSCLR_WINDOW.    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR  CnrPaintBackground(hwnd, hps, prctlPaint)
HANDLE      hwnd;
HDC         hps;
LPRECT      prctlPaint;

{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  HBRUSH     hBrush;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* PMFillRect doesn't draw top row and rightmost column pixels    */
  /* So add more pixels to each side                                 */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_DETAIL))
  {
     prctlPaint->top -= TXT_TOPMARGIN;
     prctlPaint->right += TXT_SIDEMARGIN;
  }

  if (hBrush = CreateSolidBrush (
                 pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
  {
    PMFillRect (hps, prctlPaint, hBrush);
    DeleteObject(hBrush);
  }

  /*******************************************************************/
  /* Go back to the original rectangle                               */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_DETAIL))
  {
     prctlPaint->top += TXT_TOPMARGIN;
     prctlPaint->right -= TXT_SIDEMARGIN;
  }
  return TRUE;
}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrStrcspn                                     */
/*                                                                    */
/*  Descriptive Name:  String counter for Container                   */
/*                                                                    */
/*  Functional Description:                                           */
/*     This function always returns number of bytes whether DBCS      */
/*     or not.  Need this to use calls like GpiCalcTextBox and        */
/*     GpiCharStringPosAt, because they accept only bytes and are     */
/*     not DBCS enabled.                                              */
/*                                                                    */
/**********************************************************************/
WORD FAR CnrStrcspn (LPSTR pszSrc, LPSTR pszSet)
{
  WORD  i=0, j;

  while (pszSrc[i])
  {
    for (j=0; pszSet[j]; j++)
    {
      if (pszSrc[i] == pszSet[j])
        return i;
    }
    i++;
  }
  return i;
}
