/* Revision: 15 1.2.2.2 source/ui/cclnt/cnr/cnrs2p4.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS2P4.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS2P4.C - Details view routines for container control  */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* RKK1   4/30/96  RKK  Add changes required for Wind/U.              */
/**********************************************************************/
#include <windows.h>
#include <stdlib.h>                    /* for labs                    */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS2P4_C
#include "cnrpragw.h"
#endif


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrIconVCtlProc                                   */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LRESULT CALLBACK CnrIconVCtlProc( HWND hwndCnr, UINT msg,               /*DAB1*/
                                  WPARAM wParam, LPARAM lParam )        /*DAB1*/
{
  HDC         hps;
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  RECT        rctlWin;
  RECT        rctlInvalid;
  PAINTSTRUCT paint;
  RECT        rctlTitle;
  int         cyOld;
  BOOL        fHeightChanged;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Process the messages sent to this view                          */
  /*******************************************************************/
  switch (msg)
  {
    case WM_CREATE:
      return FALSE;

    case WM_SIZE:
      pccinfon->ulFlagSet2 |= FL2_WINDOWSIZED;

      /*-----------------------------------------*/
      /* get the frame for the container, if any */
      /*-----------------------------------------*/
      if (wParam == SIZEICONIC)
      {
        pccinfon->ulFlagSet2 |= FL2_MINIMIZED;
        pccinfon->ulFlagSet &= ~FL_CNRWINOPEN;
      }
      else
      {
        if (pccinfon->ulFlagSet2 & FL2_MINIMIZED)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          }
          else
          {
            if (pccinfon->ulFlagSet2 & FL2_MAXIMIZED)
            {
              pccinfo->ptlOrigin = pccinfon->ptlPrevIconOrg;
              CnrQuerySizeRect(hwndCnr, NULL, &rctlTitle, NULL,
                               NULL, NULL);
              pccinfon->rctlWorkplace.bottom =
                           max((pccinfo->ptlOrigin.y +
                                (HIWORD(lParam) - cyOld)),
                               pccinfon->rctlWorkplaceItems.bottom);
            }
          } /* endif */

          pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;
          pccinfon->ulFlagSet &= ~(FL_CNRWINOPEN | FL_CNRPOSITION |
                              FL_SETSCROLL | FL_NEWVIEWSCROLLSET |
                              FL_SETCNRTITLE);
        } /* endif */

        if (pccinfon->ulFlagSet2 & FL2_MAXIMIZED)
        {
          pccinfon->ulFlagSet2 &= ~FL2_MAXIMIZED;
        }
        if (wParam == SIZEFULLSCREEN)
        {
          pccinfon->ulFlagSet2 |= FL2_MAXIMIZED;
          pccinfon->ptlPrevIconOrg = pccinfo->ptlOrigin;
        }
      }

      /***************************************************************/
      /* If window is already open,      then                        */
      /* cy of viewport in icon view inc, so does y of icon  to      */
      /* keep the relative pos of the icon to the top left coner     */
      /* of the window.(in any view we need to) The way to do is     */
      /* changing ptlOrigin.                                         */
      /***************************************************************/
      if ((pccinfon->ulFlagSet & FL_CNRWINOPEN))
      {
        fHeightChanged = FALSE;

        /*************************************************************/
        /* Only change of the cy of the window changes the relative  */
        /* position of the item.                                     */
        /*************************************************************/
        cyOld = (int)OLDHEIGHTOF(hwndCnr);
        if (HIWORD(lParam) != (WORD)cyOld)
        {
          pccinfon->rctlWorkplace.bottom =
                       max((int)(pccinfo->ptlOrigin.y +
                                (HIWORD(lParam) - cyOld)),
                           (int)pccinfon->rctlWorkplaceItems.bottom);
          fHeightChanged = TRUE;

          /***********************************************************/
          /* When the height of the window changes, we need reset    */
          /* the title window iff:                                   */
          /*    1) The container has a title                         */
          /*    2) The old container height equals the title height  */
          /***********************************************************/
          if (pccinfo->flWindowAttr & CA_CONTAINERTITLE)
          {
            CnrQuerySizeRect(hwndCnr, NULL, &rctlTitle, NULL,
                             NULL, NULL);
            if (cyOld == rctlTitle.bottom)
              pccinfon->ulFlagSet &= ~FL_SETCNRTITLE;

          }
        } /* endif */

        if (LOWORD(lParam) != OLDWIDTHOF(hwndCnr))
        {
          /***********************************************************/
          /* If CCS_AUTOPOSITION is set on, and the width of the      */
          /* window chnaged, we need rearrange icons in icon view.   */
          /***********************************************************/
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
            pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;
          } /* endif */

          /***********************************************************/
          /* If the width of the window changed, we need reset the   */
          /* title window.                                           */
          /***********************************************************/
          pccinfon->ulFlagSet &= ~FL_SETCNRTITLE;
          fHeightChanged = TRUE;

        } /* endif */

        if (fHeightChanged)
          SAVESIZEOF(hwndCnr, lParam);
      }
      else
      {
        /*************************************************************/
        /* If window is not open, this msg is not for the resizing   */
        /* of window. So, don't change origin and set flag on so     */
        /* that next time we get this msg, we update the origin      */
        /*************************************************************/
        SAVESIZEOF(hwndCnr, lParam);
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
      } /* endif */

      /***************************************************************/
      /* We need resize the child windows such as H scroll window.   */
      /***************************************************************/
      pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;

      PMInvalidateRect( hwndCnr, NULL, TRUE );
      return(TRUE);

    case WM_PAINT:
      hps = BeginPaint(hwndCnr, &paint);
      rctlInvalid = paint.rcPaint;

      PMFlipDC(hwndCnr, hps);
      PMFlipRect((LPRECT)&rctlInvalid);

      /*-----------------------------------------*/
      /* get the frame for the container, if any */
      /*-----------------------------------------*/
      if (!(IsIconic(hwndCnr)))
      {
        pccinfon->ulFlagSet |= FL_CNRWINOPEN;
        pccinfon->ulFlagSet2 &= ~FL2_MINIMIZED;

        /**************************************************************/
        /* If FL_SETCNRTITLE is off, reset the title window.          */
        /**************************************************************/
        if (!(pccinfon->ulFlagSet & FL_SETCNRTITLE))
        {
          PMGetClientRect(hwndCnr, &rctlWin);

          /************************************************************/
          /* We need reset Cnr Title Window. (The cnr window is not   */
          /* initialized or the font of the Title Window is changed). */
          /* CnrSizeCnrTitle in cnrcfunc.c                            */
          /************************************************************/
          CnrSizeCnrTitle (hwndCnr, hps, rctlWin);

          /************************************************************/
          /* Because the size of the title bar window might change    */
          /* we need resize the child windows.                        */
          /************************************************************/
          pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;

          /************************************************************/
          /* If CCS_AUTOPOSITION is on, we need rearrange icons.       */
          /************************************************************/
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            pccinfon->ulFlagSet |= FL_NEEDARRANGE;
          } /* endif */
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_CNRPOSITION))
        {
          /************************************************************/
          /* Make sure FL_CNRWINOPEN is on(for the case of the        */
          /* initial painting without sending WM_SIZE                 */
          /************************************************************/
          pccinfon->ulFlagSet |= FL_CNRPOSITION;
        } /* endif */


        if (pccinfon->ulFlagSet & FL_NEEDARRANGE)
        {
          /************************************************************/
          /* If FL_NEEDARRANGE is on arrange the icons by calling     */
          /* CnrArrangeIconView(in cnrgen.c).                         */
          /************************************************************/
          CnrArrangeIconView(hwndCnr, hps);
          pccinfon->ulFlagSet &= ~(FL_SETSCROLL | FL_NEEDARRANGE);
          pccinfon->ulFlagSet |= FL_SETWORKPLACE;
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
        {
          CalcTextSizeInIV(hwndCnr, hps);
          pccinfon->ulFlagSet |= FL_TEXTCALC;
        }

        if (!(pccinfon->ulFlagSet & FL_SETWORKPLACE))
        {
          /************************************************************/
          /* If FL_SETWORKPLACE is off, update the workplace rectangle*/
          /* by calling CnrUpdateWorkplace (in cnrcfunc.c).           */
          /************************************************************/
          CnrUpdateWorkplace(hwndCnr, hps, NULL, FALSE);
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
          pccinfon->ulFlagSet |= FL_SETWORKPLACE;
        } /* endif */

        if (CheckNeedResize(hwndCnr))
        {
          pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
        }

        if (!(pccinfon->ulFlagSet & FL_SIZECHILDREN))
        {
          /************************************************************/
          /* resize the scroll bars                                   */
          /************************************************************/
          PMGetClientRect(hwndCnr, &rctlWin);

          CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);

          /************************************************************/
          /* Set the scroll bar                                       */
          /************************************************************/
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;
          pccinfon->ulFlagSet |= FL_SIZECHILDREN;
        } /* endif */

        if (!(pccinfon->ulFlagSet & FL_SETSCROLL))
        {
          /************************************************************/
          /* Set the scroll bar                                       */
          /************************************************************/
          CnrSetIconViewScroll(hwndCnr);
          pccinfon->ulFlagSet |= FL_SETSCROLL;
        } /* endif */

        CnrEraseRect(hwndCnr, hps, rctlInvalid);
        if (!(CnrPaintIconView( hwndCnr, hps, &rctlInvalid)))
        {
           EndPaint(hwndCnr, &paint);
           return ((long)NULL);
        }

        EndPaint(hwndCnr, &paint);

        if (pccinfon->ulFlagSet2 & FL2_WINDOWSIZED)
        {
           /************************************************************/
           /* Windows will call CnrEnumWindowsProc for each child      */
           /************************************************************/
           EnumChildWindows(hwndCnr, CnrEnumWindowsProc, 0);            /*DAB1*/

           pccinfon->ulFlagSet2 &= ~FL2_WINDOWSIZED;
        }
        else
        {
          PMInvalidateRect (GetDlgItem (hwndCnr, CID_VSCROLL),
                             NULL, FALSE);

          PMInvalidateRect (GetDlgItem (hwndCnr, CID_HSCROLL),
                             NULL, FALSE);
        }
      }
      else
      {
        EndPaint(hwndCnr, &paint);
      }

      pccinfon->ulFlagSet2 |= FL2_INITPAINT;
      return(TRUE);

    case WM_VSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewVScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/
    case WM_HSCROLL:
    {                                                                   /*DAB1*/
      HWND hwndScroll = (HWND)(UINT)lParam;                             /*DAB1*/
      int  nThumbPos  = HIWORD(wParam);                                 /*DAB1*/
      WORD wNotifyCode= LOWORD(wParam);                                 /*DAB1*/

      CnrINTViewHScroll(hwndCnr, hwndScroll, nThumbPos, wNotifyCode);   /*DAB1*/
      return 0;
    }                                                                   /*DAB1*/

    /*****************************************************************/
    /* All other messages are sent on to the superclass              */
    /*****************************************************************/
    default:
      return ((long)CnrGateProc( hwndCnr, msg, wParam, lParam ));
  }

}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CheckNeedResize                                   */
/*                                                                   */
/* Usage:                                                            */
/*                                                                   */
/* DESCRIPTIVE NAME:                                                 */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:                                           */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR CheckNeedResize(hwndCnr)
HANDLE    hwndCnr;
{
#define NEED_H          0x0001
#define NEED_HwithV     0x0002
#define NEED_V          0x0004
#define NEED_VwithH     0x0008


  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  RECT          rctlCnr;
  RECT          rctlVscroll;
  RECT          rctlHscroll;
  RECT          rctlLTitle;
  int           sNeed = 0;
  int           cyHscroll;
  int           cxVscroll;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  PMGetClientRect(hwndCnr, &rctlCnr);
  CnrQuerySizeRect(hwndCnr, NULL, &rctlLTitle, NULL,
                   &rctlVscroll, &rctlHscroll);

  cyHscroll = GetSystemMetrics( SM_CYHSCROLL);
  cxVscroll = GetSystemMetrics( SM_CXVSCROLL);

  if ((pccinfon->rctlWorkplace.top <
       (rctlCnr.top + rctlLTitle.bottom + pccinfo->ptlOrigin.y)) ||
      (pccinfon->rctlWorkplace.bottom >
       (rctlCnr.bottom - cyHscroll + pccinfo->ptlOrigin.y)))

  {
    sNeed |= NEED_VwithH;
  }

  if ((pccinfon->rctlWorkplace.top <
       (rctlCnr.top + rctlLTitle.bottom + pccinfo->ptlOrigin.y)) ||
      (pccinfon->rctlWorkplace.bottom >
       (rctlCnr.bottom + pccinfo->ptlOrigin.y)))

  {
    sNeed |= NEED_V;
  }


  if ((pccinfon->rctlWorkplace.left <
       (rctlCnr.left + pccinfo->ptlOrigin.x)) ||
      (pccinfon->rctlWorkplace.right >
       (rctlCnr.right - cxVscroll + pccinfo->ptlOrigin.x)) )
  {
    sNeed |= NEED_HwithV;
  } /* endif */

  if ((pccinfon->rctlWorkplace.left <
       (rctlCnr.left + pccinfo->ptlOrigin.x)) ||
      (pccinfon->rctlWorkplace.right >
       (rctlCnr.right + pccinfo->ptlOrigin.x)) )
  {
    sNeed |= NEED_H;
  } /* endif */

  if ((sNeed & NEED_H) && (sNeed & NEED_VwithH) && !(sNeed & NEED_V))
  {
    sNeed |= NEED_V;
  }

  if ((sNeed & NEED_V) && (sNeed & NEED_HwithV) && !(sNeed & NEED_H))
  {
    sNeed |= NEED_H;
  }

  if ((rctlHscroll.right == 0) && (sNeed & NEED_H) ||
      (rctlVscroll.bottom == 0) && (sNeed & NEED_V) ||
      (rctlVscroll.bottom > 0) && !(sNeed & NEED_V) ||
      (rctlHscroll.right > 0) && !(sNeed & NEED_H))
  {
    return TRUE;
  }
  else
  {
    return FALSE;
  } /* endif */
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  FindFirstIteminColumn                             */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Find First Item In a Column                    */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in a  */
/*     a column searching backward from preccCurrent. If             */
/*     preccCurrent is NULL the first item is the first item         */
/*     of the list(the first item of the first column).              */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPVOID FAR FindFirstIteminColumn (hwndCnr, preccCurrent)
HANDLE          hwndCnr;
LPRECORDCOREN  preccCurrent;

{

  LPCNRINFON        pccinfon;
  LPRECORDCOREN     preccPrev;
  LPRECORDCOREN     preccFirst;

  if (preccCurrent)
  {
    preccFirst = preccCurrent;
    while (preccCurrent)
    {
      preccPrev = preccCurrent->preccPrevItem;

      /***************************************************************/
      /* If previous item  exists and it's x position is the same as */
      /* x position of the current item, continue checking the       */
      /* previous items. Else, if the current item is not filtered   */
      /* out, the current item is the first item in the column.      */
      /***************************************************************/
      if (preccPrev &&
          (preccPrev->ptlNonIconPos.x == preccCurrent->ptlNonIconPos.x))
      {
        preccCurrent = preccPrev;
        if (ISRECORDVISIBLE(preccPrev))
        {
          preccFirst = preccCurrent;
        }
      }
      else
      {
        break;
      }
    }
  }
  else
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    preccFirst = pccinfon->preccItem;
  }

  return (preccFirst);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrPaintIconView                                  */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Container Control Paint Icon View              */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints the icon view.      */
/*   The BeginPaint API is used to get the painting region.          */
/*   The first item in Z-order in the container data structure is    */
/*   used the starting record to begin painting. Checks each item.   */
/*   If it is in the viewport, draws it by calling DrawIcon function */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CnrPaintIconView( hwndCnr, hps, prctlInvalid)
HANDLE          hwndCnr;
HDC           hps;
LPRECT        prctlInvalid;

{
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  pRecord;           /* record to be checked */
  POINT         iconSize;          /* size of the icon */

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size.                                                  */
  /*******************************************************************/
  CnrCalcIconSize(hwndCnr, &iconSize);

  /*******************************************************************/
  /* Get first record in Z-order                                     */
  /*******************************************************************/
  pRecord = pccinfon->preccZ;

  while (pRecord)
  {
    if (ISRECORDVISIBLE(pRecord))
    {
      /***************************************************************/
      /* Calculate text size if it is not calculated already.        */
      /***************************************************************/
      if (!(pccinfon->ulFlagSet & FL_TEXTCALC))
      {
        IcnCalcTextSize( hps, iconSize, pccinfon,
                         pRecord->preccX->pszIcon,
                         IS_BENEATHTEXT,
                         &(pRecord->slTextSize));
      }

      /***************************************************************/
      /* Paint the record if it is in the viewport.                  */
      /***************************************************************/
      if (IsInIconViewport(pRecord, &(pccinfo->ptlOrigin),
                           prctlInvalid, &iconSize))
      {
         EraseTextOnlyInIV(hwndCnr, hps, pRecord);

         if (!(xDrawIcon (hwndCnr, hps, pRecord, &iconSize)))
         {
            return (FALSE);
         }
      }

    } /* endif */
    /*****************************************************************/
    /* Get the next record.                                          */
    /*****************************************************************/
    pRecord = pRecord->preccNextZ;
  }

  /*******************************************************************/
  /* Because it is certain that textsize is calculated,              */
  /* set the flag FL_TEXTCALC on.                                    */
  /*******************************************************************/
  pccinfon->ulFlagSet |= FL_TEXTCALC;

  return (TRUE);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrINTViewHScroll                                 */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Process H Scroll Msg                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function processes the holizontal   */
/*     scroll messages                                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*   xtai                                                            */
/*********************************************************************/
VOID FAR CnrINTViewHScroll( HWND hwndCnr, HWND hwndHscroll,             /*DAB1*/
                            int nThumbPos, WORD wNotifyCode )           /*DAB1*/
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  NOTIFYSCROLL NotifyScroll;
  RECT       rctlClient;           /* rectangle of the invalidated */
                                   /* client area */
  RECT       rctlTitle;
  RECT       rctlHscroll;          /* rectangle of the H scroll bar */
  int        lHscrollInc;          /* increase of the slider position */
  int        sHscrollInc;          /* increase of the slider position */
  int        dx;                   /* moving distance of each scroll */
  POINT      iconSize;             /* icon size */
  BOOL       scrollDirection;
  int        sScrollPos;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size.             */
  /* left-top point is (0, 0)                                        */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlClient, NULL,
                   &rctlHscroll);

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  sScrollPos = GetScrollPos(hwndHscroll, SB_CTL);

  /*******************************************************************/
  /* Distance of the each moving                                     */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) &&
      (pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
       !(pccinfo->flWindowAttr & CV_FLOW))
  {
    dx = pccinfon->lAveCharWidth / pccinfon->sHscrollFactor;
  }
  else
  {
    dx = iconSize.x / pccinfon->sHscrollFactor;
  } /* endif */

  switch (wNotifyCode)
  {
    case SB_LINEUP:
      sHscrollInc = -dx;
      break;

    case SB_LINEDOWN:
      sHscrollInc = dx;
      break;

    case SB_PAGEUP:

      /***************************************************************/
      /* Move  the width  of the client area                         */
      /***************************************************************/
      sHscrollInc = -(rctlClient.right / pccinfon->sHscrollFactor );
      break;

    case SB_PAGEDOWN:

      /***************************************************************/
      /* Move  the width  of the client area                         */
      /***************************************************************/
      sHscrollInc = rctlClient.right / pccinfon->sHscrollFactor;
      break;

    case SB_THUMBTRACK:

      return;
      break;

    case SB_THUMBPOSITION:

      /***************************************************************/
      /* User can use this msg to move by the distance they want.    */
      /***************************************************************/
      sHscrollInc = nThumbPos - pccinfon->sHscrollPos;
      break;

    default:
      sHscrollInc = 0;
      break;
  } /* endswitch */

  /*******************************************************************/
  /* Slider pos can be increased at most (max slider pos - current   */
  /* slider pos) and can be decreased at most current slider pos     */
  /*******************************************************************/
  sHscrollInc = max (-(pccinfon->sHscrollPos), min (sHscrollInc,
                     pccinfon->sHscrollMax - pccinfon->sHscrollPos)) ;

  /********************************************************************/
  /* If scrolling negatively, set an indicator for industrial size    */
  /* delta processing.                                                */
  /********************************************************************/
  if (sHscrollInc < 0)
  {
    scrollDirection = FALSE;
  }

  if (sHscrollInc != 0)
  {
    /*****************************************************************/
    /* If slider moves down, slider pos increase and ptlOrigin       */
    /* increase                                                      */
    /*****************************************************************/
    pccinfon->sHscrollPos += sHscrollInc;
    lHscrollInc = sHscrollInc * pccinfon->sHscrollFactor;

    if (!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_ICON))
    {
      pccinfo->ptlOrigin.x += lHscrollInc;
      pccinfon->rctlWorkplace.left = min(pccinfo->ptlOrigin.x,
                                  pccinfon->rctlWorkplaceItems.left);
    }
    else
    {          /* name view / text view */
      pccinfon->ptlNonIconOrg.x += lHscrollInc;
    } /* endif */

    /*****************************************************************/
    /* Get clip area relative to the window                          */
    /*****************************************************************/
    rctlClient.bottom += rctlTitle.bottom;
    rctlClient.top += rctlTitle.bottom;

    /*****************************************************************/
    /* We need to include the win target area for the repainting.    */
    /* So, we invalidate rectangle by ourselves, not by the system.  */
    /******************************************************************/
    /* Set the first item in the viewport if the current view is the  */
    /* is name/text view.                                             */
    /*  If lHscroll < 0, the window moves left, the current           */
    /*   preccFirstinXV is to the left of the updated preccFirstinXV  */
    /*   and therefore search from current preccFirstinXV with        */
    /*   flFromLeft (= FALSE).                                        */
    /*  If lHscroll > 0, the window moves right, the current          */
    /*   preccFirstinXV is left, the updated preccFirstinXV and       */
    /*   therefore search from current preccFirstinXV with            */
    /*   flFromLeft (= TRUE)                                          */
    /******************************************************************/
    if (!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_FLOW))
    {
      /****************************************************************/
      /* If flowed name view, call appropriate routines for view      */
      /****************************************************************/
      if (pccinfo->flWindowAttr & CV_NAME)
      {
        CnrSetFirstItemInNTV( hwndCnr, pccinfon->preccFirstinFNV,
                              ((BOOL)(lHscrollInc > 0)));
      }
      /****************************************************************/
      /* If flowed text view, call appropriate routines for view      */
      /****************************************************************/
      else if (pccinfo->flWindowAttr & CV_TEXT)
      {
        CnrSetFirstItemInNTV( hwndCnr, pccinfon->preccFirstinFTV,
                              ((BOOL)(lHscrollInc > 0)));
      } /* endif */
    }

    /******************************************************************/
    /*                                                                */
    /******************************************************************/
    if (pccinfo->cDelta &&
       !(pccinfo->flWindowAttr & (CV_TREE | CV_ICON)))
    {
      CnrProcessDelta(hwndCnr, scrollDirection);
    }

      /****************************************************************/
      /* Notify the application that the window is scrolled           */
      /* horizontally by the amount of sHscrollInc.                   */
      /****************************************************************/
      NotifyScroll.hwndCnr = hwndCnr;
      NotifyScroll.lScrollInc = lHscrollInc;
      NotifyScroll.fScroll = CMA_HORIZONTAL;
      NotifyOwner (hwndCnr, CN_SCROLL, (LPVOID)&NotifyScroll);

      /****************************************************************/
      /* Scroll the window right by  sHscrollInc.                     */
      /****************************************************************/
      if (rctlClient.right < abs(lHscrollInc))
      {
        PMInvalidateRect(hwndCnr, &rctlClient, TRUE);
      }
      else
      {
        ScrollWindow (hwndCnr, -lHscrollInc, 0, &rctlClient, &rctlClient);
      }

      /****************************************************************/
      /* Set the slider position                                      */
      /****************************************************************/
      SetScrollPos( hwndHscroll, SB_CTL, pccinfon->sHscrollPos, TRUE);

      UpdateWindow(hwndCnr);

  } /* endif */
  return;
}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrINTViewVScroll                                 */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Process V Scroll Msg                           */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function processes the vertical     */
/*     scroll messages                                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrINTViewVScroll( HWND hwndCnr, HWND hwndVscroll,
                            int nThumbPos, WORD wNotifyCode )
{
  int        dy;               /* moving distance for the each scroll */
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  NOTIFYSCROLL NotifyScroll;
  RECT       rctlClient;
  RECT       rctlHscroll;
  RECT       rctlTitle;
  int        lVscrollInc;
  int        sVscrollInc;
  POINT      iconSize;
  BOOL       scrollDirection=TRUE;
  int        sScrollPos;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size.             */
  /* left-bottom point is (0, 0)                                     */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlClient, NULL,
                   &rctlHscroll) ;

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  sScrollPos  = GetScrollPos(hwndVscroll, SB_CTL);

  /*******************************************************************/
  /* Distance of the each moving                                     */
  /*******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) &&
      (pccinfo->flWindowAttr & (CV_TEXT | CV_FLOW)))
  {
    dy = pccinfon->lMaxBaselineExt / pccinfon->sVscrollFactor;
  }
  else
  {
    dy = iconSize.y / pccinfon->sVscrollFactor;
  } /* endif */

  switch ( wNotifyCode)
  {
    case SB_LINEUP:
      sVscrollInc = -dy;
      break;

    case SB_LINEDOWN:
      sVscrollInc = dy;
      break;

    case SB_PAGEUP:

      /****************************************************************/
      /* Move the height of the client area                           */
      /****************************************************************/
      sVscrollInc = -(rctlClient.bottom / pccinfon->sVscrollFactor);
      break;

    case SB_PAGEDOWN:

      /****************************************************************/
      /* Move the height of the client area                           */
      /****************************************************************/
      sVscrollInc = (rctlClient.bottom / pccinfon->sVscrollFactor);
      break;

    case SB_THUMBTRACK:

      return;
      break;


    case SB_THUMBPOSITION:

      /***************************************************************/
      /* User can use this msg to move by the distance they want.    */
      /***************************************************************/
      sVscrollInc = nThumbPos - pccinfon->sVscrollPos;
      break;

    default:
       sVscrollInc = 0;
       break;
  } /* endswitch */

  /********************************************************************/
  /* Slider pos can be increased at most (max slider pos - current    */
  /* slider pos) and can be decreased at most current slider pos      */
  /********************************************************************/
  sVscrollInc = max (-(pccinfon->sVscrollPos), min (sVscrollInc,
                     pccinfon->sVscrollMax - pccinfon->sVscrollPos));

  /********************************************************************/
  /* If scrolling negatively, set an indicator for industrial size    */
  /* delta processing.                                                */
  /********************************************************************/
  if (sVscrollInc < 0)
  {
    scrollDirection = FALSE;
  }

  if (sVscrollInc != 0) {

    /*****************************************************************/
    /* If slider moves down, slider pos increases and ptlOrigin      */
    /* increases                                                     */
    /*****************************************************************/
    pccinfon->sVscrollPos += sVscrollInc;
    lVscrollInc = sVscrollInc * pccinfon->sVscrollFactor;

    if (!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_ICON))
    {
      int  cyHscroll;

      cyHscroll = GetSystemMetrics( SM_CYHSCROLL);
      pccinfo->ptlOrigin.y += lVscrollInc;
      pccinfon->rctlWorkplace.bottom =
                   max((pccinfo->ptlOrigin.y + rctlClient.bottom
                        + rctlTitle.bottom),
                       pccinfon->rctlWorkplaceItems.bottom);
    }
    else
    {          /* name view / text view */
      pccinfon->ptlNonIconOrg.y += lVscrollInc;
    } /* endif */

    /*****************************************************************/
    /* Get scroll area relative to the window(exclude H Scroll bar ) */
    /*****************************************************************/
    rctlClient.bottom += rctlTitle.bottom;
    rctlClient.top += rctlTitle.bottom;

    /*****************************************************************/
    /* We need to include the win target area for the repainting.    */
    /* So, we invalidate rectangle by ourselves, not by the system.  */
    /*****************************************************************/
    /* Set the first item in the viewport if the the current view    */
    /* is icon/text view. Because sVscroll > 0, the window moves     */
    /* down, the current preccFirstiXV is above the updated          */
    /* preccFirstinXV and therefore search from current              */
    /* preccFirstinXV with flFromAbove (= TRUE)                      */
    /* Set the first item in the viewport if the the current view    */
    /* is icon/text view. Because sVscroll < 0, the window moves     */
    /* up, the current preccFirstinXV is  below the updated          */
    /* preccFirstinXV and therefore search from current              */
    /* preccFirstinXV with flFromAbove (= FALSE).                    */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_TREE)
    {
      CnrTRVSetFirstItem (hwndCnr, pccinfon->preccFirstinTRV,
                          ((BOOL)(lVscrollInc > 0)));
    }
    /******************************************************************/
    /* If non-flowed name view, set first item based on direction     */
    /******************************************************************/
    else if ((pccinfo->flWindowAttr & CV_NAME) &&
      !(pccinfo->flWindowAttr & CV_FLOW) )
    {
      CnrSetFirstItemInNTV (hwndCnr, pccinfon->preccFirstinNV,
                            ((BOOL)(lVscrollInc > 0)));
    }
    /******************************************************************/
    /* If non-flowed text view, set first item based on direction     */
    /******************************************************************/
    else if ((pccinfo->flWindowAttr & CV_TEXT) &&
          !(pccinfo->flWindowAttr & CV_FLOW))
    {
      CnrSetFirstItemInNTV (hwndCnr, pccinfon->preccFirstinTV,
                            ((BOOL)(lVscrollInc > 0)));
    }
    /******************************************************************/
    /* If autoposition in icon view                                   */
    /******************************************************************/
    else if ((pccinfo->flWindowAttr & CV_ICON) &&
      (GetWindowLong( hwndCnr, GWL_STYLE ) & CCS_AUTOPOSITION))
    {
      CnrSetFirstItemAutoPosIconV (hwndCnr, pccinfon->preccFirstinAutoPosIV,
                                   ((BOOL)(lVscrollInc > 0)));
    }

    /******************************************************************/
    /*                                                                */
    /******************************************************************/
    if (pccinfo->cDelta &&
       !(pccinfo->flWindowAttr & (CV_TREE | CV_ICON)))
    {
      CnrProcessDelta(hwndCnr, scrollDirection);
    }
      /*****************************************************************/
      /* Notify the application that the window is scrolled            */
      /* vertically   by the amount of sVscrollInc.                    */
      /*****************************************************************/
      NotifyScroll.hwndCnr = hwndCnr;
      NotifyScroll.lScrollInc = lVscrollInc ;
      NotifyScroll.fScroll = CMA_VERTICAL;
      NotifyOwner (hwndCnr, CN_SCROLL, (LPVOID)&NotifyScroll);

      /****************************************************************/
      /* Scroll the window down by sVscrollInc.                       */
      /****************************************************************/
      if ((rctlClient.bottom - rctlClient.top) <
           abs(lVscrollInc))
      {
        PMInvalidateRect(hwndCnr, &rctlClient, TRUE);
      }
      else
      {
        ScrollWindow (hwndCnr, 0, -lVscrollInc, &rctlClient, &rctlClient);
      } /* endif */

      /****************************************************************/
      /* Set the slider position                                      */
      /****************************************************************/
      SetScrollPos( hwndVscroll, SB_CTL, pccinfon->sVscrollPos, TRUE);

      UpdateWindow(hwndCnr);

  } /* endif */
  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetFirstItemInAutoPosIconV                     */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set the First Item in the icon view with       */
/*     CCS_AUTOPOSITION                                               */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in    */
/*     viewport searching backward or forward depending on the flag. */
/*     If preccCurrent is NULL, search from the last item            */
/*     (flFromLeft = FALSE) or from the first item (flFromLeft =     */
/*     TRUE)  of the list.                                           */
/*     Then, set the first item in the cnrinfon.                     */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetFirstItemAutoPosIconV(hwndCnr, preccCurrent, flFromAbove)
HANDLE         hwndCnr;
LPRECORDCOREN preccCurrent;
BOOL         flFromAbove;

{
  LPCNRINFON    pccinfon;
  RECT         rctlHscroll;        /* H scroll bar rectangle */
  RECT         rctlViewport;       /* rectangle of the viewport */
  RECT         rctlTitle;
  RECT         rctlText;
  RECT         rctlIcon;
  POINT        iconSize;           /* size of the icon */
  LPRECORDCOREN preccNext;          /* next item of the current item */
  BOOL         found = FALSE;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size              */
  /* left-bottom point of a rectangle is (0, 0)                      */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlViewport, NULL,
                   &rctlHscroll);


  if (flFromAbove)
  {
    /*******************************************************************/
    /* If preccCurrent is NULL set it as as the first item in the      */
    /* list.                                                           */
    /*******************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccItem;
    }

    while (preccCurrent && !found)
    {
      /*****************************************************************/
      /* Get the top of the item to check if it is in the              */
      /* viewport                                                      */
      /*****************************************************************/
      CnrQueryIconTextRect(hwndCnr, preccCurrent, &iconSize,
                           FALSE, NULL, &rctlText);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
          (rctlText.bottom < rctlViewport.top + rctlTitle.bottom))
      {
        preccCurrent = preccCurrent->preccNextItem;
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      pccinfon->preccFirstinAutoPosIV = preccCurrent;
    }
    else
    {
      /***************************************************************/
      /* If no item is in the viewport set the first item as the     */
      /* first item of the list.                                     */
      /***************************************************************/
      pccinfon->preccFirstinAutoPosIV = pccinfon->preccItem;
    } /* endif*/
  }
  else
  {
    /*****************************************************************/
    /* If preccCurrent is NULL set it as as the last  item in the    */
    /* list.                                                         */
    /*****************************************************************/
    if (!preccCurrent)
    {
      preccCurrent = pccinfon->preccLastItem;
    } /* endif */

    preccNext = pccinfon->preccFirstinAutoPosIV;
    /*****************************************************************/
    /* Find the first item of which left boundary is out of the      */
    /* viewport.                                                     */
    /*****************************************************************/
    while (preccCurrent && !found)
    {
      /***************************************************************/
      /* Get the top of the item to check if it is in the            */
      /* viewport                                                    */
      /***************************************************************/
      CnrQueryIconTextRect(hwndCnr, preccCurrent, &iconSize,
                           FALSE, &rctlIcon, NULL);

      if (!(ISRECORDVISIBLE(preccCurrent)) ||
           (rctlIcon.top > (rctlViewport.top + rctlTitle.bottom)))
      {
        /*************************************************************/
        /* Save preccCurrent as preccPrev for the case that          */
        /* preccCurrent is totally out of viewport.(Above the        */
        /* viewport.)                                                */
        /*************************************************************/
        if (ISRECORDVISIBLE(preccCurrent))
        {
          preccNext = preccCurrent;
        }
        preccCurrent = preccCurrent->preccPrevItem;
      }
      else
      {
        found = TRUE;
      }
    } /* end while */

    if (preccCurrent)
    {
      /***************************************************************/
      /* If the first item whose top is above the viewport has       */
      /* bottom in the viewport, it is the first item.(Patially      */
      /* shown) Else, the first item in the viewport is the          */
      /* next item of the current item.                              */
      /***************************************************************/
      CnrQueryIconTextRect(hwndCnr, preccCurrent, &iconSize,
                           FALSE, NULL, &rctlText);
      if (rctlText.bottom > (rctlViewport.top + rctlTitle.bottom))
      {
        pccinfon->preccFirstinAutoPosIV = FindFirstIteminRow(hwndCnr,
                                          preccCurrent);
      }
      else
      {
        pccinfon->preccFirstinAutoPosIV = preccNext;
      }
    }
    else
    {
      /***************************************************************/
      /* No item is above the viewport.                              */
      /* The first item in viewport is the first item in the         */
      /* list.                                                       */
      /***************************************************************/
      pccinfon->preccFirstinAutoPosIV = pccinfon->preccItem;
    }
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  FindFirstIteminRow                                */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Find First Item In a Row                       */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function finds the first item in a  */
/*     a row    searching backward from preccCurrent. If             */
/*     preccCurrent is NULL the first item is the first item         */
/*     of the list(the first item of the first column).              */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
LPRECORDCOREN FAR FindFirstIteminRow (hwndCnr, preccCurrent)
HANDLE          hwndCnr;
LPRECORDCOREN  preccCurrent;

{
  LPCNRINFON        pccinfon;
  LPRECORDCOREN     preccPrev;
  LPRECORDCOREN     preccFirst;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);

  if (preccCurrent)
  {
    preccFirst = preccCurrent;

    while (preccCurrent)
    {
      preccPrev = preccCurrent->preccPrevItem;

      /***************************************************************/
      /* If previous item  exists and it's x position is the same as */
      /* x position of the current item, continue checking the       */
      /* previous items. Else, if the current item is not filtered   */
      /* out, the current item is the first item in the column.      */
      /***************************************************************/
      if (preccPrev)
      {
        if (ISRECORDVISIBLE(preccPrev))
        {
          if (preccPrev->ptlIcon.y == preccFirst->ptlIcon.y)
          {
            preccFirst = preccPrev;
          }
          else
          {
            break;
          } /* endif */
        }
        preccCurrent = preccCurrent->preccPrevItem;
      }
      else
      {
        preccFirst = pccinfon->preccItem;
        if (!ISRECORDVISIBLE(preccFirst))
        {
          preccFirst = GetNextUnfilteredItem(preccFirst);
        }
        break;
      }
    } /* endwhile */
  }
  else
  {
    preccFirst = pccinfon->preccItem;

    if (preccFirst)
    {
      if (!ISRECORDVISIBLE(preccFirst))
      {
        preccFirst = GetNextUnfilteredItem(preccFirst);
      }
    }
  } /* endif */

  return (preccFirst);
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrSetIconViewScroll                              */
/*                                                                   */
/* Usage: External                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the icon view                         */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrSetIconViewScroll(hwndCnr)
HANDLE        hwndCnr;
{
  HANDLE      hwndHscroll;
  HANDLE      hwndVscroll;
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  RECT        rctlWin;
  RECT        rctlClient;          /* rectangle of the client area */
  RECT        rctlHScroll;         /* rectangle of the H scroll bar */
  RECT        rctlTitle;           /* rectangle of the title */
  POINT       iconSize;            /* size of the icon */

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (CheckNeedResize(hwndCnr))
  {
    pccinfon->ulFlagSet &= ~FL_SIZECHILDREN;
    /******************************************************************/
    /* resize the scroll bars                                         */
    /******************************************************************/
    PMGetClientRect(hwndCnr, &rctlWin);

    CnrSize(hwndCnr, rctlWin.right, rctlWin.bottom);
    pccinfon->ulFlagSet |= FL_SIZECHILDREN;
  }

  /*******************************************************************/
  /* Get viewport rect, H scroll bar rect and icon size.             */
  /* left-bottom point is (0, 0)                                     */
  /*******************************************************************/
  CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlClient, NULL,
                   &rctlHScroll);

  /*******************************************************************/
  /* Get handle of the scroll bar                                    */
  /*******************************************************************/
  hwndHscroll = GetDlgItem (hwndCnr, CID_HSCROLL);
  hwndVscroll = GetDlgItem (hwndCnr, CID_VSCROLL);

  if (pccinfon->preccItem) {

    /*****************************************************************/
    /* Set Scroll bar range and pos                                  */
    /*****************************************************************/
    SetScrollBar(hwndCnr, hwndHscroll, rctlClient.right,
                 pccinfon->rctlWorkplace.right,
                 pccinfon->rctlWorkplace.left,
                 pccinfo->ptlOrigin.x,
                 &(pccinfon->sHscrollPos),
                 &(pccinfon->sHscrollMax),
                 &(pccinfon->sHscrollFactor), FALSE);

    SetScrollBar(hwndCnr, hwndVscroll, rctlClient.bottom,
                 pccinfon->rctlWorkplace.bottom,
                 pccinfon->rctlWorkplace.top,
                 pccinfo->ptlOrigin.y + rctlTitle.bottom,
                 &(pccinfon->sVscrollPos),
                 &(pccinfon->sVscrollMax),
                 &(pccinfon->sVscrollFactor), TRUE);
  }
  else
  {
    /*****************************************************************/
    /* Disable the scroll bar if nothing to show                     */
    /*****************************************************************/
    EnableWindow (hwndHscroll, FALSE);
    EnableWindow (hwndVscroll, FALSE);
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  SetScrollBar                                      */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Set Scroll Bar Range and Pos                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function set scroll bar range and   */
/*   position of the slider in the icon view                         */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR SetScrollBar(hwndCnr, hwndScroll, clientRB, workplaceRB, workplaceLT,
               originXY, psScrollPos, psScrollMax, psScrollFactor, isV)

HANDLE      hwndCnr;
HANDLE      hwndScroll;
int         clientRB;       /* right for H, bottom for V of client rect */
int         workplaceRB;    /* right for H, bottom for V of workplace rect */
int         workplaceLT;    /* left for H, top for V of workplace rect */
int         originXY;       /* x for H, y for V of client window origin */
LPINT       psScrollPos;
LPINT       psScrollMax;
LPINT       psScrollFactor;
BOOL        isV;            /* if V scroll, TRUE */

{
  int         sLTPos;         /* left or top # of scroll                  */
  int         sRBPos;         /* right or bottom # of scroll              */
  int         sMaxPos;        /* total # of scroll                        */
  int         bottom;         /* bottom of the certain range */
  int         top;            /* top of the certain range */
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  int         sLTPosTemp;
  int         sLTPosMove;
  POINT       iconSize;
  RECT        rctlIcon;
  RECT        rctlHScroll;
  RECT        rctlClient;
  RECT        rctlTitle;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* The range of the left or top is  the space beyond client        */
  /* area to the left or top of the client area                      */
  /*******************************************************************/
  sLTPos = originXY - workplaceLT;

  /*******************************************************************/
  /* The range of the right or bottom is  the space beyond client    */
  /* area to the right or bottom of the client area                  */
  /*******************************************************************/
  sRBPos = workplaceRB - originXY - clientRB;

  if ((GetWindowLong( hwndCnr, GWL_STYLE ) & CCS_AUTOPOSITION) &&
      isV )
  {
    /*****************************************************************/
    /* Get viewport rect, H scroll bar rect and icon size            */
    /* left-bottom point of a rectangle is (0, 0)                    */
    /*****************************************************************/
    CnrQuerySizeRect(hwndCnr, &iconSize, &rctlTitle, &rctlClient, NULL,
                     &rctlHScroll) ;

    pccinfon->preccFirstinAutoPosIV = FindFirstIteminRow(hwndCnr,
                                      pccinfon->preccFirstinAutoPosIV);

    if (pccinfon->ulFlagSet & FL_NEWVIEWSCROLLSET )
    {
      /***************************************************************/
      /* If this is not the first setting of the scroll bar in the   */
      /* current view, just consider the scroll pos to get top       */
      /* scroll range in pixels.                                     */
      /***************************************************************/
      sLTPos = pccinfon->sVscrollPos * pccinfon->sVscrollFactor;
      if (pccinfon->ulFlagSet & FL_ITEMNUMCHANGED)
      {
        pccinfo->ptlOrigin.y = sLTPos;
        sRBPos = pccinfon->rctlWorkplace.bottom -
                 pccinfo->ptlOrigin.y -
                 clientRB - rctlTitle.bottom;
      }
    }
    else
    {

      /***************************************************************/
      /* If the first item in the view is not specified, assign      */
      /* the first item in the list as the first item in the view.   */
      /* And no more scroll up (sRTPos = 0 )                         */
      /***************************************************************/
      if (!pccinfon->preccFirstinAutoPosIV)
      {
        pccinfon->preccFirstinAutoPosIV = pccinfon->preccItem;
      }
      else
      {
        /*************************************************************/
        /* Get the distance from top of list and top of the first    */
        /* item in viewport. This is the current pos of slider,      */
        /* and shows the possible distance to scroll up(sRTPos)      */
        /*************************************************************/
        if (pccinfon->preccItem &&
           (pccinfon->preccItem != pccinfon->preccFirstinAutoPosIV))
        {
          CnrQueryIconTextRect(hwndCnr, pccinfon->preccItem, &iconSize,
                               FALSE, &rctlIcon, NULL);
          top = rctlIcon.top;
          CnrQueryIconTextRect(hwndCnr,
                               pccinfon->preccFirstinAutoPosIV,
                               &iconSize, FALSE, &rctlIcon, NULL);
          bottom = rctlIcon.top;

          sLTPos = bottom - top;
        } /* endif */
      } /* endif */
      pccinfon->ulFlagSet |= FL_NEWVIEWSCROLLSET;
      pccinfo->ptlOrigin.y = sLTPos;

      sRBPos = pccinfon->rctlWorkplace.bottom -
               pccinfo->ptlOrigin.y -
               clientRB - rctlTitle.bottom;

    } /* endif */

    /*****************************************************************/
    /* Calculate total scroll range in pixels.                       */
    /*****************************************************************/
    sLTPosTemp = max(0, min(sLTPos, sLTPos + sRBPos));
    sLTPosMove = sLTPosTemp - sLTPos;
    pccinfo->ptlOrigin.y += sLTPosMove;
    sLTPos = sLTPosTemp;
    sRBPos = max(0, sRBPos);

    if (sLTPosMove < 0)
    {
      CnrSetFirstItemAutoPosIconV(hwndCnr, pccinfon->preccFirstinAutoPosIV,
                                  FALSE);
    } /* endif */
    pccinfon->ulFlagSet &= ~FL_ITEMNUMCHANGED;
  }
  else
  {
    sLTPos = max(0, sLTPos);
    sRBPos = max(0, sRBPos);
  } /* endif */

  /*******************************************************************/
  /* Calculate total scroll range                                    */
  /*******************************************************************/
  sMaxPos = sLTPos + sRBPos;

  /*******************************************************************/
  /* Calculate the minimum unit of scrolling.                        */
  /*******************************************************************/
  *psScrollFactor = CalcScrollFactor(hwndCnr, sMaxPos, isV);

  /*******************************************************************/
  /* Calculate the scroll ranges  in the unit.                       */
  /*******************************************************************/
  if (sRBPos % (*psScrollFactor))
  {
    sRBPos = sRBPos / (*psScrollFactor) + 1;
  }
  else
  {
    sRBPos = sRBPos / (*psScrollFactor) ;
  } /* endif */

  if (sLTPos % (*psScrollFactor))
  {
    sLTPos = sLTPos / (*psScrollFactor) + 1;
  }
  else
  {
    sLTPos = sLTPos / (*psScrollFactor) ;
  } /* endif */

  /*******************************************************************/
  /* Calculate total scroll range in the unit.                       */
  /*******************************************************************/
  sMaxPos = sRBPos + sLTPos;

  if (sMaxPos)
  {
    /*****************************************************************/
    /* Enable the scroll bar if some outside  viewport               */
    /*****************************************************************/
     EnableWindow (hwndScroll, TRUE);

      /***************************************************************/
      /* Set scroll bar. Range is 0 to sMaxPos. Current Pos is sLTPos*/
      /***************************************************************/
      SetScrollRange(hwndScroll, SB_CTL, 0, sMaxPos, FALSE);
      SetScrollPos  (hwndScroll, SB_CTL, sLTPos, TRUE);

      *psScrollPos = sLTPos;
  }
  else
  {
    /*****************************************************************/
    /* Disable the scroll bar if nothing outside  viewport           */
    /*****************************************************************/
     EnableWindow (hwndScroll, FALSE);
     *psScrollPos = 0;

  } /* endif */
  *psScrollMax = sMaxPos;

  return;

}


/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  IsInIconViewport                                  */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Query An Item Is In Icon View                  */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function query if an icon is in     */
/*   the icon viewport                                               */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT: LPRECORDCORE pRecord                                        */
/*        LPPOINT pptlOrigin                                         */
/*        LPRECT  prctlWindow                                        */
/*        LPRECT  prctlLTitle                                        */
/*        LPRECT  prctlHScroll                                       */
/*        LPRECT  prctlVScroll                                       */
/*        LPPOINT piconSize                                          */
/*        LPPOINT ptextSize                                          */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
BOOL FAR IsInIconViewport (pRecord, pptlOrigin, prctlWindow, piconSize)

LPRECORDCOREN pRecord;
LPPOINT       pptlOrigin;
LPRECT        prctlWindow;
LPPOINT       piconSize;
{
  int     ixRecordLeft;
  int     ixRecordRight;

  /*******************************************************************/
  /* If bottom of the item (ptlIcon.y - cy of text size -            */
  /* emph space) is below of the top of the invalidated rect,        */
  /* top of the item is above of the bottom of the invalidated rect, */
  /* the right of the item is to the right of the left of the        */
  /* invalidated rect and                                            */
  /* the left of the item is to the left of the right of the         */
  /* invalidated rect,                                               */
  /* then return TRUE.                                               */
  /*******************************************************************/
  if ((long)(piconSize->x + 2*ICN_EMPHSPACE) > pRecord->slTextSize.cx)  /*DAB1*/
  {
    ixRecordLeft  = pRecord->ptlIcon.x - ICN_EMPHSPACE;
    ixRecordRight = pRecord->ptlIcon.x + piconSize->x + ICN_EMPHSPACE;
  }
  else
  {
    ixRecordLeft  = ((long)pRecord->ptlIcon.x -                         /*DAB1*/
                    (pRecord->slTextSize.cx - piconSize->x) / 2);       /*DAB1*/
    ixRecordRight = (pRecord->ptlIcon.x +                               /*DAB1*/
                    (piconSize->x + pRecord->slTextSize.cx) / 2);       /*DAB1*/
  }

  if (((pptlOrigin->y + prctlWindow->top ) <
       (pRecord->ptlIcon.y + piconSize->y + pRecord->slTextSize.cy      /*DAB1*/
        + ICN_EMPHSPACE)) &&
      ((pRecord->ptlIcon.y - ICN_EMPHSPACE) <
       (pptlOrigin->y + prctlWindow->bottom)) &&
       (ixRecordRight > (prctlWindow->left + pptlOrigin->x)) &&
       (ixRecordLeft < (pptlOrigin->x + prctlWindow->right )))
  {
    return TRUE;
  }
  else
  {
    return FALSE;
  }
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrInvalidateRecord                               */
/*                                                                    */
/*  Description:    This function is only called when the application */
/*                  sends a CM_INVALIDATERECORD message.  The function*/
/*                  copies the external data for flRecordAttr and     */
/*                  ptlIcon to the internal recordcore, then          */
/*                  invalidates those items by calling                */
/*                  InvalidateRecord.                                 */
/*                                                                    */
/*  Input:          hwndCnr  - container window handle                */
/*                  mp1      - pointer to an array of records to be   */
/*                             invalidated.                           */
/*                  mp2      - number of records in array.            */
/*                                                                    */
/*  Output:         BOOL - TRUE:  Function successful.                */
/*                         FALSE: Function unseccessful.              */
/*                                                                    */
/*  Programmer: Peter Haggar and Tai Woo Nam                          */
/**********************************************************************/
BOOL FAR CnrInvalidateRecord (hwndCnr, mp1, mp2)

HANDLE    hwndCnr;
long  mp1;
long  mp2;
{
  LPCNRINFON     pccinfon;
  LPRECORDCORE   FAR *pRecordArray;
  LPRECORDCOREN  preccnTemp;
  WORD           cNumRecord;
  WORD           i;
  BOOL           rc;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = PCCINFONOF(hwndCnr);

  /*******************************************************************/
  /* Get array of the record and the  number of the records          */
  /* to be invalidated                                               */
  /*******************************************************************/
  pRecordArray = (LPVOID)mp1;
  cNumRecord = LOWORD(mp2);

  /*******************************************************************/
  /* If we need to verify the pointers, do it.                       */
  /*******************************************************************/
  if ((GetWindowLong (hwndCnr, GWL_STYLE) & CCS_VERIFYPOINTERS) &&
        !(CnrVerifyPointers (hwndCnr, pRecordArray, (long)cNumRecord)))
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return FALSE;
  }

  if (cNumRecord)
  {
    /******************************************************************/
    /* Loop through the array passed in by the application and copy   */
    /* the flRecordAttr and ptlIcon fields into the internal record   */
    /* for each one.  THEN, and pay attention to this, we reassign    */
    /* the array pointer to point to the internal record.  This is so */
    /* we can then call the InvalidateRecord function which takes an  */
    /* array of internal records.                                     */
    /******************************************************************/
    for (i = 0; i < cNumRecord; i++)
    {
      preccnTemp = GetRecordNFromX (hwndCnr, pRecordArray[i]);

      /***************************************************************/
      /* Ensure both COLLAPSED and EXPANDED are not set by turning   */
      /* off the one if the other is set                             */
      /***************************************************************/
      if (pRecordArray[i]->flRecordAttr & CRA_COLLAPSED)
      {
        pRecordArray[i]->flRecordAttr &= ~CRA_EXPANDED;
      }

      /***************************************************************/
      /* Initialize CRA_COLLAPSED if CRA_EXPANDED not specified.     */
      /***************************************************************/
      if (!(pRecordArray[i]->flRecordAttr & CRA_EXPANDED))
      {
        pRecordArray[i]->flRecordAttr |= CRA_COLLAPSED;
      }

      preccnTemp->flRecordAttr = (pRecordArray[i])->flRecordAttr;
      preccnTemp->ptlIcon = (pRecordArray[i])->ptlIcon;
      pRecordArray[i] = (LPRECORDCORE)preccnTemp;
    }
  }
  else
  {
    /******************************************************************/
    /* Since the app wants to refresh all items, start with the first */
    /* and blast though to the end.  For each one, copy the external  */
    /* information into the internal record.                          */
    /******************************************************************/
    preccnTemp = pccinfon->preccItem;
    while (preccnTemp)
    {
      /***************************************************************/
      /* Ensure both COLLAPSED and EXPANDED are not set by turning   */
      /* off the one if the other is set                             */
      /***************************************************************/
      if (preccnTemp->preccX->flRecordAttr & CRA_COLLAPSED)
      {
        preccnTemp->preccX->flRecordAttr &= ~CRA_EXPANDED;
      }

      /***************************************************************/
      /* Initialize CRA_COLLAPSED if CRA_EXPANDED not specified.     */
      /***************************************************************/
      if (!(preccnTemp->preccX->flRecordAttr & CRA_EXPANDED))
      {
        preccnTemp->preccX->flRecordAttr |= CRA_COLLAPSED;
      }

      preccnTemp->flRecordAttr = preccnTemp->preccX->flRecordAttr;
      preccnTemp->ptlIcon = preccnTemp->preccX->ptlIcon;
      preccnTemp = GetNextItem (pccinfon, preccnTemp);
    }

    pRecordArray = NULL;
  }

  rc = InvalidateRecord (hwndCnr, (PLPRECORDCOREN)pRecordArray,
                         cNumRecord, HIWORD(mp2));

  /********************************************************************/
  /* Now, before we return we need to reset the array that the app.   */
  /* passed in to be the same as it was when they passed it to us.    */
  /* Above, we changed the pointers in the array to point to the      */
  /* internal record structure so we could call InvalidateRecord.     */
  /* The pointers we changed were the applications pointers, and we   */
  /* are not supposed to change them.  So, we will now change them    */
  /* back to the way they were......ie. the pointers in the array     */
  /* will point to the external records.                              */
  /********************************************************************/
  if (cNumRecord)
  {
    for (i = 0; i < cNumRecord; i++)
    {
      pRecordArray[i] = ((LPRECORDCOREN)(pRecordArray[i]))->preccX;
    }
  }

  return (rc);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:       InvalidateRecord                             */
/*                                                                    */
/*  Descriptive Name:   Control Invalidate Record to Refresh          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*           mp1 -                                                    */
/*           mp2 -                                                    */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR InvalidateRecord (					/*RKK1*/
  HANDLE             hwnd,
  LPRECORDCOREN FAR *pRecordArray,
  WORD               cNumRecord,
  WORD               flInvalidate  )				/*RKK1*/
{
  HDC             hps;
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  LPRECORDCOREN   precord;          /* record to be invalidated       */
  LPRECORDCOREN   prcFirstInvalid;  /* first item to be invalidated   */
  LPSTR           pszString;        /* string of the text             */
  RECT            rctlInvalidate;
  POINT           iconSize;         /* size of the icon               */
  DWORD           i;                /* counter                        */
  BOOL            flReposition = FALSE;
  WORD            usLongestChanged = 0;
  SIZEL           slNewTextSize;
  RECT            CnrRect;
//ERRORID         ErrorID;
  LPFIELDINFO     pFieldInfo;
  HDC             hpsLeft;
  HDC             hpsRight;
  RECT            RowRectLeft;
  RECT            RowRectRight;
  BOOL            bRowHeightChg;                                        //@B01
  BOOL            flRepaintAll = FALSE;
  int             sOldVscrollPos;
  int             sOldHscrollPos;

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = PCCINFONOF (hwnd);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    return (TRVInvalidateRecord (hwnd, pRecordArray,
                                 cNumRecord, flInvalidate));
  }

  hps = PMGetFlippedDC(hwnd);

  if (flInvalidate & CMA_NOREPOSITION)
  {
    flReposition = FALSE;
  }
  else
  {
    if (flInvalidate & CMA_TEXTCHANGED)
    {
      if (!cNumRecord)
      {
        flReposition = TRUE;
      }
      else
      {
        /***********************************************************/
        /* If CCS_AUTOPOSITION is set on, set FL_NEEDARRANGE on     */
        /* to rearrange icons when painting in icon view           */
        /***********************************************************/
        if (!(pccinfo->flWindowAttr & CV_ICON) &&
            (GetWindowLong( hwnd, GWL_STYLE ) &
             CCS_AUTOPOSITION))
        {
          pccinfon->ulFlagSet |= FL_NEEDARRANGE;
        } /* endif */

        CnrCalcIconSize( hwnd, &iconSize );
        for (i = 0; i < cNumRecord ; i++)
        {
          precord = *(pRecordArray + i);

          if (pccinfo->flWindowAttr & CV_DETAIL)
          {
             /*******************************************************/
             /* Check to see if the height of the row changed or if */
             /* any of the columns are not fixed.  If either of     */
             /* these are ture, we need to reposition the records.  */
             /*******************************************************/
             if ((bRowHeightChg =
                      CnrRowHeightChanged (hwnd, precord)) ||
                 (!(CnrAllColumnsFixedWidth (hwnd))))
             {
                flReposition = TRUE;
             }
             else
             /*******************************************************/
             /* We need to calculate the new size of the text or    */
             /* Icon in the first column.  If this were to change,  */
             /* the size that is stored in the RECORDCORE needs to  */
             /* be updated.                                         */
             /*******************************************************/
             {
                if (!(pFieldInfo = pccinfon->pFirstVisibleFI))
                {
                   goto exiterror;
                }

                if (pFieldInfo->flData & CFA_BITMAPORICON)
                {
                  precord->slTextSize.cx = iconSize.x;                  /*DAB1*/
                  precord->slTextSize.cy = iconSize.y;                  /*DAB1*/
                }
                else
                {
                   if (!(pszString = CnrGetColumnData (hwnd,
                                                       precord->preccX,
                                                       pFieldInfo)))
                   {
//                    ErrorID = WinGetLastError (WinQueryAnchorBlock
//                                               (HANDLE_DESKTOP));

//                    if (ErrorID == PMERR_INSUFFICIENT_MEMORY)
//                    {
//                       SETERROR(ErrorID);
//                       goto exiterror;
//                    }

//                    SETERROR(ErrorID);
                   }

                   IcnCalcTextSize (hps,
                                    iconSize,
                                    pccinfon,
                                    pszString,
                                    IS_SIDETEXT,
                                    &slNewTextSize);

                   precord->slTextSize = slNewTextSize;
                }
             }
          }
          else
          {
            if (pccinfo->flWindowAttr & CV_ICON)
            {
              IcnCalcTextSize (hps, iconSize, pccinfon,
                               precord->preccX->pszIcon,
                               IS_BENEATHTEXT,
                               &slNewTextSize);
              if (GetWindowLong( hwnd, GWL_STYLE ) &
                    CCS_AUTOPOSITION)
              {
                if (!flReposition &&
                   ((precord->slTextSize.cx != slNewTextSize.cx ) ||    /*DAB1*/
                    ((precord->slTextSize.cy != slNewTextSize.cy) &&    /*DAB1*/
                     ChangeRowHeight(hwnd, precord, slNewTextSize))))
                {
                  flReposition = TRUE;
                } /* endif */
              }
              else
              {
                flReposition = TRUE;
              }
            }
            else
            {
              if (pccinfo->flWindowAttr & CV_NAME)
              {
                pszString = GETLPSTRFORNV(precord->preccX);
              }
              else
              {
                pszString = GETLPSTRFORTV(precord->preccX);
              }

              IcnCalcTextSize (hps, iconSize, pccinfon,
                               pszString, IS_SIDETEXT,
                               &slNewTextSize);

              if ((pccinfo->flWindowAttr & CV_FLOW) && !flReposition)
              {
                if (((precord->slTextSize.cy != slNewTextSize.cy ) &&   /*DAB1*/
                      ChangeItemHeight(hwnd, precord, slNewTextSize,
                                       iconSize)) ||
                     ((precord->slTextSize.cx != slNewTextSize.cx) &&   /*DAB1*/
                      ChangeColumnLength (precord, slNewTextSize)))
                {
                  flReposition = TRUE;
                }
              }
              else
              {
                if (usLongestChanged)
                {
                  if (usLongestChanged < (WORD)slNewTextSize.cx)        /*DAB1*/
                  {
                    usLongestChanged = (WORD)slNewTextSize.cx;          /*DAB1*/
                  }
                }
                else
                {
                  if (precord->slTextSize.cx != slNewTextSize.cx)       /*DAB1*/
                  {
                    usLongestChanged =
                          ChangeColumnLength (precord, slNewTextSize);
                  }
                }

                if (!flReposition &&
                     ChangeItemHeight(hwnd, precord, slNewTextSize,
                                         iconSize))
                {
                  flReposition = TRUE;
                }
              } /* endif */
            }
          }
        } /* end for */
      } /* endif */
    }
    else
    {
      flReposition = TRUE;
    }
  }
  if (!flReposition)
  {
    if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
        (!(pccinfo->flWindowAttr & CV_FLOW)) &&
         usLongestChanged )
    {
      sOldVscrollPos = pccinfon->sVscrollPos;
      sOldHscrollPos = pccinfon->sHscrollPos;
      UpdateColumnLongest( hwnd, pccinfon->preccItem,
                           usLongestChanged);
      pccinfon->ulFlagSet &= ~(FL_SETSCROLL | FL_SIZECHILDREN );
      CnrSetUpNewView(hwnd, hps);

      if ((sOldVscrollPos != pccinfon->sVscrollPos) ||
          (sOldHscrollPos != pccinfon->sHscrollPos))
      {
        flRepaintAll = TRUE;
      }
    }

    if (!cNumRecord)
    {
#if 0
      if (pccinfo->flWindowAttr & CV_DETAIL)
      {
         PMGetClientRect (hwnd, &CnrRect);

         if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
             ((long)pccinfo->xVertSplitbar > CnrRect.right))
         {
            CnrDetailsVCtlProc (hwnd, WM_PAINT, NULL, NULL);
         }
      }
#endif
      InvalidateChildren (hwnd, FALSE, FALSE);

      UpdateWindow (hwnd);
    }
    else
    {
      CnrCalcIconSize (hwnd, &iconSize);

      /**************************************************************/
      /* In the icon view, calculate the text size and redraw       */
      /* each item in the array.                                    */
      /**************************************************************/
      for (i = 0; i < cNumRecord ; i++)
      {
        precord = *(pRecordArray + i);

        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          /*********************************************************/
          /* Erase the row, then draw it.                          */
          /*********************************************************/
          if (!(CnrDrawEraseRow (hwnd, precord, FALSE)))
          {
            goto exiterror;
          }

          if (!(CnrDrawEraseRow (hwnd, precord, TRUE)))
          {
            goto exiterror;
          }

          if (precord->flRecordAttr & CRA_CURSORED)
          {
               hpsLeft = PMGetFlippedDC (GetDlgItem (hwnd,
                                                 CID_LEFTDVWND));
             CnrCalcRowRect (hwnd, precord, TRUE, &RowRectLeft);
             RowRectLeft.top ++;
             RowRectLeft.bottom --;
             RowRectLeft.right += 2;

             if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
             {
               hpsRight = PMGetFlippedDC (GetDlgItem (hwnd,
                                                    CID_RIGHTDVWND));
               CnrCalcRowRect (hwnd, precord, FALSE, &RowRectRight);
               RowRectRight.top ++;
               RowRectRight.bottom --;
               RowRectRight.left -= 2;
               RowRectRight.right += 2;
             }

             CnrDrawCursorBox (hwnd, hpsLeft, precord,
                               RowRectLeft, TRUE);

             if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
             {
               CnrDrawCursorBox (hwnd, hpsRight, precord,
                                 RowRectRight, TRUE);
             }

             ReleaseDC(GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);

             if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
             {
               ReleaseDC(GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
             }
          }
        }
        else
        {
          if (pccinfo->flWindowAttr & CV_ICON)
          {
            if (flInvalidate & CMA_ERASE)
            {
              CnrEraseIcon(hwnd, hps, precord, TRUE);
            }
            else
            {
              EraseTextOnlyInIV(hwnd, hps, precord);
            } /* endif */

            if (!(flInvalidate & CMA_NOREPOSITION))
            {
              IcnCalcTextSize (hps, iconSize, pccinfon,
                               precord->preccX->pszIcon,
                               IS_BENEATHTEXT,
                               &(precord->slTextSize));
            }

            EraseTextOnlyInIV(hwnd, hps, precord);

            if (!(CnrDrawRecord(hwnd, hps, precord, TRUE)))
            {
               goto exiterror;
            }
          }
          else
          {
            /****************************************************/
            /* Update the text size                             */
            /****************************************************/
            if (!(flInvalidate & CMA_NOREPOSITION))
            {
              if (pccinfo->flWindowAttr & CV_NAME)
              {
                pszString = GETLPSTRFORNV(precord->preccX);
              }
              else
              {
                pszString = GETLPSTRFORTV(precord->preccX);
              }
              IcnCalcTextSize (hps, iconSize, pccinfon,
                               pszString, IS_SIDETEXT,
                               &(precord->slTextSize));
            }

            if (!(pccinfo->flWindowAttr & CV_FLOW) &&
                usLongestChanged)
            {
              if (!((precord->flRecordAttr & CRA_SELECTED) ||
                    ((precord->flRecordAttr & CRA_CURSORED) &&
                     (pccinfo->flWindowAttr & CV_TEXT))))
              {
                if (!flRepaintAll)
                {
                  if (!(CnrEraseRecord(hwnd, hps, precord)))
                  {
                    goto exiterror;
                  }

#if 0
                  /****************************************************/
                  /* Update the text size                             */
                  /****************************************************/
                  if (!(flInvalidate & CMA_NOREPOSITION))
                  {
                    IcnCalcTextSize (hps, iconSize, pccinfon,
                                     pszString, IS_SIDETEXT,
                                     &(precord->slTextSize));
                    precord->usLongestLine = usLongestChanged;
                  }
#endif

                  if (!(CnrDrawRecord(hwnd, hps, precord, TRUE)))
                  {
                    goto exiterror;
                  }
                }
              }
            }
            else
            {
              if (!(CnrEraseRecord(hwnd, hps, precord)))
              {
                 goto exiterror;
              }

#if 0
              /*******************************************************/
              /* Update the text size                                */
              /*******************************************************/
              if (!(flInvalidate & CMA_NOREPOSITION))
              {
                IcnCalcTextSize (hps, iconSize, pccinfon,
                                 pszString, IS_SIDETEXT,
                                 &(precord->slTextSize));
              }
#endif

              if (!(CnrDrawRecord(hwnd, hps, precord, TRUE)))
              {
                 goto exiterror;
              }
            }
          }
        }
      } /* end for */

      if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
          (!(pccinfo->flWindowAttr & CV_FLOW)) &&
           usLongestChanged )
      {
        if (!flRepaintAll)
        {
          if (!(DrawSelCurItemsInView(hwnd, hps, FALSE)))
          {
             goto exiterror;
          }

          if (!(DrawSelCurItemsInView(hwnd, hps, TRUE)))
          {
            goto exiterror;
          }
        }
        else
        {
          PMInvalidateRect(hwnd, NULL, TRUE);
          UpdateWindow (hwnd);
        }
      }
    }

    goto exitOK;
  }
  else
  {
    /****************************************************************/
    /* If CCS_AUTOPOSITION is set on, set FL_NEEDARRANGE on          */
    /* to rearrange icons when painting in icon view                */
    /****************************************************************/
    if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
    {
      pccinfon->ulFlagSet |= FL_NEEDARRANGE;
    } /* endif */

    /****************************************************************/
    /* If the number of the records is zero, refresh all records.   */
    /* Else, refresh the records in the array                       */
    /****************************************************************/
    if (!cNumRecord)
    {
      /**************************************************************/
      /* To reset workplace when painting in icon view,             */
      /* turn off FL_SETWORKPLACE                                   */
      /**************************************************************/
      pccinfon->preccLastValid = NULL;
      pccinfon->ulFlagSet &= ~(FL_POSITIONCOLTITLES | FL_CNRPOSITION |
                          FL_TEXTCALC | FL_NEWVIEWSCROLLSET |
                          FL_SETWORKPLACE );
      pccinfon->lMaxHeight = 0;

#if 0
      if (pccinfo->flWindowAttr & CV_DETAIL)
      {
         PMGetClientRect (hwnd, &CnrRect);

         if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
             ((long)pccinfo->xVertSplitbar > CnrRect.right))
         {
            CnrDetailsVCtlProc (hwnd, WM_PAINT, NULL, NULL);
         }
      }
#endif
      InvalidateChildren (hwnd, FALSE, FALSE);

      UpdateWindow (hwnd);
    }
    else
    {
      /**************************************************************/
      /* We need repositioning in the name view and text view.      */
      /**************************************************************/
      pccinfon->ulFlagSet &= ~FL_CNRPOSITION;

      CnrCalcIconSize (hwnd, &iconSize);
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        /************************************************************/
        /* In the icon view, calculate the text size and redraw     */
        /* each item in the array.                                  */
        /************************************************************/
        for (i = 0; i < cNumRecord ; i++)
        {
          precord = *(pRecordArray + i);

          if (!(GetWindowLong( hwnd, GWL_STYLE ) &
               CCS_AUTOPOSITION))
          {
            if (flInvalidate & CMA_ERASE)
            {
              CnrEraseIcon(hwnd, hps, precord, TRUE);
            }
            else
            {
              EraseTextOnlyInIV(hwnd, hps, precord);
            }
          }

          if (!(flInvalidate & CMA_NOREPOSITION))
          {
            IcnCalcTextSize (hps, iconSize, pccinfon,
                             precord->preccX->pszIcon,
                             IS_BENEATHTEXT,
                             &(precord->slTextSize));
          }

          /***********************************************************/
          /* If CCS_AUTOPOSITION is set off, we draw each invalidated */
          /* item one by one                                         */
          /***********************************************************/
          if (!(GetWindowLong( hwnd, GWL_STYLE ) &
               CCS_AUTOPOSITION))
          {
            EraseTextOnlyInIV(hwnd, hps, precord);
            if (!(CnrDrawIcon(hwnd, hps, precord, TRUE)))
            {
               goto exiterror;
            }
          }
          else
          {
            prcFirstInvalid = *pRecordArray;
            /*******************************************************/
            /* For the flowed view, the item lefter, upper item    */
            /* becomes first invalid item                          */
            /*******************************************************/
            if ((prcFirstInvalid->ptlIcon.y < precord->ptlIcon.y) ||
                ((prcFirstInvalid->ptlIcon.y == precord->ptlIcon.y) &&
                 (prcFirstInvalid->ptlIcon.x < precord->ptlIcon.x)))
            {
              prcFirstInvalid = precord;
            }
          }

        } /* endfor */

        if (!(GetWindowLong (hwnd, GWL_STYLE) &
              CCS_AUTOPOSITION))
        {
          CnrUpdateWorkplace(hwnd, hps, NULL, FALSE);

          /***********************************************************/
          /* Set the scroll bar                                      */
          /***********************************************************/
          CnrSetIconViewScroll(hwnd);
        }
      }
      else
      {
        /************************************************************/
        /* To reset workplace when painting in icon view,           */
        /* turn off FL_SETWORKPLACE                                 */
        /************************************************************/
        pccinfon->ulFlagSet &= ~FL_SETWORKPLACE;

        /************************************************************/
        /* Initialize First invalid as the start of the array       */
        /************************************************************/
        prcFirstInvalid = *pRecordArray;

        /************************************************************/
        /* Refresh for Details View.                                */
        /************************************************************/
        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          pccinfon->ulFlagSet &= ~FL_SETSCROLL;

          /**********************************************************/
          /* Since the array passed in by the app. is not guaranteed*/
          /* to be in any order, we search the array and find the   */
          /* record that is the first one to be invalidated.  We    */
          /* will start invalidating from this record until we get  */
          /* to the end.                                            */
          /**********************************************************/
          for (i = 0; i < cNumRecord; i++)
          {
            precord = *(pRecordArray + i);

            if (prcFirstInvalid->ptlNonIconPos.y >
                precord->ptlNonIconPos.y )
            {
              prcFirstInvalid = precord;
            }
          }
        }
        /************************************************************/
        /* Refresh for the name view and text view                  */
        /************************************************************/
        else
        {
          /**********************************************************/
          /* Get the first item in the item order                   */
          /**********************************************************/
          if (pccinfo->flWindowAttr & CV_FLOW)
          {
            for (i = 0; i < cNumRecord ; i++)
            {
              precord = *(pRecordArray + i);

              if (pccinfo->flWindowAttr & CV_NAME)
              {
                pszString = GETLPSTRFORNV(precord->preccX);
              }
              else
              {
                pszString = GETLPSTRFORTV(precord->preccX);
              }

              /******************************************************/
              /* If cy of the item to be invalidated is lMaxHeight  */
              /* update lMaxHeight by calling function              */
              /* UpdateMaxHeightForDel (cnrcfunc.c)                 */
              /******************************************************/
              UpdateMaxHeightForDel(hwnd, precord);

              /******************************************************/
              /* Update the text size                               */
              /******************************************************/
              IcnCalcTextSize (hps, iconSize, pccinfon,
                               pszString, IS_SIDETEXT,
                               &(precord->slTextSize));

              /******************************************************/
              /* Update the lMaxHeight if the updated item is       */
              /* the tallest one.                                   */
              /******************************************************/
              if (pccinfon->lMaxHeight < precord->slTextSize.cy)        /*DAB1*/
              {
                pccinfon->lMaxHeight = precord->slTextSize.cy;          /*DAB1*/
              }

              /******************************************************/
              /* For the flowed view, the upper, leftmost item      */
              /* becomes first invalid item                         */
              /******************************************************/
              if ((prcFirstInvalid->ptlNonIconPos.x >
                   precord->ptlNonIconPos.x) ||
                  ((prcFirstInvalid->ptlNonIconPos.x ==
                    precord->ptlNonIconPos.x) &&
                   (prcFirstInvalid->ptlNonIconPos.y >
                    precord->ptlNonIconPos.y)))
              {
                prcFirstInvalid = precord;
              }
            }
          }
          else
          {
            /*********************************************************/
            /* For the non-flowed view, get the uppermost y and      */
            /* paint from the point to the bottom                    */
            /*********************************************************/
            for (i = 0; i < cNumRecord ; i++)
            {
              precord = *(pRecordArray + i);

              if (pccinfo->flWindowAttr & CV_NAME)
              {
                pszString = GETLPSTRFORNV(precord->preccX);
              }
              else
              {
                pszString = GETLPSTRFORTV(precord->preccX);
              } /* endif */

              IcnCalcTextSize (hps, iconSize, pccinfon,
                               pszString, IS_SIDETEXT,
                               &(precord->slTextSize));

              if (prcFirstInvalid->ptlNonIconPos.y >
                  precord->ptlNonIconPos.y)
              {
                prcFirstInvalid = precord;
              }
            }
          } /* endif */
        } /* endelse */
      }

      if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT | CV_DETAIL)) ||
          ((pccinfo->flWindowAttr & CV_ICON) &&
           (GetWindowLong( hwnd, GWL_STYLE ) &
            CCS_AUTOPOSITION)))
      {
        pccinfon->preccLastValid = prcFirstInvalid->preccPrevItem;

        /************************************************************/
        /* Reposition for the possible change in the icon/text      */
        /* size when painting.                                      */
        /* Move the window by the distance scrolled already.        */
        /************************************************************/
        pccinfon->ulFlagSet &= ~FL_CNRPOSITION;

        /************************************************************/
        /* Redraw only the rectangle, not whole window              */
        /************************************************************/
        if (Before1stInView(hwnd, prcFirstInvalid))
        {
          pccinfon->ulFlagSet &=~FL_NEWVIEWSCROLLSET;
        }
        if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          PMGetClientRect (hwnd, &CnrRect);

          if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
              ((long)pccinfo->xVertSplitbar > CnrRect.right))
          {
             CnrDetailsVCtlProc (hwnd, WM_PAINT, 0, 0);                 /*DAB1*/
          }
        }

        if (AfterLastInViewport(hwnd, prcFirstInvalid))
        {
          GetMinInvalidateRect(hwnd, &rctlInvalidate);

          if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
              (!(pccinfo->flWindowAttr & CV_FLOW)) &&
              (0 < usLongestChanged) &&
              (usLongestChanged < pccinfon->preccItem->usLongestLine))
          {
            if (!(DrawSelCurItemsInView(hwnd, hps, FALSE)))
            {
               goto exiterror;
            }
          } /* endif */

          PMInvalidateRect(hwnd, &rctlInvalidate, TRUE);
          UpdateWindow (hwnd);

          if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
              (!(pccinfo->flWindowAttr & CV_FLOW)) &&
               usLongestChanged)
          {
            if (!(DrawSelCurItemsInView(hwnd, hps, TRUE)))
            {
               goto exiterror;
            }
          }
        }
        else
        {
          /************************************************************/
          /* Since InvalidateRect does not work with WS_CLIPCHILDREN, */
          /* we must invalidate each details window ...               */
          /************************************************************/
          if (pccinfo->flWindowAttr & CV_DETAIL)
          {

            /**********************************************************/
            /* Left Details or left title window was edited.          */
            /**********************************************************/
            switch (pccinfon->ceData.id)
            {
              case CID_LEFTDVWND:
              case CID_LEFTCOLTITLEWND:
                PMInvalidateRect (GetDlgItem (hwnd,
                                   CID_LEFTCOLTITLEWND),
                                   NULL,
                                   FALSE);
                PMInvalidateRect (GetDlgItem (hwnd,
                                   CID_LEFTDVWND),
                                   NULL,
                                   FALSE);

                /********************************************************/
                /* If the row height changed and if Details view is     */
                /* split, invalidate the right details windows...       */
                /********************************************************/
                if ((bRowHeightChg)  &&
                    (!(pccinfon->ulFlagSet & FL_UNSPLIT)))
                {
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_RIGHTCOLTITLEWND),
                                     NULL,
                                     FALSE);
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_RIGHTDVWND),
                                     NULL,
                                     FALSE);
                }
                break;

              case CID_RIGHTDVWND:
              case CID_RIGHTCOLTITLEWND:
                if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
                {
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_RIGHTCOLTITLEWND),
                                     NULL,
                                     FALSE);
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_RIGHTDVWND),
                                     NULL,
                                     FALSE);

                /********************************************************/
                /* If the row height changed, then invalidate the left  */
                /* details windows...                                   */
                /********************************************************/
                if (bRowHeightChg)
                {
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_LEFTCOLTITLEWND),
                                     NULL,
                                     FALSE);
                  PMInvalidateRect (GetDlgItem (hwnd,
                                     CID_LEFTDVWND),
                                     NULL,
                                     FALSE);
                  }
                }
                break;

              default:
                break;
            } /* end switch */
          }

          InvalidateRect(hwnd, NULL, TRUE);
          UpdateWindow (hwnd);
        }

        pccinfon->preccLastValid = NULL;
      } /* endif */
    }
    goto exitOK;
  }

exitOK:
  ReleaseDC(hwnd, hps);
  return TRUE;

exiterror:
  ReleaseDC(hwnd, hps);
  return FALSE;
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:        TRVInvalidateRecord                         */
/*                                                                    */
/*  Descriptive Name:   Control Invalidate Record to Refresh          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*           mp1 -                                                    */
/*           mp2 -                                                    */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR TRVInvalidateRecord (					/*RKK1*/
  HANDLE             hwnd,
  LPRECORDCOREN FAR *pRecordArray,
  WORD               cNumRecord,
  WORD               flInvalidate  ) 				/*RKK1*/
{
  HDC             hps;
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  LPRECORDCOREN   preccInvalid;          /* record to be invalidated       */
  LPRECORDCOREN   preccFirstInvalid;     /* first item to be invalidated   */
  RECT            rctlInvalid;
  POINT           slIcon;
  POINT           slTreeIcon;
  DWORD           i;                     /* counter                        */
  int             right;
  int             xOldRight;
  WORD            usLeftMargin;
  BOOL            flInvalidateAll = FALSE;
  BOOL            flReposition = FALSE;
  WORD            usLongestChanged = 0;
  SIZEL           slNewTextSize;

  /*******************************************************************/
  /* Get ptr to Container instance data                              */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  CnrCalcIconSize (hwnd, &slIcon);
  CnrTRVCalcTreeIconSize (hwnd, &slTreeIcon);

  hps = PMGetFlippedDC(hwnd);

  if (flInvalidate & CMA_NOREPOSITION)
  {
    /***************************************************************/
    /* If cNumRecord is 0, refresh whole window.                   */
    /***************************************************************/
    if (!cNumRecord)
    {
      flInvalidateAll = TRUE;
    }
  }
  else
  {
    /***************************************************************/
    /* If CMA_REPOSITION is set on, we need reposition.            */
    /***************************************************************/
    if (flInvalidate & CMA_REPOSITION)
    {
      flReposition = TRUE;
    }

    /***************************************************************/
    /* If CMA_REPOSITION or CMA_TEXTCHANGED is set on              */
    /* and need refresh all the items, repositon it and refresh    */
    /* the  whole window.                                          */
    /***************************************************************/
    if (!cNumRecord)
    {
      flReposition = TRUE;
      flInvalidateAll = TRUE;
    }
    else
    {
      usLeftMargin = max((WORD)pccinfon->lAveCharWidth,
                         (WORD)MINCX_TREELEFTMARGIN);

      /*************************************************************/
      /* Get the first item to be invalidated. If the first item   */
      /* to be invalidated is before the first item in the view,   */
      /* refresh the whole window.                                 */
      /*************************************************************/
      preccFirstInvalid = *pRecordArray;

      for (i = 0; i < cNumRecord ; i++)
      {
        preccInvalid = *(pRecordArray + i);

        if (!flInvalidateAll &&
             IsBefore(hwnd, preccInvalid, preccFirstInvalid))
        {
          if (IsBefore(hwnd, preccInvalid, pccinfon->preccFirstinTRV))
          {
            flInvalidateAll = TRUE;
          }
          else
          {
            preccFirstInvalid = preccInvalid;
          }
        } /* endif */

        /***********************************************************/
        /* If flReposition is FALSE, we check that preccInvalid    */
        /* cause the need of reposition.                           */
        /***********************************************************/
        if (!flReposition)
        {
          IcnCalcTextSize (hps, slIcon, pccinfon,
                           GETLPSTRFORTRV(preccInvalid->preccX),
                           IS_SIDETEXT,
                           &slNewTextSize);

          /*********************************************************/
          /* If the height of the item changes, we need to         */
          /* reposition.                                           */
          /*********************************************************/
          if (ChangeItemHeight(hwnd, preccInvalid, slNewTextSize,
                               slIcon))
          {
            flReposition = TRUE;
          } /* endif */

          /*********************************************************/
          /* If flReposition is FALSE, we check that preccInvalid  */
          /* cause the change of the right_most point of the       */
          /* tree. If so, update it.                               */
          /*********************************************************/
          if (!flReposition &&
              (preccInvalid->slTextSize.cx != slNewTextSize.cx))        /*DAB1*/
          {
            if (pccinfo->flWindowAttr & CV_TEXT)
            {
              xOldRight = (preccInvalid->ptlNonIconPos.x +              /*DAB1*/
                          preccInvalid->slTextSize.cx);                 /*DAB1*/
              right = (preccInvalid->ptlNonIconPos.x +                  /*DAB1*/
                       slNewTextSize.cx);                               /*DAB1*/
            }
            else
            {
              xOldRight = (int)(preccInvalid->ptlNonIconPos.x +
                          slIcon.x + (long)usLeftMargin +
                          preccInvalid->slTextSize.cx);                 /*DAB1*/
              right = (int)(preccInvalid->ptlNonIconPos.x +
                       slIcon.x + (long)usLeftMargin +
                       slNewTextSize.cx);                               /*DAB1*/
            }
            xOldRight += pccinfon->lAveCharWidth;
            right += pccinfon->lAveCharWidth;

            /*******************************************************/
            /* New right of the preccInvalid is more than the     */
            /* right of the tree, update it.                      */
            /*******************************************************/
            if (pccinfon->xTreeRight <= right)
            {
              pccinfon->xTreeRight = right;
              usLongestChanged = (WORD)right;
            }
            else
            {
              /*****************************************************/
              /* If preccInvalid was one of the longest in the     */
              /* tree and become shotter, we need loop through     */
              /* the tree to figure out the longest.               */
              /*****************************************************/
              if ((xOldRight >= pccinfon->xTreeRight) &&
                  (pccinfon->xTreeRight >= right))
              {
                flReposition = TRUE;
              }
            }
          }
        } /* endif */
      } /* end for */
    } /* endif */
  }

  if (!flReposition)
  {
    if (usLongestChanged )
    {
      int         sOldVscrollPos;
      int         sOldHscrollPos;

      sOldVscrollPos = pccinfon->sVscrollPos;
      sOldHscrollPos = pccinfon->sHscrollPos;

      /*************************************************************/
      /* Reset scroll bar. If the scroll bar position changes,     */
      /* refresh whole window.                                     */
      /*************************************************************/
      pccinfon->ulFlagSet &= ~(FL_SETSCROLL | FL_SIZECHILDREN );

      CnrTRVSetUpNewView(hwnd, hps);

      if ((sOldVscrollPos != pccinfon->sVscrollPos) ||
          (sOldHscrollPos != pccinfon->sHscrollPos))
      {
        flInvalidateAll = TRUE;
      }
    }

    /*************************************************************/
    /* If we don't need to invalidate the whole window and don't */
    /* need repositioning, draw the items to be invalidated.     */
    /*************************************************************/
    for (i = 0; i < cNumRecord ; i++)
    {
      preccInvalid = *(pRecordArray + i);

      if (!flInvalidateAll)
      {
        CnrEraseRecord(hwnd, hps, preccInvalid);
      } /* endif */

      if (!(flInvalidate & CMA_NOREPOSITION))
      {
        IcnCalcTextSize( hps, slIcon, pccinfon,
                         GETLPSTRFORTRV(preccInvalid->preccX),
                         IS_SIDETEXT,
                         &(preccInvalid->slTextSize));

        preccInvalid->usLongestLine =
                      (WORD)preccInvalid->slTextSize.cx;                /*DAB1*/
      }

      if (!flInvalidateAll)
      {
        CnrDrawRecord(hwnd, hps, preccInvalid, TRUE);
      }
    } /* end for */
  }
  else    /* flReposition = TRUE */
  {
    /***************************************************************/
    /* Need recalculation of the text for all the items if we need */
    /* refresh all of the items.                                   */
    /***************************************************************/
    if (!cNumRecord)
    {
      pccinfon->ulFlagSet &= ~FL_TEXTCALC;
    }
    else
    /***************************************************************/
    /* Else, update only the text size of the item to be           */
    /* invalidated.                                                */
    /***************************************************************/
    {
      for (i = 0; i < cNumRecord ; i++)
      {
        preccInvalid = *(pRecordArray + i);

        IcnCalcTextSize( hps, slIcon, pccinfon,
                         GETLPSTRFORTRV(preccInvalid->preccX),
                         IS_SIDETEXT,
                         &(preccInvalid->slTextSize));
      }
    }

    /***************************************************************/
    /* Need reposition.                                            */
    /***************************************************************/
    pccinfon->ulFlagSet &= ~FL_CNRPOSITION;

    /***************************************************************/
    /* Invalidation of the whole window with reposition needs      */
    /* the new view scroll setting.                                */
    /***************************************************************/
    if (flInvalidateAll)
    {
      pccinfon->ulFlagSet &= ~FL_NEWVIEWSCROLLSET;
    }
    else
    {
      TRVGetInvalidateRect(hwnd, preccFirstInvalid, &rctlInvalid);
      PMInvalidateRect(hwnd, &rctlInvalid, TRUE);
      UpdateWindow( hwnd );
    }
  }

  if (flInvalidateAll)
  {
    PMInvalidateRect(hwnd, NULL, TRUE);
    UpdateWindow (hwnd);
  }

  ReleaseDC(hwnd, hps);

  /*****************************************************************/
  /* If CCS_AUTOPOSITION is set on, set FL_NEEDARRANGE on           */
  /* to rearrange icons when painting in icon view                 */
  /*****************************************************************/
  if (GetWindowLong( hwnd, GWL_STYLE ) & CCS_AUTOPOSITION)
  {
    pccinfon->ulFlagSet |= FL_NEEDARRANGE;
  }
  else
  {
    pccinfon->ulFlagSet &= ~FL_SETWORKPLACE;
  }
  return TRUE;
}
