/* Revision: 18 1.3.1.2 source/ui/cclnt/cnr/cnrs3p1.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS3P1.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS3P1.C - Details view routines for container control  */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* DAB2  02/21/94  DAB  Added support for DFI column width queries    */
/* DAB3  06/02/94  DAB  Modified CM_* message APIs to match OS/2 2.x  */
/* D6386  1/16/96  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* PPB1  01/29/96  PPB  Ensure scroll position > 0 defect 6948        */
/* DAB4   8/01/96  DAB  Defect 8172 - Fixed CM_PAINTBACKGROUND parms  */
/**********************************************************************/
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

//#include "storwin.h"

#ifdef SEGMENTOR
#define CNRS3P1_C
#include "cnrpragw.h"
#endif

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LRESULT CALLBACK CnrLeftDetailsWndProc( HWND hwnd, UINT msg,            /*DAB1*/
                                        WPARAM wParam, LPARAM lParam )  /*DAB1*/
{
 LPCNRINFON        pccinfon;     /* Pointer to Internal control info.  */
 LPCNRINFO         pccinfo;
 HDC              hps;
 RECT             InvalidRect;
 RECT             CnrRect;
 HANDLE           hwndCnr;
 OWNERBACKGROUND  OwnerBack;
 BOOL             OwnerPaintBG = FALSE;
 PAINTSTRUCT      paint;

 hwndCnr  = PARENTOF (hwnd);
 pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 switch (msg)
 {
    case WM_CREATE:
      return FALSE;

    case WM_PAINT:
       hps = BeginPaint (hwnd, &paint);
       InvalidRect = paint.rcPaint;

       PMFlipDC(hwnd, hps);
       PMFlipRect((LPRECT)&InvalidRect);

       if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
       {
          OwnerBack.hwnd = hwnd;
          OwnerBack.hps  = hps;
          OwnerBack.rclBackground = paint.rcPaint;
          OwnerBack.idWindow = getWindowId( GetDlgItem( hwndCnr,      /*DAB1*/
                                                     CID_LEFTDVWND)); /*DAB1*/

          OwnerPaintBG = LOWORD(SENDMESSAGE(hwndCnr, CM_PAINTBACKGROUND,/*DAB1*/
                            (DWORD)((LPOWNERBACKGROUND)&OwnerBack), 0));/*DAB4*/
       }

       /***************************************************************/
       /* If the application didn't paint the background, then do it  */
       /***************************************************************/
       if (!OwnerPaintBG)
       {
         HBRUSH hBrush;

         /*************************************************************/
         /* Paint the background with the current setting             */
         /*************************************************************/
         if (hBrush = CreateSolidBrush (
                      pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
         {
           PMFillRect( hps, &InvalidRect, hBrush);
           DeleteObject( hBrush);
         }
       }

       /***************************************************************/
       /* Draw the data in the left window.                           */
       /***************************************************************/
       if (!(CnrDrawLeftDetailsWindow (hwnd, hps, InvalidRect)))
       {
          EndPaint (hwnd, &paint);
          return ((long)NULL);
       }

       EndPaint (hwnd, &paint);

       PMGetClientRect (hwndCnr, &CnrRect);

       if ((pccinfon->ulFlagSet & FL_UNSPLIT) ||
           ((long)pccinfo->xVertSplitbar > CnrRect.right))
       {
          PMInvalidateRect (GetDlgItem (hwndCnr, CID_VSCROLL),
                             NULL, FALSE);

          PMInvalidateRect (GetDlgItem (hwndCnr, CID_HSCROLL),
                             NULL, FALSE);
       }
    break;

     case WM_LBUTTONDOWN:
       SetFocus( hwndCnr);
       pccinfon->ulFlagSet |= FL_SELECTING;
       pccinfon->ulFlagSet2 &= (~FL2_BUTTONRIGHT);
       pccinfon->ulFlagSet2 |= ( FL2_BUTTONLEFT | FL2_FOCUSLEFT );
       return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

     case WM_LBUTTONDBLCLK:
       pccinfon->ulFlagSet2 |= FL2_BUTTONLEFT;
       return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_LBUTTONUP:
       pccinfon->ulFlagSet &= ~FL_SELECTING;
       return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_RBUTTONDOWN:
    case WM_MBUTTONDBLCLK:
       pccinfon->ulFlagSet2 |= ( FL2_BUTTONLEFT | FL2_FOCUSLEFT );
       return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_RBUTTONUP:
    case WM_MOUSEMOVE:
      return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_MBUTTONDOWN:
       SetFocus (hwndCnr);
       pccinfon->ulFlagSet2 |= FL2_FOCUSLEFT;
       DefWindowProc (hwnd, msg, wParam, lParam);
    break;

    case WM_RBUTTONDBLCLK:
       SetFocus ( hwndCnr);
       pccinfon->ulFlagSet2 |= FL2_FOCUSLEFT;
       DefWindowProc (hwnd, msg, wParam, lParam);
    break;

    /******************************************************************/
    /* If a WM_GETDLGCODE message comes in, the container is in a     */
    /* dialog.  Tell the dialog which keys we want to process.        */
    /******************************************************************/
    case WM_GETDLGCODE:

      return ((long)DLGC_WANTARROWS | DLGC_WANTCHARS);
      break;

   default:
     return DefWindowProc (hwnd, msg, wParam, lParam);
 }
 return 0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LRESULT CALLBACK CnrRightDetailsWndProc( HWND hwnd, UINT msg,           /*DAB1*/
                                         WPARAM wParam, LPARAM lParam ) /*DAB1*/
{
 LPCNRINFON        pccinfon;     /* Pointer to Internal control info.  */
 LPCNRINFO         pccinfo;
 HDC              hps;
 RECT             InvalidRect;
 HANDLE           hwndCnr;
 OWNERBACKGROUND  OwnerBack;
 BOOL             OwnerPaintBG = FALSE;
 PAINTSTRUCT      paint;

 hwndCnr  = PARENTOF (hwnd);
 pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 switch (msg)
 {
    case WM_CREATE:
      return FALSE;

    case WM_PAINT:
      hps = BeginPaint (hwnd, &paint);
      InvalidRect = paint.rcPaint;

      PMFlipDC(hwnd, hps);
      PMFlipRect((LPRECT)&InvalidRect);

      if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
      {
         OwnerBack.hwnd = hwnd;
         OwnerBack.hps  = hps;
         OwnerBack.rclBackground = paint.rcPaint;
         OwnerBack.idWindow = getWindowId( GetDlgItem (hwndCnr,         /*DAB1*/
                                              CID_RIGHTDVWND));         /*DAB1*/

         OwnerPaintBG = LOWORD(SENDMESSAGE (hwndCnr, CM_PAINTBACKGROUND,/*DAB1*/
                           (DWORD)((LPOWNERBACKGROUND)&OwnerBack), 0 ));/*DAB4*/
      }

      /****************************************************************/
      /* If the application didn't paint the background, then do it   */
      /****************************************************************/
      if (!OwnerPaintBG)
      {
         HBRUSH hBrush;

         /*************************************************************/
         /* Paint the background with the current setting             */
         /*************************************************************/
         if (hBrush = CreateSolidBrush (
                      pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
         {
           PMFillRect( hps, &InvalidRect, hBrush);
           DeleteObject( hBrush);
         }
      }

      /****************************************************************/
      /* Draw the data in the right window.                           */
      /****************************************************************/
      if (!(CnrDrawRightDetailsWindow (hwnd, hps, InvalidRect)))
      {
         EndPaint (hwnd, &paint);
         return ((long)NULL);
      }

      EndPaint (hwnd, &paint);
    break;

    case WM_LBUTTONDOWN:
       SetFocus( hwndCnr);
       pccinfon->ulFlagSet |= FL_SELECTING;
       pccinfon->ulFlagSet2 &= (~FL2_BUTTONLEFT);
       pccinfon->ulFlagSet2 |= FL2_BUTTONRIGHT;
       pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;
       return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_LBUTTONDBLCLK:
     pccinfon->ulFlagSet2 |= FL2_BUTTONRIGHT;
     return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_LBUTTONUP:
      pccinfon->ulFlagSet &= ~FL_SELECTING;
      return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_RBUTTONDOWN:
    case WM_MBUTTONDBLCLK:
      pccinfon->ulFlagSet2 |= FL2_BUTTONRIGHT;
      pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;
      return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_RBUTTONUP:
    case WM_MOUSEMOVE:
     return ((long)SENDMESSAGE (hwndCnr, msg, wParam, lParam));

    case WM_MBUTTONDOWN:
       SetFocus (hwndCnr);
       pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;
       DefWindowProc (hwnd, msg, wParam, lParam);
    break;

    case WM_RBUTTONDBLCLK:
       SetFocus ( hwndCnr);
       pccinfon->ulFlagSet2 &= ~FL2_FOCUSLEFT;
       DefWindowProc (hwnd, msg, wParam, lParam);
    break;

    /******************************************************************/
    /* If a WM_GETDLGCODE message comes in, the container is in a     */
    /* dialog.  Tell the dialog which keys we want to process.        */
    /******************************************************************/
    case WM_GETDLGCODE:

      return ((long)DLGC_WANTARROWS | DLGC_WANTCHARS);
      break;

   default:
     return DefWindowProc (hwnd, msg, wParam, lParam);
 }
 return 0;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrAllocDetailFieldInfo                           */
/*                                                                    */
/*  Descriptive Name:  Allocate Detail FieldInfo Structures           */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will allocate 1 or a linked list of FieldInfo       */
/*  structures.  When the allocation is complete, a pointer to the    */
/*  first FieldInfo structure in the list is returned to the          */
/*  application.  The linked list that is allocated is singly linked. */
/*  The allocation is done through the Storage Manager.               */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          mp1     - The number of FieldInfo structures to allocate. */
/*                                                                    */
/*  Output:  Pointer to the first FieldInfo structure in the list     */
/*           allocated.                                               */
/*                                                                    */
/*  Exit-Normal:  Pointer to a FieldInfo Structure.                   */
/*                                                                    */
/*  Exit-Error:   NULL with WinSetErrorInfo message specifed:         */
/*                - PMERR_INVALID_PARAMETERS                          */
/*                - PMERR_INSUFFICIENT_MEMORY                         */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LPFIELDINFO FAR CnrAllocDetailFieldInfo (hwndCnr, mp1)

HANDLE  hwndCnr;   /* Handle to the Container window.                 */
long    mp1;       /* The number of FieldInfo structures to be        */
                   /* allocated.                                      */
{

 WORD       NumFieldInfo;     /* The number of FieldInfo Structures   */
                              /* to be allocated.                     */
 LPFIELDINFON  Head;          /* Pointer to the first allocated       */
                              /* FieldInfo Structure in the List.     */
 LPFIELDINFON  pLead;         /* Pointer used in creating a linked    */
                              /* list of FieldInfo Structures.        */
 LPFIELDINFON  pTrail;        /* Pointer used in creating a linked    */
                              /* list of FieldInfo Structures.        */
// LPVOID        pSatPtr;       /* Pointer to Storage Space.            */
 DWORD         i;
 DWORD         FISize;         /* Size of FieldInfo to allocate       */
 DWORD         ExtFISize;      /* Size of FieldInfo.                  */

 /*********************************************************************/
 /* Get the pointer to the storage space.                             */
 /*********************************************************************/
// pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);

 /*********************************************************************/
 /* Make sure the number to be allocated is greater than 0.           */
 /*********************************************************************/
 NumFieldInfo = LOWORD(mp1);

 if (NumFieldInfo < 1)
 {
    SETERROR (PMERR_INVALID_PARAMETERS);
    return ((LPFIELDINFO)NULL);
 }

 /*********************************************************************/
 /* We'll chew up a local var for the case where the app. is trying   */
 /* to allocate a lot of memory at once.  We need to specify the size */
 /* of the FIELDINFO structure in the cb field of the structure. When */
 /* the app. is allocating a lot of memory at once, this sizeof       */
 /* function would be burning time in the loop if placed there.       */
 /*********************************************************************/
 ExtFISize = (DWORD)(sizeof(FIELDINFO));
 FISize = (DWORD)(sizeof(FIELDINFO) + sizeof(FIELDINFON));

 /*********************************************************************/
 /* Call the Storage Manager to allocate enough storage for a         */
 /* FIELDINFO structure, plus a FIELDINFON structure.  The            */
 /* FIELDINFON is used internally, and the application never knows    */
 /* about it.  The pointer returned to the application points to the  */
 /* external part, or FIELDINFO.  Moving between the internal and     */
 /* external portions of the structure are made easy by using 2       */
 /* MACROS:  PFINFROMX - returns an Internal pointer (LPFIELDINFON)   */
 /*                      given an External pointer (LPFIELDINFO).     */
 /*          PFIXFROMN - returns an External pointer (LPFIELDINFO)    */
 /*                      given an Internal pointer (LPFIELDINFON).    */
 /*                                                                   */
 /* Abstractly, it is arranged in the following manner:               */
 /*                                                                   */
 /*                         +-------------+                           */
 /*                         | FIELDINFON  |                           */
 /*                         |             |                           */
 /*                         | (Internal)  |                           */
 /*                         |             |                           */
 /*                         +-------------+                           */
 /*                         |  FIELDINFO  |                           */
 /*                         |             |                           */
 /*                         | (External)  |                           */
 /*                         |             |                           */
 /*                         +-------------+                           */
 /*                                                                   */
 /*********************************************************************/
 if (NumFieldInfo > 1)
 {
//  if (Head = (LPFIELDINFON)CnrAllocateStorage (pSatPtr,
//                                      (DWORD)(sizeof(FIELDINFO) +
//                                              sizeof(FIELDINFON))))
    if (Head = (LPFIELDINFON)GlobalAlloc( GPTR, FISize ))               /*DAB1*/
    {
       PFIXFROMN(Head)->cb = ExtFISize;
       PFIXFROMN(Head)->pNextFieldInfo = NULL;
       PFIXFROMN(Head)->cxWidth = 0;
       PFIXFROMN(Head)->pTitleData = NULL;
       Head->usLongestLine = 0;
       pTrail = Head;
    }
    else
    {
       SETERROR(PMERR_INSUFFICIENT_MEMORY);
       return ((LPFIELDINFO)NULL);
    }

    for (i = NumFieldInfo; i > 1; i--)
    {
//     if (pLead = (LPFIELDINFON)CnrAllocateStorage (pSatPtr,
//                                         (DWORD)(sizeof(FIELDINFO) +
//                                                 sizeof(FIELDINFON))))
       if (pLead = (LPFIELDINFON)GlobalAlloc( GPTR, FISize ))           /*DAB1*/
       {
          PFIXFROMN(pTrail)->pNextFieldInfo = PFIXFROMN(pLead);
          PFIXFROMN(pLead)->cb = ExtFISize;
          PFIXFROMN(pLead)->pNextFieldInfo = NULL;
          pTrail = pLead;
       }
       else
       {
          SETERROR(PMERR_INSUFFICIENT_MEMORY);
          return ((LPFIELDINFO)NULL);
       }
    }
 }
 else
 {
//  if (Head = (LPFIELDINFON)CnrAllocateStorage (pSatPtr,
//                                      (DWORD)(sizeof(FIELDINFO) +
//                                              sizeof(FIELDINFON))))
    if (Head = (LPFIELDINFON)GlobalAlloc( GPTR, FISize ))               /*DAB1*/
    {
       PFIXFROMN(Head)->cb = ExtFISize;
       PFIXFROMN(Head)->pNextFieldInfo = NULL;
       PFIXFROMN(Head)->cxWidth = 0;
       PFIXFROMN(Head)->pTitleData = NULL;
       Head->usLongestLine = 0;
    }
    else
    /******************************************************************/
    /* The Allocation failed so set the error info and return NULL.   */
    /******************************************************************/
    {
       SETERROR(PMERR_INSUFFICIENT_MEMORY);
       return ((LPFIELDINFO)NULL);
    }
 }

 /*********************************************************************/
 /* If no errors occured during the allocation, we return a pointer   */
 /* to the external portion of the first FieldInfo allocated.  The    */
 /* external pointer is returned because that is all the application  */
 /* needs, and knows about.                                           */
 /*********************************************************************/
 return (PFIXFROMN(Head));
}

#ifdef DRAG_DONE
/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrDrawTargetEmphBInDV (hwndCnr, pRecord, EmphPos)

HANDLE        hwndCnr;
LPRECORDCOREN  pRecord;
DWORD         EmphPos;

{
 LPCNRINFON  pccinfon;
 LPCNRINFO   pccinfo;
 long       CyBetweenRows;
 HDC        hpsLeft;
 HDC        hpsRight;
 RECT       ViewPortLeft;
 RECT       ViewPortRight;
 POINT      ptlLeft;
 POINT      ptlRight;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* Set the distance between the rows.  If the application has        */
 /* specified it, use it...otherwise use our constant.                */
 /*********************************************************************/
 if (pccinfo->cyLineSpacing >= 0)
 {
    CyBetweenRows = pccinfo->cyLineSpacing;
 }
 else
 {
    CyBetweenRows = MINCY_BETWEENITEMS;
 }

 if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
 {
    /******************************************************************/
    /* Get the hps for each of the windows.                           */
    /******************************************************************/
    hpsLeft  = PMGetFlippedDC (GetDlgItem (hwndCnr, CID_LEFTDVWND));
    hpsRight = PMGetFlippedDC (GetDlgItem (hwndCnr, CID_RIGHTDVWND));

    /******************************************************************/
    /* Get the ViewPorts of both windows.                             */
    /******************************************************************/
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPortLeft);

    PMGetClientRect (GetDlgItem (hwndCnr, CID_RIGHTDVWND),
                        &ViewPortRight);

    /******************************************************************/
    /* Set the initial x positions for both windows.                  */
    /******************************************************************/
    ptlLeft.x  = ViewPortLeft.left;
    ptlRight.x = ViewPortRight.left;

    /******************************************************************/
    /* Set the initial y position for both windows.                   */
    /******************************************************************/
    if (EmphPos == DRAGTARGETEMPHBELOW)
    {
       /***************************************************************/
       /* Get the bottom of the row, and subtract a little to leave   */
       /* some space.                                                 */
       /***************************************************************/
       ptlLeft.y  = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);
       ptlLeft.y += (CyBetweenRows / 2) + (CyBetweenRows & 1);

       /***************************************************************/
       /* Map the point to the window.                                */
       /***************************************************************/
       ptlLeft.y -= pccinfon->ptlNonIconOrg.y;
    }
    else
    /******************************************************************/
    /* Target emphasis will be drawn above the given record.          */
    /******************************************************************/
    {
       /***************************************************************/
       /* Get the top of the row and add 1 to leave some space.       */
       /***************************************************************/
       ptlLeft.y = CnrQueryRowPosInDV (pRecord, PT_TOP) -
                   (CyBetweenRows / 2);

       /***************************************************************/
       /* Map the point to the window.                                */
       /***************************************************************/
       ptlLeft.y -= pccinfon->ptlNonIconOrg.y;
    }

    /******************************************************************/
    /* The right window's y should be the same as the left window's   */
    /* y.                                                             */
    /******************************************************************/
    ptlRight.y = ptlLeft.y;

    /******************************************************************/
    /* Move to the starting positions.                                */
    /******************************************************************/
    MoveToEx( hpsLeft, ptlLeft.x, ptlLeft.y, NULL );                    /*DAB1*/
    MoveToEx( hpsRight, ptlRight.x, ptlRight.y, NULL );                 /*DAB1*/

    /******************************************************************/
    /* Set the ending positions for the lines.                        */
    /******************************************************************/
    ptlLeft.x  = ViewPortLeft.right;
    ptlRight.x = ViewPortRight.right;

    /******************************************************************/
    /* Save current PS attributes.                                    */
    /******************************************************************/
    SaveDC (hpsLeft);
    SaveDC (hpsRight);

    /******************************************************************/
    /* Set the color for the lines.                                   */
    /******************************************************************/
    SetROP2 (hpsLeft, R2_NOT);
    SetROP2 (hpsRight, R2_NOT);

    /******************************************************************/
    /* Draw the 2 lines.                                              */
    /******************************************************************/
    LineTo (hpsLeft, ptlLeft.x, ptlLeft.y);
    LineTo (hpsRight, ptlRight.x, ptlRight.y);

    /******************************************************************/
    /* Restore the previous PS attributes.  Make sure to pop them in  */
    /* reverse order from the push though.                            */
    /******************************************************************/
    RestoreDC (hpsRight, -1L);
    RestoreDC (hpsLeft, -1L);

    /******************************************************************/
    /* Be a good guy and release both hps's.                          */
    /******************************************************************/
    ReleaseDC ( GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);
    ReleaseDC ( GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
 }
 else
 /*********************************************************************/
 /* Only 1 window exists.  That means that it is the left window.     */
 /* Handle the drawing of ordered target emphasis for the left window */
 /* only.                                                             */
 /*********************************************************************/
 {
    /******************************************************************/
    /* Get the hps for the left window.                               */
    /******************************************************************/
    hpsLeft  = PMGetFlippedDC (GetDlgItem (hwndCnr, CID_LEFTDVWND));

    /******************************************************************/
    /* Get the ViewPort of the left window.                           */
    /******************************************************************/
    PMGetClientRect (GetDlgItem (hwndCnr, CID_LEFTDVWND),
                        &ViewPortLeft);

    /******************************************************************/
    /* Set the initial x position for the left window.                */
    /******************************************************************/
    ptlLeft.x = ViewPortLeft.left;

    /******************************************************************/
    /* Set the initial y positio for the left window.                 */
    /******************************************************************/
    if (EmphPos == DRAGTARGETEMPHBELOW)
    {
       /***************************************************************/
       /* Get the bottom of the row, and subtract a little to leave   */
       /* some space.                                                 */
       /***************************************************************/
       ptlLeft.y  = CnrQueryRowPosInDV (pRecord, PT_BOTTOM);
       ptlLeft.y += (CyBetweenRows / 2) + (CyBetweenRows & 1);

       /***************************************************************/
       /* Map the point to the window.                                */
       /***************************************************************/
       ptlLeft.y -= pccinfon->ptlNonIconOrg.y;
    }
    else
    /******************************************************************/
    /* Target emphasis will be drawn above the given record.          */
    /******************************************************************/
    {
       /***************************************************************/
       /* Get the top of the row and add 1 to leave some space.       */
       /***************************************************************/
       ptlLeft.y = (int)(CnrQueryRowPosInDV (pRecord, PT_TOP) -
                   (CyBetweenRows / 2));

       /***************************************************************/
       /* Map the point to the window.                                */
       /***************************************************************/
       ptlLeft.y -= pccinfon->ptlNonIconOrg.y;
    }

    /******************************************************************/
    /* Move to the starting position in the left window.              */
    /******************************************************************/
    MoveToEx( hpsLeft, ptlLeft.x, ptlLeft.y, NULL );                    /*DAB1*/

    /******************************************************************/
    /* Set the ending position for the line.                          */
    /******************************************************************/
    ptlLeft.x = ViewPortLeft.right;

    /******************************************************************/
    /* Save current PS attributes.                                    */
    /******************************************************************/
    SaveDC (hpsLeft);

    /******************************************************************/
    /* Set the color for the line.                                    */
    /******************************************************************/
    SetROP2 (hpsLeft, R2_NOT);

    /******************************************************************/
    /* Draw the line.                                                 */
    /******************************************************************/
    LineTo(hpsLeft, ptlLeft.x, ptlLeft.y);

    /******************************************************************/
    /* Restore the previous PS attributes.                            */
    /******************************************************************/
    RestoreDC (hpsLeft, -1L);

    /******************************************************************/
    /* Release the presentation space.                                */
    /******************************************************************/
    DrgReleasePS (hpsLeft);
 }

 return;
}
#endif
/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrFreeDetailFieldInfo                            */
/*                                                                    */
/*  Descriptive Name:  Free Detail FieldInfo Structures               */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function will Free the memory associated with FieldInfo      */
/*  structures.  Individual FieldInfo's can be freed, or a list of    */
/*  FieldInfo's.  The Storage Manager is used to accomplish the       */
/*  freeing of memory.                                                */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          mp1     - A pointer to an array of pointers to            */
/*                    FieldInfo structures to be freed.               */
/*          mp2     - The number of FieldInfo structures to be freed. */
/*                                                                    */
/*  Output:  TRUE  - The memory was freed sucessfully.                */
/*           FALSE - An error occured while freeing at least one of   */
/*                   the FieldInfo structures.                        */
/*                                                                    */
/*  Exit-Normal:  TRUE is returned.                                   */
/*                                                                    */
/*  Exit-Error:   NULL with WinSetErrorInfo message specifed:         */
/*                - PMERR_INVALID_PARAMETERS                          */
/*                - PMERR_MEMORY_DEALLOCATION_ERR                     */
/*                - PMERR_FI_CURRENTLY_INSERTED                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrFreeDetailFieldInfo (hwndCnr, mp1, mp2)

HANDLE  hwndCnr;   /* Handle to the Container window.                 */
long    mp1;       /* Pointer to an array of pointers of FieldInfo    */
                   /* structures to be freed.                         */
long    mp2;       /* The number of FieldInfo structures to be freed. */
{

 WORD        i;                  /* Array index for loop.             */
 WORD        NumFieldInfo;       /* Number of FieldInfo structures    */
                                 /* to be freed.                      */
 LPFIELDINFO  FAR *pFieldInfoArray;  /* Pointer to an array of pointers*/
                                 /* to FieldInfo structures to be     */
                                 /* freed.                            */
// LPVOID      pSatPtr;            /* Pointer to Storage Space.         */
 BOOL        InUse = FALSE;      /* Indicates if a FieldInfo is       */
                                 /* currently being used.             */

 pFieldInfoArray = (LPVOID)mp1;
 NumFieldInfo    = LOWORD(mp2);

 /*********************************************************************/
 /* Get the pointer to the storage space.                             */
 /*********************************************************************/
// pSatPtr = (LPVOID)GetWindowLong (hwndCnr, QWL_STORAGEPTR);

 /*********************************************************************/
 /* Verify the parameters to be sure they are valid.                  */
 /*********************************************************************/
 if ((NumFieldInfo < 1) || (!pFieldInfoArray))
 {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return (FALSE);                                                     /*DAB1*/
 }

 /*********************************************************************/
 /* Check all of the FieldInfo Structures that are to be freed, and   */
 /* make sure none of them are currently inserted in the linked list  */
 /* of FieldInfo structures.  The app. must remove them from this     */
 /* list before freeing them.  If the FieldInfo is not being used,    */
 /* try to free it, otherwise flag an error and continue.  If freeing */
 /* the FieldInfo completes successfully, set the applications        */
 /* pointer to NULL.                                                  */
 /*********************************************************************/
 for (i=0; i < NumFieldInfo; i++)
 {
    if (!CnrVerifyFieldInfoPtrs (hwndCnr, *(pFieldInfoArray + i), 1))
       {
//        if (!(StmFreeStorage (pSatPtr,
//                          (LPVOID)PFINFROMX(*(pFieldInfoArray + i)))))
          if (GlobalFree( (LPVOID)PFINFROMX(*(pFieldInfoArray + i))))   /*DAB1*/
          {
             SETERROR(PMERR_MEMORY_DEALLOCATION_ERR);
             return (FALSE);                                            /*DAB1*/
          }
       }
       else
       {
          /************************************************************/
          /* The FieldInfo is still being used, so the memory cannot  */
          /* be freed.                                                */
          /************************************************************/
          SETERROR(PMERR_FI_CURRENTLY_INSERTED);
          return (FALSE);                                               /*DAB1*/
       }
 }

 /*********************************************************************/
 /* If none of the FieldInfo's that were to be freed were inserted,   */
 /* and no errors occured, return the good news to the application.   */
 /*********************************************************************/
 return (TRUE);

}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrRemoveDetailFieldInfo                          */
/*                                                                    */
/*  Descriptive Name:  Remove Detail FieldInfo structures             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function removes FieldInfo structures from the list of       */
/*  FieldInfo's.  One or more FieldInfo's can be removed at a time.   */
/*  In addition, the application can specify that the memory          */
/*  associated with each FieldInfo be freed after removal.            */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          mp1     - Pointer to an array of Pointers to FieldInfo    */
/*                    structures to be removed.                       */
/*          mp2     - Flag to indicate if the memory associated with  */
/*                    the removed FieldInfo structures is to be       */
/*                    freed, flag to specify whether the container    */
/*                    should be invalidated, and the number of        */
/*                    FieldInfo structures to be removed.             */
/*                                                                    */
/*  Output:  The number of FieldInfo structures currently in the      */
/*           linked list.                                             */
/*                                                                    */
/*  Exit-Normal:  The number of FieldInfo structures currently in     */
/*                the linked list after removal.                      */
/*                                                                    */
/*  Exit-Error:   NULL with WinSetErrorInfo message specifed:         */
/*                - PMERR_INVALID_PARAMETERS                          */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
int FAR CnrRemoveDetailFieldInfo (hwndCnr, mp1, mp2)

HANDLE  hwndCnr;   /* Handle of the Container window.                 */
long    mp1;       /* Pointer to an array of pointers to FieldInfo    */
                   /* structures to be removed.                       */
long    mp2;       /* Flag to indicate if the memory associated with  */
                   /* the removed FieldInfo structures is to be       */
                   /* freed, flag to specify whether the container    */
                   /* should be invalidated, and the number of        */
                   /* FieldInfo structures to be removed.             */
{

 LPCNRINFON   pccinfon;          /* Pointer to Internal Control info. */
 LPCNRINFO    pccinfo;
 LPFIELDINFO  FAR *pFieldInfoArray; /* Pointer to an array of pointers */
                                 /* to FieldInfo structures.          */
 LPFIELDINFO  pFieldInfo;        /* Pointer to a FieldInfo structure. */
 LPFIELDINFO  pNextFI;           /* Pointer to a FieldInfo structure. */
 LPFIELDINFO  pPrevFI;           /* Pointer to a FieldInfo structure. */
 WORD         NumFieldInfo;      /* Number of FieldInfo's to be       */
                                 /* removed.                          */
 WORD         RemoveAndInval;    /* Flag to indicate if we are to     */
                                 /* Free the memory as well.          */
 WORD         i;                 /* Array index for loop.             */
 BOOL         Found = FALSE;     /* Indicates if we found the         */
                                 /* FieldInfo to be removed.          */
 BOOL        Removed = FALSE;    /* Indicates if any FieldInfo's were */
                                 /* removed.                          */

 pFieldInfoArray = (LPVOID)mp1;
 NumFieldInfo    = LOWORD(mp2);
 RemoveAndInval  = HIWORD(mp2);

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* If we recieve this message and there are no FieldInfo's currently */
 /* inserted, just return 0.                                          */
 /*********************************************************************/
 if (!pccinfon->pFirstFI)
 {
    return ((int) 0);
 }

 /*********************************************************************/
 /* If NumFieldInfo is equal to 0, we don't care what pFieldInfoArray */
 /* is.  But if NumFieldInfo is not 0, we must check to be sure that  */
 /* pFieldInfoArray is not NULL.                                      */
 /*********************************************************************/
 if (NumFieldInfo)
 {
    if (!pFieldInfoArray)
    {
       SETERROR(PMERR_INVALID_PARAMETERS);
       return ((int)ERRORFLAG);
    }
 }

 /********************************************************************/
 /* If the application has set the verify pointers flag, we need to  */
 /* verify all of the FieldInfo pointers that were passed to us.  If */
 /* any of them are invalid, we return an error.                     */
 /********************************************************************/
 if (GetWindowLong (hwndCnr, GWL_STYLE) & CCS_VERIFYPOINTERS)
 {
    for (i = 0; i < NumFieldInfo; i++)
    {
       if (!(CnrVerifyFieldInfoPtrs (hwndCnr,
                                     *(pFieldInfoArray + i),
                                     1)))
       {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return ((int)ERRORFLAG);
       }
    }
 }

 /********************************************************************/
 /* When NumFieldInfo is 0 and RemoveAndInval is CMA_FREE, free all  */
 /* the FieldInfo structures currently in the container.  To do this */
 /* call CnrFreeAll, specifying FALSE to tell it to free FieldInfo's */
 /* not Records.  Also, set the number of FieldInfo's (cFields) to   */
 /* 0.                                                               */
 /********************************************************************/
 if (NumFieldInfo == 0)
 {
    if (RemoveAndInval & CMA_FREE)
    {
       if (!(CnrFreeAll (hwndCnr, FALSE)))
       {
          SETERROR(PMERR_MEMORY_DEALLOCATION_ERR);
          return ((int)ERRORFLAG);
       }
    }
    else
    /*****************************************************************/
    /* We do not free the storage associated with the FieldInfo      */
    /* structures in this case.  All we do is remove them from the   */
    /* list by setting all of the internal pointers to NULL.         */
    /*****************************************************************/
    {
       pFieldInfo = pccinfon->pFirstFI;

       while (pFieldInfo)
       {
          pNextFI = NEXTFI(pFieldInfo);
          NEXTFI(pFieldInfo) = NULL;
          NEXTVISIBLEFI(pFieldInfo) = NULL;
          pFieldInfo = pNextFI;
       }
    }

    /******************************************************************/
    /* Reset these details view values since we removed all the       */
    /* columns.                                                       */
    /******************************************************************/
    pccinfo->cFields = 0;
    pccinfon->preccFirstRowInDV = NULL;
    pccinfon->pFIFirstColLeftInDV = pccinfon->pFIFirstColRightInDV = NULL;
    pccinfon->pFILastColLeftInDV = pccinfon->pFILastColRightInDV = NULL;
    pccinfon->pFirstVisibleFI = pccinfon->pFirstFI = NULL;
    pccinfo->pFieldInfoLast = pccinfon->pFieldInfoLast = NULL;
    pccinfo->pFieldInfoObject = NULL;

    Removed = TRUE;
 }
 else
 /********************************************************************/
 /* We are passed an array of pointers to FieldInfo structures which */
 /* are to be removed.                                               */
 /********************************************************************/
 {
    for (i = 0; i < NumFieldInfo; i++)
    {
       /**************************************************************/
       /* Check to see if it's the first one in the list.            */
       /**************************************************************/
       if (pccinfon->pFirstFI == (*(pFieldInfoArray + i)))
       {
          Removed = TRUE;
          /***********************************************************/
          /* Check to see if the FieldInfo (column) that we are      */
          /* removing is the current first in view column.           */
          /***********************************************************/
          if (pccinfon->pFirstFI == pccinfon->pFIFirstColLeftInDV)
          {
             pccinfon->pFIFirstColLeftInDV = NULL;
          }

          /************************************************************/
          /* If the app. is removing pFieldInfoLast, set the external */
          /* and internal pointer to it to NULL.                      */
          /************************************************************/
          if (pccinfon->pFirstFI == pccinfo->pFieldInfoLast)
          {
             pccinfo->pFieldInfoLast = NULL;
             pccinfon->pFieldInfoLast  = NULL;
          }

          /************************************************************/
          /* Proceed with the removal.                                */
          /************************************************************/
          pPrevFI = pccinfon->pFirstFI;
          pccinfon->pFirstFI = NEXTFI(pPrevFI);
          NEXTFI(pPrevFI) = NULL;
          NEXTVISIBLEFI(pPrevFI) = NULL;

          /************************************************************/
          /* Decrement the global count of the number of FieldInfo's  */
          /* currently inserted.                                      */
          /************************************************************/
          pccinfo->cFields--;
       }
       else
       {
          /************************************************************/
          /* It must be later in the list.  Traverse the rest of the  */
          /* list until we find it, then remove it.  After removing   */
          /* the FieldInfo, decrement the cFields field in CNRINFO.   */
          /* This field contains the number of FieldInfo structures   */
          /* currently inserted.                                      */
          /************************************************************/
          pPrevFI = pccinfon->pFirstFI;
          pFieldInfo = NEXTFI(pPrevFI);

          while ((pFieldInfo) && (!Found))
          {
             if (pFieldInfo == (*(pFieldInfoArray + i)))
             {
                Removed = TRUE;
                Found = TRUE;
                /******************************************************/
                /* Check to see if the FieldInfo (column) that we     */
                /* are removing is the current first in view column   */
                /* for the left window.                               */
                /******************************************************/
                if ((*(pFieldInfoArray + i)) ==
                                         pccinfon->pFIFirstColLeftInDV)
                {
                   pccinfon->pFIFirstColLeftInDV = NULL;
                }

                /******************************************************/
                /* Check to see if the column we are removing is the  */
                /* first in view in the right window, and if so set   */
                /* it to the next column in view.                     */
                /******************************************************/
                if ((*(pFieldInfoArray + i)) ==
                                        pccinfon->pFIFirstColRightInDV)
                {
                   pccinfon->pFIFirstColRightInDV =
                          NEXTVISIBLEFI(pccinfon->pFIFirstColRightInDV);
                }

                /******************************************************/
                /* If the app. is removing pFieldInfoLast, set the    */
                /* external and internal pointer to it to NULL.       */
                /******************************************************/
                if ((*(pFieldInfoArray + i)) ==
                                          pccinfo->pFieldInfoLast)
                {
                   pccinfo->pFieldInfoLast = NULL;
                   pccinfon->pFieldInfoLast  = NULL;
                }

                /******************************************************/
                /* Proceed with the removal.                          */
                /******************************************************/
                NEXTFI(pPrevFI) = NEXTFI(pFieldInfo);
                NEXTFI(pFieldInfo) = NULL;
                NEXTVISIBLEFI(pFieldInfo) = NULL;

                /******************************************************/
                /* Decrement the global count of the number of        */
                /* FieldInfos currently inserted.                     */
                /******************************************************/
                pccinfo->cFields--;
             }
             else
             {
                /******************************************************/
                /* There wasn't a match, so go to the next one.       */
                /******************************************************/
                pPrevFI = pFieldInfo;
                pFieldInfo = NEXTFI(pFieldInfo);
             }
          }
       }

       Found = FALSE;
    }

    /******************************************************************/
    /* If the application wished for the memory to be freed after     */
    /* removal, then call CnrFreeDetailFieldInfo and Free the memory  */
    /* associated with the removed FieldInfo structures.              */
    /******************************************************************/
    if ((RemoveAndInval & CMA_FREE) && (Removed))
    {
       if (!(CnrFreeDetailFieldInfo (hwndCnr,
                                     (long)pFieldInfoArray,
                                     (DWORD)NumFieldInfo)))
       {
          SETERROR(PMERR_MEMORY_DEALLOCATION_ERR);
          return ((int)ERRORFLAG);
       }
    }
    if (pccinfo->cFields == 0)
    {
       pccinfon->pFirstVisibleFI = NULL;
    }
 }

 /*********************************************************************/
 /* If anything was removed from the list of FieldInfo's, we need to  */
 /* update the visible pointers, and check to see if invalidation was */
 /* specified.                                                        */
 /*********************************************************************/
 if (Removed)
 {
    /******************************************************************/
    /* If any FieldInfo's were removed, invalidate the window to      */
    /* display any changes.                                           */
    /******************************************************************/
    if (RemoveAndInval & CMA_INVALIDATE)
    {
       CnrInvalidateDetailFieldInfo (hwndCnr);
    }
    else
    /******************************************************************/
    /* Invlidation wasn't specified, but we still need to update the  */
    /* visible pointer list.                                          */
    /******************************************************************/
    {
       CnrProcessVisibleColumns (hwndCnr);
    }
 }

 /*********************************************************************/
 /* Return the current number of FieldInfo structures inserted to     */
 /* the application after we have removed them.                       */
 /*********************************************************************/
 return (pccinfo->cFields);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrQueryDetailFieldInfo                           */
/*                                                                    */
/*  Descriptive Name:  Query Detail FieldInfo structures.             */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function returns specific FieldInfo structures to the        */
/*  application.  The FieldInfo structure returned is dependent on    */
/*  what the application asks for via the parameters.                 */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*          mp1     - Pointer to a FieldInfo structure.               */
/*          mp2     - Indicates which FieldInfo structure to return.  */
/*                                                                    */
/*  Output:  Pointer to a FieldInfo structure that the application    */
/*           asked for.                                               */
/*                                                                    */
/*  Exit-Normal:  Pointer to a FieldInfo structure the application    */
/*                asked for.                                          */
/*                                                                    */
/*  Exit-Error:   NULL with WinSetErrorInfo message specifed:         */
/*                - PMERR_INVALID_PARAMETERS                          */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
LPFIELDINFO FAR CnrQueryDetailFieldInfo (hwndCnr, mp1, mp2)

HANDLE  hwndCnr;  /* Handle of the Container window.                  */
long    mp1;      /* Pointer to a FieldInfo structure.                */
long    mp2;      /* Indicates which FieldInfo structure to return.   */
{

 LPCNRINFON  pccinfon;    /* Pointer to Internal Control information  */
 WORD        cmd;         /* Indicates which FieldInfo structure to   */
                          /* return.                                  */
 LPFIELDINFO pFieldInfo;  /* Pointer to the FieldInfo structure       */
                          /* passed in by the application.            */
 LPFIELDINFO pFI;         /* Pointer to a FieldInfo structure.        */
 LPFIELDINFO pPrevFI;     /* Pointer to a FieldInfo structure.        */

 pFieldInfo = (LPVOID)mp1;
 cmd        = LOWORD(mp2);

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);

 /*********************************************************************/
 /* Return the First FieldInfo structure in the list.                 */
 /*********************************************************************/
 if (cmd == CMA_FIRST)
 {
    return (pccinfon->pFirstFI);
 }

 /*********************************************************************/
 /* Return the Last FieldInfo structure in the list.                  */
 /*********************************************************************/
 if (cmd == CMA_LAST)
 {
    pFI = pccinfon->pFirstFI;

    if (pFI)
    {
       while (NEXTFI(pFI))
       {
          pFI = NEXTFI(pFI);
       }
    }

    return (pFI);
 }

 /*********************************************************************/
 /* If we get to here, then that means that we are looking for either */
 /* the Next or Previous FieldInfo.  Therefore, we must utilize the   */
 /* mp1 parameter passed in by the application.  We need to verify    */
 /* that it is not NULL, and if Verify Pointers is set, check to see  */
 /* if it is valid.                                                   */
 /*********************************************************************/
 if (pFieldInfo)
 {
    if (GetWindowLong (hwndCnr, GWL_STYLE) & CCS_VERIFYPOINTERS)
    {
       if (!(CnrVerifyFieldInfoPtrs (hwndCnr, pFieldInfo, 1)))
       {
          SETERROR(PMERR_INVALID_PARAMETERS);
          return ((LPFIELDINFO)ERRORFLAG);
       }
    }

    /******************************************************************/
    /* Return the FieldInfo that is next in the list after the one    */
    /* the application passed in.                                     */
    /******************************************************************/
    if (cmd == CMA_NEXT)
    {
      return (NEXTFI(pFieldInfo));
    }

    /******************************************************************/
    /* Return the FieldInfo that is before the one the application    */
    /* passed in.                                                     */
    /******************************************************************/
    if (cmd == CMA_PREV)
    {
       pFI = pccinfon->pFirstFI;
       pPrevFI = NULL;

       while ((pFI) && (pFI != pFieldInfo))
       {
          pPrevFI = pFI;
          pFI = NEXTFI(pFI);
       }

       return (pPrevFI);
    }

    /******************************************************************/
    /* Return the column width for the fieldinfo passed by application*/
    /******************************************************************/
    if (cmd == CMA_WIDTH)                                               /*DAB2*/
    {                                                                   /*DAB2*/
      if (pFieldInfo->cxWidth)                                          /*DAB2*/
        return (LPFIELDINFO)(pFieldInfo->cxWidth);                      /*DAB2*/
      else                                                              /*DAB2*/
        return( (LPFIELDINFO)((PFINFROMX(pFieldInfo))->usLongestLine) );/*DAB2*/
    }                                                                   /*DAB2*/
 }

 /*********************************************************************/
 /* If we get to here, it indicates that the cmd parameter was not    */
 /* properly set by the application or pFieldInfo was NULL, or not a  */
 /* valid FieldInfo.                                                  */
 /*********************************************************************/
 SETERROR(PMERR_INVALID_PARAMETERS);
 return ((LPFIELDINFO)ERRORFLAG);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  xxx                                               */
/*                                                                    */
/*  Descriptive Name:  xxx                                            */
/*                                                                    */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  xxx                                                               */
/*                                                                    */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:  xxx                                                      */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:  xxx                                                 */
/*                                                                    */
/*  Exit-Error:   N/A                                                 */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
BOOL FAR CnrHorzScrollSplitWindow (hwndCnr, Window, ScrollUnit)

HANDLE  hwndCnr;
int     Window;
long    ScrollUnit;

{
 LPCNRINFON  pccinfon;
 LPCNRINFO   pccinfo;
 int         Scrollbar;
 long        lScroll;
 int         sScrollPos;
 LPINT       HScrollPos;
 LPINT       HScrollMax;

 pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
 pccinfo  = PCCINFOXFROMN(pccinfon);

 /*********************************************************************/
 /* If we are not in Details View or a splitbar doesn't exist, set an */
 /* error and return NULL.                                            */
 /*********************************************************************/
 if ((!(pccinfo->flWindowAttr & CV_DETAIL)) ||
     (pccinfon->ulFlagSet & FL_UNSPLIT) ||
     (!(CnrVerifyFieldInfoPtrs (hwndCnr, pccinfon->pFieldInfoLast, 1))))
 {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return (FALSE);                                                     /*DAB1*/
 }

 /*********************************************************************/
 /* If we are in Details View and have a splitbar, then let's         */
 /* validate the parameters and return an error if they are bad.      */
 /*********************************************************************/
 if (!Window)
 {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return (FALSE);                                                     /*DAB1*/
 }

 /*********************************************************************/
 /* Well, I guess the parameters are good, so let's go and scroll     */
 /* the window we are asked to scroll.                                */
 /*********************************************************************/
 if (Window == CMA_LEFT)
 {
    Scrollbar  = CID_HSCROLL;
    HScrollPos = &pccinfon->sHscrollPos;
    HScrollMax = &pccinfon->sHscrollMax;
 }
 else if (Window == CMA_RIGHT)
 {
    Scrollbar  = CID_RIGHTHSCROLL;
    HScrollPos = &pccinfon->sHscrollPosRight;
    HScrollMax = &pccinfon->sHscrollMaxRight;
 }
 else
 {
    SETERROR(PMERR_INVALID_PARAMETERS);
    return (FALSE);                                                     /*DAB1*/
 }

 if (ScrollUnit % (long)(pccinfon->sHscrollFactor))
 {
    lScroll = (ScrollUnit / (long)(pccinfon->sHscrollFactor)) + 1;
 }
 else
 {
    lScroll = ScrollUnit / (long)(pccinfon->sHscrollFactor);
 }

 sScrollPos = min((long)(*HScrollMax), (long)(*HScrollPos) + lScroll);  /*DAB1*/

 if ( sScrollPos < 0 )                                                  /*PPB1*/
   sScrollPos = 0;                                                      /*PPB1*/

 SENDMESSAGE( hwndCnr, WM_HSCROLL,
              MAKELONG( SB_THUMBPOSITION, sScrollPos ),                 /*DAB1*/
              (DWORD)GetDlgItem(hwndCnr, Scrollbar) );                  /*DAB1*/

 return (TRUE);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:  CnrTrackSplitbar                                  */
/*                                                                    */
/*  Descriptive Name:  Track the Splitbar in Details View.            */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  This function enables the Splitbar in Details View to be moved    */
/*  back and forth in the Container Window.  The Splitbar is moved    */
/*  via the WinTrackRect API.                                         */
/*                                                                    */
/*  Notes:  The Splitbar spearates the left and right split windows.  */
/*                                                                    */
/*  Input:  hwndCnr - Handle of the Container Window.                 */
/*                                                                    */
/*  Output:  VOID                                                     */
/*                                                                    */
/*  Exit-Normal: VOID                                                 */
/*                                                                    */
/*  Exit-Error:  VOID                                                 */
/*                                                                    */
/*  Side Effects:  After the Splitbar is moved, the sizes of the 2    */
/*                 windows are changed and then updated.              */
/*                                                                    */
/*  Author:  Peter Haggar                                             */
/**********************************************************************/
VOID FAR CnrTrackSplitbar (hwndCnr)

HWND  hwndCnr;     /* Handle to the Container window.                 */

{
 LPCNRINFON  pccinfon;      /* Pointer to Internal control info.      */
 LPCNRINFO   pccinfo;
 RECT        TitleWinRect;  /* Rect of the Title window.              */
 TRACKINFO   TrackInfo;     /* Data Structure used for WinTrackRect.  */

  pccinfon = (LPCNRINFON)GetWindowLong (hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Get the Container window rectangle and put it in rclBoundary.    */
  /* Then get the rectangle of the title window so we can subtract it */
  /* off later....we don't want the split bar to track through it.    */
  /********************************************************************/
  GetClientRect (hwndCnr, &(TrackInfo.rclBoundary));
  CnrQueryTitleRect (hwndCnr, &TitleWinRect, NULL);

  /********************************************************************/
  /* The Splitbar should track below the title window and no go any   */
  /* farther right than the vertical scroll bar.                      */
  /********************************************************************/
  TrackInfo.rclBoundary.top   += TitleWinRect.bottom;
  TrackInfo.rclBoundary.right -= GetSystemMetrics ( SM_CXVSCROLL);

  /********************************************************************/
  /* This specifies the width of the Splitbar, so WinTrackRect knows  */
  /* how wide of a thing to track.  The Splitbar is the width of a    */
  /* Size Border.                                                     */
  /********************************************************************/
  TrackInfo.cxBorder = GetSystemMetrics ( SM_CXFRAME);

  /********************************************************************/
  /* This specifies the height of the splitbar so WinTrackRect knows  */
  /* how tall of a thing to track.                                    */
  /********************************************************************/
  TrackInfo.cyBorder = TrackInfo.rclBoundary.bottom;

  TrackInfo.cxGrid = 0;
  TrackInfo.cyGrid = 0;
  TrackInfo.cxKeyboard = 0;
  TrackInfo.cyKeyboard = 0;

  /********************************************************************/
  /* This specifies the tracking rectangle used by WinTrackRect.      */
  /********************************************************************/
  TrackInfo.rclTrack.bottom = TrackInfo.rclBoundary.bottom;
  TrackInfo.rclTrack.top    = TrackInfo.rclBoundary.top;
  TrackInfo.rclTrack.left   = pccinfo->xVertSplitbar;
  TrackInfo.rclTrack.right  = pccinfo->xVertSplitbar +
                               TrackInfo.cxBorder;

  /********************************************************************/
  /* Specify the minimum and maximum tracking size.                   */
  /********************************************************************/
  TrackInfo.ptlMinTrackSize.x = TrackInfo.cxBorder;
  TrackInfo.ptlMinTrackSize.y = TrackInfo.cyBorder;
  TrackInfo.ptlMaxTrackSize.x = TrackInfo.cxBorder;
  TrackInfo.ptlMaxTrackSize.y = TrackInfo.cyBorder;

  /********************************************************************/
  /* These tracking options specify:  TF_MOVE - to center the pointer */
  /* in the tracking rectangle.       TF_ALLINBOUNDARY - perfomrs     */
  /* tracking such that no part of the tracking rectangle ever falls  */
  /* outside the bounding rectangle.                                  */
  /********************************************************************/
//TrackInfo.fs = TF_MOVE | TF_ALLINBOUNDARY;

  /********************************************************************/
  /* If WinTrackRect completes successfully, we set the new position  */
  /* of the Splitbar, (which WinTrackRect keeps track of) resize the  */
  /* 2 split windows (left and right) and repaint both windows.       */
  /* If WinTrackRect does not complete successfully, the splitbar did */
  /* not change position, so we do nothing.                           */
  /********************************************************************/
  if (WinTrackRect (hwndCnr, &TrackInfo, pccinfon->hptrSplitPtr))
  {
     /*****************************************************************/
     /* When the splitbar is moved, we mush size the 2 Column Title   */
     /* Windows, size the 2 Details Windows, and reset the scroll     */
     /* bars.                                                         */
     /*****************************************************************/
     pccinfon->ulFlagSet &= ~(FL_SIZECOLTITLES | FL_SIZECHILDREN |
                         FL_SETSCROLL );

     /*****************************************************************/
     /* Set the new position of the splitbar in the global data.      */
     /*****************************************************************/
     pccinfo->xVertSplitbar = TrackInfo.rclTrack.left;

     /*****************************************************************/
     /* Invalidate the Container window, so the splitbar will be      */
     /* repainted in the new position.  Remember that the splibar is  */
     /* the only part of Details View that is part of the container   */
     /* window.                                                       */
     /*****************************************************************/
     InvalidateRect (hwndCnr, NULL, FALSE);
  }

 return;
}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrProcessSetFocus                             */
/*                                                                    */
/*  Descriptive Name:   Process WM_SETFOCUS/WM_KILLFOCUS              */
/*                                                                    */
/*  Functional Description: This function sends the notification      */
/*     messages (CN_SETFOCUS if bSettingFocus is TRUE,                */
/*     CN_KILLFOCUS if bSettingFocus is FALSE)                        */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:  long CnrProcessSetFocus(                 */
/*                               HWND       hwnd                      */
/*                               BOOL       bSettingFocus             */
/*                               )                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd           - container control handle                */
/*           bSettingFocus  - Indicates WM_SETFOCUS or WM_KILLFOCUS   */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:                                                     */
/**********************************************************************/
BOOL FAR CnrProcessSetFocus(hwnd, bSettingFocus)
HWND     hwnd;
BOOL     bSettingFocus;
{
  LPCNRINFON  pccinfon;
  HDC         hps;
  DWORD       flOldAttr;

  hps = PMGetFlippedDC(hwnd);

  /********************************************************************/
  /* Get ptr to Container instance data                               */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);

  if (pccinfon)
  {
    if (pccinfon->preccCursor)
    {
      if (bSettingFocus)
      {
        /*************************************************************/
        /* Cnr receives the focus, so draw the Cursor Emphasis       */
        /*************************************************************/
        flOldAttr = pccinfon->preccCursor->flRecordAttr & (~CRA_CURSORED);
        CnrDrawEmphChange (hwnd, hps, pccinfon->preccCursor, flOldAttr);
      }
      else
      {
        /*************************************************************/
        /* Cnr  loses  the focus, so erase the Cursor Emphasis       */
        /*************************************************************/
        flOldAttr = pccinfon->preccCursor->flRecordAttr;
        pccinfon->preccCursor->flRecordAttr &= (~CRA_CURSORED);
        CnrDrawEmphChange (hwnd, hps, pccinfon->preccCursor, flOldAttr);
        pccinfon->preccCursor->flRecordAttr = flOldAttr;
      }

      COPYNATTRTOX(pccinfon->preccCursor);
    }

    NotifyOwner( hwnd, (WORD)((bSettingFocus) ? CN_SETFOCUS :           /*DAB1*/
                 CN_KILLFOCUS), (LPVOID)hwnd );                         /*DAB1*/
  }

  ReleaseDC(hwnd, hps);
  return TRUE;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  DrawBlinkEmphBox                                 */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Draw Box Arround Item                        */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function draws a box according to the attribute          */
/*     arround the given item.                                       */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd - handle of the container window.                  */
/*           hps - presentation space                                */
/*           preccObject - pointer to the record                     */
/*           flOn - flag to indicate on or off                       */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR DrawBlinkEmphBox(hwnd, hps, preccObject, flAttr)
HANDLE       hwnd;
HDC          hps;
LPRECORDCOREN preccObject;
DWORD        flAttr;
{

  LPCNRINFON  pccinfon;            /* Internal CNR control data  */
  LPCNRINFO   pccinfo;
  RECT       rclBox;              /* rectangle where the box */
                                   /* will be drawn */
  POINT      iconSize;            /* size of the icon */
  HDC        hpsLeft;
  HDC        hpsRight = NULL;
  RECT       RowRectLeft;
  RECT       RowRectRight;
  HPEN       hPenOld;
  HPEN       hPenCurrent = NULL;
  HBRUSH     hBrushOld;
  int        nSavedDC;

  /*******************************************************************/
  /* Get pointer to locals                                           */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get size of the icon                                            */
  /*******************************************************************/
  CnrCalcIconSize(hwnd, &iconSize);

  if (pccinfo->flWindowAttr & CV_TEXT)
  {
    /*****************************************************************/
    /* Query TextRectangle                                           */
    /*****************************************************************/
    CnrQueryIconTextRect(hwnd, preccObject, &iconSize, FALSE,
                         NULL, &rclBox);

    /*****************************************************************/
    /* Need A little space for the drawing outside the text rect.    */
    /*****************************************************************/
    rclBox.left -= TXT_TOPMARGIN;
    rclBox.bottom += TXT_TOPMARGIN;
    rclBox.right += TXT_TOPMARGIN;
  }
  else
  {
    if (pccinfo->flWindowAttr & (CV_ICON | CV_NAME))
    {
      /***************************************************************/
      /* Query Icon Rectangle                                        */
      /***************************************************************/
      CnrQueryIconTextRect(hwnd, preccObject, &iconSize, FALSE,
                           &rclBox, NULL);
      rclBox.left -= ICN_EMPHSPACE;
      rclBox.bottom += ICN_EMPHSPACE;

      /***************************************************************/
      /* Note: Draw Bitmap doesn't paint top row and rightmost clmn  */
      /*    so, we need to take out those space by ICN_SIDEMARGIN    */
      /*    and ICN_TOPMARGIN                                        */
      /***************************************************************/
      rclBox.right += ICN_EMPHSPACE;
      rclBox.top -= (ICN_EMPHSPACE - ICN_SIDEMARGIN);

    }
    else if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       hpsLeft = PMGetFlippedDC (GetDlgItem (hwnd, CID_LEFTDVWND));
       CnrCalcRowRect (hwnd, preccObject, TRUE, &RowRectLeft);
       RowRectLeft.top ++;
       RowRectLeft.bottom --;
       RowRectLeft.right += 2;

       if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
           (CnrVerifyFieldInfoPtrs (hwnd,
                                    pccinfon->pFieldInfoLast,
                                    1)))
       {
          /************************************************************/
          /* Check to see if there is any data in the right window.   */
          /************************************************************/
          if (PFINFROMX(pccinfon->pFieldInfoLast)->pNextFieldInfo)
          {
             hpsRight = PMGetFlippedDC (GetDlgItem (hwnd, CID_RIGHTDVWND));

             CnrCalcRowRect (hwnd, preccObject, FALSE, &RowRectRight);
             RowRectRight.top ++;
             RowRectRight.bottom --;
             RowRectRight.left -= 2;
             RowRectRight.right += 2;
          }
       }
    }
  }

  /*******************************************************************/
  /* Draw the box according to the attribute of the record           */
  /*******************************************************************/
  if (flAttr & CRA_CURSORED)
  {
     if (pccinfo->flWindowAttr & CV_DETAIL)
     {
        CnrDrawCursorBox (hwnd, hpsLeft, preccObject,
                          RowRectLeft, TRUE);
        ReleaseDC (GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);

        /**************************************************************/
        /* If hpsRight is set, then that means that there is some     */
        /* data in the right window.                                  */
        /**************************************************************/
        if (hpsRight)
        {
           CnrDrawCursorBox (hwnd, hpsRight, preccObject,
                             RowRectRight, TRUE);
           ReleaseDC (GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
        }
     }
     else
     {
        IcnDrawBox( hps, flAttr, &rclBox);
     }
  }
  else
  {
    /*****************************************************************/
    /* Set bottom left position of the box to be drawn               */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
      CnrDrawCursorBox (hwnd, hpsLeft, preccObject,
                        RowRectLeft, FALSE);
      CnrDrawCursorBox (hwnd, hpsRight, preccObject,
                        RowRectRight, FALSE);
      ReleaseDC(GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);
      ReleaseDC(GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
    }
    else
    {
      /****************************************************************/
      /* Save current PS attributes                                   */
      /****************************************************************/
      if (!(nSavedDC = SaveDC (hps)))
        return;

      /****************************************************************/
      /* Draw the box from the current pos to the top right pos       */
      /****************************************************************/
      if (hPenCurrent = CreatePen (PS_SOLID, 1,
                             pccinfon->ulColorPPSupport[CMA_BACKGROUNDCOLOR]))
      {
        hPenOld = SelectObject (hps, hPenCurrent);
        hBrushOld = SelectObject (hps, GetStockObject(NULL_BRUSH));
        Rectangle(hps, rclBox.left, rclBox.top, rclBox.right, rclBox.bottom);
        SelectObject ( hps, hBrushOld);
        SelectObject ( hps, hPenOld);
        DeleteObject (hPenCurrent);
      }

      /****************************************************************/
      /* Restore the previous PS attributes                           */
      /****************************************************************/
      RestoreDC (hps, nSavedDC);
    }
  } /* endif */

  return;
}

/*********************************************************************/
/* FUNCTION NAME:  DrawCursorTargetRecord                            */
/*                                                                   */
/* DESCRIPTIVE NAME: Draw Or Erase Cursored or Target Emphasis Item  */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints the specific        */
/*   record to draw or erase the cursored or target emphasis.        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC  hps - handle of the presentation space               */
/*         LPRECORDCORE precord - pointer to the record structure    */
/*         DWORD ulWhich -                                           */
/*               CRA_CURSORED - indicates cursored emph              */
/*         BOOL flChangeZorder - flag to tell to change z-order      */
/*              TRUE - change the z order                            */
/*              FALSE - do not change the z order                    */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR DrawCursorTargetRecord( hwndCnr, hps, precord, ulWhich,
                             flChangeZorder)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN  precord;
DWORD         ulWhich;
BOOL          flChangeZorder;
{
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;

  if (ISRECORDVISIBLE(precord))
  {
    /*****************************************************************/
    /* If the record is not filtered out.........                    */
    /*****************************************************************/
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    if (!(precord->flRecordAttr & ulWhich))
    {
      /***************************************************************/
      /* If the cursored/target emphasis is off ...                  */
      /***************************************************************/
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        /*************************************************************/
        /* If is in icon view, erase icon only including the         */
        /* cursored or target emph space. (cnriconv.c)               */
        /*************************************************************/
        EraseIconOnlyInIV(hwndCnr, hps, precord, TRUE, TRUE);
        EraseTextOnlyInIV(hwndCnr, hps, precord);
      }
      else
      {
         if (pccinfo->flWindowAttr & CV_DETAIL)
         {
            /**********************************************************/
            /* Erase the row in Details View.                         */
            /**********************************************************/
            if (!(CnrDrawEraseRow (hwndCnr, precord, FALSE)))
            {
               return (FALSE);
            }
         }
         else
         {
           if (!(CnrEraseRecord(hwndCnr, hps, precord)))
           {
              return (FALSE);
           }
         }
      } /* endif */
    }
    else
    {
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        if (!(precord->flRecordAttr & CRA_SELECTED))
        {
          /***********************************************************/
          /* If is in icon view, erase text only if it is not        */
          /* selected.                                               */
          /***********************************************************/
          EraseTextOnlyInIV(hwndCnr, hps, precord);
        } /* endif */
      }

    } /* endif */
    /*****************************************************************/
    /* Redraw the record (cnrcfunc.c) or row if we are in Details    */
    /* View.                                                         */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       if (!(CnrDrawEraseRow (hwndCnr, precord, TRUE)))
       {
          return (FALSE);
       }
    }
    else
    {
       if (!(CnrDrawRecord( hwndCnr, hps, precord, flChangeZorder)))
       {
          return (FALSE);
       }
    }
  } /* endif */

  return (TRUE);
}

/*********************************************************************/
/* FUNCTION NAME:  DrawSelectionRecord                               */
/*                                                                   */
/* DESCRIPTIVE NAME: Draw Or Erase Selection Emphasis Of Item        */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function paints the specific        */
/*   record to draw or erase the selection emphasis.                 */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC  hps - handle of the presentation space               */
/*         LPRECORDCORE precord - pointer to the record structure    */
/*         BOOL flChangeZorder - flag to tell to change z-order      */
/*              TRUE - change the z order                            */
/*              FALSE - do not change the z order                    */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR DrawSelectionRecord (hwndCnr, hps, precord,
                               flChangeZorder, flBoth)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN  precord;
BOOL          flChangeZorder;
BOOL          flBoth;
{
  BOOL         flErase;
  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;

  if (ISRECORDVISIBLE(precord))
  {
    /*****************************************************************/
    /* If the record is not filtered out.........                    */
    /*****************************************************************/
    pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    if (flBoth)
    {
      flErase = !(precord->flRecordAttr & (CRA_SELECTED | CRA_CURSORED));
    }
    else
    {
      flErase = !(precord->flRecordAttr & CRA_SELECTED);
    } /* endif */

    if (flErase)
    {
      /***************************************************************/
      /* If the selected emphasis is off ...                         */
      /***************************************************************/
      if (pccinfo->flWindowAttr & CV_ICON)
      {
        /*************************************************************/
        /* If is in icon view, erase icon only including the         */
        /* selection emph space. (cnriconv.c)                        */
        /*************************************************************/
        EraseIconOnlyInIV(hwndCnr, hps, precord, TRUE, TRUE);
        EraseTextOnlyInIV(hwndCnr, hps, precord);
      }
      else if (pccinfo->flWindowAttr & CV_DETAIL)
      {
         /*************************************************************/
         /* If we are in Details View, erase the row.                 */
         /*************************************************************/
         if (!(CnrDrawEraseRow (hwndCnr, precord, FALSE)))
         {
            return (FALSE);
         }
      }
      else
      {
        if (!(CnrEraseRecord(hwndCnr, hps, precord)))
        {
           return (FALSE);
        }
      }

    }
    /*****************************************************************/
    /* Redraw the record (cnrcfunc.c) or row if we are in Details    */
    /* View (cnrdetv.c)                                              */
    /*****************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       if (!(CnrDrawEraseRow (hwndCnr, precord, TRUE)))
       {
          return (FALSE);
       }
    }
    else
    {
       if (!(CnrDrawRecord( hwndCnr, hps, precord, flChangeZorder)))
       {
          return (FALSE);
       }
    }
  } /* endif */

  return (TRUE);
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  xSetTimer                                        */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Set Timer                                    */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function sets timer on or off.                           */
/*     If flOn is TRUE, start timer and send WM_TIMER every          */
/*     usDuration/1000 seconds, else stop timer.                     */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd - handle of the container window.                  */
/*           flOn - flag to indicate on or off                       */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR xSetTimer(HANDLE hwnd, WORD usDuration, BOOL flOn)
{
  if (flOn)
  {
    if (!(SetTimer( hwnd, ID_TIMER, usDuration, NULL)))
    {
      return FALSE;
    }
  }
  else
  {
    if (!(KillTimer( hwnd, (WORD)ID_TIMER)))
    {
      return FALSE;
    }
  }

  return TRUE;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  WinTrackRect                                     */
/*                                                                   */
/*  Usage: Internal                                                  */
/*                                                                   */
/*  Descriptive Name:                                                */
/*                                                                   */
/*  Functional Description:                                          */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*                                                                   */
/*  Input:                                                           */
/*                                                                   */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
BOOL FAR WinTrackRect (HWND hwnd, LPTRACKINFO lpTrackInfo, HICON hcurSplit)
{
  HWND    hwndFocus = GetFocus ();
  BOOL    fTrack = TRUE;
  RECT    rcTrack, rcBoundary;
  POINT   ptCursor;
  int     xLast;
  MSG     msg;
  HDC     hdc;
  WORD    wQuitMessage;
  HBITMAP hbmHalftone;
  HBRUSH  hbrshHalftone;
  static  WORD wHalftoneBits[] = { 0xAA, 0x55, 0xAA, 0x55,
                                   0xAA, 0x55, 0xAA, 0x55 };

  if (!(hdc           = GetWindowDC (hwnd))                               ||
      !(hbmHalftone   = CreateBitmap (8, 8, 1, 1, (LPSTR) wHalftoneBits)) ||
      !(hbrshHalftone = CreatePatternBrush (hbmHalftone)))
  {
    if (hdc)          ReleaseDC (hwnd, hdc);
    if (hbmHalftone)  DeleteObject (hbmHalftone);
    return (FALSE);
  }

  /*-----------------------------------------------------------*/
  /* figure out which button will signal the end of the drag;  */
  /* if more than one button is pressed, the left takes        */
  /* precedence, followed by the right, followed by the middle */
  /*-----------------------------------------------------------*/
  wQuitMessage = (GetKeyState (VK_LBUTTON) < 0) ? WM_LBUTTONUP :
                 (GetKeyState (VK_RBUTTON) < 0) ? WM_RBUTTONUP :
                                                  WM_MBUTTONUP;

  /*------------------------------------------------------*/
  /* get the current mouse position in window coordinates */
  /* (we're overloading rcTrack here, but it saves stack) */
  /*------------------------------------------------------*/
  GetCursorPos (&ptCursor);
  GetWindowRect (hwnd, &rcTrack);
  xLast = ptCursor.x - rcTrack.left;

  /*------------------------------------------------*/
  /* make a working copy of the tracking rectangles */
  /*------------------------------------------------*/
  rcTrack    = lpTrackInfo->rclTrack;

  /*-----------------------------------------------------------------*/
  /* make sure the mouse doesn't move outside the boundary rectangle */
  /*-----------------------------------------------------------------*/
  rcBoundary = lpTrackInfo->rclBoundary;
  ClientToScreen (hwnd,  (LPPOINT) &rcBoundary);
  ClientToScreen (hwnd, ((LPPOINT) &rcBoundary) + 1);
  rcBoundary.top    = ptCursor.y;
  rcBoundary.bottom = ptCursor.y;

  SetCapture (hwnd);
  ClipCursor (&rcBoundary);
  SetCursor  (hcurSplit);

  hbrshHalftone = SelectObject (hdc, hbrshHalftone);
  DrawTrackRect (hdc, &rcTrack);

  while (fTrack &&
         (GetFocus() == hwndFocus) &&
         (GetCapture() == hwnd)    &&
         GetMessage (&msg, 0, 0, 0))
  {
    if (msg.message == wQuitMessage)  fTrack = FALSE;

    else switch (msg.message)
    {
      /*-----------------------*/
      /* eat keyboard messages */
      /*-----------------------*/
      case WM_CHAR:
      case WM_KEYUP:
      case WM_KEYDOWN:
      case WM_SYSCHAR:
      case WM_SYSKEYUP:
      case WM_SYSKEYDOWN:
        if (msg.wParam == VK_ESCAPE)  fTrack = FALSE;
        break;

#define xNow  (LOWORD (msg.lParam))
      case WM_MOUSEMOVE:
        /*-------------------*/
        /* erase, move, draw */
        /*-------------------*/
        DrawTrackRect (hdc, &rcTrack);
        OffsetRect (&rcTrack, xNow - xLast, 0);
        DrawTrackRect (hdc, &rcTrack);
        xLast = xNow;
        break;
#undef xNow

      /*---------------------------------*/
      /* eat other mouse button messages */
      /*---------------------------------*/
      case WM_LBUTTONUP:
      case WM_MBUTTONUP:
      case WM_RBUTTONUP:
      case WM_LBUTTONDOWN:
      case WM_MBUTTONDOWN:
      case WM_RBUTTONDOWN:
      case WM_LBUTTONDBLCLK:
      case WM_MBUTTONDBLCLK:
      case WM_RBUTTONDBLCLK:
        break;

      default:
        DispatchMessage (&msg);
        break;
    }
  }

  /*----------------------------------------------------------------------*/
  /* if we fell out of the loop because GetMessage returned FALSE         */
  /* (i.e. PostQuitMessage was called), put the WM_QUIT back on the queue */
  /*----------------------------------------------------------------------*/
  if (fTrack && (GetFocus() == hwndFocus) && (GetCapture() == hwnd))
  {
    PostQuitMessage (msg.wParam);
  }

  DrawTrackRect (hdc, &rcTrack);
  ClipCursor (NULL);
  ReleaseCapture ();

  DeleteObject (SelectObject (hdc, hbrshHalftone));
  DeleteObject (hbmHalftone);
  ReleaseDC (hwnd, hdc);

  /*---------------------------------------*/
  /* if the user pressed ESC, return FALSE */
  /*---------------------------------------*/
  if ((msg.message == WM_KEYDOWN) && (msg.wParam == VK_ESCAPE))
  {
    return (FALSE);
  }
  /*---------------------------------------*/
  /* otherwise return the drop coordinates */
  /*---------------------------------------*/
  else
  {
    lpTrackInfo->rclTrack = rcTrack;
    return (TRUE);
  }
}


/*---------------------------------------------------------------------------*
 * IsOverSplitbar
 *
 * Determines if a given point is over the splitbar of a container.
 *
 * History:
 * 05-Feb-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL FAR IsOverSplitbar (HWND hwnd, POINT ptMouse)
{
  RECT rcCnr, rcLeftDV, rcRightDV, rcSplit;
  HWND hwndLeftDVTitle;

  /*--------------------------------------------*/
  /* get the window rects in screen coordinates */
  /*--------------------------------------------*/
  GetWindowRect (hwnd, &rcCnr);
  GetWindowRect (GetDlgItem (hwnd, CID_RIGHTDVWND), &rcRightDV);

  if ((hwndLeftDVTitle = GetDlgItem (hwnd, CID_LEFTCOLTITLEWND)) &&
       IsWindowVisible (hwndLeftDVTitle))
    GetWindowRect (hwndLeftDVTitle, &rcLeftDV);
  else
    GetWindowRect (GetDlgItem (hwnd, CID_LEFTDVWND),  &rcLeftDV);

  /*-----------------------------------------------------*/
  /* convert the details view window rects to container- */
  /* relative coordinates, to match the mouse            */
  /*-----------------------------------------------------*/
  OffsetRect (&rcLeftDV,  -rcCnr.left, -rcCnr.top);
  OffsetRect (&rcRightDV, -rcCnr.left, -rcCnr.top);

  /*-------------------------------------------------*/
  /* set up the rectangle representing the split bar */
  /*-------------------------------------------------*/
  SetRect (&rcSplit, rcLeftDV.right, rcLeftDV.top, rcRightDV.left,
                     rcCnr.bottom - rcCnr.top);

  return (PtInRect (&rcSplit, ptMouse));
}


void FAR DrawTrackRect (HDC hdc, LPRECT lprc)
{
  PatBlt (hdc, lprc->left, lprc->top, lprc->right-lprc->left,
               lprc->bottom-lprc->top, PATINVERT);
}


/*---------------------------------------------------------------------------*
 * CnrDragIcon
 *
 * Moves an icon within the same container when in icon view.
 *
 * History:
 * 04-Feb-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL FAR CnrDragIcon (HWND hwnd, int xMouse, int yMouse)
{
  LPCNRINFON    pccinfon = (LPCNRINFON) GetWindowLong (hwnd, QWL_CONTAINER);
  LPCNRINFO     pccinfo  = PCCINFOXFROMN (pccinfon);
  LPRECORDCOREN preccnDrag;
  LPRECORDCORE  aprecc[1];
  BOOL          fDrag = TRUE,
                fNoDragging = TRUE;
  RECT          rcViewport;
  MSG           msg;
  HDC           hdc;
  HWND          hwndFocus = GetFocus ();
  int           dxIcon, dyIcon;
  WORD          fsDrag;
  POINT         ptIcon, ptCursor;
  WORD          wParam;
  DWORD         lParam;

  /*----------------------------------------------------------------*/
  /* we must meet certain conditions before we can drag in Windows: */
  /*                                                                */
  /*   1.  we must be in icon view (Not Tree Icon View)             */
  /*   2.  the drag button (right) must be pressed                  */
  /*   3.  the mouse must be over a record                          */
  /*   4.  we must be able to get a DC for the container            */
  /*                                                                */
  /* if any of these conditions are not true, punt                  */
  /*----------------------------------------------------------------*/
  if ((pccinfo->flWindowAttr & CV_TREE)                    ||
      !(pccinfo->flWindowAttr & CV_ICON)                   ||
       (GetKeyState (VK_RBUTTON) >= 0)                     ||
      !(preccnDrag = CnrOverRecord (hwnd, xMouse, yMouse)) ||
      !(hdc = GetDC (hwnd)))
  {
    return FALSE;
  }

  /*----------------------------------------------------------------*/
  /* get the coordinates of the container's viewport rectangle      */
  /* and make sure the mouse doesn't travel outside this rectangle, */
  /* since we don't support dragging between containers             */
  /*----------------------------------------------------------------*/
  pccinfon->ulFlagSet |= FL_DRAGPROGRESS;
  CnrQueryViewportRect (hwnd, (DWORD) (LPRECT) &rcViewport, CMA_WINDOW);
  ClientToScreen (hwnd,  (LPPOINT) &rcViewport);
  ClientToScreen (hwnd, ((LPPOINT) &rcViewport) + 1);
  OffsetRect (&rcViewport, -GetSystemMetrics (SM_CXICON) / 2,
                           -GetSystemMetrics (SM_CYICON) / 2);
  ClipCursor (&rcViewport);

  ShowCursor (FALSE);
  SetCapture (hwnd);

  fsDrag = (pccinfo->flWindowAttr & CA_DRAWICON) ? DDI_ICON : DDI_BITMAP;

  /*-------------------------------------------------*/
  /* move the cursor to the upper-left of the record */
  /* so the icon will be drawn in the correct place  */
  /*-------------------------------------------------*/
  GetCursorPos (&ptCursor);
  ptIcon = preccnDrag->preccX->ptlIcon;
  WorkspaceToScreen (hwnd, &ptIcon);
  SetCursorPos (ptIcon.x, ptIcon.y);

  dxIcon = ptCursor.x - ptIcon.x;
  dyIcon = ptCursor.y - ptIcon.y;

  ScreenToClient (hwnd, &ptIcon);

  /*---------------------------------*/
  /* erase the record we're dragging */
  /*---------------------------------*/
  SENDMESSAGE( hwnd, CM_ERASERECORD,
               (WPARAM)(LPSTR)preccnDrag->preccX, 0 );                  /*DAB3*/

  /*-------------------------*/
  /* show the "cursor" image */
  /*-------------------------*/
  if (!DrawDraggedIcon (hdc, preccnDrag->preccX->hptrIcon,
                        LOWORD(ptIcon.x), LOWORD(ptIcon.y),             /*DAB1*/
                        (WORD)(DDI_START | fsDrag)))
  {
    goto Err;
  }

  /*----------------------------------------------------------*/
  /* loop until the drag button is released or ESC is pressed */
  /*----------------------------------------------------------*/
  while ( fDrag                      &&
         (GetFocus()   == hwndFocus) &&
         (GetCapture() == hwnd)      &&
          GetMessage (&msg, 0, 0, 0))
  {
    switch (msg.message)
    {
      /*-----------------------*/
      /* eat keyboard messages */
      /*-----------------------*/
      case WM_CHAR:
      case WM_KEYUP:
      case WM_KEYDOWN:
      case WM_SYSCHAR:
      case WM_SYSKEYUP:
      case WM_SYSKEYDOWN:
        if (msg.wParam == VK_ESCAPE)  fDrag = FALSE;
        break;

      /*------------------*/
      /* item was dropped */
      /*------------------*/
      case WM_RBUTTONUP:
        /*----------------------------------------------------*/
        /* update the item position, in workspace coordinates */
        /*----------------------------------------------------*/
        preccnDrag->preccX->ptlIcon.x = LOWORD(msg.lParam);             /*DAB1*/
        preccnDrag->preccX->ptlIcon.y = HIWORD(msg.lParam);             /*DAB1*/
        ClientToScreen    (hwnd, &preccnDrag->preccX->ptlIcon);
        ScreenToWorkspace (hwnd, &preccnDrag->preccX->ptlIcon);

        /*-----------------------------------------------*/
        /* move the cursor to the same position it was   */
        /* before the drag, relative to the dragged icon */
        /*-----------------------------------------------*/
        GetCursorPos (&ptCursor);
        SetCursorPos (ptCursor.x + dxIcon, ptCursor.y + dyIcon);

        fDrag = FALSE;

        if (fNoDragging)
        {
          wParam = msg.wParam;
          lParam = msg.lParam;
        }
        pccinfon->ulFlagSet &= ~FL_DRAGPROGRESS;
        break;

      /*----------------------------------------*/
      /* mouse moved, update the "cursor" image */
      /*----------------------------------------*/
      case WM_MOUSEMOVE:
        DrawDraggedIcon (hdc, preccnDrag->preccX->hptrIcon,
                         LOWORD (msg.lParam), HIWORD (msg.lParam),
                         (WORD)(DDI_DRAG | fsDrag));                    /*DAB1*/
        fNoDragging=FALSE;
        break;

      /*---------------------------------*/
      /* eat other mouse button messages */
      /*---------------------------------*/
      case WM_LBUTTONUP:
      case WM_MBUTTONUP:
      case WM_LBUTTONDOWN:
      case WM_MBUTTONDOWN:
      case WM_RBUTTONDOWN:
      case WM_LBUTTONDBLCLK:
      case WM_MBUTTONDBLCLK:
      case WM_RBUTTONDBLCLK:
        break;

      default:
        DispatchMessage (&msg);
        break;
    }
  }

  /*---------------------------*/
  /* remove the "cursor" image */
  /*---------------------------*/
  DrawDraggedIcon (hdc, preccnDrag->preccX->hptrIcon, 0, 0, DDI_END);

  /*----------------------------------------------------------------------*/
  /* if we fell out of the loop because GetMessage returned FALSE         */
  /* (i.e. PostQuitMessage was called), put the WM_QUIT back on the queue */
  /*----------------------------------------------------------------------*/
  if (msg.message == WM_QUIT)
  {
    PostQuitMessage (msg.wParam);
  }

Err:
  /*--------------------------------------------*/
  /* invalidate the record we erased so we can  */
  /* see it again (perhaps in its new location) */
  /*--------------------------------------------*/
  aprecc[0] = preccnDrag->preccX;
  CnrInvalidateRecord (hwnd, (LONG) (LPSTR) aprecc, MAKELONG (1, CMA_REPOSITION));

  /*---------------------*/
  /* let go of the mouse */
  /*---------------------*/
  ClipCursor (NULL);
  ReleaseCapture ();
  ShowCursor (TRUE);
  ReleaseDC (hwnd, hdc);

  if (fNoDragging)
     SENDMESSAGE (hwnd, WM_RBUTTONUP, wParam, lParam);
  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * WorkspaceToScreen
 *
 * Converts a point from container workspace to screen coordinates.
 *
 * History:
 * 13-Feb-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void FAR WorkspaceToScreen (HWND hwnd, LPPOINT lppt)
{
  LPCNRINFO pccinfo = PCCINFOXFROMN ((LPCNRINFON) GetWindowLong (hwnd, QWL_CONTAINER));

  /*---------------------*/
  /* workspace -> window */
  /*---------------------*/
  lppt->x -= pccinfo->ptlOrigin.x;
  lppt->y -= pccinfo->ptlOrigin.y;

  /*------------------*/
  /* window -> screen */
  /*------------------*/
  ClientToScreen (hwnd, lppt);
}


/*---------------------------------------------------------------------------*
 * ScreenToWorkspace
 *
 * Converts a point from screen to container workspace coordinates.
 *
 * History:
 * 13-Feb-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void FAR ScreenToWorkspace (HWND hwnd, LPPOINT lppt)
{
  LPCNRINFO pccinfo = PCCINFOXFROMN ((LPCNRINFON) GetWindowLong (hwnd, QWL_CONTAINER));

  /*------------------*/
  /* screen -> window */
  /*------------------*/
  ScreenToClient (hwnd, lppt);

  /*---------------------*/
  /* window -> workspace */
  /*---------------------*/
  lppt->x += pccinfo->ptlOrigin.x;
  lppt->y += pccinfo->ptlOrigin.y;
}


/*---------------------------------------------------------------------------*
 * DrawDraggedIcon
 *
 * Simulates mouse cursor movement by drawing an icon where the cursor
 * should be.
 *
 * History:
 * 13-Feb-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL FAR DrawDraggedIcon (HDC hdc, HICON hIcon, WORD x, WORD y, WORD fs)
{
  static int     xLast, yLast, cx, cy;
  static HDC     hdcScreenSave, hdcWork;
  static HBITMAP hbmScreenSave, hbmWork;

  /*----------------------*/
  /* If we're starting... */
  /*----------------------*/
  if (fs & DDI_START)
  {
    /*-----------------------------------------*/
    /* Compute how big the bitmaps need to be. */
    /*-----------------------------------------*/
    if (fs & DDI_BITMAP)
    {
      BITMAP bm;

      if (GetObject ((HBITMAP) hIcon, sizeof (bm), (LPSTR) &bm))
      {
        cx = bm.bmWidth;
        cy = bm.bmHeight;
      }
      else
      {
        cx = GetSystemMetrics (SM_CXICON);
        cy = GetSystemMetrics (SM_CYICON);
      }
    }
    else
    {
      cx = GetSystemMetrics (SM_CXICON);
      cy = GetSystemMetrics (SM_CYICON);
    }

    /*-----------------------------------------------------------*/
    /* Create working bitmaps and DCs we'll need to use for the  */
    /* duration of the drag.  They'll be deleted at DDI_END time. */
    /*-----------------------------------------------------------*/
    if (!(hdcScreenSave = CreateCompatibleDC (hdc))                  ||
        !(hdcWork       = CreateCompatibleDC (hdc))                  ||
        !(hbmScreenSave = CreateCompatibleBitmap (hdc, 2*cx, 2*cy))  ||
        !(hbmWork       = CreateCompatibleBitmap (hdc, 2*cx, 2*cy)))
    {
      if (hdcScreenSave)  DeleteDC (hdcScreenSave);
      if (hdcWork)        DeleteDC (hdcWork);
      if (hbmScreenSave)  DeleteObject (hbmScreenSave);
      return (FALSE);
    }

    /*------------------------------------------------------------------*/
    /* Select the screen save and work bitmaps into their memory device */
    /* contexts and leave them there for the duration of the drag.      */
    /*------------------------------------------------------------------*/
    hbmScreenSave = SelectObject (hdcScreenSave, hbmScreenSave);
    hbmWork       = SelectObject (hdcWork, hbmWork);

    /*--------------------------------*/
    /* Save the current screen image. */
    /*--------------------------------*/
    BitBlt (hdcScreenSave, 0, 0, cx, cy, hdc, x, y, SRCCOPY);
  }

  /*----------------------*/
  /* If this is a drag... */
  /*----------------------*/
  if (fs & DDI_DRAG)
  {
    RECT  rcUnion, rcTo, rcFrom;

    /*-------------------------------------------------------*/
    /* If dragging, must calculate positions of old and new. */
    /*-------------------------------------------------------*/
    SetRect (&rcFrom, xLast, yLast, xLast+cx, yLast+cy);
    SetRect (&rcTo,   x, y, x+cx, y+cy);

    /*-----------------------------------------------------*/
    /* If "from" and "to" overlap, perform special update. */
    /*-----------------------------------------------------*/
    if (IntersectRect (&rcUnion, &rcFrom, &rcTo))
    {
      int  xTo, yTo, xFrom, yFrom;

      /*-------------------------------------------------------*/
      /* Calculate the union of the "from" and "to" rectangles */
      /*-------------------------------------------------------*/
      UnionRect (&rcUnion, &rcFrom, &rcTo);

      /*---------------------------------------*/
      /* Normalize the from and to coordinates */
      /* with respect to the union rectangle.  */
      /*---------------------------------------*/
      xFrom = xLast - rcUnion.left;   xTo = x - rcUnion.left;
      yFrom = yLast - rcUnion.top;    yTo = y - rcUnion.top;

      /*-------------------------------------------------------*/
      /* Copy the screen around the "from" and "to" positions. */
      /*-------------------------------------------------------*/
      BitBlt (hdcWork, 0, 0, rcUnion.right - rcUnion.left,
              rcUnion.bottom - rcUnion.top, hdc, rcUnion.left, rcUnion.top,
              SRCCOPY);

      /*----------------------------------------------*/
      /* Restore the saved "from" part of the screen. */
      /*----------------------------------------------*/
      BitBlt (hdcWork, xFrom, yFrom, cx, cy, hdcScreenSave, 0, 0, SRCCOPY);

      /*--------------------------------------*/
      /* Save the "to" portion of the screen. */
      /*--------------------------------------*/
      BitBlt (hdcScreenSave, 0, 0, cx, cy, hdcWork,
              rcTo.left - rcUnion.left, rcTo.top - rcUnion.top, SRCCOPY);

      /*-----------------------------------*/
      /* Draw the icon into the work image */
      /*-----------------------------------*/
      if (fs & DDI_BITMAP)
      {
        hIcon = SelectObject (hdcScreenSave, hIcon);
        BitBlt (hdcWork, xTo, yTo, cx, cy, hdcScreenSave, 0, 0, SRCCOPY);
        SelectObject (hdcScreenSave, hIcon);
      }
      else
      {
        DrawIcon (hdcWork, xTo, yTo, hIcon);
      }

      /*-----------------------------------*/
      /* Copy the new image to the screen. */
      /*-----------------------------------*/
      BitBlt (hdc, rcUnion.left, rcUnion.top, rcUnion.right - rcUnion.left,
              rcUnion.bottom - rcUnion.top, hdcWork, 0, 0, SRCCOPY);
    }

    /*------------------------------------------------------*/
    /* Otherwise, the source and destination don't overlap. */
    /*------------------------------------------------------*/
    else
    {
      /*---------------------------------------------------------*/
      /* Restore the screen to its original look, save the area  */
      /* about to be overwritten, and clear out the DDI_DRAG flag */
      /* so we'll fall into the normal drawing code below.       */
      /*---------------------------------------------------------*/
      BitBlt (hdc, xLast, yLast, cx, cy, hdcScreenSave, 0, 0, SRCCOPY);
      BitBlt (hdcScreenSave, 0, 0, cx, cy, hdc, x, y, SRCCOPY);

      fs &= ~DDI_DRAG;
    }
  }

  /*--------------------*/
  /* If we're ending... */
  /*--------------------*/
  if (fs & DDI_END)
  {
    /*-------------------------------------------*/
    /* Restore the screen to its original state. */
    /*-------------------------------------------*/
    BitBlt (hdc, xLast, yLast, cx, cy, hdcScreenSave, 0, 0, SRCCOPY);

    /*---------------------------------*/
    /* Clean up all these GDI objects. */
    /*---------------------------------*/
    DeleteObject (SelectObject (hdcScreenSave, hbmScreenSave));
    DeleteObject (SelectObject (hdcWork, hbmWork));
    DeleteDC (hdcScreenSave);
    DeleteDC (hdcWork);
    return (TRUE);
  }

  /*------------------------------------------------------*/
  /* If not drag (either start or non overlapped drag)... */
  /*------------------------------------------------------*/
  if (!(fs & DDI_DRAG))
  {
    if (fs & DDI_BITMAP)
    {
      hIcon = SelectObject (hdcScreenSave, hIcon);
      BitBlt (hdc, x, y, cx, cy, hdcScreenSave, 0, 0, SRCCOPY);
      SelectObject (hdcScreenSave, hIcon);
    }
    else
    {
      DrawIcon (hdc, x, y, hIcon);
    }
  }

  /*------------------------------------------------------*/
  /* Update the static variables with their new settings. */
  /*------------------------------------------------------*/
  xLast = x;
  yLast = y;

  return (TRUE);
}
