/* Revision: 20 1.2.2.2 source/ui/cclnt/cnr/cnrs3p3.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS3P3.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS3P3.C - Assorted draw routines for container control */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/**********************************************************************/
#include <windows.h>
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS3P3_C
#include "cnrpragw.h"
#endif


/*********************************************************************/
/*                                                                   */
/*   FUNCTION NAME:  EraseIconOnlyInIV                               */
/*                                                                   */
/*   Usage: External                                                 */
/*                                                                   */
/*   DESCRIPTIVE NAME:  Container Control Erase Icon                 */
/*                                                                   */
/*   FUNCTIONAL DESCRIPTION: This function erase an item in the      */
/*      icon view. If flag flRedraw is TRUE, redraw all the items    */
/*      which are overlapped with the erased item.                   */
/*                                                                   */
/*   NOTES:                                                          */
/*      FORWARD DECLARATION:                                         */
/*                                                                   */
/*   INPUT:  HANDLE hwndCnr - handle to the container window         */
/*           LPRECORDCORE pRecord - pointer to the record structure  */
/*           BOOL flRedraw - flag to tell to redraw the item         */
/*                           which were overlapped with erased item  */
/*                                                                   */
/*   OUTPUT:                                                         */
/*                                                                   */
/*                                                                   */
/*   EXIT-NORMAL:                                                    */
/*                                                                   */
/*   EXIT-ERROR:                                                     */
/*                                                                   */
/*   SIDE EFFECTS:                                                   */
/*                                                                   */
/*********************************************************************/
VOID FAR EraseIconOnlyInIV (hwndCnr, hps, precord,
                            flRedraw, flInclEmph)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precord;
BOOL          flRedraw;
BOOL          flInclEmph;
{
  HRGN           regionHdl = NULL; /* handle of the new clipping */
                                   /* region */
  LPCNRINFON     pccinfon;
  LPCNRINFO      pccinfo;
  LPRECORDCOREN  prcZ;             /* record to be compared */
  RECT           iconRect;         /* rectangle of the icon */
  RECT           textRect;         /* rectangle of the text */
  RECT           arcl[2];          /* array of the rectangles of the */
                                   /* erased item */
  RECT           rclDest;          /* intersection of the rectangles */
  RECT           windowRect;       /* rectangle of the window */
  POINT          iconSize;         /* size of the icon */
  LPMARQUEEREC   pmrMarquee;
  int            nSavedDC;


  PMGetClientRect(hwndCnr, &windowRect);
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size.                                                  */
  /*******************************************************************/
  CnrCalcIconSize(hwndCnr, &iconSize);

  /*******************************************************************/
  /* Erase the record if it is in the viewport.                      */
  /*******************************************************************/
  if (IsInIconViewport (precord, &(pccinfo->ptlOrigin),
                        &windowRect, &iconSize))
  {
    /*****************************************************************/
    /* Calculate icon rectangle including cursored emph space.       */
    /*****************************************************************/
    CnrQueryIconTextRect(hwndCnr, precord, &iconSize,
                         TRUE, &iconRect, NULL);

    if (!flInclEmph)
    {
      /***************************************************************/
      /* reduce the rectangle by 2 pixels from each dimension.       */
      /***************************************************************/
      iconRect.left   += ICN_SELSPACE;
      iconRect.right  -= ICN_SELSPACE;
      iconRect.top    += ICN_SELSPACE;
      iconRect.bottom -= ICN_SELSPACE;
    }
    /*****************************************************************/
    /* Save the rectangles to be erased in the array.                */
    /*****************************************************************/
    arcl[0] = iconRect;
    nSavedDC = SaveDC (hps);

    /*****************************************************************/
    /* Create the clipping region. Save old region as oldRegionHdl   */
    /*****************************************************************/
    if (regionHdl = CreateRectRgn(arcl[0].left, arcl[0].top,
                              arcl[0].right, arcl[0].bottom))
    {
      SelectClipRgn (hps, regionHdl);
    }

    CnrEraseRect(hwndCnr, hps, iconRect);

    /*****************************************************************/
    /* If flRedraw is TRUE, redraw the overlapped items.             */
    /*****************************************************************/
    if ((flRedraw) &&
        !(GetWindowLong(hwndCnr, GWL_STYLE) & CCS_AUTOPOSITION))
    {
      /***************************************************************/
      /* Set prcZ as the bottommost item.                            */
      /***************************************************************/
      if (!pccinfon->fFirstMouseMove)
      {
        pmrMarquee = pccinfon->pmrLastMarquee;
        prcZ = pmrMarquee->pRecord;
      }
      else
      {
        prcZ = pccinfon->preccZ;
      }

      /***************************************************************/
      /* Redraw the overlapped items from top to bottom.             */
      /***************************************************************/
      while (prcZ)
      {
        /*************************************************************/
        /* If the item to be checked is the item erased, we don't    */
        /* draw again.                                               */
        /*************************************************************/
        if (prcZ != precord)
        {
          CnrQueryIconTextRect(hwndCnr, prcZ, &iconSize,
                               TRUE, &iconRect, &textRect);

          /***********************************************************/
          /* If the item checked was overlapped, redraw it and       */
          /* it's text and icon rectangle from clipping region so    */
          /* that the item under it is not drawn over it.            */
          /***********************************************************/
          if (PMIntersectRect( &rclDest, &arcl[0], &textRect) ||
              PMIntersectRect( &rclDest, &arcl[0], &iconRect) )
          {
            xDrawIcon (hwndCnr, hps, prcZ, &iconSize);
          }
        }

        if (!pccinfon->fFirstMouseMove)
        {
          pmrMarquee = pmrMarquee->pmrPrevItem;
          if (pmrMarquee)
          {
            prcZ = pmrMarquee->pRecord;
          }
          else
          {
            prcZ = NULL;
          }
        }
        else
        {
          prcZ = prcZ->preccNextZ;
        }
      } /* endwhile */
    } /* endif */

    /*****************************************************************/
    /* Set the previous region and destroy the current clipping      */
    /* region.                                                       */
    /*****************************************************************/
    if (nSavedDC)
      RestoreDC (hps, nSavedDC);

    if (regionHdl)
      DeleteObject(regionHdl);
  } /* endif */

  return;
}

/*********************************************************************/
/*                                                                   */
/*   FUNCTION NAME:  EraseTextOnlyInIV                               */
/*                                                                   */
/*   Usage: External                                                 */
/*                                                                   */
/*   DESCRIPTIVE NAME:  Container Control Erase Text                 */
/*                                                                   */
/*   FUNCTIONAL DESCRIPTION: This function erase an item in the      */
/*      icon view. If flag flRedraw is TRUE, redraw all the items    */
/*      which are overlapped with the erased item.                   */
/*                                                                   */
/*   NOTES:                                                          */
/*      FORWARD DECLARATION:                                         */
/*                                                                   */
/*   INPUT:  HANDLE hwndCnr - handle to the container window         */
/*           LPRECORDCORE pRecord - pointer to the record structure   */
/*           BOOL flRedraw - flag to tell to redraw the item         */
/*                           which were overlapped with erased item  */
/*                                                                   */
/*   OUTPUT:                                                         */
/*                                                                   */
/*                                                                   */
/*   EXIT-NORMAL:                                                    */
/*                                                                   */
/*   EXIT-ERROR:                                                     */
/*                                                                   */
/*   SIDE EFFECTS:                                                   */
/*                                                                   */
/*********************************************************************/
VOID FAR  EraseTextOnlyInIV( hwndCnr, hps, precord)
HANDLE         hwndCnr;
HDC          hps;
LPRECORDCOREN precord;
{
  LPCNRINFON      pccinfon;
  LPCNRINFO       pccinfo;
  RECT           textRect;         /* rectangle of the text */
  RECT           windowRect;       /* rectangle of the window */
  POINT          iconSize;         /* size of the icon */

  PMGetClientRect(hwndCnr, &windowRect);
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Get icon size.                                                  */
  /*******************************************************************/
  CnrCalcIconSize(hwndCnr, &iconSize);

  /*******************************************************************/
  /* Erase the record if it is in the viewport.                      */
  /*******************************************************************/
  if (IsInIconViewport(precord, &(pccinfo->ptlOrigin),
                       &windowRect, &iconSize))
  {
    /*****************************************************************/
    /* Calculate icon rectangle including cursored emph space.       */
    /*****************************************************************/
    CnrQueryIconTextRect(hwndCnr, precord, &iconSize,
                         TRUE, NULL, &textRect);

    textRect.right++;

    CnrEraseRect(hwndCnr, hps, textRect);
  }

  return;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  BlinkCursoredEmph                                */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Blink Cursored Emph                          */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function draws or erase cursored emphasis depending      */
/*     on flOn (if flOn = TRUE, drawit, else, erase it) around       */
/*     the given record preccObject.                                 */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd - handle of the container window.                  */
/*           preccObject - pointer to the record                     */
/*           flOn - flag to indicate on or off                       */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR BlinkCursoredEmph(hwnd, hps, preccObject, flOn)
HANDLE       hwnd;
HDC          hps;
LPRECORDCOREN preccObject;
BOOL         flOn;
{

  if (flOn)
  {
    DrawBlinkEmphBox(hwnd, hps, preccObject, IC_CURSORED);
  }
  else
  {
    DrawBlinkEmphBox(hwnd, hps, preccObject, 0L);
  }
  return;
}

/*********************************************************************/
/* FUNCTION NAME:  CnrDrawEmphChange                                 */
/*                                                                   */
/* DESCRIPTIVE NAME: Draw Record With Emph Changed                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function draws the record of which  */
/*                          the emphasis changed.                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC  hps - handle of the presentation space               */
/*         LPRECORDCORE precord - pointer to the record structure     */
/*         DWORD flRecordAttrOld - the old attribute.                */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR CnrDrawEmphChange(hwndCnr, hps, precordn, flRecordAttrOld)
HANDLE        hwndCnr;
HDC           hps;
LPRECORDCOREN precordn;
DWORD         flRecordAttrOld;
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
  POINT      iconSize;
  RECT       rctlIcon;
  RECT       rctlIconCur;
  RECT       rctlIconSel;
  RECT       rctlText;
  RECT       rctlTextCur;
  RECT       RowRectLeft;
  RECT       RowRectRight;
  HDC        hpsLeft;
  HDC        hpsRight;

  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    return (CnrTRVDrawEmphChange(hwndCnr, hps, precordn, flRecordAttrOld));
  }

  if (precordn && ISRECORDVISIBLE(precordn))
  {
    /******************************************************************/
    /* If we are in Details View, calculate the Row Rectangle for     */
    /* left window, and if a right window exists, calculate the row   */
    /* rectangle for it too.                                          */
    /******************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
      /****************************************************************/
      /* If there is no data inserted, just return.                   */
      /****************************************************************/
      if (!(pccinfon->pFirstVisibleFI))
      {
        return (TRUE);
      }

      hpsLeft = PMGetFlippedDC (GetDlgItem (hwndCnr, CID_LEFTDVWND));
      CnrCalcRowRect (hwndCnr, precordn, TRUE, &RowRectLeft);
      RowRectLeft.bottom --;
      RowRectLeft.top++;
      RowRectLeft.right += 2;

      if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
      {
        hpsRight = PMGetFlippedDC (GetDlgItem (hwndCnr, CID_RIGHTDVWND));
        CnrCalcRowRect (hwndCnr, precordn, FALSE, &RowRectRight);
        RowRectRight.bottom --;
        RowRectRight.top++;
        RowRectRight.left  -= 2;
        RowRectRight.right += 2;
      }
    }
    else
    {
      CnrCalcIconSize(hwndCnr, &iconSize);
      CnrQueryIconTextRect(hwndCnr, precordn, &iconSize,
                           FALSE, &rctlIcon, &rctlText);

      rctlTextCur.left   = rctlText.left - TXT_TOPMARGIN;
      rctlTextCur.right  = rctlText.right + TXT_TOPMARGIN;
      rctlTextCur.top = rctlText.top - TXT_TOPMARGIN;
      rctlTextCur.bottom = rctlText.bottom + TXT_TOPMARGIN;

      rctlIconCur.left  = rctlIcon.left - ICN_EMPHSPACE;
      rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
      rctlIconCur.top    = rctlIcon.top - ICN_EMPHSPACE +
                           ICN_SIDEMARGIN;
      rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;

      rctlIconSel.left  = rctlIcon.left - ICN_SELSPACE;
      rctlIconSel.right = rctlIcon.right + ICN_SELSPACE -
                          ICN_SIDEMARGIN;
      rctlIconSel.top    = rctlIcon.top - ICN_SELSPACE +
                           ICN_SIDEMARGIN;
      rctlIconSel.bottom = rctlIcon.bottom + ICN_SELSPACE;
    }

    /******************************************************************/
    /* If previously, the icon has no cursored or selected emphasis   */
    /******************************************************************/
    if (!(flRecordAttrOld & CRA_CURSORED) &&
        !(flRecordAttrOld & CRA_SELECTED))
    {
      /****************************************************************/
      /* If the record is to now get cursored emphasis                */
      /****************************************************************/
      if ((precordn->flRecordAttr & CRA_CURSORED) &&
          !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_CURSOR,
                           rctlIcon, rctlIconCur);

        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_CURSOR,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_CURSOR,
                             rctlIcon, rctlIconCur);
          }
          else
          {
            CnrEraseRect(hwndCnr, hps, rctlTextCur);
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to now get selected emphasis      */
      /****************************************************************/
      else if (!(precordn->flRecordAttr & CRA_CURSORED) &&
                (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to now get both selected/cursored */
      /****************************************************************/
      else if ((precordn->flRecordAttr & CRA_CURSORED) &&
               (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON | DRAW_CURSOR,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT | DRAW_CURSOR,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON | DRAW_CURSOR,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          /****************************************************/
          /* If Details View has a right window, we must      */
          /* draw the cursor for it too.                      */
          /****************************************************/
          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
    }
    /******************************************************************/
    /* If previously, the icon has cursored but no selected emphasis  */
    /******************************************************************/
    else if ((flRecordAttrOld & CRA_CURSORED) &&
             !(flRecordAttrOld & CRA_SELECTED))
    {
      /****************************************************************/
      /* If the record is to now have no cursored and no selected emph*/
      /****************************************************************/
      if (!(precordn->flRecordAttr & CRA_CURSORED) &&
          !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRect(hwndCnr, hps, rctlIconCur);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
          }
          else
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlIcon, rctlIconCur);
          }
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRecord( hwndCnr, hps, precordn);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlText, rctlTextCur);
          }
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (!(pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND) &&
              (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                     CCS_AUTOPOSITION))
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlIcon, rctlIconCur);

          }
          else
          {
            if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                     CCS_AUTOPOSITION)
            {
              EraseIconOnlyInIV( hwndCnr, hps, precordn, FALSE,
                                 TRUE);
              CnrDrawIconOnly( hwndCnr, hps, precordn,
                               DRAW_ICON,
                               rctlIcon, rctlIconCur);
            }
            else
            {
              EraseIconOnlyInIV( hwndCnr, hps, precordn, TRUE,
                                 TRUE);
              CnrEraseRect(hwndCnr, hps, rctlTextCur);
              CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
            }
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            /****************************************************/
            /* Erase the row, then redraw it.                   */
            /****************************************************/
            if (!(CnrDrawEraseRow (hwndCnr, precordn, FALSE)))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

              if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
              {
                ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
              }

              return (FALSE);
            }

            if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

              if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
              {
                ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
              }

              return (FALSE);
            }
          }
          else
          {
            /****************************************************/
            /* Erase the cursor using background colors.        */
            /****************************************************/
            CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                              RowRectLeft, FALSE);

            CnrDrawVertSep (hwndCnr, hpsLeft, TRUE, FALSE);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                                RowRectRight, FALSE);

              if (CnrVerifyFieldInfoPtrs (hwndCnr,
                                     pccinfon->pFieldInfoLast, 1))
              {
                 if (NEXTVISIBLEFI(pccinfon->pFieldInfoLast))
                 {
                    CnrDrawVertSep (hwndCnr, hpsRight,
                                    FALSE, FALSE);
                 }
              }
            }
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to now have both cursored/selected*/
      /****************************************************************/
      else if ((precordn->flRecordAttr & CRA_CURSORED) &&
               (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          /****************************************************/
          /* If Details View has a right window, we must      */
          /* draw the cursor for it too.                      */
          /****************************************************/
          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to have selected but no cursored  */
      /****************************************************************/
      else if (!(precordn->flRecordAttr & CRA_CURSORED) &&
                (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRect(hwndCnr, hps, rctlIconCur);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
          }
          else
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON | ERASE_CURSOR,
                             rctlIcon, rctlIconCur);
          }
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRecord( hwndCnr, hps, precordn);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT | ERASE_CURSOR,
                             rctlText, rctlTextCur);
          }
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (!(pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND) &&
              (GetWindowLong(hwndCnr, GWL_STYLE) &
                                     CCS_AUTOPOSITION))
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON | ERASE_CURSOR,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            if (GetWindowLong(hwndCnr, GWL_STYLE ) &
                                    CCS_AUTOPOSITION)
            {
              EraseIconOnlyInIV(hwndCnr, hps, precordn, FALSE,
                                TRUE);
              CnrDrawIconOnly( hwndCnr, hps, precordn,
                               DRAW_ICON,
                               rctlIcon, rctlIconCur);
              CnrDrawTextOnly( hwndCnr, hps, precordn,
                               DRAW_TEXT,
                               rctlText, rctlTextCur);
            }
            else
            {
              EraseIconOnlyInIV( hwndCnr, hps, precordn, TRUE,
                                 TRUE);
              CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
            }
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }
        }
      }
    }
    /******************************************************************/
    /* If previously, the icon has cursored but no selected emphasis  */
    /******************************************************************/
    else if (!(flRecordAttrOld & CRA_CURSORED) &&
          (flRecordAttrOld & CRA_SELECTED))
    {
      /****************************************************************/
      /* If the record is to now have no cursored and no selected emph*/
      /****************************************************************/
      if (!(precordn->flRecordAttr & CRA_CURSORED) &&
          !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrEraseRect(hwndCnr, hps, rctlIconSel);
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn, FALSE,
                               FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn, TRUE,
                               FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          /****************************************************/
          /* Erase the row.                                   */
          /****************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, FALSE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /****************************************************/
          /* Draw the row.                                    */
          /****************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to now have both cursored/selected*/
      /****************************************************************/
      else if ((precordn->flRecordAttr & CRA_CURSORED) &&
               (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_CURSOR,
                           rctlIcon, rctlIconCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_CURSOR,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_CURSOR,
                             rctlIcon, rctlIconCur);
          }
          else
          {
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to have cursored but no selected  */
      /****************************************************************/
      else if ((precordn->flRecordAttr & CRA_CURSORED) &&
               !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrEraseRect(hwndCnr, hps, rctlIconSel);
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON | DRAW_CURSOR,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT | DRAW_CURSOR,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            EraseIconOnlyInIV(hwndCnr, hps, precordn,
                              FALSE, FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON | DRAW_CURSOR,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            EraseIconOnlyInIV(hwndCnr, hps, precordn, TRUE,
                              FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawRecord(hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          /************************************************/
          /* Erase the row.                               */
          /************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, FALSE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /************************************************/
          /* Draw the row.                                */
          /************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /************************************************/
          /* Draw the cursor.                             */
          /************************************************/
          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
    }
    /******************************************************************/
    /* If previously, the icon has both cursored and selected emphasis*/
    /******************************************************************/
    else if ((flRecordAttrOld & CRA_CURSORED) &&
             (flRecordAttrOld & CRA_SELECTED))
    {
      /****************************************************************/
      /* If the record is to have cursored but no selected emphasis   */
      /****************************************************************/
      if ((precordn->flRecordAttr & CRA_CURSORED) &&
          !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrEraseRect(hwndCnr, hps, rctlIconSel);
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT | DRAW_CURSOR,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn, FALSE,
                               FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn, TRUE,
                               FALSE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          /**************************************************/
          /* Erase the row.                                 */
          /**************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, FALSE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /**************************************************/
          /* Draw the row.                                  */
          /**************************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /**************************************************/
          /* Draw the cursor.                               */
          /**************************************************/
          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, TRUE);

          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, TRUE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to have selected but no cursored  */
      /****************************************************************/
      else if (!(precordn->flRecordAttr & CRA_CURSORED) &&
                (precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRect(hwndCnr, hps, rctlIconCur);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
          }
          else
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlIcon, rctlIconCur);
          }
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
          {
            CnrEraseRecord(hwndCnr, hps, precordn);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlText, rctlTextCur);
          }
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (!(pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND) &&
              (GetWindowLong(hwndCnr, GWL_STYLE) &
                                     CCS_AUTOPOSITION))
          {
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             ERASE_CURSOR,
                             rctlIcon, rctlIconCur);

          }
          else
          {
            if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                     CCS_AUTOPOSITION)
            {
              EraseIconOnlyInIV( hwndCnr, hps, precordn,
                                 FALSE, TRUE);
              CnrDrawIconOnly( hwndCnr, hps, precordn,
                               DRAW_ICON,
                               rctlIcon, rctlIconCur);
            }
            else
            {
              EraseIconOnlyInIV(hwndCnr, hps, precordn,
                                TRUE, TRUE);
              CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
            }
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          CnrDrawCursorBox (hwndCnr, hpsLeft, precordn,
                            RowRectLeft, FALSE);

          if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
          {
            CnrDrawCursorBox (hwndCnr, hpsRight, precordn,
                              RowRectRight, FALSE);
          }
        }
      }
      /****************************************************************/
      /* otherwise if the record is to have no selected/cursored emph */
      /****************************************************************/
      else if (!(precordn->flRecordAttr & CRA_CURSORED) &&
               !(precordn->flRecordAttr & CRA_SELECTED))
      {
        if (pccinfo->flWindowAttr & CV_NAME)
        {
          CnrEraseRect(hwndCnr, hps, rctlIconCur);
          CnrEraseRect(hwndCnr, hps, rctlText);
          CnrDrawIconOnly( hwndCnr, hps, precordn,
                           DRAW_ICON,
                           rctlIcon, rctlIconCur);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrEraseRecord( hwndCnr, hps, precordn);
          CnrDrawTextOnly( hwndCnr, hps, precordn,
                           DRAW_TEXT,
                           rctlText, rctlTextCur);
        }
        else if (pccinfo->flWindowAttr & CV_ICON)
        {
          if (GetWindowLong( hwndCnr, GWL_STYLE ) &
                                   CCS_AUTOPOSITION)
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn, FALSE,
                               TRUE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawIconOnly( hwndCnr, hps, precordn,
                             DRAW_ICON,
                             rctlIcon, rctlIconCur);
            CnrDrawTextOnly( hwndCnr, hps, precordn,
                             DRAW_TEXT,
                             rctlText, rctlTextCur);
          }
          else
          {
            EraseIconOnlyInIV( hwndCnr, hps, precordn,
                               TRUE, TRUE);
            CnrEraseRect(hwndCnr, hps, rctlText);
            CnrDrawRecord( hwndCnr, hps, precordn, TRUE);
          }
        }
        else if (pccinfo->flWindowAttr & CV_DETAIL)
        {
          /**********************************************/
          /* Erase the row.                             */
          /**********************************************/
          if (!(CnrDrawEraseRow (hwndCnr,
                                 precordn,
                                 FALSE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }

          /**********************************************/
          /* Draw the row.                              */
          /**********************************************/
          if (!(CnrDrawEraseRow (hwndCnr, precordn, TRUE)))
          {
            ReleaseDC(GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

            if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
            {
              ReleaseDC(GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
            }

            return (FALSE);
          }
        }
      }
    }
  }

  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     ReleaseDC (GetDlgItem (hwndCnr, CID_LEFTDVWND), hpsLeft);

     if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
     {
        ReleaseDC (GetDlgItem (hwndCnr, CID_RIGHTDVWND), hpsRight);
     }
  }

  return (TRUE);
}

/*********************************************************************/
/* FUNCTION NAME:  CnrTRVDrawEmphChange                              */
/*                                                                   */
/* DESCRIPTIVE NAME: Draw Record With Emph Changed                   */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function draws the record of which  */
/*                          the emphasis changed.                    */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* INPUT:  HANDLE hwndCnr - handle to the container window           */
/*         HDC  hps - handle of the presentation space               */
/*         LPRECORDCORE precord - pointer to the record structure     */
/*         DWORD flRecordAttrOld - the old attribute.                */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*********************************************************************/
BOOL FAR  CnrTRVDrawEmphChange(hwnd, hps, preccnChange,
                               flRecordAttrOld)
HANDLE          hwnd;
HDC           hps;
LPRECORDCOREN  preccnChange;
DWORD         flRecordAttrOld;
{
  LPCNRINFON   pccinfon;
  LPCNRINFO    pccinfo;
  POINT       slIcon;
  POINT       slTreeIcon;
  RECT        rctlIcon;
  RECT        rctlTreeIcon;
  RECT        rctlIconCur;
  RECT        rctlText;
  RECT        rctlTextCur;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (preccnChange && ISRECORDVISIBLE(preccnChange))
  {
    CnrCalcIconSize( hwnd, &slIcon );
    CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);
    CnrTRVQueryIconTextRect(hwnd, preccnChange, FALSE,
                            slIcon, slTreeIcon,
                            &rctlIcon, &rctlTreeIcon, &rctlText);

    rctlTextCur.left   = rctlText.left - TXT_TOPMARGIN;
    rctlTextCur.right  = rctlText.right + TXT_TOPMARGIN;
//  rctlTextCur.bottom = rctlText.bottom - TXT_TOPMARGIN;
    rctlTextCur.bottom = rctlText.bottom + TXT_TOPMARGIN;        //FLIP - MAB
//  rctlTextCur.top    = rctlText.top;
    rctlTextCur.top    = rctlText.top - TXT_TOPMARGIN;                  //@B02

    rctlIconCur.left  = rctlIcon.left - ICN_EMPHSPACE;
    rctlIconCur.right = rctlIcon.right + ICN_EMPHSPACE;
//                    - ICN_SIDEMARGIN;
//  rctlIconCur.bottom = rctlIcon.bottom - ICN_EMPHSPACE;
//  rctlIconCur.top    = rctlIcon.top + ICN_EMPHSPACE -
//                       ICN_SIDEMARGIN;
    rctlIconCur.bottom = rctlIcon.bottom + ICN_EMPHSPACE;        //FLIP - MAB
    rctlIconCur.top    = rctlIcon.top - ICN_EMPHSPACE +    //FLIP - MAB //@B01
                         ICN_SIDEMARGIN;

    if (!(flRecordAttrOld & CRA_CURSORED) &&
        !(flRecordAttrOld & CRA_SELECTED))
    {
      if ((preccnChange->flRecordAttr & CRA_CURSORED) &&
          (preccnChange->flRecordAttr & CRA_SELECTED))
      {
        CnrTRVDrawItem(hwnd, hps, preccnChange,
                       rctlIcon, rctlTreeIcon, rctlText,
                       DE_ICON | DE_TEXT);
      }

    }
    else
    {
      if (!(flRecordAttrOld & CRA_CURSORED) &&
          (flRecordAttrOld & CRA_SELECTED))
      {
        if ((preccnChange->flRecordAttr & CRA_CURSORED) &&
            (preccnChange->flRecordAttr & CRA_SELECTED))
        {
          if (pccinfo->flWindowAttr & CV_TEXT)
          {
            CnrDrawCursorBox(hwnd, hps, preccnChange,
                             rctlTextCur, TRUE);
          }
          else
          {
            CnrDrawCursorBox(hwnd, hps, preccnChange,
                             rctlIconCur, TRUE);
          }
        }
      }
      else
      {
        if ((flRecordAttrOld & CRA_CURSORED) &&
            (flRecordAttrOld & CRA_SELECTED))
        {
          if (!(preccnChange->flRecordAttr & CRA_CURSORED) &&
              (preccnChange->flRecordAttr & CRA_SELECTED))
          {
            if (pccinfo->flWindowAttr & CV_TEXT)
            {
              if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
              {
                CnrTRVEraseItem(hwnd, hps, preccnChange,
                                rctlIcon, rctlTreeIcon, rctlText,
                                DE_TEXT);
                CnrTRVDrawItem(hwnd, hps, preccnChange,
                               rctlIcon, rctlTreeIcon, rctlText,
                               DE_TEXT);
              }
              else
              {
                CnrDrawCursorBox(hwnd, hps, preccnChange,
                                 rctlTextCur, FALSE);
              }
            }
            else
            {
              if (pccinfo->flWindowAttr & CA_OWNERPAINTBACKGROUND)
              {
                CnrTRVEraseItem(hwnd, hps, preccnChange,
                                rctlIcon, rctlTreeIcon, rctlText,
                                DE_ICON);
                CnrTRVDrawItem(hwnd, hps, preccnChange,
                               rctlIcon, rctlTreeIcon, rctlText,
                               DE_ICON);
              }
              else
              {
                CnrDrawCursorBox(hwnd, hps, preccnChange,
                                 rctlIconCur, FALSE);
              }
            }
          }
          else
          {
            if (!(preccnChange->flRecordAttr & CRA_CURSORED) &&
                !(preccnChange->flRecordAttr & CRA_SELECTED))
            {
              CnrTRVEraseItem(hwnd, hps, preccnChange,
                              rctlIcon, rctlTreeIcon, rctlText,
                              DE_ICON | DE_TEXT);
              CnrTRVDrawItem(hwnd, hps, preccnChange,
                             rctlIcon, rctlTreeIcon, rctlText,
                             DE_ICON | DE_TEXT);
            }
          }
        }
      }
    }
  }
  return TRUE;
}
#ifdef DRAG_DONE
/*********************************************************************/
/*                                                                   */
/*  Function Name:     CnrDrawRecTargetEmph                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Control Draw a Record   Target Emph          */
/*                                                                   */
/*  Functional Description: This function draws a box around         */
/*     a record.                                                     */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:  long CnrDrawTargetEmph(                 */
/*                               HANDLE       hwnd                   */
/*                               LPRECORDCORE precord                 */
/*                               BOOL       flEmphOn                 */
/*                               )                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*           precord - record on which a target emph is drawn/erased */
/*           flTargetOn - flag which indicates emph on or off        */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrDrawRecTargetEmph(hwnd, hps, precord)
HANDLE         hwnd;
HDC          hps;
LPRECORDCOREN precord;
{
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;
//POINT      ptlPos;
  RECT       rctlTarget;
  POINT      iconSize;
  RECT       RowRectLeft;
  RECT       RowRectRight;
  HDC        hpsLeft;
  HDC        hpsRight;
  HPEN       hPenOld;
  HPEN       hPenCurrent;
  HBRUSH     hBrushOld;

  if (precord && ISRECORDVISIBLE(precord))
  {
    pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
    pccinfo  = PCCINFOXFROMN(pccinfon);

    if (!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_DETAIL))
    {
       /***************************************************************/
       /* Since we always have a left window, calculate the row       */
       /* rectangle for the left window.                              */
       /***************************************************************/
       CnrCalcRowRect (hwnd, precord, TRUE, &RowRectLeft);
       RowRectLeft.top--;

//     hpsLeft = DrgGetPS (GetDlgItem (hwnd, CID_LEFTDVWND));
       hpsLeft = PMGetFlippedDC (GetDlgItem (hwnd, CID_LEFTDVWND));

       /***************************************************************/
       /* If the Details View window is split in 2, we calculate the  */
       /* Row rectangle for right window too.                         */
       /***************************************************************/
       if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
       {
          CnrCalcRowRect (hwnd, precord, FALSE, &RowRectRight);
          RowRectRight.left--;
          RowRectRight.top--;

//        hpsRight = DrgGetPS (GetDlgItem (hwnd, CID_RIGHTDVWND));
          hpsRight = PMGetFlippedDC (GetDlgItem (hwnd, CID_RIGHTDVWND));
       }

       /***************************************************************/
       /* Save the previous hps attributes.                           */
       /***************************************************************/
       SaveDC (hpsLeft);

       /***************************************************************/
       /* Set the color for the lines of the box                      */
       /***************************************************************/
       SetROP2 (hpsLeft, R2_NOT);

       /***************************************************************/
       /* Draw the box from the current pos to the top right pos      */
       /***************************************************************/
       hBrushOld   = SelectObject (hpsLeft, GetStockObject(NULL_BRUSH));
       hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
       hPenOld     = SelectObject (hpsLeft, hPenCurrent);
       Rectangle(hpsLeft, RowRectLeft.left, RowRectLeft.top,
                 RowRectLeft.right, RowRectLeft.bottom);
       SelectObject ( hpsLeft, hBrushOld);
       SelectObject ( hpsLeft, hPenOld);
       if (hPenCurrent)
         DeleteObject (hPenCurrent);

       /***************************************************************/
       /* Restore the previous PS attributes                          */
       /***************************************************************/
       RestoreDC (hpsLeft, -1L);

       /***************************************************************/
       /* If we have a right window, we need to draw its target emph. */
       /* too.                                                        */
       /***************************************************************/
       if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
       {
          /************************************************************/
          /* Save the previous hps attributes.                        */
          /************************************************************/
          SaveDC (hpsRight);

          /************************************************************/
          /* Set the color for the lines of the box                   */
          /************************************************************/
          SetROP2 (hpsRight, R2_NOT);

          /************************************************************/
          /* Draw the box from the current pos to the top right pos   */
          /************************************************************/
          hBrushOld   = SelectObject (hpsRight, GetStockObject(NULL_BRUSH));
          hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
          hPenOld     = SelectObject (hpsRight, hPenCurrent);
          Rectangle(hpsRight, RowRectRight.left, RowRectRight.top,
                    RowRectRight.right, RowRectRight.bottom);
          SelectObject ( hpsRight, hBrushOld);
          SelectObject ( hpsRight, hPenOld);
          if (hPenCurrent)
            DeleteObject (hPenCurrent);

          /************************************************************/
          /* Restore the previous PS attributes                       */
          /************************************************************/
          RestoreDC (hpsRight, -1L);

          /************************************************************/
          /* Release the PS.                                          */
          /************************************************************/
          ReleaseDC (GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
       }

       /***************************************************************/
       /* Release the PS.                                             */
       /***************************************************************/
       ReleaseDC (GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);
    }
    else
    {
      CnrCalcIconSize(hwnd, &iconSize);

      if (pccinfo->flWindowAttr & CV_TREE)
      {
        POINT        slTreeIcon;

        CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);

        (pccinfo->flWindowAttr & CV_TEXT) ?
               CnrTRVQueryIconTextRect(hwnd, precord, TRUE,
                                       iconSize, slTreeIcon,
                                       NULL, NULL, &rctlTarget):
               CnrTRVQueryIconTextRect(hwnd, precord, TRUE,
                                       iconSize, slTreeIcon,
                                       &rctlTarget, NULL, NULL);
      }
      else
      {
        if (pccinfo->flWindowAttr & CV_TEXT)
        {
          CnrQueryIconTextRect(hwnd, precord, &iconSize,
                               FALSE, NULL, &rctlTarget);
        }
        else
        {
          CnrQueryIconTextRect(hwnd, precord, &iconSize,
                               TRUE, &rctlTarget, NULL);
        }
      }

      if (pccinfo->flWindowAttr & CV_TEXT)
      {
        rctlTarget.right += TXT_TOPMARGIN;
      }
      rctlTarget.left -= TXT_TOPMARGIN;
      rctlTarget.bottom += TXT_TOPMARGIN;

      /*****************************************************************/
      /* Save current PS attributes                                    */
      /*****************************************************************/
      SaveDC (hps);

      /*****************************************************************/
      /* Set the color for the lines of the box                        */
      /*****************************************************************/
      SetROP2 (hps, R2_NOT);

      /*****************************************************************/
      /* Draw the box from the current pos to the top right pos        */
      /*****************************************************************/
      hBrushOld   = SelectObject (hps, GetStockObject(NULL_BRUSH));
      hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
      hPenOld     = SelectObject (hps, hPenCurrent);
      Rectangle(hps, rctlTarget.left, rctlTarget.top,
                rctlTarget.right, rctlTarget.bottom);
      SelectObject ( hps, hBrushOld);
      SelectObject ( hps, hPenOld);
      if (hPenCurrent)
        DeleteObject (hPenCurrent);

      /*****************************************************************/
      /* Restore the previous PS attributes                            */
      /*****************************************************************/
      RestoreDC (hps, -1L);
    }
  }

  return;
}

/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME:  CnrDrawTargetEmphB                                */
/*                                                                   */
/* Usage: Internal                                                   */
/*                                                                   */
/* DESCRIPTIVE NAME:  Draw Target Emphasis Below or Above Item       */
/*                                                                   */
/* FUNCTIONAL DESCRIPTION:  This function draws or erases the        */
/*     straight line target emphasis below or above the item.        */
/*                                                                   */
/* NOTES:                                                            */
/*    FORWARD DECLARATION:                                           */
/*                                                                   */
/* OUTPUT:                                                           */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL:                                                      */
/*                                                                   */
/* EXIT-ERROR:                                                       */
/*                                                                   */
/* SIDE EFFECTS:                                                     */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrDrawTargetEmphB(hwnd, hps, preccObject, emphPos)
HANDLE            hwnd;
HDC             hps;
LPRECORDCOREN    preccObject;
DWORD           emphPos;
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  POINT     ptl;
  POINT     iconSize;
  long      lQueryLoc;
  int       yOffset;
  int       cyBetweenRows;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  CnrCalcIconSize(hwnd, &iconSize);

  cyBetweenRows = (pccinfo->cyLineSpacing >= 0) ?
                         pccinfo->cyLineSpacing : MINCY_BETWEENITEMS;

  if (emphPos == DRAGTARGETEMPHABOVE)
  {
    lQueryLoc = PT_TOP;
    yOffset   = -ICN_SELSPACE;
  }
  else
  {
    lQueryLoc = PT_BOTTOM;
    yOffset   = cyBetweenRows/2;
  }

  ptl.x = CnrQueryPtOfItemInNTV (hwnd, preccObject, &iconSize,
                                 PT_LEFT) - pccinfon->ptlNonIconOrg.x;

  ptl.y = CnrQueryPtOfItemInNTV (hwnd, preccObject, &iconSize, lQueryLoc)
                                 - yOffset - pccinfon->ptlNonIconOrg.y;

  MoveToEx( hps, ptl.x, ptl.y, NULL );                                  /*DAB1*/

  ptl.x = CnrQueryPtOfItemInNTV (hwnd, preccObject, &iconSize,
                                 PT_RIGHT) - pccinfon->ptlNonIconOrg.x;

  /*******************************************************************/
  /* Save current PS attributes                                      */
  /*******************************************************************/
  SaveDC (hps);

  /*******************************************************************/
  /* Set the mix mode for the lines of the box                       */
  /*******************************************************************/
  SetROP2(hps, R2_NOT);

  LineTo(hps, ptl.x, ptl.y);

  /*******************************************************************/
  /* Restore the previous PS attributes                              */
  /*******************************************************************/
  RestoreDC (hps, -1L);

  return;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:     CnrDrawWinTargetEmph                          */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Control Draw Cnr Window Target Emph          */
/*                                                                   */
/*  Functional Description: This function draws a box around         */
/*     Cnr client area if flEmphOn is TRUE, else erase the box       */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*           flTargetOn - flag which indicates emph on or off        */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrDrawWinTargetEmph(hwnd, hps, flEmphOn)
HANDLE     hwnd;
HDC        hps;
BOOL       flEmphOn;
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
//POINT     ptlPos;
  RECT      rctlClient;
  RECT      rctlHScroll;
  RECT      rctlTitle;                                           //FLIP
  RECT      ViewPortLeft;
  RECT      ViewPortRight;
//POINT     ptlPosR;
  HDC       hpsLeft;
  HDC       hpsRight;
  HPEN      hPenOld;
  HPEN      hPenCurrent;
  HBRUSH    hBrushOld;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* If we are in Details View we need to draw the window target      */
  /* emphasis around the 2 Details View windows if they are both      */
  /* there.  We use GpiBox for each window, but it will appear as     */
  /* though only 1 box was drawn.                                     */
  /********************************************************************/
  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
     /*****************************************************************/
     /* If there are 2 windows in Details View, call GpiBox for each  */
     /* window.  However, for the left window, add 3 to the right of  */
     /* the rectangle, and for the right window subtract 3 from the   */
     /* left side of the rectangle.  This will give the impression    */
     /* of 1 box being drawn.                                         */
     /*****************************************************************/
     if ((!(pccinfon->ulFlagSet & FL_UNSPLIT)) &&
        (CnrVerifyFieldInfoPtrs(hwnd,pccinfon->pFieldInfoLast,1)))
     {
        /**************************************************************/
        /* Get the hps for each of the windows.                       */
        /**************************************************************/
//      hpsLeft  = DrgGetPS (GetDlgItem (hwnd, CID_LEFTDVWND));
//      hpsRight = DrgGetPS (GetDlgItem (hwnd, CID_RIGHTDVWND));
        hpsLeft  = PMGetFlippedDC (GetDlgItem (hwnd, CID_LEFTDVWND));
        hpsRight = PMGetFlippedDC (GetDlgItem (hwnd, CID_RIGHTDVWND));

        /**************************************************************/
        /* Get the ViewPorts of both windows.                         */
        /**************************************************************/
        PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                            &ViewPortLeft);

        PMGetClientRect (GetDlgItem (hwnd, CID_RIGHTDVWND),
                            &ViewPortRight);

        /**************************************************************/
        /* Add and subtract 3 to give the impression that only 1 box  */
        /* was drawn.                                                 */
        /**************************************************************/
        ViewPortLeft.right += 3;
        ViewPortRight.left -= 3;

        /**************************************************************/
        /* Now move 1 pixel in from the top and the right so the      */
        /* border is drawn just inside the window.                    */
        /**************************************************************/
        ViewPortLeft.top++;                                      //FLIP -- to ++
        ViewPortRight.top++;                                     //FLIP -- to ++
        ViewPortRight.right--;

        /**************************************************************/
        /* Set the starting position for both windows to be the       */
        /* bottom left hand corner.                                   */
        /**************************************************************/
//      ptlPos.x = ViewPortLeft.left;
//      ptlPos.y = ViewPortLeft.bottom;

//      ptlPosR.x = ViewPortRight.left;
//      ptlPosR.y = ViewPortRight.bottom;

        /**************************************************************/
        /* Move to the starting positions.                            */
        /**************************************************************/
//      GpiMove (hpsLeft, &ptlPos);
//      GpiMove (hpsRight, &ptlPosR);

        /**************************************************************/
        /* Save current PS attributes.                                */
        /**************************************************************/
        SaveDC (hpsLeft);
        SaveDC (hpsRight);

        /**************************************************************/
        /* Set the color for the lines of the box.                    */
        /**************************************************************/
        SetROP2 (hpsLeft, R2_NOT);
        SetROP2 (hpsRight, R2_NOT);

        /**************************************************************/
        /* Set the top right corner of the box for both windows.      */
        /**************************************************************/
//      ptlPos.x = ViewPortLeft.right;
//      ptlPos.y = ViewPortLeft.top;

//      ptlPosR.x = ViewPortRight.right;
//      ptlPosR.y = ViewPortRight.top;

        /**************************************************************/
        /* Draw the boxs from the current position to the top right   */
        /* position.                                                  */
        /**************************************************************/
//      GpiBox (hpsLeft, DRO_OUTLINE, &ptlPos, 0L, 0L);
        hBrushOld   = SelectObject (hpsLeft, GetStockObject(NULL_BRUSH));
        hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
        hPenOld     = SelectObject (hpsLeft, hPenCurrent);
        Rectangle(hpsLeft, ViewPortLeft.left, ViewPortLeft.top,
                  ViewPortLeft.right, ViewPortLeft.bottom);
        SelectObject ( hpsLeft, hPenOld);
        if (hPenCurrent)
          DeleteObject (hPenCurrent);
        SelectObject ( hpsLeft, hBrushOld);

//      GpiBox (hpsRight, DRO_OUTLINE, &ptlPosR, 0L, 0L);
        hBrushOld   = SelectObject (hpsRight, GetStockObject(NULL_BRUSH));
        hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
        hPenOld     = SelectObject (hpsRight, hPenCurrent);
        Rectangle(hpsRight, ViewPortRight.left, ViewPortRight.top,
                  ViewPortRight.right, ViewPortRight.bottom);
        SelectObject ( hpsRight, hPenOld);
        if (hPenCurrent)
          DeleteObject (hPenCurrent);
        SelectObject ( hpsRight, hBrushOld);
        /**************************************************************/
        /* Restore the previous PS attributes.  Make sure to pop them */
        /* in reverse order from the push though.                     */
        /**************************************************************/
        RestoreDC (hpsRight, -1L);
        RestoreDC (hpsLeft, -1L);

        /**************************************************************/
        /* Be a good guy and release both hps's.                      */
        /**************************************************************/
        ReleaseDC (GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);
        ReleaseDC (GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
     }
     else
     /*****************************************************************/
     /* There isn't a right window for Details View, only a left.     */
     /* Therefore we draw the target emphasis box around the left     */
     /* window only.                                                  */
     /*****************************************************************/
     {
        /**************************************************************/
        /* Get the hps for the left window.                           */
        /**************************************************************/
//      hpsLeft = DrgGetPS (GetDlgItem (hwnd, CID_LEFTDVWND));
        hpsLeft = PMGetFlippedDC (GetDlgItem (hwnd, CID_LEFTDVWND));

        /**************************************************************/
        /* Get the ViewPort rectangle of the left window.             */
        /**************************************************************/
        PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                            &ViewPortLeft);

        /**************************************************************/
        /* Now move 1 pixel in from the top and the right so the      */
        /* border is drawn just inside the window.                    */
        /**************************************************************/
        ViewPortLeft.top++;                                      //FLIP -- to ++
        ViewPortLeft.right--;

        /**************************************************************/
        /* Set the starting position to be the bottom left hand       */
        /* corner of the window.                                      */
        /**************************************************************/
//      ptlPos.x = ViewPortLeft.left;
//      ptlPos.y = ViewPortLeft.bottom;

        /**************************************************************/
        /* Move to the starting position.                             */
        /**************************************************************/
//      GpiMove (hpsLeft, &ptlPos);

        /**************************************************************/
        /* Save current PS attributes.                                */
        /**************************************************************/
        SaveDC (hpsLeft);

        /**************************************************************/
        /* Set the color for the lines of the box.                    */
        /**************************************************************/
        SetROP2 (hpsLeft, R2_NOT);

        /**************************************************************/
        /* Set the top right corner of the window.                    */
        /**************************************************************/
//      ptlPos.x = ViewPortLeft.right;
//      ptlPos.y = ViewPortLeft.top;

        /**************************************************************/
        /* Draw the box around the window.                            */
        /**************************************************************/
//      GpiBox (hpsLeft, DRO_OUTLINE, &ptlPos, 0L, 0L);
        hBrushOld   = SelectObject (hpsLeft, GetStockObject(NULL_BRUSH));
        hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
        hPenOld     = SelectObject (hpsLeft, hPenCurrent);
        Rectangle(hpsLeft, ViewPortLeft.left, ViewPortLeft.top,
                  ViewPortLeft.right, ViewPortLeft.bottom);

        SelectObject ( hpsLeft, hPenOld);
        if (hPenCurrent)
          DeleteObject (hPenCurrent);
        SelectObject ( hpsLeft, hBrushOld);

        /**************************************************************/
        /* Restore the PS.                                            */
        /**************************************************************/
        RestoreDC (hpsLeft, -1L);

        /**************************************************************/
        /* Release the PS.                                            */
        /**************************************************************/
        ReleaseDC (GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);
     }
  }
  else
  {
     /*****************************************************************/
     /* Get viewport rect and  H scroll bar rect.                     */
     /* left-top point is (0, 0)                                      */
     /*****************************************************************/
     CnrQuerySizeRect(hwnd, NULL, &rctlTitle, &rctlClient, NULL,  //FLIP
                      &rctlHScroll) ;

     /*****************************************************************/
     /* Set Rectangle of the client area relative to the Cnr window   */
     /* origin and move 1 pixel in from the top and from the right    */
     /* to draw the box just inside the border.                       */
     /*****************************************************************/
     rctlClient.right --;
//   rctlClient.bottom += rctlHScroll.top;
     rctlClient.bottom += rctlTitle.bottom;                      //FLIP
//   rctlClient.top += (rctlHScroll.top - TXT_TOPMARGIN);
     rctlClient.top += (rctlTitle.bottom + TXT_TOPMARGIN);       //FLIP

     flEmphOn = flEmphOn; /*will be deleted if invert color is accepted*/

     /*****************************************************************/
     /* Set bottom left position of the box to be drawn               */
     /*****************************************************************/
//   ptlPos.x = rctlClient.left ;
//   ptlPos.y = rctlClient.bottom ;

     /*****************************************************************/
     /* Move current position to the bottom left coner of the box     */
     /*****************************************************************/
//   GpiMove(hps, (LPPOINT)(&ptlPos));

     /*****************************************************************/
     /* Save current PS attributes                                    */
     /*****************************************************************/
     SaveDC (hps) ;

     /*****************************************************************/
     /* Set the color for the lines of the box                        */
     /*****************************************************************/
     SetROP2 (hps, R2_NOT);

     /*****************************************************************/
     /* Set top  right  position of the box to be drawn               */
     /*****************************************************************/
//   ptlPos.x = rctlClient.right ;
//   ptlPos.y = rctlClient.top ;

     /*****************************************************************/
     /* Draw the box from the current pos to the top right pos        */
     /*****************************************************************/
//   GpiBox(hps, DRO_OUTLINE, (LPPOINT)(&ptlPos), 0L, 0L) ;
     hBrushOld   = SelectObject (hps, GetStockObject(NULL_BRUSH));
     hPenCurrent = CreatePen (PS_SOLID, 1, BLACK_PEN);
     hPenOld     = SelectObject (hps, hPenCurrent);
     Rectangle(hps, rctlClient.left, rctlClient.top, rctlClient.right,
               rctlClient.bottom);
     SelectObject ( hps, hBrushOld);
     SelectObject ( hps, hPenOld);
     if (hPenCurrent)
       DeleteObject (hPenCurrent);
     /*****************************************************************/
     /* Restore the previous PS attributes                            */
     /*****************************************************************/
     RestoreDC (hps, -1L);
 }

 return;
}
#endif
/* *******************************************************************/
/*                                                                   */
/*  Function Name:     CnrPaintEdgeWin                               */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Paint Cnr Title Window                       */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function paints edge window with SYSCLR_WIN.             */
/*                                                                   */
/*                                                                   */
/*  Notes:                                                           */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd    - container control handle                      */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/
VOID FAR CnrPaintEdgeWin(HANDLE hwnd)
{
  HANDLE      hwndCnr;
  HDC         hps;
  LPCNRINFON  pccinfon;
  RECT        rctlEdge;
  PAINTSTRUCT paint;
  HBRUSH      hBrush;

  hwndCnr = PARENTOF (hwnd);
  pccinfon = (LPCNRINFON)GetWindowLong(hwndCnr,QWL_CONTAINER);

  hps = BeginPaint(hwnd, &paint);
  rctlEdge = paint.rcPaint;

  PMFlipDC(hwnd, hps);
  PMFlipRect((LPRECT)&rctlEdge);

//CnrPaintBackground (hwndCnr, hps, &rctlEdge);
  if (hBrush = CreateSolidBrush (GetSysColor (COLOR_SCROLLBAR)))        //->@Z01
  {
    FillRect(hps, &rctlEdge, hBrush);
    DeleteObject(hBrush);
  }                                                                     //@Z01<-

  EndPaint(hwnd, &paint);
  return;
}

/* *******************************************************************/
/*                                                                   */
/*  Function Name:  CnrSetBlinkTimer                                 */
/*                                                                   */
/*  Usage: External                                                  */
/*                                                                   */
/*  Descriptive Name:   Set Timer For Blinking Cursored Emph         */
/*                                                                   */
/*  Functional Description:                                          */
/*     This function sets a timer for the blinking cursored          */
/*     emphasis.                                                     */
/*                                                                   */
/*  Notes:                                                           */
/*     Forward Declaration:                                          */
/*                                                                   */
/*                                                                   */
/*  Input:   hwnd - handle of the container window.                  */
/*                                                                   */
/*  Output:                                                          */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*  Exit-Normal:                                                     */
/*                                                                   */
/*  Exit-Error:                                                      */
/*                                                                   */
/*  Side Effects:                                                    */
/*                                                                   */
/*********************************************************************/

BOOL FAR  CnrSetBlinkTimer(hwnd)
HANDLE      hwnd;

{
  HDC        hps;
  HDC        hpsLeft;
  HDC        hpsRight;
  RECT      RowRectLeft;
  RECT      RowRectRight;
  LPCNRINFON  pccinfon;
  LPCNRINFO   pccinfo;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfon->ulFlagSet & FL_TIMER)
  {
    /*****************************************************************/
    /* if timer is started already, stop it and draw proper box      */
    /* around the item.                                              */
    /*****************************************************************/
    xSetTimer(hwnd, 0, FALSE);
    pccinfon->ulFlagSet &= ~FL_TIMER;
    hps = PMGetFlippedDC(hwnd);
    if (!(DrawCursorTargetRecord(hwnd, hps, pccinfon->preccCursor,
                           CRA_CURSORED, TRUE)))
    {
       ReleaseDC(hwnd, hps);
       return (FALSE);
    }

    /******************************************************************/
    /* DrawCursorTargetRecord does not draw the cursor for the row in */
    /* Details View.  Therefore, if we are in Details View, we need   */
    /* to draw it here.                                               */
    /******************************************************************/
    if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       hpsLeft = PMGetFlippedDC (GetDlgItem (hwnd, CID_LEFTDVWND));
       CnrCalcRowRect (hwnd, pccinfon->preccCursor,
                       TRUE, &RowRectLeft);
       RowRectLeft.top += 2;                                     //FLIP - to +
       RowRectLeft.bottom--;                                     //FLIP ++ to --
       RowRectLeft.right += 2;

       if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
       {
          hpsRight = PMGetFlippedDC (GetDlgItem (hwnd, CID_RIGHTDVWND));
          CnrCalcRowRect (hwnd, pccinfon->preccCursor,
                          FALSE, &RowRectRight);
          RowRectRight.top += 2;                                 //FLIP - to +
          RowRectRight.bottom--;                                 //FLIP ++ to --
          RowRectRight.left -= 2;
          RowRectRight.right += 2;
       }

       CnrDrawCursorBox (hwnd, hpsLeft, pccinfon->preccCursor,
                         RowRectLeft, TRUE);

       /***************************************************************/
       /* If Details View has a right window, we must draw the cursor */
       /* for it too.                                                 */
       /***************************************************************/
       if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
       {
         CnrDrawCursorBox (hwnd, hpsRight, pccinfon->preccCursor,
                           RowRectRight, TRUE);
       }

       ReleaseDC(GetDlgItem (hwnd, CID_LEFTDVWND), hpsLeft);

       if (!(pccinfon->ulFlagSet & FL_UNSPLIT))
       {
          ReleaseDC(GetDlgItem (hwnd, CID_RIGHTDVWND), hpsRight);
       }
    }

    ReleaseDC(hwnd, hps);

  } else {
    /*****************************************************************/
    /* if timer is not started yet, start it. We use the system      */
    /* cursor rate as blinking rate.                                 */
    /*****************************************************************/
//  xSetTimer(hwnd, GetSystemMetrics( SV_CURSORRATE), TRUE);
    xSetTimer(hwnd, 50, TRUE);
    pccinfon->ulFlagSet |= FL_TIMER;
  } /* endif */
  return TRUE;

}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrVerifyPointers                              */
/*                                                                    */
/*  Usage: External                                                   */
/*                                                                    */
/*  Descriptive Name:   Control Verify Pointers                       */
/*                                                                    */
/*  Functional Description: Check if the first cNumRecoreds items     */
/*     in the array are in the list. If the items are in the list     */
/*     this function returns TRUE                                     */
/*                                                                    */
/**********************************************************************/
BOOL FAR  CnrVerifyPointers(hwnd, pRecordArray, cNumRecord)
HANDLE          hwnd;
LPRECORDCORE   FAR *pRecordArray;
DWORD         cNumRecord;
{
  DWORD               i;
  BOOL                found;        /* indicator to tell the record is found */
  LPCNRINFON           pccinfon;
  LPRECORDCORE         preccComp;    /* External record to be checked         */
  LPRECORDCOREN        preccList;    /* Internal record pointer               */

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd, QWL_CONTAINER);
  i = 0;
  found = TRUE;

  /*******************************************************************/
  /* Check until an item isn't found or all items are found          */
  /*******************************************************************/
  while (found && (i < cNumRecord) )
  {
    found = FALSE;
    preccComp = *(pRecordArray + i);
    preccList = pccinfon->preccItem;

    /*****************************************************************/
    /* Check an item is in the list                                  */
    /*****************************************************************/
    while (!found && preccList)
    {
      found = CnrIsRecordInTree(preccList, preccComp);
      preccList = preccList->preccNextItem;
    }
    ++i;

  }

  if (!found)
  {
    SETERROR(PMERR_INVALID_PARAMETERS);
  }
  return( found );

}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     CnrIsRecordInTree                              */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrIsRecordInTree (preccRoot, preccComp)
LPRECORDCOREN   preccRoot;
LPRECORDCORE    preccComp;
{
  LPRECORDCOREN  preccEnum;

  if (!preccRoot || !preccComp)
  {
    return  FALSE;
  }

  if (preccRoot->preccX == preccComp)
  {
    return  TRUE;
  }

  preccEnum = preccRoot->preccFirstChild;

  while (preccEnum)
  {
    if (CnrIsRecordInTree(preccEnum, preccComp))
    {
      return TRUE;
    }
    preccEnum = preccEnum->preccNextItem;

  }
  return FALSE;
}
