/* Revision: 23 1.3.1.2 source/ui/cclnt/cnr/cnrs3p6.c, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                         C O N T A I N E R                          */
/*                                                                    */
/*                        Filename: CNRS3P6.C                         */
/*                                                                    */
/*             ( Windows NT Container Control Component )             */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: CNRS3P6.C - Selection processing routines for container  */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* DAB1  11/16/93  DAB  Initial modifications for NT execution        */
/* D6386  1/16/95  ASM  Enable message passing in Win32s environment  */
/*                      Change SendMessage to SENDMESSAGE             */
/* PPB1  01/29/96  PPB  Ensure scroll position > 0 defect 6948        */
/**********************************************************************/
#include <math.h>
#include <stdlib.h>
#include <windows.h>
#include <iclccl.h>     /* CCL common include file */ 
#include "cnr2win.h"
#include "iclcnrw.h"

#include "cnrcbn.h"

#ifdef SEGMENTOR
#define CNRS3P6_C
#include "cnrpragw.h"
#endif


/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrExtendedSelection                           */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:  long CnrExtendedSelection(               */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:   FALSE                                              */
/*                                                                    */
/*  Exit-Error:    TRUE                                               */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrExtendedSelection (HANDLE hwnd, HDC hps,
                                LPRECORDCOREN pMouseRecord,
                                WORD fSelectionState, BOOL fClearSelect)
{
  LPRECORDCOREN pRecord;
  LPCNRINFON pccinfon;                 /* Internal control data      */
  BOOL fStillInRange = TRUE;          /* Loop exit variable         */
  BOOL fForwardDirection;             /* TRUE  -> forward direction */
                                      /* FALSE -> backward direction*/
  BOOL fForwardDirection2;            /* TRUE  -> New end of Range is in */
                                      /* the forward dir. from the old.  */
  BOOL fForwardDirection3;            /* TRUE  -> Prev end of Range is in*/
                                      /* the forward dir. from the anchor*/
  BOOL fReachedCriticalRec = FALSE;
  LPRECORDCOREN pCriticalRecord=NULL;

  DWORD       flMouseRecAttrOld;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);

  /********************************************************************/
  /* Check that both an anchor record and mouse record were passed    */
  /********************************************************************/
  if (!pMouseRecord || (!pccinfon->preccAnchor))
  {
     return (TRUE);
  }

  if (!pccinfon->preccPrevEndOfRange)
  {
    pccinfon->preccPrevEndOfRange = pccinfon->preccAnchor;
  }

  /*******************************************************************/
  /*  Determine the relative positions of the new end of range, the  */
  /*  old end of range and the anchor point.                         */
  /*******************************************************************/
  if ( (pMouseRecord != pccinfon->preccAnchor) ||
       (pccinfon->preccAnchor != pccinfon->preccPrevEndOfRange) )
  {
  /********************************************************************/
  /* Check if the mouse record is not the same as the anchor record   */
  /********************************************************************/
    pccinfon->ulFlagSet &= ~FL_SHIFTARROW;

    if ((pMouseRecord->ptlNonIconPos.x >
         pccinfon->preccAnchor->ptlNonIconPos.x)  ||
         ((pMouseRecord->ptlNonIconPos.x ==
          pccinfon->preccAnchor->ptlNonIconPos.x) &&
          (pMouseRecord->ptlNonIconPos.y >
          pccinfon->preccAnchor->ptlNonIconPos.y)))
    {
      fForwardDirection = TRUE;
    }
    else
    {
      fForwardDirection = FALSE;
    }
  }
  else
  {
     /****************************************************************/
     /* if click on anchor point, don't change                       */
     /****************************************************************/
     return (TRUE);
  }

  if (!fClearSelect)
  {
    if ((pMouseRecord->ptlNonIconPos.x >
         pccinfon->preccPrevEndOfRange->ptlNonIconPos.x) ||
         ((pMouseRecord->ptlNonIconPos.x ==
         pccinfon->preccPrevEndOfRange->ptlNonIconPos.x) &&
         (pMouseRecord->ptlNonIconPos.y >
         pccinfon->preccPrevEndOfRange->ptlNonIconPos.y)))
    {
      fForwardDirection2 = TRUE;
    }
    else
    {
      fForwardDirection2 = FALSE;
    }
    if ((pccinfon->preccPrevEndOfRange->ptlNonIconPos.x >
          pccinfon->preccAnchor->ptlNonIconPos.x)  ||
         ( (pccinfon->preccPrevEndOfRange->ptlNonIconPos.x ==
            pccinfon->preccAnchor->ptlNonIconPos.x)
            && (pccinfon->preccPrevEndOfRange->ptlNonIconPos.y >
                pccinfon->preccAnchor->ptlNonIconPos.y)))
    {
      fForwardDirection3 = TRUE;
    }
    else
    {
      fForwardDirection3 = FALSE;
    }
    if ( (!fForwardDirection && !fForwardDirection2 &&
          fForwardDirection3) ||
         (fForwardDirection && fForwardDirection2 &&
          !fForwardDirection3) ||
         (pccinfon->preccAnchor == pccinfon->preccPrevEndOfRange) )
    {
      pCriticalRecord = pccinfon->preccAnchor;
    }
    else
    {
      if ( (fForwardDirection && fForwardDirection2 &&
            fForwardDirection3) ||
           (!fForwardDirection && !fForwardDirection2 &&
            !fForwardDirection3) )
      {
        pCriticalRecord = pccinfon->preccPrevEndOfRange;
      }
    }
  }

  /*******************************************************************/
  /*  Select/Deselect new range.                                     */
  /*******************************************************************/
  pRecord = pccinfon->preccAnchor;

  while (fStillInRange && pRecord)
  {
    if (pRecord == pMouseRecord)
    {
       fStillInRange = FALSE;
       flMouseRecAttrOld = pMouseRecord->flRecordAttr;
    }
    if (fReachedCriticalRec)
    {
      if (pRecord->flRecordAttr & CRA_SELECTED)
      {
        pRecord->flSelecting = TRUE;
      }
      else
      {
        pRecord->flSelecting = FALSE;
      }
    }
    switch (fSelectionState)
    {
      case TOGGLE:
        SetEmphasisBits (hwnd, hps, pRecord, (DWORD)CRA_SELECTED,
              (WORD)((pccinfon->fAnchorState) ? SEB_SET : SEB_CLEAR));
        break;

      case SELECT:
        if (!(pRecord->flRecordAttr & CRA_SELECTED) &&
             (pRecord != pccinfon->preccAnchor))
        {
          SetEmphasisBits (hwnd, hps, pRecord, CRA_SELECTED, SEB_SET);
        }
        break;

      case DESELECT:
        if (pRecord->flRecordAttr & CRA_SELECTED)
        {
          SetEmphasisBits (hwnd, hps, pRecord, CRA_SELECTED, SEB_CLEAR);
        }
        break;

    } /* endswitch */

    if (pRecord == pCriticalRecord)
    {
      fReachedCriticalRec = TRUE;
    }
    if (fStillInRange)
    {
      if (fForwardDirection)
      {
        pRecord = GetNextUnfilteredItem (pRecord);
      }
      else
      {
        pRecord = GetPrevUnfilteredItem (pRecord);
      }
    }
  } /* endwhile */



  /*******************************************************************/
  /* Deselect all others not in range if not extended selection or   */
  /* insert mode.                                                    */
  /*******************************************************************/
  if (fClearSelect)
  {
    if (fForwardDirection)
    {
      pRecord = GetNextUnfilteredItem (pMouseRecord);
    }
    else
    {
      pRecord = GetPrevUnfilteredItem (pMouseRecord);
    }

    while (pRecord)
    {
      if (pRecord->flRecordAttr & CRA_SELECTED)
      {
        if (!SetEmphasisBits (hwnd, hps, pRecord, CRA_SELECTED, SEB_CLEAR))
        {
           return (FALSE);
        }
      }

      if (fForwardDirection)
      {
        pRecord = GetNextUnfilteredItem (pRecord);
      }
      else
      {
        pRecord = GetPrevUnfilteredItem (pRecord);
      }

    }

    if (fForwardDirection)
    {
      pRecord = GetPrevUnfilteredItem (pccinfon->preccAnchor);
    }
    else
    {
      pRecord = GetNextUnfilteredItem (pccinfon->preccAnchor);
    }

    while (pRecord)
    {
      if (pRecord->flRecordAttr & CRA_SELECTED)
      {
        if (!SetEmphasisBits (hwnd, hps, pRecord, CRA_SELECTED, SEB_CLEAR))
        {
          return (FALSE);
        }
      }

      if (fForwardDirection)
      {
        pRecord = GetPrevUnfilteredItem (pRecord);
      }
      else
      {
        pRecord = GetNextUnfilteredItem (pRecord);
      }
    }
  }
  else
  /*******************************************************************/
  /* Undo(return to original state) all records from prev. range not */
  /* in the current range.                                           */
  /*******************************************************************/
  {
    if ( (pccinfon->preccPrevEndOfRange != pccinfon->preccAnchor) &&
         (pccinfon->preccPrevEndOfRange != pMouseRecord) )
    {
      if ( (fForwardDirection && !fForwardDirection2 &&
            fForwardDirection3) ||
           (!fForwardDirection && !fForwardDirection2 &&
            fForwardDirection3) )
      {
        if (fForwardDirection && !fForwardDirection2 &&
            fForwardDirection3)
        {
          pRecord = pMouseRecord;
        }
        else
        {
          pRecord = pccinfon->preccAnchor;
        }
        pRecord = GetNextUnfilteredItem (pRecord);
        while (pRecord)
        {
          if (!SetEmphasisBits (hwnd, hps, pRecord, (DWORD)CRA_SELECTED,
               (WORD)((pRecord->flSelecting) ? SEB_SET : SEB_CLEAR)))
          {
            return (FALSE);
          }
          if (pRecord == pccinfon->preccPrevEndOfRange)
          {
            pRecord = NULL;
          }
          else
          {
            pRecord = GetNextUnfilteredItem (pRecord);
          }
        }
      }
      if ( (fForwardDirection && fForwardDirection2 &&
            !fForwardDirection3) ||
           (!fForwardDirection && fForwardDirection2 &&
            !fForwardDirection3) )
      {
        if (fForwardDirection && fForwardDirection2 &&
            !fForwardDirection3)
        {
          pRecord = pccinfon->preccAnchor;
        }
        else
        {
          pRecord = pMouseRecord;
        }
        pRecord = GetPrevUnfilteredItem (pRecord);

        while (pRecord)
        {
          if (!SetEmphasisBits (hwnd, hps, pRecord, (DWORD)CRA_SELECTED,
                 (WORD)((pRecord->flSelecting) ? SEB_SET : SEB_CLEAR)))
          {
            return (FALSE);
          }
          if (pRecord == pccinfon->preccPrevEndOfRange)
          {
            pRecord = NULL;
          }
          else
          {
            pRecord = GetPrevUnfilteredItem (pRecord);
          }
        }
      }
    }
  }

  if (pMouseRecord != pccinfon->preccCursor)
  {
    /******************************************************************/
    /* Turn the cursor emphasis off the record which is               */
    /* no longer under cursor.                                        */
    /******************************************************************/
    if (!CnrChangeCursor (hwnd, pMouseRecord, hps, FALSE ) ||
        !CnrDrawEmphChange (hwnd, hps, pccinfon->preccCursor,
                            flMouseRecAttrOld))
    {
      return (FALSE);
    }
  }

  /*******************************************************************/
  /* Save the last record over which the mouse pointer was positioned*/
  /* so that the selection state can be reversed when the mouse moves*/
  /* back over a record.                                             */
  /*******************************************************************/
  pccinfon->preccLastMouse = pMouseRecord;
  pccinfon->preccPrevEndOfRange = pMouseRecord;

  return (TRUE);
}


/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrSetRecordSelection                          */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Set the selection state of the record(s)  */
/*      in the rectangle.  Used for marquee selection.                */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:  long CnrSetRecordSelection(              */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:  pSearchRect (LPRECT) - rectangle to search in             */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:   FALSE                                              */
/*                                                                    */
/*  Exit-Error:    TRUE                                               */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR CnrSetRecordSelection(HANDLE hwnd, HDC hps, LPRECT pSearchRect)
{
  QUERYRECFROMRECT queryRecFromRect;   /* CM_QUERYRECORDFROMRECT     */
  LPRECORDCOREN     pRecord;
  LPRECORDCOREN     pLastRecord = NULL;
  LPCNRINFON        pccinfon;           /* Internal control data      */
  RECT            rclDst;
  RECT            rclIconRect;
  QUERYRECORDRECT  queryRecordRect;

  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);


  /*******************************************************************/
  /* Find the first record in the given rectangle                    */
  /*******************************************************************/
  queryRecFromRect.cb = (DWORD)sizeof(QUERYRECFROMRECT);
  queryRecFromRect.rect = *(pSearchRect);
  queryRecFromRect.fsSearch = CMA_PARTIAL | CMA_ZORDER;

  if (!(pRecord = CnrQueryRecordFromRect(hwnd, (long)(LPRECORDCORE)CMA_FIRST,
                                         (long)(LPVOID)&queryRecFromRect)))
  {
     return (FALSE);
  }

  /*******************************************************************/
  /* If there is a record in the rectangle, set the record selection */
  /* attribute acccording to the selectState flag.                   */
  /*******************************************************************/
  while (pRecord)
  {
    /******************************************************************/
    /* Calculate the icon rectangle                                   */
    /******************************************************************/
    queryRecordRect.pRecord = pRecord->preccX;
    queryRecordRect.fsExtent = CMA_ICON;
    CnrQueryRecordRect (hwnd, (long)(LPVOID)&rclIconRect,
                        (long)(LPVOID)&queryRecordRect);

    /*****************************************************************/
    /* If the icon rectangle is completely within the marquee        */
    /* selection rect, the rectangle returned by PMIntersectRect    */
    /* is equal to the icon rectangle.  If entire icon rect is       */
    /* enclosed, the record is selected. 12/90 CUA decision.         */
    /*****************************************************************/
    if (PMIntersectRect( &rclDst, pSearchRect, &rclIconRect))
    {
      if (PMEqualRect( &rclDst, &rclIconRect))
      {
        SetEmphasisBits (hwnd, NULL, pRecord, CRA_SELECTED, SEB_SET);

        pLastRecord = pRecord;

        /*************************************************************/
        /* Draw the record in the new state and notify the app.      */
        /* Note:  This function must be used because the z-order     */
        /* cannot change when drawing the records in the marquee     */
        /* selection box. The fDraw flag in DrawSelectionRecord must */
        /* be false.  This indicates don't change the z-order.  This */
        /* should be the only place in the code where this occurs.   */
        /*************************************************************/
        if (!(DrawSelectionRecord (hwnd, hps, pRecord, FALSE, FALSE)))
        {
          return (FALSE);
        }
      }
    }
    pRecord = CnrQueryRecordFromRect (hwnd, (long)pRecord,
                                      (long)(LPVOID)&queryRecFromRect);
  } /* endwhile */

  if ((pLastRecord) && (pccinfon->preccCursor != pLastRecord))
  {
    SetEmphasisBits (hwnd, NULL, pccinfon->preccCursor, CRA_CURSORED,
                     SEB_CLEAR);

    /****************************************************************/
    /* Draw the cursor.  This  function is used to be consistent    */
    /* with the previous usage of this function                     */
    /****************************************************************/
    if (!(DrawSelectionRecord (hwnd, hps, pccinfon->preccCursor,
                               FALSE, TRUE)))
    {
       return (FALSE);
    }

    SetEmphasisBits (hwnd, NULL, pLastRecord, CRA_CURSORED, SEB_SET);
    pccinfon->preccCursor = pLastRecord;

    /****************************************************************/
    /* Draw the cursor.  This  function is used to be consistent    */
    /* with the previous usage of this function                     */
    /****************************************************************/
    if (!(DrawSelectionRecord (hwnd, hps, pccinfon->preccCursor,
                               FALSE, TRUE)))
    {
       return (FALSE);
    }
  }

  return (TRUE);
}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrSetRect                                     */
/*                                                                    */
/*  Descriptive Name:                                                 */
/*                                                                    */
/*  Functional Description: Set the selection state of the record(s)  */
/*      in the rectangle, depending on the flag passed in.            */
/*      This function is intended to be the center of work for        */
/*      extended selection.                                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:  pSearchRect (LPRECT) - rectangle to search in             */
/*          selectState (WORD) - selection state (SELECT, DESELECT, */
/*                                 TOGGLE)                            */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:   FALSE                                              */
/*                                                                    */
/*  Exit-Error:    TRUE                                               */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
VOID FAR CnrSetRect(LPRECT pSearchRect, LPPOINT ptlEnd, LPPOINT ptlStart)
{
  if (ptlStart->x > ptlEnd->x)
  {
    pSearchRect->left = ptlEnd->x;
    pSearchRect->right = ptlStart->x;
  }
  else
  {
    pSearchRect->left = ptlStart->x;
    pSearchRect->right = ptlEnd->x;
  }

  if (ptlEnd->y > ptlStart->y)
  {
    pSearchRect->top = ptlEnd->y;
    pSearchRect->bottom = ptlStart->y;
  }
  else
  {
    pSearchRect->top = ptlStart->y;
    pSearchRect->bottom = ptlEnd->y;
  }

  return;
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     FindRecordVert                                 */
/*                                                                    */
/*  Descriptive Name:  Find the closest record above or below the     */
/*                     given record.  If no record is found, return   */
/*                     NULL.                                          */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR FindRecordVert(HANDLE hwnd, LPRECORDCOREN pRecord,
                                DWORD dirFlag)
{

  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN pRecordNext=NULL;
  LPRECORDCOREN pRecordReturn=NULL;
  BOOL         found=FALSE;
  long         cxNext;
  long         cyNext;
  long         distanceMin;
  long         distanceNext;
  POINT        iconSize;

  /********************************************************************/
  /* Get pointer to Container internal data.                          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /*******************************************************************/
  /* Verify we have a valid record pointer                            */
  /********************************************************************/
  if (!pRecord)
  {
    return NULL;
  }

  /******************************************************************/
  /* Calculate the icon rectangle                                   */
  /******************************************************************/
  CnrCalcIconSize (hwnd, &iconSize);

  /******************************************************************/
  /* In icon view.....                                              */
  /******************************************************************/
  if (!(pccinfo->flWindowAttr & CV_TREE) &&
      (pccinfo->flWindowAttr & CV_ICON))
  {
    /****************************************************************/
    /* Search for closest record using z_order.                     */
    /****************************************************************/
    pRecordNext = pccinfon->preccZ;
    while (pRecordNext)
    {
      cxNext = (pRecord->ptlIcon.x + iconSize.x / 2) -
                 (pRecordNext->ptlIcon.x + iconSize.x / 2);
      cxNext = labs(cxNext);
      cyNext = (pRecord->ptlIcon.y + iconSize.y / 2) -
                 (pRecordNext->ptlIcon.y + iconSize.y / 2);
      cyNext = labs(cyNext);

      if ( (((pRecordNext->ptlIcon.y > pRecord->ptlIcon.y) &&
             (dirFlag & RD_BELOW)) ||
            ((pRecordNext->ptlIcon.y < pRecord->ptlIcon.y) &&
             (dirFlag & RD_ABOVE))) &&
           (cyNext > cxNext) &&
           ISRECORDVISIBLE(pRecordNext) &&
           (pRecord != pRecordNext))
      {
        /***********************************************************/
        /* Give cxMin and cyMin initial values if not set.         */
        /***********************************************************/
        if (!found)
        {
          distanceMin = cyNext*cyNext + cxNext*cxNext;
          pRecordReturn = pRecordNext;
          found = TRUE;
        }

        /***********************************************************/
        /* If the distance from the cursored item to the next item */
        /* is less than the minimal distance, reset the minimal    */
        /* distance.                                               */
        /***********************************************************/
        distanceNext = cyNext*cyNext + cxNext*cxNext;
        if (distanceNext < distanceMin)
        {
          distanceMin = cyNext*cyNext + cxNext*cxNext;
          pRecordReturn = pRecordNext;
        }

        /***********************************************************/
        /* If the current closest item and the next item are on the*/
        /* same x-axis, check the y-axis to determine which is     */
        /* closer.                                                 */
        /***********************************************************/
      }
      pRecordNext = pRecordNext->preccNextZ;
    }
  }
  else
  {
    if (dirFlag & RD_ABOVE)
    {
      pRecordReturn = GetPrevVisualItem(hwnd, pRecord);
    }
    else
    {
      pRecordReturn = GetNextVisualItem(hwnd, pRecord);
    }
  }

  if (!pRecordReturn)
  {
    return(pRecord);
  }
  else
  {
    return(pRecordReturn);
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     FindRecordHorz                                 */
/*                                                                    */
/*  Descriptive Name:  Find the closest record to the left of the     */
/*                     passed in record.  If no record is found,      */
/*                     return NULL.                                   */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
LPRECORDCOREN FAR FindRecordHorz(HANDLE hwnd, LPRECORDCOREN pRecord,
                                DWORD dirFlag)
{

  LPCNRINFON    pccinfon;
  LPCNRINFO     pccinfo;
  LPRECORDCOREN pRecordNext=NULL;
  LPRECORDCOREN pRecordReturn=NULL;
  POINT        iconSize;
  BOOL         found=FALSE;
  long         cxNext;
  long         cyNext;
  long         cxMin;
  long         cyMin;
  long         distanceMin;
  long         distanceNext;

  /*******************************************************************/
  /* Get pointer to Container internal data.                         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  if (pccinfo->flWindowAttr & (CV_TREE | CV_DETAIL))
  {
    return NULL;
  }

  /******************************************************************/
  /* Verify we have a valid record pointer.                         */
  /******************************************************************/
  if (!pRecord)
  {
    return NULL;
  }

  /******************************************************************/
  /* Calculate the icon rectangle                                   */
  /******************************************************************/
  CnrCalcIconSize (hwnd, &iconSize);

  if (pccinfo->flWindowAttr & CV_ICON)
  {
    /*****************************************************************/
    /* Search for closest record using z_order.                      */
    /*****************************************************************/
    pRecordNext = pccinfon->preccZ;
    while (pRecordNext)
    {
      cxNext = (pRecord->ptlIcon.x + iconSize.x / 2) -
                (pRecordNext->ptlIcon.x + iconSize.x / 2);
      cxNext = labs(cxNext);
      cyNext = (pRecord->ptlIcon.y + iconSize.y / 2) -
                (pRecordNext->ptlIcon.y + iconSize.y / 2);
      cyNext = labs(cyNext);

      if ( (((pRecordNext->ptlIcon.x < pRecord->ptlIcon.x) &&
             (dirFlag & RD_LEFT)) ||
            ((pRecordNext->ptlIcon.x > pRecord->ptlIcon.x) &&
             (dirFlag & RD_RIGHT))) &&
           (cxNext > cyNext) &&
           ISRECORDVISIBLE(pRecordNext) &&
           (pRecord != pRecordNext))
      {
        /************************************************************/
        /* Give cxMin and cyMin initial values if not set.          */
        /************************************************************/
        if (!found)
        {
          distanceMin = cxNext*cxNext + cyNext*cyNext;
          pRecordReturn = pRecordNext;
          found = TRUE;
        }
        /************************************************************/
        /* If the distance from the cursored item to the next item  */
        /* is less than the minimal distance, reset the minimal     */
        /* distance.                                                */
        /************************************************************/
        distanceNext = cxNext*cxNext + cyNext*cyNext;
        if (distanceNext < distanceMin)
        {
          distanceMin = distanceNext;
          pRecordReturn = pRecordNext;
        }
        /************************************************************/
        /* If the current closest item and the next item are on the */
        /* same x-axis, check the y-axis to determine which is      */
        /* closer.                                                  */
        /************************************************************/
      }
      pRecordNext = pRecordNext->preccNextZ;
    }
  }
  else
  if ((pccinfo->flWindowAttr & (CV_NAME | CV_TEXT)) &&
       (pccinfo->flWindowAttr & CV_FLOW))
  {
    /****************************************************************/
    /* If we are in a flowed view find the item closest to the left */
    /* of the cursored item.  If we are not in flowed view, there   */
    /* will never be an item to the left.                           */
    /****************************************************************/
    if (dirFlag & RD_LEFT)
    {
      pRecordNext = pRecord->preccPrevItem;
    }
    else
    {
      pRecordNext = pRecord->preccNextItem;
    }
    while (pRecordNext)
    {
      if ((((pRecordNext->ptlNonIconPos.x < pRecord->ptlNonIconPos.x) &&
            (dirFlag & RD_LEFT)) ||
            ((pRecordNext->ptlNonIconPos.x > pRecord->ptlNonIconPos.x) &&
             (dirFlag & RD_RIGHT))) && ISRECORDVISIBLE(pRecordNext))
      {
        cxNext = (pRecord->ptlNonIconPos.x -
                  pRecordNext->ptlNonIconPos.x);
        cxNext = labs(cxNext);
        cyNext = (pRecord->ptlNonIconPos.y -
                  pRecordNext->ptlNonIconPos.y);
        cyNext = labs(cyNext);

        /***********************************************************/
        /* Give cxMin and cyMin initial values if not set.         */
        /***********************************************************/
        cxMin = cxNext;
        cyMin = cyNext;
        pRecordReturn = pRecordNext;

        /***********************************************************/
        /* Loop finding the closest record in the y-axis until we  */
        /* get to a record in the next column.                     */
        /***********************************************************/
        while ((cxMin == cxNext) && (pRecordNext))
        {
          if ((cyNext < cyMin) && ISRECORDVISIBLE(pRecordNext))
          {
            pRecordReturn = pRecordNext;
            cyMin       = cyNext;
          }
          if (dirFlag & RD_LEFT)
          {
            pRecordNext = pRecordNext->preccPrevItem;
          }
          else
          {
            pRecordNext = pRecordNext->preccNextItem;
          }
          if (pRecordNext)
          {
            cxNext = (pRecord->ptlNonIconPos.x -
                      pRecordNext->ptlNonIconPos.x);
            cxNext = labs(cxNext);
            cyNext = (pRecord->ptlNonIconPos.y -
                      pRecordNext->ptlNonIconPos.y);
            cyNext = labs(cyNext);
          }
        }

        return(pRecordReturn);
      }

      if (dirFlag & RD_LEFT)
      {
        pRecordNext = pRecordNext->preccPrevItem;
      }
      else
      {
        pRecordNext = pRecordNext->preccNextItem;
      }
    }
  }
  if (!pRecordReturn)
  {
    return(pRecord);
  }
  else
  {
    return(pRecordReturn);
  }
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CalcScrollIncrementVert                        */
/*                                                                    */
/*  Descriptive Name:  Calculate vertical scroll increment from the   */
/*                     viewport boundary to the record.               */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
long FAR CalcScrollIncrementVert(HANDLE hwnd, LPRECORDCOREN pRecord)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT      rctlViewPort;
  RECT      rctlHScroll;
  RECT      rctlTitle;
  POINT     iconSize;
  long      cy=0;
  long      bottom;
  long      top;
  long      CyBetweenRows;

  /*******************************************************************/
  /* Get pointer to Container internal data.                         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /******************************************************************/
  /* Calculate the size of the horizontal scroll bar and viewport.  */
  /******************************************************************/
  CnrQuerySizeRect(hwnd, &iconSize, &rctlTitle, &rctlViewPort,  //FLIP
                   NULL, &rctlHScroll);

  /******************************************************************/
  /* If not details view.................                           */
  /******************************************************************/
  if (!(!(pccinfo->flWindowAttr & CV_TREE) &&
        (pccinfo->flWindowAttr & CV_DETAIL)))
  {
    if (pccinfo->flWindowAttr & (CV_TREE | CV_NAME | CV_TEXT))
    {
      rctlViewPort.top += (pccinfon->ptlNonIconOrg.y +
                            rctlTitle.bottom);
      rctlViewPort.bottom += (pccinfon->ptlNonIconOrg.y +
                               rctlTitle.bottom);
    }
    else
    {
      rctlViewPort.top    += rctlTitle.bottom + pccinfo->ptlOrigin.y;
      rctlViewPort.bottom += rctlTitle.bottom + pccinfo->ptlOrigin.y;
    }
  }

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    POINT       slTreeIcon;
    RECT        rctlIcon;
    RECT        rctlTreeIcon;
    RECT        rctlText;

    CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);
    /*****************************************************************/
    /* Find bottommost point of the current record                   */
    /*****************************************************************/
    CnrTRVQueryIconTextRect(hwnd, pRecord, TRUE, iconSize, slTreeIcon,
                            &rctlIcon, &rctlTreeIcon, &rctlText);

    bottom = CnrTRVQueryPtOfItem(hwnd, rctlIcon, rctlTreeIcon,
                   rctlText, PT_BOTTOM) + pccinfon->lAveCharWidth;

    top = CnrTRVQueryPtOfItem( hwnd, rctlIcon, rctlTreeIcon,
                   rctlText, PT_TOP) - pccinfon->lAveCharWidth;
  }
  else
  {
    if (pccinfo->flWindowAttr & CV_ICON)
    {
      bottom = pRecord->ptlIcon.y + iconSize.y + pRecord->slTextSize.cy /*DAB1*/
               + pccinfon->lAveCharWidth + ICN_EMPHSPACE + ICN_SELSPACE;
      top    = pRecord->ptlIcon.y - 2 * ICN_EMPHSPACE;
    }
    else
    if (pccinfo->flWindowAttr & (CV_TEXT | CV_NAME))
    {
      bottom = CnrQueryPtOfItemInNTV (hwnd, pRecord, &iconSize,
                          PT_BOTTOM) + pccinfon->lAveCharWidth;
      top    = CnrQueryPtOfItemInNTV (hwnd, pRecord, &iconSize,
                          PT_TOP) - pccinfon->lAveCharWidth;
    }
    else if (pccinfo->flWindowAttr & CV_DETAIL)
    {
       /**************************************************************/
       /* Set the distance between the rows.  If the application has */
       /* specified it, use it...otherwise use our constant.         */
       /**************************************************************/
       if (pccinfo->cyLineSpacing >= 0)
       {
          CyBetweenRows = pccinfo->cyLineSpacing;
       }
       else
       {
          CyBetweenRows = MINCY_BETWEENITEMS;
       }

       /**************************************************************/
       /* Query the viewport.                                        */
       /**************************************************************/
       PMGetClientRect (GetDlgItem (hwnd, CID_LEFTDVWND),
                           &rctlViewPort);

       bottom = CnrQueryRowPosInDV (pRecord, PT_BOTTOM) +
                 (CyBetweenRows / 2) - pccinfon->ptlNonIconOrg.y;

       top    = CnrQueryRowPosInDV (pRecord, PT_TOP) -
                 (CyBetweenRows / 2) - pccinfon->ptlNonIconOrg.y;
    }
  }
  if ((bottom - top) > (rctlViewPort.bottom - rctlViewPort.top))
  {
    cy = top - rctlViewPort.top;
  }
  else
  if (bottom > rctlViewPort.bottom)
  {
    cy = bottom - rctlViewPort.bottom;
  }
  else
  if (top < rctlViewPort.top)
  {
    cy = top - rctlViewPort.top;
  }

  return(cy);
}

/**********************************************************************/
/*                                                                    */
/*  Function Name:     CalcScrollIncrementHorz                        */
/*                                                                    */
/*  Descriptive Name:  Calculate vertical scroll increment from the   */
/*                     viewport boundary to the record.               */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
long FAR CalcScrollIncrementHorz(HANDLE hwnd, LPRECORDCOREN pRecord)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  RECT     rctlViewPort;
  POINT     iconSize;
  long      cx=0;
  long      left;
  long      right;

  /*******************************************************************/
  /* Get pointer to Container internal data.                         */
  /*******************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /******************************************************************/
  /* Calculate the size of the horizontal scroll bar and viewport.  */
  /******************************************************************/
  CnrQuerySizeRect(hwnd, &iconSize, NULL, &rctlViewPort, NULL, NULL);

  if (pccinfo->flWindowAttr & (CV_TREE | CV_NAME | CV_TEXT))
  {
    rctlViewPort.right  += pccinfon->ptlNonIconOrg.x;
    rctlViewPort.left   += pccinfon->ptlNonIconOrg.x;
  }
  else
  {
    rctlViewPort.right  += pccinfo->ptlOrigin.x;
    rctlViewPort.left   += pccinfo->ptlOrigin.x;
  }

  if (pccinfo->flWindowAttr & CV_TREE)
  {
    POINT        slTreeIcon;
    RECT        rctlIcon;
    RECT        rctlTreeIcon;
    RECT        rctlText;

    CnrTRVCalcTreeIconSize( hwnd, &slTreeIcon);
    /*****************************************************************/
    /* Find bottommost point of the current record                   */
    /*****************************************************************/
    CnrTRVQueryIconTextRect(hwnd, pRecord, TRUE, iconSize, slTreeIcon,
                            &rctlIcon, &rctlTreeIcon, &rctlText);

    left = CnrTRVQueryPtOfItem(hwnd, rctlIcon, rctlTreeIcon,
                   rctlText, PT_LEFT) - pccinfon->lAveCharWidth;

    right = CnrTRVQueryPtOfItem(hwnd, rctlIcon, rctlTreeIcon,
                   rctlText, PT_RIGHT) + pccinfon->lAveCharWidth;
  }
  else
  if (pccinfo->flWindowAttr & CV_ICON)
  {
    left   = pRecord->ptlIcon.x - ((pRecord->slTextSize.cx -            /*DAB1*/
              iconSize.x)/2) - pccinfon->lAveCharWidth;
    right  = left + pRecord->slTextSize.cx +                            /*DAB1*/
              2 * pccinfon->lAveCharWidth;
  }
  else
  if (pccinfo->flWindowAttr & (CV_TEXT | CV_NAME))
  {
    left   = CnrQueryPtOfItemInNTV(hwnd,pRecord,&iconSize,PT_LEFT) -
              pccinfon->lAveCharWidth;
    right  = CnrQueryPtOfItemInNTV(hwnd,pRecord,&iconSize,PT_RIGHT) +
              pccinfon->lAveCharWidth;
  }
  else
  if (pccinfo->flWindowAttr & CV_DETAIL)
  {
    /*****************************************************************/
    /* Return 0 for Details View for Now ...                         */
    /*****************************************************************/
    return(cx);
  }

  if (left < rctlViewPort.left)
  {
    cx = left - rctlViewPort.left;
  }
  else
  if (right > rctlViewPort.right)
  {
    cx = right - rctlViewPort.right;
    if ((right-left) > (rctlViewPort.right - rctlViewPort.left))
    {
      cx = left - rctlViewPort.left;
    }
  }

  return(cx);
}


/**********************************************************************/
/*                                                                    */
/*  Function Name:     ScrollToRecord                                 */
/*                                                                    */
/*  Descriptive Name:  Scroll from current record to new record.      */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*  Input:   none                                                     */
/*                                                                    */
/*  Output:  none                                                     */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR ScrollToRecord(HANDLE hwnd, LPRECORDCOREN pRecordCursor,
                    LPRECORDCOREN pRecordCursorNew, BOOL fSelect,
                    BOOL fHToHawaii)
{
  LPCNRINFON pccinfon;
  LPCNRINFO  pccinfo;
  long      scrollInc;
  long      scrollPos;
  HDC       hps;
  long      windowStyle;                   /* Determine selections style */
  long      flRecordAttrOld;
  BOOL      fDeselect = FALSE;
  BOOL      fExtended = FALSE;

  if (!pRecordCursor || !pRecordCursorNew)
  {
     return (TRUE);
  }
  /********************************************************************/
  /* Get presentation space for drawing cursor emphasis               */
  /********************************************************************/
  hps = PMGetFlippedDC(hwnd);

  /********************************************************************/
  /* Get pointer to Container internal data.                          */
  /********************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong(hwnd,QWL_CONTAINER);
  pccinfo  = PCCINFOXFROMN(pccinfon);

  /********************************************************************/
  /* Apply cursor emphasis to the new record under the cursor. Update */
  /* cursored record indicator.                                       */
  /********************************************************************/
  flRecordAttrOld = pRecordCursorNew->flRecordAttr;

  /*******************************************************************/
  /* In the tree view, move the cursor from the preccCursor in       */
  /* pccinfon to pRecordCursorNew.                                   */
  /*******************************************************************/
  if (pccinfo->flWindowAttr & CV_TREE)
  {
    TRVMoveCursor(hwnd, hps, pRecordCursorNew);
    fExtended = TRUE;
  }
  else
  {
    /********************************************************************/
    /* Turn the cursor emphasis off the record which is no longer under */
    /* the cursor.                                                      */
    /********************************************************************/
    if (fSelect && !fHToHawaii)
    {
      windowStyle = GetWindowLong( hwnd, GWL_STYLE );
      if ((!(windowStyle & CCS_MULTIPLESEL)) &&
          (!(pccinfon->ulFlagSet & FL_ADDMODE)))
      {
        if ((GetKeyState (VK_SHIFT) >= 0 ) ||
           (!(windowStyle & CCS_EXTENDSEL)) ||
           ((windowStyle & CCS_EXTENDSEL) &&
           (pccinfo->flWindowAttr & CV_ICON)))
        {
         if (!(CnrFindAndDeemphasizeRecords(hwnd, hps,
            pRecordCursorNew,CRA_SELECTED | CRA_CURSORED)))
         {
            ReleaseDC (hwnd, hps);
            return (FALSE);
         }
         else
         {
            fDeselect = TRUE;
         }

         SetEmphasisBits (hwnd, NULL, pRecordCursorNew, CRA_SELECTED,
                          SEB_SET);
         pccinfon->preccAnchor = pRecordCursorNew;
         pccinfon->preccPrevEndOfRange = pRecordCursorNew;

         if (pRecordCursorNew->flRecordAttr & CRA_SELECTED)
         {
            pccinfon->fAnchorState = TRUE;
         }
         else
         {
            pccinfon->fAnchorState = FALSE;
         }

        }
        else
        {
          BOOL  fClear;

          if (GetKeyState (VK_CONTROL) < 0)
          {
            fClear = FALSE;
          }
          else
          {
            fClear = TRUE;
          }
          fExtended = TRUE;

          if (!(CnrExtendedSelection(hwnd, hps, pRecordCursorNew,
                (WORD)((pccinfon->fAnchorState) ? SELECT : DESELECT),
                fClear)))
          {
             ReleaseDC (hwnd, hps);
             return (FALSE);
          }
        }
      }
    }
    else
    {
      if (fHToHawaii)
      {
        windowStyle = GetWindowLong( hwnd, GWL_STYLE );
        if ((windowStyle & CCS_MULTIPLESEL) ||
           ((windowStyle & CCS_EXTENDSEL) &&
           (GetKeyState (VK_CONTROL) < 0)))
        {
          SetEmphasisBits (hwnd, NULL, pRecordCursorNew, CRA_SELECTED,
                           SEB_TOGGLE);
          pccinfon->preccAnchor = pRecordCursorNew;
          pccinfon->preccPrevEndOfRange = pRecordCursorNew;

          if (pRecordCursorNew->flRecordAttr & CRA_SELECTED)
          {
             pccinfon->fAnchorState = TRUE;
          }
          else
          {
             pccinfon->fAnchorState = FALSE;
          } /* endif */

        }
        else
        {
          if (!(CnrFindAndDeemphasizeRecords(hwnd, hps,
             pRecordCursorNew,CRA_SELECTED | CRA_CURSORED)))
          {
             ReleaseDC (hwnd, hps);
             return (FALSE);
          }
          else
          {
             fDeselect = TRUE;
          }

          SetEmphasisBits (hwnd, NULL, pRecordCursorNew,
                           CRA_SELECTED, SEB_SET);
          pccinfon->preccAnchor = pRecordCursorNew;
          pccinfon->preccPrevEndOfRange = pRecordCursorNew;

          if (pRecordCursorNew->flRecordAttr & CRA_SELECTED)
          {
             pccinfon->fAnchorState = TRUE;
          }
          else
          {
             pccinfon->fAnchorState = FALSE;
          }
        }
      }
    }
  }

  if (!fExtended)
  {
    if (!(CnrChangeCursor( hwnd, pRecordCursorNew, hps, fDeselect )))
    {
       return (FALSE);
    }

    /******************************************************************/
    /* Paint the record with accumulated cursor and selection changes */
    /******************************************************************/
      if (!(CnrDrawEmphChange( hwnd, hps, pRecordCursorNew,
                               flRecordAttrOld)))
    {
       ReleaseDC (hwnd, hps);
       return (FALSE);
    }
  }

  scrollInc = CalcScrollIncrementVert(hwnd,pRecordCursorNew);
  if (scrollInc)
  {
    scrollInc /= pccinfon->sVscrollFactor;
    scrollPos = scrollInc + pccinfon->sVscrollPos;                      /*PPB1*/
    if (scrollPos < 0)                                                  /*PPB1*/
      scrollPos = 0;                                                    /*PPB1*/

    SENDMESSAGE( hwnd, WM_VSCROLL, MAKELONG( SB_THUMBPOSITION,          /*DAB1*/
                 (WORD)(scrollPos)),                                    /*DAB1*/
                 (LPARAM)GetDlgItem( hwnd, CID_VSCROLL ));              /*DAB1*/
  }

  scrollInc = CalcScrollIncrementHorz(hwnd,pRecordCursorNew);
  if (scrollInc)
  {
    scrollInc /= pccinfon->sHscrollFactor;
    scrollPos = scrollInc + pccinfon->sHscrollPos;                      /*PPB1*/
    if (scrollPos < 0)                                                  /*PPB1*/
      scrollPos = 0;                                                    /*PPB1*/

    SENDMESSAGE( hwnd, WM_HSCROLL, MAKELONG( SB_THUMBPOSITION,          /*DAB1*/
                 (WORD)(scrollPos)),                                    /*DAB1*/
                 (LPARAM)GetDlgItem( hwnd, CID_HSCROLL ));              /*DAB1*/
  }

  /********************************************************************/
  /* Relesase presentation space                                      */
  /********************************************************************/
  ReleaseDC (hwnd, hps);

  return (TRUE);
}

/* ********************************************************************/
/*                                                                    */
/*  Function Name:     CnrFindAndDeemphasizeRecords                   */
/*                                                                    */
/*  Descriptive Name:  Search through records for one selected and    */
/*                     deselects it.                                  */
/*                                                                    */
/*  Functional Description:                                           */
/*                                                                    */
/*  Notes:                                                            */
/*     Forward Declaration:                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Input:   hwnd    - container control handle                       */
/*           LPRECORDCORE - pointer to record compared in search      */
/*                                                                    */
/*  Output:                                                           */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Exit-Normal:                                                      */
/*                                                                    */
/*  Exit-Error:                                                       */
/*                                                                    */
/*  Side Effects:  None                                               */
/*                                                                    */
/**********************************************************************/
BOOL FAR  CnrFindAndDeemphasizeRecords(HANDLE hwnd, HDC hps,
                                  LPRECORDCOREN pRecord, long attribute)
{
  LPCNRINFON    pccinfon;                /* Internal CNR control data */
  LPCNRINFO     pccinfo;
  LPRECORDCOREN searchRecord;            /* Search through records    */

  /*****************************************************************/
  /* Get ptr to internal CNRINFO control data.  The pointer to the */
  /* external control data is beyond the pointer to the internal   */
  /* control data.                                                 */
  /*****************************************************************/
  pccinfon = (LPCNRINFON)GetWindowLong( hwnd, QWL_CONTAINER );
  pccinfo  = PCCINFOXFROMN(pccinfon);

  searchRecord = pccinfon->preccItem;

  /*****************************************************************/
  /* Search through the records, if one is selected, deselect it.  */
  /*****************************************************************/
  while (searchRecord)
  {
    if ((searchRecord->flRecordAttr & attribute) &&
        (searchRecord != pRecord))
    {
      /*************************************************************/
      /* If Icon view and marquee selection has started and record */
      /* is not in view then don't attempt to paint it.            */
      /*************************************************************/
      if ((pccinfo->flWindowAttr & CV_ICON) &&
           (!pccinfon->fFirstMouseMove))
      {
        RECT         rctlInvalid;
        RECT         rctlHScroll;
        RECT         rctlTitle;
        POINT        slIcon;

        CnrQuerySizeRect(hwnd, &slIcon, &rctlTitle, &rctlInvalid, NULL,
                         &rctlHScroll) ;
        rctlInvalid.bottom += rctlTitle.bottom;
        rctlInvalid.top    += rctlTitle.bottom;

        if (!IsInIconViewport(searchRecord, &(pccinfo->ptlOrigin),
                              &rctlInvalid, &slIcon))
        {
          hps = NULL;
        }
      }

      /****************************************************************/
      /* If the record in chain is not the one passed in, deselect    */
      /* it, redraw it, and tell the app.                             */
      /****************************************************************/
      if (!SetEmphasisBits( hwnd, hps, searchRecord, attribute,
                            SEB_CLEAR))
      {
        return (FALSE);
      }
    }
     searchRecord = searchRecord->preccNextItem;
  }
  return (TRUE);
}
