#ifndef _CNRWIN_
#define _CNRWIN_

/* Revision: 26 1.3.2.3 source/ui/cclnt/cnr/cnrwin.h, cclcnr, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997.           */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Container Control                                                  */
/*                                                                    */
/* File:  cnrwin.h                                                    */
/*                                                                    */
/* This header file includes macro includes for the container sample  */
/* application (cnrwin.c).                                            */
/*                                                                    */
/**********************************************************************/

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#define  ID_CONTAINER  1               /* container id                */
#define  ID_FIRSTVIEWITEM 100          /* menu item id's for views    */
#define  ID_CLIENT_RESOURCES 100       /* client window resources     */
#define  IDM_SUBICON   100             /* icon view                   */
#define  IDM_SUBDETAIL 101             /* details view                */
#define  IDM_SUBNAME   102             /* name view                   */
#define  IDM_SUBTEXT   103             /* text view                   */
#define  IDM_SUBTREEICON 104           /* tree icon view              */

/**********************************************************************/
/* id's for object icons                                              */
/**********************************************************************/

#define  ID_ICON_PERSON 200            /* person                      */
#define  ID_ICON_DEPT  201             /* department                  */
#define  ID_ICON_PERSMINW 202          /* mini person                 */
#define  ID_ICON_DEPTMINW 203          /* mini department             */

/**********************************************************************/
/* indices into client data                                           */
/**********************************************************************/

#define  WINDATA_SIZE  0               /* window size                 */
#define  WINDATA_PARENT_CNR 4          /* parent container window     */
                                       /* handle                      */
#define  WINDATA_CURR_ICON_X 8         /* pointer to current window's */
                                       /* icon pos.                   */
#define  WINDATA_CURR_ICON_Y 12        /* pointer to current window's */
                                       /* icon pos.                   */
#define  WINDATA_ITEM_INDEX 16         /* index of record item        */
                                       /* represented                 */
#define  WINDATA_CURR_VIEW 20          /* current view index          */
#define  WINDATA_LENGTH 24             /* number of bytes to allocate */
                                       /* for window                  */
#define  WM_CLIENT_INIT (WM_USER + 1)

/**********************************************************************/
/* sent when parent has created client window and its container.      */
/* mp1 = parent container handle mp2 = index of object client         */
/* container represents frame size with respect to parent window      */
/**********************************************************************/

#define  WIN_RATIO_X   3/5             /* length is 3/5 of parent     */
                                       /* length                      */
#define  WIN_RATIO_Y   3/5             /* height is 3/5 of parent     */
                                       /* height                      */

/**********************************************************************/
/* container margins with respect to client window                    */
/**********************************************************************/

#define  CNR_MARGIN_XLEFT 20           /* left margin                 */
#define  CNR_MARGIN_XRIGHT 20          /* right margin                */
#define  CNR_MARGIN_YTOP 20            /* top margin                  */
#define  CNR_MARGIN_YBOTTOM 20         /* bottom margin               */
#define  COLOR_CLIENT_BACKGROUND RGB (0,150,150) /* DARK CYAN         */


/**********************************************************************/
/* Calculate the byte offset of a field in a structure of type type. */
/**********************************************************************/

#define FLDOFFSET(type, field)    ((WORD)&(((type *)0)->field))


/**********************************************************************/
/* The following structure defines a container item, including all    */
/* details view data                                                  */
/**********************************************************************/

typedef struct _USERDATA {
  RECORDCORE  recordCore;
  WORD        wIndex;
  LPSTR       lpSocialSec;
  WORD        wHourlyWage;
  LPSTR       lpTask;
  LPSTR       lpNickName;
} USERDATA;
typedef USERDATA FAR *LPUSERDATA;

/**********************************************************************/
/* The following structure defines an internal representation of      */
/* all of the objects in our container                                */
/**********************************************************************/

typedef struct _DATAINFO {
  WORD  wItemType;
  char  *pItemName;
  char  *pTask;
  char  *pNickName;
  WORD  wParentIdx;
  char  *pSocialSec;
  WORD  wHourlyWage;
  WORD  fDirectEdit;                   /* Used to track direct edit   */
} DATAINFO;

#define  ITEM_TYPE_PERSON 1            /* person type                 */
#define  ITEM_TYPE_DEPT 2              /* department type             */

#define  DE_NAME_EDITED      0x0001    /* name edited                 */
#define  DE_SOCSEC_EDITED    0x0002    /* social security edited      */
#define  DE_TASK_EDITED      0x0004    /* task edited                 */
#define  DE_NICKNAME_EDITED  0x0008    /* nickname edited             */

/**********************************************************************/
/* prototypes                                                         */
/**********************************************************************/
LRESULT CALLBACK ClientWndProc(HWND, UINT, WPARAM, LPARAM );
int  FAR PASCAL RecordCoreCompare (LPRECORDCORE, LPRECORDCORE, LPVOID);

BOOL   ancestorOf (WORD, WORD);
int    childCount(WORD);
BOOL   createContainer (HWND, WORD);
LPRECORDCORE createContainerItemTree (HWND, RECORDINSERT, WORD, LPPOINT);
VOID   createDetailsView (HWND);
LPFIELDINFO createDetailsViewItems (HWND, LPFIELDINFOINSERT);
VOID   createIconView (HWND);
VOID   createNameView (HWND);
VOID   createTextView (HWND);
VOID   createTreeIconView (HWND);
VOID   destroyClient (HWND, HWND);
VOID   destroyContainer (HWND);
VOID   destroyContainerItemTree (HWND, LPRECORDCORE);
VOID   destroyDetailsView (HWND);
VOID   destroyIconView (HWND);
VOID   destroyNameView (HWND);
VOID   destroyTextView (HWND);
VOID   destroyTreeIconView (HWND);
BOOL   drawContainerItem (LPDRAWITEMSTRUCT);
LPRECORDCORE getContainerRecord (HWND, LPRECORDCORE, WORD);
WORD   getRecordIndex (LPRECORDCORE);
BOOL   handleReallocPsz (LPCNREDITDATA, HWND, BOOL, BOOL);
VOID   initClient (HWND, HWND, HWND, WORD);
HWND   newLevel (HWND, WORD, WORD);
long   rtJustCharStringAt (HDC, LPPOINT, long, char *);
VOID   setViewNames (LPRECORDCORE, WORD);

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // CNRWIN header guard
