/* Revision: 29 1.2.2.2 source/ui/cclnt/cppwot3.c, cclnt, ioc.v400, 000302  */
/*******************************************************************************
* FILE NAME: cppwot3.c                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   DLL initialization code for controls DLL                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: CPPWOT3.C - Initialization code for the Win32 CCL DLL.           */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  -----  --------------------------------------------------- */
/* MAB2  01/23/96  6714   Initialization of cppwot3.dll failing sporatically. */
/* RKK1  04/30/96  7680   Add changes required for Wind/U.                    */
/******************************************************************************/
#include <windows.h>

#ifdef IC_WU                           //->@RKK1
#ifdef DEBUG
#include <stdio.h>
#endif
#endif                             //@RKK1<-

/*******************************************************************************
    This module is only needed when the controls are being built into
    a single DLL.  To build separate DLLs you need to define
    CCL_SEPARATE_DLL on the compile command line.
*******************************************************************************/
#if !defined(CCL_SEPARATE_DLL)


/* Declare the initialization routines for the individual controls */
INT  APIENTRY ContainerLibMain(HANDLE hInstance,
                               DWORD ul_reason_being_called,
                               LPVOID lpReserved);

INT  APIENTRY NotebookLibMain(HANDLE hInstance,
                              DWORD ul_reason_being_called,
                              LPVOID lpReserved);

INT  APIENTRY SliderLibMain(HANDLE hInstance,
                            DWORD ul_reason_being_called,
                            LPVOID lpReserved);

INT  APIENTRY CircularSliderLibMain(HANDLE hInstance,
                                    DWORD ul_reason_being_called,
                                    LPVOID lpReserved);

INT  APIENTRY SpinbuttonLibMain(HANDLE hInstance,
                                DWORD ul_reason_being_called,
                                LPVOID lpReserved);

//_TlsWin32s is a function that needs to be called for the Win32s versions
//of our dlls.  It needs to be the first call in PROCESS_ATTACH and the last
//call in PROCESS_DETACH.  This will properly initialize the Thread Local
//Storage environment used for Win32s.
#ifdef _WIN32S
unsigned _TlsWin32s (DWORD ulFlag);
#endif

#ifndef _WIN32S
// Used by the page tuning tools
HANDLE        CPPOO_dllHandle;
#endif

// Debug function and macro
void  WriteErrorOutput( const char* message );


// Segment definitions
#ifdef IC_PAGETUNE
  #define _CPPWOT3_C_
  #include "icltune.h"
#endif

#if defined(__IBMC__)


/* _CRT_init is the C run-time environment initialization function.         */
/* It will return 0 to indicate success and -1 to indicate failure.         */
int _CRT_init  (void);
void _CRT_term (void);

/* _DLL_InitTerm is the function that gets called by the operating system   */
/* loader when it loads and frees this DLL for each process that accesses   */
/* this DLL.  However, it only gets called the first time the DLL is loaded */
/* and the last time it is freed for a particular process.  The system      */
/* linkage convention MUST be used because the operating system loader is   */
/* calling this function.                                                   */
/* See DllEntryPoint in SDK documentation                                   */

#ifdef IC_WU                           //->@RKK1
/* Wind/U currently only supports a DllEntryPoint of DllMain() */

INT APIENTRY          DllMain(HANDLE hInstance,
               DWORD  reasonCalled,
               LPVOID reserved );

INT APIENTRY          DllMain(HANDLE hInstance,
               DWORD  reasonCalled,
               LPVOID reserved )
#else                              //@RKK1<-

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved );

INT APIENTRY          _DLL_InitTerm(HANDLE hInstance,
                                    DWORD  reasonCalled,
                                    LPVOID reserved )
#endif                                 //@RKK1
{
   INT  rc = FALSE;     /* failed */
   switch (reasonCalled)
      {
      case  DLL_PROCESS_ATTACH:
#ifdef _WIN32S
         _TlsWin32s(reasonCalled);
#endif
#ifndef _WIN32S
         CPPOO_dllHandle = hInstance; // DLL being loaded
#endif
         /*******************************************************************/
         /* The C run-time environment initialization function must be      */
         /* called before any calls to C run-time functions that are not    */
         /* inlined.                                                        */
         /*******************************************************************/
#ifndef IC_WU                          //@RKK1

         if (_CRT_init() != -1)
            {
#endif                             //@RKK1
            /* initialize the controls */
            rc = ContainerLibMain( hInstance, reasonCalled, reserved );
            if (!rc) WriteErrorOutput("ContainerLibMain failed\n");
            if (rc)
               {
               rc = NotebookLibMain( hInstance, reasonCalled, reserved );
               if (!rc) WriteErrorOutput("NotebookLibMain failed\n");
               }
            if (rc)
               {
               rc = SliderLibMain( hInstance, reasonCalled, reserved );
               if (!rc) WriteErrorOutput("SliderLibMain failed\n");
               }
            if (rc)
               {
               rc = CircularSliderLibMain( hInstance, reasonCalled, reserved );
               if (!rc) WriteErrorOutput("CircularSliderLibMain failed\n");
               }
            if (rc)
               {
               rc = SpinbuttonLibMain( hInstance, reasonCalled, reserved );
               if (!rc) WriteErrorOutput("SpinbuttonLibMain failed\n");
               }
#ifndef IC_WU                          //@RKK1
            }
         else
            {
            WriteErrorOutput("_CRT_Init failed\n");
            }
#endif                             //@RKK1
         break;

      case  DLL_THREAD_ATTACH:
      case  DLL_THREAD_DETACH:
         /* We currently don't do anything with these */
         break;

      case  DLL_PROCESS_DETACH:

         // Call notebook to give it a chance to clean up hooks
         rc = NotebookLibMain( hInstance, reasonCalled, reserved );    //->@MAB2
         if (!rc)
           WriteErrorOutput("NotebookLibMain (detach) failed\n");      //@MAB2<-

#ifndef IC_WU                          //@RKK1
         _CRT_term();
#endif                             //@RKK1
#ifdef _WIN32S
         _TlsWin32s(reasonCalled);
#endif
         break;
      }
   return rc;
}

#endif  // IBMC

/* Debug function to write message to standard error */
#pragma library ("advapi32.lib")
void  WriteErrorOutput( const char* message )
{
   /* Log the event in the Application event log */
   HANDLE  hlog = RegisterEventSource( 0,           /* local computer */
                                       "cppwot3" ); /* source name    */
   if (hlog)
      {
      const char* strArray[1] = {message};
      ReportEvent(
        hlog,                 /* event log handle            */
        EVENTLOG_ERROR_TYPE,  /* event type                  */
        0,                    /* category zero               */
        0x1,                  /* event identifier            */
        0,                    /* no user security identifier */
        1,                    /* one string                  */
        0,                    /* no data                     */
        strArray,             /* address of string array     */
        0);                   /* address of data             */

      DeregisterEventSource(hlog);
      }

#ifdef DEBUG
   {
#ifdef IC_WU                           //->@RKK1
      fprintf( stderr, "%lu\n", message );
#else                              //@RKK1<-
      HANDLE  herr = GetStdHandle( STD_ERROR_HANDLE );
      DWORD   bytesToWrite = strlen( message );
      DWORD   bytesWritten;
      if (herr != INVALID_HANDLE_VALUE)
        {
        WriteFile(herr, message, bytesToWrite, &bytesWritten, 0);
        }
#endif                             //@RKK1
   }
#endif
   return;
}

#endif // CCL_SEPARATE_DLL
