/* Revision: 33 1.2.2.2 source/ui/cclnt/i32to16.c, cclnt, ioc.v400, 000302  */
/*******************************************************************************
* FILE NAME: i32to16.c                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the special functions for message handling in           *
*   the Win32s environment.                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
* NOTE:                                                                        *
*    This code written in C, rather than C++, because it is for CCL (which is  *
*    C code) and could be incorporated later, directly into the CCL DLL        *
*                                                                              *
*******************************************************************************/
#include <windows.h>

#include <iclccl.h>

/*********************************************************************/
/* Function prototypes                                               */
/*********************************************************************/

WORD getFreeCacheIndex(VOID);


/*********************************************************************/
/* Types/defines used in this module                                 */
/*********************************************************************/
#define  CACHE_SIZE   100

typedef struct {
   DWORD wparam;
   LPARAM lparam;
   UINT  Msg;
} S32TO16;

DWORD     wCacheTableIndex = 0;

static S32TO16 dw32to16CacheTable[CACHE_SIZE];  /*static to set to 0x00*/


/**********************************************************************/
/*  Table lookup to resolve message                                   */
/*                                                                    */
/**********************************************************************/
LRESULT WINAPI DllS16to32Message(HWND hWnd, unsigned int * Msg,
                              DWORD * wParam, LPARAM * lParam)
{
  wCacheTableIndex = LOWORD(*wParam);

  *Msg = dw32to16CacheTable[wCacheTableIndex].Msg;
  *lParam = dw32to16CacheTable[wCacheTableIndex].lparam;
  *wParam = dw32to16CacheTable[wCacheTableIndex].wparam;

  dw32to16CacheTable[wCacheTableIndex].lparam = 0;
  dw32to16CacheTable[wCacheTableIndex].wparam = 0;
  dw32to16CacheTable[wCacheTableIndex].Msg = 0;

  return(0);
}


/**********************************************************************/
/*  Place message in table                                            */
/*                                                                    */
/**********************************************************************/
LRESULT WINAPI DllS32to16SendMessage(HWND hWnd, UINT Msg,
                                  WPARAM wParam, LPARAM lParam)
{
  //return first available slot in table
  wCacheTableIndex = getFreeCacheIndex();

  //Put Msg in table
  dw32to16CacheTable[wCacheTableIndex].Msg = Msg;

  //Put lParam in table (special for WM_COMMAND coming from CCL)
  dw32to16CacheTable[wCacheTableIndex].lparam = lParam;

  //Put wParam value in table
  dw32to16CacheTable[wCacheTableIndex].wparam = wParam;

  //Create new wParam with table index (LOWORD,HIWORD)
  wParam = MAKEWPARAM(wCacheTableIndex,0);

  if (Msg == WM_COMMAND) //&& CCL notification code
    {
     //had to come from CCL
     return(SendMessage(hWnd, IC_UM_CMD_CCLTOICLUI_32TO16, wParam, lParam));
    }
  else  // User msg or CCL msg
    {
     return(SendMessage(hWnd, IC_UM_CCL_32TO16, wParam, lParam));
    }
}


/**********************************************************************/
/*  Place message in table                                            */
/*                                                                    */
/**********************************************************************/
LRESULT WINAPI DllS32to16PostMessage(HWND hWnd, UINT Msg,
                                  WPARAM wParam, LPARAM lParam)
{
  //return first available slot in table
  wCacheTableIndex = getFreeCacheIndex();

  //Put Msg in table
  dw32to16CacheTable[wCacheTableIndex].Msg = Msg;

  //Put lParam in table (special for WM_COMMAND coming from CCL)
  dw32to16CacheTable[wCacheTableIndex].lparam = lParam;

  //Put wParam value in table
  dw32to16CacheTable[wCacheTableIndex].wparam = wParam;

  //Create new wParam with table index (LOWORD,HIWORD)
  wParam = MAKEWPARAM(wCacheTableIndex,0);

  if (Msg == WM_COMMAND) //&& CCL notification code
    {
     //had to come from CCL
     return(PostMessage(hWnd, IC_UM_CMD_CCLTOICLUI_32TO16, wParam, lParam));
    }
  else  // User msg or CCL msg
    {
     return(PostMessage(hWnd, IC_UM_CCL_32TO16, wParam, lParam));
    }
}


/**********************************************************************/
/*  Get table index                                                   */
/*                                                                    */
/**********************************************************************/
WORD getFreeCacheIndex(VOID)
{
  //Search the table until you find an empty entry.
  //An entry with a 0 value is considered empty, because a 0 value
  //would never be put into the table.
  WORD i;

  for (i=0; ((i <= CACHE_SIZE) && (dw32to16CacheTable[i].wparam != 0)); i++) ;

  return(i);
}

/**********************************************************************/
/*   CCL32to16SendMessage                                             */
/*                                                                    */
/*   If the message to be sent is destined for the CCL controls       */
/*   then call a special function in a .DLL which then calls          */
/*   SendMessage                                                      */
/*   This is due to the fact the CCL messages are user messages       */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
LRESULT WINAPI CCL32to16SendMessage(HWND hWnd, UINT Msg,
                                  WPARAM wParam, LPARAM lParam)
{
   unsigned long rc;
   /*************************************************************/
   /* If it turns out that all the notification codes have a    */
   /* handle as the lparam, that means the 2nd portion          */
   /* (the WM_COMMAND... test) of this test will never be true. */
   /* But if we ever changed a code, or added a new             */
   /* notification code that had some 32-bit value in the       */
   /* lParam, other than a handle, (container has this)         */
   /* then this test would pass, and the .DLL code would be     */
   /* called.                                                   */
   /*    On a 16-bit windows environment (Win32s) a handle is a */
   /* short.  The Win-32 SDK documents that the WM_COMMAND      */
   /* message should hold a handle in the lParam.  Therefore,   */
   /* 16-bit Windows does not bother to preserve the HIWORD of  */
   /* lParam, since it should be 0.  (It is 0 when lParam       */
   /* contains an handle and the run-time environment is Win32s.*/
   /* Hence the reason for this test -- If we use the lParam    */
   /* for a 32-bit value instead, this .DLL code will preserve  */
   /* the value, by storing it in a table.  The .DLL is called  */
   /* on message retrieval (in ICLUI, and here in CLL) as       */
   /* well.                                                     */
   /*************************************************************/
   if ( (IS_WIN32S) &&
        ( ( (HIWORD(wParam) != 0) &&
            (Msg >= CCL_MESSAGE_BASE) &&
            (Msg <= CCL_MESSAGE_TOP) ) ||
            ( (Msg == WM_COMMAND) &&
              (!IsWindow((HWND)lParam)) &&
              (HIWORD(wParam) >= CCL_NOTIFY_CODE_BASE) &&
              (HIWORD(wParam) <= CCL_NOTIFY_CODE_TOP) ) ) )
      {
       rc = DllS32to16SendMessage(hWnd, Msg, wParam, lParam);
      }
   else
      {
       rc = SendMessage(hWnd, Msg, wParam, lParam);
      }
   return(rc);
}

/**********************************************************************/
/*   CCL32to16PostMessage                                             */
/*                                                                    */
/*   If the message to be sent is destined for the CCL controls       */
/*   then call a special function in a .DLL which then calls          */
/*   SendMessage                                                      */
/*   This is due to the fact the CCL messages are user messages       */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
LRESULT WINAPI CCL32to16PostMessage(HWND hWnd, UINT Msg,
                                  WPARAM wParam, LPARAM lParam)
{
   unsigned long rc;
   /*************************************************************/
   /* If it turns out that all the notification codes have a    */
   /* handle as the lparam, that means the 2nd portion          */
   /* (the WM_COMMAND... test) of this test will never be true. */
   /* But if we ever changed a code, or added a new             */
   /* notification code that had some 32-bit value in the       */
   /* lParam, other than a handle, (container has this)         */
   /* then this test would pass, and the .DLL code would be     */
   /* called.                                                   */
   /*    On a 16-bit windows environment (Win32s) a handle is a */
   /* short.  The Win-32 SDK documents that the WM_COMMAND      */
   /* message should hold a handle in the lParam.  Therefore,   */
   /* 16-bit Windows does not bother to preserve the HIWORD of  */
   /* lParam, since it should be 0.  (It is 0 when lParam       */
   /* contains an handle and the run-time environment is Win32s.*/
   /* Hence the reason for this test -- If we use the lParam    */
   /* for a 32-bit value instead, this .DLL code will preserve  */
   /* the value, by storing it in a table.  The .DLL is called  */
   /* on message retrieval (in ICLUI, and here in CLL) as       */
   /* well.                                                     */
   /*************************************************************/
   if ( (IS_WIN32S) &&
        ( ( (HIWORD(wParam) != 0) &&
            (Msg >= CCL_MESSAGE_BASE) &&
            (Msg <= CCL_MESSAGE_TOP) ) ||
            ( (Msg == WM_COMMAND) &&
              (!IsWindow((HWND)lParam)) &&
              (HIWORD(wParam) >= CCL_NOTIFY_CODE_BASE) &&
              (HIWORD(wParam) <= CCL_NOTIFY_CODE_TOP) ) ) )
      {
       rc = DllS32to16PostMessage(hWnd, Msg, wParam, lParam);
      }
   else
      {
       rc = PostMessage(hWnd, Msg, wParam, lParam);
      }
   return(rc);
}
