/* Revision: 40 1.6.1.3 source/ui/cclnt/iclspbw.h, cclspin, ioc.v400, 000302  */
/* NOSHIP */
#ifndef _ICLSPBW_
#define _ICLSPBW_
/*******************************************************************************
* FILE NAME: iclspbw.h                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the header information for the 32-bit version of        *
*    the spin button control.                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#ifndef CCL_MESSAGE_BASE
  #define CCL_MESSAGE_BASE ( 0xF000 )
#endif

#ifndef _ICLCLB_
	#include <iclclb.h>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

#pragma enum(4)
#pragma pack(push,4)

/**********************************************************************/
/* Spinbutton Window Class                                            */
/**********************************************************************/
#define ICL_SPINBTN     "ICL Spin Button"

#if defined(CCL_SEPARATE_DLL)
#define ICL_SPINBTN_DLL "ICLSPBW.DLL"
#else
#define ICL_SPINBTN_DLL ICL_DLL
#endif

/**********************************************************************/
/* SPINBUTTON Creation Flags                                          */
/**********************************************************************/

/**********************************************************************/
/* Character Acceptance                                               */
/**********************************************************************/
#define SPBS_ALLCHARACTERS 0x00000000L /* Default: All chars accepted */
#define SPBS_NUMERICONLY   0x00000001L /* Only 0 - 9 accepted & VKeys */
#define SPBS_READONLY      0x00000002L /* No chars allowed in entryfld*/

/**********************************************************************/
/* Type of Component                                                  */
/**********************************************************************/
#define SPBS_MASTER        0x00000010L
#define SPBS_SERVANT       0x00000000L /* Default: Servant            */

/**********************************************************************/
/* Type of Justification                                              */
/**********************************************************************/
#define SPBS_JUSTDEFAULT  0x00000000L /* Default: Same as Left        */
#define SPBS_JUSTLEFT     0x00000008L
#define SPBS_JUSTRIGHT    0x00000004L
#define SPBS_JUSTCENTER   0x0000000CL

/**********************************************************************/
/* Border or not                                                      */
/**********************************************************************/
#define SPBS_NOBORDER     0x00000020L /* Borderless SpinField         */
                                      /* Default is to have a border. */

/**********************************************************************/
/* Fast spin or not                                                   */
/**********************************************************************/
#define SPBS_FASTSPIN     0x00000100L /* Allow fast spinning.  Fast   */
                                      /* spinning is performed by     */
                                      /* skipping over numbers        */

/**********************************************************************/
/* Pad numbers on front with 0's                                      */
/**********************************************************************/
#define SPBS_PADWITHZEROS 0x00000080L /* Pad the number with zeroes   */

/**********************************************************************/
/* SPINBUTTON Messages                                                */
/**********************************************************************/

/**********************************************************************/
/* Notification from Spinbutton to the application is sent in a       */
/* WM_COMMAND message.                                                */
/**********************************************************************/
/**********************************************************************/
/* Note that these notification codes are in a specific range.  When  */
/* adding a new code, be sure to add within the CCL ranges defined    */
/* in iclccl.h.  For SpinButton, limit the notification code to       */
/* 150 to 159.  Other CCL controls use other sections of the entire   */
/* range defined in iclccl.h.                                         */
/**********************************************************************/
#define SPBN_UPARROW       150        /* up arrow button was pressed  */
#define SPBN_DOWNARROW     151        /* down arrow button was pressed*/
#define SPBN_ENDSPIN       152        /* mouse button was released    */
#define SPBN_CHANGE        153        /* spinfield text has changed   */
#define SPBN_SETFOCUS      154        /* spinfield received focus     */
#define SPBN_KILLFOCUS     155        /* spinfield lost focus         */

/**********************************************************************/
/* Messages from application to Spinbutton                            */
/**********************************************************************/
/**********************************************************************/
/* Note that these message are in a specific range.  When adding a    */
/* new message, be sure to add within the CCL message ranges defined  */
/* in iclccl.h.  For SpinButton, limit the message to the range       */
/* CCL_MESSAGE_BASE+0x00a0 to CCL_MESSAGE_BASE+0x00af.  Other CCL     */
/* controls use other sections of the entire range defined in         */
/* iclccl.h                                                           */
/**********************************************************************/
#define SPBM_OVERRIDESETLIMITS (CCL_MESSAGE_BASE+0x00a0) /* Set spinbtn limits   */
#define SPBM_QUERYLIMITS       (CCL_MESSAGE_BASE+0x00a1) /* Query spinbtn limits */
#define SPBM_SETTEXTLIMIT      (CCL_MESSAGE_BASE+0x00a2) /* Max entryfield chars */
#define SPBM_SPINUP            (CCL_MESSAGE_BASE+0x00a3) /* spin entry field up  */
#define SPBM_SPINDOWN          (CCL_MESSAGE_BASE+0x00a4) /* spin entry field down*/
#define SPBM_QUERYVALUE        (CCL_MESSAGE_BASE+0x00a5) /* Tell entry field to  */
                                                         /*  send current value  */
#define SPBM_SETARRAY          (CCL_MESSAGE_BASE+0x00a6) /* Change data to spin  */
#define SPBM_SETLIMITS         (CCL_MESSAGE_BASE+0x00a7) /* Change numeric Limits*/
#define SPBM_SETCURRENTVALUE   (CCL_MESSAGE_BASE+0x00a8) /* Change current value */
#define SPBM_SETMASTER         (CCL_MESSAGE_BASE+0x00a9) /* Tell entryfield who  */
                                                         /* its master is        */
#define SPBM_QUERYSPINCOLORS   (CCL_MESSAGE_BASE+0x00aa) /* Query colors of a    */
                                                         /*  spinbtn             */
#define SPBM_SETSPINCOLORS     (CCL_MESSAGE_BASE+0x00ab) /* Set colors of a      */
                                                         /*  spinbtn             */
#define SPBM_RESETSPINCOLORS   (CCL_MESSAGE_BASE+0x00ac) /* Reset colors of a    */
                                                         /*  spinbtn             */

/**********************************************************************/
/* Query Flags                                                        */
/**********************************************************************/
#define SPBQ_UPDATEIFVALID    0       /* Default                      */
#define SPBQ_ALWAYSUPDATE     1
#define SPBQ_DONOTUPDATE      3

/**********************************************************************/
/* Attributes for setting the colors of a spinbtn                     */
/**********************************************************************/
#define SPBA_FOREGROUNDCOLOR  0x0001
#define SPBA_BACKGROUNDCOLOR  0x0002

/**********************************************************************/
/* Spin button limits structure                                       */
/**********************************************************************/
typedef struct _SPBLIMITS
{
   LONG       lUpperLimit;
   LONG       lLowerLimit;
} SPBLIMITS;
typedef SPBLIMITS FAR *LPSPBLIMITS;

/**********************************************************************/
/* Spin button query value structure                                  */
/**********************************************************************/
typedef struct _SPBQUERYVALUE
{
   LPVOID     lpValueBuf;
   WORD       wBufSize;
   WORD       wValue;
} SPBQUERYVALUE;
typedef SPBQUERYVALUE FAR *LPSPBQUERYVALUE;

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // _ICLSPBW_

