/* Revision: 43 1.5 source/ui/cclnt/notebook/bookdlg.c, cclbook, ioc.v400, 000302  */
/*---------------------------------------------------------------------------*
 *
 *                              IBM CONFIDENTIAL
 *
 *            (IBM Confidential-Restricted when combined with the
 *              aggregated OCO source modules for this program)
 *
 *                            OCO SOURCE MATERIALS
 *
 *             20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)
 *
 *---------------------------------------------------------------------------*
 * bookdlg.c
 *
 * Notebook interface routines for the Windows SDK Dialog Editor.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

#include <windows.h>
#include <custcntl.h>
#include <string.h>
#include "dlgedit.h"
#include "pmtowin.h"
#include "iclnbw.h"
#include "bookp.h"
#include "bookdlg.h"

HANDLE FAR PASCAL BookInfo  (void);
BOOL   FAR PASCAL BookStyle (HWND, HANDLE, LPFNSTRTOID, LPFNIDTOSTR);
WORD   FAR PASCAL BookFlags (WORD, LPSTR, WORD);
BOOL   FAR PASCAL BookDlgFn (HWND, WORD, WORD, DWORD);

void   EnableMajors (HWND, WORD, WORD, WORD, WORD, WORD);
WORD   QueryStyles  (HWND);
void   SetStyles    (HWND, DWORD);
LPSTR  QueryBackpageString   (WORD fs);
LPSTR  QueryMajorTabString   (WORD fs);
LPSTR  QueryTabTextString    (WORD fs);
LPSTR  QueryStatusTextString (WORD fs);
LPSTR  QueryTabStyleString   (WORD fs);
LPSTR  QueryBindingString    (WORD fs);

extern HANDLE vhInstance;


/*---------------------------------------------------------------------------*
 * BookInfo
 *
 * Allocates a CTLINFO structure from the global heap to identify the
 * controls that are supported by this DLL.  The caller of this function
 * is responsible for freeing the structure.
 *
 * This function must be ordinal 2.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

HANDLE FAR PASCAL BookInfo (void)
{
  HANDLE    hCtlInfo;
  LPCTLINFO lpCtlInfo;

  /*-----------------------------------------------------*/
  /* allocate a CTLINFO that the Dialog Editor will free */
  /*-----------------------------------------------------*/
  if (hCtlInfo = GlobalAlloc (GHND, sizeof (CTLINFO)))
  {
    /*-----------------------------------*/
    /* lock it down so we can address it */
    /*-----------------------------------*/
    if (lpCtlInfo = (LPCTLINFO) GlobalLock (hCtlInfo))
    {
      lpCtlInfo->wVersion        = CCL_VERSION;
      lpCtlInfo->wCtlTypes       = 1;

      lpCtlInfo->Type[0].wType   = 0;
      lpCtlInfo->Type[0].wWidth  = 75;
      lpCtlInfo->Type[0].wHeight = 75;
      lpCtlInfo->Type[0].dwStyle = WS_CHILD | WS_CLIPCHILDREN;

      lstrcpy (lpCtlInfo->szClass, ICL_NOTEBOOK);
      lstrcpy (lpCtlInfo->szTitle, CCL_COPYRIGHT);
      lstrcpy (lpCtlInfo->Type[0].szDescr, ICL_NOTEBOOK);

      GlobalUnlock (hCtlInfo);
    }

    /*---------------------------------------------------------------*/
    /* otherwise, we couldn't lock it down to initialize it, so punt */
    /*---------------------------------------------------------------*/
    else
    {
      GlobalFree (hCtlInfo);
      hCtlInfo = NULL;
    }
  }

  return (hCtlInfo);
}


/*---------------------------------------------------------------------------*
 * BookStyle
 *
 * This function displays a modal dialog that is to collect the style of
 * the notebook from the user.
 *
 * This function must be ordinal 3.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

BOOL FAR PASCAL BookStyle (HWND hwnd, HANDLE hCtlStyle,
                           LPFNSTRTOID lpfnStrToId, LPFNIDTOSTR lpfnIdToStr)
{
  PSTYLESTRUCT pss;
  BOOL         rc;

  if (!(pss = BookAllocMem (sizeof (STYLESTRUCT))))  return (FALSE);

  pss->hCtlStyle   = hCtlStyle;
  pss->lpCtlStyle  = NULL;
  pss->lpfnStrToId = lpfnStrToId;
  pss->lpfnIdToStr = lpfnIdToStr;

  rc = DialogBoxParam (vhInstance, MAKEINTRESOURCE (ID_NBSTYLES), hwnd,
                       (FARPROC) BookDlgFn, (WORD) pss);

  BookFreeMem (pss);

  return (rc);
}


/*---------------------------------------------------------------------------*
 * BookDlgFn
 *
 * This function must be ordinal 6.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

BOOL FAR PASCAL BookDlgFn (HWND hwnd, WORD msg, WORD wParam, DWORD lParam)
{
  PSTYLESTRUCT pss;
  DWORD        dwRet;
  char         szID[33];

  switch (msg)
  {
    case WM_INITDIALOG:
      pss = (PSTYLESTRUCT) lParam;

      /*----------------------------------------------------------------*/
      /* set the property that contains the pointer to the STYLESTRUCT, */
      /* lock down the CTLSTYLE structure, and make sure this is really */
      /* a notebook; if any of these failed, bail out now               */
      /*----------------------------------------------------------------*/
      if (!SetProp (hwnd, MAKEINTRESOURCE (1), (HANDLE) pss) ||
          !(pss->lpCtlStyle = (LPCTLSTYLE) GlobalLock (pss->hCtlStyle)))
      {
        EndDialog (hwnd, FALSE);
        break;
      }

      /*-------------------------*/
      /* initialize the ID field */
      /*-------------------------*/
      if ((*pss->lpfnIdToStr) (pss->lpCtlStyle->wId, szID, sizeof (szID)))
      {
        SetDlgItemText (hwnd, ID_ID, szID);
      }
      else
      {
        SetDlgItemInt (hwnd, ID_ID, pss->lpCtlStyle->wId, FALSE);
      }

      /*-------------------------------------*/
      /* set the text limit for the ID field */
      /*-------------------------------------*/
      SendDlgItemMessage (hwnd, ID_ID, EM_LIMITTEXT, sizeof (szID)-1, 0);

      /*---------------------------------*/
      /* check the default style buttons */
      /*---------------------------------*/
      SetStyles (hwnd, pss->lpCtlStyle->dwStyle);
      break;

    case WM_DESTROY:
      if (pss = (PSTYLESTRUCT) RemoveProp (hwnd, MAKEINTRESOURCE (1)))
      {
        if (pss->lpCtlStyle)  GlobalUnlock (pss->hCtlStyle);
      }
      break;

    case WM_COMMAND:
      switch (wParam)
      {
        /*-------------------------------------------------------------*/
        /* make sure only the valid major tab orientations are enabled */
        /* for any given selection of back page orientation            */
        /*-------------------------------------------------------------*/
        case ID_BACKPAGESBR:
          EnableMajors (hwnd, ID_MAJORTABSLEFT, ID_MAJORTABSTOP,
                        ID_MAJORTABSRIGHT, ID_MAJORTABSBOTTOM,
                        ID_MAJORTABSRIGHT);
          break;

        case ID_BACKPAGESBL:
          EnableMajors (hwnd, ID_MAJORTABSRIGHT, ID_MAJORTABSTOP,
                        ID_MAJORTABSBOTTOM, ID_MAJORTABSLEFT,
                        ID_MAJORTABSBOTTOM);
          break;

        case ID_BACKPAGESTR:
          EnableMajors (hwnd, ID_MAJORTABSLEFT, ID_MAJORTABSBOTTOM,
                        ID_MAJORTABSTOP, ID_MAJORTABSRIGHT,
                        ID_MAJORTABSTOP);
          break;

        case ID_BACKPAGESTL:
          EnableMajors (hwnd, ID_MAJORTABSRIGHT, ID_MAJORTABSBOTTOM,
                        ID_MAJORTABSLEFT, ID_MAJORTABSTOP,
                        ID_MAJORTABSLEFT);
          break;

        case IDOK:
          pss = (PSTYLESTRUCT) GetProp (hwnd, MAKEINTRESOURCE (1));

          /*----------------------------------------------*/
          /* if the ID field was modified, get the new ID */
          /*----------------------------------------------*/
          if (SendDlgItemMessage (hwnd, ID_ID, EM_GETMODIFY, 0, 0))
          {
            GetDlgItemText (hwnd, ID_ID, szID, sizeof (szID));

            /*---------------------------------------------------*/
            /* if the new ID is not a defined constant, bail out */
            /*---------------------------------------------------*/
            if (!LOWORD (dwRet = (*pss->lpfnStrToId) (szID)))  break;

            pss->lpCtlStyle->wId = HIWORD (dwRet);
          }

          LOWORD (pss->lpCtlStyle->dwStyle) = QueryStyles (hwnd);
          /* flow-through */

        case IDCANCEL:
          EndDialog (hwnd, (wParam == IDOK));
          break;
      }
      break;
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * EnableMajors
 *
 * Enables the appropriate major tab orientations for a back page orientation.
 * If the currently selected major tab orientation is invalid for the chosen
 * back page orientation, a default major tab orientation is chosen.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

void EnableMajors (HWND hwnd, WORD idInvalid1, WORD idInvalid2,
                   WORD idValid1, WORD idValid2, WORD idDefault)
{
  EnableWindow (GetDlgItem (hwnd, idValid1),   TRUE);
  EnableWindow (GetDlgItem (hwnd, idValid2),   TRUE);
  EnableWindow (GetDlgItem (hwnd, idInvalid1), FALSE);
  EnableWindow (GetDlgItem (hwnd, idInvalid2), FALSE);

  if (IsDlgButtonChecked (hwnd, idInvalid1) ||
      IsDlgButtonChecked (hwnd, idInvalid2))
  {
    CheckRadioButton (hwnd, ID_MAJORTABSRIGHT, ID_MAJORTABSBOTTOM, idDefault);
  }
}


/*---------------------------------------------------------------------------*
 * QueryStyles
 *
 * Queries the styles selected from the dialog box.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

WORD QueryStyles (HWND hwnd)
{
  DWORD flStyle;

  flStyle  = IsDlgButtonChecked (hwnd, ID_BACKPAGESBR)      ? BKS_BACKPAGESBR :
             IsDlgButtonChecked (hwnd, ID_BACKPAGESBL)      ? BKS_BACKPAGESBL :
             IsDlgButtonChecked (hwnd, ID_BACKPAGESTR)      ? BKS_BACKPAGESTR :
                                                              BKS_BACKPAGESTL;

  flStyle |= IsDlgButtonChecked (hwnd, ID_MAJORTABSRIGHT)   ? BKS_MAJORTABRIGHT :
             IsDlgButtonChecked (hwnd, ID_MAJORTABSLEFT)    ? BKS_MAJORTABLEFT  :
             IsDlgButtonChecked (hwnd, ID_MAJORTABSTOP)     ? BKS_MAJORTABTOP   :
                                                              BKS_MAJORTABBOTTOM;

  flStyle |= IsDlgButtonChecked (hwnd, ID_TABTEXTLEFT)      ? BKS_TABTEXTLEFT  :
             IsDlgButtonChecked (hwnd, ID_TABTEXTRIGHT)     ? BKS_TABTEXTRIGHT :
                                                              BKS_TABTEXTCENTER;

  flStyle |= IsDlgButtonChecked (hwnd, ID_STATUSTEXTCENTER) ? BKS_STATUSTEXTCENTER :
             IsDlgButtonChecked (hwnd, ID_STATUSTEXTRIGHT)  ? BKS_STATUSTEXTRIGHT  :
                                                              BKS_STATUSTEXTLEFT;

  flStyle |= IsDlgButtonChecked (hwnd, ID_ROUNDTABS)        ? BKS_ROUNDEDTABS :
             IsDlgButtonChecked (hwnd, ID_POLYGONTABS)      ? BKS_POLYGONTABS :
                                                              BKS_SQUARETABS;

  flStyle |= IsDlgButtonChecked (hwnd, ID_SPIRALBIND)       ? BKS_SPIRALBIND :
                                                              BKS_SOLIDBIND;

  return (LOWORD (flStyle));
}


/*---------------------------------------------------------------------------*
 * SetStyles
 *
 * Sets the styles for the dialog box from what was passed in by the
 * Dialog Editor
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

void SetStyles (HWND hwnd, DWORD flStyle)
{
  WORD idBackPage = (flStyle & BKS_BACKPAGESTL) ? ID_BACKPAGESTL :
                    (flStyle & BKS_BACKPAGESBL) ? ID_BACKPAGESBL :
                    (flStyle & BKS_BACKPAGESTR) ? ID_BACKPAGESTR :
                                                  ID_BACKPAGESBR;

  /*------------------------------------------------------------*/
  /* check the appropriate backpage button and simulate a WM_-  */
  /* COMMAND so the major tab buttons will be enabled correctly */
  /*------------------------------------------------------------*/
  CheckDlgButton (hwnd, idBackPage, TRUE);
  SendMessage (hwnd, WM_COMMAND, idBackPage, NULL);

  CheckDlgButton (hwnd, (flStyle & BKS_MAJORTABBOTTOM)   ? ID_MAJORTABSBOTTOM:
                        (flStyle & BKS_MAJORTABLEFT)     ? ID_MAJORTABSLEFT  :
                        (flStyle & BKS_MAJORTABTOP)      ? ID_MAJORTABSTOP   :
                                                           ID_MAJORTABSRIGHT,
                  TRUE);

  CheckDlgButton (hwnd, (flStyle & BKS_TABTEXTLEFT)      ? ID_TABTEXTLEFT  :
                        (flStyle & BKS_TABTEXTRIGHT)     ? ID_TABTEXTRIGHT :
                                                           ID_TABTEXTCENTER,
                  TRUE);

  CheckDlgButton (hwnd, (flStyle & BKS_STATUSTEXTCENTER) ? ID_STATUSTEXTCENTER :
                        (flStyle & BKS_STATUSTEXTRIGHT)  ? ID_STATUSTEXTRIGHT  :
                                                           ID_STATUSTEXTLEFT,
                  TRUE);

  CheckDlgButton (hwnd, (flStyle & BKS_ROUNDEDTABS)      ? ID_ROUNDTABS   :
                        (flStyle & BKS_POLYGONTABS)      ? ID_POLYGONTABS :
                                                           ID_SQUARETABS,
                  TRUE);

  CheckDlgButton (hwnd, (flStyle & BKS_SPIRALBIND)       ? ID_SPIRALBIND :
                                                           ID_SOLIDBIND,
                  TRUE);
}


/*---------------------------------------------------------------------------*
 * BookFlags
 *
 * This function must be ordinal 4.
 *
 * History:
 * 13-Jan-92 JeffR    Created
 *---------------------------------------------------------------------------*/

WORD FAR PASCAL BookFlags (WORD fs, LPSTR lpStyle, WORD cbMax)
{
  char  szStyles[128];
  LPSTR lpBKS = "BKS";

  /*-------------------------*/
  /* format the style string */
  /*-------------------------*/
  wsprintf (szStyles, "%s_BACKPAGES%s | %s_MAJORTAB%s | %s_TABTEXT%s | "
                      "%s_STATUSTEXT%s | %s_%sTABS | %s_%sBIND",
            lpBKS, QueryBackpageString (fs), lpBKS, QueryMajorTabString (fs),
            lpBKS, QueryTabTextString  (fs), lpBKS, QueryStatusTextString (fs),
            lpBKS, QueryTabStyleString (fs), lpBKS, QueryBindingString (fs));

  /*-------------------------------------------------------------*/
  /* copy the style string we build into DlgEdit's buffer,       */
  /* make sure it's null-terminated, and return the bytes copied */
  /*-------------------------------------------------------------*/
  strncpy (lpStyle, szStyles, cbMax);
  lpStyle[cbMax-1] = '\0';
  return (lstrlen (lpStyle));
}


/*---------------------------------------------------------------------------*
 * QueryBackpageString
 *
 * Returns the string for the backpage style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryBackpageString (WORD fs)
{
  return ((fs & BKS_BACKPAGESBR) ? "BR" :
          (fs & BKS_BACKPAGESBL) ? "BL" :
          (fs & BKS_BACKPAGESTR) ? "TR" : "TL");
}


/*---------------------------------------------------------------------------*
 * QueryMajorTabString
 *
 * Returns the string for the major tab style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryMajorTabString (WORD fs)
{
  return ((fs & BKS_MAJORTABRIGHT) ? "RIGHT" :
          (fs & BKS_MAJORTABLEFT)  ? "LEFT"  :
          (fs & BKS_MAJORTABTOP)   ? "TOP"   : "BOTTOM");
}


/*---------------------------------------------------------------------------*
 * QueryTabTextString
 *
 * Returns the string for the tab text style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryTabTextString (WORD fs)
{
  return ((fs & BKS_TABTEXTRIGHT) ? "RIGHT" :
          (fs & BKS_TABTEXTLEFT)  ? "LEFT"  : "CENTER");
}


/*---------------------------------------------------------------------------*
 * QueryStatusTextString
 *
 * Returns the string for the status text style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryStatusTextString (WORD fs)
{
  return ((fs & BKS_STATUSTEXTRIGHT)  ? "RIGHT"  :
          (fs & BKS_STATUSTEXTCENTER) ? "CENTER" : "LEFT");
}


/*---------------------------------------------------------------------------*
 * QueryTabStyleString
 *
 * Returns the string for the tab style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryTabStyleString (WORD fs)
{
  return ((fs & BKS_ROUNDEDTABS) ? "ROUNDED" :
          (fs & BKS_POLYGONTABS) ? "POLYGON" : "SQUARE");
}


/*---------------------------------------------------------------------------*
 * QueryBindingString
 *
 * Returns the string for the binding style bit in a given flag set.
 *
 * History:
 * 13-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

LPSTR QueryBindingString (WORD fs)
{
  return ((fs & BKS_SPIRALBIND) ? "SPIRAL" : "SOLID");
}
