/* Revision: 45 1.3 source/ui/cclnt/notebook/bookdraw.c, cclbook, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                       Filename: BOOKDRAW.C                         */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: ICLUINT.DLL                                               */
/* Filename: BOOKDRAW.C - General painting routines for the Notebook  */
/*                        control                                     */
/*                                                                    */
/* Flag    Date    Defect  Description                                */
/* ----  --------  -----  ------------------------------------------- */
/* JEH1  06/22/93         Initial modifications for NT execution      */
/* DAB2  10/04/94         Updated notifications to return information */
/*                        supported in PM which was left out of Win3.1*/
/* MAB1  01/06/96  6370   Repaint problems with spiral bindings       */
/**********************************************************************/
#include <windows.h>
#include "iclnbw.h"
#include "bookp.h"

extern USHORT vsDevIndX;

/*---------------------------------------------------------------------------*
 * DrawBackPagesBottom
 *
 *            Ŀ          Ŀ
 *         1Ĵ0                            0Ŀ1
 *                                          
 *                                          
 *                                          
 *                   4            4         
 *                      
 *         2 3          3 2
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBackPagesBottom(
  PBOOK  pbk,                               /* book instance data (locked)   */
  SHORT  sLoop,
  HDC    hdc,
  LPRECT lprcDraw)
{
  POINT  pt[5];
  RECT   rc1, rc2;
  SHORT  cxPageSpacing;                    /* space between back pages  */

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxPageSpacing = BKD_PAGESPACING * vsDevIndX;

  /*-----------------------------------------------------*/
  /* Set up points to draw backpages on the bottom left. */
  /*-----------------------------------------------------*/
  if (pbk->flStyle & BKS_BACKPAGESBL)
  {
    pt[0].x = lprcDraw->left   - cxPageSpacing * (sLoop-1);
    pt[0].y = lprcDraw->top    + cxPageSpacing * sLoop;

    pt[1].x = pt[0].x - cxPageSpacing;
    pt[1].y = pt[0].y;

    pt[2].x = pt[1].x;
    pt[2].y = lprcDraw->bottom + cxPageSpacing * sLoop;

    pt[3].x = lprcDraw->right  - cxPageSpacing * sLoop;
    pt[3].y = pt[2].y;

    pt[4].x = pt[3].x;
    pt[4].y = pt[3].y - cxPageSpacing;

    SetRect (&rc1, pt[1].x, lprcDraw->top, pt[0].x, pt[0].y);
    SetRect (&rc2, pt[3].x+1, pt[4].y+1, lprcDraw->right, pt[3].y+1);
  }

  /*------------------------------------------------------*/
  /* Set up points to draw backpages on the bottom right. */
  /*------------------------------------------------------*/
  else
  {
    pt[0].x = lprcDraw->right  + cxPageSpacing * (sLoop-1);
    pt[0].y = lprcDraw->top    + cxPageSpacing * sLoop;

    pt[1].x = pt[0].x + cxPageSpacing;
    pt[1].y = pt[0].y;

    pt[2].x = pt[1].x;
    pt[2].y = lprcDraw->bottom + cxPageSpacing * sLoop;

    pt[3].x = lprcDraw->left   + cxPageSpacing * sLoop;
    pt[3].y = pt[2].y;

    pt[4].x = pt[3].x;
    pt[4].y = pt[3].y - cxPageSpacing;

    SetRect (&rc1, pt[0].x+1, lprcDraw->top, pt[1].x+1, pt[0].y);
    SetRect (&rc2, lprcDraw->left, pt[4].y+1, pt[3].x+1, pt[3].y+1);
  }

  /*--------------------------*/
  /* Fill in background color */
  /*--------------------------*/
  BookFillRect (hdc, &rc1, pbk->crColor[CLR_NOTEBOOK]);
  BookFillRect (hdc, &rc2, pbk->crColor[CLR_NOTEBOOK]);
  BookPolyline (hdc, pt, 5);
}


/*---------------------------------------------------------------------------*
 * DrawBackPagesTop
 *
 *       2Ŀ1                         1Ŀ2
 *         Ŀ                       Ŀ 
 *                 0                         0         
 *                                                     
 *                                                     
 *                                                     
 *       3Ĵ4                                         43
 *                                 
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBackPagesTop(
  PBOOK  pbk,                               /* book instance data (locked)   */
  SHORT  sLoop,
  HDC    hdc,
  LPRECT lprcDraw)
{
  POINT  pt[5];
  RECT   rc1, rc2;
  SHORT  cxPageSpacing;                    /* space between back pages  */

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxPageSpacing = BKD_PAGESPACING * vsDevIndX;

  /*--------------------------------------------------*/
  /* Set up points to draw backpages on the top left. */
  /*--------------------------------------------------*/
  if (pbk->flStyle & BKS_BACKPAGESTL)
  {
    pt[0].x = lprcDraw->right  - cxPageSpacing * sLoop;
    pt[0].y = lprcDraw->top    - cxPageSpacing * (sLoop-1);

    pt[1].x = pt[0].x;
    pt[1].y = pt[0].y - cxPageSpacing;

    pt[2].x = lprcDraw->left   - cxPageSpacing * sLoop;
    pt[2].y = pt[1].y;

    pt[3].x = pt[2].x;
    pt[3].y = lprcDraw->bottom - cxPageSpacing * sLoop;

    pt[4].x = pt[3].x + cxPageSpacing;
    pt[4].y = pt[3].y;

    SetRect (&rc1, pt[0].x, pt[1].y, pbk->rcBookPages.right, pt[0].y);
    SetRect (&rc2, pt[3].x, pt[3].y, pt[4].x, pbk->rcBookPages.bottom);
  }

  /*--------------------------------------------------*/
  /* Set up points to draw backpages on the top right */
  /*--------------------------------------------------*/
  else
  {
    pt[0].x = lprcDraw->left   + cxPageSpacing * sLoop;
    pt[0].y = lprcDraw->top    - cxPageSpacing * (sLoop-1);

    pt[1].x = pt[0].x;
    pt[1].y = pt[0].y - cxPageSpacing;

    pt[2].x = lprcDraw->right  + cxPageSpacing * sLoop;
    pt[2].y = pt[1].y;

    pt[3].x = pt[2].x;
    pt[3].y = lprcDraw->bottom - cxPageSpacing * sLoop;

    pt[4].x = pt[3].x - cxPageSpacing;
    pt[4].y = pt[3].y;

    SetRect (&rc1, pbk->rcBookPages.left, pt[1].y, pt[0].x, pt[0].y);
    SetRect (&rc2, pt[4].x+1, pt[3].y, pt[3].x+1, pbk->rcBookPages.bottom);
  }

  /*--------------------------*/
  /* Fill in background color */
  /*--------------------------*/
  BookFillRect (hdc, &rc1, pbk->crColor[CLR_NOTEBOOK]);
  BookFillRect (hdc, &rc2, pbk->crColor[CLR_NOTEBOOK]);
  BookPolyline (hdc, pt, 5);
}


/*---------------------------------------------------------------------------*
 * DrawBookBackpages
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBookBackpages(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcTopPage,
  LPRECT lprcInvalid)
{
  SHORT cxBackPageSpacing;
  RECT  rcHorz, rcVert, rcIntersect;
  HPEN  hpenOutline;

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxBackPageSpacing = BKD_NUMVISPAGEEDGES * BKD_PAGESPACING * vsDevIndX;

  switch (LOWORD (pbk->flStyle & FC_BACKPAGES))
  {
    case BKS_BACKPAGESBR:
      SetRect (&rcVert, lprcTopPage->right + 1, lprcTopPage->top,
               lprcTopPage->right + cxBackPageSpacing, lprcTopPage->bottom);
      SetRect (&rcHorz, lprcTopPage->left, lprcTopPage->bottom + 1,
               lprcTopPage->right, lprcTopPage->bottom + cxBackPageSpacing);
      break;

    case BKS_BACKPAGESBL:
      SetRect (&rcVert, lprcTopPage->left - cxBackPageSpacing, lprcTopPage->top,
               lprcTopPage->left, lprcTopPage->bottom);
      SetRect (&rcHorz, lprcTopPage->left, lprcTopPage->bottom + 1,
               lprcTopPage->right, lprcTopPage->bottom + cxBackPageSpacing);
      break;

    case BKS_BACKPAGESTR:
      SetRect (&rcVert, lprcTopPage->right + 1, lprcTopPage->top,
               lprcTopPage->right + cxBackPageSpacing, lprcTopPage->bottom);
      SetRect (&rcHorz, lprcTopPage->left, lprcTopPage->top - cxBackPageSpacing,
               lprcTopPage->right, lprcTopPage->top);
      break;

    case BKS_BACKPAGESTL:
      SetRect (&rcVert, lprcTopPage->left - cxBackPageSpacing, lprcTopPage->top,
               lprcTopPage->left, lprcTopPage->bottom);
      SetRect (&rcHorz, lprcTopPage->left, lprcTopPage->top - cxBackPageSpacing,
               lprcTopPage->right, lprcTopPage->top);
      break;
  }

  /*-------------------------------------------------------------*/
  /* if the backpages intersect the invalid rectangle, draw them */
  /*-------------------------------------------------------------*/
  if ((IntersectRect (&rcIntersect, &rcHorz, lprcInvalid)  ||
       IntersectRect (&rcIntersect, &rcVert, lprcInvalid)) &&
      (hpenOutline = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_OUTLINE])))
  {
    hpenOutline = SelectObject (hdc, hpenOutline);

    /*---------------------------------------*/
    /* fill the background of the back pages */
    /*---------------------------------------*/
    BookFillRect (hdc, &rcHorz, pbk->crColor[CLR_PAGEBG]);
    BookFillRect (hdc, &rcVert, pbk->crColor[CLR_PAGEBG]);
    DrawTopPage (pbk, hdc, lprcTopPage);

    /*---------------------*/
    /* draw the back pages */
    /*---------------------*/
    if (pbk->flStyle & (BKS_BACKPAGESBR | BKS_BACKPAGESBL))
      DrawBookBottom (pbk, hdc, lprcTopPage);
    else
      DrawBookTop (pbk, hdc, lprcTopPage);

    DeleteObject (SelectObject (hdc, hpenOutline));
  }

  /*----------------------------------------------------------------------*/
  /* otherwise, if the top page intersects the invalid rectangle, draw it */
  /*----------------------------------------------------------------------*/
  else if (IntersectRect (&rcIntersect, lprcTopPage, lprcInvalid))
  {
    DrawTopPage (pbk, hdc, lprcTopPage);
  }
}


/*---------------------------------------------------------------------------*
 * DrawBookBinding
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBookBinding(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcUpdate)
{
  RECT    rcToUpdate;
  HBITMAP hbmMem;

  SetRectEmpty( &rcToUpdate );                                           //@MAB1

  /*------------------------------------------------------------------*/
  /* Offset update rectangle for creation of memory bitmap            */
  /*------------------------------------------------------------------*/
  rcToUpdate = pbk->rcBinding;                                           //@MAB1
  OffsetRect( &rcToUpdate, -lprcUpdate->left, -lprcUpdate->top );

  /*------------------------------------------------------------------*/
  /* create our memory DC for pre-drawing                             */
  /*------------------------------------------------------------------*/
  if ( !(hbmMem = CreateCompatibleBitmap( hdc,
                                          rcToUpdate.right,
                                          rcToUpdate.bottom )) )
  {
    return;
  }

  hbmMem = SelectObject (pbk->hdcMem, hbmMem);

  /*------------------------------------------------------------------*/
  /* if we have a spiral binding, draw the spirals                    */
  /*------------------------------------------------------------------*/
  if (pbk->flStyle & BKS_SPIRALBIND)
  {
    DrawSpiralBinding( pbk, pbk->hdcMem, &rcToUpdate );
  }
  else
  {
    /*----------------------------------------------------------------*/
    /* otherwise, just fill the binding region                        */
    /*----------------------------------------------------------------*/
    BookFillRect (pbk->hdcMem, &rcToUpdate, pbk->crColor[CLR_OUTLINE]);
  }

  /*------------------------------------------------------------------*/
  /* Copy memory image to screen.                                     */
  /*------------------------------------------------------------------*/
  BitBlt( hdc, lprcUpdate->left, lprcUpdate->top,
          rcToUpdate.right, rcToUpdate.bottom,
          pbk->hdcMem, 0, 0, SRCCOPY );

  DeleteObject( SelectObject( pbk->hdcMem, hbmMem ) );
}


/*---------------------------------------------------------------------------*
 * DrawSpiralBinding
 *
 * Draws a spiral binding on a notebook.
 *
 * History:
 * 15-Jan-92 JeffR    Created.
 *---------------------------------------------------------------------------*/

void DrawSpiralBinding(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcToUpdate)
{
  RECT   rcPage, rcBook;
  UINT   cIcons, xPos, yPos, i;                                         /*JEH1*/
  USHORT id;                                                            /*JEH1*/
  HPEN   hpen;
  HICON  hIcon;                                                         /*JEH1*/
  POINT  pt[3];

  /*------------------------------------------------------------------*/
  /* Save two update rectangles for clearing the binding bitmap       */
  /* background. Half is on the inside of the book and the other on   */
  /* the outside of the book                                          */
  /*------------------------------------------------------------------*/
  SetRectEmpty( &rcPage );                                               //@MAB1
  SetRectEmpty( &rcBook );                                               //@MAB1
  rcPage = rcBook = *lprcToUpdate;

  /*------------------------------------------------------------------*/
  /* Determine number of icons that must be drawn to make the         */
  /* binding.                                                         */
  /*------------------------------------------------------------------*/
  if (pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM))
  {
    /*----------------------------------------------------------------*/
    /* Determine number of binding rings that will fit on horizontal  */
    /* edge of book                                                   */
    /*----------------------------------------------------------------*/
    cIcons = (pbk->rcTopPage.right -
                    pbk->rcTopPage.left) /
                    BKD_SPIRALBINDLENGTH;

    /*----------------------------------------------------------------*/
    /* Determine coordinates for redrawing top page border line which */
    /* will be overlayed by binding bitmap                            */
    /*----------------------------------------------------------------*/
    pt[1].y = rcPage.top + BKD_SPIRALBINDWIDTH/2;
    pt[1].x = rcPage.left;
    pt[2].x = rcPage.right+1;
    pt[2].y = pt[1].y;
    pt[0].x = pt[1].x;

    /*----------------------------------------------------------------*/
    /* Get resource id for binding icon                               */
    /*----------------------------------------------------------------*/
    if (pbk->flStyle & BKS_MAJORTABTOP)
    {
      pt[0].y = rcPage.top;
      rcBook.top = pt[1].y;
      id = ID_BINDBOTTOMSPIRAL;
    }
    else
    {
      pt[0].y = rcPage.bottom;
      rcBook.bottom = pt[1].y;
      id = ID_BINDTOPSPIRAL;
    }

    /*----------------------------------------------------------------*/
    /* Determine starting position for drawing binding icon.          */
    /*----------------------------------------------------------------*/
    yPos = rcPage.top;
    xPos = rcPage.left +
           ((((pbk->rcBinding.right -
               pbk->rcBinding.left) % BKD_SPIRALBINDLENGTH)+7)/2);       //@MAB1
  }
  else
  {
    /*----------------------------------------------------------------*/
    /* Determine number of binding rings that will fit on vertical    */
    /* edge of book                                                   */
    /*----------------------------------------------------------------*/
    cIcons = (pbk->rcBinding.bottom -
                   pbk->rcBinding.top) /
                   BKD_SPIRALBINDLENGTH;

    /*----------------------------------------------------------------*/
    /* Determine coordinates for redrawing top page border line which */
    /* will be overlayed by binding bitmap                            */
    /*----------------------------------------------------------------*/
    pt[1].x = rcPage.left + BKD_SPIRALBINDWIDTH/2;
    pt[1].y = rcPage.top;
    pt[2].x = pt[1].x;
    pt[2].y = rcPage.bottom+1;                                           //@MAB1
    pt[0].y = pt[1].y;

    /*----------------------------------------------------------------*/
    /* Get resource id for binding icon                               */
    /*----------------------------------------------------------------*/
    if (pbk->flStyle & BKS_MAJORTABRIGHT)
    {
      pt[0].x = rcPage.right;
      rcBook.right = pt[1].x;
      id = ID_BINDLEFTSPIRAL;
    }
    else
    {
      pt[0].x = rcPage.left;
      rcBook.left = pt[1].x;
      id = ID_BINDRIGHTSPIRAL;
    }

    /*----------------------------------------------------------------*/
    /* Determine starting position for drawing binding icon           */
    /*----------------------------------------------------------------*/
    yPos = rcPage.top +
            ((((pbk->rcBinding.bottom -
               pbk->rcBinding.top) % BKD_SPIRALBINDLENGTH)+7)/2);        //@MAB1
    xPos = rcPage.left;
  }

  /*------------------------------------------------------------------*/
  /* Clear the binding bitmap background.  We gave 2 rectangles       */
  /* because half of the background is on the inside of the book and  */
  /* the other on the outside of the book.                            */
  /*------------------------------------------------------------------*/
  BookFillRect( hdc, &rcPage, pbk->crColor[CLR_PAGEBG] );
  BookFillRect( hdc, &rcBook, pbk->crColor[CLR_NOTEBOOK] );

  /*------------------------------------------------------------------*/
  /* Redraw top page border line which will be overlayed by bitmap.   */
  /*------------------------------------------------------------------*/
  if (hpen = CreatePen( PS_SOLID, 1, pbk->crColor[CLR_OUTLINE] ) )
  {
    hpen = SelectObject( hdc, hpen );
    BookPolyline( hdc, pt, 3 );
    DeleteObject( SelectObject( hdc, hpen ) );
  }

  /*------------------------------------------------------------------*/
  /* Draw binding icons                                               */
  /*------------------------------------------------------------------*/
  if (hIcon = QueryIconResource (id))
  {
    for (i=0; i<cIcons; i++)
    {
      DrawIcon (hdc, xPos, yPos, hIcon);

      /*--------------------------------------------------------------*/
      /* Set next horizontal position if major tabs are on the top or */
      /* bottom.  Other, set next vertical position.                  */
      /*--------------------------------------------------------------*/
      if (pbk->flStyle & (BKS_MAJORTABTOP | BKS_MAJORTABBOTTOM))
        xPos += BKD_SPIRALBINDLENGTH;
      else
        yPos += BKD_SPIRALBINDLENGTH;
    }
  }
}


/*---------------------------------------------------------------------------*
 * DrawBookBottom
 *
 *            Ŀ                    Ŀ
 *          Ĵ4        3                    4        3Ŀ
 *                                                    
 *                                                    
 *                                                    
 *           1        2                    1        2 
 *                                
 *                                
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBookBottom(
  PBOOK  pbk,
  HDC    hdc,
  LPRECT lprcDraw)
{
  SHORT  i;

  for (i = 1; i <= BKD_NUMVISPAGEEDGES; i++)
  {
    DrawBackPagesBottom (pbk, i, hdc, lprcDraw);
  }

  DrawBorderBottom (pbk, hdc, lprcDraw);
}


/*---------------------------------------------------------------------------*
 * DrawBookTop
 *
 *         Ŀ                         Ŀ
 *          Ŀ                     Ŀ 
 *          4         3                     4         3 
 *                                                      
 *                                                      
 *                                                      
 *         Ĵ1         2                     1         2
 *                                
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBookTop(
  PBOOK  pbk,
  HDC    hdc,
  LPRECT lprcDraw)
{
  SHORT  i;

  for (i = 1; i <= BKD_NUMVISPAGEEDGES; i++)
  {
    DrawBackPagesTop (pbk, i, hdc, lprcDraw);
  }

  DrawBorderTop (pbk, hdc, lprcDraw);
}


/*---------------------------------------------------------------------------*
 * DrawBorderBottom
 *
 *            Ŀ                     Ŀ
 *         0Ĵ                                         Ŀ0
 *                                                     
 *                                                     
 *                                                     
 *                                                     
 *                                 
 *         1 2                     2 1
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBorderBottom(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcDraw)
{
  SHORT  cxPages;                           /* width of all pages            */
  SHORT  cxPagesB;                          /* width of all pages + border   */
  RECT   rc1;
  RECT   rc2;
  POINT  pt[6];
  SHORT  dx;
  HPEN   hpenNew;

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxPages  = BKD_NUMVISPAGEEDGES * BKD_PAGESPACING * vsDevIndX;
  cxPagesB = cxPages + BKD_BORDERWIDTH;

  /*-----------------------*/
  /* BackPages Bottom Left */
  /*-----------------------*/
  if (pbk->flStyle & BKS_BACKPAGESBL)
  {
    if (pbk->flStyle & BKS_MAJORTABLEFT)
    {
      SetRect (&rc1, lprcDraw->left   - cxPagesB,
                     lprcDraw->top    + cxPages,
                     lprcDraw->left   - cxPages,
                     lprcDraw->bottom + cxPages);

      SetRect (&rc2, lprcDraw->left   - cxPagesB,
                     lprcDraw->bottom + cxPages,
                     lprcDraw->right  - cxPagesB,
                     lprcDraw->bottom + cxPagesB);

      SetRect ((LPRECT) pt, lprcDraw->right, lprcDraw->bottom,
                            lprcDraw->right  - (cxPagesB - 1),
                            lprcDraw->bottom + (cxPagesB - 1));
      dx = -1;
    }
    else
    {
      SetRect (&rc1, lprcDraw->left   - cxPagesB,
                     lprcDraw->bottom + cxPages,
                     lprcDraw->right  - cxPages + 1,
                     lprcDraw->bottom + cxPagesB);

      SetRect (&rc2, lprcDraw->left   - cxPagesB,
                     lprcDraw->top    + cxPagesB,
                     lprcDraw->left   - cxPages,
                     lprcDraw->bottom + cxPages);

      SetRect ((LPRECT) pt, lprcDraw->left, lprcDraw->top,
                            lprcDraw->left - cxPagesB,
                            lprcDraw->top  + cxPagesB);
      dx = 1;
    }
  }

  /*------------------------*/
  /* BackPages Bottom Right */
  /*------------------------*/
  else
  {
    if (pbk->flStyle & BKS_MAJORTABRIGHT)
    {
      SetRect (&rc1, lprcDraw->right  + cxPages,
                     lprcDraw->top    + cxPages,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom + cxPages);

      SetRect (&rc2, lprcDraw->left   + cxPagesB,
                     lprcDraw->bottom + cxPages,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom + cxPagesB);

      SetRect ((LPRECT) pt, lprcDraw->left, lprcDraw->bottom,
                            lprcDraw->left   + (cxPagesB - 1),
                            lprcDraw->bottom + (cxPagesB - 1));
      dx = 1;
    }
    else
    {
      SetRect (&rc1, lprcDraw->left   + cxPages,
                     lprcDraw->bottom + cxPages,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom + cxPagesB);

      SetRect (&rc2, lprcDraw->right  + cxPages,
                     lprcDraw->top    + cxPagesB,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom + cxPages);

      SetRect ((LPRECT) pt, lprcDraw->right, lprcDraw->top,
                            lprcDraw->right + cxPagesB - 1,
                            lprcDraw->top   + cxPagesB - 1);
      dx = -1;
    }
  }

  if (hpenNew = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_OUTLINE]))
  {
    SetRect ((LPRECT) &pt[2], pt[1].x + dx, pt[1].y, pt[0].x + dx, pt[0].y);
    SetRect ((LPRECT) &pt[4], pt[3].x + dx, pt[3].y, pt[2].x + dx, pt[2].y);

    hpenNew = SelectObject (hdc, hpenNew);
    BookPolyline (hdc, pt, 6);
    DeleteObject (SelectObject (hdc, hpenNew));
  }

  BookFillRect (hdc, &rc1, pbk->crColor[CLR_OUTLINE]);
  BookFillRect (hdc, &rc2, pbk->crColor[CLR_OUTLINE]);
}


/*---------------------------------------------------------------------------*
 * DrawBorderTop
 *
 *        1          0                          0         1
 *        Ŀ                          Ŀ
 *         Ŀ                     Ŀ 
 *                                                   
 *                                                   
 *                                                   
 *                                                   
 *                                                  
 *       2                        2
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawBorderTop(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcDraw)
{
  SHORT  cxPages;                           /* width of all pages            */
  SHORT  cxPagesB;                          /* width of all pages + border   */
  RECT   rc1;
  RECT   rc2;
  POINT  pt[6];
  SHORT  dx;
  HPEN   hpenNew;

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxPages  = BKD_NUMVISPAGEEDGES * BKD_PAGESPACING * vsDevIndX;
  cxPagesB = cxPages + BKD_BORDERWIDTH;

  /*---------------------*/
  /* BackPages Top Right */
  /*---------------------*/
  if (pbk->flStyle & BKS_BACKPAGESTR)
  {
    if (pbk->flStyle & BKS_MAJORTABRIGHT)
    {
      SetRect (&rc1, lprcDraw->left   + cxPagesB,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->right  + cxPages,
                     lprcDraw->top    - cxPages);

      SetRect (&rc2, lprcDraw->right  + cxPages,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom - cxPages + 1);

      SetRect ((LPRECT) pt, lprcDraw->left, lprcDraw->top,
                            lprcDraw->left + cxPagesB,
                            lprcDraw->top  - cxPagesB);
      dx = 1;
    }
    else
    {
      SetRect (&rc1, lprcDraw->right  + cxPages,
                     lprcDraw->top    - cxPages,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->bottom - cxPages - 1);

      SetRect (&rc2, lprcDraw->left   + cxPages,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->right  + cxPagesB,
                     lprcDraw->top    - cxPages);

      SetRect ((LPRECT) pt, lprcDraw->right, lprcDraw->bottom,
                            lprcDraw->right  + (cxPagesB - 1),
                            lprcDraw->bottom - (cxPagesB - 1));
      dx = -1;
    }
  }

  /*--------------------*/
  /* BackPages Top Left */
  /*--------------------*/
  else
  {
    if (pbk->flStyle & BKS_MAJORTABLEFT)
    {
      SetRect (&rc1, lprcDraw->left   - cxPages,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->right  - cxPagesB + 1,
                     lprcDraw->top    - cxPages);

      SetRect (&rc2, lprcDraw->left   - cxPagesB,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->left   - cxPages,
                     lprcDraw->bottom - cxPages + 1);

      SetRect ((LPRECT) pt, lprcDraw->right, lprcDraw->top,
                            lprcDraw->right - cxPagesB,
                            lprcDraw->top   - cxPagesB);
      dx = -1;
    }
    else
    {
      SetRect (&rc1, lprcDraw->left   - cxPages,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->right  - cxPages + 1,
                     lprcDraw->top    - cxPages);

      SetRect (&rc2, lprcDraw->left   - cxPagesB,
                     lprcDraw->top    - cxPagesB,
                     lprcDraw->left   - cxPages,
                     lprcDraw->bottom - cxPagesB + 1);

      SetRect ((LPRECT) pt, lprcDraw->left, lprcDraw->bottom,
                            lprcDraw->left   - (cxPagesB - 1),
                            lprcDraw->bottom - (cxPagesB - 1));
      dx = 1;
    }
  }

  if (hpenNew = CreatePen (PS_SOLID, 1, pbk->crColor[CLR_OUTLINE]))
  {
    SetRect ((LPRECT) &pt[2], pt[1].x + dx, pt[1].y, pt[0].x + dx, pt[0].y);
    SetRect ((LPRECT) &pt[4], pt[3].x + dx, pt[3].y, pt[2].x + dx, pt[2].y);

    hpenNew = SelectObject (hdc, hpenNew);
    BookPolyline (hdc, pt, 6);
    DeleteObject (SelectObject (hdc, hpenNew));
  }

  BookFillRect (hdc, &rc1, pbk->crColor[CLR_OUTLINE]);
  BookFillRect (hdc, &rc2, pbk->crColor[CLR_OUTLINE]);
}


/*---------------------------------------------------------------------------*
 * DrawStatusLineText
 *
 * Draw the status area text if it exist for the current top page.  If the
 * status text string is null, the status text area if filled with the
 * notebook page window color.
 *
 * History:
 * 11-Sep-91 JeffR Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID DrawStatusLineText (
  PBOOK pbk,
  PPAGE ppg,
  HDC   hdc,
  BOOL  fDraw)
{
  USHORT   fsCmd = DT_VCENTER | DT_SINGLELINE;
  BOOL     fReleaseDC, fSetFont;
  COLORREF crFG, crBG;

  /*------------------------------------------------------*/
  /* Get a presentation space to draw into (if necessary) */
  /*------------------------------------------------------*/
  if ((fReleaseDC = (hdc == NULL)) && !(hdc = GetDC (pbk->hwndBook)))  return;

  /*--------------------------------*/
  /* clear out the status text area */
  /*--------------------------------*/
  BookFillRect (hdc, &pbk->rcStatus, pbk->crColor[CLR_PAGEBG]);

  /*----------------------------------------------------------------*/
  /* if there is text to draw and we want to draw it, then go ahead */
  /*----------------------------------------------------------------*/
  if (ppg && ppg->npszStatus && (ppg->fsAttr & BKA_STATUSTEXTON) && fDraw)
  {
#define STATUS_MARGIN
#ifdef STATUS_MARGIN
    RECT rcStatus = pbk->rcStatus;
    WORD cxMargin = LOWORD (GetDialogBaseUnits ()) / 2;
#else
    #define rcStatus  (pbk->rcStatus)
#endif

    /*----------------------------------------------------------*/
    /* Determine the tab text justification based on the window */
    /*----------------------------------------------------------*/
    fsCmd |= (pbk->flStyle & BKS_STATUSTEXTCENTER) ? DT_CENTER :
             (pbk->flStyle & BKS_STATUSTEXTRIGHT)  ? DT_RIGHT  : DT_LEFT;

#ifdef STATUS_MARGIN
    /*-----------------------------------------------*/
    /* allow a half-character margin around the text */
    /*-----------------------------------------------*/
    InflateRect (&rcStatus, -cxMargin, 0);
#endif

    /*---------------*/
    /* draw the text */
    /*---------------*/
    crFG = SetTextColor (hdc, pbk->crColor[CLR_STATUSTEXT]);
    crBG = SetBkColor   (hdc, pbk->crColor[CLR_PAGEBG]);

    if (fSetFont = (BOOL)pbk->hFont)                                    /*JEH1*/
      pbk->hFont = SelectObject (hdc, pbk->hFont);
    DrawText (hdc, ppg->npszStatus, -1, &rcStatus, fsCmd);
    if (fSetFont)   pbk->hFont = SelectObject (hdc, pbk->hFont);

    SetTextColor (hdc, crFG);
    SetBkColor   (hdc, crBG);

#ifdef STATUS_MARGIN
    #undef rcStatus
#endif
  }

  /*----------------------------------------------------*/
  /* Release the presentation space if one was created. */
  /*----------------------------------------------------*/
  if (fReleaseDC)  ReleaseDC (pbk->hwndBook, hdc);
}


/*---------------------------------------------------------------------------*
 * DrawTopPage
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void DrawTopPage(
  PBOOK  pbk,                               /* book instance data (locked)   */
  HDC    hdc,
  LPRECT lprcDraw)
{
  RECT rcT;

  /*--------------------------------------------------*/
  /* move the right and bottom sides out by 1 to com- */
  /* pensate for Windows' FrameRect drawing algorithm */
  /*--------------------------------------------------*/
  SetRect (&rcT, lprcDraw->left, lprcDraw->top, lprcDraw->right+1,
           lprcDraw->bottom+1);
  BookFrameRect (hdc, &rcT, pbk->crColor[CLR_OUTLINE]);
}


/*---------------------------------------------------------------------------*
 * BltStretch
 *
 * Blits a bitmap into a rectangle in a device context
 *
 * History:
 * 10-Dec-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

VOID BltStretch (HDC hdc, HBITMAP hbm, LPRECT lprcDst, BOOL fAdjust)
{
  HDC    hdcMem;
  BITMAP bmp;
  RECT   rcSrc;

  /*-----------------------------------------------*/
  /* create the memory device context to blit from */
  /*-----------------------------------------------*/
  if (!(hdcMem = CreateCompatibleDC (hdc)))  return;
  GetObject (hbm, sizeof (BITMAP), (LPSTR) &bmp);
  hbm = SelectObject (hdcMem, hbm);
  SetMapMode (hdcMem, GetMapMode (hdc));
  SetStretchBltMode (hdc, COLORONCOLOR);

  /*----------------------------------------------------------*/
  /* set up the source rectangle; if the the bitmap came from */
  /* a bitmap file created by SDK Paint, adjust for that      */
  /* annoying black line on the right edge of the bitmap      */
  /*----------------------------------------------------------*/
  SetRect (&rcSrc, 0, 0, bmp.bmWidth, bmp.bmHeight);
  if (fAdjust)  rcSrc.right--;

  /*-------------------------------------------*/
  /* map from display points to logical points */
  /*-------------------------------------------*/
  DPtoLP (hdcMem, (LPPOINT) &rcSrc,  2);
  DPtoLP (hdc,    (LPPOINT) lprcDst, 2);

  /*-----------------*/
  /* blit the bitmap */
  /*-----------------*/
  StretchBlt (hdc, lprcDst->left, lprcDst->top,
              lprcDst->right - lprcDst->left, lprcDst->bottom - lprcDst->top,
              hdcMem, rcSrc.left, rcSrc.top, rcSrc.right, rcSrc.bottom,
              SRCCOPY);

  SelectObject (hdcMem, hbm);
  DeleteDC (hdcMem);
}


/*---------------------------------------------------------------------------*
 * DrawScrollArrows
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL DrawScrollArrows(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTab,
  LPRECT lprcDraw)
{
  SHORT  cxPageSpacing;
  USHORT usSide;
  UINT  x1, x2, y1, y2;                                                 /*JEH1*/
  HWND   hwnd1, hwnd2;

  cxPageSpacing = (BKD_NUMVISPAGEEDGES * BKD_PAGESPACING * vsDevIndX) + 1;
  usSide        = DetermineSide (pbk, ppgTab, 0);                       /*JEH1*/

  if ((usSide == BKV_BOTTOM) || (usSide == BKV_TOP))
  {
    hwnd1 = GetDlgItem (pbk->hwndBook, ID_HORIZONTALLEFT);
    hwnd2 = GetDlgItem (pbk->hwndBook, ID_HORIZONTALRIGHT);
  }
  else
  {
    hwnd1 = GetDlgItem (pbk->hwndBook, ID_VERTICALTOP);
    hwnd2 = GetDlgItem (pbk->hwndBook, ID_VERTICALBOTTOM);
  }

  if (!hwnd1 || !hwnd2)  return (FALSE);

  switch (usSide)
  {
    /*--------------------*/
    /* Scroll arrows left */
    /*--------------------*/
    case BKV_LEFT:
      y1 = lprcDraw->top;
      y2 = lprcDraw->bottom;

      if (pbk->flStyle & BKS_BACKPAGESTL)
      {
        y1 -= BKD_BORDERWIDTH;
        y2 -= pbk->cxScrollArrow + cxPageSpacing - BKD_BOOKBORDER;
      }
      else
      {
        y1 += cxPageSpacing;
        y2 -= pbk->cxScrollArrow - BKD_BORDERWIDTH;
      }

      x1 = x2 = lprcDraw->left - pbk->cxScrollArrow - BKD_BORDERWIDTH;
      break;

    /*---------------------*/
    /* Scroll arrows right */
    /*---------------------*/
    case BKV_RIGHT:
      y1 = lprcDraw->top;
      y2 = lprcDraw->bottom - pbk->cxScrollArrow;

      if (pbk->flStyle & BKS_BACKPAGESTR)
      {
        y1 -= BKD_BORDERWIDTH;
        y2 -= cxPageSpacing - BKD_BOOKBORDER;
      }
      else
      {
        y1 += cxPageSpacing - BKD_BOOKBORDER;
        y2 += BKD_BORDERWIDTH;
      }

      x1 = x2 = lprcDraw->right + BKD_BORDERWIDTH;
      break;

    /*----------------------*/
    /* Scroll arrows bottom */
    /*----------------------*/
    case BKV_BOTTOM:
      x1 = lprcDraw->left;
      x2 = lprcDraw->right;

      if (pbk->flStyle & BKS_BACKPAGESBL)
      {
        x1 -= BKD_BORDERWIDTH;
        x2 -= pbk->cxScrollArrow + cxPageSpacing - BKD_BOOKBORDER;
      }
      else
      {
        x1 += cxPageSpacing - BKD_BOOKBORDER;
        x2 -= pbk->cxScrollArrow - BKD_BORDERWIDTH;
      }

      y1 = y2 = lprcDraw->bottom + BKD_BORDERWIDTH;
      break;

    /*-------------------*/
    /* Scroll arrows top */
    /*-------------------*/
    case BKV_TOP:
      x1 = lprcDraw->left;
      x2 = lprcDraw->right - pbk->cxScrollArrow;

      if (pbk->flStyle & BKS_BACKPAGESTL)
      {
        x1 -= BKD_BORDERWIDTH;
        x2 -= BKD_BORDERWIDTH + pbk->cxScrollArrow;
      }
      else
      {
        x1 += cxPageSpacing - BKD_BOOKBORDER;
        x2 += BKD_BORDERWIDTH;
      }

      y1 = y2 = lprcDraw->top - BKD_BORDERWIDTH - pbk->cxScrollArrow;
      break;
  }

  SetWindowPos (hwnd1, NULL, x1, y1, pbk->cxScrollArrow,
                pbk->cxScrollArrow, 0);                                 /*JEH1*/

  SetWindowPos (hwnd2, NULL, x2, y2, pbk->cxScrollArrow,
                pbk->cxScrollArrow, 0);                                 /*JEH1*/

  BookShowWindow (hwnd1, TRUE);
  BookShowWindow (hwnd2, TRUE);

  return (TRUE);
}


/*---------------------------------------------------------------------------*
 * BookFillRect
 *
 * History:
 * 18-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL BookFillRect (
  HDC      hdc,                             /* device context to draw in     */
  LPRECT   lprc,                            /* rectangle to fill             */
  COLORREF crColor)                         /* color to fill with            */
{
  HBRUSH hbrsh;

  if (hbrsh = CreateSolidBrush (crColor))
  {
    FillRect (hdc, lprc, hbrsh);
    DeleteObject (hbrsh);
    return (TRUE);
  }

  return (FALSE);
}


/*---------------------------------------------------------------------------*
 * BookFrameRect
 *
 * History:
 * 18-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

BOOL BookFrameRect (
  HDC      hdc,                             /* device context to draw in     */
  LPRECT   lprc,                            /* rectangle to fill             */
  COLORREF crColor)                         /* color to fill with            */
{
  HBRUSH hbrsh;

  if (hbrsh = CreateSolidBrush (crColor))
  {
    FrameRect (hdc, lprc, hbrsh);
    DeleteObject (hbrsh);
    return (TRUE);
  }

  return (FALSE);
}


#ifdef USE_OWN_POLYLINE
/*---------------------------------------------------------------------------*
 * BookPolyline
 *
 * Simulates Windows' Polyline function by drawing the lines one at a time.
 * This is here only so we can see the lines being drawn individually, later
 * we can just call Polyline directly.
 *
 * History:
 * 24-Sep-91 JeffR    Created.
 *---------------------------------------------------------------------------*/

VOID BookPolyline (
  HDC     hdc,
  LPPOINT lppt,
  USHORT  cpt)
{
  ULONG ulCurrentPos = GetCurrentPosition (hdc);

  MoveTo (hdc, lppt->x, lppt->y);

  for (lppt++; --cpt; lppt++)
  {
    LineTo (hdc, lppt->x, lppt->y);
  }

  MoveTo (hdc, LOWORD (ulCurrentPos), HIWORD (ulCurrentPos));
}
#endif


/*---------------------------------------------------------------------------*
 * ResetNotebookRegions
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void ResetNotebookRegions(
  PBOOK pbk)                                /* book instance data (locked)   */
{
  HideAllWindows(pbk);

  /*---------------------------------------------------------*/
  /* Calculate the spacing needed for the backpages and tabs */
  /*---------------------------------------------------------*/
  CalcVertHorzSpace(pbk);

  /*----------------------------------------------------------------*/
  /* Calculate the book top page rectangle. This is the book        */
  /* rectangle less the outside binding, backpage space , and tabs. */
  /*----------------------------------------------------------------*/
  GetClientRect (pbk->hwndBook, &pbk->rcTopPage);
  CalcTopPage(pbk, &pbk->rcTopPage, TRUE);

  /*------------------------------------------------------------------*/
  /* Calculate the book top page rectangle with the backpage spacing. */
  /*------------------------------------------------------------------*/
  pbk->rcBookPages = pbk->rcTopPage;
  CalcPages(pbk, &pbk->rcBookPages);

  SetPageWndPos (pbk,&pbk->rcAppPageWnd);
  CalcStatusLine (pbk);
  CalcBinding (pbk);
  CalcPageButton (pbk);
  CalcMaxVisTabs (pbk);

  if (pbk->ppgCursorMajor)
  {
    ResetVisPagePtrs (pbk, pbk->ppgCursorMajor, BKA_MAJOR, 0);          /*JEH1*/

    if (pbk->ppgFirstVisMajor)
      ResetTabData (pbk, pbk->ppgCursorMajor,
                    QueryStartTabLevel (pbk, BKA_MAJOR));
  }

  if (pbk->ppgCursorMinor)
  {
    ResetVisPagePtrs (pbk, pbk->ppgCursorMinor, BKA_MINOR, 0);          /*JEH1*/

    if (pbk->ppgFirstVisMinor)
      ResetTabData (pbk, pbk->ppgCursorMinor,
                    QueryStartTabLevel (pbk, BKA_MINOR));
  }

  /*----------------------------------------------------*/
  /* Send a notification that the book size has changed */
  /*----------------------------------------------------*/
  NotifyParent (pbk->hwndBook, BKN_NEWPAGESIZE, FALSE, 0 );             /*DAB2*/
}


/*---------------------------------------------------------------------------*
 * BookShowWindow
 *
 * Shows or hides a window as requested and *if necessary*.  For instance,
 * if a window is hidden and it is requested that it be hidden again, nothing
 * happens.
 *
 * History:
 * 26-Sep-91 JeffR    Created
 *---------------------------------------------------------------------------*/

VOID BookShowWindow (
  HWND hwnd,                                /* window to show or hide        */
  BOOL fShow)                               /* TRUE=show, FALSE=hide         */
{
  if (IsWindowVisible (hwnd) != fShow)
  {
    ShowWindow (hwnd, (fShow) ? SW_SHOW : SW_HIDE);
  }
}


/*---------------------------------------------------------------------------*
 * BookEnableWindow
 *
 * Enables or disables a window as requested and *if necessary*.  For
 * instance, if a window is enabled and it is requested that it be enabled
 * again, nothing happens.
 *
 * History:
 * 26-Sep-91 JeffR    Created
 *---------------------------------------------------------------------------*/

VOID BookEnableWindow (
  HWND hwnd,                                /* window to enable or disable   */
  BOOL fEnable)                             /* TRUE=enable, FALSE=disable    */
{
  if (IsWindowEnabled (hwnd) != fEnable)
  {
    EnableWindow (hwnd, fEnable);
  }
}
