#ifndef _BOOKP_
#define _BOOKP_

/* Revision: 48 1.5 source/ui/cclnt/notebook/bookp.h, cclbook, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                         Filename: BOOKP.H                          */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: BOOKP.H - Private header file for the Notebook control           */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  ------  -------------------------------------------------- */
/* JEH1  06/22/93          Initial modifications for NT execution             */
/* MAB1  02/09/96  6613    Rename "CCL Xxxx" window classes to "ICL Xxxx"     */
/******************************************************************************/
#if defined(__IBMC__)
#pragma info(nocnd)
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

/*-------------------------------*/
/* Notebook color region indexes */
/*-------------------------------*/
#define CLR_NOTEBOOK     0           /* Notebook window background    */
#define CLR_STATUSTEXT   1           /* Status line text color        */
#define CLR_OUTLINE      2           /* Notebook outline color        */
#define CLR_CURSOR       3           /* Selection cursor              */
#define CLR_PAGEBG       4           /* Notebook page background      */
#define CLR_MAJORBG      5           /* Major Tab Color               */
#define CLR_MINORBG      6           /* Minor Tab Color               */
#define CLR_MAJORFG      7           /* Major Tab Text Color          */
#define CLR_MINORFG      8           /* Minor Tab Text Color          */

#define CCOLORS          9

/*----------------------*/
/* Tab record structure */
/*----------------------*/
typedef struct _TAB {
  struct _PAGE *ppgPrevTab;          /* handle to previous tab        *//*DAB1*/
  struct _PAGE *ppgNextTab;          /* handle to next tab            *//*DAB1*/
  RECT    rc;
  HBITMAP hbmTab;                    /* tab bitmap handle             */
  PSZ     npszText;                  /* handle to tab text            *//*JEH1*/
  USHORT  usLevel;                                                      /*JEH1*/
  USHORT  usPosition;                                                   /*JEH1*/
  CHAR    chMnemonic;                /* page mnemonic key             */
} TAB;
typedef TAB *PTAB;                                                      /*DAB1*/

/*-----------------------*/
/* Page record structure */
/*-----------------------*/
typedef struct _PAGE {
  struct _PAGE *ppgPrev;             /* ptr to previous page          *//*DAB1*/
  struct _PAGE *ppgNext;             /* ptr to next page              *//*DAB1*/
  HWND   hwndPage;                   /* application window handle     */
  HWND   hwndPageParent;             /* app window's old parent       */
  ULONG  ulAppData;                  /* ptr to app window data        */
  PTAB   ptab;                       /* handle to the page's tab      */
  PSZ    npszStatus;                 /* ptr to status line text       *//*JEH1*/
  USHORT fsAttr;                     /* page attribute                *//*JEH1*/
} PAGE;
typedef PAGE *PPAGE;                                                    /*DAB1*/

/*-----------------------------------------*/
/* Notebook global instance data structure */
/*-----------------------------------------*/
typedef struct _BOOK {
  HWND     hwndBook;                 /* notebook window handle        */
  HWND     hwndBookPage;             /* parent of all app pages       */
  HWND     hwndTopPage;              /* currently displayed app page  */
  FARPROC  lpfnTopPageWndProc;       /* winproc for top app page      */
  HDC      hdcMem;                   /* memory DC for drawing tabs    */
  HFONT    hFont;                    /* app's custom font             */
  PPAGE    ppgTopMajor;              /* handle to top major tab page  */
  PPAGE    ppgTopMinor;              /* handle to top minor tab page  */
  PPAGE    ppgTop;                   /* handle to top page            */
  PPAGE    ppgFirst;                 /* handle to first page in list  */
  PPAGE    ppgLast;                  /* handle to last page in list   */
  PPAGE    ppgFirstTab;              /* handle to first tab page      */
  PPAGE    ppgLastTab;               /* handle to last tab page       */
  PPAGE    ppgFirstVisMajor;         /* handle to first vis major tab */
  PPAGE    ppgFirstVisMinor;         /* handle to first vis minor tab */
  PPAGE    ppgLastVisMajor;          /* handle to last vis major tab  */
  PPAGE    ppgLastVisMinor;          /* handle to last vis minor tab  */
  PPAGE    ppgCursor;                /* handle to cursored page       */
  PPAGE    ppgCursorMajor;           /* handle to cursored major tab  */
  PPAGE    ppgCursorMinor;           /* handle to cursored minor tab  */
  RECT     rcTopPage;                /* Notebook top page rect        */
  RECT     rcAppPageWnd;             /* app page window               */
  RECT     rcBookPages;              /* Top page + backpages rect     */
  RECT     rcStatus;                 /* Status line text rect         */
  RECT     rcBinding;                /* Notebook binding rect         */
  COLORREF crColor[CCOLORS];         /* Notebook color values         */
  ULONG    flState;                  /* Notebook visual states        */
  ULONG    flStyle;                  /* Notebook style bits           */
  SHORT    cxPageSpacing;            /* horizontal backpage space     *//*JEH1*/
  SHORT    cyPageSpacing;            /* vertical backpage space       *//*JEH1*/
  SHORT    cxMajorTab;               /* Major tab width               *//*JEH1*/
  SHORT    cyMajorTab;               /* Major tab height              *//*JEH1*/
  SHORT    cxMinorTab;               /* Minor tab width               *//*JEH1*/
  SHORT    cyMinorTab;               /* Minor tab height              *//*JEH1*/
  SHORT    cxPageButton;             /* Page Button window width      *//*JEH1*/
  SHORT    cyPageButton;             /* Page Button window height     *//*JEH1*/
  SHORT    cxScrollArrow;            /* scroll arrow width/height     *//*JEH1*/
  SHORT    cMaxVisMajors;                                               /*JEH1*/
  SHORT    cMaxVisMinors;                                               /*JEH1*/
  SHORT    cMaxVisMajorsScroll;                                         /*JEH1*/
  SHORT    cMaxVisMinorsScroll;                                         /*JEH1*/
} BOOK;
typedef BOOK *PBOOK;                                                    /*DAB1*/

typedef void *NPVOID;                                                   /*DAB1*/
typedef HANDLE     HMEM;
NPVOID BookAllocMem (UINT);                                             /*JEH1*/
NPVOID BookFreeMem  (NPVOID);

#ifndef IDOF
#define IDOF(hwnd)            (GetWindowLong (hwnd, GWL_ID))            /*JEH1*/
#endif
#ifndef PARENTOF
#define PARENTOF(hwnd)        (GetParent (hwnd))
#endif
#ifndef OWNEROF
#define OWNEROF(hwnd)         (GetWindow (hwnd, GW_OWNER))
#endif
#define QUERYFOCUS(hwnd)      (GetFocus ())
#define QUERYCAPTURE(hwnd)    (GetCapture ())

/*---------------*/
/* helper macros */
/*---------------*/
#define WNDPROC          LRESULT CALLBACK                               /*JEH1*/

/*--------------------------------*/
/* Window word offset definitions */
/*--------------------------------*/
#define QWL_NOTEBOOK         4

/*-------------------------------*/
/* Define mnemonic key indicator */
/*-------------------------------*/
#define MNEMONIC_CHAR       '&'

/*------------------------------------------------------------*/
/* Scroll arrow window ids.  Note that InitializeScrollArrows */
/* depends on these IDs being consecutive.                    */
/*------------------------------------------------------------*/
#define ID_VERTICALTOP      8000
#define ID_VERTICALBOTTOM   8001
#define ID_HORIZONTALLEFT   8002
#define ID_HORIZONTALRIGHT  8003

/*------------------------*/
/* Page Button window ids */
/*------------------------*/
#define ID_PAGEBUTTONLEFT   8004
#define ID_PAGEBUTTONRIGHT  8005

/*-------------------------*/
/* Notebook Page Window id */
/*-------------------------*/
#define ID_BOOKPAGE         8006

/*-------------------------------*/
/* Scroll arrow visual placement */
/*-------------------------------*/
#define CX_HORZARROWIMAGE        10
#define CY_HORZARROWIMAGE         6
#define CX_VERTARROWIMAGE         6
#define CY_VERTARROWIMAGE        10

/*--------------------------*/
/* Tab Scroll Arrow Visuals */
/*--------------------------*/
#define ID_TABDWN                 1
#define ID_TABLFT                 2
#define ID_TABRGT                 3
#define ID_TABUP                  4
#define ID_TABDWN_D               5
#define ID_TABLFT_D               6
#define ID_TABRGT_D               7
#define ID_TABUP_D                8
#define ID_BINDLEFTSPIRAL         9
#define ID_BINDRIGHTSPIRAL       10
#define ID_BINDTOPSPIRAL         11
#define ID_BINDBOTTOMSPIRAL      12

/*---------------------------*/
/* Page Cycle Button Visuals */
/*---------------------------*/
#define ID_PAGEBUTTONRUP         13
#define ID_PAGEBUTTONRDOWN       14
#define ID_PAGEBUTTONLUP         15
#define ID_PAGEBUTTONLDOWN       16
#define ID_PAGEBUTTONRUP_D       17
#define ID_PAGEBUTTONRDOWN_D     18
#define ID_PAGEBUTTONLUP_D       19
#define ID_PAGEBUTTONLDOWN_D     20

#define DISABLE_DELTA             4

#define CRESOURCES               21

/*-----------------*/
/* Window max pels */
/*-----------------*/
#define VGAX                640
#define VGAY                480
#define VGALROUND           15
#define BGALROUND           20

/*---------------------------------------------*/
/* amount tab cursor is inset from edge of tab */
/*---------------------------------------------*/
#define CX_CURSORINSET      2

/*------------------------*/
/* Notebook visual states */
/*------------------------*/
#define BKV_HSCROLLRIGHTPRESSED  0x00000004  /* Horz. scroll right pressed */
#define BKV_HSCROLLLEFTPRESSED   0x00000008  /* Horz. scroll left pressed  */
#define BKV_VSCROLLTOPPRESSED    0x00000010  /* Horz. scroll top pressed   */
#define BKV_VSCROLLBOTPRESSED    0x00000020  /* Horz. scroll bottom pressed*/
#define BKV_LPAGEBUTTONDOWN      0x00000400  /* Left page button pressed   */
#define BKV_RPAGEBUTTONDOWN      0x00000800  /* Right page button pressed  */
#define BKV_PAGEBUTTONDOWN       (BKV_LPAGEBUTTONDOWN | BKV_RPAGEBUTTONDOWN)
#define BKV_SELFFOCUS            0x80000000  /* we set focus to ourselves  */

/*------------------*/
/* Tab orientations */
/*------------------*/
#define BKV_TOP                  1
#define BKV_BOTTOM               2
#define BKV_RIGHT                3
#define BKV_LEFT                 4

/*-------------------*/
/* Notebook defaults */
/*-------------------*/
#define BKD_MAJORTABWIDTH        0
#define BKD_MAJORTABHEIGHT       0
#define BKD_MINORTABWIDTH        0
#define BKD_MINORTABHEIGHT       0
#define BKD_PAGEBUTTONWIDTH      (GetSystemMetrics (SM_CXVSCROLL))
#define BKD_PAGEBUTTONHEIGHT     (GetSystemMetrics (SM_CYHSCROLL))
#define BKD_SCROLLARROWSIZE      16
#define BKD_TABTYPE              BKA_TEXT
#define BKD_TABOFFSET            5
#define BKD_TABSPACING           2
#define BKD_NUMVISPAGEEDGES      6
#define BKD_MAXMAJORTABLEVEL     BKD_NUMVISPAGEEDGES
#define BKD_MAXMINORTABLEVEL     2
#define BKD_MINTABLEVEL          0
#define BKD_MINTABPOSITION       1
#define BKD_PAGESPACING          3
#define BKD_BOOKBORDER           1
#define BKD_BORDERWIDTH          3
#define BKD_TOPPAGEBORDER        1
#define BKD_SOLIDBINDWIDTH       20
#define BKD_SPIRALBINDWIDTH      16
#define BKD_SPIRALBINDLENGTH     15
#define BKD_TABSIZEMIN           6

/*-------------------------*/
/* handy flag combinations */
/*-------------------------*/
#define FC_BACKPAGES             (BKS_BACKPAGESBL | BKS_BACKPAGESBR | \
                                  BKS_BACKPAGESTL | BKS_BACKPAGESTR)

#define BKV_TABSIZEMIN      7

/*-----------------------*/
/* Subroutine prototypes */
/*-----------------------*/
void    AdjustBinding (PBOOK, LPRECT, BOOL);
void    BltStretch (HDC, HBITMAP, LPRECT, BOOL);
void    BookEnableWindow (HWND, BOOL);
BOOL    BookFillRect (HDC, LPRECT, COLORREF);
BOOL    BookFrameRect (HDC, LPRECT, COLORREF);

#ifdef USE_OWN_POLYLINE
void    BookPolyline (HDC, LPPOINT, USHORT);                            /*JEH1*/
#else
#define BookPolyline  Polyline
#endif

void    BookSetFocus (HWND);
void    BookShowWindow (HWND, BOOL);
void    CalcBinding (PBOOK);
LPRECT  CalcInvalidateRect (PBOOK, PPAGE, PPAGE, LPRECT);
void    CalcMaxVisTabs (PBOOK);
void    CalcNotebookRect (PBOOK, LPRECT);
void    CalcPageButton (PBOOK);
BOOL    CalcPageRect (PBOOK, LPRECT, BOOL);
void    CalcPageWnd (PBOOK, LPRECT, BOOL);
void    CalcPages (PBOOK, LPRECT);
void    CalcStatusLine (PBOOK);
void    CalcTabRectl (PBOOK, PPAGE, USHORT, USHORT, USHORT, USHORT);    /*JEH1*/
void    CalcTopPage (PBOOK, LPRECT, BOOL);
void    CalcVertHorzSpace (PBOOK);
SHORT   CountDisplayableTabs (PBOOK, int, int, int, int);               /*JEH1*/
USHORT  CountTabs (PBOOK, USHORT, SHORT);                               /*JEH1*/
BOOL    DeletePage  (PBOOK, PPAGE, USHORT, BOOL);                       /*JEH1*/
void    DestroyBookResources (void);
void    DestroyPageWindows (PBOOK, PPAGE);
SHORT   DetermineSide (PBOOK, PPAGE, USHORT);                           /*JEH1*/
void    DrawBackPagesBottom (PBOOK, SHORT, HDC, LPRECT);                /*JEH1*/
void    DrawBackPagesTop (PBOOK, SHORT, HDC, LPRECT);                   /*JEH1*/
void    DrawBookBackpages (PBOOK, HDC, LPRECT, LPRECT);
void    DrawBookBinding (PBOOK, HDC, LPRECT);
void    DrawBookBottom (PBOOK, HDC, LPRECT);
void    DrawBookTop (PBOOK, HDC, LPRECT);
void    DrawBorderBottom (PBOOK, HDC, LPRECT);
void    DrawBorderTop (PBOOK, HDC, LPRECT);
void    DrawPolygonTabBorder  (PBOOK, HDC, LPRECT, PPAGE, USHORT, COLORREF);/*JEH1*/
void    DrawPolygonTabCursor (HDC, LPPOINT, USHORT);                    /*JEH1*/
void    DrawRoundTabArc (HDC, LPRECT, int, int, int, int);
void    DrawRoundTabBorder  (PBOOK, HDC, LPRECT, PPAGE, USHORT, COLORREF);/*JEH1*/
void    DrawRoundTabCursor (HDC, LPRECT, USHORT, USHORT, USHORT);       /*JEH1*/
BOOL    DrawScrollArrows (PBOOK, PPAGE, LPRECT);
void    DrawSpiralBinding (PBOOK, HDC, LPRECT);
void    DrawSquareTabBorder  (PBOOK, HDC, LPRECT, PPAGE, USHORT, COLORREF);/*JEH1*/
void    DrawSquareTabCursor (HDC, LPRECT);
void    DrawStatusLineText (PBOOK, PPAGE, HDC, BOOL);
void    DrawTabs (PBOOK, HDC, PPAGE, PPAGE, LPRECT);
void    DrawTabArc (HDC, LPRECT, int, int, int, int);
void    DrawTabBitmap (HDC, HBITMAP, LPRECT);
void    DrawTabBorder (PBOOK, HDC, LPRECT, PPAGE);
void    DrawTabCursor (PBOOK, HDC, LPRECT, LPPOINT, USHORT, USHORT, USHORT);/*JEH1*/
void    DrawTabText (PBOOK, PPAGE, HDC, LPRECT);
void    DrawTopPage (PBOOK, HDC, LPRECT);
void    DrawTopTabLine (HDC, LPRECT, USHORT, COLORREF, USHORT);         /*JEH1*/
void    DrawUpdateRect (PBOOK, PPAGE, PPAGE, LPRECT, HDC);
HWND    FindTabstop (HWND, USHORT);                                     /*JEH1*/
void    FreePageData (PBOOK, PPAGE, BOOL);
PPAGE   GetTabPtr (PPAGE, USHORT, UINT);                                /*JEH1*/
BOOL    GotoPage (PBOOK, PPAGE, ULONG, BOOL);                           /*JEH1*/
void    HideAllWindows (PBOOK);
void    HideScrollArrows (PBOOK, SHORT, SHORT);                         /*JEH1*/
void    HideTopPage (PBOOK, PPAGE);
BOOL    Initialize (HWND);
BOOL    InitializeResources (void);
BOOL    InitializeScrollArrows (PBOOK);
BOOL    InsertPage (PBOOK, PPAGE, PPAGE, USHORT);                       /*JEH1*/
BOOL    InsertPageList (PBOOK, PPAGE, PPAGE, USHORT);                   /*JEH1*/
void    InsertTabPageList (PBOOK, PPAGE, PPAGE, USHORT);                /*JEH1*/
BOOL    IsDialogPageMessage (int, WPARAM, LPARAM, LPHANDLE);            /*JEH1*/
BOOL    IsSibling (HWND, HWND);
BOOL    IsTabVisible (PBOOK, PPAGE);
BOOL    IsReverseNavigation (PBOOK);
HWND    MakeBookPageWnd (HWND);
PPAGE   MakePage (PBOOK, USHORT);                                       /*JEH1*/
BOOL    MakePageButton (HWND);
void    NotifyParent (HWND, USHORT, BOOL, DWORD );                      /*DAB2*/
void    PaintBook (PBOOK);
void    PolyHorizontal (PPOINTL, LPRECT, USHORT, SHORT, SHORT);         /*JEH1*/
void    PolyVertical (PPOINTL, LPRECT, USHORT, SHORT, SHORT);           /*JEH1*/
LONG    ProcessKeyDown (PBOOK, UINT, WPARAM, LPARAM);                   /*JEH1*/
PPAGE   ProcessMouseButtonDown (PBOOK, POINT);
HBITMAP QueryBitmapResource (USHORT);                                   /*JEH1*/
HICON   QueryIconResource (USHORT);                                     /*JEH1*/
LONG    QueryPageCount (PBOOK, PPAGE, USHORT);                          /*JEH1*/
LONG    QueryPageId (PBOOK, PPAGE, USHORT, USHORT);                     /*JEH1*/
USHORT  QueryStartTabLevel (PBOOK, USHORT);                             /*JEH1*/
int     QueryStatusLineText (PBOOK, PPAGE, PBOOKTEXT);                  /*JEH1*/
int     QueryTabText (PBOOK, PPAGE, PBOOKTEXT);                         /*JEH1*/
PPAGE   QueryTopPage (PBOOK);
void    RedrawCursoredTabs (PBOOK);
void    ResetCursorPagePtr (PBOOK, UINT, USHORT);                       /*JEH1*/
void    ResetGlobalPagePtrs (PBOOK, PPAGE);
void    ResetNotebookRegions (PBOOK);
void    ResetPagePtrs (PBOOK, PPAGE);
void    ResetTabData (PBOOK, PPAGE, USHORT);                            /*JEH1*/
void    ResetTopPagePtrs (PBOOK, PPAGE, USHORT);                        /*JEH1*/
void    ResetVisCursorPage (PBOOK, USHORT, PPAGE);                      /*JEH1*/
void    ResetVisPagePtrs (PBOOK, PPAGE, USHORT, UINT);                  /*JEH1*/
void    SendSetCursor (HWND, UINT);                                     /*JEH1*/
BOOL    SetDimensions (PBOOK, USHORT, USHORT, USHORT);                  /*JEH1*/
BOOL    SetNotebookColors (PBOOK, LONG, USHORT);                        /*JEH1*/
void    SetNotebookStyle (PBOOK, BOOL);
BOOL    SetPageButtonDimensions (PBOOK, SHORT, SHORT);                  /*JEH1*/
BOOL    SetPageWindowHwnd (PBOOK, PPAGE, HWND);
void    SetPageWndPos (PBOOK, LPRECT);
BOOL    SetStatusLineText (PBOOK, PPAGE, PSZ);
BOOL    SetTabBitmap (PBOOK, PPAGE, HBITMAP);
BOOL    SetTabDimensions (PBOOK, USHORT, USHORT, USHORT);               /*JEH1*/
BOOL    SetTabText (PBOOK, PPAGE, PSZ);
void    ShowFocus (PBOOK);
void    ShowTopPage (PBOOK, HDC);
void    SubclassTopPage (PBOOK, PPAGE, BOOL);
void    TerminateNotebook (PBOOK);
void    UpdatePageButtonStates (PBOOK);
void    UpdateScrollStates (PBOOK, USHORT);                             /*JEH1*/
BOOL    VerifyPageId (PBOOK, PPAGE);

/*----------------------*/
/* Window class defines */
/*----------------------*/
#define WC_PAGE        "ICL Notebook Page"                             //->@MAB1
#define WC_PAGEBUTTON  "ICL Notebook Page Button"
#define WC_SCROLLARROW "ICL Notebook Scroll Tab"                       //@MAB1<-

/*-------------------------------*/
/* Window procedure declarations */
/*-------------------------------*/
WNDPROC BookWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);/*JEH1*/
WNDPROC PageWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);/*JEH1*/
WNDPROC PageButtonWndProc (HWND hwnd, UINT msg, WPARAM wParam,          /*JEH1*/
                           LPARAM lParam);                              /*JEH1*/
WNDPROC ScrollWndProc (HWND hwnd, UINT msg, WPARAM wParam,              /*JEH1*/
                       LPARAM lParam);                                  /*JEH1*/
WNDPROC TopPageSubclassProc (HWND hwnd, UINT msg, WPARAM wParam,        /*JEH1*/
                             LPARAM lParam);                            /*JEH1*/

INT  APIENTRY LibMain(HANDLE hInstance, DWORD ul_reason_being_called,   /*JEH1*/
                      LPVOID lpReserved);                               /*JEH1*/
LRESULT CALLBACK BookKeyboardHookProc (int nCode, WPARAM wParam,        /*JEH1*/
                                       LPARAM lParam);                  /*JEH1*/

#if 0
#pragma alloc_text (_HOOK,       BookKeyboardHookProc, IsDialogPageMessage)
#pragma alloc_text (INIT_TEXT,   LibMain)
#pragma alloc_text (_ROUNDTABS,  DrawRoundTabBorder, DrawRoundTabCursor)
#pragma alloc_text (_ROUNDTABS,  DrawRoundTabArc)
#pragma alloc_text (_POLYTABS,   DrawPolygonTabBorder, DrawPolygonTabCursor)
#pragma alloc_text (_POLYTABS,   PolyHorizontal, PolyVertical)
#pragma alloc_text (_SQUARETABS, DrawSquareTabBorder, DrawSquareTabCursor)
#pragma alloc_text (_BOTTOM,     DrawBookBottom, DrawBorderBottom)
#pragma alloc_text (_BOTTOM,     DrawBackPagesBottom)
#pragma alloc_text (_TOP,        DrawBookTop, DrawBorderTop, DrawBackPagesTop)
#pragma alloc_text (_WNDPROC,    BookWndProc, PageWndProc, PageButtonWndProc)
#pragma alloc_text (_WNDPROC,    ScrollWndProc)
#pragma alloc_text (_INIT,       Initialize, InitializeResources)
#pragma alloc_text (_INIT,       InitializeScrollArrows, MakeBookPageWnd)
#pragma alloc_text (_INIT,       MakePage, MakePageButton, TerminateNotebook)
#pragma alloc_text (_SPIRAL,     DrawSpiralBinding)
#endif


#define BKS_BACKPAGEMASK (BKS_BACKPAGESBR | BKS_BACKPAGESBL | \
                          BKS_BACKPAGESTR | BKS_BACKPAGESTL)
#define BKS_MAJORTABMASK (BKS_MAJORTABRIGHT | BKS_MAJORTABLEFT | \
                          BKS_MAJORTABTOP   | BKS_MAJORTABBOTTOM)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif // BOOKP header guard
