/* Revision: 50 1.3 source/ui/cclnt/notebook/booktabs.c, cclbook, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992 (Unpublished)         */
/*                                                                    */
/* ================================================================== */
/*                                                                    */
/*                          N O T E B O O K                           */
/*                                                                    */
/*                       Filename: BOOKTABS.C                         */
/*                                                                    */
/*             ( Windows NT Notebook Control Component )              */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/******************************************************************************/
/* Product: ICLUINT.DLL                                                       */
/* Filename: BOOKTABS.C - Tab support routines for the Notebook  control      */
/*                                                                            */
/* Flag    Date    Defect  Description                                        */
/* ----  --------  -----  --------------------------------------------------- */
/* JEH1  06/22/93         Initial modifications for NT execution              */
/* MAB2  01/08/96  50612  Disabled scroll arrow visible when it should be     */
/*                        invisible.                                          */
/* MAB4  01/19/96  6557   Scroll arrow drawn over first tab, and not          */
/*                        repainting properly.                                */
/******************************************************************************/
#include <windows.h>
#include "iclnbw.h"
#include "bookp.h"

extern USHORT vsDevIndX;


/*---------------------------------------------------------------------------*
 * CountDisplayableTabs
 *
 * Calculates the number tabs that will fit along the edge of the book.
 *
 *
 *   Ŀ    } OFFSET
 *     Ŀ |
 *     ^  |- SIZESCROLLARROW * MULTIPLIER
 *      |
 *         \
 *  B       |
 *  O       |- area for displayable tabs
 *  O       |
 *  K      /
 *     Ŀ |
 *     V  |- SIZESCROLLARROW * MULTIPLIER
 *      |
 *       } OFFSET
 *
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

SHORT CountDisplayableTabs (
  PBOOK pbk,                         /* book instance data (locked)   */
  int   pt1,                         /* xLeft or yTop of page rect    *//*JEH1*/
  int   pt2,                         /* xRight or yBottom of page rect*//*JEH1*/
  int   cxTab,                       /* width of tab                  *//*JEH1*/
  int   cxScrollArrow)               /* width of scroll arrow         *//*JEH1*/
{
  SHORT cxBackPageSpacing;
  SHORT cxTabSpacing;
  SHORT cxPageSpacing;
  SHORT cPages;

  /*-----------------------*/
  /* Calculate tabspacing. */
  /*-----------------------*/
  cxTabSpacing  = BKD_TABSPACING * vsDevIndX;
  cxPageSpacing = BKD_PAGESPACING * vsDevIndX;

  /*----------------------------*/
  /* Calculate backpage spacing */
  /*----------------------------*/
  cxBackPageSpacing = (BKD_NUMVISPAGEEDGES * cxPageSpacing) + 1;

  cPages = (pt2 - pt1 - cxBackPageSpacing - cxScrollArrow) /
                                           (cxTab + cxTabSpacing);

  return (max (cPages, 0));
}

/*---------------------------------------------------------------------------*
 * CountTabs
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

USHORT CountTabs (                                                      /*JEH1*/
  PBOOK pbk,                         /* book instance data (locked)   */
  USHORT sItemType,                  /* type of page to count         *//*JEH1*/
  SHORT cMaxTabs)                    /* max number of tabs to count   */
{
  PPAGE ppg;
  SHORT cTabs = 0;

  ppg = ((sItemType & BKA_MAJOR) || !pbk->ppgTopMajor) ?
               pbk->ppgFirstTab : pbk->ppgTopMajor->ptab->ppgNextTab;

  if (sItemType & BKA_MAJOR)
  {
    while (ppg && (cTabs <= cMaxTabs))
    {
      if (ppg->fsAttr & BKA_MAJOR)  cTabs++;
      ppg = ppg->ptab->ppgNextTab;
    }
  }

  else while (ppg && (cTabs <= cMaxTabs))
  {
    if      (ppg->fsAttr & BKA_MINOR)  cTabs++;
    else if (ppg->fsAttr & BKA_MAJOR)  return (cTabs);
    ppg = ppg->ptab->ppgNextTab;
  }

  return (cTabs);
}


/*---------------------------------------------------------------------------*
 * DetermineSide
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

SHORT DetermineSide(
  PBOOK  pbk,                                /* book instance data (locked)   */
  PPAGE  ppg,
  USHORT fTabType)                                                      /*JEH1*/
{
  if (!ppg && !fTabType)  return (0);                                   /*JEH1*/

  if ((fTabType & BKA_MAJOR) || (ppg && (ppg->fsAttr & BKA_MAJOR)))
  {
    if      (pbk->flStyle & BKS_MAJORTABRIGHT)   return (BKV_RIGHT);
    else if (pbk->flStyle & BKS_MAJORTABLEFT)    return (BKV_LEFT);
    else if (pbk->flStyle & BKS_MAJORTABTOP)     return (BKV_TOP);
    else if (pbk->flStyle & BKS_MAJORTABBOTTOM)  return (BKV_BOTTOM);
  }

  else if ((fTabType & BKA_MINOR) || (ppg && (ppg->fsAttr & BKA_MINOR)))
  {
    switch (LOWORD ((pbk->flStyle & BKS_BACKPAGEMASK)))
    {
      case BKS_BACKPAGESBR:
        return ((pbk->flStyle & BKS_MAJORTABRIGHT) ? BKV_BOTTOM : BKV_RIGHT);
        break;

      case BKS_BACKPAGESTR:
        return ((pbk->flStyle & BKS_MAJORTABRIGHT) ? BKV_TOP : BKV_RIGHT);
        break;

      case BKS_BACKPAGESBL:
        return ((pbk->flStyle & BKS_MAJORTABLEFT)  ? BKV_BOTTOM : BKV_LEFT);
        break;

      case BKS_BACKPAGESTL:
        return ((pbk->flStyle & BKS_MAJORTABLEFT)  ? BKV_TOP : BKV_LEFT);
        break;
    }
  }

  return (0);                                                           /*JEH1*/
}


/*---------------------------------------------------------------------------*
 * PolyHorizontal
 *
 *         top side              bottom side
 *
 *        2      3
 *        /-------\            0|          |5
 *      1|         |4           |          |
 *       |         |           1|          |4
 *      0|         |5            \--------/
 *                                2      3
 *
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void PolyHorizontal(
  PPOINTL apt,                       /* point array to fill in        */
  LPRECT  lprc,                      /* bounding rectangle            */
  USHORT  usSide,                    /* BKV_TOP or BKV_BOTTOM         *//*JEH1*/
  SHORT   cyCornerRatio,             /* y ratio of 0-1 to 1-2         */
  SHORT   cxCornerRatio)             /* x ratio of 1-2 to 2-3         */
{
  int   yStart;                                                         /*JEH1*/
  int   dx;                                                             /*JEH1*/
  int   cx = lprc->right - lprc->left;                                  /*JEH1*/
  int   cy = lprc->bottom - lprc->top;                                  /*JEH1*/

  if (usSide == BKV_TOP)                                                /*JEH1*/
  {
    yStart = lprc->bottom;
    dx = 1;
  }
  else
  {
    yStart = lprc->top;
    dx = -1;
  }

  apt[0].x = lprc->left;
  apt[0].y = yStart;

  apt[1].x = apt[0].x;
  apt[1].y = apt[0].y - dx * (100-cyCornerRatio) * cy / 100;

  apt[2].x = lprc->left + cx / cxCornerRatio;
  apt[2].y = apt[1].y - dx * cyCornerRatio * cy / 100;

  apt[3].x = lprc->right - cx / cxCornerRatio;
  apt[3].y = apt[2].y;

  apt[4].x = lprc->right;
  apt[4].y = apt[1].y;

  apt[5].x = apt[4].x;
  apt[5].y = apt[0].y;

  apt[6]   = apt[0];
}


/*---------------------------------------------------------------------------*
 * PolyVertical
 *
 *       left side     right side
 *
 *          1   0       0   1
 *         /-----       -----\
 *       2|                   |2
 *        |                   |
 *       3|                   |3
 *         \-----       -----/
 *          4   5       5   4
 *
 *   width - the reciprocal forms the percentage which is the
 *           horizontal distance between 1 & 2
 *   size  - the reciprocal forms the percentage which is the vertical
 *           distance between 1 & 2
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

void PolyVertical(
  PPOINTL apt,                       /* array of points to fill       */
  LPRECT  lprc,                      /* bounding rectangle            */
  USHORT  usSide,                    /* BKV_RIGHT or BKV_LEFT         *//*JEH1*/
  SHORT   cxCornerRatio,             /* x ratio of 0-1 to 1-2         */
  SHORT   cyCornerRatio)             /* y ratio of 1-2 to 2-3         */
{
  int   xStart;                                                         /*JEH1*/
  int   dx;                                                             /*JEH1*/
  int   cx = lprc->right - lprc->left;                                  /*JEH1*/
  int   cy = lprc->bottom - lprc->top;                                  /*JEH1*/

  if (usSide == BKV_RIGHT)                                              /*JEH1*/
  {
    xStart = lprc->left;
    dx = 1;
  }
  else
  {
    xStart = lprc->right;
    dx = -1;
  }

  apt[0].x = xStart;
  apt[0].y = lprc->top;

  apt[1].x = apt[0].x + dx * (100-cxCornerRatio) * cx / 100;
  apt[1].y = apt[0].y;

  apt[2].x = apt[1].x + dx * cxCornerRatio * cx / 100;
  apt[2].y = apt[1].y + cy / cyCornerRatio;

  apt[3].x = apt[2].x;
  apt[3].y = lprc->bottom - cy / cyCornerRatio;

  apt[4].x = apt[1].x;
  apt[4].y = lprc->bottom;

  apt[5].x = apt[0].x;
  apt[5].y = apt[4].y;

  apt[6]   = apt[0];
}


/*---------------------------------------------------------------------------*
 * CalcTabRect
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID CalcTabRect(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTab,
  USHORT usLevel,
  USHORT usPosition,
  USHORT cx,
  USHORT cy)
{
  USHORT usSide;
  SHORT  cxPageSpacing;
  SHORT  cxBackPageSpacing;
  SHORT  cxScrollArrow;
  SHORT  cxTabSpacing;
  HWND   hwnd;                                                           //@MAB4
  RECT   rcT;

  /*----------------------------------------*/
  /* Determine side tabs are to be drawn on */
  /*----------------------------------------*/
  usSide = DetermineSide (pbk, ppgTab, 0);                              /*JEH1*/

  /*---------------------------------------------------------------*/  //->@MAB4
  /* Set the scroll arrow size  if the scroll arrow is visible.    */
  /* We have to query the WS_VISIBLE style directly, because       */
  /* IsWindowVisible() fails when the parent of a visible child    */
  /* is invisible.                                                 */
  /*---------------------------------------------------------------*/
  if ( (usSide == BKV_LEFT) || (usSide == BKV_RIGHT) )
  {
    hwnd = GetDlgItem (pbk->hwndBook, ID_VERTICALTOP);
    cxScrollArrow =
      (GetWindowLong( hwnd, GWL_STYLE ) & WS_VISIBLE) ? BKD_SCROLLARROWSIZE
                                                      : 0;
  }
  else
  {
    if ( (usSide == BKV_BOTTOM) || (usSide == BKV_TOP) )
    {
      hwnd = GetDlgItem (pbk->hwndBook, ID_HORIZONTALLEFT);
      cxScrollArrow =
        (GetWindowLong( hwnd, GWL_STYLE ) & WS_VISIBLE) ? BKD_SCROLLARROWSIZE
                                                        : 0;
    }
  }                                                                    //@MAB4<-

  /*-------------------------------------*/
  /* Calculate space between back pages. */
  /*-------------------------------------*/
  cxPageSpacing     = BKD_PAGESPACING     * vsDevIndX;
  cxTabSpacing      = BKD_TABSPACING      * vsDevIndX;
  cxBackPageSpacing = BKD_NUMVISPAGEEDGES * cxPageSpacing;

  /*----------------------------------------------------------------*/
  /* Determine tab starting coordinates. Adjust by 1 when needed to */
  /* account for PM rectangle drawing algorithm                     */
  /*----------------------------------------------------------------*/
  switch (pbk->flStyle & FC_BACKPAGES)
  {
    case BKS_BACKPAGESBR:
      /*-------------*/
      /* tabs BOTTOM */
      /*-------------*/
      if (usSide == BKV_BOTTOM)
      {
        rcT.left   = pbk->rcTopPage.left + cxScrollArrow + cxBackPageSpacing +
                     (usPosition-1) * (cxTabSpacing+cx);
        rcT.bottom = pbk->rcTopPage.bottom + cy + usLevel * cxPageSpacing;
      }

      /*------------*/
      /* tabs RIGHT */
      /*------------*/
      else
      {
        rcT.left   = pbk->rcTopPage.right + usLevel * cxPageSpacing;
        rcT.bottom = pbk->rcTopPage.top + cxScrollArrow + cxBackPageSpacing +
                     usPosition * (cxTabSpacing+cy);
      }
      break;

    case BKS_BACKPAGESBL:
      /*-------------*/
      /* tabs BOTTOM */
      /*-------------*/
      if (usSide == BKV_BOTTOM)
      {
        rcT.left   = pbk->rcTopPage.right - cxScrollArrow - cxBackPageSpacing -
                     usPosition * (cxTabSpacing+cx);
        rcT.bottom = pbk->rcTopPage.bottom + cy + usLevel * cxPageSpacing;
      }

      /*-----------*/
      /* tabs LEFT */
      /*-----------*/
      else
      {
        rcT.left   = pbk->rcTopPage.left - cx + BKD_BOOKBORDER -
                     usLevel * cxPageSpacing;
        rcT.bottom = pbk->rcTopPage.top + cxScrollArrow + cxBackPageSpacing +
                     usPosition * (cxTabSpacing+cy);
      }
      break;

    case BKS_BACKPAGESTR:
      /*----------*/
      /* tabs TOP */
      /*----------*/
      if (usSide == BKV_TOP)
      {
        rcT.left   = pbk->rcTopPage.left + cxScrollArrow + cxBackPageSpacing +
                     (usPosition-1) * (cxTabSpacing+cx);
        rcT.bottom = pbk->rcTopPage.top - usLevel * cxPageSpacing + 1;
      }

      /*------------*/
      /* tabs RIGHT */
      /*------------*/
      else
      {
        rcT.left   = pbk->rcTopPage.right + usLevel * cxPageSpacing;
        rcT.bottom = pbk->rcTopPage.bottom - cxScrollArrow - cxBackPageSpacing -
                     (usPosition-1) * (cxTabSpacing+cy);
      }
      break;

    case BKS_BACKPAGESTL:
      /*----------*/
      /* tabs TOP */
      /*----------*/
      if (usSide == BKV_TOP)
      {
        rcT.left   = pbk->rcTopPage.right - cxScrollArrow - cxBackPageSpacing -
                     usPosition * (cxTabSpacing+cx);
        rcT.bottom = pbk->rcTopPage.top - usLevel * cxPageSpacing + 1;
      }

      /*-----------*/
      /* tabs LEFT */
      /*-----------*/
      else
      {
        rcT.left   = pbk->rcTopPage.left - cx + BKD_BOOKBORDER -
                                  usLevel * cxPageSpacing;
        rcT.bottom = pbk->rcTopPage.bottom - cxScrollArrow - cxBackPageSpacing -
                     (usPosition-1) * (cxTabSpacing+cy);
      }
      break;
  }

  rcT.right = rcT.left   + cx;
  rcT.top   = rcT.bottom - cy;

  ppgTab->ptab->rc = rcT;
}


/*---------------------------------------------------------------------------*
 * CalcInvalidateRect
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

LPRECT CalcInvalidateRect(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTabStart,
  PPAGE  ppgTabEnd,
  LPRECT lprc)
{
  UINT   usSide;                                                        /*JEH1*/
  RECT   rcBook;
  UINT   usTempPos;                                                     /*JEH1*/

  if (!ppgTabStart)
  {
    SetRectEmpty (lprc);
    return (lprc);
  }

  /*------------------------*/
  /* Get Notebook rectangle */
  /*------------------------*/
  GetClientRect (pbk->hwndBook, &rcBook);

  /*----------------------------------------*/
  /* Determine side tabs are to be drawn on */
  /*----------------------------------------*/
  switch (usSide = DetermineSide (pbk, ppgTabStart, 0))                 /*JEH1*/
  {
    case BKV_LEFT:
      SetRect (lprc, rcBook.left,
                     ppgTabStart->ptab->rc.top,
                     pbk->rcTopPage.left + 1,
                     ppgTabEnd->ptab->rc.bottom + 1);
      break;

    case BKV_RIGHT:
      SetRect (lprc, pbk->rcTopPage.right,
                     ppgTabStart->ptab->rc.top,
                     rcBook.right + 1,
                     ppgTabEnd->ptab->rc.bottom + 1);
      break;

    case BKV_TOP:
      SetRect (lprc, ppgTabStart->ptab->rc.left,
                     rcBook.top,
                     ppgTabEnd->ptab->rc.right + 1,
                     pbk->rcTopPage.top + 1);
      break;

    case BKV_BOTTOM:
      SetRect (lprc, ppgTabStart->ptab->rc.left,
                     pbk->rcTopPage.bottom,
                     ppgTabEnd->ptab->rc.right + 1,
                     rcBook.bottom + 1);
      break;
  }

  /*---------------------------------------------------------------*/
  /* Make adjustments for the different tab backpage orientations. */
  /* for example, if backpage are bottom right, tabs will be drawn */
  /* from top to bottom. If backpages are top right, tabs will be  */
  /* drawn from bottom to top.                                     */
  /*---------------------------------------------------------------*/
  if (lprc->left > lprc->right)
  {
    usTempPos   = lprc->left;
    lprc->left  = lprc->right;
    lprc->right = usTempPos;
  }
  if (lprc->top > lprc->bottom)
  {
    usTempPos    = lprc->bottom;
    lprc->bottom = lprc->top;
    lprc->top    = usTempPos;
  }

  if (((pbk->flStyle & BKS_BACKPAGESTL) && (usSide == BKV_LEFT)) ||
      ((pbk->flStyle & BKS_BACKPAGESTR) && (usSide == BKV_RIGHT)))
  {
    lprc->top    = ppgTabEnd->ptab->rc.top - 1;
    lprc->bottom = ppgTabStart->ptab->rc.bottom;
  }

  else if (((pbk->flStyle & BKS_BACKPAGESTL) && (usSide == BKV_TOP)) ||
           ((pbk->flStyle & BKS_BACKPAGESBL) && (usSide == BKV_BOTTOM)))
  {
    lprc->left  = ppgTabEnd->ptab->rc.left;
    lprc->right = ppgTabStart->ptab->rc.right + 1;
  }

  return (lprc);
}


/*---------------------------------------------------------------------------*
 * IsTabVisible
 *
 * Determines if the tab for a given page is visible.
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

BOOL IsTabVisible(
  PBOOK pbk,                                /* book instance data (locked)   */
  PPAGE ppgTab)
{
  PPAGE   ppgCurrentTab=NULL;
  SHORT   cMaxTabs;
  SHORT   i;
  USHORT  usTabType;

  /*----------------------------------------------------------------*/
  /* If the tab page is NULL, it cannot be visible                  */
  /*----------------------------------------------------------------*/
  if (!ppgTab)
    return FALSE;

  /*----------------------------------------------------------------*/
  /* If page is a major get pointer to first visible major tab page */
  /*----------------------------------------------------------------*/
  if (ppgTab->fsAttr & BKA_MAJOR)
  {
    cMaxTabs = CountTabs (pbk, ppgTab->fsAttr, pbk->cMaxVisMajors);

    if (!cMaxTabs)
    {
      return FALSE;
    }
    else if (cMaxTabs > pbk->cMaxVisMajors)
    {
      cMaxTabs = pbk->cMaxVisMajorsScroll;
    }

    ppgCurrentTab = pbk->ppgFirstVisMajor;
    usTabType = BKA_MAJOR;
  }

  else
  {
    cMaxTabs = CountTabs (pbk, ppgTab->fsAttr, pbk->cMaxVisMinors);

    if (!cMaxTabs)
    {
      return FALSE;
    }
    else if (cMaxTabs > pbk->cMaxVisMinors)
    {
      cMaxTabs = pbk->cMaxVisMinorsScroll;
    }

    ppgCurrentTab = pbk->ppgFirstVisMinor;
    usTabType     = BKA_MINOR;
  }

  for (i=1; ppgCurrentTab && (i <= cMaxTabs); i++)
  {
    if (ppgCurrentTab == ppgTab)  return TRUE;

    ppgCurrentTab = GetTabPtr (ppgCurrentTab->ptab->ppgNextTab,
                               usTabType, BKA_NEXT);
  }

  return(FALSE);
}


/*---------------------------------------------------------------------------*
 * QueryStartTabLevel
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

USHORT QueryStartTabLevel(
  PBOOK pbk,                                /* book instance data (locked)   */
  USHORT usTabType)
{
  USHORT  usMaxTabLevel;
  USHORT  usLevel;
  USHORT  cMaxTabs;
  USHORT  cTabs;
  PPAGE   ppgVis;
  PPAGE   ppgTab;
  PPAGE   ppgTop;

  if (usTabType & BKA_MAJOR)
  {
    ppgVis = pbk->ppgFirstVisMajor;
    ppgTop = pbk->ppgTopMajor;

    cMaxTabs      = pbk->cMaxVisMajors;
    usMaxTabLevel = BKD_MAXMAJORTABLEVEL;
    usLevel       = BKD_MINTABLEVEL;
  }
  else
  {
    ppgVis = pbk->ppgFirstVisMinor;
    ppgTop = pbk->ppgTopMinor;

    cMaxTabs      = pbk->cMaxVisMinors;
    usMaxTabLevel = BKD_MAXMINORTABLEVEL;
    usLevel       = BKD_MINTABLEVEL;
  }

  cTabs = CountTabs (pbk, usTabType, cMaxTabs);

  if (cTabs > cMaxTabs)
  {
    cMaxTabs = (usTabType & BKA_MAJOR) ? pbk->cMaxVisMajorsScroll :
                                         pbk->cMaxVisMinorsScroll;
  }
  else
  {
    cMaxTabs = cTabs;
  }

  ppgTab = ppgTop;

  while (ppgTab && (ppgTab != ppgVis) && (usLevel != usMaxTabLevel))
  {
    usLevel++;
    ppgTab = GetTabPtr (ppgTab->ptab->ppgNextTab, usTabType, BKA_NEXT);
  }

  if (ppgTab != ppgVis)
  {
    usLevel = (usTabType & BKA_MINOR) ? BKD_MAXMINORTABLEVEL :
                                        BKD_MAXMAJORTABLEVEL;
  }

  return usLevel;
}


/*---------------------------------------------------------------------------*
 * ResetTabData
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID ResetTabData(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTab,
  USHORT usStartLevel)
{
  PPAGE   ppgCurrentTab;
  PPAGE   ppgTabTop;
  USHORT  usMaxTabLevel;
  USHORT  usLevel;
  USHORT  usPosition;
  USHORT  cMaxTabs;
  USHORT  i;
  USHORT  cxTab,cyTab;
  USHORT  cTabs;
  USHORT  usTabType;

  /*-----------------------------*/
  /* If tab page is NULL, return */
  /*-----------------------------*/
  if (!ppgTab)  return;

  usLevel = usStartLevel;

  if (ppgTab->fsAttr & BKA_MAJOR)
  {
    ppgCurrentTab = pbk->ppgFirstVisMajor;
    cMaxTabs      = pbk->cMaxVisMajors;
    ppgTabTop     = pbk->ppgTopMajor;
    usMaxTabLevel = BKD_MAXMAJORTABLEVEL;
    cxTab         = pbk->cxMajorTab;
    cyTab         = pbk->cyMajorTab;
    usTabType     = BKA_MAJOR;

    if (!usLevel) usLevel = BKD_MAXMAJORTABLEVEL;
  }

  else
  {
    ppgCurrentTab = pbk->ppgFirstVisMinor;
    cMaxTabs      = pbk->cMaxVisMinors;
    ppgTabTop     = pbk->ppgTopMinor;
    usMaxTabLevel = BKD_MAXMINORTABLEVEL;
    cxTab         = pbk->cxMinorTab;
    cyTab         = pbk->cyMinorTab;
    usTabType     = BKA_MINOR;

    if (!usLevel) usLevel  = BKD_MAXMINORTABLEVEL;
  }

  cTabs = CountTabs (pbk, ppgTab->fsAttr, cMaxTabs);

  if (cTabs > cMaxTabs)
  {
    cMaxTabs = (ppgTab->fsAttr & BKA_MAJOR) ? pbk->cMaxVisMajorsScroll :
                                              pbk->cMaxVisMinorsScroll;
  }
  else
  {
    cMaxTabs = cTabs;
  }

  /*--------------------------------------*/
  /* Initialize tab position to the first */
  /*--------------------------------------*/
  usPosition = BKD_MINTABPOSITION;

  for (i=1; ppgCurrentTab && (i <= cMaxTabs); i++)
  {
    /*------------------------------*/
    /* Reset current page tab level */
    /*------------------------------*/
    if (ppgCurrentTab == ppgTabTop)   usLevel = BKD_MINTABLEVEL;
    ppgCurrentTab->ptab->usLevel = usLevel;

    /*---------------------------------*/
    /* Reset current page tab position */
    /*---------------------------------*/
    ppgCurrentTab->ptab->usPosition = usPosition;

    /*--------------------------------------*/
    /* Calculate current page tab rectangle */
    /*--------------------------------------*/
    CalcTabRect (pbk, ppgCurrentTab, ppgCurrentTab->ptab->usLevel,
                 ppgCurrentTab->ptab->usPosition, cxTab, cyTab);

    /*----------------------------------*/
    /* Increment tab level and position */
    /*----------------------------------*/
    if (usLevel != usMaxTabLevel)
    {
      usLevel += ((ppgCurrentTab == ppgTabTop) &&
                  (ppgCurrentTab->fsAttr & BKA_MAJOR) &&
                  (ppgCurrentTab->ptab->ppgNextTab) &&
                  (ppgCurrentTab->ptab->ppgNextTab->fsAttr & BKA_MINOR)) ? 2 : 1;
    }

    usPosition++;

    /*------------------------*/
    /* Index to next tab page */
    /*------------------------*/
    ppgCurrentTab = GetTabPtr (ppgCurrentTab->ptab->ppgNextTab,
                               usTabType, BKA_NEXT);
  }
}


/*---------------------------------------------------------------------------*
 * ResetVisPagePtrs
 *
 * History:
 * 11-Sep-91 JeffR    Initial port from PM.
 *---------------------------------------------------------------------------*/

VOID ResetVisPagePtrs(
  PBOOK  pbk,                               /* book instance data (locked)   */
  PPAGE  ppgTab,
  USHORT usTabType,
  UINT   usTabOrder)                                                    /*JEH1*/
{
  PPAGE   ppgCurrentTab;
  SHORT   cMaxTabs=0;
  SHORT   i;
  SHORT   cTabs;
  PPAGE   ppgVisTab;
  USHORT  usSide;

  /*-----------------------------------------------*/
  /* Determine the number of tabs to be displayed. */
  /*-----------------------------------------------*/
  if (usTabType & BKA_MAJOR)
  {
    cMaxTabs = CountTabs (pbk,usTabType, pbk->cMaxVisMajors);

    if (usTabOrder & BKA_PREV)
    {
      ppgVisTab = NULL;                                                //->@MAB2
      if (pbk->ppgFirstVisMajor)
      {
        ppgVisTab = GetTabPtr( pbk->ppgFirstVisMajor->ptab->ppgPrevTab,
                               usTabType, usTabOrder );
      }
      if (ppgVisTab)
      {
        pbk->ppgFirstVisMajor = ppgVisTab;

        if (cMaxTabs > pbk->cMaxVisMajors)
        {
          pbk->ppgLastVisMajor =
                      GetTabPtr( pbk->ppgLastVisMajor->ptab->ppgPrevTab,
                                 usTabType, usTabOrder );
        }
      }
    }
    else if (usTabOrder & BKA_NEXT)
    {
      ppgVisTab = NULL;
      if (pbk->ppgLastVisMajor)
      {
        ppgVisTab = GetTabPtr( pbk->ppgLastVisMajor->ptab->ppgNextTab,
                               usTabType,usTabOrder );
      }
      if (ppgVisTab)
      {
        pbk->ppgLastVisMajor = ppgVisTab;

        if (cMaxTabs > pbk->cMaxVisMajors)
        {
          pbk->ppgFirstVisMajor =
                       GetTabPtr( pbk->ppgFirstVisMajor->ptab->ppgNextTab,
                                  usTabType, usTabOrder );
        }
      }
    }
    else
    {
      pbk->ppgFirstVisMajor = NULL;
      pbk->ppgLastVisMajor  = NULL;

      if (cMaxTabs > pbk->cMaxVisMajors)
      {
        DrawScrollArrows( pbk, ppgTab, &pbk->rcBookPages );
        cMaxTabs = pbk->cMaxVisMajorsScroll;
      }
      else
      {
        usSide = DetermineSide (pbk, ppgTab, 0);                        /*JEH1*/

        if ((usSide == BKV_TOP) || (usSide == BKV_BOTTOM))
          HideScrollArrows (pbk, ID_HORIZONTALLEFT, ID_HORIZONTALRIGHT);

        else if ((usSide == BKV_RIGHT) || (usSide == BKV_LEFT))
          HideScrollArrows (pbk, ID_VERTICALTOP, ID_VERTICALBOTTOM);
      }
    }
  }

  if (usTabType & BKA_MINOR)
  {
    cMaxTabs = CountTabs (pbk, usTabType, pbk->cMaxVisMinors);

    if (usTabOrder & BKA_PREV)
    {
      ppgVisTab = NULL;
      if (pbk->ppgFirstVisMinor)
      {
        ppgVisTab = GetTabPtr( pbk->ppgFirstVisMinor->ptab->ppgPrevTab,
                               usTabType, usTabOrder );
      }
      if (ppgVisTab)
      {
        pbk->ppgFirstVisMinor = ppgVisTab;
        if (cMaxTabs > pbk->cMaxVisMinors)
        {
          pbk->ppgLastVisMinor =
                      GetTabPtr( pbk->ppgLastVisMinor->ptab->ppgPrevTab,
                                 usTabType,usTabOrder );
        }
      }
    }
    else if (usTabOrder & BKA_NEXT)
    {
      ppgVisTab = NULL;
      if (pbk->ppgLastVisMinor)
      {
        ppgVisTab = GetTabPtr( pbk->ppgLastVisMinor->ptab->ppgNextTab,
                               usTabType, usTabOrder );
      }
      if (ppgVisTab)
      {
        pbk->ppgLastVisMinor = ppgVisTab;

        if (cMaxTabs > pbk->cMaxVisMinors)
        {
          pbk->ppgFirstVisMinor =
                       GetTabPtr( pbk->ppgFirstVisMinor->ptab->ppgNextTab,
                                  usTabType,usTabOrder );
        }                                                              //@MAB2<-
      }
    }
    else
    {
      pbk->ppgFirstVisMinor = NULL;
      pbk->ppgLastVisMinor  = NULL;

      if (cMaxTabs > pbk->cMaxVisMinors)
      {
        DrawScrollArrows (pbk, ppgTab, &pbk->rcBookPages);
        cMaxTabs = pbk->cMaxVisMinorsScroll;
      }
      else
      {
        usSide = DetermineSide (pbk, ppgTab, 0);                        /*JEH1*/

        if ((usSide == BKV_TOP) || (usSide == BKV_BOTTOM))
          HideScrollArrows (pbk, ID_HORIZONTALLEFT, ID_HORIZONTALRIGHT);

        else if ((usSide == BKV_RIGHT) || (usSide == BKV_LEFT))
          HideScrollArrows (pbk, ID_VERTICALTOP, ID_VERTICALBOTTOM);
      }
    }
  }

  if (!cMaxTabs || usTabOrder)
  {
    UpdateScrollStates (pbk, usTabType);
    return;
  }
  else
  {
    ppgCurrentTab = ppgTab;
    cTabs = cMaxTabs/2;
  }

  /*--------------------------------------------------------*/
  /* Reset the starting and ending visible page pointers so */
  /* that the tab page will come into view in the middle    */
  /*--------------------------------------------------------*/
  if (usTabType & BKA_MAJOR)
  {
    pbk->ppgFirstVisMajor = ppgCurrentTab;
    pbk->ppgLastVisMajor  = ppgCurrentTab;
  }
  else
  {
    pbk->ppgFirstVisMinor = ppgCurrentTab;
    pbk->ppgLastVisMinor  = ppgCurrentTab;
  }

  /*-------------------------------*/
  /* Set last visible page pointer */
  /*-------------------------------*/
  for (i=0; ppgCurrentTab && (i <= cTabs); i++)
  {
    if (usTabType & BKA_MAJOR)                                         //->@MAB2
      pbk->ppgLastVisMajor = ppgCurrentTab;
    else
      pbk->ppgLastVisMinor = ppgCurrentTab;                            //@MAB2<-

    if ((ppgCurrentTab != ppgTab) && (cMaxTabs > 0))
      cMaxTabs--;

    ppgCurrentTab = GetTabPtr( ppgCurrentTab->ptab->ppgNextTab,
                               usTabType, BKA_NEXT );
  }

  /*--------------------------------*/
  /* Set first visible page pointer */
  /*--------------------------------*/
  ppgCurrentTab = ppgTab;
  cTabs = cMaxTabs;

  for (i=0; ppgCurrentTab && (i < cTabs); i++, cMaxTabs--)
  {
    if (usTabType & BKA_MAJOR)                                         //->@MAB2
      pbk->ppgFirstVisMajor = ppgCurrentTab;
    else
      pbk->ppgFirstVisMinor = ppgCurrentTab;                           //@MAB2<-

    ppgCurrentTab = GetTabPtr( ppgCurrentTab->ptab->ppgPrevTab,
                               usTabType, BKA_PREV );
  }

  cTabs  = cMaxTabs;

  if (usTabType & BKA_MAJOR)                                           //->@MAB2
    ppgCurrentTab = pbk->ppgLastVisMajor;
  else
    ppgCurrentTab = pbk->ppgLastVisMinor;                              //@MAB2<-

  for (i=0; ppgCurrentTab && (i <= cTabs); i++)
  {
    if (usTabType & BKA_MAJOR)                                         //->@MAB2
      pbk->ppgLastVisMajor = ppgCurrentTab;
    else
      pbk->ppgLastVisMinor = ppgCurrentTab;                            //@MAB2<-

    ppgCurrentTab = GetTabPtr( ppgCurrentTab->ptab->ppgNextTab,
                               usTabType, BKA_NEXT );
  }

  UpdateScrollStates (pbk, usTabType);
}
