/* Revision: 51 1.4 source/ui/cclnt/notebook/bookwin.c, cclbook, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997            */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Notebook Control                                                   */
/*                                                                    */
/* File:  bookwin.c                                                   */
/*                                                                    */
/* This application creates a notebook containing a calendar.  Major  */
/* tabs represent five year intervals, and minor tabs allow selection */
/* of years/quarter years.  Menu items are included to delete and     */
/* restore calendar pages, as well as to jump to an arbitrary         */
/* calendar month/year.                                               */
/*                                                                    */
/**********************************************************************/

#include <windows.h>
#include <stdlib.h>                    /* itoa, atoi                  */
#include <time.h>                      /* for getting current date    */
#include "iclnbw.h"                    /* include file for CCL        */
#include "bookwin.h"                   /* private application include */
                                       /* file                        */

/**********************************************************************/
/* globals monthNames are the months of the year                      */
/**********************************************************************/

static char *monthNames[] =
{
  "January",     "February",    "March",       "April",       "May",
  "June",        "July",        "August",      "September",
  "October",     "November",    "December"
} ;

#define  YEAR_MONTHS   12

/**********************************************************************/
/* monthTabs is the minor tab text associated with each year          */
/**********************************************************************/

static char *monthTabs[] =
{
  " Jan-Mar",    " Apr-Jun",    " Jul-Sep",    " Oct-Dec"
} ;

#define  MONTHTABCOUNT (sizeof (monthTabs) / sizeof (monthTabs [0]))
#define  MONTHTABINTERVAL (YEAR_MONTHS / MONTHTABCOUNT)

/**********************************************************************/
/* dayNames are the days of the week                                  */
/**********************************************************************/

static char *dayNames[] =
{
  "Sun",         "Mon",         "Tues",        "Wed",         "Thurs",
  "Fri",         "Sat"
} ;

#define  DAYCOUNT      7

/**********************************************************************/
/* daysPerMonth are the number of days in each month                  */
/**********************************************************************/

static int daysPerMonth[] =
{
  31,            28,            31,            30,            31,
  30,            31,            31,            30,            31,
  30,            31
} ;

static DATE currDate =
{
  0,             0
} ;
                                       /* current page on top of      */
                                       /* notebook                    */
static char szPageClass[] = "Calendar Page";

int PASCAL WinMain(HANDLE hInstance,HANDLE hPrevInstance,
                   LPSTR lpCmdLine,int nCmdShow);

#if defined(__IBMC__)
int main( int argc, char** argv )
{
   HANDLE hInstance = GetModuleHandle(0);
   return WinMain( hInstance, 0, 0, SW_SHOWNORMAL );
}
#endif

/**********************************************************************/
/*  Function:  WinMain                                                */
/*  Inputs:    hInstance        instance handle of execution          */
/*             hPrevInstance    instance handle of previous execution */
/*             lpCmdLine        command line arguments                */
/*             nCmdShow         startup mode (iconized/maximized)     */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function registers the window class and contains the message */
/*  loop. It loads the application icons and cursors and is           */
/*  responsible for message dispatching.                              */
/**********************************************************************/

int PASCAL WinMain(HANDLE hInstance,HANDLE hPrevInstance,
                   LPSTR lpCmdLine,int nCmdShow)

  /********************************************************************/
  /* windows application entry point                                  */
  /********************************************************************/

{
  HANDLE hLibrary;
  HANDLE hwnd;                         /* client window handle        */
  MSG msg;                             /* message number              */
  WNDCLASS wndClass;                   /* for registration            */
  HBRUSH hbrshClient;                  /* brush for client window     */
  HBRUSH hbrshPage;                    /* brush for page window       */
  static char szClientClass[] = "Notebook Sample Application";

  lpCmdLine;                           /* prevent unused warning      */

  /********************************************************************/
  /* register window class for client...a basic window with no fancy  */
  /* aspects...we use a colored brush for its background              */
  /********************************************************************/

  if (!hPrevInstance)
  {
    /******************************************************************/
    /* create a background brush                                      */
    /******************************************************************/
    hbrshClient = CreateSolidBrush(COLOR_CLIENT_BACKGROUND);

    wndClass.lpszClassName = szClientClass;
    wndClass.hInstance     = hInstance;
    wndClass.lpfnWndProc   = ClientWndProc;
    wndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wndClass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wndClass.lpszMenuName  = MAKEINTRESOURCE(ID_CLIENT_RESOURCES);
    wndClass.hbrBackground = hbrshClient;
    wndClass.style         = CS_HREDRAW | CS_VREDRAW;
    wndClass.cbClsExtra    = 0;
    wndClass.cbWndExtra    = 0;

    RegisterClass(&wndClass);

    /******************************************************************/
    /* register a page for the notebook                               */
    /******************************************************************/
    hbrshPage = CreateSolidBrush (COLOR_CALENDARPAGE);

    wndClass.lpszClassName = szPageClass;
    wndClass.lpfnWndProc   = PageWndProc;
    wndClass.lpszMenuName  = NULL;
    wndClass.hbrBackground = hbrshPage;

    RegisterClass(&wndClass);
  }

  /********************************************************************/
  /* register the notebook control                                    */
  /********************************************************************/
  if (!(hLibrary = LoadLibrary(ICL_NOTEBOOK_DLL)))                      /*JEH1*/
    return 0;

  /********************************************************************/
  /* create a big client window                                       */
  /********************************************************************/

  hwnd = CreateWindow(szClientClass,           /* window class        */
                      "Calendar",              /* window title        */
                      WS_OVERLAPPEDWINDOW |
                      WS_CLIPCHILDREN,         /* window              */
                                               /* style               */
                      CW_USEDEFAULT,           /* x position          */
                      CW_USEDEFAULT,           /* y position          */
                      788,                     /* cx size             */
                      500,                     /* cy size             */
                      NULL,                    /* no parent           */
                      NULL,                    /* class menu          */
                      hInstance,               /* creator             */
                      NULL);                   /* params              */

  ShowWindow(hwnd, nCmdShow);
  UpdateWindow(hwnd);

  /********************************************************************/
  /* message loop                                                     */
  /********************************************************************/
  while (GetMessage(&msg,  NULL, 0, 0))                                 /*JEH1*/
  {
    TranslateMessage(&msg);            /* keyboard interface          */
    DispatchMessage(&msg);             /* to client window procedure  */
  }                                    /* end of while loop           */

  FreeLibrary(hLibrary);

  return 0;
}                                      /* end of winMain              */

/**********************************************************************/
/*  Function:  ClientWndProc                                          */
/*  Inputs:    hwnd                 client window handle              */
/*             msg                  message id                        */
/*             wParam               first parameter (word)            */
/*             lParam               second parameter (long)           */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/*  This function processes messages sent to the client window,       */
/*  including system messages and messages from controls.             */
/**********************************************************************/

LRESULT CALLBACK ClientWndProc(HWND hwnd, UINT msg, WPARAM wParam,      /*JEH1*/
                              LPARAM lParam)                            /*JEH1*/

  /********************************************************************/
  /* Client window procedure                                          */
  /********************************************************************/

{
  static int winXLen;                  /* x-length of client window   */
  static int winYLen;                  /* y-length of client window   */
  static HANDLE hInstance;             /* instance handle of          */
                                       /* execution                   */
  static DLGPROC lpfnJumpDlgProc;      /* far pointer to dialog       *//*JEH1*/
                                       /* procedure                   */

  switch (msg)
  {

    /******************************************************************/
    /* WM_CREATE is sent when the window is created. It is here that  */
    /* we initialize the window by creating its children and loading  */
    /* its cursor resource                                            */
    /******************************************************************/

    case  WM_CREATE :
      {
        RECT windowSize;               /* size of client window       */

        /**************************************************************/
        /* get client window size and store it                        */
        /**************************************************************/

        GetClientRect(hwnd, &windowSize);
        winXLen = (int)windowSize.right;
        winYLen = (int)windowSize.bottom;

        /**************************************************************/
        /* initialize the current page to NULL                        */
        /**************************************************************/

        currDate.month = YEAR_TITLEPAGE;
        currDate.year = 0;

        /**************************************************************/
        /* store instance handle and far pointer to dialog procedure  */
        /**************************************************************/

        hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
        lpfnJumpDlgProc = MakeProcInstance((FARPROC)JumpDlgProc,        /*JEH1*/
                                                             hInstance);
        /**************************************************************/
        /* now create the child notebook and fill it                  */
        /**************************************************************/

        if (!createChildren(hwnd, hInstance))
          return FALSE;
      }                                /* end of WM_CREATE processing */
      break;

    /****************************************************************/
    /* We don't want the client area to have the focus, since it    */
    /* doesn't accept input.  When it gets the focus, which it will */
    /* when the window is restored from a minimized state, for      */
    /* instance, post a user message to ourselves to reset the      */
    /* focus to the notebook.  Posting the message is necessary     */
    /* because the focus shouldn't be tampered with when processing */
    /* a WM_SETFOCUS or WM_KILLFOCUS message.                       */
    /****************************************************************/
    case WM_SETFOCUS:
      PostMessage (hwnd, WM_USER, 0, 0L);                               /*JEH1*/
      break;

    case WM_USER:
      SetFocus (GetDlgItem (hwnd, ID_BOOK_CALENDAR));
      break;

    /******************************************************************/
    /* The WM_SIZE message is received whenever the window size       */
    /* changes. lParam contains the new size (length in low word,     */
    /* height in high word). We extract the new size and adjust the   */
    /* notebook window to fit the client window                       */
    /******************************************************************/

    case  WM_SIZE :

      /****************************************************************/
      /* update window size                                           */
      /****************************************************************/

      winXLen = LOWORD(lParam);
      winYLen = HIWORD(lParam);

      /****************************************************************/
      /* now move the notebook window                                 */
      /****************************************************************/

      MoveWindow(GetDlgItem(hwnd, ID_BOOK_CALENDAR),
                 BOOK_MARGIN_XLEFT,
                 BOOK_MARGIN_YTOP,
                 (int)(winXLen -
                       BOOK_MARGIN_XLEFT -
                       BOOK_MARGIN_XRIGHT),
                 (int)(winYLen -
                       BOOK_MARGIN_YBOTTOM -
                       BOOK_MARGIN_YTOP),
                 TRUE);

      /****************************************************************/
      /* adjust page turn button size                                 */
      /****************************************************************/
//       SendDlgItemMessage (hwnd, ID_BOOK_CALENDAR
//                           BKM_SETDIMENSIONS,
//                           MAKELONG (winXLen / BOOK_BUTTON_WIDTH,
//                                     winYLen / BOOK_BUTTON_HEIGHT),
//                           BKA_PAGEBUTTON );

      break;

      /****************************************************************/
      /* The client window receives a WM_COMMAND message to inform it */
      /* of important events in the child windows. wParam is the id   */
      /* of the control, and HIWORD of lParam is the message code     */
      /****************************************************************/

    case  WM_COMMAND :

      /****************************************************************/
      /* based on source of message                                   */
      /****************************************************************/

      switch (LOWORD(wParam))                                           /*JEH1*/
      {
        case  ID_BOOK_CALENDAR :

          /************************************************************/
          /* based on notification code of message...                 */
          /************************************************************/

          switch (HIWORD(wParam))                                       /*JEH1*/
          {

            /**********************************************************/
            /* The BKN_PAGESELECTED message is sent by the notebook   */
            /* control whenever a new page is about to be displayed.  */
            /* Low word of lParam contains the notebook window        */
            /* handle. When this message is received, we perform any  */
            /* necessary initializations to the internal data and the */
            /* notebook--including enabling and disabling menu items, */
            /* setting the current page in currPage, and setting the  */
            /* background color for the page                          */
            /**********************************************************/

            case  BKN_PAGESELECTED :
              newPageSelected( hwnd, (LPVOID)lParam );                  /*DAB2*/
              break;
            default  :                 /* other messages from         */
                                       /* ID_BOOK_CALENDAR            */
              break;
          }                            /* end of switch on messages   */
                                       /* from ID_BOOK_CALENDAR       */
          break;

        /**************************************************************/
        /* The IDM_INSERT_AFTER and IDM_INSERT_BEFORE menu commands   */
        /* insert a previously deleted calendar page to the           */
        /* notebook. since the menu item was selected, we assume      */
        /* that the insert is legal... error checking was done when   */
        /* the current page was brought to the top of the notebook    */
        /* (we enabled or disabled the menu item)                     */
        /**************************************************************/

        case  IDM_INSERT_AFTER :
        case  IDM_INSERT_BEFORE :
          menuInsertPage(GetDlgItem(hwnd, ID_BOOK_CALENDAR),
                         LOWORD(wParam));                               /*DAB1*/
          break;

          /************************************************************/
          /* The IDM_DELETE_CURRENT menu item refers to the "delete   */
          /* current page" item in the user menu. It is only valid for*/
          /* calendar month pages, and we assume that validation was  */
          /* done when the new page was selected (menu item           */
          /* enabled/disabled). We delete the current page from the   */
          /* notebook, set a flag internally allowing this page to be */
          /* reinserted again later, and move to the next page in the */
          /* notebook.                                                */
          /************************************************************/

        case  IDM_DELETE_CURRENT :
          {
            UINT pageDeleted = PAGE_ID_FLAG_DELETED; /* used to set   *//*DAB1*/
                                       /* internal data               */
            UINT delPageId;            /* page id being deleted       *//*DAB1*/
            UINT newPageId;            /* page id being turned to     *//*DAB1*/
            DATE delDate;              /* date being deleted          */

            /**********************************************************/
            /* since the menu item was selected, we assume that the   */
            /* insert is legal... error checking was done when        */
            /* enabling or disabling the menu item first find out     */
            /* page ID of current and next page                       */
            /**********************************************************/

            delPageId = SendDlgItemMessage(hwnd,
                                           ID_BOOK_CALENDAR,
                                           BKM_QUERYPAGEID,
                                           0,                           /*JEH1*/
                                           (LONG)BKA_TOP);
            newPageId = SendDlgItemMessage(hwnd,
                                           ID_BOOK_CALENDAR,
                                           BKM_QUERYPAGEID,
                                           delPageId,
                                           (LONG)BKA_NEXT);

            /**********************************************************/
            /* date being deleted is the current page                 */
            /**********************************************************/

            delDate = currDate;

            /**********************************************************/
            /* we save the old date because as we select the new      */
            /* page, the static current date will change set internal */
            /* flag...we can recover page                             */
            /**********************************************************/

            pageRecord(PAGE_ID_CMD_STORE, &pageDeleted, &delDate);

            /**********************************************************/
            /* turn to next page                                      */
            /**********************************************************/

            SendDlgItemMessage(hwnd,
                               ID_BOOK_CALENDAR,
                               BKM_TURNTOPAGE,
                               newPageId,
                               0L);

            /**********************************************************/
            /* ... and delete current                                 */
            /**********************************************************/

            SendDlgItemMessage(hwnd,
                               ID_BOOK_CALENDAR,
                               BKM_DELETEPAGE,
                               delPageId,
                               (LONG)BKA_SINGLE);
          }                            /* end of case                 */
                                       /* IDM_DELETE_CURRENT          */
          break;

        /**************************************************************/
        /* The IDM_JUMP_TO_PAGE menu item is the "Jump" menu item.    */
        /* It brings up a dialog box allowing the user to jump to     */
        /* an arbitrary point in the notebook                         */
        /**************************************************************/

        case  IDM_JUMP_TO_PAGE :
          {
            UINT newPageId;            /* new page to jump to         *//*DAB1*/
            DATE newDate;              /* used to store new page data */

            /**********************************************************/
            /* initialize dialog box with current date                */
            /**********************************************************/

            newDate = currDate;

            /**********************************************************/
            /* if dialog box returns successfully then process user   */
            /* input                                                  */
            /**********************************************************/

            if (DialogBoxParam(hInstance,
                               MAKEINTRESOURCE(ID_DLG_JUMP_PAGE),
                               hwnd,
                               lpfnJumpDlgProc,
                               (DWORD)((LPVOID)&newDate)))
            {

              /********************************************************/
              /* user decided to jump, so find page ID                */
              /********************************************************/

              pageRecord(PAGE_ID_CMD_RETRIEVE, &newPageId, &newDate);

              /********************************************************/
              /* if page has been deleted, then tell user             */
              /********************************************************/

              if (newPageId == PAGE_ID_FLAG_DELETED)
                MessageBox(hwnd,
                           "That page has been deleted from the "
                           "calendar!  Selection cancelled.",
                           NULL,
                           MB_APPLMODAL   |
                           MB_DEFBUTTON1  |
                           MB_OK          |
                           MB_ICONEXCLAMATION);
              else

                /******************************************************/
                /* valid page...so jump                               */
                /******************************************************/

                SendDlgItemMessage(hwnd,
                                   ID_BOOK_CALENDAR,
                                   BKM_TURNTOPAGE,
                                   newPageId,
                                   0L);
            }                          /* end of if we are jumping    */
          }                            /* end of case: IDM_JUMP_TO    */
          break;

        /**************************************************************/
        /* The IDM_SUBEXIT id is the "Exit" menu option. To process   */
        /* it, we send ourselves a close message.                     */
        /**************************************************************/

        case  IDM_SUBEXIT :
          SendMessage(hwnd, WM_CLOSE, 0, 0);                            /*JEH1*/
          return  FALSE;

        default  :                     /* other sources of command    */
                                       /* messages                    */
          break;
      }                                /* end of switch: commands     */
      break;

    /******************************************************************/
    /* client painting is simple... just let Windows paint the        */
    /* window in our client background color! The notebook will       */
    /* paint itself.                                                  */
    /******************************************************************/

    case  WM_PAINT :
      {
        PAINTSTRUCT ps;                /* paint structure             */

        BeginPaint(hwnd, &ps);
        EndPaint(hwnd, &ps);
      }                                /* end of case WM_PAINT        */
      break;

    /******************************************************************/
    /* WM_DESTROY is received when the window has been hidden and     */
    /* is about to be removed from memory.                            */
    /******************************************************************/

    case  WM_DESTROY :

      /****************************************************************/
      /* deallocate window resources                                  */
      /****************************************************************/

      FreeProcInstance((FARPROC)lpfnJumpDlgProc);/* unlock dlg proc   *//*JEH1*/
      PostQuitMessage(0);
      break;
    default  :                         /* other messages              */
      break;
  }                                    /* endswitch                   */
  return (DefWindowProc(hwnd, msg, wParam, lParam));
}                                      /* end of client window        */
                                       /* procedure                   */

/**********************************************************************/
/*  Function:  createChildren                                         */
/*  Inputs:    hwndClient           client window handle              */
/*             hInstance            instance handle of execution      */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function creates a notebook and fills it with pages--year    */
/*  title and month calendar pages.                                   */
/**********************************************************************/

BOOL createChildren(HANDLE hwndClient,HANDLE hInstance)
{
  HANDLE hwndNotebook;                 /* notebook window handle      */
  HANDLE hwndPage;                     /* page handle                 */
  DWORD ulBookStyle;                   /* notebook style              */
  RECT rcl;                            /* client window size          */
  UINT pageIndex;                      /* current page inserting      *//*JEH1*/
  time_t todayRawTime;                 /* current time...in seconds   */
  struct tm *today;                    /* today's date...from C lib.  */
  DATE todayDate;                      /* date to initialize notebook */

  /********************************************************************/
  /* get size of client rectangle                                     */
  /********************************************************************/

  GetClientRect(hwndClient, &rcl);
  ulBookStyle = BKS_BACKPAGESBR    |   /* back pages on bottom right  */
                BKS_MAJORTABBOTTOM |   /* major tabs on bottom...     */
                                       /* implies minor tabs on right */
                BKS_STATUSTEXTLEFT |   /* status text left-justified  */
                WS_CHILD;              /* child window                */

  /********************************************************************/
  /* create a notebook                                                */
  /********************************************************************/

  hwndNotebook = CreateWindow(ICL_NOTEBOOK,       /* class            */
                              NULL,               /* title            */
                              ulBookStyle,        /* style            */
                              BOOK_MARGIN_XLEFT,  /* origin           */
                              BOOK_MARGIN_YTOP,
                              (int)(rcl.right -   /* size             */
                                    BOOK_MARGIN_XLEFT -
                                    BOOK_MARGIN_XRIGHT),
                              (int)(rcl.bottom -
                                    BOOK_MARGIN_YBOTTOM -
                                    BOOK_MARGIN_YTOP),
                              hwndClient,        /* parent            */
                              (HMENU)ID_BOOK_CALENDAR,  /* class menu *//*JEH1*/
                              hInstance,         /* creator           */
                              NULL);             /* data              */

  if (!hwndNotebook)
    return FALSE;

  /********************************************************************/
  /* set up notebook parameters set minor tab size                    */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETDIMENSIONS,
              MAKELONG(BOOK_MINOR_WIDTH,       /* width               *//*DAB2*/
                       BOOK_MINOR_HEIGHT),     /* height              *//*DAB2*/
              BKA_MINORTAB );                                           /*DAB2*/

  /********************************************************************/
  /* set major tab size                                               */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETDIMENSIONS,
              MAKELONG(BOOK_MAJOR_WIDTH,     /* width                 *//*DAB2*/
                       BOOK_MAJOR_HEIGHT),   /* height                *//*DAB2*/
              BKA_MAJORTAB );                                           /*DAB2*/

  /********************************************************************/
  /* page turn button size                                            */
  /********************************************************************/
// SendMessage (hwndNotebook,
//              BKM_SETDIMENSIONS,
//              MAKELONG (rcl.xRight / BOOK_BUTTON_WIDTH,   /* width  */
//                        rcl.yTop / BOOK_BUTTON_HEIGHT),   /* height */
//              BKA_PAGEBUTTON );
  /********************************************************************/
  /* major tab color                                                  */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETNOTEBOOKCOLORS,
              (DWORD)(COLOR_MAJORTABS),       /* color                *//*DAB2*/
              BKA_BACKGROUNDMAJORCOLOR );                               /*DAB2*/

  /********************************************************************/
  /* minor tab color                                                  */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETNOTEBOOKCOLORS,
              (DWORD)(COLOR_MINORTABS),      /* color                 *//*DAB2*/
              BKA_BACKGROUNDMINORCOLOR );                               /*DAB2*/

  /********************************************************************/
  /* notebook window color                                            */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETNOTEBOOKCOLORS,
              (DWORD)(COLOR_CLIENT_BACKGROUND),   /* color            *//*DAB2*/
              BKA_BACKGROUNDCOLOR );                                    /*DAB2*/

  /********************************************************************/
  /* status text color                                                */
  /********************************************************************/

  SendMessage(hwndNotebook,
              BKM_SETNOTEBOOKCOLORS,
              (DWORD)(COLOR_STATUSTEXT),         /* color             *//*DAB2*/
              BKA_FOREGROUNDCOLOR );                                    /*DAB2*/

  /********************************************************************/
  /* first create a window for the client page parent gets reset by   */
  /* notebook to be notebooks's own current page window               */
  /********************************************************************/

  hwndPage = CreateWindow(szPageClass,           /* class             */
                          NULL,                  /* title             */
                          WS_CHILD,              /* style             */
                          0,
                          0,                     /* origin            */
                          1,
                          1,                     /* size              */
                          hwndNotebook,          /* parent            */
                          (HMENU)ID_WIN_CALENDAR_PAGE,  /* class menu *//*JEH1*/
                          hInstance,             /* creator           */
                          NULL);                 /* data              */

  /********************************************************************/
  /* now add pages to the book loop by year                           */
  /********************************************************************/

  for (todayDate.year = YEAR_START;
       todayDate.year <= YEAR_STOP;
       todayDate.year++)
  {

    /******************************************************************/
    /* add major tab at start of year                                 */
    /******************************************************************/

    todayDate.month = YEAR_TITLEPAGE;

    /******************************************************************/
    /* and add it to the notebook: BKA_MAJOR: add a major tab to the  */
    /* page BKA_AUTOPAGESIZE: automatically resize page to fit        */
    /* notebook page                                                  */
    /******************************************************************/

      createPage(hwndNotebook,               /* notebook              */
                 hwndPage,                   /* page window           */
                 todayDate,                  /* date of page          */
                 0L,                         /* ordering              *//*JEH1*/
                 MAKELONG(BKA_MAJOR |
                             BKA_AUTOPAGESIZE,  /* style              */
                          BKA_LAST));          /* placement           */

    /******************************************************************/
    /* a page for each month                                          */
    /******************************************************************/

    for (todayDate.month = 1;
         todayDate.month <= YEAR_MONTHS;
         todayDate.month++)

      /****************************************************************/
      /* if the page is a minor tab page, add that attribute when     */
      /* creating it: BKA_STATUSTEXTON: add status text to the page   */
      /****************************************************************/

      createPage(hwndNotebook, hwndPage, todayDate, 0L,                 /*JEH1*/
                 MAKELONG(
                    (((todayDate.month-1)%MONTHTABINTERVAL)== 0)?
                      (BKA_STATUSTEXTON | BKA_AUTOPAGESIZE | BKA_MINOR):
                      (BKA_STATUSTEXTON | BKA_AUTOPAGESIZE),
                  BKA_LAST));
  }                                    /* end of loop through years   */

  /********************************************************************/
  /* set initial page to today use standard C time functions to get   */
  /* date and time                                                    */
  /********************************************************************/

  time(&todayRawTime);                 /* seconds since 1970          */
  today = localtime(&todayRawTime);

  /********************************************************************/
  /* initialize a structure for today's date                          */
  /********************************************************************/

  todayDate.month = (int)(1+(today->tm_mon));
  todayDate.year = 1900+(today->tm_year);

  /********************************************************************/
  /* get the id for the page with today on it                         */
  /********************************************************************/

  pageRecord(PAGE_ID_CMD_RETRIEVE, &pageIndex, &todayDate);

  /********************************************************************/
  /* and go to that page                                              */
  /********************************************************************/

  SendMessage(hwndNotebook, BKM_TURNTOPAGE, pageIndex, 0L);

  /********************************************************************/
  /* finally, show the notebook                                       */
  /********************************************************************/

  ShowWindow(hwndNotebook, SW_SHOWNORMAL);

  return  TRUE;
}                                      /* end of function             */
                                       /* createChildren              */

/**********************************************************************/
/*  Function:  createPage                                             */
/*  Inputs:    hwndBook     notebook window handle                    */
/*             hwndPage     page window to associate with new page    */
/*             pageDate     date to associate with new page           */
/*             refIndex     reference page id...used when positioning */
/*                          page into notebook                        */
/*             style        page style                                */
/*             order        page positioning with respect to refIndex */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function creates a notebook page, associates it with the     */
/*  given date and window procedure, and stores the new page id       */
/*  internally for future reference.                                  */
/**********************************************************************/

VOID createPage(HANDLE hwndBook,HANDLE hwndPage,DATE pageDate,
                UINT refIndex, DWORD style)                             /*DAB1*/

  /********************************************************************/
  /* insert an appropriate page into the calendar book                */
  /********************************************************************/

{
  UINT  pageIndex;                        /* page id being inserted   *//*JEH1*/
  char text[MAXTEXTLENGTH];               /* tab and status text      */

  /********************************************************************/
  /* insert a page into notebook                                      */
  /********************************************************************/

  pageIndex = SendMessage(hwndBook, BKM_INSERTPAGE,                     /*JEH1*/
                                refIndex, (DWORD)style);

  /********************************************************************/
  /* associate new page with a window procedure                       */
  /********************************************************************/

  SendMessage(hwndBook, BKM_SETPAGEWINDOWHWND, pageIndex, (DWORD)hwndPage);

  /********************************************************************/
  /* and finally store the new page id internally; associate it with  */
  /* the appropriate date                                             */
  /********************************************************************/

  pageRecord(PAGE_ID_CMD_STORE, &pageIndex, &pageDate);

  /********************************************************************/
  /* now set tab text and status line text                            */
  /********************************************************************/

  switch (pageDate.month)
  {
    case  YEAR_TITLEPAGE :

      /****************************************************************/
      /* year title pages have a minor tab containng the year number  */
      /****************************************************************/

      wsprintf(text, "%d", pageDate.year);

      /****************************************************************/
      /* set the major tab text                                       */
      /****************************************************************/

      SendMessage(hwndBook, BKM_SETTABTEXT, pageIndex, (LONG)(LPSTR)text);
      break;
    default  :

      /****************************************************************/
      /* Some month pages have minor tabs containing quarters... if   */
      /* we have such a page, then add the tab text                   */
      /****************************************************************/

      if ((LOWORD(style))&BKA_MINOR)
        SendMessage(hwndBook,
                    BKM_SETTABTEXT,
                    pageIndex,            /* page to set text to      */
                                          /* associate                */
                    (DWORD)(LPSTR)monthTabs[(pageDate.month-1)/
                                             MONTHTABINTERVAL]);

      /****************************************************************/
      /* all minor tabs have status text containing the current year  */
      /****************************************************************/

      wsprintf(text, "   %d", pageDate.year);

      /****************************************************************/
      /* and associate the text with the page                         */
      /****************************************************************/

      SendMessage(hwndBook,
                  BKM_SETSTATUSLINETEXT,
                  pageIndex,
                  (DWORD)(LPSTR)text);
      break;
  }                                    /* end of switch               */
}                                      /* end of function createPage  */

/**********************************************************************/
/*  Function:  pageRecord                                             */
/*  Inputs:    function       id of command to perform                */
/*             pageId         page id to perform function upon        */
/*             date           date structure to perform function upon */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function maintains a table matching each notebook page id    */
/*  with a valid date.  All interfaces with page id's are performed   */
/*  through this function.  The following operations are supported:   */
/*     PAGE_ID_CMD_STORE     associate the given pageId with the given*/
/*                           date                                     */
/*     PAGE_ID_CMD_RETRIEVE  return the page id associated with the   */
/*                           given date                               */
/*     PAGE_ID_CMD_CONVERT   return the date associated with the given*/
/*                           page id                                  */
/**********************************************************************/

VOID pageRecord(int function,UINT *pageId,DATE *date)                   /*DAB1*/

  /********************************************************************/
  /* maintains the page ID for each page in the book                  */
  /********************************************************************/

{

  /********************************************************************/
  /* static data to hold page id: month and year pages are stored in  */
  /* the pages array--with year title pages in element [0].           */
  /********************************************************************/

  static DWORD pages[YEAR_COUNT][YEAR_MONTHS+1];

  switch (function)
  {
    case  PAGE_ID_CMD_STORE :

      /****************************************************************/
      /* if we are storing the id, place the id in the array location */
      /* for the month                                                */
      /****************************************************************/

      pages[(WORD)INDEXFROMYEAR(date->year)][date->month] = *pageId;
      break;

      break;

    /******************************************************************/
    /* to retrive the id for the given date, simply retrieve it       */
    /* from the proper location                                       */
    /******************************************************************/

    case  PAGE_ID_CMD_RETRIEVE :
      (*pageId) = pages[(WORD)INDEXFROMYEAR(date->year)][date->month];  /*JEH1*/
      break;

    /******************************************************************/
    /* to convert a page id into a date, we must search the data      */
    /* until a matching id is found                                   */
    /******************************************************************/

    case  PAGE_ID_CMD_CONVERT :
      {
        WORD   index;                  /* loop index                  */
        WORD   index2;                 /* loop index                  */

        /**************************************************************/
        /* search for page id in year and month pages                 */
        /**************************************************************/

        for (index = YEAR_START; index <= YEAR_STOP; index++)
        {
          for (index2 = 0; index2 <= YEAR_MONTHS; index2++)
          {
            if (pages[INDEXFROMYEAR(index)][index2] == (*pageId))
            {
              (date->year) = index;
              (date->month) = index2;
              return ;
            }                          /* end of if matching page     */
                                       /* found                       */
          }                            /* end of loop through months  */
        }                              /* end of loop through years   */

        /**************************************************************/
        /* if we are still here, then id has not been registered      */
        /**************************************************************/

        (date->month) = YEAR_TITLEPAGE;
        (date->year) = YEAR_INVALID;
        return ;
      }                                /* end of case                 */
      break;
  }                                    /* end of switch               */
}                                      /* end of function pageRecord  */

/**********************************************************************/
/*  Function:  newPageSelected                                        */
/*  Inputs:    hwnd                 handle of client window           */
/*             lParam               pointer to PAGESELECTNOTIFY       */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called when a new page is brought to the top of  */
/*  the notebook.  It performs necessary initializations to the       */
/*  internal data and the notebook--including enabling and disabling  */
/*  menu items, setting the current page in currPage, and setting the */
/*  background color for the page.                                    */
/**********************************************************************/

VOID newPageSelected( HANDLE hwnd, PPAGESELECTNOTIFY ppgSelNfy )        /*DAB2*/
{
  UINT  newPageId;                     /* page id of new top page     *//*DAB1*/
  HMENU hwndMenu;                      /* handle of user menu         */
  DATE  newDate;                       /* used to store new page data */
  HWND  hwndBook;                                                       /*DAB2*/

  /********************************************************************/
  /* if a new calendar page selected, decide whether it can be        */
  /* deleted, and whether the next or previous pages can be inserted  */
  /* get client window's menu handle                                  */
  /********************************************************************/
  hwndBook = ppgSelNfy->hwndBook;                                       /*DAB2*/
  hwndMenu = GetMenu(hwnd);

  /********************************************************************/
  /* get id of new top page                                           */
  /********************************************************************/
//newPageId = SendMessage(hwndBook, BKM_QUERYPAGEID, 0, (LONG)BKA_TOP); /*DAB1*/
  newPageId = ppgSelNfy->ulPageIdNew;                                   /*DAB2*/

  /********************************************************************/
  /* ... and translate it into a date                                 */
  /********************************************************************/

  pageRecord(PAGE_ID_CMD_CONVERT, &newPageId, &newDate);

  /********************************************************************/
  /* if current page is a calendar page, it may be deleted            */
  /********************************************************************/

  if (newDate.year != YEAR_INVALID)
  {

    /******************************************************************/
    /* enable "delete current page" menu item                         */
    /******************************************************************/

    EnableMenuItem(hwndMenu,
                   IDM_DELETE_CURRENT,
                   ((newDate.month >= 1) &&
                    (newDate.month <= YEAR_MONTHS))?
                   MF_ENABLED:
                   MF_GRAYED);

    /******************************************************************/
    /* now look at next page and see if can be inserted               */
    /******************************************************************/

    newDate.month++;

    /******************************************************************/
    /* if it is a valid calendar month, then get its page id          */
    /******************************************************************/

    if ((newDate.month >= 1) && (newDate.month <= YEAR_MONTHS))
      pageRecord(PAGE_ID_CMD_RETRIEVE, &newPageId, &newDate);

    /******************************************************************/
    /* if next page is a calendar page and it has been deleted, then  */
    /* it may be re-inserted--set menu item appropriately             */
    /******************************************************************/

    EnableMenuItem(hwndMenu,
                   IDM_INSERT_AFTER,
                   ((newDate.month >= 1) &&
                    (newDate.month <= YEAR_MONTHS) &&
                    (newPageId == PAGE_ID_FLAG_DELETED))?
                   MF_ENABLED:
                   MF_GRAYED);

    /******************************************************************/
    /* now check the page before the new top page, and see whether it */
    /* can be re-inserted                                             */
    /******************************************************************/

    newDate.month -= 2;

    /******************************************************************/
    /* if it is a valid calendar month, then get its page id          */
    /******************************************************************/

    if (newDate.month >= 1)
      pageRecord(PAGE_ID_CMD_RETRIEVE, &newPageId, &newDate);

    /******************************************************************/
    /* if previous page is a calendar page and it has been deleted,   */
    /* then it may be re-inserted--set menu item appropriately        */
    /******************************************************************/

    EnableMenuItem(hwndMenu,
                   IDM_INSERT_BEFORE,
                   ((newDate.month >= 1) &&
                    (newPageId == PAGE_ID_FLAG_DELETED))?
                   MF_ENABLED:
                   MF_GRAYED);

    /******************************************************************/
    /* now, come back to current top page                             */
    /******************************************************************/

    newDate.month++;

    /******************************************************************/
    /* store new date internally                                      */
    /******************************************************************/

    currDate = newDate;
  }                                    /* end of if valid ID          */
}                                      /* end of newPageSelected      */
                                       /* function                    */

/**********************************************************************/
/*  Function:  menuInsertPage                                         */
/*  Inputs:    hwndBook             notebook control handle           */
/*             commandId            menu item id                      */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function is called when the user wishes to re-insert a       */
/*  deleted calendar page either before or after the current page.    */
/*  We get the current date, create and insert the appropriate new    */
/*  page, and turn to the inserted page.                              */
/**********************************************************************/

VOID menuInsertPage(HANDLE hwndBook,WORD commandId)
{
  UINT currPageId;                     /* current page id             *//*DAB1*/
  DATE newDate;                        /* temporary date storage      */

  /********************************************************************/
  /* get id of top notebook page                                      */
  /********************************************************************/

  currPageId = SendMessage(hwndBook, BKM_QUERYPAGEID, 0, (LONG)BKA_TOP);/*DAB1*/

  /********************************************************************/
  /* store the current date                                           */
  /********************************************************************/

  newDate = currDate;

  /********************************************************************/
  /* if inserting after current page                                  */
  /********************************************************************/

  if (commandId == IDM_INSERT_AFTER)
  {

    /******************************************************************/
    /* set new date                                                   */
    /******************************************************************/

    newDate.month++;

    /******************************************************************/
    /* we can now recreate the new page                               */
    /******************************************************************/

    createPage(hwndBook,               /* notebook handle             */
               (HANDLE)SendMessage(hwndBook, /* page window           */
                                   BKM_QUERYPAGEWINDOWHWND,
                                   currPageId,
                                   0L),
               newDate,                /* date of page                */
               currPageId,             /* page id                     */
               MAKELONG(                         /* lParam            */
                  (((newDate.month-1)%MONTHTABINTERVAL) == 0)?
                  (BKA_STATUSTEXTON|BKA_AUTOPAGESIZE|BKA_MINOR):
                  (BKA_STATUSTEXTON|BKA_AUTOPAGESIZE),
                 BKA_NEXT) );
  }                                    /* end of if inserting after   */
                                       /* current page                */
  else                                 /* otherwise, insert before    */
                                       /* current page                */
  {

    /******************************************************************/
    /* set new date                                                   */
    /******************************************************************/

    newDate.month--;

    /******************************************************************/
    /* we can now recreate the new page                               */
    /******************************************************************/

    createPage(hwndBook,               /* notebook handle             */
               (HANDLE)SendMessage(hwndBook, /* page window           */
                                   BKM_QUERYPAGEWINDOWHWND,
                                   currPageId,
                                   0L),
               newDate,                /* date of page                */
               currPageId,             /* page id                     */
               MAKELONG(                         /* lParam            */
                  (((newDate.month-1)%MONTHTABINTERVAL) == 0)?
                  (BKA_STATUSTEXTON|BKA_AUTOPAGESIZE|BKA_MINOR):
                  (BKA_STATUSTEXTON|BKA_AUTOPAGESIZE),
                BKA_PREV) );
  }                                    /* end of if inserting before  */
                                       /* current page                */

  /********************************************************************/
  /* lastly, we set the current page to be the newly inserted page:   */
  /* get the id for it, and then turn to it                           */
  /********************************************************************/

  pageRecord(PAGE_ID_CMD_RETRIEVE, &currPageId, &newDate);
  SendMessage(hwndBook,
              BKM_TURNTOPAGE,
              currPageId,              /* new page id                 */
              0L);
}                                      /* end of function             */
                                       /* menuInsertPage              */

/**********************************************************************/
/*  Function:  PageWndProc                                            */
/*  Inputs:    hwnd                 client window handle              */
/*             msg                  message id                        */
/*             wParam               first parameter (word)            */
/*             lParam               second parameter (long)           */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/*  This function processes messages sent to the client page window.  */
/**********************************************************************/

LRESULT CALLBACK PageWndProc(HWND hwnd, UINT msg, WPARAM wParam,        /*JEH1*/
                             LPARAM lParam)                             /*JEH1*/
{
  static RECT windowSize;              /* current window size         */

  switch (msg)
  {

    /******************************************************************/
    /* WM_CREATE is sent when the window is created.                  */
    /******************************************************************/

    case  WM_CREATE :

      /****************************************************************/
      /* store the window rectangle size                              */
      /****************************************************************/

      GetClientRect(hwnd, &windowSize);

      break;

    /******************************************************************/
    /* The WM_SIZE message is received whenever the window size       */
    /* changes. lParam contains the new size (length in low word,     */
    /* height in high word). We extract the new size and adjust the   */
    /* notebook window to fit the client window                       */
    /******************************************************************/

    case  WM_SIZE :

      /****************************************************************/
      /* update window size                                           */
      /****************************************************************/

      windowSize.right = LOWORD(lParam);
      windowSize.bottom = HIWORD(lParam);
      break;

      /****************************************************************/
      /* To paint the page window, we determine the type of page we   */
      /* are drawing and call the appropriate procedure               */
      /****************************************************************/

    case  WM_PAINT :
      {
        HDC hdc;
        PAINTSTRUCT ps;

        hdc = BeginPaint(hwnd, &ps);

        switch (currDate.month)
        {
          case  YEAR_TITLEPAGE :
            drawYearTitlePage(hdc, windowSize, currDate.year);
            break;
          default  :
            drawMonthCalendar(hdc,
                              windowSize,
                              currDate.year,
                              currDate.month);
            break;
        }                              /* end of switch               */

        /**************************************************************/
        /* complete painting                                          */
        /**************************************************************/

        EndPaint(hwnd, &ps);
      }                                /* end of WM_PAINT processing  */
      break;
    default  :                         /* other messages              */
      break;
  }                                    /* endswitch                   */
  return (DefWindowProc(hwnd, msg, wParam, lParam));
}                                      /* end of page window          */
                                       /* procedure                   */

/**********************************************************************/
/*  Function:  drawYearTitlePage                                      */
/*  Inputs:    hdc                  device context for page           */
/*             windowSize           size of page                      */
/*             year                 year whose page is being drawn    */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function draws the title page for a year.  The title page    */
/*  contains the current year.                                        */
/**********************************************************************/

VOID drawYearTitlePage(HDC hdc,RECT windowSize,int year)

  /********************************************************************/
  /* draws year title page                                            */
  /********************************************************************/

{
  POINT ptl;                           /* text draw point             */
  char buffer[MAXTEXTLENGTH];          /* which text to draw          */
  WORD oldBackMode;                    /* old background mode         */
  DWORD oldTextColor;                  /* old text color              */

  /********************************************************************/
  /* set the appropriate background mode and text color               */
  /********************************************************************/

  oldBackMode = SetBkMode(hdc, TRANSPARENT);
  oldTextColor = SetTextColor(hdc, COLOR_YEARTITLETEXT);

  /********************************************************************/
  /* find the center of the page                                      */
  /********************************************************************/

  ptl.x = windowSize.right/2;
  ptl.y = windowSize.bottom/2;

  /********************************************************************/
  /* and draw the text centered at this point                         */
  /********************************************************************/

  centerJustCharStringAt(hdc,
                         &ptl,
                         (WORD)wsprintf(buffer, "%d", year),
                         buffer);

  /********************************************************************/
  /* restore device context                                           */
  /********************************************************************/

  SetBkMode(hdc, oldBackMode);
  SetTextColor(hdc, oldTextColor);
}                                      /* end of function             */
                                       /* drawYearTitlePage           */

/**********************************************************************/
/*  Function:  drawMonthCalendar                                      */
/*  Inputs:    hdc                  device context for page           */
/*             windowSize           size of page                      */
/*             year                 year whose page is being drawn    */
/*             month                month whose page is being drawn   */
/*  Outputs:   none                                                   */
/*                                                                    */
/*  This function draws a calendar page for the given month and year. */
/*  The heading month is drawn at a fixed location.  The page size is */
/*  divided into rows and columns, into which the days and dates are  */
/*  painted.                                                          */
/**********************************************************************/

VOID drawMonthCalendar(HDC hdc,RECT windowSize,int year,int month)

  /********************************************************************/
  /* draw a calendar page                                             */
  /********************************************************************/

{
  int colLength;                       /* length of each day column   */
  int rowHeight;                       /* height of each week row     */
  int day;                             /* loop index representing day */
                                       /* of week                     */
  POINT ptl;                           /* text drawing point          */
  char buffer[MAXTEXTLENGTH];          /* buffer for title text       */
  int index;                           /* days in month loop index    */
  int monthDays;                       /* number of days in current   */
                                       /* month                       */
  WORD oldBkMode;                      /* old background mix mode     */
  DWORD oldTextColor;                  /* old text color              */

  /********************************************************************/
  /* print header set background mode and color                       */
  /********************************************************************/

  oldBkMode = SetBkMode(hdc, OPAQUE);
//  oldBkMode = SetBkMode(hdc, TRANSPARENT);
  oldTextColor = SetTextColor(hdc, COLOR_CALENDARHEADER);

  /********************************************************************/
  /* center horizontally, place at fixed distance from top            */
  /********************************************************************/

  ptl.x = (windowSize.right-windowSize.left)/2;
  ptl.y = CAL_MARGIN_HEADER;

  /********************************************************************/
  /* and center header at this point                                  */
  /********************************************************************/

  centerJustCharStringAt(hdc,
                         &ptl,
                         (WORD)wsprintf((LPSTR)buffer,
                                        (LPSTR)"%s",
                                        (LPSTR)monthNames[month-1]),
                         buffer);

  /********************************************************************/
  /* adjust windowSize for actual calendar margins                    */
  /********************************************************************/

  windowSize.left += CAL_MARGIN_LEFT;
  windowSize.right -= CAL_MARGIN_RIGHT;
  windowSize.bottom -= CAL_MARGIN_BOTTOM;
  windowSize.top += CAL_MARGIN_TOP;

  /********************************************************************/
  /* calculate size of each cell: we need to fit enough columns for   */
  /* each day of the week; we need one row for days of week and an    */
  /* additional six rows for all of the dates                         */
  /********************************************************************/

  colLength = (windowSize.right-windowSize.left)/DAYCOUNT;
  rowHeight = (windowSize.bottom-windowSize.top)/7;

  /********************************************************************/
  /* place ourselves in middle of first row and just before first     */
  /* column                                                           */
  /********************************************************************/

  ptl.y = windowSize.top+(rowHeight/2);
  ptl.x = windowSize.left-(colLength/2);

  /********************************************************************/
  /* set text color                                                   */
  /********************************************************************/

  SetTextColor(hdc, COLOR_CALENDARDAYS);

  /********************************************************************/
  /* now output all of the days of the week                           */
  /********************************************************************/

  for (index = 0; index < DAYCOUNT; index++)
  {

    /******************************************************************/
    /* move to the next column                                        */
    /******************************************************************/

    ptl.x += colLength;

    /******************************************************************/
    /* if the columns are not big enough, then only print first       */
    /* letter of each day; otherwise, print full day name             */
    /******************************************************************/

    centerJustCharStringAt(hdc,
                           &ptl,
                           (WORD)(colLength < BOOK_COL_LENGTH_THRESH) ? /*JEH1*/
                              (WORD)1:                                  /*JEH1*/
                              (WORD)lstrlen(dayNames[index]),
                           (char *)(dayNames[(WORD)index]));            /*DAB1*/
  }                                    /* end of loop to print days   */
                                       /* at top of calendar          */

  /********************************************************************/
  /* now print calendar set the date text color                       */
  /********************************************************************/

  SetTextColor(hdc, COLOR_CALENDARDATES);

  /********************************************************************/
  /* find what day the month starts on                                */
  /********************************************************************/

  day = convertDateToDay(month, year);

  /********************************************************************/
  /* move to the the middle of the starting column                    */
  /********************************************************************/

  ptl.x = windowSize.left+(int)(colLength *(day+0.5));
  ptl.y += rowHeight;

  /********************************************************************/
  /* determine how many days are in month                             */
  /********************************************************************/

  monthDays = daysPerMonth[month-1];

  /********************************************************************/
  /* increase days to 29 if February of leap year                     */
  /********************************************************************/

  if ((month == 2) && (leapYear(year)))
    monthDays++;

  /********************************************************************/
  /* now draw the calendar numbers                                    */
  /********************************************************************/

  for (index = 1; index <= monthDays; index++)
  {

    /******************************************************************/
    /* right justify text in proper location                          */
    /******************************************************************/

    rtJustCharStringAt(hdc,
                       &ptl,
                       (WORD)wsprintf(buffer, "%d", index),
                       buffer);

    /******************************************************************/
    /* determine which day of week is next                            */
    /******************************************************************/

    day = (day+1)%DAYCOUNT;

    /******************************************************************/
    /* if day is Sunday, then move to next row                        */
    /******************************************************************/

    if (day == 0)
    {
      ptl.y += rowHeight;                   /* next row               */
      ptl.x = windowSize.left-(colLength/2);/* first column           */
    }                                       /* if we need to move to  */
                                            /* next line              */

    /******************************************************************/
    /* move to next column                                            */
    /******************************************************************/

    ptl.x += colLength;
  }                                    /* end of loop to print days   */
                                       /* of month                    */

  /********************************************************************/
  /* restore device context                                           */
  /********************************************************************/

  SetTextColor(hdc, oldTextColor);
  SetBkMode(hdc, oldBkMode);
}                                      /* end of function             */
                                       /* drawMonthCalendar           */

/**********************************************************************/
/*  Function:  rtJustCharStringAt                                     */
/*  Inputs:    hdc              device context for page               */
/*             endPt            point to justify against              */
/*             length           number of characters in text string   */
/*             text             pointer to string to justify          */
/*  Outputs:   TRUE if successful; FALSE otherwise (same as TextOut)  */
/*                                                                    */
/*  This function right justifies text at a given point.              */
/**********************************************************************/

BOOL rtJustCharStringAt(HDC hdc,POINT *endPt,WORD length,char *text)

  /********************************************************************/
  /* right justifies text at the given point                          */
  /********************************************************************/

{
  WORD oldTextAlign;                   /* old text alignment          */
  BOOL returnVal;                      /* return value from TextOut   */

  /********************************************************************/
  /* set text alignment to right justify and baseline                 */
  /********************************************************************/

  oldTextAlign = SetTextAlign(hdc,
                              TA_RIGHT|TA_BASELINE);

  /********************************************************************/
  /* draw the text                                                    */
  /********************************************************************/

  returnVal = TextOut(hdc,
                      endPt->x,
                      endPt->y,
                      text,
                      length);

  /********************************************************************/
  /* restore the device context                                       */
  /********************************************************************/

  SetTextAlign(hdc, oldTextAlign);
  return (returnVal);
}                                      /* end of function             */
                                       /* rtJustCharStringAt          */

/**********************************************************************/
/*  Function:  centerJustCharStringAt                                 */
/*  Inputs:    hdc             device context for page                */
/*             midPt           point to center against                */
/*             length          number of characters in text string    */
/*             text            pointer to string to center            */
/*  Outputs:   TRUE if successful; FALSE otherwise (same as TextOut)  */
/*                                                                    */
/*  This function centers text at a given point.                      */
/**********************************************************************/

BOOL centerJustCharStringAt(HDC hdc,POINT *midPt,WORD length,
                            char *text)

  /********************************************************************/
  /* centers text at the given point                                  */
  /********************************************************************/

{
  WORD oldTextAlign;                   /* old text alignment          */
  BOOL returnVal;                      /* return value from TextOut   */

  /********************************************************************/
  /* set text alignment to center and baseline                        */
  /********************************************************************/

  oldTextAlign = SetTextAlign(hdc, TA_CENTER | TA_BASELINE);

  /********************************************************************/
  /* draw the text                                                    */
  /********************************************************************/

  returnVal = TextOut(hdc,
                      midPt->x,
                      midPt->y,
                      text,
                      length);

  /********************************************************************/
  /* restore the device context                                       */
  /********************************************************************/

  SetTextAlign(hdc, oldTextAlign);
  return (returnVal);
}                                      /* end of function             */
                                       /* centerJustCharStringAt      */

/**********************************************************************/
/*  Function:  leapYear                                               */
/*  Inputs:    year             year to check for leap year status    */
/*  Outputs:   TRUE if year is a leap year; FALSE otherwise           */
/*                                                                    */
/*  This function determines whether the given year is a leap year.   */
/**********************************************************************/

BOOL leapYear(int year)

  /********************************************************************/
  /* determines whether current year is a leap year If 1900, 2000,    */
  /* 2100, etc. then not a leap year; otherwise, if 1904, 1908, 1912, */
  /* etc. then a leap year; otherwise, not a leap year                */
  /********************************************************************/

{

  /********************************************************************/
  /* if century, then not a leap year                                 */
  /********************************************************************/

  if (!(year%1000))
    return  FALSE;

  /********************************************************************/
  /* if divisible by 4, then a leap year                              */
  /********************************************************************/

  if (!(year%4))
    return  TRUE;

  /********************************************************************/
  /* otherwise, not a leap year                                       */
  /********************************************************************/

  return  FALSE;
}                                      /* end of function leapYear    */

/**********************************************************************/
/*  Function:  convertDatetoDay                                       */
/*  Inputs:    month                month of date to convert          */
/*             year                 year of date to convert           */
/*  Outputs:   day of week that first day of month, year falls:       */
/*             0=Sun., 6=Sat.                                         */
/*                                                                    */
/*  This function determines which day of the week the first day of   */
/*  the month falls.                                                  */
/**********************************************************************/

int convertDateToDay(int month,int year)

  /********************************************************************/
  /* receives a month and year and returns the day on which the first */
  /* of the month falls: 0=Sunday, 6=Saturday;                        */
  /********************************************************************/

{

  /********************************************************************/
  /* January 1, 1991 was a Tuesday!                                   */
  /********************************************************************/

  int currentDay = 2;
  int currentYear = 1991;
  int currentMonth = 1;

  /********************************************************************/
  /* now we simply brute force by adding and subtracting the day      */
  /* offset if the sought after date is later, then move forward in   */
  /* time                                                             */
  /********************************************************************/

  for (; currentYear < year; currentYear++)
    currentDay = (currentDay+((leapYear(currentYear))?366:365))%
       DAYCOUNT;

  /********************************************************************/
  /* if the sought after date is prior, then move backward in time    */
  /********************************************************************/

  for (; currentYear > year; currentYear--)
    currentDay = (currentDay-((leapYear(currentYear))?366:365))%
       DAYCOUNT;

  /********************************************************************/
  /* now that we have the date of (January 1, year), we proceed by    */
  /* month                                                            */
  /********************************************************************/

  for (; currentMonth < month; currentMonth++)
  {
    currentDay = (currentDay+daysPerMonth[currentMonth-1])%DAYCOUNT;

    /******************************************************************/
    /* if February of leap year, we have to add an extra day          */
    /******************************************************************/

    if (leapYear(currentYear) && (currentMonth == 2))
      currentDay = (currentDay+1)%DAYCOUNT;
  }                                    /* end of loop through months  */
                                       /* in year                     */

  /********************************************************************/
  /* we now have (month 1, year) and are done; just make certain that */
  /* currentDay is positive                                           */
  /********************************************************************/

  return ((currentDay+DAYCOUNT)%DAYCOUNT);
}                                      /* end of function             */
                                       /* convertDateToDay            */

/**********************************************************************/
/*  Function:  JumpDlgProc                                            */
/*  Inputs:    hwnd                 dialog window handle              */
/*             msg                  message id                        */
/*             wParam               first parameter (word)            */
/*             lParam               second parameter (long)           */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/*  This function processes messages sent to the jump dialog window.  */
/**********************************************************************/

BOOL CALLBACK JumpDlgProc(HWND hwnd, UINT msg, WPARAM wParam,           /*JEH1*/
                          LPARAM lParam)                                /*JEH1*/

  /********************************************************************/
  /* dialog window procedure                                          */
  /********************************************************************/

{
  static pDATE dateInfo;               /* the selected date           */
  static char buffer[MAXTEXTLENGTH];   /* string for year from edit   */
                                       /* field                       */

  switch (msg)
  {

    /******************************************************************/
    /* The WM_INITDIALOG message is sent when the dialog box is       */
    /* created. lParam contains the parameter passed by the calling   */
    /* function. We initialize the radiobuttons and edit field        */
    /******************************************************************/

    case  WM_INITDIALOG :

      /****************************************************************/
      /* get the currently selected date                              */
      /****************************************************************/

      dateInfo = (pDATE)(LPVOID)(lParam);

      /****************************************************************/
      /* select the correct radio button                              */
      /****************************************************************/

      SendDlgItemMessage(hwnd,
                         IDD_BOX_MONTH+(dateInfo->month),
                         BM_SETCHECK,
                         TRUE,
                         0L);

      /****************************************************************/
      /* set the maximum text length in the edit control to be four   */
      /* digits...the size of a year                                  */
      /****************************************************************/

      SendDlgItemMessage(hwnd,
                         IDD_EDIT_YEAR,
                         EM_LIMITTEXT,
                         4,
                         0L);

      /****************************************************************/
      /* initialize the edit field with the given year                */
      /****************************************************************/

      wsprintf(buffer, "%d", dateInfo->year);

      /****************************************************************/
      /* set the edit field text                                      */
      /****************************************************************/

      SetDlgItemText(hwnd, IDD_EDIT_YEAR, buffer);

      /****************************************************************/
      /* and give the focus to the first button                       */
      /****************************************************************/

      SetFocus(GetDlgItem(hwnd, IDD_BOX_MONTH+(dateInfo->month)));
      return (FALSE);

    /******************************************************************/
    /* The WM_COMMAND message informs the dialog procedure of         */
    /* significant occurrences inside dialog controls. wParam         */
    /* identifies the control                                         */
    /******************************************************************/

    case  WM_COMMAND :

      /****************************************************************/
      /* a push button was pressed                                    */
      /****************************************************************/

      switch (wParam)
      {

        /**************************************************************/
        /* before letting the user terminate with "OK", we make sure  */
        /* that the year in the edit field is valid                   */
        /**************************************************************/

        case  IDD_BTN_OK :

          /************************************************************/
          /* find out what is in the edit field                       */
          /************************************************************/

          GetDlgItemText(hwnd,
                         IDD_EDIT_YEAR,
                         buffer,
                         sizeof(buffer));

          /************************************************************/
          /* convert it into a number                                 */
          /************************************************************/

          dateInfo->year = atoi(buffer);

          /************************************************************/
          /* if the year is valid, then dismiss the dialog with a     */
          /* TRUE return value                                        */
          /************************************************************/

          if (((dateInfo->year) >= YEAR_START) &&
              ((dateInfo->year) <= YEAR_STOP))
          {
            EndDialog(hwnd, TRUE);
          }
          else
          {

            /**********************************************************/
            /* if an invalid year, then warn user and do not allow    */
            /* the dialog to be dismissed                             */
            /**********************************************************/

            MessageBox(hwnd,
                       "You have entered an invalid year!",
                       NULL,
                       MB_OK|MB_ICONEXCLAMATION|MB_DEFBUTTON1);

            /**********************************************************/
            /* put focus on erroneous edit field                      */
            /**********************************************************/

            SetFocus(GetDlgItem(hwnd, IDD_EDIT_YEAR));
          }                            /* end of else: do not dismiss */
                                       /* dialog yet                  */
          return  TRUE;
        case  IDD_BTN_CANCEL :

          /************************************************************/
          /* if "CANCEL" selected, dismiss dialog with FALSE return   */
          /* value                                                    */
          /************************************************************/

          EndDialog(hwnd, FALSE);
          return  TRUE;
        default  :

          /************************************************************/
          /* if a radio button is pressed, deselect the old one and   */
          /* select the new one                                       */
          /************************************************************/

          if ((wParam > IDD_BOX_MONTH) &&
              (wParam <= (IDD_BOX_MONTH + YEAR_MONTHS)))
          {

            /**********************************************************/
            /* deselect the old one                                   */
            /**********************************************************/

            SendDlgItemMessage(hwnd,
                               IDD_BOX_MONTH+(dateInfo->month),
                               BM_SETCHECK,
                               FALSE,
                               0L);

            /**********************************************************/
            /* store the new button                                   */
            /**********************************************************/

            dateInfo->month = wParam-IDD_BOX_MONTH;

            /**********************************************************/
            /* and select it                                          */
            /**********************************************************/

            SendDlgItemMessage(hwnd,
                               IDD_BOX_MONTH+(dateInfo->month),
                               BM_SETCHECK,
                               TRUE,
                               0L);
            return  TRUE;
          }                            /* end of if a month selection */
                                       /* button was checked          */
          break;
      }                                /* end of switch on button id  */
      break;

    default  :                         /* other messages              */
      break;
  }                                    /* end of switch on message    */
  return  FALSE;
}                                      /* end of dialog window        */
                                       /* procedure                   */
