/* Revision: 58 1.4 source/ui/cclnt/slider/sliderd.c, cclslid, ioc.v400, 000302  */
/*---------------------------------------------------------------------------*
 *
 *                              IBM CONFIDENTIAL
 *
 *            (IBM Confidential-Restricted when combined with the
 *              aggregated OCO source modules for this program)
 *
 *                            OCO SOURCE MATERIALS
 *
 *             20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)
 *
 *---------------------------------------------------------------------------*/
/**********************************************************************/
/* Sliderd.c                                                          */
/*                                                                    */
/* Slider interface routines for the Windows SDK Dialog Editor.       */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

#include <windows.h>
#include <custcntl.h>
#include <string.h>
#include "dlgedit.h"
#include "iclsldw.h"
#include "sliderp.h"
#include "sliderd.h"

HANDLE FAR PASCAL SldInfo  (void);
BOOL   FAR PASCAL SldStyle (HWND, HANDLE, LPFNSTRTOID, LPFNIDTOSTR);
WORD   FAR PASCAL SldFlags (WORD, LPSTR, WORD);
BOOL   FAR PASCAL SldDlgFn (HWND, WORD, WORD, DWORD);

void   EnableStyles (HWND,WORD,WORD,WORD,WORD,WORD,WORD,WORD);
WORD   QueryStyles  (HWND);
void   SetStyles    (HWND, DWORD);
LPSTR  QueryArmString         (WORD fs);
LPSTR  QueryButtonString      (WORD fs);
LPSTR  QueryOrientationString (WORD fs);
LPSTR  QuerySliderPosString   (WORD fs);
LPSTR  QueryScaleString       (WORD fs);
LPSTR  QuerySnapString        (WORD fs);
LPSTR  QueryReadOnlyString    (WORD fs);
LPSTR  QueryRibbonStripString (WORD fs);
LPSTR  QueryOwnerDrawString   (WORD fs);

extern HANDLE hSliderInstance;


/**********************************************************************/
/* SldInfo                                                            */
/*                                                                    */
/* Allocates a CTLINFO structure from the global heap to identify the */
/* controls that are supported by this DLL.  The caller of this       */
/* function is responsible for freeing the structure.                 */
/*                                                                    */
/* This function must be ordinal 2.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/
HANDLE FAR PASCAL SldInfo (void)
{
  HANDLE    hCtlInfo;
  LPCTLINFO lpCtlInfo;

  /********************************************************************/
  /* allocate a CTLINFO that the Dialog Editor will free              */
  /********************************************************************/
  if (hCtlInfo = GlobalAlloc (GHND, sizeof (CTLINFO)))
  {
    /******************************************************************/
    /* lock it down so we can address it                              */
    /******************************************************************/
    if (lpCtlInfo = (LPCTLINFO) GlobalLock (hCtlInfo))
    {
      lpCtlInfo->wVersion        = CCL_VERSION;
      lpCtlInfo->wCtlTypes       = 1;

      lpCtlInfo->Type[0].wType   = 0;
      lpCtlInfo->Type[0].wWidth  = 80;
      lpCtlInfo->Type[0].wHeight = 12;
      lpCtlInfo->Type[0].dwStyle = WS_CHILD | WS_CLIPCHILDREN;

      lstrcpy (lpCtlInfo->szClass, ICL_SLIDER);
      lstrcpy (lpCtlInfo->szTitle, CCL_COPYRIGHT);
      lstrcpy (lpCtlInfo->Type[0].szDescr, ICL_SLIDER);

      GlobalUnlock (hCtlInfo);
    }

    /******************************************************************/
    /* otherwise, we couldn't lock it down to initialize it, so punt */
    /******************************************************************/
    else
    {
      GlobalFree (hCtlInfo);
      hCtlInfo = NULL;
    }
  }

  return (hCtlInfo);
}


/**********************************************************************/
/* SldStyle                                                           */
/*                                                                    */
/* This function displays a modal dialog that is to collect the style */
/* of the slider from the user.                                       */
/*                                                                    */
/* This function must be ordinal 3.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/
BOOL FAR PASCAL SldStyle (HWND hwnd, HANDLE hCtlStyle,
                          LPFNSTRTOID lpfnStrToId, LPFNIDTOSTR lpfnIdToStr)
{
  PSTYLESTRUCT pss;
  BOOL         rc;

  if (!(pss = (PSTYLESTRUCT) LocalAlloc (LMEM_FIXED | LMEM_ZEROINIT, sizeof(STYLESTRUCT))))     return (FALSE);

  pss->hCtlStyle   = hCtlStyle;
  pss->lpCtlStyle  = NULL;
  pss->lpfnStrToId = lpfnStrToId;
  pss->lpfnIdToStr = lpfnIdToStr;

  rc = DialogBoxParam (hSliderInstance, MAKEINTRESOURCE (ID_SLDSTYLES), hwnd,
                       (FARPROC) SldDlgFn, (WORD) pss);

  LocalFree((HANDLE) pss);

  return (rc);
}


/**********************************************************************/
/* SldDlgFn                                                           */
/*                                                                    */
/* This function must be ordinal 6.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/

BOOL FAR PASCAL SldDlgFn (HWND hwnd, WORD msg, WORD wParam, DWORD lParam)
{
  PSTYLESTRUCT pss;
  DWORD        dwRet;
  char         szID[33];

  switch (msg)
  {
    case WM_INITDIALOG:
      pss = (PSTYLESTRUCT) lParam;

      /****************************************************************/
      /* Set the property that contains the pointer to the            */
      /* STYLESTRUCT /* lock down the CTLSTYLE structure, and make    */
      /* sure this is really  a slider; if any of these failed,       */
      /* bail out now                                                 */
      /****************************************************************/
      if (!SetProp (hwnd, MAKEINTRESOURCE (1), (HANDLE) pss) ||
          !(pss->lpCtlStyle = (LPCTLSTYLE) GlobalLock (pss->hCtlStyle)))
      {
        EndDialog (hwnd, FALSE);
        break;
      }

      /****************************************************************/
      /* initialize the ID field                                      */
      /****************************************************************/
      if ((*pss->lpfnIdToStr) (pss->lpCtlStyle->wId, szID, sizeof (szID)))
      {
        SetDlgItemText (hwnd, ID_ID, szID);
      }
      else
      {
        SetDlgItemInt (hwnd, ID_ID, pss->lpCtlStyle->wId, FALSE);
      }

      /****************************************************************/
      /* set the text limit for the ID field                          */
      /****************************************************************/
      SendDlgItemMessage (hwnd, ID_ID, EM_LIMITTEXT, sizeof (szID)-1, 0);

      /****************************************************************/
      /* check the default style buttons                              */
      /****************************************************************/
      SetStyles (hwnd, pss->lpCtlStyle->dwStyle);
      break;

    case WM_DESTROY:
      if (pss = (PSTYLESTRUCT) RemoveProp (hwnd, MAKEINTRESOURCE (1)))
      {
        if (pss->lpCtlStyle)  GlobalUnlock (pss->hCtlStyle);
      }
      break;

    case WM_COMMAND:
      switch (wParam)
      {
        case ID_HORIZONTAL:
          EnableStyles (hwnd, ID_SLIDERLEFT, ID_SLIDERRIGHT,
                        ID_SLIDERTOP, ID_SLIDERBOTTOM, ID_SLIDERCENTER,
                        ID_SLIDERCENTER, ID_SLIDERTOP);
          EnableStyles (hwnd, ID_HOMETOP, ID_HOMEBOTTOM,
                        ID_HOMELEFT, ID_HOMERIGHT, ID_HOMELEFT,
                        ID_HOMELEFT, ID_HOMETOP);
          EnableStyles (hwnd, ID_BUTTONSTOP, ID_BUTTONSBOTTOM,
                        ID_BUTTONSLEFT, ID_BUTTONSRIGHT, ID_NONE,
                        ID_BUTTONSLEFT, ID_BUTTONSTOP);
          break;

        case ID_VERTICAL:
          EnableStyles (hwnd, ID_SLIDERBOTTOM, ID_SLIDERTOP,
                        ID_SLIDERRIGHT, ID_SLIDERLEFT, ID_SLIDERCENTER,
                        ID_SLIDERCENTER, ID_SLIDERTOP);
          EnableStyles (hwnd, ID_HOMELEFT, ID_HOMERIGHT,
                        ID_HOMEBOTTOM, ID_HOMETOP, ID_HOMEBOTTOM,
                        ID_HOMELEFT, ID_HOMETOP);
          EnableStyles (hwnd, ID_BUTTONSLEFT, ID_BUTTONSRIGHT,
                        ID_BUTTONSBOTTOM, ID_BUTTONSTOP, ID_NONE,
                        ID_BUTTONSLEFT, ID_BUTTONSTOP);
          break;

        case IDOK:
          pss = (PSTYLESTRUCT) GetProp (hwnd, MAKEINTRESOURCE (1));

          /************************************************************/
          /* if the ID field was modified, get the new ID             */
          /************************************************************/
          if (SendDlgItemMessage (hwnd, ID_ID, EM_GETMODIFY, 0, 0))
          {
            GetDlgItemText (hwnd, ID_ID, szID, sizeof (szID));

            /**********************************************************/
            /* if the new ID is not a defined constant, bail out      */
            /**********************************************************/
            if (!LOWORD (dwRet = (*pss->lpfnStrToId) (szID)))  break;

            pss->lpCtlStyle->wId = HIWORD (dwRet);
          }

          LOWORD (pss->lpCtlStyle->dwStyle) = QueryStyles (hwnd);
          /* flow-through */

        case IDCANCEL:
          EndDialog (hwnd, (wParam == IDOK));
          break;
      }
      break;
  }

  return (FALSE);
}
/**********************************************************************/
/* EnableStyles                                                       */
/*                                                                    */
/* Enables the appropriate styles                                     */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/

void EnableStyles (HWND hwnd, WORD idInvalid1, WORD idInvalid2,
                   WORD idValid1, WORD idValid2, WORD idDefault,
                   WORD idFirst, WORD idLast)
{
  EnableWindow (GetDlgItem (hwnd, idValid1),   TRUE);
  EnableWindow (GetDlgItem (hwnd, idValid2),   TRUE);
  EnableWindow (GetDlgItem (hwnd, idInvalid1), FALSE);
  EnableWindow (GetDlgItem (hwnd, idInvalid2), FALSE);

  if (IsDlgButtonChecked (hwnd, idInvalid1) ||
      IsDlgButtonChecked (hwnd, idInvalid2))
  {
    CheckRadioButton (hwnd, idFirst, idLast, idDefault);
  }
}


/**********************************************************************/
/* QueryStyles                                                        */
/*                                                                    */
/* Queries the styles selected from the dialog box.                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/

WORD QueryStyles (HWND hwnd)
{
  DWORD flStyle;

  flStyle  = IsDlgButtonChecked (hwnd, ID_VERTICAL)        ? SLS_VERTICAL :
                                                             SLS_HORIZONTAL;

  flStyle |= IsDlgButtonChecked (hwnd, ID_PRIMARYSCALE2)   ? SLS_PRIMARYSCALE2 :
                                                             SLS_PRIMARYSCALE1;

  flStyle |= IsDlgButtonChecked (hwnd, ID_SLIDERLEFT)      ? SLS_LEFT   :
             IsDlgButtonChecked (hwnd, ID_SLIDERRIGHT)     ? SLS_RIGHT  :
             IsDlgButtonChecked (hwnd, ID_SLIDERBOTTOM)    ? SLS_BOTTOM :
             IsDlgButtonChecked (hwnd, ID_SLIDERTOP)       ? SLS_TOP    :
                                                             SLS_CENTER;

  flStyle |= IsDlgButtonChecked (hwnd, ID_HOMETOP)         ? SLS_HOMETOP   :
             IsDlgButtonChecked (hwnd, ID_HOMERIGHT)       ? SLS_HOMERIGHT :
             IsDlgButtonChecked (hwnd, ID_HOMEBOTTOM)      ? SLS_HOMEBOTTOM:
                                                             SLS_HOMELEFT;

  flStyle |= IsDlgButtonChecked (hwnd, ID_BUTTONSTOP)      ? SLS_BUTTONSTOP    :
             IsDlgButtonChecked (hwnd, ID_BUTTONSRIGHT)    ? SLS_BUTTONSRIGHT  :
             IsDlgButtonChecked (hwnd, ID_BUTTONSBOTTOM)   ? SLS_BUTTONSBOTTOM :
             IsDlgButtonChecked (hwnd, ID_BUTTONSLEFT)     ? SLS_BUTTONSLEFT   :
                                                             0;

  if (IsDlgButtonChecked (hwnd, ID_SNAP))        flStyle |= SLS_SNAPTOINCREMENT;
  if (IsDlgButtonChecked (hwnd, ID_READONLY))    flStyle |= SLS_READONLY;
  if (IsDlgButtonChecked (hwnd, ID_RIBBONSTRIP)) flStyle |= SLS_RIBBONSTRIP;
  if (IsDlgButtonChecked (hwnd, ID_OWNERDRAW))   flStyle |= SLS_OWNERDRAW;

  return (LOWORD (flStyle));
}


/**********************************************************************/
/* SetStyles                                                          */
/*                                                                    */
/* Sets the styles for the dialog box from what was passed in by the  */
/* Dialog Editor                                                      */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/

void SetStyles (HWND hwnd, DWORD flStyle)
{
  WORD idOrientation;

  /*---------------------------------------------------------*/
  /* if this is a vertical slider, use vertical defaults     */
  /* for shaft alignment, home position, and button position */
  /*---------------------------------------------------------*/
  if (flStyle & SLS_VERTICAL)
  {
    idOrientation = ID_VERTICAL;

    CheckDlgButton (hwnd, (flStyle & SLS_LEFT)          ? ID_SLIDERLEFT    :
                          (flStyle & SLS_RIGHT)         ? ID_SLIDERRIGHT   :
                                                          ID_SLIDERCENTER,  TRUE);
    CheckDlgButton (hwnd, (flStyle & SLS_HOMETOP)       ? ID_HOMETOP       :
                                                          ID_HOMEBOTTOM,    TRUE);
    CheckDlgButton (hwnd, (flStyle & SLS_BUTTONSTOP)    ? ID_BUTTONSTOP    :
                          (flStyle & SLS_BUTTONSBOTTOM) ? ID_BUTTONSBOTTOM :
                                                          ID_NONE,          TRUE);
  }

  /*----------------------------------------------------------------*/
  /* otherwise this is a horizontal slider, use horizontal defaults */
  /* for shaft alignment, home position, and button position        */
  /*----------------------------------------------------------------*/
  else
  {
    idOrientation = ID_HORIZONTAL;

    CheckDlgButton (hwnd, (flStyle & SLS_TOP)          ? ID_SLIDERTOP    :
                          (flStyle & SLS_BOTTOM)       ? ID_SLIDERBOTTOM :
                                                         ID_SLIDERCENTER, TRUE);
    CheckDlgButton (hwnd, (flStyle & SLS_HOMERIGHT)    ? ID_HOMERIGHT    :
                                                         ID_HOMELEFT,     TRUE);
    CheckDlgButton (hwnd, (flStyle & SLS_BUTTONSLEFT)  ? ID_BUTTONSLEFT  :
                          (flStyle & SLS_BUTTONSRIGHT) ? ID_BUTTONSRIGHT :
                                                         ID_NONE,         TRUE);
  }

  /*-------*/
  /* scale */
  /*-------*/
  CheckDlgButton (hwnd, (flStyle & SLS_PRIMARYSCALE2) ? ID_PRIMARYSCALE2 :
                                                        ID_PRIMARYSCALE1, TRUE);

  /*---------*/
  /* options */
  /*---------*/
  CheckDlgButton (hwnd, ID_SNAP,        (flStyle & SLS_SNAPTOINCREMENT) ? TRUE : FALSE);
  CheckDlgButton (hwnd, ID_READONLY,    (flStyle & SLS_READONLY)        ? TRUE : FALSE);
  CheckDlgButton (hwnd, ID_RIBBONSTRIP, (flStyle & SLS_RIBBONSTRIP)     ? TRUE : FALSE);
  CheckDlgButton (hwnd, ID_OWNERDRAW,   (flStyle & SLS_OWNERDRAW)       ? TRUE : FALSE);

  /*-------------*/
  /* orientation */
  /*-------------*/
  CheckDlgButton (hwnd, idOrientation, TRUE);
  SendMessage    (hwnd, WM_COMMAND, idOrientation, NULL);
}


/**********************************************************************/
/* SldFlags                                                           */
/*                                                                    */
/* This function must be ordinal 4.                                   */
/*                                                                    */
/* The Dialog Editor doesn't give us a big enough buffer to hold      */
/* all of the possible styles for the slider, so we return a string   */
/* containing abbreviations for the styles that are defined in        */
/* FCLSLDW.H                                                          */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in Slider                      */
/**********************************************************************/

WORD FAR PASCAL SldFlags (WORD fs, LPSTR lpStyle, WORD cbMax)
{
  char  szStyles[200];
  LPSTR lpSLS = "SLS";

  /********************************************************************/
  /* format the style string                                          */
  /********************************************************************/
  wsprintf (szStyles, "%s_%s | %s_%s | %s_%s | %s_%s%s%s%s%s%s",
            lpSLS, QueryOrientationString (fs),
            lpSLS, QueryScaleString (fs),
            lpSLS, QuerySliderPosString (fs),
            lpSLS, QueryArmString   (fs),
            QueryButtonString (fs),
            QuerySnapString (fs),
            QueryReadOnlyString (fs),
            QueryRibbonStripString (fs),
            QueryOwnerDrawString (fs));

  /********************************************************************/
  /* copy the style string we build into DlgEdit's buffer, make sure  */
  /* it's null-terminated, and return the bytes copied                */
  /********************************************************************/
  strncpy (lpStyle, szStyles, cbMax);
  lpStyle[cbMax-1] = '\0';
  return (lstrlen (lpStyle));
}


/**********************************************************************/
/* QueryArmString                                                     */
/*                                                                    */
/* Returns the string for the arm position style bit in a given flag  */
/* set.                                                               */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryArmString (WORD fs)
{
  if (fs & SLS_VERTICAL)
  {
    return ((fs & SLS_HOMETOP) ? "HT" : "HB");
  }
  else
  {
    return ((fs & SLS_HOMERIGHT) ? "HR" : "HL");
  }
}

/**********************************************************************/
/* QueryButtonString                                                  */
/*                                                                    */
/* Returns the string for the button location style bit in a given    */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryButtonString (WORD fs)
{
  if (fs & SLS_VERTICAL)
  {
    return ((fs & SLS_BUTTONSTOP)    ? " | SLS_BT" :
            (fs & SLS_BUTTONSBOTTOM) ? " | SLS_BB" : "");
  }
  else
  {
    return ((fs & SLS_BUTTONSRIGHT)  ? " | SLS_BR" :
            (fs & SLS_BUTTONSLEFT)   ? " | SLS_BL" : "");
  }
}

/**********************************************************************/
/* QueryOrientationString                                             */
/*                                                                    */
/* Returns the string for the slider orientation style bit in a given */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryOrientationString (WORD fs)
{
  return ((fs & SLS_VERTICAL) ? "VERT" : "HORZ");
}

/**********************************************************************/
/* QuerySliderPosString                                               */
/*                                                                    */
/* Returns the string for the slider position style bit in a given    */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QuerySliderPosString (WORD fs)
{
  if (fs & SLS_VERTICAL)
  {
    return ((fs & SLS_LEFT)   ? "L" :
            (fs & SLS_RIGHT)  ? "R" : "C");
  }
  else
  {
    return ((fs & SLS_BOTTOM) ? "B" :
            (fs & SLS_TOP)    ? "T" : "C");
  }
}


/**********************************************************************/
/* QueryScaleString                                                   */
/*                                                                    */
/* Returns the string for the scale location style bit in a given     */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryScaleString (WORD fs)
{
  return ((fs & SLS_PRIMARYSCALE2) ? "SCALE2" : "SCALE1");
}

/**********************************************************************/
/* QuerySnapStringString                                              */
/*                                                                    */
/* Returns the string for the snap to increment style bit in a given  */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QuerySnapString (WORD fs)
{
  return ((fs & SLS_SNAPTOINCREMENT) ? " | SLS_SNAP" : "");
}

/**********************************************************************/
/* QueryReadOnlyString                                                */
/*                                                                    */
/* Returns the string for the read-only style bit in a given          */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryReadOnlyString (WORD fs)
{
  return ((fs & SLS_READONLY) ? " | SLS_READONLY" : "");
}

/**********************************************************************/
/* QueryRibbonStripString                                             */
/*                                                                    */
/* Returns the string for the ribbon strip style bit in a given       */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryRibbonStripString (WORD fs)
{
  return ((fs & SLS_RIBBONSTRIP) ? " | SLS_RIBBON" : "");
}

/**********************************************************************/
/* QueryOwnerDraw                                                     */
/*                                                                    */
/* Returns the string for the ownder draw style bit in a given        */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryOwnerDrawString (WORD fs)
{
  return ((fs & SLS_OWNERDRAW) ? " | SLS_OWNERDRAW" : "");
}
