/* Revision: 60 1.5 source/ui/cclnt/slider/sliderm.c, cclslid, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*        (IBM Confidential-Restricted when combined with the         */
/*         aggregated OCO source modules for this program)            */
/*                                                                    */
/*                       OCO SOURCE MATERIALS                         */
/*                                                                    */
/*         20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)   */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/*                           S L I D E R                              */
/*                                                                    */
/*                       Filename: SLIDERM.C                          */
/*                                                                    */
/*              ( Windows NT Slider Control Component )               */
/*                                                                    */
/*                         IBM CONFIDENTIAL                           */
/*                                                                    */
/*                (C) Copyright IBM Corporation 1997                  */
/*                       All Rights Reserved                          */
/*            Licensed Material - Program Property of IBM             */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/* Product: SLIDER.DLL                                                */
/* Filename: SLIDERM.C - Main routines for the Slider control         */
/*                                                                    */
/* Flag    Date    By:  Description                                   */
/* ----  --------  ---  --------------------------------------------- */
/* JEH1  06/21/93  JEH  Initial modifications for NT execution        */
/* DAB1  05/20/94  DAB  Modify message API to match OS/2 (32 bit)     */
/* D4856 09/28/94  ASM  Adding Detent returns invalid ID              */
/* D5046 10/26/95  EVB1 Adding SLM_QUERYSLIDERCOLORS                  */
/**********************************************************************/
#include <string.h>

#include <windows.h>
#include "iclsldw.h"               /* Slider control public includes  */
#include "sliderp.h"               /* Slider control private includes */

/**********************************************************************/
/* Global control variables for slider                                */
/**********************************************************************/
HDC hdc = 0;
HDC hpsDragArm = 0;
WORD wUseCount = 0;

/**********************************************************************/
/* Global variables for DrawMaskBitMap                                */
/**********************************************************************/
HBITMAP   hbmScrSave = 0;          /* background preservation bitmap  */
HBITMAP   hbmIcon = 0;             /* background preservation bitmap  */
HBITMAP   hbmCopy = 0;             /* copy of arm bitmap stretched    */
HDC       hpsMem = 0;

/**********************************************************************/
/* Global handles for Arrow Bitmaps                                   */
/**********************************************************************/
HBITMAP  hbmVIncr = NULL;
HBITMAP  hbmVDecr = NULL;
HBITMAP  hbmHIncr = NULL;
HBITMAP  hbmHDecr = NULL;
HBITMAP  hbmVIncrS = NULL;
HBITMAP  hbmVDecrS = NULL;
HBITMAP  hbmHIncrS = NULL;
HBITMAP  hbmHDecrS = NULL;
HBITMAP  hbmVIncrD = NULL;
HBITMAP  hbmVDecrD = NULL;
HBITMAP  hbmHIncrD = NULL;
HBITMAP  hbmHDecrD = NULL;
HBITMAP  hbmVIncrSD = NULL;
HBITMAP  hbmVDecrSD = NULL;
HBITMAP  hbmHIncrSD = NULL;
HBITMAP  hbmHDecrSD = NULL;

/**********************************************************************/
/* Function: SliderWndProc - This is the main window procedure        */
/*           which handles all of the initialization and processing   */
/*           of all user input.                                       */
/*                                                                    */
/* Input:    hwnd - window handle of control                          */
/*           msg  - Id of the message which was sent/dispatched       */
/*                  to this window procedure                          */
/*           mp1  - First message parameter                           */
/*           mp2  - Second message parameter                          */
/*                                                                    */
/* Output:   Return value of the message result                       */
/**********************************************************************/
LRESULT CALLBACK SliderWndProc( HWND hwnd, UINT message,                /*JEH1*/
                                WPARAM wParam, LPARAM lParam )          /*JEH1*/
{
  PISLDDATA pIntData;
  ULONG     ulStyle;                                                    /*JEH1*/
  USHORT    usScale;                                                    /*JEH1*/
  BOOL      bSelectBtn=TRUE;
  PTHDATA   pthCurrent;
  BOOL bCreateReceived = FALSE;
  SLDCDATA  sldcDummyCB;

  /********************************************************************/
  /* Query the window words for the private data pointer for slider   */
  /********************************************************************/
  pIntData = (PISLDDATA)GetWindowLong( hwnd, QWL_SLDCTL );

  /********************************************************************/
  /* Query for the style of the slider control                        */
  /********************************************************************/
  ulStyle = GetWindowLong( hwnd, GWL_STYLE );

  /*------------------------------------------------------------------*/
  /* search the task-based linked list to find the DC for this slider */
  /*------------------------------------------------------------------*/
  if (pthCurrent = FindTaskData ())

    /******************************************************************/
    /* Get device context for this slider                             */
    /******************************************************************/
    hpsMem = pthCurrent->hdcMem;

  else if ((message != WM_CREATE) && bCreateReceived)
    /******************************************************************/
    /* Otherwise, if not in list, error as this should not be.        */
    /******************************************************************/
    return((long) FALSE);

  /********************************************************************/
  /* Check if a non-dynamic style bit of the slider control has been  */
  /* modified by the application - If so, take action accordingly     */
  /********************************************************************/
  if ( pIntData && ((ulStyle & SLS_TYPEMASK) != pIntData->ulStyle ))
  {
    /******************************************************************/
    /* Update the slider according to which style bit was changed     */
    /******************************************************************/
    UpdateSliderValues( hwnd, pIntData, (ulStyle & SLS_TYPEMASK) );
  }


  /********************************************************************/
  /* Now depending on which message was sent, take appropriate action */
  /********************************************************************/
  switch (message)
  {
    /******************************************************************/
    /* Initialize any control specific items before becoming visible  */
    /*   mp1: pctldata - Pointer to slider control data structure     */
    /*   mp2: pcreate  - not used                                     */
    /*   reply: Return TRUE if failure on create                      */
    /******************************************************************/
    case WM_CREATE:
      bCreateReceived = TRUE;
      return (InitializeSlider (hwnd,
          (LPSLDCDATA) ((LPCREATESTRUCT)lParam)->lpCreateParams));


    /******************************************************************/
    /* Terminate the control, freeing all memory allocated before exit*/
    /*   mp1: NULL                                                    */
    /*   mp2: NULL                                                    */
    /******************************************************************/
    case WM_DESTROY:
      TerminateSlider( hwnd, pIntData );
      break;

    /******************************************************************/
    /* When the slider window is sized, recalculate the slider size   */
    /******************************************************************/
    case WM_SIZE:
      sldcDummyCB.cbSize = sizeof (SLDCDATA);

      sldcDummyCB.usScale1Spacing =
          (pIntData->usStatusSld & FL_AUTOINCREMENT1) ? 0 :
          pIntData->usScale1Spacing;
      sldcDummyCB.usScale2Spacing =
          (pIntData->usStatusSld & FL_AUTOINCREMENT2) ? 0 :
          pIntData->usScale2Spacing;
      sldcDummyCB.usScale1Increments = pIntData->usScale1Increments;
      sldcDummyCB.usScale2Increments = pIntData->usScale2Increments;
      SetupSliderFromCB( hwnd, pIntData, &sldcDummyCB );
      break;

    /******************************************************************/
    /* If the slider appears in a dialog, indicate which input it     */
    /* needs (arrows only -- home and end get through by default)     */
    /******************************************************************/
    case WM_GETDLGCODE:
      return (DLGC_WANTARROWS);
      break;

    /******************************************************************/
    /* If mouse button 1 or 2 is depressed over the slider, find out  */
    /*  where it was depressed.  Reset flag if button two pressed     */
    /*  to differentiate - fall thru intentional                      */
    /*   mp1: xmouse - pointing device x coordinate                   */
    /*        ymouse - pointing device y coordinate                   */
    /*   mp2: hittestres - Not used                                   */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_MBUTTONDOWN:
    case WM_MBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONDBLCLK:
      bSelectBtn=FALSE;

    case WM_LBUTTONDBLCLK:
    case WM_LBUTTONDOWN:

      /****************************************************************/
      /* Check where the mouse was clicked in the window              */
      /****************************************************************/
      return ((long)SldProcessMouseDown( hwnd, pIntData,
                    LOWORD(lParam), HIWORD(lParam), bSelectBtn ));
      break;
    /******************************************************************/
    /* When button released, complete action in process.  Reset flag  */
    /*  if button two pressed to differentiate - fall thru intentional*/
    /*   mp1: xmouse - pointing device x coordinate                   */
    /*        ymouse - pointing device y coordinate                   */
    /*   mp2: hittestres - Not used                                   */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_MBUTTONUP:
    case WM_RBUTTONUP:
      bSelectBtn=FALSE;

    case WM_LBUTTONUP:

      /****************************************************************/
      /* Reset any action occurring with button down                  */
      /****************************************************************/
      return ((long)SldProcessMouseUp( hwnd, pIntData ));
      break;
    /******************************************************************/
    /* If character keys are pressed, move slider position accordingly*/
    /*   mp1: usKeyCtl   - Keyboard control code                      */
    /*   mp2: usCharCode - Not passed as no character processing      */
    /*        usVrtlKey  - Virtual key translation value of kbd event */
    /*   reply: TRUE - if keystroke processed                         */
    /*          FALSE - if keystroke ignored                          */
    /******************************************************************/
    case WM_KEYDOWN:
      if ( SldProcessVirtKey( hwnd, pIntData, wParam, lParam))
        return ((long)TRUE);
      else
        return ( DefWindowProc( hwnd, message, wParam, lParam) );
      break;

    /******************************************************************/
    /* If a focus message is received, update the window focus visual */
    /*   mp1: hwnd   - window handle receiving focus)                 */
    /*   mp2: not used                                                */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_KILLFOCUS:

      /****************************************************************/
      /* Call process focus message passing focus flag                */
      /****************************************************************/
      SldProcessFocusChg( hwnd, pIntData, FALSE );
      break;

    /******************************************************************/
    /* If an enable message is received, update slider to reflect it  */
    /*   mp1: bEnable - If TRUE, window is to be enabled              */
    /*                  If FALSE, window is to be disabled            */
    /*   mp2: none                                                    */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_ENABLE:

      /****************************************************************/
      /* Repaint entire slider to reflect enabled status change       */
      /****************************************************************/
      InvalidateRect (hwnd, NULL, FALSE);
      break;

    /******************************************************************/
    /* If a mouse move message comes in, process it                   */
    /*   mp1: xmouse - pointing device x coordinate                   */
    /*        ymouse - pointing device y coordinate                   */
    /*   mp2: hittestres - Not used                                   */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_MOUSEMOVE:

      /****************************************************************/
      /* Call mouse processing routine passing current mouse position */
      /****************************************************************/
      SldProcessMouseMove( hwnd, pIntData, LOWORD(lParam),
                           HIWORD(lParam) );
      return ((long)TRUE);

    /******************************************************************/
    /* Update the slider, painting the invalidated region             */
    /*   mp1: NULL                                                    */
    /*   mp2: NULL                                                    */
    /******************************************************************/
    case WM_PAINT:

      DrawSlider( hwnd, pIntData );
      return (FALSE);

    /******************************************************************/
    /* If a focus message is received, update the window focus visual */
    /*   mp1: hwnd   - window handle losing focus)                    */
    /*   mp2: not used                                                */
    /*   reply: none                                                  */
    /******************************************************************/
    case WM_SETFOCUS:

      /****************************************************************/
      /* Call process focus message passing focus flag                */
      /****************************************************************/
      SldProcessFocusChg( hwnd, pIntData, TRUE );
      break;

    /******************************************************************/
    /* If a timer message comes in, process it (in case it is ours)   */
    /*   mp1: timerid - Timer identity                                */
    /*   mp2: NULL                                                    */
    /******************************************************************/
    case WM_TIMER:

      /****************************************************************/
      /* Call timer routine passing in timer id for updating arm      */
      /****************************************************************/
      SldProcessTimer( hwnd, pIntData, wParam);
      break;

    /******************************************************************/
    /* If a WM_SETFONT message comes in, set the sliderFont field of  */
    /* the slider's internal data.                                    */
    /*   mp1: font handle - handle of the new font, if NULL then      */
    /*                      system font will be used                  */
    /*   mp2: if TRUE, redraw slider immediately, if FALSE, don't     */
    /******************************************************************/
    case WM_SETFONT:

      pIntData->hSliderFont = (HFONT) wParam;
      if (lParam)
        InvalidateRect( hwnd, NULL, TRUE );
      break;

    /******************************************************************/
    /* If a WM_GETFONT message comes in, return the sliderFont field  */
    /* of the slider's internal data.                                 */
    /******************************************************************/
    case WM_GETFONT:

      return (LRESULT)(pIntData->hSliderFont);                          /*JEH1*/
      break;

    /******************************************************************/
    /* If the system colors are changed, update the internal color    */
    /* data (Windows will invalidate the window)                      */
    /******************************************************************/
    case WM_SYSCOLORCHANGE:
      SldInitPresParam (pIntData);
      break;

    /******************************************************************/
    /* Add a detent to the slider at the specified location           */
    /*   mp1: usDetentPos - specifies location of detent represented  */
    /*                      as # of pixels from home                  */
    /*   mp2: NULL                                                    */
    /*   reply: usDetentId - unique detent identifier                 */
    /******************************************************************/
    case SLM_ADDDETENT:

      //D4856 return MAKELONG(AddDetent( hwnd, pIntData, LOWORD(wParam) ),0);   /*JEH1*/
      return (long)AddDetent( hwnd, pIntData, LOWORD(wParam) ); /*JEH1*/ /*D4856*/
    break;
    /******************************************************************/
    /* Query the position of a detent in the slider                   */
    /*   mp1: usDetentId - unique detent identifier which indicates   */
    /*                     the position to return                     */
    /*   mp2: NULL                                                    */
    /*   reply: usDetentPos - specifies location of detent represented*/
    /*                        as # of pixels from home                */
    /*          fDetentLocation - Denotes which scale located on      */
    /******************************************************************/
    case SLM_QUERYDETENTPOS:

      return (long)(QueryDetentPos( pIntData, (PDETENTDATA)wParam));
    break;
    /******************************************************************/
    /* Query the scale text at a specific tick position               */
    /*   mp1: usTickNum - this specifies the tick location to query   */
    /*                    for the text                                */
    /*        usBufLen  - Length of buffer to copy the text into      */
    /*   mp2: pszTickText - pointer to buffer to copy text into       */
    /*   reply: usTextLen - count of bytes copied to buffer           */
    /******************************************************************/
    case SLM_QUERYSCALETEXT:
      return (long)(QueryScaleText( hwnd,pIntData, LOWORD(wParam),      /*DAB1*/
                                    HIWORD(wParam), (LPSTR)lParam ));   /*DAB1*/
    break;
    /******************************************************************/
    /* Query specific information about the dimensions or position    */
    /* of the slider shaft or arm                                     */
    /*   mp1: usInfoType - Identifier of type of information requested*/
    /*        usArmPosType - Format of arm position information       */
    /*   mp2: NULL                                                    */
    /*   reply: ulInfo - Specific information requested               */
    /******************************************************************/
    case SLM_QUERYSLIDERINFO:

      return (long)(QuerySliderInfo( hwnd, pIntData,
                          LOWORD(wParam), HIWORD(wParam) ));            /*DAB1*/
    break;
    /******************************************************************/
    /* Query the window coordinates of a tick position                */
    /*   mp1: usTickNum - this specifies the tick location to query   */
    /*   mp2: NULL                                                    */
    /*   reply: x, y    - x,y coordinates of tick (end next to shaft) */
    /******************************************************************/
    case SLM_QUERYTICKPOS:

      /****************************************************************/
      /* Pass in as a parameter the current scale                     */
      /****************************************************************/
      if (ulStyle & SLS_PRIMARYSCALE2)
        usScale = SMA_SCALE2;
      else
        usScale = SMA_SCALE1;

      return (long)(QueryTickPos( hwnd, pIntData, usScale,
                                  LOWORD(wParam) ));                    /*JEH1*/
    break;
    /******************************************************************/
    /* Query the size of a tick at a specific tick position           */
    /*   mp1: usTickNum - this specifies the tick location to query   */
    /*   mp2: NULL                                                    */
    /*   reply: usTickSize - size of tick (in pixels)                 */
    /******************************************************************/
    case SLM_QUERYTICKSIZE:

      return (long)(QueryTickSize( hwnd, pIntData, LOWORD(wParam)));    /*JEH1*/
    break;
    /******************************************************************/
    /* Remove a specific detent on the shaft                          */
    /*   mp1: usDetentId - unique detent identifier which indicates   */
    /*                     the position to return                     */
    /*   mp2: NULL                                                    */
    /*   reply: fSuccess - TRUE, successful        FALSE, error       */
    /******************************************************************/
    case SLM_REMOVEDETENT:

      return (long)(RemoveDetent( hwnd, pIntData, (PDETENTDATA)wParam));
    break;
    /******************************************************************/
    /* Set the text placed over a tick at a specific tick position    */
    /*   mp1: usTickNum - this specifies the tick location to query   */
    /*   mp2: pszTickText - pointer to text to place at tick mark     */
    /*   reply: fSuccess - TRUE, successful        FALSE, error       */
    /******************************************************************/
    case SLM_SETSCALETEXT:

      return (long)(SetScaleText( hwnd, pIntData,
                                  LOWORD(wParam), (LPSTR)lParam ));     /*JEH1*/
    break;

    /******************************************************************/
    /* If presentation parameters changed message received, get       */
    /* updated presentation data and reflect in control.              */
    /*   mp1: colorToChange                                           */
    /*   mp2: crColorValue                                            */
    /*   reply: none                                                  */
    /******************************************************************/
    case SLM_SETSLIDERCOLORS:

      /****************************************************************/
      /* Pass change information to routine and refresh control       */
      /****************************************************************/
      if (SldSetPresParam( pIntData, LOWORD(wParam), (COLORREF)lParam ))/*JEH1*/
      {
        InvalidateRect( hwnd, NULL, FALSE );
        return ((long)TRUE);
      }
      else
        return ((long)FALSE);
      break;

    /******************************************************************/
    /* If presentation parameters query message received, get         */
    /* current presentation data and return queried value.            */
    /*   mp1: colorToReturn                                           */
    /*   mp2: NULL                                                    */
    /*   reply: ulong : presentation parameter color long RGB value   */
    /******************************************************************/
    case SLM_QUERYSLIDERCOLORS:

      /****************************************************************/
      /* Pass query information to routine and return presentation    */
      /* parameter color value.                                       */
      /****************************************************************/
      return (long)(SldQueryPresParam( pIntData, LOWORD(wParam)));        /*EVB1A*/
      break;

    /******************************************************************/
    /* If presentation parameters changed message received, get       */
    /* updated presentation data and reflect in control.              */
    /*   mp1: colorToChange                                           */
    /*   mp2: NULL                                                    */
    /*   reply: none                                                  */
    /******************************************************************/
    case SLM_RESETSLIDERCOLORS:

      /****************************************************************/
      /* Pass change information to routine and refresh control       */
      /****************************************************************/
      if (SldResetPresParam( pIntData, LOWORD(wParam)))                   /*EVB1A*/
      {
        InvalidateRect( hwnd, NULL, FALSE );
        return ((long)TRUE);
      }
      else
        return ((long)FALSE);
      break;

    /******************************************************************/
    /* Set the dimensions or position of the slider shaft or arm      */
    /*   mp1: usInfoType - Identifier of type of information requested*/
    /*        usArmPosType - Format of arm position information       */
    /*   mp2: ulInfo - Specific information to be set                 */
    /*   reply: fSuccess - TRUE, successful        FALSE, error       */
    /******************************************************************/
    case SLM_SETSLIDERINFO:
      return (long)(SetSliderInfo( hwnd, pIntData, LOWORD(wParam),      /*DAB1*/
                                   HIWORD(wParam), (ULONG)lParam ));    /*DAB1*/
    break;
    /******************************************************************/
    /* Set the size of a tick to be displayed at a tick position      */
    /*   mp1: usTickNum - this specifies the tick location to set     */
    /*        usTickSize - size of tick (in pixels)                   */
    /*   mp2: NULL                                                    */
    /*   reply: fSuccess - TRUE, successful        FALSE, error       */
    /******************************************************************/
    case SLM_SETTICKSIZE:

      return (long)(SetTickSize( hwnd, pIntData, LOWORD(wParam),        /*DAB1*/
                                 HIWORD(wParam) ));                     /*DAB1*/
    break;
    default:
      return ( DefWindowProc( hwnd, message, wParam, lParam) );
  }
  return ( DefWindowProc( hwnd, message, wParam, lParam) );
}


/**********************************************************************/
/* Procedure: InitializeSlider - This function is called when the     */
/*            slider window is created. It will initialize the values */
/*            for the slider control in the internal control block    */
/*            and save information away for future use.               */
/*                                                                    */
/* Inputs: hwndSld  - handle of the slider window                     */
/*         pSldData - Pointer to SLDCDATA passed on creation          */
/*         pIntData - Pointer to the internal control block           */
/*                                                                    */
/* Output: If FALSE, successful creation                              */
/*         If TRUE, error during creation                             */
/**********************************************************************/
long InitializeSlider( HANDLE hwndSld, LPSLDCDATA pSldData )
{
  PBYTE    pData=NULL;
  LONG     rc;

  /*------------------------------------------------------------------*/
  /* add a node to describe this task in the task handle list.  If it */
  /* fails, return a failure code                                     */
  /*------------------------------------------------------------------*/
  if (!AddTaskNode ())
  {
    return (-1);
  }

  /********************************************************************/
  /* Allocate memory from the heap for the internal data needed       */
  /********************************************************************/
  if (!(pData = (PBYTE)LocalAlloc( LMEM_FIXED | LMEM_ZEROINIT,
                                    sizeof(ISLDDATA) )))
    return (-1);

  /********************************************************************/
  /* Place allocated data block into the window words for future use  */
  /********************************************************************/
  SetWindowLong( hwndSld, QWL_SLDCTL, (LONG)pData );                    /*JEH1*/
  SetWindowLong( hwndSld, QWL_SLDBRUSH, (LONG)0 );                      /*MJS1*/

  SldInitPresParam( (PISLDDATA)pData );

  /********************************************************************/
  /* Setup slider internal data from the control block passed (if any)*/
  /********************************************************************/
  rc = (SetupSliderFromCB (hwndSld,(PISLDDATA)pData,pSldData)) ? 0 : -1;

  ++wUseCount;
  return (rc);
}

/**********************************************************************/
/* Procedure: SetupSliderFromCB - This procedure will initialize the  */
/*            slider internal data from the control block passed.     */
/*            It will recalculate all of the slider dimensions from   */
/*            the new data, and cause the slider to update itself.    */
/*                                                                    */
/* Inputs: hwndSld  - Window handle of the slider control             */
/*         pIntData - Pointer to the slider control data block        */
/*         pSldData - Pointer to SLDCDATA passed on creation          */
/*                                                                    */
/* Output: If TRUE, successful update from control block              */
/*         If FALSE, error during update                              */
/**********************************************************************/
BOOL SetupSliderFromCB( HANDLE hwndSld, PISLDDATA pIntData,
                        LPSLDCDATA pSldData )
{
  ULONG   ulStyle=0;                                                    /*JEH1*/
  USHORT  usScale1Increments = 2;                                       /*JEH1*/
  USHORT  usScale1Spacing = 0;                                          /*JEH1*/
  USHORT  usScale2Increments = 2;                                       /*JEH1*/
  USHORT  usScale2Spacing = 0;                                          /*JEH1*/
  USHORT  usMaxValue;                                                   /*JEH1*/

  /********************************************************************/
  /* Get slider style for creation                                    */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* Check for a valid control block pointer first                    */
  /* Check if control block is of correct size                        */
  /********************************************************************/
  if ((pSldData) && (pSldData->cbSize == sizeof(SLDCDATA)))
  {
    /******************************************************************/
    /* Validate information passed                                    */
    /*  If Ruler 1 is primary then check increments and spacing for   */
    /*  ruler 1, otherwise check increments and spacing for ruler 2   */
    /******************************************************************/
    if ((!(ulStyle & SLS_PRIMARYSCALE2) &&
        (pSldData->usScale1Increments < 2)) ||
        ((ulStyle & SLS_PRIMARYSCALE2) &&
        (pSldData->usScale2Increments < 2)))
    {
      /****************************************************************/
      /* If the primary rulers information is null, return error      */
      /****************************************************************/
      return ( FALSE );
    }

    /******************************************************************/
    /* Transfer all of the control block info into local area         */
    /******************************************************************/
    usScale1Increments = pSldData->usScale1Increments;
    usScale1Spacing = pSldData->usScale1Spacing;
    usScale2Increments = pSldData->usScale2Increments;
    usScale2Spacing = pSldData->usScale2Spacing;
  }
  /********************************************************************/
  /* Transfer all of the control block info into internal data        */
  /********************************************************************/
  pIntData->usScale1Increments = usScale1Increments;
  pIntData->usScale1Spacing = usScale1Spacing;
  pIntData->usScale2Increments = usScale2Increments;
  pIntData->usScale2Spacing = usScale2Spacing;

  /********************************************************************/
  /* Set up initial shaft dimensions and arm dimensions               */
  /* but only if the shaft has no size yet                            */
  /********************************************************************/
  if (!pIntData->usShaftBreadth)
    pIntData->usShaftBreadth = GetDefaultSize( SLD_DEFSHAFTBREADTH,
        ulStyle);

  /********************************************************************/
  /* Check to insure that buttons were specified for only one side    */
  /********************************************************************/
  if ((ulStyle & SLS_BUTTONSRIGHT) && (ulStyle & SLS_BUTTONSLEFT))
  {
    /******************************************************************/
    /* If both sides specified, null out all buttons                  */
    /******************************************************************/
    ulStyle &= ~(SLS_BUTTONSRIGHT | SLS_BUTTONSLEFT);
    SetWindowLong( hwndSld, GWL_STYLE, ulStyle );
  }

  /********************************************************************/
  /* Set up default arm dimensions                                    */
  /********************************************************************/
  if (ulStyle & SLS_READONLY)
  {
    /******************************************************************/
    /* If readonly slider, make it a narrow, shaft wide slider arm    */
    /******************************************************************/
    pIntData->usArmLength = SLD_ROARMLENGTH;
    pIntData->usArmBreadth = pIntData->usShaftBreadth;

    /******************************************************************/
    /* If readonly, null out any button styles                        */
    /******************************************************************/
    ulStyle &= ~(SLS_BUTTONSRIGHT | SLS_BUTTONSLEFT);
    SetWindowLong( hwndSld, GWL_STYLE, ulStyle );
  }
  else
  {
    /******************************************************************/
    /* Otherwise normal so use normal slider default sizing           */
    /* but only if arm has no size yet                                */
    /******************************************************************/
    if (!pIntData->usArmLength)
      pIntData->usArmLength = GetDefaultSize( SLD_DEFARMLENGTH,
          ulStyle);
    if (!pIntData->usArmBreadth)
      pIntData->usArmBreadth = GetDefaultSize( SLD_DEFARMBREADTH,
          ulStyle);
  }

  /********************************************************************/
  /* Since at this point we don't know for sure whether or not the    */
  /* slider is autosize (the spacing may have changed), zero out both */
  /* flags to make sure one isn't carried over by mistake.            */
  /********************************************************************/
  pIntData->usStatusSld &= ~FL_AUTOINCREMENT1;
  pIntData->usStatusSld &= ~FL_AUTOINCREMENT2;

  /********************************************************************/
  /* Calculate the size of the shaft based on the number of increments*/
  /* and the default spacing (if any)                                 */
  /********************************************************************/
  CalcShaftSize( hwndSld, pIntData );

  /********************************************************************/
  /* Now that the default values are in, calculate the shaft and      */
  /* possible button positions based on alignment and styles          */
  /********************************************************************/
  CalcShaftnButtonPos( hwndSld, pIntData );

  /********************************************************************/
  /* Before calculating arm stuff, make sure the arm hasn't gone      */
  /* off the high end of the slider as a result of making the slider  */
  /* window smaller (with an autosizing slider)                       */
  /********************************************************************/
  usMaxValue = QueryMaxRange (hwndSld, pIntData);
  if (pIntData->usArmPos > usMaxValue)
    pIntData->usArmPos = usMaxValue;

  /********************************************************************/
  /* Now calculate the arm rectangle from the current settings        */
  /* and create it                                                    */
  /********************************************************************/
  CalcArmRectangle( hwndSld, pIntData, &(pIntData->rclSliderArm),
                    pIntData->usArmPos );
  pIntData->usStatusSld &= ~FL_SLDFOCUSCHANGE;
  CreateSliderArm( hwndSld, pIntData );

  /********************************************************************/
  /* Now initialize button status based on arm initial position       */
  /********************************************************************/
  CheckForEndOfScale (hwndSld, pIntData, 0);                            /*JEH1*/

  return ( TRUE );
}


/**********************************************************************/
/* Procedure: UpdateSliderValues - This procedure will update the     */
/*            slider internal data because of a dynamic modification  */
/*            of a slider style which was not intended to be changed  */
/*            dynamically.                                            */
/*                                                                    */
/* Inputs: hwndSld  - Window handle of the slider control             */
/*         pIntData - Pointer to the slider control data block        */
/*         ulStyle  - New style values masked for important bits      */
/*                                                                    */
/* Output: none                                                       */
/**********************************************************************/
VOID UpdateSliderValues( HANDLE hwndSld, PISLDDATA pIntData,
                         ULONG ulStyle )                                /*JEH1*/
{
  ULONG  ulFlag;                                                        /*JEH1*/
  USHORT usMaxValue;                                                    /*JEH1*/

  /********************************************************************/
  /* Determine which bits have changed (using XOR operator)           */
  /********************************************************************/
  ulFlag = ulStyle ^ pIntData->ulStyle;

  /********************************************************************/
  /* Now check the style bits which changed to determine proper action*/
  /*  If readonly style changed, modify arm size                      */
  /********************************************************************/
  if (ulFlag & SLS_READONLY)
  {
    /******************************************************************/
    /* If new style is readonly, then change to thin arm size         */
    /******************************************************************/
    if (ulStyle & SLS_READONLY)
    {
      /****************************************************************/
      /* If readonly slider, make it a narrow, shaft wide slider arm  */
      /****************************************************************/
      pIntData->usArmLength = SLD_ROARMLENGTH;
      pIntData->usArmBreadth = pIntData->usShaftBreadth;
    }
    else
    {
      /****************************************************************/
      /* Otherwise normal so use normal slider default sizing, but    */
      /* only if the arm length is still the read-only default        */
      /****************************************************************/
      if (pIntData->usArmLength == SLD_ROARMLENGTH)
        pIntData->usArmLength = GetDefaultSize( SLD_DEFARMLENGTH,
          pIntData->ulStyle);
    }
  }
  /********************************************************************/
  /* If home position changed, or ownerdraw flag changed, recalculate */
  /* slider arm rectangle and slider arm bitmap used                  */
  /********************************************************************/
  if ((ulFlag & SLS_HOMERIGHT) || (ulFlag & SLS_OWNERDRAW) ||
      (ulFlag & SLS_READONLY))
  {
    /******************************************************************/
    /* Calculate the size of the shaft based on the number of         */
    /* increments and the default spacing (if any)                    */
    /******************************************************************/
    CalcShaftSize( hwndSld, pIntData );

    /******************************************************************/
    /* Now that the default values are in, calculate the shaft and    */
    /* possible button positions based on alignment and styles        */
    /******************************************************************/
    CalcShaftnButtonPos( hwndSld, pIntData );

    /******************************************************************/
    /* Before calculating arm stuff, make sure the arm hasn't gone    */
    /* off the high end of the slider as a result of making the slider*/
    /* window smaller (with an autosizing slider)                     */
    /******************************************************************/
    usMaxValue = QueryMaxRange (hwndSld, pIntData);
    if (pIntData->usArmPos > usMaxValue)
      pIntData->usArmPos = usMaxValue;

    /******************************************************************/
    /* If one of the styles has been modified, update slider arm      */
    /******************************************************************/
    CalcArmRectangle( hwndSld, pIntData, &(pIntData->rclSliderArm),
                      pIntData->usArmPos );
    pIntData->usStatusSld &= ~FL_SLDFOCUSCHANGE;
    CreateSliderArm( hwndSld, pIntData );

    /******************************************************************/
    /* Now initialize button status based on arm initial position     */
    /******************************************************************/
    CheckForEndOfScale (hwndSld, pIntData, 0);                          /*JEH1*/
  }

  /********************************************************************/
  /* If snap style changed, check if snap was turned on               */
  /********************************************************************/
  if ((ulFlag & SLS_SNAPTOINCREMENT) && (ulStyle & SLS_SNAPTOINCREMENT))
  {
    /******************************************************************/
    /* If so, move to the closest increment if snap turned on         */
    /******************************************************************/
    pIntData->ulStyle = ulStyle & SLS_TYPEMASK;
    MoveSliderArm( hwndSld, pIntData, pIntData->usArmPos, TRUE, TRUE);
  }

  /********************************************************************/
  /* Update previous style with latest setting                        */
  /********************************************************************/
  pIntData->ulStyle = ulStyle & SLS_TYPEMASK;

  return;
}

/**********************************************************************/
/* Procedure: TerminateSlider - This procedure will terminate the     */
/*            slider control.  It will free any memory allocated      */
/*            for control usage.  It will traverse and free and       */
/*            memory used for detent and tick linked lists, including */
/*            strings associated with tick marks.  It will also free  */
/*            the internal control block.                             */
/*                                                                    */
/* Inputs: hwndSld  - Window handle of the slider control             */
/*         pIntData - Pointer to the slider control data block        */
/*                                                                    */
/* Output: none                                                       */
/**********************************************************************/
VOID TerminateSlider( HANDLE hwndSld, PISLDDATA pIntData )
{
  PTICKDATA   pTickCurr;
  PTICKDATA   pTickNext;
  PDETENTDATA pDetentCurr;
  PDETENTDATA pDetentNext;
  HBRUSH      temp;

  /********************************************************************/
  /* Get pointer to start of tick data                                */
  /********************************************************************/
  pTickCurr = pIntData->pTickData;

  /********************************************************************/
  /* While there is a valid tick pointer                              */
  /********************************************************************/
  while ( pTickCurr )
  {
    /******************************************************************/
    /* Check if there is a text string associated with this tick      */
    /******************************************************************/
    if ( pTickCurr->pszText )
    {
      /****************************************************************/
      /* If tick text exists, free the text buffer                    */
      /****************************************************************/
      LocalFree((HANDLE)pTickCurr->pszText );
    }
    /******************************************************************/
    /* Save pointer to the next tick following this one (if any)      */
    /******************************************************************/
    pTickNext = pTickCurr->pNext;

    /******************************************************************/
    /* Free the memory for this tick node                             */
    /******************************************************************/
    LocalFree((HANDLE)pTickCurr );

    /******************************************************************/
    /* Point to the next tick in the list (if any)                    */
    /******************************************************************/
    pTickCurr = pTickNext;
  }

  /********************************************************************/
  /* Get pointer to start of detent data                              */
  /********************************************************************/
  pDetentCurr = pIntData->pDetentData;

  /********************************************************************/
  /* While there is a valid detent pointer                            */
  /********************************************************************/
  while ( pDetentCurr )
  {
    /******************************************************************/
    /* Save pointer to the next detent following this one (if any)    */
    /******************************************************************/
    pDetentNext = pDetentCurr->pNext;

    /******************************************************************/
    /* Free the memory for this detent node                           */
    /******************************************************************/
    LocalFree((HANDLE)pDetentCurr );

    /******************************************************************/
    /* Point to the next detent in the list (if any)                  */
    /******************************************************************/
    pDetentCurr = pDetentNext;
  }

  /********************************************************************/
  /* Delete the bitmap used for this slider                           */
  /********************************************************************/
  if (pIntData->hbmArm)
    DeleteObject(pIntData->hbmArm);

  /********************************************************************/
  /* Finally deallocate memory from heap for the internal control data*/
  /********************************************************************/
  LocalFree((HANDLE)pIntData );

  /********************************************************************/
  /* Once memory is freed, reset invalid pointer so it is not used    */
  /********************************************************************/
  SetWindowLong( hwndSld, QWL_SLDCTL, 0 );                              /*JEH1*/

  /********************************************************************/
  /* Delete the brush stored in the background                        */
  /********************************************************************/
  temp = (HBRUSH)GetWindowLong( hwndSld, QWL_SLDBRUSH );

  if (temp)
    DeleteObject( temp );

  SetWindowLong( hwndSld, QWL_SLDBRUSH, 0 );

  /********************************************************************/
  /* Free the node for this slider from the linked list of task       */
  /* handles                                                          */
  /********************************************************************/
  DeleteTaskNode ();

  /********************************************************************/
  /* Decrement the use counter, and if this is the last slider to be  */
  /* destroyed, free the slider bitmaps                               */
  /********************************************************************/
  if (!wUseCount)
    DeleteSliderBitmaps();

  return;
}

/**********************************************************************/
/* Procedure: GetDefaultSize - This procedure will get the dimension  */
/*            value for a specific variable based on the resolution   */
/*            of the device being used to display the slider.  The    */
/*            routine queries the system for the dimenstions of       */
/*            the particular hardware being used, and then calculates */
/*            default value based on proportion from VGA defaults:    */
/*                                                                    */
/* Inputs: usValue - Index of the parameter to return the default     */
/*                   value of for the current resolution              */
/*                                                                    */
/* Output: usSize  - value of parameter requested                     */
/**********************************************************************/
USHORT GetDefaultSize( USHORT usValue, ULONG ulStyle)                   /*JEH1*/
{
 USHORT cxScreen;                                                       /*JEH1*/
 USHORT cyScreen;                                                       /*JEH1*/
 static USHORT ausVal[3] =                                              /*JEH1*/
         { VGA_DEFARMLENGTH, VGA_DEFSHAFTBREADTH, VGA_DEFARMBREADTH };

  /********************************************************************/
  /* Get the current dimensions for the screen in use                 */
  /********************************************************************/
  cxScreen = GetSystemMetrics( SM_CXSCREEN );
  cyScreen = GetSystemMetrics( SM_CYSCREEN );

  /********************************************************************/
  /* Now calculate nearest default size based on screen resolution    */
  /* If vertical, breadth calculations are for x, length for y        */
  /********************************************************************/
  if (((ulStyle & SLS_VERTICAL) && usValue) ||
      (!(ulStyle & SLS_VERTICAL) && !usValue))
    return (WORD) ((((ausVal[usValue]*cxScreen)/VGA_CX)+1) & ~1);
  else
    return (WORD) ((((ausVal[usValue]*cyScreen)/VGA_CY)+1) & ~1);
}


/**********************************************************************/
/* Procedure: CalcShaftSize - This function calculates the spacing    */
/*            of the increments on the slider.                        */
/*                                                                    */
/* Inputs: hwndSld  - handle of the slider window                     */
/*         pIntData - Pointer to the internal control block           */
/*                                                                    */
/* Output: none                                                       */
/**********************************************************************/
VOID CalcShaftSize( HWND hwndSld, PISLDDATA pIntData)
{
  ULONG   ulStyle=0;                                                    /*JEH1*/
  RECT    rclWindow;
  USHORT  usButtonLength;                                               /*JEH1*/
  SHORT   sWindowSize;                                                  /*JEH1*/

  /********************************************************************/
  /* Get slider style for creation                                    */
  /********************************************************************/
  ulStyle = GetWindowLong( hwndSld, GWL_STYLE );

  /********************************************************************/
  /* If spacing for scales is not specified, automatically            */
  /*  calculate the spacing based on the window size                  */
  /********************************************************************/
  GetClientRect( hwndSld, &rclWindow );

  /********************************************************************/
  /* Set overall window length based on orientation                   */
  /********************************************************************/
  if (ulStyle & SLS_VERTICAL)
    sWindowSize = LOWORD(rclWindow.bottom);                             /*JEH1*/
  else
    sWindowSize = LOWORD(rclWindow.right);                              /*JEH1*/

  /********************************************************************/
  /* Get the length of a button based on current shaft breadth        */
  /********************************************************************/
  usButtonLength = CalcButtonLength( pIntData );

  /********************************************************************/
  /* If buttons are included, update value to subtract out            */
  /********************************************************************/
  if ((ulStyle & SLS_BUTTONSLEFT) || (ulStyle & SLS_BUTTONSRIGHT))
  {
    /******************************************************************/
    /* If buttons included subtract 2 times a single button           */
    /* length plus a two pel space between shaft and buttons          */
    /******************************************************************/
    sWindowSize -= ((usButtonLength << 1) + BUTTONSPACE);
  }

  /********************************************************************/
  /* Calculate shaft length based on ruler information passed         */
  /********************************************************************/
  if (ulStyle & SLS_PRIMARYSCALE2)
  {

    /******************************************************************/
    /* If scale two spacing not specified and number of increments    */
    /* is greater than one, calculate spacing                         */
    /******************************************************************/
    if ((!pIntData->usScale2Spacing &&
        (pIntData->usScale2Increments > 1)) ||
        (pIntData->usStatusSld & FL_AUTOINCREMENT2))
    {
      pIntData->usStatusSld |= FL_AUTOINCREMENT2;

      /****************************************************************/
      /* Spacing is window size minus arm length and two times        */
      /* border width divided by number of increments minus one       */
      /****************************************************************/
      pIntData->usScale2Spacing = (sWindowSize -
          (pIntData->usArmLength + (BORDERWIDTH << 1))) /
          (pIntData->usScale2Increments - 1);

      /****************************************************************/
      /* If there are too many increments specified for the window,   */
      /* then spacing will have come out zero, which means no space   */
      /* will be saved for the slider shaft in the next step.  If the */
      /* spacing is zero, reset it to one as a safety step (this will */
      /* cause the oversize slider to be clipped by the window).      */
      /****************************************************************/
      if (!pIntData->usScale2Spacing)
        pIntData->usScale2Spacing = 1;
      /****************************************************************/
      /* Protect against initial window size of 0.  Resetting         */
      /* usScale2Spacing to 0 will cause spacing to be recalculated   */
      /* on subsequent resize.                                        */
      /****************************************************************/
      if (!sWindowSize)
        pIntData->usScale2Spacing = 0;
    }

    /******************************************************************/
    /* If primary ruler is second ruler, calculate from it            */
    /******************************************************************/
    pIntData->usShaftLength = (pIntData->usScale2Increments - 1) *
        pIntData->usScale2Spacing + pIntData->usArmLength +
        (BORDERWIDTH << 1);
    pIntData->usStatusSld |= FL_SLDINITIALSCALE2;

    /******************************************************************/
    /* If scale one spacing is also not specified and number of       */
    /* increments is greater than one, calc spacing from shaftsize    */
    /******************************************************************/
    if ((!pIntData->usScale1Spacing &&
        (pIntData->usScale1Increments > 1)) ||
        (pIntData->usStatusSld & FL_AUTOINCREMENT1))
    {
      pIntData->usStatusSld |= FL_AUTOINCREMENT1;

      /****************************************************************/
      /* Spacing is window size minus arm length and two times        */
      /* border width divided by number of increments minus one       */
      /****************************************************************/
      pIntData->usScale1Spacing = (pIntData->usShaftLength -
          (pIntData->usArmLength + (BORDERWIDTH << 1))) /
          (pIntData->usScale1Increments - 1);
    }
  }
  else
  {
    /******************************************************************/
    /* If scale one spacing not specified and number of increments    */
    /*  is greater than one, calculate spacing                        */
    /******************************************************************/
    if ((!pIntData->usScale1Spacing &&
        (pIntData->usScale1Increments > 1)) ||
        (pIntData->usStatusSld & FL_AUTOINCREMENT1))
    {
      pIntData->usStatusSld |= FL_AUTOINCREMENT1;

      /****************************************************************/
      /* Spacing is window size minus arm length and two times        */
      /*  border width divided by number of increments minus one      */
      /****************************************************************/
      pIntData->usScale1Spacing = (sWindowSize -
           (pIntData->usArmLength + (BORDERWIDTH << 1))) /
                                 (pIntData->usScale1Increments - 1);

      /****************************************************************/
      /* If there are too many increments specified for the window,   */
      /* then spacing will have come out zero, which means no space   */
      /* will be saved for the slider shaft in the next step.  If the */
      /* spacing is zero, reset it to one as a safety step (this will */
      /* cause the oversize slider to be clipped by the window).      */
      /****************************************************************/
      if (!pIntData->usScale1Spacing)
        pIntData->usScale1Spacing = 1;
      /****************************************************************/
      /* Protect against initial window size of 0.  Resetting         */
      /* usScale1Spacing to 0 will cause spacing to be recalculated   */
      /* on subsequent resize.                                        */
      /****************************************************************/
      if (!sWindowSize)
        pIntData->usScale1Spacing = 0;
    }

    /******************************************************************/
    /* Otherwise calculate length from the first ruler                */
    /******************************************************************/
    pIntData->usShaftLength = (pIntData->usScale1Increments - 1) *
        pIntData->usScale1Spacing + pIntData->usArmLength +
        (BORDERWIDTH << 1);
    pIntData->usStatusSld &= ~FL_SLDINITIALSCALE2;

    /******************************************************************/
    /* If scale two spacing not specified and number of increments    */
    /*  is greater than one, calculate spacing                        */
    /******************************************************************/
    if ((!pIntData->usScale2Spacing &&
        (pIntData->usScale2Increments > 1)) ||
        (pIntData->usStatusSld & FL_AUTOINCREMENT2))
    {
      pIntData->usStatusSld |= FL_AUTOINCREMENT2;

      /****************************************************************/
      /* Spacing is window size minus arm length and two times        */
      /*  border width divided by number of increments minus one      */
      /****************************************************************/
      pIntData->usScale2Spacing = (pIntData->usShaftLength -
           (pIntData->usArmLength + (BORDERWIDTH << 1))) /
                                 (pIntData->usScale2Increments - 1);
    }
  }
}
