/* Revision: 67 1.2.2.2 source/ui/cclnt/spinbtn/spinbtnd.c, cclspin, ioc.v400, 000302  */
/*---------------------------------------------------------------------------*
 *
 *                              IBM CONFIDENTIAL
 *
 *            (IBM Confidential-Restricted when combined with the
 *              aggregated OCO source modules for this program)
 *
 *                            OCO SOURCE MATERIALS
 *
 *             20G0400 (C) Copyright IBM Corp. 1992, 1997 (Unpublished)
 *
 *---------------------------------------------------------------------------*/
/**********************************************************************/
/* Spinbtnd.c                                                         */
/*                                                                    */
/* Spin Button interface routines for the Windows SDK Dialog Editor.  */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

#include <windows.h>
#include <custcntl.h>
#include "dlgedit.h"
#include "iclspbw.h"
#include "spinbtnp.h"
#include "spinbtnd.h"

HANDLE FAR PASCAL SBInfo  (void);
BOOL   FAR PASCAL SBStyle (HWND, HANDLE, LPFNSTRTOID, LPFNIDTOSTR);
WORD   FAR PASCAL SBFlags (WORD, LPSTR, WORD);
BOOL   FAR PASCAL SBDlgFn (HWND, WORD, WORD, DWORD);

void   EnableStyles (HWND,WORD,WORD,WORD,WORD,WORD,WORD,WORD);
WORD   QueryStyles  (HWND);
void   SetStyles    (HWND, DWORD);
LPSTR  QueryTypeString         (WORD fs);
LPSTR  QueryAttributeString    (WORD fs);
LPSTR  QueryTextJustString     (WORD fs);
LPSTR  QueryNoBorderString     (WORD fs);
LPSTR  QueryFastSpinString     (WORD fs);
LPSTR  QueryPadWithZerosString (WORD fs);

extern HANDLE hSpinInstance;

LPSTR strncpy(LPSTR,LPSTR,WORD);

/**********************************************************************/
/* SBInfo                                                             */
/*                                                                    */
/* Allocates a CTLINFO structure from the global heap to identify the */
/* controls that are supported by this DLL.  The caller of this       */
/* function is responsible for freeing the structure.                 */
/*                                                                    */
/* This function must be ordinal 2.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/
HANDLE FAR PASCAL SBInfo (void)
{
  HANDLE    hCtlInfo;
  LPCTLINFO lpCtlInfo;

  /********************************************************************/
  /* allocate a CTLINFO that the Dialog Editor will free              */
  /********************************************************************/
  if (hCtlInfo = GlobalAlloc (GHND, sizeof (CTLINFO)))
  {
    /******************************************************************/
    /* lock it down so we can address it                              */
    /******************************************************************/
    if (lpCtlInfo = (LPCTLINFO) GlobalLock (hCtlInfo))
    {
      lpCtlInfo->wVersion        = CCL_VERSION;
      lpCtlInfo->wCtlTypes       = 1;

      lpCtlInfo->Type[0].wType   = 0;
      lpCtlInfo->Type[0].wWidth  = 40;
      lpCtlInfo->Type[0].wHeight = 13;
      lpCtlInfo->Type[0].dwStyle = WS_CHILD | WS_CLIPCHILDREN | SPBS_MASTER;

      lstrcpy (lpCtlInfo->szClass, ICL_SPINBTN);
      lstrcpy (lpCtlInfo->szTitle, CCL_COPYRIGHT);
      lstrcpy (lpCtlInfo->Type[0].szDescr, ICL_SPINBTN);

      GlobalUnlock (hCtlInfo);
    }

    /******************************************************************/
    /* otherwise, we couldn't lock it down to initialize it, so punt  */
    /******************************************************************/
    else
    {
      GlobalFree (hCtlInfo);
      hCtlInfo = NULL;
    }
  }

  return (hCtlInfo);
}


/**********************************************************************/
/* SBStyle                                                            */
/*                                                                    */
/* This function displays a modal dialog that is to collect the style */
/* of the SpinButton from the user.                                   */
/*                                                                    */
/* This function must be ordinal 3.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/
BOOL FAR PASCAL SBStyle (HWND hwnd, HANDLE hCtlStyle,
                          LPFNSTRTOID lpfnStrToId, LPFNIDTOSTR lpfnIdToStr)
{
  PSTYLESTRUCT pss;
  BOOL         rc;

  if (!(pss = (PSTYLESTRUCT) LocalAlloc (LMEM_FIXED | LMEM_ZEROINIT,
                                         sizeof(STYLESTRUCT))))
    return (FALSE);

  pss->hCtlStyle   = hCtlStyle;
  pss->lpCtlStyle  = NULL;
  pss->lpfnStrToId = lpfnStrToId;
  pss->lpfnIdToStr = lpfnIdToStr;

  rc = DialogBoxParam (hSpinInstance, MAKEINTRESOURCE (ID_SBSTYLES), hwnd,
                       (FARPROC) SBDlgFn, (WORD) pss);

  LocalFree((HANDLE) pss);

  return (rc);
}


/**********************************************************************/
/* SBDlgFn                                                            */
/*                                                                    */
/* This function must be ordinal 6.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/

BOOL FAR PASCAL SBDlgFn (HWND hwnd, WORD msg, WORD wParam, DWORD lParam)
{
  PSTYLESTRUCT pss;
  DWORD        dwRet;
  char         szID[33];

  switch (msg)
  {
    case WM_INITDIALOG:
      pss = (PSTYLESTRUCT) lParam;

      /****************************************************************/
      /* Set the property that contains the pointer to the            */
      /* STYLESTRUCT /* lock down the CTLSTYLE structure, and make    */
      /* sure this is really  a SpinButton; if any of these failed,   */
      /* bail out now                                                 */
      /****************************************************************/
      if (!SetProp (hwnd, MAKEINTRESOURCE (1), (HANDLE) pss) ||
          !(pss->lpCtlStyle = (LPCTLSTYLE) GlobalLock (pss->hCtlStyle)))
      {
        EndDialog (hwnd, FALSE);
        break;
      }

      /****************************************************************/
      /* initialize the ID field                                      */
      /****************************************************************/
      if ((*pss->lpfnIdToStr) (pss->lpCtlStyle->wId, szID, sizeof (szID)))
      {
        SetDlgItemText (hwnd, ID_ID, szID);
      }
      else
      {
        SetDlgItemInt (hwnd, ID_ID, pss->lpCtlStyle->wId, FALSE);
      }

      /****************************************************************/
      /* set the text limit for the ID field                          */
      /****************************************************************/
      SendDlgItemMessage (hwnd, ID_ID, EM_LIMITTEXT, sizeof (szID)-1, 0);

      /****************************************************************/
      /* check the default style buttons                              */
      /****************************************************************/
      SetStyles (hwnd, pss->lpCtlStyle->dwStyle);
      break;

    case WM_DESTROY:
      if (pss = (PSTYLESTRUCT) RemoveProp (hwnd, MAKEINTRESOURCE (1)))
      {
        if (pss->lpCtlStyle)  GlobalUnlock (pss->hCtlStyle);
      }
      break;

    case WM_COMMAND:
      switch (wParam)
      {
        case IDOK:
          pss = (PSTYLESTRUCT) GetProp (hwnd, MAKEINTRESOURCE (1));

          /************************************************************/
          /* if the ID field was modified, get the new ID             */
          /************************************************************/
          if (SendDlgItemMessage (hwnd, ID_ID, EM_GETMODIFY, 0, 0))
          {
            GetDlgItemText (hwnd, ID_ID, szID, sizeof (szID));

            /**********************************************************/
            /* if the new ID is not a defined constant, bail out      */
            /**********************************************************/
            if (!LOWORD (dwRet = (*pss->lpfnStrToId) (szID)))  break;

            pss->lpCtlStyle->wId = HIWORD (dwRet);
          }

          LOWORD (pss->lpCtlStyle->dwStyle) = QueryStyles (hwnd);
          /* flow-through */

        case IDCANCEL:
          EndDialog (hwnd, (wParam == IDOK));
          break;
      }
      break;
  }

  return (FALSE);
}

/**********************************************************************/
/* QueryStyles                                                        */
/*                                                                    */
/* Queries the styles selected from the dialog box.                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/

WORD QueryStyles (HWND hwnd)
{
  DWORD flStyle;

  flStyle  = IsDlgButtonChecked (hwnd, ID_MASTER)       ? SPBS_MASTER :
                                                          SPBS_SERVANT;

  flStyle |= IsDlgButtonChecked (hwnd, ID_NUMERICONLY)  ? SPBS_NUMERICONLY :
             IsDlgButtonChecked (hwnd, ID_READONLY)     ? SPBS_READONLY    :
                                                          SPBS_ALLCHARACTERS;

  flStyle |= IsDlgButtonChecked (hwnd, ID_JUSTRIGHT)    ? SPBS_JUSTRIGHT   :
             IsDlgButtonChecked (hwnd, ID_JUSTCENTER)   ? SPBS_JUSTCENTER  :
                                                          SPBS_JUSTLEFT;

  if (IsDlgButtonChecked (hwnd, ID_NOBORDER))      flStyle |= SPBS_NOBORDER;
  if (IsDlgButtonChecked (hwnd, ID_FASTSPIN))      flStyle |= SPBS_FASTSPIN;
  if (IsDlgButtonChecked (hwnd, ID_PADWITHZEROS))  flStyle |= SPBS_PADWITHZEROS;


  return (LOWORD (flStyle));
}


/**********************************************************************/
/* SetStyles                                                          */
/*                                                                    */
/* Sets the styles for the dialog box from what was passed in by the  */
/* Dialog Editor                                                      */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/

void SetStyles (HWND hwnd, DWORD flStyle)
{

  CheckDlgButton (hwnd, (flStyle & SPBS_MASTER)     ? ID_MASTER :
                                                      ID_SERVANT,
                  TRUE);

  CheckDlgButton (hwnd, (flStyle & SPBS_NUMERICONLY) ? ID_NUMERICONLY :
                        (flStyle & SPBS_READONLY)    ? ID_READONLY    :
                                                       ID_ALLCHARACTERS,
                  TRUE);

  CheckDlgButton (hwnd, ((flStyle & SPBS_JUSTRIGHT) &&
                         (flStyle & SPBS_JUSTLEFT))  ? ID_JUSTCENTER :
                        (flStyle & SPBS_JUSTRIGHT)   ? ID_JUSTRIGHT :
                                                       ID_JUSTLEFT,
                  TRUE);

  CheckDlgButton (hwnd, ID_NOBORDER,
                  (flStyle & SPBS_NOBORDER)         ? TRUE : FALSE );

  CheckDlgButton (hwnd, ID_FASTSPIN,
                  (flStyle & SPBS_FASTSPIN)         ? TRUE : FALSE );

  CheckDlgButton (hwnd, ID_PADWITHZEROS,
                  (flStyle & SPBS_PADWITHZEROS)     ? TRUE : FALSE );


}


/**********************************************************************/
/* SBFlags                                                            */
/*                                                                    */
/* This function must be ordinal 4.                                   */
/*                                                                    */
/* History:                                                           */
/* 13-Jan-92 JeffR    Created                                         */
/* 03-Feb-92 DianaM   Modified for use in SpinButton                  */
/**********************************************************************/

WORD FAR PASCAL SBFlags (WORD fs, LPSTR lpStyle, WORD cbMax)
{
  char  szStyles[200];
  LPSTR lpSB = "SPBS";

  /********************************************************************/
  /* format the style string                                          */
  /********************************************************************/
  wsprintf (szStyles, "%s_%s | %s_%s | %s_%s"
                      "%s%s%s",
            lpSB, QueryTypeString (fs), lpSB, QueryAttributeString (fs),
            lpSB, QueryTextJustString (fs),   QueryNoBorderString (fs),
            QueryFastSpinString (fs),         QueryPadWithZerosString (fs));

  /********************************************************************/
  /* copy the style string we build into DlgEdit's buffer, make sure  */
  /* it's null-terminated, and return the bytes copied                */
  /********************************************************************/
  strncpy (lpStyle, szStyles, cbMax);
  lpStyle[cbMax-1] = '\0';
  return (lstrlen (lpStyle));
}


/**********************************************************************/
/* QueryTypeString                                                    */
/*                                                                    */
/* Returns the string for the spinbtn type style bit in a given flag  */
/* set.                                                               */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryTypeString (WORD fs)
{
  return ((fs & SPBS_MASTER) ? "MASTER" : "SERVANT");
}


/**********************************************************************/
/* QueryAttributeString                                               */
/*                                                                    */
/* Returns the string for the attribute style bit in a given          */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryAttributeString (WORD fs)
{
  return ((fs & SPBS_READONLY)    ? "READONLY"    :
          (fs & SPBS_NUMERICONLY) ? "NUMERICONLY" : "ALLCHARACTERS");
}

/**********************************************************************/
/* QueryTextJustString                                                */
/*                                                                    */
/* Returns the string for the text justification style bit in a given */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryTextJustString (WORD fs)
{
  return (((fs & SPBS_JUSTRIGHT) && (fs & SPBS_JUSTLEFT))  ? "JUSTCENTER" :
           (fs & SPBS_JUSTRIGHT)                           ? "JUSTRIGHT" :
                                                             "JUSTLEFT");
}


/**********************************************************************/
/* QueryNoBorderString                                                */
/*                                                                    */
/* Returns the string for the no border style bit in a given          */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryNoBorderString (WORD fs)
{
  return ((fs & SPBS_NOBORDER) ? " | SPBS_NOBORDER" : "");
}

/**********************************************************************/
/* QueryFastSpinString                                                */
/*                                                                    */
/* Returns the string for the fast spin style bit in a given          */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryFastSpinString (WORD fs)
{
  return ((fs & SPBS_FASTSPIN) ? " | SPBS_FASTSPIN" : "");
}

/**********************************************************************/
/* QueryPadWithZerosString                                            */
/*                                                                    */
/* Returns the string for the no border style bit in a given          */
/* flag set.                                                          */
/*                                                                    */
/* History:                                                           */
/* 03-Feb-92 DianaM   Created                                         */
/**********************************************************************/

LPSTR QueryPadWithZerosString (WORD fs)
{
  return ((fs & SPBS_PADWITHZEROS) ? " | SPBS_PADWITHZEROS" : "");
}

