/* Revision: 71 1.2.2.2 source/ui/cclnt/spinbtn/spinwin.c, cclspin, ioc.v400, 000302  */
/**********************************************************************/
/*                                                                    */
/*              Licensed Materials - Property of IBM                  */
/*              20G0400 (C) Copyright IBM Corp. 1992, 1997.           */
/*                    See Copyright Instructions.                     */
/*                                                                    */
/**********************************************************************/
/**********************************************************************/
/*                                                                    */
/* CUA Controls Library/2 Version 1.0                       (Windows) */
/*                                                                    */
/* Sample Application Code                                            */
/* Spinbutton Control                                                 */
/*                                                                    */
/* File:  spinwin.c                                                   */
/*                                                                    */
/* This application creates seven spinbutton controls allowing the    */
/* user to select the red, green, and blue color intensities, x- and  */
/* y-axis length, shape, and fill pattern for a polygon drawn on the  */
/* screen. Read-only spinbuttons and master-servant spinbuttons are   */
/* also demonstrated.                                                 */
/*                                                                    */
/**********************************************************************/

#include <windows.h>
#include <string.h>
#include "iclspbw.h"              /* include file for CCL             */
#include "spinwin.h"              /* private application include file */

/**********************************************************************/
/*  Globals                                                           */
/**********************************************************************/
/**********************************************************************/
/* spinButtonNames declares the strings to the left of the spinbuttons*/
/**********************************************************************/
static char *spinButtonNames []    =   {
                                          "Red Intensity",
                                          "Green Intensity",
                                          "Blue Intensity",
                                          "X-Axis Length",
                                          "Y-Axis Length",
                                          "Shape / Pattern"
                                       };

/**********************************************************************/
/*  pattern_names declares the strings inside the pattern spinbutton  */
/**********************************************************************/
static LPSTR pattern_names []      =   {
                                          "Solid",
                                          "Horiz. Lines",
                                          "Vert. Lines",
                                          "Empty"
                                       };
#define PATTERN_INDEX_EMPTY            3

/**********************************************************************/
/*  The pattern_mapping array represents how the above patterns are   */
/*  drawn. The low byte is the brush style and the high byte is the   */
/*  hatch style.                                                      */
/**********************************************************************/
static LONG pattern_mapping []     =   {
                                   MAKELONG (BS_SOLID,   0            ),
                                   MAKELONG (BS_HATCHED, HS_HORIZONTAL),
                                   MAKELONG (BS_HATCHED, HS_VERTICAL  ),
                                   MAKELONG (BS_SOLID,   0            )
                                       };

/**********************************************************************/
/*  The shape_names array represents the strings in the shape spin    */
/*  button                                                            */
/**********************************************************************/
static LPSTR shape_names []       =    {
                                          "Circle",
                                          "Square"
                                       };
#define SHAPE_INDEX_CIRCLE             0
#define SHAPE_INDEX_SQUARE             1

WORD      wdrawX = 0;       /* y-position where polygon will be drawn */
WORD      wdrawY = 0;       /* y-position where polygon will be drawn */




INT APIENTRY WinMain (HANDLE hInstance,
                      HANDLE hPrevInstance,
                      LPSTR  lpCmdLine,
                      int    nCmdShow);

#if defined(__IBMC__)
int main( int argc, char** argv )
{
   HANDLE hInstance = GetModuleHandle(0);
   return WinMain( hInstance, 0, 0, SW_SHOWNORMAL );
}
#endif

/**********************************************************************/
/*  Function:  WinMain                                                */
/*                                                                    */
/*  Functional Description:                                           */
/*  This function registers the window class and contains the message */
/*  loop. It loads the application icons and cursors and is           */
/*  responsible for message dispatching.                              */
/*                                                                    */
/*  Notes:  Windows program entry point                               */
/*                                                                    */
/*  Inputs:    hInstance            instance handle of execution      */
/*             hPrevInstance        instance handle of prev execution */
/*             lpCmdLine            command line arguments            */
/*             nCmdShow             startup mode (iconized/maximized) */
/*                                                                    */
/*  Outputs:   none                                                   */
/*                                                                    */
/**********************************************************************/

INT APIENTRY WinMain (HANDLE hInstance,
                      HANDLE hPrevInstance,
                      LPSTR  lpCmdLine,
                      int    nCmdShow)
{
   HANDLE      hLibrary;            /* handle to spinbutton DLL       */
   HWND        hwnd;                /* client window handle           */
   MSG         msg;                 /* message number                 */
   WNDCLASS    wndClass;            /* window class for registration  */
   static char szClientClass []  = "Spin Button Sample Application";
   POINT       ptScreenSize;        /* screen size                    */

   lpCmdLine;                       /* prevent unused warning         */

  /********************************************************************/
  /*  Register window class for client...a basic window with no fancy */
  /*  aspects.                                                        */
  /********************************************************************/
   if (! hPrevInstance)
   {
      wndClass.lpszClassName = szClientClass;
      wndClass.hInstance     = hInstance;
      wndClass.lpfnWndProc   = ClientWndProc;
      wndClass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      wndClass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      wndClass.lpszMenuName  = NULL;
      wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
      wndClass.style         = CS_HREDRAW | CS_VREDRAW;
      wndClass.cbClsExtra    = 0;
      wndClass.cbWndExtra    = 0;

      RegisterClass (&wndClass);

   }

  /********************************************************************/
  /*  Register the spin button window class                           */
  /*******************************************************************/
   if (!(hLibrary = LoadLibrary(ICL_SPINBTN_DLL)))
     return 0;

  /********************************************************************/
  /*  Get screen size                                                 */
  /********************************************************************/
   ptScreenSize.x = GetSystemMetrics (SM_CXFULLSCREEN);
   ptScreenSize.y = GetSystemMetrics (SM_CYFULLSCREEN);

  /********************************************************************/
  /*  Create a big client window                                      */
  /********************************************************************/
   hwnd = CreateWindow (szClientClass,           /* window class      */
                        "Build Shape",           /* window title      */
                        WS_OVERLAPPEDWINDOW,     /* window style      */
                        (ptScreenSize.x < 700)   /* x-position        */
                           ?  0
                           :  (ptScreenSize.x - 700) / 2,
                        (ptScreenSize.y < 400)   /* y-position        */
                           ?  0
                           :  (ptScreenSize.y - 400) / 2,
                        (ptScreenSize.x < 700)   /* x-size            */
                           ?  ptScreenSize.x
                           :  700,
                        (ptScreenSize.y < 400)   /* y-size            */
                           ?  ptScreenSize.y
                           :  400,
                        NULL,                    /* no parent         */
                        NULL,                    /* class menu        */
                        hInstance,               /* creator           */
                        NULL);                   /* params            */

   ShowWindow (hwnd, nCmdShow);
   UpdateWindow (hwnd);


  /********************************************************************/
  /*  Message loop                                                    */
  /********************************************************************/
   while (GetMessage (&msg, NULL, 0, 0))
   {
      TranslateMessage (&msg);      /* keyboard interface             */
      DispatchMessage (&msg);       /* to client window procedure     */
   }  /* end of while loop */

   FreeLibrary(hLibrary);

   return 0;
}




/**********************************************************************/
/*  Function:  ClientWndProc                                          */
/*                                                                    */
/*  Functional Description:                                           */
/*  This function processes messages sent to the client window,       */
/*  including system messages and messages from controls.             */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Inputs:    hwnd                 client window handle              */
/*             msg                  message id                        */
/*             wParam               first parameter (WORD)            */
/*             lParam               second parameter (DWORD)          */
/*                                                                    */
/*  Outputs:   return values appropriate to message                   */
/*                                                                    */
/**********************************************************************/

LRESULT CALLBACK ClientWndProc (HWND hwnd, UINT msg,
                                WPARAM wParam, LPARAM lParam)
{
   static int     nWinXLen;             /* length of client window    */
   static int     nWinYLen;             /* height of client window    */
   static BOOL    bChangeSpinning = FALSE;

   switch (msg)
   {

     /*****************************************************************/
     /*  WM_CREATE                                                    */
     /*     wParam - Not used                                         */
     /*     lParam - CREATESTRUCT pointer                             */
     /*****************************************************************/
      case WM_CREATE:
         {
            RECT     rclWindowSize;     /* size of window             */
            WORD     wSpinHeight;       /* height of spin button      */
            HDC      hdc;               /* hdc to get text metrics    */
            TEXTMETRIC tm;              /* text metrics for font      */

           /***********************************************************/
           /*  Get size of client window and store width and height   */
           /***********************************************************/
            GetClientRect (hwnd, &rclWindowSize);
            nWinXLen     = (int) rclWindowSize.right;
            nWinYLen     = (int) rclWindowSize.bottom;

           hdc = GetDC (hwnd);

           GetTextMetrics (hdc, &tm);
           wSpinHeight = tm.tmHeight + tm.tmExternalLeading + 2;

           ReleaseDC (hwnd, hdc);
           /***********************************************************/
           /*  Set location for polygon draw to be center of draw     */
           /*  field. Average left and right to get horizontal center */
           /***********************************************************/
            wdrawX        = (SHAPE_FIELD_LEFT
                            + (nWinXLen - SHAPE_FIELD_RIGHT))    / 2;
            /* average top and bottom to get vertical center          */
            wdrawY        = ((ID_SPINBUTTON_COLOR_RED *
                             (wSpinHeight + SPIN_VERT_SPACING)) +
                             (ID_SPINBUTTON_SIZE_Y *
                             (wSpinHeight + SPIN_VERT_SPACING))) / 2;

           /***********************************************************/
           /*  Create the spin buttons                                */
           /***********************************************************/
            createChildren (hwnd,
                            ((LPCREATESTRUCT)lParam)->hInstance,
                            wSpinHeight);

         }  /* end of WM_CREATE processing */

         break;


     /*****************************************************************/
     /*  WM_SIZE                                                      */
     /*     wParam - Not used                                         */
     /*     lParam - New size (length in loword, height in hiword)    */
     /*****************************************************************/
      case WM_SIZE:
         {
            WORD       wIdx;            /* loop index                 */
            WORD       wSpinHeight;     /* height of spin button      */
            HDC        hdc;             /* hdc to get text metrics    */
            TEXTMETRIC tm;              /* text metrics for font      */

           /***********************************************************/
           /*  Update window size                                     */
           /***********************************************************/
            nWinXLen   = LOWORD (lParam);
            nWinYLen   = HIWORD (lParam);

           hdc = GetDC (hwnd);

           GetTextMetrics (hdc, &tm);
           wSpinHeight = tm.tmHeight + tm.tmExternalLeading + 2;

           ReleaseDC (hwnd, hdc);
           /***********************************************************/
           /*  Adjust spinbutton positions relative to top of window  */
           /***********************************************************/
            for (wIdx = ID_SPINBUTTON_COLOR_RED;
                 wIdx <= ID_SPINBUTTON_SIZE_Y;
                 wIdx ++)
               MoveWindow (GetDlgItem (hwnd, wIdx),
                           SPIN_MARGIN_X,
                           wIdx * (wSpinHeight + SPIN_VERT_SPACING),
                           SPIN_LENGTH_X,
                           wSpinHeight,
                           TRUE);

           /***********************************************************/
           /*  The shape and pattern spinbuttons are special cases    */
           /***********************************************************/
            MoveWindow (GetDlgItem (hwnd, ID_SPINBUTTON_SHAPE),
                        SPIN_MARGIN_X,
                        ID_SPINBUTTON_SHAPE * (wSpinHeight
                           + SPIN_VERT_SPACING),
                        SPIN_TEXTLENGTH_X,
                        wSpinHeight,
                        TRUE);
            MoveWindow (GetDlgItem (hwnd, ID_SPINBUTTON_PATTERN),
                        SPIN_MARGIN_X
                           + SPIN_TEXTLENGTH_X
                           + SPIN_HORZ_SPACING,
                        ID_SPINBUTTON_SHAPE * (wSpinHeight
                           + SPIN_VERT_SPACING),
                        SPIN_TEXTLENGTH_X,
                        wSpinHeight,
                        TRUE);

           /***********************************************************/
           /*  Set location for polygon draw to be center of draw     */
           /*  field. Average left and right to get horizontal center */
           /***********************************************************/
            wdrawX         = (SHAPE_FIELD_LEFT
                             + (nWinXLen - SHAPE_FIELD_RIGHT)) / 2;

           /***********************************************************/
           /*  Average top and bottom to get vertical center          */
           /***********************************************************/
            wdrawY         = ((ID_SPINBUTTON_COLOR_RED *
                              (wSpinHeight + SPIN_VERT_SPACING)) +
                              (ID_SPINBUTTON_SIZE_Y *
                              (wSpinHeight + SPIN_VERT_SPACING))) / 2;
         }  /* end of WM_SIZE processing */

         break;


     /*****************************************************************/
     /*  WM_KEYDOWN                                                   */
     /*     wParam - Virtual key code (ASCII)                         */
     /*     lParam - Not used                                         */
     /*                                                               */
     /*  Process Enter, Tab, and Backtab keys ...                     */
     /*  Enter - Redraws the polygon                                  */
     /*  Tab and Backtab  - Shifts keyboard focus to the next         */
     /*                     and previous spinbuttons, respectively    */
     /*****************************************************************/
      case WM_KEYDOWN:
         {
            HDC            hDC;         /* device context             */
            UINT           wOldID;      /* current spinbutton focus   */

           /***********************************************************/
           /*  If enter pressed, then simply paint the window         */
           /*  (e.g. \n = enter,  \r = return) ...                    */
           /*  We do not have to repaint when processing tab/backtab, */
           /*  since the spin button will send a SPBN_KILLFOCUS       */
           /*  notification ...   We will draw the polygon whilest    */
           /*  processing SPBN_KILLFOCUS ...                          */
           /***********************************************************/
            if ((wParam == '\n') || (wParam == '\r'))
            {
               hDC   = GetDC (hwnd);
               polyDraw (NULL, hDC, ID_NULL,          /* draw polygon */
                         NOT_SPINNING);
               ReleaseDC (hwnd, hDC);
            }

           /***********************************************************/
           /*  Find ID of control having focus                        */
           /***********************************************************/
            wOldID = getWindowId(PARENTOF(GetFocus ()));

           /***********************************************************/
           /*  Process tab (\t) and backtab (<SHIFT> + \t) ... Obtain */
           /*  the <SHIFT> key's state to determine which we are      */
           /*  processing ...                                         */
           /*  TAB -                                                  */
           /*  Increment (and mod) the ID, convert back to a handle,  */
           /*  and finally give that spinbutton the focus ...         */
           /***********************************************************/
            if ((wParam == '\t') && (GetKeyState (VK_SHIFT) >= 0))
               SetFocus (GetDlgItem (hwnd, (wOldID % SPINBUTTON_COUNT)
                                               + 1));

           /***********************************************************/
           /*  BACKTAB -                                              */
           /*  Decrement (and mod) the ID, convert back to a handle,  */
           /*  and finally give that spinbutton the focus ...         */
           /***********************************************************/
            if ((wParam == '\t') && (GetKeyState (VK_SHIFT) < 0))
               SetFocus (GetDlgItem (hwnd,
                                     (wOldID - 1)
                                        ? (wOldID - 1)
                                        : SPINBUTTON_COUNT));

         }  /* end of WM_KEYDOWN processing */

         break;


     /*****************************************************************/
     /*  WM_COMMAND                                                   */
     /*     wParam - loword - Control ID                              */
     /*              hiword - Notification code                       */
     /*     lParam - Window handle of the control                     */
     /*                       sending the message                     */
     /*                                                               */
     /*  Process Spin Button notification messages in the following   */
     /*      manner: When an arrow is first pressed, a SPBN_CHANGE    */
     /*      message arrives, followed by either a SPBN_UPARROW or    */
     /*      SPBN_DOWNARROW.  If the button is immediately released,  */
     /*      a SPBN_ENDSPIN is sent next.  However, if the button is  */
     /*      held down, then the client window receives alternating   */
     /*      SPBN_CHANGE and SPBN_UPARROW/SPBN_DOWNARROW messages     */
     /*      until SPBN_ENDSPIN is sent on a button up.  For our      */
     /*      purposes, the UPARROW/DOWNARROW messages are redundant,  */
     /*      and we therefore only look at the CHANGE and ENDSPIN     */
     /*      messages.                                                */
     /*****************************************************************/
      case WM_COMMAND:
#if 0
         switch (HIWORD (wParam))                                       /*DAB1*/
         {

           /***********************************************************/
           /*  SPBN_ENDSPIN                                           */
           /*  SPBN_KILLFOCUS                                         */
           /*                                                         */
           /*    When the spin button is initialized, only a          */
           /*    SPBN_CHANGE message arrives:  no SPBN_UPARROW,       */
           /*    SPBN_DOWNARROW, or SPBN_ENDSPIN message is sent from */
           /*    the control.  The same holds true when the user edits*/
           /*    the entry field and then clicks on another window.   */
           /*    Thus, whenever a spinbutton loses the focus, we      */
           /*    assume the user has finished setting the parameter,  */
           /*    and treat it like an SPBN_ENDSPIN                    */
           /***********************************************************/
            case SPBN_ENDSPIN:
            case SPBN_KILLFOCUS:

               /*******************************************************/
               /*  Reset change spinning/not spinning flag            */
               /*                                    (not spinning)   */
               /*******************************************************/
                bChangeSpinning = FALSE;

               switch (LOWORD(wParam))                                  /*DAB1*/
               {

                 /*****************************************************/
                 /*  Spin Button (all of the above) processing        */
                 /*****************************************************/
                  case ID_SPINBUTTON_COLOR_RED:
                  case ID_SPINBUTTON_COLOR_GREEN:
                  case ID_SPINBUTTON_COLOR_BLUE:
                  case ID_SPINBUTTON_SIZE_X:
                  case ID_SPINBUTTON_SIZE_Y:
                  case ID_SPINBUTTON_PATTERN:
                  case ID_SPINBUTTON_SHAPE:

                     {
                        HDC   hDC;                 /* device context  */


                       /***********************************************/
                       /*  Draw polygon in still form                 */
                       /***********************************************/
                        hDC   = GetDC (hwnd);
                        polyDraw (GetDlgItem (hwnd, LOWORD(wParam)),    /*DAB1*/
                                  hDC,
                                  LOWORD(wParam),                       /*DAB1*/
                                  NOT_SPINNING);
                        ReleaseDC (hwnd, hDC);
                     }  /* end of KILLFOCUS and ENDSPIN processing */

                     break;

                  default:
                     break;
               }  /* end of switch on origin of SPBN_ENDSPIN */

               break;

           /***********************************************************/
           /*  SPBN_CHANGE                                            */
           /*                                                         */
           /*    The SPBN_CHANGE message indicates that the value of  */
           /*    a spinbutton is changing.  This message is sent when */
           /*    the user has clicked one of the arrow keys, and      */
           /*    therefore, the spinbutton is changing rapidly.  If   */
           /*    processing is slow, then the spinbutton response     */
           /*    time will appear to be slow too.  Therefore, we      */
           /*    must always try to minimize paint update time by     */
           /*    sending poly_draw the SPINNING flag so that only     */
           /*    the polygon outline is drawn as the shape changes.   */
           /*    The final form of the polygon is drawn when the      */
           /*    ENDSPIN message is received.                         */
           /*    We have another consideration, however:  the shape   */
           /*    spinbutton is a servant that we control based upon   */
           /*    input from the pattern spinbutton.  As a result,     */
           /*    we will not necessarily get an ENDSPIN message       */
           /*    from it. Thus, we must always paint it in final form.*/
           /***********************************************************/
            case SPBN_CHANGE:

             switch (LOWORD(wParam))                                    /*DAB1*/
             {

              /********************************************************/
              /*  Spin Button Shape processing                        */
              /********************************************************/
               case ID_SPINBUTTON_SHAPE:
                {

                  /****************************************************/
                  /*  For circle, show sheer windows, for square      */
                  /*  hide them                                       */
                  /****************************************************/
                   DWORD           dwCurrent;      /* current value   */
                   HDC             hDC;            /* device context  */

                  /****************************************************/
                  /*  Get current value of the spin button ...        */
                  /****************************************************/
                   SendMessage( GetDlgItem (hwnd, LOWORD(wParam)),      /*DAB1*/
                                SPBM_QUERYVALUE,
                                (DWORD)(LPVOID)&dwCurrent,              /*DAB1*/
                                (LPARAM)MAKELONG(0,SPBQ_UPDATEIFVALID));/*DAB1*/

                  /****************************************************/
                  /*  and draw polygon in final form                  */
                  /****************************************************/
                   hDC   = GetDC (hwnd);
                   polyDraw (GetDlgItem (hwnd, LOWORD(wParam)),         /*DAB1*/
                             hDC,
                             LOWORD(wParam),                            /*DAB1*/
                             NOT_SPINNING);
                   ReleaseDC (hwnd, hDC);
                }  /* end of if we get a SPBN_CHANGE from
                      ID_SPINBUTTON_SHAPE */

                break;

              /********************************************************/
              /*  Spin Button Color processing                        */
              /********************************************************/
               case ID_SPINBUTTON_COLOR_RED:
               case ID_SPINBUTTON_COLOR_GREEN:
               case ID_SPINBUTTON_COLOR_BLUE:
               case ID_SPINBUTTON_PATTERN:

                {
                  /****************************************************/
                  /*  In this case, we draw the polygon in SPINNING   */
                  /*  mode                                            */
                  /****************************************************/
                   HDC    hDC;                 /* device context      */

                   hDC   = GetDC (hwnd);
                   polyDraw (GetDlgItem (hwnd, LOWORD(wParam)),         /*DAB1*/
                             hDC,
                             LOWORD(wParam),                            /*DAB1*/
                             SPINNING);
                   ReleaseDC (hwnd, hDC);
                }  /* end of CHANGE processing from other controls */

                break;

              /********************************************************/
              /*  Spin Button Shape processing                        */
              /********************************************************/
               case ID_SPINBUTTON_SIZE_X:
               case ID_SPINBUTTON_SIZE_Y:

                {
                  /****************************************************/
                  /*  Determine se, whether we draw the polygon in    */
                  /*  SPINNING or NOT_SPINNING mode                   */
                  /****************************************************/
                   HDC    hDC;                 /* device context      */

                   hDC   = GetDC (hwnd);
                   polyDraw (GetDlgItem (hwnd, LOWORD(wParam)),         /*DAB1*/
                             hDC,
                             LOWORD(wParam),                            /*DAB1*/
                             ((bChangeSpinning) ? SPINNING
                                                : NOT_SPINNING));
                   ReleaseDC (hwnd, hDC);
                }  /* end of CHANGE processing from other controls */

                break;

               default:   /* other sources */
                  break;

             } /* endswitch on origin of SPBN_CHANGE */

             break;

           /***********************************************************/
           /*  SPBN_UPARROW                                           */
           /*                                                         */
           /*    The SPBN_UPARROW message is sent when the user       */
           /*    presses the up arrow or if the user clicks on        */
           /*    the up arrow button.  In most cases, we don't        */
           /*    really worry about this message since it is          */
           /*    followed by a SPBN_CHANGE message.  However, we      */
           /*    are implementing a master-servant relationship       */
           /*    between pattern and shape.  Thus, whenever the       */
           /*    pattern spinbutton rolls over, we must cause the     */
           /*    shape spinbutton to spin up or spin down.            */
           /***********************************************************/
            case SPBN_UPARROW:

            /**********************************************************/
            /*  Set change spinning/not spinning flag (spinning)      */
            /**********************************************************/
             bChangeSpinning = TRUE;

             switch (LOWORD(wParam))                                    /*DAB1*/
             {

              /********************************************************/
              /*  Spin Button Shape processing                        */
              /********************************************************/
               case ID_SPINBUTTON_PATTERN:
                {
                  /****************************************************/
                  /*  Spinning up is caused by clicking the up arrow  */
                  /*  pressing the pgUp key, or pressing cursorUp.  It*/
                  /*  causes the spin button to move DOWN the list of */
                  /*  text (e.g. increment the array index).          */
                  /*                                                  */
                  /*  Check whether we have spun up back to 0: if we  */
                  /*  have, then spin up shape field                  */
                  /****************************************************/
                   DWORD          dwCurrent;      /* curr spinbtn valu*/

                  /****************************************************/
                  /*  Get current spinbutton value                    */
                  /****************************************************/
                   SendMessage( GetDlgItem( hwnd,
                                            ID_SPINBUTTON_PATTERN),
                                SPBM_QUERYVALUE,
                                (DWORD)(LPVOID)&dwCurrent,              /*DAB1*/
                                (LPARAM)MAKELONG(0,SPBQ_UPDATEIFVALID));/*DAB1*/

                  /****************************************************/
                  /*If we have returned to 0, then spinup shape by one*/
                  /****************************************************/
                   if (dwCurrent == 0)
                      SendMessage ( GetDlgItem (hwnd,
                                                ID_SPINBUTTON_SHAPE),
                                    SPBM_SPINUP,
                                    1L,                                 /*DAB1*/
                                    0L );                               /*DAB1*/
                }  /* end of spin up from pattern spin button */

                break;

               default:
                  break;

             }  /* end of switch on source of SBPN_UPARROW */

             break;

           /***********************************************************/
           /*  SPBN_DOWNARROW                                         */
           /*                                                         */
           /*    The SPBN_DOWNARROW message is sent when the user     */
           /*    presses the down arrow or if the user clicks on      */
           /*    the down arrow button.  In most cases, we don't      */
           /*    really worry about this message since it is          */
           /*    followed by a SPBN_CHANGE message.  However, we      */
           /*    are implementing a master-servant relationship       */
           /*    between pattern and shape.  Thus, whenever the       */
           /*    pattern spinbutton rolls over, we must cause the     */
           /*    shape spinbutton to spin up or spin down.            */
           /***********************************************************/
            case SPBN_DOWNARROW:

            /**********************************************************/
            /*  Set change spinning/not spinning flag (spinning)      */
            /**********************************************************/
             bChangeSpinning = TRUE;

             switch (LOWORD(wParam))                                    /*DAB1*/
             {

              /********************************************************/
              /*  Spin Button Shape processing                        */
              /********************************************************/
               case ID_SPINBUTTON_PATTERN:
                {
                  /****************************************************/
                  /*  Spinning down is caused by clicking the up arrow*/
                  /*  pressing the pgDown key, or pressing cursorDown.*/
                  /*  It causes the spin button to move UP the list of*/
                  /*  text (e.g. decrement the array index).          */
                  /*                                                  */
                  /*  Check whether we have spun down to end of list: */
                  /*  if we have, then spin down shape field ..       */
                  /****************************************************/
                   DWORD          dwCurrent;     /* curr spinbtn value*/

                  /****************************************************/
                  /*  Get current spinbutton value                    */
                  /****************************************************/
                   SendMessage( GetDlgItem (hwnd,
                                            ID_SPINBUTTON_PATTERN),
                                SPBM_QUERYVALUE,
                                (DWORD)(LPVOID)&dwCurrent,              /*DAB1*/
                                (LPARAM)MAKELONG(0,SPBQ_UPDATEIFVALID));/*DAB1*/

                  /****************************************************/
                  /*  If we have hit end of list, then spindown shape */
                  /*  by one ...                                      */
                  /****************************************************/
                   if ( dwCurrent == (sizeof (pattern_names)
                                    / sizeof (pattern_names [0])
                                    - 1) )
                      SendMessage ( GetDlgItem (hwnd,
                                                ID_SPINBUTTON_SHAPE),
                                    SPBM_SPINDOWN,
                                    1L,                                 /*DAB1*/
                                    0L );                               /*DAB1*/

                }  /* end of spin down from pattern spin button */

                break;

               default:
                  break;

            }  /* end of switch on source of SBPN_DOWNARROW */

            break;

          default:
             break;

       } /* endswitch on id of message command */
#endif
       break;


     /*****************************************************************/
     /*  WM_PAINT                                                     */
     /*     wParam - Not used                                         */
     /*     lParam - Not used                                         */
     /*                                                               */
     /*  Our paint processing consists of filling the drawing field   */
     /*  background, drawing the spinbutton labels to the left, and   */
     /*  drawing the polygon in NOT_SPINNING state.                   */
     /*****************************************************************/
      case WM_PAINT:
       {
          PAINTSTRUCT   ps;           /* paint structure              */
          RECT          rclFillRgn;   /* area we are filling backgrnd */
          HDC           hDC;          /* device context               */
          WORD          wOldBKMode;   /* old background color mode    */
          WORD          wIndex;       /* loop index                   */
          WORD          wSpinHeight;  /* height of spin button        */
          TEXTMETRIC    tm;           /* text metrics for font        */

         /*************************************************************/
         /*  Initialize paint and fill background                     */
         /*************************************************************/
          hDC         = BeginPaint (hwnd, &ps);

          GetTextMetrics (hDC, &tm);
          wSpinHeight = tm.tmHeight + tm.tmExternalLeading + 2;

         /*************************************************************/
         /*  Fill drawing window                                      */
         /*************************************************************/
          rclFillRgn.left    = SHAPE_FIELD_LEFT;
          rclFillRgn.right   = nWinXLen - SHAPE_FIELD_RIGHT;
          rclFillRgn.bottom  = ID_SPINBUTTON_SIZE_Y *
                               (wSpinHeight + SPIN_VERT_SPACING);
          rclFillRgn.top     = ID_SPINBUTTON_COLOR_RED *
                               (wSpinHeight + SPIN_VERT_SPACING);
          FillRect (hDC, &rclFillRgn, GetStockObject (WHITE_BRUSH));

         /*************************************************************/
         /*  Initialize text starting point and drawing mode          */
         /*************************************************************/
          wOldBKMode  = SetBkMode (hDC,          /* leave bkgnd alone */
                                   TRANSPARENT);
          SetTextColor (hDC, MAIN_TEXT_COLOR);   /*  set text color   */
          SetTextAlign (hDC, TA_LEFT | TA_TOP);

         /*************************************************************/
         /*  For each available spin button label, move down the      */
         /*  screen and draw the text.                                */
         /*************************************************************/
          for (wIndex = 0;
               wIndex < (sizeof (spinButtonNames)
                           / sizeof (spinButtonNames [0]));
               wIndex ++)
          {
             TextOut (hDC,
                      TEXT_MARGIN_X,
                      (wIndex+1) * (wSpinHeight + SPIN_VERT_SPACING),
                      spinButtonNames [wIndex],
                      strlen (spinButtonNames [wIndex]));
          }

         /*************************************************************/
         /*  Restore the drawing mode                                 */
         /*************************************************************/
          SetBkMode (hDC, wOldBKMode);

         /*************************************************************/
         /*  Finally, draw polygon in its current form                */
         /*************************************************************/
          polyDraw (NULL, hDC, ID_NULL, NOT_SPINNING);

          EndPaint (hwnd, &ps);
       }  /* end of WM_PAINT processing */

       break;


     /*****************************************************************/
     /*  WM_DESTROY                                                   */
     /*     wParam - Not used                                         */
     /*     lParam - Not used                                         */
     /*                                                               */
     /*  With WM_DESTROY, we have nothing to deallocate, so just quit */
     /*****************************************************************/
      case WM_DESTROY:
         PostQuitMessage (0);
         break;

      default:      /* other messages */
         break;

   }  /* end switch on message */

   return (DefWindowProc (hwnd, msg, wParam, lParam));
}




/**********************************************************************/
/*  Function:  polyDraw                                               */
/*                                                                    */
/*  Functional Description:                                           */
/*  This function is called whenever the polygon needs to be updated. */
/*  That is, whenever a spinbutton changes value or when a paint is   */
/*  necessary.  The parameters determine which information to update. */
/*  We first erase the current polygon by repainting it with the view */
/*  field background color.  Next, we get the new data from the spin  */
/*  button being updated. Note that if update_id is ID_NULL and       */
/*  hwndSpin is NULL, then no data is changing--and hence no data is  */
/*  actually updated at all.   Finally, we draw the new polygon.      */
/*  There are two drawing modes which are specified by the fourth     */
/*  parameter.  If it is SPINNING, then the spin button is in motion  */
/*  (or it has not reached a stable value).  In this case, painting   */
/*  must be done quickly...and if the polygon's shape is changing,    */
/*  it is faster to only draw its outline (erasing and refreshing     */
/*  the polygon takes too long).  If the parameter is NOT_SPINNING,   */
/*  then the full polygon is drawn and filled.                        */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Inputs:    hwndSpin             current spin button handle        */
/*             hDC                  device context for client window  */
/*             wUpdateID            id of spinbutton being updated    */
/*             nStatus              whether or not we are currently   */
/*                                  spinning                          */
/*  Outputs:   None                                                   */
/*                                                                    */
/**********************************************************************/

VOID polyDraw (HWND hwndSpin, HDC hDC, WORD wUpdateID, int nStatus)
{
 static DWORD  dwColorIndexRed   = OPTION_COLOR_INIT;  /* R intensity */
 static DWORD  dwColorIndexGreen = OPTION_COLOR_INIT;  /* G intensity */
 static DWORD  dwColorIndexBlue  = OPTION_COLOR_INIT;  /* B intensity */
 static DWORD  dwSizeX           = OPTION_SIZE_X_INIT; /* Size-X-Axis */
 static DWORD  dwSizeY           = OPTION_SIZE_Y_INIT; /* Size-Y-Axis */
 static DWORD  dwPattern         = OPTION_PATTERN_INIT;/* Pattern     */
 static DWORD  dwShape           = OPTION_SHAPE_INIT;  /* Shape       */
 static BOOL   bRedrawing        = NOT_SPINNING;       /* Draw mode   */

 WORD             wOldBKMode;         /* Old background mix mode      */
 HPEN             hPen;               /* Polygon outline pen          */
 HPEN             hPenOld;            /* Old pen                      */
 HBRUSH           hBrush;             /* Polygon fill brush           */
 HBRUSH           hBrushOld;          /* Old brush                    */
 LOGBRUSH         logBrush;           /* Used to create hBrush        */
 LPVOID           lpValueBuf;         /* Used to get spinbutton value *//*DAB1*/
 WORD             wValue;                                               /*DAB1*/


 SetBkColor (hDC, SHAPE_FIELD_COLOR);             /* white background */
 wOldBKMode       = SetBkMode (hDC, OPAQUE);      /* overpaint        */

 /*********************************************************************/
 /* Only redraw if outline of object is changing -- if size or shape  */
 /* is changing, or if a paint message is received.                   */
 /* If color is changing, we can just paint on top of the old shape   */
 /* without erasing it first.                                         */
 /*********************************************************************/
 if ( (wUpdateID == ID_SPINBUTTON_SIZE_X)                    ||
      (wUpdateID == ID_SPINBUTTON_SIZE_Y)                    ||
      (wUpdateID == ID_SPINBUTTON_SHAPE)                     ||
      (wUpdateID == ID_NULL) )
 {
   /*******************************************************************/
   /*  Create brush for filling area ... If we have only been drawing */
   /*  the outline, then there is no need to fill the inside.         */
   /*******************************************************************/
    logBrush.lbStyle      = (bRedrawing)
                               ? BS_HOLLOW
                               : LOWORD (pattern_mapping [dwPattern]);
    logBrush.lbColor      = SHAPE_FIELD_COLOR;
    logBrush.lbHatch      = HIWORD (pattern_mapping [dwPattern]);
    hBrush                = CreateBrushIndirect (&logBrush);
    hBrushOld             = SelectObject (hDC, hBrush);

   /*******************************************************************/
   /*  Use a solid pen whose color is the background color            */
   /*******************************************************************/
    hPen               = CreatePen (PS_SOLID, 0, SHAPE_FIELD_COLOR);
    hPenOld            = SelectObject (hDC, hPen);

   /*******************************************************************/
   /*  Draw the shape                                                 */
   /*******************************************************************/
    switch ((WORD) dwShape)
    {
       case SHAPE_INDEX_CIRCLE:
          Ellipse (hDC,
                   (int) (wdrawX - dwSizeX),
                   (int) (wdrawY - dwSizeY),
                   (int) (wdrawX + dwSizeX),
                   (int) (wdrawY + dwSizeY));
          break;

       case SHAPE_INDEX_SQUARE:
          Rectangle (hDC,
                     (int) (wdrawX - dwSizeX),
                     (int) (wdrawY - dwSizeY),
                     (int) (wdrawX + dwSizeX),
                     (int) (wdrawY + dwSizeY));
          break;
    }  /* end of case on type of polygon to erase */

   /*******************************************************************/
   /*  Deselect our objects and delete brushes and pens               */
   /*******************************************************************/
    SelectObject (hDC, hBrushOld);
    DeleteObject (hBrush);
    SelectObject (hDC, hPenOld);
    DeleteObject (hPen);

   /*******************************************************************/
   /*  If spin status is changing, then update it for next time       */
   /*******************************************************************/
    if (nStatus != NO_SPIN_CHANGE)
       bRedrawing  = nStatus;

 }  /* end of if we must erase previous shape */

 /*********************************************************************/
 /* Now, if a spinbutton is changing value (id != ID_NULL), query it  */
 /* for the new value.  Unless the shape is changing, always get      */
 /* a valid value.  Shape can be invalid while the user is typing.    */
 /* However, as soon as a valid shape is entered, update it.          */
 /*********************************************************************/
 wValue      = SPBQ_ALWAYSUPDATE;                                       /*DAB1*/

 /*********************************************************************/
 /*  Set pointer to the value to change                               */
 /*********************************************************************/
 switch (wUpdateID)
 {
    case ID_SPINBUTTON_COLOR_RED:
       lpValueBuf   = &dwColorIndexRed;                                 /*DAB1*/
       break;

    case ID_SPINBUTTON_COLOR_GREEN:
       lpValueBuf   = &dwColorIndexGreen;                               /*DAB1*/
       break;

    case ID_SPINBUTTON_COLOR_BLUE:
       lpValueBuf   = &dwColorIndexBlue;                                /*DAB1*/
       break;

    case ID_SPINBUTTON_SIZE_X:
       lpValueBuf   = &dwSizeX;                                         /*DAB1*/
       break;

    case ID_SPINBUTTON_SIZE_Y:
       lpValueBuf   = &dwSizeY;                                         /*DAB1*/
       break;

    case ID_SPINBUTTON_PATTERN:
      /****************************************************************/
      /*  Since we want the array index instead of the actual string, */
      /*  we use the pattern variable instead of a PSTR buffer        */
      /****************************************************************/
       lpValueBuf   = &dwPattern;                                       /*DAB1*/
       break;

    case ID_SPINBUTTON_SHAPE:
      /****************************************************************/
      /*  Since we want the array index instead of the actual string, */
      /*  we use the shape variable instead of a PSTR buffer          */
      /****************************************************************/
       wValue      = SPBQ_UPDATEIFVALID;                                /*DAB1*/
       lpValueBuf   = &dwShape;                                         /*DAB1*/
       break;

    default:
       lpValueBuf   = 0;                                                /*DAB1*/
       break;
 }  /* end of switch on spinbutton to update */

 /*********************************************************************/
 /*  Now get new value if one is coming                               */
 /*********************************************************************/
 if (lpValueBuf)
    SendMessage ( hwndSpin,
                  SPBM_QUERYVALUE,
                  (DWORD)(LPVOID)lpValueBuf,                            /*DAB1*/
                  (LPARAM)MAKELONG(0,wValue));                          /*DAB1*/

 /*********************************************************************/
 /*  Create a logical brush that matches our parameters; if redrawing,*/
 /*  don't fill it in.                                                */
 /*********************************************************************/
 logBrush.lbStyle      = (bRedrawing)
                            ? BS_HOLLOW
                            : LOWORD (pattern_mapping [dwPattern]);
 logBrush.lbColor      = RGB (dwColorIndexRed,
                              dwColorIndexGreen,
                              dwColorIndexBlue);
 logBrush.lbHatch      = HIWORD (pattern_mapping [dwPattern]);

 /*********************************************************************/
 /* In Windows, a BS_HOLLOW brush implies that Windows will not       */
 /* attempt to paint the interior at all ... Not even with the        */
 /* background color.  Thus, to get an "empty" region, we must use a  */
 /* SOLID brush with SHAPE_FIELD_COLOR as the foreground color.       */
 /*********************************************************************/
 if (dwPattern == PATTERN_INDEX_EMPTY)
    logBrush.lbColor   = SHAPE_FIELD_COLOR;

 /*********************************************************************/
 /* Create and select our brush and pen; note that we use a solid pen */
 /* with the appropriate color.                                       */
 /*********************************************************************/
 hBrush                = CreateBrushIndirect (&logBrush);
 hBrushOld             = SelectObject (hDC, hBrush);
 hPen                  = CreatePen (PS_SOLID, 0,
                                    RGB (dwColorIndexRed,
                                         dwColorIndexGreen,
                                         dwColorIndexBlue));
 hPenOld               = SelectObject (hDC, hPen);

 /*********************************************************************/
 /*  Finally, draw the polygon                                        */
 /*********************************************************************/
 switch ((WORD) dwShape)
 {
    case SHAPE_INDEX_CIRCLE:
       Ellipse (hDC,
                (int) (wdrawX - dwSizeX),
                (int) (wdrawY - dwSizeY),
                (int) (wdrawX + dwSizeX),
                (int) (wdrawY + dwSizeY));
       break;

    case SHAPE_INDEX_SQUARE:
       Rectangle (hDC,
                  (int) (wdrawX - dwSizeX),
                  (int) (wdrawY - dwSizeY),
                  (int) (wdrawX + dwSizeX),
                  (int) (wdrawY + dwSizeY));
       break;
 }  /* end of switch on shape */

 /*********************************************************************/
 /*  Cleanup our objects and restore things to the way they were      */
 /*********************************************************************/
 SelectObject (hDC, hBrushOld);
 DeleteObject (hBrush);
 SelectObject (hDC, hPenOld);
 DeleteObject (hPen);

 SetBkMode (hDC, wOldBKMode);
}




/**********************************************************************/
/*  Function:  createChildren                                         */
/*                                                                    */
/*  Functional Description:                                           */
/*  This function is called during WM_CREATE processing.  It creates  */
/*  the seven spinbutton controls, sets their limits, and sets their  */
/*  initial values.  It also sets up master-servant relationships and */
/*  sets the focus to the first spinbutton.                           */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Inputs:    hwndClient           client window handle              */
/*             hInstance            execution instance handle         */
/*             wSpinHeight          height of the spin button         */
/*                                                                    */
/*  Outputs:   None                                                   */
/*                                                                    */
/**********************************************************************/

VOID createChildren (HWND hwndClient, HANDLE hInstance, WORD wSpinHeight)
{
   DWORD      dwSpinStyle;              /* spin button style          */
   WORD       wIdx;                     /* loop index                 */

  /********************************************************************/
  /*  Set spin button style                                           */
  /********************************************************************/
   dwSpinStyle    = SPBS_MASTER      |   /* spn btn has its own btns  */
                    SPBS_NUMERICONLY |   /* and it only holds numbers */
                    SPBS_JUSTRIGHT   |   /* numbers are rght justified*/
                    SPBS_FASTSPIN    |   /* and it spins faster as the*/
                                         /* arrows are held down      */
                    WS_CHILD;            /* and is a child window     */

  /********************************************************************/
  /*  Create the first five numeric spinbuttons with this style       */
  /********************************************************************/
   for (wIdx = ID_SPINBUTTON_COLOR_RED;
        wIdx <= ID_SPINBUTTON_SIZE_Y;
        wIdx ++)
   {
      LONG       lMin;         /* minimum value */
      LONG       lMax;         /* maximum value */
      LONG       lInit;        /* initial value */
      WORD       wTextLimit;   /* text limit    */

     /*****************************************************************/
     /*  Set up initialization values                                 */
     /*****************************************************************/
      switch (wIdx)
      {
        /**************************************************************/
        /*  Color spin buttons                                        */
        /**************************************************************/
         case ID_SPINBUTTON_COLOR_RED:
         case ID_SPINBUTTON_COLOR_GREEN:
         case ID_SPINBUTTON_COLOR_BLUE:
            lMax  = OPTION_COLOR_MAX;
            lMin  = OPTION_COLOR_MIN;
            lInit = OPTION_COLOR_INIT;
            wTextLimit = (WORD)SPIN_TEXT_MAXCHARS;
            break;

        /**************************************************************/
        /*  X-size                                                    */
        /**************************************************************/
         case ID_SPINBUTTON_SIZE_X:
            dwSpinStyle   ^= SPBS_FASTSPIN;  /* update style: anything*/
                                             /* further does not      */
                                             /* have FASTSPIN         */
            lMax  = OPTION_SIZE_X_MAX;
            lMin  = OPTION_SIZE_X_MIN;
            lInit = OPTION_SIZE_X_INIT;
            wTextLimit = (WORD)(SPIN_TEXT_MAXCHARS - 1);
            break;

        /**************************************************************/
        /*  Y-size                                                    */
        /**************************************************************/
         case ID_SPINBUTTON_SIZE_Y:
            lMax  = OPTION_SIZE_Y_MAX;
            lMin  = OPTION_SIZE_Y_MIN;
            lInit = OPTION_SIZE_Y_INIT;
            wTextLimit = (WORD)(SPIN_TEXT_MAXCHARS - 1);
            break;
      }  /* end of switch on which spin button is being created */

     /*****************************************************************/
     /*  Create colors/sizes spin buttons                             */
     /*****************************************************************/
      createSpinButton (hwndClient,          /* client window         */
                        hInstance,           /* instance handle       */
                        SPIN_MARGIN_X,       /* x-position            */
                        wIdx * (wSpinHeight  /* y-position            */
                           + SPIN_VERT_SPACING),
                        SPIN_LENGTH_X,       /* width of spin button  */
                        wSpinHeight,         /* height of spin button */
                        dwSpinStyle,         /* style                 */
                        wIdx,                /* id                    */
                        TRUE,                /* numeric type          */
                        lMin,                /* minimum               */
                        lMax,                /* maximum               */
                        NULL,                /* strings               */
                        0,                   /* no strings            */
                        (LONG)lInit);        /* initial value         */

     /*****************************************************************/
     /*  Set how many characters the user can type into the spin btn  */
     /*****************************************************************/
      SendMessage ( GetDlgItem (hwndClient, wIdx),
                    SPBM_SETTEXTLIMIT,
                    wTextLimit,
                    0L );
   } /* endfor */

  /********************************************************************/
  /*  Style for shape spin button                                     */
  /********************************************************************/
   dwSpinStyle    = SPBS_SERVANT       |    /* no buttons             */
                    SPBS_ALLCHARACTERS |    /* string only            */
                    SPBS_JUSTCENTER    |    /* center text            */
                    WS_CHILD;               /* child window           */

  /********************************************************************/
  /*  Create a spinbutton for shape                                   */
  /********************************************************************/
   createSpinButton (hwndClient,            /* client window          */
                     hInstance,             /* instance handle        */
                     SPIN_MARGIN_X,         /* x-position             */
                     ID_SPINBUTTON_SHAPE    /* y-position             */
                        * (wSpinHeight
                        + SPIN_VERT_SPACING),
                     SPIN_TEXTLENGTH_X,     /* width of spin button   */
                     wSpinHeight,           /* height of spin button  */
                     dwSpinStyle,           /* style                  */
                     ID_SPINBUTTON_SHAPE,   /* id                     */
                     FALSE,                 /* not numeric type       */
                     0L,                    /* no minimum             */
                     0L,                    /* no maximum             */
                     shape_names,           /* strings                */
                     sizeof (shape_names)   / sizeof (shape_names [0]),
                                            /* number of strings      */
                     (LONG)OPTION_SHAPE_INIT);  /* initial value      */


  /********************************************************************/
  /*  Style for pattern spin button                                   */
  /********************************************************************/
   dwSpinStyle    = SPBS_MASTER        |     /* has buttons           */
                    SPBS_READONLY      |     /* user cannot type in it*/
                    SPBS_JUSTCENTER    |     /* center text           */
                    WS_CHILD;                /* child window          */

  /********************************************************************/
  /*  Create a spinbutton for pattern                                 */
  /********************************************************************/
   createSpinButton (hwndClient,                  /* client window    */
                     hInstance,                   /* instance handle  */
                     SPIN_MARGIN_X                /* x-position       */
                        + SPIN_TEXTLENGTH_X
                        + SPIN_HORZ_SPACING,
                     ID_SPINBUTTON_SHAPE          /* y-position       */
                        * (wSpinHeight
                        + SPIN_VERT_SPACING),
                     SPIN_TEXTLENGTH_X,           /* width of spin    */
                     wSpinHeight,                 /* height of spin   */
                     dwSpinStyle,                 /* style            */
                     ID_SPINBUTTON_PATTERN,       /* id               */
                     FALSE,                       /* not numeric type */
                     0L,                          /* no minimum       */
                     0L,                          /* no maximum       */
                     pattern_names,               /* strings          */
                     (sizeof (pattern_names) /
                     sizeof (pattern_names [0])),
                                                  /* number of strings*/
                     (LONG)OPTION_PATTERN_INIT);  /* initial value    */

  /********************************************************************/
  /*  Set pattern spin button as master to shape spin button ... by   */
  /*  doing this, when the focus is on the servant spin button, the   */
  /*  master's arrow buttons manipulate the servant's value.          */
  /********************************************************************/
   SendMessage (GetDlgItem (hwndClient, ID_SPINBUTTON_SHAPE),
                SPBM_SETMASTER,
                (LONG)GetDlgItem (hwndClient, ID_SPINBUTTON_PATTERN),
                0L);

  /********************************************************************/
  /* Set focus to first spinbutton                                    */
  /********************************************************************/
   SetFocus (GetDlgItem (hwndClient, ID_SPINBUTTON_COLOR_RED));
}

/**********************************************************************/
/*  Function:  createSpinButton                                       */
/*                                                                    */
/*  Functional Description:                                           */
/*  This function creates and sets up a spinbutton.  It calls         */
/*  CreateWindow to create the spinbutton.  It then sends messages to */
/*  initialize its values and set a starting value.                   */
/*                                                                    */
/*  Notes:                                                            */
/*                                                                    */
/*  Inputs:    hwndClient           client window handle              */
/*             hInstance            execution instance handle         */
/*             wPosX                x-position                        */
/*             wPosY                y-position                        */
/*             wCx                  x-length                          */
/*             wCy                  y-length                          */
/*             dwSpinStyle          spinbutton style                  */
/*             wID                  spinbutton id                     */
/*             bNumericType         TRUE if numeric, FALSE if string  */
/*             lMin                 minimum value, if numeric         */
/*             lMax                 maximum value, if numeric         */
/*             lpStrings            string array, if string           */
/*             wStringCount         number of strings, if string      */
/*             lInit                initial value                     */
/*                                                                    */
/*  Outputs:   None                                                   */
/*                                                                    */
/**********************************************************************/

VOID createSpinButton (HWND       hwndClient,
                       HANDLE     hInstance,
                       WORD       wPosX,
                       WORD       wPosY,
                       WORD       wCx,
                       WORD       wCy,
                       DWORD      dwSpinStyle,
                       WORD       wID,
                       BOOL       bNumericType,
                       LONG       lMin,
                       LONG       lMax,
                       LPSTR      lpStrings [],
                       WORD       wStringCount,
                       LONG       lInit)
{
   HWND       hwndSpin;                  /* spinbutton handle         */


  /********************************************************************/
  /*  Create a spinbutton with the given parameters                   */
  /********************************************************************/
   hwndSpin = CreateWindow (ICL_SPINBTN,            /* class          */
                            NULL,                   /* title          */
                            dwSpinStyle,            /* style          */
                            wPosX, wPosY,           /* origin         */
                            wCx, wCy,               /* size           */
                            hwndClient,             /* parent         */
                            (HMENU)wID,             /* ID             */
                            hInstance,              /* creator        */
                            NULL);                  /* data           */

  /********************************************************************/
  /*  If numeric only spin button, then set limits                    */
  /********************************************************************/
   if (bNumericType)
   {
      SendMessage ( hwndSpin,                        /* set them      */
                    SPBM_SETLIMITS,
                    lMax, lMin );                                       /*DAB1*/
   }
   else
   {
     /*****************************************************************/
     /*  In a string spin button, we set the string array as elements:*/
     /*  wParam is the number of elements, lParam points to the array */
     /*****************************************************************/
      SendMessage ( hwndSpin,
                    SPBM_SETARRAY,
                    ((DWORD)(LPVOID)(lpStrings)),                       /*DAB1*/
                    wStringCount );                                     /*DAB1*/
    } /* end if */

  /********************************************************************/
  /*  Finally, give the spinbutton an initial value ...               */
  /********************************************************************/
   SendMessage ( hwndSpin,
                 SPBM_SETCURRENTVALUE,
                 (DWORD)(lInit),            /* initial value goes here*//*DAB1*/
                 0 );                                                   /*DAB1*/

  /********************************************************************/
  /*  and make it visible ...                                         */
  /********************************************************************/
   ShowWindow (hwndSpin, SW_SHOWNORMAL);
}

