// Revision: 74 1.2.2.2 source/ui/cnr/icnrccur.cpp, container, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: icnrccur.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrccur.hpp. This includes IContainerColumnCursor, used to             *
*   iterate the columns in the Details View of the container.                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/


/*--------------------------------------------------------------*/
/* declare the include file switches and the include files.     */
/*--------------------------------------------------------------*/
#define INCL_WINSTDCNR
#define INCL_GPILCIDS
#define INCL_DOSMODULEMGR

extern "C" {
  #include <stdio.h>
  #include <string.h>
  }

#include <icnrrec.hpp>  // Must be first for OS/2 flags

#include <irect.hpp>
#include <iexcept.hpp>
#include <icnrctl.hpp>
#include <icnrcol.hpp>
#undef IC_DEVELOP            //  Patch to compiler trap
#include <itrace.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCCUR_CPP_
  #include <ipagetun.h>
#endif

IContainerControl::ColumnCursor :: ~ColumnCursor ( ) {}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::IContainerColumnCursor

 Implementation: Construct a column cursor.
------------------------------------------------------------------------------*/
IContainerControl::ColumnCursor::ColumnCursor(const IContainerControl& cnrctl,
                                              bool fVisibleOnly)
               :  ulClChanges(cnrctl.numberOfColumnChanges())
{
   pcnrctlClOwner = (IContainerControl*)&cnrctl;
   pcnrcolClCurrent=0;
   fClVisible= fVisibleOnly;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::current

 Implementation: Return the current column.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::ColumnCursor::current()  const
{
   return(pcnrcolClCurrent);
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::setCurrent

 Implementation: Set the current column.
------------------------------------------------------------------------------*/
IContainerControl::ColumnCursor& IContainerControl::ColumnCursor::setCurrent(const IContainerColumn* pcnrcolCurrent)
{
  ulClChanges = pcnrctlClOwner->numberOfColumnChanges();
  pcnrcolClCurrent=(IContainerColumn*)pcnrcolCurrent;
  return *this;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::setToFirst

 Implementation: Point to the first column.
------------------------------------------------------------------------------*/
bool IContainerControl::ColumnCursor::setToFirst()
{
  ulClChanges = pcnrctlClOwner->numberOfColumnChanges();
  pcnrcolClCurrent = pcnrctlClOwner->column(0,CMA_FIRST, fClVisible);
  if(pcnrcolClCurrent)
     return true;
  else
     return false;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::setToNext

 Implementation: Point to the next (or first) column.
------------------------------------------------------------------------------*/
bool IContainerControl::ColumnCursor::setToNext()
{
  if(pcnrcolClCurrent)
     pcnrcolClCurrent = pcnrctlClOwner->column(pcnrcolClCurrent,CMA_NEXT,
                                               fClVisible);
  else
     pcnrcolClCurrent = pcnrctlClOwner->column(0,CMA_FIRST,fClVisible);
  if(pcnrcolClCurrent)
     return true;
  else
     return false;

}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::setToPrevious

 Implementation: Point to the previous (or last) column.
------------------------------------------------------------------------------*/
bool IContainerControl::ColumnCursor::setToPrevious()
{
  if(pcnrcolClCurrent)
    pcnrcolClCurrent = pcnrctlClOwner->column(pcnrcolClCurrent,CMA_PREV,
                                              fClVisible);
  else
    pcnrcolClCurrent = pcnrctlClOwner->column(0,CMA_LAST,fClVisible);
  if(pcnrcolClCurrent)
     return true;
  else
     return false;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::setToLast

 Implementation: Point to the last column.
------------------------------------------------------------------------------*/
bool IContainerControl::ColumnCursor::setToLast()
{
  ulClChanges = pcnrctlClOwner->numberOfColumnChanges();
  pcnrcolClCurrent = pcnrctlClOwner->column(0,CMA_LAST,fClVisible);
  if (pcnrcolClCurrent)
     return true;
  else
     return false;
}



/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::isValid

 Implementation: Check if the current column is valid.
------------------------------------------------------------------------------*/
bool IContainerControl::ColumnCursor::isValid() const
{
  if(ulClChanges == pcnrctlClOwner->numberOfColumnChanges()  &&
     pcnrcolClCurrent != 0)
     return true;
  else
     return false;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::invalidate

 Implementation: Indicate that the cursor does not point to a column.
------------------------------------------------------------------------------*/
IContainerControl::ColumnCursor& IContainerControl::ColumnCursor::invalidate()
{
  pcnrcolClCurrent = 0;
  return *this;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::first

 Implementation: Return the first column.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::ColumnCursor::first()
{
  setToFirst();
  return pcnrcolClCurrent;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::next

 Implementation: Return the next column.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::ColumnCursor::next()
{
  setToNext();
  return pcnrcolClCurrent;
}

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::last

 Implementation: Return the last column.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::ColumnCursor::last()
{
  setToLast();
  return pcnrcolClCurrent;
}
/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::ColumnCursor::previous

 Implementation: Return the previous column.
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::ColumnCursor::previous()
{
  setToPrevious();
  return pcnrcolClCurrent;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl::column

 Implementation: Return the column with the indicated association.
    (FIRST, LAST, NEXT, PREVIOUS)
------------------------------------------------------------------------------*/
IContainerColumn* IContainerControl::column(IContainerColumn* pcnrcol,
                                                   unsigned long ulCmd,
                                                   bool fVisible)
{

  PFIELDINFO pfldinfoRelative = 0;
  if(pcnrcol!=0)
     pfldinfoRelative = pcnrcol->columnInfo();

  PFIELDINFO pfldinfo;
  pfldinfo =  (PFIELDINFO)(char*)(sendEvent(
                                 CM_QUERYDETAILFIELDINFO,
                                 MPFROMP(pfldinfoRelative),
                                 MPFROMSHORT((unsigned short)ulCmd)));

  if (fVisible && pfldinfo && ((int)pfldinfo!=-1) &&
         (pfldinfo->flData & CFA_INVISIBLE) )
  {
    if (ulCmd == CMA_FIRST)
      ulCmd = CMA_NEXT;
    if (ulCmd == CMA_LAST)
      ulCmd = CMA_PREV;
    while (pfldinfo  && (pfldinfo->flData & CFA_INVISIBLE) )
    {
       pfldinfo =  (PFIELDINFO)(char*)(sendEvent(
                                 CM_QUERYDETAILFIELDINFO,
                                     MPFROMP(pfldinfo),
                                     MPFROMSHORT((unsigned short)ulCmd)));
    }
  }
  if ((int)pfldinfo==-1)
  {
     ITHROWGUIERROR("CM_QUERYDETAILFILEDINFO");
  }


  if(pfldinfo)
     return (IContainerColumn*)(pfldinfo->pUserData);
  else
     return 0;
}

