// Revision: 85 1.5.2.4 source/ui/cnr/icnrctl4.cpp, container, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrctl4.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the Tree View functions for       *
*    the IContainerControl class declared in icnrctl.hpp.                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags


#include <itrace.hpp>
#include <iexcept.hpp>
#include <ireslib.hpp>
#include <icnrctl.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTL4_CPP_
  #include <ipagetun.h>
#endif

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: showTreeLine

 Implementation: Show the tree line in tree view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: showTreeLine(long lTreeLineWidth)
{
  ICnrInfo cnrInfo;
  containerInfo(&cnrInfo);
  cnrInfo.cxTreeLine = (short)lTreeLineWidth;
  cnrInfo.flWindowAttr |= CA_TREELINE;
  setContainerInfo(&cnrInfo, CMA_CXTREELINE | CMA_FLWINDOWATTR);
  return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: hideTreeLine

 Implementation: Hide the tree line in tree view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: hideTreeLine()
{
   setContainerAttributes(CA_TREELINE, 0);
   if(isVisible() && isRefreshOn())
      refresh();
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setTreeExpandIconSize

 Implementation: Set icon size of expand and collapse icons.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setTreeExpandIconSize(const ISize& sizeIcon)
{
//ITRACE_MOTIF_NOTYET();
#ifdef IC_PMWIN
  ICnrInfo cnrInfo;
  cnrInfo.slTreeBitmapOrIcon.cx = sizeIcon.width();
  cnrInfo.slTreeBitmapOrIcon.cy = sizeIcon.height();
  setContainerInfo(&cnrInfo, CMA_SLTREEBITMAPORICON);
  return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setTreeViewIndent

 Implementation: Set the pels to indent in the tree view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setTreeViewIndent(long lIndentPels)
{
  ICnrInfo cnrInfo;
  cnrInfo.cxTreeIndent = lIndentPels;
  setContainerInfo(&cnrInfo, CMA_CXTREEINDENT);
  return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setTreeItemIcons

 Implementation: Set icons for expanded & collapsed icons.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setTreeItemIcons(const IPointerHandle& hptrExpanded,
                                           const IPointerHandle& hptrCollapsed)
{
  IASSERT(hptrExpanded!=0);
  IASSERT(hptrCollapsed!=0);
  ICnrInfo cnrInfo;

  cnrInfo.hptrExpanded  = hptrExpanded;
  cnrInfo.hptrCollapsed = hptrCollapsed;
  setContainerInfo(&cnrInfo, CMA_TREEICON);
  return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: setTreeItemIcons

 Implementation: Set icons for expanded & collapsed icons.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: setTreeItemIcons(const IResourceId& residExpanded,
                                           const IResourceId& residCollapsed)
{
   setTreeItemIcons(residExpanded.resourceLibrary().loadIcon(residExpanded),
                    residCollapsed.resourceLibrary().loadIcon(residCollapsed));
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: expand

 Implementation: Expand direct descendents of this object in Tree.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: expand(IContainerObject* pcnrobj)
{
   IMiniCnrRecord* pcnrrec = 0;
   if(pcnrobj)
       pcnrrec = IRecFromObj(pcnrobj);
   unsigned fSuccess = sendEvent(CM_EXPANDTREE, pcnrrec);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_EXPANDTREE");
   }
   return *this;
}


/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: collapse

 Implementation: Collapse direct descendents of this object in Tree.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: collapse(IContainerObject* pcnrobj)
{
   IMiniCnrRecord* pcnrrec = 0;
   if(pcnrobj)
       pcnrrec = IRecFromObj(pcnrobj);
   unsigned fSuccess = sendEvent(CM_COLLAPSETREE, pcnrrec);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_COLLAPSETREE");
   }
   return *this;
}


IContainerControl& IContainerControl :: expandTree()
{return expand(0); }


IContainerControl& IContainerControl :: collapseTree()
{
   if (objectCount() == 0)
      return *this ;
   else
      return collapse(0);
}

