// Revision: 04 1.5.2.2 source/ui/cnr/icnrehdr.cpp, container, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: icnrehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrehdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ilanglvl.hpp>

#define INCL_WININPUT
#define INCL_WINSTDCNR
#define INCL_WINWINDOWMGR

#ifdef IC_MOTIF
  #include <cnr.h>
  #include <icong.h>
#endif
extern "C" {
  #include <stdlib.h>
  #include <icconst.h>
  }

#include <icnrrec.hpp>  // Must be first for OS flags

#include <icnrehdr.hpp>
#include <icnreevt.hpp>
#include <icnrctl.hpp>
#include <imle.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <iwcname.hpp>

#ifdef IC_PMWIN
  #include <icnrctlw.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNREHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICnrEditHandler::ICnrEditHandler                                             |
------------------------------------------------------------------------------*/
ICnrEditHandler::ICnrEditHandler(StringType UserTextType)
             : _phdrMLE(0), _stringType(UserTextType) {}

/*------------------------------------------------------------------------------
| ICnrEditHandler::~ICnrEditHandler                                            |
------------------------------------------------------------------------------*/
ICnrEditHandler :: ~ICnrEditHandler    ( ) {}

/*------------------------------------------------------------------------------
| ICnrEditHandler::dispatchHandlerEvent                                        |
|                                                                              |
| Routes the edit events to the appropriate event functions.                   |
------------------------------------------------------------------------------*/
bool ICnrEditHandler::dispatchHandlerEvent(IEvent& evt)
{
   bool fStopDispatch = false;
#ifdef IC_PMWIN
   /*****************************************************************/
   /* Make sure that we are not getting MLE keystrokes.  Seems that */
   /* the MLE passes them to the default winproc when the MLE is    */
   /* disabled or the text limit is reached.                        */
   /*****************************************************************/
   if (evt.eventId() == WM_CHAR)
   {
      IContainerControl* pcnrctl = (IContainerControl*)evt.window();
      if (pcnrctl)
      {
         if(pcnrctl->currentEditMLE())
           return true;
      } /* endif */
   }
   else
#endif //IC_PMWIN
   if (evt.eventId() == WM_CONTROL)
   {                              // Allocate class name string.
         ITRACE_ALL(IString("CnrEditHandler WM_CONTROL: HWND=") +
                    IString(evt.handle().asUnsigned()).d2x() +
                    IString(", Msg=") + IString(evt.eventId()).d2x() +
                    IString(", MP1=") +
                    IString((unsigned long)evt.parameter1()).d2x() +
                    IString(", MP2=") +
                    IString((unsigned long)evt.parameter2()).d2x());

      IControlEvent ctlevt(evt);
      IWindow* pwndControl = ctlevt.controlWindow();

      if(pwndControl)
      {
        if (pwndControl->handle())
        {
          unsigned long ulMsg = evt.parameter1().number2();
          IWindowClassName className( pwndControl->handle() );

#ifdef IC_WIN
          if (className == WC_CONTAINER || className == WC_NATIVECONTAINER)
#endif
#ifdef IC_PM
          if (className == WC_CONTAINER)
#endif
          {                        // container
            switch(ulMsg)
            {
               case CN_BEGINEDIT:
               {
                     ICnrBeginEditEvent beevt(ctlevt);
                     fStopDispatch =  beginEdit(beevt);
                     evt.setResult(beevt.result());
                     break;
               }

               case CN_REALLOCPSZ:
               {
                     ICnrReallocStringEvent rseevt(ctlevt);
                     fStopDispatch = reallocateString(rseevt);
                     evt.setResult(rseevt.result());
                     break;
               }

               case CN_ENDEDIT:
               {
                     ICnrEndEditEvent eeevt(ctlevt);
                     IContainerControl* pcnrctl = eeevt.container();
                     IMultiLineEdit* pmle = pcnrctl->currentEditMLE();

                     if(pmle)
                         eeevt.strNewText = pmle->text();

                     fStopDispatch = endEdit(eeevt);
                     evt.setResult(eeevt.result());
                     break;
               }
            } /* end switch ulMsg */
          } /* endif  ulClassName == WC_CONTAINER */
        } /* endif handle valid */
      } /* pwndControl != 0 */
   } /* endif WM_CONTROL */

   return fStopDispatch;
}


/*------------------------------------------------------------------------------
| ICnrEditHandler::reallocateString                                            |
|                                                                              |
| Called when text has been modified in the container and storage needs to be  |
| reallocated.                                                                 |
------------------------------------------------------------------------------*/
bool ICnrEditHandler :: reallocateString(ICnrReallocStringEvent& evt)
{
  IMODTRACE_ALL("CnrEHdr::reallocateString");

  IContainerControl* pcnrctl = evt.container();
#ifdef IC_MOTIF
  if(evt.isDetailsData())
  {
    /********************************************************************/
    /* Need to check here to make sure we are dealing with IString's    */
    /* as the user data could be either IString or char*.              */
    /********************************************************************/
    if (_stringType == isCharacterPointer)
    {
       evt.reallocateText();
    }
    else
    {
       evt.reallocateString();
    }
    evt.setResult(true);
  }
  return true;
#endif //IC_MOTIF
  IMultiLineEdit* pmle = pcnrctl->currentEditMLE();
  IString strNewText;
  if(pmle)
  {
    strNewText = pmle->text();
    if(strNewText != evt.currentText())
    {
      /********************************************************************/
      /* Need to check here to make sure we are dealing with IString's.   */
      /* Title's and Icon text is always an IString. Heading is a char*,  */
      /* and the user data could be either IString or char*.              */
      /********************************************************************/
      if ((evt.isDetailsData() && _stringType == isCharacterPointer) ||
              evt.isRightDetailsHeading() || evt.isLeftDetailsHeading() )
         evt.reallocateText();
      else
         evt.reallocateString();
      evt.setResult(true);
    }
  }
  return true;
}

/*------------------------------------------------------------------------------
| ICnrEditHandler::beginEdit                                                   |
|                                                                              |
| Called when an edit window has been opened in the container.                 |
| Notes: The default behavior is to create an instance of an IMultiLineEdit.   |
|        If setMLEHandler has been previously called to store an edit handler, |
|          the handler will be added to the MLE control.                       |
------------------------------------------------------------------------------*/
bool ICnrEditHandler :: beginEdit(ICnrBeginEditEvent& evt)
{
  IMODTRACE_ALL("CnrEHdr::beginEdit");
  /********************************************************/
  /* Subclass the MLE.                                    */
  /********************************************************/
  IContainerControl* pcnrctl = evt.container();
  IASSERT(pcnrctl!=0);

#ifdef IC_MOTIF
  IMultiLineEdit* pmle = pcnrctl->currentEditMLE();
#endif

#ifdef IC_PMWIN
  IMultiLineEdit* pmle;
  if (pcnrctl->ppd->pmCompatible)
  {
  pmle = multiLineEdit( IWindow::handleWithParent(CID_MLE,
                                                  pcnrctl->handle()));
  }
#ifdef IC_WIN
  else
  {
    if (pcnrctl->ppd->pActiveview == pcnrctl->ppd->pListview)
    {
      if (pcnrctl->ppd->pDetailsEdit)
        pmle = pcnrctl->ppd->pDetailsEdit;
      else
      {
        pmle = multiLineEdit( IWindowHandle( ListView_GetEditControl( pcnrctl->ppd->pListview->handle() )));
        // The edit control for the common controls should not be destroyed.
        pmle->setAutoDestroyWindow( false );
      }
    }
    else
    {
      pmle = multiLineEdit( IWindowHandle( TreeView_GetEditControl( pcnrctl->ppd->pTreeview->handle() )));
      // The edit control for the common controls should not be destroyed.
      pmle->setAutoDestroyWindow( false );
    }
  }
#endif  //IC_WIN
#endif  //IC_PMWIN

  if(pmle)
  {
    pcnrctl->setEditMLE(pmle);
    pcnrctl->setEditColumn(evt.column());
    pcnrctl->setEditObject(evt.object());

    /*******************************************************/
    /* If an edit handler provided, add it.                */
    /*******************************************************/
    if(_phdrMLE)
       _phdrMLE->handleEventsFor(pmle);
  }

  return true;
};

/*------------------------------------------------------------------------------
| ICnrEditHandler::multiLineEdit                                               |
|                                                                              |
| Return an IMultiLineEdit "wrapper".                                          |
------------------------------------------------------------------------------*/
IMultiLineEdit* ICnrEditHandler::multiLineEdit( const IWindowHandle& handleMLE)
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  IMultiLineEdit* pmle = new IMultiLineEdit(handleMLE);
  return pmle;
#endif
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrEditHandler::endEdit                                                     |
|                                                                              |
| Default end edit event.                                                      |
------------------------------------------------------------------------------*/
bool ICnrEditHandler :: endEdit(ICnrEndEditEvent& evt)
{
  IMODTRACE_ALL("CnrEHdr::endEdit");

  /* If we setup to process the MLE, clean up */
  IContainerControl* pcnrctl = evt.container();
  IASSERT(pcnrctl);
  IMultiLineEdit* pmle = pcnrctl->currentEditMLE();

  if(pmle)
  {
     if(_phdrMLE)
       _phdrMLE->stopHandlingEventsFor(pmle);
#ifdef IC_PMWIN
     if (pcnrctl->ppd->pmCompatible)
       delete pmle;
#endif
     pcnrctl->setEditMLE(0);
     pcnrctl->setEditColumn(0);
     pcnrctl->setEditObject(0);
  }
  return true;
};

/*------------------------------------------------------------------------------
| ICnrEditHandler::handleEventsFor                                             |
|                                                                              |
| Attaches the handler to the container control passed in the argument.        |
------------------------------------------------------------------------------*/
ICnrEditHandler& ICnrEditHandler :: handleEventsFor(
                                                 IContainerControl *container )
{
  IASSERTPARM(container != 0);
  IHandler::handleEventsFor(container);
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrEditHandler::stopHandlingEventsFor                                       |
|                                                                              |
| Detaches the handler from the container control passed in the argument.      |
------------------------------------------------------------------------------*/
ICnrEditHandler& ICnrEditHandler :: stopHandlingEventsFor(
                                                IContainerControl *container )
{
  IASSERTPARM(container != 0);
  IHandler::stopHandlingEventsFor(container);
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrEditHandler::handleEventsFor                                             |
------------------------------------------------------------------------------*/
IHandler& ICnrEditHandler :: handleEventsFor( IWindow *window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
    return *this;
}


/*------------------------------------------------------------------------------
| ICnrEditHandler::stopHandlingEventsFor                                       |
------------------------------------------------------------------------------*/
IHandler& ICnrEditHandler :: stopHandlingEventsFor( IWindow *window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
    return *this;
}


/*------------------------------------------------------------------------------
| ICnrEditHandler::setMLEHandler                                               |
------------------------------------------------------------------------------*/
void ICnrEditHandler::setMLEHandler(IHandler* pMLEHandler)
{
  _phdrMLE = pMLEHandler;
}

