// Revision: 07 1.9.1.3 source/ui/cnr/icnrevt.cpp, container, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrevt.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrevt.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags

#include <icnrobj.hpp>
#include <icnrevt.hpp>
#include <icnrctl.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNREVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICnrEvent::~ICnrEvent                                                        |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrEvent :: ~ICnrEvent      ( ) {}

/*------------------------------------------------------------------------------
| ICnrEnterEvent::~ICnrEnterEvent                                              |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrEnterEvent :: ~ICnrEnterEvent () {}

/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::~ICnrEmphasisEvent                                        |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrEmphasisEvent :: ~ICnrEmphasisEvent  ( ) {}

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::~ICnrQueryDeltaEvent                                    |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrQueryDeltaEvent :: ~ICnrQueryDeltaEvent ( ) {}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::~ICnrScrollEvent                                            |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrScrollEvent :: ~ICnrScrollEvent ( ) {}

/*------------------------------------------------------------------------------
| ICnrHelpEvent::~ICnrHelpEvent                                                |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
ICnrHelpEvent :: ~ICnrHelpEvent ( ) {}

/*------------------------------------------------------------------------------
| ICnrEvent::ICnrEvent                                                         |
------------------------------------------------------------------------------*/
ICnrEvent :: ICnrEvent( const IControlEvent& evt)
        : IControlEvent(evt) {}

/*------------------------------------------------------------------------------
| ICnrEnterEvent::ICnrEnterEvent                                               |
------------------------------------------------------------------------------*/
ICnrEnterEvent ::  ICnrEnterEvent( const IControlEvent& evt)
        : ICnrEvent(evt)  {}

/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::ICnrEmphasisEvent                                         |
------------------------------------------------------------------------------*/
ICnrEmphasisEvent :: ICnrEmphasisEvent( const IControlEvent& evt)
        : ICnrEvent(evt)  {}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::ICnrEmphasisEvent                                         |
/   Motif specific
------------------------------------------------------------------------------*/
ICnrEmphasisEvent :: ICnrEmphasisEvent( IControlEvent& evt,
                                        IContainerObject* obj,
                                        bool acquiring, 
                                        IContainerObject::Emphasis emphasis)
        : ICnrEvent(evt),
          pObject(obj),
          acquired(acquiring),
          evtEmphasis(emphasis)
{}

/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::isAcquired                                                |
/   Motif specific
------------------------------------------------------------------------------*/
bool ICnrEmphasisEvent :: isAcquired() const
{ 
   return (bool) ((PNOTIFYRECORDEMPHASIS) (void*)parameter2())->fAcquired ; 
}
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::ICnrQueryDeltaEvent                                     |
------------------------------------------------------------------------------*/
ICnrQueryDeltaEvent :: ICnrQueryDeltaEvent( const IControlEvent& evt)
        : ICnrEvent(evt)
{
  ITRACE_MOTIF_NOP();
}

/*------------------------------------------------------------------------------
| ICnrHelpEvent::ICnrHelpEvent                                                 |
------------------------------------------------------------------------------*/
ICnrHelpEvent :: ICnrHelpEvent( const IControlEvent& chpevt)
        : ICnrEvent(chpevt) {}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::ICnrScrollEvent                                             |
------------------------------------------------------------------------------*/
ICnrScrollEvent :: ICnrScrollEvent( const IControlEvent& chpevt)
        : ICnrEvent(chpevt) {}

/*------------------------------------------------------------------------------
| ICnrEvent::containerId                                                       |
|                                                                              |
| Returns the control identifier for the container.                            |
------------------------------------------------------------------------------*/
unsigned long ICnrEvent :: containerId() const
{
  return  parameter1().number1();
}

/*-----------------------------------------------------------------------------
| ICnrEnterEvent Functions                                                    |
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| ICnrEnterEvent::enterPressed                                                 |
|                                                                              |
| Determine if the enter key was pressed (as opposed to a mouse double-click). |
------------------------------------------------------------------------------*/
bool ICnrEnterEvent :: enterPressed() const
{
  return ((PNOTIFYRECORDENTER)(void*)parameter2())->fKey;
}

/*------------------------------------------------------------------------------
| ICnrEnterEvent::validObject                                                  |
|                                                                              |
| Return true if the event occurred over an object.                            |
------------------------------------------------------------------------------*/
bool ICnrEnterEvent :: validObject() const
{
#ifdef IC_PMWIN
  return (((PNOTIFYRECORDENTER)(void*)parameter2())->pRecord !=0);
#endif
#ifdef IC_MOTIF
  // Container widget only gets defaultActionCallback if you double-click
  // over an IconGadget
  return true;
#endif
}

/*------------------------------------------------------------------------------
| ICnrEnterEvent::object                                                       |
|                                                                              |
| Return the object over which the Enter occurred.                             |
------------------------------------------------------------------------------*/
IContainerObject* ICnrEnterEvent :: object()  const
{
  IMiniCnrRecord* prec = (IMiniCnrRecord*)((PNOTIFYRECORDENTER)
                                                 (void*)parameter2())->pRecord;
  return IObjFromRec(prec);
}

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent Functions                                                |
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::atTopDelta                                              |
|                                                                              |
| Return true if the object that represents the delta value scrolls into view  |
| at the top of the client area.                                               |
------------------------------------------------------------------------------*/
bool ICnrQueryDeltaEvent :: atTopDelta() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return ((((PNOTIFYDELTA)(void*)parameter2())->fDelta &
                                           CMA_DELTATOP)) ? true : false;
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::atBottomDelta                                           |
|                                                                              |
| Returns true if the object that represents the delta value  scrolls into     |
| view at the bottom of the client area.                                       |
------------------------------------------------------------------------------*/
bool ICnrQueryDeltaEvent :: atBottomDelta() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return ((((PNOTIFYDELTA)(void*)parameter2())->fDelta &
                                           CMA_DELTABOT)? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::atFirstObject                                           |
|                                                                              |
| Returns true if the container has scrolled to the first object in the        |
| container.                                                                   |
------------------------------------------------------------------------------*/
bool ICnrQueryDeltaEvent :: atFirstObject() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return ((((PNOTIFYDELTA)(void*)parameter2())->fDelta &
                                           CMA_DELTAHOME)? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrQueryDeltaEvent::atLastObject                                            |
|                                                                              |
| Returns true if the container has scrolled to the last object in the         |
| container.                                                                   |
------------------------------------------------------------------------------*/
bool ICnrQueryDeltaEvent :: atLastObject() const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
  return ((((PNOTIFYDELTA)(void*)parameter2())->fDelta & CMA_DELTAEND)? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}


/*------------------------------------------------------------------------------
| ICnrEmphasisEvent Functions                                                  |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::object                                                    |
|                                                                              |
| Return the object which has changed emphasis.                                |
------------------------------------------------------------------------------*/
IContainerObject* ICnrEmphasisEvent :: object()  const
{
  IMiniCnrRecord* prec = (IMiniCnrRecord*)((PNOTIFYRECORDEMPHASIS)
                                                  (void*)parameter2())->pRecord;
  return IObjFromRec(prec);
}

/*------------------------------------------------------------------------------
| ICnrEmphasisEvent::changed                                                   |
|                                                                              |
| Determine it the passed emphasis has changed.                                |
------------------------------------------------------------------------------*/
bool ICnrEmphasisEvent :: changed(IContainerObject::Emphasis emphasis) const
{
  unsigned long ulEmphasisChanged = ((PNOTIFYRECORDEMPHASIS)
                                           (void*)parameter2())->fEmphasisMask;
  bool bEmphasisChanged(false) ;

  switch(emphasis)
  {
     case IContainerObject::inuse    :
        if(ulEmphasisChanged & CRA_INUSE)
           bEmphasisChanged = true;
        break;

     case IContainerObject::selected :
        if(ulEmphasisChanged & CRA_SELECTED)
           bEmphasisChanged = true;
        break;

     case IContainerObject::cursored :
        if(ulEmphasisChanged & CRA_CURSORED)
           bEmphasisChanged = true;
        break;
  }

  return bEmphasisChanged ;
};

/*------------------------------------------------------------------------------
| ICnrScrollEvent Functions                                                    |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| ICnrScrollEvent::amount                                                      |
|                                                                              |
| Returns the amount the window has scrolled (in pixels).                      |
------------------------------------------------------------------------------*/
long ICnrScrollEvent :: amount() const
{
  // ITRACE_MOTIF_NOTYET();
#ifdef IC_PMWIN
  return ((PNOTIFYSCROLL)(void*)parameter2())->lScrollInc;
#endif
#ifdef IC_MOTIF
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::isLeftDetails                                               |
|                                                                              |
| Return true if the scrolling occurred in the left (or only) side of the      |
| details view.                                                                |
------------------------------------------------------------------------------*/
bool ICnrScrollEvent :: isLeftDetails() const
{
#ifdef IC_PMWIN
  return ((((PNOTIFYSCROLL)(void*)parameter2())->fScroll &
                                           CMA_LEFT)? true : false);
#endif
#ifdef IC_MOTIF
  return true;
#endif
}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::isRightDetails                                              |
|                                                                              |
| Returns true if the scrolling occurred in the right side of a split details  |
| view.                                                                        |
------------------------------------------------------------------------------*/
bool ICnrScrollEvent :: isRightDetails() const
{
#ifdef IC_PMWIN
  return ((((PNOTIFYSCROLL)(void*)parameter2())->fScroll &
                                           CMA_RIGHT)? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::isHorizontal                                                |
|                                                                              |
| Return true if scrolling horizontally.                                       |
------------------------------------------------------------------------------*/
bool ICnrScrollEvent :: isHorizontal() const
{
  // ITRACE_MOTIF_NOTYET();
#ifdef IC_PMWIN
  return ((((PNOTIFYSCROLL)(void*)parameter2())->fScroll &
                                           CMA_HORIZONTAL)? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrScrollEvent::isVertical                                                  |
|                                                                              |
| Return true if scrolling vertically.                                         |
------------------------------------------------------------------------------*/
bool ICnrScrollEvent :: isVertical() const
{
  // ITRACE_MOTIF_NOTYET();
#ifdef IC_PMWIN
  return ((((PNOTIFYSCROLL)(void*)parameter2())->fScroll &
                                           CMA_VERTICAL) ? true : false);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}

/*------------------------------------------------------------------------------
| ICnrHelpEvent Functions                                                      |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| ICnrHelpEvent::object                                                        |
|                                                                              |
| Return the object over which help is requested.                              |
------------------------------------------------------------------------------*/
IContainerObject* ICnrHelpEvent :: object()  const
{
   IContainerControl* pcnrctl = (IContainerControl*)controlWindow();
   IContainerObject* pcnrobj = 0;

  /******************************************************************/
  /* The "object" that is passed in a CN_HELP event is the cursored */
  /* object.  If we have an mle open, we want to return the mle     */
  /* object so long as we are editing an object, otherwise return   */
  /* the cursored object.                                           */
  /******************************************************************/

  if(pcnrctl)
  {
     pcnrobj = pcnrctl->currentEditObject();
     // if there was no cursored object, then -1 was passed as parm2.
     IMiniCnrRecord *recordFromEvent = 0;
     if ( (void*)(this->parameter2()) != (void*)(-1) )
       recordFromEvent = (IMiniCnrRecord*)(void*)(this->parameter2());

     if (!pcnrobj && !pcnrctl->currentEditMLE() && recordFromEvent)
       pcnrobj = IObjFromRec(recordFromEvent);

  }

  return pcnrobj;
}

/*------------------------------------------------------------------------------
| ICnrHelpEvent::column                                                        |
|                                                                              |
| Returns the column that was being edited if help was requested within a      |
| multiple-line entry field; otherwise zero is returned.                       |
------------------------------------------------------------------------------*/
IContainerColumn* ICnrHelpEvent :: column()  const
{
  IContainerControl* pcnrctl = (IContainerControl*)controlWindow();
  if(pcnrctl)
    return pcnrctl->currentEditColumn();
  else
    return 0;
}
