// Revision: 18 1.11.1.8 source/ui/cnr/icnrobj0.cpp, container, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrobj0.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrobj.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINSTDDRAG
#define INCL_WINSTDCNR
extern "C" {
  #include <string.h>
  #include <stdio.h>
}

#include <icnrrec.hpp>  // Must be first for OS flags

#include <ireslib.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <istring.hpp>
#include <icnrobj.hpp>
#include <icnrctl.hpp>
#include <icconst.h>
#include <icnrfilt.hpp>


#ifdef IC_MOTIF
  #include <icnrctlm.hpp>
  #include <icong.h>
  #include <imstring.hpp>
  #include <ibmpstat.hpp>
#endif

#include <icnrobjp.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNROBJ0_CPP_
  #include <ipagetun.h>
#endif



/*------------------------------------------------------------------------------
| IContainerObject::initialize                                                 |
|                                                                              |
| Common constructor initialization                                            |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject::initialize()
{

   this->ppd = ::new ICnrObjPrivateData(this);

   this->pcnrobjClParent = 0;
   this->pbase = new(this) IMiniCnrRecord();

   this->pbase->hptrIcon  = 0;
   this->pbase->ptlIcon.x = 0;
   this->pbase->ptlIcon.y = 0;

   #ifdef IC_PMWIN
   #ifdef IC_NOBINARYCOMP
     this->pbase->pRecordIcon = (IHandle::Value)(this->pbase->hptrIcon);
   #endif
   #endif

   // Check to make sure that we were created with operator new
   if (this->pbase->cb != sizeof(IMiniCnrRecord))
   {
      ITHROWLIBRARYERROR(IC_CNROBJ_INVALID_CREATE,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
   }

   this->pbase->flRecordAttr = CRA_RECORDREADONLY | CRA_DROPONABLE;

// Ewald claimed have to set flClState this way under Motif to prevent
// setIcon() from being invisible until call to setRefreshOn()
#ifdef IC_MOTIF
   this->flClState = ok;      //ER:
#endif
#ifdef IC_PMWIN
   this->flClState |= ok;
#endif

   this->ulClUseCount = 0;
#ifdef IC_PMWIN
   this->delayFree = false;
#endif
   return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Default constructor with no arguments.                                       |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject()
{
   IMODTRACE_ALL("CnrObj:CnrObj()");
   initialize();

   return;
}

/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Create an container object using an IString for text and an IPointerHandle   |
| for the icon.                                                                |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject(const IString& strIconName,
                                     const IPointerHandle& ptrhIconInit)
{
   IMODTRACE_ALL("CnrObj:CnrObj(str,ptrh)");
   initialize();

   this->pbase->strIcon = strIconName;

#ifdef IC_PMWIN
#ifdef IC_NOBINARYCOMP
   this->pbase->cRecordText = (char*)(this->pbase->strIcon);
#endif
#endif //IC_PMWIN

   setRefreshOff();
   if(ptrhIconInit)
     setIcon(ptrhIconInit);
   setRefreshOn();

   return;
}


/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Create an container object by providing an IString for text and an           |
| IResourceId for the icon.                                                    |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject(const IString& strIconName,
                                     const IResourceId& residIcon)
{
   IMODTRACE_ALL("CnrObj:CnrObj(str,resid)");
   initialize();

   this->pbase->strIcon = strIconName;

#ifdef IC_PMWIN
#ifdef IC_NOBINARYCOMP
   this->pbase->cRecordText = (char*)(this->pbase->strIcon);
#endif
#endif //IC_PMWIN

   setRefreshOff();
   if(residIcon)
     setIcon(residIcon);
   setRefreshOn();

   return;
}

/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Create a container object by providing an IString for text and an unsigned   |
| long that is used as a resource ID.                                          |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject(const IString& strIconName,
                                     unsigned long ResidIcon)
{
   IMODTRACE_ALL("CnrObj:CnrObj(str,ulResid)");
   initialize();

   this->pbase->strIcon = strIconName;

#ifdef IC_PMWIN
#ifdef IC_NOBINARYCOMP
   this->pbase->cRecordText = (char*)(this->pbase->strIcon);
#endif
#endif

   setRefreshOff();
   if(ResidIcon)
     setIcon(ResidIcon);
   setRefreshOn();

   return;
}

/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Create a container object by providing resource IDs for both the text and    |
| the icon.                                                                    |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject(const IResourceId& residIconName,
                                     const IResourceId& residIcon)
{
   IMODTRACE_ALL("CnrObj:CnrObj(resid,resid)");
   initialize();

   setRefreshOff();
   if(residIcon)
      setIcon(residIcon) ;

   if(residIconName)
     setIconText(residIconName);
   setRefreshOn();

   return;
}


/*------------------------------------------------------------------------------
| IContainerObject::IContainerObject                                           |
|                                                                              |
| Copy Constructor.                                                            |
------------------------------------------------------------------------------*/
IContainerObject :: IContainerObject (const IContainerObject& cnrobjOld)
{
   IMODTRACE_ALL("CnrObj:CnrObj(&)");
   initialize();

   IMiniCnrRecord* precOld    = IRecFromObj(&cnrobjOld);
   this->pbase->flRecordAttr  = precOld->flRecordAttr;
   this->pbase->flRecordAttr &=
                  (unsigned long)(~(CRA_CURSORED | CRA_INUSE | CRA_SELECTED));
   this->pbase->ptlIcon       = precOld->ptlIcon;

   if (cnrobjOld.ppd) {
      ppd->hptrInUseIcon   = (cnrobjOld.ppd)->hptrInUseIcon ;
      ppd->hptrRegularIcon = (cnrobjOld.ppd)->hptrRegularIcon ;
   }

   setRefreshOff();
   setIcon(cnrobjOld.icon());
   setIconText(cnrobjOld.iconText());
   setRefreshOn();
}

/*------------------------------------------------------------------------------
| IContainerObject::operator =                                                 |
|                                                                              |
| assignment operator                                                          |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: operator = (const IContainerObject& cnrObject)
{
   if (this != &cnrObject) {
      IMiniCnrRecord* precOld    = IRecFromObj(&cnrObject);
      this->pbase->flRecordAttr  = precOld->flRecordAttr;
      this->pbase->flRecordAttr &=
                     (unsigned long)(~(CRA_CURSORED | CRA_INUSE | CRA_SELECTED));
      this->pbase->ptlIcon       = precOld->ptlIcon;

      if (ppd) {
         ::delete ppd ;
         ppd = ::new ICnrObjPrivateData(this) ;
      }
      if (cnrObject.ppd) {
         ppd->hptrInUseIcon   = (cnrObject.ppd)->hptrInUseIcon ;
         ppd->hptrRegularIcon = (cnrObject.ppd)->hptrRegularIcon ;
      }

      setRefreshOff();
      setIcon(cnrObject.icon());
      setIconText(cnrObject.iconText());
      setRefreshOn();
   }

   return (*this) ;

} // operator =

/*------------------------------------------------------------------------------
| IContainerObject::newRecord                                                  |
|                                                                              |
| Implementation of operator new.                                              |
------------------------------------------------------------------------------*/
void* IContainerObject :: newRecord(size_t size)
{
  IMiniCnrRecord* pcnrrec;
  IContainerObject* pcnrobj;

#ifdef IC_PMWIN
  pcnrrec = (IMiniCnrRecord*)(void*)
                IContainerControl::hwndAllocation().sendEvent(
                                                           CM_ALLOCRECORD,
 #ifdef IC_NOBINARYCOMP
                                                           MPFROMLONG(size+48),  //DAB hack
 #else
                                                           MPFROMLONG(size),
 #endif
                                                           MPFROMLONG(1L));
#endif //IC_PMWIN

#ifdef IC_MOTIF
  pcnrrec = allocateRecords(1, size);
#endif

  if(pcnrrec==0)
  {
     ITHROWGUIERROR2("CM_ALLOCRECORD",
                          IBaseErrorInfo::outOfMemory,
                          IException::recoverable);
  }
  pcnrrec->cb = sizeof(IMiniCnrRecord);
  pcnrobj = IObjFromRec(pcnrrec);

  if(size > sizeof(IContainerObject))
     pcnrobj->flClState = subclassed;

  return(pcnrobj);
}

/*------------------------------------------------------------------------------
| IContainerObject::deleteRecord                                               |
|                                                                              |
| Implementation of operator delete.                                           |
------------------------------------------------------------------------------*/
void IContainerObject :: deleteRecord(void* pcnrobj)
{
#ifdef IC_PMWIN
if (!(((IContainerObject*)pcnrobj)->isDelayedFree()))
{
  /********************************************************************/
  /* Call Container control to free the MINIRECORDCORE and associated */
  /* memory that was allocated for this object via CM_ALLOCRECORD     */
  /* during the construction of this object.                          */
  /********************************************************************/
  void* pcnrrec = IRecFromObj((IContainerObject*)pcnrobj);
  int fSuccess = (int) IContainerControl::hwndAllocation().sendEvent(
                                   CM_FREERECORD,
                                   MPFROMP(&pcnrrec),
                                   MPFROMSHORT(1));

  if (fSuccess == -1)
  {
    ITHROWGUIERROR("CM_FREERECORD");
  }
}
#endif

#ifdef IC_MOTIF
    void* pcnrrec = IRecFromObj((IContainerObject*)pcnrobj);
    delete [] pcnrrec;
#endif
}

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| allocateRecords                                                              |
------------------------------------------------------------------------------*/
IMiniCnrRecord* IContainerObject::allocateRecords(unsigned short numRecords,
                                                  long size)
{
  IMiniCnrRecord* pHead = 0;
  IMiniCnrRecord* pLead = 0;
  IMiniCnrRecord* pTrail = 0;

  //Add in the size of the IMiniCnrRecord.
  size += sizeof(IMiniCnrRecord);
  if (numRecords > 1)
  {
    for (int i = 0; i < numRecords; i++)
    {
      pLead = (IMiniCnrRecord*)new char[size];
      if (pTrail)
      {
        pTrail->preccNextRecord = pLead;
      }
      else
      {
        pHead = pLead;
      }
      pLead->cb = sizeof(IMiniCnrRecord);
      pTrail = pLead;
    }
    pLead->preccNextRecord = 0;
  }
  else
  {
    pHead = (IMiniCnrRecord*)new char[size];
    pHead->cb = sizeof(IMiniCnrRecord);
    pHead->preccNextRecord = 0;
  }
  return pHead;
}
#endif // IC_MOTIF

/*------------------------------------------------------------------------------
| IContainerObject::setCnrObjIcon                                              |
|                                                                              |
| Sets an object's icon.                                                       |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  setCnrObjIcon(const IPointerHandle& ptrh)
{
#ifdef IC_PMWIN
   this->pbase->hptrIcon = ptrh;
#ifdef IC_NOBINARYCOMP
  this->pbase->pRecordIcon = (IHandle::Value)(this->pbase->hptrIcon);
#endif

   if(isRefreshOn())
     refresh();
#endif //IC_PMWIN

#ifdef IC_MOTIF
   Arg args[5];
   int n = 0;

   this->pbase->hptrIcon = ptrh;

   XtSetArg(args[n], XmNsmallIconPixmap, ptrh); n++;
   XtSetArg(args[n], XmNlargeIconPixmap, ptrh); n++;

   Pixmap shapeMask =
     IBitmapStaticPtr::shapeMaskFor(ptrh);
   if (shapeMask)
   {
     XtSetArg(args[n], XmNsmallIconMask, shapeMask); n++;
     XtSetArg(args[n], XmNlargeIconMask, shapeMask); n++;
   }

   ContainedInListType::Cursor cursor(ppd->containedInList);
   for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
   {
      XtSetValues(cursor.element()->iconGadget, args, n); 
      cursor.element()->containerKey->ppd->resetIconSize(*this) ;
   }
//   if(isRefreshOn())
//     refresh();
#endif //IC_MOTIF
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setIcon                                                    |
|                                                                              |
| Sets an object's icon.                                                       |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  setIcon(const IPointerHandle& ptrh)
{
   ppd->hptrRegularIcon = ptrh ;

   setCnrObjIcon(ptrh) ;

   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setIcon                                                    |
|                                                                              |
| Sets an object's icon via an resource id.                                    |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  setIcon(const IResourceId& resid)
{
   setIcon(resid.resourceLibrary().loadIcon(resid));
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setIcon                                                    |
|                                                                              |
| Sets an object's icon via an resource id.                                    |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  setIcon(unsigned long ulId)
{
   IResourceId resid(ulId);
   if(ulId)
     setIcon(resid.resourceLibrary().loadIcon(resid));
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setInUseIcon                                               |
|                                                                              |
| Sets an object's in use icon using a pointer handle.                         |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setInUseIcon(const IPointerHandle& ptrh)
{
   ppd->hptrInUseIcon = ptrh ;
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setInUseIcon                                               |
|                                                                              |
| Sets an object's in use icon using a resource id.                            |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setInUseIcon(const IResourceId& resid)
{
   setInUseIcon(resid.resourceLibrary().loadIcon(resid));
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setInUseIcon                                               |
|                                                                              |
| Sets an object's in use icon using a resource id.                            |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setInUseIcon(unsigned long ulId)
{
   if(ulId) {
      IResourceId resid(ulId);
      setInUseIcon(resid.resourceLibrary().loadIcon(resid));
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setIconText                                                |
|                                                                              |
| Indicate the text for Icon View.  Refresh if necessary.                      |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setIconText(const IString& strIconNew)
{
  this->pbase->strIcon = strIconNew;
#ifdef IC_PMWIN
#ifdef IC_NOBINARYCOMP
  this->pbase->cRecordText = (char*)(this->pbase->strIcon);
#endif
  if(isRefreshOn())
    refresh();
#endif //IC_PMWIN

#ifdef IC_MOTIF
  IMString string(strIconNew);

  ContainedInListType::Cursor cursor(ppd->containedInList);
  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    XtVaSetValues(cursor.element()->iconGadget,
                  XmNlabelString,(XmString)string,
                  NULL);

//  if(isRefreshOn())
//    refresh();
#endif  //IC_MOTIF
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setIconText                                                |
|                                                                              |
| Indicate the text for Icon View.                                             |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setIconText(const char* pszIconNew)
{
   IString strIconNew(pszIconNew);
   setIconText(strIconNew);
   return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::setIconText                                                |
|                                                                              |
| Indicate the text for Icon View from resource id.                            |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setIconText(const IResourceId& resid)
{
   IString strText = resid.resourceLibrary().loadString(resid);
   setIconText(strText);
   return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::iconText                                                   |
|                                                                              |
| Query an object's icon text.                                                 |
------------------------------------------------------------------------------*/
IString IContainerObject :: iconText() const
{
   return this->pbase->strIcon;
}

/*------------------------------------------------------------------------------
| IContainerObject::icon                                                       |
|                                                                              |
| Query an object's icon .                                                     |
------------------------------------------------------------------------------*/
IPointerHandle IContainerObject :: icon() const
{
     return ppd->hptrRegularIcon ;
}


/*------------------------------------------------------------------------------
| IContainerObject::inUseIcon                                                  |
|                                                                              |
| Query an object's in use icon .                                              |
------------------------------------------------------------------------------*/
IPointerHandle IContainerObject :: inUseIcon() const
{
  return this->ppd->hptrInUseIcon ;
}


/*------------------------------------------------------------------------------
| IContainerObject::setInUse                                                   |
|                                                                              |
| Set the object's in-use emphasis                                             |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setInUse(bool fInUse,
                                               IContainerControl* pcnrctl )
{
   if (fInUse)
   {
      setEmphasis(CRA_INUSE, fInUse, pcnrctl);
      if (ppd->hptrInUseIcon)
      {
         setCnrObjIcon(ppd->hptrInUseIcon) ;
      }
   }
   else
      removeInUse() ;
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::removeInUse                                                |
|                                                                              |
| Remove the object's in-use emphasis.                                         |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: removeInUse(
                                               IContainerControl* pcnrctl )
{
   setEmphasis(CRA_INUSE, false, pcnrctl);
   if (this->pbase->hptrIcon == ppd->hptrInUseIcon)
      setCnrObjIcon(ppd->hptrRegularIcon) ;
   return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::enableDataUpdate                                           |
|                                                                              |
| Make an object editable.                                                     |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: enableDataUpdate(
                                               bool fEnable,
                                               IContainerControl* pcnrctl)
{
   if(fEnable)
     setAttributes(CRA_RECORDREADONLY, 0, pcnrctl);
   else
     setAttributes(0, CRA_RECORDREADONLY, pcnrctl);
   return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::disableDataUpdate                                          |
|                                                                              |
| Make an object non-editable.                                                 |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: disableDataUpdate(
                                                 IContainerControl* pcnrctl)
{
   setAttributes(0, CRA_RECORDREADONLY, pcnrctl);
   return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::enableDrop                                                 |
|                                                                              |
| Allow other objects to be dropped on this object.                            |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: enableDrop(bool fEnable,
                                                 IContainerControl* pcnrctl)
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   if(fEnable)
     setAttributes(0, CRA_DROPONABLE, pcnrctl);
   else
     setAttributes(CRA_DROPONABLE, 0, pcnrctl);
   return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

/*------------------------------------------------------------------------------
| IContainerObject::disableDrop                                                |
|                                                                              |
| Prevent any object from being dropped on this object.                        |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: disableDrop( IContainerControl* pcnrctl)
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   setAttributes(CRA_DROPONABLE, 0, pcnrctl);
   return *this;
#endif
#ifdef IC_MOTIF
  return *this;
#endif
}

/*------------------------------------------------------------------------------
| IContainerObject::isDropOnAble                                               |
|                                                                              |
| Returns true if this object can accept a drop.                               |
------------------------------------------------------------------------------*/
bool IContainerObject :: isDropOnAble(IContainerControl* pcnrctl) const
{
  ITRACE_MOTIF_NOP();
#ifdef IC_PMWIN
   return isAttribute(CRA_DROPONABLE, pcnrctl);
#endif
#ifdef IC_MOTIF
  return false;
#endif
}


/*------------------------------------------------------------------------------
| IContainerObject::isInUse                                                    |
|                                                                              |
| Query if this object has in-use emphasis.                                    |
------------------------------------------------------------------------------*/
bool IContainerObject :: isInUse(IContainerControl* pcnrctl) const
{
   return isAttribute(CRA_INUSE, pcnrctl);
}


/*------------------------------------------------------------------------------
| IContainerObject::isWriteable                                                |
|                                                                              |
| Query if this object can be edited.                                          |
------------------------------------------------------------------------------*/
bool IContainerObject :: isWriteable(IContainerControl* pcnrctl) const
{
   return ( !isAttribute(CRA_RECORDREADONLY, pcnrctl) );
}

/*------------------------------------------------------------------------------
| IContainerObject::hide                                                       |
|                                                                              |
| Makes this object invisible by filtering it out of the collection.           |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject::hide(IContainerControl* pcnrctl)
{
  return show(false, pcnrctl);
}

/*------------------------------------------------------------------------------
| IContainerObject::isVisible                                                  |
|                                                                              |
| Determine if an object is filtered out of a container.                       |
------------------------------------------------------------------------------*/
bool IContainerObject::isVisible(IContainerControl* pcnrctl) const
{
  return !isAttribute(CRA_FILTERED, pcnrctl);
}


/*------------------------------------------------------------------------------
| IContainerHideFilter::IContainerHideFilter                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IContainerHideFilter ::  IContainerHideFilter ( ) {}

/*------------------------------------------------------------------------------
| IContainerHideFilter::~IContainerHideFilter                                  |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
IContainerHideFilter ::  ~IContainerHideFilter ( ) {}

/*------------------------------------------------------------------------------
| IContainerHideFilter::isMemberOf                                             |
|                                                                              |
| Determine if an object is filtered out of a container.                       |
------------------------------------------------------------------------------*/
bool IContainerHideFilter :: isMemberOf( IContainerObject* object,
                                                   IContainerControl* ) const
{
  IMiniCnrRecord* pcnrrec = IRecFromObj(object);
  if (pcnrrec->flRecordAttr & CRA_FILTERED)
     return false;
  else
     return true;
}


/*------------------------------------------------------------------------------
| IContainerObject::incrementUseCount                                          |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  incrementUseCount()
{
  ((IContainerObject*)this)->ulClUseCount++;
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::decrementUseCount                                          |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::  decrementUseCount()
{
  ((IContainerObject*)this)->ulClUseCount--;
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setBase                                                    |
|                                                                              |
| Stores the address of the real base of this object.                          |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setBase( const IMiniCnrRecord* pcnrrec)
{
  this->pbase = (IMiniCnrRecord*)pcnrrec;
  return *this;
}


/*------------------------------------------------------------------------------
| IContainerObject::setRefreshOff                                              |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setRefreshOff()
{
  //flClState |=  noRefresh;
  //return *this;
   return setRefreshOn(false) ;
}

/*------------------------------------------------------------------------------
| IContainerObject::setRefreshOn                                               |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject :: setRefreshOn(bool fOn)
{
  if ( fOn )
     flClState &= ~noRefresh;
  else
     flClState |= noRefresh;

#ifdef IC_MOTIF
     ppd->setRefreshOn(fOn) ;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::isRefreshOn                                                |
------------------------------------------------------------------------------*/
bool IContainerObject :: isRefreshOn() const
{
  return ((flClState&noRefresh) ? false : true);
}

/*------------------------------------------------------------------------------
| IContainerObject::setClosed                                                  |
|                                                                              |
| Removes the open emphasis in all containers.                                 |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::setClosed()
{
  flClState &= ~opened;
  removeInUse();
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerObject::setOpen                                                    |
|                                                                              |
| Sets the open emphasis.                                                      |
------------------------------------------------------------------------------*/
IContainerObject& IContainerObject ::setOpen(bool fOpen)
{
  if(fOpen)
  {
    flClState |= opened;
    setInUse();
    return *this;
  }
  return setClosed();
}

/*------------------------------------------------------------------------------
| IContainerObject::handleOpen                                                 |
|                                                                              |
| Called when Enter is pressed on an object or a mouse double-click occurs on  |
| an object and the IContainerControl containing the object has been provided  |
| with an ICnrHandler.                                                         |
------------------------------------------------------------------------------*/
void IContainerObject :: handleOpen ( IContainerControl* )
{
  setOpen();
}

/*------------------------------------------------------------------------------
| IContainerObject::isOpen                                                     |
|                                                                              |
| Returns true if this object is open.                                         |
------------------------------------------------------------------------------*/
bool IContainerObject :: isOpen() const
{
  return isInUse();
}

/*------------------------------------------------------------------------------
| IContainerObject::handleCursoredChange                                       |
|                                                                              |
| Called when cursored emphasis changes on an object.                          |
------------------------------------------------------------------------------*/
void IContainerObject :: handleCursoredChange ( IContainerControl*,
                                                bool )
{  }

/*------------------------------------------------------------------------------
| IContainerObject::handleSelectedChange                                       |
|                                                                              |
| Called when selection emphasis changes on an object and the IContainerControl|
| containing the object has been provided with an ICnrHandler.                 |
------------------------------------------------------------------------------*/
void IContainerObject :: handleSelectedChange ( IContainerControl*,
                                                bool )
{  }

/*------------------------------------------------------------------------------
| IContainerObject::handleInuseChange                                          |
|                                                                              |
| Called when in-use emphasis changes on an object and the IContainerControl   |
| containing the object has been provided with an ICnrHandler.                 |
------------------------------------------------------------------------------*/
void IContainerObject :: handleInuseChange ( IContainerControl*,
                                             bool )
{  }

/*------------------------------------------------------------------------------
| IContainerObject::handleTreeCollapse                                         |
|                                                                              |
| Called when this object is a parent in the tree view and the descendent      |
| objects have been collapsed.                                                 |
------------------------------------------------------------------------------*/
void IContainerObject :: handleTreeCollapse ( IContainerControl* )
{  }

/*------------------------------------------------------------------------------
| IContainerObject::handleTreeExpand                                           |
|                                                                              |
| Called when this object is a parent in the tree view and the descendent      |
| objects have been expanded.                                                  |
------------------------------------------------------------------------------*/
void IContainerObject :: handleTreeExpand ( IContainerControl* )
{  }

/*------------------------------------------------------------------------------
| IContainerObject::helpId                                                     |
|                                                                              |
| Called when an ICnrHelpEvent is dispatched to ICnrHandler.                   |
------------------------------------------------------------------------------*/
unsigned long IContainerObject :: helpId() const
{
  return 0;
}

/*------------------------------------------------------------------------------
| ICnrAllocator::ICnrAllocator                                                 |
|                                                                              |
| ICnrAllocator constructor.                                                   |
------------------------------------------------------------------------------*/
ICnrAllocator::ICnrAllocator(unsigned long num, unsigned long size)
{
  IMiniCnrRecord* pcnrRec;
  if (num)
  {
#ifdef IC_PMWIN
    pcnrRec = (IMiniCnrRecord*)(void*)
                  IContainerControl::hwndAllocation().sendEvent(
                                                  CM_ALLOCRECORD,
                                                  MPFROMLONG(size),
                                                  MPFROMLONG(num));
#endif
#ifdef IC_MOTIF
  pcnrRec = IContainerObject::allocateRecords(num, size);
#endif
    if (!pcnrRec)
    {
      ITHROWGUIERROR2("CM_ALLOCRECORD",
                      IBaseErrorInfo::outOfMemory,
                      IException::recoverable);
    }
  }
  else
  {
    ITHROWLIBRARYERROR(IC_CNRALLOCATOR_INVALIDNUMVALUE,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }
  this->pFirst     = pcnrRec;
  this->pAvailable = pcnrRec;
  this->numInitialized = 0;
}

/*------------------------------------------------------------------------------
| ICnrAllocator::~ICnrAllocator                                                |
|                                                                              |
| ICnrAllocator destructor.                                                    |
------------------------------------------------------------------------------*/
ICnrAllocator::~ICnrAllocator()
{
  void* pFreeRec = this->nextAvailable();

  while (pFreeRec)
  {
#ifdef IC_PMWIN
    int fSuccess = (int)IContainerControl::hwndAllocation().sendEvent(
                                           CM_FREERECORD,
                                           MPFROMP(&pFreeRec),
                                           MPFROMSHORT(1));
    if (fSuccess == -1)
    {
      ITHROWGUIERROR("CM_FREERECORD");
    }
#endif //IC_PMWIN
#ifdef IC_MOTIF
  delete [] pFreeRec;
#endif
    pFreeRec = this->nextAvailable();
  }
}

/*------------------------------------------------------------------------------
| ICnrAllocator::nextAvailable                                                 |
|                                                                              |
| Returns the next available item that has not been initialized.               |
------------------------------------------------------------------------------*/
void* ICnrAllocator::nextAvailable()
{
  void* pAvail = pAvailable;
  if (pAvailable)
  {
    pAvailable = ((IMiniCnrRecord*)pAvailable)->preccNextRecord;
  }
  return pAvail;
}

/*------------------------------------------------------------------------------
| ICnrAllocator::next                                                          |
|                                                                              |
| Returns the next item of the item passed in.                                 |
------------------------------------------------------------------------------*/
void* ICnrAllocator::next(void* pRec)
{
  return ((IMiniCnrRecord*)pRec)->preccNextRecord;
}

/*------------------------------------------------------------------------------
| ICnrAllocator::updateForInsert                                               |
|  Updates the state of an ICnrAllocator object after addObjects or            |
|  addObjectsAfter is called.                                                  |
------------------------------------------------------------------------------*/
ICnrAllocator& ICnrAllocator::updateForInsert()
{
  this->pFirst = this->pAvailable;
  this->numInitialized = 0;
  return *this;
}
