// Revision: 20 1.6.2.4 source/ui/cnr/icnrobjd.cpp, container, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrobjd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the memory-debug implementations of operators new and   *
*   delete for IContainerObject (see icnrobj.hpp).                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icnrobj.hpp>
#include <icconst.h>
#include <icnrrec.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>

#ifdef __DEBUG_ALLOC__

// This file provides the __DEBUG_ALLOC__ implementations of the
// operator new and delete for IContainerObject.  The non-memory-debug
// version of the same operators is provided in icnrobjn.cpp and
// icnrobjn.inl/ncnrobjn.cpp.
// This file (icnrobjd.cpp) exists so that an application can build
// with /Tm+, with or without I_NO_INLINES, and link dynamically to the
// class library DLL, without encountering an unresolved external--even
// if the class library code is built with /Tm-.
//
// This file must be compiled with /Tm+.  This compiler option is forced
// via the profile used to generate the makefile, which in turn is used
// to compile this file.

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICNROBJD_CPP_
  #include <ipagetun.h>
#endif

void* IContainerObject::operator new ( size_t size,
                                       const char* fileName,
                                       size_t lineNumber )
{
  void* p = newRecord( size );
  ITRACE_DEVELOP( IString( "CnrObj::new at: " ) +
                  IString( (unsigned long)p ).d2x() +
                  IString( " line: " ) + IString( lineNumber ) +
                  IString( " in file: " ) + IString( fileName ));
  return p;
}

void* IContainerObject::operator new ( size_t size,
                                       const char* fileName,
                                       size_t lineNumber,
                                       ICnrAllocator& allocator )
{
  void* pcnrRec = allocator.nextAvailable();
  IContainerObject* pcnrobj;

  ITRACE_DEVELOP( IString( "CnrObj::new(allocator) at: " ) +
                  IString( (unsigned long)pcnrRec ).d2x() +
                  IString( " line: " ) + IString( lineNumber ) +
                  IString( " in file: " ) + IString( fileName ));

  if ( pcnrRec )
  {
     allocator.numInitialized++;
     pcnrobj = IObjFromRec( (IMiniCnrRecord*)(void*)pcnrRec );

     if ( size > sizeof( IContainerObject ))
     {
        pcnrobj->flClState = subclassed;
     }
  }
  else
  {
     ITHROWLIBRARYERROR( IC_CNRALLOCATOR_LIMIT_EXCEEDED,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );
  }

  return pcnrobj;
}

void IContainerObject::operator delete ( void* p,
                                         const char* fileName,
                                         size_t lineNumber )
{
  ITRACE_DEVELOP( IString( "CnrObj::delete: " ) +
                  IString( (unsigned long)p ).d2x() +
                  IString( " line: " ) + IString( lineNumber ) +
                  IString( " in file: " ) + IString( fileName ));
  deleteRecord( p );
}

#else  // ! __DEBUG_ALLOC__
#ifdef IC_PMWIN
  #error Error: icnrobjd.cpp must be compiled with /Tm+. Check the makefile (or its profile) for a missing /Tm+ option.
#endif
#ifdef IC_MOTIF
  #error Error: icnrobjd.cpp must be compiled with -qheapdebug on. Check the makefile (or its profile) for a missing /Tm+ option.
#endif

#endif // __DEBUG_ALLOC__

