// Revision: 89 1.2.1.2 source/ui/cnr/icnrolst.cpp, container, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icnrolst.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of ICnrObjectSet.                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <itrace.hpp>
#include <icnrolst.hpp>
#include <iseq2.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNROLST_CPP_
  #include <ipagetun.h>
#endif



class ICnrObjectSetImp : public IVPtrSequence<IContainerObject*>
{
  
};

                                       


ICnrObjectSet::Cursor::Cursor (const ICnrObjectSet& objectList)
         : fObjectListImp(objectList.fObjectListImp),
           fObjectCursorImp(new ICnrObjectSetImp::Cursor(*fObjectListImp))  
{}
       
ICnrObjectSet::Cursor::Cursor ( const Cursor& cursor)
         : fObjectListImp(cursor.fObjectListImp),
           fObjectCursorImp(new ICnrObjectSetImp::Cursor(
                  *(ICnrObjectSetImp::Cursor*)(cursor.fObjectCursorImp))) 
{}

ICnrObjectSet::Cursor::~Cursor ( )
{
   delete (ICnrObjectSetImp::Cursor*)fObjectCursorImp;
}

ICnrObjectSet::Cursor& ICnrObjectSet::Cursor::operator=(const Cursor& cursor)
{
  if(this != &cursor)
  {
     fObjectListImp = cursor.fObjectListImp;
     delete (ICnrObjectSetImp::Cursor*)fObjectCursorImp;
     fObjectCursorImp = new ICnrObjectSetImp::Cursor(
                       *(ICnrObjectSetImp::Cursor*)(cursor.fObjectCursorImp)); 
  }
  return *this;
}

bool ICnrObjectSet::Cursor::setToFirst ()
{ 
   return ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->setToFirst();
}


bool ICnrObjectSet::Cursor::setToLast ()
{ 
   return ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->setToLast();
}

bool ICnrObjectSet::Cursor::setToNext ()
{ 
   return ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->setToNext();
}

bool ICnrObjectSet::Cursor::setToPrevious ()
{ 
   return ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->setToPrevious();
}

bool ICnrObjectSet::Cursor::isValid() const
{ 
   return ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->isValid();
}

void ICnrObjectSet::Cursor::invalidate ( )
{ 
   ((ICnrObjectSetImp::Cursor*)fObjectCursorImp)->invalidate();
}


IContainerObject* const ICnrObjectSet::Cursor::element ( ) const
{ 
   return ((const ICnrObjectSetImp::Cursor*)fObjectCursorImp)->element();
}     




ICnrObjectSet :: ICnrObjectSet() 
    : fObjectListImp(new ICnrObjectSetImp)
{}

ICnrObjectSet :: ICnrObjectSet(const ICnrObjectSet& objectSet)
    : fObjectListImp(new ICnrObjectSetImp(*objectSet.fObjectListImp))
{}

ICnrObjectSet& ICnrObjectSet::operator= (const ICnrObjectSet& objectSet)
{
   if (this != &objectSet)
   {
      if (fObjectListImp) 
         delete fObjectListImp ;
      fObjectListImp = new ICnrObjectSetImp(*objectSet.fObjectListImp) ;
   }

   return (*this) ;
}

ICnrObjectSet :: ~ICnrObjectSet() 
{
  delete fObjectListImp;   
}


bool ICnrObjectSet::add ( IContainerObject* const& object)
{
   return fObjectListImp->add(object);
}


void ICnrObjectSet::addAllFrom ( const ICnrObjectSet& objectList)
{
   fObjectListImp->addAllFrom(*(objectList.fObjectListImp));
}

void ICnrObjectSet::addAsLast ( IContainerObject* const& object)
{
   fObjectListImp->addAsLast(object);
}
 

void ICnrObjectSet::addAsFirst ( IContainerObject* const& object)
{
   fObjectListImp->addAsFirst(object);
}

void ICnrObjectSet::addAsNext  ( IContainerObject* const& object, 
                                 Cursor& cursor)
{  
   fObjectListImp->addAsNext(object, 
                             *(ICnrObjectSetImp::Cursor*)(cursor.fObjectCursorImp));
}

void ICnrObjectSet::addAtPosition ( unsigned long pos, 
                                    IContainerObject* const& object)
{
   fObjectListImp->addAtPosition(pos, object);
}

void ICnrObjectSet::addAsPrevious  ( IContainerObject* const& object, 
                                     Cursor& cursor)
{
   fObjectListImp->addAsPrevious(object, 
                                 *(ICnrObjectSetImp::Cursor*)(cursor.fObjectCursorImp));
}

bool ICnrObjectSet::allElementsDo( 
                    bool (*function) (IContainerObject* &, void*),
                    void* additionalArgument)
{
  return fObjectListImp->allElementsDo(function, additionalArgument);
}

bool ICnrObjectSet::allElementsDo   ( 
                    bool (*function) (IContainerObject* const&, void*),
                    void* additionalArgument ) const
{
  return fObjectListImp->allElementsDo(function, additionalArgument);
}

long ICnrObjectSet::compare ( const ICnrObjectSet& objectList,
            long (*compFunction)(IContainerObject* const& , 
                                 IContainerObject* const&)) const
{
  return fObjectListImp->compare(*(objectList.fObjectListImp), compFunction);
}

IContainerObject* ICnrObjectSet::elementAt   ( const Cursor& cursor) const
{
  return fObjectListImp->elementAt(*(ICnrObjectSetImp::Cursor*)(cursor.fObjectCursorImp));
}

IContainerObject* ICnrObjectSet::elementAtPosition ( unsigned long position) const
{
  return fObjectListImp->elementAtPosition(position);
}

IContainerObject* ICnrObjectSet::firstElement ( ) const
{
  return fObjectListImp->firstElement();
}

IContainerObject* ICnrObjectSet::lastElement ( ) const
{
  return fObjectListImp->lastElement();
}


bool ICnrObjectSet::isEmpty ( ) const 
{
  return fObjectListImp->isEmpty();
}

unsigned long ICnrObjectSet::position ( Cursor const& cursor ) const
{
  return fObjectListImp->position(*(const ICnrObjectSetImp::Cursor*)cursor.fObjectCursorImp);
}

unsigned long ICnrObjectSet::numberOfElements ( ) const 
{
  return fObjectListImp->numberOfElements();
}

unsigned long ICnrObjectSet::removeAll ( )
{
  return fObjectListImp->removeAll();
}

unsigned long ICnrObjectSet::removeAll ( bool (*predicateFunction)(IContainerObject* const& , void*), void* data)
{
  return fObjectListImp->removeAll(predicateFunction, data);
}

void ICnrObjectSet::removeAt ( Cursor& cursor)
{
  fObjectListImp->removeAt(*(ICnrObjectSetImp::Cursor*)cursor.fObjectCursorImp);
}

void ICnrObjectSet::removeAtPosition ( unsigned long position)
{
  fObjectListImp->removeAtPosition(position);
}

void ICnrObjectSet::removeFirst ( )
{
  fObjectListImp->removeFirst();
}

void ICnrObjectSet::removeLast ( )
{
  fObjectListImp->removeLast();
}

void ICnrObjectSet::replaceAt ( const Cursor& cursor, 
                                IContainerObject* const& object)
{
  fObjectListImp->replaceAt(*(const ICnrObjectSetImp::Cursor*)cursor.fObjectCursorImp, object);
}

void ICnrObjectSet::setToPosition ( unsigned long pos, Cursor& cursor) const
{
  fObjectListImp->setToPosition( pos, 
                                 *(ICnrObjectSetImp::Cursor*)cursor.fObjectCursorImp);
}


void ICnrObjectSet::sort ( long (*comparisonFunction)
                             (IContainerObject* const& element1, 
                             IContainerObject* const& element2))
{
  fObjectListImp->sort(comparisonFunction);
}



