// Revision: 65 1.7.1.2 source/ui/drag/idmdrsrc.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmdrsrc.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMDropSource                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <idmdrsrc.hpp>
#include <itrace.hpp>
#include <iimage.hpp>
#include <iwindow.hpp>
#include <idmimage.hpp>
#include <icoordsy.hpp>
#include <icolor.hpp>
#include <igrport.hpp>
#include <irect.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <iimagels.hpp>
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMDRSRC_CPP_
  #include <ipagetun.h>
#endif

#pragma enum(4)
#pragma pack(push,4)

class IDMDropSourceData
  {
  public:
    enum TypeOfDrag { Move, Copy, Link, None, Cursor };
    enum TypeOfImage { Bitmap, Icon, Default };

    IDMDropSourceData()  ;
    ~IDMDropSourceData() ;

    enum TypeOfDrag
      dragType;
    enum TypeOfImage
      imageType;

    int
      m_refs;
    IImageList
      imageList;
    bool
      skipEnter;
    int
      cursorShowing;
    DWORD
      cDwEffect;
    DWORD
      dwKeyState;
    int
      moveIndex,
      copyIndex,
      scutIndex,
      noIndex;
    ISize
      iconSize( HICON icon );
    IPoint
      hotSpot;
  };

#pragma enum(pop)
#pragma pack(pop)


IDMDropSourceData::IDMDropSourceData()
      : m_refs(0)
      , dragType( Cursor )
      , imageType( Default )
      , skipEnter( True )
      , cDwEffect( 0 )
      , dwKeyState( 0 )
      , hotSpot( 16, 16 )
{
} // IDMDropSourceData

IDMDropSourceData::~IDMDropSourceData()
{
}

ISize IDMDropSourceData::iconSize( HICON icon )

  {
  ICONINFO iconInfo;
  if ( GetIconInfo( icon, &iconInfo ) == FALSE )
    ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
      IBaseErrorInfo::invalidRequest,
      IException::recoverable);

  HBITMAP hSourceBitmap = iconInfo.hbmMask;
  BITMAP  iSourceBitmap;

  GetObject( hSourceBitmap, sizeof(BITMAP), (LPSTR)&iSourceBitmap);

  ISize iconSize( iSourceBitmap.bmWidth, iSourceBitmap.bmHeight);
  ITRACE_DEVELOP("Icon size is " + iconSize.asString() );

  // Clean up bitmaps created and passed from GetIconInfo
  ::DeleteObject( iconInfo.hbmMask );
  ::DeleteObject( iconInfo.hbmColor );
  return iconSize;
  }


IDMDropSource::IDMDropSource()
  : fDMDropSourceData(new IDMDropSourceData())
  {}

IDMDropSource::~IDMDropSource()
  {
  IMODTRACE_DEVELOP( "IDMDropSource::~IDMDropSource");
  if ( ( IDMDropSourceData::Bitmap == fDMDropSourceData->imageType ) ||
       ( IDMDropSourceData::Icon   == fDMDropSourceData->imageType ) )
    {
    ITRACE_DEVELOP("Release, show cursor");
    ::ShowCursor( TRUE );
    }
  if ( fDMDropSourceData )
    delete fDMDropSourceData;
  }

HRESULT __stdcall IDMDropSource::QueryInterface( REFIID riid,
                                       void **ppvoid )
  {
  IMODTRACE_DEVELOP("IDMDropSource::QueryInterface");

  *ppvoid = NULL;       // Always null our output parameters

  if ( IsEqualIID( riid, IID_IUnknown ) ||
       IsEqualIID( riid, IID_IDropSource ))
    {
    *ppvoid = (void *) this;
    AddRef();
    }
  if ( NULL != *ppvoid )
    return NOERROR;

  return ResultFromScode( E_NOINTERFACE);
  }

ULONG __stdcall IDMDropSource::AddRef( )

  {
  ULONG cRefT = ++fDMDropSourceData->m_refs;
  IMODTRACE_DEVELOP("IDMDropSource::AddRef");
  ITRACE_DEVELOP( IString("exit count == ") +
                  IString(cRefT) );
  return cRefT;
  }

ULONG __stdcall IDMDropSource::Release( )

  {
  IMODTRACE_DEVELOP("IDMDropSource::Release");
  ULONG cRefT = --fDMDropSourceData->m_refs;
  ITRACE_DEVELOP(IString("exit count == ") +
                 IString(cRefT) );
  if ( cRefT == 0 )
    {
    ITRACE_DEVELOP("IDMDropSource::Release will delete self");
    delete this;
    }

  return cRefT;
  }

HRESULT __stdcall IDMDropSource::QueryContinueDrag( BOOL fEsc,
                                          DWORD grfKeyState)
  {
  IMODTRACE_DEVELOP("IDMDropSource::QueryContinueDrag");

  // save the mouse button used for drag action
  // in case this is a drag to a non OpenClass target.
  if ((grfKeyState & MK_LBUTTON) || (grfKeyState & MK_RBUTTON))
     fDMDropSourceData->dwKeyState = grfKeyState ;

  if ( fEsc)
    {
    if ( (IDMDropSourceData::Bitmap == fDMDropSourceData->imageType ) ||
         (IDMDropSourceData::Icon   == fDMDropSourceData->imageType ) )
      {
      // The drag has been canceled, hide the image, and end the drag.
      ImageList_DragLeave( NULL );
      ImageList_EndDrag();
      }
    return ResultFromScode( DRAGDROP_S_CANCEL );
    }

  if (!(grfKeyState & MK_LBUTTON) &&
      !(grfKeyState & MK_RBUTTON))
    {
    if ( (IDMDropSourceData::Bitmap == fDMDropSourceData->imageType ) ||
         (IDMDropSourceData::Icon   == fDMDropSourceData->imageType ) )
      {
      // We have a drop, hide the image, and end the drag.
      ::ImageList_DragLeave( NULL );
      ::ImageList_EndDrag();
      }

    if ( fDMDropSourceData->cDwEffect == DROPEFFECT_NONE  )
       // dropping on an invalid drop site. make equivalent to a <cancel> so we
       // can clean up the source side
       return ResultFromScode (DRAGDROP_S_CANCEL) ;
    else
       return ResultFromScode( DRAGDROP_S_DROP );
    }

  if ( (IDMDropSourceData::Bitmap == fDMDropSourceData->imageType ) ||
       (IDMDropSourceData::Icon   == fDMDropSourceData->imageType ) )
    {
    POINT pt;
    GetCursorPos( &pt );
    IPoint cursorPos( pt );

    IPoint& hotSpot( fDMDropSourceData->hotSpot );

    if ( fDMDropSourceData->skipEnter == True )
      {
      fDMDropSourceData->skipEnter = False;
      return NOERROR;
      }

    if ( (fDMDropSourceData->cDwEffect & 0x0000000F) == 0 )
      {
      if ( fDMDropSourceData->dragType != IDMDropSourceData::Cursor )
        {
        fDMDropSourceData->dragType = IDMDropSourceData::Cursor;
        // Hide the drag image.
        ::ImageList_DragLeave( NULL );
        // If cursor is not showing then show it.
        if ( fDMDropSourceData->cursorShowing == -1 )
          fDMDropSourceData->cursorShowing = ::ShowCursor( TRUE );
        fDMDropSourceData->skipEnter = True;
        }
      return NOERROR;
      }
    else
      {
      // Make sure the to hide the cursor.
      while ( fDMDropSourceData->cursorShowing != -1 )
        fDMDropSourceData->cursorShowing = ::ShowCursor( FALSE );
      }


    if ( fDMDropSourceData->cDwEffect & DROPEFFECT_COPY )
      {
      ITRACE_DEVELOP("cDwEffect is Copy");
      if ( fDMDropSourceData->dragType != IDMDropSourceData::Copy )
        {
        ITRACE_DEVELOP("COPY");
        fDMDropSourceData->dragType = IDMDropSourceData::Copy;
        fDMDropSourceData->imageList.setActiveImage(
          fDMDropSourceData->copyIndex );
        fDMDropSourceData->imageList.switchDragImage( hotSpot, cursorPos );
        }
      }
    else if ( fDMDropSourceData->cDwEffect & DROPEFFECT_MOVE )
      {
      ITRACE_DEVELOP("cDwEffect is Move");
      if ( fDMDropSourceData->dragType != IDMDropSourceData::Move )
        {
        ITRACE_DEVELOP("Move");
        fDMDropSourceData->dragType = IDMDropSourceData::Move;
        fDMDropSourceData->imageList.setActiveImage(
          fDMDropSourceData->moveIndex );
        fDMDropSourceData->imageList.switchDragImage( hotSpot, cursorPos );
        }
      }
    else if ( fDMDropSourceData->cDwEffect & DROPEFFECT_LINK )
      {
      ITRACE_DEVELOP("cDwEffect is Link");
      if ( fDMDropSourceData->dragType != IDMDropSourceData::Link )
        {
        ITRACE_DEVELOP("Link");
        fDMDropSourceData->dragType = IDMDropSourceData::Link;
        fDMDropSourceData->imageList.setActiveImage(
          fDMDropSourceData->scutIndex );
        fDMDropSourceData->imageList.switchDragImage( hotSpot, cursorPos );
        }
      }
//    else if ( fDMDropSourceData->cDwEffect & DROPEFFECT_NONE )
    else if ( fDMDropSourceData->cDwEffect == DROPEFFECT_NONE )
      {
      ITRACE_DEVELOP("cDwEffect is None");
      if ( fDMDropSourceData->dragType != IDMDropSourceData::None )
        {
        ITRACE_DEVELOP("None");
        fDMDropSourceData->dragType = IDMDropSourceData::None;
        fDMDropSourceData->imageList.setActiveImage(
          fDMDropSourceData->noIndex );
        fDMDropSourceData->imageList.switchDragImage( hotSpot, cursorPos );
        }
      }
    ::ImageList_DragMove( (int)pt.x, (int)pt.y );
    }
  return NOERROR;
  }

HRESULT __stdcall IDMDropSource::GiveFeedback( DWORD dwEffect)

  {
  IMODTRACE_DEVELOP("IDMDropSource::GiveFeedback");

  SCODE sc = DRAGDROP_S_USEDEFAULTCURSORS;
//  if ( (IDMDropSourceData::Bitmap == fDMDropSourceData->imageType ) ||
//       (IDMDropSourceData::Icon   == fDMDropSourceData->imageType ) )
//    {
    // We can use default cursors because the cursor is not
    // shown.
    // We have to use QueryContinueDrag to draw the images because
    // Give Feedback is only called when there is a IDropTarget on the
    // other side. Therefore we just save the dwEffect and set the
    // image in QueryContinueDrag, which is called for every mouse move.
    fDMDropSourceData->cDwEffect = dwEffect;
//    }
  return sc;
  }

void IDMDropSource::setImage( IDMImage &image, const IPoint& stackHotspot )

  {
  IMODTRACE_DEVELOP("IDMDropSource::setImage");

  if ( image.bitmap() )
    {
    ITRACE_DEVELOP("Image is a bitmap");
    fDMDropSourceData->imageType = IDMDropSourceData::Bitmap;
    }
  else if ( image.pointer() )
    {
    ITRACE_DEVELOP("Image is a Icon");
    fDMDropSourceData->imageType = IDMDropSourceData::Icon;
    }
  else
    {
    fDMDropSourceData->imageType = IDMDropSourceData::Default;
    return;
    }

  if ( (fDMDropSourceData->imageType == IDMDropSourceData::Bitmap) ||
       (fDMDropSourceData->imageType == IDMDropSourceData::Icon) )
    {
    try
      {
      ISize imageOffset;
      IImageList tImageList, tCursorList;

      // Get the hotspot.
      IPoint& hotSpot = fDMDropSourceData->hotSpot;
      hotSpot = stackHotspot; // This hotspot is local
      imageOffset = hotSpot;

      HCURSOR currentCursor = ::SetCursor( NULL );
      ::SetCursor( currentCursor );

      ISize cursorSize( GetSystemMetrics( SM_CXICON),
                        GetSystemMetrics( SM_CYICON));

      int
        bIndex;

      HICON
        ditheredIcon;

      if ( fDMDropSourceData->imageType == IDMDropSourceData::Bitmap )
        {
        IGImage  theBitmap( image.bitmap() );

        // Figure out how big to make the bitmap.
        IRectangle bitmapRectangle( IPoint(0,0), ISize(theBitmap.rasterBounds().width(),
                                                       theBitmap.rasterBounds().height()) );
        // Resize bitmap.
        IImage
          theImage( theBitmap.handle() );

        theImage.grafPort()->draw( bitmapRectangle );

        theImage.grafPort()->draw( theBitmap );

        tImageList.create( bitmapRectangle.size(),
                           ILC_MASK,
                           6,
                           4 );

        // Add the add.
        HBITMAP bitmap = theImage.image()->handle();
        bIndex = tImageList.add( bitmap );

        ditheredIcon = tImageList.icon(bIndex);

        bIndex = tImageList.addIconAsSeeThru( ditheredIcon );

        ::DestroyIcon( ditheredIcon );
        }
      else if ( fDMDropSourceData->imageType == IDMDropSourceData::Icon )
        {
        ITRACE_DEVELOP("Its an icon");
        ditheredIcon = image.pointer();

        tImageList.create( fDMDropSourceData->iconSize( ditheredIcon),
                           ILC_MASK,
                           6,
                           4 );

        bIndex = tImageList.addIconAsSeeThru( ditheredIcon );
        }

      int
        t_copyIndex = 0,
        t_scutIndex = 0,
        t_noIndex = 0;


      tCursorList.create( cursorSize,
                           ILC_MASK,
                           4,
                           4 );

      int
        t_arrowIndex = tCursorList.addIcon( currentCursor );


      t_copyIndex = tCursorList.addIcon(
          IApplication::current().resourceLibrary().loadPointer(IC_DM_COPY));
      t_scutIndex = tCursorList.addIcon(
          IApplication::current().resourceLibrary().loadPointer(IC_DM_SHORTCUT));
      t_noIndex = tCursorList.addIcon(
          IApplication::current().resourceLibrary().loadPointer(IC_DM_NO));

      if ( ( t_arrowIndex == -1) ||
           ( t_copyIndex  == -1) ||
           ( t_scutIndex  == -1) ||
           ( t_noIndex    == -1) ||
           ( bIndex       == -1) )
        ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                          IBaseErrorInfo::invalidRequest,
                          IException::recoverable);

      // Make the cursors we need and add them to the image list.

      // Create the move cursor.
      HIMAGELIST newImageList = ::ImageList_Merge(
                                    tImageList.imageList(),
                                    bIndex,
                                    tCursorList.imageList(),
                                    t_arrowIndex,
                                    hotSpot.x(),
                                    hotSpot.y() );
      int width, height;

      ImageList_GetIconSize( newImageList, &width, &height );
      // Calculate the hot spot to be used when displaying the drag image.
      if ( hotSpot.x() < 0)
        hotSpot.setX( 0 );
      if ( hotSpot.y() < 0)
        hotSpot.setY( 0 );


      fDMDropSourceData->imageList.create( ISize(width, height ),
                                           ILC_MASK,
                                           4,
                                           4 );
      HICON tempIcon = ImageList_GetIcon(newImageList, 0, ILD_NORMAL );

      fDMDropSourceData->moveIndex =
        fDMDropSourceData->imageList.addIcon( tempIcon );
      ::DestroyIcon( tempIcon );

      ImageList_Destroy( newImageList );

      // Create the copy cursor.
      fDMDropSourceData->copyIndex =
          fDMDropSourceData->imageList.mergeImagesIntoDest(
              fDMDropSourceData->moveIndex,
              tCursorList,
              t_copyIndex,
              imageOffset,
              fDMDropSourceData->imageList );

      // Create the link cursor.
      fDMDropSourceData->scutIndex =
          fDMDropSourceData->imageList.mergeImagesIntoDest(
              fDMDropSourceData->moveIndex,
              tCursorList,
              t_scutIndex,
              imageOffset,
              fDMDropSourceData->imageList );

      // Create the no cursor.
      fDMDropSourceData->noIndex =
          fDMDropSourceData->imageList.mergeImagesIntoDest(
              fDMDropSourceData->moveIndex,
              tCursorList,
              t_noIndex,
              imageOffset,
              fDMDropSourceData->imageList );

      fDMDropSourceData->skipEnter = True;

#ifdef IC_TRACE_DEVELOP
      ITRACE_DEVELOP("The image list size is " +
          fDMDropSourceData->imageList.imageSize().asString() );
      ITRACE_DEVELOP("Number of images " +
          IString(fDMDropSourceData->imageList.numberOfImages()) );
#endif
      }
    catch ( IException &exp )
      {
      fDMDropSourceData->imageType = IDMDropSourceData::Default;
      }
    }
  }

unsigned long IDMDropSource::keyState()
{
   return fDMDropSourceData->dwKeyState ;
}
