// Revision: 77 1.10.1.4 source/ui/drag/idmhndlr.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmhndlr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMHandler                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
  /* define what subsections of PM headers to include */
  #define INCL_WINWINDOWMGR     // General window management
  #include <iwindefs.h>
  #ifdef IC_MOTIF
  #include <Xm/Xm.h>
  #include <cnr.h>
  #endif // IC_MOTIF
}

#include <idmhndlr.hpp>
#include <idmsrch.hpp>
#include <idmtgth.hpp>
#include <idmrendr.hpp>
#include <idmprov.hpp>
#include <ientryfd.hpp>
#include <idmefit.hpp>
#include <imle.hpp>
#include <idmmleit.hpp>
#include <icnrctl.hpp>
#include <idmcnrit.hpp>
#include <imenubar.hpp>
#include <isubmenu.hpp>
#include <idmmenit.hpp>

#include <itbar.hpp> 
#include <itbarbut.hpp>

#include <idmtbbit.hpp>
#include <idmtbrit.hpp>
#include <itrace.hpp>
#include <idmseq.hpp>

#ifdef IC_MOTIF
  #include <idmenab.hpp>
#endif

#ifdef IC_WIN
  #include <idmdptgh.hpp>
  #include <idmdragh.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMHNDLR_CPP_
  #include <ipagetun.h>
#endif

#pragma enum(4)
#pragma pack(push,4)
/*------------------------------------------------------------------------------
| IDMRendererSeq                                                               |
------------------------------------------------------------------------------*/
class IDMRendererSeq : public ISequence< IDMRenderer* > {
public:
  IDMRendererSeq ( );
 ~IDMRendererSeq ( );
};

#pragma enum(pop)
#pragma pack(pop)

/*------------------------------------------------------------------------------
| IDMRendererSeq::IDMRendererSeq                                               |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMRendererSeq :: IDMRendererSeq ( )
  : ISequence< IDMRenderer* >( 10 )
{
}

/*------------------------------------------------------------------------------
| IDMRendererSeq::~IDMRendererSeq                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMRendererSeq :: ~IDMRendererSeq ( )
{
}


/*------------------------------------------------------------------------------
| Static objects to ensure default handlers get deleted at shutdown.           |
------------------------------------------------------------------------------*/
static struct IDMHandler_Static {
static IDMSourceHandler
 *pSrcHandler;
static IDMTargetHandler
 *pTgtHandler;
static IDMItemProvider
 *pEFProvider,
 *pMLEProvider,
 *pCnrProvider,
 *pSMenuProvider,
 *pTBarProvider,
 *pTBarBtnProvider;
#ifdef IC_WIN
static IDMDropTargetHandler
 *pDropTargetHandler;
static IDMDragHandler
 *pDragHandler;
#endif

  ~IDMHandler_Static ( )
         {
         IMODTRACE_DEVELOP( "IDMHandler_Static dtor" );
         delete pSrcHandler;
         delete pTgtHandler;
         delete pEFProvider;
         delete pMLEProvider;
         delete pCnrProvider;
         delete pSMenuProvider;
         delete pTBarProvider;
         delete pTBarBtnProvider;
#ifdef IC_WIN
         // The drop target handler needs to be around as long as the process
         // delete pDropTargetHandler;
         delete pDragHandler;
#endif
         }

} statics;

IDMSourceHandler
 *IDMHandler_Static::pSrcHandler = 0;
IDMTargetHandler
 *IDMHandler_Static::pTgtHandler = 0;
IDMItemProvider
 *IDMHandler_Static::pEFProvider = 0,
 *IDMHandler_Static::pMLEProvider = 0,
 *IDMHandler_Static::pCnrProvider = 0,
 *IDMHandler_Static::pSMenuProvider = 0,
 *IDMHandler_Static::pTBarProvider = 0,
 *IDMHandler_Static::pTBarBtnProvider = 0;
#ifdef IC_WIN
IDMDropTargetHandler
 *IDMHandler_Static::pDropTargetHandler = 0;
IDMDragHandler
 *IDMHandler_Static::pDragHandler = 0;
#endif
 // IDMHandler static members:
IDMSourceHandler
 *IDMHandler::pSource = 0;
IDMTargetHandler
 *IDMHandler::pTarget = 0;
#ifdef IC_WIN
IDMDropTargetHandler
 *IDMHandler::pDropTarget = 0;
IDMDragHandler
 *IDMHandler::pDragHandler = 0;
#endif


/*------------------------------------------------------------------------------
| IDMHandler::IDMHandler                                                       |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMHandler :: IDMHandler ()
  : IHandler ()
  , pRendererSeqCl(0)
{
}

/*------------------------------------------------------------------------------
| IDMHandler::~IDMHandler                                                      |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMHandler :: ~IDMHandler ()
{
  delete this->pRendererSeqCl;
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
|                                                                              |
| Add the default source handler to the argument window.                       |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IWindow* pWin )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pWin );
#ifdef IC_WIN
  IDMHandler::defaultDragHandler().handleEventsFor( pWin );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pWin->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IEntryField* pEF )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );

#ifdef IC_WIN
  IDMHandler::defaultDragHandler().handleEventsFor( pEF );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pEF->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IMultiLineEdit* pMLE )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );

#ifdef IC_WIN
  IDMHandler::defaultDragHandler().handleEventsFor( pMLE );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pMLE->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IContainerControl* pCnr )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );

#ifdef IC_WIN
  IDMHandler::defaultDragHandler().handleEventsFor( pCnr );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pCnr->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
//void IDMHandler :: enableDragFrom ( IMenuBar* pMenuBar )
//  {
//
//#ifndef IC_WIN
  /********************************************************************/
  /* Iterate menu items for corresponding submenus                    */
  /********************************************************************/
//  IMenu::Cursor  cursor( *pMenuBar );
//  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
//    {
//      IWindowHandle hSubmenu = (IWindowHandle) pMenuBar->elementAt( cursor ).submenuHandle();
//      ITRACE_DEVELOP( "Hooking submenu->" + hSubmenu.asString() );
//      IWindow *pSubmenu = new IWindow( hSubmenu );
//          pSubmenu->setAutoDeleteObject();

//      IDMHandler::enableDragFrom( (ISubmenu *)pSubmenu );
//    }
//#endif
//  }

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
//void IDMHandler :: enableDragFrom ( ISubmenu* pSubmenu )
//{
//#ifndef IC_WIN
  /********************************************************************/
  /* Enable this submenu as a drag source, as well as setting the     */
  /* item provider.                                                   */
  /********************************************************************/
//  IDMHandler::defaultSourceHandler().handleEventsFor( pSubmenu->window() );
//  IDMHandler::setItemProvider( pSubmenu );
//#ifdef IC_MOTIF
//  IDMDragDropEnabler::enableDragFrom( pSubmenu->window()->handle() );
//#endif

//#endif
//}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
|                                                                              |
| Add the default target handler to the argument window.                       |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IWindow* pWin )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultTargetHandler().handleEventsFor( pWin );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pWin );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDropOn( pWin->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IEntryField* pEF )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultTargetHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pEF );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDropOn( pEF->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IMultiLineEdit* pMLE )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultTargetHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pMLE );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDropOn( pMLE->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IContainerControl* pCnr )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultTargetHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pCnr );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDropOn( pCnr->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IToolBar* pTBar )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
#endif

  /********************************************************************/
  /* Enable this toolbar button as a drag target, as well as setting  */
  /* the item provider.                                               */
  /********************************************************************/
  IDMHandler::defaultTargetHandler().handleEventsFor( pTBar );
  IDMHandler::setItemProvider( pTBar );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pTBar );
#endif

#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDropOn( pTBar->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
|                                                                              |
| Add the default source and target handlers to the argument window.           |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IWindow* pWin )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /*                                                                  */
  /* NOTE: initialization is called twice, once for the drop target   */
  /*       handler and once for the drag handler.  This is necessary  */
  /*       because Ole is uninitialized on WM_DESTROY for each        */
  /*       handler.  Reference counting is used here, so the actual   */
  /*       OleInitialize and OleUninitialize are called only once     */
  /*       per process.                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pWin );
  IDMHandler::defaultTargetHandler().handleEventsFor( pWin );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pWin );
  IDMHandler::defaultDragHandler().handleEventsFor( pWin );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pWin->handle() );
  IDMDragDropEnabler::enableDropOn( pWin->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IEntryField* pEF )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pEF );
  IDMHandler::defaultTargetHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pEF );
  IDMHandler::defaultDragHandler().handleEventsFor( pEF );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pEF->handle() );
  IDMDragDropEnabler::enableDropOn( pEF->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IMultiLineEdit* pMLE )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pMLE );
  IDMHandler::defaultTargetHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pMLE );
  IDMHandler::defaultDragHandler().handleEventsFor( pMLE );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pMLE->handle() );
  IDMDragDropEnabler::enableDropOn( pMLE->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IContainerControl* pCnr )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
  IDMDragHandler::initOle();
#endif

  IDMHandler::defaultSourceHandler().handleEventsFor( pCnr );
  IDMHandler::defaultTargetHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pCnr );
  IDMHandler::defaultDragHandler().handleEventsFor( pCnr );
#endif
#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pCnr->handle() );
  IDMDragDropEnabler::enableDropOn( pCnr->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IToolBarButton* pTBarBtn )
{
#ifdef IC_WIN
  /********************************************************************/
  /* OLE initialization                                               */
  /********************************************************************/
  IDMDragHandler::initOle();
  IDMDragHandler::initOle();
#endif

  /********************************************************************/
  /* Enable this toolbar button as a drag target and source, as well  */
  /* as setting the item provider.                                    */
  /********************************************************************/
  IDMHandler::defaultSourceHandler().handleEventsFor( pTBarBtn );
  IDMHandler::defaultTargetHandler().handleEventsFor( pTBarBtn );
  IDMHandler::setItemProvider( pTBarBtn );

#ifdef IC_WIN
  IDMHandler::defaultDropTargetHandler().handleEventsFor( pTBarBtn );
  IDMHandler::defaultDragHandler().handleEventsFor( pTBarBtn );
#endif

#ifdef IC_MOTIF
  IDMDragDropEnabler::enableDragFrom( pTBarBtn->handle() );
  IDMDragDropEnabler::enableDropOn( pTBarBtn->handle() );
#endif
}

/*------------------------------------------------------------------------------
| IDMHandler::defaultSourceHandler                                             |
|                                                                              |
| Return default source handler, initializing if this is the first time.       |
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMHandler :: defaultSourceHandler ( )
{
  if (!IDMHandler::pSource)
  {
    if (!statics.pSrcHandler)
      statics.pSrcHandler = new IDMSourceHandler;

    IDMHandler::pSource = statics.pSrcHandler;
  }
  return( *IDMHandler::pSource );
}

/*------------------------------------------------------------------------------
| IDMHandler::defaultTargetHandler                                             |
|                                                                              |
| Return default target handler, initializing if this is the first time.       |
------------------------------------------------------------------------------*/
IDMTargetHandler& IDMHandler :: defaultTargetHandler ( )
{
  if (!IDMHandler::pTarget)
  {
    if (!statics.pTgtHandler)
      statics.pTgtHandler = new IDMTargetHandler;

    IDMHandler::pTarget = statics.pTgtHandler;
  }
  return( *IDMHandler::pTarget );
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDMHandler::defaultDropTargetHandler                                         |
|                                                                              |
| Return default drop target handler, initializing if this is the first time.  |
------------------------------------------------------------------------------*/
IDMDropTargetHandler& IDMHandler :: defaultDropTargetHandler ( )
{
  if (!IDMHandler::pDropTarget)
  {
    if (!statics.pDropTargetHandler)
      statics.pDropTargetHandler = new IDMDropTargetHandler;

    IDMHandler::pDropTarget = statics.pDropTargetHandler;
  }
  return( *IDMHandler::pDropTarget );
}

/*------------------------------------------------------------------------------
| IDMHandler::defaultDragHandler                                         |
|                                                                              |
| Return default drag handler, initializing if this is the first time.  |
------------------------------------------------------------------------------*/
IDMDragHandler& IDMHandler :: defaultDragHandler ( )
{
  if (!IDMHandler::pDragHandler)
  {
    if (!statics.pDragHandler)
      statics.pDragHandler = new IDMDragHandler;

    IDMHandler::pDragHandler = statics.pDragHandler;
  }
  return( *IDMHandler::pDragHandler );
}
#endif

/*------------------------------------------------------------------------------
| IDMHandler::setDefaultSourceHandler                                          |
|                                                                              |
| Set default source handler pointer to point to argument handler.             |
------------------------------------------------------------------------------*/
void IDMHandler :: setDefaultSourceHandler ( IDMSourceHandler& source )
{
  IDMHandler::pSource = &source;
}

/*------------------------------------------------------------------------------
| IDMHandler::setDefaultTargetHandler                                          |
|                                                                              |
| Set default target handler pointer to point to argument handler.             |
------------------------------------------------------------------------------*/
void IDMHandler :: setDefaultTargetHandler ( IDMTargetHandler& target )
{
  IDMHandler::pTarget = &target;
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
|                                                                              |
| If the argument window doesn't already have an item provider, use            |
| appropriate default.                                                         |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IEntryField* pEF )
{
  if (!pEF->itemProvider())
  {
    if (!statics.pEFProvider)
      statics.pEFProvider = new IDMItemProviderFor< IDMEFItem >;

    pEF->setItemProvider( statics.pEFProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IMultiLineEdit* pMLE )
{
  if (!pMLE->itemProvider())
  {
    if (!statics.pMLEProvider)
      statics.pMLEProvider = new IDMItemProviderFor< IDMMLEItem >;

    pMLE->setItemProvider( statics.pMLEProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IContainerControl* pCnr )
{
  if (!pCnr->itemProvider())
  {
    if (!statics.pCnrProvider)
      statics.pCnrProvider = new IDMItemProviderFor< IDMCnrItem >;

    pCnr->setItemProvider( statics.pCnrProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
//void IDMHandler :: setItemProvider ( ISubmenu* pSubMenu )
//{
//#ifndef IC_WIN
//  if (!pSubMenu->itemProvider())
//  {
//    if (!statics.pSMenuProvider)
//      statics.pSMenuProvider = new IDMItemProviderFor< IDMMenuItem >;
//
//    pSubMenu->setItemProvider( statics.pSMenuProvider );
//  }
//#endif
//}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IToolBar* pTBar )
{
  if (!pTBar->itemProvider())
  {
    if (!statics.pTBarProvider)
      statics.pTBarProvider = new IDMItemProviderFor< IDMToolBarItem >;

    pTBar->setItemProvider( statics.pTBarProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IToolBarButton* pTBarBtn )
{
  if (!pTBarBtn->itemProvider())
  {
         if (!statics.pTBarBtnProvider)
                statics.pTBarBtnProvider = new IDMItemProviderFor< IDMTBarButtonItem >;

         pTBarBtn->setItemProvider( statics.pTBarBtnProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::isContainerControl                                               |
|                                                                              |
| Determines if the window is a container control.                             |
------------------------------------------------------------------------------*/
bool IDMHandler :: isContainerControl(const IWindow* pCtrlWin) const

{
  /*-------------------------------------------------------------------
     Verify that we have a non-zero window pointer
   -------------------------------------------------------------------*/
  if (pCtrlWin == 0)
    return(false);

  /*-------------------------------------------------------------------
     Determine if its a container by checking its window class.
   -------------------------------------------------------------------*/
#ifdef IC_PMWIN
  char pszClassName[30];
  long lLen = IQUERYCLASSNAME(pCtrlWin->handle(),
                                sizeof(pszClassName),
                                (PSZ)pszClassName);

#ifdef IC_PM
  if (lLen > 0  &&  *pszClassName == '#')
  {
     unsigned long ulClassName = atoi(pszClassName + 1);
     if (ulClassName == (unsigned short)(unsigned long)WC_CONTAINER)
       return(true);
  }
#endif
#ifdef IC_WIN
  if ( lLen > 0 )
  {
    IString className( pszClassName);
    if ( (className == IString(WC_CONTAINER)) ||
         (className == IString(WC_NATIVECONTAINER)) )
      return(true);
  }
#endif
  return(false);
#endif // IC_PMWIN

#ifdef IC_MOTIF
  return (IDMMotif::isContainerControl(pCtrlWin)) ;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IDMHandler::numberOfRenderers                                                |
|                                                                              |
| Returns the number of renderers.                                             |
------------------------------------------------------------------------------*/
unsigned IDMHandler :: numberOfRenderers ()
{
  return( rendererCollection()->numberOfElements() );
}

/*------------------------------------------------------------------------------
| IDMHandler::renderer                                                         |
|                                                                              |
| Returns the renderer with the given position.                                |
------------------------------------------------------------------------------*/
IDMRenderer* IDMHandler :: renderer (unsigned position)
{
  IMODTRACE_DEVELOP("IDMHandler::renderer");
  ITRACE_DEVELOP("The number of renderers is " +
    IString(numberOfRenderers()) );

  return( rendererCollection()->elementAtPosition(position) );
}

/*------------------------------------------------------------------------------
| IDMHandler::replaceRenderer                                                  |
|                                                                              |
| Replaces a given IDMRenderer with another.                                   |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: replaceRenderer (unsigned position,
                                                                                                                 const IDMRenderer &replacement)
{
  rendererCollection()->removeAtPosition(position);
  rendererCollection()->addAtPosition(position, (IDMRenderer *)&replacement);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::addRenderer                                                      |
|                                                                              |
| Adds another IDMRenderer to this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: addRenderer (const IDMRenderer &newRenderer)
{
  ITRACE_DEVELOP("AddRenderer, the number of renderers is " +
    IString(numberOfRenderers()) );
  rendererCollection()->add((IDMRenderer *)&newRenderer);
  ITRACE_DEVELOP("AddRenderer, after add the number of renderers is " +
    IString(numberOfRenderers()) );
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::removeRenderer                                                   |
|                                                                              |
| Removes an IDMRenderer from this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: removeRenderer (unsigned position)
{
  rendererCollection()->removeAtPosition(position);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::removeRenderer                                                   |
|                                                                              |
| Removes an IDMRenderer from this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: removeRenderer (const IDMRenderer &rendererToRemove)
{
  /************************************************************************/
  /* Get number of renderers                                              */
  /************************************************************************/
  unsigned count = numberOfRenderers();

  /************************************************************************/
  /* Loop thru the collection and remove renderer if a match is found     */
  /************************************************************************/
  for (unsigned position = 1; (position <= count); position++)
  {
         if ((rendererCollection()->elementAtPosition(position))
                                                 == &rendererToRemove)
    {
      removeRenderer(position);
      break;
         }
  }
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::setRenderer                                                      |
|                                                                              |
| Sets the renderer for this operation.  Removes any existing renderers.       |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: setRenderer (const IDMRenderer &newRenderer)
{
  if (this->pRendererSeqCl)
  {
         delete this->pRendererSeqCl;
    this->pRendererSeqCl = 0;
  }

  IDMHandler::addRenderer(newRenderer);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::rendererCollection                                               |
|                                                                              |
| Returns a pointer to the collection of IDMRenderers.                         |
------------------------------------------------------------------------------*/
IDMRendererSeq* IDMHandler :: rendererCollection ()
{
  if (!this->pRendererSeqCl)
    this->pRendererSeqCl = new IDMRendererSeq;

  return( this->pRendererSeqCl );
}


