// Revision: 86 1.3.2.3 source/ui/drag/idmopma.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmopma.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMOperationMarshaller                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1997                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>

// Tracing functions
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// IDMOperation
#ifndef _IDMOPER_
  #include <idmoper.hpp>
#endif

// Include our classes's stuff
#ifndef _IDMOPMA_
  #include <idmopma.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMOPMA_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMOperationMarshaller::IDMOperationMarshaller                               |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMOperationMarshaller :: IDMOperationMarshaller( IDMOperation& anOperation )
  : theOperation( anOperation )
{ }

/*------------------------------------------------------------------------------
| IDMOperationMarshaller::IDMOperationMarshaller                               |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
IDMOperationMarshaller :: IDMOperationMarshaller(
                                     const IDMOperationMarshaller& aMarshaller )
  : theOperation( aMarshaller.theOperation )
{ }

/*------------------------------------------------------------------------------
| IDMOperationMarshaller::~IDMOperationMarshaller                              |
------------------------------------------------------------------------------*/
IDMOperationMarshaller :: ~IDMOperationMarshaller()
{ }

/*------------------------------------------------------------------------------
| IDMOperationMarshaller::marshall                                             |
|                                                                              |
| Create a string representation of theOperation, starting at outStr[startPos].|
| Return startPos + the length of the string representation.                   |
|                                                                              |
| The return value is calculated correctly even if outStr is 0.                |
------------------------------------------------------------------------------*/
unsigned long IDMOperationMarshaller :: marshall ( unsigned long startPos,
                                                   char* outStr ) const
{
 unsigned long newPos = startPos;
 unsigned count = theOperation.numberOfItems();
 for (unsigned i=1; i <= count; i++ )
  {
   IDMItem::Handle itemH(theOperation.item(i)) ; 
   newPos = IDMMarshall::marshall( (long)itemH->object(), newPos, outStr );
   newPos = IDMMarshall::marshall( itemH->contents(), newPos, outStr);
  }
 return newPos;
}

/*------------------------------------------------------------------------------
| IDMOperationMarshaller::unmarshall                                           |
|                                                                              |
| Initialize theOperation from a string representation, previously created     |
| using marshall.                                                              |
|                                                                              |
| Return inStr +                                                               |
| the number of characters used from the string representation.                |
------------------------------------------------------------------------------*/
const char* IDMOperationMarshaller :: unmarshall ( const char* inStr )
{
 const char* newStr = inStr;
 long result;
 IString aString;
 unsigned count = theOperation.numberOfItems();
 for (unsigned i=1; i <= count; i++ )
  {
   IDMItem::Handle itemH(theOperation.item(i)) ; 

   newStr = IDMMarshall::unmarshall( result, newStr );
   itemH->setObject ( (void*)result );

   newStr = IDMMarshall::unmarshall( aString, newStr );
   itemH->setContents( aString );
  }

 return newStr;
}
