// Revision: 94 1.9.1.4 source/ui/drag/idmprov.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmprov.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMItemProvider                                                          *
*       IDMItemProviderFor< >                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINWINDOWMGR     // General window management
  #define INCL_WINBUTTONS       // Button controls
  #define INCL_WINSTATICS       // Static controls
  #include <iwindefs.h>
#ifdef IC_MOTIF  
extern "C" {
  #include <cnr.h>
  #include <Xm/Text.h>
  #include <Xm/TextF.h>
}
#endif // IC_MOTIF
}

// Include our header
#include <idmprov.hpp>
#include <idmitem.hpp>
#include <idmevent.hpp>
#include <icnrctl.hpp>
#include <itbar.hpp>
#include <itbpriv.hpp>
#include <itrace.hpp>
#include <idmtgtrn.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMPROV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMItemProvider::IDMItemProvider                                             |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMItemProvider :: IDMItemProvider ()
{
}

/*------------------------------------------------------------------------------
| IDMItemProvider::~IDMItemProvider                                            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMItemProvider :: ~IDMItemProvider ()
{
}

/*------------------------------------------------------------------------------
| IDMItemProvider::provideSourceItems                                          |
|                                                                              |
| Called to give provider opportunity to attach derived IDMItems to the        |
| argument IDMSourceOperation object.  This function is called when a drag     |
| operation commences over a control object.  Returns true of items were       |
| provided, false otherwise.                                                   |
------------------------------------------------------------------------------*/
bool IDMItemProvider ::
        provideSourceItems(const IDMSourceOperation::Handle &sourceOperation)
{
  return( IDMItem::generateSourceItems( sourceOperation.getAlias() ) );
}

/*------------------------------------------------------------------------------
| IDMItemProvider::provideEnterSupport                                         |
|                                                                              |
| Gives the provider an opportunity to provide support for the target enter    |
| event.                                                                       |
------------------------------------------------------------------------------*/
bool IDMItemProvider :: provideEnterSupport(IDMTargetEnterEvent &event)
{
  IMODTRACE_DEVELOP("IDMItemProvider::provideEnterSupport");
  IWindow *ctrlWindow;

  /********************************************************************/
  /* Get the pointer to the target window - query container directly  */
  /********************************************************************/
  if (event.container())
    ctrlWindow = (IWindow *)event.container();
  else
    ctrlWindow = event.window();

#ifdef IC_PMWIN 
  char pszClassName[30];
  long lLen = IQUERYCLASSNAME(ctrlWindow->handle(),
                                sizeof(pszClassName),
                                (PSZ)pszClassName);

#ifdef IC_WIN
  if (lLen > 0)
#endif
#ifdef IC_PM 
  if (lLen > 0  &&  lLen <= 6  &&  *pszClassName == '#')
#endif
  {
#ifdef IC_PM
    unsigned long ulClassName = atoi(pszClassName + 1);
#endif
#endif // IC_PMWIN

    IDMTargetOperation::Handle
                        pTgtOpH = IDMTargetOperation::targetOperation();

    /******************************************************************/
    /* If the window is a container, then ...                         */
    /******************************************************************/
#ifdef IC_WIN
    IString className(pszClassName);
    if ( (className == IString(WC_CONTAINER)) ||
         (className == IString(WC_NATIVECONTAINER)) )
#endif
#ifdef IC_PM 
    if (ulClassName == (unsigned short)(unsigned long)WC_CONTAINER)
#endif
#ifdef IC_MOTIF 
    Widget containerW = ctrlWindow->handle();
//@@       if ( XmIsScrolledWindow( ctrlWindow->handle() ) )
//@@         XtVaGetValues( ctrlWindow->handle(), XmNworkWindow, containerW, NULL );
    if ( XmIsContainer( containerW) )
#endif // IC_MOTIF
    {
      /****************************************************************/
      /* If our target is a container object, we need to make sure    */
      /* that a drop is allowed on the object, iff we are over the    */
      /* object.                                                      */
      /****************************************************************/
      IContainerObject *pCnrObj = event.object();
      if (pCnrObj  &&  (!event.isDragAfter()))
      {
        if (!pCnrObj->isDropOnAble())
        {
          event.setDropIndicator( IDM::notOk );
          return( false );
        }
      }

#ifdef IC_WIN
      if (pTgtOpH->firstTimeEntered())
        {
        ITRACE_DEVELOP("Setting supported types for container control");
        /************************************************************/
        /* Get the default target renderer and set the supported    */
        /* types                                                    */
        /************************************************************/
        IDMTargetRenderer* pTgtRender = &IDMRenderer::defaultTargetRenderer();
        pTgtRender->setSupportedTypes(IDM::containerObject);

        pTgtOpH->setFirstTimeEntered(false);
        }
#endif

#ifdef IC_WIN
      // Check to see if the source Window is a container.
      long lLen = IQUERYCLASSNAME(pTgtOpH->sourceWindowHandle(),
                                      sizeof(pszClassName),
                                      (PSZ)pszClassName);

      if (lLen > 0)
      {
        IString className(pszClassName);
        if ( (className != IString(WC_CONTAINER)) &&
             (className != IString(WC_NATIVECONTAINER)) )
          // if the sourceWindow is not a container then we are out of here.
          return false;

      }
#endif
        /****************************************************************/
        /* Get pointers to the source and target container control      */
        /* objects                                                      */
        /****************************************************************/
      IContainerControl *pSrcCnr =
                        (IContainerControl *)pTgtOpH->sourceWindow();
      IContainerControl *pTgtCnr =
                        (IContainerControl *)pTgtOpH->targetWindow();

      /****************************************************************/
      /* Loop thru all the items defined per the drag operation       */
      /****************************************************************/
      unsigned count = pTgtOpH->numberOfItems();
      for (unsigned i = 1; (i <= count); i++)
      {
        IDMItem::Handle pTgtDIH(pTgtOpH->item(i)) ;

        /**************************************************************/
        /* If the operation is a move ... (IDMOperation::drag is      */
        /* treated as a move per the CUA default) ...                 */
        /**************************************************************/
        if ((pTgtOpH->operation() == IDMOperation::move) ||
            (pTgtOpH->operation() == IDMOperation::drag))
        {
          /************************************************************/
          /* Get the source item in order to get the move object      */
          /* (which we can do if we're in the same process).          */
          /* pSrcDIH will be 0 if our target is in another process    */
          /************************************************************/
          IDMItem::Handle pSrcDIH(IDMItem::sourceItemFor(pTgtDIH)) ;
          if ( pSrcDIH.valid())
          {
            /**********************************************************/
            /* Validate the move operation, and set drop indicator    */
            /* if move is not allowed ...                             */
            /**********************************************************/
            if (!(pSrcCnr->isMoveValid (
                      (IContainerObject *)pSrcDIH->object(),
                      (pTgtCnr->isTreeView() ? pTgtOpH->containerObject()
                                             : 0),
                       pTgtCnr,
                       (event.isDragAfter()  ? pTgtOpH->containerObject()
                                             : 0) )))
            {
              event.setDropIndicator( IDM::notOk );
              return( false );
            }
          }
        } /* end move validate */
      } /* end for */

      return(true );
    } /* end container if */

    /****************************************************************/
    /* If the window is a button, then ...                          */
    /****************************************************************/
#ifdef IC_PMWIN 
#ifdef IC_WIN
    if (IString(pszClassName) == IString(WC_BUTTON))
#endif
#ifdef IC_PM
    if (ulClassName == (unsigned short)(unsigned long)WC_BUTTON)
#endif
    {
      /**************************************************************/
      /* Verify that we are working with a toolbar button based     */
      /* upon a user defined event that we send to the target       */
      /* window.                                                    */
      /**************************************************************/
      bool bRc =
              pTgtOpH->targetWindow()->sendEvent( IC_UM_TOOLBAR,
                                                  IC_UM_TBAR_TARGET_ENTER )
              .asUnsignedLong();
      if (bRc)
      {
        /************************************************************/
        /* If we get to this point, we know we are dealing with a   */
        /* toolbar button.                                          */
        /*----------------------------------------------------------*/
        /* If the source and target window are the same, we want to */
        /* prevent a drop.                                          */
        /************************************************************/
        if (pTgtOpH->sourceWindowHandle() == pTgtOpH->targetWindowHandle())
        {
          event.setDropIndicator( IDM::neverOk );
          return( false );
        }

        /************************************************************/
        /* Filter the types of items that we allow to be dropped on */
        /* a toolbar button.                                        */
        /************************************************************/
        IDMItem::Handle pTgtDIH(pTgtOpH->item(1)) ; 
        IString strTypes = pTgtDIH->types();
        if ((strTypes.includes( IDM::toolBarButton ))  ||
            (strTypes.includes( IDM::menuItem ))       ||
            (strTypes.includes( IDM::bitmap )))
        {
          if (strTypes.includes( IDM::menuItem ))
          {
            ITRACE_DEVELOP("Testing drop of menu item on tool bar button");
            // If a control already exists for the id, do not allow a drop
            // of the menu item
            IWindow* parentWindow = ctrlWindow->parent();
            if ( parentWindow )
            {
              IDMItem::Handle pSrcDIH(IDMItem::sourceItemFor(pTgtDIH)) ;
              unsigned long id = (unsigned long) pSrcDIH->object();
              ITRACE_DEVELOP(IString("menu item id = ")+IString(id));
              if ( IWindow::windowWithParent (id, parentWindow ))
              {
                ITRACE_DEVELOP("Preventing drop");
                event.setDropIndicator( IDM::neverOk );
                return( false );
              }
            }
          }
          if (strTypes.includes( IDM::toolBarButton ))
          {
            ITRACE_DEVELOP("Testing drop of button on button");
            // If a control already exists for the id, do not allow a drop
            // of the tool bar button
            IWindow* parentWindow = ctrlWindow->parent();
            if ( parentWindow )
            {
              // Protect against drop of toolbar button from another process
              if (!pTgtOpH->sourceWindow())
              {
                event.setDropIndicator( IDM::neverOk );
                return( false );
              }
              ITRACE_DEVELOP(IString("tool bar button id = ") +
                             IString(pTgtOpH->sourceWindow()->id()));
              IWindow* ctrl = IWindow::windowWithParent
                              ( pTgtOpH->sourceWindow()->id(), parentWindow );
              if ( ctrl && ( ctrl->handle() != pTgtOpH->sourceWindowHandle() ))
              {
                ITRACE_DEVELOP("Preventing drop");
                event.setDropIndicator( IDM::neverOk );
                return( false );
              }
            }
          }
          return( true );
        }
        else
        {
          if ((strTypes.includes( IDM::plainText ))  &&
              (pTgtDIH->sourceName().lowerCase().includes( ".bmp" )))
          {
            return( true );
          }
          else
          {
            event.setDropIndicator( IDM::neverOk );
            return( false );
          }
        }
      }

      return( true );
    } /* end button if */
#endif // IC_PMWIN

    /****************************************************************/
    /* If the window is a static, then ...                          */
    /****************************************************************/
#ifdef IC_PMWIN 
    if (IString(pszClassName) == IString(IC_WC_IOC_CANVAS))
    {
      /**************************************************************/
      /* Verify that we are working with a toolbar based upon a     */
      /* user defined event that we send to the target window.      */
      /**************************************************************/
      bool bRc =
              pTgtOpH->targetWindow()->sendEvent( IC_UM_TOOLBAR,
                                                  IC_UM_TBAR_TARGET_ENTER )
              .asUnsignedLong();
      if (bRc)
      {
        /************************************************************/
        /* If we get to this point, we know we are dealing with a   */
        /* toolbar.                                                 */
        /*----------------------------------------------------------*/
        /* Filter the types of items that we allow to be dropped on */
        /* a toolbar button.                                        */
        /************************************************************/
        IDMItem::Handle pTgtDIH(pTgtOpH->item(1)) ;
        IString strTypes = pTgtDIH->types();
        if ((strTypes.includes( IDM::toolBarButton ))  ||
            (strTypes.includes( IDM::menuItem )))
        {
          if (strTypes.includes( IDM::menuItem ))
          {
            ITRACE_DEVELOP("Testing drop of menu item on tool bar");
            IDMItem::Handle pSrcDIH(IDMItem::sourceItemFor(pTgtDIH)) ;
            unsigned long id = (unsigned long) pSrcDIH->object();
            ITRACE_DEVELOP(IString("menu item id = ")+IString(id));
            // If a control already exists for the id, do not allow a drop
            // of the menu item
            if ( IWindow::windowWithParent (id, ctrlWindow ))
            {
              ITRACE_DEVELOP("Preventing drop");
              event.setDropIndicator( IDM::neverOk );
              return( false );
            }
          }
          if (strTypes.includes( IDM::toolBarButton ))
          {
            ITRACE_DEVELOP("Testing drop of button on tool bar");
            // Protect against drop of toolbar button from another process
            if (!pTgtOpH->sourceWindow())
            {
              event.setDropIndicator( IDM::neverOk );
              return( false );
            }
            ITRACE_DEVELOP(IString("tool bar button id = ") +
                           IString(pTgtOpH->sourceWindow()->id()));
            IWindow* ctrl = IWindow::windowWithParent
                            ( pTgtOpH->sourceWindow()->id(), ctrlWindow );
            // If a control already exists for the id, do not allow a drop
            // of the tool bar button
            if ( ctrl && ( ctrl->handle() != pTgtOpH->sourceWindowHandle() ))
            {
              ITRACE_DEVELOP("Preventing drop");
              event.setDropIndicator( IDM::neverOk );
              return( false );
            }
          }
          return( true );
        }
        else
        {
          event.setDropIndicator( IDM::neverOk );
          return( false );
        }
      }

      return( true );
    } /* end static if */
#endif // IC_PMWIN

    /******************************************************************/
    /* If the window is an MLE or entryfield, then ...                */
    /******************************************************************/
#ifdef IC_WIN
    if ((IString(pszClassName) == IString(WC_ENTRYFIELD))  ||
        (IString(pszClassName) == IString(WC_MLE)))
#endif
#ifdef IC_PM 
    if ((ulClassName == (unsigned short)(unsigned long)WC_ENTRYFIELD)  ||
        (ulClassName == (unsigned short)(unsigned long)WC_MLE))
#endif
#ifdef IC_MOTIF 
    if ( XmIsText( ctrlWindow->handle()) ||
         XmIsTextField( ctrlWindow->handle()) )
#endif
    {
      /****************************************************************/
      /* If the source and target window are the same, we want to     */
      /* prevent a drop.                                              */
      /****************************************************************/
      // from Motif code
      //IDragInfoWrapper dragInfoWrapper( *pTgtOpH );
      //if ( dragInfoWrapper.sourcePID() == IApplication::currentPID()
      //  && pTgtOpH->sourceWindowHandle() == pTgtOpH->targetWindowHandle() )
      if (pTgtOpH->sourceWindowHandle() == pTgtOpH->targetWindowHandle())
      {
        event.setDropIndicator( IDM::neverOk );
        return( false );
      }

#ifdef IC_MOTIFWIN
      if (pTgtOpH->firstTimeEntered())
        {
        ITRACE_DEVELOP("Setting supported types for edit control");
        /************************************************************/
        /* Get the default target renderer and set the supported    */
        /* types                                                    */
        /************************************************************/
        IDMTargetRenderer* pTgtRender = &IDMRenderer::defaultTargetRenderer();
        pTgtRender->setSupportedTypes(IDM::text);

        pTgtOpH->setFirstTimeEntered(false);
        }
#endif // IC_MOTIFWIN

    } /* end MLE if */

  return( true );
#ifdef IC_PMWIN 
  } // className if
#endif
  return false ;
} // provideEnterSupport

/*------------------------------------------------------------------------------
| IDMItemProvider::provideLeaveSupport                                         |
|                                                                              |
| Gives the provider an opportunity to provide support for the target leave    |
| event.                                                                       |
------------------------------------------------------------------------------*/
bool IDMItemProvider :: provideLeaveSupport(IDMTargetLeaveEvent &event)
{
  return(false);
}

/*------------------------------------------------------------------------------
| IDMItemProvider::provideHelpFor                                              |
|                                                                              |
| Gives the provider an opportunity to provide window specific drop help.      |
------------------------------------------------------------------------------*/
bool IDMItemProvider :: provideHelpFor(IDMTargetHelpEvent &event)
{
  return(false);
}

/*------------------------------------------------------------------------------
| IDMItemProvider::provideTargetItemFor                                        |
|                                                                              |
| Called to give provider opportunity to replace generic IDMItems with a       |
| derived class.  This function is called when a drop event occurs.  Base      |
| behavior is to return the argument.                                          |
------------------------------------------------------------------------------*/
IDMItem::Handle IDMItemProvider ::
                provideTargetItemFor(const IDMItem::Handle &pDragItem)
{

  return((IDMItem::Handle) pDragItem);
}

