// Revision: 06 1.5.1.3 source/ui/drag/idmtbrit.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmtbrit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMToolBarItem                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #include <iwindefs.h>
}

#include <idmtbrit.hpp>
#include <idmevent.hpp>
#include <itbar.hpp>
#include <itbarbut.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMTBRIT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMToolBarItem::IDMToolBarItem                                               |
|                                                                              |
| Constructor for toolbar target items.                                        |
------------------------------------------------------------------------------*/
IDMToolBarItem::IDMToolBarItem( const IDMItem::Handle& pTBarItem )  :
                          IDMItem(pTBarItem)
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::ctor (target)" );
}

/*------------------------------------------------------------------------------
| IDMToolBarItem::~IDMToolBarItem                                              |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMToolBarItem :: ~IDMToolBarItem( )
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::dtor" );
}

/*------------------------------------------------------------------------------
| IDMToolBarItem::targetDrop                                                   |
|                                                                              |
| Sets button bitmap and text into the target toolbar button,  or moves the    |
| toolbar button to a new position within the toolbar.                         |
------------------------------------------------------------------------------*/
bool IDMToolBarItem :: targetDrop( IDMTargetDropEvent& evt )
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::targetDrop" );

  /********************************************************************/
  /* Get the target toolbar                                           */
  /********************************************************************/
  IToolBar *pTgtTBar = (IToolBar *)evt.window();

  /********************************************************************/
  /* If type includes "menuItem" then process the menuitem link       */
  /* request.                                                         */
  /********************************************************************/
  if (types().includes( IDM::menuItem ))
  {
    /******************************************************************/
    /* Create the toolbar button (and make it the last item in the    */
    /* list for now)                                                  */
    /******************************************************************/
    IToolBarButton *pButton = new IToolBarButton( (unsigned long)object(),
                                                  pTgtTBar,
                                                  pTgtTBar );
    pButton->setAutoDeleteObject();
    pTgtTBar->addAsLast( pButton, true );

    return( true );
  } //end menuItem processing

  /********************************************************************/
  /* If type includes "toolBarButton" then process the move request   */
  /********************************************************************/
  if (types().includes( IDM::toolBarButton ))
  {
    /******************************************************************/
    /* Get the source button and the toolbar it resides in            */
    /******************************************************************/
    IToolBarButton *pSrcBtn = (IToolBarButton *)sourceWindow();
    IToolBar *pSrcTBar = (IToolBar *)pSrcBtn->parent();

    /******************************************************************/
    /* If we're in the same toolbar ...                               */
    /******************************************************************/
    if (pSrcTBar == pTgtTBar)
    {
      /******************************************************************/
      /* Move it to the end for now ...                                 */
      /******************************************************************/
      pTgtTBar->moveToLast ( pSrcBtn );
    }
#ifdef IC_PMWIN
    else
    {
      /****************************************************************/
      /* Add it to the new toolbar at the end for now and remove it   */
      /* from the old one ...                                         */
      /****************************************************************/
      pSrcTBar->remove( pSrcBtn );
      pSrcBtn->setParent( pTgtTBar );
      pSrcBtn->setOwner( pTgtTBar );
      pTgtTBar->addAsLast( pSrcBtn, true );
    }
#endif // IC_PMWIN

    return( true );
  } //end toolBarButton processing

  return( false );
}

