// Revision: 26 1.4.1.2 source/ui/extapp/icliphdr.cpp, clipboard, ioc.v400, 000302  
/*******************************************************************************
* FILE NAME: icliphdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icliphdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINCLIPBOARD
extern "C" {
   #include <iwindefs.h>
}
#include <icliphdr.hpp>
#include <iclipbrd.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <iclipimp.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICLIPHDR_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_MOTIF
  #include <Xm/CutPaste.h>
  #include <ixevtids.h>
#endif

#ifdef IC_WIN
  #define WM_RENDERALLFMTS WM_RENDERALLFORMATS
  #define WM_RENDERFMT     WM_RENDERFORMAT
#endif


#ifdef IC_MOTIF
  #define WM_RENDERFMT        (XmCR_CLIPBOARD_DATA_REQUEST + motifEventOffset)
  #define WM_DESTROYCLIPBOARD (XmCR_CLIPBOARD_DATA_DELETE + motifEventOffset)
  #define WM_RENDERALLFMTS    0  // not used on Motif
#endif  


IClipboardHandler::IClipboardHandler()
{

}

IClipboardHandler::~IClipboardHandler()
{

}

bool IClipboardHandler::dispatchHandlerEvent ( IEvent& event )
{
   if (event.eventId() == WM_RENDERALLFMTS)
   {
      IMODTRACE_DEVELOP("WM_RENDERALLFMTS");
      return renderAllFormats(event);
   }
   else if (event.eventId() == WM_RENDERFMT)
   {
      IMODTRACE_DEVELOP("WM_RENDERFMT");
      // On Windows and OS/2, parm1 is the format name.
      // On Motif, parm1 is the format handle and parm2 is the data id stored
      // with the data.  On Motif, it is the dataId and not the format name
      // that is necessary to render the data.  We acquire the
      // format on Motif just for api consistency.
      #ifdef IC_MOTIF
        IClipboard clipboard(event.window()->handle());
        IString format = clipboard.format(event.parameter1().asUnsignedLong());
      #else
        IString format = IClipboard::format(event.parameter1().asUnsignedLong());
      #endif
      return renderFormat( event, format);
   }
   else if (event.eventId() == WM_DESTROYCLIPBOARD)
   {
      IMODTRACE_DEVELOP("WM_DESTROYCLIPBOARD");
      return clipboardEmptied(event);
   }
   return false;
}

void IClipboardHandler::setDelayedData ( IEvent&        event,
                                         const char*    format,
                                         const void*    data,
                                         unsigned long  dataLength )
{
   IFUNCTRACE_DEVELOP();

   IClipboard clipboard(event.window()->handle());
   clipboard.fClipboardImp->copyDelayedData(format,
                                            data,
                                            dataLength,
                                            event);
}
