// Revision: 32 1.5.1.3 source/ui/extapp/iclipimp.hpp, clipboard, ioc.v400, 000302  
/* NOSHIP */
#ifndef _ICLIPIMP_
#define _ICLIPIMP_
/*******************************************************************************
* FILE NAME: iclipimp.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IClipboardImp                                                            *
*     IWindowsClipboardImp                                                     *
*     IOS2ClipboardImp                                                         *
*     IMotifClipboardImp                                                       *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandle.hpp>
#ifdef IC_MOTIF
  #include <X11/Xlib.h>
#endif

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IString;
class IEvent;

#pragma enum(4)
#pragma pack(push,4)

class IClipboardImpDeleteList;

class IClipboardImp
{
public:

virtual ~IClipboardImp( );

IWindowHandle
  owner ( );

// AutoOpen means that we opened the clipboard because we needed
// it open.  This is turned off when the user calls open.
bool
  isAutoOpen ( ),
  isOpen     ( );

void
  setOpen  ( bool open = true),
  setAutoOpen  ( bool open = true),
  setOwner ( const IWindowHandle& owner);

void
  open      ( ),
  autoOpen  ( ),
  close     ( ),
  autoClose ( );

// The following are derived class overrides.
virtual void
  openClipboard   ( ) = 0,
  closeClipboard  ( ) = 0,
  emptyClipboard  ( ) = 0,
  copyData        ( const char*   formatName,
                    const void*         buffer,
                    unsigned long bufferSize) = 0,
  copyDelayedData ( const char*   formatName,
                    const void*         buffer,
                    unsigned long bufferSize,
                    IEvent& event) = 0,
  copyHandle      ( const char*   formatName,
                    unsigned long dataHandle ) = 0;

virtual const void*
  retrieveData    ( const char*    formatName,
                    unsigned long& bytesReturned) = 0;

virtual unsigned long
  registerFormat  ( const char* formatName,
                    unsigned long elementSizeInBits = 8 ) = 0;

virtual bool
  clipboardHasFormat ( const char* formatName ) = 0;

virtual IString
  formatForItem   ( unsigned long itemHandle) = 0,
  formatForAtom   ( unsigned long atom ) = 0;

virtual unsigned long
  atomFromName    ( const char* name) = 0,
  itemCount  ( ) = 0;

virtual unsigned long
  nextItemHandle ( unsigned long priorItemHandle) = 0;

virtual IWindowHandle
  realOwner      ( ) = 0;

protected:
IClipboardImp (const IWindowHandle& owner  = 0);

private:


IWindowHandle
  fOwnerHandle;
bool
  fOpened,
  fAutoOpened;
};

#ifdef IC_WIN
class IWindowsClipboardImp : public IClipboardImp
{
public:
IWindowsClipboardImp ( const IWindowHandle& owner );

virtual ~IWindowsClipboardImp ( );

virtual void
  openClipboard   ( ),
  closeClipboard  ( ),
  emptyClipboard  ( ),
  copyData        ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize),
  copyDelayedData ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize,
                    IEvent& event),
  copyHandle      ( const char*   formatName,
                    unsigned long dataHandle );

virtual const void*
  retrieveData    ( const char*    formatName,
                    unsigned long& bytesReturned);

virtual unsigned long
  registerFormat  ( const char*    formatName,
                    unsigned long  elementSizeInBits = 8 );

virtual bool
  clipboardHasFormat ( const char* formatName );

virtual IString
  formatForItem   ( unsigned long itemHandle),
  formatForAtom   ( unsigned long atom );

virtual unsigned long
  atomFromName    ( const char* name);

virtual unsigned long
  itemCount  ( );

virtual unsigned long
  nextItemHandle ( unsigned long priorItemHandle);

virtual IWindowHandle
  realOwner      ( );

protected:
private:
void
  addToDeleteList ( void* memoryToDelete),
  cleanUpDeleteList ( );
IString
  publicFormat( unsigned long formatHandle );

IClipboardImpDeleteList
 *fDeleteList;
};
#endif


#ifdef IC_PM
class IOS2ClipboardImp : public IClipboardImp
{
public:
IOS2ClipboardImp ( const IWindowHandle& owner );

virtual ~IOS2ClipboardImp ( );

virtual void
  openClipboard   ( ),
  closeClipboard  ( ),
  emptyClipboard  ( ),
  copyData        ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize),
  copyDelayedData ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize,
                    IEvent& event),
  copyHandle      ( const char*   formatName,
                    unsigned long dataHandle );

virtual const void*
  retrieveData    ( const char*    formatName,
                    unsigned long& bytesReturned);

virtual unsigned long
  registerFormat  ( const char*    formatName,
                    unsigned long  elementSizeInBits = 8 );
virtual bool
  clipboardHasFormat ( const char* formatName );

virtual IString
  formatForItem   ( unsigned long itemHandle),
  formatForAtom   ( unsigned long atom );

virtual unsigned long
  atomFromName    ( const char* name);

virtual unsigned long
  itemCount  ( );

virtual unsigned long
  nextItemHandle ( unsigned long priorItemHandle);

virtual IWindowHandle
  realOwner      ( );

void
  setRealOwner   ( );

private:
IClipboardImpDeleteList
 *fDeleteList;
};
#endif

#ifdef IC_MOTIF
class IMotifClipboardImp : public IClipboardImp
{
public:

IMotifClipboardImp ( const IWindowHandle& owner );

virtual ~IMotifClipboardImp ( );

virtual void
  openClipboard   ( ),
  closeClipboard  ( ),
  emptyClipboard  ( ),
  copyData        ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize),
  copyDelayedData ( const char*   formatName,
                    const void*   buffer,
                    unsigned long bufferSize,
                    IEvent& event),
  copyHandle      ( const char*   formatName,
                    unsigned long dataHandle );

virtual const void*
  retrieveData    ( const char*    formatName,
                    unsigned long& bytesReturned);

virtual unsigned long
  registerFormat  ( const char*    formatName,
                    unsigned long  elementSizeInBits = 8 );
virtual bool
  clipboardHasFormat ( const char* formatName );

virtual IString
  formatForItem   ( unsigned long itemHandle),
  formatForAtom   ( unsigned long atom );

virtual unsigned long
  atomFromName    ( const char* name);

virtual unsigned long
  itemCount  ( );

virtual unsigned long
  nextItemHandle ( unsigned long priorItemHandle);

virtual IWindowHandle
  realOwner      ( );

protected:

private:
void
  addToDeleteList ( void* memoryToDelete),
  cleanUpDeleteList ( );
IClipboardImpDeleteList
 *fDeleteList;
long
  fStartItemId;
Display
 *fDisplay;
Window     
  fWindow;
};
#endif

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#include <iclipimp.inl>

#endif /* _ICLIPIMP_ */
