// Revision: 28 1.15.1.12 source/ui/extapp/icustbev.cpp, toolbar, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icustbev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// #define IC_TRACE_ALL

extern "C" {
  #define INCL_WINBUTTONS
  #include <iwindefs.h>
}

#include <icconst.h>
#include <ictlevt.hpp>
#include <icustbdd.hpp>
#include <icustbev.hpp>
#include <icustbut.hpp>
#include <iexcept.hpp>
#include <itrace.hpp>

#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )
  #include <ivbase.hpp>  // Needed for fGraphicContext
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICUSTBEV_CPP_
  #include <ipagetun.h>
#endif


#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )
// To ensure that this file doesn't bind the old graphics code, we
// store and IGraphicContext as an IVBase pointer so we can delete it
// in our destructor without knowing it is an IGraphicContext.  Further,
// we've moved the functions that do deal with IGraphicContext to
// graphic\iuiobs.cpp.
#endif

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
|                                                                              |
| This version of the constructor creates a grafport directly from the         |
| presentation space associated with the window, or from the system message.   |
| This grafport has the property that drawing into it immediately renders      |
| the drawing to the screen.                                                   |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent ( const IEvent& event )
   : IControlEvent ( event ),
     fImage( 0 ), //No image needed in this case.
   #ifdef IC_MOTIFPM
     fGrafPort( new IRootGrafPort( controlWindow()->presSpace() ) ),
   #endif
   #ifdef IC_WIN
     fGrafPort( new IRootGrafPort(
                IPresSpaceHandle( ((LPDRAWITEMSTRUCT)event.parameter2()
                                                     .asUnsignedLong())->hDC,
                                  this->controlHandle() ) ) ),
   #endif //IC_WIN
     fGraphicContext( 0 ),
     fManageGrafPort(true)
{
  fRectangle = controlWindow()->size();
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const ICustomButtonDrawEvent& event )
   : IControlEvent ( event ),
     fImage( 0 ),
     fGrafPort( 0 ),
     fGraphicContext( 0 ),
     fManageGrafPort(event.fManageGrafPort)
{
   if (event.fImage)
   {
      fImage = new IImage( *event.fImage );
   }
   else if (fManageGrafPort==true)
   {
  #ifdef IC_MOTIFPM
      fGrafPort =
         new IRootGrafPort( controlWindow()->presSpace() );
  #endif
  #ifdef IC_WIN
      fGrafPort =
         new IRootGrafPort( IPresSpaceHandle( ((LPDRAWITEMSTRUCT)event.parameter2().asUnsignedLong())->hDC,
                                              this->controlHandle() ) );
  #endif //IC_WIN
   }
   else
      fGrafPort = event.fGrafPort;

  fRectangle = event.fRectangle;
}



/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
|                                                                              |
| This version of the constructor stores but does not maintain the passed      |
| grafport.  This allows a memory grafport to be passed in so drawing doesn't  |
| occur on the screen. To later render the drawing to the screen, you must     |
| draw to a screen grafport.                                                   |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const IEvent& event,
                            const ISize&  size,
                            IGrafPort*    grafPort)
   : IControlEvent ( event ),
     fImage( 0 ),
     fGrafPort( grafPort ),
     fGraphicContext (0),
     fManageGrafPort(false)
{
fRectangle = size;
}
/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
|                                                                              |
| This version of the constructor creates an IImage, from which the grafport   |
| is later obtained.  This grafport has the property that drawing into it does |
| not render the drawing to the screen.  This is a memory grapfport.  To later |
| render the drawing to the screen, you must draw to a screen grafport.        |
| ICustomButtonDrawEvent::ICustomButtonDrawEvent(const IEvent& event)          |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const IEvent& event, const ISize& size,
                            const bool trueColor )
   : IControlEvent ( event ),
     fImage( 0 ),
     fGrafPort( 0 ), // Not needed in this case.
     fGraphicContext (0)
{

// Note: This ctor is being obsoleted and is no longer used by toolbar buttons.
  if (trueColor)
    {
     //Note that creating an IImage with IGImage::kTrueColor24Bit
     //allows for all user supplied images and drawing to display
     //at maximum color allowed by the runtime system.  This is
     //slower performing than a k256Color8Bit image.

     // Cast to (long) as required by IImage Contructor
     fImage = new IImage( (long)size.width(),
                          (long)size.height(),
                          IGImage::kTrueColor24Bit,
                          ICoordinateSystem::kOriginUpperLeft);
    }
  else
    {
     //An IImage created with IGImage::k256Color8Bit has much
     //better performances.  It should be used when what is
     //drawn does not exceed the 8bit color depth, 256 colors total.

     // Cast to (long) as required by IImage Contructor
     fImage = new IImage( (long)size.width(),
                          (long)size.height(),
                          IGImage::k256Color8Bit,
                          ICoordinateSystem::kOriginUpperLeft);
    }

  fRectangle = size;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::~ICustomButtonDrawEvent                              |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ~ICustomButtonDrawEvent ()
{
   if ( fImage )
   {
      delete fImage;
   }
   if ( fGrafPort && fManageGrafPort )
   {
   #ifdef IC_MOTIFPM
      controlWindow()->releasePresSpace(fGrafPort->orphanDeviceContext());
   #endif
      delete fGrafPort;
   }

#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )
   if ( fGraphicContext )
      delete (IVBase*)fGraphicContext;
#endif
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::grafPort                                             |
------------------------------------------------------------------------------*/
IGrafPort& ICustomButtonDrawEvent :: grafPort () const
{
   if ( fImage )
   {
      return *( fImage->grafPort() );
   }
   else
   {
      // Return the regular port (attached to screen DC/PS).
      return (*((ICustomButtonDrawEvent*)this)->fGrafPort);
   }
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ButtonState ICustomButtonDrawEvent :: buttonState    |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent::ButtonState ICustomButtonDrawEvent :: buttonState () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
    return buttonDown;
#endif
#ifdef IC_PM
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return buttonDown;
#endif
#ifdef IC_MOTIF
  //The button selection state is maintained in the draw data object attached
  //to the button.  If this object is present, a pointer to it is passed in
  //parameter2 of the event.  Otherwise, use parameter1 (draw up or down)
  //to determine the state.
  ICustomButtonDrawData* drawData = (ICustomButtonDrawData*)(char*)parameter2();
  if (drawData)
     return drawData->buttonState();
  else
     {
     if ( parameter1() == true)
        return buttonDown;
     }
#endif

  if ( customButton() && customButton()->isLatched() )
    return buttonLatched;
  return buttonUp;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawUp                                               |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawEvent :: drawUp () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
    return false;
#endif
#ifdef IC_PM
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return false;
#endif
#ifdef IC_MOTIF
  //The button selection state is maintained.  When the event was created,
  //the first parameter contained the button state.
  if ( parameter1() == true)
    return false;
#endif

  if ( customButton() && customButton()->isLatched() )
    return false;
  return true;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawDown                                             |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawEvent :: drawDown () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
    return true;
#endif
#ifdef IC_PM
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return true;
#endif
#ifdef IC_MOTIF
  //The button selection state is maintained.  When the event was created,
  //the first parameter contained the button state.
  if ( parameter1() == true)
    return true;
#endif
  else
    return false;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawLatched                                          |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawEvent :: drawLatched () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
    return false;
#endif
#ifdef IC_PM
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return false;
#endif
#ifdef IC_MOTIF
  //The button selection state is maintained.  When the event was created,
  //the first parameter contained the button state.
  if ( parameter1() == true)
    return false;
#endif

  if ( customButton() && customButton()->isLatched() )
    return true;
  return false;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::isButtonEnabled                                      |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawEvent :: isButtonEnabled () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_DISABLED )
    return false;
  return true;
#endif
#ifdef IC_PM
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_DISABLED )
    return false;
  return true;
#endif
#ifdef IC_MOTIF
  ICustomButtonDrawData* drawData = (ICustomButtonDrawData*)(char*)parameter2();
  if (drawData)
     return drawData->isDrawEnabled();
  else if ( customButton() && customButton()->isEnabled() )
     return true;
  return false;

#endif
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::customButton                                         |
------------------------------------------------------------------------------*/
ICustomButton* ICustomButtonDrawEvent :: customButton () const
{
  ICustomButton* customButton = dynamic_cast<ICustomButton*>(controlWindow());
  return customButton;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawingArea                                          |
------------------------------------------------------------------------------*/
IRectangle ICustomButtonDrawEvent :: drawingArea () const
{
  return fRectangle;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::setDrawingArea                                       |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent& ICustomButtonDrawEvent :: setDrawingArea
                                                  ( const IRectangle& rectangle )
{
  fRectangle = rectangle;
  return *this;
}


#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ICustomButtonDrawData::dataHandle                                            |
------------------------------------------------------------------------------*/
IWindow::DataHandle    ICustomButtonDrawData::dataHandle()
{
   // Handle for accessing ICustomButtonDrawData with IWindow::windowData
   static IWindow::DataHandle fghandle(0);
   if (!fghandle)
      fghandle = IWindow::dataHandleWithKey("ICustomButtonDrawData");
   return fghandle;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::ICustomButtonDrawData                                 |
------------------------------------------------------------------------------*/
ICustomButtonDrawData::ICustomButtonDrawData  ( )
  : fUpImage              (0),
    fUpImageDisabled      (0),
    fLatchedImage         (0),
    fLatchedImageDisabled (0),
    fDownImage            (0),
    fDownImageDisabled    (0),
    fState                (ICustomButtonDrawEvent::buttonUp),
    fSystemData           (0),
    fInvalidateReason     (0),
    fDrawEnabled          (true)
{
   IMODTRACE_ALL("ICustomButtonDrawData::ICustomButtonDrawData");
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::ICustomButtonDrawData                                 |
------------------------------------------------------------------------------*/
ICustomButtonDrawData::ICustomButtonDrawData( const ICustomButtonDrawData& another )
{
   // Private dummy function.
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
}


/*------------------------------------------------------------------------------
| ICustomButtonDrawData::operator=                                             |
------------------------------------------------------------------------------*/
ICustomButtonDrawData& ICustomButtonDrawData::operator=( const ICustomButtonDrawData& another )
{
   // Private dummy function.
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::~ICustomButtonDrawData                                |
------------------------------------------------------------------------------*/
ICustomButtonDrawData::~ICustomButtonDrawData ( )
{
   delete fUpImage;
   delete fUpImageDisabled;
   delete fLatchedImage;
   delete fLatchedImageDisabled;
   delete fDownImage;
   delete fDownImageDisabled;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::image                                                 |
------------------------------------------------------------------------------*/
IGImage* ICustomButtonDrawData::image( ICustomButtonDrawEvent::ButtonState state,
                                       bool enabledImage ) const
{
   if (state == ICustomButtonDrawEvent::buttonUp)
      return enabledImage ? fUpImage : fUpImageDisabled;
   else if (state == ICustomButtonDrawEvent::buttonDown)
      return enabledImage ? fDownImage : fDownImageDisabled;
   else
      return enabledImage ? fLatchedImage : fLatchedImageDisabled;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::adoptImage                                            |
------------------------------------------------------------------------------*/
ICustomButtonDrawData& ICustomButtonDrawData::adoptImage(ICustomButtonDrawEvent::ButtonState state,
                                                         IGImage* image,
                                                         bool enabledImage )
{
   IMODTRACE_ALL("ICustomButtonDrawData::adoptImage");
   ITRACE_ALL(IString("image=") + IString((unsigned long)image).d2x() );
   IGImage**      pimage(0);
   if (state == ICustomButtonDrawEvent::buttonUp)
   {
      ITRACE_ALL("buttonUp");
      pimage = enabledImage ? &fUpImage : &fUpImageDisabled;
   }
   else if (state == ICustomButtonDrawEvent::buttonDown)
   {
      ITRACE_ALL("buttonDown");
      pimage = enabledImage ? &fDownImage : &fDownImageDisabled;
   }
   else
   {
      ITRACE_ALL("buttonLatched");
      pimage = enabledImage ? &fLatchedImage : &fLatchedImageDisabled;
   }

   if (*pimage && (*pimage != image))
      delete *pimage;
   *pimage = image;
   return *this;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::imageHandle                                           |
------------------------------------------------------------------------------*/
IBitmapHandle ICustomButtonDrawData::imageHandle( ICustomButtonDrawEvent::ButtonState state,
                                                  bool enabledImage ) const
{
   IMODTRACE_ALL("ICustomButtonDrawData::imageHandle");
   IGImage*  pimage = this->image( state, enabledImage );
   return (pimage ? pimage->handle() : IBitmapHandle(0) );
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::isImageValid                                          |
------------------------------------------------------------------------------*/
bool  ICustomButtonDrawData::isImageValid( ICustomButtonDrawEvent::ButtonState state ) const
{
   ITRACE_ALL(IString("isImageValid(")+IString((unsigned long)state)
                  +") fUpImage="+IString((unsigned long)fUpImage)
                   +" fDownImage="+IString((unsigned long)fDownImage)
                   +" fLatchedImage="+IString((unsigned long)fLatchedImage));
   if (state == ICustomButtonDrawEvent::buttonUp)
      return (fUpImage != 0);
   else if (state == ICustomButtonDrawEvent::buttonDown)
      return (fDownImage != 0);
   else
      return (fLatchedImage != 0);
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::invalidateAll                                         |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawData::invalidateAll( unsigned long invalidateReason )
{
   IMODTRACE_ALL("ICustomButtonDrawData::invalidateAll");
   bool result(false);
   fInvalidateReason |= invalidateReason;
   if (fUpImage)
   {
      delete fUpImage;
      fUpImage = 0;
      result = true;
   }
   if (fUpImageDisabled)
   {
      delete fUpImageDisabled;
      fUpImageDisabled = 0;
      result = true;
   }
   if (fDownImage)
   {
      delete fDownImage;
      fDownImage = 0;
      result = true;
   }
   if (fDownImageDisabled)
   {
      delete fDownImageDisabled;
      fDownImageDisabled = 0;
      result = true;
   }
   if (fLatchedImage)
   {
      delete fLatchedImage;
      fLatchedImage = 0;
      result = true;
   }
   if (fLatchedImageDisabled)
   {
      delete fLatchedImageDisabled;
      fLatchedImageDisabled = 0;
      result = true;
   }

   return result;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::invalidateReason                                      |
------------------------------------------------------------------------------*/
unsigned long ICustomButtonDrawData::invalidateReason( ) const
{
   ITRACE_ALL( IString("invalidateReason=") + IString(fInvalidateReason).d2x());
   return fInvalidateReason;
}


/*------------------------------------------------------------------------------
| ICustomButtonDrawData::setButtonState                                        |
------------------------------------------------------------------------------*/
ICustomButtonDrawData& ICustomButtonDrawData::setButtonState( ICustomButtonDrawEvent::ButtonState state )
{
   IMODTRACE_ALL("ICustomButtonDrawData::setButtonState");
   ITRACE_ALL( IString("Old state=") + IString((unsigned long)fState) +
               IString(" New state=") + IString((unsigned long)state) );
   fState = state;
   return *this;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::setDrawEnabled                                        |
------------------------------------------------------------------------------*/
ICustomButtonDrawData& ICustomButtonDrawData::setDrawEnabled (bool drawEnabled)
{
   fDrawEnabled = drawEnabled;
   return *this;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::isDrawEnabled                                         |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawData::isDrawEnabled  () const
{
   return fDrawEnabled;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::buttonState                                           |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent::ButtonState ICustomButtonDrawData::buttonState( ) const
{
   return fState;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::setSystemData                                         |
------------------------------------------------------------------------------*/
ICustomButtonDrawData& ICustomButtonDrawData::setSystemData( void* data )
{
  fSystemData = data;
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawData::systemData                                            |
------------------------------------------------------------------------------*/
void*  ICustomButtonDrawData::systemData( ) const
{
  return fSystemData;
}
#endif //IC_MOTIF
