// Revision: 30 1.25.1.18 source/ui/extapp/icustbhd.cpp, toolbar, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icustbhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// #define IC_TRACE_ALL

extern "C" {
  #define INCL_WINMESSAGEMGR
  #define INCL_WINBUTTONS
  #define INCL_WININPUT
  #define INCL_WINSYS
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include <Xm/PushB.h>
  #endif
}


#include <ibmpstat.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <icoordsy.hpp>
#include <icustbdd.hpp>
#include <icustbev.hpp>
#include <icustbhd.hpp>
#include <icustbut.hpp>
#include <ifont.hpp>
#include <igimage.hpp>
#include <igraftxt.hpp>
#include <igrport.hpp>
#ifdef IC_MOTIF
  #include <isizrect.hpp>
#endif
#include <istring.hpp>
#include <istyles.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBHD_CPP_
  #include <ipagetun.h>
#endif


#define DRAW_STANDARD_BUTTON  1  // undefine to get Win95 toolbar button
                                 // appearance

static unsigned long gCxSysBorder = 0;
static unsigned long gCySysBorder = 0;

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::ICustomButtonDrawHandler                           |
------------------------------------------------------------------------------*/
ICustomButtonDrawHandler :: ICustomButtonDrawHandler ()
{
  //These borders can safely be queried once and the values used later.
  //In OS/2 and Windows, these values will not change dynamically.  They
  //cannot be set by an end user or a programmer
#ifdef IC_PMWIN
  if (gCxSysBorder == 0)
     gCxSysBorder = IQUERYSYSVALUE(SV_CXBORDER);
  if (gCySysBorder == 0)
     gCySysBorder = IQUERYSYSVALUE(SV_CYBORDER);
#endif
#ifdef IC_MOTIF
  if (gCxSysBorder == 0)
     gCxSysBorder = 1;
  if (gCySysBorder == 0)
     gCySysBorder = 1;
#endif
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::~ICustomButtonDrawHandler                          |
------------------------------------------------------------------------------*/
ICustomButtonDrawHandler :: ~ICustomButtonDrawHandler ()
{
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::dispatchHandlerEvent                               |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawHandler :: dispatchHandlerEvent ( IEvent &event )
{
  bool rc = false;

  switch ( event.eventId() )
  {
    // Check for paint message
#ifdef IC_PM
    case WM_CONTROL:
         if ( event.parameter1().number2() == BN_PAINT )
         {
           ICustomButtonDrawEvent evt(event);
           ICustomButton* button = evt.customButton();
           if (button)
           {
              if ( evt.drawDown() )
                button->sendEvent(IC_UM_DRAW_DOWN, true);
              else
                button->sendEvent(IC_UM_DRAW_DOWN, false);
           }
           if ( rc = drawButton(evt) )
           {
             event.setResult(evt.result());
           }
         }
         break;
#endif
#ifdef IC_WIN
    case WM_DRAWITEM:
         {
           ICustomButtonDrawEvent evt(event);
           ICustomButton* button = evt.customButton();
           if (button)
           {
              if ( evt.drawDown() )
                button->sendEvent(IC_UM_DRAW_DOWN, true);
              else
                button->sendEvent(IC_UM_DRAW_DOWN, false);
           }
           if ( rc = drawButton(evt) )
           {
             event.setResult(evt.result());
           }
         }
         break;
#endif
#ifdef IC_MOTIF
    case xEvent( MapNotify ):
       if ( event.parameter1().asUnsignedLong() == true )
       {  // First time the window is being shown.
          // Don't use bit gravity, so the button always repaints
          // whenever it gets resized.  Otherwise changing views
          // to a smaller button will not repaint the button.
          ITRACE_ALL("ICustomButtonDrawHandler::dispatchHandlerEvent - MapNotify");
          IWindowHandle
            hwnd( event.handle() );
          XSetWindowAttributes
            attributes;
          attributes.bit_gravity = ForgetGravity;
          XChangeWindowAttributes( XtDisplay( (Widget) hwnd ),
                                   XtWindow( (Widget) hwnd ),
                                   CWBitGravity,
                                   &attributes );

          ICustomButton* button =
             dynamic_cast<ICustomButton*>( event.dispatchingWindow() );
          if (button)
          {
             ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
                (button->windowData( ICustomButtonDrawData::dataHandle() ) );
             if (drawData)
             {
                drawData->setSystemData(0);
                button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                  false, drawData);
             }
          }
       }
       break;
    case IC_UM_MOTIF_CUST_BUT_PAINT:
       {
          // This event is used to force drawing of the button.  It is
          // dispatched whenever the image on the button is invalid but
          // is needed for display.  This event is the only way that
          // the drawButton() function gets called.
          //
          IMODTRACE_ALL("IC_UM_MOTIF_CUST_BUT_PAINT");
          ICustomButtonDrawData* drawData =
              (ICustomButtonDrawData*)(char*)event.parameter2();
          if (drawData)
          {
             // We have draw data which contains a cache of the images
             // used in the buttons.  We supply the drawButton function
             // with an event whose grafPort is actually an image that
             // we will cache away.
             IWindow* button = event.dispatchingWindow();
             if (!button)
                return false;   // should not happen
             ISize    imageSize( button->size() );
             if (imageSize == ISize())
                return false;    // 0 sized image.  No need to draw.

             // Draw the enabled button.
             drawData->setDrawEnabled( true );
             IGImage* cache = new IGImage( (long)imageSize.width(),
                                           (long)imageSize.height(),
                                           IGImage::kTrueColor24Bit,
                                           ICoordinateSystem::kOriginUpperLeft);
             ICustomButtonDrawEvent memoryEvent( event,
                                                 imageSize,
                                                 cache->grafPort() );
             // Draw into the cache image.
             rc = drawButton(memoryEvent);
             if ( rc )
             {
                 // Store the image into the drawData for use later.
                 drawData->adoptImage( drawData->buttonState(), cache, true);

                 // Draw the disabled button.
                 drawData->setDrawEnabled( false );
                 cache = new IGImage( (long)imageSize.width(),
                                      (long)imageSize.height(),
                                      IGImage::kTrueColor24Bit,
                                      ICoordinateSystem::kOriginUpperLeft);
                 ICustomButtonDrawEvent memoryEvent( event,
                                                     imageSize,
                                                     cache->grafPort() );
                 if ( drawButton(memoryEvent) )
                 {
                    // Store the image into the drawData for use later.
                    drawData->adoptImage( drawData->buttonState(), cache, false);
                 }
                 else
                 {
                    delete cache;
                 }

                 event.setResult( 1 );  // drawing complete

                 // Update the widget with the image just drawn.
                 // This is one of the places where the widget's
                 // image resources get set.
                 // Valid images can be set by:
                 // - IC_UM_MOTIF_CUST_BUT_PAINT case of
                 //   ICustomButtonDrawHandler::dispatchHandlerEvent()
                 // - ICustomButton::latch
                 // - XmCR_ARM and XmCR_DISARM cases of
                 //   ICustomButtonLatchHandler::dispatchHandlerEvent()
                 //    (these change the insensitive pixmap only)
                 // The images can be set to XmUNSPECIFIED_PIXMAP by:
                 // - ICustomButton::setLayoutDistorted function
                 // - ConfigureNotify case of ICustomButtonDrawHandler::
                 //   dispatchHandlerEvent()
                 Pixmap labelPixmap = (IBitmapHandle::Value)
                    drawData->imageHandle( drawData->buttonState(), true );
                 if (!labelPixmap)
                    labelPixmap = XmUNSPECIFIED_PIXMAP;
                 Pixmap  disabledPixmap = (IBitmapHandle::Value)
                    drawData->imageHandle( drawData->buttonState(), false);
                 if (!disabledPixmap)
                    disabledPixmap = XmUNSPECIFIED_PIXMAP;
                 if ( drawData->buttonState() == ICustomButtonDrawEvent::buttonDown )
                    XtVaSetValues( button->handle(),
                                   XmNarmPixmap, labelPixmap,
                                   XmNlabelInsensitivePixmap, disabledPixmap,
                                   0);
                 else
                    XtVaSetValues( button->handle(),
                                   XmNlabelPixmap, labelPixmap,
                                   XmNlabelInsensitivePixmap, disabledPixmap,
                                   0);
             }
             else
             {
                 delete cache;
             }
          }
          else
          {
             // No draw data available.  Dispatch the event with
             // a grafport for the widget on screen to draw directly to
             // the screen.
             ICustomButtonDrawEvent evt(event);

             if ( evt.drawingArea().size() == ISize() )
             {  // Zero-sized area to paint, so there is nothing to do.
                // Also avoid calling functions that may cause a trap.
                return false;
             }

             rc = drawButton(evt);
             if ( rc )
             {
               event.setResult( 1 );
             }
          }
       }
       break;

     case xEvent( Expose ):
       {
          // Optimization: only draw button when Expose count is 0
          XEvent* xevent = (XEvent*)(char*)event.parameter2();
          if ( xevent->xexpose.count != 0 )
             return rc;

          // Optimization: only draw button when caching turned off
          ICustomButton* button =
             dynamic_cast<ICustomButton*>(event.window() );
          if (button)
          {
             ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
                (button->windowData( ICustomButtonDrawData::dataHandle() ) );
             if (!drawData || drawData->isDrawEnabled()==false)
                return rc;
          }
          // else Expose count 0 or caching off, so fall into IC_UM_PAINT logic
       }
     case IC_UM_PAINT :
       {
          IMODTRACE_ALL("ICustomButtonDrawHandler - IC_UM_PAINT");
          ICustomButton* button =
             dynamic_cast<ICustomButton*>(event.window() );
          if (button)
          {
             ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
                (button->windowData( ICustomButtonDrawData::dataHandle() ) );
             if (drawData)
             {
                // If the image is invalid, send the draw event to
                // refresh it.
                if (!drawData->isImageValid( drawData->buttonState() ))
                {
                   unsigned long reason = drawData->invalidateReason();
                   if ((reason & IWindow::fontChanged) ||
                       (reason & IWindow::colorChanged) )
                   {
                      event.setResult(
                         button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                           false, drawData) );
                   }
                }
             }
             else
             {
                // No cache.  Always send the draw event.
                event.setResult(
                   button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                     button->isLatched(), 0 ) );
             }
             rc = event.result().asUnsignedLong() ? true : false;
          }
       }
       break;

    case xEvent( ConfigureNotify ):
       {
          IMODTRACE_ALL("ICustomButtonDrawHandler - ConfigureNotify");
          ISizeRectangle
             *sizeRect( (ISizeRectangle *)(char*)event.parameter1() );
          if ( sizeRect->sizeChanged() )
          {
             ICustomButton *button = dynamic_cast<ICustomButton*>(
                event.dispatchingWindow() );
             if (button)
             {
                ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
                   ( button->windowData( ICustomButtonDrawData::dataHandle() ) );
                if (drawData)
                {
                   // DrawData is available.  A change in size event means
                   // that the images must be redrawn.
                   if (drawData->invalidateAll( 0 ))
                   {
                       // This is one of the places where the widget's
                       // image resources get set.
                       // Valid images can be set by:
                       // - IC_UM_MOTIF_CUST_BUT_PAINT case of
                       //   ICustomButtonDrawHandler::dispatchHandlerEvent()
                       // - ICustomButton::latch
                       // - XmCR_ARM and XmCR_DISARM cases of
                       //   ICustomButtonLatchHandler::dispatchHandlerEvent()
                       //    (these change the insensitive pixmap only)
                       // The images can be set to XmUNSPECIFIED_PIXMAP by:
                       // - ICustomButton::setLayoutDistorted function
                       // - ConfigureNotify case of ICustomButtonDrawHandler::
                       //   dispatchHandlerEvent()
                       ITRACE_ALL("ConfigureNotify invalidated image(s)");
                       XtVaSetValues(
                          button->handle(),
                          XmNarmPixmap,   XmUNSPECIFIED_PIXMAP,
                          XmNlabelPixmap, XmUNSPECIFIED_PIXMAP,
                          XmNlabelInsensitivePixmap, XmUNSPECIFIED_PIXMAP,
                          0);
                   }
                   // Draw new image for current button state.
                   drawData->setSystemData ( 0 );
                   button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                     false,
                                     drawData);
                }
             }
          }
       }
       break;

    case xEvent( VisibilityNotify ):
       {
         IMODTRACE_ALL("ICustomButtonDrawHandler - VisibilityNotify");
       }
       break;
#endif  //IC_MOTIF

  }
  return rc;
}

/*------------------------------------------------------------------------------
| bool ICustomButtonDrawHandler::drawButton                                    |
------------------------------------------------------------------------------*/
bool ICustomButtonDrawHandler::drawButton ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawButton");

  //Save the event rectangle for use when drawing disabledEmphasis.
  IRectangle entireRect = event.drawingArea();

  drawBorder ( event );
  drawBackground ( event );
  drawForeground ( event );
  if ( !event.isButtonEnabled() )
  {
    //Set the event rectangle to the entire button area to draw
    //disabled emphasis over entire button.  Save the current
    //drawing area to reset it after drawing disabled emphasis
    IRectangle middleRect = event.drawingArea();
    event.setDrawingArea ( entireRect );

    drawDisabledEmphasis ( event );

    //Restore the event rectangle to the entire button area
    event.setDrawingArea ( middleRect );
  }
  return true;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawBorder                                         |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawBorder ( ICustomButtonDrawEvent& event,
                                              ISize margin )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawBorder");

  //Initialize local variables
  IGrafPort&  gp = event.grafPort();
  ICustomButton*    customButton = event.customButton();
  if (customButton)
  {
    bool              bDrawUp = !(event.drawDown() || customButton->isLatched());

    IColor  parentBgnd     (customButton->parent()->backgroundColor());
    IColor  borderColor    (IColor::kDefaultButton);
    IColor  highlightDark  (IColor::kButtonDark);
    IColor  highlightBright(IColor::kButtonLight);

    IRectangle rect = event.drawingArea();

    ITRACE_ALL(IString("rect=")  + rect.asString() +
               IString(" parentBgnd=")      + IString(parentBgnd.asRGBLong()).d2x() +
               IString(" borderColor=")     + IString(borderColor.asRGBLong()).d2x() +
               IString(" highlightDark=")   + IString(highlightDark.asRGBLong()).d2x() +
               IString(" highlightBright=") + IString(highlightBright.asRGBLong()).d2x() );

    // Clear the outermost border area to the parent background color
    // (This includes the margin and notched border area)
    unsigned long limit;
    unsigned long loop;
    IGrafBundle bundle;
    IPaint newPenPaint( parentBgnd );
    bundle.setFramePaint( newPenPaint );
    bundle.setDrawingOperation( IAttributeState::kFrame );

    //Set pen to hairline, the default is solid.  We must use a hairline
    //pen to assure pixel perfect drawing accross platforms.
    bundle.setFramePen(IPen(IPen::kHairline,(GCoordinate)gCxSysBorder));
    ILinkedGrafPort linkedPort( &gp, &bundle );
    limit = gCxSysBorder + margin.width();
    if (limit < (gCySysBorder + margin.height()) )
      limit = gCySysBorder + margin.height();
    for ( loop = 0; loop < limit; loop++ )
    {
       linkedPort.draw(IGRect2D( rect.minX() + loop,
                                 rect.minY() + loop,
                                 rect.maxX() - loop - 1,
                                 rect.maxY() - loop - 1 ));
       ITRACE_ALL( IString("clearing border rect=") +
                   IRectangle ( rect.minX() + loop,
                                rect.minY() + loop,
                                rect.maxX() - loop - 1,
                                rect.maxY() - loop - 1 ).asString() );
    }
    rect.shrinkBy ( margin );
    ITRACE_ALL( IString("margin removed rect=") + rect.asString() );

#ifdef DRAW_STANDARD_BUTTON
    // adjust the rectangle by one since we are drawing using lines.
    // This is to insure that we draw bottom/right sides inside the region,
    // and the bottom/right dimensions of the rectangle are NOT part of
    // the region (definition of IRectangle)
    rect = IRectangle( rect.minX(), rect.minY(),
                       rect.maxX()-1, rect.maxY()-1 );

    // Draw the notched border

    // For each of the draws done below, draw one beyond what we really want,
    // because the definition of hairline pens is to not draw the end point.
    bundle.setFrameColor( borderColor );
    ILinkedGrafPort linkedBorderPort( &gp, &bundle );
    //top
    linkedBorderPort.draw( IGLine2D( IGPoint2D( rect.minXMinY() +
                                                IPoint(gCxSysBorder,0) ),
                                     IGPoint2D(
                                     IPoint(1,0) +
                                     rect.maxXMinY() -
                                     IPoint(gCxSysBorder,0)) ));
    //left
    linkedBorderPort.draw( IGLine2D( IGPoint2D( rect.minXMinY() +
                                                IPoint(0,gCySysBorder) ),
                                     IGPoint2D(
                                     IPoint(0,1) +
                                     rect.minXMaxY() -
                                     IPoint(0,gCySysBorder)) ));
    //bottom
    linkedBorderPort.draw( IGLine2D( IGPoint2D( rect.minXMaxY() +
                                                IPoint(gCxSysBorder,0) ),
                                     IGPoint2D(
                                     IPoint(1,0) +
                                     rect.maxXMaxY() -
                                     IPoint(gCxSysBorder,0)) ));
    //right
    linkedBorderPort.draw( IGLine2D( IGPoint2D( rect.maxXMinY() +
                                     IPoint(0,gCySysBorder) ),
                                     IGPoint2D(
                                     IPoint(0,1) +
                                     rect.maxXMaxY() -
                                     IPoint(0,gCySysBorder)) ));

    // adjust rect for next inner line set
    rect.shrinkBy ( IPair ( gCxSysBorder, gCySysBorder ));

    // draw top and left
    if (bDrawUp)
       bundle.setFrameColor( highlightBright );
    else
       bundle.setFrameColor( highlightDark );

    IGPoint2DArray anArray(3);
    anArray[0] = rect.minXMaxY();
    anArray[1] = rect.minXMinY();

    // Draw one beyond what we really want, because the definition of hairline
    // pens is to not draw the end point.
    anArray[2] = IGPoint2D( rect.maxXMinY() + IGPoint2D(1,0) );

    gp.draw( IGPolyline2D( anArray ), bundle );

    // Draw the 3-D borders
    if (bDrawUp)
    {
       // Draw dark highlight on bottom and right
       bundle.setFrameColor( highlightDark );
       anArray[1] = rect.maxXMaxY();
       // Draw one beyond what we really want, because the definition of
       // hairline pens is to not draw the end point.
       anArray[2] = IGPoint2D( rect.maxXMinY() - IGPoint2D(0,1) );
       gp.draw( IGPolyline2D( anArray ), bundle );

       // draw the double wide dark highlight
       // type case (double) needed because of negative sign
       anArray[0] = IGPoint2D( anArray[0] +
                               IGPoint2D(gCxSysBorder,
                                         -(double)gCySysBorder) ); // lower left
       anArray[1] = IGPoint2D( anArray[1] +
                               IGPoint2D(-(double)gCxSysBorder,
                                         -(double)gCySysBorder) ); // lower right
       anArray[2] = IGPoint2D( anArray[2] +
                               IGPoint2D(-(double)gCxSysBorder,
                                         +(double)gCySysBorder) ); // upper right
       gp.draw( IGPolyline2D( anArray ), bundle );

       // adjust rect leaving only "client area" of button.  Be sure to put back
       // the ISize(1,1) we took out before drawing.
       rect = IRectangle ( rect.minX() + gCxSysBorder,
                           rect.minY() + gCySysBorder,
                           rect.maxX() - gCxSysBorder*2 + 1,
                           rect.maxY() - gCySysBorder*2 + 1 );
    }
    else
    {
       // draw down.  Done drawing.  Just need to adjust the rect.
       rect = IRectangle ( rect.minX() + gCxSysBorder,
                           rect.minY() + gCySysBorder,
                           rect.maxX() + 1,          // restoring ISize(1,1)
                           rect.maxY() + 1 );
    }
#else //DRAW_STANDARD_BUTTON
    // This code draws Win95 native appearance toolbar buttons
    IColor  highlightMiddle(IColor(IColor::kButtonMiddle));

    // draw down simply flips colors used
    if (!bDrawUp)
       {
         IColor  tempBorderColor(borderColor),
                 tempHighlightDark(highlightDark);

         borderColor = highlightBright;
         highlightDark = highlightMiddle;
         highlightBright = tempBorderColor;
         highlightMiddle = tempHighlightDark;
       }

    // draw top and left
    bundle.setFrameColor( highlightBright );

    IGPoint2DArray anArray(3);
    anArray[0] = rect.minXMaxY();      // lower left
    anArray[1] = rect.minXMinY();      // upper left
    anArray[2] = rect.maxXMinY();      // upper right
    gp.draw( IGPolyline2D( anArray ), bundle );

    // draw bottom and right
    bundle.setFrameColor( borderColor );
    anArray[1] = rect.maxXMaxY();      // lower right
    gp.draw( IGPolyline2D( anArray ), bundle );

    // adjust rect for next inner line set
    rect = IRectangle ( rect.minX() + gCxSysBorder,
                        rect.minY() + gCySysBorder,
                        rect.maxX() - gCxSysBorder,
                        rect.maxY() - gCySysBorder );

    // draw top and left
    bundle.setFrameColor( highlightMiddle );

    anArray[0] = rect.minXMaxY();      // lower left
    anArray[1] = rect.minXMinY();      // upper left
    anArray[2] = rect.maxXMinY();      // upper right
    gp.draw( IGPolyline2D( anArray ), bundle );

    // draw bottom and right
    bundle.setFrameColor( highlightDark );

    anArray[1] = rect.maxXMaxY();      // lower right
    gp.draw( IGPolyline2D( anArray ), bundle );

    // adjust rect leaving only "client area" of button.  Be sure to put back
    // the ISize(1,1) we took out before drawing.
    rect = IRectangle ( rect.minX() + gCxSysBorder,
                        rect.minY() + gCySysBorder,
                        rect.maxX() - gCxSysBorder + 1,
                        rect.maxY() - gCySysBorder + 1 );
#endif // DRAW_STANDARD_BUTTON

    // Save the new rectangle as the drawing area
    ITRACE_ALL( IString("New drawing area rect=") + rect.asString() );
    event.setDrawingArea ( rect );
  } // is a custom button
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawBackground                                     |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawBackground ( ICustomButtonDrawEvent& event )
{
  IFUNCTRACE_ALL();
  IRectangle rect = event.drawingArea();

  //Initialize local variables
  IGrafPort&  gp = event.grafPort();
  ICustomButton*    customButton = event.customButton();
  if (customButton)
  {
    bool        bDrawLatched = (!event.drawDown() && customButton->isLatched());

    ITRACE_ALL(IString("bDrawLatched =: ")+IString((unsigned long)bDrawLatched));

    // Get the background color
    IColor bgndColor(0,0,0);
    if ( bDrawLatched )
      bgndColor = customButton->latchedBackgroundColor();
    else
      bgndColor = customButton->backgroundColor();

    // Fill the background.
    // Need to paint the entire background with color in case the bitmap is
    // smaller than the button.
    IGrafBundle bundle;
    bundle.setFrameColor( bgndColor );
    bundle.setFillColor( bgndColor );
    bundle.setDrawingOperation( IAttributeState::kFillAndFrame );

    //If latched, draw the background with a halftone pattern.
    //We need to paint the entire background with a latched half-tone look
    //in case the bitmap is smaller than the button, or if the bitmap is
    //transparent.
    if ( bDrawLatched && customButton->isLatchedBackgroundColorHalftone() )
      {
        ITRACE_ALL("Drawing half-tone");
        //Create an image to draw into, so drawing is not rendered to screen.
        //The image is the size of the background area we are painting.
        IImage memoryImage( (long)rect.width(),
                            (long)rect.height(),
                            IGImage::kTrueColor24Bit,
                            ICoordinateSystem::kOriginUpperLeft);

        //Fill the background area with a solid color.
        //Fill in the entire area on the memory image.
        IGrafPort* imagePort = memoryImage.grafPort();
        imagePort->draw( IGRect2D( 0,0,
                                   rect.width() - 1,
                                   rect.height() - 1 ),
                          bundle );

        //Half-tone the target grafPort.
        IGrafBundle halfToneBundle;
        halfToneBundle.setImageTransferMode(IImageTransferMode::kOCHalfTone);
        ILinkedGrafPort halfTonePort(&gp, &halfToneBundle);

        //Draw the half-tone background from the image to the screen grafPort.
        IGrafMatrix matrix;
        matrix.translateBy(rect.minXMinY());
        ILinkedGrafPort translateHalfTonePort(&halfTonePort,
                                              kViewMatrix,
                                              &matrix );

        memoryImage.draw(translateHalfTonePort);
      }
    else
      {
        ITRACE_ALL("Drawing, not half-tone");
        gp.draw( IGRect2D( rect.minX(),
                           rect.minY(),
                           rect.maxX() - 1,
                           rect.maxY() - 1 ),
                 bundle );
      }

    ITRACE_ALL( IString(" drawBackground rect=") +
                IRectangle ( rect.minX(),
                             rect.minY(),
                             rect.maxX() - 1,
                             rect.maxY() - 1).asString());
  } // not a custom button
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawForeground                                     |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawForeground ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawForeground");
  IRectangle rect(event.drawingArea());
  ITRACE_ALL(IString("rect=") + rect.asString() );

  IGrafPort&  gp = event.grafPort();
  ICustomButton*    customButton = event.customButton();

  if (customButton)
  {
    // Get the old clip region and create a new clip region based on
    // the drawing area rectangle (so we do not draw outside of the
    // foreground area)
    IGArea
      startRegion( *gp.clipArea() ),
      clipRegion( rect );

    ILinkedGrafPort
      clippedPort( &gp, &clipRegion );

    IGrafBundle
      bundle;

#ifdef IC_PMWIN
    IFont  fnt(event.controlWindow());
#endif
#ifdef IC_MOTIF
    IFont  fnt = event.controlWindow()->font();
#endif

    // Start drawing text on the top of the button
    IPoint textPos;

    // Text origin is baseline of left character
    textPos.setY ( rect.minY() + gCySysBorder + fnt.maxAscender() );
    ITRACE_ALL( IString("textPos=") + textPos.asString() );

    // If button is not "up", shift foregroung drawing down 1 pel
    if ( !event.drawUp() )
      {
      textPos.setY ( textPos.y() + 1 );
      }

    // Use the foreground color to draw text
    if ( event.drawLatched() ) {
      IColor
        base = customButton->latchedForegroundColor();
      bundle.setFillColor( base );
    } else {
      IColor
        base = customButton->foregroundColor();
      bundle.setFillColor( base );
    }

    // Loop through text to draw each line
    unsigned index = 0;
    unsigned start = 0;

    IString txt(customButton->text());
    do
    {
      // Get the text for the current line
      IString subtxt;
      start = index + 1;
      index = txt.indexOf('\n',start);
      if (index != 0)
        subtxt = txt.subString(start,index-start).strip();
      else
        subtxt = txt.subString(start).strip();

      // Get the text width
      unsigned long txtWidth;
#ifdef IC_PMWIN
      if ( IBidiSettings::isBidiSupported() )
      {
        IBidiSettings settings(*(customButton));
        txtWidth = fnt.textWidth(subtxt,settings);
      }
      else
#endif
      {
#ifdef IC_PMWIN
        txtWidth = customButton->displaySize(subtxt).width();
#endif
#ifdef IC_MOTIF
        txtWidth = fnt.textWidth(subtxt);
#endif
      }

      // Center text (or clip if text is too big)
      if ( rect.width() >= txtWidth )
      {
        textPos.setX ( rect.minX() + (( rect.width() - txtWidth ) / 2 ));
      }
      else
      {
        textPos.setX ( rect.left() );
#ifdef IC_PMWIN
        if ( IBidiSettings::isBidiSupported() )
        {
          IBidiSettings settings(*(customButton));
          if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
          {
            textPos.setX ( rect.right() - txtWidth );
          }
        }
#endif
      }

      // Draw text
      ITRACE_ALL( IString("drawGString pos=") + textPos.asString() +
                  IString(" string=") + subtxt );

      IText text( subtxt );
      text.addStyles(ITextTypefaceStyle(fnt.name()));
      text.addStyles(ITextPointSizeStyle(fnt.pointSize()));
      text.addStyles(ITextOutlineStyle(fnt.isOutline()));
      text.addStyles(ITextBoldfaceStyle(fnt.isBold()));
      text.addStyles(ITextItalicStyle(fnt.isItalic()));
      text.addStyles(ITextUnderlineStyle(fnt.isUnderscore()));
      text.addStyles(ITextStrikethroughStyle(fnt.isStrikeout()));
      text.addStyles(ITextBitmapStyle(fnt.isBitmap()));

      IColor fgnd( customButton->foregroundColor() );
      text.addStyles(ITextColorStyle(fgnd.redMix(),
                                     fgnd.greenMix(),
                                     fgnd.blueMix()));
      IGraphicText theText( text, IGraphicText::kSingleLine );
      theText.setLocation(textPos);

      ILinkedGrafPort linkedPort( &clippedPort, &bundle );
      theText.draw( linkedPort );

      // Move position for next line of text (overlap one pel)
      textPos.setY ( textPos.y() + fnt.maxCharHeight() - 1 );
    }
    while (index != 0);
  }
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawDisabledEmphasis                               |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawDisabledEmphasis
                                 ( ICustomButtonDrawEvent& event )
{
  //Initialize local variables
  IGrafPort&  gp = event.grafPort();
  IRectangle rect(event.drawingArea());

  IGRect2D rect2D( rect );
  rect2D.orderPoints();

  IPresSpaceHandle hdc(gp.orphanDeviceContext(),
                       event.handle() );
  gp.adoptDeviceContext( hdc );

  IGrafBundle bundle;
  bundle.setImageTransferMode( IImageTransferMode::kOCHalfTone );

  //When this function is called, we do not know what has been
  //drawn into the grafport.  We need to retrieve the contents
  //of the grafport and redraw with disabled emphasis.
  gp.draw( IGImage( IGImage::captureImage(hdc, rect2D) ) ,bundle );
}
