// Revision: 33 1.17.1.13 source/ui/extapp/icustbut.cpp, toolbar, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: icustbut.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbut.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481524 )

// #define IC_TRACE_ALL

extern "C" {
  #define INCL_WINMESSAGEMGR
  #define INCL_WINBUTTONS
  #define INCL_WININPUT
  #define INCL_WINSYS
  #define INCL_WINDIALOGS
  #include <iwindefs.h>
  #ifdef IC_MOTIF
    #include <Xm/PushB.h>
  #endif //IC_MOTIF
}

#define USE_DRAWNBUTTON 1
#define USE_DRAWDATA 1

#include <ibmpstat.hpp>
#include <icconst.h>
#include <icustbdd.hpp>
#include <icustbhd.hpp>
#include <icustbut.hpp>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <iinhratt.hpp>
#include <ikeyevt.hpp>
#include <inotifev.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#ifdef IC_MOTIF
  #include <icmdevt.hpp>
  #include <imstring.hpp>
  #include <ixmlabel.hpp>
#endif //IC_MOTIF

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBUT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ICustomButtonLatchHandler : public IHandler
{
public:
  ICustomButtonLatchHandler ();
  ~ICustomButtonLatchHandler ();
virtual bool
  dispatchHandlerEvent(IEvent& event);
ICustomButton*
  buttonDown;
};

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
ICustomButtonLatchHandler :: ICustomButtonLatchHandler ()
                           : buttonDown ( 0 )
{
}

ICustomButtonLatchHandler :: ~ICustomButtonLatchHandler ()
{
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
bool ICustomButtonLatchHandler :: dispatchHandlerEvent(IEvent& event)
{
  bool rc = false;

  switch ( event.eventId() )
  {
#ifdef IC_PMWIN
    // Check for button up message
    // This is needed to smoothly change the button latch state.
    // We first check to see if the button was last drawn in the
    // down state.  If so, then we need to toggle the latch state.
    // The refreshDisplay flag is set to false since PM will
    // already redraw the button and we do not want double drawing.
    case WM_BUTTON1UP:
       {
         ICustomButton *button = (ICustomButton *) event.dispatchingWindow();

         if ( buttonDown == button )
         {
           if ( button->isLatched() )
           {
             if ( button->isLatchingEnabled() )
               button->latch(false,false);
           }
           else
           {
             if ( button->isLatchingEnabled() || button->isAutoLatchEnabled() )
             {
               button->latch(true,false);
             }
           }
         }
       }
       break;
#endif

#ifdef IC_MOTIF
     case motifEvent(XmCR_ACTIVATE):
     {
         IMODTRACE_ALL("XmCR_ACTIVATE");
         ICustomButton* button =
            dynamic_cast<ICustomButton*>( event.dispatchingWindow() );
         if (button)
         {

            ITRACE_ALL(IString("isLatched=")+IString((unsigned long)button->isLatched())
                        +IString(", isLatchingEnabled=")+IString((unsigned long)button->isLatchingEnabled())
                        +IString(", isAutoLatchingEnabled=")+IString((unsigned long)button->isAutoLatchEnabled()));

            // Handle the transition from down to up at this point (instead
            // of in DISARM) so that we get smooth latching.
            if ( button->isLatched() )
            {
              if ( button->isLatchingEnabled() )
                button->latch(false,false);
            }
            else
            {
              if ( button->isLatchingEnabled() || button->isAutoLatchEnabled() )
              {
                button->latch(true,false);
              }
            }

            //In Motif, we must generate the command event when a button
            //is pressed.
            IWindow* powner = button->owner();
            if (powner)
              powner->postEvent ( IWindow::command,
                                  button->id(),
                                  IEventData(ICommandEvent::pushButton, 0) );
         }
         break;
     }
     case motifEvent(XmCR_ARM):
     {
         // When the button is armed, we need to draw it in the down or
         // latched state.
         IMODTRACE_ALL("XmCR_ARM");
         ICustomButton *button = dynamic_cast<ICustomButton*>(event.dispatchingWindow() );
         if (button)
         {
            // Handle setting the image into the widget.
            ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
               ( button->windowData( ICustomButtonDrawData::dataHandle() ) );
            if (drawData)
            {
               // DrawData is available.  Check the state of the cache to
               // see if image needs to be redrawn, and send event to do
               // the drawing if needed.  Set the armPixmap in the
               // widget to the appropriate image.
               drawData->setButtonState( ICustomButtonDrawEvent::buttonDown );
               drawData->setSystemData ( event.parameter1() );
               if ( !drawData->isImageValid( drawData->buttonState() ) )
               {
                  button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                    true,
                                    drawData);
               }
               else
               {  // Image is valid, but we need to be sure disabled image
                  // is the correct one, because there is only one resource
                  // for this image.
                  // This is one of the places where the widget's
                  // image resources get set.
                  // Valid images can be set by:
                  // - IC_UM_MOTIF_CUST_BUT_PAINT case of
                  //   ICustomButtonDrawHandler::dispatchHandlerEvent()
                  // - ICustomButton::latch
                  // - XmCR_ARM and XmCR_DISARM cases of
                  //   ICustomButtonLatchHandler::dispatchHandlerEvent()
                  //    (these change the insensitive pixmap only)
                  // The images can be set to XmUNSPECIFIED_PIXMAP by:
                  // - ICustomButton::setLayoutDistorted function
                  // - ConfigureNotify case of ICustomButtonDrawHandler::
                  //   dispatchHandlerEvent()
                  Pixmap  disabledPixmap = (IBitmapHandle::Value)
                     drawData->imageHandle( drawData->buttonState(), false);
                  if (!disabledPixmap)
                     disabledPixmap = XmUNSPECIFIED_PIXMAP;
                  XtVaSetValues(
                     button->handle(),
                     XmNlabelInsensitivePixmap, disabledPixmap,
                     0);
               }
            }
            else
            {
               // No DrawData available.  Send event to do drawing to
               // the screen.
               button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT, true, 0);
            }
         }
         break;
     }
    case motifEvent(XmCR_DISARM):
     {
         // When the button is disarmed, we typically need to draw it
         // in the up state.  However, if it is has latching enabled
         // we need to handle latch state transitions and then draw the
         // button down if it is now latched.
         IMODTRACE_ALL("XmCR_DISARM");
         ICustomButton *button = dynamic_cast<ICustomButton*>(
            event.dispatchingWindow() );
         if (button)
         {
            ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
               ( button->windowData( ICustomButtonDrawData::dataHandle() ) );
            if (drawData)
            {
               // DrawData is available.  Check the state of the cache to
               // see if image needs to be redrawn, and send event to do
               // the drawing if needed.  Set the labelPixmap in the
               // widget to the appropriate image.
               drawData->setButtonState(
                    button->isLatched() ?
                    ICustomButtonDrawEvent::buttonLatched :
                    ICustomButtonDrawEvent::buttonUp );
               drawData->setSystemData ( event.parameter1() );
               if ( !drawData->isImageValid( drawData->buttonState() ) )
               {
                  button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                    false,
                                    drawData);
               }
               else
               {  // Image is valid, but we need to be sure disabled image
                  // is the correct one, because there is only one resource
                  // for this image.
                  // This is one of the places where the widget's
                  // image resources get set.
                  // Valid images can be set by:
                  // - IC_UM_MOTIF_CUST_BUT_PAINT case of
                  //   ICustomButtonDrawHandler::dispatchHandlerEvent()
                  // - ICustomButton::latch
                  // - XmCR_ARM and XmCR_DISARM cases of
                  //   ICustomButtonLatchHandler::dispatchHandlerEvent()
                  //    (these change the insensitive pixmap only)
                  // The images can be set to XmUNSPECIFIED_PIXMAP by:
                  // - ICustomButton::setLayoutDistorted function
                  // - ConfigureNotify case of ICustomButtonDrawHandler::
                  //   dispatchHandlerEvent()
                  Pixmap  disabledPixmap = (IBitmapHandle::Value)
                     drawData->imageHandle( drawData->buttonState(), false);
                  if (!disabledPixmap)
                     disabledPixmap = XmUNSPECIFIED_PIXMAP;
                  XtVaSetValues(
                     button->handle(),
                     XmNlabelInsensitivePixmap, disabledPixmap,
                     0);
               }
            }
            else
            {
               // No DrawData available.  Send event to do drawing to
               // the screen.
               button->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT,
                                 button->isLatched(), 0);
            }
         }
         break;
     }
#endif


    // Check for unlatch message
    //
    // NOTE: This is an internal message used by the class library to
    //       implement the autoLatch feature of custom buttons

    case IC_UM_UNLATCH:
       {
         ICustomButton *button = (ICustomButton *) event.dispatchingWindow();
         if ( button->isAutoLatchEnabled() )
           button->unlatch();
         rc = true;
       }
       break;

    // Check draw down message
    //
    // NOTE: This is an internal message used by the class library to
    //       implement smooth latching

    case IC_UM_DRAW_DOWN:
       {
         ICustomButton *button = (ICustomButton *) event.dispatchingWindow();
         if ( event.parameter1().asUnsignedLong() )
         {
            buttonDown = button;
         }
         else
         {
            if ( buttonDown == button)
               buttonDown = 0;
         }
         rc = true;
       }
       break;

#ifdef IC_WIN
    // Generate WM_COMMAND message expected by command handlers.
    case WM_CONTROL:
       if (event.parameter1().highNumber() == BN_CLICKED)
          {
          ICustomButton *button = (ICustomButton *) event.dispatchingWindow();
          // generate the needed events due to a button click
          if (button)
             {
             IWindow* owner = button->owner();
             if ((owner == 0) || (owner == IWindow::desktopWindow()) )
                {
                owner = button->parent();
                }
             if (owner)
                {
                ITRACE_ALL( IString("owner=") +
                            IString( (unsigned long)((void*)owner) ).d2x() +
                            IString(" hwnd=") +
                            IString( owner->handle().asUnsigned() ).d2x() );
                owner->postEvent( WM_COMMAND,
                                  IEventData(
                                     event.parameter1().lowNumber(),
                                     CMDSRC_PUSHBUTTON),
                                  0 );
                rc = true;
                } // if owner
             } // if button
          } // if BN_CLICKED
       break;
#endif

#ifdef IC_PMWIN
    // Remove the DLGC_PUSHBUTTON dialog code for custom buttons because
    // we do supprt the "default button" for custom buttons and the canvas
    // logic looks at this code to determine if a button can be the
    // default
    case WM_QUERYDLGCODE:
        event.window()->dispatchRemainingHandlers(event);
        event.setResult(event.result().asUnsignedLong() &
                           (unsigned long)(~DLGC_PUSHBUTTON));
        rc = true;
        break;

    // Make the enter key and spacebar work the same as the mouse
    //
    // Note: Normally we would only want the spacebar to act this way,
    //       but we shipped with a bug regarding default pushbutton
    //       support and current applications will expect the enter
    //       key to press the button (so we are adding enter key support
    //       to try and maintain better backward compatibility)

    case WM_CHAR:
        {
          IKeyboardEvent keyevt(event);
          if ( !(keyevt.isUpTransition()) && keyevt.isVirtual() )
          {
            if (( keyevt.virtualKey() == IKeyboardEvent::space ) ||
                ( keyevt.virtualKey() == IKeyboardEvent::enter ) ||
                ( keyevt.virtualKey() == IKeyboardEvent::newLine ))
            {
              ICustomButton *button =
                            (ICustomButton *)event.dispatchingWindow();
              button->click();
              rc = true;
            }
          }
        }
        break;
#endif //IC_PMWIN

  }
  return rc;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
#pragma enum(4)
#pragma pack(push,4)

class ICustomButtonData
{
public:
  ICustomButtonData ( );
bool
  latched,
  halftoneLatch,
  bkgndSet;
IColor
  *latchColor;
IColor
  *latchForeground;
unsigned long
  userData;
ICustomButtonLatchHandler
  latchHandler;
ICustomButtonDrawHandler
  drawHandler;

};

#pragma pack(pop)
#pragma enum(pop)

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
ICustomButtonData :: ICustomButtonData ( )
                   : latched ( false ),
                     halftoneLatch ( true ),
                     bkgndSet( false ),
                     latchColor ( 0 ),
                     latchForeground ( 0 ),
                     userData ( 0 ),
                     latchHandler( ),
                     drawHandler( )
{
}

#ifdef IC_MOTIF
// Only the setText override is actually needed now, but we need to keep
// them for backward compatiblity.
/*------------------------------------------------------------------------------
| ICustomButton::setText                                                       |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: setText(const char* pszText)
{
  Inherited::setText( pszText );
  setLayoutDistorted( IWindow::minimumSizeChanged, 0);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::setText                                                       |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: setText(const IResourceId& residText)
{
  Inherited::setText(residText);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::text                                                          |
------------------------------------------------------------------------------*/
IString  ICustomButton :: text() const
{
  return Inherited::text();

}

/*------------------------------------------------------------------------------
| ICustomButton::setFont                                                       |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: setFont(const IFont& font)
{
  Inherited::setFont( font );
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::resetFont                                                     |
|                                                                              |
| Causes the window to use the default font.                                   |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: resetFont()
{
  Inherited::resetFont( );
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::font                                                          |
------------------------------------------------------------------------------*/
IFont  ICustomButton :: font() const
{
  return Inherited::font( );
}

/*------------------------------------------------------------------------------
| ICustomButton::characterSize                                                 |
------------------------------------------------------------------------------*/
ISize  ICustomButton :: characterSize ( ) const
{
  return Inherited::characterSize( );
}
/*------------------------------------------------------------------------------
| ICustomButton::enable                                                        |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: enable( bool enableWindow )
{
  Inherited::enable(enableWindow);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::disable                                                       |
|                                                                              |
| This function overrided exists to complement the enable() override           |
| to present a consistent interface the user.                                  |
------------------------------------------------------------------------------*/
ICustomButton&  ICustomButton :: disable( )
{
  return( this->enable( false ) );
}
#endif  //IC_MOTIF

/*------------------------------------------------------------------------------
| ICustomButton styles                                                         |
------------------------------------------------------------------------------*/
const ICustomButton::Style
  ICustomButton::latchable         ( 0, IBS_CUSTOMLATCHABLE ),
  ICustomButton::autoLatch         ( 0, IBS_CUSTOMAUTOLATCH ),
  ICustomButton::classDefaultStyle = WS_VISIBLE;

  ICustomButton::Style ICustomButton::fgCurrentDefaultStyle = WS_VISIBLE;

/*------------------------------------------------------------------------------
| ICustomButton ctor                                                           |
------------------------------------------------------------------------------*/
ICustomButton :: ICustomButton (unsigned long id,
                                IWindow* parent,
                                IWindow* owner,
                                const IRectangle& initial,
                                const Style& style )
               : fCustomButtonData(0)
{
  IASSERTPARM(parent!=0);
  fCustomButtonData = new ICustomButtonData;

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  IWindowHandle whButton =
      this -> create( id,
                      0,
                      convertToGUIStyle( style ),
                      WC_BUTTON,
                      parent->handleForChildCreation(),
                      (owner == 0) ? IWindowHandle(0) : owner->handle(),
                      initial,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( style, true ) );

  startHandlingEventsFor(whButton);
#endif

#ifdef IC_MOTIF

#if USE_DRAWDATA
   this->adoptWindowData( ICustomButtonDrawData::dataHandle(),
                          new ICustomButtonDrawData );
#endif

   IArgList argList;
   Widget widgetID;

   // Button face is drawn by the drawHandler.  Set the system provided
   // border widths to 0 to suppress standard system drawing.
   argList.setResource( XmNmarginHeight, 0);
   argList.setResource( XmNmarginWidth, 0);
   argList.setResource( XmNborderWidth, 0);
   argList.setResource( XmNshadowThickness,   0);    //no shadow highlighting
   argList.setResource( XmNdefaultButtonShadowThickness, 0);
   argList.setResource( XmNmultiClick, XmMULTICLICK_DISCARD);
   argList.setResource( XmNshowAsDefault, 0);
   argList.setResource( XmNfillOnArm, 0);
   argList.setResource( XmNrecomputeSize, 0);
   argList.setResource( XmNlabelType, XmPIXMAP);   // label is image

   widgetID = Inherited::create(id,
                                0,
                                style,
                                (IXmCreateFunction)XmCreatePushButton,
                                parent,
                                owner,
                                initial);

   XtSetValues( widgetID, (ArgList)argList.argList(), argList.numberOfArgs() );

   startHandlingEventsFor(widgetID);

   // Set up pointerFocus based on the IButton::noPointerFocus style. (the
   // convertToGUIStyle functions handle this on Intel)
   this->enableMouseClickFocus( (style & noPointerFocus) ? false : true );

   // Add callbacks used by latchHandler.
   XtAddCallback( widgetID,
                  XmNactivateCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);
   XtAddCallback( widgetID,
                  XmNarmCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);
   XtAddCallback( widgetID,
                  XmNdisarmCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);

#endif // IC_MOTIF

  fCustomButtonData->latchHandler.handleEventsFor(this);
  fCustomButtonData->drawHandler.handleEventsFor(this);

#ifdef IC_MOTIFWIN
  // Add the inheritColor attribute to the window.  ICustomButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );

#endif // IC_MOTIFWIN
#ifdef IC_WIN
  setColor( PP_BACKGROUNDCOLOR, IColor(IColor::kButtonMiddle));
#endif // IC_WIN
}

/*------------------------------------------------------------------------------
| ICustomButton (wrapper) ctor                                                 |
------------------------------------------------------------------------------*/
ICustomButton :: ICustomButton (const IWindowHandle& handle )
               : fCustomButtonData(0)
{
  setAutoDestroyWindow ( false );

  fCustomButtonData = new ICustomButtonData;

#ifdef IC_MOTIF

#if USE_DRAWDATA
   this->adoptWindowData( ICustomButtonDrawData::dataHandle(),
                          new ICustomButtonDrawData );
#endif

   // Add callbacks used by latchHandler.
   XtAddCallback( handle,
                  XmNactivateCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);
   XtAddCallback( handle,
                  XmNarmCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);
   XtAddCallback( handle,
                  XmNdisarmCallback,
                  iwindowMotifCallback,
                  (XtPointer)this);
#endif

  reserveUserWindowWord( false );
  startHandlingEventsFor(handle);

  fCustomButtonData->latchHandler.handleEventsFor(this);
  fCustomButtonData->drawHandler.handleEventsFor(this);

#ifdef IC_MOTIFWIN
  // add the inheritColor attribute to the window.  ICustomButton does
  // not inherit background or foreground color.
  IInheritColorAttribute
    inheritColor( IInheritColorAttribute::kNone );
  this->addOrReplaceAttribute( IAttributeName("IInheritColorAttribute"),
                               inheritColor );

#endif // IC_MOTIFWIN
}

/*------------------------------------------------------------------------------
| ICustomButton (protected) ctor                                               |
------------------------------------------------------------------------------*/
ICustomButton :: ICustomButton ()
               : fCustomButtonData(0)
{
  fCustomButtonData = new ICustomButtonData;
#ifdef IC_MOTIF

#if USE_DRAWDATA
   this->adoptWindowData( ICustomButtonDrawData::dataHandle(),
                          new ICustomButtonDrawData );
#endif
   // Note: user of this ctor will have to add callbacks for latch handler
   // to work correctly.
#endif

  fCustomButtonData->latchHandler.handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| ICustomButton dtor                                                           |
------------------------------------------------------------------------------*/
ICustomButton :: ~ICustomButton()
{
#ifdef IC_MOTIF
   if (this->isValid())
   {
      Widget widgetId = this->handle();
      XtRemoveCallback( widgetId,
                     XmNactivateCallback,
                     iwindowMotifCallback,
                     (XtPointer)this);
      XtRemoveCallback( widgetId,
                     XmNarmCallback,
                     iwindowMotifCallback,
                     (XtPointer)this);
      XtRemoveCallback( widgetId,
                     XmNdisarmCallback,
                     iwindowMotifCallback,
                     (XtPointer)this);
   }
#endif
  fCustomButtonData->latchHandler.stopHandlingEventsFor( this );
  fCustomButtonData->drawHandler.stopHandlingEventsFor( this );
  if (fCustomButtonData->latchColor)
    delete fCustomButtonData->latchColor;
  if (fCustomButtonData)
    delete fCustomButtonData;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
ICustomButton::Style ICustomButton :: defaultStyle()
{
  return fgCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
void ICustomButton :: setDefaultStyle(const Style& style)
{
  fgCurrentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| ICustomButton::convertToGUIStyle                                             |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ICustomButton::convertToGUIStyle(const IBitFlag& guiStyle,
                                               bool bExtOnly) const
{
  // Obtain the style from the class (IButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

// Motif (ConvertToGUIStyle does nothing in Motif)
#ifdef IC_PMWIN
  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // Set the GUI user button style
#ifdef IC_PM
    ulStyle |= BS_USERBUTTON;
#endif
#ifdef IC_WIN
    ulStyle |= BS_OWNERDRAW | WS_CHILD;
#endif
  }
#endif
  return( ulStyle );
}

/*------------------------------------------------------------------------------
| ICustomButton::enableLatching                                                |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: enableLatching ( bool enable )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  if ( enable )
  {
    ulExtStyle |= latchable.asExtendedUnsignedLong();
  }
  else
  {
    latch(false);
    ulExtStyle &= ~latchable.asExtendedUnsignedLong();
  }

  if ( ulExtStyle != ulOldExtStyle )
  {
    setExtendedStyle(ulExtStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::disableLatching                                               |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: disableLatching ( )
{
  enableLatching(false);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::enableAutoLatch                                               |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: enableAutoLatch ( bool enable )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  if ( enable )
  {
    ulExtStyle |= autoLatch.asExtendedUnsignedLong();
  }
  else
  {
    ulExtStyle &= ~autoLatch.asExtendedUnsignedLong();
  }

  if ( ulExtStyle != ulOldExtStyle )
  {
    setExtendedStyle(ulExtStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::disableAutoLatch                                              |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: disableAutoLatch ( )
{
  enableAutoLatch(false);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::isLatchable                                                   |
------------------------------------------------------------------------------*/
bool ICustomButton :: isLatchingEnabled () const
{
  return( (extendedStyle() & latchable.asExtendedUnsignedLong()) ? true
                                                                 : false );
}

/*------------------------------------------------------------------------------
| ICustomButton::isAutoLatch                                                   |
------------------------------------------------------------------------------*/
bool ICustomButton :: isAutoLatchEnabled () const
{
  return( (extendedStyle() & autoLatch.asExtendedUnsignedLong()) ? true
                                                                 : false );
}

/*------------------------------------------------------------------------------
| ICustomButton::latch                                                         |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: latch ( bool latched,
                                        bool refresh )
{
  IMODTRACE_ALL("ICustomButton::latch");
  unsigned long ulMask = latchable.asExtendedUnsignedLong() |
                         autoLatch.asExtendedUnsignedLong();

  if ( (extendedStyle() & ulMask) &&
       ( fCustomButtonData->latched != latched ))
  {
    fCustomButtonData->latched = latched;

    notifyObservers(INotificationEvent(ICustomButton::latchId, *this,
                                       true, (void*)latched));
#ifdef IC_MOTIF
    ICustomButtonDrawData* drawData = (ICustomButtonDrawData*)
       this->windowData( ICustomButtonDrawData::dataHandle() );
    if (drawData && this->isValid())
    {
       ICustomButtonDrawEvent::ButtonState state =
           latched ?
           ICustomButtonDrawEvent::buttonLatched :
           ICustomButtonDrawEvent::buttonUp ;
       drawData->setButtonState( state );
       if (!drawData->isImageValid( state ))
       {
          // Draw image and update the widget with the image.
          drawData->setSystemData(0);
          this->sendEvent(IC_UM_MOTIF_CUST_BUT_PAINT, false, drawData);
       }
       else
       {
          // Update the widget with the saved image.

          // This is one of the places where the widget's
          // image resources get set.
          // Valid images can be set by:
          // - IC_UM_MOTIF_CUST_BUT_PAINT case of
          //   ICustomButtonDrawHandler::dispatchHandlerEvent()
          // - ICustomButton::latch
          // - XmCR_ARM and XmCR_DISARM cases of
          //   ICustomButtonLatchHandler::dispatchHandlerEvent()
          //    (these change the insensitive pixmap only)
          // The images can be set to XmUNSPECIFIED_PIXMAP by:
          // - ICustomButton::setLayoutDistorted function
          // - ConfigureNotify case of ICustomButtonDrawHandler::
          //   dispatchHandlerEvent()
          Pixmap labelPixmap = (IBitmapHandle::Value)
             drawData->imageHandle( drawData->buttonState(), true );
          if (!labelPixmap)
             labelPixmap = XmUNSPECIFIED_PIXMAP;
          Pixmap  disabledPixmap = (IBitmapHandle::Value)
             drawData->imageHandle( drawData->buttonState(), false);
          if (!disabledPixmap)
             disabledPixmap = XmUNSPECIFIED_PIXMAP;
          XtVaSetValues( this->handle(),
                         XmNlabelPixmap, labelPixmap,
                         XmNlabelInsensitivePixmap, disabledPixmap,
                         0);
       }
    }
#endif

    if ( refresh )
      this->refresh();
    if ( latched )
      checkAutoLatch ();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::unlatch                                                       |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: unlatch ()
{
  return latch(false);
}

/*------------------------------------------------------------------------------
| ICustomButton::isLatched                                                     |
------------------------------------------------------------------------------*/
bool ICustomButton :: isLatched () const
{
  return fCustomButtonData->latched;
}

/*------------------------------------------------------------------------------
| ICustomButton::latchedBackgroundColor                                        |
------------------------------------------------------------------------------*/
IColor ICustomButton :: latchedBackgroundColor () const
{
  if (fCustomButtonData->latchColor)
    return (*(fCustomButtonData->latchColor));
  else
    return(backgroundColor());
}

/*------------------------------------------------------------------------------
| ICustomButton::latchedForegroundColor                                        |
------------------------------------------------------------------------------*/
IColor ICustomButton :: latchedForegroundColor () const
{
  if (fCustomButtonData->latchForeground)
    return (*(fCustomButtonData->latchForeground));
  else
    return(foregroundColor());
}

/*------------------------------------------------------------------------------
| ICustomButton::setLatchedBackgroundColor                                     |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: setLatchedBackgroundColor ( const IColor& color,
                                                            bool halftone )
{
  if (fCustomButtonData->latchColor)
    *(fCustomButtonData->latchColor) = color;
  else
    fCustomButtonData->latchColor = new IColor(color);
  fCustomButtonData->halftoneLatch = halftone;
  setLayoutDistorted( IWindow::colorChanged, 0);
  if (fCustomButtonData->latched)
    refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::setLatchedForegroundColor                                     |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: setLatchedForegroundColor (const IColor& color)
{
  if (fCustomButtonData->latchForeground)
    *(fCustomButtonData->latchForeground) = color;
  else
    fCustomButtonData->latchForeground = new IColor(color);
  setLayoutDistorted( IWindow::colorChanged, 0);
  if (fCustomButtonData->latched)
    refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::resetLatchedBackgroundColor                                   |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: resetLatchedBackgroundColor ()
{
  if (fCustomButtonData->latchColor)
  {
    delete fCustomButtonData->latchColor;
    fCustomButtonData->latchColor = 0;
    setLayoutDistorted( IWindow::colorChanged, 0);
  }
  fCustomButtonData->halftoneLatch = true;
  if (fCustomButtonData->latched)
    refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::resetLatchedForegroundColor                                   |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: resetLatchedForegroundColor ()
{
  if (fCustomButtonData->latchForeground)
  {
    delete fCustomButtonData->latchForeground;
    fCustomButtonData->latchForeground = 0;
    setLayoutDistorted( IWindow::colorChanged, 0);
  }
  if (fCustomButtonData->latched)
    refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::isLatchedBackgroundColorHalftone                              |
------------------------------------------------------------------------------*/
bool ICustomButton :: isLatchedBackgroundColorHalftone () const
{
  return fCustomButtonData->halftoneLatch;
}

/*------------------------------------------------------------------------------
| ICustomButton::backgroundColor                                               |
------------------------------------------------------------------------------*/
IColor ICustomButton :: backgroundColor () const
{
#ifdef IC_MOTIFWIN
  if (!fCustomButtonData->bkgndSet)
    return ( IColor( IColor::kButtonMiddle ));
  return Inherited::backgroundColor();
#else
  return ( IWindow::color ( PP_BACKGROUNDCOLOR,
                            IColor ( IColor::kButtonMiddle )));
#endif
}

#ifdef IC_WIN
// The overrides below (and the query logic above) are required on Windows
// to get the color queries to work correctly in this class and derived
// classes (like IToolBarButton or any application derived class).
/*------------------------------------------------------------------------------
| ICustomButton::setBackgroundColor                                            |
|                                                                              |
| Sets the background color of the window.                                     |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton::setBackgroundColor( const IColor &color )
{
  fCustomButtonData->bkgndSet = true;
  Inherited::setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::setBackgroundColor                                            |
|                                                                              |
| Sets the background color of the window.                                     |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton::setColor( unsigned long colorArea,
                                        const IColor &color )
{
  if (colorArea == PP_BACKGROUNDCOLOR )
  {
    fCustomButtonData->bkgndSet = true;
  }
  Inherited::setColor( colorArea, color );
  return *this;
}


/*------------------------------------------------------------------------------
| ICustomButton::resetBackgroundColor                                          |
|                                                                              |
| Resets the background color to the default button value                      |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton::resetBackgroundColor ()
{
  return setBackgroundColor( IColor::kButtonMiddle );
}
#endif

/*------------------------------------------------------------------------------
| ICustomButton::addDefaultDrawHandler                                         |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: addDefaultDrawHandler ()
{
  //handleEventsFor calls IWindow::addHandler, which will not add the
  //handler twice.
  fCustomButtonData->drawHandler.handleEventsFor(this);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::removeDefaultHandler                                          |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: removeDefaultDrawHandler ()
{
  //handleEventsFor calls IWindow::removeHandler, which will not cause a
  //problem even if the handler has never been added.
  fCustomButtonData->drawHandler.stopHandlingEventsFor(this);
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::calcMinimumSize                                               |
------------------------------------------------------------------------------*/
ISize ICustomButton :: calcMinimumSize () const
{
  ISize sz;

  // Get the current font and text
  IFont fnt = this->font();

  IString       txt(text());

  // Set up temporary variables used for calculations
  unsigned long index = 0;
  unsigned long start;
  unsigned long cx;

  // For each line add to height and test width
  sz.setHeight(1);
  do
  {
    // Only add max height - 1 (so that one pel may overlap between lines)
    sz.setHeight(sz.height()+fnt.maxCharHeight()-1);
    start = index + 1;
    index = txt.indexOf('\n', (unsigned)start);
#ifdef IC_PMWIN
    if ( IBidiSettings::isBidiSupported() )
    {
      IBidiSettings settings(*this);
      if (index != 0)
        cx = fnt.textWidth(txt.subString((unsigned)start,
                                         (unsigned)(index-start)).strip(),settings);
      else
        cx = fnt.textWidth(txt.subString((unsigned)start).strip(),settings);
    }
    else
#endif
    {

      if (index != 0)
        cx = displaySize(txt.subString((unsigned)start,
                                       (unsigned)(index-start)).strip()).width();
      else
        cx = displaySize(txt.subString((unsigned)start).strip()).width();

    }
    if (cx > sz.width())
      sz.setWidth(cx);
  }
  while (index != 0);

  // Add room for border painting and pad before returning
  // There may be up to 5 borders and add 6x3 pels of pad

#ifdef IC_PMWIN
  unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);
#endif
#ifdef IC_MOTIF
  unsigned long cxBorder = 1;
  unsigned long cyBorder = 1;
#endif

  sz.setWidth ( sz.width() + ( cxBorder * 5 ) + 6 );
  sz.setHeight ( sz.height() + ( cyBorder * 5 ) + 3 );

  return sz;
}

/*------------------------------------------------------------------------------
| ICustomButton::checkAutoLatch                                                |
------------------------------------------------------------------------------*/

void ICustomButton :: checkAutoLatch ()
{
  // Check if the autoLatch is set

  if ( isAutoLatchEnabled() )
  {
    // Unlatch all other buttons in the current group

    IWindow *top = parent();
    IWindow *first = 0;
    IWindow *current = 0;

    IWindow::ChildCursor cursor(*top);

    // Find the first window in the same group

    cursor.setToFirst();
    while ( cursor.isValid() )
    {
      current = IWindow::windowWithHandle(top->childAt(cursor));
      if (( first == 0 ) || (( current != 0 ) && ( current->isGroup() )))
      {
        first = current;
      }
      if ( current == this )
        cursor.invalidate();
      else
        cursor.setToNext();
    }

    // Send IC_UM_UNLATCH to all other buttons in the group.

    if ( first )
    {
      cursor.setToFirst();
      while ( cursor.isValid() )
      {
        current = IWindow::windowWithHandle(top->childAt(cursor));
        if ( current != 0 )
        {
          if ( first == current )
          {
            if ( current != this )
              current->sendEvent(IC_UM_UNLATCH);
            first = 0;
            cursor.setToNext();
          }
          else
          {
            if ( first == 0 )
            {
              if (current->isGroup())
                cursor.invalidate();
              else
              {
                if ( current != this )
                  current->sendEvent(IC_UM_UNLATCH);
                cursor.setToNext();
              }
            }
            else
              cursor.setToNext();
          }
        }
        else
          cursor.setToNext();
      }
    }
  }
}

/*------------------------------------------------------------------------------
| ICustomButton::setUserData                                                   |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton :: setUserData ( unsigned long data )
{
  fCustomButtonData->userData = data;
  return *this;
}

/*------------------------------------------------------------------------------
| ICustomButton::userData                                                      |
------------------------------------------------------------------------------*/
unsigned long ICustomButton :: userData ( ) const
{
  return fCustomButtonData->userData;
}

#ifdef IC_MOTIF
void _System iCustomButtonMotifCallback(
                     Widget     handle,
                     XtPointer  client_data,
                     XtPointer  call_data)
{
  IMODTRACE_ALL("iCustomButtonMotifCallback");

  // Retrieve the IWindow associated with the handle (Widget ID)
  ICustomButton* pwin = (ICustomButton *)client_data;
  if (!pwin)
    ITHROWLIBRARYERROR1(IC_INVALID_CLASS,
                       IBaseErrorInfo::accessError,
                       IException::recoverable,
                       "IToolBarButton");
  else
    {
     IWindow * powner = pwin->owner();
     if (powner)
        powner->postEvent ( IWindow::command,
                            pwin->id(),
                            IEventData(ICommandEvent::pushButton, 0) );
    }
  return;
}

/*------------------------------------------------------------------------------
| ICustomButton::passEventToOwner                                              |
|                                                                              |
| Returns whether or not the event can be passed up to the owner of this       |
| control.                                                                     |
------------------------------------------------------------------------------*/
bool ICustomButton::passEventToOwner( IEvent& event )

{
  switch ( event.eventId() )
  {
    case WM_BUTTON1DOWN:
    case WM_BUTTON2DOWN:
    case WM_BUTTON3DOWN:
    case WM_BUTTON1UP:
    case WM_BUTTON2UP:
    case WM_BUTTON3UP:
    case WM_BUTTON1CLICK:
    case WM_BUTTON2CLICK:
    case WM_BUTTON3CLICK:
    case WM_BUTTON1DBLCLK:
    case WM_BUTTON2DBLCLK:
    case WM_BUTTON3DBLCLK:
    case WM_CHORD:
      event.setPassToOwner( false );
      break;
    default:
      Inherited::passEventToOwner( event );
      break;
    }
  // return value is ignored.
  return true;
}
#endif  // IC_MOTIF

/*------------------------------------------------------------------------------
| ICustomButton::setLayoutDistorted                                            |
------------------------------------------------------------------------------*/
ICustomButton& ICustomButton::setLayoutDistorted
                                ( unsigned long layoutAttributeOn,
                                  unsigned long layoutAttributeOff )
{
   IMODTRACE_ALL("ICustomButton::setLayoutDistorted");
#ifdef IC_MOTIF
   if ( (layoutAttributeOn & IWindow::minimumSizeChanged ) ||
        (layoutAttributeOn & IWindow::colorChanged) ||
        (layoutAttributeOn & IWindow::fontChanged ) )
   {
      // Any of the above changes invalidate the cached image(s).
      ICustomButtonDrawData* drawData = dynamic_cast<ICustomButtonDrawData*>
         ( this->windowData( ICustomButtonDrawData::dataHandle() ) );
      if (drawData)
      {
         bool imagesInvalidated = drawData->invalidateAll( layoutAttributeOn );
         if (imagesInvalidated && this->isValid())
         {
            // Clear the images stored in the widget to insure residual
            // images do not cause flashing or visual artifacts.
            // This is one of the places where the widget's
            // image resources get set.
            // Valid images can be set by:
            // - IC_UM_MOTIF_CUST_BUT_PAINT case of
            //   ICustomButtonDrawHandler::dispatchHandlerEvent()
            // - ICustomButton::latch
            // - XmCR_ARM and XmCR_DISARM cases of
            //   ICustomButtonLatchHandler::dispatchHandlerEvent()
            //    (these change the insensitive pixmap only)
            // The images can be set to XmUNSPECIFIED_PIXMAP by:
            // - ICustomButton::setLayoutDistorted function
            // - ConfigureNotify case of ICustomButtonDrawHandler::
            //   dispatchHandlerEvent()

            ITRACE_ALL("setLayoutDistorted - setting XmUNSPECIFIED_PIXMAP");
            XtVaSetValues( this->handle(),
                           XmNarmPixmap,   XmUNSPECIFIED_PIXMAP,
                           XmNlabelPixmap, XmUNSPECIFIED_PIXMAP,
                           XmNlabelInsensitivePixmap, XmUNSPECIFIED_PIXMAP,
                           0);
         }
      }
   }
#endif
   Inherited::setLayoutDistorted(layoutAttributeOn,layoutAttributeOff);
   return *this;
}
