/*******************************************************************************
* FILE NAME: iflypain.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iflypain.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
{
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#include <ibase.hpp>

#ifdef IC_PM

#include <iflypain.hpp>
#include <iflytext.hpp>
#include <iexgrprt.hpp>
#include <igrport.hpp>
#include <igraftxt.hpp>
#include <igraph2d.hpp>
#include <ibundles.hpp>
#include <istyles.hpp>
#include <ifont.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFLYPAIN_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IFlyTextPaintHandler::IFlyTextPaintHandler                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyTextPaintHandler::IFlyTextPaintHandler()
{}

/*------------------------------------------------------------------------------
| IFlyTextPaintHandler::~IFlyTextPaintHandler                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyTextPaintHandler::~IFlyTextPaintHandler()
{}

/*------------------------------------------------------------------------------
| IFlyTextPaintHandler::paintWindow                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IFlyTextPaintHandler::paintWindow( IPaintEvent& event )
{
   // Build a graf port with the current clip region, which is
   // in native coordinates.
   IGArea
     invalidArea( event.rect() );
   IExtendedRootGrafPort
     nativePort( event.presSpaceHandle(), kRightHand );
   ILinkedGrafPort
     gport( &nativePort, &invalidArea );

   IFlyText*       flyText(((IFlyText*)event.window()));
   IRectangle      box(event.rect());

   box.sizeTo(box.size() - ISize(1,1));
   IRectangle      rc(box);
   IGRect2D        outline( box );

   GpiSetTextAlignment( event.presSpaceHandle(),  TA_STANDARD_HORIZ, TA_BOTTOM);

   // Draw the filled box upon which the text will be drawn.
   IGRect2D   fill(rc);
   IGrafBundle bundle;
   bundle.setFramePaint( IBasicColor( flyText->backgroundColor()) );
   bundle.setFillPaint ( IBasicColor( flyText->backgroundColor() ) );
   bundle.setDrawingOperation( IAttributeState::kFillAndFrame );
   //bundle.setFillColor(flyText->backgroundColor());
   gport.draw( fill, bundle);

   bundle.setDrawingOperation( IAttributeState::kFrame );

   IText  text( flyText->text() );
   IColor fgnd( flyText->foregroundColor() );
   text.addStyles(
     ITextColorStyle(fgnd.redMix(), fgnd.greenMix(), fgnd.blueMix()));
   IFont fnt(event.window()->font());
   text.addStyles(ITextTypefaceStyle(fnt.name()));
   text.addStyles(ITextPointSizeStyle( fnt.pointSize() ) );
   if ( fnt.isBold() )
   {
     text.addStyles(ITextBoldfaceStyle( true ) );
   }
   if ( fnt.isItalic() )
   {
     text.addStyles(ITextItalicStyle( true ) );
   }
   if ( fnt.isUnderscore() )
   {
     text.addStyles(ITextUnderlineStyle( true ) );
   }
   if ( fnt.isStrikeout() )
   {
     text.addStyles(ITextStrikethroughStyle( true ) );
   }

   IGraphicText    theText( text, IGraphicText::kSingleLine );  // BRO Moved

   // Add 3 to both x and y values so that text appears centered
   theText.setLocation( IGPoint2D( rc.left()+3, rc.bottom() ));

   theText.adoptBundle(&bundle);   // BRO Added
   theText.draw( gport );
   IGrafBundle *tempBundle = theText.orphanBundle();   // BRO Added

   // Draw the three outline boxes.  

   bundle.setFramePaint( IBasicColor( flyText->borderColor() ) );
   bundle.setDrawingOperation( IAttributeState::kFrame );
   gport.draw( outline, bundle);
   box.shrinkBy(IPair(1,1));
   outline.extendTo(box);
   bundle.setFramePaint( IBasicColor( flyText->borderColor() ) );
   gport.draw( outline, bundle);
   box.shrinkBy(IPair(1,1));
   outline.extendTo(box);
   bundle.setFrameColor( IBasicColor( flyText->shadowColor() ) );
   gport.draw( outline, bundle);
   return true;
}

/*------------------------------------------------------------------------------
| IFlyTextPaintHandler::dispatchHandlerEvent                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IFlyTextPaintHandler::dispatchHandlerEvent( IEvent& event )
{
  if (event.eventId() == IC_UM_FLY_PAINT)
  {
    event.window()->positionOnSiblings();
    event.window()->show();
  }
  return Inherited::dispatchHandlerEvent( event );
}

#endif   // IC_PM

