// Revision: 39 1.6.1.5 source/ui/extapp/iflypriv.hpp, staticctls, ioc.v400, 000302  
/* NOSHIP */
#ifndef _IFLYPRIV_
#define _IFLYPRIV_
/*******************************************************************************
* FILE NAME: iflypriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of iflyhhdr.hpp                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandle.hpp>
#include <iksstab2.h>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itimer.hpp>
#include <iwindefs.h>
#include <iwindow.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IFlyOverData;
class IFlyOverHelpHandler;
class IFlyText;
class ITextControl;
struct tagNMHDR;

#pragma enum(4)
#pragma pack(push,4)

class IHandleStringElement {
public:
  IHandleStringElement ( unsigned long      handle,
                         const IString&     flyText,
                         const IString&     longText );

  IHandleStringElement ( unsigned long      handle,
                         const IResourceId& flyText,
                         const IResourceId& longText );

 ~IHandleStringElement ( );

  // Note:  A union would appear to be storage representation of choice
  //        for this class since it will contain either strings or
  //        resource IDs.  It is my understanding, though, that the
  //        compiler will generate 32 bytes of storage at a minumum for
  //        each structure which negates any savings by using a union.
unsigned long
  handleKey;
IString
  flyStr,
  longStr;
unsigned long
  flyResId,
  longResId;
bool
  fStrings;
#ifdef IC_WIN
  // Save the pointer to the subclassed window proc.
unsigned long
  (* __stdcall subclassProc )( HWND          hwnd,
                               unsigned long ulMsg,
                               WPARAM        mp1,
                               LPARAM        mp2 );
IFlyOverData
 *flyData;
#endif
private:
  IHandleStringElement ( const IHandleStringElement& );
IHandleStringElement
 &operator=            ( const IHandleStringElement& );
}; // IHandleStringElement

inline unsigned long const& key ( IHandleStringElement* const& k )
{
  return k->handleKey;
}

class IHandleStringSet : public IKeySortedSetAsTable < IHandleStringElement*, unsigned long >
{
typedef IKeySortedSetAsTable< IHandleStringElement*, unsigned long >
  Inherited;
public:
  IHandleStringSet ( );
 ~IHandleStringSet ( );
}; // IHandleStringSet

/*----------------------------------------------------------------------------*/
// An instance of IFlyOverWindowData is attached to each IWindow to
// which an IFlyOverHelpHandler is attached.
/*----------------------------------------------------------------------------*/
class IFlyOverWindowData : public IWindowData {
public:
  IFlyOverWindowData ( IFlyOverHelpHandler* handler );
virtual
 ~IFlyOverWindowData ( );

IFlyOverHelpHandler
 *fHandler;
unsigned long
  fEventMask;
static IWindow::DataHandle
  fgDataHandle;
}; // IFlyOverWindowData

/*******************************************************************************
* Objects of IFlyOverData contain the private data for the                     *
* IFlyOverHelpHandler class.                                                   *
*******************************************************************************/
class IFlyOverData {
friend class IFlyOverHelpHandler;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 ways to construct instances of this class:                       |
|   1. From a pointer to a fly text control, long text control and             |
|      an unsigned long indicating the timer delay.                            |
------------------------------------------------------------------------------*/
  IFlyOverData ( IFlyText*            flyText,
                 ITextControl*        longText,
                 unsigned long        delay,
                 unsigned long        initialDelay,
                 IFlyOverHelpHandler* flyHelp );

 ~IFlyOverData ( );

#ifdef IC_WIN
void
  notifyEvent  ( tagNMHDR*             notifyHeader );

IFlyOverData
 &setHelpText  ( const IWindowHandle&  handle,
                 IHandleStringElement& element );
#endif

#ifdef IC_PM
void
  timerpop     ( unsigned long timerId );
#endif
#ifdef IC_MOTIF
enum ChangeCode {
  timerExpired,                 // A timer expired
  flytextInvalid,               // The fly text is invalid ... remove it
  pointerMoved                  // Need to evaluate pointer motion
};
void
  timerpop     ( ChangeCode code );
#endif

/*------------------------------ Data Members ----------------------------------
|  The following data members are used to different font values:               |
|   flytxt      - Pointer to fly text control.                                 |
|   longtxt     - Pointer to long text control.                                |
|   flyOffset - Offset into the string table for the short text control.       |
|   longOffset  - Offset into the string table for the long text control.      |
|   timer       - Timer object.                                                |
|   prevPos     - last position of the mouse from a WM_CONTROL pointer.        |
|   controlId   - Id of window that the mouse was over from the last control   |
|                 pointer message.                                             |
------------------------------------------------------------------------------*/
IFlyText
 *fFlytxt;
ITextControl
 *fLongtxt;
long
  fFlyOffset,
  fLongOffset;

#ifdef IC_PM
ITimer
  fTimer;
#endif
#ifdef IC_MOTIF
XtIntervalId
  fTimer;
#endif

#ifdef IC_MOTIFPM
IWindowHandle
  fHandle;
IPoint
  fPrevPos;
#endif

#ifdef IC_PM
IWindowHandle
  fPrevHandle;
IWindow
 *fControlWindow;
#endif

IString
  fDefault;
IWindowHandle
  fControlHandle;
IFlyOverHelpHandler
 *fFly;
IResourceLibrary
 *fResLib;
unsigned long
  fTimerDelay,
  fInitialTimerDelay;

private:
  IFlyOverData ( const IFlyOverData& );
IFlyOverData
 &operator=    ( const IFlyOverData& );

IFlyOverData
 &setFlyText          ( const IString& helpText,
                        char*          szText,
                        bool           useWideChar,
                        unsigned int*  flags );
IFlyOverData
 &setFlyTextForHandle ( IWindowHandle  controlHandle,
                        char*          szText,
                        bool           useWideChar,
                        unsigned int*  flags );
#ifdef IC_MOTIFPM
IFlyOverData
 &positionTextWindow  ( );
#endif

IFlyOverData
 &setLongText         ( const IString& helpText );
}; // IFlyOverData;

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _IFLYPRIV_ */
