// Revision: 48 1.18.1.2 source/ui/extapp/itbarfrm.cpp, toolbar, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: itbarfrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbarfrm.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C"
{
  #define INCL_WINSYS
  #define INCL_WINFRAMEMGR
  #define INCL_WINFRAMECTLS
  #define INCL_WINWINDOWMGR
  #define INCL_WINCOUNTRY
  #define INCL_WINHOOKS
  #include <iwindefs.h>
}

#include <itbarfrm.hpp>
#include <itbfpriv.hpp>
#include <itbar.hpp>
#include <iswp.hpp>
#include <icoordsy.hpp>
#include <ifont.hpp>
#include <igrport.hpp>
#include <igraftxt.hpp>
#include <istyles.hpp>
#include <igimage.hpp>
#include <igarea2d.hpp>
 #ifdef IC_PM
  #include <ibundles.hpp>
 #endif
#include <itbpriv.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBARFRM_CPP_
  #include <ipagetun.h>
#endif

static const unsigned long titleBarHeight(19);

unsigned long IToolBarTitle::offSet(0);

/*------------------------------------------------------------------------------
| IToolBarFrameData::IToolBarFrameData                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::IToolBarFrameData( IToolBarFrameWindow* toolBarFrameWindow )
  : systemMenu( toolBarFrameWindow ),
    toolBarTitle( toolBarFrameWindow, "" ),
#ifdef IC_PM
    titleFrameHandler(),
    systemMenuDrawHandler(),
#endif
    titleBarCommandHandler()
#ifdef IC_PM
    ,
    showHideButton( IC_ID_SHOWHIDE,
                    toolBarFrameWindow,
                    toolBarFrameWindow,
                    IRectangle(),
                    IToolBarButton::bitmapVisible |
                    IToolBarButton::transparentBitmap |
                    IWindow::visible              |
                    IButton::noPointerFocus ),
    moveButton( IC_ID_OWNERMOVE,
                toolBarFrameWindow,
                toolBarFrameWindow,
                IRectangle(),
                IToolBarButton::bitmapVisible  |
                IToolBarButton::transparentBitmap |
                ICustomButton::latchable       |
                IWindow::visible               |
                IButton::noPointerFocus )
#endif //IC_PM
{
#ifdef IC_PM
  //Set the transparent color for the toolbar frame to be (255,0,255),
  //which is the color that the button were created with.
  //Although this is the default, a user can change it.  We do not
  //want a user's change to affect these buttons.
  IColor transparentColor(255,0,255);
  showHideButton.setTransparentColor(transparentColor);
  moveButton.setTransparentColor(transparentColor);
#endif
}

/*------------------------------------------------------------------------------
| IToolBarFrameData::~IToolBarFrameData                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::~IToolBarFrameData( )
{ }

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::IToolBarFrameWindow                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::IToolBarFrameWindow( unsigned long windowId,
                                          IFrameWindow* owner )
  : IToolBarFrameWindow::Inherited( windowId,
                                    IWindow::desktopWindow(),
                                    owner,
                                    IRectangle(),
                                    IFrameWindow::titleBar      |
                                    IFrameWindow::systemMenu    |
                                    IFrameWindow::border        |
#ifdef IC_WIN
// Toggling of floating toolbar not supported on Motif
                                    IFrameWindow::maximizeButton |
#endif
                                    IFrameWindow::noMoveWithOwner ),
    fToolBarFrameData( new IToolBarFrameData( this ))
{
#ifdef IC_PMWIN
  //IToolBarTitle::offSet not used in Motif
  IToolBarTitle::offSet = IQUERYSYSVALUE( SV_CYTITLEBAR ) - titleBarHeight;
#endif

#ifdef IC_MOTIF
  // ISystemMenu not supported under Motif.  Therefore we cannot remove
  // unwanted menu items such as:  Lower
  //                               Occupy Workspace...
  //                               Occupy All Workspaces
  //                               Unoccupy Workspace
  // Motif does not use the showHideButton, the moveButton, the
  // titleFrameHandler, or the systemMenuDrawHandler.
#endif
#ifdef IC_WIN
  // Remove all the unnecessary menu options.
  fToolBarFrameData->systemMenu.deleteItem(ISystemMenu::idSize)
                               .deleteItem(ISystemMenu::idMinimize);

  // Windows does not use the showHideButton, the moveButton,  the
  // titleFrameHandler, or the systemMenuDrawHandler.
#endif
#ifdef IC_PM
  // Make the system menu bitmap owner draw &
  // Remove all the unnecessary menu options.
  fToolBarFrameData->systemMenu.setItem(
                fToolBarFrameData->systemMenu.menuItem(SC_SYSMENU).setDrawItem())
                              .deleteItem(ISystemMenu::idRestore)
                              .deleteItem(ISystemMenu::idMinimize)
                              .deleteItem(ISystemMenu::idMaximize)
                              .deleteItem(ISystemMenu::idHide);

  // Set up bitmaps for move with owner button
  fToolBarFrameData->moveButton.setBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_NOOWNERMOVE));
  fToolBarFrameData->moveButton.setLatchedBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_OWNERMOVE));

  fToolBarFrameData->showHideButton.setUserData( false );

  fToolBarFrameData->showHideButton.setBitmap(IApplication::current()
                                   .resourceLibrary().tryToLoadBitmap(IC_ID_HIDE));

//.setLatchedColor( fToolBarFrameData->showHideButton.backgroundColor(), false );

  fToolBarFrameData->titleFrameHandler.handleEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.handleEventsFor(
                                  fToolBarFrameData->systemMenu.window());
#endif

  fToolBarFrameData->titleBarCommandHandler.handleEventsFor( this );

#ifdef IC_PM
  // These buttons are not shown in Windows, since title bar extensions
  // are not supported.  We use the maximize and restore buttons for the
  // collapse/expand functions
  addExtension( &fToolBarFrameData->showHideButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );
  addExtension( &fToolBarFrameData->moveButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );
#endif
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::~IToolBarFrameWindow                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::~IToolBarFrameWindow( )
{
#ifdef IC_PM
  fToolBarFrameData->titleFrameHandler.stopHandlingEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.stopHandlingEventsFor(
                                  fToolBarFrameData->systemMenu.window());
#endif
  fToolBarFrameData->titleBarCommandHandler.stopHandlingEventsFor( this );
  delete fToolBarFrameData;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const char* text )
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const IResourceId& text)
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::titleText                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarFrameWindow::titleText( ) const
{
  return fToolBarFrameData->toolBarTitle.text();
}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::ITitleBarFrameHandler                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::ITitleBarFrameHandler()
 : ITitleBarFrameHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::~ITitleBarFrameHandler                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::~ITitleBarFrameHandler()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::format                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool ITitleBarFrameHandler::format( IFrameFormatEvent &formatEvent )
{
#ifdef IC_MOTIFWIN
   // ITitleBarFrameHandler currently not attached in Windows and Motif
   // implementation.

   // Since we use the default title bar size, we can use the standard
   // format as well.
   return Inherited::format( formatEvent );
#endif
#ifdef IC_PM
  RECTL
    temp = { 0, 0 };

  IFrameFormatEvent
    localEvent = formatEvent;

  if ( formatEvent.parameter2() == 0 )
  {
    localEvent = IEvent( formatEvent.handle(),
                         formatEvent.eventId(),
                         formatEvent.parameter1(),
                         &temp );
  }

  // Get count of standard controls (and thus index of extensions)...
  unsigned long
    controls(this -> defaultProcedure( localEvent ));

  ISWPArray
   swpArray( formatEvent.swpArray(),
             controls + this->numberOfExtensions( formatEvent.frame() ) );

  ISWP &titleSWP     (swpArray[ swpArray.indexOf( FID_TITLEBAR )]);
  ISWP &clientSWP    (swpArray[ swpArray.indexOf( FID_CLIENT )]);
  ISWP &systemMenuSWP(swpArray[ swpArray.indexOf( FID_SYSMENU )]);

  // make the titlebar wider and smaller in height
  titleSWP.setSize( ISize( titleSWP.size().width() + IToolBarTitle::offSet,
                           titleSWP.size().height() - IToolBarTitle::offSet));

  // move the title bar up and over
  titleSWP.setPosition( IPoint( titleSWP.position().x() - IToolBarTitle::offSet,
                                titleSWP.position().y() + IToolBarTitle::offSet));

  // make the system menu smaller in both x and y direction
  systemMenuSWP.setSize( ISize( systemMenuSWP.size().width() - IToolBarTitle::offSet,
                                systemMenuSWP.size().height() - IToolBarTitle::offSet));
  // move the system menu up
  systemMenuSWP.setPosition( IPoint( systemMenuSWP.position().x(),
                                     systemMenuSWP.position().y() + IToolBarTitle::offSet));

  // make the client taller
  clientSWP.setSize( ISize( clientSWP.size().width(),
                            clientSWP.size().height() + IToolBarTitle::offSet));

  unsigned long widthBeforePositioningCtrls(titleSWP.size().width());

  // position the remaining extensions
  formatEvent.setResult( this->positionExtensions( localEvent, controls ));

  // Remove the two buttons to the left of the titlebar if there
  // is insufficient room to display them.
  if (widthBeforePositioningCtrls < 2 * titleBarHeight)
  {
    ISWP &moveButton    (swpArray[controls]);
    ISWP &showHideButton(swpArray[controls+1]);
    moveButton.setSize(ISize(0,0));
    showHideButton.setSize(ISize(0,0));
    titleSWP.setSize( ISize(widthBeforePositioningCtrls, titleSWP.size().height()));
  } /* endif */
  return true;
#endif //IC_PM
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::ITitlePaintHandler                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PM
ITitlePaintHandler::ITitlePaintHandler()
 : ITitlePaintHandler::Inherited()
{}
#endif

/*------------------------------------------------------------------------------
| ITitlePaintHandler::~ITitlePaintHandler                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PM
ITitlePaintHandler::~ITitlePaintHandler()
{}
#endif

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintWindow                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PM
bool ITitlePaintHandler::paintWindow( IPaintEvent& event )
{
  IMODTRACE_ALL("ITitlePaintHandler::paintWindow");

  IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());
  IRootGrafPort gp(event.presSpaceHandle());
  this->defaultProcedure(event);

  paintTitleText(*toolBarTitle, gp);

  return true;
}
#endif

/*------------------------------------------------------------------------------
| ITitlePaintHandler::dispatchHandlerEvent                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PM
bool ITitlePaintHandler::dispatchHandlerEvent( IEvent& event )
{
  // If the BIDI attributes change, repaint the title bar
  if (event.eventId() == 0xBD0)
     event.window()->postEvent(TBM_SETHILITE);

  if (event.eventId() == TBM_SETHILITE)
  {
    IMODTRACE_ALL("ITitlePaintHandler::dispatchHandlerEvent - activated");
    IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());

    IManagedPresSpaceHandle presSpace(event.window());
    IRootGrafPort gc(presSpace);

    this->defaultProcedure(event);

    // Get the title bar rectangle and set it as the
    // clip region to ensure the title text string does
    // not paint outside the title bar window.
    IRectangle toolBarTitleRect(toolBarTitle->rect());
    toolBarTitleRect.moveTo(IPoint(0,0));

    // Set the clip region to the window rect and destroy
    // any existing clip region.
    IGArea titleBarRegion(toolBarTitleRect);

    { // scope for the clipPort...
      ILinkedGrafPort clipPort(&gc, &titleBarRegion);

      // Paint the title text string.
      paintTitleText(*toolBarTitle, clipPort);
    }
    return true;
  }
  else
  {
    return ITitlePaintHandler::Inherited::dispatchHandlerEvent( event );
  }
}
#endif

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintTitleText                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PM
ITitlePaintHandler& ITitlePaintHandler::paintTitleText(
  const IToolBarTitle& toolBarTitle,
  IGrafPort& gp)
{
  IMODTRACE_ALL("ITitlePaintHandler::paintTitleText");

  //Function not needed in Windows since we use the standard title bar.
  IColor titleStrColor(0,0,0), bgndColor(0,0,0);

  IFont titleFont(&toolBarTitle);

  if (IQUERYACTIVEWINDOW(HWND_DESKTOP) == toolBarTitle.owner()->handle())
  {
    titleStrColor = IColor(IColor::activeTitleText);
    bgndColor = IColor(IColor::activeTitleTextBgnd);
  }
  else
  {
    titleStrColor = IColor(IColor::inactiveTitleText);
    bgndColor = IColor(IColor::inactiveTitleTextBgnd);
  }

  // Assume we should start the text from the left side but check for
  // right-to-left layout
  IGPoint2D pt(titleFont.avgCharWidth(),
               titleFont.maxAscender() + 1);

  if (IBidiSettings::isBidiSupported())
  {
    IBidiSettings bidi(toolBarTitle);
    if (bidi.windowLayout() == IBidiSettings::layoutRightToLeft)
    {
      unsigned long txtWidth = titleFont.textWidth(toolBarTitle.text(),bidi);
      pt.fX = (GCoordinate)(toolBarTitle.size().width() - txtWidth) - pt.fX;
    }
  }

  IGrafBundle   bundle;
  bundle.setFillColor(titleStrColor);

  IText titleText(toolBarTitle.text());
  titleText.addStyles(ITextTypefaceStyle(titleFont.name()));
  titleText.addStyles(ITextPointSizeStyle(titleFont.pointSize()));
  titleText.addStyles(ITextColorStyle(titleStrColor.redMix(),
                                      titleStrColor.greenMix(),
                                      titleStrColor.blueMix()));
  titleText.addStyles(ITextBoldfaceStyle(titleFont.isBold()));
  titleText.addStyles(ITextItalicStyle(titleFont.isItalic()));
  titleText.addStyles(ITextOutlineStyle(titleFont.isOutline()));
  titleText.addStyles(ITextUnderlineStyle(titleFont.isUnderscore()));
  titleText.addStyles(ITextStrikethroughStyle(titleFont.isStrikeout()));
  titleText.addStyles(ITextBitmapStyle(titleFont.isBitmap()));
  IGraphicText theText( titleText, IGraphicText::kSingleLine );
  theText.setLocation(pt);

  ILinkedGrafPort linkedPort( &gp, &bundle );
  theText.draw( linkedPort );
  return *this;
}
#endif //IC_PM

/*------------------------------------------------------------------------------
| IToolBarTitle::IToolBarTitle                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_MOTIFWIN
IToolBarTitle::IToolBarTitle( IWindow* owner, char* titleText )
                            : IToolBarTitle::Inherited( owner, titleText )
{
   // In Windows, we use the default system painting.  Therefore,
   // we let the base class (ITitle) handle everything.
}
#endif
#ifdef IC_PM
IToolBarTitle::IToolBarTitle( IWindow* owner, char* titleText )
                            : IToolBarTitle::Inherited( owner, "" ),
                              paintHandler(),
                              title(titleText)
{
  this->Inherited::setText("");
  paintHandler.handleEventsFor( this );

  // Set the default font to be Helv.8
  // Note: DBCS OS/2 systems should use the default font instead of Helv
  //       since the double-byte characters do not exist fot this font

  unsigned long codepage = WinQueryCp(HMQ_CURRENT);
  switch (codepage)
  {
    // Note: The following list represents the DBCS code pages supported
    //       by OS/2 2.1 and Warp as of June 1995

    case  949: // Korean                      (replaces older 934/944)
    case  944: // Korean
    case  934: // Korean
    case 1381: // People's Republic of China  (replaces older 936/946)
    case  936: // People's Republic of China
    case  946: // People's Republic of China
    case  950: // Taiwan                      (replaces older 938/948)
    case  938: // Taiwan
    case  948: // Taiwan
    case  932: // Japanese
    case  942: // Japanese
    {
      IFont fnt(this);
      fnt.setPointSize(8);
      fnt.setWindowFont(this);
    }
    break;
    default:
    {
      IFont fnt("Helv",8);
      fnt.setWindowFont(this);
    }
    break;
  }
}
#endif //IC_PM

/*------------------------------------------------------------------------------
| IToolBarTitle::~IToolBarTitle                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle::~IToolBarTitle( )
{
#ifdef IC_PM
  paintHandler.stopHandlingEventsFor( this );
#endif
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const char* text )
{
#ifdef IC_MOTIFWIN
  // Not needed since we use standard title bar
  Inherited::setText(text);
#endif
#ifdef IC_PM
  title = text;
  refresh();
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const IResourceId& residText )
{
#ifdef IC_MOTIFWIN
  Inherited::setText( residText );
#endif
#ifdef IC_PM
  title = residText.resourceLibrary().loadString(residText);
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::text                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarTitle::text( ) const
{
#ifdef IC_MOTIFWIN
  return Inherited::text();
#endif
#ifdef IC_PM
  return title;
#endif
}

#ifdef IC_PM
//Note that the ISystemMenuDrawHandler class is only used in the
//OS/2 implementation
/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::ISystemMenuDrawHandler                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::ISystemMenuDrawHandler() :
  ISystemMenuDrawHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::~ISystemMenuDrawHandler                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::~ISystemMenuDrawHandler()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::draw                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool ISystemMenuDrawHandler::draw(IMenuDrawItemEvent& evt, DrawFlag& flag)
{
  IBitmapHandle bmp = IApplication::current().resourceLibrary()
                      .tryToLoadBitmap ( IC_SMALLSYSTEMMENU );
  if (bmp)
  {
    IRootGrafPort gc(evt.itemPresSpaceHandle());
    IGImage systemMenu(bmp);
    gc.draw(systemMenu);
    return true;
  }
  return false;
}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::setSize                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool ISystemMenuDrawHandler::setSize(IMenuDrawItemEvent& evt, ISize& newSize)
{
  newSize.setWidth(IQUERYSYSVALUE( SV_CYTITLEBAR )).setHeight( titleBarHeight );
  return true;
}
#endif //IC_PM

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::ITitleBarCommandHandler                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::~ITitleBarCommandHandler                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::~ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::command                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool ITitleBarCommandHandler::command( ICommandEvent& event )
{
  switch (event.commandId())
  {
    case IC_ID_SHOWHIDE:
    {
      IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
#ifdef IC_MOTIFWIN
      if (toolBar)
      {
        toolBar->expand( !toolBar->isExpanded() );
        // toggle state of tool bar expand/collapse
      }
#endif
#ifdef IC_PM
      IToolBarButton *showHideButton((IToolBarButton*)
                     IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
      if (showHideButton->userData())
      {
        showHideButton->setBitmap( IC_ID_HIDE );
        showHideButton->setUserData( false );
        toolBar->expand( );
        // show the tool bar
      }
      else
      {
        showHideButton->setBitmap( IC_ID_SHOW );
        showHideButton->setUserData( true );
        toolBar->collapse( );
        // hide the tool bar
      }
#endif
    }
    break;
#ifdef IC_PM  //ifndef IC_WIN
    // IFrameWindow::noMoveWithOwner ignored in Windows and Motif (always on)
    case IC_ID_OWNERMOVE:
    {
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IToolBarButton *moveButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_OWNERMOVE, event.window()));

      unsigned long tbfStyle(ISTYLEOF(toolBarFrame->handle()));
      if (moveButton->isLatched())
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle & ~FS_NOMOVEWITHOWNER ));
      }
      else
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle | FS_NOMOVEWITHOWNER ));
        // move with owner
      }
    }
    break;
#endif
    default:
    {
      // Route the command event to the frame which owns the toolbar frame,
      // if there is one.
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IWindow             *owner (toolBarFrame ? toolBarFrame->owner() : 0);
      if (owner)
         owner->sendEvent( event );
    }
    break;
  } /* endswitch */
  return false;
}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::systemCommand                                       |
------------------------------------------------------------------------------*/
bool ITitleBarCommandHandler::systemCommand( ICommandEvent& event )
{
  // We use restore and maximize to collapse and expand the toolbar,
  // respectively
#ifdef IC_WIN
  switch  ( event.commandId() & 0xFFFFFFF0 )
#endif
#ifdef IC_MOTIFPM
  switch  ( event.commandId() )
#endif
  {
#ifdef IC_WIN
     case  SC_RESTORE:
     {
        IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
        if (toolBar && toolBar->isExpanded())
           {
           // Need to set the normal position so the toolbar will not
           // move during the restore.
           WINDOWPLACEMENT wp;
           RECT            position;
           POINT           delta;
           wp.length = sizeof( WINDOWPLACEMENT );
           GetWindowRect( event.handle(), &position);    // current position
           GetWindowPlacement( event.handle(), &wp);     // current placement
           delta.x =  position.left - wp.rcNormalPosition.left;
           delta.y =  position.top  - wp.rcNormalPosition.top ;
           wp.rcNormalPosition.left   += delta.x;
           wp.rcNormalPosition.right  += delta.x;
           wp.rcNormalPosition.top    += delta.y;
           wp.rcNormalPosition.bottom += delta.y;
           SetWindowPlacement( event.handle(), &wp);
           event.handle().postEvent( WM_COMMAND,
                                     IEventData(IC_ID_SHOWHIDE),
                                     IEventData(0) );
           }
     }
     break;
     case  SC_MAXIMIZE:
     {
        IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
        if (toolBar && !toolBar->isExpanded())
           {
           // We ensure the maximize position is correct in
           // dispatchHandlerEvent since we need to set the size also.
           event.handle().postEvent( WM_COMMAND,
                                     IEventData(IC_ID_SHOWHIDE),
                                     IEventData(0) );
           }
     }
     break;
#endif //IC_WIN
  } /* endswitch */
  return false;
}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::dispatchHandlerEvent                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool ITitleBarCommandHandler::dispatchHandlerEvent( IEvent& event )
{
#ifdef IC_PM
  if ( event.eventId() == IC_UM_TOOLBAR &&
       event.parameter1().asUnsignedLong() == IC_UM_TBAR_SETEXPAND_BTN )
  {
    IToolBarButton *showHideButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
    if (event.parameter2().asUnsignedLong() == true)
    {
      showHideButton->setUserData( false );
      showHideButton->setBitmap( IC_ID_HIDE );
    }
    else
    {
      showHideButton->setUserData( true );
      showHideButton->setBitmap( IC_ID_SHOW );
    }
    return false;
  } // if
  else
#endif
//Toggling of toolbar not supported on Motif.  Floating toolbar does not
//have a maximize button
#ifdef IC_WIN
  if ( event.eventId() == WM_GETMINMAXINFO)
  {
    // We basically allow no changes in the size/position for maximize,
    // since all we really want changed by the system is the visuals.
    // The setLayoutDistorted function in IToolBar handles actual sizing.
    RECT rect;
    GetWindowRect( event.handle(), &rect );
    MINMAXINFO* mmi( (MINMAXINFO*)(event.parameter2().asUnsignedLong()) );
    mmi->ptMaxSize.x = rect.right - rect.left;
    mmi->ptMaxSize.y = rect.bottom - rect.top;
    mmi->ptMaxPosition.x = rect.left;
    mmi->ptMaxPosition.y = rect.top;
    // other fields allowed to default
    return true;
  } // if
  else
#endif
  {
    return Inherited::dispatchHandlerEvent( event );
  }
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::convertToGUIStyle                                       |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
| In Windows, this routine causes WS_EX_TOOLWINDOW style to be set.            |
| This gives us the system toolbar frame behavior.                             |
------------------------------------------------------------------------------*/
unsigned long IToolBarFrameWindow::convertToGUIStyle ( const IBitFlag& style,
                                                bool extendedOnly ) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle =
             Inherited::convertToGUIStyle( style, extendedOnly );

#ifdef IC_WIN
  if (extendedOnly)
  {
    // Turn on the WS_EX_TOOLWINDOW style
    ulStyle |= WS_EX_TOOLWINDOW;
  }
#endif
  return( ulStyle );
}

