// Revision: 50 1.6.1.3 source/ui/extapp/itbfpriv.hpp, toolbar, ioc.v400, 000302 
/* NOSHIP */
#ifndef _ITBFPRIV_
#define _ITBFPRIV_
/*******************************************************************************
* FILE NAME: itbfpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ITitleBarFrameHandler                                                    *
*     ITitleBarCommandHandler                                                  *
*     ISystemMenuDrawHandler                                                   *
*     IToolBarTitle                                                            *
*     IToolBarFrameData                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <iframhdr.hpp>
#include <ipainhdr.hpp>
#include <icmdhdr.hpp>
#include <imndihdr.hpp>
#include <ititle.hpp>
#include <itbarbut.hpp>
#include <isysmenu.hpp>

#if __IBMCPP__ >= 400
#pragma namemangling(compat)
#endif

class IToolBarTitle;
class IResourceId;
class IGrafPort;
class IToolBarFrameWindow;

#pragma enum(4)
#pragma pack(push,4)

class ITitleBarFrameHandler : public IFrameHandler {
typedef IFrameHandler
  Inherited;
/*******************************************************************************
* Objects of this class are used to size the IToolBarTitle objects.            *
* This handler should be attached to the frame window for which you want to    *
* change the title bar height.                                                 *
* This handler should be attached to the frame window before any other frame   *
* handlers you wish to attach to the frame window.                             *
*******************************************************************************/
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 way to construct instances of this class:                        |
|   1. default ctor.                                                           |
------------------------------------------------------------------------------*/

  ITitleBarFrameHandler  ( );
virtual
  ~ITitleBarFrameHandler ( );

protected:
/*----------------------------- Event Handling ---------------------------------
| These functions are passed by an IFrameEvent that provides an argument       |
| value and in which an event result can be set.  Each has a bool return    |
| type that indicates whether an additional handler should be dispatched.      |
|   format      - Fills in an array of window position structures, one for     |
|                 each frame control.  This function does this for the frame   |
|                 extensions that have been added to the frame using           |
|                 IFrameWindow::addExtension.                                  |
------------------------------------------------------------------------------*/
virtual bool
  format ( IFrameFormatEvent &formatEvent );
};

class ITitlePaintHandler : public IPaintHandler {
typedef IPaintHandler
  Inherited;
/*******************************************************************************
* Objects of this class are used to paint the tool bar title text.  This is    *
* necessary since there is a bug in PM that will always paint the title bar    *
* text as if the title bar is the default height.                              *
*******************************************************************************/
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 way to construct instances of this class:                        |
|   1. default ctor.                                                           |
------------------------------------------------------------------------------*/
  ITitlePaintHandler  ( );
virtual
  ~ITitlePaintHandler ( );

protected:

/*----------------------------- Event Handling ---------------------------------
| paintWindow          - Overridden to handle painting the title bar text.     |
| dispatchHandlerEvent - Overridden to handle events other that paint events   |
|                        that require repainting the title bar text.           |
------------------------------------------------------------------------------*/
virtual bool
  paintWindow          ( IPaintEvent &event ),
  dispatchHandlerEvent ( IEvent& event      );


/*---------------------------- Title text drawing ------------------------------
| paintTitleText - Draws the title text using a small font.                    |
------------------------------------------------------------------------------*/
virtual ITitlePaintHandler
  &paintTitleText(const IToolBarTitle& toolBarTitle, IGrafPort& gc);
};

class ITitleBarCommandHandler : public ICommandHandler
{
typedef ICommandHandler
  Inherited;
/*******************************************************************************
* Objects of this class are used to handle command events for buttons that     *
* are added to the floating tool bar frame window.                             *
*******************************************************************************/

public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 way to construct instances of this class:                        |
|   1. default ctor.                                                           |
------------------------------------------------------------------------------*/
  ITitleBarCommandHandler  ( );
virtual
  ~ITitleBarCommandHandler ( );

protected:

/*----------------------------- Event Handling ---------------------------------
| command              - Overriden to handle command events for the buttons    |
|                        that are added to the floating tool bar frame window. |
| dispatchHandlerEvent - Overriden to handle IC_UM_TBAR_SETEXPAND_BTN messages.|
|                        This message is used to set the start of the floating |
|                        toolbar expand/collapse button.                       |
------------------------------------------------------------------------------*/
virtual bool
  command                  ( ICommandEvent& event );

virtual bool
  systemCommand            ( ICommandEvent& event );

virtual bool
  dispatchHandlerEvent     ( IEvent& event        );
};

#ifdef IC_PMWIN
class ISystemMenuDrawHandler : public IMenuDrawItemHandler {
typedef IMenuDrawItemHandler
  Inherited;
/*******************************************************************************
* Objects of this class are used to draw the system menu.  The user of this    *
* class should attach this handler to a system menu bar.  You will need to     *
* change the style of the system menu to owner draw for this handler to        *
* function properly.                                                           *
*                                                                              *
* Example:                                                                     *
* IFrameWindow frameWindow;                                                    *
* ISystemMenu  systemMenu( &frameWindow );                                     *
* ISystemMenuDrawHandler systemMenuDrawHandler;                                *
* systemMenu.setItem(systemMenu.menuItem(SC_SYSMENU).setDrawItem());           *
* ISystemMenuDrawHandler systemMenuDrawHandler;                                *
* systemMenuDrawHandler.handleEventsFor( &systemMenu );                        *
*******************************************************************************/
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 way to construct instances of this class:                        |
|   1. default ctor.                                                           |
------------------------------------------------------------------------------*/
  ISystemMenuDrawHandler  ( );
virtual
  ~ISystemMenuDrawHandler ( );

protected:

virtual bool
  setSize (IMenuDrawItemEvent& evt, ISize& newSize),
  draw    (IMenuDrawItemEvent& evt, DrawFlag& flag);
};
#endif

class IToolBarTitle : public ITitle {
typedef ITitle
  Inherited;
/*******************************************************************************
* Objects of this class are used to create title bars that are smaller in      *
* height than the system title bar.                                            *
*******************************************************************************/
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 way to construct instances of this class:                        |
|   1. From a pointer to the owning window (usually a frame window) and        |                                                  |
|      a string indicating the text of the title bar.                          |
------------------------------------------------------------------------------*/
  IToolBarTitle  ( IWindow* owner,
                   char*    titleText );
virtual
  ~IToolBarTitle ( );

/*------------------------ Setting/Retrieving title text -----------------------
| setText - Sets the text of the title bar.  This is overriden to prevent      |
|           calling the inherited setText member functions which call the      |
|           system function directly to set the title text.                    |
| text    - Retrieves the text of the title bar.                               |
------------------------------------------------------------------------------*/
virtual IToolBarTitle
  &setText                ( const char* text        ),
  &setText                ( const IResourceId& text );

virtual IString
  text                    ( ) const;

static unsigned long
  offSet;

private:
/*------------------------------ Hidden Members ------------------------------*/
   IToolBarTitle ( const IToolBarTitle& title );
IToolBarTitle
  &operator=     ( const IToolBarTitle& title );

#ifdef IC_PM
ITitlePaintHandler
  paintHandler;
IString
  title;
#endif
}; //IC_PM

class IToolBarFrameData {
/*******************************************************************************
* Objects of IToolBarFrameData contain the private data for                    *
* IToolBarFrameWindow.  This class is necessary so that the private data       *
* and handlers for the floating toolbar are not externalized.                  *
*******************************************************************************/
public:
/*------------------------ Constructors/Destructor -----------------------------
| You construct object of this class:                                          |
|   -  From a pointer to a toolbar frame window.                               |
------------------------------------------------------------------------------*/
  IToolBarFrameData  ( IToolBarFrameWindow* toolBarFrameWindow );
  ~IToolBarFrameData ( );

ISystemMenu
  systemMenu;
IToolBarTitle
  toolBarTitle;

#ifdef IC_PM
ITitleBarFrameHandler
  titleFrameHandler;
ISystemMenuDrawHandler
  systemMenuDrawHandler;
#endif

ITitleBarCommandHandler
  titleBarCommandHandler;

#ifdef IC_PM
IToolBarButton
  showHideButton,
  moveButton;
#endif

private:
/*------------------------------ Hidden Members ------------------------------*/
   IToolBarFrameData ( const IToolBarFrameData& data );
IToolBarFrameData
  &operator=         ( const IToolBarFrameData& data );

}; // IToolBarFrameData

#pragma pack(pop)
#pragma enum(pop)

#if __IBMCPP__ >= 400
#pragma namemangling()
#endif

#endif /* _ITBFPRIV_ */
