// Revision: 82 1.9.1.3 source/ui/drag/idmitem.cpp, dragdrop, ioc.v400, 000302 
/*******************************************************************************
* FILE NAME: idmitem.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMItem                                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
  // include PM headers
  #define INCL_WINSTDDRAG
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
}

#include <itrace.hpp>
#include <iexcept.hpp>
#include <idmitem.hpp>
#include <idmsrch.hpp>
#include <idmimage.hpp>
#include <iapp.hpp>

#ifdef IC_WIN
  #include <idmformt.hpp>
#endif

#include <idmditm.hpp>
#include <idmdnfo.hpp>

#ifdef IC_MOTIF 
  #include <idmmotf.hpp>
#endif

#ifdef IC_PM
#pragma enum(4) 
#pragma pack(push,4)

// Utility functions.  This class is a friend so it's members can
// access private data members of items.
class IDMItemUtilities {
public:
static void
  setDragItemHSTR ( IDMItem* pItem,
                    unsigned long (_DRAGITEM::*pHSTR),
                    const IString&            aString );
}; // class IDMItemUtilities

#pragma enum(pop)
#pragma pack(pop)
#endif // IC_PM

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMITEM_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Static supported operation definitions.                                      |
|                                                                              |
| These values exploit the specific values used for the PM toolkit DO_*        |
| values.  Item creation must support the operations corresponding to the      |
| DO_* values.                                                                 |
------------------------------------------------------------------------------*/
const unsigned long
  IDMItem::unknown  ( 0 ),
  IDMItem::copyable ( DO_COPYABLE ),
  IDMItem::moveable ( DO_MOVEABLE ),
  IDMItem::linkable ( DO_LINKABLE );

/*------------------------------------------------------------------------------
| Static attribute definitions.                                                |
|                                                                              |
| These values exploit the specific values used for the PM toolkit DC_*        |
| values.  Item creation must support the attributes corresponding to the      |
| DC_* values.                                                                 |
------------------------------------------------------------------------------*/
const unsigned long
  IDMItem::none           ( 0 ),
  IDMItem::open           ( DC_OPEN ),
  IDMItem::reference      ( DC_REF  ),
  IDMItem::group          ( DC_GROUP ),
  IDMItem::container      ( DC_CONTAINER ),
  IDMItem::prepare        ( DC_PREPARE ),
  IDMItem::removableMedia ( DC_REMOVEABLEMEDIA );


/*------------------------------------------------------------------------------
| IDMItem::IDMItem                                                             |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMItem::IDMItem(IDMSourceOperation* pSrcOp,
                 const IString& types,
                 const unsigned long drgSupOps,
                 const unsigned long drgAttrs )
  : pDMSrcOperation(pSrcOp),
    pDMTgtOperation(0),
    strTypes(types),
    dragSupOps(drgSupOps),
    dragAttrs(drgAttrs),
    strContainerName(),
    strSourceName(),
    strTargetName(),
    strRMFs(),
    strSelectedRMF(),
    strContents(),
    sizeImageOffset(0,0),
    rendererPosition(0),
    pObject(0),
    bTargetTheSource(false),
#ifdef IC_PM
    pPMDragItem(0),
#endif
#ifdef IC_MOTIFWIN
    pDragItemWrapper(0),
#endif
    pIDMImage(0),
    drpStatus(IDM::notOk)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Assert and trace section                                         */
  /********************************************************************/

  /********************************************************************/
  /* Set our source window                                            */
  /********************************************************************/
  setSourceWindowHandle(pDMSrcOperation->sourceWindowHandle());
}

#ifdef IC_MOTIFPM
/*------------------------------------------------------------------------------
| IDMItem::IDMItem                                                             |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMItem::IDMItem(IDMTargetOperation* pTgtOp,
                 _DRAGITEM*          pDI )
  : strContents(),
    strSelectedRMF(),
    pDMSrcOperation(0),
    pDMTgtOperation(pTgtOp),
    rendererPosition(0),
    pObject(0),
    pIDMImage(0),
#ifdef IC_PM
    pPMDragItem(pDI),
#endif
#ifdef IC_MOTIF
    pDragItemWrapper(0),
#endif
    drpStatus(IDM::notOk)
{
  IFUNCTRACE_DEVELOP();

#ifdef IC_MOTIF
  pDragItemWrapper = new IDragItemWrapper( *pDI );
  /********************************************************************/
  /* Initialize ICLUI drag item object from PM counterpart            */
  /********************************************************************/
  setTypes(pDragItemWrapper->types());
  this->dragSupOps = pDragItemWrapper->dragSupOps();
  this->dragAttrs = pDragItemWrapper->dragAttrs();
  setSourceWindowHandle(pDragItemWrapper->sourceWindowHandle());
  setContainerName(pDragItemWrapper->containerName());
  setSourceName(pDragItemWrapper->sourceName());
  setTargetName(pDragItemWrapper->targetName());
  this->sizeImageOffset = pDragItemWrapper->imageOffset();
  setRMFs(IDMItem::rmfsFrom( pDragItemWrapper->rmfs() ));
#endif // IC_MOTIF

#ifdef IC_PM
  /********************************************************************/
  /* Initialize ICLUI drag item object from PM counterpart            */
  /********************************************************************/
  setTypes(stringFromHandle(pDI->hstrType));
  this->dragSupOps = pDI->fsSupportedOps;
  this->dragAttrs = pDI->fsControl;
  setSourceWindowHandle(pDI->hwndItem);
  setContainerName(stringFromHandle(pDI->hstrContainerName));
  setSourceName(stringFromHandle(pDI->hstrSourceName));
  setTargetName(stringFromHandle(pDI->hstrTargetName));
  this->sizeImageOffset = ISize(pDI->cxOffset,pDI->cyOffset);
  setRMFs(IDMItem::rmfsFrom( stringFromHandle(pDI->hstrRMF) ));
#endif // IC_PM

  /********************************************************************/
  /* Determine if source and target window are the same and set the   */
  /* result in both the source and target drag items.  Note that      */
  /* the default setting is source and target not the same ...        */
  /********************************************************************/
  //  bTargetTheSource is never used anywhere ????
  //     using the handle inside the ctor is mucking up the useCount !
  //     coming out of the ctor, the useCount should be 0 but instead is
  //     1 because this @#$@#@%$% handle is used here
  this->bTargetTheSource = false;
  //if (this->pDMTgtOperation)
  //{
  //  if (this->pDMTgtOperation->sourceWindowHandle() ==
  //      this->pDMTgtOperation->targetWindowHandle())
  //  {
  //    Handle pThisDI(this); this->addRef() ;
  //    // pThisDI = this;
  //    IDMItem::Handle pSrcDIH = IDMItem::sourceItemFor( pThisDI );
  //    if (pSrcDIH.valid())
  //      pSrcDIH->bTargetTheSource = this->bTargetTheSource = true;
  //  }
  //}
}
#endif //IC_MOTIFPM

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDMItem::IDMItem                                                             |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMItem::IDMItem(IDMTargetOperation* pTgtOp,
                 _DRAGITEM*          pDI )
  : strContents(),
    strSelectedRMF(),
    pDMSrcOperation(0),
    pDMTgtOperation(pTgtOp),
    rendererPosition(0),
    pObject(0),
    pDragItemWrapper(new IDragItemWrapper(*pDI)),
    pIDMImage(0),
    drpStatus(IDM::notOk)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Initialize ICLUI drag item object from pDragItemWrapper          */
  /********************************************************************/
  setTypes(pDragItemWrapper->types());
  this->dragSupOps = pDragItemWrapper->dragSupOps();
  this->dragAttrs = pDragItemWrapper->dragAttrs();
  setSourceWindowHandle(pDragItemWrapper->sourceWindowHandle());
  setContainerName(pDragItemWrapper->containerName());
  setSourceName(pDragItemWrapper->sourceName());
  setTargetName(pDragItemWrapper->targetName());
  this->sizeImageOffset = pDragItemWrapper->imageOffset();
  setRMFs(pDragItemWrapper->rmfs() );

  /********************************************************************/
  /* Determine if source and target window are the same and set the   */
  /* result in both the source and target drag items.  Note that      */
  /* the default setting is source and target not the same ...        */
  /********************************************************************/
  //  bTargetTheSource is never used anywhere ????
  //     using the handle inside the ctor is mucking up the useCount !
  //     coming out of the ctor, the useCount should be 0 but instead is
  //     1 because this @#$@#@%$% handle is used here
  this->bTargetTheSource = false;
  //if (this->pDMTgtOperation)
  //{
  //  if (this->pDMTgtOperation->sourceWindowHandle() ==
  //      this->pDMTgtOperation->targetWindowHandle())
  //  {
  //    Handle pThisDI(this) ; this->addRef() ;
  //    pThisDI = this;
  //    IDMItem::Handle pSrcDIH = IDMItem::sourceItemFor( pThisDI );
  //    if ( pSrcDIH.valid() )
  //      pSrcDIH->bTargetTheSource = this->bTargetTheSource = true;
  //  }
  //}
}
#endif // IC_WIN

/*------------------------------------------------------------------------------
| IDMItem::IDMItem                                                             |
|                                                                              |
| Copy constructor.                                                            |
------------------------------------------------------------------------------*/
IDMItem::IDMItem(const IDMItem& item)
   : pIDMImage(0)
#ifdef IC_MOTIFWIN
     , pDragItemWrapper(0)
#endif
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Make a copy of the drag item per the copy constructor            */
  /********************************************************************/
  this->dragSupOps       = item.dragSupOps;
  this->dragAttrs        = item.dragAttrs;
  this->sourceWndh       = item.sourceWndh;
  if (item.pIDMImage)
    this->pIDMImage        = new IDMImage(*(item.pIDMImage));
  this->sizeImageOffset  = item.sizeImageOffset;
  this->strContainerName = item.strContainerName;
  this->strSourceName    = item.strSourceName;
  this->strTargetName    = item.strTargetName;
  this->strTypes         = item.strTypes;
  this->strRMFs          = item.strRMFs;
  this->strSelectedRMF   = item.strSelectedRMF;
  this->strContents      = item.strContents;
  this->pDMSrcOperation  = item.pDMSrcOperation;
  this->pDMTgtOperation  = item.pDMTgtOperation;
  this->rendererPosition = item.rendererPosition;
  this->drpStatus        = item.drpStatus;
  this->pObject          = item.pObject;
  this->bTargetTheSource = item.bTargetTheSource;
#ifdef IC_PM
  this->pPMDragItem      = item.pPMDragItem;
#endif
#ifdef IC_WIN
  this->pDragItemWrapper = item.pDragItemWrapper;
#endif
#ifdef IC_MOTIF
  // TODO : does IDragItemWrapper live long enough to do what ic-win is doing ?
  if ( item.pDragItemWrapper )
    this->pDragItemWrapper = new IDragItemWrapper(*(item.pDragItemWrapper));
#endif

  /********************************************************************/
  /* Bump the operation's use count per the copy constructor ...      */
  /********************************************************************/
  if (this->pDMSrcOperation)
  {
    ITRACE_DEVELOP( "IDMItem (Src Operation) - addRef" );
    this->pDMSrcOperation->addRef();
  }
  else
  {
    ITRACE_DEVELOP( "IDMItem (Tgt Operation) - addRef" );
    this->pDMTgtOperation->addRef();
  }
}

/*------------------------------------------------------------------------------
| IDMItem::IDMItem                                                             |
|                                                                              |
| Constructor - Construct from a IDMItem::Handle.                              |
------------------------------------------------------------------------------*/
IDMItem::IDMItem(const Handle& pDI)
   : pIDMImage(0)
#ifdef IC_MOTIFWIN
     , pDragItemWrapper(0)
#endif
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Make a copy of the drag item per the copy constructor            */
  /********************************************************************/
  this->dragSupOps       = pDI->dragSupOps;
  this->dragAttrs        = pDI->dragAttrs;
  this->sourceWndh       = pDI->sourceWndh;
  if (pDI->pIDMImage)
    this->pIDMImage = new IDMImage(*(pDI->pIDMImage));
  this->sizeImageOffset  = pDI->sizeImageOffset;
  this->strContainerName = pDI->strContainerName;
  this->strSourceName    = pDI->strSourceName;
  this->strTargetName    = pDI->strTargetName;
  this->strTypes         = pDI->strTypes;
  this->strRMFs          = pDI->strRMFs;
  this->strSelectedRMF   = pDI->strSelectedRMF;
  this->strContents      = pDI->strContents;
  this->pDMSrcOperation  = pDI->pDMSrcOperation;
  this->pDMTgtOperation  = pDI->pDMTgtOperation;
  this->rendererPosition = pDI->rendererPosition;
  this->drpStatus        = pDI->drpStatus;
  this->pObject          = pDI->pObject;
  this->bTargetTheSource = pDI->bTargetTheSource;
#ifdef IC_PM
  this->pPMDragItem      = pDI->pPMDragItem;
#endif
#ifdef IC_WIN
  this->pDragItemWrapper = pDI->pDragItemWrapper;
#endif
#ifdef IC_MOTIF
  // TODO : does IDragItemWrapper live long enough to do what ic-win is doing ?
  if ( pDI->pDragItemWrapper )
    this->pDragItemWrapper = new IDragItemWrapper(*(pDI->pDragItemWrapper));
#endif

  /********************************************************************/
  /* Bump the operation's use count per the copy constructor ...      */
  /********************************************************************/
  if (this->pDMSrcOperation)
  {
    this->pDMSrcOperation->addRef();
    ITRACE_DEVELOP( "IDMItem (Src Operation) - addRef, useCount = " +
                    IString( pDMSrcOperation->count() ) );
  }
  else
  {
    this->pDMTgtOperation->addRef();
    ITRACE_DEVELOP( "IDMItem (Tgt Operation) - addRef, useCount = " +
                    IString( pDMTgtOperation->count() ) );
  }
}

/*------------------------------------------------------------------------------
| IDMItem::~IDMItem                                                            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMItem::~IDMItem()
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Decrement the operations use count.                              */
  /********************************************************************/
  if (this->pDMSrcOperation)
  {
    ITRACE_DEVELOP( "IDMItem (Src Operation) - removeRef" );
    this->pDMSrcOperation->removeRef();
  }
  if (this->pDMTgtOperation)
  {
    ITRACE_DEVELOP( "IDMItem (Tgt Operation) - removeRef" );
    this->pDMTgtOperation->removeRef();
  }
  if (pIDMImage)
     delete pIDMImage;
#ifdef IC_MOTIF
  if (pDragItemWrapper)
     delete pDragItemWrapper ; // TODO : how come ic-win not doint this ?
#endif
}

/*------------------------------------------------------------------------------
| IDMItem::operator =                                                          |
|                                                                              |
| Overload definition to handle assignments of drag items to drag items.       |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: operator = (const IDMItem& item)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Make a copy of the drag item per the assignment ...              */
  /********************************************************************/
  if (this != &item)
  {
    this->dragSupOps       = item.dragSupOps;
    this->dragAttrs        = item.dragAttrs;
    this->sourceWndh       = item.sourceWndh;
    if (pIDMImage)
       delete this->pIDMImage;
    if (item.pIDMImage)
      this->pIDMImage      = new IDMImage(*(item.pIDMImage));
    else
       this->pIDMImage     = 0 ;
    this->sizeImageOffset  = item.sizeImageOffset;
    this->strContainerName = item.strContainerName;
    this->strSourceName    = item.strSourceName;
    this->strTargetName    = item.strTargetName;
    this->strTypes         = item.strTypes;
    this->strRMFs          = item.strRMFs;
    this->strSelectedRMF   = item.strSelectedRMF;
    this->strContents      = item.strContents;
    this->pDMSrcOperation  = item.pDMSrcOperation;
    this->pDMTgtOperation  = item.pDMTgtOperation;
    this->rendererPosition = item.rendererPosition;
    this->drpStatus        = item.drpStatus;
    this->pObject          = item.pObject;
    this->bTargetTheSource = item.bTargetTheSource;
#ifdef IC_PM
    this->pPMDragItem      = item.pPMDragItem;
#endif
#ifdef IC_WIN
    this->pDragItemWrapper = item.pDragItemWrapper;
#endif
#ifdef IC_MOTIF
  // TODO : does IDragItemWrapper live long enough to do what ic-win is doing ?
  if (pDragItemWrapper)
     delete this->pDragItemWrapper ;
  pDragItemWrapper = 0 ;
  if ( item.pDragItemWrapper )
    this->pDragItemWrapper = new IDragItemWrapper(*(item.pDragItemWrapper));
#endif

    /******************************************************************/
    /* Bump the reference count for the operation to indicate that    */
    /* the item refers to it ...                                      */
    /******************************************************************/
    if (this->pDMSrcOperation)
    {
      ITRACE_DEVELOP( "IDMItem (Src Operation) - addRef" );
      this->pDMSrcOperation->addRef();
    }
    else
    {
      ITRACE_DEVELOP( "IDMItem (Tgt Operation) - addRef" );
      this->pDMTgtOperation->addRef();
    }
  }

  return( *this );
}

/*------------------------------------------------------------------------------
| IDMItem::operator =                                                          |
|                                                                              |
| Overload definition to handle assignments of drag items to drag items.       |
------------------------------------------------------------------------------*/
IDMItem::Handle IDMItem :: operator = (const Handle& pDI)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Make a copy of the drag item per the assignment ...              */
  /********************************************************************/
  Handle pThisDI(this) ; 
  if (pThisDI != pDI)
  {
    pThisDI->dragSupOps       = pDI->dragSupOps;
    pThisDI->dragAttrs        = pDI->dragAttrs;
    pThisDI->sourceWndh       = pDI->sourceWndh;
    if (pIDMImage)
       delete this->pIDMImage;
    if (pDI->pIDMImage)
      this->pIDMImage         = new IDMImage(*(pDI->pIDMImage));
    else
       this->pIDMImage        = 0 ;
    pThisDI->sizeImageOffset  = pDI->sizeImageOffset;
    pThisDI->strContainerName = pDI->strContainerName;
    pThisDI->strSourceName    = pDI->strSourceName;
    pThisDI->strTargetName    = pDI->strTargetName;
    pThisDI->strTypes         = pDI->strTypes;
    pThisDI->strRMFs          = pDI->strRMFs;
    pThisDI->strSelectedRMF   = pDI->strSelectedRMF;
    pThisDI->strContents      = pDI->strContents;
    pThisDI->pDMSrcOperation  = pDI->pDMSrcOperation;
    pThisDI->pDMTgtOperation  = pDI->pDMTgtOperation;
    pThisDI->rendererPosition = pDI->rendererPosition;
    pThisDI->drpStatus        = pDI->drpStatus;
    pThisDI->pObject          = pDI->pObject;
    pThisDI->bTargetTheSource = pDI->bTargetTheSource;
#ifdef IC_PM
    pThisDI->pPMDragItem      = pDI->pPMDragItem;
#endif
#ifdef IC_WIN
    pThisDI->pDragItemWrapper = pDI->pDragItemWrapper;
#endif
#ifdef IC_MOTIF
  // TODO : does IDragItemWrapper live long enough to do what ic-win is doing ?
  if (pDragItemWrapper)
     delete this->pDragItemWrapper ;
   pDragItemWrapper = 0 ;
  if ( pDI->pDragItemWrapper )
    this->pDragItemWrapper = new IDragItemWrapper(*(pDI->pDragItemWrapper));
#endif

    /******************************************************************/
    /* Bump the reference count for the operation to indicate that    */
    /* the item refers to it ...                                      */
    /******************************************************************/
    if (pThisDI->pDMSrcOperation)
    {
      ITRACE_DEVELOP( "IDMItem (Src Operation) - addRef" );
      pThisDI->pDMSrcOperation->addRef();
    }
    else
    {
      ITRACE_DEVELOP( "IDMItem (Tgt Operation) - addRef" );
      pThisDI->pDMTgtOperation->addRef();
    }
  }

  return( pThisDI );
}

/*------------------------------------------------------------------------------
| IDMItem::sourceWindowHandle                                                  |
|                                                                              |
| Returns the handle of the source window for this item.                       |
------------------------------------------------------------------------------*/
IWindowHandle IDMItem :: sourceWindowHandle() const
{
  return(this->sourceWndh);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceWindow                                                        |
|                                                                              |
| Returns a pointer to the source window for this item.                        |
------------------------------------------------------------------------------*/
IWindow* IDMItem :: sourceWindow() const
{
#ifdef IC_PMWIN
  return(IWindow::windowWithHandle(this->sourceWndh));
#endif
#ifdef IC_MOTIF
  return (IDMMotif::controlWindow(this->sourceWndh)) ; 
#endif
}

/*------------------------------------------------------------------------------
| IDMItem::containerName                                                       |
|                                                                              |
| Returns the source container name.                                           |
------------------------------------------------------------------------------*/
IString IDMItem :: containerName() const
{
  return(this->strContainerName);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceName                                                          |
|                                                                              |
| Returns the source object name.                                              |
------------------------------------------------------------------------------*/
IString IDMItem :: sourceName() const
{
  return(this->strSourceName);
}

/*------------------------------------------------------------------------------
| IDMItem::targetName                                                          |
|                                                                              |
| Returns the suggested target object name.                                    |
------------------------------------------------------------------------------*/
IString IDMItem :: targetName() const
{
  return(this->strTargetName);
}

/*------------------------------------------------------------------------------
| IDMItem::attributes                                                          |
|                                                                              |
| Returns the control attributes for this item.                                |
------------------------------------------------------------------------------*/
unsigned long IDMItem :: attributes() const
{
  return(this->dragAttrs);
}

/*------------------------------------------------------------------------------
| IDMItem::supportedOperations                                                 |
|                                                                              |
| Returns the operations supported by this item.                               |
------------------------------------------------------------------------------*/
unsigned long IDMItem :: supportedOperations() const
{
  return(this->dragSupOps);
}

/*------------------------------------------------------------------------------
| IDMItem::imageOffset                                                         |
|                                                                              |
| Returns the offset of the origin of the image from the pointer hotspot.      |
------------------------------------------------------------------------------*/
ISize IDMItem :: imageOffset() const
{
  return(this->sizeImageOffset);
}

/*------------------------------------------------------------------------------
| IDMItem::isOpen                                                              |
|                                                                              |
| Indicates whether the dragged object is "open".                              |
------------------------------------------------------------------------------*/
bool IDMItem :: isOpen() const
{
  return( (this->attributes() & open) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::isReference                                                         |
|                                                                              |
| Indicates whether the dragged object is a "reference" to another object.     |
------------------------------------------------------------------------------*/
bool IDMItem :: isReference() const
{
  return( (this->attributes() & reference) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::isGroup                                                             |
|                                                                              |
| Indicates whether the item is a group of objects.                            |
------------------------------------------------------------------------------*/
bool IDMItem :: isGroup() const
{
  return( (this->attributes() & group) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::isContainer                                                         |
|                                                                              |
| Indicates whether the item is a container of other objects.                  |
------------------------------------------------------------------------------*/
bool IDMItem :: isContainer() const
{
  return( (this->attributes() & container) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::isOnRemovableMedia                                                  |
|                                                                              |
| Indicates whether the item is on removable media or cannot be recovered      |
| after a move operation.                                                      |
------------------------------------------------------------------------------*/
bool IDMItem :: isOnRemovableMedia() const
{
  return( (this->attributes() & removableMedia) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::requiresPreparation                                                 |
|                                                                              |
| Indicates whether the source requires preparation prior to rendering.        |
------------------------------------------------------------------------------*/
bool IDMItem :: requiresPreparation() const
{
  return( (this->attributes() & prepare) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::canBeCopied                                                         |
|                                                                              |
| Indicates whether a copy operation is supported.                             |
------------------------------------------------------------------------------*/
bool IDMItem :: canBeCopied() const
{
  return( (this->supportedOperations() & copyable) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::canBeLinked                                                         |
|                                                                              |
| Indicates whether a link operation is supported.                             |
------------------------------------------------------------------------------*/
bool IDMItem :: canBeLinked() const
{
  return( (this->supportedOperations() & linkable) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::canBeMoved                                                          |
|                                                                              |
| Indicates whether a move operation is supported.                             |
------------------------------------------------------------------------------*/
bool IDMItem :: canBeMoved() const
{
  return( (this->supportedOperations() & moveable) !=0 );
}

/*------------------------------------------------------------------------------
| IDMItem::trueType                                                            |
|                                                                              |
| Returns the "true type" of the item.                                         |
------------------------------------------------------------------------------*/
IString IDMItem :: trueType() const
{
  /********************************************************************/
  /* First type in the string, delimited by ','                       */
  /********************************************************************/
  if (this->strTypes.indexOf(','))
    return(this->strTypes.subString(1, (this->strTypes.indexOf(',')-1)));
  else
  {
    /******************************************************************/
    /* Return the string if no delimiter (',')                        */
    /******************************************************************/
    return(this->strTypes);
  }
}

/*------------------------------------------------------------------------------
| IDMItem::types                                                               |
|                                                                              |
| Returns all types for the item.                                              |
------------------------------------------------------------------------------*/
IString IDMItem :: types() const
{
  return(this->strTypes);
}

/*------------------------------------------------------------------------------
| IDMItem::hasType                                                             |
|                                                                              |
| Indicates whether the item supports an argument type.                        |
------------------------------------------------------------------------------*/
bool IDMItem :: hasType(const char* pType) const
{
  /********************************************************************/
  /* Scan stored types for argument                                   */
  /********************************************************************/
  if (this->strTypes.indexOf(pType))
    return(true);
  else
    return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::nativeRMF                                                           |
|                                                                              |
| Returns the "native" rendering mechanism and format of the item.             |
------------------------------------------------------------------------------*/
IString IDMItem :: nativeRMF() const
{
  /********************************************************************/
  /* First RMF in the string, end of RMF's marked by '>' or ')'       */
  /********************************************************************/
  if ( rmfs().length() )
    return( IDMItem::rmfFrom(nativeRM(), nativeRF()) );
  else
    return( IString() );
}

/*------------------------------------------------------------------------------
| IDMItem::nativeRM                                                            |
|                                                                              |
| Returns the "native" rendering mechanism of the item.                        |
------------------------------------------------------------------------------*/
IString IDMItem :: nativeRM() const
{
  /********************************************************************/
  /* Find the first RM in the string                                  */
  /********************************************************************/
  return(IDMItem::rmFrom(rmfs()));
}

/*------------------------------------------------------------------------------
| IDMItem::nativeRF                                                            |
|                                                                              |
| Returns the "native" rendering format of the item.                           |
------------------------------------------------------------------------------*/
IString IDMItem :: nativeRF() const
{
  /********************************************************************/
  /* Find the first RF in the string                                  */
  /********************************************************************/
  return(IDMItem::rfFrom(rmfs()));
}

/*------------------------------------------------------------------------------
| IDMItem::rmfs                                                                |
|                                                                              |
| Returns all rendering mechanisms and formats of the item.                    |
------------------------------------------------------------------------------*/
IString IDMItem :: rmfs() const
{
  return(this->strRMFs);
}

/*------------------------------------------------------------------------------
| IDMItem::supportsRMF                                                         |
|                                                                              |
| Indicates whether the item supports a specific rendering mechanism and       |
| format.                                                                      |
------------------------------------------------------------------------------*/
bool IDMItem :: supportsRMF(const IString& pRMF)
{
  /********************************************************************/
  /* Scan stored RMFs for argument                                    */
  /********************************************************************/
  return IDMItem::matchingRMFs( this->rmfs(),
                                IDMItem::rmfsFrom( pRMF ),
                                true ).length();
}

/*------------------------------------------------------------------------------
| IDMItem::supportsRMF                                                         |
|                                                                              |
| Indicates whether the item supports a specific rendering mechanism and       |
| format.                                                                      |
------------------------------------------------------------------------------*/
bool IDMItem :: supportsRMF(const IString& pRM, const IString& pRF)
{
  /********************************************************************/
  /* Scan for multiple rendering mechanisms                           */
  /********************************************************************/
  return IDMItem::matchingRMFs( this->rmfs(),
                                IDMItem::rmfsFrom( pRM, pRF ),
                                true ).length();
}

/*------------------------------------------------------------------------------
| IDMItem::setNativeRMF                                                        |
|                                                                              |
| Sets the "native" RMF to the argument string.                                |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setNativeRMF(const IString& native)
{
  if ( native.length() )
  {
    IString
      old = this->nativeRMF();

    if ( old.length() )
      this->strRMFs.change( old, native, 1, 1 );
    else
      this->strRMFs = native;
  }

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrRMF,
                                     IDMItem::compressedRMFs( this->strRMFs ) );
#endif
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setRMFs( IDMItem::compressedRMFs( this->strRMFs ) );
#endif

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setNativeRMF                                                        |
|                                                                              |
| Sets the "native" RMF to the argument type.                                  |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setNativeRMF(const IString& pRM, const IString& pRF)
{
  return this->setNativeRMF( IDMItem::rmfFrom( pRM, pRF ) );
}

/*------------------------------------------------------------------------------
| IDMItem::setRenderer                                                         |
|                                                                              |
| Sets the position of the selected renderer for this item.                    |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setRenderer(unsigned position)
{
  this->rendererPosition = position;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setSelectedRMF                                                      |
|                                                                              |
| Sets selected RMF for this item.  Expects the argument to be in ordered pair |
| format (i.e. "<rm,rf>") or cross product (i.e. "(rm_x(rf1,rf2)").            |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setSelectedRMF(const IString& pRMF)
{
  this->strSelectedRMF = pRMF;
  this->dragSupOps = this->supportedOperationsFor( this->strSelectedRMF );
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::dropStatus                                                          |
|                                                                              |
| Returns the drop status for this item.                                       |
------------------------------------------------------------------------------*/
IDM::DropIndicator IDMItem :: dropStatus() const
{
  return( this->drpStatus );
}

/*------------------------------------------------------------------------------
| IDMItem::setDropStatus                                                       |
|                                                                              |
| Sets drop status for this item.                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setDropStatus(DropIndicator status)
{
  this->drpStatus = status;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::contents                                                            |
|                                                                              |
| Return the "contents" of the drag item.                                      |
------------------------------------------------------------------------------*/
IString IDMItem :: contents() const
{
  return(this->strContents);
}


/*------------------------------------------------------------------------------
| IDMItem::contentsSize                                                        |
|                                                                              |
| Returns the size of the contents of the drag item.                           |
------------------------------------------------------------------------------*/
unsigned long IDMItem :: contentsSize() const
{
  return(contents().size());
}

/*------------------------------------------------------------------------------
| IDMItem::setContents                                                         |
|                                                                              |
| Sets contents of the drag item.  Derived class should override to provide    |
| specific inplementation based upon characteristics of the window.            |
------------------------------------------------------------------------------*/
bool IDMItem :: setContents(const IString& data)
{
  this->strContents=data;
  return(true);
}

/*------------------------------------------------------------------------------
| IDMItem::object                                                              |
|                                                                              |
| Returns pointer to the object.                                               |
------------------------------------------------------------------------------*/
void* IDMItem :: object() const
{
  return(this->pObject);
}

/*------------------------------------------------------------------------------
| IDMItem::setObject                                                           |
|                                                                              |
| Sets the pointer to the object.                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem::setObject(void* pObj)
{
  this->pObject = pObj;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setSourceWindowHandle                                               |
|                                                                              |
| Sets the source window for this item.                                        |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setSourceWindowHandle(IWindowHandle srcWindow)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  this->sourceWndh = srcWindow;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field.  */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->hwndItem = this->sourceWndh;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setSourceWindowHandle( this->sourceWndh );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setContainerName                                                    |
|                                                                              |
| Sets the source container name.                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setContainerName(const char* pszContainerName)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  this->strContainerName = IString(pszContainerName);

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrContainerName,
                                     this->strContainerName );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setContainerName( this->strContainerName );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setSourceName                                                       |
|                                                                              |
| Sets the source object "name".                                               |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setSourceName(const char* pszSourceName)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  this->strSourceName = IString(pszSourceName);

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrSourceName,
                                     this->strSourceName );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setSourceName( this->strSourceName );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setTargetName                                                       |
|                                                                              |
| Sets the suggested target object "name".                                     |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setTargetName(const char* pszTargetName)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  this->strTargetName = IString(pszTargetName);

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrTargetName,
                                     this->strTargetName );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setTargetName( this->strTargetName );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::image                                                               |
|                                                                              |
| Returns the dragimage for this IDMItem.                                      |
------------------------------------------------------------------------------*/
IDMImage& IDMItem :: image ()
{
  if (!pIDMImage)
  {
    pIDMImage = new IDMImage();
  }
  return (*pIDMImage);
}

/*------------------------------------------------------------------------------
| IDMItem::setImage                                                            |
|                                                                              |
| Sets the dragimage for this IDMItem.                                         |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setImage (IDMImage& img)
{
  image() = img;
  sizeImageOffset = image().pointerOffset() ;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceItemFor                                                       |
|                                                                              |
| Retrieves the source item handle based upon the target item if and only if   |
| we're in the same process, and the parameter is a target item.  Otherwise,   |
| 0 is returned.                                                               |
------------------------------------------------------------------------------*/
IDMItem::Handle IDMItem::sourceItemFor(const Handle& pTgtDIH)
{
  /********************************************************************/
  /* Verify that the item is a target item (i.e. sourceOperation will */
  /* only be non-zero for a source item) ...                          */
  /********************************************************************/
  if (pTgtDIH->sourceOperation())
    return(0);

#ifdef IC_MOTIF
  // TODO : check if we can fold this into the ic-win logic
  IDragInfoWrapper dragInfoWrapper( *pTgtDIH->targetOperation() );
  if ( dragInfoWrapper.sourcePID() == IApplication::currentPID() )
  {
    IDragItemWrapper diw( *pTgtDIH->PMDragItem() );
    IDMItem::Handle pSrcDIH((IDMItem *)diw.itemID()) ; 
    return ( pSrcDIH );
  }
  else
    return(0);
#endif // IC_MOTIF

#ifdef IC_PMWIN
#ifdef IC_PM
  PID  srcPid,
       tgtPid;
  TID  tid;
#endif // IC_PM
#ifdef IC_WIN
  DWORD  srcPid = 0,
         tgtPid = 0;
  DWORD  tid;
#endif // IC_WIN

  if ((!IQUERYWINDOWPROCESS(pTgtDIH->sourceWindowHandle(),
                               &srcPid,
                               &tid)) ||
      (!IQUERYWINDOWPROCESS (pTgtDIH->targetOperation()->targetWindowHandle(),
                              &tgtPid,
                              &tid)))
  {
#ifdef IC_WIN
    return (0);
#else
    ITHROWGUIERROR("WinQueryWindowProcess");
#endif // IC_WIN
  }

  if (srcPid == tgtPid)
  {
     IDMItem::Handle pSrcDIH ;
    /*------------------------------------------------------------------
      Given a target item, return its corresponding source item
    ------------------------------------------------------------------*/
    IDMItem *pItem(0) ;
#ifdef IC_PM
    pItem = (IDMItem *)pTgtDIH->PMDragItem()->ulItemID;
#endif // IC_PM

#ifdef IC_WIN

    if (pTgtDIH->pDragItemWrapper)
      pItem = (IDMItem*)pTgtDIH->pDragItemWrapper->itemID();
#endif // IC_WIN

    if (pItem) 
    {
       pSrcDIH = pItem ;
    }

    return (pSrcDIH);
  }
  else
    return(0);
#endif // IC_PMWIN
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDMItem::tokenForWPSObject                                                   |
|                                                                              |
| Returns the token for the Workplace Shell object that can be passed to the   |
| OBJECT_FOR_PREC macro that returns a WPObject pointer.  Please note that the |
| application must be shell enabled or OBJECT_FOR_PREC will trap.              |
------------------------------------------------------------------------------*/
unsigned long IDMItem::tokenForWPSObject() const
{
  IString wpsRMF = IDMItem::rmfFrom( IDM::rmObject, IDM::rfObject );
  if (selectedRMF() == wpsRMF)
    return( ((IDMItem *)this)->PMDragItem()->ulItemID );
  else
    return( 0 );
}
#endif //IC_PM

/*------------------------------------------------------------------------------
| IDMItem::generateSourceItems                                                 |
|                                                                              |
| Generates generic items.                                                     |
------------------------------------------------------------------------------*/
bool IDMItem :: generateSourceItems(IDMSourceOperation* pDMSrcOp)
{
  IFUNCTRACE_DEVELOP();

  /********************************************************************/
  /* Create the generic item                                          */
  /********************************************************************/
  IDMItem *pItem = new IDMItem(pDMSrcOp, IDM::unknown);

  /********************************************************************/
  /* Add item to the source operation.                                */
  /********************************************************************/
  pDMSrcOp->addItem(pItem);

  /********************************************************************/
  /* Indicate that the item was successfully created.                 */
  /********************************************************************/
  return(true);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceOperation                                                     |
|                                                                              |
| Returns the source operation for this item, or 0 if request was made by      |
| target.                                                                      |
------------------------------------------------------------------------------*/
IDMSourceOperation* IDMItem :: sourceOperation()
{
  return(this->pDMSrcOperation);
}

/*------------------------------------------------------------------------------
| IDMItem::targetOperation                                                     |
|                                                                              |
| Returns the target operation for this item, or 0 if request was made by      |
| source.                                                                      |
------------------------------------------------------------------------------*/
IDMTargetOperation* IDMItem :: targetOperation()
{
  return(this->pDMTgtOperation);
}

/*------------------------------------------------------------------------------
| IDMItem::setOpen                                                             |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setOpen(bool bOpen)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bOpen)
    this->dragAttrs |= IDMItem::open;
  else
    this->dragAttrs &= ~IDMItem::open;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setReference                                                        |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setReference(bool bReference)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bReference)
    this->dragAttrs |= IDMItem::reference;
  else
    this->dragAttrs &= ~IDMItem::reference;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setGroup                                                            |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setGroup(bool bGroup)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bGroup)
    this->dragAttrs |= IDMItem::group;
  else
    this->dragAttrs &= ~IDMItem::group;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setContainer                                                        |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setContainer(bool bContainer)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bContainer)
    this->dragAttrs |= IDMItem::container;
  else
    this->dragAttrs &= ~IDMItem::container;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setOnRemovableMedia                                                 |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setOnRemovableMedia(bool bRemovableMedia)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bRemovableMedia)
    this->dragAttrs |= IDMItem::removableMedia;
  else
    this->dragAttrs &= ~IDMItem::removableMedia;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setRequiresPreparation                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setRequiresPreparation(bool bRequiresPrep)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bRequiresPrep)
    this->dragAttrs |= IDMItem::prepare;
  else
    this->dragAttrs &= ~IDMItem::prepare;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsControl = (unsigned short)this->dragAttrs;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragAttrs( this->dragAttrs );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::enableCopy                                                          |
|                                                                              |
| Enables or disables copy operation for the item.                             |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: enableCopy(bool bCopyable)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bCopyable)
    this->dragSupOps |= IDMItem::copyable;
  else
    this->dragSupOps &= ~IDMItem::copyable;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsSupportedOps = (unsigned short)this->dragSupOps;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragSupOps( this->dragSupOps );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::enableLink                                                          |
|                                                                              |
| Enables or disables link operation for the item.                             |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: enableLink(bool bLinkable)
{

  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bLinkable)
    this->dragSupOps |= IDMItem::linkable;
  else
    this->dragSupOps &= ~IDMItem::linkable;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsSupportedOps = (unsigned short)this->dragSupOps;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragSupOps( this->dragSupOps );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::enableMove                                                          |
|                                                                              |
| Enables or disables move operation for the item.                             |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: enableMove(bool bMoveable)
{

  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  if (bMoveable)
    this->dragSupOps |= IDMItem::moveable;
  else
    this->dragSupOps &= ~IDMItem::moveable;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  _DRAGITEM *pDI = this->PMDragItem();
  if (pDI)
  {
    if (pDI->ulItemID == (unsigned long)this)
      pDI->fsSupportedOps = (unsigned short)this->dragSupOps;
  }
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMyDragItem() )
    pDragItemWrapper->setDragSupOps( this->dragSupOps );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setTrueType                                                         |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setTrueType(const char* pType)
{
  /********************************************************************/
  /* Has a true type been previously set?                             */
  /********************************************************************/
  if (this->strTypes.length())
  {
    /******************************************************************/
    /* Yes, get current true type                                     */
    /******************************************************************/
    const IString strTrue = this->trueType();

    /******************************************************************/
    /* Replace it with the new type                                   */
    /******************************************************************/
    this->strTypes.change(strTrue, pType, 1, 1);
  }
  else
  {
    /******************************************************************/
    /* No, set it                                                     */
    /******************************************************************/
    this->strTypes = pType;
  }

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrType,
                                     this->strTypes );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setTypes( this->strTypes );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setTypes                                                            |
|                                                                              |
| Sets the types for the item.  Issue setTypes("") to remove all types for the |
| item.                                                                        |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setTypes(const char* pTypes)
{
  /********************************************************************/
  /* Set the types                                                    */
  /********************************************************************/
  this->strTypes = pTypes;

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrType,
                                     this->strTypes );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setTypes( this->strTypes );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::addType                                                             |
|                                                                              |
| Adds an additional type (or types).                                          |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: addType(const char* pType)
{
  /********************************************************************/
  /*  Store in the class member data                                  */
  /********************************************************************/

  /********************************************************************/
  /* Are there types defined?                                         */
  /********************************************************************/
  if (types().length())
  {
    /******************************************************************/
    /* Append new type(s)                                            */
    /******************************************************************/
    this->strTypes += "," + IString(pType);
  }
  else
  {
    /******************************************************************/
    /* No, add new type(s)                                            */
    /******************************************************************/
    this->strTypes = pType;
  }

  /********************************************************************/
  /* If this "set" occurred on a drag item that was instantiated by a */
  /* source window, also update the corresponding PM drag item field. */
  /********************************************************************/
#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrType,
                                     this->strTypes );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setTypes( this->strTypes );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::removeType                                                          |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: removeType(const char* pType)
{
  IDMItem::deleteRMF( this->strTypes, pType );
  this -> setTypes( this->strTypes );

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::setRMFs                                                             |
|                                                                              |
| Sets the RMFs for the item.  Issue setRMFs("") to remove all RMFs for the    |
| item.                                                                        |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: setRMFs( const IString& pRMF)
{
  /********************************************************************/
  /* Store in the class member data                                   */
  /********************************************************************/
  this->strRMFs = IDMItem::rmfsFrom( pRMF );

#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrRMF,
                                     IDMItem::compressedRMFs( this->strRMFs ) );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setRMFs( IDMItem::compressedRMFs( this->strRMFs ) );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::addRMF                                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: addRMF(const IString& pRMF)
{
  IDMItem::appendRMF( this->strRMFs, IDMItem::rmfsFrom( pRMF ) );

#ifdef IC_PM
  IDMItemUtilities::setDragItemHSTR( this,
                                     &DRAGITEM::hstrRMF,
                                     IDMItem::compressedRMFs( this->strRMFs ) );
#endif // IC_PM
#ifdef IC_MOTIF
  if ( iHaveMySrcDragItem() )
    pDragItemWrapper->setRMFs( IDMItem::compressedRMFs( this->strRMFs ) );
#endif // IC_MOTIF

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::addRMF                                                              |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: addRMF ( const IString& rm, const IString& rf )
{
  return this -> addRMF( IDMItem::rmfsFrom( rm, rf ) );
}

/*------------------------------------------------------------------------------
| IDMItem::removeRMF                                                           |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: removeRMF(const IString& pRMF)
{
  IDMItem::deleteRMF( this->strRMFs, pRMF );
  this -> setRMFs( this->strRMFs );

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMItem::removeRMF                                                           |
------------------------------------------------------------------------------*/
IDMItem& IDMItem :: removeRMF ( const IString& rm, const IString& rf )
{
  return this -> removeRMF( IDMItem::rmfFrom( rm, rf ) );
}

/*------------------------------------------------------------------------------
| IDMItem::sourceRender                                                        |
|                                                                              |
| Called to give the item opportunity to acquire the data that will be         |
| transferred to the target.                                                   |
------------------------------------------------------------------------------*/
bool IDMItem :: sourceRender(IDMSourceRenderEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::sourcePrepare                                                       |
|                                                                              |
| Called to give the item opportunity to acquire information needed by the     |
| target, perform a task required by a target prior to target rendering, or    |
| perform setup required by a source renderer.                                 |
------------------------------------------------------------------------------*/
bool IDMItem :: sourcePrepare(IDMSourcePrepareEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceEnd                                                           |
|                                                                              |
| Called by the source handler when the target has informed it that the target |
| has finished processing the data that was associated with the dropped item.  |
| Source cleanup for the item should occur here.                               |
------------------------------------------------------------------------------*/
bool IDMItem :: sourceEnd(IDMSourceEndEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::sourceDiscard                                                       |
|                                                                              |
| Called by a derived IDMSourceHandler::sourceDiscard() function when it was   |
| indicated that the source window has responsibility for discarding the item. |
------------------------------------------------------------------------------*/
bool IDMItem :: sourceDiscard(IDMSourceDiscardEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::sourcePrint                                                         |
|                                                                              |
| Called by a derived IDMSourceHandler::sourcePrint() function when it was     |
| indicated that the source window has responsibility for printing the item.   |
------------------------------------------------------------------------------*/
bool IDMItem :: sourcePrint(IDMSourcePrintEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::targetDrop                                                          |
|                                                                              |
| Called to give the item the opportunity to "write" the data that was         |
| transferred to the target window.                                            |
------------------------------------------------------------------------------*/
bool IDMItem :: targetDrop(IDMTargetDropEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::targetEnd                                                           |
|                                                                              |
| Called by the target handler when the source has informed it that the source |
| has finished rendering the data that was associated with the dropped item.   |
| Target cleanup for the item should occur here.                               |
------------------------------------------------------------------------------*/
bool IDMItem :: targetEnd(IDMTargetEndEvent& event)
{
  IFUNCTRACE_DEVELOP();

  return(false);
}

/*------------------------------------------------------------------------------
| IDMItem::generateSourceName                                                  |
|                                                                              |
| Returns the data to be dragged so that it can be placed into the sourceName  |
| field of the dragitem structure which is used in conjunction with the        |
| IDM::rfText rendering format.  0 will be returned if the data does not       |
| meet the following criteria:                                                 |
|    - The data must be <= 255 bytes.                                          |
|    - The data must not contain any embedded NULL characters.                 |
------------------------------------------------------------------------------*/
IString IDMItem :: generateSourceName ()
{
  IString data = contents();

  /**********************************************************************/
  /* If the data that is to be dragged, is less than 255 characters and */
  /* does not contain embedded null characters, return it.              */
  /**********************************************************************/
  unsigned dataLength = data.length();
  if ((dataLength) && (dataLength <= 255) && (!(data.includes((char)0))))
  {
    return (data);
  }
  else
  {
    return ("");
  }
}

/*------------------------------------------------------------------------------
| IDMItem::PMDragItem                                                          |
|                                                                              |
| Returns corresponding PM DRAGITEM pointer.                                   |
------------------------------------------------------------------------------*/
_DRAGITEM* IDMItem :: PMDragItem()
{
#ifdef IC_PM
  return(this->pPMDragItem);
#endif
#ifdef IC_WIN
  return 0;
#endif
#ifdef IC_MOTIF
  if ( pDragItemWrapper )
    return( this->pDragItemWrapper->asDragItemPtr() );
  else
    return 0;
#endif
}

/*------------------------------------------------------------------------------
| IDMItem::setPMDragItem                                                       |
|                                                                              |
| Sets corresponding PM DRAGITEM pointer.                                      |
------------------------------------------------------------------------------*/
void IDMItem :: setPMDragItem(_DRAGITEM* pDI)
{
  IFUNCTRACE_DEVELOP();
#ifdef IC_MOTIFWIN
  if (pDragItemWrapper)
     delete pDragItemWrapper;
  pDragItemWrapper = new IDragItemWrapper( *pDI );
#endif // IC_MOTIFWIN
#ifdef IC_PM
  this->pPMDragItem = pDI;
#endif // IC_PM
}

/*------------------------------------------------------------------------------
| IDMItem::asPMDragItem                                                        |
|                                                                              |
| Fills in a PM DRAGITEM structure                                             |
------------------------------------------------------------------------------*/
void IDMItem :: asPMDragItem(_DRAGITEM* pDI)
{
  IFUNCTRACE_DEVELOP();

#ifdef IC_MOTIFWIN
  IDragItemWrapper diw( *pDI );
  /********************************************************************/
  /* Save a pointer to this ICLUI drag item object in the ulItemID    */
  /* field of the PM DRAGITEM that represents the object              */
  /********************************************************************/
  diw.setItemID( (unsigned long)this );

  diw.setSourceWindowHandle( this->sourceWindowHandle() );
  diw.setTypes( this->types() );
  diw.setRMFs( IDMItem::compressedRMFs( this->rmfs() ) );
  diw.setContainerName( this->containerName() );
  diw.setTargetName( this->targetName() );
  diw.setImageOffset( this->imageOffset() );
  diw.setDragAttrs( (unsigned short)this->attributes() );
  diw.setDragSupOps( (unsigned short)this->supportedOperations() );
  diw.setSourceName( this->sourceName() );
  diw.setImageOffset( this->imageOffset() );
#endif // IC_MOTIFWIN

#ifdef IC_PM
  /********************************************************************/
  /* Save a pointer to this ICLUI drag item object in the ulItemID    */
  /* field of the PM DRAGITEM that represents the object              */
  /********************************************************************/
  pDI->ulItemID = (unsigned long)this;

  pDI->hwndItem = this->sourceWindowHandle();
  pDI->hstrType = handleFromString(this->types());
  pDI->hstrRMF = handleFromString( IDMItem::compressedRMFs( this->rmfs() ));
  pDI->hstrContainerName = handleFromString(this->containerName());
  pDI->hstrTargetName = handleFromString(this->targetName());
  pDI->cxOffset = (short)this->imageOffset().width();
  pDI->cyOffset = (short)this->imageOffset().height();
  pDI->fsControl = (unsigned short)this->attributes();
  pDI->fsSupportedOps = (unsigned short)this->supportedOperations();
  pDI->hstrSourceName = handleFromString(this->sourceName());
#endif // IC_PM
}

/*------------------------------------------------------------------------------
| IDMItem::stringFromHandle                                                    |
|                                                                              |
| Converts a PM Atom (HSTR) into an IString.                                   |
------------------------------------------------------------------------------*/
IString IDMItem :: stringFromHandle (const IStringHandle& hstr)
{

  /********************************************************************/
  /* Retrieve string from PM atom table                               */
  /********************************************************************/
#ifdef IC_PM
  char szBuf[256]={'\0'};
  DrgQueryStrName(hstr, sizeof(szBuf), (PSZ)szBuf);
  return(IString(szBuf));
#endif
#ifdef IC_WIN
  return IDMFormat::stringFromFormat( (short) hstr.asUnsigned() );
#endif
#ifdef IC_MOTIF
   return IString() ;
#endif // IC_MOTIF
}

/*------------------------------------------------------------------------------
| IDMItem::handleFromString                                                    |
|                                                                              |
| Converts an IString into a PM Atom (HSTR).                                   |
------------------------------------------------------------------------------*/
IStringHandle IDMItem :: handleFromString(IString inputStr)
{
  if (inputStr.length())
  {
#ifdef IC_PM
    IStringHandle sh = DrgAddStrHandle((PSZ)inputStr);
    if (sh)
    {
      return (sh);
    }
    else
    {
      ITHROWGUIERROR("DrgAddStrHandle");
    }
#endif

#ifdef IC_WIN
    return IStringHandle((void *) (unsigned long) IDMFormat::formatFromString(inputStr) );
#endif
#ifdef IC_MOTIF
    return IStringHandle(0) ;
#endif // IC_MOTIF
  }

  return (IStringHandle(0));
}

/*------------------------------------------------------------------------------
| IDMItem::supportedOperationsFor                                              |
|                                                                              |
| The default implementation return all the supported operations for the item. |
------------------------------------------------------------------------------*/
unsigned long IDMItem :: supportedOperationsFor ( const IString& ) const
{
  return this->supportedOperations();
}

/*------------------------------------------------------------------------------
| IDMItem:appendRMF                                                            |
|                                                                              |
| Appends an RMF to an canonical "rmfs" string.  It will separate additional   |
| RMFs from ones already there with a comma.                                   |
------------------------------------------------------------------------------*/
void IDMItem :: appendRMF( IString& rmfs, const IString& rmf )
{
  unsigned pos = rmfs.length();

  // Add 1 plus another if non-zero...
//if ( pos++ && pos++ );
  if (pos)
    pos += 2;

  rmfs.overlayWith( rmf, pos, ',' );
}

/*------------------------------------------------------------------------------
| IDMItem::deleteRMF                                                           |
|                                                                              |
| Delete rmf from rmf string.  Manage commas as appropriate.                   |
------------------------------------------------------------------------------*/
void IDMItem :: deleteRMF ( IString& rmfs, const IString& rmf )
{
  IString delRMF( rmf );

  rmfs
    .center( rmfs.length() + 2, ',' )
    .change( delRMF.center( delRMF.length() + 2, ',' ), ",", 1, 1 )
    .strip( "," );
}

/*------------------------------------------------------------------------------
| IDMItem::compressedRMFs                                                      |
|                                                                              |
| This function returns a "compressed" version of the argument rmf string,     |
| which must be in "canonical" form.  It will convert rmfs to cross-product    |
| form, where possible.                                                        |
------------------------------------------------------------------------------*/
IString IDMItem :: compressedRMFs( const IString& rmfsIn )
{
  unsigned rmfIndex = 1;
  IString
    rmfs( rmfsIn ),
    result,
    currentRM,
    currentRFs,
    xRMs,
    xRFs;

  // Add dummy <rm,rf>s to force last real one to be processed...
  appendRMF( rmfs, "<\a1,\a1>,<\a2,\a2>" );
  while ( rmfIndex <= rmfs.length() )
  {
    // Get next RMF pair...
    IString
      rm = nextRMOrRFFrom( rmfs, rmfIndex ),
      rf = nextRMOrRFFrom( rmfs, rmfIndex );

    // Compare rm to current one...
    if ( rm == currentRM )
    {
      // Add this rf to current set for this rm...
      appendRMF( currentRFs, rf );
    }
    else
    {
      // End of rfs for currentRM.  We will add this RM to
      // pending cross product, or, complete pending
      // cross product and initialize new pending cross-product
      // to (currentRM)x(currentRFs)...
      if ( currentRM.length() )
      {
        // See if currentRM can be added to cross product...
        if ( currentRFs == xRFs )
          // Match, so add current RM to cross product...
          appendRMF( xRMs, currentRM );
        else
        {
          // Previous cross-product done...
          // See if cross product pending...
          if ( xRMs.length() )
          {
            // See if cross-product or simple rmf...
            if ( xRMs.includes(',')  ||
                 xRFs.includes(',') )
            {
              // Need cross product, add (rms)x(rfs)...
              unsigned
                resLen = result.length(),
                rmsLen = xRMs.length(),
                rfsLen = xRFs.length();
              // If current length > 0, add a comma...
              if ( resLen )
                resLen++;

              result
                .leftJustify( resLen + rmsLen + rfsLen + 5, ',' )
                .overlayWith( "(", resLen + 1 )
                .overlayWith( xRMs, resLen + 2 )
                .overlayWith( ")x(", resLen + 2 + rmsLen )
                .overlayWith( xRFs, resLen + 2 + rmsLen + 3 )
                .overlayWith( ")", resLen + 2 + rmsLen + 3 + rfsLen );
            }
            else
            {
              // Simple rmf will do, add <rm,rf>...
              appendRMF( result, rmfFrom( xRMs, xRFs ) );
            }
          }

          // Current rm/rfs become pending cross-product...
          xRMs = currentRM;
          xRFs = currentRFs;
        }
      }

      currentRM  = rm;
      currentRFs = rf;
    }// end Compare rm to current one...
  } //end while

  return( result );
}

/*------------------------------------------------------------------------------
| IDMItem::matchingRMFs                                                        |
|                                                                              |
| This function returns the intersection of the argument rmf strings.  Each    |
| argument must be in "canonical" form as returned by either of the            |
| rmfsFrom functions.  An optional third argument can be used to specify that  |
| only the first element of the intersection be returned.                      |
|                                                                              |
| The elements of the result are ordered as they appear in the first           |
| argument.                                                                    |
------------------------------------------------------------------------------*/
IString IDMItem :: matchingRMFs ( const IString& rmfs1,
                                  const IString& rmfs2,
                                  bool        firstOnly )
{
  IString result;
  unsigned rmfIndex = 1;

  // Loop through rmfs in first argument...
  while ( rmfIndex <= rmfs1.length() )
  {
    // Get next rmf from first argument...
    rmfIndex = rmfs1.indexOf( '<', rmfIndex );
    if ( !rmfIndex )
      break;

    unsigned end = rmfs1.indexOf( '>', rmfIndex );
    if ( !end )
      // Invalid input so quit...
      break;

    IString rmf1 = rmfs1.subString( rmfIndex, end - rmfIndex + 1 );
    rmfIndex = end;
    // See if second arg contains this rmf...
    if ( rmfs2.includes( rmf1 ) )
    {
      appendRMF( result, rmf1 );
      if ( firstOnly )
        break;
    }
  } //end while

  return( result );
}

/*------------------------------------------------------------------------------
| IDMItem::rfForThisProcess                                                    |
|                                                                              |
| Returns DRF_PROCESS=pid where pid is the current process id.                 |
------------------------------------------------------------------------------*/
IString IDMItem :: rfForThisProcess()
{
  IString pid( IApplication::currentPID().asString() );

  /********************************************************************/
  /* Create "DRF_PROCESS=pid" and return                              */
  /********************************************************************/
  IString
    tempStr( IDM::rfProcess, strlen( IDM::rfProcess ),
             "=", 1,
             (char *)pid, pid.length() );
  return( tempStr );
}

/*------------------------------------------------------------------------------
| IDMItem::rfFrom                                                              |
|                                                                              |
| Returns the rendering format component from the argument rmf.                |
------------------------------------------------------------------------------*/
IString IDMItem :: rfFrom ( const IString& rmf )
{
  IString result;
  unsigned i = 1;

  // Skip rm...
  nextRMOrRFFrom( rmf, i );
  return( nextRMOrRFFrom( rmf, i ) );
}

/*------------------------------------------------------------------------------
| IDMItem::rmfFrom                                                             |
|                                                                              |
| Constructs a valid PM rmf from argument rm and rf.  The arguments must       |
| be singular!  That is, not multiple rms or rfs separated by commas.          |
------------------------------------------------------------------------------*/
IString IDMItem :: rmfFrom ( const IString& rm, const IString& rf )
{
  IString
    result( 0, rm.length() + rf.length() + 3 );
  result
    .overlayWith( "<", 1 )
    .overlayWith( rm, 2 )
    .overlayWith( ",", rm.length() + 2 )
    .overlayWith( rf, rm.length() + 3 )
    .overlayWith( ">", result.length() );

  return( result );
}

/*------------------------------------------------------------------------------
| IDMItem::rmFrom                                                              |
|                                                                              |
| Returns the rendering mechanism component from the argument rmf.             |
------------------------------------------------------------------------------*/
IString IDMItem :: rmFrom ( const IString& rmf )
{
  IString result;
  unsigned i = 1;
  return( nextRMOrRFFrom( rmf, i ) );
}

/*------------------------------------------------------------------------------
| IDMItem::rmfsFrom                                                            |
|                                                                              |
| Returns rmfs from separate rm/rf specifications.  Either rm or rf can        |
| have multiple values which must be separated with commas and can be enclosed |
| in parentheses.  The result is in "canonical" format (e.g., no cross         |
| products).                                                                   |
------------------------------------------------------------------------------*/
IString IDMItem :: rmfsFrom( const IString& rms, const IString& rfs )
{
  IString rmfs;
  unsigned rmIndex = 1;

  // Loop on rms till done...
  while ( rmIndex <= rms.length() )
  {
    // Get next rm...
    IString rm = nextRMOrRFFrom( rms, rmIndex );

    //Loop on rfs till done...
    unsigned rfIndex = 1;
    while ( rfIndex <= rfs.length() )
    {
      // Get next rf...
      IString rf = nextRMOrRFFrom( rfs, rfIndex );

      // Add canonical RMF to result...
      appendRMF( rmfs, rmfFrom( rm, rf ) );
    }
  }

  return( rmfs );
}

/*------------------------------------------------------------------------------
| IDMItem::rmfsFrom                                                            |
|                                                                              |
| Expand argument RMFs to canonical format.  The input can be comprised of     |
| any valid RMF specification (including cross products).                      |
------------------------------------------------------------------------------*/
IString IDMItem :: rmfsFrom( const IString& input )
{
  IString result;
  unsigned rmIndex = 1;

  // Loop on <rm,rf>s or ()x()s till done...
  while ( rmIndex <= input.length() )
  {
    rmIndex = input.indexOfAnyOf( "<(", rmIndex );
    if ( rmIndex )
    {
      if ( input[ rmIndex ] == '<' )
      {
        // Copy next <rm,rf> to result...
        // Split rmf into rm/rf...
        IString
          rm = nextRMOrRFFrom( input, rmIndex ),
          rf = nextRMOrRFFrom( input, rmIndex );

        appendRMF( result, rmfFrom( rm, rf ) );
      }
      else
      {
        // Expand next (rm1,rm2)x(rf1,rf2) to result...
        // First, pick off rms...
        unsigned end = input.indexOf( ')', rmIndex );
        if ( !end )
          end = input.length() + 1;

        IString
          rms = input.subString( rmIndex, end - rmIndex + 1 );
        rmIndex = end + 1;
        // Next, get rfs...
        unsigned rfIndex = input.indexOf( '(', rmIndex );
        end = input.indexOf( ')', rmIndex );
        IString rfs;

        if ( rfIndex && end )
        {
          rfs = input.subString( rfIndex + 1, end - rfIndex + 1 );
          rmIndex = end + 1;
        }
        else
        {
          // Missing '(' or ')' so quit...
          rfs = IDM::rfUnknown;
          rmIndex = input.length() + 1;
        }
        // Add expanded rmfs to result...
        appendRMF( result, rmfsFrom( rms, rfs ) );
      }
    }
    else
      // All done...
      break;
  } // end while

  return( result );
}

/*------------------------------------------------------------------------------
| IDMItem::nextRMOrRFFrom                                                      |
|                                                                              |
| Returns the next rm or rf from the argument string, starting at the          |
| argument index.  The index is also updated to point to the place where       |
| the next rm or rf will be fetched from.                                      |
------------------------------------------------------------------------------*/
IString IDMItem :: nextRMOrRFFrom( const IString& source, unsigned& index )
{
  unsigned end = source.indexOf( ',', index );
  if ( !end )
    end = source.length() + 1;

  IString result = source.subString( index, end - index );
  index = end + 1;
  return( result.strip( " ()<>" ) );
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDMItem::setHSTR                                                             |
|                                                                              |
| Set the HSTR to the new value.  If it previously was not null, release       |
| it.                                                                          |
------------------------------------------------------------------------------*/
void IDMItem :: setHSTR ( unsigned long& oldHSTR, const IStringHandle& newHSTR )
{
  IStringHandle old = oldHSTR;
  oldHSTR = newHSTR;

  if ( old )
  {
    if ( !DrgDeleteStrHandle( old ) )
      ITHROWGUIERROR( "DrgDeleteStrHandle" );
  }
}

/*------------------------------------------------------------------------------
| IDMItemUtilities::setDragItemHSTR                                                       |
|                                                                              |
| If there is a PM drag image, and this object is the corresponding source     |
| object, then reset the HSTR within it (pointed to by the first argument)     |
| to the string specified as the second argument.                              |
------------------------------------------------------------------------------*/
void IDMItemUtilities::setDragItemHSTR( IDMItem* pItem,
                                        unsigned long ( DRAGITEM :: *pHSTR ),
                                        const IString& string )
{
  DRAGITEM *pPMItem = pItem->PMDragItem();
  if ( pItem->pDMSrcOperation  &&
       pPMItem                 &&
       ( pPMItem->ulItemID == (unsigned long)pItem ) )
  {
    pItem->setHSTR( pPMItem->*pHSTR, pItem->handleFromString( string ) );
  }
}
#endif // IC_PM

#ifdef IC_MOTIF

bool IDMItem::iHaveMyDragItem() const
{
  return pDragItemWrapper && pDragItemWrapper->itemID() == (unsigned long) this;
}

bool IDMItem::iHaveMySrcDragItem() const
{
  return pDMSrcOperation
      && pDragItemWrapper && pDragItemWrapper->itemID() == (unsigned long) this;
}

#endif // IC_MOTIF
